

package BondLib "Modelica library for Bond Graph modeling" 
  extends Modelica.Icons.Library;
  
  annotation (
    Window(
      x=0.01,
      y=0.55,
      width=0.4,
      height=0.4,
      library=1,
      autolayout=1),
    Documentation(info="<html>
The <b>BondLib</b> library is designed as a graphical library for modeling physical systems using the bond graph metaphor.  This library has been created by Prof. Fran&ccedil;ois Cellier and his students.<p>

When modeling systems in this fashion, it is common to hide many connections behind the bondgraphic connectors.  To this end, one first places neighboring connected elements next to each other <br>
<img src=\"../Images/BG_Fig1.png\"> <br>
making the connection between the connectors.  The two models are then shifted together, until the two connectors are in the same position on the screen <br>
<img src=\"../Images/BG_Fig2.png\"> <br>
thereby hiding the connection from view. <p>

Bond graphs are described in terms of four basic variables: <p>

<ol>
<li> efforts (<b>e</b>), <br>
<li> flows (<b>f</b>), <br>
<li> generalized momentums (<b>p</b>), and <br>
<li> generalized positions (<b>q</b>).
</ol> <p>

The four variables are related to each other in the following fashion: <br>
<img src=\"../Images/BG_Fig3.png\"> <p>

Power is always the product of effort and flow: <br>
<b>P = e*f</b> <p>

The three basic OnePort elements of the bond graph methodology are: <p>

<ol>
<li> resistors (<b>R</b>): <b>e = R(f)</b>, <br>
<li> capacitors (<b>C</b>): <b>e = C(q)</b>, and <br>
<li> inductors (<b>I</b>): <b>f = I(p)</b>,
</ol> <p>

where <b>R</b>, <b>C</b>, and <b>I</b> can be arbitrarily non-linear functions operating in the first and third quadrants only. <p>

Bond graphs are domain independent.  The four basic variables of a number of physical domains are summarized below: <br>
<img src=\"../Images/BG_Fig4.png\"> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, ISBN: 0-387-97502-0, 755p.
<li> Cellier, F.E. (1992), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/HierarchBG.pdf\">Hierarchical Non-linear
     Bond Graphs: A Unified Methodology for Modeling Complex Physical Systems</a>,\" <i>Simulation</i>, <b>58</b>(4),
     pp. 230-248.
<li> Cellier, F.E. (1995), \"Bond Graphs: The Right Choice for Educating Students in Modeling Continuous-time Physical
     Systems,\" <i>Simulation</i>, <b>64</b>(3), pp. 154-159.
<li> Cellier, F.E. and R.T. McBride (2003), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/icbgm03_bglib.pdf\">Object-oriented
     Modeling of Complex Physical Systems Using the Dymola Bond-graph Library</a>,\" <i>Proc. ICBGM'03,
     6<sup>th</sup> SCS Intl. Conf. on Bond Graph Modeling and Simulation</i>, Orlando, Florida, pp. 157-162.
<li> Cellier, F.E. and A. Nebot (2005), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/modelica2005_cellier.pdf\">The
     Modelica Bond Graph Library</a>,\" <i>Proc. 4<sup>th</sup> Modelica Conference</i>, Hamburg, Germany.
</ol> <p>

<hr>

<b>Main Author:</b> <p>

Prof. Dr. Fran&ccedil;ois E. Cellier <br>
Institut f&uuml;r Computational Science <br>
ETH Z&uuml;rich <br>
ETH Zentrum CAB G82.1 <br>
CH-8092 Z&uuml;rich <br>
Switzerland <p>

Phone: +41(1)632-7474 <br>
Fax: +41(1)632-1374 <br>
Email: <a href=\"mailto:FCellier@Inf.ETHZ.CH\"> FCellier@Inf.ETHZ.CH</a> <br>
URL: <a href=\"http://www.inf.ethz.ch/~fcellier/\"> http://www.inf.ethz.ch/~fcellier/</a> <p>

<hr>

<b>Release Notes:</b> <p>

<ul>
<li>Version 1.0: February 10, 2005 <br>
<li>Version 1.1: June 21, 2005 <br>
<li>Version 1.2: November 21, 2005 <br>
<li>Version 1.3: December 13, 2006 <br>
<li>Version 2.0: July 18, 2007 <br>
<li>Version 2.1: August 2, 2007
</ul> <p>

<hr>

<b>Copyright (C) 2005-2007, Fran&ccedil;ois E. Cellier.</b> <p>

<i>The ModelicaAdditions.BondLib package is <b>free</b> software;
it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i>
</html>
"), Diagram(
      Rectangle(extent=[-100,100; 100,-100],style(
          color=0,
          thickness=2,
          fillColor=9)),
      Text(
        extent=[-60,68; 52,46],
        style(
          color=0,
          thickness=2,
          fillColor=9,
          fillPattern=1),
        string="The bond graph"),
      Text(
        extent=[-73.9,15.05; 68,-14],
        style(
          color=0,
          thickness=2,
          fillColor=9,
          fillPattern=1),
        string="domain independent"),
      Text(
        extent=[-55.8,34.1; 54,4],
        style(
          color=0,
          thickness=2,
          fillColor=9,
          fillPattern=1),
        string="offers a uniform, "),
      Text(
        extent=[-47.7,-8.85; 38,-26],
        style(
          color=0,
          thickness=2,
          fillColor=9,
          fillPattern=1),
        string="interface to"),
      Text(
        extent=[-79.6,-19.8; 82,-50],
        style(
          color=0,
          thickness=2,
          fillColor=9,
          fillPattern=1),
        string="physical system modeling."),
      Text(
        extent=[-45.9,51.05; 40,24],
        style(
          color=0,
          thickness=2,
          fillColor=9,
          fillPattern=1),
        string="methodology")),
    uses(                         MultiBondLib(version="1.2"), Modelica(version=
           "2.2.1")),
    version="2.1",
    conversion(from(version="", script="ConvertFromBondLib_.mos"),
        noneFromVersion="2.0"));
  
  package UsersGuide "User's Guide" 
    annotation (DocumentationClass=true, Documentation(info="<html>
<h3><font color=\"#008000\" size=5>Users Guide of package BondLib</font></h3>
<p>
Library <b>BondLib</b> is a <b>free</b> Modelica package providing components to model multi-energy physical systems in a convenient domain-independent format. This package contains the <b>users guide</b> to the library, and has the following content:
</p>
<ol>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview\">Overview of library</a>
       gives an overview of the library.
</ol>
</html>
"));
    package Overview "Overview of Library" 
      annotation (Documentation(info="<html>
This overview contains the following sections: <p>

<ol>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Introduction\">An introduction to bond graph modeling</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Connectors\">Bond graph connectors</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Bonds\">Causal and a-causal bonds</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Basic\">Basic bond graph elements</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Advanced\">Advanced bond graph elements</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Wrapping\">Wrapping bond graphs</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Electrical\">The electrical library</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Mechanical\">The mechanical libraries</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Thermal\">The thermal library</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Encapsulating\">Encapsulating bond graphs</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Hydraulic\">The hydraulic library</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Examples\">Examples</a>
<li><a href=\"Modelica://BondLib.UsersGuide.Overview.Literature\">Literature</a>
</ol>
</html>
"));
      class Introduction "An introduction to bond graph modeling" 
        annotation (Documentation(info="<html>
Bond graphs offer a generic domain-independent means to graphically modeling physical systems in an object-oriented fashion. <p>

All physical systems are characterized by the fact that they observe a number of conservation principles.  Prime among those is the conservation of energy.  When modeling a physical system, it is important for the modeler to be aware of these conservation principles, and ensure that the models constructed adhere to them. <p>

Yet, if the modeling formalism doesn't guarantee that the conservation principles are being observed, i.e., if it is up to the modeler to verify that his or her models adhere to them, these principles can get easily violated, because modelers (just like all other people) are constantly under time pressure, and usually won't take the time to validate their models, once they have been constructed and compile/simulate without generating error messages. <p>

It may therefore be a good idea to embrace a modeling formalism that checks the conservation principles implicitly for the modeler.  For example, if a modeler wishes to model an electrical capacitor, he or she knows that: <p>

<b>i = C*<font color=red>der</font>(u)</b> <p>

Yet, this model may fail, when the capacitance <b>C</b> happens to be a function of time.  It is better to ask oneself, what happens physically in the capacitor. <p>

We know that the capacitor stores <i>electrical charge</i>, <b>Q</b>.  The electrical charge is the integral of the current.  Thus, when current flows into the capacitor, electrical charge builds up.  The charge is a physical quantity having mass.  It thus needs to be conserved.  Thus, the charge, <b>Q</b>, rather than the voltage, <b>u</b>, is a much better choice for a state variable, since voltage doesn't need to be conserved. <p>

The voltage across the capacitor is proportional to the charge stored in the capacitor: <p>

<b>u = C*Q</b> <p>

What happens to the energy?  The power, <b>P</b>, flowing into the capacitor is the product of current and voltage: <p>

<b>P = u*i</b> <p>

The energy, <b>E</b>, stored in the capacitor is the integral of the power.  It also needs to be conserved.  Yet, we only need one state variable, thus energy doesn't get conserved automatically.  We must ensure explicitly that this happens. <p>

Bond graphs ensure that no energy is ever lost or (worse!) generated out of thin air in interconnections between models.  They do this by explicitly modeling the power flow through a physical system. <p>

Power, in all physical systems, can always be expressed as a product of two adjugate variables, or possibly in the most complex cases, as the sum of products of adjugate variables. <p>

Bond graphs work with these adjugate variables as the basic modeling variables: the <i>effort</i>, <b>e</b>, and the <i>flow</i>, <b>f</b>. Power is always the product of effort and flow: <p>

<b>P = e*f</b> <p>

What these adjugate variables represent, depends on the physical nature of the model.  For example in electrical systems, the effort would be the <i>voltage</i>, whereas the flow would be the <i>current</i>.  In mechanical translational systems, the effort would be the <i>force</i>, whereas the flow would be the <i>velocity</i>. <p>

A power flow is represented by a harpoon, with which the two adjugate variables are being associated.  The hook of the harpoon points always to the \"left\" (relative to the direction, in which the harpoon points), and the effort variable is written on the side of the harpoon, whereas the flow variable is written on the opposite side: <p>

<img src=\"../Images/BG_Fig5.png\"> <p>

The direction of the harpoon is the direction of positive power flow. <p>

Evidently, since <i>efforts</i> and <i>flows</i> can represent different physical properties, no strong type checking is possible in bond graph modeling.  All variables need to be declared as <b><font color=red>Real</font></b> rather than <b><font color=red>Modelica.SIunits.Voltage</font></b>, for example.  This is a price that the modeler must be willing to pay, if he or she wishes to use bond graphs. <p>

Sometimes, as in the case of the capacitor, it is useful to also consider the integrals of the two adjugate variables, as these are in fact natural state variables.  Since bond graphs were invented by a mechanical engineer, the integral of the effort is called the <i>generalized momentum</i>, <b>p</b>, whereas the integral of the flow is called the <i>generalized position</i>, <b>q</b>, since the integral of <i>force</i> is a <i>momentum</i>, whereas the integral of <i>velocity</i> is a position. <p>

The below table lists the four basic bond graph variables, <b>e</b>, <b>f</b>, <b>p</b>, and <b>q</b>, for a number of different energy domains: <p>

<img src=\"../Images/BG_Fig4.png\"> <p>

The bond graph model of a non-linear capacitor would thus take the form: <p>

<img src=\"../Images/BG_Fig6.png\"> <p>

where <b><font color=red>ModPassiveOnePort</font></b> is one of the standard bond graph interfaces offered in the <b><font color=\"#008000\">Interfaces</font</b> sub-library of the bond graph library.  It represents a modulated one-port, where the modulation signal (a real-valued input) is given the name <b>s</b>.  The variables <b>e</b> and <b>f</b> don't need to be declared either, as they are already declared inside the (partial) interface model.
</html>
"));
      end Introduction;
      
      class Connectors "Bond graph connectors" 
        annotation (Documentation(info="<html>
As bonds carry two physical variables, the <i>effort</i>, <b>e</b> and the <i>flow</i>, <b>f</i>, the bond graph connectors need to carry at least these two variables.  Yet, also the direction of positive power flow is explicitly encoded in the bond models, and thereby also in the connectors.  To this end, a (non-physical) directional variable, <b>d</b>, is associated with each bond: <p>

<img src=\"../Images/BG_Fig7.png\"> <p>

<b>d</b> assumes a value of <b>-1</b> at the connector, from which the bond emanates, and a value of <b>+1</b> at the  connector, where it ends. <p>

All bond graph variables are declared as <i>potential variables</i>, i.e., rather than making use of the <i>flow variables</i> of Modelica, the bond graph library encodes the direction of flow explicitly using the <b>d</b> variable. <p>

The bond graph connector is a grey dot.  Its definition is given as follows: <p>

<img src=\"../Images/BG_Fig8.png\"> <p>

The corresponding bond model has been encoded as: <p>

<img src=\"../Images/BG_Fig9.png\"> <p>

It passes the physical variables along, like an electrical wire, and it defines the directional variable, <b>d</b>, at the two ends. <p>

In the bond graph methodology, there are two types of nodes, called <i>junctions</i>, the <b>0-junction</b>, which corresponds to a regular Modelica node, and the <b>1-junction</b>, which offers the dual functionality. <p>

<img src=\"../Images/BG_Fig10.png\"> <p>

The flows into a 0-junction add up to zero, whereas the efforts around a 0-junction are the same.  Correspondingly, the efforts into a 1-junction add up to zero, whereas the flows around it are the same.  This, by the way, is the reason, why the bond graph library doesn't make use of the Modelica flow variables, since Modelica offers only one type of nodes. <p>

For example, the 1-junction with three bond connectors is defined as: <p>

<img src=\"../Images/BG_Fig11.png\"> <p>

It inherits the effort and flow vectors from the (partial) <b><font color=red>ThreePortOne</font></b> interface model: <p>

<img src=\"../Images/BG_Fig12.png\"> <p>

which makes use of the directional <b>d</b> variables to decide on the sign in summing the efforts (or flows) into the junction.
</html>
"));
      end Connectors;
      
      class Bonds "Causal and a-causal bonds" 
        annotation (Documentation(info="<html>
Modelica is perfectly capable of deciding on its own the computational causality of all equations.  Thus, models can be generated in the form of implicitly defined differential and algebraic equation (DAE) systems.  If we use the a-causal bonds, as presented so far, this is precisely, what we get. <p>

However, it may be useful to write models in their causal form, as the computational causality information may offer additional insight into the system. <p>

Since bonds carry two physical variables, we need to generate two equations to compute their values.  It turns out that, in every case, one of the two variables is computed at each end of the bond.  We can denote the side, where the flow variable is computed, by marking it with a vertical stroke, the so-called <i>causality stroke</i>. <p>

To this end, the bond graph library offers two additional types of bonds: <p>

<img src=\"../Images/BG_Fig13.png\"> <p>

In this case, the equations can be read out of the bond graph in their causal forms: <p>

<b>B1.e = B2.e</b> <br>
<b>B3.e = B2.e</b> <br>
<b>B2.f = B1.f - B3.f</b> <p>

for the 0-junction, and: <p>

<b>B4.f = B5.f</b> <br>
<b>B6.f = B5.f</b> <br>
<b>B5.e = B4.e - B6.e</b> <p>

for the 1-junction, respectively.  Since the 0-junction has only one flow equation, there must be exactly one causality stroke at the junction.  Similarly, since the 1-junction has only one effort equation, there must be exactly one causality stroke away from the junction. <p>

Causal and a-causal bonds can be arbitrarily mixed.  We recommend to use causal bonds as much as possible.
</html>
"));
      end Bonds;
      
      class Basic "Basic bond graph elements" 
        annotation (Documentation(info="<html>
The four basic bond graph variables can be related in the following way: <p>

<img src=\"../Images/BG_Fig3.png\"> <p>

<i>Resistors</i> relate efforts and flows to each other in a static fashion, i.e.: <p>

<b>e = R(f)</b> <p>

Similarly, <i>capacitors</i> relate efforts and generalized positions to each other, i.e.: <p>

<b>e = C(q)</b> <p>

Finally, <i>inductors</i> relate flows and generalized momentums to each other, i.e.: <p>

<b>f = I(p)</b> <p>

All of these relationships must be defined in the first and thrid quadrants only for the element to be physical. There cannot exist any other relationships in physics; thus, all bond graphs can be written in terms of the three variables <b>R</b>, <b>C</b>, and <b>I</b>. <p>

In addition, energy can be transformed, either from one domain to another, or within a domain.  The transformation of free energy can follow two different patterns: <p>

<b>e1 = m*e2</b> <br>
<b>e1*f1 = e2*f2</b> <p>

or: <p>

<b>e1 = r*f2</b> <br>
<b>e1*f1 = e2*f2</b> <p>

We call the former relationship a <i>transformer</i>, <b>TF</b>, whereas the latter relationship is called a <i>gyrator</i>, <b>GY</i>.  The second equation in both cases ensures that no power is either lost or generated by the transformation. <p>

In their causal forms, transformers and gyrators can be written as follows: <p>

<img src=\"../Images/BG_Fig14.png\"> <p>

Thus, both the transformers and the gyrators have two possible causalities.  <i>Transformers</i> have always one causality stroke at the transformer and the other away from it, whereas <i>gyrators</i> have either both causality strokes at the gyrator, or both away from it. <p>

Finally, we need two types of sources, the effort source, <b>Se</b>, and the flow source, <b>Sf</b>: <p>

<img src=\"../Images/BG_Fig15.png\"> <p>

The causality strokes of the two sources are fixed. <p>

As we haven't discussed the causality strokes for the three basic modeling elements yet, let us cover that topic now: <p>

<img src=\"../Images/BG_Fig16.png\"> <p>

Both capacitors and inductors have <i>preferred causalities</i>, so-called <i>integral causalities</i>, whereas the causality of the resistors is free. <p>

That is all there is.  These elements exhaust all of physics, and puritans among the bond graph practitioners would argue that no other elements should be offered. <p>

If the preferred causality of a bond graph can be assigned in a unique way, the bond graph has neither algebraic loops nor structural singularities, i.e., can be described by an index-0 ordinary differential equation (ODE) system.  If the modeler has free choice in assigning causality strokes, the model contains one or several algebraic loops.  If a conflict exists on either capacitors or inductors, the model consists of a higher-index DAE system.
</html>
"));
      end Basic;
      
      class Advanced "Advanced bond graph elements" 
        annotation (Documentation(info="<html>
For practical purposes, it makes sense to add more elements to the list. <p>

On the one hand, we may wish to offer different types of sources, such as sinusoidal and trapezoidal sources, to make the tool more practical for simulation.  Similarly, we may wish to offer different types of non-linear resistors, e.g. for hydraulic systems or thermal systems.  These are not really new elements, only specializations of the elements introduced previously. <p>

It may also make sense to offer sensor (detector) elements.  We may wish to sense any one of the four basic variables.  An effort detector, <b>De</b>, receives the effort from the model.  It computes the flow, by setting it to zero.  Thus, an effort sensor is in fact the same as a zero flow source: <p>

<img src=\"../Images/BG_Fig17.png\"> <p>

with the model: <p>

<img src=\"../Images/BG_Fig18.png\"> <p>

A position sensor: <p>

<img src=\"../Images/BG_Fig19.png\"> <p>

on the other hand, has the model: <p>

<img src=\"../Images/BG_Fig20.png\"> <p>

i.e., a position sensor is a macro element that is itself described by a bond graph.  A position sensor is a flow sensor followed by an integrator. <p>

Finally, we ought to discuss the switching elements.  The bond graph switch: <p>

<img src=\"../Images/BG_Fig21.png\"> <p>

has a Boolean input signal using the convention that, if the input signal has a value of <i>true</i>, the switch is <i>open</i>, i.e., there is zero flow.  On the other hand, if the Boolean input signal assumes a value of <i>false</i>, the switch is <i>closed</i>, i.e., there is zero effort. <p>

Switch elements are strange, because they change the location of their causality stroke as a function of the value of the Boolean input signal.  Thus, switch elements must always be attached to a-causal bonds. <p>

Furthermore, if the causality of the bond connecting to the switch is fixed from the outside, an ideal switch will lead to a division by zero in one of the two switch positions.  Thus, ideal switch elements must always be placed inside algebraic loops. <p>

To avoid this pitfall, the library offers also a leaking switch element, <b>Sw2</i>, that contains a parasitic resistor in the closed state, and a parasitic conductance in the open state. <p>

The library also offers special switching elements, such as electrical diodes.
</html>
"));
      end Advanced;
      
      class Wrapping "Wrapping bond graphs" 
        annotation (Documentation(info="<html>
Although bond graphs can be used to describe arbitrarily complex physical systems, they offer a fairly low-level interface and are therefore not always easily readable.  Furthermore, different modelers may prefer different modeling formalisms, because they are more familiar with them. <p>

Since bond graphs offer a low-level interface, it is always possible to wrap bond graphs inside other modeling metaphors.  For example, the electrical resistor model: <p>

<img src=\"../Images/BG_Fig22.png\"> <p>

can easily be reimplemented as follows: <p>

<img src=\"../Images/BG_Fig23.png\"> <p>

The <b>El2BG</b> and <b>BG2El</b> blocks are wrappers that convert the electrical connectors to bond graph connectors, and vice versa. <p>

<font color=red><b>BondLib</b></font> features wrapped <font color=blue><b>electrical (analog)</b></font>, <font color=blue><b>mechanical 1D (translational, rotational)</b></font>, and <font color=blue><b>thermal 1D (heat transfer)</b></font> libraries that are analogous to the corresponding sub-libraries of the <font color=red><b>Modelica standard library</b></font>. The wrapped electrical analog library contains a complete wrapped <font color=blue><b>Spice implementation</b></font> for DC analysis, AC analysis, transient analysis, and thermal analysis. <p>

Wrapped <font color=blue><b>mechanical 2D and 3D libraries</b></font> are also available,  but they were included in the <font color=red><b>MultiBondLib</b></font> library, as they are based on multi-bond graphs rather than on regular bond graphs. <p>

The wrapped (internal) bond graph variables are not visible from the outside.  They were <b>protected</b> in order to make the wrapped models look externally more similar to the corresponding models of the <font color=red><b>Modelica standard library</b></font>. <p>

Also in the case of capacitors and inductors, Modelica's <b>stateSelect</b> feature has been used to make the wrapped models use the same state variables as the corresponding non-wrapped models of the <font color=red><b>Modelica standard library</b></font>, i.e. <b>C.v</b> instead of <b>C.C1.e</b> in the case of a capacitor. <p>

A Boolean flag <b>enforceStates</b> is by default set to <b>true</b>, but can be set to <b>false</b> if the internal bond graph variables are to be used as state variables.
</html>
"));
      end Wrapping;
      
      class Electrical "The electrical library" 
        annotation (Documentation(info="<html>
We made use of the wrapping technique to re-implement large portions of the standard electrical analog library of Modelica using bond graph technology.  These models have the same touch and feel as the corresponding models contained in the standard library.  They are interchangeable and combinable in an arbitrary fashion. <p>

The new electrical analog library is a subset of the standard library, because not all elements have been recoded yet.  However, it is also a superset, because additional elements have been provided in some places. <p>

Especially, the new library contains <font color=red size=5><b><i>ModelicaSpice</i></b></font>, a Spice-style implementation of semiconductor models, including the four classes of bipolar transistors (the laterally and vertically diffused NPN and PNP transistors with all of the technological parameters that Spice offers), the two classes of MOSFETs (NMOS and PMOS), the two classes of JFETs (NJF and PJF), and an accurate model of diodes.  Just as in Spice, each of these models contains several alternative descriptions that can be selected by choosing the appropriate modeling level.
</html>
"));
      end Electrical;
      
      class Mechanical "The mechanical libraries" 
        
        annotation (Documentation(info="<html>
We made use of the wrapping technique to re-implement large portions of the translational and rotational 1D mechanical libraries of the Modelica standard library using bond graph technology.  These models have the same touch and feel as the corresponding models contained in the standard library.  However, they are not combinable with the corresponding models of the Modelica standard library, because the translational and rotational connectors of the wrapped libraries are different from the corresponding connectors of the standard library. Since bond graphs operate on forces (torques) and velocities (angular velocities), it was important to include velocity information in the connectors, whereas the corresponding connectors of the standard library carry only positional information.
</html>"));
      end Mechanical;
      
      class Thermal "The thermal library" 
        
        annotation (Documentation(info="<html>
We made use of the wrapping technique to re-implement large portions of the heat transfer library of the Modelica standard library using bond graph technology.  These models have the same touch and feel as the corresponding models contained in the standard library.  They are interchangeable and combinable in an arbitrary fashion. <p>

Not all models of the wrapped library are identical to the corresponding models of the standard library.  Especially the <b>conductance</b> and <b>convection</b> models behave differently, because the additional generated heat (entropy) by these resistors has been taken into account in the wrapped library, whereas it is ignored in the corresponding models of the standard library.
</html>"));
      end Thermal;
      
      class Encapsulating "Encapsulating bond graphs" 
        annotation (Documentation(info="<html>
<i>Encapsulation</i> of bond graph models is a technique similar to <i>wrapping</i>, however, it doesn't require special wrapping models.  The bond graph methodology already offers standard interfaces to mathematical signals.  On the one hand, there are the modulated sources, <b>mSe</b> and <b>mSf</b>, which convert input signals to bond graph signals.  On the other hand, there are the four detector elements, <b>De</b>, <b>Df</b>, <b>Dp</b>, and <b>Dq</b>, that offer the inverse functionality by converting bond graph signals to mathematical signals. <p>

These elements can be used to encapsulate bond graphs inside block diagrams, which is particularly useful for the description of control systems, the individual components of which may still be composed using bond graph technology.
</html>"));
      end Encapsulating;
      
      class Hydraulic "The hydraulic library" 
        annotation (Documentation(info="<html>
The hydraulic library contains a few bond graph models of hydraulic systems, such as a hydraulic motor with a servo valve. <p>

This library was built using the encapsulation technique discussed earlier.  The hydraulic motor is described by a bond graph with two bond graph connectors at the input, and one signal connector at the output: <p>

<img src=\"../Images/BG_Fig24.png\"> <p>

The servo valve, on the other hand, is described by another bond graph with one signal connector at the input, and two bond graph connectors at the output: <p>

<img src=\"../Images/BG_Fig25.png\"> <p>

In this way, the two components can be connected in the form of a block diagram: <p>

<img src=\"../Images/BG_Fig26.png\"> <p>

where they can then be used as part of a control loop: <p>

<img src=\"../Images/BG_Fig27.png\">
</html>
"));
      end Hydraulic;
      
      class Examples "Examples" 
        annotation (Documentation(info="<html>
The library contains an interesting example section: <p>

<ol>
<li> <b>SimpleBondGraph</b> describes a simple bond graph of a small electrical circuit.  The development of this bond graph was explained in [3].
<li> <b>ElectricalCircuit</b> contains a model of the same circuit, this time using the wrapping technology and components of the new electrical analog library contained as part of the bond graph library.
<li><b>OperationalAmplifierCircuit</b> describes a commercial hifi operational amplifier built in bipolar technology with six NPN and six PNP transistors [1].
<li><b>HydraulicControl</b> contains a hydraulic motor control problem, also explained in [2,3], that explains the encapsulation technique.
<li><b>SolarHouse</b> shows the use of bond graphs for thermodynamic modeling.  The example contains a fairly elaborate model of a passive solar heating system implemented in an experimental house in Tucson [5,6].
<li><b>Biosphere </b> shows another example of thermodynamic modeling, describing a model of the thermodynamic budget of the Biosphere 2 structure, located outside Tucson, without air conditioning [3,4].
<li><b>ModelicaSpice</b> is a collection of examples demonstrating the use of <font color=red><b><i>ModelicaSpice</i></b></font> for electronic device modeling and simulation (transient analysis, DC analysis, and AC analysis).
</ol> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, ISBN: 0-387-97502-0, 755p.
<li> Cellier, F.E. and R.T. McBride (2003), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/icbgm03_bglib.pdf\">Object-oriented
     Modeling of Complex Physical Systems Using the Dymola Bond-graph Library</a>,\" <i>Proc. ICBGM'03,
     6<sup>th</sup> SCS Intl. Conf. on Bond Graph Modeling and Simulation</i>, Orlando, Florida, pp. 157-162.
<li> Cellier, F.E. and A. Nebot (2005), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/modelica2005_cellier.pdf\">The
     Modelica Bond Graph Library</a>,\" <i>Proc. 4<sup>th</sup> Modelica Conference</i>, Hamburg, Germany.
<li> Nebot, A., F.E. Cellier, and F. Mugica (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/ecoeng_bio2.pdf\">Simulation
     of Heat and Humidity Budget of Biosphere 2 without its Air Conditioning</a>,\" <i>Ecological Engineering</i>,
     <b>13</b>, pp. 333-356.
<li> Weiner, M. (1992), <i>Bond Graph Model of a Passive Solar Heating System</i>, MS Thesis, Dept. of Electr. &amp;
     Comp. Engr., University of Arizona, Tucson, Arizona.
<li> Weiner, M., and F.E. Cellier (1993), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/weiner.pdf\">Modeling
     and Simulation of a Solar Energy System by Use of Bond Graphs</a>,\" <i>Proc. 1<sup>st</sup> SCS Intl. Conf. on Bond Graph
     Modeling</i>, San Diego, California, pp.301-306.
</ol>
</html>
"));
      end Examples;
      
      class Literature "Literature" 
        annotation (Documentation(info="<html>
We offer a fairly substantial bibliography of bond graph modeling in the \"<a href=\"http://www.inf.ethz.ch/~fcellier/BondGraphs/bg.html\">Bond Graph Compendium</a>.\"
</html>
"));
      end Literature;
    end Overview;
    
    package Versions "Release Notes" 
      
      annotation (Documentation(info="<html>
This section explains the major improvements made to the library from one version to the next: <p>

<ol>
<li><a href=\"Modelica://BondLib.UsersGuide.Versions.Version_1_0\">old -> Version 1.0</a>
    (February 10, 2005)

<li><a href=\"Modelica://BondLib.UsersGuide.Versions.Version_1_1\">Version 1.0 -> Version 1.1</a>
    (June 21, 2005)

<li><a href=\"Modelica://BondLib.UsersGuide.Versions.Version_1_2\">Version 1.1 -> Version 1.2</a>
    (November 21, 2005)

<li><a href=\"Modelica://BondLib.UsersGuide.Versions.Version_1_3\">Version 1.2 -> Version 1.3</a>
    (December 13, 2006)

<li><a href=\"Modelica://BondLib.UsersGuide.Versions.Version_2_0\">Version 1.3 -> Version 2.0</a>
    (July 18, 2007)

<li><a href=\"Modelica://BondLib.UsersGuide.Versions.Version_2_1\">Version 2.0 -> Version 2.1</a>
    (August 2, 2007)
</ol>
</html>"));
      class Version_1_0 "old -> Version 1.0 (February 10, 2005)" 
        
        annotation (Documentation(info="<html>
<font color=red><b>BondLib 1.0</b></font> constitutes the first official release of BondLib,
a library that has been around in one form or another since 1987.  Whereas the original version
of BondLib (based on the original Dymola syntax) had been purely alphanumeric, a first
graphical version of BondLib, based on Dymola 4.0, was created in 2001.  However, that
version hadn't been documented yet and was strictly meant to be used by my students. <p>

<ul>
<li><font color=red><b>Conversion from Dymola 4.0:</b></font> The library has been
    upgraded to the new graphical interface of Dymola 5.0.</li>

<li><font color=red><b>Documentation:</b></font> Appropriate documentation has been
    added to all modules.</li>

<li><font color=red><b>Electrical library:</b></font> An electrical analog library,
    similar in scope to the corresponding sublibrary of the Modelica standard library
    but based on an internal (wrapped) bond graph representation has been added.</li>

<li><font color=red><b>Examples:</b></font> A set of examples has been added to the
    library for improved documentation.</li>

</ol>
</html>"));
      end Version_1_0;
      
      class Version_1_1 "Version 1.0 -> Version 1.1 (June 21, 2005)" 
        
        annotation (Documentation(info="<html>
The following improvements were made between Version 1.0 and Version 1.1: <p>

<ol>
<li><font color=red><b>Units:</b></font> Proper measurement units were added to
    the modules of the electrical library and to all examples.  The basic
    bond graph methodology is, of course, application independent, and
    therefore, its modules continue to use generic units (\"Real\").</li> <p>

<li><font color=red><b>User's Guide:</b></font> A user's guide has been
    added introducing the bond graph methodology.</li>
</ol>
</html>"));
      end Version_1_1;
      
      class Version_1_2 "Version 1.1 -> Version 1.2 (November 21, 2005)" 
        
        annotation (Documentation(info="<html>
The following improvements were made between Version 1.1 and Version 1.2: <p>

<ol>
<li><font color=red><b>ModelicaSpice:</b></font> A new sublibrary has been
    added to the electrical analog sublibrary of BondLib implementing a
    full-fledged version of HSpice for DC analysis, AC analysis, and
    transient analysis.</li>
</ol>
</html>"));
      end Version_1_2;
      
      class Version_1_3 "Version 1.2 -> Version 1.3 (December 13, 2006)" 
        
        annotation (Documentation(info="<html>
The following improvements were made between Version 1.2 and Version 1.3: <p>

<ol>
<li><font color=red><b>Sensor models:</b></font> The <b>Dp</b> and <b>Dq</b>
    models were enhanced by the possibility of specifying non-zero initial
    conditions.</li> <p>

<li><font color=red><b>Electrical library:</b></font> The internal variables
    and models of wrapped models were <b>protected</b> to make them invisible
    from the outside.  In this way, the models of the standard library and those
    of the wrapped bond graph library look much more similar to each other in the
    <b>Simulation window</b>, and the simulation doesn't store unnecessarily
    internal variables that are of no interest to the modeler. Dymola's
    <b>stateSelect algorithm</b> is being used to enforce that the electrical voltages
    across capacitors [and the electrical currents through inductors] are being used as
    state variables instead of the (internal) effort variables of the underlying bond
    graph C elements [and the flow variables of the bond graph I elements].</li>
</ol>
</html>"));
      end Version_1_3;
      
      class Version_2_0 "Version 1.3 -> Version 2.0 (July 18, 2007)" 
        
        annotation (Documentation(info="<html>
The following improvements were made between Version 1.3 and Version 2.0: <p>

<ol>
<li><font color=red><b>Mechanical 1D libraries:</b></font> Two mechanical
    1D libraries for modeling translational and rotational motions were added.
    Although these libraries are similar to the corresponding libraries of the
    Modelica standard library, their connectors are incompatible due to the
    underlying bond graph technology being used.  Rather than only transferring
    positional and force (torque) information, the connectors of the new
    libraries also carry velocity information. Consequently, the models of the
    new libraries cannot be mixed with the models of the corresponding libraries
    that form part of the Modelica standard library.</li> <p>

<li><font color=red><b>Spice MOSFET models:</b></font> The MOSFET models of the
    SPICE library (part of the electrical analog library) still contained errors
    that have meanwhile been corrected by Alfonso Urqua and Carla Martn of the
    Universidad de Educacin a Distancia in Madrid.</li>
</ol>
</html>"));
      end Version_2_0;
      
      class Version_2_1 "Version 2.0 -> Version 2.1 (August 2, 2007)" 
        
        annotation (Documentation(info="<html>
The following improvements were made between Version 2.0 and Version 2.1: <p>

<ol>
<li><font color=red><b>HeatTransfer library:</b></font> A heat transfer library
    based on bond graph technology was added.  Its connectors are compatible
    with the connectors of the corresponding heat transfer library of the Modelica
    standard library. Consequently, the models of the new library can be
    arbitrarily mixed with the models of the corresponding library that
    forms part of the Modelica standard library.</li> <p>

<li><font color=red><b>Electrical analog library:</b></font> The heat port
    and substrate port of the electrical analog library have been enhanced
    to allow them to remain unconnected from the outside.  An unconnected
    heat port corresponds to the assumption of a constant operating
    temperature of Tnom.  An unconnected substrate port of a bipolar transistor
    corresponds to the assumption of an internal connection between the substrate
    and either the collector or the emitter, depending on whether the transistor
    is laterally or vertically diffused.</li>
</ol>
</html>"));
      end Version_2_1;
    end Versions;
  end UsersGuide;
  
  package Interfaces 
    "Connectors and partial models for Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(
            color=58,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
            color=58,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
            color=58,
            fillColor=30,
            fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=58)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0.02,
        y=0.53,
        width=0.4,
        height=0.4,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package contains the three basic bond graph connectors as well as some partial models that invoke them. <p>

The <b>BondCon</b> connector is the basic bi-directional bondgraphic connector that is used always.  The <b>eBondCon</b> and <b>fBondCon</b> connectors are uni-directional connectors that are only used in the definitions of the two causal bonds.
</html>
"));
    connector BondCon "Bi-directional bondgraphic connector" 
      Real e "Bondgraphic effort variable";
      Real f "Bondgraphic flow variable";
      Real d "Directional variable";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Ellipse(extent=[-60, 60; 60, -60], style(color=8, fillColor=8))),
        Icon(Ellipse(extent=[-60, 60; 60, -60], style(color=8, fillColor=8))),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
This is the basic bi-directional bondgraphic connector that is used everywhere, except in the definitions of the two causal bonds. <p>

The bondgraphic connector references three variables, two of which are physical variables, whereas the third one is a topological variable.  All three are defined as potential variables. <p>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable

 d:  Directional variable indicating the direction of positive power flow
</pre>
</html>
"));
    end BondCon;
    
    connector eBondCon "Uni-directional bondgraphic connector" 
      input Real e "Bondgraphic effort variable";
      output Real f "Bondgraphic flow variable";
      output Real d "Direction variable";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Ellipse(extent=[-60, 60; 60, -60], style(color=8, fillColor=8)),
              Text(
            extent=[-34, 34; 38, -28],
            string="e",
            style(color=41, fillColor=41))),
        Icon(Ellipse(extent=[-60, 60; 60, -60], style(color=8, fillColor=8)),
            Text(
            extent=[-34, 34; 38, -28],
            string="e",
            style(color=41, fillColor=41))),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
This is one of two uni-directional bondgraphic connectors used in the definitions of the two causal bonds. <p>

The bondgraphic connectors reference three variables, two of which are physical variables, whereas the third one is a topological variable.  All three are defined as potential variables. <p>

<pre>
Potential variables:

 e:  Bondgraphic effort variable (input)

 f:  Bondgraphic flow variable (output)

 d:  Directional variable indicating the direction of positive power flow (output)
</pre>
</html>
"));
    end eBondCon;
    
    connector fBondCon "Uni-directional bondgraphic connector" 
      output Real e "Bondgraphic effort variable";
      input Real f "Bondgraphic flow variable";
      output Real d "Direction variable";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Ellipse(extent=[-60, 60; 60, -60], style(color=8, fillColor=8)),
              Text(
            extent=[-36, 38; 36, -34],
            string="f",
            style(color=41, fillColor=41))),
        Icon(Ellipse(extent=[-60, 60; 60, -60], style(color=8, fillColor=8)),
            Text(
            extent=[-36, 38; 36, -34],
            string="f",
            style(color=41, fillColor=41))),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
This is one of two uni-directional bondgraphic connectors used in the definitions of the two causal bonds. <p>

The bondgraphic connectors reference three variables, two of which are physical variables, whereas the third one is a topological variable.  All three are defined as potential variables. <p>

<pre>
Potential variables:

 e:  Bondgraphic effort variable (output)

 f:  Bondgraphic flow variable (input)

 d:  Directional variable indicating the direction of positive power flow (output)
</pre>
</html>
"));
    end fBondCon;
    
    partial model PassiveOnePort 
      "Partial model invoking one bondgraphic connector" 
      Real e "Bondgraphic effort";
      Real f "Bondgraphic flow";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram,
        Icon,
        Documentation(info="<html>
This is a partial model invoking a single bondgraphic connector.  It is used to describe passive OnePort elements.<p>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model
</pre>
</html>
"));
      BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
    equation 
      e = BondCon1.e;
      f = BondCon1.d*BondCon1.f;
    end PassiveOnePort;
    
    partial model ModPassiveOnePort 
      "Partial model invoking one bondgraphic connector and one signal connector" 
      
      Real e "Bondgraphic effort";
      Real f "Bondgraphic flow";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
This is a partial model invoking a single bondgraphic connector and a signal connector.  It is used to describe passive modulated OnePort elements.<p>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 s:  Real-valued signal input used for modulation
</pre>
</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      Modelica.Blocks.Interfaces.RealInput s "Signal input" 
                                                annotation (extent=[100, -10;
            80, 10]);
    equation 
      e = BondCon1.e;
      f = BondCon1.d*BondCon1.f;
    end ModPassiveOnePort;
    
    partial model ActiveOnePort 
      "Partial model invoking one bondgraphic connector" 
      Real e "Bondgraphic effort";
      Real f "Bondgraphic flow";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram,
        Icon,
        Documentation(info="<html>
This is a partial model invoking a single bondgraphic connector.  It is used to describe active OnePort elements.<p>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows out of the model
</pre>
</html>
"));
      BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
    equation 
      e = BondCon1.e;
      f = -BondCon1.d*BondCon1.f;
    end ActiveOnePort;
    
    partial model ModActiveOnePort 
      Real e "Bondgraphic effort";
      Real f "Bondgraphic flow";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Diagram);
      BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      Modelica.Blocks.Interfaces.RealInput s "Signal input" 
                                                annotation (extent=[100, -10;
            80, 10]);
    equation 
      e = BondCon1.e;
      f = -BondCon1.d*BondCon1.f;
    end ModActiveOnePort;
    
    partial model TwoPort "Partial model invoking two bondgraphic connectors" 
      Real e1 "Bondgraphic primary effort";
      Real f1 "Bondgraphic primary flow";
      Real e2 "Bondgraphic secondary effort";
      Real f2 "Bondgraphic secondary flow";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram,
        Icon(Line(points=[42, -44; 68, -50; 42, -56], style(color=0)), Line(
              points=[-70, -50; 68, -50], style(color=0))),
        Documentation(info="<html>
This is a partial model invoking two bondgraphic connectors.  It is used to describe directed TwoPort elements.  Positive flow is from inflow to outflow.  The direction of positive flow is indicated by an arrow. <p>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model
</pre>

</html>
"));
      BondLib.Interfaces.BondCon BondCon1 "Left bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Right bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
    equation 
      e1 = BondCon1.e;
      f1 = BondCon1.d*BondCon1.f;
      e2 = BondCon2.e;
      f2 = -BondCon2.d*BondCon2.f;
    end TwoPort;
    
    partial model TwoPort2 "Partial model invoking two bondgraphic connectors" 
      Real e1 "Bondgraphic primary effort";
      Real f1 "Bondgraphic primary flow";
      Real e2 "Bondgraphic secondary effort";
      Real f2 "Bondgraphic secondary flow";
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
    equation 
      e1 = BondCon1.e;
      f1 = BondCon1.d*BondCon1.f;
      e2 = BondCon2.e;
      f2 = BondCon2.d*BondCon2.f;
      annotation (Documentation(info="<html>
This is a partial model invoking two bondgraphic connectors.  It is used to describe symmetric TwoPort elements.  Both inflow and outflow are assumed positive into the model. <p>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows into the model
</pre>

</html>
"), Diagram);
    end TwoPort2;
    
    partial model TwoPortZero 
      "Partial model invoking two bondgraphic connectors" 
      Real e1 "Bondgraphic primary effort";
      Real f1 "Bondgraphic primary flow";
      Real e2 "Bondgraphic secondary effort";
      Real f2 "Bondgraphic secondary flow";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking two bondgraphic connectors.  It is used only for the description of the TwoPort 0-junction. <p>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of first connector

 f1:  Bondgraphic flow variable of first connector, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of second connector

 f2:  Bondgraphic flow variable of second connector, normalized positive for flows into the model
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
    equation 
      e1 = BondCon1.e;
      f1 = BondCon1.d*BondCon1.f;
      e2 = BondCon2.e;
      f2 = BondCon2.d*BondCon2.f;
    end TwoPortZero;
    
    partial model TwoPortOne 
      "Partial model invoking two bondgraphic connectors" 
      Real e1 "Bondgraphic primary effort";
      Real f1 "Bondgraphic primary flow";
      Real e2 "Bondgraphic secondary effort";
      Real f2 "Bondgraphic secondary flow";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking two bondgraphic connectors.  It is used only for the description of the TwoPort 1-junction. <p>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of first connector, normalized positive for flows into the model

 f1:  Bondgraphic flow variable of first connector

 e2:  Bondgraphic effort variable of second connector, normalized positive for flows into the model

 f2:  Bondgraphic flow variable of second connector
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
    equation 
      e1 = BondCon1.d*BondCon1.e;
      f1 = BondCon1.f;
      e2 = BondCon2.d*BondCon2.e;
      f2 = BondCon2.f;
    end TwoPortOne;
    
    partial model ModTwoPort 
      "Partial model invoking two bondgraphic connectors and one signal connector" 
      
      Real e1 "Bondgraphic primary effort";
      Real f1 "Bondgraphic primary flow";
      Real e2 "Bondgraphic secondary effort";
      Real f2 "Bondgraphic secondary flow";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Diagram,
        Icon(Line(points=[-70, -50; 68, -50], style(color=0)), Line(points=[42,
                  -44; 68, -50; 42, -56], style(color=0))),
        Documentation(info="<html>
This is a partial model invoking two bondgraphic connectors and a signal connector.  It is used to describe modulated directed TwoPort elements.  Positive flow is from inflow to outflow.  The direction of positive flow is indicated by an arrow. <p>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 s:   Real-valued signal input used for modulation
</pre>

</html>
"));
      BondLib.Interfaces.BondCon BondCon1 "Left bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Right bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      Modelica.Blocks.Interfaces.RealInput s "Signal input" 
                                                annotation (extent=[10, -100; -
            10, -80], rotation=90);
    equation 
      e1 = BondCon1.e;
      f1 = BondCon1.d*BondCon1.f;
      e2 = BondCon2.e;
      f2 = -BondCon2.d*BondCon2.f;
    end ModTwoPort;
    
    partial model ThreePortZero 
      "Partial model invoking three bondgraphic connectors" 
      Real e[3] "Bondgraphic effort vector";
      Real f[3] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking three bondgraphic connectors.  It is used only for the description of the ThreePort 0-junction. <p>

<pre>
Potential variables:

 e[3]:  Vector of bondgraphic effort variables of connectors

 f[3]:  Vector of bondgraphic flow variables of connectors, normalized positive for flows into the model
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-10, -110; 10, -
            90]);
    equation 
      e[1] = BondCon1.e;
      f[1] = BondCon1.d*BondCon1.f;
      e[2] = BondCon2.e;
      f[2] = BondCon2.d*BondCon2.f;
      e[3] = BondCon3.e;
      f[3] = BondCon3.d*BondCon3.f;
    end ThreePortZero;
    
    partial model ThreePortOne 
      "Partial model invoking three bondgraphic connectors" 
      Real e[3] "Bondgraphic effort vector";
      Real f[3] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking three bondgraphic connectors.  It is used only for the description of the ThreePort 1-junction. <p>

<pre>
Potential variables:

 e[3]:  Vector of bondgraphic effort variables of connectors, normalized positive for flows into the model

 f[3]:  Vector of bondgraphic flow variables of connectors
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-10, -110; 10, -
            90]);
    equation 
      e[1] = BondCon1.d*BondCon1.e;
      f[1] = BondCon1.f;
      e[2] = BondCon2.d*BondCon2.e;
      f[2] = BondCon2.f;
      e[3] = BondCon3.d*BondCon3.e;
      f[3] = BondCon3.f;
    end ThreePortOne;
    
    partial model FourPortZero 
      "Partial model invoking four bondgraphic connectors" 
      Real e[4] "Bondgraphic effort vector";
      Real f[4] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking four bondgraphic connectors.  It is used only for the description of the FourPort 0-junction. <p>

<pre>
Potential variables:

 e[4]:  Vector of bondgraphic effort variables of connectors

 f[4]:  Vector of bondgraphic flow variables of connectors, normalized positive for flows into the model
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-10, -110; 10, -
            90]);
      BondLib.Interfaces.BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[-10, 90; 10, 110]);
    equation 
      e[1] = BondCon1.e;
      f[1] = BondCon1.d*BondCon1.f;
      e[2] = BondCon2.e;
      f[2] = BondCon2.d*BondCon2.f;
      e[3] = BondCon3.e;
      f[3] = BondCon3.d*BondCon3.f;
      e[4] = BondCon4.e;
      f[4] = BondCon4.d*BondCon4.f;
    end FourPortZero;
    
    partial model FourPortZero2 
      "Partial model invoking up to four bondgraphic connectors" 
      Real e[4] "Bondgraphic effort vector";
      Real f[4] "Bondgraphic flow vector";
      BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110,-10; -90,10]);
      BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90,-10; 110,10]);
      BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-10,-110; 10,-90]);
      BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[-10,90; 10,110]);
    equation 
      e[1] = BondCon1.e;
      f[1] = BondCon1.d*BondCon1.f;
      e[2] = BondCon2.e;
      f[2] = BondCon2.d*BondCon2.f;
      e[3] = BondCon3.e;
      f[3] = BondCon3.d*BondCon3.f;
      e[4] = BondCon4.e;
      f[4] = BondCon4.d*BondCon4.f;
      
      if cardinality(BondCon1) == 0 then
        BondCon1.d = 1;
        f[1] = 0;
      end if;
      if cardinality(BondCon2) == 0 then
        BondCon2.d = 1;
        f[2] = 0;
      end if;
      if cardinality(BondCon3) == 0 then
        BondCon3.d = 1;
        f[3] = 0;
      end if;
      if cardinality(BondCon4) == 0 then
        BondCon4.d = 1;
        f[4] = 0;
      end if;
      annotation (Documentation(info="<html>
This is a partial model invoking four bondgraphic connectors.  It is used only for the description of a 0-junction with up to four bonds connecting to it. <p>

<pre>
Potential variables:

 e[4]:  Vector of bondgraphic effort variables of connectors

 f[4]:  Vector of bondgraphic flow variables of connectors, normalized positive for flows into the model
</pre>

</html>"), Diagram);
    end FourPortZero2;
    
    partial model FourPortOne 
      "Partial model invoking four bondgraphic connectors" 
      Real e[4] "Bondgraphic effort vector";
      Real f[4] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking four bondgraphic connectors.  It is used only for the description of the FourPort 1-junction. <p>

<pre>
Potential variables:

 e[4]:  Vector of bondgraphic effort variables of connectors, normalized positive for flows into the model

 f[4]:  Vector of bondgraphic flow variables of connectors
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-10, -110; 10, -
            90]);
      BondLib.Interfaces.BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[-10, 90; 10, 110]);
    equation 
      e[1] = BondCon1.d*BondCon1.e;
      f[1] = BondCon1.f;
      e[2] = BondCon2.d*BondCon2.e;
      f[2] = BondCon2.f;
      e[3] = BondCon3.d*BondCon3.e;
      f[3] = BondCon3.f;
      e[4] = BondCon4.d*BondCon4.e;
      f[4] = BondCon4.f;
    end FourPortOne;
    
    partial model FourPortOne2 
      "Partial model invoking up to four bondgraphic connectors" 
      Real e[4] "Bondgraphic effort vector";
      Real f[4] "Bondgraphic flow vector";
      BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110,-10; -90,10]);
      BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90,-10; 110,10]);
      BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-10,-110; 10,-90]);
      BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[-10,90; 10,110]);
    equation 
      e[1] = BondCon1.d*BondCon1.e;
      f[1] = BondCon1.f;
      e[2] = BondCon2.d*BondCon2.e;
      f[2] = BondCon2.f;
      e[3] = BondCon3.d*BondCon3.e;
      f[3] = BondCon3.f;
      e[4] = BondCon4.d*BondCon4.e;
      f[4] = BondCon4.f;
      
      if cardinality(BondCon1) == 0 then
        BondCon1.d = 1;
        e[1] = 0;
      end if;
      if cardinality(BondCon2) == 0 then
        BondCon2.d = 1;
        e[2] = 0;
      end if;
      if cardinality(BondCon3) == 0 then
        BondCon3.d = 1;
        e[3] = 0;
      end if;
      if cardinality(BondCon4) == 0 then
        BondCon4.d = 1;
        e[4] = 0;
      end if;
      annotation (Documentation(info="<html>
This is a partial model invoking four bondgraphic connectors.  It is used only for the description of a 1-junction with up to four bonds connecting to it. <p>

<pre>
Potential variables:

 e[4]:  Vector of bondgraphic effort variables of connectors, normalized positive for flows into the model

 f[4]:  Vector of bondgraphic flow variables of connectors
</pre>

</html>"), Diagram);
    end FourPortOne2;
    
    partial model FivePortZero 
      "Partial model invoking five bondgraphic connectors" 
      Real e[5] "Bondgraphic effort vector";
      Real f[5] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking five bondgraphic connectors.  It is used only for the description of the FivePort 0-junction. <p>

<pre>
Potential variables:

 e[5]:  Vector of bondgraphic effort variables of connectors

 f[5]:  Vector of bondgraphic flow variables of connectors, normalized positive for flows into the model
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-50, -110; -30, -
            90]);
      BondLib.Interfaces.BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[30, -110; 50, -90]);
      BondLib.Interfaces.BondCon BondCon5 "Fifth bond graph connector" 
                                          annotation (extent=[-10, 90; 10, 110]);
    equation 
      e[1] = BondCon1.e;
      f[1] = BondCon1.d*BondCon1.f;
      e[2] = BondCon2.e;
      f[2] = BondCon2.d*BondCon2.f;
      e[3] = BondCon3.e;
      f[3] = BondCon3.d*BondCon3.f;
      e[4] = BondCon4.e;
      f[4] = BondCon4.d*BondCon4.f;
      e[5] = BondCon5.e;
      f[5] = BondCon5.d*BondCon5.f;
    end FivePortZero;
    
    partial model FivePortOne 
      "Partial model invoking five bondgraphic connectors" 
      Real e[5] "Bondgraphic effort vector";
      Real f[5] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking five bondgraphic connectors.  It is used only for the description of the FivePort 1-junction. <p>

<pre>
Potential variables:

 e[5]:  Vector of bondgraphic effort variables of connectors, normalized positive for flows into the model

 f[5]:  Vector of bondgraphic flow variables of connectors
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-50, -110; -30, -
            90]);
      BondLib.Interfaces.BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[30, -110; 50, -90]);
      BondLib.Interfaces.BondCon BondCon5 "Fifth bond graph connector" 
                                          annotation (extent=[-10, 90; 10, 110]);
    equation 
      e[1] = BondCon1.d*BondCon1.e;
      f[1] = BondCon1.f;
      e[2] = BondCon2.d*BondCon2.e;
      f[2] = BondCon2.f;
      e[3] = BondCon3.d*BondCon3.e;
      f[3] = BondCon3.f;
      e[4] = BondCon4.d*BondCon4.e;
      f[4] = BondCon4.f;
      e[5] = BondCon5.d*BondCon5.e;
      f[5] = BondCon5.f;
    end FivePortOne;
    
    partial model SixPortZero 
      "Partial model invoking six bondgraphic connectors" 
      Real e[6] "Bondgraphic effort vector";
      Real f[6] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking six bondgraphic connectors.  It is used only for the description of the SixPort 0-junction. <p>

<pre>
Potential variables:

 e[6]:  Vector of bondgraphic effort variables of connectors

 f[6]:  Vector of bondgraphic flow variables of connectors, normalized positive for flows into the model
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-50, -110; -30, -
            90]);
      BondLib.Interfaces.BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[30, -110; 50, -90]);
      BondLib.Interfaces.BondCon BondCon5 "Fifth bond graph connector" 
                                          annotation (extent=[-50, 90; -30, 110]);
      BondLib.Interfaces.BondCon BondCon6 "Sixth bond graph connector" 
                                          annotation (extent=[30, 90; 50, 110]);
    equation 
      e[1] = BondCon1.e;
      f[1] = BondCon1.d*BondCon1.f;
      e[2] = BondCon2.e;
      f[2] = BondCon2.d*BondCon2.f;
      e[3] = BondCon3.e;
      f[3] = BondCon3.d*BondCon3.f;
      e[4] = BondCon4.e;
      f[4] = BondCon4.d*BondCon4.f;
      e[5] = BondCon5.e;
      f[5] = BondCon5.d*BondCon5.f;
      e[6] = BondCon6.e;
      f[6] = BondCon6.d*BondCon6.f;
    end SixPortZero;
    
    partial model SixPortOne 
      "Partial model invoking six bondgraphic connectors" 
      Real e[6] "Bondgraphic effort vector";
      Real f[6] "Bondgraphic flow vector";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Documentation(info="<html>
This is a partial model invoking six bondgraphic connectors.  It is used only for the description of the SixPort 1-junction. <p>

<pre>
Potential variables:

 e[6]:  Vector of bondgraphic effort variables of connectors, normalized positive for flows into the model

 f[6]:  Vector of bondgraphic flow variables of connectors
</pre>

</html>
"),     Diagram);
      BondLib.Interfaces.BondCon BondCon1 "First bond graph connector" 
                                          annotation (extent=[-110, -10; -90,
            10]);
      BondLib.Interfaces.BondCon BondCon2 "Second bond graph connector" 
                                          annotation (extent=[90, -10; 110, 10]);
      BondLib.Interfaces.BondCon BondCon3 "Third bond graph connector" 
                                          annotation (extent=[-50, -110; -30, -
            90]);
      BondLib.Interfaces.BondCon BondCon4 "Fourth bond graph connector" 
                                          annotation (extent=[30, -110; 50, -90]);
      BondLib.Interfaces.BondCon BondCon5 "Fifth bond graph connector" 
                                          annotation (extent=[-50, 90; -30, 110]);
      BondLib.Interfaces.BondCon BondCon6 "Sixth bond graph connector" 
                                          annotation (extent=[30, 90; 50, 110]);
    equation 
      e[1] = BondCon1.d*BondCon1.e;
      f[1] = BondCon1.f;
      e[2] = BondCon2.d*BondCon2.e;
      f[2] = BondCon2.f;
      e[3] = BondCon3.d*BondCon3.e;
      f[3] = BondCon3.f;
      e[4] = BondCon4.d*BondCon4.e;
      f[4] = BondCon4.f;
      e[5] = BondCon5.d*BondCon5.e;
      f[5] = BondCon5.f;
      e[6] = BondCon6.d*BondCon6.e;
      f[6] = BondCon6.f;
    end SixPortOne;
    
    partial model EightPortZero 
      "Partial model invoking up to eight bondgraphic connectors" 
      Real e[8] "Bondgraphic effort vector";
      Real f[8] "Bondgraphic flow vector";
      annotation (
        Coordsys(extent=[-200,-200; 200,200]),
        Diagram,
        Documentation(info="<html>
This is a partial model invoking eight bondgraphic connectors.  It is used only for the description of a 0-junction with up to eight bonds connecting to it. <p>

<pre>
Potential variables:

 e[8]:  Vector of bondgraphic effort variables of connectors

 f[8]:  Vector of bondgraphic flow variables of connectors, normalized positive for flows into the model
</pre>

</html>"));
      BondCon BondCon1 "First bond graph connector" 
                       annotation (extent=[-210,-10; -190,10]);
      BondCon BondCon2 "Second bond graph connector" 
                       annotation (extent=[-110,90; -90,110]);
      BondCon BondCon3 "Third bond graph connector" 
                       annotation (extent=[-10,190; 10,210]);
      BondCon BondCon4 "Fourth bond graph connector" 
                       annotation (extent=[90,90; 110,110]);
      BondCon BondCon5 "Fifth bond graph connector" 
                       annotation (extent=[190,-10; 210,10]);
      BondCon BondCon6 "Sixth bond graph connector" 
                       annotation (extent=[90,-110; 110,-90]);
      BondCon BondCon7 "Seventh bond graph connector" 
                       annotation (extent=[-10,-210; 10,-190]);
      BondCon BondCon8 "Eighth bond graph connector" 
                       annotation (extent=[-110,-110; -90,-90]);
    equation 
      e[1] = BondCon1.e;
      f[1] = BondCon1.d*BondCon1.f;
      e[2] = BondCon2.e;
      f[2] = BondCon2.d*BondCon2.f;
      e[3] = BondCon3.e;
      f[3] = BondCon3.d*BondCon3.f;
      e[4] = BondCon4.e;
      f[4] = BondCon4.d*BondCon4.f;
      e[5] = BondCon5.e;
      f[5] = BondCon5.d*BondCon5.f;
      e[6] = BondCon6.e;
      f[6] = BondCon6.d*BondCon6.f;
      e[7] = BondCon7.e;
      f[7] = BondCon7.d*BondCon7.f;
      e[8] = BondCon8.e;
      f[8] = BondCon8.d*BondCon8.f;
      
      if cardinality(BondCon1) == 0 then
        BondCon1.d = 1;
        f[1] = 0;
      end if;
      if cardinality(BondCon2) == 0 then
        BondCon2.d = 1;
        f[2] = 0;
      end if;
      if cardinality(BondCon3) == 0 then
        BondCon3.d = 1;
        f[3] = 0;
      end if;
      if cardinality(BondCon4) == 0 then
        BondCon4.d = 1;
        f[4] = 0;
      end if;
      if cardinality(BondCon5) == 0 then
        BondCon5.d = 1;
        f[5] = 0;
      end if;
      if cardinality(BondCon6) == 0 then
        BondCon6.d = 1;
        f[6] = 0;
      end if;
      if cardinality(BondCon7) == 0 then
        BondCon7.d = 1;
        f[7] = 0;
      end if;
      if cardinality(BondCon8) == 0 then
        BondCon8.d = 1;
        f[8] = 0;
      end if;
    end EightPortZero;
    
    partial model EightPortOne 
      "Partial model invoking up to eight bondgraphic connectors" 
      Real e[8] "Bondgraphic effort vector";
      Real f[8] "Bondgraphic flow vector";
      annotation (Coordsys(extent=[-200,-200; 200,200]), Documentation(info="<html>
This is a partial model invoking eight bondgraphic connectors.  It is used only for the description of a 1-junction with up to eight bonds connecting to it. <p>

<pre>
Potential variables:

 e[8]:  Vector of bondgraphic effort variables of connectors, normalized positive for flows into the model

 f[8]:  Vector of bondgraphic flow variables of connectors
</pre>

</html>"),
        Diagram);
      BondCon BondCon1 "First bond graph connector" 
                       annotation (extent=[-210,-10; -190,10]);
      BondCon BondCon2 "Second bond graph connector" 
                       annotation (extent=[-110,90; -90,110]);
      BondCon BondCon3 "Third bond graph connector" 
                       annotation (extent=[-10,190; 10,210]);
      BondCon BondCon4 "Fourth bond graph connector" 
                       annotation (extent=[90,90; 110,110]);
      BondCon BondCon5 "Fifth bond graph connector" 
                       annotation (extent=[190,-10; 210,10]);
      BondCon BondCon6 "Sixth bond graph connector" 
                       annotation (extent=[90,-110; 110,-90]);
      BondCon BondCon7 "Seventh bond graph connector" 
                       annotation (extent=[-10,-210; 10,-190]);
      BondCon BondCon8 "Eighth bond graph connector" 
                       annotation (extent=[-110,-110; -90,-90]);
    equation 
      e[1] = BondCon1.d*BondCon1.e;
      f[1] = BondCon1.f;
      e[2] = BondCon2.d*BondCon2.e;
      f[2] = BondCon2.f;
      e[3] = BondCon3.d*BondCon3.e;
      f[3] = BondCon3.f;
      e[4] = BondCon4.d*BondCon4.e;
      f[4] = BondCon4.f;
      e[5] = BondCon5.d*BondCon5.e;
      f[5] = BondCon5.f;
      e[6] = BondCon6.d*BondCon6.e;
      f[6] = BondCon6.f;
      e[7] = BondCon7.d*BondCon7.e;
      f[7] = BondCon7.f;
      e[8] = BondCon8.d*BondCon8.e;
      f[8] = BondCon8.f;
      
      if cardinality(BondCon1) == 0 then
        BondCon1.d = 1;
        e[1] = 0;
      end if;
      if cardinality(BondCon2) == 0 then
        BondCon2.d = 1;
        e[2] = 0;
      end if;
      if cardinality(BondCon3) == 0 then
        BondCon3.d = 1;
        e[3] = 0;
      end if;
      if cardinality(BondCon4) == 0 then
        BondCon4.d = 1;
        e[4] = 0;
      end if;
      if cardinality(BondCon5) == 0 then
        BondCon5.d = 1;
        e[5] = 0;
      end if;
      if cardinality(BondCon6) == 0 then
        BondCon6.d = 1;
        e[6] = 0;
      end if;
      if cardinality(BondCon7) == 0 then
        BondCon7.d = 1;
        e[7] = 0;
      end if;
      if cardinality(BondCon8) == 0 then
        BondCon8.d = 1;
        e[8] = 0;
      end if;
    end EightPortOne;
  end Interfaces;
  
  package Bonds "Bonds of Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30, fillPattern=
                1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
              fillColor=30, fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
              fillColor=30, fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=3)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0.55,
        y=0.46,
        width=0.35,
        height=0.49,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package contains models of the three bonds of the Modelica bond graph library.  The <b>Bond</b> model describes the a-causal bonds, whereas the <b>eBond</b> and <b>fBond</b> blocks describe the two causal bonds. <p>

Although it is possible to work with a-causal bonds throughout, it is recommended to use causal bonds whenever possible. <p>

Bonds must be connected to either elements or junctions.  It is illegal to connect two bonds in series.
</html>
"));
    model Bond "The a-causal bond model of the bond graph library" 
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-80, 0; 80, 0], style(color=0, thickness=2)),
          Line(points=[80, 0; 40, 40], style(color=0, thickness=2)),
          Text(
            extent=[-40, -8; 20, -40],
            string="f",
            style(color=0)),
          Text(
            extent=[-40, 40; 18, 10],
            string="e",
            style(color=0))),
        Icon(
          Line(points=[-80, 0; 80, 0], style(color=0, thickness=2)),
          Line(points=[80, 0; 40, 40], style(color=0, thickness=2)),
          Text(
            extent=[-70, -4; 72, -42],
            string="%name",
            style(color=0))),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
This is the model of the general a-causal bond of the Modelica bond graph library. <p>

The model invokes two general bond connectors.  It does so directly, i.e., without inheriting them from one of the TwoPort partial models. <p>

The Model passes the effort and flow variables from the inflow to the outflow, and it sets the directional variable <b>d</b> to <b>-1</b> at the inflow, and to <b>+1</b> at the outflow.
</html>
"));
      Interfaces.BondCon BondCon1 "Left bond graph connector" 
                                  annotation (extent=[-108, -10; -88, 10]);
      Interfaces.BondCon BondCon2 "Right bond graph connector" 
                                  annotation (extent=[90, -10; 110, 10]);
    equation 
      BondCon2.e = BondCon1.e;
      BondCon2.f = BondCon1.f;
      BondCon1.d = -1;
      BondCon2.d = +1;
    end Bond;
    
    block eBond "One of two causal bond models of the bond graph library" 
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Icon(
          Line(points=[-80, 0; 80, 0], style(color=0, thickness=2)),
          Line(points=[80, 0; 40, 40], style(color=0, thickness=2)),
          Line(points=[-80, 40; -80, -40], style(color=0, thickness=2)),
          Text(
            extent=[-68, -4; 64, -42],
            string="%name",
            style(color=0))),
        Diagram(
          Line(points=[-80, 0; 80, 0], style(color=0, thickness=2)),
          Line(points=[80, 0; 40, 40], style(color=0, thickness=2)),
          Line(points=[-80, 40; -80, -40], style(color=0, thickness=2)),
          Text(
            extent=[-38, 42; 20, 12],
            string="e",
            style(color=0)),
          Text(
            extent=[-40, -8; 20, -40],
            string="f",
            style(color=0)),
          Line(points=[4, -24; 34, -24], style(
              color=0,
              thickness=2,
              arrow=1)),
          Line(points=[-24, 24; -56, 24], style(
              color=0,
              thickness=2,
              arrow=1))),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
This is the model of one of two causal bonds of the Modelica bond graph library. <p>

The model invokes one <b>fBond</b> and one <b>eBond</b> connector.  The <b>fBond</b> connector is used at the inflow, i.e., where the flow variable is being computed, whereas the <b>eBond</b> connector is used at the outflow, i.e., where the effort variable is being computed. Thus, the causality stroke of this causal bond is at the inflow. <p>

The model passes the effort and flow variables from the inflow to the outflow, and it sets the directional variable <b>d</b> to <b>-1</b> at the inflow, and to <b>+1</b> at the outflow.
</html>
"));
      Interfaces.fBondCon fBondCon1 "Left bond graph connector" 
                                    annotation (extent=[-110, -10; -90, 10]);
      Interfaces.eBondCon eBondCon1 "Right bond graph connector" 
                                    annotation (extent=[90, -10; 110, 10]);
    equation 
      fBondCon1.e = eBondCon1.e;
      eBondCon1.f = fBondCon1.f;
      fBondCon1.d = -1;
      eBondCon1.d = +1;
    end eBond;
    
    block fBond "One of two causal bond models of the bond graph library" 
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-80, 0; 80, 0], style(color=0, thickness=2)),
          Text(
            extent=[-40, -8; 20, -40],
            string="f",
            style(color=0)),
          Text(
            extent=[-38, 42; 20, 12],
            string="e",
            style(color=0)),
          Line(points=[80, 0; 40, 40], style(color=0, thickness=2)),
          Line(points=[80, 40; 80, -40], style(color=0, thickness=2)),
          Line(points=[-22, -24; -54, -24], style(
              color=0,
              thickness=2,
              arrow=1)),
          Line(points=[2, 26; 32, 26], style(
              color=0,
              thickness=2,
              arrow=1))),
        Icon(
          Line(points=[-80, 0; 80, 0], style(color=0, thickness=2)),
          Line(points=[80, 0; 40, 40], style(color=0, thickness=2)),
          Line(points=[80, 40; 80, -40], style(color=0, thickness=2)),
          Text(
            extent=[-66, -4; 62, -44],
            string="%name",
            style(color=0))),
        Window(
          x=0.52,
          y=0.42,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
This is the model of one of two causal bonds of the Modelica bond graph library. <p>

The model invokes one <b>eBond</b> and one <b>fBond</b> connector.  The <b>eBond</b> connector is used at the inflow, i.e., where the effort variable is being computed, whereas the <b>fBond</b> connector is used at the outflow, i.e., where the flow variable is being computed. Thus, the causality stroke of this causal bond is at the outflow. <p>

The model passes the effort and flow variables from the inflow to the outflow, and it sets the directional variable <b>d</b> to <b>-1</b> at the inflow, and to <b>+1</b> at the outflow.
</html>
"));
      Interfaces.eBondCon eBondCon1 "Left bond graph connector" 
                                    annotation (extent=[-110, -10; -90, 10]);
      Interfaces.fBondCon fBondCon1 "Right bond graph connector" 
                                    annotation (extent=[90, -10; 110, 10]);
    equation 
      fBondCon1.e = eBondCon1.e;
      eBondCon1.f = fBondCon1.f;
      eBondCon1.d = -1;
      fBondCon1.d = +1;
    end fBond;
  end Bonds;
  
  package Junctions "Junctions of Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30, fillPattern=
                1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
              fillColor=30, fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
              fillColor=30, fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=3)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0.49,
        y=0.48,
        width=0.35,
        height=0.49,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package contains models of the different junctions of the Modelica bond graph library.  The library distinguishes explicitly between junctions with 2, 3, 4, 5, and 6 connectors.  If more than 6 connectors are needed, the junction needs to be split.  Since neighboring same-sex junctions can always be treated logically like a single junction, it is possible to construct junctions with arbitrarily many connectors.<p>

Junctions must be connected to bonds.  It is illegal to connect junctions to either modeling elements or other junctions directly.  If junctions are being split, a bond must be placed between them.
</html>
"));
    model J0p2 "Model of TwoPort 0-junction" 
      extends Interfaces.TwoPortZero;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Documentation(info="<html>
This is the model of the TwoPort 0-junction.  It has one causality stroke.
</html>
"));
    equation 
      e2 = e1;
      f1 + f2 = 0;
    end J0p2;
    
    model J0p3 "Model of ThreePort 0-junction" 
      extends Interfaces.ThreePortZero;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Documentation(info="<html>
This is the model of the ThreePort 0-junction.  It has one causality stroke.
</html>
"));
    equation 
      e[2:3] = e[1:2];
      sum(f) = 0;
    end J0p3;
    
    model J0p4 "Model of FourPort 0-junction" 
      extends Interfaces.FourPortZero;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Documentation(info="<html>
This is the model of the FourPort 0-junction.  It has one causality stroke.
</html>
"));
    equation 
      e[2:4] = e[1:3];
      sum(f) = 0;
    end J0p4;
    
    model J0p5 "Model of FivePort 0-junction" 
      extends Interfaces.FivePortZero;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Documentation(info="<html>
This is the model of the FivePort 0-junction.  It has one causality stroke.
</html>
"));
    equation 
      e[2:5] = e[1:4];
      sum(f) = 0;
    end J0p5;
    
    model J0p6 "Model of SixPort 0-junction" 
      extends Interfaces.SixPortZero;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="0",
            style(color=0))),
        Documentation(info="<html>
This is the model of the SixPort 0-junction.  It has one causality stroke.
</html>
"));
    equation 
      e[2:6] = e[1:5];
      sum(f) = 0;
    end J0p6;
    
    model J0 "Model of a standard 0-junction with up to four ports" 
      extends Interfaces.FourPortZero2;
      annotation (
        Diagram(Text(
            extent=[-60,80; 60,-80],
            string="0",
            style(color=0))),
        Icon(   Text(
            extent=[-60,80; 60,-80],
            string="0",
            style(color=0))),
        Documentation(info="<html>
This is the model of the generalized FourPort 0-junction.  It has one causality stroke. <p>

Contrary to the regular FourPoint 0-junction (<b>J0p4</b>), this model allows connectors to remain unconnected externally.  Hence the junction can be used for up to four bond connections.
</html>"));
    equation 
      e[2:4] = e[1:3];
      sum(f) = 0;
    end J0;
    
    model J0large "Model of a large 0-junction with up to eight ports" 
      extends Interfaces.EightPortZero;
      annotation (
        Icon(
          Line(points=[-102,0; -194,0], style(color=10, rgbcolor={135,135,135})),
          Line(points=[0,100; 0,194], style(color=10, rgbcolor={135,135,135})),
          Line(points=[104,0; 194,0], style(color=10, rgbcolor={135,135,135})),
          Line(points=[0,-104; 0,-194], style(color=10, rgbcolor={135,135,135})),
          Line(points=[-70,70; -96,96], style(color=10, rgbcolor={135,135,135})),
          Line(points=[70,70; 96,96], style(color=10, rgbcolor={135,135,135})),
          Line(points=[70,-70; 96,-96], style(color=10, rgbcolor={135,135,135})),
          Line(points=[-70,-70; -96,-96], style(color=10, rgbcolor={135,135,135})),
          Ellipse(extent=[120,120; -122,-120], style(
              color=10,
              rgbcolor={135,135,135},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1)),
             Text(
            extent=[-140,140; 140,-140],
            string="0",
            style(color=0, rgbcolor={0,0,0}))),
        Diagram(
             Text(
            extent=[-140,140; 140,-140],
            string="0",
            style(color=0, rgbcolor={0,0,0}))),
        Documentation(info="<html>
This is the model of the EightPort 0-junction.  It has one causality stroke. <p>

This model allows connectors to remain unconnected externally.  Hence the junction can be used for up to eight bond connections.
</html>"));
    equation 
      e[2:8] = e[1:7];
      sum(f) = 0;
    end J0large;
    
    model J1p2 "Model of TwoPort 1-junction" 
      extends Interfaces.TwoPortOne;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Documentation(info="<html>
This is the model of the TwoPort 1-junction.  It has one causality stroke.
</html>
"));
    equation 
      f2 = f1;
      e1 + e2 = 0;
    end J1p2;
    
    model J1p3 "Model of ThreePort 1-junction" 
      extends Interfaces.ThreePortOne;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Documentation(info="<html>
This is the model of the ThreePort 1-junction.  It has two causality strokes.
</html>
"));
    equation 
      f[2:3] = f[1:2];
      sum(e) = 0;
    end J1p3;
    
    model J1p4 "Model of FourPort 1-junction" 
      extends Interfaces.FourPortOne;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Documentation(info="<html>
This is the model of the FourPort 1-junction.  It has three causality strokes.
</html>
"));
    equation 
      f[2:4] = f[1:3];
      sum(e) = 0;
    end J1p4;
    
    model J1p5 "Model of FivePort 1-junction" 
      extends Interfaces.FivePortOne;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Documentation(info="<html>
This is the model of the FivePort 1-junction.  It has four causality strokes.
</html>
"));
    equation 
      f[2:5] = f[1:4];
      sum(e) = 0;
    end J1p5;
    
    model J1p6 "Model of SixPort 1-junction" 
      extends Interfaces.SixPortOne;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="1",
            style(color=0))),
        Documentation(info="<html>
This is the model of the SixPort 1-junction.  It has two causality strokes.
</html>
"));
    equation 
      f[2:6] = f[1:5];
      sum(e) = 0;
    end J1p6;
    
    model J1 "Model of a standard 1-junction with up to four ports" 
      extends Interfaces.FourPortZero2;
      annotation (
        Diagram(Text(
            extent=[-60,80; 60,-80],
            string="1",
            style(color=0))),
        Icon(   Text(
            extent=[-60,80; 60,-80],
            string="1",
            style(color=0))),
        Documentation(info="<html>
This is the model of the generalized FourPort 1-junction. <p>

Contrary to the regular FourPoint 1-junction (<b>J1p4</b>), this model allows connectors to remain unconnected externally.  Hence the junction can be used for up to four bond connections.
</html>"));
    equation 
      f[2:4] = f[1:3];
      sum(e) = 0;
    end J1;
    
    model J1large "Model of a large 1-junction with up to eight ports" 
      extends Interfaces.EightPortOne;
      annotation (
        Icon(
          Line(points=[-102,0; -194,0], style(color=10, rgbcolor={135,135,135})),
          Line(points=[0,100; 0,194], style(color=10, rgbcolor={135,135,135})),
          Line(points=[104,0; 194,0], style(color=10, rgbcolor={135,135,135})),
          Line(points=[0,-104; 0,-194], style(color=10, rgbcolor={135,135,135})),
          Line(points=[-70,70; -96,96], style(color=10, rgbcolor={135,135,135})),
          Line(points=[70,70; 96,96], style(color=10, rgbcolor={135,135,135})),
          Line(points=[70,-70; 96,-96], style(color=10, rgbcolor={135,135,135})),
          Line(points=[-70,-70; -96,-96], style(color=10, rgbcolor={135,135,135})),
          Ellipse(extent=[120,120; -122,-120], style(
              color=10,
              rgbcolor={135,135,135},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1)),
             Text(
            extent=[-140,140; 140,-140],
            style(color=0, rgbcolor={0,0,0}),
            string="1")),
        Diagram(
             Text(
            extent=[-140,140; 140,-140],
            style(color=0, rgbcolor={0,0,0}),
            string="1")),
        Documentation(info="<html>
This is the model of the EightPort 1-junction. <p>

This model allows connectors to remain unconnected externally.  Hence the junction can be used for up to eight bond connections.
</html>"));
    equation 
      f[2:8] = f[1:7];
      sum(e) = 0;
    end J1large;
  end Junctions;
  
  package Passive "Passive elements of Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30, fillPattern=
                1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
              fillColor=30, fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
              fillColor=30, fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=3)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0.48,
        y=0.48,
        width=0.35,
        height=0.49,
        library=1,
        autolayout=1),
      Documentation(info="<html>
The basic passive modeling elements used in the bond graph methodology are: <p>

<ul>
<li> the resistors (<b>R</b>),
<li> the capacitors (<b>C</b>),
<li> the inductors (<b>I</b>),
<li> the transformers (<b>TF</b>), and
<li> the gyrators (<b>GY</b>).
</ul> <p>

These elements are collected here.
</html>

"));
    model R "The bondgraphic linear resistor element" 
      extends Interfaces.PassiveOnePort;
      parameter Real R=1 "Bondgraphic Resistance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="R",
            style(color=0, fillColor=0))),
        Icon(
          Text(extent=[-70, -50; 70, -90], string="R=%R"),
          Text(
            extent=[-60, 80; 60, -80],
            string="R",
            style(color=0, fillColor=0)),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Documentation(info="<html>
The bondgraphic linear resistor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The resistance is modeled as a parameter. <p>

The causality of the resistor is free. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 R:  Resistance (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = R*f</b>
</html>
"));
    equation 
      e = R*f;
    end R;
    
    model mR "The bondgraphic modulated resistor element" 
      extends Interfaces.ModPassiveOnePort;
      Real R "Bondgraphic Resistance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mR",
            style(color=0))),
        Icon(Text(
            extent=[-58,80; 62,-80],
            string="mR",
            style(color=0)), Text(extent=[-70, 80; 70, 40], string="%name")),
        Documentation(info="<html>
The bondgraphic modulated resistor is a modulated passive OnePort element.  It inherits the effort and flow variables as well as the modulating signal from the modulated passive OnePort.  The resistance is modeled as a real-valued variable. <p>

The causality of the resistor is free. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 R:  Resistance (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = R*f</b>
</html>
"));
    equation 
      R = s;
      e = R*f;
    end mR;
    
    model HydroR "The bondgraphic hydraulic resistor element" 
      extends Interfaces.PassiveOnePort;
      parameter Real k(unit="m^3.5/kg^0.5") = 1 "Hydraulic Valve Constant";
      Real sf "Sign of f";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="R",
            style(color=0, fillColor=0)), Text(
            extent=[14, 10; 122, -12],
            string="Hydro",
            style(color=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="R",
            style(color=0, fillColor=0)),
          Text(
            extent=[14, 10; 122, -12],
            string="Hydro",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-70, -50; 70, -90], string="k=%k")),
        Documentation(info="<html>
The bondgraphic hydraulic resistor is a passive OnePort element.  It models the turbulent flow through a hydraulic valve.  The model inherits the effort and flow variables from the passive OnePort.  The valve constant is modeled as a parameter. <p>

This non-linear model is formulated in resistive causality.  There exists a second model, <b>HydroG</b>, that models the same device in conductive causality.  It is useful to make use of the model in its correct causality, as Modelica may be unable to automatically turn the causality of this non-linear equation around. <p>

The hydraulic resistor model has a preferred causality away from the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 k:  Valve constant (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = sign(f)*f<sup>2</sup>/k</b>
</html>
"));
    equation 
      sf = noEvent(if f > 0 then 1 else -1);
      e = sf*f*f/k;
    end HydroR;
    
    model HydromR "The bondgraphic hydraulic modulated resistor element" 
      extends Interfaces.ModPassiveOnePort;
      Real k(unit="m^3.5/kg^0.5") "Hydraulic Valve Constant";
      Real sf "Sign of f";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mR",
            style(color=0)), Text(
            extent=[-48, -26; 60, -48],
            string="Hydro",
            style(color=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="mR",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(
            extent=[-48, -26; 60, -48],
            string="Hydro",
            style(color=0))),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The bondgraphic hydraulic modulated resistor is a modulated passive OnePort element.  It models the turbulent flow through a hydraulic valve.  The model inherits the effort and flow variables as well as the modulation signal from the modulated passive OnePort.  The valve constant is modeled as a real-valued variable. <p>

This non-linear model is formulated in resistive causality.  There exists a second model, <b>HydromG</b>, that models the same device in conductive causality.  It is useful to make use of the model in its correct causality, as Modelica may be unable to automatically turn the causality of this non-linear equation around. <p>

The hydraulic resistor model has a preferred causality away from the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 k:  Valve constant (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = sign(f)*f<sup>2</sup>/k</b>
</html>
"));
    equation 
      sf = noEvent(if f > 0 then 1 else -1);
      k = s;
      e = sf*f*f/k;
    end HydromR;
    
    model RF "The bondgraphic linear TwoPort resistive field element" 
      extends Interfaces.TwoPort2;
      parameter Real R1=1 "Primary Resistance";
      parameter Real R2=1 "Secondary Resistance";
      parameter Real R12=0 "Coupling Resistance";
      annotation (
        Icon(Text(
            extent=[-62, 80; 58, -80],
            style(color=0),
            string="RF"), Text(extent=[-68, 90; 72, 50], string="%name")),
        Diagram(Text(
            extent=[-62, 80; 58, -80],
            style(color=0),
            string="RF")),
        Documentation(info="<html>
The bondgraphic TwoPort resistive field is a symmetric TwoPort element.  It inherits the effort and flow variables from the symmetric TwoPort.  The resistance values are modeled as a set of three parameters. <p>

The TwoPort resistive field has free causality. <p>

<hr>

<pre>
Potential variables:

 e1:   Bondgraphic effort variable of inflow

 f1:   Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:   Bondgraphic effort variable of outflow

 f2:   Bondgraphic flow variable of outflow, normalized positive for flows into the model


Parameters:

 R1:   Primary resistance (default value = 1)

 R2:   Secondary resistance (default value = 1)

 R12:  Coupling resistance (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e1 = R1*f1 + R12*f2</b> <br>
<b>e2 = R12*f1 + R2*f2</b>
</html>
"));
    equation 
      e1 = R1*f1 + R12*f2;
      e2 = R12*f1 + R2*f2;
    end RF;
    
    model G "The bondgraphic linear conductor element" 
      extends Interfaces.PassiveOnePort;
      parameter Real G=1 "Bondgraphic Conductance";
      annotation (
        Icon(
          Text(extent=[-68,-52; 72,-92], string="G=%G"),
          Text(
            extent=[-58,78; 62,-82],
            style(color=0, fillColor=0),
            string="G"),
          Text(extent=[-68,88; 72,48],   string="%name")),
        Diagram(
          Text(
            extent=[-58,78; 62,-82],
            style(color=0, fillColor=0),
            string="G")),
        Documentation(info="<html>
The bondgraphic linear conductor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The conductance is modeled as a parameter. <p>

The causality of the conductor is free. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 G:  Conductance (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = G*e</b>
</html>"));
    equation 
      f = G*e;
    end G;
    
    model mG "The bondgraphic modulated rconductor element" 
      extends Interfaces.ModPassiveOnePort;
      Real G "Bondgraphic Conductance";
      annotation (
        Icon(Text(
            extent=[-62,78; 58,-82],
            style(color=0),
            string="mG"),    Text(extent=[-70,80; 70,40],   string="%name")),
        Diagram(
             Text(
            extent=[-62,78; 58,-82],
            style(color=0),
            string="mG")),
        Documentation(info="<html>
The bondgraphic modulated conductor is a modulated passive OnePort element.  It inherits the effort and flow variables as well as the modulating signal from the modulated passive OnePort.  The conductance is modeled as a real-valued variable. <p>

The causality of the conductor is free. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 G:  Conductance (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = G*e</b>
</html>"));
    equation 
      G = s;
      f = G*e;
    end mG;
    
    model HydroG "The bondgraphic hydraulic conductor element" 
      extends Interfaces.PassiveOnePort;
      parameter Real k(unit="m^3.5/kg^0.5") = 1 "Hydraulic Valve Constant";
      Real se "Sign of e";
      Modelica.SIunits.Pressure ae "Absolute value of e";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="G",
            style(color=0, fillColor=0)), Text(
            extent=[16, 10; 124, -12],
            string="Hydro",
            style(color=0))),
        Icon(
          Text(
            extent=[16, 10; 124, -12],
            string="Hydro",
            style(color=0)),
          Text(
            extent=[-60, 80; 60, -80],
            string="G",
            style(color=0, fillColor=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-70, -50; 70, -90], string="k=%k")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49),
        Documentation(info="<html>
The bondgraphic hydraulic conductor is a passive OnePort element.  It models the turbulent flow through a hydraulic valve.  The model inherits the effort and flow variables from the passive OnePort.  The valve constant is modeled as a parameter. <p>

This non-linear model is formulated in conductive causality.  There exists a second model, <b>HydroR</b>, that models the same device in resistive causality.  It is useful to make use of the model in its correct causality, as Modelica may be unable to automatically turn the causality of this non-linear equation around. <p>

The hydraulic conductor model has a preferred causality at the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 k:  Valve constant (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = k*sign(e)*sqrt(abs(e))</b>
</html>
"));
    equation 
      se = noEvent(if e > 0 then 1 else -1);
      ae = noEvent(if e > 0 then e else -e);
      f = k*se*sqrt(ae);
    end HydroG;
    
    model HydromG "The bondgraphic hydraulic modulated conductor element" 
      extends Interfaces.ModPassiveOnePort;
      Real k(unit="m^3.5/kg^0.5") "Hydraulic Valve Constant";
      Real se "Sign of e";
      Modelica.SIunits.Pressure ae "Absolute value of e";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mG",
            style(color=0)), Text(
            extent=[-48, -26; 60, -48],
            string="Hydro",
            style(color=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="mG",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(
            extent=[-48, -26; 60, -48],
            string="Hydro",
            style(color=0))),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The bondgraphic hydraulic modulated conductor is a modulated passive OnePort element.  It models the turbulent flow through a hydraulic valve.  The model inherits the effort and flow variables as well as the modulation signal from the modulated passive OnePort.  The valve constant is modeled as a real-valued variable. <p>

This non-linear model is formulated in conductive causality.  There exists a second model, <b>HydromR</b>, that models the same device in resistive causality.  It is useful to make use of the model in its correct causality, as Modelica may be unable to automatically turn the causality of this non-linear equation around. <p>

The hydraulic conductor model has a preferred causality at the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 k:  Valve constant (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = k*sign(e)*sqrt(abs(e))</b>
</html>
"));
    equation 
      se = noEvent(if e > 0 then 1 else -1);
      ae = noEvent(if e > 0 then e else -e);
      k = s;
      f = k*se*sqrt(ae);
    end HydromG;
    
    model GF "The bondgraphic linear TwoPort conductive field element" 
      extends Interfaces.TwoPort2;
      parameter Real G1=1 "Primary Conductance";
      parameter Real G2=1 "Secondary Conductance";
      parameter Real G12=0 "Coupling Conductance";
      annotation (
        Icon(Text(
            extent=[-60, 78; 60, -82],
            style(color=0),
            string="GF"), Text(extent=[-66, 86; 74, 46], string="%name")),
        Diagram(Text(
            extent=[-60, 78; 60, -82],
            style(color=0),
            string="GF")),
        Documentation(info="<html>
The bondgraphic TwoPort conductive field is a symmetric TwoPort element.  It inherits the effort and flow variables from the symmetric TwoPort.  The conductance values are modeled as a set of three parameters. <p>

The TwoPort conductive field has free causality. <p>

<hr>

<pre>
Potential variables:

 e1:   Bondgraphic effort variable of inflow

 f1:   Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:   Bondgraphic effort variable of outflow

 f2:   Bondgraphic flow variable of outflow, normalized positive for flows into the model


Parameters:

 G1:   Primary conductance (default value = 1)

 G2:   Secondary conductance (default value = 1)

 G12:  Coupling conductance (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f1 = G1*e1 + G12*e2</b> <br>
<b>f2 = G12*e1 + G2*e2</b>
</html>
"));
    equation 
      f1 = G1*e1 + G12*e2;
      f2 = G12*e1 + G2*e2;
    end GF;
    
    model C "The bondgraphic linear capacitor element" 
      extends Interfaces.PassiveOnePort;
      parameter Real C=1 "Bondgraphic Capacitance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="C",
            style(color=0, fillColor=0))),
        Icon(
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(
            extent=[-60, 80; 60, -80],
            string="C",
            style(color=0, fillColor=0)),
          Text(extent=[-70, -50; 70, -90], string="C=%C")),
        Documentation(info="<html>
The bondgraphic linear capacitor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The capacitance is modeled as a parameter. <p>

The capacitor model has a preferred causality away from the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 C:  Capacitance (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = C*der(e)</b>
</html>
"));
    equation 
      f = C*der(e);
    end C;
    
    model mC "The bondgraphic modulated capacitor element" 
      extends Interfaces.ModPassiveOnePort;
      Real C "Bondgraphic Capacitance";
      Real q "Bondgraphic generalized position";
      annotation (
        Icon(Text(
            extent=[-64,78; 56,-82],
            style(color=0),
            string="mC"),    Text(extent=[-70,80; 70,40],   string="%name")),
        Diagram(
             Text(
            extent=[-64,78; 56,-82],
            style(color=0),
            string="mC")),
        Documentation(info="<html>
The bondgraphic modulated capacitor is a modulated passive OnePort element.  It inherits the effort and flow variables as well as the modulating signal from the modulated passive OnePort.  The capacitance is modeled as a real-valued variable. <p>

The capacitor has a preferred causality at the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 q:  Bondgraphic generalized position variable

 C:  Capacitance (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>q = C*e</b> <br>
<b>f = der(q)</b>
</html>"));
    equation 
      C = s;
      q = C*e;
      f = der(q);
    end mC;
    
    model CF "The bondgraphic linear TwoPort capacitive field element" 
      extends Interfaces.TwoPort2;
      parameter Real C1=1 "Primary Capacitance";
      parameter Real C2=1 "Secondary Capacitance";
      parameter Real C12=0 "Coupling Capacitance";
      final parameter Real Det=C1*C2 - C12^2;
      final parameter Real Cinv1=C2/Det;
      final parameter Real Cinv2=C1/Det;
      final parameter Real Cinv12=-C12/Det;
      annotation (
        Icon(Text(
            extent=[-60, 78; 60, -82],
            style(color=0),
            string="CF"), Text(extent=[-66, 88; 74, 48], string="%name")),
        Diagram(Text(
            extent=[-60, 78; 60, -82],
            style(color=0),
            string="CF")),
        Documentation(info="<html>
The bondgraphic TwoPort capacitive field is a symmetric TwoPort element.  It inherits the effort and flow variables from the symmetric TwoPort.  The capacitance values are modeled as a set of three parameters.  The symmetric capacitance matrix is manually inverted to speed up the compilation. <p>

The TwoPort capacitive field has preferred causalities at the <b>CF</b> element. <p>

<hr>

<pre>
Potential variables:

 e1:   Bondgraphic effort variable of inflow

 f1:   Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:   Bondgraphic effort variable of outflow

 f2:   Bondgraphic flow variable of outflow, normalized positive for flows into the model


Parameters:

 C1:   Primary capacitance (default value = 1)

 C2:   Secondary capacitance (default value = 1)

 C12:  Coupling capacitance (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>Det = C1*C2 - C12<sup>2</sup></b> <br>
<b>Cinv1 = C2/Det</b> <br>
<b>Cinv2 = C1/Det</b> <br>
<b>Cinv12 = -C12/Det</b> <p>
<b>der(e1) = Cinv1*f1 + Cinv12*f2</b> <br>
<b>der(e2) = Cinv12*f1 + Cinv2*f2</b>
</html>
"));
    equation 
      der(e1) = Cinv1*f1 + Cinv12*f2;
      der(e2) = Cinv12*f1 + Cinv2*f2;
    end CF;
    
    model I "The bondgraphic linear inductor element" 
      extends Interfaces.PassiveOnePort;
      parameter Real I=1 "Bondgraphic Inductance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="I",
            style(color=0, fillColor=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="I",
            style(color=0, fillColor=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-70, -50; 70, -90], string="I=%I")),
        Documentation(info="<html>
The bondgraphic linear inductor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The inductance is modeled as a parameter. <p>

The inductor model has a preferred causality at the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 I:  Inductance (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = I*der(f)</b>
</html>
"));
    equation 
      e = I*der(f);
    end I;
    
    model mI "The bondgraphic modulated inductor element" 
      extends Interfaces.ModPassiveOnePort;
      Real I "Bondgraphic Inductance";
      Real p "Bondgraphic generalized momentum";
      annotation (
        Icon(Text(
            extent=[-50,48; 48,-50],
            style(color=0),
            string="mI"),    Text(extent=[-70,80; 70,40],   string="%name")),
        Diagram(
             Text(
            extent=[-50,48; 48,-50],
            style(color=0),
            string="mI")),
        Documentation(info="<html>
The bondgraphic modulated inductor is a modulated passive OnePort element.  It inherits the effort and flow variables as well as the modulating signal from the modulated passive OnePort.  The inductance is modeled as a real-valued variable. <p>

The inductor has a preferred causality away from the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 p:  Bondgraphic generalized momentum variable

 I:  inductance (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>p = I*f</b> <br>
<b>e = der(p)</b>
</html>"));
    equation 
      I = s;
      p = I*f;
      e = der(p);
    end mI;
    
    model IF "The bondgraphic linear TwoPort inductive field element" 
      extends Interfaces.TwoPort2;
      parameter Real I1=1 "Primary Inductance";
      parameter Real I2=1 "Secondary Inductance";
      parameter Real I12=0 "Coupling Inductance";
      final parameter Real Det=I1*I2 - I12^2;
      final parameter Real Iinv1=I2/Det;
      final parameter Real Iinv2=I1/Det;
      final parameter Real Iinv12=-I12/Det;
      annotation (
        Icon(Text(extent=[-68, 88; 72, 48], string="%name"), Text(
            extent=[-48, 72; 44, -74],
            style(color=0),
            string="IF")),
        Diagram(Text(
            extent=[-48, 72; 44, -74],
            style(color=0),
            string="IF")),
        Documentation(info="<html>
The bondgraphic TwoPort inductive field is a symmetric TwoPort element.  It inherits the effort and flow variables from the symmetric TwoPort.  The inductance values are modeled as a set of three parameters.  The symmetric inductance matrix is manually inverted to speed up the compilation. <p>

The TwoPort inductive field has preferred causalities away from the <b>IF</b> element. <p>

<hr>

<pre>
Potential variables:

 e1:   Bondgraphic effort variable of inflow

 f1:   Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:   Bondgraphic effort variable of outflow

 f2:   Bondgraphic flow variable of outflow, normalized positive for flows into the model


Parameters:

 I1:   Primary inductance (default value = 1)

 I2:   Secondary inductance (default value = 1)

 I12:  Coupling inductance (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>Det = I1*I2 - I12<sup>2</sup></b> <br>
<b>Iinv1 = I2/Det</b> <br>
<b>Iinv2 = I1/Det</b> <br>
<b>Iinv12 = -I12/Det</b> <p>
<b>der(f1) = Iinv1*e1 + Iinv12*e2</b> <br>
<b>der(f2) = Iinv12*e1 + Iinv2*e2</b>
</html>
"));
    equation 
      der(f1) = Iinv1*e1 + Iinv12*e2;
      der(f2) = Iinv12*e1 + Iinv2*e2;
    end IF;
    
    model TF "The bondgraphic linear transformer element" 
      extends Interfaces.TwoPort;
      parameter Real m=1 "Bondgraphic Transformation Constant";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="TF",
            style(color=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="TF",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-70, -50; 70, -90], string="m=%m")),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The bondgraphic linear transformer is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The transformation constant is modeled as a parameter. <p>

The bond graph literature is not consistent in the definition of the transformation constant.  In this library, the constant is defined as the amplification of <b>flow</b> from the inflow to the outflow.  Some other bond graph references define the transformation constant as the amplification of <b>effort</b> from the inflow to the outflow. <p>

The transformer model has one causality stroke. <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 m:   Transformation constant (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f2 = m*f1</b> <br>
<b>e1 = m*e2</b>
</html>
"));
    equation 
      e1 = m*e2;
      f2 = m*f1;
    end TF;
    
    model mTF "The bondgraphic modulated transformer element" 
      extends Interfaces.ModTwoPort;
      Real m "Bondgraphic Transformation Constant";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mTF",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="mTF",
            style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The bondgraphic modulated transformer is a modulated directed TwoPort element.  It inherits the effort and flow variables from the modulated directed TwoPort.  The transformation constant is modeled as a real-valued variable. <p>

The bond graph literature is not consistent in the definition of the transformation constant.  In this library, the constant is defined as the amplification of <b>flow</b> from the inflow to the outflow.  Some other bond graph references define the transformation constant as the amplification of <b>effort</b> from the inflow to the outflow. <p>

The transformer model has one causality stroke. <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 m:   Transformation constant (modulation input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f2 = m*f1</b> <br>
<b>e1 = m*e2</b>
</html>
"));
    equation 
      m = s;
      e2 = m*e1;
      f1 = m*f2;
    end mTF;
    
    model GY "The bondgraphic linear gyrator element" 
      extends Interfaces.TwoPort;
      parameter Real r=1 "Bondgraphic Gyration Constant";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="GY",
            style(color=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="GY",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-70, -50; 70, -90], string="r=%r")),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The bondgraphic linear gyrator is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The gyration constant is modeled as a parameter. <p>

The bond graph literature is not consistent in the definition of the gyration constant.  In this library, the constant is defined as the amplification of <b>flow</b> at the inflow to <b>effort</b> at the outflow.  Some other bond graph references define the gyration constant as the amplification of <b>effort</b> at the inflow to <b>flow</b> at the outflow. <p>

The gyrator model has either zero or two causality strokes. <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 r:   Gyration constant (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e2 = r*f1</b> <br>
<b>e1 = r*f2</b>
</html>
"));
    equation 
      e1 = r*f2;
      e2 = r*f1;
    end GY;
    
    model mGY "The bondgraphic modulated gyrator element" 
      extends Interfaces.ModTwoPort;
      Real r "Bondgraphic Gyration Constant";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mGY",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="mGY",
            style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The bondgraphic modulated gyrator is a modulated directed TwoPort element.  It inherits the effort and flow variables from the modulated directed TwoPort.  The gyration constant is modeled as a real-valued variable. <p>

The bond graph literature is not consistent in the definition of the gyration constant.  In this library, the constant is defined as the amplification of <b>flow</b> at the inflow to <b>effort</b> at the outflow.  Some other bond graph references define the gyration constant as the amplification of <b>effort</b> at the inflow to <b>flow</b> at the outflow. <p>

The gyrator model has either zero or two causality strokes. <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 r:   Gyration constant (modulation input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e2 = r*f1</b> <br>
<b>e1 = r*f2</b>
</html>
"));
    equation 
      r = s;
      e1 = r*f2;
      e2 = r*f1;
    end mGY;
  end Passive;
  
  package Semiconductors "Bondgraphic description of semiconductor elements" 
    annotation (preferedView="info",
      Icon(
        Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30, fillPattern=
                1)),
        Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
              fillColor=30, fillPattern=1)),
        Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
              fillColor=30, fillPattern=1)),
        Text(
          extent=[-85,35; 65,-85],
          string="Library",
          style(color=3)),
        Text(
          extent=[-120,122; 120,73],
          string="%name",
          style(color=1))), Documentation(info="<html>
This package contains bond graph models of electronic semiconductors using the standard bond graph interface.
</html>
"));
    model Dreal "Electrical real diode of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      parameter Modelica.SIunits.Current Ids=1.e-6 "Saturation current";
      parameter Modelica.SIunits.Voltage Vt=0.04 
        "Voltage equivalent of temperature (kT/qn)";
      parameter Real Maxexp(final min=Modelica.Constants.small) = 15 
        "Max. exponent for linear continuation";
      parameter Modelica.SIunits.Resistance R=1.e8 "Parallel Ohmic resistance";
      annotation (
        Icon(
          Text(
            extent=[-88,68; 74,-76],
            string="D",
            style(color=0)),
          Text(
            extent=[26,-18; 88,-52],
            style(color=0, pattern=0),
            string="real"),
          Text(extent=[-76,92; 64,52],   string="%name")),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                fillColor=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[4,82; 14,72],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Text(
            extent=[70,-4; 80,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[-66,-6; -36,-14],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[16,50; 48,42],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Line(points=[0,0; 6,10; 12,28; 16,66],            style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1))),
        Documentation(info="<html>
The real diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  Real diodes are a special class of non-linear resistors. <p>

The real diode model consists of the diode itself and an parallel Ohmic resistance <b>R</b>. The diode formula essentially is: <p>

<b>f = Ids*(exp(e/Vt) - 1)</b> <p>

However, if the exponent <b>e/Vt</b> reaches the limit <b>Maxexp</b>, the diode characterisic is linearly continued to avoid overflow. <p>

The causality of the real diode element is free. <p>

<hr>

<pre>
Potential variables:

 e:       Bondgraphic effort variable

 f:       Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 Ids:     Saturation current (default value = 1e-6 Amp)

 Vt:      Thermal voltage (default value = 0.04 Volt)

 Maxexp:  Maximum exponent (default value = 15)

 R:       Parallel Ohmic leakage resistance (default value = 1e8 Ohm)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = if (e/Vt &gt; Maxexp) then Ids*(exp(Maxexp)*(1 + e/Vt - Maxexp) - 1) + e/R else Ids*(exp(e/Vt) - 1) + e/R</b>
</html>
"));
    equation 
      f = if (e/Vt > Maxexp) then Ids*(exp(Maxexp)*(1 + e/Vt - Maxexp) - 1) +
        e/R else Ids*(exp(e/Vt) - 1) + e/R;
    end Dreal;
    
    model NPN "NPN transistor of the Modelica bond graph library" 
      parameter Real Bf=50 "Forward beta coefficient";
      parameter Real Br=0.1 "Reverse beta coefficient";
      parameter Modelica.SIunits.Current Is=1.e-16 
        "Transport saturation current";
      parameter Modelica.SIunits.InversePotential Vak=0.02 
        "Inverse early voltage";
      parameter Modelica.SIunits.Time Tauf=0.12e-9 "Ideal forward transit time";
      parameter Modelica.SIunits.Time Taur=5e-9 "Ideal reverse transit time";
      parameter Modelica.SIunits.Capacitance Ccs=1e-12 
        "Collector-substrate capacitance";
      parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
        "Base-emitter zero bias depletion capacitance";
      parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
        "Base-collector zero bias depletion capacitance";
      parameter Modelica.SIunits.Voltage Phie=0.8 
        "Base-emitter diffusion voltage";
      parameter Real Me=0.4 "Base-emitter gradation exponent";
      parameter Modelica.SIunits.Voltage Phic=0.8 
        "Base-collector diffusion voltage";
      parameter Real Mc=0.333 "Base-collector gradation exponent";
      parameter Modelica.SIunits.Conductance Gbc=1e-15 
        "Base-collector leakage conductance";
      parameter Modelica.SIunits.Conductance Gbe=1e-15 
        "Base-emitter leakage conductance";
      parameter Modelica.SIunits.Voltage Vt=0.02585 "Thermal voltage";
      parameter Real EMin=-100 "if x < EMin, the exp(x) function is linearized";
      parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
      Interfaces.BondCon B "Base" 
                           annotation (extent=[-110,-10; -90,10]);
      Interfaces.BondCon C "Collector" 
                           annotation (extent=[-10,90; 10,110]);
      Interfaces.BondCon E "Emitter" 
                           annotation (extent=[-10,-110; 10,-90]);
      annotation (
        Diagram(
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Ellipse(extent=[-112,42; -108,38], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Ellipse(extent=[-122,-42; -118,-38], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Text(
            extent=[-98,48; -88,40],
            string="ibc",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[-96,-66; -86,-74],
            string="ibc",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[78,72; 88,64],
            string="ibc",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[54,72; 64,64],
            style(color=9, rgbcolor={149,164,171}),
            string="ibe"),
          Text(
            extent=[-96,-86; -86,-94],
            style(color=9, rgbcolor={149,164,171}),
            string="ibe"),
          Text(
            extent=[-100,-32; -90,-40],
            style(color=9, rgbcolor={149,164,171}),
            string="ibe"),
          Text(
            extent=[-36,-12; -26,-20],
            style(color=9, rgbcolor={149,164,171}),
            string="vbc"),
          Text(
            extent=[48,58; 58,50],
            style(color=9, rgbcolor={149,164,171}),
            string="vbc"),
          Text(
            extent=[78,8; 88,0],
            style(color=9, rgbcolor={149,164,171}),
            string="IC0"),
          Text(
            extent=[-38,-90; -28,-98],
            style(color=9, rgbcolor={149,164,171}),
            string="IB0")),
        Icon(
          Text(
            extent=[-82,82; 86,-86],
            style(color=0),
            string="NPN"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Line(points=[0,80; 0,-80], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[-20,-60; 0,-80], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[0,-80; 20,-60], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1))),
        Documentation(info="<html>
The NPN element of the bond graph library implements the Ebers-Moll model of a NPN bipolar transistor.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

The NPN bipolar transistor is a directed ThreePort.  The direction of positive power flow is assumed into the model at the base, <b>B</b>, and at the collector, <b>C</b>, whereas it is assumed out of the model at the emitter, <b>E</b>. <p>

The causality of the NPN model is relatively free.  However, if there is a causality stroke placed at the collector from the outside, the substrate capacitor will assume differential causality. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"));
      Junctions.J0p3 J0p3_1 annotation (extent=[-10,-10; 10,10], rotation=-90);
      Bonds.Bond B1 annotation (extent=[-10,50; 10,70], rotation=90);
      Bonds.Bond B2 annotation (extent=[-10,10; 10,30], rotation=90);
      Bonds.Bond B3 annotation (extent=[-10,-30; 10,-10], rotation=-90);
      Bonds.Bond B4 annotation (extent=[-10,-70; 10,-50], rotation=-90);
      Junctions.J1p3 J1p3_1 annotation (extent=[-10,30; 10,50], rotation=-90);
      Junctions.J0p4 J0p4_1 annotation (extent=[-10,70; 10,90]);
      Junctions.J1p3 J1p3_2 annotation (extent=[-10,-50; 10,-30], rotation=-90);
      Junctions.J0p3 J0p3_2 annotation (extent=[10,-90; -10,-70], rotation=-90);
      Junctions.J1p3 J1p3_3 annotation (extent=[40,-10; 20,10], rotation=-90);
      Bonds.Bond B5 annotation (extent=[20,10; 40,30], rotation=-90);
      Bonds.Bond B6 annotation (extent=[20,-30; 40,-10], rotation=-90);
      Bonds.Bond B7 annotation (extent=[-30,70; -10,90], rotation=180);
      Passive.C C1(C=Ccs) annotation (extent=[-30,70; -50,90]);
      Bonds.eBond B8 annotation (extent=[40,-10; 60,10], rotation=180);
      Sources.mSf Ic0 annotation (extent=[60,-10; 80,10]);
      Junctions.J0p3 J0p3_3 annotation (extent=[-50,50; -30,30], rotation=-180);
      Bonds.eBond B9 annotation (extent=[-30,30; -10,50], rotation=180);
      Bonds.fBond B10 annotation (extent=[-50,10; -30,30], rotation=-90);
      Sensors.De Vbc annotation (extent=[-50,-10; -30,10], rotation=-90);
      Bonds.eBond B11 annotation (extent=[-70,30; -50,50], rotation=180);
      Utilities.Cj Dbc(
        Is=Is,
        Tau=Taur,
        Cj=Cjc,
        Phi=Phic,
        M=Mc,
        Gb=Gbc,
        Vt=Vt,
        EMin=EMin,
        EMax=EMax) annotation (extent=[-70,30; -90,50]);
      Bonds.eBond B12 annotation (extent=[-30,-50; -10,-30], rotation=180);
      Junctions.J0p3 J0p3_4 
        annotation (extent=[-50,-30; -30,-50], rotation=-180);
      Bonds.eBond B13 annotation (extent=[-50,-70; -30,-50], rotation=90);
      Sources.mSf Ib0 annotation (extent=[-50,-90; -30,-70], rotation=-90);
      Bonds.eBond B14 annotation (extent=[-70,-50; -50,-30], rotation=180);
      Utilities.Cj Dbe(
        Is=Is,
        Vt=Vt,
        EMin=EMin,
        EMax=EMax,
        Tau=Tauf,
        Cj=Cje,
        Phi=Phie,
        M=Me,
        Gb=Gbe) annotation (extent=[-70,-50; -90,-30]);
      Utilities.IC0 IC0_1(
        Br=Br,
        Vak=Vak) annotation (extent=[60,40; 80,60], rotation=-90);
      Utilities.IB0 IB0_1(Bf=Bf, Br=Br) annotation (extent=[-80,-90; -60,-70]);
    equation 
      connect(J0p3_1.BondCon1, B2.BondCon1) annotation (points=[-6.12303e-016,
            10; -6.00057e-016,10; -6.00057e-016,10.2], style(color=8, rgbcolor=
              {192,192,192}));
      connect(J1p3_1.BondCon2, B2.BondCon2) annotation (points=[6.12303e-016,30;
            6.12303e-016,30], style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon1, J1p3_1.BondCon1) annotation (points=[-6.00057e-016,
            50.2; -6.00057e-016,56.1; -6.12303e-016,56.1; -6.12303e-016,50],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon3, B1.BondCon2) annotation (points=[0,70;
            6.12303e-016,70], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon4, C) annotation (points=[0,90; 0,100], style(color=
             8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2, B3.BondCon1) annotation (points=[6.12303e-016,
            -10; -6.00057e-016,-10; -6.00057e-016,-10.2], style(color=8,
            rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon1, B3.BondCon2) annotation (points=[-6.12303e-016,
            -30; 6.12303e-016,-30], style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon1, J1p3_2.BondCon2) annotation (points=[-6.00057e-016,
            -50.2; -6.00057e-016,-55.1; 6.12303e-016,-55.1; 6.12303e-016,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon2, J0p3_2.BondCon1) annotation (points=[6.12303e-016,
            -70; 6.12303e-016,-70], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon2, E) annotation (points=[-6.12303e-016,-90; 0,-90;
            0,-100], style(color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon2, J1p3_3.BondCon1) annotation (points=[30,10; 30,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon2, B6.BondCon1) annotation (points=[30,-10; 30,
            -10.2], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon2, B5.BondCon1) annotation (points=[10,80; 30,80;
            30,29.8], style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon2, J0p3_2.BondCon3) annotation (points=[30,-30; 30,-80;
            10,-80], style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1, J0p4_1.BondCon1) annotation (points=[-10.2,80; -10,
            80], style(color=8, rgbcolor={192,192,192}));
      connect(C1.BondCon1, B7.BondCon2) annotation (points=[-30,80; -30,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon3, B8.eBondCon1) annotation (points=[40,
            -6.12303e-016; 44,-6.12303e-016; 44,1.22461e-015; 40,1.22461e-015],
          style(color=8, rgbcolor={192,192,192}));
      connect(Ic0.BondCon1, B8.fBondCon1) annotation (points=[60,0; 64,0; 64,
            -1.22461e-015; 60,-1.22461e-015], style(color=8, rgbcolor={192,192,
              192}));
      connect(B9.fBondCon1, J1p3_1.BondCon3) annotation (points=[-10,40; -10,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon1, B9.eBondCon1) annotation (points=[-30,40; -30,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon3, B10.eBondCon1) annotation (points=[-40,30; -40,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(Vbc.BondCon1, B10.fBondCon1) annotation (points=[-40,10; -40,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(B11.fBondCon1, J0p3_3.BondCon2) annotation (points=[-50,40; -50,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(Dbc.BondCon1, B11.eBondCon1) annotation (points=[-70,40; -70,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B12.fBondCon1, J1p3_2.BondCon3) annotation (points=[-10,-40; -10,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon1, B12.eBondCon1) annotation (points=[-30,-40; -30,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(B13.eBondCon1, J0p3_4.BondCon3) annotation (points=[-40,-50; -40,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(Ib0.BondCon1, B13.fBondCon1) annotation (points=[-40,-70; -40,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B14.fBondCon1, J0p3_4.BondCon2) annotation (points=[-50,-40; -50,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(Dbe.BondCon1, B14.eBondCon1) annotation (points=[-70,-40; -70,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Vbc.OutPort1, IC0_1.u3) annotation (points=[-40,-11; -40,-20; -20,
            -20; -20,34; 50,34; 50,50; 60,50], style(color=74, rgbcolor={0,0,
              127}));
      connect(Dbc.y, IC0_1.u1) annotation (points=[-90,40; -110,40; -110,112;
            76,112; 76,62], style(color=74, rgbcolor={0,0,127}));
      connect(Dbc.y, IB0_1.u1) annotation (points=[-90,40; -110,40; -110,-74;
            -82,-74], style(color=74, rgbcolor={0,0,127}));
      connect(Dbe.y, IB0_1.u2) annotation (points=[-90,-40; -120,-40; -120,-86;
            -82,-86], style(color=74, rgbcolor={0,0,127}));
      connect(Dbe.y, IC0_1.u2) annotation (points=[-90,-40; -120,-40; -120,122;
            64,122; 64,62], style(color=74, rgbcolor={0,0,127}));
      connect(J0p3_1.BondCon3, B) annotation (points=[-10,-6.12303e-016; -52,
            -6.12303e-016; -52,0; -100,0], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=74,
          rgbfillColor={0,0,185},
          fillPattern=1));
      connect(IB0_1.y, Ib0.s) annotation (points=[-59,-80; -50,-80; -50,-96;
            -40,-96; -40,-89], style(color=74, rgbcolor={0,0,127}));
      connect(IC0_1.y, Ic0.s) annotation (points=[70,39; 70,30; 90,30; 90,0; 79,
            0], style(color=74, rgbcolor={0,0,127}));
    end NPN;
    
    model NPNth 
      "NPN transistor of the Modelica bond graph library with temperature variation" 
      parameter Real Bf=50 "Forward beta coefficient";
      parameter Real Br=0.1 "Reverse beta coefficient";
      parameter Modelica.SIunits.Temperature Tnom=300.15 
        "Reference temperature";
      parameter Modelica.SIunits.Current Is=1.e-16 
        "Transport saturation current";
      parameter Modelica.SIunits.InversePotential Vak=0.02 
        "Inverse early voltage";
      parameter Modelica.SIunits.Time Tauf=0.12e-9 "Ideal forward transit time";
      parameter Modelica.SIunits.Time Taur=5e-9 "Ideal reverse transit time";
      parameter Real NF=1 "Forward current emission coefficient";
      parameter Real NR=1 "Reverse current emission coefficient";
      parameter Modelica.SIunits.Capacitance Ccs=1e-12 
        "Collector-substrate capacitance";
      parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
        "Base-emitter zero bias depletion capacitance";
      parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
        "Base-collector zero bias depletion capacitance";
      parameter Modelica.SIunits.Voltage Phie=0.8 
        "Base-emitter diffusion voltage";
      parameter Real Me=0.4 "Base-emitter gradation exponent";
      parameter Modelica.SIunits.Voltage Phic=0.8 
        "Base-collector diffusion voltage";
      parameter Real Mc=0.333 "Base-collector gradation exponent";
      parameter Modelica.SIunits.Voltage EG=1.11 
        "Energy gap for temperature effect on saturation current";
      parameter Real XTI=3 "Saturation current temperature exponent";
      parameter Real XTB=0 "Forward and reverse beta temperature coefficient";
      parameter Modelica.SIunits.Conductance Gbc=1e-15 
        "Base-collector leakage conductance";
      parameter Modelica.SIunits.Conductance Gbe=1e-15 
        "Base-emitter leakage conductance";
      parameter Real EMin=-100 "if x < EMin, the exp(x) function is linearized";
      parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
      Interfaces.BondCon B "Base" 
                           annotation (extent=[-110,-10; -90,10]);
      Interfaces.BondCon C "Collector" 
                           annotation (extent=[-10,90; 10,110]);
      Interfaces.BondCon E "Emitter" 
                           annotation (extent=[-10,-110; 10,-90]);
      Junctions.J0p3 J0p3_1 annotation (extent=[-10,-10; 10,10], rotation=-90);
      Bonds.Bond B1 annotation (extent=[-10,50; 10,70], rotation=90);
      Bonds.Bond B2 annotation (extent=[-10,10; 10,30], rotation=90);
      Bonds.Bond B3 annotation (extent=[-10,-30; 10,-10], rotation=-90);
      Bonds.Bond B4 annotation (extent=[-10,-70; 10,-50], rotation=-90);
      Junctions.J1p3 J1p3_1 annotation (extent=[-10,30; 10,50], rotation=-90);
      Junctions.J0p4 J0p4_1 annotation (extent=[-10,70; 10,90]);
      Junctions.J1p3 J1p3_2 annotation (extent=[-10,-50; 10,-30], rotation=-90);
      Junctions.J0p3 J0p3_2 annotation (extent=[10,-90; -10,-70], rotation=-90);
      Junctions.J1p3 J1p3_3 annotation (extent=[40,-10; 20,10], rotation=-90);
      Bonds.Bond B5 annotation (extent=[20,10; 40,30], rotation=-90);
      Bonds.Bond B6 annotation (extent=[20,-30; 40,-10], rotation=-90);
      Bonds.Bond B7 annotation (extent=[-30,70; -10,90], rotation=180);
      Passive.C C1(C=Ccs) annotation (extent=[-30,70; -50,90]);
      Bonds.eBond B8 annotation (extent=[40,-10; 60,10], rotation=180);
      Junctions.J0p3 J0p3_3 annotation (extent=[-50,50; -30,30], rotation=-180);
      Bonds.eBond B9 annotation (extent=[-30,30; -10,50], rotation=180);
      Bonds.fBond B10 annotation (extent=[-50,10; -30,30], rotation=-90);
      Sensors.De Vbc annotation (extent=[-50,-10; -30,10], rotation=-90);
      Bonds.eBond B11 annotation (extent=[-70,30; -50,50], rotation=180);
      Bonds.eBond B12 annotation (extent=[-30,-50; -10,-30], rotation=180);
      Junctions.J0p3 J0p3_4 
        annotation (extent=[-50,-30; -30,-50], rotation=-180);
      Bonds.eBond B13 annotation (extent=[-50,-70; -30,-50], rotation=90);
      Bonds.eBond B14 annotation (extent=[-70,-50; -50,-30], rotation=180);
      annotation (
        Diagram(
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Ellipse(extent=[-112,62; -108,58], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Ellipse(extent=[-116,-22; -112,-18], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Text(
            extent=[-80,60; -70,52],
            string="ibc",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[-96,-66; -86,-74],
            string="ibc",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[78,72; 88,64],
            string="ibc",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[54,72; 64,64],
            style(color=9, rgbcolor={149,164,171}),
            string="ibe"),
          Text(
            extent=[-96,-86; -86,-94],
            style(color=9, rgbcolor={149,164,171}),
            string="ibe"),
          Text(
            extent=[-80,-20; -70,-28],
            style(color=9, rgbcolor={149,164,171}),
            string="ibe"),
          Text(
            extent=[-36,-12; -26,-20],
            style(color=9, rgbcolor={149,164,171}),
            string="vbc"),
          Text(
            extent=[48,56; 58,48],
            style(color=9, rgbcolor={149,164,171}),
            string="vbc"),
          Text(
            extent=[72,18; 82,10],
            style(color=9, rgbcolor={149,164,171}),
            string="IC0"),
          Text(
            extent=[-30,-72; -20,-80],
            style(color=9, rgbcolor={149,164,171}),
            string="IB0"),
          Text(
            extent=[88,26; 120,0],
            style(color=9, rgbcolor={149,164,171}),
            string="Heat"),
          Ellipse(extent=[48,-12; 52,-8],      style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Text(
            extent=[50,40; 62,32],
            style(color=9, rgbcolor={149,164,171}),
            string="Tdev"),
          Text(
            extent=[-84,-92; -72,-100],
            style(color=9, rgbcolor={149,164,171}),
            string="Tdev")),
        Documentation(info="<html>
The <b>NPNth</b> element of the bond graph library implements the Ebers-Moll model of a NPN bipolar transistor with temperature variation.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

The <b>NPNth</b> bipolar heating transistor is a directed FourPort.  The direction of positive power flow is assumed into the model at the base, <b>B</b>, and at the collector, <b>C</b>, whereas it is assumed out of the model at the emitter, <b>E</b>, and at the heat port, <b>Heat</b>. <p>

The causality of the <b>NPNth</b> model is relatively free.  However, if there is a causality stroke placed at the collector from the outside, the substrate capacitor will assume differential causality. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 NF:    Forward current emission coefficient (default value = 1)

 NR:    Reverse current emission coefficient (default value = 1)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 XTI:   Saturation current temperature exponent (default value = 3)

 XTB:   Forward and reverse beta temperature coefficient (default value = 0)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"),     Icon(
          Text(
            extent=[-84,82; 62,-86],
            style(color=0),
            string="NPN"),
          Line(points=[0,80; 0,-80], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[-20,-60; 0,-80], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Line(points=[0,-80; 20,-60], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Text(
            extent=[84,28; 116,2],
            style(color=9, rgbcolor={149,164,171}),
            string="Heat"),
          Line(points=[-72,-50; 72,-50], style(color=3, rgbcolor={0,0,255})),
          Line(points=[72,-50; 50,-40], style(color=3, rgbcolor={0,0,255})),
          Line(points=[72,-50; 50,-60], style(color=3, rgbcolor={0,0,255})),
          Text(
            extent=[56,2; 92,-32],
            string="th",
            style(color=0, rgbcolor={0,0,0}))));
      Utilities.IB02 IB0_1(
        Tnom=Tnom,
        Bf=Bf,
        Br=Br,
        XTB=XTB) annotation (extent=[-80,-90; -60,-70]);
      Interfaces.BondCon H "Heat" 
                           annotation (extent=[90,-10; 110,10]);
    Modelica.Blocks.Math.Gain Gain1(k=-1) 
      annotation (extent=[-14,-90; -34,-110],  rotation=-90);
      Utilities.IC02 IC0_1(
        Tnom=Tnom,
        Br=Br,
        Vak=Vak,
        XTB=XTB) annotation (extent=[60,40; 80,60], rotation=-90);
      Utilities.CjS Dbc(
        Tnom=Tnom,
        Is=Is,
        EG=EG,
        XTI=XTI,
        EMin=EMin,
        EMax=EMax,
        Tau=Taur,
        Cj=Cjc,
        Phi=Phic,
        M=Mc,
        N=NR,
        Gb=Gbc) annotation (extent=[-70,30; -90,50]);
      Utilities.CjS Dbe(
        Tnom=Tnom,
        Is=Is,
        EG=EG,
        XTI=XTI,
        EMin=EMin,
        EMax=EMax,
        Tau=Tauf,
        Cj=Cje,
        Phi=Phie,
        M=Me,
        N=NF,
        Gb=Gbe) annotation (extent=[-70,-50; -90,-30]);
      Bonds.fBond B15 annotation (extent=[56,-60; 76,-40], rotation=90);
      Sensors.De Tdev 
                     annotation (extent=[56,-40; 76,-20],  rotation=90);
      Bonds.eBond B16 annotation (extent=[56,-100; 76,-80],  rotation=90);
      Bonds.eBond B17 annotation (extent=[40,-80; 60,-60],   rotation=0);
      Junctions.J0p6 J0p6_1 annotation (extent=[60,-80; 80,-60]);
    Sources.mSf_int Ic0 annotation (extent=[80,10; 60,-10]);
      Bonds.fBond B18 annotation (extent=[70,-30; 90,-10], rotation=90);
    Sources.mSf_int Ib0 
      annotation (extent=[-50,-90; -30,-70],   rotation=90);
      Bonds.fBond B19 annotation (extent=[74,-90; 94,-70], rotation=0);
    equation 
      connect(J0p3_1.BondCon1,B2. BondCon1) annotation (points=[-6.12303e-016,
            10; -6.12303e-016,10.2; -6.00057e-016,10.2],
                                                       style(color=8, rgbcolor=
              {192,192,192}));
      connect(J1p3_1.BondCon2,B2. BondCon2) annotation (points=[6.12303e-016,30;
            6.12303e-016,30], style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon1,J1p3_1. BondCon1) annotation (points=[-6.00057e-016,
            50.2; -6.00057e-016,56.1; 0,50; -6.12303e-016,50],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon3,B1. BondCon2) annotation (points=[0,70;
            6.12303e-016,70], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon4,C)  annotation (points=[0,90; 0,100], style(color=
             8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2,B3. BondCon1) annotation (points=[6.12303e-016,
            -10; 6.12303e-016,-10.2; -6.00057e-016,-10.2],style(color=8,
            rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon1,B3. BondCon2) annotation (points=[-6.12303e-016,
            -30; 6.12303e-016,-30], style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon1,J1p3_2. BondCon2) annotation (points=[-6.00057e-016,
            -50.2; -6.00057e-016,-55.1; 0,-50; 6.12303e-016,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon2,J0p3_2. BondCon1) annotation (points=[6.12303e-016,
            -70; 6.12303e-016,-70], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon2,E)  annotation (points=[-6.12303e-016,-90; 0,-90;
            0,-100], style(color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon2,J1p3_3. BondCon1) annotation (points=[30,10; 30,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon2,B6. BondCon1) annotation (points=[30,-10; 30,
            -10.2], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon2,B5. BondCon1) annotation (points=[10,80; 30,80;
            30,29.8], style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon2,J0p3_2. BondCon3) annotation (points=[30,-30; 30,-80;
            10,-80], style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1,J0p4_1. BondCon1) annotation (points=[-10.2,80; -10,
            80], style(color=8, rgbcolor={192,192,192}));
      connect(C1.BondCon1,B7. BondCon2) annotation (points=[-30,80; -30,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.fBondCon1,J1p3_1. BondCon3) annotation (points=[-10,40; -10,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon1,B9. eBondCon1) annotation (points=[-30,40; -30,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon3,B10. eBondCon1) annotation (points=[-40,30; -40,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(Vbc.BondCon1,B10. fBondCon1) annotation (points=[-40,10; -40,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(B11.fBondCon1,J0p3_3. BondCon2) annotation (points=[-50,40; -50,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(B12.fBondCon1,J1p3_2. BondCon3) annotation (points=[-10,-40; -10,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon1,B12. eBondCon1) annotation (points=[-30,-40; -30,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(B13.eBondCon1,J0p3_4. BondCon3) annotation (points=[-40,-50; -40,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(B14.fBondCon1,J0p3_4. BondCon2) annotation (points=[-50,-40; -50,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon3,B)  annotation (points=[-10,-6.12303e-016; -52,
            -6.12303e-016; -52,0; -100,0], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=74,
          rgbfillColor={0,0,185},
          fillPattern=1));
      connect(IB0_1.y, Gain1.u) annotation (points=[-59,-80; -56,-80; -56,-118;
            -24,-118; -24,-112], style(color=74, rgbcolor={0,0,127}));
      connect(Vbc.OutPort1, IC0_1.u3) annotation (points=[-40,-11; -40,-20; -20,
            -20; -20,56; 60,56], style(color=74, rgbcolor={0,0,127}));
      connect(Dbc.BondCon1, B11.eBondCon1) annotation (points=[-70,40; -70,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Dbc.y, IB0_1.u1) annotation (points=[-80,51; -80,60; -110,60;
            -110,-74; -82,-74], style(color=74, rgbcolor={0,0,127}));
      connect(Dbc.y, IC0_1.u1) annotation (points=[-80,51; -80,60; -110,60;
            -110,114; 76,114; 76,62], style(color=74, rgbcolor={0,0,127}));
      connect(Dbe.BondCon1, B14.eBondCon1) annotation (points=[-70,-40; -70,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Dbe.y, IB0_1.u2) annotation (points=[-80,-29; -80,-20; -114,-20;
            -114,-86; -82,-86], style(color=74, rgbcolor={0,0,127}));
      connect(Dbe.y, IC0_1.u2) annotation (points=[-80,-29; -80,-20; -114,-20;
            -114,110; 64,110; 64,62], style(color=74, rgbcolor={0,0,127}));
      connect(Tdev.BondCon1, B15.fBondCon1) annotation (points=[66,-40; 66,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Tdev.OutPort1, IC0_1.Tdev) annotation (points=[66,-19; 66,-10; 50,
            -10; 50,44; 60,44], style(color=74, rgbcolor={0,0,127}));
      connect(Tdev.OutPort1, IB0_1.Tdev) annotation (points=[66,-19; 66,-10; 50,
            -10; 50,-122; -70,-122; -70,-90], style(color=74, rgbcolor={0,0,127}));
      connect(Dbe.BondCon2, B17.fBondCon1) annotation (points=[-90,-40; -118,
            -40; -118,-126; 34,-126; 34,-70; 40,-70], style(color=8, rgbcolor={
              192,192,192}));
      connect(Dbc.BondCon2, B16.fBondCon1) annotation (points=[-90,40; -122,40;
            -122,-130; 66,-130; 66,-100], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon1, B17.eBondCon1) annotation (points=[60,-70; 60,-70], style(
            color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon3, B16.eBondCon1) annotation (points=[66,-80; 66,
            -80], style(color=8, rgbcolor={192,192,192}));
      connect(B15.eBondCon1, J0p6_1.BondCon5) annotation (points=[66,-60; 66,
            -60], style(color=8, rgbcolor={192,192,192}));
      connect(Ic0.BondCon2, B8.fBondCon1) annotation (points=[60,0; 60,
            -1.22461e-015], style(color=8, rgbcolor={192,192,192}));
      connect(B8.eBondCon1, J1p3_3.BondCon3) annotation (points=[40,
            1.22461e-015; 40,-6.12303e-016], style(color=8, rgbcolor={192,192,
              192}));
      connect(B18.fBondCon1, Ic0.BondCon1) annotation (points=[80,-10; 80,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon6, B18.eBondCon1) annotation (points=[74,-60; 74,
            -40; 80,-40; 80,-30], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon2, H)  annotation (points=[80,-70; 90,-70; 90,0;
            100,0], style(color=8, rgbcolor={192,192,192}));
      connect(Ib0.BondCon2, B13.fBondCon1) annotation (points=[-40,-70; -40,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon4, B19.eBondCon1) annotation (points=[74,-80; 74,
            -80], style(color=8, rgbcolor={192,192,192}));
      connect(B19.fBondCon1, Ib0.BondCon1) annotation (points=[94,-80; 100,-80;
            100,-134; -40,-134; -40,-90], style(color=8, rgbcolor={192,192,192}));
      connect(Gain1.y, Ib0.s) annotation (points=[-24,-89; -24,-80; -31,-80], style(
            color=74, rgbcolor={0,0,127}));
      connect(IC0_1.y, Ic0.s) 
        annotation (points=[70,39; 70,9], style(color=74, rgbcolor={0,0,127}));
    end NPNth;
    
    model PNP "PNP transistor of the Modelica bond graph library" 
      parameter Real Bf=50 "Forward beta coefficient";
      parameter Real Br=0.1 "Reverse beta coefficient";
      parameter Modelica.SIunits.Current Is=1.e-16 
        "Transport saturation current";
      parameter Modelica.SIunits.InversePotential Vak=0.02 
        "Inverse early voltage";
      parameter Modelica.SIunits.Time Tauf=0.12e-9 "Ideal forward transit time";
      parameter Modelica.SIunits.Time Taur=5e-9 "Ideal reverse transit time";
      parameter Modelica.SIunits.Capacitance Ccs=1e-12 
        "Collector-substrate capacitance";
      parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
        "Base-emitter zero bias depletion capacitance";
      parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
        "Base-collector zero bias depletion capacitance";
      parameter Modelica.SIunits.Voltage Phie=0.8 
        "Base-emitter diffusion voltage";
      parameter Real Me=0.4 "Base-emitter gradation exponent";
      parameter Modelica.SIunits.Voltage Phic=0.8 
        "Base-collector diffusion voltage";
      parameter Real Mc=0.333 "Base-collector gradation exponent";
      parameter Modelica.SIunits.Conductance Gbc=1e-15 
        "Base-collector leakage conductance";
      parameter Modelica.SIunits.Conductance Gbe=1e-15 
        "Base-emitter leakage conductance";
      parameter Modelica.SIunits.Voltage Vt=0.02585 "Thermal voltage";
      parameter Real EMin=-100 "if x < EMin, the exp(x) function is linearized";
      parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
      annotation (
        Icon(
          Text(
            extent=[-80,82; 88,-86],
            style(color=0),
            string="PNP"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Line(points=[0,80; 0,-80], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[0,80; 20,60],    style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[-20,60; 0,80],  style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1))),
        Diagram(
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Ellipse(extent=[-112,42; -108,38], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Ellipse(extent=[-122,-42; -118,-38], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Text(
            extent=[-98,48; -88,40],
            style(color=9, rgbcolor={149,164,171}),
            string="icb"),
          Text(
            extent=[-96,-66; -86,-74],
            style(color=9, rgbcolor={149,164,171}),
            string="icb"),
          Text(
            extent=[78,72; 88,64],
            style(color=9, rgbcolor={149,164,171}),
            string="icb"),
          Text(
            extent=[54,72; 64,64],
            style(color=9, rgbcolor={149,164,171}),
            string="ieb"),
          Text(
            extent=[-96,-86; -86,-94],
            style(color=9, rgbcolor={149,164,171}),
            string="ieb"),
          Text(
            extent=[-100,-32; -90,-40],
            style(color=9, rgbcolor={149,164,171}),
            string="ieb"),
          Text(
            extent=[-36,-12; -26,-20],
            style(color=9, rgbcolor={149,164,171}),
            string="vcb"),
          Text(
            extent=[48,58; 58,50],
            style(color=9, rgbcolor={149,164,171}),
            string="vcb"),
          Text(
            extent=[78,8; 88,0],
            style(color=9, rgbcolor={149,164,171}),
            string="IC0"),
          Text(
            extent=[-38,-90; -28,-98],
            style(color=9, rgbcolor={149,164,171}),
            string="IB0")),
        Documentation(info="<html>
The PNP element of the bond graph library implements the Ebers-Moll model of a PNP bipolar transistor.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

The PNP bipolar transistor is a directed ThreePort.  The direction of positive power flow is assumed out of the model at the base, <b>B</b>, and at the collector, <b>C</b>, whereas it is assumed into the model at the emitter, <b>E</b>. <p>

The causality of the PNP model is relatively free.  However, if there is a causality stroke placed at the collector from the outside, the substrate capacitor will assume differential causality. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"));
      Interfaces.BondCon B "Base" 
                           annotation (extent=[-110,-10; -90,10]);
      Interfaces.BondCon C "Collector" 
                           annotation (extent=[-10,90; 10,110]);
      Interfaces.BondCon E "Emitter" 
                           annotation (extent=[-10,-110; 10,-90]);
      Junctions.J0p3 J0p3_1 annotation (extent=[-10,-10; 10,10], rotation=-90);
      Bonds.Bond B1 annotation (extent=[-10,50; 10,70], rotation=270);
      Bonds.Bond B2 annotation (extent=[-10,10; 10,30], rotation=270);
      Bonds.Bond B3 annotation (extent=[-10,-30; 10,-10], rotation=90);
      Bonds.Bond B4 annotation (extent=[-10,-70; 10,-50], rotation=90);
      Junctions.J1p3 J1p3_1 annotation (extent=[-10,30; 10,50], rotation=-90);
      Junctions.J0p4 J0p4_1 annotation (extent=[-10,70; 10,90]);
      Junctions.J1p3 J1p3_2 annotation (extent=[-10,-50; 10,-30], rotation=-90);
      Junctions.J0p3 J0p3_2 annotation (extent=[10,-90; -10,-70], rotation=-90);
      Junctions.J1p3 J1p3_3 annotation (extent=[40,-10; 20,10], rotation=-90);
      Bonds.Bond B5 annotation (extent=[20,10; 40,30], rotation=90);
      Bonds.Bond B6 annotation (extent=[20,-30; 40,-10], rotation=90);
      Bonds.Bond B7 annotation (extent=[-30,70; -10,90], rotation=180);
      Passive.C C1(C=Ccs) annotation (extent=[-30,70; -50,90]);
      Bonds.eBond B8 annotation (extent=[40,-10; 60,10], rotation=180);
      Sources.mSf Ic0 annotation (extent=[60,-10; 80,10]);
      Junctions.J0p3 J0p3_3 annotation (extent=[-50,50; -30,30], rotation=-180);
      Bonds.eBond B9 annotation (extent=[-30,30; -10,50], rotation=180);
      Bonds.fBond B10 annotation (extent=[-50,10; -30,30], rotation=-90);
      Sensors.De Vcb annotation (extent=[-50,-10; -30,10], rotation=-90);
      Bonds.eBond B11 annotation (extent=[-70,30; -50,50], rotation=180);
      Utilities.Cj Dcb(
        Is=Is,
        Tau=Taur,
        Cj=Cjc,
        Phi=Phic,
        M=Mc,
        Gb=Gbc,
        Vt=Vt,
        EMin=EMin,
        EMax=EMax) annotation (extent=[-70,30; -90,50]);
      Bonds.eBond B12 annotation (extent=[-30,-50; -10,-30], rotation=180);
      Junctions.J0p3 J0p3_4 
        annotation (extent=[-50,-30; -30,-50], rotation=-180);
      Bonds.eBond B13 annotation (extent=[-50,-70; -30,-50], rotation=90);
      Sources.mSf Ib0 annotation (extent=[-50,-90; -30,-70], rotation=-90);
      Bonds.eBond B14 annotation (extent=[-70,-50; -50,-30], rotation=180);
      Utilities.Cj Deb(
        Is=Is,
        Vt=Vt,
        EMin=EMin,
        EMax=EMax,
        Tau=Tauf,
        Cj=Cje,
        Phi=Phie,
        M=Me,
        Gb=Gbe) annotation (extent=[-70,-50; -90,-30]);
      Utilities.IC0 IC0_1(
        Br=Br,
        Vak=Vak) annotation (extent=[60,40; 80,60], rotation=-90);
      Utilities.IB0 IB0_1(Bf=Bf, Br=Br) annotation (extent=[-80,-90; -60,-70]);
    equation 
      connect(J0p4_1.BondCon4,C)  annotation (points=[0,90; 0,100], style(color=
             8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon2,E)  annotation (points=[-6.12303e-016,-90; 0,-90;
            0,-100], style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1,J0p4_1. BondCon1) annotation (points=[-10.2,80; -10,
            80], style(color=8, rgbcolor={192,192,192}));
      connect(C1.BondCon1,B7. BondCon2) annotation (points=[-30,80; -30,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon3,B8. eBondCon1) annotation (points=[40,
            -6.12303e-016; 44,-6.12303e-016; 44,1.22461e-015; 40,1.22461e-015],
          style(color=8, rgbcolor={192,192,192}));
      connect(Ic0.BondCon1,B8. fBondCon1) annotation (points=[60,0; 64,0; 64,
            -1.22461e-015; 60,-1.22461e-015], style(color=8, rgbcolor={192,192,
              192}));
      connect(B9.fBondCon1,J1p3_1. BondCon3) annotation (points=[-10,40; -10,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon1,B9. eBondCon1) annotation (points=[-30,40; -30,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon3,B10. eBondCon1) annotation (points=[-40,30; -40,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(Vcb.BondCon1,B10. fBondCon1) annotation (points=[-40,10; -40,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(B11.fBondCon1,J0p3_3. BondCon2) annotation (points=[-50,40; -50,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(Dcb.BondCon1,B11. eBondCon1) annotation (points=[-70,40; -70,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B12.fBondCon1,J1p3_2. BondCon3) annotation (points=[-10,-40; -10,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon1,B12. eBondCon1) annotation (points=[-30,-40; -30,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(B13.eBondCon1,J0p3_4. BondCon3) annotation (points=[-40,-50; -40,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(Ib0.BondCon1,B13. fBondCon1) annotation (points=[-40,-70; -40,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B14.fBondCon1,J0p3_4. BondCon2) annotation (points=[-50,-40; -50,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(Deb.BondCon1,B14. eBondCon1) annotation (points=[-70,-40; -70,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Vcb.OutPort1,IC0_1. u3) annotation (points=[-40,-11; -40,-20; -20,
            -20; -20,34; 50,34; 50,50; 60,50], style(color=74, rgbcolor={0,0,
              127}));
      connect(Dcb.y,IC0_1. u1) annotation (points=[-90,40; -110,40; -110,112;
            76,112; 76,62], style(color=74, rgbcolor={0,0,127}));
      connect(Dcb.y,IB0_1. u1) annotation (points=[-90,40; -110,40; -110,-74;
            -82,-74], style(color=74, rgbcolor={0,0,127}));
      connect(Deb.y,IB0_1. u2) annotation (points=[-90,-40; -120,-40; -120,-86;
            -82,-86], style(color=74, rgbcolor={0,0,127}));
      connect(Deb.y,IC0_1. u2) annotation (points=[-90,-40; -120,-40; -120,122;
            64,122; 64,62], style(color=74, rgbcolor={0,0,127}));
      connect(J0p3_1.BondCon3,B)  annotation (points=[-10,-6.12303e-016; -52,
            -6.12303e-016; -52,0; -100,0], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=74,
          rgbfillColor={0,0,185},
          fillPattern=1));
      connect(J1p3_1.BondCon1, B1.BondCon2) annotation (points=[-6.12303e-016,
            50; -1.83691e-015,50], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon1, B2.BondCon2) annotation (points=[-6.12303e-016,
            10; -1.83691e-015,10], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2, B3.BondCon2) annotation (points=[6.12303e-016,
            -10; 6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon2, B4.BondCon2) annotation (points=[6.12303e-016,
            -50; 6.12303e-016,-50], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon1, B5.BondCon1) annotation (points=[30,10; 30,10.2],
          style(color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon2, J0p4_1.BondCon2) annotation (points=[30,30; 30,80;
            10,80], style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon2, J1p3_3.BondCon2) annotation (points=[30,-10; 30,-10],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon3, B6.BondCon1) annotation (points=[10,-80; 30,-80;
            30,-29.8], style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon1, J0p4_1.BondCon3) annotation (points=[1.80017e-015,
            69.8; 1.80017e-015,65.9; 0,65.9; 0,70], style(color=8, rgbcolor={
              192,192,192}));
      connect(B2.BondCon1, J1p3_1.BondCon2) annotation (points=[1.80017e-015,
            29.8; 1.80017e-015,27.9; 6.12303e-016,27.9; 6.12303e-016,30], style(
            color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon1, B3.BondCon1) annotation (points=[-6.12303e-016,
            -30; -6.00057e-016,-30; -6.00057e-016,-29.8], style(color=8,
            rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon1, B4.BondCon1) annotation (points=[6.12303e-016,
            -70; -6.00057e-016,-70; -6.00057e-016,-69.8], style(color=8,
            rgbcolor={192,192,192}));
      connect(IC0_1.y, Ic0.s) annotation (points=[70,39; 70,30; 90,30; 90,0; 79,
            0], style(color=74, rgbcolor={0,0,127}));
      connect(IB0_1.y, Ib0.s) annotation (points=[-59,-80; -50,-80; -50,-96;
            -40,-96; -40,-89], style(color=74, rgbcolor={0,0,127}));
    end PNP;
    
    model PNPth 
      "PNP transistor of the Modelica bond graph library with temperature variation" 
      parameter Real Bf=50 "Forward beta coefficient";
      parameter Real Br=0.1 "Reverse beta coefficient";
      parameter Modelica.SIunits.Temperature Tnom=300.15 
        "Reference temperature";
      parameter Modelica.SIunits.Current Is=1.e-16 
        "Transport saturation current";
      parameter Modelica.SIunits.InversePotential Vak=0.02 
        "Inverse early voltage";
      parameter Modelica.SIunits.Time Tauf=0.12e-9 "Ideal forward transit time";
      parameter Modelica.SIunits.Time Taur=5e-9 "Ideal reverse transit time";
      parameter Real NF=1 "Forward current emission coefficient";
      parameter Real NR=1 "Reverse current emission coefficient";
      parameter Modelica.SIunits.Capacitance Ccs=1e-12 
        "Collector-substrate capacitance";
      parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
        "Base-emitter zero bias depletion capacitance";
      parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
        "Base-collector zero bias depletion capacitance";
      parameter Modelica.SIunits.Voltage Phie=0.8 
        "Base-emitter diffusion voltage";
      parameter Real Me=0.4 "Base-emitter gradation exponent";
      parameter Modelica.SIunits.Voltage Phic=0.8 
        "Base-collector diffusion voltage";
      parameter Real Mc=0.333 "Base-collector gradation exponent";
      parameter Modelica.SIunits.Voltage EG=1.11 
        "Energy gap for temperature effect on saturation current";
      parameter Real XTI=3 "Saturation current temperature exponent";
      parameter Real XTB=0 "Forward and reverse beta temperature coefficient";
      parameter Modelica.SIunits.Conductance Gbc=1e-15 
        "Base-collector leakage conductance";
      parameter Modelica.SIunits.Conductance Gbe=1e-15 
        "Base-emitter leakage conductance";
      parameter Real EMin=-100 "if x < EMin, the exp(x) function is linearized";
      parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
      Interfaces.BondCon B "Base" 
                           annotation (extent=[-110,-10; -90,10]);
      Interfaces.BondCon C "Collector" 
                           annotation (extent=[-10,90; 10,110]);
      Interfaces.BondCon E "Emitter" 
                           annotation (extent=[-10,-110; 10,-90]);
      Junctions.J0p3 J0p3_1 annotation (extent=[-10,-10; 10,10], rotation=-90);
      Bonds.Bond B1 annotation (extent=[-10,50; 10,70], rotation=270);
      Bonds.Bond B2 annotation (extent=[-10,10; 10,30], rotation=270);
      Bonds.Bond B3 annotation (extent=[-10,-30; 10,-10], rotation=90);
      Bonds.Bond B4 annotation (extent=[-10,-70; 10,-50], rotation=90);
      Junctions.J1p3 J1p3_1 annotation (extent=[-10,30; 10,50], rotation=-90);
      Junctions.J0p4 J0p4_1 annotation (extent=[-10,70; 10,90]);
      Junctions.J1p3 J1p3_2 annotation (extent=[-10,-50; 10,-30], rotation=-90);
      Junctions.J0p3 J0p3_2 annotation (extent=[10,-90; -10,-70], rotation=-90);
      Junctions.J1p3 J1p3_3 annotation (extent=[40,-10; 20,10], rotation=-90);
      Bonds.Bond B5 annotation (extent=[20,10; 40,30], rotation=90);
      Bonds.Bond B6 annotation (extent=[20,-30; 40,-10], rotation=90);
      Bonds.Bond B7 annotation (extent=[-30,70; -10,90], rotation=180);
      Passive.C C1(C=Ccs) annotation (extent=[-30,70; -50,90]);
      Bonds.eBond B8 annotation (extent=[40,-10; 60,10], rotation=180);
      Junctions.J0p3 J0p3_3 annotation (extent=[-50,50; -30,30], rotation=-180);
      Bonds.eBond B9 annotation (extent=[-30,30; -10,50], rotation=180);
      Bonds.fBond B10 annotation (extent=[-50,10; -30,30], rotation=-90);
      Sensors.De Vcb annotation (extent=[-50,-10; -30,10], rotation=-90);
      Bonds.eBond B11 annotation (extent=[-70,30; -50,50], rotation=180);
      Bonds.eBond B12 annotation (extent=[-30,-50; -10,-30], rotation=180);
      Junctions.J0p3 J0p3_4 
        annotation (extent=[-50,-30; -30,-50], rotation=-180);
      Bonds.eBond B13 annotation (extent=[-50,-70; -30,-50], rotation=90);
      Bonds.eBond B14 annotation (extent=[-70,-50; -50,-30], rotation=180);
      annotation (
        Diagram(
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Ellipse(extent=[-112,62; -108,58], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Ellipse(extent=[-116,-22; -112,-18], style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Text(
            extent=[-80,60; -70,52],
            style(color=9, rgbcolor={149,164,171}),
            string="icb"),
          Text(
            extent=[-96,-66; -86,-74],
            style(color=9, rgbcolor={149,164,171}),
            string="icb"),
          Text(
            extent=[78,72; 88,64],
            style(color=9, rgbcolor={149,164,171}),
            string="icb"),
          Text(
            extent=[54,72; 64,64],
            style(color=9, rgbcolor={149,164,171}),
            string="ieb"),
          Text(
            extent=[-96,-86; -86,-94],
            style(color=9, rgbcolor={149,164,171}),
            string="ieb"),
          Text(
            extent=[-80,-20; -70,-28],
            style(color=9, rgbcolor={149,164,171}),
            string="ieb"),
          Text(
            extent=[-36,-12; -26,-20],
            style(color=9, rgbcolor={149,164,171}),
            string="vcb"),
          Text(
            extent=[46,56; 56,48],
            style(color=9, rgbcolor={149,164,171}),
            string="vcb"),
          Text(
            extent=[72,18; 82,10],
            style(color=9, rgbcolor={149,164,171}),
            string="IC0"),
          Text(
            extent=[-28,-72; -18,-80],
            style(color=9, rgbcolor={149,164,171}),
            string="IB0"),
          Ellipse(extent=[48,-12; 52,-8],      style(
              color=9,
              rgbcolor={149,164,171},
              fillColor=74,
              rgbfillColor={0,0,185})),
          Text(
            extent=[88,26; 120,0],
            style(color=9, rgbcolor={149,164,171}),
            string="Heat"),
          Text(
            extent=[52,38; 64,30],
            style(color=9, rgbcolor={149,164,171}),
            string="Tdev"),
          Text(
            extent=[-84,-92; -72,-100],
            style(color=9, rgbcolor={149,164,171}),
            string="Tdev")),
        Icon(
          Text(
            extent=[-82,82; 68,-86],
            style(color=0),
            string="PNP"),
          Line(points=[0,80; 0,-80], style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[0,80; 20,60],    style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[-20,60; 0,80],  style(
              color=9,
              rgbcolor={149,164,171},
              thickness=2,
              fillColor=74,
              rgbfillColor={0,0,185},
              fillPattern=1)),
          Line(points=[-72,-50; 72,-50], style(color=3, rgbcolor={0,0,255})),
          Line(points=[72,-50; 50,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[72,-50; 50,-40], style(color=3, rgbcolor={0,0,255})),
          Text(
            extent=[84,28; 116,2],
            style(color=9, rgbcolor={149,164,171}),
            string="Heat"),
          Text(
            extent=[6,110; 26,90],
            style(color=9, rgbcolor={149,164,171}),
            string="C"),
          Text(
            extent=[-108,28; -88,8],
            string="B",
            style(color=9, rgbcolor={149,164,171})),
          Text(
            extent=[6,-90; 26,-110],
            style(color=9, rgbcolor={149,164,171}),
            string="E"),
          Text(
            extent=[58,0; 94,-34],
            string="th",
            style(color=0, rgbcolor={0,0,0}))),
        Documentation(info="<html>
The <b>PNPth</b> element of the bond graph library implements the Ebers-Moll model of a PNP bipolar transistor with temperature variation.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

The <b>PNPth</b> bipolar heating transistor is a directed FourPort.  The direction of positive power flow is assumed out of the model at the base, <b>B</b>, at the collector, <b>C</b>, and at the heat port, <b>Heat</b>, whereas it is assumed into the model at the emitter, <b>E</b>. <p>

The causality of the <b>PNPth</b> model is relatively free.  However, if there is a causality stroke placed at the collector from the outside, the substrate capacitor will assume differential causality. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 NF:    Forward current emission coefficient (default value = 1)

 NR:    Reverse current emission coefficient (default value = 1)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 XTI:   Saturation current temperature exponent (default value = 3)

 XTB:   Forward and reverse beta temperature coefficient (default value = 0)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"));
      Utilities.CjS Dcb(
        Tnom=Tnom,
        Is=Is,
        EG=EG,
        XTI=XTI,
        EMin=EMin,
        EMax=EMax,
        Tau=Taur,
        Cj=Cjc,
        Phi=Phic,
        M=Mc,
        N=NR,
        Gb=Gbc) annotation (extent=[-70,30; -90,50]);
      Utilities.IB02 IB0_1(
        Tnom=Tnom,
        Bf=Bf,
        Br=Br,
        XTB=XTB) annotation (extent=[-80,-90; -60,-70]);
    Modelica.Blocks.Math.Gain Gain1(k=-1) 
      annotation (extent=[-14,-90; -34,-110],  rotation=-90);
      Utilities.IC02 IC0_1(
        Tnom=Tnom,
        Br=Br,
        Vak=Vak,
        XTB=XTB) annotation (extent=[60,40; 80,60], rotation=-90);
      Utilities.CjS Deb(
        Tnom=Tnom,
        Is=Is,
        EG=EG,
        XTI=XTI,
        EMin=EMin,
        EMax=EMax,
        Tau=Tauf,
        Cj=Cje,
        Phi=Phie,
        M=Me,
        N=NF,
        Gb=Gbe) annotation (extent=[-70,-50; -90,-30]);
      Interfaces.BondCon H "Heat" 
                           annotation (extent=[90,-10; 110,10]);
      Bonds.fBond B15 annotation (extent=[56,-60; 76,-40], rotation=90);
      Bonds.eBond B17 annotation (extent=[40,-80; 60,-60],   rotation=0);
      Bonds.eBond B16 annotation (extent=[56,-100; 76,-80],  rotation=90);
      Sensors.De Tdev 
                     annotation (extent=[56,-40; 76,-20],  rotation=90);
      Junctions.J0p6 J0p6_1 annotation (extent=[60,-80; 80,-60]);
    Sources.mSf_int Ic0 annotation (extent=[80,10; 60,-10]);
      Bonds.fBond B18 annotation (extent=[70,-30; 90,-10], rotation=90);
      Bonds.fBond B19 annotation (extent=[74,-90; 94,-70], rotation=0);
    Sources.mSf_int Ib0 
      annotation (extent=[-50,-90; -30,-70],   rotation=90);
    equation 
      connect(J0p4_1.BondCon4,C)  annotation (points=[0,90; 0,100], style(color=
             8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon2,E)  annotation (points=[-6.12303e-016,-90; 0,-90;
            0,-100], style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1,J0p4_1. BondCon1) annotation (points=[-10.2,80; -10,
            80], style(color=8, rgbcolor={192,192,192}));
      connect(C1.BondCon1,B7. BondCon2) annotation (points=[-30,80; -30,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon3,B8. eBondCon1) annotation (points=[40,
            -6.12303e-016; 44,-6.12303e-016; 44,1.22461e-015; 40,1.22461e-015],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.fBondCon1,J1p3_1. BondCon3) annotation (points=[-10,40; -10,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon1,B9. eBondCon1) annotation (points=[-30,40; -30,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon3,B10. eBondCon1) annotation (points=[-40,30; -40,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(Vcb.BondCon1,B10. fBondCon1) annotation (points=[-40,10; -40,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(B11.fBondCon1,J0p3_3. BondCon2) annotation (points=[-50,40; -50,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(B12.fBondCon1,J1p3_2. BondCon3) annotation (points=[-10,-40; -10,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon1,B12. eBondCon1) annotation (points=[-30,-40; -30,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(B13.eBondCon1,J0p3_4. BondCon3) annotation (points=[-40,-50; -40,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(B14.fBondCon1,J0p3_4. BondCon2) annotation (points=[-50,-40; -50,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon3,B)  annotation (points=[-10,-6.12303e-016; -52,
            -6.12303e-016; -52,0; -100,0], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=74,
          rgbfillColor={0,0,185},
          fillPattern=1));
      connect(J1p3_1.BondCon1,B1. BondCon2) annotation (points=[-6.12303e-016,
            50; -1.83691e-015,50], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon1,B2. BondCon2) annotation (points=[-6.12303e-016,
            10; -1.83691e-015,10], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2,B3. BondCon2) annotation (points=[6.12303e-016,
            -10; 6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon2,B4. BondCon2) annotation (points=[6.12303e-016,
            -50; 6.12303e-016,-50], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon1,B5. BondCon1) annotation (points=[30,10; 30,10.2],
          style(color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon2,J0p4_1. BondCon2) annotation (points=[30,30; 30,80;
            10,80], style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon2,J1p3_3. BondCon2) annotation (points=[30,-10; 30,-10],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon3,B6. BondCon1) annotation (points=[10,-80; 30,-80;
            30,-29.8], style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon1,J0p4_1. BondCon3) annotation (points=[1.80017e-015,
            69.8; 1.80017e-015,65.9; 0,65.9; 0,70], style(color=8, rgbcolor={
              192,192,192}));
      connect(B2.BondCon1,J1p3_1. BondCon2) annotation (points=[1.80017e-015,
            29.8; 1.80017e-015,27.9; 0,30; 6.12303e-016,30],              style(
            color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon1,B3. BondCon1) annotation (points=[-6.12303e-016,
            -30; -6.12303e-016,-29.8; -6.00057e-016,-29.8],
                                                          style(color=8,
            rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon1,B4. BondCon1) annotation (points=[6.12303e-016,
            -70; 6.12303e-016,-69.8; -6.00057e-016,-69.8],style(color=8,
            rgbcolor={192,192,192}));
      connect(Dcb.BondCon1, B11.eBondCon1) annotation (points=[-70,40; -70,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(IB0_1.y, Gain1.u) annotation (points=[-59,-80; -56,-80; -56,-116;
            -24,-116; -24,-112], style(color=74, rgbcolor={0,0,127}));
      connect(Dcb.y, IB0_1.u1) annotation (points=[-80,51; -80,60; -110,60;
            -110,-74; -82,-74], style(color=74, rgbcolor={0,0,127}));
      connect(Vcb.OutPort1, IC0_1.u3) annotation (points=[-40,-11; -40,-20; -20,
            -20; -20,56; 60,56], style(color=74, rgbcolor={0,0,127}));
      connect(Dcb.y, IC0_1.u1) annotation (points=[-80,51; -80,60; -110,60;
            -110,114; 76,114; 76,62], style(color=74, rgbcolor={0,0,127}));
      connect(Deb.BondCon1, B14.eBondCon1) annotation (points=[-70,-40; -70,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Deb.y, IB0_1.u2) annotation (points=[-80,-29; -80,-20; -114,-20;
            -114,-86; -82,-86], style(color=74, rgbcolor={0,0,127}));
      connect(Deb.y, IC0_1.u2) annotation (points=[-80,-29; -80,-20; -114,-20;
            -114,110; 64,110; 64,62], style(color=74, rgbcolor={0,0,127}));
      connect(B15.fBondCon1, Tdev.BondCon1) annotation (points=[66,-40; 66,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Tdev.OutPort1, IC0_1.Tdev) annotation (points=[66,-19; 66,-10; 50,
            -10; 50,44; 60,44], style(color=74, rgbcolor={0,0,127}));
      connect(Tdev.OutPort1, IB0_1.Tdev) annotation (points=[66,-19; 66,-10; 50,
            -10; 50,-120; -70,-120; -70,-90], style(color=74, rgbcolor={0,0,127}));
      connect(Deb.BondCon2, B17.fBondCon1) annotation (points=[-90,-40; -118,
            -40; -118,-124; 34,-124; 34,-70; 40,-70], style(color=8, rgbcolor={
              192,192,192}));
      connect(Dcb.BondCon2, B16.fBondCon1) annotation (points=[-90,40; -122,40;
            -122,-128; 66,-128; 66,-100], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon3, B16.eBondCon1) annotation (points=[66,-80; 66,-80], style(
            color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon1, B17.eBondCon1) annotation (points=[60,-70; 60,
            -70], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon5, B15.eBondCon1) annotation (points=[66,-60; 66,
            -60], style(color=8, rgbcolor={192,192,192}));
      connect(Ic0.BondCon2, B8.fBondCon1) annotation (points=[60,0; 63,0; 63,
            -1.22461e-015; 60,-1.22461e-015], style(color=8, rgbcolor={192,192,
              192}));
      connect(J0p6_1.BondCon6, B18.eBondCon1) annotation (points=[74,-60; 74,
            -40; 80,-40; 80,-30], style(color=8, rgbcolor={192,192,192}));
      connect(B18.fBondCon1, Ic0.BondCon1) annotation (points=[80,-10; 80,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon4, B19.eBondCon1) annotation (points=[74,-80; 74,
            -80], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon2, H)  annotation (points=[80,-70; 90,-70; 90,0;
            100,0], style(color=8, rgbcolor={192,192,192}));
      connect(Ib0.BondCon2, B13.fBondCon1) annotation (points=[-40,-70; -40,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B19.fBondCon1, Ib0.BondCon1) annotation (points=[94,-80; 100,-80;
            100,-132; -40,-132; -40,-90], style(color=8, rgbcolor={192,192,192}));
      connect(IC0_1.y, Ic0.s) 
        annotation (points=[70,39; 70,9], style(color=74, rgbcolor={0,0,127}));
      connect(Gain1.y, Ib0.s) annotation (points=[-24,-89; -24,-80; -31,-80], style(
            color=74, rgbcolor={0,0,127}));
    end PNPth;
    
    package Utilities "Utility models for bond graph semiconductor package" 
      annotation (preferedView="info",
        Icon(
          Rectangle(extent=[-100,-100; 80,50], style(
              color=42,
              rgbcolor={127,0,0},
              fillColor=30,
              rgbfillColor={235,235,235},
              fillPattern=1)),
          Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
              color=42,
              rgbcolor={127,0,0},
              fillColor=30,
              rgbfillColor={235,235,235},
              fillPattern=1)),
          Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
              color=42,
              rgbcolor={127,0,0},
              fillColor=30,
              rgbfillColor={235,235,235},
              fillPattern=1)),
          Text(
            extent=[-85,35; 65,-85],
            string="Library",
            style(color=42, rgbcolor={127,0,0})),
          Text(
            extent=[-120,122; 120,73],
            string="%name",
            style(color=1))), Documentation(info="<html>
This package contains some utility models used inside some of the semiconductor bond graph models.
</html>
"));
      block nlC "Non-linear capacitance computation of Ebers-Moll BJT" 
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Modelica.SIunits.Current Is=1.e-16 
          "Transport saturation current";
        parameter Modelica.SIunits.Time Tau=5e-9 "Ideal transit time";
        parameter Modelica.SIunits.Voltage Phi=0.8 "Diffusion voltage";
        parameter Modelica.SIunits.Capacitance Cj=0.5e-12 
          "Zero bias depletion capacitance";
        parameter Real M=0.333 "Gradation exponent";
        parameter Modelica.SIunits.Voltage Vt=0.02585 "Thermal voltage";
        parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
        output Modelica.SIunits.Voltage v 
          "Voltage across the junction capacitance";
        output Modelica.SIunits.Capacitance C 
          "Non-linear junction capacitance value";
        
      protected 
        Modelica.SIunits.Capacitance Cjunc "Junction capacitance";
        Modelica.SIunits.Capacitance Cdepl "Depletion capacitance";
        Real ExMin;
        Real ExMax;
        
        function pow "Just a helper function for x^y" 
          input Real x;
          input Real y;
          output Real z;
        algorithm 
          z:=x^y;
        end pow;
        annotation (
          Icon(Text(
              extent=[-76,64; 78,-64],
              string="nl C",
              style(color=0, rgbcolor={0,0,0})), Text(
              extent=[-80,-52; 84,-76],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT")),
          Diagram(Text(
              extent=[-76,64; 78,-64],
              string="nl C",
              style(color=0, rgbcolor={0,0,0})), Text(
              extent=[-80,-52; 84,-76],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT")),
          Documentation(info="<html>
This block computes the junction capacitance value of a junction capacitance in accordance with the Ebers-Moll junction diode model.  No temperature variation is considered in this model.
</html>
"));
      equation 
        ExMin = exp(EMin);
        ExMax = exp(EMax);
        v = u;
        Cjunc = if (v/Vt < EMin) then Tau*Is/Vt*ExMin*(v/Vt - EMin + 1) else 
                if (v/Vt > EMax) then Tau*Is/Vt*ExMax*(v/Vt - EMax + 1) else Tau*Is/Vt*exp(v/Vt);
        Cdepl = if (v/Phi > 0) then Cj*(1 + M*v/Phi) else Cj*pow(1 - v/Phi, -M);
        C = Cjunc + Cdepl;
        y = C;
      end nlC;
      
      block nlC2 
        "Non-linear capacitance computation of Ebers-Moll BJT with thermal port" 
        extends Modelica.Blocks.Interfaces.SISO;
        constant Real k=1.3806226e-23 "Boltzmann's constant";
        constant Modelica.SIunits.ElectricCharge q=1.6021918e-19 
          "Electron charge";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
        parameter Modelica.SIunits.Current Is=1.e-16 
          "Transport saturation current";
        parameter Modelica.SIunits.Time Tau=5e-9 "Ideal transit time";
        parameter Modelica.SIunits.Voltage Phi=0.8 "Diffusion voltage";
        parameter Modelica.SIunits.Capacitance Cj=0.5e-12 
          "Zero bias depletion capacitance";
        parameter Real M=0.333 "Gradation exponent";
        parameter Modelica.SIunits.Voltage EG=1.11 
          "Energy gap for temperature effect on saturation current";
        parameter Real N=1 "Current emission coefficient";
        parameter Real XTI=3 "Saturation current temperature exponent";
        parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
        output Modelica.SIunits.Voltage v 
          "Voltage across the junction capacitance";
        output Modelica.SIunits.Capacitance C 
          "Non-linear junction capacitance value";
        output Modelica.SIunits.Voltage Vt "Thermal voltage";
        output Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
        
      protected 
        Modelica.SIunits.Capacitance Cjunc "Junction capacitance";
        Modelica.SIunits.Capacitance Cdepl "Depletion capacitance";
        Real RTemp "Temperature quotient";
        Real ExMin;
        Real ExMax;
        Real et;
        
        function pow "Just a helper function for x^y" 
          input Real x;
          input Real y;
          output Real z;
        algorithm 
          z:=x^y;
        end pow;
        annotation (
          Diagram(Text(
              extent=[-78,74; 76,-54],
              string="nl C",
              style(color=0, rgbcolor={0,0,0})), Text(
              extent=[-82,-42; 82,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT")),
          Icon(   Text(
              extent=[-78,76; 76,-52],
              string="nl C",
              style(color=0, rgbcolor={0,0,0})), Text(
              extent=[-82,-40; 82,-64],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT")),
          Documentation(info="<html>
This block computes the junction capacitance value of a junction capacitance in accordance with the Ebers-Moll junction diode model.  Temperature variation is considered in this model.
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
              Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[-10,-110; 10,-90], rotation=90);
      equation 
        /* Compute thermal voltage as function of temperature */
        Vt = k*Tdev/q;
        
        /* Compute temperature dependence of saturation current */
        RTemp = Tdev/Tnom;
        ISval = Is*exp((RTemp - 1)*EG/Vt + XTI*ln(RTemp));
        
        /* Compute diode characteristic */
        ExMin = exp(EMin);
        ExMax = exp(EMax);
        v = u;
        et = v/(N*Vt);
        Cjunc = Tau*ISval/Vt*(if et < EMin then ExMin*(et - EMin + 1) else 
                              if et > EMax then ExMax*(et - EMax + 1) else exp(et));
        Cdepl = if (v/Phi > 0) then Cj*(1 + M*v/Phi) else Cj*pow(1 - v/Phi, -M);
        C = Cjunc + Cdepl;
        y = C;
      end nlC2;
      
      block IB0 "Non-linear base-emitter current source of the Ebers-Moll BJT" 
        extends Modelica.Blocks.Interfaces.SI2SO;
        parameter Real Bf=50 "Forward beta coefficient";
        parameter Real Br=0.1 "Reverse beta coefficient";
        output Modelica.SIunits.Current ibc "Base-collector diode current";
        output Modelica.SIunits.Current ibe "Base-emitter diode current";
        output Modelica.SIunits.Current IB0 "Injected base current";
        annotation (
          Icon(Text(
              extent=[-72,62; 82,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IB 0"), Text(
              extent=[-76,-54; 88,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT"),
            Text(
              extent=[-96,68; -76,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[-96,-52; -76,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe")),
          Diagram(Text(
              extent=[-72,62; 82,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IB 0"), Text(
              extent=[-76,-54; 88,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT"),
            Text(
              extent=[-96,68; -76,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[-96,-52; -76,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe")),
          Documentation(info="<html>
This block computes the injected base current of a BJT in accordance with the Ebers-Moll model. <p>

<pre>
Parameters:

 Bf:   Forward beta coefficient (default value = 50)

 Br:   Reverse beta coefficient (default value = 0.1)
</pre>
</html>
"));
      equation 
        ibc = u1;
        ibe = u2;
        IB0 = ibe/Bf + ibc/Br;
        y = IB0;
      end IB0;
      
      block IB02 
        "Non-linear base-emitter current source of the Ebers-Moll BJT with thermal variation" 
        extends Modelica.Blocks.Interfaces.SI2SO;
        parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
        parameter Real Bf=50 "Forward beta coefficient";
        parameter Real Br=0.1 "Reverse beta coefficient";
        parameter Real XTB=0 "Forward and reverse beta temperature coefficient";
        output Modelica.SIunits.Current ibc "Base-collector diode current";
        output Modelica.SIunits.Current ibe "Base-emitter diode current";
        output Modelica.SIunits.Current IB0 "Injected base current";
        
      protected 
        Real RTemp "Temperature quotient";
        Real Rxtb "Exponentiated temperature quotient";
        Real BFval "Forward beta coefficient at device temperature";
        Real BRval "Backward beta coefficient at device temperature";
        annotation (Icon(
            Text(
              extent=[-96,68; -76,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[-96,-52; -76,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe"),
               Text(
              extent=[-72,62; 82,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IB 0"), Text(
              extent=[-76,-54; 88,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT")), Diagram(
            Text(
              extent=[-96,68; -76,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[-96,-52; -76,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe"),
               Text(
              extent=[-72,62; 82,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IB 0"), Text(
              extent=[-76,-54; 88,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT")),
          Documentation(info="<html>
This block computes the injected base current of a BJT with temperature variation in accordance with the Ebers-Moll model. <p>

<pre>
Parameters:

 Bf:   Forward beta coefficient (default value = 50)

 Br:   Reverse beta coefficient (default value = 0.1)

 XTB:  Forward and reverse beta temperature coefficient (default value = 0)
</pre>
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
              Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[-20,-120; 20,-80], rotation=90);
      equation 
        RTemp = Tdev/Tnom;
        Rxtb = RTemp^XTB;
        BFval = Bf*Rxtb;
        BRval = Br*Rxtb;
        ibc = u1;
        ibe = u2;
        IB0 = ibe/BFval + ibc/BRval;
        y = IB0;
      end IB02;
      
      block IC0 
        "Non-linear collector-emitter current source of the Ebers-Moll BJT" 
        extends Modelica.Blocks.Interfaces.SI2SO;
        parameter Real Br=0.1 "Reverse beta coefficient";
        parameter Modelica.SIunits.InversePotential Vak=0.02 
          "Inverse early voltage";
        output Modelica.SIunits.Current ibc "Base-collector diode current";
        output Modelica.SIunits.Current ibe "Base-emitter diode current";
        output Modelica.SIunits.Voltage vbc "Base-collector voltage";
        output Modelica.SIunits.Current IC0 "Injected collector current";
        
      protected 
        Real qbk "Inverse base charge";
        annotation (
          Icon(Text(
              extent=[-72,62; 82,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IC 0"), Text(
              extent=[-76,-54; 88,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT"),
            Text(
              extent=[-96,-52; -76,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe"),
            Text(
              extent=[-96,68; -76,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[10,-82; 32,-98],
              style(color=3, rgbcolor={0,0,255}),
              string="vbc")),
          Diagram(Text(
              extent=[-72,62; 82,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IC 0"), Text(
              extent=[-76,-54; 88,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT"),
            Text(
              extent=[-96,-52; -76,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe"),
            Text(
              extent=[-96,68; -76,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[10,-82; 32,-98],
              style(color=3, rgbcolor={0,0,255}),
              string="vbc")),
          Documentation(info="<html>
This block computes the injected collector current of a BJT in accordance with the Ebers-Moll model. <p>

<pre>
Parameters:

 Br:   Reverse beta coefficient (default value = 0.1)

 Vak:  Inverse early voltage (default value = 0.02 /Volt)
</pre>
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput u3 
          "Connector of Real input signal 2" 
          annotation (extent=[-20,-120; 20,-80], rotation=90);
      equation 
        ibc = u1;
        ibe = u2;
        vbc = u3;
        qbk = 1 - vbc*Vak;
        IC0 = (ibe - ibc)*qbk - ibc/Br;
        y = IC0;
      end IC0;
      
      block IC02 
        "Non-linear collector-emitter current source of the Ebers-Moll BJT with thermal variation" 
        extends Modelica.Blocks.Interfaces.SI2SO;
        parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
        parameter Real Br=0.1 "Reverse beta coefficient";
        parameter Modelica.SIunits.InversePotential Vak=0.02 
          "Inverse early voltage";
        parameter Real XTB=0 "Forward and reverse beta temperature coefficient";
        output Modelica.SIunits.Current ibc "Base-collector diode current";
        output Modelica.SIunits.Current ibe "Base-emitter diode current";
        output Modelica.SIunits.Voltage vbc "Base-collector voltage";
        output Modelica.SIunits.Current IC0 "Injected collector current";
        
      protected 
        Real RTemp "Temperature quotient";
        Real Rxtb "Exponentiated temperature quotient";
        Real BRval "Backward beta coefficient at device temperature";
        Real qbk "Inverse base charge";
      public 
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
              Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[40,-120; 80,-80], rotation=90);
        annotation (Diagram(
            Text(
              extent=[-98,68; -78,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[-98,-52; -78,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe"),
                  Text(
              extent=[-74,62; 80,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IC 0"), Text(
              extent=[-78,-54; 86,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT"),
            Text(
              extent=[-50,-82; -28,-98],
              style(color=3, rgbcolor={0,0,255}),
              string="vbc")), Icon(
            Text(
              extent=[-98,68; -78,52],
              style(color=3, rgbcolor={0,0,255}),
              string="ibc"),
            Text(
              extent=[-98,-52; -78,-68],
              style(color=3, rgbcolor={0,0,255}),
              string="ibe"),
                  Text(
              extent=[-74,62; 80,-66],
              style(color=0, rgbcolor={0,0,0}),
              string="IC 0"), Text(
              extent=[-78,-54; 86,-78],
              style(color=0, rgbcolor={0,0,0}),
              string="Ebers-Moll BJT"),
            Text(
              extent=[-50,-82; -28,-98],
              style(color=3, rgbcolor={0,0,255}),
              string="vbc")),
          Documentation(info="<html>
This block computes the injected collector current of a BJT with temperature variation in accordance with the Ebers-Moll model. <p>

<pre>
Parameters:

 Br:   Reverse beta coefficient (default value = 0.1)

 Vak:  Inverse early voltage (default value = 0.02 /Volt)

 XTB:  Forward and reverse beta temperature coefficient (default value = 0)
</pre>
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput u3 
          "Connector of Real input signal 3" 
          annotation (extent=[-80,-120; -40,-80],rotation=90);
      equation 
        RTemp = Tdev/Tnom;
        Rxtb = RTemp^XTB;
        BRval = Br*Rxtb;
        ibc = u1;
        ibe = u2;
        vbc = u3;
        qbk = 1 - vbc*Vak;
        IC0 = (ibe - ibc)*qbk - ibc/BRval;
        y = IC0;
      end IC02;
      
      model mC2 "Bondgraphic modulated capacitor element" 
        extends Interfaces.ModPassiveOnePort;
        Real C "Bondgraphic Capacitance";
        annotation (
          Icon(Text(
              extent=[-66,78; 54,-82],
              style(color=0),
              string="mC"),    Text(extent=[-72,80; 68,40],   string="%name"),
               Text(
              extent=[50,-2; 80,-54],
              style(color=0),
              string="2")),
          Diagram(
               Text(
              extent=[50,-2; 80,-54],
              style(color=0),
              string="2"),
               Text(
              extent=[-66,78; 54,-82],
              style(color=0),
              string="mC")),
          Documentation(info="<html>
This is an alternative model to the <b>mC</b> modulated capacitance model of the bond graph library.  This model is physically dubious.  Modelers are advised to always use the <b>mC</b> model, never the <b>mC2</b> model.  However, since this is the model that is being used in the Ebers-Moll description of bipolar transistors, it is being reproduced here.  However, it is hidden under utilities to discourage modelers from using it. <p>

<hr> <p>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 C:  Capacitance (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = C*der(e)</b>
</html>
"));
      equation 
        C = s;
        f = C*der(e);
      end mC2;
      
      model Cj "Junction capacitance model of Ebers-Moll BJT" 
        extends Interfaces.PassiveOnePort;
        parameter Modelica.SIunits.Current Is=1.e-16 
          "Transport saturation current";
        parameter Modelica.SIunits.Time Tau=5e-9 "Ideal transit time";
        parameter Modelica.SIunits.Capacitance Cj=0.5e-12 
          "Zero bias depletion capacitance";
        parameter Modelica.SIunits.Voltage Phi=0.8 "Diffusion voltage";
        parameter Real M=0.333 "Gradation exponent";
        parameter Modelica.SIunits.Conductance Gb=1e-15 "Leakage conductance";
        parameter Modelica.SIunits.Voltage Vt=0.02585 "Thermal voltage";
        parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
        Modelica.Blocks.Interfaces.RealOutput y 
          "Connector of Real output signal" 
          annotation (extent=[90,-10; 110,10]);
        annotation (
          Icon(Text(
              extent=[2,-6; 92,-72],
              style(color=0),
              string="Junction"),
               Text(
              extent=[-74,62; 12,-70],
              style(color=0),
              string="C"),     Text(extent=[-70,94; 70,54],   string="%name")),
          Diagram,
          Documentation(info="<html>
The junction capacitance element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  Junction capacitances are used in semiconductor technology to model the charge accumulated in the vicinity of a pn-junction. <p>

The junction capacitance element has its preferred causality away from the element. <p>

The model also computes the diodic current through the junction and puts it out as an output signal. <p>

<hr>

<pre>
Potential variables:

 e:     Bondgraphic effort variable

 f:     Bondgraphic flow variable, normalized positive for flows into the model

 id:    Diodic current through the junction (output variable)


Parameters:

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Tau:   Ideal transit time (default value = 5e-9 sec)

 Cj:    Zero bias depletion capacitance (default value = 0.5e-12 F)

 Phi:   Diffusion voltage (default value = 0.8 Volt)

 M:     Gradation exponent (default value = 0.333)

 Gb:    Leakage conductance (default value = 1e-15 Mho)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre>
</html>
"));
        Junctions.J0p3 J0p3_1 annotation (extent=[-90,10; -70,-10]);
        Bonds.fBond B1 annotation (extent=[-70,-10; -50,10]);
        Sensors.De vbi annotation (extent=[-50,-10; -30,10]);
        Bonds.eBond B2 annotation (extent=[-90,10; -70,30], rotation=90);
        mC2 Cj1 annotation (extent=[-90,30; -70,50], rotation=90);
        Sources.mSe Vbi annotation (extent=[0,-10; -20,10]);
        Bonds.fBond B3 annotation (extent=[0,-10; 20,10]);
        Junctions.J1p3 J1p3_1 annotation (extent=[20,-10; 40,10]);
        Bonds.eBond B4 annotation (extent=[40,-10; 60,10], rotation=0);
        Sensors.Df id annotation (extent=[60,-10; 80,10]);
        Bonds.fBond B5 annotation (extent=[20,-30; 40,-10], rotation=270);
        Junctions.J0p3 J0p3_2 annotation (extent=[20,-30; 40,-50]);
        Bonds.fBond B6 annotation (extent=[40,-50; 60,-30], rotation=0);
        Bonds.fBond B7 annotation (extent=[0,-50; 20,-30], rotation=180);
        Passive.G G1(G=Gb) annotation (extent=[60,-50; 80,-30]);
        nlC nlC1(
          Is=Is,
          Tau=Tau,
          Phi=Phi,
          Cj=Cj,
          M=M,
          Vt=Vt,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-40,50; -60,70]);
        Dj Dbi(
          Is=Is,
          Vt=Vt,
          EMin=EMin,
          EMax=EMax) annotation (extent=[0,-50; -20,-30]);
      equation 
        connect(J0p3_1.BondCon1, BondCon1) annotation (points=[-90,0; -100,0], style(
              color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon2, B1.eBondCon1) annotation (points=[-70,0; -70,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B1.fBondCon1, vbi.BondCon1) annotation (points=[-50,0; -50,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon3, B2.fBondCon1) annotation (points=[-80,10; -80,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(B2.eBondCon1, Cj1.BondCon1) annotation (points=[-80,30; -80,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(Vbi.BondCon1, B3.eBondCon1) annotation (points=[0,0; 0,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B3.fBondCon1, J1p3_1.BondCon1) annotation (points=[20,0; 20,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon2, B4.fBondCon1) annotation (points=[40,0; 40,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B4.eBondCon1, id.BondCon1) annotation (points=[60,0; 60,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(id.OutPort1, y) annotation (points=[81,0; 100,0], style(color=
                74, rgbcolor={0,0,127}));
        connect(J0p3_2.BondCon2, B6.eBondCon1) annotation (points=[40,-40; 40,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B7.eBondCon1, J0p3_2.BondCon1) annotation (points=[20,-40; 20,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B6.fBondCon1, G1.BondCon1) annotation (points=[60,-40; 60,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon3, B5.eBondCon1) annotation (points=[30,-10; 30,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(B5.fBondCon1, J0p3_2.BondCon3) annotation (points=[30,-30; 30,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(vbi.OutPort1, nlC1.u) annotation (points=[-29,0; -24,0; -24,60;
              -38,60], style(color=74, rgbcolor={0,0,127}));
        connect(Dbi.BondCon1, B7.fBondCon1) annotation (points=[0,-40; 0,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(vbi.OutPort1, Vbi.s) annotation (points=[-29,0; -19,0], style(
              color=74, rgbcolor={0,0,127}));
        connect(nlC1.y, Cj1.s) annotation (points=[-61,60; -80,60; -80,49],
            style(color=74, rgbcolor={0,0,127}));
      end Cj;
      
      model CjS 
        "Junction capacitance model of Ebers-Moll BJT with thermal port" 
        extends Interfaces.TwoPort;
        parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
        parameter Modelica.SIunits.Current Is=1.e-16 
          "Transport saturation current";
        parameter Modelica.SIunits.Time Tau=5e-9 "Ideal transit time";
        parameter Modelica.SIunits.Capacitance Cj=0.5e-12 
          "Zero bias depletion capacitance";
        parameter Modelica.SIunits.Voltage Phi=0.8 "Diffusion voltage";
        parameter Real M=0.333 "Gradation exponent";
        parameter Modelica.SIunits.Voltage EG=1.11 
          "Energy gap for temperature effect on saturation current";
        parameter Real N=1 "Current emission coefficient";
        parameter Real XTI=3 "Saturation current temperature exponent";
        parameter Modelica.SIunits.Conductance Gb=1e-15 "Leakage conductance";
        parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
        Modelica.Blocks.Interfaces.RealOutput y 
          "Connector of Real output signal" 
          annotation (extent=[-10,100; 10,120], rotation=90);
        Junctions.J0p3 J0p3_1 annotation (extent=[-90,10; -70,-10]);
        Bonds.fBond B1 annotation (extent=[-70,-10; -50,10]);
        Sensors.De vbi annotation (extent=[-50,-10; -30,10]);
        Bonds.eBond B2 annotation (extent=[-90,10; -70,30], rotation=90);
        mC2 Cj1 annotation (extent=[-90,30; -70,50], rotation=90);
        Sources.mSe Vbi annotation (extent=[0,-10; -20,10]);
        Bonds.fBond B3 annotation (extent=[0,-10; 20,10]);
        Junctions.J1p3 J1p3_1 annotation (extent=[20,-10; 40,10], rotation=-90);
        Bonds.eBond B4 annotation (extent=[20,10; 40,30],  rotation=90);
        Sensors.Df id annotation (extent=[20,30; 40,50], rotation=90);
        Bonds.fBond B5 annotation (extent=[20,-30; 40,-10], rotation=270);
        Junctions.J0p3 J0p3_2 annotation (extent=[20,-30; 40,-50]);
        Bonds.fBond B6 annotation (extent=[40,-50; 60,-30], rotation=0);
        Bonds.fBond B7 annotation (extent=[0,-50; 20,-30], rotation=180);
        Passive.G G1(G=Gb) annotation (extent=[0,-50; -20,-30]);
        nlC2 nlC1(
          Tnom=Tnom,
          Is=Is,
          Tau=Tau,
          Phi=Phi,
          Cj=Cj,
          M=M,
          EG=EG,
          N=N,
          XTI=XTI,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-40,50; -60,70]);
        DjS DjS1(
          Tnom=Tnom,
          Is=Is,
          EG=EG,
          N=N,
          XTI=XTI,
          EMin=EMin,
          EMax=EMax) annotation (extent=[60,-50; 80,-30]);
        Junctions.J0p3 J0p3_3 annotation (extent=[90,10; 70,-10], rotation=-90);
        Bonds.fBond B8 annotation (extent=[70,10; 90,30], rotation=90);
        Sensors.De Tdev 
                       annotation (extent=[70,30; 90,50], rotation=90);
        Bonds.eBond B9 annotation (extent=[70,-30; 90,-10],rotation=90);
      equation 
        connect(J0p3_1.BondCon2,B1. eBondCon1) annotation (points=[-70,0; -70,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B1.fBondCon1,vbi. BondCon1) annotation (points=[-50,0; -50,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon3,B2. fBondCon1) annotation (points=[-80,10; -80,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(B2.eBondCon1,Cj1. BondCon1) annotation (points=[-80,30; -80,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(Vbi.BondCon1,B3. eBondCon1) annotation (points=[0,0; 0,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon2,B6. eBondCon1) annotation (points=[40,-40; 40,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B7.eBondCon1,J0p3_2. BondCon1) annotation (points=[20,-40; 20,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B5.fBondCon1,J0p3_2. BondCon3) annotation (points=[30,-30; 30,
              -30], style(color=8, rgbcolor={192,192,192}));
        annotation (Diagram,
          Icon(                Text(extent=[-68,92; 72,52],   string="%name"),
               Text(
              extent=[-88,64; 20,-68],
              style(color=0),
              string="CS"),
               Text(
              extent=[8,4; 98,-62],
              style(color=0),
              string="Junction")),
          Documentation(info="<html>
The junction capacitance source element is a directed TwoPort.  It inherits the effort and flow variables from the directed TwoPort.  Junction capacitances are used in semiconductor technology to model the charge accumulated in the vicinity of a pn-junction. <p>

The junction capacitance element has its preferred causality away from the element. <p>

The model also computes the diodic current through the junction and puts it out as an output signal. <p>

<hr>

<pre>
Potential variables:

 e1:    Bondgraphic effort variable of inflow

 f1:    Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:    Bondgraphic effort variable of outflow

 f2:    Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 id:    Diodic current through the junction (output variable)


Parameters:

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Tau:   Ideal transit time (default value = 5e-9 sec)

 Cj:    Zero bias depletion capacitance (default value = 0.5e-12 F)

 Phi:   Diffusion voltage (default value = 0.8 Volt)

 M:     Gradation exponent (default value = 0.333)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 N:     Current emission coefficient (default value = 1)

 XTI:   Saturation current temperature exponent (default value = 3)

 Gb:    Leakage conductance (default value = 1e-15 Mho)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre>
</html>
"));
        connect(J0p3_1.BondCon1, BondCon1) annotation (points=[-90,0; -100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(vbi.OutPort1, nlC1.u) annotation (points=[-29,0; -24,0; -24,60;
              -38,60], style(color=74, rgbcolor={0,0,127}));
        connect(G1.BondCon1, B7.fBondCon1) annotation (points=[0,-40; 0,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(B6.fBondCon1, DjS1.BondCon1) annotation (points=[60,-40; 60,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(B5.eBondCon1, J1p3_1.BondCon2) annotation (points=[30,-10; 30,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(B3.fBondCon1, J1p3_1.BondCon3) annotation (points=[20,0; 23,0;
              23,-6.12303e-016; 20,-6.12303e-016], style(color=8, rgbcolor={192,
                192,192}));
        connect(B4.fBondCon1, J1p3_1.BondCon1) annotation (points=[30,10; 30,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(id.BondCon1, B4.eBondCon1) annotation (points=[30,30; 30,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(id.OutPort1, y) annotation (points=[30,51; 30,80; 0,80; 0,110],
            style(color=74, rgbcolor={0,0,127}));
        connect(J0p3_3.BondCon3, BondCon2) annotation (points=[90,6.12303e-016;
              92,6.12303e-016; 92,0; 100,0], style(color=8, rgbcolor={192,192,
                192}));
        connect(B8.eBondCon1, J0p3_3.BondCon2) annotation (points=[80,10; 80,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.BondCon1, B8.fBondCon1) annotation (points=[80,30; 80,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.OutPort1, nlC1.Tdev) annotation (points=[80,51; 80,60; 0,
              60; 0,40; -50,40; -50,50], style(color=74, rgbcolor={0,0,127}));
        connect(B9.eBondCon1, J0p3_3.BondCon1) annotation (points=[80,-10; 80,-10],
            style(color=8, rgbcolor={192,192,192}));
        connect(DjS1.BondCon2, B9.fBondCon1) annotation (points=[80,-40; 80,-30],
            style(color=8, rgbcolor={192,192,192}));
        connect(vbi.OutPort1, Vbi.s) annotation (points=[-29,0; -19,0], style(
              color=74, rgbcolor={0,0,127}));
        connect(nlC1.y, Cj1.s) annotation (points=[-61,60; -80,60; -80,49],
            style(color=74, rgbcolor={0,0,127}));
      end CjS;
      
      model Dj "Junction diode model of Ebers-Moll BJT" 
        extends Interfaces.PassiveOnePort;
        parameter Modelica.SIunits.Current Is=1.e-16 
          "Transport saturation current";
        parameter Modelica.SIunits.Voltage Vt=0.02585 
          "Voltage equivalent of temperature (kT/qn)";
        parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
        
      protected 
        Real ExMin;
        Real ExMax;
        annotation (
          Icon(Text(
              extent=[-76,64; 10,-68],
              style(color=0),
              string="D"),
               Text(
              extent=[-4,-4; 86,-70],
              style(color=0),
              string="Junction"),
                               Text(extent=[-70,96; 70,56],   string="%name")),
          Diagram(
            Line(points=[-80,0; 80,0],   style(color=10)),
            Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                  fillColor=10)),
            Line(points=[0,80; 0,-80],   style(
                color=10,
                fillColor=10,
                fillPattern=1)),
            Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
                color=10,
                fillColor=10,
                fillPattern=1)),
            Text(
              extent=[4,82; 14,72],
              style(
                color=10,
                fillColor=10,
                fillPattern=1),
              string="f"),
            Text(
              extent=[70,-4; 80,-14],
              style(
                color=10,
                fillColor=10,
                fillPattern=1),
              string="e"),
            Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
                color=0,
                thickness=2,
                fillColor=10,
                fillPattern=1)),
            Text(
              extent=[-66,-6; -36,-14],
              style(
                color=10,
                pattern=3,
                fillColor=10,
                fillPattern=1),
              string="blocking"),
            Text(
              extent=[16,50; 48,42],
              style(
                color=10,
                pattern=3,
                fillColor=10,
                fillPattern=1),
              string="conducting"),
            Line(points=[0,0; 6,10; 12,28; 16,66],            style(
                color=0,
                thickness=2,
                fillColor=10,
                fillPattern=1))),
          Documentation(info="<html>
The junction diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  Junction diodes are a special class of non-linear resistors. <p>

The junction diode model is a simple diode with an exponential characteristic. The diode formula essentially is: <p>

<b>f = Is*(exp(e/Vt) - 1)</b> <p>

However, if the exponent <b>e/Vt</b> reaches either an upper limit <b>EMax</b> or a lower limit <b>Emin</b>, the diode characterisic is linearly continued to avoid overflow or underflow, respectively. <p>

The causality of the junction diode element is free. <p>

<hr>

<pre>
Potential variables:

 e:     Bondgraphic effort variable

 f:     Bondgraphic flow variable, normalized positive for flows into the model


Parameters:

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

Equations: <p>

<b>ExMin = exp(EMin)</b> <br>
<b>ExMax = exp(EMax)</b> <br>
<b>f = if (e/Vt &lt; EMin) then Is*(ExMin*(e/Vt - EMin + 1) - 1) else 
       if (e/Vt &gt; EMax) then Is*(ExMax*(e/Vt - EMax + 1) - 1) else Is*(exp(e/Vt) - 1)</b>
</html>
"));
      equation 
        ExMin = exp(EMin);
        ExMax = exp(EMax);
        f = if (e/Vt < EMin) then Is*(ExMin*(e/Vt - EMin + 1) - 1) else 
            if (e/Vt > EMax) then Is*(ExMax*(e/Vt - EMax + 1) - 1) else Is*(exp(e/Vt) - 1);
      end Dj;
      
      model DjS "Ebers-Moll junction diode model for bipolar transistors" 
        extends Interfaces.TwoPort;
        constant Real k=1.3806226e-23 "Boltzmann's constant";
        constant Modelica.SIunits.ElectricCharge q=1.6021918e-19 
          "Electron charge";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
        parameter Modelica.SIunits.Current Is=1e-16 
          "Saturation current at reference temperature";
        parameter Modelica.SIunits.Voltage EG=1.11 
          "Energy gap for temperature effect on saturation current";
        parameter Real N=1 "Current emission coefficient";
        parameter Real XTI=3 "Saturation current temperature exponent";
        parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 "if x > EMax, the exp(x) function is linearized";
        Modelica.SIunits.Voltage Vt "Thermal voltage";
        Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
        
      protected 
        Real RTemp "Temperature quotient";
        Real ExMin;
        Real ExMax;
        Real et;
        annotation (
          Diagram(
            Line(points=[-80,0; 80,0],   style(color=10)),
            Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                  fillColor=10)),
            Line(points=[0,80; 0,-80],   style(
                color=10,
                fillColor=10,
                fillPattern=1)),
            Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
                color=10,
                fillColor=10,
                fillPattern=1)),
            Text(
              extent=[4,82; 14,72],
              style(
                color=10,
                fillColor=10,
                fillPattern=1),
              string="f"),
            Text(
              extent=[70,-4; 80,-14],
              style(
                color=10,
                fillColor=10,
                fillPattern=1),
              string="e"),
            Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
                color=0,
                thickness=2,
                fillColor=10,
                fillPattern=1)),
            Text(
              extent=[-66,-6; -36,-14],
              style(
                color=10,
                pattern=3,
                fillColor=10,
                fillPattern=1),
              string="blocking"),
            Text(
              extent=[16,50; 48,42],
              style(
                color=10,
                pattern=3,
                fillColor=10,
                fillPattern=1),
              string="conducting"),
            Line(points=[0,0; 6,10; 12,28; 16,66],            style(
                color=0,
                thickness=2,
                fillColor=10,
                fillPattern=1))),
          Icon(Text(
              extent=[-84,74; 20,-82],
              style(color=0),
              string="DS"),
               Text(
              extent=[8,8; 98,-58],
              style(color=0),
              string="Junction"),
                               Text(extent=[-72,102; 68,62],  string="%name")),
          Documentation(info="<html>
The junction diodic source element is a directed TwoPort.  It inherits the effort and flow variables from the directed TwoPort.  Junction diodes are a special class of non-linear resistors. <p>

The junction diode model is a simple diode with an exponential characteristic. The diode formula essentially is: <p>

<b>f = Is*(exp(e/Vt) - 1)</b> <p>

However, if the exponent <b>e/Vt</b> reaches either an upper limit <b>EMax</b> or a lower limit <b>Emin</b>, the diode characterisic is linearly continued to avoid overflow or underflow, respectively. <p>

The causality of the junction diode element is free. <p>

<hr>

<pre>
Potential variables:

 e1:    Bondgraphic effort variable of inflow

 f1:    Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:    Bondgraphic effort variable of outflow

 f2:    Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 Tnom:  Reference temperature (default value = 300.15 K)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 N:     Current emission coefficient (default value = 1)

 XTI:   Saturation current temperature exponent (default value = 3)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

Equations: <p>

<b>RTemp = e2/Tnom</b> <br>
<b>ISval = IS*exp((RTemp - 1)*EG/Vt + XTI*ln(RTemp))</b> <br>
<b>ExMin = exp(EMin)</b> <br>
<b>ExMax = exp(EMax)</b> <br>
<b>et = e1/(Vt*N)</b> <br>
<b>f1 = ISval*(if et &lt; EMin then ExMin*(et - EMin + 1) - 1 else 
               if et &gt; EMax then ExMax*(et - EMax + 1) - 1 else exp(et) - 1)</b> <br>
<b>f2 = 0</b>
</html>
"));
      equation 
        /* Compute thermal voltage as function of temperature */
        Vt = k*e2/q;
        
        /* Compute temperature dependence of saturation current */
        RTemp = e2/Tnom;
        ISval = Is*exp((RTemp - 1)*EG/Vt + XTI*ln(RTemp));
        
        /* Compute diode characteristic */
        ExMin = exp(EMin);
        ExMax = exp(EMax);
        et = e1/(Vt*N);
        f1 = ISval*(if et < EMin then ExMin*(et - EMin + 1) - 1 else 
                    if et > EMax then ExMax*(et - EMax + 1) - 1 else exp(et) - 1);
        
        /* Compute heat flow */
        f2 = 0;
      end DjS;
    end Utilities;
  end Semiconductors;
  
  package Sensors "Sensors (detectors) of Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30, fillPattern=
                1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
              fillColor=30, fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
              fillColor=30, fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=3)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0,
        y=0.6,
        width=0.4,
        height=0.4,
        library=1,
        autolayout=1),
      Documentation(info="<html>
Sensors (detectors) are not among the standard bond graph elements advertized regularly in the bond graph literature.  Yet, they are very useful elements indeed, as they enable the modeler to convert bond graph variables to signal variables.  This is useful for modeling blocks of block diagrams internally by bond graphs.  It is also useful for extracting bond graph variables needed for the modeling of so-called \"activated bonds,\" bond graph elements that are internally modulated by variables extracted elsewhere in the bond graph. <p>

This package contains four types of detectors: <p>

<ol>
<li> effort detectors (<b>De</b>), <br>
<li> flow detectors (<b>Df</b>), <br>
<li> detectors of generalized momentums (<b>Dp</b>), and <br>
<li> detectors of generalized positions (<b>Dq</b>).
</ol> <p>
</html>
"));
    model De "Effort sensor of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="De",
            style(color=0, fillColor=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="De",
            style(color=0, fillColor=0)), Text(extent=[-70, 90; 70, 50], string=
               "%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The effort sensor (detector) senses the common effort of a 0-junction.  Thus, effort sensors must always be connected to 0-junctions via a bond. <p>

An effort sensor does not affect the behavior of the system at all.  This is accomplished by setting the flow into the sensor equal to zero.  Thus, an effort sensor is in its effects identical with a flow source of zero flow. <p>

The effort sensor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The effort sensor has an additional output signal, through which it transmits the effort value. <p>

Since the effort sensor computes the flow (by setting it equal to zero), it has a causality stroke at the sensor element.

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = 0</b>
</html>
"));
      Modelica.Blocks.Interfaces.RealOutput OutPort1 
        "Connector of Real output signal"         annotation (extent=[100, -10;
              120, 10], layer="icon");
    equation 
      OutPort1 = e;
      f = 0;
    end De;
    
    model Df "Flow sensor of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="Df",
            style(color=0, fillColor=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="Df",
            style(color=0, fillColor=0)), Text(extent=[-70, 90; 70, 50], string=
               "%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The flow sensor (detector) senses the common flow of a 1-junction.  Thus, flow sensors must always be connected to 1-junctions via a bond. <p>

A flow sensor does not affect the behavior of the system at all.  This is accomplished by setting the effort at the sensor equal to zero.  Thus, a flow sensor is in its effects identical with an effort source of zero effort. <p>

The flow sensor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The flow sensor has an additional output signal, through which it transmits the flow value. <p>

Since the flow sensor computes the effort (by setting it equal to zero), it has a causality stroke away from the sensor element.

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = 0</b>
</html>
"));
      Modelica.Blocks.Interfaces.RealOutput OutPort1 
        "Connector of Real output signal"         annotation (extent=[100, -10;
              120, 10]);
    equation 
      OutPort1 = f;
      e = 0;
    end Df;
    
    model Dp "Generalized momentum sensor of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      parameter Real p0 "Initial momentum";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Icon(Text(
            extent=[-58, 80; 62, -80],
            string="Dp",
            style(color=0, fillColor=0)), Text(extent=[-68, 90; 72, 50], string=
               "%name")),
        Diagram,
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The sensor (detector) of generalized momentum senses the common effort of a 0-junction, then integrates it.  Thus, momentum sensors must always be connected to 0-junctions via a bond. <p>

A momentum sensor does not affect the behavior of the system at all.  This is accomplished by setting the flow into the sensor equal to zero.  Thus, a momentum sensor is in its effects similar to a flow source of zero flow. <p>

The momentum sensor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The momentum sensor has an additional output signal, through which it transmits the generalized momentum value. <p>

Since the momentum sensor computes the flow (by setting it equal to zero), it has a causality stroke at the sensor element.

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 p:  Generalized momentum
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = 0</b> <br>
<b>der(p) = e</b>
</html>
"));
      Modelica.Blocks.Interfaces.RealOutput OutPort1 
        "Connector of Real output signal"         annotation (extent=[100, -10;
              120, 10], layer="icon");
      Modelica.Blocks.Continuous.Integrator Integrator1(y_start=p0) 
                                                        annotation (extent=[50,
              -20; 90, 20]);
      De De1 annotation (extent=[-10, -20; 30, 20]);
      Junctions.J0p2 J0p2_1 annotation (extent=[-90, -20; -50, 20]);
      Bonds.fBond B1 annotation (extent=[-50, -20; -10, 20]);
    equation 
      connect(Integrator1.y,       OutPort1) annotation (points=[92,0; 110,0], style(
            color=74, rgbcolor={0,0,135}));
      connect(De1.OutPort1,Integrator1.u)       annotation (points=[32, 0; 46,
            0], style(color=74, rgbcolor={0,0,135}));
      connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -90,0],
          style(color=8));
      connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-50, 0; -50, 0],
               style(color=8));
      connect(B1.fBondCon1, De1.BondCon1) annotation (points=[-10, 0; -10, 0],
          style(color=8));
    end Dp;
    
    model Dq "Generalized position sensor of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      parameter Real q0 "Initial position";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Icon(Text(
            extent=[-58, 80; 62, -80],
            string="Dq",
            style(color=0, fillColor=0)), Text(extent=[-68, 90; 72, 50], string=
               "%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The sensor (detector) of generalized position senses the common flow of a 1-junction, then integrates it.  Thus, position sensors must always be connected to 1-junctions via a bond. <p>

A position sensor does not affect the behavior of the system at all.  This is accomplished by setting the effort at the sensor equal to zero.  Thus, a position sensor is in its effects similar to an effort source of zero effort. <p>

The position sensor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The position sensor has an additional output signal, through which it transmits the generalized position value. <p>

Since the position sensor computes the effort (by setting it equal to zero), it has a causality stroke away from the sensor element.

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 q:  Generalized position
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = 0</b> <br>
<b>der(q) = f</b>
</html>
"),     Diagram);
      Modelica.Blocks.Interfaces.RealOutput OutPort1 
        "Connector of Real output signal"         annotation (extent=[100, -10;
              120, 10], layer="icon");
      Modelica.Blocks.Continuous.Integrator Integrator1(y_start=q0) 
                                                        annotation (extent=[50,
              -20; 90, 20]);
      Df Df1 annotation (extent=[-10, -20; 30, 20]);
      Bonds.eBond B1 annotation (extent=[-50,-20; -10,20]);
      Junctions.J1p2 J1p2_1 annotation (extent=[-90,-20; -50,20]);
    equation 
      connect(Integrator1.y,       OutPort1) annotation (points=[92,0; 110,0], style(
            color=74, rgbcolor={0,0,135}));
      connect(Df1.OutPort1,Integrator1.u)       annotation (points=[32, 0; 46,
            0], style(color=74, rgbcolor={0,0,135}));
      connect(B1.eBondCon1, Df1.BondCon1) annotation (points=[-10,0; -10,0],
          style(color=8));
      connect(J1p2_1.BondCon2, B1.fBondCon1) annotation (points=[-50,0; -50,0],
               style(color=8));
      connect(BondCon1, J1p2_1.BondCon1) annotation (points=[-100,0; -90,0],
          style(color=8));
    end Dq;
    
    model Pbond "Power flow sensor" 
      extends Bonds.Bond;
      Modelica.Blocks.Interfaces.RealOutput P(redeclare type SignalType = 
            Modelica.SIunits.Power) "Power" 
        annotation (extent=[-10,80; 10,100], rotation=90);
    equation 
      P = BondCon1.e*BondCon1.f;
      annotation (Documentation(info="<html>
The non-directed power flow sensor is built as an acausal bond with an additional signal output that makes available the power flowing through the bond in the direction of the harpoon.
</html>
"), Diagram);
    end Pbond;
    
    block ePbond "Power flow sensor" 
      extends Bonds.eBond;
      Modelica.Blocks.Interfaces.RealOutput P(redeclare type SignalType = 
            Modelica.SIunits.Power) "Power" 
        annotation (extent=[-10,80; 10,100], rotation=90);
    equation 
      P = eBondCon1.e*fBondCon1.f;
      annotation (Documentation(info="<html>
The two directed power flow sensors are built as causal bonds with an additional signal output that makes available the power flowing through the bond in the direction of the harpoon.
</html>
"), Diagram);
    end ePbond;
    
    block fPbond "Power flow sensor" 
      extends Bonds.fBond;
      Modelica.Blocks.Interfaces.RealOutput P(redeclare type SignalType = 
            Modelica.SIunits.Power) "Power" 
        annotation (extent=[-10,80; 10,100], rotation=90);
    equation 
      P = eBondCon1.e*fBondCon1.f;
      annotation (Documentation(info="<html>
The non-directed power flow sensor is built as an acausal bond with an additional signal output that makes available the power flowing through the bond in the direction of the harpoon.
</html>
"), Diagram);
    end fPbond;
  end Sensors;
  
  package Sources "Sources of Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30, fillPattern=
                1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
              fillColor=30, fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
              fillColor=30, fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=3)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0.04,
        y=0.26,
        width=0.35,
        height=0.49,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package contains a collection of effort sources (<b>Se</b>) and flow sources (<b>Sf</b>). <p>

As of now, only the most common types of sources are being provided.  These are: <p>

<ol>
<li> constant sources, <br>
<li> modulated sources, and <br>
<li> sinusoidal sources.
</ol><p>

Other types of sources, such as rectangular sources can easily be constructed using the electrical library as a model.
</html>

"));
    model Se "Constant effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Bondgraphic Effort Source";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="Se",
            style(color=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="Se",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-70, -50; 70, -90], string="e0=%e0")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The constant effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The constant effort to be generated is modeled as a parameter. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:   Bondgraphic effort variable

 f:   Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:  Constant effort to be generated (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = e0</b>
</html>
"));
    equation 
      e = e0;
    end Se;
    
    model mSe "Modulated effort source of the Modelica bond graph library" 
      extends Interfaces.ModActiveOnePort;
      Real e0 "Bondgraphic Effort Source";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mSe",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="mSe",
            style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The modulated effort source is a modulated active OnePort element.  It inherits the effort and flow variables from the modulated active OnePort.  The modulated effort to be generated is modeled as a real-valued variable. <p>

Modulated sources are commonly used to model actuators of control systems.  They are the most common and convenient way for converting signal variables to bondgraphic variables, thereby allowing individual blocks of a block diagram, e.g. describing a control system, to be modeled internally using bond graphs. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:   Bondgraphic effort variable

 f:   Bondgraphic flow variable, normalized positive for flows out of the model

 e0:  Effort to be generated (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = e0</b>
</html>
"));
    equation 
      e0 = s;
      e = e0;
    end mSe;
    
    model sinSe "Sinusoidal effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Amplitude of sine wave";
      parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
      parameter Modelica.SIunits.Frequency freqHz=1 "Frequency of sine wave";
      parameter Real offset=0 "Offset of sine wave";
      parameter Modelica.SIunits.Time startTime=0 "Start time of sine wave";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-80, -90; -80, 84], style(color=8)),
          Polygon(points=[-80, 100; -86, 84; -74, 84; -80, 100], style(color=8,
                  fillColor=8)),
          Line(points=[-99, -40; 85, -40], style(color=8)),
          Polygon(points=[101, -40; 85, -34; 85, -46; 101, -40], style(color=8,
                  fillColor=8)),
          Line(points=[-40, 0; -31.6, 34.2; -26.1, 53.1; -21.3, 66.4; -17.1,
                74.6; -12.9, 79.1; -8.64, 79.8; -4.42, 76.6; -0.201, 69.7; 4.02,
                  59.4; 8.84, 44.1; 14.9, 21.2; 27.5, -30.8; 33, -50.2; 37.8, -
                64.2; 42, -73.1; 46.2, -78.4; 50.5, -80; 54.7, -77.6; 58.9, -
                71.5; 63.1, -61.9; 67.9, -47.2; 74, -24.8; 80, 0], style(color=
                  0, thickness=2)),
          Line(points=[-41, -2; -80, -2], style(color=0, thickness=2)),
          Text(
            extent=[-106, -11; -60, -29],
            string="offset",
            style(color=9)),
          Line(points=[-41, -2; -41, -40], style(color=8, pattern=2)),
          Text(
            extent=[-60, -43; -14, -61],
            string="startTime",
            style(color=9)),
          Text(
            extent=[84, -52; 108, -72],
            string="time",
            style(color=9)),
          Line(points=[-9, 79; 43, 79], style(color=8, pattern=2)),
          Line(points=[-42, -1; 50, 0], style(color=8, pattern=2)),
          Polygon(points=[33, 80; 30, 67; 37, 67; 33, 80], style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[33, 1; 30, 14; 36, 14; 33, 1; 33, 1], style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[33, 79; 33, 0], style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[-88, 100; -44, 80],
            string="e",
            style(color=9, fillColor=8)),
          Text(
            extent=[32, 52; 72, 34],
            string="e0",
            style(color=9))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="Se",
            style(color=0)),
          Line(points=[-70, 0; -60.2, 29.9; -53.8, 46.5; -48.2, 58.1; -43.3,
                65.2; -38.3, 69.2; -33.4, 69.8; -28.5, 67; -23.6, 61; -18.6, 52;
                  -13, 38.6; -5.98, 18.6; 8.79, -26.9; 15.1, -44; 20.8, -56.2;
                25.7, -64; 30.6, -68.6; 35.5, -70; 40.5, -67.9; 45.4, -62.5;
                50.3, -54.1; 55.9, -41.3; 63, -21.7; 70, 0], style(color=8)),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The sinusoidal effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The sinusoidal effort to be generated is modeled as a set of five parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:         Amplitude of sine wave to be generated (default value = 1)

 phase:      Phase of sine wave to be generated (default value = 0)

 freqHz:     Frequency of sine wave to be generated (default value = 1)

 offset:     Offset of sine wave to be generated (default value = 0)

 startTime:  Start time of sine wave to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = offset + e0*sin(2*pi*freqHz*(time - startTime))</b>
</html>
"));
      Modelica.Blocks.Sources.Sine Sine1(
        amplitude=e0,
        freqHz=freqHz,
        offset=offset,
        phase=phase,
        startTime=startTime)   annotation (extent=[70, 66; 90, 86]);
    equation 
      e = Sine1.y;
    end sinSe;
    
    model stepSe "Step effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Height of step";
      parameter Real offset=0 "Offset of step";
      parameter Modelica.SIunits.Time startTime=0 "Start time of step";
      annotation (
        Icon(
          Text(
            extent=[-62,78; 58,-82],
            string="Se",
            style(color=0)),
          Text(extent=[-72,100; 68,60],  string="%name"),
             Line(points=[-70,-80; 0,-80; 0,60; 69,60],     style(color=8))),
        Diagram(
          Polygon(points=[-80,90; -88,68; -72,68; -80,90],     style(color=8,
                fillColor=8)),
          Line(points=[-80,68; -80,-80],   style(color=8)),
          Line(points=[-80,-18; 0,-18; 0,50; 80,50],     style(color=0, thickness=
                 2)),
          Line(points=[-90,-70; 82,-70],   style(color=8)),
          Polygon(points=[90,-70; 68,-62; 68,-78; 90,-70],     style(color=8,
                fillColor=8)),
          Text(
            extent=[70,-80; 94,-100],
            string="time",
            style(color=9)),
          Text(
            extent=[-21,-72; 25,-90],
            string="startTime",
            style(color=9)),
          Line(points=[0,-17; 0,-71],   style(color=8, pattern=2)),
          Text(
            extent=[-68,-36; -22,-54],
            string="offset",
            style(color=9)),
          Line(points=[-13,50; -13,-17],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[2,50; -19,50; 2,50],    style(color=8, pattern=2)),
          Polygon(points=[-13,-17; -16,-4; -10,-4; -13,-17; -13,-17],      style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-13,50; -16,37; -9,37; -13,50],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-50,26; -4,8],
            style(color=9),
            string="e0"),
          Polygon(points=[-13,-69; -16,-56; -10,-56; -13,-69; -13,-69],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-13,-18; -13,-70],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-13,-18; -16,-31; -9,-31; -13,-18],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-91,93; -32,70],
            style(color=9),
            string="e")),
        Documentation(info="<html>
The step effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The step effort to be generated is modeled as a set of three parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:         Height of step to be generated (default value = 1)

 offset:     Offset of step to be generated (default value = 0)

 startTime:  Start time of step to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = offset + (if time &lt; startTime then 0 else e0)</b>
</html>
"));
      Modelica.Blocks.Sources.Step Step1(
        offset=offset,
        startTime=startTime,
        height=e0) annotation (extent=[70,68; 90,88]);
    equation 
      e = Step1.y;
    end stepSe;
    
    model rampSe "Ramp effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Height of ramp";
      parameter Modelica.SIunits.Time duration=2 "Duration of ramp";
      parameter Real offset=0 "Offset of ramp";
      parameter Modelica.SIunits.Time startTime=0 "Start time of ramp";
      annotation (
        Icon(
          Text(
            extent=[-60,70; 60,-90],
            string="Se",
            style(color=0)),
          Text(extent=[-70,100; 70,60],  string="%name"),
             Line(points=[-80,-70; -50,-70; 50,50; 80,50],     style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,-30; -20,-30; 50,40],    style(color=0, thickness=2)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-40,-30; -42,-40; -37,-40; -40,-30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-30; -40,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-40,-80; -43,-70; -38,-70; -40,-80; -40,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-80,-43; -41,-59],
            string="offset",
            style(color=9)),
          Text(
            extent=[-40,-80; 6,-98],
            string="startTime",
            style(color=9)),
          Text(
            extent=[-83,83; -38,58],
            style(color=9),
            string="e"),
          Text(
            extent=[70,-86; 94,-106],
            string="time",
            style(color=9)),
          Line(points=[-20,-30; -20,-80],   style(color=8, pattern=2)),
          Line(points=[-19,-30; 50,-30],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Line(points=[50,40; 101,40],   style(color=0, thickness=2)),
          Line(points=[50,40; 50,-30],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[50,-30; 42,-28; 42,-32; 50,-30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-20,-30; -11,-28; -11,-32; -20,-30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[50,40; 48,30; 53,30; 50,40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[50,-30; 47,-20; 52,-20; 50,-30; 50,-30],      style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[53,15; 82,-3],
            style(color=9),
            string="e0"),
          Text(
            extent=[0,-27; 35,-47],
            string="duration",
            style(color=9))),
        Documentation(info="<html>
The ramp effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The ramp effort to be generated is modeled as a set of four parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:         Height of ramp to be generated (default value = 1)

 duration:   Duration of ramp to be generated (default value = 2)

 offset:     Offset of ramp to be generated (default value = 0)

 startTime:  Start time of ramp to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = offset + (if time &lt; startTime then 0 else if time &lt; (startTime + duration)
                 then (time - startTime)*e0/duration else e0);</b>
</html>
"));
      Modelica.Blocks.Sources.Ramp Ramp1(
        height=e0,
        duration=duration,
        offset=offset,
        startTime=startTime) annotation (extent=[68,66; 90,88]);
    equation 
      e = Ramp1.y;
    end rampSe;
    
    model expsinSe 
      "Damped sinusoidal effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Amplitude of sine wave";
      parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
      parameter Modelica.SIunits.Frequency freqHz=2 "Frequency of sine wave";
      parameter Real damping=1 "Damping coefficient of sine wave";
      parameter Real offset=0 "Offset of sine wave";
      parameter Modelica.SIunits.Time startTime=0 "Start time of sine wave";
      annotation (
        Icon(
          Text(
            extent=[-60,80; 60,-80],
            string="Se",
            style(color=0)),
          Text(extent=[-64,98; 76,58],   string="%name"),
             Line(points=[-70,-4; -65.2,28.3; -62,46.3; -58.7,60.5; -55.5,70.2;
                -52.3,75.3; -49.1,75.6; -45.9,71.3; -42.7,63.1; -38.6,48.2; -33,
                21.8; -25,-17.9; -20.2,-37.7; -16.1,-49.9; -12.1,-57.2; -8.1,
                -59.3; -4.1,-56.5; -0.1,-49.3; 4.77,-36.1; 18.44,9.7; 23.3,22.4;
                28.1,30.8; 32.1,34; 36.9,33.2; 41.8,27.8; 48.2,15.4; 61.1,-14.5;
                67.5,-25.2; 73.1,-29.9; 78.7,-29.9; 85.2,-24.5; 90,-17.8],
                                               style(color=8))),
        Diagram(
          Line(points=[-86,-94; -86,80],   style(color=8)),
          Polygon(points=[-86,96; -92,80; -80,80; -86,96],       style(color=8,
                fillColor=8)),
          Line(points=[-105,-44; 79,-44],  style(color=8)),
          Polygon(points=[95,-44; 79,-38; 79,-50; 95,-44],       style(color=8,
                fillColor=8)),
          Line(points=[-56,-4; -52.1,24.2; -49.5,40; -46.9,52.4; -44.2,60.9;
                -41.6,65.4; -39,65.6; -36.4,61.9; -33.8,54.7; -30.5,41.7; -25.9,
                18.5; -19.4,-16.2; -15.5,-33.5; -12.23,-44.1; -8.96,-50.5;
                -5.698,-52.4; -2.43,-49.9; 0.83,-43.6; 4.8,-32.1; 15.9,8; 19.8,
                19.1; 23.7,26.5; 27,29.3; 30.9,28.5; 34.8,23.8; 40,12.9; 50.5,
                -13.2; 55.7,-22.6; 60.3,-26.7; 64.9,-26.6; 70.1,-22; 74,-16.1],
                                style(color=0, thickness=2)),
          Text(
            extent=[-84,-3; -61,-23],
            string="offset",
            style(color=9)),
          Text(
            extent=[-78,-40; -32,-58],
            string="startTime",
            style(color=9)),
          Text(
            extent=[78,-56; 102,-76],
            string="time",
            style(color=9)),
          Line(points=[-56,-4; 12,-4], style(color=8, pattern=2)),
          Line(points=[-56,-4; -87,-4], style(color=0, thickness=2)),
          Line(points=[-56,73; -56,-4],  style(color=8, pattern=2)),
          Line(points=[12,-5; 12,72],   style(color=8, pattern=2)),
          Line(points=[12,69; -56,69],   style(color=8)),
          Text(
            extent=[-48,84; 3,70],
            string="1/freqHz",
            style(color=9)),
          Polygon(points=[-55,69; -46,71; -46,67; -55,69],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[12,69; 4,71; 4,67; 12,69],       style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-56,-65; -25,-65],   style(color=8)),
          Polygon(points=[-24,-65; -32,-63; -32,-67; -24,-65],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-57,-67; -33,-79],
            string="t",
            style(color=9)),
          Text(
            extent=[-88,-71; 102,-100],
            style(color=9),
            string="e0*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"),
          Line(points=[-56,-4; -56,-44],  style(color=8, pattern=2)),
          Line(points=[-56,-58; -56,-76],   style(color=8, pattern=2)),
          Line(points=[12,-80; -7,-52],   style(color=8, pattern=2)),
          Text(
            extent=[-91,101; -52,82],
            style(color=9),
            string="e")),
        Documentation(info="<html>
The damped sinusoidal effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The damped sinusoidal effort to be generated is modeled as a set of six parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:         Amplitude of damped sine wave to be generated (default value = 1)

 phase:      Phase of damped sine wave to be generated (default value = 0)

 freqHz:     Frequency of damped sine wave to be generated (default value = 2)

 damping:    Damping coefficient of damped sine wave to be generated (default value = 1)

 offset:     Offset of damped sine wave to be generated (default value = 0)

 startTime:  Start time of damped sine wave to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e = offset + (if time &lt; startTime then 0 else e0*exp(-(time
       - startTime)*damping)*sin(2*pi*freqHz*(time - startTime) + phase))</b>
</html>
"));
      Modelica.Blocks.Sources.ExpSine ExpSine1(
        amplitude=e0,
        freqHz=freqHz,
        phase=phase,
        damping=damping,
        offset=offset,
        startTime=startTime) annotation (extent=[68,66; 90,88]);
    equation 
      e = ExpSine1.y;
    end expsinSe;
    
    model expSe 
      "Step response effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Height of step response after infinite time";
      parameter Modelica.SIunits.Time riseTime(min=0) = 0.5 "Rise time";
      parameter Modelica.SIunits.Time riseTimeConst(min=Modelica.Constants.small) = 0.1 
        "Rise time constant";
      parameter Modelica.SIunits.Time fallTimeConst(min=Modelica.Constants.small) = riseTimeConst 
        "Fall time constant";
      parameter Real offset=0 "Offset of step response";
      parameter Modelica.SIunits.Time startTime=0 "Start time of step response";
      annotation (
        Icon(
          Text(
            extent=[-60,78; 60,-82],
            string="Se",
            style(color=0)),
          Text(extent=[-68,98; 72,58],   string="%name"),
             Line(points=[-70,-69; -67.2,-54.3; -64.3,-41.1; -60.8,-26.6; -57.3,
                -14; -53.7,-3.08; -49.5,8.18; -45.3,17.7; -40.3,27; -34.6,35.5;
                -28.3,43.1; -21.2,49.6; -12.7,55.3; -2.1,60.2; 0,61; 2.12,48.5;
                4.95,33.7; 7.78,20.8; 10.606,9.45; 14.14,-2.7; 17.68,-13; 21.9,
                -23.2; 26.2,-31.6; 31.1,-39.5; 36.8,-46.4; 43.1,-52.3; 50.9,
                -57.5; 60.8,-61.8; 70,-64.4],                            style(
                color=8))),
        Diagram(
          Line(points=[-100,-80; 84,-80],   style(color=8)),
          Polygon(points=[100,-80; 84,-74; 84,-86; 100,-80],     style(color=8,
                fillColor=8)),
          Line(points=[-40,-40; -37.2,-25.3; -34.3,-12.1; -30.8,2.4; -27.3,15;
                -23.7,25.92; -19.5,37.18; -15.3,46.7; -10.3,56; -4.6,64.5; 1.7,
                72.1; 8.8,78.6; 17.3,84.3; 27.9,89.2; 30,90; 32.12,77.5; 34.95,
                62.7; 37.78,49.8; 40.606,38.45; 44.14,26.3; 47.68,16; 51.9,5.8;
                56.2,-2.6; 61.1,-10.5; 66.8,-17.4; 73.1,-23.3; 80.9,-28.5; 90.8,
                -32.8; 100,-35.4],                              style(color=0,
                thickness=2)),
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Text(
            extent=[-78,-53; -46,-66],
            string="offset",
            style(color=9)),
          Polygon(points=[-40,-80; -43,-70; -38,-70; -40,-80; -40,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-39; -40,-70],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-40,-40; -42,-50; -37,-50; -40,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-39,-40; -80,-40],   style(color=0, thickness=2)),
          Text(
            extent=[-59,-81; -13,-99],
            string="startTime",
            style(color=9)),
          Polygon(points=[-41,-40; -32,-38; -32,-42; -41,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-40; 29,-40],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[29,-40; 21,-38; 21,-42; 29,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-26,-22; 19,-38],
            string="riseTime",
            style(color=9)),
          Text(
            extent=[78,-86; 102,-106],
            string="time",
            style(color=9)),
          Line(points=[-100,-80; 84,-80],   style(color=8)),
          Polygon(points=[100,-80; 84,-74; 84,-86; 100,-80],     style(color=8,
                fillColor=8)),
          Line(points=[-40,-40; -37.2,-25.3; -34.3,-12.1; -30.8,2.4; -27.3,15;
                -23.7,25.92; -19.5,37.18; -15.3,46.7; -10.3,56; -4.6,64.5; 1.7,
                72.1; 8.8,78.6; 17.3,84.3; 27.9,89.2; 30,90; 32.12,77.5; 34.95,
                62.7; 37.78,49.8; 40.606,38.45; 44.14,26.3; 47.68,16; 51.9,5.8;
                56.2,-2.6; 61.1,-10.5; 66.8,-17.4; 73.1,-23.3; 80.9,-28.5; 90.8,
                -32.8; 100,-35.4],                              style(color=0,
                thickness=2)),
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Text(
            extent=[-82,81; -41,61],
            style(color=9),
            string="e"),
          Text(
            extent=[-78,-53; -46,-66],
            string="offset",
            style(color=9)),
          Polygon(points=[-40,-80; -43,-70; -38,-70; -40,-80; -40,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-39; -40,-70],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-40,-40; -42,-50; -37,-50; -40,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-39,-40; -80,-40],   style(color=0, thickness=2)),
          Text(
            extent=[-59,-81; -13,-99],
            string="startTime",
            style(color=9)),
          Polygon(points=[-41,-40; -32,-38; -32,-42; -41,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-40; 29,-40],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[29,-40; 21,-38; 21,-42; 29,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-26,-22; 19,-38],
            string="riseTime",
            style(color=9)),
          Text(
            extent=[78,-86; 102,-106],
            string="time",
            style(color=9)),
          Line(points=[30,90; 30,-44],    style(color=8, pattern=2))),
        Documentation(info="<html>
The step response effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The step response effort to be generated is modeled as a set of six parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:              Bondgraphic effort variable

 f:              Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:             Height of step response after infinite time (default value = 1)

 riseTime:       Rise time of step response to be generated (default value = 0.5)

 riseTimeConst:  Rise time constant of step response to be generated (default value = 0.1)

 fallTimeConst:  Fall time constant of step response to be generated (default value = riseTimeConst)

 offset:         Offset of step response to be generated (default value = 0)

 startTime:      Start time of step response to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e_riseTime = e0*(1 - exp(-riseTime/riseTimeConst))</b> <br>
<b>e = offset + (if (time &lt; startTime) then 0 else if (time &lt; (startTime
     + riseTime)) then e0*(1 - exp(-(time - startTime)/riseTimeConst)) else 
          e_riseTime*exp(-(time - startTime - riseTime)/fallTimeConst));</b>
</html>
"));
      Modelica.Blocks.Sources.Exponentials Exponentials1(
        outMax=e0,
        riseTime=riseTime,
        riseTimeConst=riseTimeConst,
        fallTimeConst=fallTimeConst,
        offset=offset,
        startTime=startTime) annotation (extent=[68,66; 90,88]);
    equation 
      e = Exponentials1.y;
    end expSe;
    
    model pulseSe "Pulse effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Amplitude of pulse";
      parameter Modelica.SIunits.Time width(
        final min=Modelica.Constants.small,
        final max=100) = 50 "Width of pulse in % of period";
      parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
        "Time for one period";
      parameter Real offset=0 "Offset of pulse";
      parameter Modelica.SIunits.Time startTime=0 "Start time of pulse";
      annotation (
        Icon(
          Text(
            extent=[-62,70; 58,-90],
            string="Se",
            style(color=0)),
          Text(extent=[-70,100; 70,60],  string="%name"),
             Line(points=[-76,-80; -46,-80; -46,60; -6,60; -6,-80; 34,-80; 35,
                60; 74,60],    style(color=8))),
        Diagram(
          Polygon(points=[-80,86; -88,64; -72,64; -80,86],     style(color=8,
                fillColor=8)),
          Line(points=[-80,64; -80,-84],   style(color=8)),
          Line(points=[-90,-74; 82,-74],   style(color=8)),
          Polygon(points=[90,-74; 68,-66; 68,-82; 90,-74],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,-3; -37,-16; -30,-16; -34,-3],   style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-34,-5; -34,-74],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-33,-74; -36,-61; -30,-61; -33,-74; -33,-74],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-78,-28; -35,-40],
            string="offset",
            style(color=9)),
          Text(
            extent=[-39,-73; 7,-91],
            string="startTime",
            style(color=9)),
          Text(
            extent=[70,-84; 94,-104],
            string="time",
            style(color=9)),
          Line(points=[-18,-4; -18,-74],  style(color=8, pattern=2)),
          Line(points=[-80,-4; -20,-4; -20,46; 20,46; 20,-4; 40,-4; 40,46; 80,
                46],   style(color=0, thickness=2)),
          Line(points=[-19,84; -19,45],   style(color=8, pattern=2)),
          Line(points=[21,70; 21,46],   style(color=9, pattern=2)),
          Line(points=[41,84; 41,46],   style(color=8, pattern=2)),
          Line(points=[-19,79; 42,79],   style(color=8)),
          Line(points=[-19,65; 21,65],   style(color=8)),
          Text(
            extent=[-9,93; 37,81],
            string="period",
            style(color=9)),
          Text(
            extent=[-18,77; 21,65],
            string="width",
            style(color=9)),
          Line(points=[-52,46; -19,46],   style(color=8, pattern=2)),
          Line(points=[-34,46; -34,-3],  style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[-66,28; -25,14],
            style(color=9),
            string="e0"),
          Polygon(points=[-34,45; -37,32; -30,32; -34,45],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,-3; -37,10; -31,10; -34,-3; -34,-3],   style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[80,46; 80,-4; 92,-4],   style(color=0, thickness=2)),
          Polygon(points=[-19,65; -10,67; -10,63; -19,65],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[21,65; 13,67; 13,63; 21,65],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-19,79; -10,81; -10,77; -19,79],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[41,79; 33,81; 33,77; 41,79],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-70,87; -52,68],
            style(color=9),
            string="e")),
        Documentation(info="<html>
The pulse effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The pulse effort to be generated is modeled as a set of five parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:         Amplitude of pulse to be generated (default value = 1)

 width:      Percentage of period in on state (default value = 50)

 period:     Length of period of pulse to be generated (default value = 1)

 offset:     Offset of pulse to be generated (default value = 0)

 startTime:  Start time of pulse to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>when sample(startTime, period) then</b> <br>
<b>T0 = time</b> <br>
<b>end when</b> <br>
<b>e = offset + (if time &lt; startTime or time &gt;= T0 + T_width then 0 else e0)</b>
</html>
"));
      Modelica.Blocks.Sources.Pulse Pulse1(
        amplitude=e0,
        width=width,
        period=period,
        offset=offset,
        startTime=startTime) annotation (extent=[68,66; 90,88]);
    equation 
      e = Pulse1.y;
    end pulseSe;
    
    model sawSe "Saw tooth effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Amplitude of saw tooth";
      parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
        "Time for one period";
      parameter Real offset=0 "Offset of saw tooth";
      parameter Modelica.SIunits.Time startTime=0 "Start time of saw tooth";
      annotation (
        Icon(
          Text(
            extent=[-66,68; 54,-92],
            string="Se",
            style(color=0)),
          Text(extent=[-68,104; 72,64],  string="%name"),
             Line(points=[-78,-79; -58,-79; 2,62; 2,-79; 62,62; 62,-79],
              style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,-29; -37,-42; -30,-42; -34,-29],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-34,-30; -34,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-34,-80; -37,-67; -31,-67; -34,-80; -34,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-78,-34; -35,-46],
            string="offset",
            style(color=9)),
          Text(
            extent=[-31,-79; 15,-97],
            string="startTime",
            style(color=9)),
          Text(
            extent=[74,-84; 98,-104],
            string="time",
            style(color=9)),
          Line(points=[-10,-30; -10,-80],   style(color=8, pattern=2)),
          Line(points=[-10,78; -10,-30],   style(color=8, pattern=2)),
          Line(points=[30,78; 30,49],   style(color=8, pattern=2)),
          Line(points=[-10,73; 30,73],   style(color=8)),
          Text(
            extent=[-12,87; 34,75],
            string="period",
            style(color=9)),
          Line(points=[-44,50; 30,50],   style(color=8, pattern=2)),
          Line(points=[-34,37; -34,-17],  style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[-66,18; -25,4],
            style(color=9),
            string="e0"),
          Polygon(points=[-34,50; -37,37; -30,37; -34,50],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,-30; -37,-17; -31,-17; -34,-30; -34,-30],    style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-10,73; -1,75; -1,71; -10,73],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[30,73; 22,75; 22,71; 30,73],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-80,-30; -10,-30; 30,50; 30,-30; 72,50; 72,-30],
              style(color=0, thickness=2)),
          Text(
            extent=[-69,82; -50,62],
            style(color=9),
            string="e")),
        Documentation(info="<html>
The saw tooth effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The saw tooth effort to be generated is modeled as a set of four parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:         Amplitude of saw tooth to be generated (default value = 1)

 period:     Length of period of saw tooth to be generated (default value = 1)

 offset:     Offset of saw tooth to be generated (default value = 0)

 startTime:  Start time of saw tooth to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>when sample(startTime, period) then</b> <br>
<b>T0 = time</b> <br>
<b>end when</b> <br>
<b>e = offset + (if time &lt; startTime then 0 else (e0/period)*(time - T0))</b>
</html>
"));
      Modelica.Blocks.Sources.SawTooth SawTooth1(
        amplitude=e0,
        period=period,
        offset=offset,
        startTime=startTime) annotation (extent=[68,66; 90,88]);
    equation 
      e = SawTooth1.y;
    end sawSe;
    
    model trapSe "Trapezoidal effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real e0=1 "Amplitude of trapezoidal function";
      parameter Modelica.SIunits.Time rising(final min=0) = 0 
        "Rising duration of trapezoid";
      parameter Modelica.SIunits.Time width(final min=0) = 0.5 
        "Width duration of trapezoid";
      parameter Modelica.SIunits.Time falling(final min=0) = 0 
        "Falling duration of trapezoid";
      parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
        "Time for one period";
      parameter Integer nperiod=-1 
        "Number of periods (< 0 means infinite number of periods)";
      parameter Real offset=0 "Offset of trapezoidal function";
      parameter Modelica.SIunits.Time startTime=0 
        "Start time of trapezoidal function";
      annotation (
        Icon(
          Text(
            extent=[-56,78; 64,-82],
            string="Se",
            style(color=0)),
          Text(extent=[-64,104; 76,64],  string="%name"),
             Line(points=[-79,-78; -58,-78; -28,62; 3,62; 32,-78; 53,-78; 82,62],
                         style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-45,-40; -47,-51; -43,-51; -45,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-45,-41; -45,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-45,-80; -47,-70; -43,-70; -45,-80; -45,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-86,-53; -43,-65],
            string="offset",
            style(color=9)),
          Text(
            extent=[-47,-79; -1,-97],
            string="startTime",
            style(color=9)),
          Text(
            extent=[76,-84; 100,-104],
            string="time",
            style(color=9)),
          Line(points=[-30,71; -30,-80],   style(color=8, pattern=2)),
          Line(points=[-10,49; -10,30],   style(color=8, pattern=2)),
          Line(points=[20,49; 20,29],   style(color=9, pattern=2)),
          Line(points=[40,49; 40,-40],   style(color=8, pattern=2)),
          Line(points=[-29,46; 40,46],   style(color=8)),
          Text(
            extent=[-8,60; 21,50],
            string="width",
            style(color=9)),
          Line(points=[-42,30; -10,30],   style(color=8, pattern=2)),
          Line(points=[-39,30; -39,-29],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[-67,6; -30,-8],
            style(color=9),
            string="e0"),
          Polygon(points=[-29,46; -22,48; -22,44; -29,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-10,46; -17,48; -17,44; -10,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-80,-40; -30,-40; -9,36; 21,36; 40,-40; 60,-40; 80,36;
                100,36],      style(color=0, thickness=2)),
          Polygon(points=[-39,30; -41,19; -37,19; -39,30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-39,-39; -41,-29; -37,-29; -39,-39; -39,-39],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[60,71; 60,-40],   style(color=8, pattern=2)),
          Polygon(points=[39,46; 32,48; 32,44; 39,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[20,46; 27,48; 27,44; 20,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[20,46; 13,48; 13,44; 20,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-12,46; -5,48; -5,44; -12,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-34,60; -5,50],
            string="rising",
            style(color=9)),
          Text(
            extent=[16,60; 45,50],
            string="falling",
            style(color=9)),
          Text(
            extent=[-75,85; -46,62],
            style(color=9),
            string="e"),
          Line(points=[-20,66; 61,66],   style(color=8)),
          Polygon(points=[-29,66; -20,68; -20,64; -29,66],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[61,66; 53,68; 53,64; 61,66],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-2,76; 25,67],
            string="period",
            style(color=9))),
        Documentation(info="<html>
The trapezoidal effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The trapezoidal effort to be generated is modeled as a set of eight parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 e0:         Amplitude of trapezoid to be generated (default value = 1)

 rising:     Length of rising time of trapezoid to be generated (default value = 0)

 width :     Length of on state of trapezoid to be generated (default value = 0.5)

 falling:    Length of falling time of trapezoid to be generated (default value = 0)

 period:     Length of period of trapezoid to be generated (default value = 1)

 nperiod:    Number of periods of trapezoid to be generated
             an infinite number of periods is being generated if nperiod &lt; 0 (default value = -1)

 offset:     Offset of trapezoid to be generated (default value = 0)

 startTime:  Start time of trapezoid to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>when pre(counter2) &lt;&gt; 0 and sample(startTime, period) then</b> <br>
<b>T0 = time</b> <br>
<b>counter2 = pre(counter)</b> <br>
<b>counter = pre(counter) - (if pre(counter) &gt; 0 then 1 else 0)</b> <br>
<b>end when</b> <br>
<b>e = offset + (if (time &lt; startTime or counter2 == 0 or time &gt;= T0 +
    T_falling) then 0 else if (time &lt; T0 + T_rising) then (time - T0)*e0/T_rising
    else if (time &lt; T0 + T_width) then e0 else 
    (T0 + T_falling - time)*e0/(T_falling - T_width))</b>
</html>
"));
      Modelica.Blocks.Sources.Trapezoid Trapezoid1(
        amplitude=e0,
        rising=rising,
        width=width,
        falling=falling,
        period=period,
        nperiod=nperiod,
        offset=offset,
        startTime=startTime) annotation (extent=[68,66; 90,88]);
    equation 
      e = Trapezoid1.y;
    end trapSe;
    
    model tableSe "Tabular effort source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real table[:, :]=[0, 0; 1, 1; 2, 4] 
        "Table matrix (time = first column, voltage = second column)";
      parameter Real offset=0 "Offset of time function";
      parameter Modelica.SIunits.Time startTime=0 "Start time of time function";
      annotation (
        Icon(
          Text(
            extent=[-58,78; 62,-82],
            string="Se",
            style(color=0)),
          Text(extent=[-66,104; 74,64],  string="%name"),
             Line(points=[-64,-62; -64,58; 36,58; 36,-62; -64,-62; -64,-32; 36,
                -32; 36,-2; -64,-2; -64,28; 36,28; 36,58; -14,58; -14,-63],
                     style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Rectangle(extent=[-20,80; 30,-40],   style(
              color=7,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-20,-40; -20,80; 80,80; 80,-40; -20,-40; -20,-10; 80,-10;
                80,20; -20,20; -20,50; 80,50; 80,80; 30,80; 30,-41],
              style(color=0)),
          Text(
            extent=[-77,-52; -38,-68],
            string="offset",
            style(color=9)),
          Polygon(points=[-31,-40; -33,-50; -28,-50; -31,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-30,-80; -33,-70; -28,-70; -30,-80; -30,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-31,-41; -31,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Line(points=[-20,-42; -20,-80],   style(color=8, pattern=2)),
          Text(
            extent=[-38,-80; 8,-98],
            string="startTime",
            style(color=9)),
          Line(points=[-20,-40; -80,-40],   style(color=8, pattern=2)),
          Text(
            extent=[72,-87; 97,-99],
            string="time",
            style(color=9)),
          Text(
            extent=[-15,73; 24,58],
            string="time",
            style(color=0)),
          Text(
            extent=[33,73; 76,57],
            string="v",
            style(color=0)),
          Text(
            extent=[-73,84; -50,62],
            style(color=9),
            string="e")),
        Documentation(info="<html>
The tabular effort source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The tabular effort to be generated is modeled as a two-dimensional parameter table plus a set of two scalar parameters. <p>

The effort source has its causality stroke away from the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 table:      Two-dimensional table with supporting values (default value = [0, 0; 1, 1; 2, 4])

 offset:     Offset of time function to be generated (default value = 0)

 startTime:  Start time of time function to be generated (default value = 0)
</pre> <p>

<hr> <p>

The model generates an effort source by <b>linear interpolation</b> in
a table. The time points and effort values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
time points, and the second column contains the effort to be interpolated.
The table interpolation has the following proporties: <p>

<ul>
<li>The time points need to be <b>monotonically increasing</b>.
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table.
<li>Values <b>outside</b> of the table range, are computed by
    <b>extrapolation</b> through the last or first two points of the
    table.
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the voltage value is just returned independantly of the
    actual time instant, i.e., this is a constant voltage source.
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined
    by the table can be shifted both in time and in the voltage.
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries,
    in order to not integrate over a discontinuous or not differentiable
    points.
</ul> <p>

<hr> <p>

<b>Example:</b> <p>

<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the effort e =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the effort e =  2.5,
    e.g., time = 2.0, the effort e =  4.0,
    e.g., time = 5.0, the effort e = 23.0 (i.e. extrapolation).
</pre>
</html>
"));
      Modelica.Blocks.Sources.TimeTable TimeTable1(
        table=table,
        offset=offset,
        startTime=startTime) annotation (extent=[68,68; 90,90]);
    equation 
      e = TimeTable1.y;
    end tableSe;
    
    model mSe_int "Internal modulated effort source of the bond graph library" 
      extends Interfaces.ModTwoPort;
      Real e0 "Bondgraphic Effort Source";
      annotation (
        Diagram(Text(
            extent=[-60,80; 60,-80],
            string="mSe",
            style(color=0))),
        Icon(   Text(
            extent=[-60,80; 60,-80],
            string="mSe",
            style(color=0)), Text(extent=[-70,90; 70,50],   string="%name")),
        Documentation(info="<html>
The modulated internal effort source is a modulated directed TwoPort element.  It inherits the effort and flow variables from the modulated directed TwoPort.  The modulated effort to be generated is modeled as a real-valued variable. <p>

The modulated internal effort source is a modulated effort source on its secondary side.  The primary side provides the power needed by the effort source.  This element can e.g. be used to model a DC power supply together with the power cable that draws the required energy from the grid. <p>

The effort source has its secondary causality stroke away from the source.  Its primary causality stroke is free. <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 e0:  Effort to be generated (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e2 = e0</b> <br>
<b>e1*f1 = e2*f2</b>
</html>
"));
    equation 
      e0 = s;
      e2 = e0;
      e1*f1 = e2*f2;
    end mSe_int;
    
    model Sf "Constant flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Bondgraphic Flow Source";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="Sf",
            style(color=0))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="Sf",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-70, -50; 70, -90], string="f0=%f0")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The constant flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The constant flow to be generated is modeled as a parameter. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:   Bondgraphic effort variable

 f:   Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:  Constant flow to be generated (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = f0</b>
</html>
"));
    equation 
      f = f0;
    end Sf;
    
    model mSf "Modulated flow source of the Modelica bond graph library" 
      extends Interfaces.ModActiveOnePort;
      Real f0 "Bondgraphic Effort Source";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mSf",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="mSf",
            style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The modulated flow source is a modulated active OnePort element.  It inherits the effort and flow variables from the modulated active OnePort.  The modulated flow to be generated is modeled as a real-valued variable. <p>

Modulated sources are commonly used to model actuators of control systems.  They are the most common and convenient way for converting signal variables to bondgraphic variables, thereby allowing individual blocks of a block diagram, e.g. describing a control system, to be modeled internally using bond graphs. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:   Bondgraphic effort variable

 f:   Bondgraphic flow variable, normalized positive for flows out of the model

 f0:  Flow to be generated (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = f0</b>
</html>
"));
    equation 
      f0 = s;
      f = f0;
    end mSf;
    
    model sinSf "Sinusoidal flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Amplitude of sine wave";
      parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
      parameter Modelica.SIunits.Frequency freqHz=1 "Frequency of sine wave";
      parameter Real offset=0 "Offset of sine wave";
      parameter Modelica.SIunits.Time startTime=0 "Start time of sine wave";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-80, -90; -80, 84], style(color=8)),
          Polygon(points=[-80, 100; -86, 84; -74, 84; -80, 100], style(color=8,
                  fillColor=8)),
          Line(points=[-99, -40; 85, -40], style(color=8)),
          Polygon(points=[101, -40; 85, -34; 85, -46; 101, -40], style(color=8,
                  fillColor=8)),
          Line(points=[-40, 0; -31.6, 34.2; -26.1, 53.1; -21.3, 66.4; -17.1,
                74.6; -12.9, 79.1; -8.64, 79.8; -4.42, 76.6; -0.201, 69.7; 4.02,
                  59.4; 8.84, 44.1; 14.9, 21.2; 27.5, -30.8; 33, -50.2; 37.8, -
                64.2; 42, -73.1; 46.2, -78.4; 50.5, -80; 54.7, -77.6; 58.9, -
                71.5; 63.1, -61.9; 67.9, -47.2; 74, -24.8; 80, 0], style(color=
                  0, thickness=2)),
          Line(points=[-41, -2; -80, -2], style(color=0, thickness=2)),
          Text(
            extent=[-106, -11; -60, -29],
            string="offset",
            style(color=9)),
          Line(points=[-41, -2; -41, -40], style(color=8, pattern=2)),
          Text(
            extent=[-60, -43; -14, -61],
            string="startTime",
            style(color=9)),
          Text(
            extent=[84, -52; 108, -72],
            string="time",
            style(color=9)),
          Line(points=[-9, 79; 43, 79], style(color=8, pattern=2)),
          Line(points=[-42, -1; 50, 0], style(color=8, pattern=2)),
          Polygon(points=[33, 80; 30, 67; 37, 67; 33, 80], style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[33, 1; 30, 14; 36, 14; 33, 1; 33, 1], style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[33, 79; 33, 0], style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[32, 52; 72, 34],
            string="f0",
            style(color=9)),
          Text(
            extent=[-86, 96; -44, 78],
            string="f",
            style(color=9))),
        Icon(
          Text(
            extent=[-60, 80; 60, -80],
            string="Sf",
            style(color=0)),
          Line(points=[-70, 0; -60.2, 29.9; -53.8, 46.5; -48.2, 58.1; -43.3,
                65.2; -38.3, 69.2; -33.4, 69.8; -28.5, 67; -23.6, 61; -18.6, 52;
                  -13, 38.6; -5.98, 18.6; 8.79, -26.9; 15.1, -44; 20.8, -56.2;
                25.7, -64; 30.6, -68.6; 35.5, -70; 40.5, -67.9; 45.4, -62.5;
                50.3, -54.1; 55.9, -41.3; 63, -21.7; 70, 0], style(color=8)),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The sinusoidal flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The sinusoidal flow to be generated is modeled as a set of five parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:         Amplitude of sine wave to be generated (default value = 1)

 phase:      Phase of sine wave to be generated (default value = 0)

 freqHz:     Frequency of sine wave to be generated (default value = 1)

 offset:     Offset of sine wave to be generated (default value = 0)

 startTime:  Start time of sine wave to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = offset + f0*sin(2*pi*freqHz*(time - startTime))</b>
</html>
"));
      Modelica.Blocks.Sources.Sine Sine1(
        amplitude=f0,
        freqHz=freqHz,
        offset=offset,
        phase=phase,
        startTime=startTime)   annotation (extent=[70, 66; 90, 86]);
    equation 
      f = Sine1.y;
    end sinSf;
    
    model stepSf "Step flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Height of step";
      parameter Real offset=0 "Offset of step";
      parameter Modelica.SIunits.Time startTime=0 "Start time of step";
      annotation (
        Icon(
          Text(
            extent=[-70,74; 36,-78],
            string="Sf",
            style(color=0)),
          Text(extent=[-76,102; 64,62],  string="%name"),
             Line(points=[-70,-80; 0,-80; 0,60; 69,60],     style(color=8))),
        Diagram(
          Polygon(points=[-80,90; -88,68; -72,68; -80,90],     style(color=8,
                fillColor=8)),
          Line(points=[-80,68; -80,-80],   style(color=8)),
          Line(points=[-80,-18; 0,-18; 0,50; 80,50],     style(color=0, thickness=
                 2)),
          Line(points=[-90,-70; 82,-70],   style(color=8)),
          Polygon(points=[90,-70; 68,-62; 68,-78; 90,-70],     style(color=8,
                fillColor=8)),
          Text(
            extent=[70,-80; 94,-100],
            string="time",
            style(color=9)),
          Text(
            extent=[-21,-72; 25,-90],
            string="startTime",
            style(color=9)),
          Line(points=[0,-17; 0,-71],   style(color=8, pattern=2)),
          Text(
            extent=[-68,-36; -22,-54],
            string="offset",
            style(color=9)),
          Line(points=[-13,50; -13,-17],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[2,50; -19,50; 2,50],    style(color=8, pattern=2)),
          Polygon(points=[-13,-17; -16,-4; -10,-4; -13,-17; -13,-17],      style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-13,50; -16,37; -9,37; -13,50],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-50,26; -4,8],
            style(color=9),
            string="f0"),
          Polygon(points=[-13,-69; -16,-56; -10,-56; -13,-69; -13,-69],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-13,-18; -13,-70],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-13,-18; -16,-31; -9,-31; -13,-18],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-91,93; -32,70],
            style(color=9),
            string="f")),
        Documentation(info="<html>
The step flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The step flow to be generated is modeled as a set of three parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:         Height of step to be generated (default value = 1)

 offset:     Offset of step to be generated (default value = 0)

 startTime:  Start time of step to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = offset + (if time &lt; startTime then 0 else f0)</b>
</html>
"));
      Modelica.Blocks.Sources.Step Step1(
        offset=offset,
        startTime=startTime,
        height=f0) annotation (extent=[70,68; 90,88]);
    equation 
      f = Step1.y;
    end stepSf;
    
    model rampSf "Ramp flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Height of ramp";
      parameter Modelica.SIunits.Time duration=2 "Duration of ramp";
      parameter Real offset=0 "Offset of ramp";
      parameter Modelica.SIunits.Time startTime=0 "Start time of ramp";
      annotation (
        Icon(
          Text(
            extent=[-60,64; 36,-68],
            style(color=0),
            string="Sf"),
          Text(extent=[-70,106; 70,66],  string="%name"),
             Line(points=[-80,-60; -50,-60; 50,60; 80,60],     style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,-30; -20,-30; 50,40],    style(color=0, thickness=2)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-40,-30; -42,-40; -37,-40; -40,-30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-30; -40,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-40,-80; -43,-70; -38,-70; -40,-80; -40,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-80,-43; -41,-59],
            string="offset",
            style(color=9)),
          Text(
            extent=[-40,-80; 6,-98],
            string="startTime",
            style(color=9)),
          Text(
            extent=[-83,83; -38,58],
            style(color=9),
            string="f"),
          Text(
            extent=[70,-86; 94,-106],
            string="time",
            style(color=9)),
          Line(points=[-20,-30; -20,-80],   style(color=8, pattern=2)),
          Line(points=[-19,-30; 50,-30],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Line(points=[50,40; 101,40],   style(color=0, thickness=2)),
          Line(points=[50,40; 50,-30],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[50,-30; 42,-28; 42,-32; 50,-30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-20,-30; -11,-28; -11,-32; -20,-30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[50,40; 48,30; 53,30; 50,40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[50,-30; 47,-20; 52,-20; 50,-30; 50,-30],      style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[53,15; 82,-3],
            style(color=9),
            string="f0"),
          Text(
            extent=[0,-27; 35,-47],
            string="duration",
            style(color=9))),
        Documentation(info="<html>
The ramp flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The ramp flow to be generated is modeled as a set of four parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:         Height of ramp to be generated (default value = 1)

 duration:   Duration of ramp to be generated (default value = 2)

 offset:     Offset of ramp to be generated (default value = 0)

 startTime:  Start time of ramp to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = offset + (if time &lt; startTime then 0 else if time &lt; (startTime + duration)
                 then (time - startTime)*f0/duration else f0);</b>
</html>
"));
      Modelica.Blocks.Sources.Ramp Ramp1(
        duration=duration,
        offset=offset,
        startTime=startTime,
        height=f0)           annotation (extent=[68,66; 90,88]);
    equation 
      f = Ramp1.y;
    end rampSf;
    
    model expsinSf 
      "Damped sinusoidal flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Amplitude of sine wave";
      parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
      parameter Modelica.SIunits.Frequency freqHz=2 "Frequency of sine wave";
      parameter Real damping=1 "Damping coefficient of sine wave";
      parameter Real offset=0 "Offset of sine wave";
      parameter Modelica.SIunits.Time startTime=0 "Start time of sine wave";
      annotation (
        Icon(
          Text(
            extent=[-70,62; 24,-64],
            style(color=0),
            string="Sf"),
          Text(extent=[-68,102; 72,62],  string="%name"),
             Line(points=[-80,-4; -75.2,28.3; -72,46.3; -68.7,60.5; -65.5,70.2;
                -62.3,75.3; -59.1,75.6; -55.9,71.3; -52.7,63.1; -48.6,48.2; -43,
                21.8; -35,-17.9; -30.2,-37.7; -26.1,-49.9; -22.1,-57.2; -18.1,
                -59.3; -14.1,-56.5; -10.1,-49.3; -5.23,-36.1; 8.44,9.7; 13.3,
                22.4; 18.1,30.8; 22.1,34; 26.9,33.2; 31.8,27.8; 38.2,15.4; 51.1,
                -14.5; 57.5,-25.2; 63.1,-29.9; 68.7,-29.9; 75.2,-24.5; 80,-17.8],
                                               style(color=8))),
        Diagram(
          Line(points=[-88,-92; -88,82],   style(color=8)),
          Polygon(points=[-88,98; -94,82; -82,82; -88,98],       style(color=8,
                fillColor=8)),
          Line(points=[-107,-42; 77,-42],  style(color=8)),
          Polygon(points=[93,-42; 77,-36; 77,-48; 93,-42],       style(color=8,
                fillColor=8)),
          Line(points=[-58,-2; -54.1,26.2; -51.5,42; -48.9,54.4; -46.2,62.9;
                -43.6,67.4; -41,67.6; -38.4,63.9; -35.8,56.7; -32.5,43.7; -27.9,
                20.5; -21.4,-14.2; -17.5,-31.5; -14.23,-42.1; -10.96,-48.5;
                -7.698,-50.4; -4.43,-47.9; -1.17,-41.6; 2.8,-30.1; 13.9,10;
                17.8,21.1; 21.7,28.5; 25,31.3; 28.9,30.5; 32.8,25.8; 38,14.9;
                48.5,-11.2; 53.7,-20.6; 58.3,-24.7; 62.9,-24.6; 68.1,-20; 72,
                -14.1],         style(color=0, thickness=2)),
          Text(
            extent=[-86,-1; -63,-21],
            string="offset",
            style(color=9)),
          Text(
            extent=[-80,-38; -34,-56],
            string="startTime",
            style(color=9)),
          Text(
            extent=[76,-54; 100,-74],
            string="time",
            style(color=9)),
          Line(points=[-58,-2; 10,-2], style(color=8, pattern=2)),
          Line(points=[-58,-2; -89,-2], style(color=0, thickness=2)),
          Line(points=[-58,75; -58,-2],  style(color=8, pattern=2)),
          Line(points=[10,-3; 10,74],   style(color=8, pattern=2)),
          Line(points=[10,71; -58,71],   style(color=8)),
          Text(
            extent=[-50,86; 1,72],
            string="1/freqHz",
            style(color=9)),
          Polygon(points=[-57,71; -48,73; -48,69; -57,71],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[10,71; 2,73; 2,69; 10,71],       style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-58,-63; -27,-63],   style(color=8)),
          Polygon(points=[-26,-63; -34,-61; -34,-65; -26,-63],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-59,-65; -35,-77],
            string="t",
            style(color=9)),
          Text(
            extent=[-90,-69; 100,-98],
            style(color=9),
            string="f0*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"),
          Line(points=[-58,-2; -58,-42],  style(color=8, pattern=2)),
          Line(points=[-58,-56; -58,-74],   style(color=8, pattern=2)),
          Line(points=[10,-78; -9,-50],   style(color=8, pattern=2)),
          Text(
            extent=[-93,101; -54,82],
            style(color=9),
            string="f")),
        Documentation(info="<html>
The damped sinusoidal flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The damped sinusoidal flow to be generated is modeled as a set of six parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:         Amplitude of damped sine wave to be generated (default value = 1)

 phase:      Phase of damped sine wave to be generated (default value = 0)

 freqHz:     Frequency of damped sine wave to be generated (default value = 2)

 damping:    Damping coefficient of damped sine wave to be generated (default value = 1)

 offset:     Offset of damped sine wave to be generated (default value = 0)

 startTime:  Start time of damped sine wave to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = offset + (if time &lt; startTime then 0 else f0*exp(-(time
       - startTime)*damping)*sin(2*pi*freqHz*(time - startTime) + phase))</b>
</html>
"));
      Modelica.Blocks.Sources.ExpSine ExpSine1(
        freqHz=freqHz,
        phase=phase,
        damping=damping,
        offset=offset,
        startTime=startTime,
        amplitude=f0) annotation (extent=[66,68; 88,90]);
    equation 
      f = ExpSine1.y;
    end expsinSf;
    
    model expSf "Step response flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Height of step response after infinite time";
      parameter Modelica.SIunits.Time riseTime(min=0) = 0.5 "Rise time";
      parameter Modelica.SIunits.Time riseTimeConst(min=Modelica.Constants.small) = 0.1 
        "Rise time constant";
      parameter Modelica.SIunits.Time fallTimeConst(min=Modelica.Constants.small) = riseTimeConst 
        "Fall time constant";
      parameter Real offset=0 "Offset of step response";
      parameter Modelica.SIunits.Time startTime=0 "Start time of step response";
      annotation (
        Icon(
          Text(
            extent=[-60,78; 34,-80],
            style(color=0),
            string="Sf"),
          Text(extent=[-68,98; 72,58],   string="%name"),
             Line(points=[-70,-69; -67.2,-54.3; -64.3,-41.1; -60.8,-26.6; -57.3,
                -14; -53.7,-3.08; -49.5,8.18; -45.3,17.7; -40.3,27; -34.6,35.5;
                -28.3,43.1; -21.2,49.6; -12.7,55.3; -2.1,60.2; 0,61; 2.12,48.5;
                4.95,33.7; 7.78,20.8; 10.606,9.45; 14.14,-2.7; 17.68,-13; 21.9,
                -23.2; 26.2,-31.6; 31.1,-39.5; 36.8,-46.4; 43.1,-52.3; 50.9,
                -57.5; 60.8,-61.8; 70,-64.4],                            style(
                color=8))),
        Diagram(
          Line(points=[-100,-80; 84,-80],   style(color=8)),
          Polygon(points=[100,-80; 84,-74; 84,-86; 100,-80],     style(color=8,
                fillColor=8)),
          Line(points=[-40,-40; -37.2,-25.3; -34.3,-12.1; -30.8,2.4; -27.3,15;
                -23.7,25.92; -19.5,37.18; -15.3,46.7; -10.3,56; -4.6,64.5; 1.7,
                72.1; 8.8,78.6; 17.3,84.3; 27.9,89.2; 30,90; 32.12,77.5; 34.95,
                62.7; 37.78,49.8; 40.606,38.45; 44.14,26.3; 47.68,16; 51.9,5.8;
                56.2,-2.6; 61.1,-10.5; 66.8,-17.4; 73.1,-23.3; 80.9,-28.5; 90.8,
                -32.8; 100,-35.4],                              style(color=0,
                thickness=2)),
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Text(
            extent=[-78,-53; -46,-66],
            string="offset",
            style(color=9)),
          Polygon(points=[-40,-80; -43,-70; -38,-70; -40,-80; -40,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-39; -40,-70],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-40,-40; -42,-50; -37,-50; -40,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-39,-40; -80,-40],   style(color=0, thickness=2)),
          Text(
            extent=[-59,-81; -13,-99],
            string="startTime",
            style(color=9)),
          Polygon(points=[-41,-40; -32,-38; -32,-42; -41,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-40; 29,-40],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[29,-40; 21,-38; 21,-42; 29,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-26,-22; 19,-38],
            string="riseTime",
            style(color=9)),
          Text(
            extent=[78,-86; 102,-106],
            string="time",
            style(color=9)),
          Line(points=[-100,-80; 84,-80],   style(color=8)),
          Polygon(points=[100,-80; 84,-74; 84,-86; 100,-80],     style(color=8,
                fillColor=8)),
          Line(points=[-40,-40; -37.2,-25.3; -34.3,-12.1; -30.8,2.4; -27.3,15;
                -23.7,25.92; -19.5,37.18; -15.3,46.7; -10.3,56; -4.6,64.5; 1.7,
                72.1; 8.8,78.6; 17.3,84.3; 27.9,89.2; 30,90; 32.12,77.5; 34.95,
                62.7; 37.78,49.8; 40.606,38.45; 44.14,26.3; 47.68,16; 51.9,5.8;
                56.2,-2.6; 61.1,-10.5; 66.8,-17.4; 73.1,-23.3; 80.9,-28.5; 90.8,
                -32.8; 100,-35.4],                              style(color=0,
                thickness=2)),
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Text(
            extent=[-82,81; -41,61],
            style(color=9),
            string="f"),
          Text(
            extent=[-78,-53; -46,-66],
            string="offset",
            style(color=9)),
          Polygon(points=[-40,-80; -43,-70; -38,-70; -40,-80; -40,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-39; -40,-70],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-40,-40; -42,-50; -37,-50; -40,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-39,-40; -80,-40],   style(color=0, thickness=2)),
          Text(
            extent=[-59,-81; -13,-99],
            string="startTime",
            style(color=9)),
          Polygon(points=[-41,-40; -32,-38; -32,-42; -41,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-40,-40; 29,-40],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[29,-40; 21,-38; 21,-42; 29,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-26,-22; 19,-38],
            string="riseTime",
            style(color=9)),
          Text(
            extent=[78,-86; 102,-106],
            string="time",
            style(color=9)),
          Line(points=[30,90; 30,-44],    style(color=8, pattern=2))),
        Documentation(info="<html>
The step response flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The step response flow to be generated is modeled as a set of six parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:              Bondgraphic effort variable

 f:              Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:             Height of step response after infinite time (default value = 1)

 riseTime:       Rise time of step response to be generated (default value = 0.5)

 riseTimeConst:  Rise time constant of step response to be generated (default value = 0.1)

 fallTimeConst:  Fall time constant of step response to be generated (default value = riseTimeConst)

 offset:         Offset of step response to be generated (default value = 0)

 startTime:      Start time of step response to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f_riseTime = f0*(1 - exp(-riseTime/riseTimeConst))</b> <br>
<b>f = offset + (if (time &lt; startTime) then 0 else if (time &lt; (startTime
     + riseTime)) then f0*(1 - exp(-(time - startTime)/riseTimeConst)) else 
          f_riseTime*exp(-(time - startTime - riseTime)/fallTimeConst));</b>
</html>
"));
      Modelica.Blocks.Sources.Exponentials Exponentials1(
        riseTime=riseTime,
        riseTimeConst=riseTimeConst,
        fallTimeConst=fallTimeConst,
        offset=offset,
        startTime=startTime,
        outMax=f0) annotation (extent=[68,66; 90,88]);
    equation 
      f = Exponentials1.y;
    end expSf;
    
    model pulseSf "Pulse flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Amplitude of saw tooth";
      parameter Modelica.SIunits.Time width(
        final min=Modelica.Constants.small,
        final max=100) = 50 "Width of saw tooth in % of period";
      parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
        "Time for one period";
      parameter Real offset=0 "Offset of saw tooth";
      parameter Modelica.SIunits.Time startTime=0 "Start time of saw tooth";
      annotation (
        Icon(
          Text(
            extent=[-62,58; 50,-60],
            style(color=0),
            string="Sf"),
          Text(extent=[-70,100; 70,60],  string="%name"),
             Line(points=[-76,-80; -46,-80; -46,60; -6,60; -6,-80; 34,-80; 35,
                60; 74,60],    style(color=8))),
        Diagram(
          Polygon(points=[-80,90; -88,68; -72,68; -80,90],     style(color=8,
                fillColor=8)),
          Line(points=[-80,68; -80,-80],   style(color=8)),
          Line(points=[-90,-70; 82,-70],   style(color=8)),
          Polygon(points=[90,-70; 68,-62; 68,-78; 90,-70],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,1; -37,-12; -30,-12; -34,1],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-34,-1; -34,-70],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-33,-70; -36,-57; -30,-57; -33,-70; -33,-70],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-78,-24; -35,-36],
            string="offset",
            style(color=9)),
          Text(
            extent=[-39,-69; 7,-87],
            string="startTime",
            style(color=9)),
          Text(
            extent=[70,-80; 94,-100],
            string="time",
            style(color=9)),
          Line(points=[-18,0; -18,-70],   style(color=8, pattern=2)),
          Line(points=[-80,0; -20,0; -20,50; 20,50; 20,0; 40,0; 40,50; 80,50],
                       style(color=0, thickness=2)),
          Line(points=[-19,88; -19,49],   style(color=8, pattern=2)),
          Line(points=[21,74; 21,50],   style(color=9, pattern=2)),
          Line(points=[41,88; 41,50],   style(color=8, pattern=2)),
          Line(points=[-19,83; 42,83],   style(color=8)),
          Line(points=[-19,69; 21,69],   style(color=8)),
          Text(
            extent=[-9,97; 37,85],
            string="period",
            style(color=9)),
          Text(
            extent=[-18,81; 21,69],
            string="width",
            style(color=9)),
          Line(points=[-52,50; -19,50],   style(color=8, pattern=2)),
          Line(points=[-34,50; -34,1],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[-66,32; -25,18],
            style(color=9),
            string="f0"),
          Polygon(points=[-34,49; -37,36; -30,36; -34,49],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,1; -37,14; -31,14; -34,1; -34,1],      style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[80,50; 80,0; 92,0],     style(color=0, thickness=2)),
          Polygon(points=[-19,69; -10,71; -10,67; -19,69],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[21,69; 13,71; 13,67; 21,69],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-19,83; -10,85; -10,81; -19,83],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[41,83; 33,85; 33,81; 41,83],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-74,91; -56,72],
            style(color=9),
            string="f")),
        Documentation(info="<html>
The pulse flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The pulse flow to be generated is modeled as a set of five parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:         Amplitude of pulse to be generated (default value = 1)

 width:      Percentage of period in on state (default value = 50)

 period:     Length of period of pulse to be generated (default value = 1)

 offset:     Offset of pulse to be generated (default value = 0)

 startTime:  Start time of pulse to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>when sample(startTime, period) then</b> <br>
<b>T0 = time</b> <br>
<b>end when</b> <br>
<b>f = offset + (if time &lt; startTime or time &gt;= T0 + T_width then 0 else f0)</b>
</html>
"));
      Modelica.Blocks.Sources.Pulse Pulse1(
        width=width,
        period=period,
        offset=offset,
        startTime=startTime,
        amplitude=f0) annotation (extent=[68,70; 90,92]);
    equation 
      f = Pulse1.y;
    end pulseSf;
    
    model sawSf "Saw tooth flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Amplitude of saw tooth";
      parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
        "Time for one period";
      parameter Real offset=0 "Offset of saw tooth";
      parameter Modelica.SIunits.Time startTime=0 "Start time of saw tooth";
      annotation (
        Icon(
          Text(
            extent=[-58,50; 38,-70],
            style(color=0),
            string="Sf"),
          Text(extent=[-72,104; 68,64],  string="%name"),
             Line(points=[-82,-79; -62,-79; -2,62; -2,-79; 58,62; 58,-79],
              style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,-29; -37,-42; -30,-42; -34,-29],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-34,-30; -34,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-34,-80; -37,-67; -31,-67; -34,-80; -34,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-78,-34; -35,-46],
            string="offset",
            style(color=9)),
          Text(
            extent=[-31,-79; 15,-97],
            string="startTime",
            style(color=9)),
          Text(
            extent=[74,-84; 98,-104],
            string="time",
            style(color=9)),
          Line(points=[-10,-30; -10,-80],   style(color=8, pattern=2)),
          Line(points=[-10,78; -10,-30],   style(color=8, pattern=2)),
          Line(points=[30,78; 30,49],   style(color=8, pattern=2)),
          Line(points=[-10,73; 30,73],   style(color=8)),
          Text(
            extent=[-12,87; 34,75],
            string="period",
            style(color=9)),
          Line(points=[-44,50; 30,50],   style(color=8, pattern=2)),
          Line(points=[-34,37; -34,-17],  style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[-66,18; -25,4],
            style(color=9),
            string="f0"),
          Polygon(points=[-34,50; -37,37; -30,37; -34,50],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-34,-30; -37,-17; -31,-17; -34,-30; -34,-30],    style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-10,73; -1,75; -1,71; -10,73],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[30,73; 22,75; 22,71; 30,73],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-80,-30; -10,-30; 30,50; 30,-30; 72,50; 72,-30],
              style(color=0, thickness=2)),
          Text(
            extent=[-75,82; -56,62],
            style(color=9),
            string="f")),
        Documentation(info="<html>
The saw tooth flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The saw tooth flow to be generated is modeled as a set of four parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:         Amplitude of saw tooth to be generated (default value = 1)

 period:     Length of period of saw tooth to be generated (default value = 1)

 offset:     Offset of saw tooth to be generated (default value = 0)

 startTime:  Start time of saw tooth to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>when sample(startTime, period) then</b> <br>
<b>T0 = time</b> <br>
<b>end when</b> <br>
<b>f = offset + (if time &lt; startTime then 0 else (f0/period)*(time - T0))</b>
</html>
"));
      Modelica.Blocks.Sources.SawTooth SawTooth1(
        period=period,
        offset=offset,
        startTime=startTime,
        amplitude=f0) annotation (extent=[68,66; 90,88]);
    equation 
      f = SawTooth1.y;
    end sawSf;
    
    model trapSf "Trapezoidal flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real f0=1 "Amplitude of trapezoidal function";
      parameter Modelica.SIunits.Time rising(final min=0) = 0 
        "Rising duration of trapezoid";
      parameter Modelica.SIunits.Time width(final min=0) = 0.5 
        "Width duration of trapezoid";
      parameter Modelica.SIunits.Time falling(final min=0) = 0 
        "Falling duration of trapezoid";
      parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
        "Time for one period";
      parameter Integer nperiod=-1 
        "Number of periods (< 0 means infinite number of periods)";
      parameter Real offset=0 "Offset of trapezoidal function";
      parameter Modelica.SIunits.Time startTime=0 
        "Start time of trapezoidal function";
      annotation (
        Icon(
          Text(
            extent=[-60,46; 36,-68],
            style(color=0),
            string="Sf"),
          Text(extent=[-66,104; 74,64],  string="%name"),
             Line(points=[-81,-78; -60,-78; -30,62; 1,62; 30,-78; 51,-78; 80,62],
                         style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-45,-40; -47,-51; -43,-51; -45,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-45,-41; -45,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Polygon(points=[-45,-80; -47,-70; -43,-70; -45,-80; -45,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-86,-53; -43,-65],
            string="offset",
            style(color=9)),
          Text(
            extent=[-47,-79; -1,-97],
            string="startTime",
            style(color=9)),
          Text(
            extent=[76,-84; 100,-104],
            string="time",
            style(color=9)),
          Line(points=[-30,71; -30,-80],   style(color=8, pattern=2)),
          Line(points=[-10,49; -10,30],   style(color=8, pattern=2)),
          Line(points=[20,49; 20,29],   style(color=9, pattern=2)),
          Line(points=[40,49; 40,-40],   style(color=8, pattern=2)),
          Line(points=[-29,46; 40,46],   style(color=8)),
          Text(
            extent=[-8,60; 21,50],
            string="width",
            style(color=9)),
          Line(points=[-42,30; -10,30],   style(color=8, pattern=2)),
          Line(points=[-39,30; -39,-29],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Text(
            extent=[-67,6; -30,-8],
            style(color=9),
            string="f0"),
          Polygon(points=[-29,46; -22,48; -22,44; -29,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-10,46; -17,48; -17,44; -10,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-80,-40; -30,-40; -9,36; 21,36; 40,-40; 60,-40; 80,36;
                100,36],      style(color=0, thickness=2)),
          Polygon(points=[-39,30; -41,19; -37,19; -39,30],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-39,-39; -41,-29; -37,-29; -39,-39; -39,-39],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[60,71; 60,-40],   style(color=8, pattern=2)),
          Polygon(points=[39,46; 32,48; 32,44; 39,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[20,46; 27,48; 27,44; 20,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[20,46; 13,48; 13,44; 20,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-12,46; -5,48; -5,44; -12,46],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-34,60; -5,50],
            string="rising",
            style(color=9)),
          Text(
            extent=[16,60; 45,50],
            string="falling",
            style(color=9)),
          Text(
            extent=[-77,83; -48,60],
            style(color=9),
            string="f"),
          Line(points=[-20,66; 61,66],   style(color=8)),
          Polygon(points=[-29,66; -20,68; -20,64; -29,66],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[61,66; 53,68; 53,64; 61,66],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Text(
            extent=[-2,76; 25,67],
            string="period",
            style(color=9))),
        Documentation(info="<html>
The trapezoidal flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The trapezoidal flow to be generated is modeled as a set of eight parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 f0:         Amplitude of trapezoid to be generated (default value = 1)

 rising:     Length of rising time of trapezoid to be generated (default value = 0)

 width :     Length of on state of trapezoid to be generated (default value = 0.5)

 falling:    Length of falling time of trapezoid to be generated (default value = 0)

 period:     Length of period of trapezoid to be generated (default value = 1)

 nperiod:    Number of periods of trapezoid to be generated
             an infinite number of periods is being generated if nperiod &lt; 0 (default value = -1)

 offset:     Offset of trapezoid to be generated (default value = 0)

 startTime:  Start time of trapezoid to be generated (default value = 0)
</pre> <p>

<hr> <p>

Equations: <p>

<b>when pre(counter2) &lt;&gt; 0 and sample(startTime, period) then</b> <br>
<b>T0 = time</b> <br>
<b>counter2 = pre(counter)</b> <br>
<b>counter = pre(counter) - (if pre(counter) &gt; 0 then 1 else 0)</b> <br>
<b>end when</b> <br>
<b>f = offset + (if (time &lt; startTime or counter2 == 0 or time &gt;= T0 +
    T_falling) then 0 else if (time &lt; T0 + T_rising) then (time - T0)*f0/T_rising
    else if (time &lt; T0 + T_width) then f0 else 
    (T0 + T_falling - time)*f0/(T_falling - T_width))</b>
</html>
"));
      Modelica.Blocks.Sources.Trapezoid Trapezoid1(
        rising=rising,
        width=width,
        falling=falling,
        period=period,
        nperiod=nperiod,
        offset=offset,
        startTime=startTime,
        amplitude=f0) annotation (extent=[68,66; 90,88]);
    equation 
      f = Trapezoid1.y;
    end trapSf;
    
    model tableSf "Tabular flow source of the Modelica bond graph library" 
      extends Interfaces.ActiveOnePort;
      parameter Real table[:, :]=[0, 0; 1, 1; 2, 4] 
        "Table matrix (time = first column, voltage = second column)";
      parameter Real offset=0 "Offset of time function";
      parameter Modelica.SIunits.Time startTime=0 "Start time of time function";
      annotation (
        Icon(
          Text(
            extent=[-54,66; 46,-68],
            style(color=0),
            string="Sf"),
          Text(extent=[-66,102; 74,62],  string="%name"),
             Line(points=[-64,-64; -64,56; 36,56; 36,-64; -64,-64; -64,-34; 36,
                -34; 36,-4; -64,-4; -64,26; 36,26; 36,56; -14,56; -14,-65],
                     style(color=8))),
        Diagram(
          Polygon(points=[-80,80; -88,58; -72,58; -80,80],     style(color=8,
                fillColor=8)),
          Line(points=[-80,58; -80,-90],   style(color=8)),
          Line(points=[-90,-80; 82,-80],   style(color=8)),
          Polygon(points=[90,-80; 68,-72; 68,-88; 90,-80],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Rectangle(extent=[-20,80; 30,-40],   style(
              color=7,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-20,-40; -20,80; 80,80; 80,-40; -20,-40; -20,-10; 80,-10;
                80,20; -20,20; -20,50; 80,50; 80,80; 30,80; 30,-41],
              style(color=0)),
          Text(
            extent=[-77,-52; -38,-68],
            string="offset",
            style(color=9)),
          Polygon(points=[-31,-40; -33,-50; -28,-50; -31,-40],     style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Polygon(points=[-30,-80; -33,-70; -28,-70; -30,-80; -30,-80],
              style(
              color=8,
              fillColor=8,
              fillPattern=1)),
          Line(points=[-31,-41; -31,-80],   style(
              color=8,
              pattern=1,
              thickness=1,
              arrow=0)),
          Line(points=[-20,-42; -20,-80],   style(color=8, pattern=2)),
          Text(
            extent=[-38,-80; 8,-98],
            string="startTime",
            style(color=9)),
          Line(points=[-20,-40; -80,-40],   style(color=8, pattern=2)),
          Text(
            extent=[72,-87; 97,-99],
            string="time",
            style(color=9)),
          Text(
            extent=[-15,73; 24,58],
            string="time",
            style(color=0)),
          Text(
            extent=[33,73; 76,57],
            string="v",
            style(color=0)),
          Text(
            extent=[-77,80; -54,58],
            style(color=9),
            string="f")),
        Documentation(info="<html>
The tabular flow source is an active OnePort element.  It inherits the effort and flow variables from the active OnePort.  The tabular flow to be generated is modeled as a two-dimensional parameter table plus a set of two scalar parameters. <p>

The flow source has its causality stroke at the source. <p>

<hr>

<pre>
Potential variables:

 e:          Bondgraphic effort variable

 f:          Bondgraphic flow variable, normalized positive for flows out of the model


Parameters:

 table:      Two-dimensional table with supporting values (default value = [0, 0; 1, 1; 2, 4])

 offset:     Offset of time function to be generated (default value = 0)

 startTime:  Start time of time function to be generated (default value = 0)
</pre> <p>

<hr> <p>

The model generates a flow source by <b>linear interpolation</b> in
a table. The time points and flow values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
time points, and the second column contains the flow to be interpolated.
The table interpolation has the following proporties: <p>

<ul>
<li>The time points need to be <b>monotonically increasing</b>.
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table.
<li>Values <b>outside</b> of the table range, are computed by
    <b>extrapolation</b> through the last or first two points of the
    table.
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the voltage value is just returned independantly of the
    actual time instant, i.e., this is a constant voltage source.
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined
    by the table can be shifted both in time and in the voltage.
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries,
    in order to not integrate over a discontinuous or not differentiable
    points.
</ul> <p>

<hr> <p>

<b>Example:</b> <p>

<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the flow f =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the flow f =  2.5,
    e.g., time = 2.0, the flow f =  4.0,
    e.g., time = 5.0, the flow f = 23.0 (i.e. extrapolation).
</pre>
</html>
"));
      Modelica.Blocks.Sources.TimeTable TimeTable1(
        table=table,
        offset=offset,
        startTime=startTime) annotation (extent=[68,68; 90,90]);
    equation 
      f = TimeTable1.y;
    end tableSf;
    
    model mSf2 
      extends Interfaces.ModTwoPort;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-60, 80; 60, -80],
            string="mSf",
            style(color=0))),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="mSf",
            style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.63),
        Documentation(info="<html>
The modulated internal flow source is a modulated directed TwoPort element.  It inherits the effort and flow variables from the modulated directed TwoPort. <p>

The internal source takes power out of the system to the left, and feeds it into the system to the right.  The modulation signal is used to set the <b>flow</b> of the inflow.  The <b>flow</b> of the outflow is computed such that power in equals power out. <p>

The internal flow source is primarily used in the computation of convective flows, when entropy flow has to accompany mass flow. <p>

The modulated internal flow source has two causality strokes. <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 s:   Modulation input signal
</pre> <p>

<hr> <p>

Equations: <p>

<b>f1 = s</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      f1 = s;
      e1*f1 = e2*f2;
    end mSf2;
    
    model mSf_int "Internal modulated flow source of the bond graph library" 
      extends Interfaces.ModTwoPort;
      Real f0 "Bondgraphic Flow Source";
      annotation (
        Icon(   Text(
            extent=[-62,80; 58,-80],
            style(color=0),
            string="mSf"),   Text(extent=[-70,90; 70,50],   string="%name")),
        Diagram(Text(
            extent=[-62,80; 58,-80],
            style(color=0),
            string="mSf")),
        Documentation(info="<html>
The modulated internal flow source is a modulated directed TwoPort element.  It inherits the effort and flow variables from the modulated directed TwoPort.  The modulated flow to be generated is modeled as a real-valued variable. <p>

The modulated internal flow source is a modulated flow source on its secondary side.  The primary side provides the power needed by the flow source.  This element can e.g. be used to model a DC current supply together with the power cable that draws the required energy from the grid. <p>

The flow source has its secondary causality stroke away from the source.  Its primary causality stroke is free. <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 f0:  Flow to be generated (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f2 = f0</b> <br>
<b>e1*f1 = e2*f2</b>
</html>
"));
    equation 
      f0 = s;
      f2 = f0;
      e1*f1 = e2*f2;
    end mSf_int;
    
    model mSP "Source of power" 
      extends Interfaces.ModActiveOnePort;
      Real P0 "Bondgraphic Power Source";
      annotation (
        Diagram(Text(
            extent=[-62, 80; 58, -80],
            style(color=0),
            string="mSP")),
        Icon(Text(
            extent=[-62, 80; 58, -80],
            style(color=0),
            string="mSP"), Text(extent=[-70, 92; 70, 52], string="%name")),
        Documentation(info="<html>
The modulated power source is a modulated active OnePort element.  It inherits the effort and flow variables from the modulated active OnePort.  The modulated power to be generated is modeled as a real-valued variable. <p>

Modulated sources are commonly used to model actuators of control systems.  They are the most common and convenient way for converting signal variables to bondgraphic variables, thereby allowing individual blocks of a block diagram, e.g. describing a control system, to be modeled internally using bond graphs. <p>

The power source has free causality. <p>

<hr>

<pre>
Potential variables:

 e:   Bondgraphic effort variable

 f:   Bondgraphic flow variable, normalized positive for flows out of the model

 P0:  Power to be generated (modulated input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e*f = P0</b>
</html>
"));
    equation 
      P0 = s;
      P0 = e*f;
    end mSP;
  end Sources;
  
    package Spice "Spice-style semiconductor models" 
      annotation (preferedView="info",
        Icon(
          Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30,
                fillPattern=
                  1)),
          Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                fillColor=30, fillPattern=1)),
          Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                fillColor=30, fillPattern=1)),
          Text(
            extent=[-85,35; 65,-85],
            string="Library",
            style(color=3)),
          Text(
            extent=[-120,122; 120,73],
            string="%name",
            style(color=1))), Documentation(info="<html>
This package contains Spice-style models of electronic circuit elements built in bond graph technology.  The library implements facets of a dialect of <b>HSpice</b>, called <b>BBSpice</b>, further developed from HSpice at <i>Burr Brown, Inc.</i> in Tucson Arizona, a company that has meanwhile been acquired by <i>Texas Instruments, Inc.</i> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, Chapter 6.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
      package Interfaces "Interfaces for Spice-style electronic circuit models" 
      annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=58)),
            Text(
              extent=[-120,122; 120,73],
              string="%name",
              style(color=1))), Documentation(info="<html>
This package contains some common interfaces for Spice-style electronic circuit models.
</html>
"));
        partial model SpiceTwoPort 
        "Special twoPort element used by Spice models" 
          extends BondLib.Interfaces.TwoPort;
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          Modelica.SIunits.Temperature DTemp 
          "Difference between circuit temperature and reference temperature";
          Real DTempSq "Square of DTemp";
        annotation (Icon(
              Text(
                extent=[24,-4; 92,-34],
                string="Spice",
                style(color=0, rgbcolor={0,0,0})),
              Text(extent=[-40,86; 40,56],   string="%name")), Diagram(
              Text(
                extent=[24,-4; 92,-34],
                string="Spice",
                style(color=0, rgbcolor={0,0,0}))));
        equation 
          DTemp = e2 - Tnom;
          DTempSq = DTemp*DTemp;
        end SpiceTwoPort;
      end Interfaces;
    
      model RS "Spice-style resistor model" 
        extends Interfaces.SpiceTwoPort;
        parameter Modelica.SIunits.Resistance R=1 
        "Resistance at reference temperature";
        parameter Real TR1(unit="Ohm/K") = 0 "Linear temperature coefficient";
        parameter Real TR2(unit="Ohm/K2") = 0 
        "Quadratic temperature coefficient";
        parameter Real Area=1 "Relative area occupied by resistor";
        Modelica.SIunits.Resistance Rval 
        "Resistance value at circuit temperature";
        annotation (
          Icon(
            Text(
              extent=[-82,76; 28,-80],
              string="RS",
              style(color=0)),
            Text(extent=[-72,-58; 66,-86],   string="R=%R")),
          Diagram(
            Text(
              extent=[-82,76; 28,-80],
              string="RS",
              style(color=0))),
          Documentation(info="<html>
The Spice-style resistor has a reference resistance value computed at the reference temperature, <b>Tnom</b>, and for a relative area of <b>Area=1</b>.  It changes its value as a function of the circuit temperature and the relative area occupied by the resistor. <p>

<hr> <p>

<pre>
Parameters:

 R:     Reference resistance (default value = 1 Ohm)

 Tnom:  Reference temperature (default value = 300.15 <sup>o</sup>K)

 TR1:   Linear temperature variation coefficient (default value = 0 Ohm/<sup>o</sup>K)

 TR2:   Quadratic temperature variation coefficient (default value = 0 Ohm/<sup>o</sup>K<sup>2</sup>)

 Area:  Relative area occupied by resistor (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>DTemp = e2 - Tnom</b> <br>
<b>Rval = R*(1 + TR1*DTemp + TR2*DTemp*DTemp)/Area</b> <br>
<b>e1 = Rval*f1</b> <br>
<b>f2 = (e1/e2)*f1</b>
</html>
"));
      equation 
        Rval = R*(1 + TR1*DTemp + TR2*DTempSq)/Area;
        e1 = Rval*f1;
        e1*f1 = e2*f2;
      end RS;
    
      model CS "Spice-style capacitor model" 
        extends Interfaces.SpiceTwoPort;
        parameter Modelica.SIunits.Capacitance C=1 
        "Capacitance at reference temperature";
        parameter Real TC1(unit="F/K") = 0 "Linear temperature coefficient";
        parameter Real TC2(unit="F/K2") = 0 "Quadratic temperature coefficient";
        parameter Real Area=1 "Relative area occupied by capacitor";
        Modelica.SIunits.Capacitance Cval 
        "Capacitance value at circuit temperature";
        annotation (
          Icon(
            Text(
              extent=[-84,76; 26,-80],
              style(color=0),
              string="CS"),
            Text(extent=[-62,-58; 48,-86], string="C=%C")),
          Diagram(
            Text(
              extent=[-84,76; 26,-80],
              style(color=0),
              string="CS")),
          Documentation(info="<html>
The Spice-style capacitor has a reference capacitance value computed at the reference temperature, <b>Tnom</b>, and for a relative area of <b>Area=1</b>.  It changes its value as a function of the circuit temperature and the relative area occupied by the capacitor. <p>

<hr> <p>

<pre>
Parameters:

 C:     Reference capacitance (default value = 1 F)

 Tnom:  Reference temperature (default value = 300.15 <sup>o</sup>K)

 TC1:   Linear temperature variation coefficient (default value = 0 F/<sup>o</sup>K)

 TC2:   Quadratic temperature variation coefficient (default value = 0 F/<sup>o</sup>K<sup>2</sup>)

 Area:  Relative area occupied by capacitor (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>DTemp = e2 - Tnom</b> <br>
<b>Cval = C*(1 + TC1*DTemp + TC2*DTemp*DTemp)*Area</b> <br>
<b>q1 = Cval*e1</b> <br>
<b>der(q1) = f1</b> <br>
<b>f2 = 0</b>
</html>
"));
      equation 
        Cval = C*(1 + TC1*DTemp + TC2*DTempSq)*Area;
        f1 = Cval*der(e1);
        f2 = 0;
      end CS;
    
      model LS "Spice-style inductor model" 
        extends Interfaces.SpiceTwoPort;
        parameter Modelica.SIunits.Inductance L=1 
        "Inductance at reference temperature";
        parameter Real TL1(unit="H/K") = 0 "Linear temperature coefficient";
        parameter Real TL2(unit="H/K2") = 0 "Quadratic temperature coefficient";
        Modelica.SIunits.Capacitance Lval 
        "Inductance value at circuit temperature";
        annotation (
          Icon(
            Text(
              extent=[-82,76; 28,-80],
              style(color=0),
              string="LS"),
            Text(extent=[-74,-58; 70,-88], string="L=%L")),
          Diagram(
            Text(
              extent=[-82,76; 28,-80],
              style(color=0),
              string="LS")),
          Documentation(info="<html>
The Spice-style inductor has a reference inductance value computed at the reference temperature, <b>Tnom</b>.  It changes its value as a function of the circuit temperature. <p>

<hr> <p>

<pre>
Parameters:

 L:     Reference inductance (default value = 1 H)

 Tnom:  Reference temperature (default value = 300.15 <sup>o</sup>K)

 TL1:   Linear temperature variation coefficient (default value = 0 H/<sup>o</sup>K)

 TL2:   Quadratic temperature variation coefficient (default value = 0 H/<sup>o</sup>K<sup>2</sup>)
</pre> <p>

<hr> <p>

Equations: <p>

<b>DTemp = e2 - Tnom</b> <br>
<b>Lval = L*(1 + TL1*DTemp + TL2*DTemp*DTemp)</b> <br>
<b>p1 = Lval*f1</b> <br>
<b>der(p1) = e1</b> <br>
<b>f2 = 0</b>
</html>
"));
      equation 
        Lval = L*(1 + TL1*DTemp + TL2*DTempSq);
        e1 = Lval*der(f1);
        f2 = 0;
      end LS;
    
        model DS 
      "Spice-style diode model used inside the bipolar transistor models" 
          import BondLib;
          extends BondLib.Interfaces.TwoPort;
          constant Real inf=Modelica.Constants.inf;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
        "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
        "Electron charge";
      
          /* General parameters */
          parameter Integer Level(min=1,max=3) = 1 
        "DIODE modeling level (check documentation window for details)";
          parameter Real Area(min=Modelica.Constants.small) = 1 
        "Relative device area";
      
          /* Process parameters */
          parameter Modelica.SIunits.Length TOX(min=Modelica.Constants.small) = 1e-7 
        "|Process parameters|Thin oxide thickness";
      
          /* Basic Electrical parameters */
          parameter Modelica.SIunits.Voltage PB(min=0) = 0.8 
        "|Electrical parameters|Built-in junction potential at reference temperature";
          parameter Modelica.SIunits.Voltage BV(min=0) = 0 
        "|Electrical parameters|Reverse breakdown voltage at reference temperature";
          parameter Modelica.SIunits.Current IBV(min=0) = 0 
        "|Electrical parameters|Reverse breakdown current";
          parameter Modelica.SIunits.Voltage PHP(min=0) = 0.8 
        "|Electrical parameters|Built-in perimeter potential at reference temperature";
          parameter Real N=1 
        "|Electrical parameters|Current emission coefficient";
          parameter Modelica.SIunits.Current IS(min=Modelica.Constants.small) = 1e-14 
        "|Electrical parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISW(min=0) = 0 
        "|Electrical parameters|Saturation current of perimeter at reference temperature";
          parameter Real PJ(min=0) = 1 
        "|Electrical parameters|Relative perimeter width";
      
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF(min=Modelica.Constants.small) = inf 
        "|High current beta degradation effect|Forward knee current";
          parameter Modelica.SIunits.Current IKR(min=Modelica.Constants.small) = inf 
        "|High current beta degradation effect|Reverse knee current";
      
          /* Parasitic resistor parameters */
          parameter Modelica.SIunits.Resistance RS(min=0) = 0 
        "|Parasitic resistor parameters|Resistance at reference temperature";
      
          /* Junction capacitance parameters */
          parameter Modelica.SIunits.Capacitance CJ(min=0) = 0.5e-12 
        "|Junction capacitance parameters|Zero-bias bulk capacitance at reference temperature";
          parameter Real MJ(min=0.1,max=0.9) = 0.33 
        "|Junction capacitance parameters|Bulk junction grading coefficient";
          parameter Modelica.SIunits.Capacitance CJSW(min=0) = 0.5e-12 
        "|Junction capacitance parameters|Zero-bias perimeter capacitance at reference temperature";
          parameter Real MJSW(min=0.1,max=0.9) = 0.33 
        "|Junction capacitance parameters|Perimeter capacitance grading coefficient";
          parameter Real FC(min=0,max=0.95) = 0.5 
        "|Junction capacitance parameters|Forward-bias depletion capacitance coefficient";
      
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TT(min=0) = 0 
        "|Transit time parameters|Transit time at reference temperature";
      
          /* Temperature compensation parameters */
          parameter Real TRS(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of resistance";
          parameter Real TCV(unit="V/K")=0 
        "|Temperature compensation parameters|Linear temperature coefficient of breakdown voltage";
          parameter Real TTT1(unit="s/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of transit time";
          parameter Real TTT2(unit="s/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of transit time";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI(min=0) = 3 
        "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage EG(min=0.1) = 1.16 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
      
          /* Level=2 General parameters */
          parameter Integer M(min=1) = 1 "|Level=2||Diode multiplier";
      
          /* Level=2 Geometric parameters */
          parameter Modelica.SIunits.Length L(min=0) = 0 
        "|Level=2|Geometric parameters|Diode length";
          parameter Modelica.SIunits.Length W(min=0) = 0 
        "|Level=2|Geometric parameters|Diode width";
      
          /* Level=2 Electrical parameters */
          parameter Real JF(min=0,unit="A/V2") = 0 
        "|Level=2|Electrical parameters|Forward Fowler-Nordheim current coefficient";
          parameter Real JR(min=0,unit="A/V2") = 0 
        "|Level=2|Electrical parameters|Reverse Fowler-Nordheim current coefficient";
          parameter Modelica.SIunits.ElectricFieldStrength EF(min=0) = 0 
        "|Level=2|Electrical parameters|Forward Fowler-Nordheim critical electrical field";
          parameter Modelica.SIunits.ElectricFieldStrength ER(min=0) = 0 
        "|Level=2|Electrical parameters|Reverse Fowler-Nordheim critical electrical field";
      
          /* Level=3 Geometric parameters */
          parameter Modelica.SIunits.Length LP(min=0) = 0 
        "|Level=3|Geometric parameters|Polysilicon length";
          parameter Modelica.SIunits.Length WP(min=0) = 0 
        "|Level=3|Geometric parameters|Polysilicon width";
          parameter Modelica.SIunits.Length XP(min=0) = 0 
        "|Level=3|Geometric parameters|Widening of polysilicon due to masking and etching";
          parameter Modelica.SIunits.Length XOI(min=0) = 0 
        "|Level=3|Geometric parameters|Polysilicon thickness";
          parameter Modelica.SIunits.Length LM(min=0) = 0 
        "|Level=3|Geometric parameters|Metal length";
          parameter Modelica.SIunits.Length WM(min=0) = 0 
        "|Level=3|Geometric parameters|Metal width";
          parameter Modelica.SIunits.Length XM(min=0) = 0 
        "|Level=3|Geometric parameters|Widening of metal due to masking and etching";
          parameter Modelica.SIunits.Length XOM(min=0) = 0 
        "|Level=3|Geometric parameters|Metal thickness";
      
          /* Level=3 Electrical parameters */
          parameter Modelica.SIunits.CurrentDensity JS(min=Modelica.Constants.small) = 1e-4 
        "|Level=3|Electrical parameters|Saturation current density at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JSW(min=0) = 0 
        "|Level=3|Electrical parameters|Saturation current density of perimeter at reference temperature";
      
          /* Level=3 Junction capacitance parameters */
          parameter Real CJ2(min=Modelica.Constants.small,unit="F/m2") = 1e-4 
        "|Level=3|Junction capacitance parameters|Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real CJSW2(min=0,unit="F/m") = 1e-9 
        "|Level=3|Junction capacitance parameters|Zero-bias perimeter capacitance per meter at reference temperature";
      
          /* Numerical parameters */
          parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=Modelica.Constants.small) = 1e-16 
        "|Advanced|Numerical parameters|Leakage conductance";
      
    protected 
          parameter Modelica.SIunits.Area AREAeff = W*L*M;
          parameter Modelica.SIunits.Length PJeff = 2*(W + L)*M;
          parameter Modelica.SIunits.Voltage Vt = k*Tnom/q;
          parameter Modelica.SIunits.Current ISeff = if Level==1 then Area*IS + PJ*ISW else 
                                                     if Level==3 then AREAeff*JS + PJeff*JSW else 0;
          parameter Modelica.SIunits.Current Ibreak = -ISeff*(exp(-BV/(N*Vt)) - 1);
          parameter Modelica.SIunits.Current IBVval = if IBV < Ibreak then Ibreak else IBV;
          parameter Modelica.SIunits.Voltage BVval = if BV > 0 then BVval - (N*Vt)*ln(IBVval/Ibreak) else 0;
        annotation (
          Icon(
              Text(
                extent=[-82,76; 28,-80],
                style(color=0),
              string="DS"), Text(
            extent=[8,-10; 108,-38],
            string="Spice",
            style(
              color=0,
              rgbcolor={0,0,0},
              pattern=0))),
          Diagram,
          Documentation(info="<html>
The Spice-style diode offers three modeling levels [1].  The <b>Level=1</b> model characterizes a junction diode by its electrical properties.  This model is similar to the models of the junction diodes used within the BJT and MOSFET models, but it is more accurate.  In particular, it includes a description of avalanche breakdown behavior.  The <b>Level=3</b> model is similar to the <b>Level=1</b> model, but here, the diode is characterized by its physical parameters, such as length and width of the junction.  Also, this model is yet more accurate as it allows the modeler to specify a polysilicon and a metal layer, in which case the additional capacitances associated with these layers will be computed as well.  The <b>Level=2</b> model is an entirely different device.  It represents essentially a non-linear resistor with a quadratic characteristic on both sides.  Thus, the Fowler-Nordheim diode is a symmetric device. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   DIODE modeling level (default value = 1)
            Level=1: Non-geometric junction diode
            Level=2: Fowler-Nordheim diode
            Level=3: Geometric junction diode</b></font>

 Area:    Relative area occupied by the diode (default value = 1) <font color=red>Level 1</font>


<b>Process Parameters:</b>

 TOX:     Thin oxide thickness (default value = 1e-7 m) <font color=red>Levels 1-3</font>


<b>Basic Electrical Parameters:</b>

 PB:      Built-in junction potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1,3</font>

 BV:      Reverse breakdown voltage at reference temperature (default value = 0 Volt) <font color=red>Levels 1,3</font>
            <font color=blue><b>BV must be specified as a positive number.  A value of 0 indicates that the breakdown voltage is infinite.</b></font>

 IBV:     Reverse breakdown current (default value = 0 Amp) <font color=red>Levels 1,3</font>
            <font color=blue><b>IBV must be specified as a positive number.  A value of 0 indicates that the breakdown voltage is infinite.
            Either BV or IBV (the diodic current at breakdown) can be specified.  If both are specified and
            the values are inconsistent with each other, they will be suitably adjusted to match.</b></font>

 PHP:     Built-in perimeter potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1,3</font>

 N:       Current emission coefficient (default value = 1) <font color=red>Levels 1,3</font>

 IS:      Saturation current at reference temperature (default value = 1e-14 Amp) <font color=red>Level 1</font>

 ISW:     Saturation current of perimeter at reference temperature (default value = 0 Amp) <font color=red>Level 1</font>

 PJ:      Relative perimeter width (default value = 1) <font color=red>Level 1</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Forward knee current (default value = &infin; Amp) <font color=red>Levels 1,3</font>

 IKR:     Reverse knee current (default value = &infin; Amp) <font color=red>Levels 1,3</font>


<b>Parasitic Resistor Parameters:</b>

 RS:      Resistance at reference temperature (default value = 0 Ohm) <font color=red>Levels 1-3</font>


<b>Junction Capacitance Parameters:</b>

 CJ:      Zero-bias bulk capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Level 1</font>

 MJ:      Bulk junction grading coefficient (default value = 0.33) <font color=red>Levels 1,3</font>

 CJSW:    Zero-bias perimeter capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Level 1</font>

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33) <font color=red>Levels 1,3</font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1,3</font>


<b>Transit Time Parameters:</b>

 TT:      Transit time at reference temperature (default value = 0 sec) <font color=red>Levels 1,3</font>


<b>Temperature Compensation Parameters:</b>

 TRS:     Linear temperature coefficient of resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TCV:     Linear temperature coefficient of breakdown voltage (default value = 0 V/K) <font color=red>Levels 1,3</font>

 TTT1:    Linear temperature coefficient of transit time (default value = 0 s/K) <font color=red>Levels 1,3</font>
 
 TTT2:    Quadratic temperature coefficient of transit time (default value = 0 s/K2) <font color=red>Levels 1,3</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1-3</font>

 XTI:     Temperature exponent of saturation current (default value = 3) <font color=red>Levels 1,3</font>
            <font color=blue><b>XTI = 3 for <i>pn</i>-junction
            XTI = 2 for Schottky barrier diode</b></font>

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt) <font color=red>Levels 1,3</font>
            <font color=blue><b>EG = 1.16 for Si
            EG = 0.73 for Schottky barrier diode
            EG = 0.71 for Ge</b></font>


<b>Level=2 General Parameters:</b>

 M:       Diode multiplier (default value = 1) <font color=red>Levels 2,3</font>


<b>Level=2 Geometric Parameters:</b>

 L:       Diode length (default value = 0 m) <font color=red>Levels 2,3</font>

 W:       Diode width (default value = 0 m) <font color=red>Levels 2,3</font>


<b>Level=2 Electrical Parameters:</b>

 JF:       Forward Fowler-Nordheim current coefficient (default value = 0 A/V<sup>2</sup>) <font color=red>Level 2</font>

 JR:       Reverse Fowler-Nordheim current coefficient (default value = 0 A/V<sup>2</sup>) <font color=red>Level 2</font>

 EF:       Forward Fowler-Nordheim critical electrical field (default value = 0 V/m) <font color=red>Level 2</font>

 ER:       Reverse Fowler-Nordheim critical electrical field (default value = 0 V/m) <font color=red>Level 2</font>


<b>Level=3 Geometric Parameters:</b>

 LP:       Polysilicon length (default value = 0 m) <font color=red>Level 3</font>

 WP:       Polysilicon width (default value = 0 m) <font color=red>Level 3</font>

 XP:       Widening of polysilicon due to masking and etching (default value = 0 m) <font color=red>Level 3</font>

 XOI:      Polysilicon thickness (default value = 0 m) <font color=red>Level 3</font>

 LM:       Metal length (default value = 0 m) <font color=red>Level 3</font>

 WM:       Metal width (default value = 0 m) <font color=red>Level 3</font>

 XM:       Widening of metal due to masking and etching (default value = 0 m) <font color=red>Level 3</font>

 XOM:      Metal thickness (default value = 0 m) <font color=red>Level 3</font>


<b>Level=3 Electrical Parameters:</b>

 JS:      Saturation current density at reference temperature (default value = 1e-4 Amp/m<sup>2</sup>) <font color=red>Level 3</font>

 JSW:     Saturation current density of perimeter at reference temperature (default value = 0 Amp/m<sup>2</sup>) <font color=red>Level 3</font>


<b>Level=2 Junction Capacitance Parameters:</b>

 CJ2:     Zero-bias bulk capacitance per meter square at reference temperature (default value = 1e-4 F/m<sup>2</sup>) <font color=red>Level 3</font>

 CJSW2:   Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1,3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1,3</font>

 GminDC:  Leakage conductance (default value = 1e-16 mho) <font color=red>Levels 1,3</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
    public 
      Junctions.J0p2 J0p2_1 annotation (extent=[-90,-10; -70,10]);
      Junctions.J0p5 J0p5_1 annotation (extent=[70,-10; 90,10]);
      Bonds.Bond B1 annotation (extent=[-70,-10; -50,10]);
      Junctions.J1p3 J1p3_1 annotation (extent=[-50,-10; -30,10]);
      Bonds.Bond B2 annotation (extent=[-30,-10; -10,10]);
      Junctions.J0p4 J0p4_1 annotation (extent=[-10,-10; 10,10]);
      Bonds.fBond B4 annotation (extent=[50,-10; 70,10], rotation=180);
      Bonds.eBond B3 annotation (extent=[10,-10; 30,10], rotation=180);
      Bonds.Bond B5 annotation (extent=[-50,-30; -30,-10], rotation=-90);
      Bonds.eBond B6 annotation (extent=[-50,-90; -30,-70], rotation=270);
      Bonds.Bond B7 annotation (extent=[-10,10; 10,30], rotation=90);
      Utilities.CjD CjD1(
        Level=Level,
        Tnom=Tnom,
        IS=IS,
        N=N,
        TT=TT,
        CJ=CJ,
        PB=PB,
        MJ=MJ,
        EG=EG,
        XTI=XTI,
        FC=FC,
        BV=BVval,
        IBV=IBVval,
        TCV=TCV,
        TTT1=TTT1,
        TTT2=TTT2,
        ISW=ISW,
        CJSW=CJSW,
        PHP=PHP,
        MJSW=MJSW,
        TOX=TOX,
        GminDC=GminDC,
        Area=Area,
        PJ=PJ,
        EMin=EMin,
        EMax=EMax,
        L=L,
        W=W,
        M=M,
        JS=JS,
        CJ2=CJ2,
        JSW=JSW,
        CJSW2=CJSW2,
        LP=LP,
        WP=WP,
        XP=XP,
        XOI=XOI,
        LM=LM,
        WM=WM,
        XM=XM,
        XOM=XOM,
        JF=JF,
        JR=JR,
        EF=EF,
        ER=ER) annotation (extent=[10,30; -10,70], rotation=90);
      Utilities.ID0 ID0(
        Level=Level,
        N=N,
        IKF=IKF,
        IKR=IKR) annotation (extent=[20,60; 40,40]);
      Bonds.eBond B8 annotation (extent=[-10,70; 10,90], rotation=90);
      Modelica.Blocks.Math.Gain Gain1(k=-1) annotation (extent=[50,44; 62,56]);
      Bonds.fBond B9 annotation (extent=[70,10; 90,30], rotation=90);
      Sensors.De Tdev annotation (extent=[70,30; 90,50], rotation=90);
      Bonds.fBond B10 annotation (extent=[-10,-30; 10,-10], rotation=270);
      Sensors.De vd annotation (extent=[-10,-50; 10,-30], rotation=270);
      BondLib.Spice.RS RS1(
        Tnom=Tnom,
        R=RS,
        TR1=TRS,
        TR2=0,
        Area=Area) annotation (extent=[-50,-70; -30,-30], rotation=-90);
      BondLib.Sources.mSf_int mSf_int1 annotation (extent=[50,10; 30,-10]);
        equation 
      connect(J0p2_1.BondCon1, BondCon1) annotation (points=[-90,0; -100,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_1.BondCon2, BondCon2) annotation (points=[90,0; 100,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon1, J0p2_1.BondCon2) annotation (points=[-69.8,0; -70,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-50,0; -50,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[-30,0; -29.8,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B2.BondCon2, J0p4_1.BondCon1) annotation (points=[-10,0; -10,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon2, B3.eBondCon1) annotation (points=[10,0; 15,0; 15,
            1.22461e-015; 10,1.22461e-015], style(color=8, rgbcolor={192,192,
              192}));
      connect(B4.eBondCon1, J0p5_1.BondCon1) annotation (points=[70,
            -1.22461e-015; 70,0], style(color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon1, J1p3_1.BondCon3) annotation (points=[-40,-10.2; -40,
            -10], style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1, J0p4_1.BondCon4) annotation (points=[-6.00057e-016,
            10.2; -6.00057e-016,17.1; 0,17.1; 0,10], style(color=8, rgbcolor={
              192,192,192}));
      connect(CjD1.BondCon1, B7.BondCon2) annotation (points=[6.12303e-016,30; 
            6.12303e-016,30], style(color=8, rgbcolor={192,192,192}));
      connect(CjD1.id, ID0.u) annotation (points=[10,50; 18,50], style(color=74,
            rgbcolor={0,0,127}));
      connect(B8.fBondCon1, CjD1.BondCon2) annotation (points=[-6.12303e-016,70; 
            -6.12303e-016,70], style(color=8, rgbcolor={192,192,192}));
      connect(Gain1.u, ID0.y) annotation (points=[48.8,50; 41,50], style(color=
              74, rgbcolor={0,0,127}));
      connect(B9.eBondCon1, J0p5_1.BondCon5) annotation (points=[80,10; 80,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(Tdev.BondCon1, B9.fBondCon1) annotation (points=[80,30; 80,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(Tdev.OutPort1, ID0.Tdev) annotation (points=[80,51; 80,70; 30,70; 
            30,60], style(color=74, rgbcolor={0,0,127}));
      connect(B10.eBondCon1, J0p4_1.BondCon3) annotation (points=[1.83691e-015,
            -10; 0,-10], style(color=8, rgbcolor={192,192,192}));
      connect(vd.BondCon1, B10.fBondCon1) annotation (points=[1.83691e-015,-30; 
            -1.83691e-015,-30], style(color=8, rgbcolor={192,192,192}));
      connect(vd.OutPort1, ID0.v) annotation (points=[-2.0206e-015,-51; 
            -2.0206e-015,-60; 20,-60; 20,20; 30,20; 30,40], style(color=74,
            rgbcolor={0,0,127}));
      connect(B6.eBondCon1, J0p5_1.BondCon3) annotation (points=[-40,-90; -40,
            -96; 76,-96; 76,-10], style(color=8, rgbcolor={192,192,192}));
      connect(B8.eBondCon1, J0p5_1.BondCon4) annotation (points=[6.12303e-016,90; 
            0,90; 0,94; 110,94; 110,-20; 84,-20; 84,-10],     style(color=8,
            rgbcolor={192,192,192}));
      connect(RS1.BondCon1, B5.BondCon2) annotation (points=[-40,-30; -40,-30],
          style(color=8, rgbcolor={192,192,192}));
      connect(RS1.BondCon2, B6.fBondCon1) annotation (points=[-40,-70; -40,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B3.fBondCon1, mSf_int1.BondCon2) annotation (points=[30,
            -1.22461e-015; 30,0], style(color=8, rgbcolor={192,192,192}));
      connect(B4.fBondCon1, mSf_int1.BondCon1) annotation (points=[50,
            1.22461e-015; 50,0], style(color=8, rgbcolor={192,192,192}));
      connect(Gain1.y, mSf_int1.s) annotation (points=[62.6,50; 66,50; 66,20; 
            40,20; 40,9], style(color=74, rgbcolor={0,0,127}));
        end DS;
    
      model NPNlat "Spice-style laterally diffused NPN bipolar transistor" 
        constant Real inf=Modelica.Constants.inf;
      
        /* General parameters */
        parameter Integer Level(min=1,max=2) = 2 
        "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
        parameter Real Area=1 "Relative area occupied by device";
      
        /* DC model parameters */
        parameter Real BF=100 
        "|DC model parameters|Maximum forward current gain at reference temperature";
        parameter Real BR=1 
        "|DC model parameters|Maximum reverse current gain at reference temperature";
        parameter Modelica.SIunits.Current IS=1e-16 
        "|DC model parameters|Saturation current at reference temperature";
        parameter Modelica.SIunits.Current ISS=IS 
        "|DC model parameters|Saturation current used for current injection";
        parameter Real NF=1 
        "|DC model parameters|Forward current emission coefficient";
        parameter Real NR=1 
        "|DC model parameters|Reverse current emission coefficient";
      
        /* Low current beta degradation effect parameters */
        parameter Real C4(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
        parameter Modelica.SIunits.Current ISC(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
        parameter Real C2(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
        parameter Modelica.SIunits.Current ISE(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
        parameter Real NC=2 
        "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
        parameter Real NE=1.5 
        "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
      
        /* High current beta degradation effect parameters */
        parameter Modelica.SIunits.Current IKF=inf 
        "|High current beta degradation effect|Corner for forward beta high-current roll-off";
        parameter Modelica.SIunits.Current IKR=inf 
        "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
      
        /* Base width modulation parameters */
        parameter Modelica.SIunits.Voltage VAF=inf 
        "|Base width modulation parameters|Forward early voltage";
        parameter Modelica.SIunits.Voltage VAR=inf 
        "|Base width modulation parameters|Reverse early voltage";
      
        /* Parasitic resistor parameters */
        parameter Modelica.SIunits.Current IRB=inf 
        "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
        parameter Modelica.SIunits.Resistance RB(min=0) = 0 
        "|Parasitic resistor parameters|Zero-bias base resistance";
        parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
        "|Parasitic resistor parameters|Minimum base resistance at high currents";
        parameter Modelica.SIunits.Resistance RC(min=0) = 0 
        "|Parasitic resistor parameters|Collector resistance";
        parameter Modelica.SIunits.Resistance RE(min=0) = 0 
        "|Parasitic resistor parameters|Emitter resistance";
      
        /* Junction capacitor parameters */
        parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
        parameter Real MJC(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-collector junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJC=0.75 
        "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
        parameter Real MJE(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-emitter junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJE=0.75 
        "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
        parameter Real MJS(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Substrate junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJS=0.75 
        "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
        parameter Real XCJC(min=0,max=1) = 1 
        "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
        parameter Real FC(min=0,max=1) = 0.5 
        "|Junction capacitor parameters|Depletion capacitance factor for linearization";
      
        /* Transit time parameters */
        parameter Modelica.SIunits.Time TF(min=0) = 0 
        "|Transit time parameters|Ideal forward transit time";
        parameter Modelica.SIunits.Time TR(min=0) = 0 
        "|Transit time parameters|Ideal reverse transit time";
      
        /* Temperature compensation parameters */
        parameter Real TRB1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
        parameter Real TRB2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
        parameter Real TRM1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
        parameter Real TRM2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
        parameter Real TRC1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
        parameter Real TRC2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
        parameter Real TRE1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
        parameter Real TRE2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Real XTI=3 
        "|Temperature compensation parameters|Saturation current temperature exponent";
        parameter Real XTB=0 
        "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
        parameter Modelica.SIunits.Voltage EG=1.11 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
        "|Advanced|Numerical parameters|Leakage conductance";
      
    protected 
        parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
        parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
      annotation (Coordsys(extent=[-120,-120; 120,120]), Diagram(
          Text(
            extent=[-126,22; -114,8],
            string="B",
            style(color=10, rgbcolor={95,95,95})),
          Text(
            extent=[-20,128; -8,114],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[-20,-114; -8,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E"),
          Text(
            extent=[108,-6; 134,-22],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[78,50; 104,34],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[42,-34; 52,-46],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[-28,66; -18,54],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[0,20; 10,8],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[-60,70; -50,58],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[-126,-38; -114,-52],
            style(color=10, rgbcolor={95,95,95}),
            string="S")),
        Icon(
          Text(
            extent=[-126,24; -114,10],
            string="B",
            style(color=10, rgbcolor={95,95,95})),
          Text(
            extent=[10,126; 22,112],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[8,-114; 20,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E"),
          Text(
            extent=[106,26; 132,6],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
            Text(
              extent=[-96,84; 68,-92],
              style(color=0),
              string="NPN"),
            Text(
              extent=[62,4; 102,-38],
              style(color=0),
            string="lat"),
            Line(points=[0,100; 0,-100],
                                       style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,-100; 20,-80],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[-20,-80; 0,-100],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Line(points=[-100,-60; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-80; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-40; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Text(
            extent=[-80,100; 80,60],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
          Text(
            extent=[-88,-72; -44,-108],
            string="%Area x",
            style(color=3, rgbcolor={0,0,255}))),
        Documentation(info="<html>
The <b>NPNlat</b> element of the Spice bond graph library implements a full-fledges Spice-stye Gummel-Poon model of the laterally diffused NPN bipolar transistor [1-3]. The model contains a <b>NPNint</b> model implementing the inner parts of the NPN transistor up to the internal nodes. <p>

The NPN bipolar transistor is a directed FourPort.  The direction of positive power flow is assumed into the model at the base, <b>B</b>, and at the collector, <b>C</b>, whereas it is assumed out of the model at the emitter, <b>E</b>, and at the <b>Heat</b> port. <p>

The causality of the NPN model is free. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
    public 
      BondLib.Interfaces.BondCon B "Base" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon E "Emitter" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon H "Heat" annotation (extent=[110,-10; 130,10]);
      BondLib.Interfaces.BondCon C "Collector" 
                                          annotation (extent=[-10,110; 10,130]);
      BondLib.Spice.Utilities.NPNint Qni(
        BF=BF,
        BR=BR,
        IS=IS,
        NF=NF,
        NR=NR,
        GminDC=GminDC,
        NC=NC,
        NE=NE,
        VAF=VAF,
        VAR=VAR,
        IKF=IKF,
        IKR=IKR,
        CJC=CJC,
        MJC=MJC,
        VJC=VJC,
        CJE=CJE,
        MJE=MJE,
        VJE=VJE,
        XCJC=XCJC,
        TF=TF,
        TR=TR,
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        EMin=EMin,
        EMax=EMax,
        FC=FC,
        Level=Level,
        ISC=ISCeff,
        ISE=ISEeff,
        ISS=ISS) annotation (extent=[0,-40; 40,0]);
      BondLib.Bonds.Bond B1 annotation (extent=[-20,-30; 0,-10], rotation=0);
      BondLib.Junctions.J0p3 J0p3_1 
        annotation (extent=[-40,-30; -20,-10], rotation=90);
      BondLib.Bonds.Bond B2 annotation (extent=[-50,-10; -30,10], rotation=0);
      BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-70,10; -50,-10]);
      BondLib.Bonds.Bond B3 annotation (extent=[-90,-10; -70,10], rotation=0);
      BondLib.Bonds.Bond B4 annotation (extent=[-70,10; -50,30], rotation=90);
      BondLib.Spice.Utilities.RBS Rb(
        Tnom=Tnom,
        RB=RB,
        RBM=RBM,
        IRB=IRB,
        TRB1=TRB1,
        TRB2=TRB2,
        TRM1=TRM1,
        TRM2=TRM2,
        Area=Area,
        Level=Level) 
                   annotation (extent=[-60,30; -20,50], rotation=0);
      BondLib.Bonds.eBond B5 annotation (extent=[-20,30; 0,50]);
      BondLib.Bonds.Bond B6 annotation (extent=[-50,-50; -30,-30], rotation=180);
      BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[-70,-50; -50,-30]);
      BondLib.Bonds.Bond B7 annotation (extent=[-90,-50; -70,-30], rotation=180);
      BondLib.Junctions.J0p2 J0p2_2 
        annotation (extent=[-110,-60; -90,-40], rotation=90);
      BondLib.Bonds.Bond B9 annotation (extent=[-70,-70; -50,-50], rotation=270);
      BondLib.Bonds.eBond B10 annotation (extent=[-20,-90; 0,-70]);
      BondLib.Bonds.Bond B11 annotation (extent=[10,40; 30,60], rotation=270);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[30,80; 10,60], rotation=-90);
      BondLib.Bonds.Bond B12 annotation (extent=[10,80; 30,100], rotation=270);
      BondLib.Junctions.J0p2 J0p2_3 
        annotation (extent=[10,100; 30,120], rotation=90);
      BondLib.Bonds.Bond B13 annotation (extent=[30,60; 50,80], rotation=0);
      BondLib.Bonds.eBond B14 annotation (extent=[90,60; 110,80], rotation=0);
      BondLib.Bonds.Bond B15 annotation (extent=[10,-60; 30,-40], rotation=270);
      BondLib.Junctions.J1p3 J1p3_4 
        annotation (extent=[30,-60; 10,-80], rotation=-90);
      BondLib.Bonds.Bond B16 annotation (extent=[10,-100; 30,-80], rotation=270);
      BondLib.Junctions.J0p2 J0p2_4 
        annotation (extent=[10,-120; 30,-100], rotation=90);
      BondLib.Bonds.Bond B17 annotation (extent=[30,-80; 50,-60], rotation=0);
      BondLib.Bonds.eBond B18 annotation (extent=[90,-80; 110,-60], rotation=0);
      BondLib.Bonds.eBond B19 annotation (extent=[40,-30; 60,-10]);
      BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[60,20; 80,40]);
      BondLib.Bonds.Bond B20 annotation (extent=[10,0; 30,20], rotation=270);
      BondLib.Junctions.J0p3 J0p3_2 
        annotation (extent=[10,20; 30,40], rotation=90);
      BondLib.Junctions.J0p3 J0p3_3 
        annotation (extent=[-110,-10; -90,10], rotation=180);
      BondLib.Bonds.Bond B21 annotation (extent=[-110,10; -90,30], rotation=90);
      BondLib.Junctions.J1p3 J1p3_5 
        annotation (extent=[-110,30; -90,50], rotation=90);
      BondLib.Bonds.Bond B22 annotation (extent=[-110,50; -90,70], rotation=90);
      BondLib.Bonds.Bond B23 annotation (extent=[-90,50; -70,70], rotation=90);
      BondLib.Bonds.eBond B24 annotation (extent=[-40,70; -20,90]);
      BondLib.Bonds.eBond B25 annotation (extent=[80,20; 100,40]);
      BondLib.Junctions.J0p3 J0p3_4 
        annotation (extent=[100,20; 120,40], rotation=180);
      Utilities.CbxS Cbx   annotation (extent=[-80,70; -40,90]);
      Utilities.RS2 Rc(
        Tnom=Tnom,
        R=RC,
        TR1=TRC1,
        TR2=TRC2,
        Area=Area,
        Level=Level) annotation (extent=[50,58; 90,82]);
      Utilities.RS2 Re(
        Tnom=Tnom,
        Area=Area,
        Level=Level,
        R=RE,
        TR1=TRE1,
        TR2=TRE2) annotation (extent=[50,-82; 90,-58]);
      BondLib.Interfaces.BondCon S "Substrate" 
        annotation (extent=[-130,-70; -110,-50]);
      Utilities.CjS Dbs(
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        Level=Level,
        BE=false,
        EMin=EMin,
        EMax=EMax,
        MJ=MJS,
        VJ=VJS,
        XCJ=1,
        Tau=0,
        CJ=CJS,
        FC=0,
        IS=0,
        GminDC=0,
        N=1)       annotation (extent=[-60,-94; -20,-66]);
      Modelica.Blocks.Sources.Constant C0(k=0) 
        annotation (extent=[-66,-116; -54,-104]);
      Modelica.Blocks.Sources.Constant qb(k=1) 
        annotation (extent=[-46,-116; -34,-104]);
      equation 
      connect(B2.BondCon2, J0p3_1.BondCon2) annotation (points=[-30,0; -30,-10],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[-50,0; -49.8,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B3.BondCon2, J1p3_1.BondCon1) annotation (points=[-70,0; -70,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_1.BondCon3, B4.BondCon1) annotation (points=[-60,10; -60,
            10.2], style(color=8, rgbcolor={192,192,192}));
      connect(Rb.BondCon1, B4.BondCon2)  annotation (points=[-60,40; -60,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rb.BondCon2, B5.fBondCon1)  annotation (points=[-20,40; -20,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon1, B6.BondCon1) annotation (points=[-30,-30; -30,
            -40; -30.2,-40], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon2, B6.BondCon2) annotation (points=[-50,-40; -50,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1, J1p3_2.BondCon1) annotation (points=[-70.2,-40; -70,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_2.BondCon2, B7.BondCon2) annotation (points=[-100,-40; -90,
            -40],          style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon1, J1p3_2.BondCon3) annotation (points=[-60,-50.2; -60,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon1, B11.BondCon1) annotation (points=[20,60; 20,59.9;
            20,59.8; 20,59.8], style(color=8, rgbcolor={192,192,192}));
      connect(B12.BondCon2, J1p3_3.BondCon2) annotation (points=[20,80; 20,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon3, B13.BondCon1) annotation (points=[30,70; 30.2,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B15.BondCon2, J1p3_4.BondCon2) annotation (points=[20,-60; 20,-60],
          style(color=8, rgbcolor={192,192,192}));
      connect(B16.BondCon1, J1p3_4.BondCon1) annotation (points=[20,-80.2; 20,
            -80.1; 20,-80.1; 20,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_4.BondCon3, B17.BondCon1) annotation (points=[30,-70; 30.2,
            -70], style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_3.BondCon1, B12.BondCon1) annotation (points=[20,100; 20,
            99.9; 20,99.8; 20,99.8], style(color=8, rgbcolor={192,192,192}));
      connect(B16.BondCon2, J0p2_4.BondCon2) annotation (points=[20,-100; 20,
            -100], style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_4.BondCon1, E)        annotation (points=[20,-120; 0,-120],
          style(color=8, rgbcolor={192,192,192}));
      connect(B11.BondCon2, J0p3_2.BondCon2) annotation (points=[20,40; 20,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon1, B20.BondCon1) annotation (points=[20,20; 20,19.8],
                               style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_3.BondCon2, C)        annotation (points=[20,120; 0,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon2, B)        annotation (points=[-110,1.22461e-015;
            -116,1.22461e-015; -114,0; -120,0], style(color=8, rgbcolor={192,
              192,192}));
      connect(J0p3_3.BondCon1, B3.BondCon1) annotation (points=[-90,
            -1.22461e-015; -90,0; -89.8,0], style(color=8, rgbcolor={192,192,
              192}));
      connect(J0p3_3.BondCon3, B21.BondCon1) annotation (points=[-100,10; -100,
            10.2], style(color=8, rgbcolor={192,192,192}));
      connect(B21.BondCon2, J1p3_5.BondCon1) annotation (points=[-100,30; -100,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon1, J1p3_5.BondCon2) annotation (points=[-100,50.2;
            -100,50], style(color=8, rgbcolor={192,192,192}));
      connect(B23.BondCon1, J1p3_5.BondCon3) annotation (points=[-80,50.2; -80,
            40; -90,40], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon2, J0p3_2.BondCon3) annotation (points=[-100,70; -100,
            94; 40,94; 40,30; 30,30], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon2, B25.fBondCon1) annotation (points=[80,30; 80,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(B25.eBondCon1, J0p3_4.BondCon2) annotation (points=[100,30; 100,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon1, H)        annotation (points=[120,30; 120,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B14.eBondCon1, J0p3_4.BondCon3) annotation (points=[110,70; 110,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(B19.eBondCon1, J0p6_1.BondCon1) annotation (points=[60,-20; 60,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(B5.eBondCon1, J0p6_1.BondCon5) annotation (points=[0,40; 0,48; 66,
            48; 66,40], style(color=8, rgbcolor={192,192,192}));
      connect(B24.eBondCon1, J0p6_1.BondCon6) annotation (points=[-20,80; 0,80;
            0,52; 74,52; 74,40], style(color=8, rgbcolor={192,192,192}));
      connect(B18.eBondCon1, J0p6_1.BondCon3) annotation (points=[110,-70; 110,
            -40; 66,-40; 66,20], style(color=8, rgbcolor={192,192,192}));
      connect(B10.eBondCon1, J0p6_1.BondCon4) annotation (points=[0,-80; 8,-80;
            8,-90; 114,-90; 114,-36; 74,-36; 74,20], style(color=8, rgbcolor={
              192,192,192}));
      connect(Qni.qb, Rb.qb)      annotation (points=[41.4286,-33.7143; 52,
            -33.7143; 52,-48; -10,-48; -10,56; -40,56; -40,50], style(color=74,
            rgbcolor={0,0,127}));
      connect(B1.BondCon1, J0p3_1.BondCon3) annotation (points=[-19.8,-20; -20,
            -20], style(color=8, rgbcolor={192,192,192}));
      connect(Cbx.BondCon2, B24.fBondCon1)   annotation (points=[-40,80; -40,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(Cbx.BondCon1, B23.BondCon2)   annotation (points=[-80,80; -80,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Qni.ix, Cbx.ix)       annotation (points=[10,0; 10,20; -70,20;
            -70,60; -60,60; -60,70], style(color=74, rgbcolor={0,0,127}));
      connect(Rc.BondCon1, B13.BondCon2) annotation (points=[50,70; 50,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rc.BondCon2, B14.fBondCon1) annotation (points=[90,70; 90,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Re.BondCon1, B17.BondCon2) annotation (points=[50,-70; 50,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Re.BondCon2, B18.fBondCon1) annotation (points=[90,-70; 90,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_2.BondCon1, S)        annotation (points=[-100,-60; -120,-60],
          style(color=8, rgbcolor={192,192,192}));
      connect(Dbs.BondCon2, B10.fBondCon1) annotation (points=[-20,-80; -20,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon2, Dbs.BondCon1) annotation (points=[-60,-70; -60,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(C0.y, Dbs.C0) annotation (points=[-53.4,-110; -50,-110; -50,-94],
          style(color=74, rgbcolor={0,0,127}));
      connect(qb.y, Dbs.qb) annotation (points=[-33.4,-110; -30,-110; -30,-94],
          style(color=74, rgbcolor={0,0,127}));
      connect(B1.BondCon2, Qni.Bint) annotation (points=[0,-20; 0,-20], style(
            color=8, rgbcolor={192,192,192}));
      connect(B15.BondCon1, Qni.Eint) annotation (points=[20,-40.2; 20,-40.1;
            20,-40.1; 20,-40], style(color=8, rgbcolor={192,192,192}));
      connect(B19.fBondCon1, Qni.H) annotation (points=[40,-20; 40,-20], style(
            color=8, rgbcolor={192,192,192}));
      connect(B20.BondCon2, Qni.Cint) annotation (points=[20,0; 20,0], style(
            color=8, rgbcolor={192,192,192}));
      end NPNlat;
    
      model NPNvert "Spice-style vertically diffused NPN bipolar transistor" 
        constant Real inf=Modelica.Constants.inf;
      
        /* General parameters */
        parameter Integer Level(min=1,max=2) = 2 
        "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
        parameter Real Area=1 "Relative area occupied by device";
      
        /* DC model parameters */
        parameter Real BF=100 
        "|DC model parameters|Maximum forward current gain at reference temperature";
        parameter Real BR=1 
        "|DC model parameters|Maximum reverse current gain at reference temperature";
        parameter Modelica.SIunits.Current IS=1e-16 
        "|DC model parameters|Saturation current at reference temperature";
        parameter Modelica.SIunits.Current ISS=IS 
        "|DC model parameters|Saturation current used for current injection";
        parameter Real NF=1 
        "|DC model parameters|Forward current emission coefficient";
        parameter Real NR=1 
        "|DC model parameters|Reverse current emission coefficient";
      
        /* Low current beta degradation effect parameters */
        parameter Real C4(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
        parameter Modelica.SIunits.Current ISC(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
        parameter Real C2(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
        parameter Modelica.SIunits.Current ISE(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
        parameter Real NC=2 
        "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
        parameter Real NE=1.5 
        "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
      
        /* High current beta degradation effect parameters */
        parameter Modelica.SIunits.Current IKF=inf 
        "|High current beta degradation effect|Corner for forward beta high-current roll-off";
        parameter Modelica.SIunits.Current IKR=inf 
        "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
      
        /* Base width modulation parameters */
        parameter Modelica.SIunits.Voltage VAF=inf 
        "|Base width modulation parameters|Forward early voltage";
        parameter Modelica.SIunits.Voltage VAR=inf 
        "|Base width modulation parameters|Reverse early voltage";
      
        /* Parasitic Resistor Parameters */
        parameter Modelica.SIunits.Current IRB=inf 
        "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
        parameter Modelica.SIunits.Resistance RB(min=0) = 0 
        "|Parasitic resistor parameters|Zero-bias base resistance";
        parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
        "|Parasitic resistor parameters|Minimum base resistance at high currents";
        parameter Modelica.SIunits.Resistance RC(min=0) = 0 
        "|Parasitic resistor parameters|Collector resistance";
        parameter Modelica.SIunits.Resistance RE(min=0) = 0 
        "|Parasitic resistor parameters|Emitter resistance";
      
        /* Junction capacitor parameters */
        parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
        parameter Real MJC(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-collector junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJC=0.75 
        "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
        parameter Real MJE(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-emitter junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJE=0.75 
        "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
        parameter Real MJS(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Substrate junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJS=0.75 
        "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
        parameter Real XCJC(min=0,max=1) = 1 
        "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
        parameter Real FC(min=0,max=1) = 0.5 
        "|Junction capacitor parameters|Depletion capacitance factor for linearization";
      
        /* Transit time parameters */
        parameter Modelica.SIunits.Time TF(min=0) = 0 
        "|Transit time parameters|Ideal forward transit time";
        parameter Modelica.SIunits.Time TR(min=0) = 0 
        "|Transit time parameters|Ideal reverse transit time";
      
        /* Temperature compensation and area parameters */
        parameter Real TRB1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
        parameter Real TRB2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
        parameter Real TRM1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
        parameter Real TRM2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
        parameter Real TRC1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
        parameter Real TRC2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
        parameter Real TRE1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
        parameter Real TRE2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Real XTI=3 
        "|Temperature compensation parameters|Saturation current temperature exponent";
        parameter Real XTB=0 
        "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
        parameter Modelica.SIunits.Voltage EG=1.11 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
        "|Advanced|Numerical parameters|Leakage conductance";
      
    protected 
        parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
        parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
    public 
      BondLib.Interfaces.BondCon B "Base" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon E "Emitter" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon H "Heat" annotation (extent=[110,-10; 130,10]);
      BondLib.Spice.Utilities.NPNint Qni(
        BF=BF,
        BR=BR,
        IS=IS,
        NF=NF,
        NR=NR,
        GminDC=GminDC,
        NC=NC,
        NE=NE,
        VAF=VAF,
        VAR=VAR,
        IKF=IKF,
        IKR=IKR,
        CJC=CJC,
        MJC=MJC,
        VJC=VJC,
        CJE=CJE,
        MJE=MJE,
        VJE=VJE,
        XCJC=XCJC,
        TF=TF,
        TR=TR,
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        EMin=EMin,
        EMax=EMax,
        FC=FC,
        Level=Level,
        ISC=ISCeff,
        ISE=ISEeff,
        ISS=ISS) annotation (extent=[0,-40; 40,0]);
      BondLib.Bonds.Bond B2 annotation (extent=[-50,-10; -30,10], rotation=0);
      BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-70,-10; -50,10]);
      BondLib.Bonds.Bond B3 annotation (extent=[-90,-10; -70,10], rotation=0);
      BondLib.Bonds.Bond B4 annotation (extent=[-70,-30; -50,-10], rotation=270);
      BondLib.Spice.Utilities.RBS Rb(
        Tnom=Tnom,
        RB=RB,
        RBM=RBM,
        IRB=IRB,
        TRB1=TRB1,
        TRB2=TRB2,
        TRM1=TRM1,
        TRM2=TRM2,
        Area=Area,
        Level=Level) 
                   annotation (extent=[-60,-50; -20,-30], rotation=0);
      BondLib.Bonds.eBond B5 annotation (extent=[-20,-50; 0,-30]);
      BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[50,40; 70,20]);
      BondLib.Bonds.Bond B7 annotation (extent=[30,20; 50,40], rotation=180);
      BondLib.Bonds.Bond B9 annotation (extent=[50,40; 70,60], rotation=90);
      BondLib.Bonds.eBond B10 annotation (extent=[110,80; 130,100], rotation=90);
      BondLib.Bonds.Bond B11 annotation (extent=[10,40; 30,60], rotation=270);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[10,80; 30,60], rotation=-90);
      BondLib.Bonds.Bond B12 annotation (extent=[10,80; 30,100], rotation=270);
      BondLib.Junctions.J0p2 J0p2_3 
        annotation (extent=[10,100; 30,120], rotation=90);
      BondLib.Bonds.Bond B13 annotation (extent=[-10,60; 10,80], rotation=180);
      BondLib.Bonds.eBond B14 annotation (extent=[-70,60; -50,80], rotation=180);
      BondLib.Bonds.Bond B15 annotation (extent=[10,-60; 30,-40], rotation=270);
      BondLib.Junctions.J1p3 J1p3_4 
        annotation (extent=[10,-60; 30,-80], rotation=-90);
      BondLib.Bonds.Bond B16 annotation (extent=[10,-100; 30,-80], rotation=270);
      BondLib.Junctions.J0p2 J0p2_4 
        annotation (extent=[10,-120; 30,-100], rotation=90);
      BondLib.Bonds.Bond B17 annotation (extent=[-10,-80; 10,-60], rotation=180);
      BondLib.Bonds.eBond B18 
        annotation (extent=[-70,-80; -50,-60], rotation=180);
      BondLib.Bonds.eBond B19 annotation (extent=[40,-30; 60,-10]);
      BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[60,-100; 80,-80]);
      BondLib.Bonds.Bond B20 annotation (extent=[10,0; 30,20], rotation=270);
      BondLib.Junctions.J0p3 J0p3_3 
        annotation (extent=[-110,-10; -90,10], rotation=180);
      BondLib.Bonds.Bond B21 annotation (extent=[-110,10; -90,30], rotation=90);
      BondLib.Junctions.J1p3 J1p3_5 
        annotation (extent=[-110,30; -90,50], rotation=90);
      BondLib.Bonds.Bond B22 annotation (extent=[-110,50; -90,70], rotation=90);
      BondLib.Bonds.Bond B23 annotation (extent=[-90,30; -70,50], rotation=0);
      BondLib.Bonds.eBond B24 annotation (extent=[-30,30; -10,50]);
      BondLib.Bonds.eBond B25 annotation (extent=[80,-100; 100,-80]);
      BondLib.Junctions.J0p3 J0p3_4 
        annotation (extent=[100,-100; 120,-80], rotation=180);
      BondLib.Interfaces.BondCon C "Collector" 
                                          annotation (extent=[-10,110; 10,130]);
      annotation (
        Diagram(
          Text(
            extent=[-126,22; -114,8],
            string="B",
            style(color=10, rgbcolor={95,95,95})),
          Text(
            extent=[-20,128; -8,114],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[-20,-114; -8,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E"),
          Text(
            extent=[92,-6; 118,-22],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[82,-96; 108,-112],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[38,-34; 48,-46],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[-38,-14; -28,-26],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[0,10; 10,-2],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[-50,30; -40,18],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[-126,-36; -114,-50],
            style(color=10, rgbcolor={95,95,95}),
            string="S")),
        Coordsys(extent=[-120,-120; 120,120]),
        Documentation(info="<html>
The <b>NPNvert</b> element of the Spice bond graph library implements a full-fledges Spice-stye Gummel-Poon model of the vertically diffused NPN bipolar transistor [1-3]. The model contains a <b>NPNint</b> model implementing the inner parts of the NPN transistor up to the internal nodes. <p>

The NPN bipolar transistor is a directed FourPort.  The direction of positive power flow is assumed into the model at the base, <b>B</b>, and at the collector, <b>C</b>, whereas it is assumed out of the model at the emitter, <b>E</b>, and at the <b>Heat</b> port. <p>

The causality of the NPN model is free. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"),     Icon(
            Text(
              extent=[-102,84; 62,-92],
              style(color=0),
              string="NPN"),
            Line(points=[0,100; 0,-100],
                                       style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[-20,-80; 0,-100],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,-100; 20,-80],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Text(
            extent=[-88,-72; -44,-108],
            string="%Area x",
            style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-80; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-40; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[-100,-60; 100,-60], style(color=3, rgbcolor={0,0,255})),
            Text(
              extent=[58,4; 110,-42],
              style(color=0),
            string="vert"),
          Text(
            extent=[-80,100; 80,60],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
          Text(
            extent=[-126,24; -114,10],
            string="B",
            style(color=10, rgbcolor={95,95,95})),
          Text(
            extent=[10,126; 22,112],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[106,26; 132,6],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[8,-114; 20,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E")));
      BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[10,20; 30,40]);
      Utilities.CbxS Cbx   annotation (extent=[-70,30; -30,50]);
      Utilities.RS2 Rc(
        Tnom=Tnom,
        R=RC,
        TR1=TRC1,
        TR2=TRC2,
        Area=Area,
        Level=Level) annotation (extent=[-10,58; -50,82]);
      Utilities.RS2 Re(
        Tnom=Tnom,
        Area=Area,
        Level=Level,
        R=RE,
        TR1=TRE1,
        TR2=TRE2) annotation (extent=[-10,-82; -50,-58]);
      Junctions.J0p2 J0p2_2 
        annotation (extent=[90,20; 110,40],     rotation=180);
      BondLib.Bonds.Bond B8 annotation (extent=[70,20; 90,40], rotation=180);
      BondLib.Interfaces.BondCon S "Substrate" 
        annotation (extent=[-130,-70; -110,-50]);
      Utilities.CjS Dsc(
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        Level=Level,
        BE=false,
        EMin=EMin,
        EMax=EMax,
        MJ=MJS,
        VJ=VJS,
        XCJ=1,
        Tau=0,
        FC=0,
        IS=0,
        GminDC=0,
        CJ=CJS,
        N=1)       annotation (extent=[70,66; 110,94]);
      Modelica.Blocks.Sources.Constant C0(k=0) 
        annotation (extent=[96,44; 84,56]);
      Modelica.Blocks.Sources.Constant qb(k=1) 
        annotation (extent=[116,44; 104,56]);
      equation 
      connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[-50,0; -49.8,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B3.BondCon2, J1p3_1.BondCon1) annotation (points=[-70,0; -70,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rb.BondCon2, B5.fBondCon1)  annotation (points=[-20,-40; -20,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1, J1p3_2.BondCon1) annotation (points=[49.8,30; 50,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon1, B11.BondCon1) annotation (points=[20,60; 20,59.9;
            20,59.8; 20,59.8], style(color=8, rgbcolor={192,192,192}));
      connect(B12.BondCon2, J1p3_3.BondCon2) annotation (points=[20,80; 20,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B15.BondCon2, J1p3_4.BondCon2) annotation (points=[20,-60; 20,-60],
          style(color=8, rgbcolor={192,192,192}));
      connect(B16.BondCon1, J1p3_4.BondCon1) annotation (points=[20,-80.2; 20,
            -80.1; 20,-80.1; 20,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_3.BondCon1, B12.BondCon1) annotation (points=[20,100; 20,
            99.9; 20,99.8; 20,99.8], style(color=8, rgbcolor={192,192,192}));
      connect(B16.BondCon2, J0p2_4.BondCon2) annotation (points=[20,-100; 20,
            -100], style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_4.BondCon1, E)        annotation (points=[20,-120; 0,-120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_3.BondCon2, C)        annotation (points=[20,120; 0,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon2, B)        annotation (points=[-110,1.22461e-015;
            -115,1.22461e-015; -115,0; -120,0], style(color=8, rgbcolor={192,
              192,192}));
      connect(J0p3_3.BondCon1, B3.BondCon1) annotation (points=[-90,
            -1.22461e-015; -89.9,-1.22461e-015; -89.9,0; -89.8,0], style(color=
              8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon3, B21.BondCon1) annotation (points=[-100,10; -100,
            10.2], style(color=8, rgbcolor={192,192,192}));
      connect(B21.BondCon2, J1p3_5.BondCon1) annotation (points=[-100,30; -100,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon1, J1p3_5.BondCon2) annotation (points=[-100,50.2;
            -100,50], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon2, B25.fBondCon1) annotation (points=[80,-90; 80,
            -90], style(color=8, rgbcolor={192,192,192}));
      connect(B25.eBondCon1, J0p3_4.BondCon2) annotation (points=[100,-90; 100,
            -90], style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon1, J1p3_1.BondCon3) annotation (points=[-60,-10.2; -60,
            -10], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_5.BondCon3, B23.BondCon1) annotation (points=[-90,40; -89.8,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon4, B11.BondCon2) annotation (points=[20,40; 20,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon3, B20.BondCon1) annotation (points=[20,20; 25,20;
            25,19.8; 20,19.8], style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon2, Rb.BondCon1)  annotation (points=[-60,-30; -60,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Qni.qb, Rb.qb)      annotation (points=[41.4286,-33.7143; 50,
            -33.7143; 50,-48; -10,-48; -10,-24; -40,-24; -40,-30], style(color=
              74, rgbcolor={0,0,127}));
      connect(B13.BondCon1, J1p3_3.BondCon3) annotation (points=[9.8,70; 10,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon2, J0p4_1.BondCon1) annotation (points=[-100,70; -100,
            88; 0,88; 0,30; 10,30], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon2, B7.BondCon2) annotation (points=[30,30; 30,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon1, J1p3_2.BondCon3) annotation (points=[60,40.2; 60,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B17.BondCon1, J1p3_4.BondCon3) annotation (points=[9.8,-70; 10,
            -70], style(color=8, rgbcolor={192,192,192}));
      connect(B10.eBondCon1, J0p3_4.BondCon1) annotation (points=[120,100; 120,
            108; 130,108; 130,-90; 120,-90], style(color=8, rgbcolor={192,192,
              192}));
      connect(B5.eBondCon1, J0p6_1.BondCon1) annotation (points=[0,-40; 8,-40;
            8,-52; 60,-52; 60,-90], style(color=8, rgbcolor={192,192,192}));
      connect(B19.eBondCon1, J0p6_1.BondCon5) annotation (points=[60,-20; 66,
            -20; 66,-80], style(color=8, rgbcolor={192,192,192}));
      connect(B24.eBondCon1, J0p6_1.BondCon6) annotation (points=[-10,40; -6,40;
            -6,12; 74,12; 74,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon3, H)        annotation (points=[110,-80; 120,-80;
            120,0], style(color=8, rgbcolor={192,192,192}));
      connect(B18.eBondCon1, J0p6_1.BondCon3) annotation (points=[-70,-70; -76,
            -70; -76,-88; 48,-88; 48,-100; 66,-100], style(color=8, rgbcolor={
              192,192,192}));
      connect(B14.eBondCon1, J0p6_1.BondCon4) annotation (points=[-70,70; -80,
            70; -80,-92; 44,-92; 44,-104; 74,-104; 74,-100], style(color=8,
            rgbcolor={192,192,192}));
      connect(Cbx.BondCon1, B23.BondCon2)   annotation (points=[-70,40; -70,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Cbx.BondCon2, B24.fBondCon1)   annotation (points=[-30,40; -30,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Qni.ix, Cbx.ix)       annotation (points=[10,0; 10,10; -50,10;
            -50,30], style(color=74, rgbcolor={0,0,127}));
      connect(Rc.BondCon1, B13.BondCon2) annotation (points=[-10,70; -10,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rc.BondCon2, B14.fBondCon1) annotation (points=[-50,70; -50,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Re.BondCon1, B17.BondCon2) annotation (points=[-10,-70; -10,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Re.BondCon2, B18.fBondCon1) annotation (points=[-50,-70; -50,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B8.BondCon2, J1p3_2.BondCon2) annotation (points=[70,30; 70,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_2.BondCon2, B8.BondCon1) annotation (points=[90,30; 89.8,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_2.BondCon1, S)        annotation (points=[110,30; 134,30;
            134,-130; -120,-130; -120,-60], style(color=8, rgbcolor={192,192,
              192}));
      connect(C0.y, Dsc.C0) annotation (points=[83.4,50; 80,50; 80,66], style(
            color=74, rgbcolor={0,0,127}));
      connect(qb.y, Dsc.qb) annotation (points=[103.4,50; 100,50; 100,66],
          style(color=74, rgbcolor={0,0,127}));
      connect(Dsc.BondCon2, B10.fBondCon1) annotation (points=[110,80; 120,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(Dsc.BondCon1, B9.BondCon2) annotation (points=[70,80; 60,80; 60,
            60], style(color=8, rgbcolor={192,192,192}));
      connect(Qni.Bint, B2.BondCon2) annotation (points=[0,-20; -20,-20; -20,0;
            -30,0], style(color=8, rgbcolor={192,192,192}));
      connect(B15.BondCon1, Qni.Eint) annotation (points=[20,-40.2; 20,-40.1;
            20,-40.1; 20,-40], style(color=8, rgbcolor={192,192,192}));
      connect(B19.fBondCon1, Qni.H) annotation (points=[40,-20; 40,-20], style(
            color=8, rgbcolor={192,192,192}));
      connect(B20.BondCon2, Qni.Cint) annotation (points=[20,0; 20,0], style(
            color=8, rgbcolor={192,192,192}));
      end NPNvert;
    
      model PNPlat "Spice-style laterally diffused PNP bipolar transistor" 
        constant Real inf=Modelica.Constants.inf;
      
        /* General parameters */
        parameter Integer Level(min=1,max=2) = 2 
        "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
        parameter Real Area=1 "Relative area occupied by device";
      
        /* DC model parameters */
        parameter Real BF=100 
        "|DC model parameters|Maximum forward current gain at reference temperature";
        parameter Real BR=1 
        "|DC model parameters|Maximum reverse current gain at reference temperature";
        parameter Modelica.SIunits.Current IS=1e-16 
        "|DC model parameters|Saturation current at reference temperature";
        parameter Modelica.SIunits.Current ISS=IS 
        "|DC model parameters|Saturation current used for current injection";
        parameter Real NF=1 
        "|DC model parameters|Forward current emission coefficient";
        parameter Real NR=1 
        "|DC model parameters|Reverse current emission coefficient";
      
        /* Low current beta degradation effect parameters */
        parameter Real C4(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
        parameter Modelica.SIunits.Current ISC(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
        parameter Real C2(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
        parameter Modelica.SIunits.Current ISE(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
        parameter Real NC=2 
        "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
        parameter Real NE=1.5 
        "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
      
        /* High current beta degradation effect parameters */
        parameter Modelica.SIunits.Current IKF=inf 
        "|High current beta degradation effect|Corner for forward beta high-current roll-off";
        parameter Modelica.SIunits.Current IKR=inf 
        "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
      
        /* Base width modulation parameters */
        parameter Modelica.SIunits.Voltage VAF=inf 
        "|Base width modulation parameters|Forward early voltage";
        parameter Modelica.SIunits.Voltage VAR=inf 
        "|Base width modulation parameters|Reverse early voltage";
      
        /* Parasitic Resistor Parameters */
        parameter Modelica.SIunits.Current IRB=inf 
        "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
        parameter Modelica.SIunits.Resistance RB(min=0) = 0 
        "|Parasitic resistor parameters|Zero-bias base resistance";
        parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
        "|Parasitic resistor parameters|Minimum base resistance at high currents";
        parameter Modelica.SIunits.Resistance RC(min=0) = 0 
        "|Parasitic resistor parameters|Collector resistance";
        parameter Modelica.SIunits.Resistance RE(min=0) = 0 
        "|Parasitic resistor parameters|Emitter resistance";
      
        /* Junction capacitor parameters */
        parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
        parameter Real MJC(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-collector junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJC=0.75 
        "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
        parameter Real MJE(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-emitter junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJE=0.75 
        "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
        parameter Real MJS(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Substrate junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJS=0.75 
        "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
        parameter Real XCJC(min=0,max=1) = 1 
        "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
        parameter Real FC(min=0,max=1) = 0.5 
        "|Junction capacitor parameters|Depletion capacitance factor for linearization";
      
        /* Transit time parameters */
        parameter Modelica.SIunits.Time TF(min=0) = 0 
        "|Transit time parameters|Ideal forward transit time";
        parameter Modelica.SIunits.Time TR(min=0) = 0 
        "|Transit time parameters|Ideal reverse transit time";
      
        /* Temperature compensation and area parameters */
        parameter Real TRB1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
        parameter Real TRB2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
        parameter Real TRM1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
        parameter Real TRM2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
        parameter Real TRC1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
        parameter Real TRC2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
        parameter Real TRE1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
        parameter Real TRE2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Real XTI=3 
        "|Temperature compensation parameters|Saturation current temperature exponent";
        parameter Real XTB=0 
        "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
        parameter Modelica.SIunits.Voltage EG=1.11 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
        "|Advanced|Numerical parameters|Leakage conductance";
      
    protected 
        parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
        parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
      annotation (
        Coordsys(extent=[-120,-120; 120,120]),
        Diagram(
          Text(
            extent=[-126,22; -114,8],
            string="B",
            style(color=10, rgbcolor={95,95,95})),
          Text(
            extent=[-20,128; -8,114],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[-20,-114; -8,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E"),
          Text(
            extent=[108,-6; 134,-22],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[78,50; 104,34],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[42,-34; 52,-46],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[-28,66; -18,54],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[-60,70; -50,58],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[0,16; 10,4],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[-126,-38; -114,-52],
            style(color=10, rgbcolor={95,95,95}),
            string="S")),
        Documentation(info="<html>
The <b>PNPlat</b> element of the Spice bond graph library implements a full-fledges Spice-stye Gummel-Poon model of the laterally diffused PNP bipolar transistor [1-3]. The model contains a <b>PNPint</b> model implementing the inner parts of the PNP transistor up to the internal nodes. <p>

The PNP bipolar transistor is a directed FourPort.  The direction of positive power flow is assumed into the model at the emitter, <b>E</b>, whereas it is assumed out of the model at the base, <b>B</b>, at the collector, <b>C</b>, and at the <b>Heat</b> port. <p>

The causality of the PNP model is free. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"),     Icon(
            Text(
              extent=[-96,84; 68,-92],
              style(color=0),
            string="PNP"),
            Text(
              extent=[64,4; 104,-38],
              style(color=0),
            string="lat"),
          Text(
            extent=[-80,100; 80,60],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
          Text(
            extent=[-126,24; -114,10],
            string="B",
            style(color=10, rgbcolor={95,95,95})),
          Text(
            extent=[10,126; 22,112],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[106,26; 132,6],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[-88,-72; -44,-108],
            string="%Area x",
            style(color=3, rgbcolor={0,0,255})),
            Line(points=[0,100; 0,-100],
                                       style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,100; 20,80],   style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[-20,80; 0,100], style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Line(points=[-100,-60; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-80; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-40; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Text(
            extent=[8,-114; 20,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E")));
    public 
      BondLib.Interfaces.BondCon B "Base" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon E "Emitter" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon H "Heat" annotation (extent=[110,-10; 130,10]);
      BondLib.Interfaces.BondCon C "Collector" 
                                          annotation (extent=[-10,110; 10,130]);
      BondLib.Bonds.Bond B1 annotation (extent=[-20,-30; 0,-10], rotation=180);
      BondLib.Junctions.J0p3 J0p3_1 
        annotation (extent=[-40,-30; -20,-10], rotation=90);
      BondLib.Bonds.Bond B2 annotation (extent=[-50,-10; -30,10], rotation=180);
      BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-70,10; -50,-10]);
      BondLib.Bonds.Bond B3 annotation (extent=[-90,-10; -70,10], rotation=180);
      BondLib.Bonds.Bond B4 annotation (extent=[-70,10; -50,30], rotation=90);
      BondLib.Spice.Utilities.RBS Rb(
        Tnom=Tnom,
        RB=RB,
        RBM=RBM,
        IRB=IRB,
        TRB1=TRB1,
        TRB2=TRB2,
        TRM1=TRM1,
        TRM2=TRM2,
        Area=Area,
        Level=Level) 
                   annotation (extent=[-60,30; -20,50], rotation=0);
      BondLib.Bonds.eBond B5 annotation (extent=[-20,30; 0,50]);
      BondLib.Bonds.Bond B6 annotation (extent=[-50,-50; -30,-30], rotation=0);
      BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[-70,-50; -50,-30]);
      BondLib.Bonds.Bond B9 annotation (extent=[-70,-70; -50,-50], rotation=270);
      BondLib.Bonds.eBond B10 annotation (extent=[-20,-90; 0,-70]);
      BondLib.Bonds.Bond B11 annotation (extent=[10,40; 30,60], rotation=90);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[30,80; 10,60], rotation=-90);
      BondLib.Bonds.Bond B12 annotation (extent=[10,80; 30,100], rotation=90);
      BondLib.Junctions.J0p2 J0p2_3 
        annotation (extent=[10,100; 30,120], rotation=90);
      BondLib.Bonds.Bond B13 annotation (extent=[30,60; 50,80], rotation=0);
      BondLib.Bonds.eBond B14 annotation (extent=[90,60; 110,80], rotation=0);
      BondLib.Bonds.Bond B15 annotation (extent=[10,-60; 30,-40], rotation=90);
      BondLib.Junctions.J1p3 J1p3_4 
        annotation (extent=[30,-60; 10,-80], rotation=-90);
      BondLib.Bonds.Bond B16 annotation (extent=[10,-100; 30,-80], rotation=90);
      BondLib.Junctions.J0p2 J0p2_4 
        annotation (extent=[10,-120; 30,-100], rotation=90);
      BondLib.Bonds.Bond B17 annotation (extent=[30,-80; 50,-60], rotation=0);
      BondLib.Bonds.eBond B18 annotation (extent=[90,-80; 110,-60], rotation=0);
      BondLib.Bonds.eBond B19 annotation (extent=[40,-30; 60,-10]);
      BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[60,20; 80,40]);
      BondLib.Bonds.Bond B20 annotation (extent=[10,0; 30,20], rotation=90);
      BondLib.Junctions.J0p3 J0p3_2 
        annotation (extent=[10,20; 30,40], rotation=90);
      BondLib.Junctions.J0p3 J0p3_3 
        annotation (extent=[-110,-10; -90,10], rotation=180);
      BondLib.Bonds.Bond B21 annotation (extent=[-110,10; -90,30], rotation=90);
      BondLib.Junctions.J1p3 J1p3_5 
        annotation (extent=[-110,30; -90,50], rotation=90);
      BondLib.Bonds.Bond B22 annotation (extent=[-110,50; -90,70], rotation=90);
      BondLib.Bonds.Bond B23 annotation (extent=[-90,50; -70,70], rotation=90);
      BondLib.Bonds.eBond B24 annotation (extent=[-40,70; -20,90]);
      BondLib.Bonds.eBond B25 annotation (extent=[80,20; 100,40]);
      BondLib.Junctions.J0p3 J0p3_4 
        annotation (extent=[100,20; 120,40], rotation=180);
      BondLib.Spice.Utilities.PNPint Qpi(
        BF=BF,
        BR=BR,
        IS=IS,
        NF=NF,
        NR=NR,
        GminDC=GminDC,
        NC=NC,
        NE=NE,
        VAF=VAF,
        VAR=VAR,
        IKF=IKF,
        IKR=IKR,
        CJC=CJC,
        MJC=MJC,
        VJC=VJC,
        CJE=CJE,
        MJE=MJE,
        VJE=VJE,
        XCJC=XCJC,
        TF=TF,
        TR=TR,
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        EMin=EMin,
        EMax=EMax,
        FC=FC,
        Level=Level,
        ISC=ISCeff,
        ISE=ISEeff,
        ISS=ISS) annotation (extent=[0,-40; 40,0]);
      Utilities.CbxS Cbx   annotation (extent=[-80,70; -40,90]);
      Utilities.RS2 Rc(
        Tnom=Tnom,
        R=RC,
        TR1=TRC1,
        TR2=TRC2,
        Area=Area,
        Level=Level) annotation (extent=[50,58; 90,82]);
      Utilities.RS2 Re(
        Tnom=Tnom,
        Area=Area,
        Level=Level,
        R=RE,
        TR1=TRE1,
        TR2=TRE2) annotation (extent=[50,-82; 90,-58]);
      BondLib.Bonds.Bond B7 annotation (extent=[-90,-50; -70,-30], rotation=0);
      Junctions.J0p2 J0p2_2 
        annotation (extent=[-110,-60; -90,-40], rotation=270);
      BondLib.Interfaces.BondCon S "Substrate" 
        annotation (extent=[-130,-70; -110,-50]);
      Utilities.CjS Dsb(
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        Level=Level,
        BE=false,
        EMin=EMin,
        EMax=EMax,
        MJ=MJS,
        VJ=VJS,
        XCJ=1,
        Tau=0,
        IS=0,
        GminDC=0,
        CJ=CJS,
        FC=0,
        N=1)       annotation (extent=[-60,-94; -20,-66]);
      Modelica.Blocks.Sources.Constant C0(k=0) 
        annotation (extent=[-66,-116; -54,-104]);
      Modelica.Blocks.Sources.Constant qb(k=1) 
        annotation (extent=[-46,-116; -34,-104]);
      equation 
      connect(J1p3_1.BondCon3, B4.BondCon1) annotation (points=[-60,10; -60,
            10.2], style(color=8, rgbcolor={192,192,192}));
      connect(Rb.BondCon1, B4.BondCon2)  annotation (points=[-60,40; -60,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rb.BondCon2, B5.fBondCon1)  annotation (points=[-20,40; -20,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon1, J1p3_2.BondCon3) annotation (points=[-60,-50.2; -60,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon3, B13.BondCon1) annotation (points=[30,70; 30.2,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_4.BondCon1, E)        annotation (points=[20,-120; 0,-120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_3.BondCon2, C)        annotation (points=[20,120; 0,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon3, B21.BondCon1) annotation (points=[-100,10; -100,
            10.2], style(color=8, rgbcolor={192,192,192}));
      connect(B21.BondCon2, J1p3_5.BondCon1) annotation (points=[-100,30; -100,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon1, J1p3_5.BondCon2) annotation (points=[-100,50.2;
            -100,50], style(color=8, rgbcolor={192,192,192}));
      connect(B23.BondCon1, J1p3_5.BondCon3) annotation (points=[-80,50.2; -80,
            40; -90,40], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon2, J0p3_2.BondCon3) annotation (points=[-100,70; -100,
            94; 40,94; 40,30; 30,30], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon1, H)        annotation (points=[120,30; 120,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B14.eBondCon1, J0p3_4.BondCon3) annotation (points=[110,70; 110,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(B19.eBondCon1, J0p6_1.BondCon1) annotation (points=[60,-20; 60,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(B5.eBondCon1, J0p6_1.BondCon5) annotation (points=[0,40; 0,48; 66,
            48; 66,40], style(color=8, rgbcolor={192,192,192}));
      connect(B24.eBondCon1, J0p6_1.BondCon6) annotation (points=[-20,80; 0,80;
            0,52; 74,52; 74,40], style(color=8, rgbcolor={192,192,192}));
      connect(B18.eBondCon1, J0p6_1.BondCon3) annotation (points=[110,-70; 110,
            -40; 66,-40; 66,20], style(color=8, rgbcolor={192,192,192}));
      connect(B10.eBondCon1, J0p6_1.BondCon4) annotation (points=[0,-80; 8,-80;
            8,-90; 114,-90; 114,-36; 74,-36; 74,20], style(color=8, rgbcolor={
              192,192,192}));
      connect(B12.BondCon2, J0p2_3.BondCon1) annotation (points=[20,100; 20,100],
          style(color=8, rgbcolor={192,192,192}));
      connect(B12.BondCon1, J1p3_3.BondCon2) annotation (points=[20,80.2; 20,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon1, B11.BondCon2) annotation (points=[20,60; 20,60],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_2.BondCon2, B11.BondCon1) annotation (points=[20,40; 20,40.2],
          style(color=8, rgbcolor={192,192,192}));
      connect(B20.BondCon2, J0p3_2.BondCon1) annotation (points=[20,20; 20,20],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_4.BondCon2, B15.BondCon1) annotation (points=[20,-60; 20,
            -59.8], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_4.BondCon1, B16.BondCon2) annotation (points=[20,-80; 20,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B16.BondCon1, J0p2_4.BondCon2) annotation (points=[20,-99.8; 20,
            -100], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon3, B1.BondCon2) annotation (points=[-20,-20; -20,
            -20], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2, B2.BondCon1) annotation (points=[-30,-10; -30,
            -1.20011e-015; -30.2,-1.20011e-015],     style(color=8, rgbcolor={
              192,192,192}));
      connect(B3.BondCon2, J0p3_3.BondCon1) annotation (points=[-90,
            1.22461e-015; -90,-1.22461e-015], style(color=8, rgbcolor={192,192,
              192}));
      connect(B6.BondCon1, J1p3_2.BondCon2) annotation (points=[-49.8,-40; -50,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon1, B6.BondCon2) annotation (points=[-30,-30; -30,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(Cbx.BondCon2, B24.fBondCon1)   annotation (points=[-40,80; -40,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(Cbx.BondCon1, B23.BondCon2)   annotation (points=[-80,80; -80,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Qpi.qb, Rb.qb)      annotation (points=[41.4286,-33.7143; 52,
            -33.7143; 52,-48; -10,-48; -10,56; -40,56; -40,50], style(color=74,
            rgbcolor={0,0,127}));
      connect(Qpi.ix, Cbx.ix)       annotation (points=[10,0; 10,20; -70,20;
            -70,60; -60,60; -60,70], style(color=74, rgbcolor={0,0,127}));
      connect(Rc.BondCon1, B13.BondCon2) annotation (points=[50,70; 50,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rc.BondCon2, B14.fBondCon1) annotation (points=[90,70; 90,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Re.BondCon2, B18.fBondCon1) annotation (points=[90,-70; 90,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon2, J1p3_2.BondCon1) annotation (points=[-70,-40; -70,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1, J0p2_2.BondCon1) annotation (points=[-89.8,-40; -100,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_2.BondCon2, S)        annotation (points=[-100,-60; -120,-60],
          style(color=8, rgbcolor={192,192,192}));
      connect(C0.y, Dsb.C0) annotation (points=[-53.4,-110; -50,-110; -50,-94],
          style(color=74, rgbcolor={0,0,127}));
      connect(qb.y, Dsb.qb) annotation (points=[-33.4,-110; -30,-110; -30,-94],
          style(color=74, rgbcolor={0,0,127}));
      connect(Dsb.BondCon2, B10.fBondCon1) annotation (points=[-20,-80; -20,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon2, Dsb.BondCon1) annotation (points=[-60,-70; -60,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B2.BondCon2, J1p3_1.BondCon2) annotation (points=[-50,
            1.22461e-015; -50,0], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_1.BondCon1, B3.BondCon1) annotation (points=[-70,0; -70,
            -1.20011e-015; -70.2,-1.20011e-015], style(color=8, rgbcolor={192,
              192,192}));
      connect(B17.BondCon2, Re.BondCon1) annotation (points=[50,-70; 50,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B17.BondCon1, J1p3_4.BondCon3) annotation (points=[30.2,-70; 30,
            -70], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon2, B25.fBondCon1) annotation (points=[80,30; 80,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon2, B25.eBondCon1) annotation (points=[100,30; 100,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon2, B)        annotation (points=[-110,1.22461e-015;
            -116,1.22461e-015; -116,0; -120,0], style(color=8, rgbcolor={192,
              192,192}));
      connect(Qpi.Bint, B1.BondCon1) annotation (points=[0,-20; -0.2,-20],
          style(color=8, rgbcolor={192,192,192}));
      connect(Qpi.Eint, B15.BondCon2) annotation (points=[20,-40; 20,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B19.fBondCon1, Qpi.H) annotation (points=[40,-20; 40,-20], style(
            color=8, rgbcolor={192,192,192}));
      connect(B20.BondCon1, Qpi.Cint) annotation (points=[20,0.2; 20,0], style(
            color=8, rgbcolor={192,192,192}));
      end PNPlat;
    
      model PNPvert "Spice-style vertically diffused PNP bipolar transistor" 
        constant Real inf=Modelica.Constants.inf;
      
        /* General parameters */
        parameter Integer Level(min=1,max=2) = 2 
        "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
        parameter Real Area=1 "Relative area occupied by device";
      
        /* DC model parameters */
        parameter Real BF=100 
        "|DC model parameters|Maximum forward current gain at reference temperature";
        parameter Real BR=1 
        "|DC model parameters|Maximum reverse current gain at reference temperature";
        parameter Modelica.SIunits.Current IS=1e-16 
        "|DC model parameters|Saturation current at reference temperature";
        parameter Modelica.SIunits.Current ISS=IS 
        "|DC model parameters|Saturation current used for current injection";
        parameter Real NF=1 
        "|DC model parameters|Forward current emission coefficient";
        parameter Real NR=1 
        "|DC model parameters|Reverse current emission coefficient";
      
        /* Low current beta degradation effect parameters */
        parameter Real C4(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
        parameter Modelica.SIunits.Current ISC(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
        parameter Real C2(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
        parameter Modelica.SIunits.Current ISE(min=0) = 0 
        "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
        parameter Real NC=2 
        "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
        parameter Real NE=1.5 
        "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
      
        /* High current beta degradation effect parameters */
        parameter Modelica.SIunits.Current IKF=inf 
        "|High current beta degradation effect|Corner for forward beta high-current roll-off";
        parameter Modelica.SIunits.Current IKR=inf 
        "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
      
        /* Base width modulation parameters */
        parameter Modelica.SIunits.Voltage VAF=inf 
        "|Base width modulation parameters|Forward early voltage";
        parameter Modelica.SIunits.Voltage VAR=inf 
        "|Base width modulation parameters|Reverse early voltage";
      
        /* Parasitic Resistor Parameters */
        parameter Modelica.SIunits.Current IRB=inf 
        "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
        parameter Modelica.SIunits.Resistance RB(min=0) = 0 
        "|Parasitic resistor parameters|Zero-bias base resistance";
        parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
        "|Parasitic resistor parameters|Minimum base resistance at high currents";
        parameter Modelica.SIunits.Resistance RC(min=0) = 0 
        "|Parasitic resistor parameters|Collector resistance";
        parameter Modelica.SIunits.Resistance RE(min=0) = 0 
        "|Parasitic resistor parameters|Emitter resistance";
      
        /* Junction capacitor parameters */
        parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
        parameter Real MJC(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-collector junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJC=0.75 
        "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
        parameter Real MJE(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Base-emitter junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJE=0.75 
        "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
        parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
        "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
        parameter Real MJS(min=0.3,max=0.5) = 0.33 
        "|Junction capacitor parameters|Substrate junction grading coefficient";
        parameter Modelica.SIunits.Voltage VJS=0.75 
        "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
        parameter Real XCJC(min=0,max=1) = 1 
        "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
        parameter Real FC(min=0,max=1) = 0.5 
        "|Junction capacitor parameters|Depletion capacitance factor for linearization";
      
        /* Transit time parameters */
        parameter Modelica.SIunits.Time TF(min=0) = 0 
        "|Transit time parameters|Ideal forward transit time";
        parameter Modelica.SIunits.Time TR(min=0) = 0 
        "|Transit time parameters|Ideal reverse transit time";
      
        /* Temperature compensation and area parameters */
        parameter Real TRB1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
        parameter Real TRB2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
        parameter Real TRM1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
        parameter Real TRM2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
        parameter Real TRC1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
        parameter Real TRC2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
        parameter Real TRE1(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
        parameter Real TRE2(unit="Ohm/K2") = 0 
        "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Real XTI=3 
        "|Temperature compensation parameters|Saturation current temperature exponent";
        parameter Real XTB=0 
        "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
        parameter Modelica.SIunits.Voltage EG=1.11 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
        "|Advanced|Numerical parameters|Leakage conductance";
      
    protected 
        parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
        parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
    public 
      BondLib.Interfaces.BondCon B "Base" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon E "Emitter" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon H "Heat" annotation (extent=[110,-10; 130,10]);
      BondLib.Interfaces.BondCon C "Collector" 
                                          annotation (extent=[-10,110; 10,130]);
      BondLib.Bonds.Bond B2 annotation (extent=[-50,-10; -30,10], rotation=180);
      BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-70,-10; -50,10]);
      BondLib.Bonds.Bond B3 annotation (extent=[-90,-10; -70,10], rotation=180);
      BondLib.Bonds.Bond B4 annotation (extent=[-70,-30; -50,-10], rotation=270);
      BondLib.Spice.Utilities.RBS Rb(
        Tnom=Tnom,
        RB=RB,
        RBM=RBM,
        IRB=IRB,
        TRB1=TRB1,
        TRB2=TRB2,
        TRM1=TRM1,
        TRM2=TRM2,
        Area=Area,
        Level=Level) 
                   annotation (extent=[-60,-50; -20,-30], rotation=0);
      BondLib.Bonds.eBond B5 annotation (extent=[-20,-50; 0,-30]);
      BondLib.Bonds.Bond B6 annotation (extent=[70,20; 90,40], rotation=0);
      BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[50,40; 70,20]);
      BondLib.Bonds.Bond B7 annotation (extent=[30,20; 50,40], rotation=0);
      BondLib.Bonds.Bond B9 annotation (extent=[50,40; 70,60], rotation=90);
      BondLib.Bonds.eBond B10 annotation (extent=[110,80; 130,100], rotation=90);
      BondLib.Bonds.Bond B11 annotation (extent=[10,40; 30,60], rotation=90);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[10,80; 30,60], rotation=-90);
      BondLib.Bonds.Bond B12 annotation (extent=[10,80; 30,100], rotation=90);
      BondLib.Junctions.J0p2 J0p2_3 
        annotation (extent=[10,100; 30,120], rotation=90);
      BondLib.Bonds.Bond B13 annotation (extent=[-10,60; 10,80], rotation=180);
      BondLib.Bonds.eBond B14 annotation (extent=[-70,60; -50,80], rotation=180);
      BondLib.Bonds.Bond B15 annotation (extent=[10,-60; 30,-40], rotation=90);
      BondLib.Junctions.J1p3 J1p3_4 
        annotation (extent=[10,-60; 30,-80], rotation=-90);
      BondLib.Bonds.Bond B16 annotation (extent=[10,-100; 30,-80], rotation=90);
      BondLib.Junctions.J0p2 J0p2_4 
        annotation (extent=[10,-120; 30,-100], rotation=90);
      BondLib.Bonds.Bond B17 annotation (extent=[-10,-80; 10,-60], rotation=180);
      BondLib.Bonds.eBond B18 
        annotation (extent=[-70,-80; -50,-60], rotation=180);
      BondLib.Bonds.eBond B19 annotation (extent=[40,-30; 60,-10]);
      BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[60,-100; 80,-80]);
      BondLib.Bonds.Bond B20 annotation (extent=[10,0; 30,20], rotation=90);
      BondLib.Junctions.J0p3 J0p3_3 
        annotation (extent=[-110,-10; -90,10], rotation=180);
      BondLib.Bonds.Bond B21 annotation (extent=[-110,10; -90,30], rotation=90);
      BondLib.Junctions.J1p3 J1p3_5 
        annotation (extent=[-110,30; -90,50], rotation=90);
      BondLib.Bonds.Bond B22 annotation (extent=[-110,50; -90,70], rotation=90);
      BondLib.Bonds.Bond B23 annotation (extent=[-90,30; -70,50], rotation=0);
      BondLib.Bonds.eBond B24 annotation (extent=[-30,30; -10,50]);
      BondLib.Bonds.eBond B25 annotation (extent=[80,-100; 100,-80]);
      BondLib.Junctions.J0p3 J0p3_4 
        annotation (extent=[100,-100; 120,-80], rotation=180);
      BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[10,20; 30,40]);
      annotation (
        Diagram(
          Text(
            extent=[-126,22; -114,8],
            string="B",
            style(color=10, rgbcolor={95,95,95})),
          Text(
            extent=[-20,128; -8,114],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[-20,-114; -8,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E"),
          Text(
            extent=[106,22; 132,6],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[82,-96; 108,-112],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[38,-34; 48,-46],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[-38,-14; -28,-26],
            style(color=10, rgbcolor={95,95,95}),
            string="qb"),
          Text(
            extent=[0,10; 10,-2],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[-50,30; -40,18],
            style(color=10, rgbcolor={95,95,95}),
            string="ix"),
          Text(
            extent=[-126,-36; -114,-50],
            style(color=10, rgbcolor={95,95,95}),
            string="S")),
        Coordsys(extent=[-120,-120; 120,120]),
        Icon(
          Text(
            extent=[-80,100; 80,60],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
            Line(points=[0,100; 0,-100],
                                       style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Text(
              extent=[-94,84; 70,-92],
              style(color=0),
            string="PNP"),
            Text(
              extent=[58,4; 110,-42],
              style(color=0),
            string="vert"),
          Line(points=[-100,-60; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-80; 100,-60], style(color=3, rgbcolor={0,0,255})),
          Line(points=[80,-40; 100,-60], style(color=3, rgbcolor={0,0,255})),
            Line(points=[-20,80; 0,100], style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,100; 20,80],   style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Text(
            extent=[-88,-72; -44,-108],
            string="%Area x",
            style(color=3, rgbcolor={0,0,255})),
          Text(
            extent=[8,-114; 20,-128],
            style(color=10, rgbcolor={95,95,95}),
            string="E"),
          Text(
            extent=[106,26; 132,6],
            style(color=10, rgbcolor={95,95,95}),
            string="Heat"),
          Text(
            extent=[10,126; 22,112],
            style(color=10, rgbcolor={95,95,95}),
            string="C"),
          Text(
            extent=[-126,24; -114,10],
            string="B",
            style(color=10, rgbcolor={95,95,95}))),
        Documentation(info="<html>
The <b>PNPvert</b> element of the Spice bond graph library implements a full-fledges Spice-stye Gummel-Poon model of the vertically diffused PNP bipolar transistor [1-3]. The model contains a <b>PNPint</b> model implementing the inner parts of the PNP transistor up to the internal nodes. <p>

The PNP bipolar transistor is a directed FourPort.  The direction of positive power flow is assumed into the model at the emitter, <b>E</b>, whereas it is assumed out of the model at the base, <b>B</b>, at the collector, <b>C</b>, and at the <b>Heat</b> port. <p>

The causality of the PNP model is free. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
      BondLib.Spice.Utilities.PNPint Qpi(
        BF=BF,
        BR=BR,
        IS=IS,
        NF=NF,
        NR=NR,
        GminDC=GminDC,
        NC=NC,
        NE=NE,
        VAF=VAF,
        VAR=VAR,
        IKF=IKF,
        IKR=IKR,
        CJC=CJC,
        MJC=MJC,
        VJC=VJC,
        CJE=CJE,
        MJE=MJE,
        VJE=VJE,
        XCJC=XCJC,
        TF=TF,
        TR=TR,
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        EMin=EMin,
        EMax=EMax,
        FC=FC,
        Level=Level,
        ISC=ISCeff,
        ISE=ISEeff,
        ISS=ISS) annotation (extent=[0,-40; 40,0]);
      BondLib.Junctions.J0p2 J0p2_1 
        annotation (extent=[90,20; 110,40], rotation=180);
      Utilities.CbxS Cbx   annotation (extent=[-70,30; -30,50]);
      Utilities.RS2 Rc(
        Tnom=Tnom,
        R=RC,
        TR1=TRC1,
        TR2=TRC2,
        Area=Area,
        Level=Level) annotation (extent=[-10,58; -50,82]);
      Utilities.RS2 Re(
        Tnom=Tnom,
        Area=Area,
        Level=Level,
        R=RE,
        TR1=TRE1,
        TR2=TRE2) annotation (extent=[-10,-82; -50,-58]);
      BondLib.Interfaces.BondCon S "Substrate" 
        annotation (extent=[-130,-70; -110,-50]);
      Utilities.CjS Dcs(
        Tnom=Tnom,
        XTI=XTI,
        XTB=XTB,
        EG=EG,
        Area=Area,
        Level=Level,
        BE=false,
        EMin=EMin,
        EMax=EMax,
        MJ=MJS,
        VJ=VJS,
        XCJ=1,
        Tau=0,
        IS=0,
        GminDC=0,
        CJ=CJS,
        FC=0,
        N=1)       annotation (extent=[70,66; 110,94]);
      Modelica.Blocks.Sources.Constant C0(k=0) 
        annotation (extent=[96,44; 84,56]);
      Modelica.Blocks.Sources.Constant qb(k=1) 
        annotation (extent=[116,44; 104,56]);
      equation 
      connect(Rb.BondCon2, B5.fBondCon1)  annotation (points=[-20,-40; -20,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_4.BondCon1, E)        annotation (points=[20,-120; 0,-120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_3.BondCon2, C)        annotation (points=[20,120; 0,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_3.BondCon2, B)        annotation (points=[-110,1.22461e-015;
            -115,1.22461e-015; -115,0; -120,0], style(color=8, rgbcolor={192,
              192,192}));
      connect(J0p3_3.BondCon3, B21.BondCon1) annotation (points=[-100,10; -100,
            10.2], style(color=8, rgbcolor={192,192,192}));
      connect(B21.BondCon2, J1p3_5.BondCon1) annotation (points=[-100,30; -100,
            30], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon1, J1p3_5.BondCon2) annotation (points=[-100,50.2;
            -100,50], style(color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon2, B25.fBondCon1) annotation (points=[80,-90; 80,
            -90], style(color=8, rgbcolor={192,192,192}));
      connect(B25.eBondCon1, J0p3_4.BondCon2) annotation (points=[100,-90; 100,
            -90], style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon1, J1p3_1.BondCon3) annotation (points=[-60,-10.2; -60,
            -10], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_5.BondCon3, B23.BondCon1) annotation (points=[-90,40; -89.8,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon2, Rb.BondCon1)  annotation (points=[-60,-30; -60,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B13.BondCon1, J1p3_3.BondCon3) annotation (points=[9.8,70; 10,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon2, J0p4_1.BondCon1) annotation (points=[-100,70; -100,
            88; 0,88; 0,30; 10,30], style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon1, J1p3_2.BondCon3) annotation (points=[60,40.2; 60,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B17.BondCon1, J1p3_4.BondCon3) annotation (points=[9.8,-70; 10,
            -70], style(color=8, rgbcolor={192,192,192}));
      connect(B10.eBondCon1, J0p3_4.BondCon1) annotation (points=[120,100; 120,
            108; 132,108; 132,-90; 120,-90], style(color=8, rgbcolor={192,192,
              192}));
      connect(B5.eBondCon1, J0p6_1.BondCon1) annotation (points=[0,-40; 8,-40;
            8,-52; 60,-52; 60,-90], style(color=8, rgbcolor={192,192,192}));
      connect(B19.eBondCon1, J0p6_1.BondCon5) annotation (points=[60,-20; 66,
            -20; 66,-80], style(color=8, rgbcolor={192,192,192}));
      connect(B24.eBondCon1, J0p6_1.BondCon6) annotation (points=[-10,40; -6,40;
            -6,12; 74,12; 74,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_4.BondCon3, H)        annotation (points=[110,-80; 120,-80;
            120,0], style(color=8, rgbcolor={192,192,192}));
      connect(B18.eBondCon1, J0p6_1.BondCon3) annotation (points=[-70,-70; -76,
            -70; -76,-88; 48,-88; 48,-100; 66,-100], style(color=8, rgbcolor={
              192,192,192}));
      connect(B14.eBondCon1, J0p6_1.BondCon4) annotation (points=[-70,70; -80,
            70; -80,-92; 44,-92; 44,-104; 74,-104; 74,-100], style(color=8,
            rgbcolor={192,192,192}));
      connect(Qpi.qb, Rb.qb)      annotation (points=[41.4286,-33.7143; 50,
            -33.7143; 50,-48; 12,-48; 12,-24; -40,-24; -40,-30], style(color=74,
            rgbcolor={0,0,127}));
      connect(B2.BondCon2, J1p3_1.BondCon2) annotation (points=[-50,
            1.22461e-015; -41,1.22461e-015; -41,0; -50,0], style(color=8,
            rgbcolor={192,192,192}));
      connect(B3.BondCon1, J1p3_1.BondCon1) annotation (points=[-70.2,
            -1.20011e-015; -70,-1.20011e-015; -70,0], style(color=8, rgbcolor={
              192,192,192}));
      connect(B3.BondCon2, J0p3_3.BondCon1) annotation (points=[-90,
            1.22461e-015; -90,-1.22461e-015], style(color=8, rgbcolor={192,192,
              192}));
      connect(J0p2_3.BondCon1, B12.BondCon2) annotation (points=[20,100; 20,100],
          style(color=8, rgbcolor={192,192,192}));
      connect(B12.BondCon1, J1p3_3.BondCon2) annotation (points=[20,80.2; 20,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon1, B11.BondCon2) annotation (points=[20,60; 20,60],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon4, B11.BondCon1) annotation (points=[20,40; 20,40.2],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon3, B20.BondCon2) annotation (points=[20,20; 20,20],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_4.BondCon2, B15.BondCon1) annotation (points=[20,-60; 20,
            -59.8], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_4.BondCon1, B16.BondCon2) annotation (points=[20,-80; 20,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_4.BondCon2, B16.BondCon1) annotation (points=[20,-100; 20,
            -99.8], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon2, B7.BondCon1) annotation (points=[30,30; 30.2,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon1, B7.BondCon2) annotation (points=[50,30; 50,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon2, B6.BondCon1) annotation (points=[70,30; 70.2,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(Cbx.BondCon1, B23.BondCon2)   annotation (points=[-70,40; -70,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Cbx.BondCon2, B24.fBondCon1)   annotation (points=[-30,40; -30,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(Qpi.ix, Cbx.ix)       annotation (points=[10,0; 10,10; -50,10;
            -50,30], style(color=74, rgbcolor={0,0,127}));
      connect(Rc.BondCon2, B14.fBondCon1) annotation (points=[-50,70; -50,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rc.BondCon1, B13.BondCon2) annotation (points=[-10,70; -10,70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Re.BondCon2, B18.fBondCon1) annotation (points=[-50,-70; -50,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(Re.BondCon1, B17.BondCon2) annotation (points=[-10,-70; -10,-70],
          style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon2, J0p2_1.BondCon2) annotation (points=[90,30; 90,30],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p2_1.BondCon1, S)        annotation (points=[110,30; 136,30;
            136,-130; -120,-130; -120,-60], style(color=8, rgbcolor={192,192,
              192}));
      connect(C0.y, Dcs.C0) annotation (points=[83.4,50; 80,50; 80,66], style(
            color=74, rgbcolor={0,0,127}));
      connect(qb.y, Dcs.qb) annotation (points=[103.4,50; 100,50; 100,66],
          style(color=74, rgbcolor={0,0,127}));
      connect(Dcs.BondCon2, B10.fBondCon1) annotation (points=[110,80; 120,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(Dcs.BondCon1, B9.BondCon2) annotation (points=[70,80; 60,80; 60,
            60], style(color=8, rgbcolor={192,192,192}));
      connect(Qpi.Bint, B2.BondCon1) annotation (points=[0,-20; -20,-20; -20,
            -1.20011e-015; -30.2,-1.20011e-015], style(color=8, rgbcolor={192,
              192,192}));
      connect(Qpi.Eint, B15.BondCon2) annotation (points=[20,-40; 20,-40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B19.fBondCon1, Qpi.H) annotation (points=[40,-20; 40,-20], style(
            color=8, rgbcolor={192,192,192}));
      connect(B20.BondCon1, Qpi.Cint) annotation (points=[20,0.2; 20,0], style(
            color=8, rgbcolor={192,192,192}));
      end PNPvert;
    
      model NMOS "Spice-style NMOS transistor in bond graph technology" 
        constant Real pi=Modelica.Constants.pi;
        constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
        "Boltzmann's constant";
        constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
        "Permittivity of free space";
        constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
        "Electron charge";
        constant Modelica.SIunits.ElectronNumberDensity ni=1.45e16 
        "Intrinsic carrier concentration at 300 K";
        constant Real kox=3.9 "Dielectric constant of SiO2";
        constant Real ks=11.8 "Dielectric constant of Si";
        constant Real GapC1=7.02E-4 
        "Sze's first bandgap correction factor Silicon";
        constant Real GapC2=1108.0 
        "Sze's second bandgap correction factor Silicon";
        constant Modelica.SIunits.Voltage Fn0=3.25 
        "Work function constant for computation of flat band voltage";
        constant Real TOXdef = 1e-7 "Default oxide thickness";
      
        /* General parameters */
        parameter Integer Level(min=0,max=4) = 3 
        "Level of MOS model (check documentation window for details)";
      
        /* Basic Geometric parameters */
        parameter Modelica.SIunits.Length L(min=0) = 1e-4 
        "|Geometric parameters|Channel length";
        parameter Modelica.SIunits.Length W(min=0) = 1e-4 
        "|Geometric parameters|Channel width";
      
        /* Basic Process parameters */
        parameter Real TPG=1 "|Process parameters|Type of gate material";
        parameter Modelica.SIunits.Length TOX=TOXdef 
        "|Process parameters|Thin oxide thickness";
        parameter Real COX(unit="F/m2") = 0 
        "|Process parameters|Specific capacitance of SiO2";
        parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
        "|Process parameters|Substrate doping";
      
        /* Basic Electrical parameters */
        parameter Real U0(unit="m2/(V.s)") = 0 
        "|Electrical parameters|Surface mobility at reference temperature";
        parameter Modelica.SIunits.Voltage VT0=0 
        "|Electrical parameters|Zero-bias threshold voltage";
        parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
        "|Electrical parameters|Channel length modulation";
        parameter Modelica.SIunits.Length LD(min=0) = 0 
        "|Electrical parameters|Lateral diffusion";
        parameter Modelica.SIunits.Length WD(min=0) = 0 
        "|Electrical parameters|Width diffusion";
        parameter Modelica.SIunits.Transconductance KP=0 
        "|Electrical parameters|Transconductance parameter at reference temperature";
        parameter Real GAMMA(unit="V^0.5") = 0 
        "|Electrical parameters|Body-effect parameter";
        parameter Modelica.SIunits.Voltage PHI=0 
        "|Electrical parameters|Surface inversion potential at reference temperature";
        parameter Real NSS(unit="1/m2") = 0 
        "|Electrical parameters|Surface state density";
      
        /* Basic temperature compensation parameters */
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Modelica.SIunits.Voltage EG=1.16 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
      
        /* Level=1 electrical parameters */
        parameter Modelica.SIunits.Area AD=0 
        "|Level=1|Electrical parameters|Drain diffusion area";
        parameter Modelica.SIunits.Length PD=0 
        "|Level=1|Electrical parameters|Drain perimeter width";
        parameter Modelica.SIunits.Area AS=0 
        "|Level=1|Electrical parameters|Source diffusion area";
        parameter Modelica.SIunits.Length PS=0 
        "|Level=1|Electrical parameters|Source perimeter width";
        parameter Modelica.SIunits.Current ISD(min=0) = 0 
        "|Level=1|Electrical parameters|Drain junction saturation current at reference temperature";
        parameter Modelica.SIunits.Current ISS(min=0) = 0 
        "|Level=1|Electrical parameters|Source junction saturation current at reference temperature";
        parameter Modelica.SIunits.CurrentDensity JS(min=0) = 0 
        "|Level=1|Electrical parameters|Saturation current density at reference temperature";
        parameter Modelica.SIunits.Voltage PB(min=0.2) = 0.8 
        "|Level=1|Electrical parameters|Built-in potential at reference temperature";
      
        /* Level=1 parasitic resistance parameters */
        parameter Modelica.SIunits.Resistance RD(min=0) = 1 
        "|Level=1|Parasitic resistance parameters|Drain Ohmic resistance";
        parameter Modelica.SIunits.Resistance RS(min=0) = 1 
        "|Level=1|Parasitic resistance parameters|Source Ohmic resistance";
      
        /* Level=1 junction capacitance parameters */
        parameter Real CJ(unit="F/m2",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Zero-bias bulk capacitance per square meter at reference temperature";
        parameter Real MJ(min=0.3,max=0.5) = 0.33 
        "|Level=1|Junction capacitance parameters|Bulk junction grading coefficient";
        parameter Real CJSW(unit="F/m",min=0) = 1e-9 
        "|Level=1|Junction capacitance parameters|Zero-bias perimeter capacitance per meter at reference temperature";
        parameter Real MJSW(min=0.3,max=0.5) = 0.33 
        "|Level=1|Junction capacitance parameters|Perimeter capacitance grading coefficient";
        parameter Real FC(min=0,max=0.95)= 0.5 
        "|Level=1|Junction capacitance parameters|Forward-bias depletion capacitance coefficient";
        parameter Real CGB0(unit="F/m",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Gate-bulk overlap capacitance per meter";
        parameter Real CGD0(unit="F/m",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Gate-drain overlap capacitance per meter";
        parameter Real CGS0(unit="F/m",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Gate-source overlap capacitance per meter";
      
        /* Level=1 temperature compensation parameters */
        parameter Real TRD1(unit="Ohm/degree") = 0 
        "|Level=1|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
        parameter Real TRD2(unit="Ohm/degree2") = 0 
        "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of drain resistance";
        parameter Real TRS1(unit="Ohm/degree") = 0 
        "|Level=1|Temperature compensation parameters|Linear temperature coefficient of source resistance";
        parameter Real TRS2(unit="Ohm/degree2") = 0 
        "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of source resistance";
      
        /* Level=2 process parameters */
        parameter Modelica.SIunits.Length XJ=0 
        "|Level=2|Process parameters|Metallurgical junction depth";
      
        /* Level=2 electrical parameters */
        parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
        "|Level=2|Electrical parameters|Critical electric field for mobility";
        parameter Real UEXP=0 
        "|Level=2|Electrical parameters|Exponential coefficient for mobility";
        parameter Real UTRA(min=0,max=0.5) = 0 
        "|Level=2|Electrical parameters|Transverse field coefficient";
        parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
        "|Level=2|Electrical parameters|Critical electric field for pinch-off";
        parameter Real DELTA=0 
        "|Level=2|Electrical parameters|Width effect on threshold voltage";
        parameter Modelica.SIunits.Velocity VMAX=0 
        "|Level=2|Electrical parameters|Maximum drift velocity of carriers";
        parameter Real NFS(unit="1/m2") = 0 
        "|Level=2|Electrical parameters|Surface fast state density";
        parameter Real NEFF(min=Modelica.Constants.small) = 1 
        "|Level=2|Electrical parameters|Total channel charge coefficient";
        parameter Real XQC=0 
        "|Level=2|Electrical parameters|Coefficient of channel charge share";
      
        /* Level=3 electrical parameters */
        parameter Real KAPPA(min=Modelica.Constants.small) = 0.2 
        "|Level=3|Electrical parameters|Saturation field factor";
        parameter Real ETA=0 
        "|Level=3|Electrical parameters|Static feedback on threshold voltage";
        parameter Modelica.SIunits.InversePotential THETA=0 
        "|Level=3|Electrical parameters|Mobility modulation";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC=1e-12 
        "|Advanced|Numerical parameters|Leakage conductance";
      
        /* Implement Spice-style MODCHK */
    protected 
        parameter Integer Type = 1 "Type=1 for NMOS; Type=-1 for PMOS";
        parameter Modelica.SIunits.Permittivity es = ks*e0 
        "Permittivity of silicon";
        parameter Modelica.SIunits.Permittivity eox = kox*e0 
        "Permittivity of SiO2";
        parameter Modelica.SIunits.Length LDeff = LD;
        parameter Modelica.SIunits.Length Leff = L - 2*LDeff;
        parameter Modelica.SIunits.Length Weff = W - 2*WD;
        parameter Modelica.SIunits.Length TOXeff = if TOX == TOXdef and COX > 0 then eox/COX else TOX;
        parameter Real COXeff(unit="F/m2") = if COX > 0 then COX else eox/TOX;
        parameter Modelica.SIunits.Capacitance CGB0eff = CGB0;
        parameter Modelica.SIunits.Capacitance CGD0eff = CGD0;
        parameter Modelica.SIunits.Capacitance CGS0eff = CGS0;
        parameter Real U0def(unit="m2/(V.s)") = 0.06;
        parameter Real U0tmp(unit="m2/(V.s)") = if KP == 0 and U0 == 0 then U0def else U0;
        parameter Modelica.SIunits.Transconductance KPeff = if KP > 0 then KP else U0tmp*COXeff;
        parameter Real U0eff(unit="m2/(V.s)") = if KP > 0 and U0 == 0 then KP/COXeff else U0tmp;
        parameter Real GAMMAtmp(unit="V^0.5") = if GAMMA == 0 and NSUB == 0 then 0.35 else GAMMA;
        parameter Modelica.SIunits.ElectronNumberDensity NSUBeff = if NSUB > 0 then NSUB else (GAMMAtmp*COXeff)^2/(2*es*q);
        parameter Real GAMMAeff(unit="V^0.5") = if GAMMA == 0 and NSUB > 0 then sqrt(2*es*q*NSUB)/COXeff else GAMMAtmp;
        parameter Modelica.SIunits.InversePotential LAMBDAeff = if LAMBDA > 0 then LAMBDA else 
                                                                if Level <> 2 then sqrt(2*es/(q*NSUBeff)) else LAMBDA;
        parameter Modelica.SIunits.Voltage EGref = EG - GapC1*Tnom*Tnom/(Tnom + GapC2) 
        "Energy gap for temperature effect on saturation current at reference temperature";
        parameter Modelica.SIunits.ElectronNumberDensity XNI = ni*(Tnom/300)^1.5 * exp(EGref*q/(2*k)*(1/300 - 1/Tnom)) 
        "Intrinsic carrier concentration at reference temperature";
        parameter Modelica.SIunits.Voltage VTref = k*Tnom/q 
        "Thermal voltage at reference temperature";
        parameter Modelica.SIunits.Voltage PHIeff = if PHI > 0 then PHI else if NSUB > 0 then max({2*VTref*ln(NSUB/XNI),0.1}) else 0.6;
        parameter Modelica.SIunits.Voltage FermiG = Type*TPG*EGref/2;
        parameter Modelica.SIunits.Voltage Fn1 = if TPG == 0 then 3.2 else Fn0 + EGref/2 - FermiG;
        parameter Modelica.SIunits.Voltage FermiS = Type*PHIeff/2;
        parameter Modelica.SIunits.Voltage Fn2 = Fn1 - Fn0 - EGref/2 - FermiS;
        parameter Modelica.SIunits.Voltage VFB = Fn2 - NSS*q/COXeff;
        parameter Modelica.SIunits.Voltage Vcorr = Type*(GAMMAeff*sqrt(PHIeff) + PHIeff);
        parameter Modelica.SIunits.Voltage VT0eff = if Type*VT0 > 0 then VT0 else VFB + Vcorr;
        parameter Modelica.SIunits.Voltage VFBeff = if Type*VT0 > 0 then VT0 - Vcorr else VFB;
        parameter Real XD(unit="m/V^0.5") = sqrt(2*es/(q*NSUBeff)) 
        "Depletion factor";
        parameter Real CJeff(unit="F/m2") = if CJ > 0 then CJ else sqrt(es*q*NSUBeff/(2*PB));
        parameter Modelica.SIunits.Voltage EGeff = if Level == 3 and EG == 0 then 0.2 else EGref;
        parameter Modelica.SIunits.Voltage VBP = UCRIT*es/COXeff 
        "Gate to channel critical voltage";
        parameter Modelica.SIunits.Current ISDeff = if ISD > 0 then ISD else if AD*JS > 0 then AD*JS else 1e-15;
        parameter Modelica.SIunits.Current ISSeff = if ISS > 0 then ISS else if AS*JS > 0 then AS*JS else 1e-15;
    public 
      BondLib.Interfaces.BondCon G "Gate" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon D "Drain" 
        annotation (extent=[-10,110; 10,130]);
      BondLib.Interfaces.BondCon B "Bulk" 
        annotation (extent=[110,-10; 130,10]);
      BondLib.Interfaces.BondCon S "Source" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon H "Heat" 
        annotation (extent=[-70,-130; -50,-110]);
      annotation (
        Diagram(
          Text(
            extent=[-90,-114; -68,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="Heat"),
          Text(
            extent=[2,-114; 24,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Text(
            extent=[110,20; 132,6],
            style(color=10, rgbcolor={135,135,135}),
            string="B"),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[-130,20; -108,6],
            style(color=10, rgbcolor={135,135,135}),
            string="G"),
            Text(
              extent=[-138,60; -124,54],
              string="vds",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-138,-50; -124,-56],
              string="vds",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-54,136; -40,130],
              string="vds",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-98,136; -84,130],
              style(color=10, rgbcolor={135,135,135}),
            string="vbs"),
            Text(
              extent=[-138,94; -124,88],
              style(color=10, rgbcolor={135,135,135}),
            string="vbs"),
            Text(
              extent=[-140,-84; -126,-90],
              style(color=10, rgbcolor={135,135,135}),
            string="vbs")),
        Coordsys(extent=[-120,-120; 120,120]),
        Documentation(info="<html>
The Spice-style n-channel MOSFET model contains the internal MOSFET plus the three external gate capacitances and the drain and source resistances. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model</b></font>


<b>Basic Geometric Parameters:</b>

 L:       Channel length (default value = 1e-4 m) <font color=red>Levels 0-4</font>

 W:       Channel width (default value = 1e-4 m) <font color=red>Levels 0-4</font>


<b>Basic Process Parameters:</b>

 TPG:     Type of gate material (default value = 1) <font color=red>Levels 0-4</font>
            <font color=blue><b>TPG = 0:  Aluminum gate</b></font>
            <font color=blue><b>TPG = 1:  Polysilicon gate</b></font>

 TOX:     Thin oxide thickness (default value = 1e-7 m) <font color=red>Levels 0-4</font>

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 0-4</font>

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>) <font color=red>Levels 0-4</font>


<b>Basic Electrical Parameters:</b>

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s)) <font color=red>Levels 0-4</font>

 VT0:     Zero-bias threshold voltage (default value = 0 Volt) <font color=red>Levels 0,1</font>
            <font color=blue><b>VT0 &gt; 0 for enhancement MOSFET
            VT0 &lt; 0 for depletion MOSFET</b></font>

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt) <font color=red>Levels 0-4</font>

 LD:      Lateral diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 WD:      Width diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 0-4</font>

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>) <font color=red>Levels 0-4</font>

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt) <font color=red>Levels 0-4</font>

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 0-4</font>


<b>Basic Temperature Compensation Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 0-4</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt) <font color=red>Levels 0-4</font>


<b>Level=1 Electrical Parameters:</b>

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PD:      Drain perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PS:      Source perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 JS:      Transport saturation current density (default value = 0 Amp/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-4</font>


<b>Level=1 Parasitic Resistor Parameters:</b>

 RD:      Drain Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>

 RS:      Source Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>


<b>Level=1 Junction Capacitance Parameters:</b>

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 MJ:      Bulk junction grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m) <font color=red>Levels 1-4</font>

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1-4</font>

 CGB0:    Gate-bulk overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGD0:    Gate-drain overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGS0:    Gate-source overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>


<b>Level=1 Temperature Compensation Parameters:</b>

 TRD1:    Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRD2:    Quadratic temperature coefficient of drain resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>

 TRS1:    Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRS2:    Quadratic temperature coefficient of source resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>


<b>Level=2 Process Parameters:</b>

 XJ:      Metallurgical Junction Depth (default value = 0 m) <font color=red>Levels 2-4</font>


<b>Level=2 Electrical Parameters:</b>

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m) <font color=red>Levels 2,4</font>

 UEXP:    Exponential coefficient for mobility (default value = 0) <font color=red>Levels 2,4</font>

 UTRA:    Transverse field coefficient (default value = 0) <font color=red>Levels 2,4</font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,4</font>

 DELTA:   Width effect on threshold voltage (default value = 0) <font color=red>Levels 2-4</font>

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s) <font color=red>Levels 2-4</font>
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation (Levels 2,4)</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation (Levels 2,4)</b></font>

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 2-4</font>

 NEFF:    Total channel charge coefficient (default value = 1) <font color=red>Levels 2,4</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2-4</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>


<b>Level=3 Electrical Parameters:</b>

 KAPPA:   Saturation field factor (default value = 0.2) <font color=red>Level 3</font>

 ETA:     Static feedback on threshold voltage (default value = 0) <font color=red>Level 3</font>

 THETA:   Mobility modulation (default value = 0 1/Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-4</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-4</font>

 GminDC:  Leakage conductance (default value = 1e-12 mho) <font color=red>Levels 0-4</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"),     Icon(
          Text(
            extent=[-80,110; 80,70],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
            Line(points=[0,100; 0,-100],
                                       style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Text(
              extent=[-114,92; 116,-96],
              style(color=0),
            string="NMOS"),
            Line(points=[-20,-80; 0,-100],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,-100; 20,-80],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[10,50; 108,50], style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[10,50; 40,60],  style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[10,50; 40,40],  style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[110,20; 132,6],
            style(color=10, rgbcolor={135,135,135}),
            string="B"),
          Text(
            extent=[2,-114; 24,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Text(
            extent=[-90,-114; -68,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="Heat"),
          Text(
            extent=[-130,20; -108,6],
            style(color=10, rgbcolor={135,135,135}),
            string="G")));
      BondLib.Spice.Utilities.NMOSint NMOSint(
        Level=Level,
        Tnom=Tnom,
        NSS=NSS,
        NFS=NFS,
        NEFF=NEFF,
        XJ=XJ,
        AD=AD,
        Type=1,
        VT0=VT0eff,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        LAMBDA=LAMBDAeff,
        TOX=TOXeff,
        COX=COXeff,
        NSUB=NSUBeff,
        L=Leff,
        W=Weff,
        LD=LDeff,
        PD=PD,
        AS=AS,
        PS=PS,
        XD=XD,
        TPG=TPG,
        U0=U0eff,
        UCRIT=UCRIT,
        UEXP=UEXP,
        UTRA=UTRA,
        VMAX=VMAX,
        ECRIT=ECRIT,
        XQC=XQC,
        DELTA=DELTA,
        KAPPA=KAPPA,
        ETA=ETA,
        THETA=THETA,
        VBP=VBP,
        ISD=ISDeff,
        ISS=ISSeff,
        PB=PB,
        CJ=CJeff,
        MJ=MJ,
        CJSW=CJSW,
        MJSW=MJSW,
        FC=FC,
        GminDC=GminDC,
        EG=EGeff,
        EMin=EMin,
        EMax=EMax) annotation (extent=[-20,-20; 20,20]);
      BondLib.Junctions.J0p4 J0p4_2 annotation (extent=[90,-10; 110,10]);
      BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[-10,80; 10,100]);
      BondLib.Bonds.Bond B1 annotation (extent=[-40,-10; -20,10],rotation=0);
      BondLib.Bonds.Bond B2 annotation (extent=[-10,20; 10,40],  rotation=-90);
      BondLib.Bonds.Bond B3 annotation (extent=[-10,-40; 10,-20],rotation=-90);
      BondLib.Bonds.Bond B4 annotation (extent=[20,-10; 40,10],  rotation=-180);
      BondLib.Junctions.J1p3 J1p3_1 
        annotation (extent=[10,40; -10,60],rotation=90);
      BondLib.Bonds.Bond B5 annotation (extent=[-10,60; 10,80],  rotation=-90);
      BondLib.Junctions.J1p3 J1p3_2 
        annotation (extent=[10,-60; -10,-40],
                                           rotation=90);
      BondLib.Bonds.Bond B6 annotation (extent=[-10,-80; 10,-60],rotation=-90);
      BondLib.Bonds.Bond B7 annotation (extent=[-30,40; -10,60], rotation=-180);
      BondLib.Bonds.Bond B8 annotation (extent=[-30,-60; -10,-40],
                                                                 rotation=-180);
      BondLib.Spice.Utilities.RM Rd(
        Tnom=Tnom,
        R=RD,
        Level=Level,
        TR1=TRD1,
        TR2=TRD2) annotation (extent=[-30,40; -50,60]);
      BondLib.Spice.Utilities.RM Rs(
        Tnom=Tnom,
        Level=Level,
        R=RS,
        TR1=TRS1,
        TR2=TRS2) annotation (extent=[-30,-60; -50,-40]);
      BondLib.Bonds.Bond B9 annotation (extent=[-30,80; -10,100],rotation=0);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[-50,80; -30,100],
                                           rotation=180);
      BondLib.Bonds.Bond B10 
                            annotation (extent=[-70,80; -50,100],rotation=0);
      BondLib.Bonds.Bond B11 
                            annotation (extent=[-50,100; -30,120],
                                                                 rotation=90);
      BondLib.Spice.Utilities.Cgd Cgd1(
        Tnom=Tnom,
        Level=Level,
        Type=1,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        COX=COXeff,
        NFS=NFS,
        XJ=XJ,
        L=Leff,
        W=Weff,
        LD=LDeff,
        XD=XD,
        XQC=XQC,
        DELTA=DELTA,
        ETA=ETA,
        CGD0=CGD0eff,
        EG=EGeff) annotation (extent=[-50,132; -90,108]);
      BondLib.Bonds.Bond B12 
                            annotation (extent=[-100,20; -80,40],rotation=90);
      BondLib.Junctions.J1p3 J1p3_4 
        annotation (extent=[-100,40; -80,60],
                                           rotation=270);
      BondLib.Bonds.Bond B13 
                            annotation (extent=[-100,60; -80,80],rotation=90);
      BondLib.Bonds.Bond B14 
                            annotation (extent=[-120,40; -100,60],
                                                                 rotation=-180);
      BondLib.Spice.Utilities.Cgb Cgb1(
        Tnom=Tnom,
        Level=Level,
        Type=1,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        COX=COXeff,
        NFS=NFS,
        XJ=XJ,
        L=Leff,
        W=Weff,
        LD=LDeff,
        XD=XD,
        XQC=XQC,
        DELTA=DELTA,
        ETA=ETA,
        CGB0=CGB0eff,
        EG=EGeff) annotation (extent=[-108,54; -132,94], rotation=90);
      BondLib.Junctions.J0p5 J0p5_1 
        annotation (extent=[-10,-100; 10,-80], rotation=90);
      BondLib.Bonds.Bond B15 
                            annotation (extent=[20,-90; 40,-70], rotation=-180);
      BondLib.Junctions.J1p3 J1p3_5 
        annotation (extent=[40,-90; 60,-70],
                                           rotation=180);
      BondLib.Bonds.Bond B16 
                            annotation (extent=[60,-90; 80,-70], rotation=-180);
      BondLib.Bonds.fBond B17 
        annotation (extent=[40,-70; 60,-50], rotation=90);
      BondLib.Sensors.De Vbs annotation (extent=[40,-50; 60,-30], rotation=90);
      BondLib.Bonds.Bond B18 
                            annotation (extent=[10,80; 30,100],  rotation=0);
      BondLib.Junctions.J1p3 J1p3_6 
        annotation (extent=[30,100; 50,80],rotation=180);
      BondLib.Bonds.Bond B19 
                            annotation (extent=[50,80; 70,100],  rotation=0);
      BondLib.Bonds.fBond B20 
                             annotation (extent=[30,60; 50,80], rotation=-90);
      BondLib.Sensors.De Vds 
        annotation (extent=[30,40; 50,60],         rotation=-90);
      BondLib.Junctions.J0p5 J0p5_2 
        annotation (extent=[-96,10; -76,-10],  rotation=0);
      BondLib.Bonds.Bond B21 
                            annotation (extent=[-96,-30; -76,-10],
                                                                 rotation=-90);
      BondLib.Junctions.J1p3 J1p3_7 
        annotation (extent=[-76,-50; -96,-30],
                                           rotation=90);
      BondLib.Bonds.Bond B22 
                            annotation (extent=[-96,-70; -76,-50],
                                                                 rotation=-90);
      BondLib.Bonds.Bond B23 
                            annotation (extent=[-116,-50; -96,-30],
                                                                 rotation=-180);
      BondLib.Spice.Utilities.Cgs Cgs1(
        Tnom=Tnom,
        Level=Level,
        Type=1,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        COX=COXeff,
        NFS=NFS,
        XJ=XJ,
        L=Leff,
        W=Weff,
        LD=LDeff,
        XD=XD,
        XQC=XQC,
        DELTA=DELTA,
        ETA=ETA,
        CGS0=CGS0eff,
        EG=EGeff) annotation (extent=[-132,-90; -108,-50], rotation=-90);
      BondLib.Junctions.J0p6 J0p6_1 
        annotation (extent=[-70,-106; -50,-86],  rotation=90);
      Bonds.eBond B24 
        annotation (extent=[-130,-110; -110,-90],
                                                rotation=-90);
      Bonds.eBond B25 
        annotation (extent=[-70,40; -50,60],    rotation=-180);
      Bonds.eBond B26 
        annotation (extent=[-70,-60; -50,-40],  rotation=-180);
      Bonds.eBond B27 
        annotation (extent=[-30,-30; -10,-10],  rotation=-180);
      Bonds.eBond B28 
        annotation (extent=[-110,110; -90,130], rotation=-180);
      Bonds.eBond B29 
        annotation (extent=[-130,94; -110,114], rotation=-270);
      Junctions.J0p3 J0p3_1         annotation (extent=[-50,-30; -30,-10]);
      Bonds.eBond B30 
        annotation (extent=[-64,-46; -44,-26],  rotation=-180);
      algorithm 
        assert(NSUBeff > XNI, "NSUB must be larger than intrinsic carrier concentration");
        assert(LAMBDAeff < 0.2, "LAMBDA value too large, leads to numerical instability");
        assert(Leff > 0, "The effective channel length must be larger than zero");
      
      equation 
      connect(J0p4_2.BondCon2, B)        annotation (points=[110,0; 120,0], style(color=8,
            rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon4, D)        annotation (points=[0,100; 0,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_2.BondCon1, B4.BondCon1) annotation (points=[90,0; 54.9,0;
            54.9,1.20011e-015; 39.8,1.20011e-015], style(color=8, rgbcolor={192,
              192,192}));
      connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[6.12303e-016,40;
            -6.00057e-016,40; -6.00057e-016,39.8], style(color=8, rgbcolor={192,
              192,192}));
      connect(B5.BondCon2, J1p3_1.BondCon2) annotation (points=[6.12303e-016,60;
            -6.12303e-016,60], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon3, B5.BondCon1) annotation (points=[0,80; 0,79.8;
            -6.00057e-016,79.8], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon2, B3.BondCon2) annotation (points=[-6.12303e-016,
            -40; 6.12303e-016,-40], style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon1, J1p3_2.BondCon1) annotation (points=[-6.00057e-016,
            -60.2; -6.00057e-016,-63.1; 6.12303e-016,-63.1; 6.12303e-016,-60],
          style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1, J1p3_1.BondCon3) annotation (points=[-10.2,50; -10,
            50], style(color=8, rgbcolor={192,192,192}));
      connect(B8.BondCon1, J1p3_2.BondCon3) annotation (points=[-10.2,-50; -10,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(Rd.BondCon1, B7.BondCon2) annotation (points=[-30,50; -30,50],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rs.BondCon1, B8.BondCon2) annotation (points=[-30,-50; -30,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon2, J0p4_1.BondCon1) annotation (points=[-10,90; -10,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_3.BondCon1, B9.BondCon1) annotation (points=[-30,90; -29.8,
            90], style(color=8, rgbcolor={192,192,192}));
      connect(B10.BondCon2, J1p3_3.BondCon2) annotation (points=[-50,90; -50,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(B11.BondCon1, J1p3_3.BondCon3) annotation (points=[-40,100.2; -40,
            100], style(color=8, rgbcolor={192,192,192}));
      connect(Cgd1.BondCon1, B11.BondCon2) annotation (points=[-50,120; -40,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_4.BondCon2, B12.BondCon2) annotation (points=[-90,40; -90,40],
          style(color=8, rgbcolor={192,192,192}));
      connect(B13.BondCon1, J1p3_4.BondCon1) annotation (points=[-90,60.2; -90,
            60], style(color=8, rgbcolor={192,192,192}));
      connect(B14.BondCon1, J1p3_4.BondCon3) annotation (points=[-100.2,50;
            -100,50], style(color=8, rgbcolor={192,192,192}));
      connect(B14.BondCon2, Cgb1.BondCon1) annotation (points=[-120,50; -120,54],
          style(color=8, rgbcolor={192,192,192}));
      connect(B13.BondCon2, J0p4_2.BondCon4) annotation (points=[-90,80; -90,
            104; 100,104; 100,10], style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon2, J0p5_1.BondCon2) annotation (points=[6.12303e-016,
            -80; 6.12303e-016,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_1.BondCon1, S)        annotation (points=[-6.12303e-016,-100;
            0,-100; 0,-120], style(color=8, rgbcolor={192,192,192}));
      connect(B15.BondCon1, J1p3_5.BondCon2) annotation (points=[39.8,-80; 40,
            -80], style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_1.BondCon4, B15.BondCon2) annotation (points=[10,-86; 10,-80;
            20,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_5.BondCon1, B16.BondCon2) annotation (points=[60,-80; 60,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B17.eBondCon1, J1p3_5.BondCon3) annotation (points=[50,-70; 50,
            -70], style(color=8, rgbcolor={192,192,192}));
      connect(Vbs.BondCon1, B17.fBondCon1) annotation (points=[50,-50; 50,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_2.BondCon3, B16.BondCon1) annotation (points=[100,-10; 100,
            -80; 79.8,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon2, B18.BondCon1) annotation (points=[10,90; 10.2,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(B18.BondCon2, J1p3_6.BondCon2) annotation (points=[30,90; 30,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_6.BondCon1, B19.BondCon1) annotation (points=[50,90; 50.2,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_6.BondCon3, B20.eBondCon1) annotation (points=[40,80; 40,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B20.fBondCon1, Vds.BondCon1) annotation (points=[40,60; 40,60],
          style(color=8, rgbcolor={192,192,192}));
      connect(B19.BondCon2, J0p5_1.BondCon3) annotation (points=[70,90; 130,90;
            130,-94; 10,-94], style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_2.BondCon2, B1.BondCon1) annotation (points=[-76,0; -39.8,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_2.BondCon1, G)        annotation (points=[-96,0; -120,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_2.BondCon3, B12.BondCon1) annotation (points=[-90,10; -90,
            20.2], style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_2.BondCon4, B10.BondCon1) annotation (points=[-82,10; -76,10;
            -76,90; -69.8,90], style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_2.BondCon5, B21.BondCon1) annotation (points=[-86,-10; -86,
            -10.2], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_7.BondCon2, B21.BondCon2) annotation (points=[-86,-30; -86,
            -30], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon1, J1p3_7.BondCon1) annotation (points=[-86,-50.2; -86,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(B23.BondCon1, J1p3_7.BondCon3) annotation (points=[-96.2,-40; -96,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(Cgs1.BondCon1, B23.BondCon2) annotation (points=[-120,-50; -120,
            -40; -116,-40], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon2, J0p5_1.BondCon5) annotation (points=[-86,-70; -86,
            -76; -10,-76; -10,-90], style(color=8, rgbcolor={192,192,192}));
      connect(Vds.OutPort1, Cgd1.u1) annotation (points=[40,39; 40,30; 80,30;
            80,138; -58,138; -58,132], style(color=74, rgbcolor={0,0,127}));
      connect(Vbs.OutPort1, Cgd1.u2) annotation (points=[50,-29; 50,28; 82,28;
            82,140; -82,140; -82,132], style(color=74, rgbcolor={0,0,127}));
      connect(Cgb1.u2, Cgd1.u2) annotation (points=[-132,86; -140,86; -140,140;
            -82,140; -82,132], style(color=74, rgbcolor={0,0,127}));
      connect(Cgb1.u1, Cgd1.u1) annotation (points=[-132,62; -138,62; -138,138;
            -58,138; -58,132], style(color=74, rgbcolor={0,0,127}));
      connect(Cgs1.u1, Cgb1.u1) annotation (points=[-132,-58; -138,-58; -138,62;
            -132,62], style(color=74, rgbcolor={0,0,127}));
      connect(Cgs1.u2, Cgb1.u2) annotation (points=[-132,-82; -140,-82; -140,86;
            -132,86], style(color=74, rgbcolor={0,0,127}));
      connect(NMOSint.y1, Cgd1.u)  annotation (points=[-10,20; -10,32; -62,32;
            -62,98; -70,98; -70,108], style(color=74, rgbcolor={0,0,127}));
      connect(Cgb1.u, Cgd1.u) annotation (points=[-108,74; -104,74; -104,98;
            -70,98; -70,108], style(color=74, rgbcolor={0,0,127}));
      connect(Cgb1.u, Cgs1.u) annotation (points=[-108,74; -104,74; -104,-70;
            -108,-70], style(color=74, rgbcolor={0,0,127}));
      connect(J0p6_1.BondCon1, H)        annotation (points=[-60,-106; -60,-120],
          style(color=8, rgbcolor={192,192,192}));
      connect(Cgs1.BondCon2, B24.fBondCon1) annotation (points=[-120,-90; -120,
            -90], style(color=8, rgbcolor={192,192,192}));
      connect(Rd.BondCon2, B25.fBondCon1) annotation (points=[-50,50; -50,50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B26.fBondCon1, Rs.BondCon2) annotation (points=[-50,-50; -50,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B28.fBondCon1, Cgd1.BondCon2) annotation (points=[-90,120; -90,
            120], style(color=8, rgbcolor={192,192,192}));
      connect(B29.fBondCon1, Cgb1.BondCon2) annotation (points=[-120,94; -120,
            94], style(color=8, rgbcolor={192,192,192}));
      connect(B26.eBondCon1, J0p6_1.BondCon3) annotation (points=[-70,-50; -74,
            -50; -74,-66; -40,-66; -40,-100; -50,-100], style(color=8, rgbcolor=
             {192,192,192}));
      connect(B25.eBondCon1, J0p6_1.BondCon4) annotation (points=[-70,50; -74,
            50; -74,10; -68,10; -68,-38; -76,-38; -76,-68; -50,-68; -50,-92],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2, B27.eBondCon1) annotation (points=[-30,-20; -30,
            -20], style(color=8, rgbcolor={192,192,192}));
      connect(B30.fBondCon1, J0p3_1.BondCon3) annotation (points=[-44,-36; -40,
            -36; -40,-30], style(color=8, rgbcolor={192,192,192}));
      connect(B30.eBondCon1, J0p6_1.BondCon2) annotation (points=[-64,-36; -78,
            -36; -78,-70; -60,-70; -60,-86], style(color=8, rgbcolor={192,192,
              192}));
      connect(B24.eBondCon1, J0p3_1.BondCon1) annotation (points=[-120,-110;
            -120,-114; -102,-114; -102,-20; -50,-20], style(color=8, rgbcolor={
              192,192,192}));
      connect(J0p6_1.BondCon6, B29.eBondCon1) annotation (points=[-70,-92; -100,
            -92; -100,-116; -142,-116; -142,118; -120,118; -120,114], style(
            color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon5, B28.eBondCon1) annotation (points=[-70,-100; -98,
            -100; -98,-118; -144,-118; -144,120; -110,120], style(color=8,
            rgbcolor={192,192,192}));
      connect(B1.BondCon2, NMOSint.Gint) annotation (points=[-20,0; -20,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B3.BondCon1, NMOSint.Sint) annotation (points=[-6.00057e-016,
            -20.2; 0,-20.2; 0,-20], style(color=8, rgbcolor={192,192,192}));
      connect(NMOSint.H, B27.fBondCon1) annotation (points=[-10,-20; -10,-20],
          style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon2, NMOSint.Bint) annotation (points=[20,-1.22461e-015;
            20,0], style(color=8, rgbcolor={192,192,192}));
      connect(B2.BondCon2, NMOSint.Dint) annotation (points=[6.12303e-016,20; 0,
            20], style(color=8, rgbcolor={192,192,192}));
      end NMOS;
    
      model PMOS "Spice-style PMOS transistor in bond graph technology" 
        constant Real pi=Modelica.Constants.pi;
        constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
        "Boltzmann's constant";
        constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
        "Permittivity of free space";
        constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
        "Electron charge";
        constant Modelica.SIunits.ElectronNumberDensity ni=1.45e16 
        "Intrinsic carrier concentration at 300 K";
        constant Real kox=3.9 "Dielectric constant of SiO2";
        constant Real ks=11.8 "Dielectric constant of Si";
        constant Real GapC1=7.02E-4 
        "Sze's first bandgap correction factor Silicon";
        constant Real GapC2=1108.0 
        "Sze's second bandgap correction factor Silicon";
        constant Modelica.SIunits.Voltage Fn0=3.25 
        "Work function constant for computation of flat band voltage";
        constant Real TOXdef = 1e-7 "Default oxide thickness";
      
        /* General parameters */
        parameter Integer Level(min=0,max=4) = 3 
        "Level of MOS model (check documentation window for details)";
      
        /* Basic Geometric parameters */
        parameter Modelica.SIunits.Length L(min=0) = 1e-4 
        "|Geometric parameters|Channel length";
        parameter Modelica.SIunits.Length W(min=0) = 1e-4 
        "|Geometric parameters|Channel width";
      
        /* Basic Process parameters */
        parameter Real TPG=1 "|Process parameters|Type of gate material";
        parameter Modelica.SIunits.Length TOX=TOXdef 
        "|Process parameters|Thin oxide thickness";
        parameter Real COX(unit="F/m2") = 0 
        "|Process parameters|Specific capacitance of SiO2";
        parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
        "|Process parameters|Substrate doping";
      
        /* Basic Electrical parameters */
        parameter Real U0(unit="m2/(V.s)") = 0 
        "|Electrical parameters|Surface mobility at reference temperature";
        parameter Modelica.SIunits.Voltage VT0=0 
        "|Electrical parameters|Zero-bias threshold voltage";
        parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
        "|Electrical parameters|Channel length modulation";
        parameter Modelica.SIunits.Length LD(min=0) = 0 
        "|Electrical parameters|Lateral diffusion";
        parameter Modelica.SIunits.Length WD(min=0) = 0 
        "|Electrical parameters|Width diffusion";
        parameter Modelica.SIunits.Transconductance KP=0 
        "|Electrical parameters|Transconductance parameter at reference temperature";
        parameter Real GAMMA(unit="V^0.5") = 0 
        "|Electrical parameters|Body-effect parameter";
        parameter Modelica.SIunits.Voltage PHI=0 
        "|Electrical parameters|Surface inversion potential at reference temperature";
        parameter Real NSS(unit="1/m2") = 0 
        "|Electrical parameters|Surface state density";
      
        /* Basic temperature compensation parameters */
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Modelica.SIunits.Voltage EG=1.16 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
      
        /* Level=1 electrical parameters */
        parameter Modelica.SIunits.Area AD=0 
        "|Level=1|Electrical parameters|Drain diffusion area";
        parameter Modelica.SIunits.Length PD=0 
        "|Level=1|Electrical parameters|Drain perimeter width";
        parameter Modelica.SIunits.Area AS=0 
        "|Level=1|Electrical parameters|Source diffusion area";
        parameter Modelica.SIunits.Length PS=0 
        "|Level=1|Electrical parameters|Source perimeter width";
        parameter Modelica.SIunits.Current ISD(min=0) = 0 
        "|Level=1|Electrical parameters|Drain junction saturation current at reference temperature";
        parameter Modelica.SIunits.Current ISS(min=0) = 0 
        "|Level=1|Electrical parameters|Source junction saturation current at reference temperature";
        parameter Modelica.SIunits.CurrentDensity JS(min=0) = 0 
        "|Level=1|Electrical parameters|Saturation current density at reference temperature";
        parameter Modelica.SIunits.Voltage PB(min=0.2) = 0.8 
        "|Level=1|Electrical parameters|Built-in potential at reference temperature";
      
        /* Level=1 parasitic resistance parameters */
        parameter Modelica.SIunits.Resistance RD(min=0) = 1 
        "|Level=1|Parasitic resistance parameters|Drain Ohmic resistance";
        parameter Modelica.SIunits.Resistance RS(min=0) = 1 
        "|Level=1|Parasitic resistance parameters|Source Ohmic resistance";
      
        /* Level=1 junction capacitance parameters */
        parameter Real CJ(unit="F/m2",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Zero-bias bulk capacitance per square meter at reference temperature";
        parameter Real MJ(min=0.3,max=0.5) = 0.33 
        "|Level=1|Junction capacitance parameters|Bulk junction grading coefficient";
        parameter Real CJSW(unit="F/m",min=0) = 1e-9 
        "|Level=1|Junction capacitance parameters|Zero-bias perimeter capacitance per meter at reference temperature";
        parameter Real MJSW(min=0.3,max=0.5) = 0.33 
        "|Level=1|Junction capacitance parameters|Perimeter capacitance grading coefficient";
        parameter Real FC(min=0,max=0.95)= 0.5 
        "|Level=1|Junction capacitance parameters|Forward-bias depletion capacitance coefficient";
        parameter Real CGB0(unit="F/m",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Gate-bulk overlap capacitance per meter";
        parameter Real CGD0(unit="F/m",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Gate-drain overlap capacitance per meter";
        parameter Real CGS0(unit="F/m",min=0) = 0 
        "|Level=1|Junction capacitance parameters|Gate-source overlap capacitance per meter";
      
        /* Level=1 temperature compensation parameters */
        parameter Real TRD1(unit="Ohm/degree") = 0 
        "|Level=1|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
        parameter Real TRD2(unit="Ohm/degree2") = 0 
        "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of drain resistance";
        parameter Real TRS1(unit="Ohm/degree") = 0 
        "|Level=1|Temperature compensation parameters|Linear temperature coefficient of source resistance";
        parameter Real TRS2(unit="Ohm/degree2") = 0 
        "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of source resistance";
      
        /* Level=2 process parameters */
        parameter Modelica.SIunits.Length XJ=0 
        "|Level=2|Process parameters|Metallurgical junction depth";
      
        /* Level=2 electrical parameters */
        parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
        "|Level=2|Electrical parameters|Critical electric field for mobility";
        parameter Real UEXP=0 
        "|Level=2|Electrical parameters|Exponential coefficient for mobility";
        parameter Real UTRA(min=0,max=0.5) = 0 
        "|Level=2|Electrical parameters|Transverse field coefficient";
        parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
        "|Level=2|Electrical parameters|Critical electric field for pinch-off";
        parameter Real DELTA=0 
        "|Level=2|Electrical parameters|Width effect on threshold voltage";
        parameter Modelica.SIunits.Velocity VMAX=0 
        "|Level=2|Electrical parameters|Maximum drift velocity of carriers";
        parameter Real NFS(unit="1/m2") = 0 
        "|Level=2|Electrical parameters|Surface fast state density";
        parameter Real NEFF(min=Modelica.Constants.small) = 1 
        "|Level=2|Electrical parameters|Total channel charge coefficient";
        parameter Real XQC=0 
        "|Level=2|Electrical parameters|Coefficient of channel charge share";
      
        /* Level=3 electrical parameters */
        parameter Real KAPPA(min=Modelica.Constants.small) = 0.2 
        "|Level=3|Electrical parameters|Saturation field factor";
        parameter Real ETA=0 
        "|Level=3|Electrical parameters|Static feedback on threshold voltage";
        parameter Modelica.SIunits.InversePotential THETA=0 
        "|Level=3|Electrical parameters|Mobility modulation";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC=1e-12 
        "|Advanced|Numerical parameters|Leakage conductance";
      
        /* Implement Spice-style MODCHK */
    protected 
        parameter Integer Type = -1 "Type=1 for NMOS; Type=-1 for PMOS";
        parameter Modelica.SIunits.Permittivity es = ks*e0 
        "Permittivity of silicon";
        parameter Modelica.SIunits.Permittivity eox = kox*e0 
        "Permittivity of SiO2";
        parameter Modelica.SIunits.Length LDeff = LD;
        parameter Modelica.SIunits.Length Leff = L - 2*LDeff;
        parameter Modelica.SIunits.Length Weff = W - 2*WD;
        parameter Modelica.SIunits.Length TOXeff = if TOX == TOXdef and COX > 0 then eox/COX else TOX;
        parameter Real COXeff(unit="F/m2") = if COX > 0 then COX else eox/TOX;
        parameter Modelica.SIunits.Capacitance CGB0eff = CGB0;
        parameter Modelica.SIunits.Capacitance CGD0eff = CGD0;
        parameter Modelica.SIunits.Capacitance CGS0eff = CGS0;
        parameter Real U0def(unit="m2/(V.s)") = 0.06;
        parameter Real U0tmp(unit="m2/(V.s)") = if KP == 0 and U0 == 0 then U0def else U0;
        parameter Modelica.SIunits.Transconductance KPeff = if KP > 0 then KP else U0tmp*COXeff;
        parameter Real U0eff(unit="m2/(V.s)") = if KP > 0 and U0 == 0 then KP/COXeff else U0tmp;
        parameter Real GAMMAtmp(unit="V^0.5") = if GAMMA == 0 and NSUB == 0 then 0.35 else GAMMA;
        parameter Modelica.SIunits.ElectronNumberDensity NSUBeff = if NSUB > 0 then NSUB else (GAMMAtmp*COXeff)^2/(2*es*q);
        parameter Real GAMMAeff(unit="V^0.5") = if GAMMA == 0 and NSUB > 0 then sqrt(2*es*q*NSUB)/COXeff else GAMMAtmp;
        parameter Modelica.SIunits.InversePotential LAMBDAeff = if LAMBDA > 0 then LAMBDA else 
                                                                if Level <> 2 then sqrt(2*es/(q*NSUBeff)) else LAMBDA;
        parameter Modelica.SIunits.Voltage EGref = EG - GapC1*Tnom*Tnom/(Tnom + GapC2) 
        "Energy gap for temperature effect on saturation current at reference temperature";
        parameter Modelica.SIunits.ElectronNumberDensity XNI = ni*(Tnom/300)^1.5 * exp(EGref*q/(2*k)*(1/300 - 1/Tnom)) 
        "Intrinsic carrier concentration at reference temperature";
        parameter Modelica.SIunits.Voltage VTref = k*Tnom/q 
        "Thermal voltage at reference temperature";
        parameter Modelica.SIunits.Voltage PHIeff = if PHI > 0 then PHI else if NSUB > 0 then max({2*VTref*ln(NSUB/XNI),0.1}) else 0.;
        parameter Modelica.SIunits.Voltage FermiG = Type*TPG*EGref/2;
        parameter Modelica.SIunits.Voltage Fn1 = if TPG == 0 then 3.2 else Fn0 + EGref/2 - FermiG;
        parameter Modelica.SIunits.Voltage FermiS = Type*PHIeff/2;
        parameter Modelica.SIunits.Voltage Fn2 = Fn1 - Fn0 - EGref/2 - FermiS;
        parameter Modelica.SIunits.Voltage VFB = Fn2 - NSS*q/COXeff;
        parameter Modelica.SIunits.Voltage Vcorr = Type*(GAMMAeff*sqrt(PHIeff) + PHIeff);
        parameter Modelica.SIunits.Voltage VT0eff = if Type*VT0 > 0 then VT0 else VFB + Vcorr;
        parameter Modelica.SIunits.Voltage VFBeff = if Type*VT0 > 0 then VT0 - Vcorr else VFB;
        parameter Real XD(unit="m/V^0.5") = sqrt(2*es/(q*NSUBeff)) 
        "Depletion factor";
        parameter Real CJeff(unit="F/m2") = if CJ > 0 then CJ else sqrt(es*q*NSUBeff/(2*PB));
        parameter Modelica.SIunits.Voltage EGeff = if Level == 3 and EG == 0 then 0.2 else EGref;
        parameter Modelica.SIunits.Voltage VBP = UCRIT*es/COXeff 
        "Gate to channel critical voltage";
        parameter Modelica.SIunits.Current ISDeff = if ISD > 0 then ISD else if AD*JS > 0 then AD*JS else 1e-15;
        parameter Modelica.SIunits.Current ISSeff = if ISS > 0 then ISS else if AS*JS > 0 then AS*JS else 1e-15;
      annotation (
        Coordsys(extent=[-120,-120; 120,120]),
        Diagram(
          Text(
            extent=[-90,-114; -68,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="Heat"),
          Text(
            extent=[2,-114; 24,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Text(
            extent=[110,20; 132,6],
            style(color=10, rgbcolor={135,135,135}),
            string="B"),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[-130,20; -108,6],
            style(color=10, rgbcolor={135,135,135}),
            string="G"),
            Text(
              extent=[-138,60; -124,54],
              style(color=10, rgbcolor={135,135,135}),
            string="vsd"),
            Text(
              extent=[-138,-50; -124,-56],
              style(color=10, rgbcolor={135,135,135}),
            string="vsd"),
            Text(
              extent=[-54,136; -40,130],
              style(color=10, rgbcolor={135,135,135}),
            string="vsd"),
            Text(
              extent=[-98,136; -84,130],
              style(color=10, rgbcolor={135,135,135}),
            string="vsb"),
            Text(
              extent=[-138,94; -124,88],
              style(color=10, rgbcolor={135,135,135}),
            string="vsb"),
            Text(
              extent=[-140,-84; -126,-90],
              style(color=10, rgbcolor={135,135,135}),
            string="vsb")),
        Documentation(info="<html>
The Spice-style p-channel MOSFET model contains the internal MOSFET plus the three external gate capacitances and the drain and source resistances. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model</b></font>


<b>Basic Geometric Parameters:</b>

 L:       Channel length (default value = 1e-4 m) <font color=red>Levels 0-4</font>

 W:       Channel width (default value = 1e-4 m) <font color=red>Levels 0-4</font>


<b>Basic Process Parameters:</b>

 TPG:     Type of gate material (default value = 1) <font color=red>Levels 0-4</font>
            <font color=blue><b>TPG = 0:  Aluminum gate</b></font>
            <font color=blue><b>TPG = 1:  Polysilicon gate</b></font>

 TOX:     Thin oxide thickness (default value = 1e-7 m) <font color=red>Levels 0-4</font>

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 0-4</font>

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>) <font color=red>Levels 0-4</font>


<b>Basic Electrical Parameters:</b>

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s)) <font color=red>Levels 0-4</font>

 VT0:     Zero-bias threshold voltage (default value = 0 Volt) <font color=red>Levels 0,1</font>
            <font color=blue><b>VT0 &lt; 0 for enhancement MOSFET
            VT0 &gt; 0 for depletion MOSFET</b></font>

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt) <font color=red>Levels 0-4</font>

 LD:      Lateral diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 WD:      Width diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 0-4</font>

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>) <font color=red>Levels 0-4</font>

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt) <font color=red>Levels 0-4</font>

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 0-4</font>


<b>Basic Temperature Compensation Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 0-4</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt) <font color=red>Levels 0-4</font>


<b>Level=1 Electrical Parameters:</b>

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PD:      Drain perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PS:      Source perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 JS:      Transport saturation current density (default value = 0 Amp/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-4</font>


<b>Level=1 Parasitic Resistor Parameters:</b>

 RD:      Drain Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>

 RS:      Source Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>


<b>Level=1 Junction Capacitance Parameters:</b>

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 MJ:      Bulk junction grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m) <font color=red>Levels 1-4</font>

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1-4</font>

 CGB0:    Gate-bulk overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGD0:    Gate-drain overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGS0:    Gate-source overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>


<b>Level=1 Temperature Compensation Parameters:</b>

 TRD1:    Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRD2:    Quadratic temperature coefficient of drain resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>

 TRS1:    Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRS2:    Quadratic temperature coefficient of source resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>


<b>Level=2 Process Parameters:</b>

 XJ:      Metallurgical Junction Depth (default value = 0 m) <font color=red>Levels 2-4</font>


<b>Level=2 Electrical Parameters:</b>

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m) <font color=red>Levels 2,4</font>

 UEXP:    Exponential coefficient for mobility (default value = 0) <font color=red>Levels 2,4</font>

 UTRA:    Transverse field coefficient (default value = 0) <font color=red>Levels 2,4</font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,4</font>

 DELTA:   Width effect on threshold voltage (default value = 0) <font color=red>Levels 2-4</font>

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s) <font color=red>Levels 2-4</font>
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation (Levels 2,4)</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation (Levels 2,4)</b></font>

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 2-4</font>

 NEFF:    Total channel charge coefficient (default value = 1) <font color=red>Levels 2,4</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2-4</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>


<b>Level=3 Electrical Parameters:</b>

 KAPPA:   Saturation field factor (default value = 0.2) <font color=red>Level 3</font>

 ETA:     Static feedback on threshold voltage (default value = 0) <font color=red>Level 3</font>

 THETA:   Mobility modulation (default value = 0 1/Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-4</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-4</font>

 GminDC:  Leakage conductance (default value = 1e-12 mho) <font color=red>Levels 0-4</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"),     Icon(
          Text(
            extent=[-78,110; 82,70],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
            Line(points=[0,100; 0,-100],
                                       style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Text(
              extent=[-114,92; 116,-96],
              style(color=0),
            string="PMOS"),
            Line(points=[0,100; 20,80],   style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[-20,80; 0,100], style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[10,50; 108,50], style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[80,40; 110,50], style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[80,60; 110,50], style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Text(
            extent=[-90,-114; -68,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="Heat"),
          Text(
            extent=[2,-114; 24,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Text(
            extent=[-130,20; -108,6],
            style(color=10, rgbcolor={135,135,135}),
            string="G"),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[110,20; 132,6],
            style(color=10, rgbcolor={135,135,135}),
            string="B")));
    public 
      BondLib.Interfaces.BondCon G "Gate" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon D "Drain" 
        annotation (extent=[-10,110; 10,130]);
      BondLib.Interfaces.BondCon B "Bulk" 
        annotation (extent=[110,-10; 130,10]);
      BondLib.Interfaces.BondCon S "Source" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon H "Heat" 
        annotation (extent=[-70,-130; -50,-110]);
      BondLib.Junctions.J0p4 J0p4_2 annotation (extent=[90,-10; 110,10]);
      BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[-10,80; 10,100]);
      BondLib.Bonds.Bond B1 annotation (extent=[-40,-10; -20,10],rotation=180);
      BondLib.Bonds.Bond B2 annotation (extent=[-10,20; 10,40],  rotation=90);
      BondLib.Bonds.Bond B3 annotation (extent=[-10,-40; 10,-20],rotation=90);
      BondLib.Bonds.Bond B4 annotation (extent=[20,-10; 40,10],  rotation=0);
      BondLib.Junctions.J1p3 J1p3_1 
        annotation (extent=[10,40; -10,60],rotation=90);
      BondLib.Bonds.Bond B5 annotation (extent=[-10,60; 10,80],  rotation=90);
      BondLib.Junctions.J1p3 J1p3_2 
        annotation (extent=[10,-60; -10,-40],
                                           rotation=90);
      BondLib.Bonds.Bond B6 annotation (extent=[-10,-80; 10,-60],rotation=90);
      BondLib.Bonds.Bond B7 annotation (extent=[-30,40; -10,60], rotation=-180);
      BondLib.Bonds.Bond B8 annotation (extent=[-30,-60; -10,-40],
                                                                 rotation=-180);
      BondLib.Spice.Utilities.RM Rd(
        Tnom=Tnom,
        R=RD,
        Level=Level,
        TR1=TRD1,
        TR2=TRD2) annotation (extent=[-30,40; -50,60]);
      BondLib.Spice.Utilities.RM Rs(
        Tnom=Tnom,
        Level=Level,
        R=RS,
        TR1=TRS1,
        TR2=TRS2) annotation (extent=[-30,-60; -50,-40]);
      BondLib.Bonds.Bond B9 annotation (extent=[-30,80; -10,100],rotation=180);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[-50,80; -30,100],
                                           rotation=180);
      BondLib.Bonds.Bond B10 
                            annotation (extent=[-70,80; -50,100],rotation=180);
      BondLib.Bonds.Bond B11 
                            annotation (extent=[-50,100; -30,120],
                                                                 rotation=90);
      BondLib.Spice.Utilities.Cgd Cdg(
        Tnom=Tnom,
        Level=Level,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        COX=COXeff,
        NFS=NFS,
        XJ=XJ,
        L=Leff,
        W=Weff,
        LD=LDeff,
        XD=XD,
        XQC=XQC,
        DELTA=DELTA,
        ETA=ETA,
        CGD0=CGD0eff,
        EG=EGeff,
        Type=-1) annotation (extent=[-50,132; -90,108]);
      BondLib.Bonds.Bond B12 
                            annotation (extent=[-100,20; -80,40],rotation=270);
      BondLib.Junctions.J1p3 J1p3_4 
        annotation (extent=[-100,40; -80,60],
                                           rotation=270);
      BondLib.Bonds.Bond B13 
                            annotation (extent=[-100,60; -80,80],rotation=270);
      BondLib.Bonds.Bond B14 
                            annotation (extent=[-120,40; -100,60],
                                                                 rotation=-180);
      BondLib.Spice.Utilities.Cgb Cbg(
        Tnom=Tnom,
        Level=Level,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        COX=COXeff,
        NFS=NFS,
        XJ=XJ,
        L=Leff,
        W=Weff,
        LD=LDeff,
        XD=XD,
        XQC=XQC,
        DELTA=DELTA,
        ETA=ETA,
        CGB0=CGB0eff,
        EG=EGeff,
        Type=-1) annotation (extent=[-108,54; -132,94], rotation=90);
      BondLib.Junctions.J0p5 J0p5_1 
        annotation (extent=[-10,-100; 10,-80], rotation=90);
      BondLib.Bonds.Bond B15 
                            annotation (extent=[20,-90; 40,-70], rotation=0);
      BondLib.Junctions.J1p3 J1p3_5 
        annotation (extent=[40,-90; 60,-70],
                                           rotation=180);
      BondLib.Bonds.Bond B16 
                            annotation (extent=[60,-90; 80,-70], rotation=0);
      BondLib.Bonds.fBond B17 
        annotation (extent=[40,-70; 60,-50], rotation=90);
      BondLib.Sensors.De Vsb annotation (extent=[40,-50; 60,-30], rotation=90);
      BondLib.Bonds.Bond B18 
                            annotation (extent=[10,80; 30,100],  rotation=180);
      BondLib.Junctions.J1p3 J1p3_6 
        annotation (extent=[30,100; 50,80],rotation=180);
      BondLib.Bonds.Bond B19 
                            annotation (extent=[50,80; 70,100],  rotation=180);
      BondLib.Bonds.fBond B20 
                             annotation (extent=[30,60; 50,80], rotation=-90);
      BondLib.Sensors.De Vsd 
        annotation (extent=[30,40; 50,60],         rotation=-90);
      BondLib.Junctions.J0p5 J0p5_2 
        annotation (extent=[-96,10; -76,-10],  rotation=0);
      BondLib.Bonds.Bond B21 
                            annotation (extent=[-96,-30; -76,-10],
                                                                 rotation=90);
      BondLib.Junctions.J1p3 J1p3_7 
        annotation (extent=[-76,-50; -96,-30],
                                           rotation=90);
      BondLib.Bonds.Bond B22 
                            annotation (extent=[-96,-70; -76,-50],
                                                                 rotation=90);
      BondLib.Bonds.Bond B23 
                            annotation (extent=[-116,-50; -96,-30],
                                                                 rotation=-180);
      BondLib.Spice.Utilities.Cgs Csg(
        Tnom=Tnom,
        Level=Level,
        Type=-1,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        COX=COXeff,
        NFS=NFS,
        XJ=XJ,
        L=Leff,
        W=Weff,
        LD=LDeff,
        XD=XD,
        XQC=XQC,
        DELTA=DELTA,
        ETA=ETA,
        CGS0=CGS0eff,
        EG=EGeff) annotation (extent=[-132,-90; -108,-50], rotation=-90);
      BondLib.Junctions.J0p6 J0p6_1 
        annotation (extent=[-70,-106; -50,-86],  rotation=90);
      BondLib.Bonds.eBond B24 
        annotation (extent=[-130,-110; -110,-90],
                                                rotation=-90);
      BondLib.Bonds.eBond B25 
        annotation (extent=[-70,40; -50,60],    rotation=-180);
      BondLib.Bonds.eBond B26 
        annotation (extent=[-70,-60; -50,-40],  rotation=-180);
      BondLib.Bonds.eBond B27 
        annotation (extent=[-30,-30; -10,-10],  rotation=-180);
      BondLib.Bonds.eBond B28 
        annotation (extent=[-110,110; -90,130], rotation=-180);
      BondLib.Bonds.eBond B29 
        annotation (extent=[-130,94; -110,114], rotation=-270);
      BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[-50,-30; -30,-10]);
      BondLib.Bonds.eBond B30 
        annotation (extent=[-64,-46; -44,-26],  rotation=-180);
      BondLib.Spice.Utilities.PMOSint PMOSint(
        Level=Level,
        Type=-1,
        Tnom=Tnom,
        VT0=VT0eff,
        VFB=VFBeff,
        KP=KPeff,
        GAMMA=GAMMAeff,
        PHI=PHIeff,
        LAMBDA=LAMBDAeff,
        TOX=TOXeff,
        COX=COXeff,
        NSUB=NSUBeff,
        NSS=NSS,
        NFS=NFS,
        NEFF=NEFF,
        L=Leff,
        W=Weff,
        XJ=XJ,
        LD=LDeff,
        AD=AD,
        PD=PD,
        AS=AS,
        PS=PS,
        XD=XD,
        TPG=TPG,
        U0=U0eff,
        UCRIT=UCRIT,
        UEXP=UEXP,
        UTRA=UTRA,
        VMAX=VMAX,
        ECRIT=ECRIT,
        XQC=XQC,
        DELTA=DELTA,
        KAPPA=KAPPA,
        ETA=ETA,
        THETA=THETA,
        VBP=VBP,
        ISD=ISDeff,
        ISS=ISSeff,
        PB=PB,
        CJ=CJeff,
        MJ=MJ,
        CJSW=CJSW,
        MJSW=MJSW,
        FC=FC,
        GminDC=GminDC,
        EG=EGeff,
        EMin=EMin,
        EMax=EMax) annotation (extent=[-20,-20; 20,20]);
      algorithm 
        assert(NSUBeff > XNI, "NSUB must be larger than intrinsic carrier concentration");
        assert(LAMBDAeff < 0.2, "LAMBDA value too large, leads to numerical instability");
        assert(Leff > 0, "The effective channel length must be larger than zero");
      
      equation 
      connect(J0p4_2.BondCon2, B)        annotation (points=[110,0; 120,0], style(color=8,
            rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon4, D)        annotation (points=[0,100; 0,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(B7.BondCon1, J1p3_1.BondCon3) annotation (points=[-10.2,50; -10,
            50], style(color=8, rgbcolor={192,192,192}));
      connect(B8.BondCon1, J1p3_2.BondCon3) annotation (points=[-10.2,-50; -10,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(Rd.BondCon1, B7.BondCon2) annotation (points=[-30,50; -30,50],
          style(color=8, rgbcolor={192,192,192}));
      connect(Rs.BondCon1, B8.BondCon2) annotation (points=[-30,-50; -30,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B11.BondCon1, J1p3_3.BondCon3) annotation (points=[-40,100.2; -40,
            100], style(color=8, rgbcolor={192,192,192}));
      connect(Cdg.BondCon1, B11.BondCon2) annotation (points=[-50,120; -40,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(B14.BondCon1, J1p3_4.BondCon3) annotation (points=[-100.2,50;
            -100,50], style(color=8, rgbcolor={192,192,192}));
      connect(B14.BondCon2, Cbg.BondCon1) annotation (points=[-120,50; -120,54],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_1.BondCon1, S)        annotation (points=[-6.12303e-016,-100;
            0,-100; 0,-120], style(color=8, rgbcolor={192,192,192}));
      connect(B17.eBondCon1, J1p3_5.BondCon3) annotation (points=[50,-70; 50,
            -70], style(color=8, rgbcolor={192,192,192}));
      connect(Vsb.BondCon1, B17.fBondCon1) annotation (points=[50,-50; 50,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_6.BondCon3, B20.eBondCon1) annotation (points=[40,80; 40,80],
          style(color=8, rgbcolor={192,192,192}));
      connect(B20.fBondCon1, Vsd.BondCon1) annotation (points=[40,60; 40,60],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_2.BondCon1, G)        annotation (points=[-96,0; -120,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B23.BondCon1, J1p3_7.BondCon3) annotation (points=[-96.2,-40; -96,
            -40], style(color=8, rgbcolor={192,192,192}));
      connect(Csg.BondCon1, B23.BondCon2) annotation (points=[-120,-50; -120,
            -40; -116,-40], style(color=8, rgbcolor={192,192,192}));
      connect(Vsd.OutPort1, Cdg.u1) annotation (points=[40,39; 40,30; 80,30; 80,
            138; -58,138; -58,132], style(color=74, rgbcolor={0,0,127}));
      connect(Vsb.OutPort1, Cdg.u2) annotation (points=[50,-29; 50,28; 82,28;
            82,140; -82,140; -82,132], style(color=74, rgbcolor={0,0,127}));
      connect(Cbg.u2, Cdg.u2) annotation (points=[-132,86; -140,86; -140,140;
            -82,140; -82,132], style(color=74, rgbcolor={0,0,127}));
      connect(Cbg.u1, Cdg.u1) annotation (points=[-132,62; -138,62; -138,138;
            -58,138; -58,132], style(color=74, rgbcolor={0,0,127}));
      connect(Csg.u1, Cbg.u1) annotation (points=[-132,-58; -138,-58; -138,62;
            -132,62], style(color=74, rgbcolor={0,0,127}));
      connect(Csg.u2, Cbg.u2) annotation (points=[-132,-82; -140,-82; -140,86;
            -132,86], style(color=74, rgbcolor={0,0,127}));
      connect(Cbg.u, Cdg.u) annotation (points=[-108,74; -104,74; -104,98; -70,
            98; -70,108], style(color=74, rgbcolor={0,0,127}));
      connect(Cbg.u, Csg.u) annotation (points=[-108,74; -104,74; -104,-70;
            -108,-70], style(color=74, rgbcolor={0,0,127}));
      connect(J0p6_1.BondCon1, H)        annotation (points=[-60,-106; -60,-120],
          style(color=8, rgbcolor={192,192,192}));
      connect(Csg.BondCon2, B24.fBondCon1) annotation (points=[-120,-90; -120,
            -90], style(color=8, rgbcolor={192,192,192}));
      connect(Rd.BondCon2, B25.fBondCon1) annotation (points=[-50,50; -50,50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B26.fBondCon1, Rs.BondCon2) annotation (points=[-50,-50; -50,-50],
          style(color=8, rgbcolor={192,192,192}));
      connect(B28.fBondCon1, Cdg.BondCon2) annotation (points=[-90,120; -90,120],
          style(color=8, rgbcolor={192,192,192}));
      connect(B29.fBondCon1, Cbg.BondCon2) annotation (points=[-120,94; -120,94],
          style(color=8, rgbcolor={192,192,192}));
      connect(B26.eBondCon1, J0p6_1.BondCon3) annotation (points=[-70,-50; -74,
            -50; -74,-66; -40,-66; -40,-100; -50,-100], style(color=8, rgbcolor=
             {192,192,192}));
      connect(B25.eBondCon1, J0p6_1.BondCon4) annotation (points=[-70,50; -74,
            50; -74,10; -68,10; -68,-38; -76,-38; -76,-68; -50,-68; -50,-92],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2, B27.eBondCon1) annotation (points=[-30,-20; -30,
            -20], style(color=8, rgbcolor={192,192,192}));
      connect(B30.fBondCon1, J0p3_1.BondCon3) annotation (points=[-44,-36; -40,
            -36; -40,-30], style(color=8, rgbcolor={192,192,192}));
      connect(B30.eBondCon1, J0p6_1.BondCon2) annotation (points=[-64,-36; -78,
            -36; -78,-70; -60,-70; -60,-86], style(color=8, rgbcolor={192,192,
              192}));
      connect(B24.eBondCon1, J0p3_1.BondCon1) annotation (points=[-120,-110;
            -120,-114; -102,-114; -102,-20; -50,-20], style(color=8, rgbcolor={
              192,192,192}));
      connect(J0p6_1.BondCon6, B29.eBondCon1) annotation (points=[-70,-92; -100,
            -92; -100,-116; -142,-116; -142,118; -120,118; -120,114], style(
            color=8, rgbcolor={192,192,192}));
      connect(J0p6_1.BondCon5, B28.eBondCon1) annotation (points=[-70,-100; -98,
            -100; -98,-118; -144,-118; -144,120; -110,120], style(color=8,
            rgbcolor={192,192,192}));
      connect(B2.BondCon2, J1p3_1.BondCon1) annotation (points=[6.12303e-016,40;
            6.12303e-016,40], style(color=8, rgbcolor={192,192,192}));
      connect(B4.BondCon2, J0p4_2.BondCon1) annotation (points=[40,0; 90,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon2, J0p5_2.BondCon2) annotation (points=[-40,
            1.22461e-015; -66,1.22461e-015; -66,0; -76,0], style(color=8,
            rgbcolor={192,192,192}));
      connect(J0p4_1.BondCon3, B5.BondCon2) annotation (points=[0,80;
            6.12303e-016,80], style(color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon1, J1p3_1.BondCon2) annotation (points=[-6.00057e-016,
            60.2; -6.00057e-016,63.1; -6.12303e-016,63.1; -6.12303e-016,60],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon1, B6.BondCon2) annotation (points=[6.12303e-016,
            -60; 6.12303e-016,-60], style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_1.BondCon2, B6.BondCon1) annotation (points=[6.12303e-016,
            -80; -6.00057e-016,-80; -6.00057e-016,-79.8], style(color=8,
            rgbcolor={192,192,192}));
      connect(B15.BondCon2, J1p3_5.BondCon2) annotation (points=[40,-80; 40,-80],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_1.BondCon4, B15.BondCon1) annotation (points=[10,-86; 10,-80;
            20.2,-80], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_5.BondCon1, B16.BondCon1) annotation (points=[60,-80; 60.2,
            -80], style(color=8, rgbcolor={192,192,192}));
      connect(B16.BondCon2, J0p4_2.BondCon3) annotation (points=[80,-80; 100,
            -80; 100,-10], style(color=8, rgbcolor={192,192,192}));
      connect(B18.BondCon2, J0p4_1.BondCon2) annotation (points=[10,90; 10,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(B18.BondCon1, J1p3_6.BondCon2) annotation (points=[29.8,90; 30,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_6.BondCon1, B19.BondCon2) annotation (points=[50,90; 50,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(B19.BondCon1, J0p5_1.BondCon3) annotation (points=[69.8,90; 130,
            90; 130,-94; 10,-94], style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon2, J1p3_3.BondCon1) annotation (points=[-30,90; -30,90],
          style(color=8, rgbcolor={192,192,192}));
      connect(B9.BondCon1, J0p4_1.BondCon1) annotation (points=[-10.2,90; -10,
            90], style(color=8, rgbcolor={192,192,192}));
      connect(B10.BondCon1, J1p3_3.BondCon2) annotation (points=[-50.2,90; -50,
            90], style(color=8, rgbcolor={192,192,192}));
      connect(B10.BondCon2, J0p5_2.BondCon4) annotation (points=[-70,90; -76,90;
            -76,10; -82,10], style(color=8, rgbcolor={192,192,192}));
      connect(B12.BondCon1, J1p3_4.BondCon2) annotation (points=[-90,39.8; -90,
            40], style(color=8, rgbcolor={192,192,192}));
      connect(B12.BondCon2, J0p5_2.BondCon3) annotation (points=[-90,20; -90,10],
          style(color=8, rgbcolor={192,192,192}));
      connect(B13.BondCon2, J1p3_4.BondCon1) annotation (points=[-90,60; -90,60],
          style(color=8, rgbcolor={192,192,192}));
      connect(B13.BondCon1, J0p4_2.BondCon4) annotation (points=[-90,79.8; -90,
            104; 100,104; 100,10], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon2, J1p3_7.BondCon1) annotation (points=[-86,-50; -86,
            -50], style(color=8, rgbcolor={192,192,192}));
      connect(B22.BondCon1, J0p5_1.BondCon5) annotation (points=[-86,-69.8; -86,
            -76; -10,-76; -10,-90], style(color=8, rgbcolor={192,192,192}));
      connect(J0p5_2.BondCon5, B21.BondCon2) annotation (points=[-86,-10; -86,
            -10], style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_7.BondCon2, B21.BondCon1) annotation (points=[-86,-30; -86,
            -29.8], style(color=8, rgbcolor={192,192,192}));
      connect(PMOSint.y1, Cdg.u)  annotation (points=[-10,20; -10,32; -62,32;
            -62,98; -70,98; -70,108], style(color=74, rgbcolor={0,0,127}));
      connect(B1.BondCon1, PMOSint.Gint) annotation (points=[-20.2,-1.20011e-015;
            -20.2,0; -20,0], style(color=8, rgbcolor={192,192,192}));
      connect(PMOSint.H, B27.fBondCon1) annotation (points=[-10,-20; -10,-20],
          style(color=8, rgbcolor={192,192,192}));
      connect(B3.BondCon2, PMOSint.Sint) annotation (points=[6.12303e-016,-20;
            0,-20], style(color=8, rgbcolor={192,192,192}));
      connect(B3.BondCon1, J1p3_2.BondCon2) annotation (points=[-6.00057e-016,
            -39.8; 0,-39.8; 0,-40; -6.12303e-016,-40], style(color=8, rgbcolor=
              {192,192,192}));
      connect(PMOSint.Bint, B4.BondCon1) annotation (points=[20,0; 20.2,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B2.BondCon1, PMOSint.Dint) annotation (points=[-6.00057e-016,20.2;
            0,20.2; 0,20], style(color=8, rgbcolor={192,192,192}));
      end PMOS;
    
      model NFET "Spice-style n-channel JFET in bond graph technology" 
      
        /* General parameters */
        parameter Integer Level(min=1,max=3) = 1 
        "JFET modeling level (check documentation layer for details)";
        parameter Real Area=1 "Relative area occupied by capacitor";
      
        /* Basic Electrical parameters */
        parameter Modelica.SIunits.Voltage VT0=0 
        "|Electrical parameters|Threshold voltage at reference temperature";
        parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
        "|Electrical parameters|Channel length modulation";
        parameter Modelica.SIunits.Transconductance BETA=0 
        "|Electrical parameters|Transconductance parameter at reference temperature";
        parameter Real N=1 
        "|Electrical parameters|Current emission coefficient";
        parameter Modelica.SIunits.Current IS(min=Modelica.Constants.small) = 1.e-14 
        "|Electrical parameters|Transport saturation current at reference temperature";
        parameter Modelica.SIunits.Voltage PB=0.8 
        "|Electrical parameters|Built-in potential at reference temperature";
      
        /* Parasitic resistor parameters */
        parameter Modelica.SIunits.Resistance RS(min=0) = 0 
        "|Parasitic resistor parameters|Source resistance";
        parameter Modelica.SIunits.Resistance RD(min=0) = 0 
        "|Parasitic resistor parameters|Drain resistance";
        parameter Modelica.SIunits.Resistance RG(min=0) = 0 
        "|Parasitic resistor parameters|Gate resistance";
      
        /* Junction capacitor parameters */
        parameter Modelica.SIunits.Capacitance CGS(min=Modelica.Constants.small) = 0.5e-12 
        "|Junction capacitor parameters|Zero-bias gate-source depletion capacitance at reference temperature";
        parameter Modelica.SIunits.Capacitance CGD(min=Modelica.Constants.small) = 0.5e-12 
        "|Junction capacitor parameters|Zero-bias gate-drain depletion capacitance at reference temperature";
        parameter Real M(min=0.1,max=0.9) = if Level==3 then 0.5 else 0.33 
        "|Junction capacitor parameters|Junction grading coefficient";
        parameter Real FC(min=0,max=0.95)= 0.5 
        "|Junction capacitor parameters|Factor for linearization of depletion capacitance formula";
      
        /* Transit time parameters */
        parameter Modelica.SIunits.Time TT(min=0) = 5e-9 
        "|Transit time parameters|Ideal transit time";
      
        /* Temperature compensation parameters */
        parameter Real TCV(unit="V/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of threshold voltage";
        parameter Real TRS(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of source resistance";
        parameter Real TRD(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
        parameter Real TRG(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of gate resistance";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Real XTI = if Level==3 then 2 else 3 
        "|Temperature compensation parameters|Saturation current temperature exponent";
        parameter Real BEX=0 
        "|Temperature compensation parameters|Mobility temperature exponent";
        parameter Modelica.SIunits.Voltage EG(min=0.1) = if Level==3 then 0.73 else 1.16 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
      
        /* Level=2 electrical parameters */
        parameter Modelica.SIunits.InversePotential LAM1=0 
        "|Level=2|Electrical parameters|Channel length modulation gate voltage parameter";
      
        /* Level=3 electrical parameters */
        parameter Modelica.SIunits.InversePotential ALPHA=1 
        "|Level=3|Electrical parameters|Hyperbolic tangient fitting parameter";
        parameter Modelica.SIunits.InversePotential B=0 
        "|Level=3|Electrical parameters|Measure of doping profile";
        parameter Modelica.SIunits.Voltage DELTA=0.2 
        "|Level=3|Electrical parameters|Voltage range for transition";
        parameter Modelica.SIunits.Voltage VMAX=0.5 
        "|Level=3|Electrical parameters|Limit voltage for carrier velocity saturation";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC=1e-16 
        "|Advanced|Numerical parameters|Leakage conductance";
      Junctions.J0p2 J0p2_1 annotation (extent=[-100,-10; -80,10]);
      BondLib.Interfaces.BondCon G "Gate" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon H "Heat" annotation (extent=[110,-10; 130,10]);
      BondLib.Interfaces.BondCon S "Source" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon D "Drain" 
                                          annotation (extent=[-10,110; 10,130]);
      annotation (
        Coordsys(extent=[-120,-120; 120,120]),
        Diagram(
          Text(
            extent=[-130,24; -108,10],
            style(color=10, rgbcolor={135,135,135}),
            string="G"),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[4,-114; 26,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Text(
            extent=[108,-8; 132,-26],
            style(
              color=10,
              rgbcolor={135,135,135},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1),
            string="Heat")),
        Icon(
          Text(
            extent=[-130,24; -108,10],
            style(color=10, rgbcolor={135,135,135}),
            string="G"),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[4,-114; 26,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Text(
            extent=[108,-6; 132,-24],
            style(
              color=10,
              rgbcolor={135,135,135},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1),
            string="Heat"),
          Text(
            extent=[-74,120; 86,80],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
            Line(points=[0,-100; 20,-80],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[-20,-80; 0,-100],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,78; 0,-100],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Line(points=[-100,-60; 100,-60; 80,-50], style(color=3, rgbcolor={0,0,
                  255})),
            Text(
              extent=[-110,92; 120,-96],
              style(color=0),
            string="NFET"),
          Line(points=[100,-60; 80,-70], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1))),
        Documentation(info="<html>
The Spice-style n-channel JFET model contains an internal n-channel JFET together with the three external resistances. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Area:    Relative area occupied by the diode (default value = 1) <font color=red>Levels 1-3</font>


<b>Basic Electrical Parameters:</b>

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt) <font color=red>Levels 1-3</font>
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 LAMBDA:  Channel length modulation (default value = 0 1/Volt) <font color=red>Levels 1-3</font>

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 1-3</font>

 N:       Current emission coefficient (default value = 1) <font color=red>Levels 1-3</font>

 IS:      Transport saturation current (default value = 1e-14 Amp) <font color=red>Levels 1-3</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-3</font>


<b>Parasitic Resistor Parameters:</b>

 RS:      Source resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RD:      Drain resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RG:      Gate resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>


<b>Junction Capacitance Parameters:</b>

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 M:       Junction grading coefficient <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1,2</font>


<b>Transit Time Parameters:</b>

 TT:      Ideal transit time (default value = 5e-9 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation Parameters:</b>

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K) <font color=red>Levels 1-3</font>

 TRS:     Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRD:     Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRG:     Linear temperature coefficient of gate resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1-3</font>

 XTI:     Saturation current temperature exponent <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 BEX:     Mobility temperature exponent (default value = 0) <font color=red>Levels 1-3</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>


<b>Level=2 Electrical Parameters:</b>

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt) <font color=red>Level 2</font>


<b>Level=3 Electrical Parameters:</b>

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1) <font color=red>Level 3</font>

 B:       Measure of doping profile (default value = 0 1/Volt) <font color=red>Level 3</font>

 DELTA:   Voltage range for transition (default value = 0.2 Volt) <font color=red>Level 3</font>

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-3</font>

 GminDC:  Leakage conductance (default value = 1e-15 Mho) <font color=red>Levels 1-3</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
      BondLib.Bonds.Bond B1 annotation (extent=[-80,-10; -60,10]);
      BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-60,10; -40,-10]);
      BondLib.Bonds.Bond B2 annotation (extent=[-40,-10; -20,10]);
      BondLib.Spice.Utilities.NFETint NFETint1(
        Level=Level,
        Tnom=Tnom,
        VT0=VT0,
        IS=IS,
        BETA=BETA,
        LAMBDA=LAMBDA,
        TCV=TCV,
        EG=EG,
        N=N,
        XTI=XTI,
        PB=PB,
        CGS=CGS,
        CGD=CGD,
        M=M,
        FC=FC,
        TT=TT,
        BEX=BEX,
        LAM1=LAM1,
        ALPHA=ALPHA,
        B=B,
        DELTA=DELTA,
        VMAX=VMAX,
        GminDC=GminDC,
        Area=Area,
        EMin=EMin,
        EMax=EMax) annotation (extent=[-20,-20; 20,20]);
      BondLib.Bonds.eBond B3 annotation (extent=[20,-10; 40,10]);
      BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[80,10; 100,-10]);
      Junctions.J0p2 J0p2_2 annotation (extent=[-10,80; 10,100], rotation=90);
      BondLib.Bonds.Bond B4 annotation (extent=[-10,60; 10,80], rotation=-90);
      BondLib.Junctions.J1p3 J1p3_2 
        annotation (extent=[10,60; -10,40], rotation=-90);
      BondLib.Bonds.Bond B5 annotation (extent=[-10,20; 10,40], rotation=-90);
      BondLib.Bonds.Bond B6 annotation (extent=[-10,-40; 10,-20], rotation=-90);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[10,-40; -10,-60], rotation=-90);
      BondLib.Bonds.Bond B7 annotation (extent=[-10,-80; 10,-60], rotation=-90);
      Junctions.J0p2 J0p2_3 annotation (extent=[-10,-100; 10,-80], rotation=90);
      BondLib.Bonds.Bond B8 annotation (extent=[-60,10; -40,30], rotation=90);
      BondLib.Bonds.Bond B9 annotation (extent=[10,40; 30,60]);
      BondLib.Bonds.Bond B10 annotation (extent=[10,-60; 30,-40]);
      BondLib.Spice.RS Rd(
        Tnom=Tnom,
        R=RD,
        TR1=TRD,
        TR2=0,
        Area=Area) annotation (extent=[30,36; 70,64]);
      BondLib.Spice.RS Rs(
        Tnom=Tnom,
        TR2=0,
        Area=Area,
        R=RS,
        TR1=TRS) annotation (extent=[30,-64; 70,-36]);
      BondLib.Spice.RS Rg(
        Tnom=Tnom,
        TR2=0,
        Area=Area,
        R=RG,
        TR1=TRG) annotation (extent=[-64,30; -36,70], rotation=90);
      BondLib.Bonds.eBond B11 annotation (extent=[-60,70; -40,90], rotation=90);
      BondLib.Bonds.eBond B12 annotation (extent=[70,40; 90,60]);
      BondLib.Bonds.eBond B13 annotation (extent=[70,-60; 90,-40]);
      equation 
      connect(J0p2_1.BondCon1, G)        annotation (points=[-100,0; -120,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p2_1.BondCon2, B1.BondCon1) annotation (points=[-80,0; -79.8,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-60,0; -60,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[-40,0; -39.8,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p5_1.BondCon2, H)        annotation (points=[100,0; 120,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B3.eBondCon1, J0p5_1.BondCon1) annotation (points=[40,0; 80,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(D, J0p2_2.BondCon2)        annotation (points=[0,120; 0,100;
            6.12303e-016,100], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p2_2.BondCon1, B4.BondCon1) annotation (points=[-6.12303e-016,
            80; -6.00057e-016,80; -6.00057e-016,79.8], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B4.BondCon2, J1p3_2.BondCon2) annotation (points=[6.12303e-016,60;
            -6.12303e-016,60], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_2.BondCon1, B5.BondCon1) annotation (points=[6.12303e-016,40;
            -6.00057e-016,40; -6.00057e-016,39.8], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_3.BondCon1, B7.BondCon1) annotation (points=[6.12303e-016,
            -60; -6.00057e-016,-60; -6.00057e-016,-60.2], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B7.BondCon2, J0p2_3.BondCon2) annotation (points=[6.12303e-016,
            -80; 6.12303e-016,-80], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p2_3.BondCon1, S)        annotation (points=[-6.12303e-016,-100;
            -6.12303e-016,-110; 0,-110; 0,-120], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B8.BondCon1, J1p3_1.BondCon3) annotation (points=[-50,10.2; -50,
            10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_2.BondCon3, B9.BondCon1) annotation (points=[10,50; 10.2,50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_3.BondCon3, B10.BondCon1) annotation (points=[10,-50; 10.2,
            -50], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B9.BondCon2, Rd.BondCon1) annotation (points=[30,50; 30,50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B10.BondCon2, Rs.BondCon1) annotation (points=[30,-50; 30,-50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(Rg.BondCon1, B8.BondCon2) annotation (points=[-50,30; -50,30],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B11.fBondCon1, Rg.BondCon2) annotation (points=[-50,70; -50,70],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(Rd.BondCon2, B12.fBondCon1) annotation (points=[70,50; 70,50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(Rs.BondCon2, B13.fBondCon1) annotation (points=[70,-50; 70,-50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B13.eBondCon1, J0p5_1.BondCon5) annotation (points=[90,-50; 100,
            -50; 100,-20; 90,-20; 90,-10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B12.eBondCon1, J0p5_1.BondCon3) annotation (points=[90,50; 100,50;
            100,30; 86,30; 86,10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B11.eBondCon1, J0p5_1.BondCon4) annotation (points=[-50,90; -50,
            106; 110,106; 110,20; 94,20; 94,10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(NFETint1.Gint, B2.BondCon2) annotation (points=[-20,0; -20,0],
          style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon2, J1p3_3.BondCon2) annotation (points=[6.12303e-016,
            -40; -6.12303e-016,-40], style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon1, NFETint1.Sint) annotation (points=[-6.00057e-016,
            -20.2; 0,-20.2; 0,-20], style(color=8, rgbcolor={192,192,192}));
      connect(B3.fBondCon1, NFETint1.H) annotation (points=[20,0; 20,0], style(
            color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon2, NFETint1.Dint) annotation (points=[6.12303e-016,20;
            0,20], style(color=8, rgbcolor={192,192,192}));
      end NFET;
    
      model PFET "Spice-style p-channel JFET in bond graph technology" 
      
        /* General parameters */
        parameter Integer Level(min=1,max=3) = 1 
        "JFET modeling level (check documentation layer for details)";
        parameter Real Area=1 "Relative area occupied by capacitor";
      
        /* Basic Electrical parameters */
        parameter Modelica.SIunits.Voltage VT0=0 
        "|Electrical parameters|Threshold voltage at reference temperature";
        parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
        "|Electrical parameters|Channel length modulation";
        parameter Modelica.SIunits.Transconductance BETA=0 
        "|Electrical parameters|Transconductance parameter at reference temperature";
        parameter Real N=1 
        "|Electrical parameters|Current emission coefficient";
        parameter Modelica.SIunits.Current IS(min=Modelica.Constants.small) = 1.e-14 
        "|Electrical parameters|Transport saturation current at reference temperature";
        parameter Modelica.SIunits.Voltage PB=0.8 
        "|Electrical parameters|Built-in potential at reference temperature";
      
        /* Parasitic resistor parameters */
        parameter Modelica.SIunits.Resistance RS(min=0) = 0 
        "|Parasitic resistor parameters|Source resistance";
        parameter Modelica.SIunits.Resistance RD(min=0) = 0 
        "|Parasitic resistor parameters|Drain resistance";
        parameter Modelica.SIunits.Resistance RG(min=0) = 0 
        "|Parasitic resistor parameters|Gate resistance";
      
        /* Junction capacitor parameters */
        parameter Modelica.SIunits.Capacitance CGS(min=Modelica.Constants.small) = 0.5e-12 
        "|Junction capacitor parameters|Zero-bias gate-source depletion capacitance at reference temperature";
        parameter Modelica.SIunits.Capacitance CGD(min=Modelica.Constants.small) = 0.5e-12 
        "|Junction capacitor parameters|Zero-bias gate-drain depletion capacitance at reference temperature";
        parameter Real M(min=0.1,max=0.9) = if Level==3 then 0.5 else 0.33 
        "|Junction capacitor parameters|Junction grading coefficient";
        parameter Real FC(min=0,max=0.95)= 0.5 
        "|Junction capacitor parameters|Factor for linearization of depletion capacitance formula";
      
        /* Transit time parameters */
        parameter Modelica.SIunits.Time TT(min=0) = 5e-9 
        "|Transit time parameters|Ideal transit time";
      
        /* Temperature compensation parameters */
        parameter Real TCV(unit="V/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of threshold voltage";
        parameter Real TRS(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of source resistance";
        parameter Real TRD(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
        parameter Real TRG(unit="Ohm/K") = 0 
        "|Temperature compensation parameters|Linear temperature coefficient of gate resistance";
        parameter Modelica.SIunits.Temperature Tnom=300.15 
        "|Temperature compensation parameters|Reference temperature";
        parameter Real XTI = if Level==3 then 2 else 3 
        "|Temperature compensation parameters|Saturation current temperature exponent";
        parameter Real BEX=0 
        "|Temperature compensation parameters|Mobility temperature exponent";
        parameter Modelica.SIunits.Voltage EG(min=0.1) = if Level==3 then 0.73 else 1.16 
        "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
      
        /* Level=2 electrical parameters */
        parameter Modelica.SIunits.InversePotential LAM1=0 
        "|Level=2|Electrical parameters|Channel length modulation gate voltage parameter";
      
        /* Level=3 electrical parameters */
        parameter Modelica.SIunits.InversePotential ALPHA=1 
        "|Level=3|Electrical parameters|Hyperbolic tangient fitting parameter";
        parameter Modelica.SIunits.InversePotential B=0 
        "|Level=3|Electrical parameters|Measure of doping profile";
        parameter Modelica.SIunits.Voltage DELTA=0.2 
        "|Level=3|Electrical parameters|Voltage range for transition";
        parameter Modelica.SIunits.Voltage VMAX=0.5 
        "|Level=3|Electrical parameters|Limit voltage for carrier velocity saturation";
      
        /* Numerical parameters */
        parameter Real EMin=-100 
        "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
        parameter Real EMax=40 
        "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
        parameter Modelica.SIunits.Conductance GminDC=1e-16 
        "|Advanced|Numerical parameters|Leakage conductance";
      annotation (
        Coordsys(extent=[-120,-120; 120,120]),
        Diagram(
          Text(
            extent=[-130,24; -108,10],
            style(color=10, rgbcolor={135,135,135}),
            string="G"),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[4,-114; 26,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Text(
            extent=[108,-8; 132,-26],
            style(
              color=10,
              rgbcolor={135,135,135},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1),
            string="Heat")),
        Icon(
            Line(points=[-20,60; 0,80],  style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,80; 20,60],    style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
            Line(points=[0,78; 0,-100],style(
                color=9,
                rgbcolor={149,164,171},
                thickness=2,
                fillColor=74,
                rgbfillColor={0,0,185},
                fillPattern=1)),
          Line(points=[-100,-60; 100,-60; 80,-50], style(color=3, rgbcolor={0,0,
                  255})),
            Text(
              extent=[-112,92; 118,-96],
              style(color=0),
            string="PFET"),
          Text(
            extent=[108,-6; 132,-24],
            style(
              color=10,
              rgbcolor={135,135,135},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1),
            string="Heat"),
          Text(
            extent=[-74,120; 86,80],
            style(color=3, rgbcolor={0,0,255}),
            string="%name"),
          Text(
            extent=[4,126; 26,112],
            style(color=10, rgbcolor={135,135,135}),
            string="D"),
          Text(
            extent=[-130,24; -108,10],
            style(color=10, rgbcolor={135,135,135}),
            string="G"),
          Text(
            extent=[4,-114; 26,-128],
            style(color=10, rgbcolor={135,135,135}),
            string="S"),
          Line(points=[100,-60; 80,-70], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1))),
        Documentation(info="<html>
The Spice-style p-channel JFET model contains an internal p-channel JFET together with the three external resistances. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Area:    Relative area occupied by the diode (default value = 1) <font color=red>Levels 1-3</font>


<b>Basic Electrical Parameters:</b>

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt) <font color=red>Levels 1-3</font>
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 LAMBDA:  Channel length modulation (default value = 0 1/Volt) <font color=red>Levels 1-3</font>

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 1-3</font>

 N:       Current emission coefficient (default value = 1) <font color=red>Levels 1-3</font>

 IS:      Transport saturation current (default value = 1e-14 Amp) <font color=red>Levels 1-3</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-3</font>


<b>Parasitic Resistor Parameters:</b>

 RS:      Source resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RD:      Drain resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RG:      Gate resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>


<b>Junction Capacitance Parameters:</b>

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 M:       Junction grading coefficient <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1,2</font>


<b>Transit Time Parameters:</b>

 TT:      Ideal transit time (default value = 5e-9 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation Parameters:</b>

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K) <font color=red>Levels 1-3</font>

 TRS:     Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRD:     Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRG:     Linear temperature coefficient of gate resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1-3</font>

 XTI:     Saturation current temperature exponent <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 BEX:     Mobility temperature exponent (default value = 0) <font color=red>Levels 1-3</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>


<b>Level=2 Electrical Parameters:</b>

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt) <font color=red>Level 2</font>


<b>Level=3 Electrical Parameters:</b>

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1) <font color=red>Level 3</font>

 B:       Measure of doping profile (default value = 0 1/Volt) <font color=red>Level 3</font>

 DELTA:   Voltage range for transition (default value = 0.2 Volt) <font color=red>Level 3</font>

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-3</font>

 GminDC:  Leakage conductance (default value = 1e-15 Mho) <font color=red>Levels 1-3</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
      Junctions.J0p2 J0p2_1 annotation (extent=[-100,-10; -80,10]);
      BondLib.Interfaces.BondCon G "Gate" 
        annotation (extent=[-130,-10; -110,10]);
      BondLib.Interfaces.BondCon H "Heat" annotation (extent=[110,-10; 130,10]);
      BondLib.Interfaces.BondCon S "Source" 
        annotation (extent=[-10,-130; 10,-110]);
      BondLib.Interfaces.BondCon D "Drain" 
                                          annotation (extent=[-10,110; 10,130]);
      BondLib.Bonds.Bond B1 annotation (extent=[-80,-10; -60,10], rotation=180);
      BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-60,10; -40,-10]);
      BondLib.Bonds.Bond B2 annotation (extent=[-40,-10; -20,10], rotation=180);
      BondLib.Bonds.eBond B3 annotation (extent=[20,-10; 40,10]);
      BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[80,10; 100,-10]);
      BondLib.Junctions.J0p2 J0p2_2 
        annotation (extent=[-10,80; 10,100], rotation=90);
      BondLib.Bonds.Bond B4 annotation (extent=[-10,60; 10,80], rotation=90);
      BondLib.Junctions.J1p3 J1p3_2 
        annotation (extent=[10,60; -10,40], rotation=-90);
      BondLib.Bonds.Bond B5 annotation (extent=[-10,20; 10,40], rotation=90);
      BondLib.Bonds.Bond B6 annotation (extent=[-10,-40; 10,-20], rotation=90);
      BondLib.Junctions.J1p3 J1p3_3 
        annotation (extent=[10,-40; -10,-60], rotation=-90);
      BondLib.Bonds.Bond B7 annotation (extent=[-10,-80; 10,-60], rotation=90);
      BondLib.Junctions.J0p2 J0p2_3 
        annotation (extent=[-10,-100; 10,-80], rotation=90);
      BondLib.Bonds.Bond B8 annotation (extent=[-60,10; -40,30], rotation=90);
      BondLib.Bonds.Bond B9 annotation (extent=[10,40; 30,60]);
      BondLib.Bonds.Bond B10 annotation (extent=[10,-60; 30,-40]);
      BondLib.Spice.RS Rd(
        Tnom=Tnom,
        R=RD,
        TR1=TRD,
        TR2=0,
        Area=Area) annotation (extent=[30,36; 70,64]);
      BondLib.Spice.RS Rs(
        Tnom=Tnom,
        TR2=0,
        Area=Area,
        R=RS,
        TR1=TRS) annotation (extent=[30,-64; 70,-36]);
      BondLib.Spice.RS Rg(
        Tnom=Tnom,
        TR2=0,
        Area=Area,
        R=RG,
        TR1=TRG) annotation (extent=[-64,30; -36,70], rotation=90);
      BondLib.Bonds.eBond B11 annotation (extent=[-60,70; -40,90], rotation=90);
      BondLib.Bonds.eBond B12 annotation (extent=[70,40; 90,60]);
      BondLib.Bonds.eBond B13 annotation (extent=[70,-60; 90,-40]);
      BondLib.Spice.Utilities.PFETint PFETint1(
        Level=Level,
        Tnom=Tnom,
        VT0=VT0,
        IS=IS,
        BETA=BETA,
        LAMBDA=LAMBDA,
        TCV=TCV,
        EG=EG,
        N=N,
        XTI=XTI,
        PB=PB,
        CGS=CGS,
        CGD=CGD,
        M=M,
        FC=FC,
        TT=TT,
        BEX=BEX,
        LAM1=LAM1,
        ALPHA=ALPHA,
        B=B,
        DELTA=DELTA,
        VMAX=VMAX,
        GminDC=GminDC,
        Area=Area,
        EMin=EMin,
        EMax=EMax) annotation (extent=[-20,-20; 20,20]);
      equation 
      connect(J0p2_1.BondCon1, G)        annotation (points=[-100,0; -120,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p5_1.BondCon2, H)        annotation (points=[100,0; 120,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B3.eBondCon1, J0p5_1.BondCon1) annotation (points=[40,0; 80,0],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(D, J0p2_2.BondCon2)        annotation (points=[0,120; 0,100;
            6.12303e-016,100], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p2_3.BondCon1, S)        annotation (points=[-6.12303e-016,-100;
            -6.12303e-016,-112; 0,-110; 0,-120], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B8.BondCon1, J1p3_1.BondCon3) annotation (points=[-50,10.2; -50,
            10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_2.BondCon3, B9.BondCon1) annotation (points=[10,50; 10.2,50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_3.BondCon3, B10.BondCon1) annotation (points=[10,-50; 10.2,
            -50], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B9.BondCon2, Rd.BondCon1) annotation (points=[30,50; 30,50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B10.BondCon2, Rs.BondCon1) annotation (points=[30,-50; 30,-50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(Rg.BondCon1, B8.BondCon2) annotation (points=[-50,30; -50,30],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B11.fBondCon1, Rg.BondCon2) annotation (points=[-50,70; -50,70],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(Rd.BondCon2, B12.fBondCon1) annotation (points=[70,50; 70,50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(Rs.BondCon2, B13.fBondCon1) annotation (points=[70,-50; 70,-50],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B13.eBondCon1, J0p5_1.BondCon5) annotation (points=[90,-50; 100,
            -50; 100,-20; 90,-20; 90,-10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B12.eBondCon1, J0p5_1.BondCon3) annotation (points=[90,50; 100,50;
            100,30; 86,30; 86,10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B11.eBondCon1, J0p5_1.BondCon4) annotation (points=[-50,90; -50,
            106; 110,106; 110,20; 94,20; 94,10], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_2.BondCon1, B5.BondCon2) annotation (points=[6.12303e-016,40;
            6.12303e-016,40], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B4.BondCon1, J1p3_2.BondCon2) annotation (points=[-6.00057e-016,
            60.2; -6.00057e-016,63.1; -6.12303e-016,63.1; -6.12303e-016,60],
          style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p2_2.BondCon1, B4.BondCon2) annotation (points=[-6.12303e-016,
            80; -6.12303e-016,85; 6.12303e-016,85; 6.12303e-016,80], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J1p3_3.BondCon2, B6.BondCon1) annotation (points=[-6.12303e-016,
            -40; -6.00057e-016,-40; -6.00057e-016,-39.8], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B7.BondCon2, J1p3_3.BondCon1) annotation (points=[6.12303e-016,
            -60; 6.12303e-016,-60], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p2_3.BondCon2, B7.BondCon1) annotation (points=[6.12303e-016,
            -80; -6.00057e-016,-80; -6.00057e-016,-79.8], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B2.BondCon2, J1p3_1.BondCon2) annotation (points=[-40,
            1.22461e-015; -40,0], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(B1.BondCon1, J1p3_1.BondCon1) annotation (points=[-60.2,
            -1.20011e-015; -66.1,-1.20011e-015; -66.1,0; -60,0], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(J0p2_1.BondCon2, B1.BondCon2) annotation (points=[-80,0; -80,
            1.22461e-015], style(
          color=8,
          rgbcolor={192,192,192},
          fillColor=3,
          rgbfillColor={0,0,255},
          fillPattern=1));
      connect(PFETint1.Gint, B2.BondCon1) annotation (points=[-20,0; -20,
            -1.20011e-015; -20.2,-1.20011e-015], style(color=8, rgbcolor={192,
              192,192}));
      connect(B6.BondCon2, PFETint1.Sint) annotation (points=[6.12303e-016,-20;
            0,-20], style(color=8, rgbcolor={192,192,192}));
      connect(B3.fBondCon1, PFETint1.H) annotation (points=[20,0; 20,0], style(
            color=8, rgbcolor={192,192,192}));
      connect(B5.BondCon1, PFETint1.Dint) annotation (points=[-6.00057e-016,
            20.2; 0,20.2; 0,20], style(color=8, rgbcolor={192,192,192}));
      end PFET;
    
      package Utilities "Utility models of Spice circuit element models" 
      annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=42, rgbcolor={127,0,0})),
            Text(
              extent=[-120,122; 120,73],
              string="%name",
              style(color=1))), Documentation(info="<html>
This package contains some utility models used within the Spice-style circuit element models.
</html>
"));
        block nlC "Non-linear capacitance computation of Gummel-Poon BJT" 
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Current IS=1.e-16 
          "Transport saturation current";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real N=1 "Current emission coefficient";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage VJ=0.8 
          "Built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJ=0.5e-12 
          "Zero-bias depletion capacitance at reference temperature";
          parameter Real MJ=0.333 "Junction grading coefficient";
          parameter Real FC=0.5 
          "Factor for linearization of depletion capacitance formula";
          parameter Modelica.SIunits.Time Tau=5e-9 "Ideal transit time";
          parameter Modelica.SIunits.Conductance GminDC=1e-15 
          "Leakage conductance";
          parameter Real Area=1 "Relative area occupied by capacitor";
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Boolean BE=true "True if base-emitter junction";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
          output Modelica.SIunits.Voltage Vt "Thermal voltage";
          output Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
          output Modelica.SIunits.Voltage VJval 
          "Built-in potential at device temperature";
          output Modelica.SIunits.Capacitance CJval 
          "Depletion capacitance at device temperature";
          output Modelica.SIunits.Energy EGval 
          "Energy gap at device temperature";
          output Modelica.SIunits.Energy EGnom 
          "Energy gap at reference temperature";
        
      protected 
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Modelica.SIunits.Temperature DTemp 
          "Difference between circuit temperature and reference temperature";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Capacitance Cdiff1 
          "Part of diffusion capacitance at device temperature";
          Real et;
        annotation (Diagram(
                    Text(
                extent=[-78,70; 76,-58],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-82,-46; 82,-70],
                style(color=0, rgbcolor={0,0,0}),
              string="Gummel-Poon BJT")), Icon(
            Rectangle(extent=[-100,100; 100,-100], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255})),
                    Text(
                extent=[-78,70; 76,-58],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-82,-46; 82,-70],
                style(color=0, rgbcolor={0,0,0}),
              string="Gummel-Poon BJT")));
      public 
        Modelica.Blocks.Interfaces.RealInput qb "Base charge" 
          annotation (extent=[-60,-110; -40,-90], rotation=90);
        Modelica.Blocks.Interfaces.RealInput C0(redeclare type SignalType = 
                Modelica.SIunits.Capacitance) "Base capacitance" 
          annotation (extent=[40,-110; 60,-90], rotation=90);
      public 
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
                Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[-10,90; 10,110],    rotation=270);
        Modelica.Blocks.Interfaces.RealInput v(redeclare type SignalType = 
                Modelica.SIunits.Voltage) "Voltage across junction" 
          annotation (extent=[-110,-10; -90,10]);
        Modelica.Blocks.Interfaces.RealOutput Cdiff(redeclare type SignalType 
            =   Modelica.SIunits.Capacitance) "Diffusion capacitance" 
          annotation (extent=[100,40; 120,60]);
        Modelica.Blocks.Interfaces.RealOutput Cdepl(redeclare type SignalType 
            =   Modelica.SIunits.Capacitance) "Depletion capacitance" 
          annotation (extent=[100,-60; 120,-40]);
        equation 
          /* Compute thermal voltage as function of temperature */
          Vt = k*Tdev/q;
          et = v/(N*Vt);
        
          /* Compute temperature dependence of saturation current and junction capacitance */
          DTemp = Tdev - Tnom;
          RTemp = Tdev/Tnom;
          if Level==2 then
            /* Gummel-Poon model */
            EGval = EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
            EGnom = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
            VJval = VJ*RTemp + EGval - EGnom*RTemp - 3*Vt*ln(RTemp);
            CJval = CJ*(1 + MJ*(1 - VJval/VJ + 4e-4*DTemp));
          else
            /* Ebers-Moll model */
            EGval = EG;
            EGnom = EG;
            VJval = VJ;
            CJval = CJ;
          end if;
          ISval = IS*exp((RTemp - 1)*EGval/Vt + XTI*ln(RTemp));
        
          /* Compute Ebers-Moll diffusion capacitance */
          Cdiff1 = Tau*ISval/(N*Vt)*(if et < EMin then ExMin*(et - EMin + 1) else 
                                     if et > EMax then ExMax*(et - EMax + 1) else exp(et));
        
          /* Compute diffusion and depletion capacitance values */
          if Level==2 then
            /* Gummel-Poon model */
            Cdiff = Area*(Tau*GminDC + (if BE then Cdiff1/qb - C0 else Cdiff1));
            Cdepl = Area*CJval*(if v > FC*VJval then (1 - FC*(1 + MJ) + MJ*v/VJval)/((1 - FC)^(1 + MJ)) else 
                                                     1/noEvent((abs(1 - v/VJval))^MJ));
          else
            /* Ebers-Moll model */
            Cdiff = Tau*GminDC + Cdiff1;
            Cdepl = CJval*(if v/VJval > 0 then 1 + MJ*v/VJval else 1/noEvent((abs(1 - v/VJval))^MJ));
          end if;
        end nlC;
      
        block BJTvars 
        "Space charge and injected currents of bipolar transistors" 
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Real BF=50 
          "Maximum forward current gain at reference temperature";
          parameter Real BR=0.1 
          "Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
          "Saturation current used for current injection";
          parameter Modelica.SIunits.Time TF=0.12e-9 
          "Ideal forward transit time";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current";
          parameter Modelica.SIunits.Voltage VAF=9e30 "Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=9e30 "Reverse early voltage";
        
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=9e30 
          "Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=9e30 
          "Corner for reverse beta high-current roll-off";
          parameter Modelica.SIunits.Current ISC=1e-16 
          "Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Modelica.SIunits.Current ISE=1e-16 
          "Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
          "Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
          "Low-current base-emitter leakage emission coefficient";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real XTB=0 
          "Forward and reverse beta temperature coefficient";
          parameter Real Area=1 "Relative area occupied by resistor";
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          output Modelica.SIunits.Voltage Vt "Thermal voltage";
          output Modelica.SIunits.Energy EGval 
          "Energy gap at device temperature";
          output Modelica.SIunits.Current ISCval 
          "Collector saturation current at device temperature";
          output Modelica.SIunits.Current ISEval 
          "Emitter saturation current at device temperature";
          output Modelica.SIunits.Current icn 
          "Beta degradation collector current";
          output Modelica.SIunits.Current ien 
          "Beta degradation emitter current";
        
      protected 
          Real RTemp "Temperature quotient";
          Real Rxtb "Exponentiated temperature quotient";
          Real q1 "Primary base charge";
          Real q2 "Secondary base charge";
          Real dq1 "Derivative of q1";
          Real dq2 "Derivative of q2";
          Real dqb "Derivative of qb";
          Real BFval "Forward beta coefficient at device temperature";
          Real BRval "Backward beta coefficient at device temperature";
          Real facln 
          "Logarithm of temperature dependence of saturation current";
      public 
        Modelica.Blocks.Interfaces.RealInput vbc(redeclare type SignalType = 
                Modelica.SIunits.Voltage) "Base-collector voltage" 
          annotation (extent=[-120,50; -100,70]);
        Modelica.Blocks.Interfaces.RealInput vbe(redeclare type SignalType = 
                Modelica.SIunits.Voltage) "Base-emitter voltage" 
          annotation (extent=[-120,-30; -100,-10]);
        Modelica.Blocks.Interfaces.RealInput ibc(redeclare type SignalType = 
                Modelica.SIunits.Current) "Base-collector current" 
          annotation (extent=[-120,10; -100,30]);
        annotation (Diagram(
            Rectangle(extent=[-100,100; 100,-100], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255})),
            Text(
              extent=[-72,58; 76,20],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="Injected currents"),
            Text(
              extent=[-18,22; 16,-2],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="and"),
            Text(
              extent=[-68,-6; 70,-28],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="base charge")), Icon(
            Rectangle(extent=[-100,100; 100,-100], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255})),
            Text(
              extent=[-70,58; 78,20],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="Injected currents"),
            Text(
              extent=[-18,22; 16,-2],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="and"),
            Text(
              extent=[-68,-6; 70,-28],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="base charge")));
        Modelica.Blocks.Interfaces.RealInput ibe(redeclare type SignalType = 
                Modelica.SIunits.Current) "Base-emitter current" 
          annotation (extent=[-120,-70; -100,-50]);
        Modelica.Blocks.Interfaces.RealOutput qb "Base charge" 
          annotation (extent=[100,-30; 120,-10]);
        Modelica.Blocks.Interfaces.RealOutput C0(redeclare type SignalType = 
                Modelica.SIunits.Capacitance) "Base capacitance" 
          annotation (extent=[100,-70; 120,-50]);
        Modelica.Blocks.Interfaces.RealOutput ib0(redeclare type SignalType = 
                Modelica.SIunits.Current) "Injected base current" 
          annotation (extent=[100,50; 120,70]);
        Modelica.Blocks.Interfaces.RealOutput ic0(redeclare type SignalType = 
                Modelica.SIunits.Current) "Injected collector current" 
          annotation (extent=[100,10; 120,30]);
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
                Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[-10,-120; 10,-100], rotation=90);
        equation 
          RTemp = Tdev/Tnom;
          Rxtb = RTemp^XTB;
        
          /* Compute thermal voltage as function of temperature */
          Vt = k*Tdev/q;
        
          /* Compute base charge */
          if Level==2 then
            /* Gummel-Poon model */
            q1 = 1/(1 - vbc/VAF - vbe/VAR);
            q2 = (ISS/IS) * (ibc/IKR + ibe/IKF) / Area;
            qb = q1*0.5*(1 + sqrt(1 + 4*q2));
            dq1 = 1/(VAR*q1*q1);
            dq2 = (ien + ISEval)/(Vt*NC*IKF);
            dqb = qb*dq1/q1 + q1*dq2/sqrt(1+4*q2);
            C0 = ibe*TF*dqb/(qb*qb);
          else
            /* Ebers-Moll model */
            q1 = 1/(1 - vbc/VAF);
            q2 = 0;
            qb = q1;
            dq1 = 0;
            dq2 = 0;
            dqb = 0;
            C0 = 0;
          end if;
        
          /* Compute non-linear current sources */
          BFval = BF*Rxtb;
          BRval = BR*Rxtb;
          EGval = EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
          facln  = (RTemp - 1)*EGval/Vt + XTI*ln(RTemp);
          ISCval = ISC*exp(facln/NC)/Rxtb;
          ISEval = ISE*exp(facln/NE)/Rxtb;
          icn = Area*ISCval*(exp(vbc/(Vt*NC)) - 1);
          ien = Area*ISEval*(exp(vbe/(Vt*NE)) - 1);
        
          if Level==2 then
            /* Gummel-Poon model */
            ic0 = (ibe - ibc)/qb - ibc/BRval - icn;
            ib0 = ibe/BFval + ibc/BRval + ien + icn;
          else
            /* Ebers-Moll model */
            ic0 = (ibe - ibc)/qb - ibc/BRval;
            ib0 = ibe/BFval + ibc/BRval;
          end if;
        end BJTvars;
      
        model RS2 "Spice-style resistor model for BJTs" 
          extends Interfaces.SpiceTwoPort;
          parameter Modelica.SIunits.Resistance R=0 
          "Resistance at reference temperature";
          parameter Real TR1(unit="Ohm/K") = 0 "Linear temperature coefficient";
          parameter Real TR2(unit="Ohm/K2") = 0 
          "Quadratic temperature coefficient";
          parameter Real Area=1 "Relative area occupied by resistor";
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          Modelica.SIunits.Resistance Rval 
          "Resistance value at circuit temperature";
        equation 
          if Level==2 then
          /* Gummel-Poon model */
            Rval = R*(1 + TR1*DTemp + TR2*DTempSq)/Area;
          else
          /* Ebers-Moll model */
            Rval = 0;
          end if;
          e1 = Rval*f1;
          e1*f1 = e2*f2;
        annotation (
          Diagram(
              Text(
                extent=[-82,78; 28,-78],
                string="RS",
                style(color=0))),
          Icon(
              Text(
                extent=[-82,78; 28,-78],
                string="RS",
                style(color=0)),
              Text(extent=[-72,-58; 66,-86],   string="R=%R")),
          Documentation(info="<html>
This is the regular Spice-style resistive source <b>RS</b> model, except that it looks at the modeling level.  If the level is not equal to 2, the resistance is set equal to zero.
</html>
"));
        end RS2;
      
        model RBS "Spice-style base resistor model of bipolar transistors" 
          extends Interfaces.SpiceTwoPort;
          constant Real pi=Modelica.Constants.pi;
          parameter Modelica.SIunits.Resistance RB=0 
          "Resistance at reference temperature";
          parameter Modelica.SIunits.Resistance RBM=0 
          "Modified resistance at reference temperature";
          parameter Modelica.SIunits.Current IRB=0 
          "Reference base current at reference temperature";
          parameter Real TRB1(unit="Ohm/K") = 0 
          "Linear temperature coefficient";
          parameter Real TRB2(unit="Ohm/K2") = 0 
          "Quadratic temperature coefficient";
          parameter Real TRM1(unit="Ohm/K") = 0 
          "Linear temperature coefficient";
          parameter Real TRM2(unit="Ohm/K2") = 0 
          "Quadratic temperature coefficient";
          parameter Real Area=1 "Relative area occupied by resistor";
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          Modelica.SIunits.Resistance RBval 
          "Base resistance value at circuit temperature";
          Modelica.SIunits.Resistance RBMval 
          "Modified base resistance value at circuit temperature";
          Modelica.SIunits.Resistance Rval 
          "Resistance value at circuit temperature";
        
      protected 
          parameter Real piSq = pi*pi;
          Real z;
          Real tz;
        annotation (
          Icon(
              Text(
                extent=[-82,76; 28,-80],
                style(color=0),
              string="RBS"),
              Text(extent=[-72,-58; 66,-86], string="R=%RB")),
          Diagram(
              Text(
                extent=[-82,76; 28,-80],
                style(color=0),
              string="RBS")),
          Documentation(info="<html>
The base resistor is the most important of the parasitic resistors of a bipolar transistor.  For this reason, Spice models this resistor using a more sophisticated model with additional parameters. <p>

<hr> <p>

<pre>
Parameters:

 RB:     Reference zero-bias base resistance (default value = 0 Ohm)

 RBM:    Reference minimum base resistance (default value = 0 Ohm)

 IRB:    Current where base resistance falls halfway to minimum value (default value = 0 Amp)

 Tnom:   Reference temperature (default value = 300.15 <sup>o</sup>K)

 TRB1:   Linear temperature variation coefficient of RB (default value = 0 Ohm/<sup>o</sup>K)

 TRB2:   Quadratic temperature variation coefficient of RB (default value = 0 Ohm/<sup>o</sup>K<sup>2</sup>)

 TRM1:   Linear temperature variation coefficient of RBM (default value = 0 Ohm/<sup>o</sup>K)

 TRM2:   Quadratic temperature variation coefficient of RBM (default value = 0 Ohm/<sup>o</sup>K<sup>2</sup>)

 Area:   Relative area occupied by resistor (default value = 1)

 Level:  Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)
</pre> <p>

<hr> <p>

Equations: <p>

<b>DTemp = e2 - Tnom</b> <br>
<b>RBval = RB*(1 + TRB1*DTemp + TRB2*DTemp*DTemp)/Area</b> <br>
<b>RBMval = RBM*(1 + TRM1*DTemp + TRM2*DTemp*DTemp)/Area</b> <br>
<b>z = if IRB &gt; 0 then (-1 + sqrt(1 + 144*f1/(pi*pi*IRB*Area)))/(24*sqrt(f1/(IRB*Area))/(pi*pi)) else 0</b> <br>
<b>tz = if IRB &gt; 0 then tan(z) else 0</b> <br>
<b>Rval = if IRB &gt; 0 then (Rbmval + 3*(Rbval - Rbmval)*(tz-z)/(z*tz*tz)) else (Rbmval + (Rbval - Rbmval)/qb)</b> <br>
<b>e1 = Rval*f1</b> <br>
<b>f2 = (e1/e2)*f1</b>
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput qb "Base charge" 
          annotation (extent=[-10,90; 10,110], rotation=-90);
        equation 
          RBval = RB*(1 + TRB1*DTemp + TRB2*DTempSq)/Area;
          RBMval = RBM*(1 + TRM1*DTemp + TRM2*DTempSq)/Area;
        
          if Level==2 then
            /* Gummel-Poon model */
            if IRB > 0 and IRB < 1e10 then
              z = (-1 + sqrt(1 + 144*f1/(piSq*IRB*Area)))/(24*sqrt(f1/(IRB*Area))/piSq);
              tz = tan(z);
              Rval = RBMval + 3*(RBval - RBMval)*(tz-z)/(z*tz*tz);
            else
              z = 0;
              tz = 0;
              Rval = RBMval + (RBval - RBMval)/qb;
            end if;
          else
            /* Ebers-Moll model */
            z = 0;
            tz = 0;
            Rval = 0;
          end if;
        
          e1 = Rval*f1;
          e1*f1 = e2*f2;
        end RBS;
      
        model Gj "Spice-style leakage conductance model" 
          extends BondLib.Interfaces.PassiveOnePort;
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
          "Leakage conductance";
          parameter Real Area=1 "Relative area occupied by device";
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
        annotation (
          Icon(
              Text(
                extent=[-84,64; 12,-68],
                style(color=0),
              string="Gj"),
            Text(
              extent=[0,-6; 84,-34],
              string="Spice",
              style(color=0, rgbcolor={0,0,0})),
                                 Text(extent=[-68,92; 72,52],   string="%name")),
          Diagram(
              Text(
                extent=[-84,64; 12,-68],
                style(color=0),
              string="Gj"), Text(
              extent=[0,-6; 84,-34],
              string="Spice",
              style(color=0, rgbcolor={0,0,0}))),
          Documentation(info="<html>
The Spice-style leakage conductance model is a conductor element without temperature variation.  The <b>Level</b> parameter is used to acknowledge the additional <b>Area</b> parameter of the Gummel-Poon transistor model. No heat port is needed here, as the model is used only as a mathematical construct to compute the currents of the non-linear current sources. The heat generation is accounted for in the non-linear sources. <p>

<hr> <p>

<pre>
Parameters:

 GminDC:   Leakage conductance (default value = 1e-16 Mho)

 Area:     Relative area occupied by resistor (default value = 1)

 Level:    Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = Area*GminDC*e</b>
</html>
"));
        equation 
          if Level==2 then
            /* Gummel-Poon model */
            f = Area*GminDC*e;
          else
            /* Ebers-Moll model */
            f = GminDC*e;
          end if;
        end Gj;
      
        model mC3 "Bondgraphic modulated capacitor element for Spice" 
          extends BondLib.Interfaces.PassiveOnePort;
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Real XCJ=1 
          "Fraction of depletion capacitance connected to internal base node";
          Real C "Bondgraphic Capacitance";
        
      protected 
          Real du "Derivative of voltage across capacitor";
        annotation (
          Diagram(
                 Text(
                extent=[-76,78; 44,-82],
                style(color=0),
                string="mC"),
                 Text(
                extent=[40,-2; 70,-54],
                style(color=0),
              string="3")),
          Icon(                  Text(extent=[-70,92; 70,52],   string="%name"),
                 Text(
                extent=[-76,78; 44,-82],
                style(color=0),
                string="mC"),
                 Text(
                extent=[40,-2; 70,-54],
                style(color=0),
              string="3")),
          Documentation(info="<html>
This model presents a generalization of the <b>mC2</b> model presented in the semiconductor section, generalized for Spice-style transistor models.  It computes also the external current, <b>ix</b>, which doesn't pass through the junction, but in parallel to it through the <b>Cbx</b> capacitor, which is realized as a current source to prevent an unnecessary higher index DAE problem. <p>

<hr> <p>

<pre>
Potential variables:

 e:      Bondgraphic effort variable

 f:      Bondgraphic flow variable, normalized positive for flows into the model

 Cdiff:  Diffusion capacitance (modulating input signal)

 Cdepl:  Depletion capacitance (modulating input signal)

 ix:     External depletion capacitive current


Parameters:

 XCJC:   Fraction of depletion capacitance connected to internal base node (default value = 1)

 Level:  Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)
</pre> <p>

<hr> <p>

Equations: <p>

<b>C = Cdiff + XCJ*Cdepl;</b> <br>
<b>du = f/C;</b> <br>
<b>ix = (1 - XCJ)*Cdepl*du;</b> <br>
<b>der(e) = du;</b>
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput Cdiff(redeclare type SignalType = 
                Modelica.SIunits.Capacitance) "Diffusion capacitance" 
          annotation (extent=[-60,-110; -40,-90], rotation=90);
        Modelica.Blocks.Interfaces.RealInput Cdepl(redeclare type SignalType = 
                Modelica.SIunits.Capacitance) "Depletion capacitance" 
          annotation (extent=[40,-110; 60,-90], rotation=90);
        Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType = 
                Modelica.SIunits.Current) 
          "External base-collector capacitance current" 
          annotation (extent=[-10,90; 10,110], rotation=90);
        equation 
          if Level==2 then
          /* Gummel-Poon model */
            C = Cdiff + XCJ*Cdepl;
            du = f/C;
            ix = (1 - XCJ)*Cdepl*du;
            der(e) = du;
          else
          /* Ebers-Moll model */
            C = Cdiff + Cdepl;
            du = f/C;
            ix = 0;
            der(e) = du;
          end if;
        end mC3;
      
        model CjS 
        "Spice-style junction capacitor model for bipolar transistors" 
          extends BondLib.Interfaces.TwoPort;
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Conductance GminDC=1e-15 
          "Leakage conductance";
          parameter Real N=1 "Current emission coefficient";
          parameter Modelica.SIunits.Capacitance CJ=0.5e-12 
          "Zero-bias depletion capacitance at reference temperature";
          parameter Real MJ=0.333 "Junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJ=0.8 
          "Built-in potential at reference temperature";
          parameter Real XCJ=1 
          "Fraction of depletion capacitance connected to internal base node";
          parameter Real FC=0.5 
          "Factor for linearization of depletion capacitance formula";
          parameter Modelica.SIunits.Time Tau=1e-9 "Ideal transit time";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real XTB=0 
          "Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current";
          parameter Real Area=1 "Relative area occupied by device";
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Boolean BE=true "True if base-emitter junction";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (
          Icon(
              Text(
                extent=[-82,76; 28,-80],
                style(color=0),
              string="CjS"),
            Text(
              extent=[14,-4; 102,-32],
              string="Spice",
              style(color=0, rgbcolor={0,0,0})),
                                 Text(extent=[-70,88; 70,48],   string="%name")),
          Diagram,
          Documentation(info="<html>
The Spice-style junction capacitance computes both the depletion capacitance value and the diffusion capacitance value of a junction used in a bipolar transistor.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model (just as in the case of the simpler Ebers-Moll model).  The numerical error should remain small, as the time constants associated with temperature change are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2].  Since: <p>

<b>f = der(q)</b> <p>

the space charge, <b>q</b>, was symbolically differentiated.  In the differentiation, only electrical signals, i.e., the voltage, <b>e</b>, were considered time-varying.  The temperature gradients were assumed to be negligible.  This produced an equation of the form: <p>

<b>f = C(e)*der(e)</b> <p>

which is the formula that was used to compute the non-linear capacitance <b>C(e)</b> in the model. <p>

<hr> <p>

<pre>
Parameters:

 Is:      Transport saturation current (default value = 1e-16 Amp)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent (default value = 3)

 VJ:      Built-in potential at reference temperature (default value = 0.75 Volt)

 CJ:      Zero-bias depletion capacitance at reference temperature (default value = 0.5e-12 F)

 XCJ:     Fraction of base-collector depletion capacitance connected to internal base node (default value = 1)

 MJ:      Junction grading coefficient (default value = 0.333)

 Tau:     Ideal transit time (default value = 1e-9 sec)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 Level:  Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)

 BE:      True if base-emitter junction (default value = true)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.69.
</ol>
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput qb "Base charge" 
          annotation (extent=[40,-110; 60,-90],   rotation=90);
        Modelica.Blocks.Interfaces.RealInput C0(redeclare type SignalType = 
                Modelica.SIunits.Capacitance) "Base capacitance" 
          annotation (extent=[-60,-110; -40,-90],
                                                rotation=90);
        Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType = 
                Modelica.SIunits.Current) 
          "External base-collector capacitance current" 
          annotation (extent=[-60,90; -40,110],rotation=90);
        Modelica.Blocks.Interfaces.RealOutput id(redeclare type SignalType = 
                Modelica.SIunits.Current) "Diode current" 
          annotation (extent=[40,90; 60,110],  rotation=90);
          Junctions.J0p3 J0p3_1 annotation (extent=[-90,-10; -70,10]);
          Bonds.fBond B1 annotation (extent=[-70,-10; -50,10]);
          Sensors.De vbi annotation (extent=[-50,-10; -30,10]);
          Bonds.eBond B2 annotation (extent=[-90,-30; -70,-10],
                                                              rotation=270);
          Sources.mSe Vbi annotation (extent=[0,-10; -20,10]);
          Bonds.fBond B3 annotation (extent=[0,-10; 20,10]);
          Junctions.J1p3 J1p3_1 annotation (extent=[20,-10; 40,10], rotation=
              -90);
          Bonds.eBond B4 annotation (extent=[20,10; 40,30],  rotation=90);
          Sensors.Df id1 
                        annotation (extent=[20,30; 40,50], rotation=90);
          Bonds.fBond B5 annotation (extent=[20,-30; 40,-10], rotation=270);
          Junctions.J0p3 J0p3_2 annotation (extent=[20,-30; 40,-50]);
          Bonds.fBond B6 annotation (extent=[40,-50; 60,-30], rotation=0);
          Bonds.fBond B7 annotation (extent=[0,-50; 20,-30], rotation=180);
        nlC nlC1(
          Tnom=Tnom,
          IS=IS,
          EG=EG,
          N=N,
          XTI=XTI,
          VJ=VJ,
          CJ=CJ,
          MJ=MJ,
          FC=FC,
          Tau=Tau,
          GminDC=GminDC,
          Area=Area,
          Level=Level,
          BE=BE,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-40,-50; -60,-30]);
          Bonds.fBond B8 annotation (extent=[50,10; 70,30], rotation=90);
          Sensors.De Tdev 
                         annotation (extent=[50,30; 70,50], rotation=90);
        DjS DjS1(
          Tnom=Tnom,
          IS=IS,
          EG=EG,
          N=N,
          XTI=XTI,
          Area=Area,
          Level=Level,
          EMin=EMin,
          EMax=EMax) annotation (extent=[60,-52; 100,-28]);
        mC3 Cj1(Level=Level, XCJ=XCJ) 
          annotation (extent=[-70,-50; -90,-30], rotation=-90);
          Junctions.J0p3 J0p3_3 annotation (extent=[70,-10; 90,10]);
        Gj Gj1(
          GminDC=GminDC,
          Area=Area,
          Level=Level) 
               annotation (extent=[0,-50; -28,-30]);
          Bonds.eBond B9 annotation (extent=[90,-36; 110,-16],
                                                             rotation=90);
        equation 
        connect(J0p3_1.BondCon1, BondCon1) annotation (points=[-90,0; -100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon3, B2.fBondCon1) annotation (points=[-80,-10; -80,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(B2.eBondCon1, Cj1.BondCon1) annotation (points=[-80,-30; -80,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(nlC1.Cdiff, Cj1.Cdiff) annotation (points=[-61,-35; -70,-35],
            style(color=74, rgbcolor={0,0,127}));
        connect(nlC1.Cdepl, Cj1.Cdepl) annotation (points=[-61,-45; -70,-45],
            style(color=74, rgbcolor={0,0,127}));
        connect(C0, nlC1.C0) annotation (points=[-50,-100; -50,-70; -55,-70;
              -55,-50], style(color=74, rgbcolor={0,0,127}));
        connect(qb, nlC1.qb) annotation (points=[50,-100; 50,-70; -44,-70; -44,
              -50; -45,-50], style(color=74, rgbcolor={0,0,127}));
        connect(J0p3_1.BondCon2, B1.eBondCon1) annotation (points=[-70,0; -70,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(Cj1.ix, ix) annotation (points=[-90,-40; -112,-40; -112,70; -50,
              70; -50,100], style(color=74, rgbcolor={0,0,127}));
        connect(vbi.BondCon1, B1.fBondCon1) annotation (points=[-50,0; -50,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(Vbi.BondCon1, B3.eBondCon1) annotation (points=[0,0; 0,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(id1.BondCon1, B4.eBondCon1) annotation (points=[30,30; 30,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(id1.OutPort1, id) annotation (points=[30,51; 30,70; 50,70; 50,
              100], style(color=74, rgbcolor={0,0,127}));
        connect(Tdev.OutPort1, nlC1.Tdev) annotation (points=[60,51; 60,60; -56,
              60; -56,-20; -50,-20; -50,-30], style(color=74, rgbcolor={0,0,127}));
        connect(J1p3_1.BondCon3, B3.fBondCon1) annotation (points=[20,
              -6.12303e-016; 20,0], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon1, B4.fBondCon1) annotation (points=[30,10; 30,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(B5.eBondCon1, J1p3_1.BondCon2) annotation (points=[30,-10; 30,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(B5.fBondCon1, J0p3_2.BondCon3) annotation (points=[30,-30; 30,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon2, B6.eBondCon1) annotation (points=[40,-40; 40,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B7.eBondCon1, J0p3_2.BondCon1) annotation (points=[20,-40; 20,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(nlC1.v, vbi.OutPort1) annotation (points=[-40,-40; -34,-40; -34,
              -12; -24,-12; -24,0; -29,0], style(color=74, rgbcolor={0,0,127}));
        connect(Tdev.BondCon1, B8.fBondCon1) annotation (points=[60,30; 60,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_3.BondCon2, BondCon2) annotation (points=[90,0; 100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_3.BondCon1, B8.eBondCon1) annotation (points=[70,0; 60,0;
              60,10], style(color=8, rgbcolor={192,192,192}));
        connect(Gj1.BondCon1, B7.fBondCon1) annotation (points=[0,-40; 0,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(B6.fBondCon1, DjS1.BondCon1) annotation (points=[60,-40; 60,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(B9.fBondCon1, DjS1.BondCon2) annotation (points=[100,-36; 100,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B9.eBondCon1, J0p3_3.BondCon3) annotation (points=[100,-16; 100,
              -14; 80,-14; 80,-10], style(color=8, rgbcolor={192,192,192}));
        connect(vbi.OutPort1, Vbi.s) annotation (points=[-29,0; -19,0], style(
              color=74, rgbcolor={0,0,127}));
        end CjS;
      
        model CbxS 
        "Spice-style external base-collector capacitance model for bipolar transistors" 
          extends BondLib.Interfaces.TwoPort;
        annotation (
          Icon(
              Text(
                extent=[-84,76; 26,-80],
                style(color=0),
              string="CbxS"), Text(
              extent=[12,2; 100,-26],
              string="Spice",
              style(color=0, rgbcolor={0,0,0})),
            Text(
              extent=[-60,96; 60,64],
              style(color=3, rgbcolor={0,0,255}),
              string="%name")),
          Diagram,
          Documentation(info="<html>
The Spice-style external base-collector capacitance computes the external depletion capacitance value of a bipolar transistor.  The external base-collector capacitance leads invariably to a higher-index DAE.  In order to prevent the Pantelides algorithm from having to be used constantly and unnecessarily, the higher index problem was reduced manually.  This leads to an implementation, whereby the external base-collector capacitance is being realized internally as a modulated current source.
</html>
"));
      public 
        Modelica.Blocks.Interfaces.RealInput ix(redeclare type SignalType = 
                Modelica.SIunits.Current) 
          "External base-collector capacitance current" 
          annotation (extent=[-10,-110; 10,-90],  rotation=90);
        Sources.mSf dbx annotation (extent=[-40,-10; -20,10]);
        Junctions.J0p2 J0p2_1 annotation (extent=[-80,-10; -60,10]);
        Bonds.eBond B1 annotation (extent=[-60,-10; -40,10], rotation=180);
        Modelica.Blocks.Math.Gain Gain1(k=-1) 
          annotation (extent=[-10,-60; 10,-40], rotation=90);
        equation 
          /* Compute heat flow */
          f2 = 0;
        connect(J0p2_1.BondCon1, BondCon1) annotation (points=[-80,0; -100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(dbx.BondCon1, B1.fBondCon1) annotation (points=[-40,0; -40,
              -1.22461e-015], style(color=8, rgbcolor={192,192,192}));
        connect(ix, Gain1.u) annotation (points=[0,-100; 0,-81; 0,-62;
              -7.34764e-016,-62], style(color=74, rgbcolor={0,0,127}));
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-60,0; -60,
              1.22461e-015], style(color=8, rgbcolor={192,192,192}));
        connect(Gain1.y, dbx.s) annotation (points=[6.73533e-016,-39;
              6.73533e-016,0; -21,0], style(color=74, rgbcolor={0,0,127}));
        end CbxS;
      
        model DjS "Spice-style junction diode model for bipolar transistors" 
          extends Interfaces.SpiceTwoPort;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Modelica.SIunits.Current IS=1e-16 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current";
          parameter Real N=1 "Current emission coefficient";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real Area=1 "Relative area occupied by diode";
          parameter Integer Level=2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
          Modelica.SIunits.Energy EGval 
          "Activation energy at device temperature";
        
      protected 
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Real RTemp "Temperature quotient";
          Real et;
        annotation (
          Icon(
              Text(
                extent=[-84,76; 26,-82],
                style(color=0),
              string="DjS")),
          Diagram(
              Line(points=[-80,0; 80,0],   style(color=10)),
              Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                    fillColor=10)),
              Line(points=[0,80; 0,-80],   style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[4,82; 14,72],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="f"),
              Text(
                extent=[70,-4; 80,-14],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="e"),
              Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[-66,-6; -36,-14],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="blocking"),
              Text(
                extent=[16,50; 48,42],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="conducting"),
              Line(points=[0,0; 6,10; 12,28; 16,66],            style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1))),
          Documentation(info="<html>
The Spice-style junction diode for BJTs is a regular diode without heat generation. A separate model was needed to implement the <b>Level</b> parameter, as the Ebers-Moll model doesn't recognize the <b>Area</b> parameter. A heat port was still needed, as the model makes use of the device temperature that it imports through the heat port.  However, the heat flow itself is zero, as the model is used only as a mathematical construct to compute the currents of the non-linear current sources. The heat generation is accounted for in the non-linear sources. <p>

<hr>

<pre>
Parameters:

 IS:     Transport saturation current (default value = 1e-16 Amp)

 EG:     Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 N:      Current emission coefficient (default value = 1)

 XTI:    Saturation current temperature exponent (default value = 3)

 Area:   Relative area occupied by the diode (default value = 1)

 Level:  Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default = 2)

 EMin:   if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:   if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre>
</html>
"));
        equation 
          /* Compute thermal voltage as function of temperature */
          Vt = k*e2/q;
          et = e1/(N*Vt);
        
          /* Compute temperature dependence of saturation current */
          RTemp = e2/Tnom;
          EGval = EG - GapC1*e2*e2/(e2 + GapC2);
          ISval = IS*exp((RTemp - 1)*EGval/Vt + XTI*ln(RTemp));
        
          /* Compute diode characteristic */
          if Level==2 then
            /* Gummel-Poon model */
            f1 = ISval*Area*(if et < EMin then ExMin*(et - EMin + 1) - 1 else 
                             if et > EMax then ExMax*(et - EMax + 1) - 1 else exp(et) - 1);
          else
            /* Ebers-Moll model */
            f1 = ISval*(if et < EMin then ExMin*(et - EMin + 1) - 1 else 
                        if et > EMax then ExMax*(et - EMax + 1) - 1 else exp(et) - 1);
          end if;
        
          /* Compute heat flow */
          f2 = 0;
        end DjS;
      
        model NPNint 
        "Spice-style NPN transistor model without parasitic resistors and substrate diode" 
          /* DC model parameters */
          constant Real inf=Modelica.Constants.inf;
          parameter Real BF=100 
          "Maximum forward current gain at reference temperature";
          parameter Real BR=1 
          "Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
          "Saturation current used for current injection";
          parameter Real NF=1 "Forward current emission coefficient";
          parameter Real NR=1 "Reverse current emission coefficient";
          parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
          "Leakage conductance";
        
          /* Low current beta degradation effect parameters */
          parameter Modelica.SIunits.Current ISC(min=0) = 0 
          "Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Modelica.SIunits.Current ISE(min=0) = 0 
          "Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
          "Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
          "Low-current base-emitter leakage emission coefficient";
        
            /* Base width modulation parameters */
          parameter Modelica.SIunits.Voltage VAF=inf "Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=inf "Reverse early voltage";
        
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=inf 
          "Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=inf 
          "Corner for reverse beta high-current roll-off";
        
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
          "Zero-bias base-collector depletion capacitance at reference temperature";
          parameter Real MJC(min=0.3,max=0.5) = 0.33 
          "Base-collector junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJC=0.8 
          "Base-collector built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
          "Zero-bias base-emitter depletion capacitance at reference temperature";
          parameter Real MJE(min=0.3,max=0.5) = 0.33 
          "Base-emitter junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJE=0.75 
          "Base-emitter built-in potential at reference temperature";
          parameter Real XCJC(min=0,max=1) = 1 
          "Fraction of base-collector depletion capacitance connected to internal base node";
          parameter Real FC(min=0,max=1) = 0.5 
          "Depletion capacitance factor for linearization";
        
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TF(min=0) = 0 
          "Ideal forward transit time";
          parameter Modelica.SIunits.Time TR(min=0) = 0 
          "Ideal reverse transit time";
        
          /* Temperature compensation and area parameters */
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real XTB=0 
          "Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.11 
          "Energy gap for temperature effect on saturation current";
          parameter Real Area=1 "Relative area occupied by device";
          parameter Integer Level(min=1,max=2) = 2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
        
          /* Numerical parameters */
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (Coordsys(extent=[-140,-140; 140,140]), Diagram(
            Text(
              extent=[-152,28; -126,12],
              style(color=10, rgbcolor={135,135,135}),
              string="Bint"),
            Text(
              extent=[10,148; 36,130],
              style(color=10, rgbcolor={135,135,135}),
              string="Cint"),
            Text(
              extent=[10,-132; 36,-150],
              style(color=10, rgbcolor={135,135,135}),
              string="Eint"),
            Text(
              extent=[126,-2; 158,-28],
              style(color=10, rgbcolor={135,135,135}),
              string="Heat"),
            Text(
              extent=[-90,22; -78,10],
              style(color=10, rgbcolor={135,135,135}),
              string="vbc"),
            Text(
              extent=[-98,-12; -86,-24],
              style(color=10, rgbcolor={135,135,135}),
              string="vbe"),
            Text(
              extent=[-126,-38; -114,-50],
              style(color=10, rgbcolor={135,135,135}),
              string="ibe"),
            Text(
              extent=[-124,122; -112,110],
              style(color=10, rgbcolor={135,135,135}),
              string="ibc"),
            Text(
              extent=[92,-124; 110,-140],
              style(color=10, rgbcolor={135,135,135}),
              string="Tdev"),
            Text(
              extent=[136,40; 154,24],
              style(color=10, rgbcolor={135,135,135}),
              string="Tdev"),
            Text(
              extent=[134,-60; 146,-74],
              style(color=10, rgbcolor={135,135,135}),
              string="IB0"),
            Text(
              extent=[-22,-114; -10,-128],
              style(color=10, rgbcolor={135,135,135}),
              string="IB0"),
            Text(
              extent=[100,-20; 112,-34],
              style(color=10, rgbcolor={135,135,135}),
              string="IC0"),
            Text(
              extent=[134,-100; 144,-112],
              style(color=10, rgbcolor={135,135,135}),
              string="qb"),
            Text(
              extent=[-112,-106; -102,-118],
              style(color=10, rgbcolor={135,135,135}),
              string="qb"),
            Text(
              extent=[106,-140; 120,-156],
              style(color=10, rgbcolor={135,135,135}),
              string="C0"),
            Text(
              extent=[-78,-108; -66,-120],
              style(color=10, rgbcolor={135,135,135}),
              string="C0")),
          Icon(
              Text(
                extent=[-112,90; 96,-84],
                style(color=0),
                string="NPN"),
              Line(points=[0,100; 0,-100],
                                         style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[-20,-80; 0,-100],style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[0,-100; 20,-80],style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Text(
                extent=[8,-126; 38,-152],
                style(color=9, rgbcolor={149,164,171}),
              string="Eint"),
              Text(
                extent=[-156,28; -124,4],
                style(color=9, rgbcolor={149,164,171}),
              string="Bint"),
              Text(
                extent=[10,150; 38,126],
                style(color=9, rgbcolor={149,164,171}),
              string="Cint"),
              Text(
                extent=[122,30; 158,6],
                style(color=9, rgbcolor={149,164,171}),
              string="Heat"),
              Text(
                extent=[142,-66; 164,-86],
                style(color=9, rgbcolor={149,164,171}),
              string="qb"),
            Line(points=[-100,-60; 100,-60], style(color=3, rgbcolor={0,0,255})),
            Line(points=[80,-40; 100,-60], style(color=3, rgbcolor={0,0,255})),
            Line(points=[80,-80; 100,-60], style(color=3, rgbcolor={0,0,255})),
              Text(
                extent=[88,4; 128,-38],
                style(color=0),
              string="int"),
            Text(
              extent=[-80,120; 80,80],
              style(color=3, rgbcolor={0,0,255}),
              string="%name")),
          Documentation(info="<html>
The NPN element of the Spice bond graph library implements a full-fledges Spice-stye Gummel-Poon model of the NPN bipolar transistor [1-3].  <b>NPNint</b> is a partial model that implements only the internal nodes of the bipolar transistor.  The external parasitic resistances, the external capacitor between base and collector, and the substrate are being added from the outside. <p>

Notice the use of the two internal modulated current sources.  They are connected on the primary side to the <b>Heat</b> port.  As shown in [4,5], these current sources are in fact non-linear resistors, as voltage and current through these sources are always pointing in the same direction.  Thus, they generate heat. <p>

The NPN bipolar transistor is a directed FourPort.  The direction of positive power flow is assumed into the model at the base, <b>B</b>, and at the collector, <b>C</b>, whereas it is assumed out of the model at the emitter, <b>E</b>, and at the <b>Heat</b> port. <p>

The causality of the NPN model is free. <p>

<hr>

<pre>
<b>Parameters:</b>

 Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100)

 BR:      Maximum reverse current gain at reference temperature (default value = 1)

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp)

 ISS:     Saturation current for injection (default value = IS Amp)

 NF:      Forward current emission coefficient (default value = 1)

 NR:      Reverse current emission coefficient (default value = 1)

 GminDC:  Leakage conductance (default value = 1e-19 Mho)


<b>Low Current Beta Degradation Effect Parameters:</b>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp)

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp)

 NC:      Low-current base-collector leakage emission coefficient (default value = 2)

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5)


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = 9e30 Volt)

 VAR:     Reverse early voltage (default value = 9e30 Volt)


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = 9e30 Amp)

 IKR:     Corner for reverse beta high-current roll-off (default value = 9e30 Amp)


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (default value = 1e-12 F)

 MJC:     Base-collector junction grading coefficient (default value = 0.33)

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt)

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (default value = 1e-12 F)

 MJE:     Base-emitter junction grading coefficient (default value = 0.33)

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt)

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1)

 FC:      Depletion capacitance factor for linearization (default value = 0.5)


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec)

 TR:      Ideal reverse transit time (default value = 0 sec)


<b>Temperature Compensation and Area Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K)

 XTI:     Saturation current temperature exponent (default value = 3)

 XTB:     Forward and reverse beta temperature coefficient (default value = 0)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 Area:    Relative area occupied by device (default value = 1)


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100)

 EMax:    Maximum exponent for linearization of junction current (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
        BondLib.Interfaces.BondCon Bint "Internal base" 
          annotation (extent=[-150,-10; -130,10]);
        BondLib.Interfaces.BondCon Eint "Internal emitter" 
          annotation (extent=[-10,-150; 10,-130]);
        BondLib.Interfaces.BondCon H "Heat" 
          annotation (extent=[130,-10; 150,10]);
        BondLib.Interfaces.BondCon Cint "Internal collector" 
          annotation (extent=[-10,130; 10,150]);
          BondLib.Junctions.J0p3 J0p3_1 
          annotation (extent=[-10,-10; 10,10], rotation=-90);
          BondLib.Bonds.Bond B1 annotation (extent=[-10,10; 10,70], rotation=90);
          BondLib.Junctions.J1p3 J1p3_1 
          annotation (extent=[-10,70; 10,90], rotation=-90);
          BondLib.Bonds.Bond B2 
          annotation (extent=[-10,-70; 10,-10], rotation=270);
          BondLib.Junctions.J1p3 J1p3_2 
          annotation (extent=[-10,-90; 10,-70], rotation=-90);
          BondLib.Bonds.Bond B3 annotation (extent=[-30,70; -10,90], rotation=180);
          BondLib.Bonds.Bond B4 
          annotation (extent=[-30,-90; -10,-70], rotation=180);
          BondLib.Junctions.J0p3 J0p3_2 
          annotation (extent=[-50,70; -30,90], rotation=0);
          BondLib.Bonds.fBond B5 
          annotation (extent=[-50,50; -30,70], rotation=-90);
          BondLib.Sensors.De Vbc 
          annotation (extent=[-50,24; -30,50], rotation=-90);
          BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[-50,-90; -30,-70]);
          BondLib.Bonds.fBond B6 
          annotation (extent=[-50,-70; -30,-50], rotation=90);
          BondLib.Sensors.De Vbe 
          annotation (extent=[-50,-50; -30,-24], rotation=90);
          BondLib.Bonds.Bond B7 annotation (extent=[-10,90; 10,110], rotation=90);
          BondLib.Bonds.Bond B8 
          annotation (extent=[-10,-110; 10,-90], rotation=270);
          BondLib.Junctions.J0p3 J0p3_3 
          annotation (extent=[10,-130; -10,-110], rotation=-90);
          BondLib.Junctions.J1p3 J1p3_5 
          annotation (extent=[24,10; 44,-10], rotation=90);
          BondLib.Bonds.Bond B9 annotation (extent=[24,10; 44,70], rotation=270);
          BondLib.Bonds.Bond B10 annotation (extent=[24,-68; 44,-8], rotation=270);
          BondLib.Bonds.eBond B11 
          annotation (extent=[44,-10; 64,10], rotation=180);
          BondLib.Bonds.Bond B13 
          annotation (extent=[-70,70; -50,90], rotation=180);
          BondLib.Bonds.eBond B14 
          annotation (extent=[-130,70; -110,90], rotation=180);
          BondLib.Bonds.Bond B15 
          annotation (extent=[-70,-90; -50,-70], rotation=180);
          BondLib.Bonds.eBond B16 
          annotation (extent=[-130,-90; -110,-70], rotation=180);
          BondLib.Bonds.eBond B12 
          annotation (extent=[-50,-110; -30,-90], rotation=90);
          BondLib.Junctions.J0p3 J0p3_4 
          annotation (extent=[10,110; -10,130], rotation=-90);
          BondLib.Spice.Utilities.BJTvars BJTvars1(
          Tnom=Tnom,
          BF=BF,
          BR=BR,
          TF=TF,
          EG=EG,
          XTI=XTI,
          VAF=VAF,
          VAR=VAR,
          IKF=IKF,
          IKR=IKR,
          ISC=ISC,
          ISE=ISE,
          NC=NC,
          NE=NE,
          XTB=XTB,
          Area=Area,
          Level=Level,
          IS=IS,
          ISS=ISS) 
          annotation (extent=[60,-120; 120,-60]);
          BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[80,80; 120,120]);
          Modelica.Blocks.Interfaces.RealOutput qb "Base charge" 
          annotation (extent=[140,-106; 160,-86]);
        Sources.mSf_int IC0 annotation (extent=[94,-14; 64,14]);
        Bonds.fBond B17 annotation (extent=[94,-10; 114,10],  rotation=180);
        Sources.mSf_int IB0 
          annotation (extent=[-56,-140; -24,-110], rotation=90);
        Bonds.fBond B18 annotation (extent=[-70,-150; -50,-130], rotation=0);
        Sensors.De Tdev annotation (extent=[96,36; 120,60], rotation=-90);
        Modelica.Blocks.Sources.Constant qb1(k=1) 
          annotation (extent=[-118,44; -106,56]);
        Modelica.Blocks.Sources.Constant C0(k=0) 
          annotation (extent=[-62,44; -74,56]);
        Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType = 
                Modelica.SIunits.Current) 
          "External base-collector capacitance current" 
          annotation (extent=[-80,130; -60,150], rotation=90);
        Modelica.Blocks.Math.Gain Gain1(k=-1) 
          annotation (extent=[150,-166; 170,-146], rotation=-90);
        CjS Dbc(
          Tnom=Tnom,
          IS=IS,
          GminDC=GminDC,
          N=NR,
          CJ=CJC,
          MJ=MJC,
          VJ=VJC,
          XCJ=XCJC,
          FC=FC,
          Tau=TR,
          XTI=XTI,
          XTB=XTB,
          EG=EG,
          Area=Area,
          Level=Level,
          BE=false,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-70,66; -110,94]);
        CjS Dbe(
          Tnom=Tnom,
          IS=IS,
          GminDC=GminDC,
          FC=FC,
          XTI=XTI,
          XTB=XTB,
          EG=EG,
          Area=Area,
          Level=Level,
          EMin=EMin,
          EMax=EMax,
          N=NF,
          CJ=CJE,
          MJ=MJE,
          VJ=VJE,
          XCJ=1,
          Tau=TF,
          BE=true) annotation (extent=[-70,-94; -110,-66]);
          BondLib.Bonds.fBond B19 
          annotation (extent=[98,60; 118,80],  rotation=-90);
        equation 
          connect(J1p3_2.BondCon1,B2. BondCon2) annotation (points=[
              -6.12303e-016,-70; -1.83691e-015,-70], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[-10.2,80;
              -10,80],
                   style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B4.BondCon1,J1p3_2. BondCon3) annotation (points=[-10.2,-80;
              -10,-80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p3_2.BondCon2,B3. BondCon2) annotation (points=[-30,80; -30,
              80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B5.eBondCon1,J0p3_2. BondCon3) annotation (points=[-40,70;
              -40,70],
                   style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(Vbc.BondCon1,B5. fBondCon1) annotation (points=[-40,50; -40,
              50],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p4_1.BondCon2,B4. BondCon2) annotation (points=[-30,-80;
              -30,-80],
                    style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p4_1.BondCon4,B6. eBondCon1) annotation (points=[-40,-70;
              -40,-70],
                    style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B6.fBondCon1,Vbe. BondCon1) annotation (points=[-40,-50; -40,
              -50], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J1p3_1.BondCon1,B7. BondCon1) annotation (points=[
              -6.12303e-016,90; -6.12303e-016,90.2; -6.00057e-016,90.2],
                                                         style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B9.BondCon2,J1p3_5. BondCon1) annotation (points=[34,10; 34,
              10],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J1p3_5.BondCon2,B10. BondCon1) annotation (points=[34,-10; 34,
              -8.6], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B15.BondCon1,J0p4_1. BondCon1) annotation (points=[-50.2,-80;
              -50,-80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B12.eBondCon1,J0p4_1. BondCon3) annotation (points=[-40,-90;
              -40,-90], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B7.BondCon2,J0p3_4. BondCon2) annotation (points=[
              6.12303e-016,110; -6.12303e-016,110],
                                       style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p3_4.BondCon3,B9. BondCon1) annotation (points=[10,120; 34,
              120; 34,69.4], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B8.BondCon1,J1p3_2. BondCon2) annotation (points=[
              1.80017e-015,-90.2; 1.80017e-015,-98.1; 0,-90; 6.12303e-016,-90],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p3_3.BondCon1,B8. BondCon2) annotation (points=[
              6.12303e-016,-110; -1.83691e-015,-110],
                                         style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p3_3.BondCon3,B10. BondCon2) annotation (points=[10,-120;
              34,-120; 34,-68],
                             style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p3_1.BondCon1,B1. BondCon1) annotation (points=[
              -6.12303e-016,10; -6.12303e-016,10.6; -6.00057e-016,10.6],
                                                         style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B1.BondCon2,J1p3_1. BondCon2) annotation (points=[
              6.12303e-016,70; 6.12303e-016,70],
                                    style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(Vbc.OutPort1,BJTvars1. vbc) annotation (points=[-40,22.7; -40,
              12; -152,12; -152,-160; 38,-160; 38,-72; 57,-72],       style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(Vbe.OutPort1,BJTvars1. vbe) annotation (points=[-40,-22.7;
              -40,-12; -160,-12; -160,-168; 46,-168; 46,-96; 57,-96],   style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(BJTvars1.qb,qb)  annotation (points=[123,-96; 150,-96],
                                   style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p3_4.BondCon1, Cint)   annotation (points=[6.12303e-016,130;
              0,130; 0,140], style(color=8, rgbcolor={192,192,192}));
          connect(J0p3_1.BondCon3, Bint)   annotation (points=[-10,
              -6.12303e-016; -74,-6.12303e-016; -74,0; -140,0], style(color=8,
              rgbcolor={192,192,192}));
          connect(J0p3_3.BondCon2, Eint)   annotation (points=[-6.12303e-016,
              -130; 0,-130; 0,-140], style(color=8, rgbcolor={192,192,192}));
          connect(J0p6_1.BondCon2, H)      annotation (points=[120,100; 128,100;
              128,0; 140,0], style(color=8, rgbcolor={192,192,192}));
        connect(IB0.BondCon2, B12.fBondCon1) annotation (points=[-40,-110; -40,
              -110], style(color=8, rgbcolor={192,192,192}));
        connect(B17.eBondCon1, J0p6_1.BondCon3) annotation (points=[114,
              -1.22461e-015; 114,20; 92,20; 92,80], style(color=8, rgbcolor={
                192,192,192}));
        connect(B18.fBondCon1, IB0.BondCon1) annotation (points=[-50,-140; -40,
              -140], style(color=8, rgbcolor={192,192,192}));
        connect(B16.eBondCon1, J0p6_1.BondCon5) annotation (points=[-130,-80;
              -168,-80; -168,156; 92,156; 92,120], style(color=8, rgbcolor={192,
                192,192}));
        connect(B14.eBondCon1, J0p6_1.BondCon1) annotation (points=[-130,80;
              -164,80; -164,152; 80,152; 80,100], style(color=8, rgbcolor={192,
                192,192}));
        connect(B18.eBondCon1, J0p6_1.BondCon6) annotation (points=[-70,-140;
              -172,-140; -172,160; 108,160; 108,120], style(color=8, rgbcolor={
                192,192,192}));
        connect(Tdev.OutPort1, BJTvars1.Tdev) annotation (points=[108,34.8; 108,
              28; 164,28; 164,-136; 90,-136; 90,-123],   style(color=74,
              rgbcolor={0,0,127}));
        connect(BJTvars1.ib0, Gain1.u) annotation (points=[123,-72; 160,-72;
              160,-144],        style(color=74, rgbcolor={0,0,127}));
        connect(IC0.BondCon2, B11.fBondCon1) annotation (points=[64,
              1.77636e-015; 64,-1.22461e-015], style(color=8, rgbcolor={192,192,
                192}));
        connect(B17.fBondCon1, IC0.BondCon1) annotation (points=[94,
              1.22461e-015; 94,1.77636e-015], style(color=8, rgbcolor={192,192,
                192}));
        connect(B11.eBondCon1, J1p3_5.BondCon3) annotation (points=[44,
              1.22461e-015; 44,6.12303e-016], style(color=8, rgbcolor={192,192,
                192}));
        connect(Dbc.ix, ix) annotation (points=[-80,94; -80,120; -70,120; -70,
              140], style(color=74, rgbcolor={0,0,127}));
        connect(Dbc.id, BJTvars1.ibc) annotation (points=[-100,94; -100,110;
              -156,110; -156,-164; 42,-164; 42,-84; 57,-84], style(color=74,
              rgbcolor={0,0,127}));
        connect(C0.y, Dbc.C0) annotation (points=[-74.6,50; -80,50; -80,66],
            style(color=74, rgbcolor={0,0,127}));
        connect(qb1.y, Dbc.qb) annotation (points=[-105.4,50; -100,50; -100,66],
            style(color=74, rgbcolor={0,0,127}));
        connect(Dbe.id, BJTvars1.ibe) annotation (points=[-100,-66; -100,-48;
              -164,-48; -164,-172; 50,-172; 50,-108; 57,-108], style(color=74,
              rgbcolor={0,0,127}));
        connect(BJTvars1.C0, Dbe.C0) annotation (points=[123,-108; 128,-108;
              128,-152; -80,-152; -80,-94], style(color=74, rgbcolor={0,0,127}));
        connect(BJTvars1.qb, Dbe.qb) annotation (points=[123,-96; 132,-96; 132,
              -156; -100,-156; -100,-94], style(color=74, rgbcolor={0,0,127}));
        connect(B13.BondCon1, J0p3_2.BondCon1) annotation (points=[-50.2,80; -50,80],
            style(color=8, rgbcolor={192,192,192}));
        connect(Dbc.BondCon1, B13.BondCon2) annotation (points=[-70,80; -70,80],
            style(color=8, rgbcolor={192,192,192}));
        connect(B14.fBondCon1, Dbc.BondCon2) annotation (points=[-110,80; -110,
              80], style(color=8, rgbcolor={192,192,192}));
        connect(Dbe.BondCon1, B15.BondCon2) annotation (points=[-70,-80; -70,
              -80], style(color=8, rgbcolor={192,192,192}));
        connect(B16.fBondCon1, Dbe.BondCon2) annotation (points=[-110,-80; -110,
              -80], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon2, B2.BondCon1) annotation (points=[6.12303e-016,
              -10; 1.80017e-015,-10; 1.80017e-015,-10.6], style(color=8,
              rgbcolor={192,192,192}));
        connect(B19.eBondCon1, J0p6_1.BondCon4) annotation (points=[108,80; 108,
              80], style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.BondCon1, B19.fBondCon1) annotation (points=[108,60; 108,
              60], style(color=8, rgbcolor={192,192,192}));
          connect(BJTvars1.ic0, IC0.s) annotation (points=[123,-84; 130,-84;
              130,-30; 79,-30; 79,-12.6], style(color=74, rgbcolor={0,0,127}));
          connect(Gain1.y, IB0.s) annotation (points=[160,-167; 160,-176; -16,
              -176; -16,-125; -25.6,-125], style(color=74, rgbcolor={0,0,127}));
        end NPNint;
      
        model PNPint 
        "Spice-style PNP transistor model without parasitic resistors and substrate diode" 
          /* DC model parameters */
          constant Real inf=Modelica.Constants.inf;
          parameter Real BF=100 
          "Maximum forward current gain at reference temperature";
          parameter Real BR=1 
          "Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
          "Saturation current used for current injection";
          parameter Real NF=1 "Forward current emission coefficient";
          parameter Real NR=1 "Reverse current emission coefficient";
          parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
          "Leakage conductance";
        
          /* Low current beta degradation effect parameters */
          parameter Modelica.SIunits.Current ISC(min=0) = 0 
          "Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Modelica.SIunits.Current ISE(min=0) = 0 
          "Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
          "Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
          "Low-current base-emitter leakage emission coefficient";
        
            /* Base width modulation parameters */
          parameter Modelica.SIunits.Voltage VAF=inf "Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=inf "Reverse early voltage";
        
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=inf 
          "Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=inf 
          "Corner for reverse beta high-current roll-off";
        
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
          "Zero-bias base-collector depletion capacitance at reference temperature";
          parameter Real MJC(min=0.3,max=0.5) = 0.33 
          "Base-collector junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJC=0.8 
          "Base-collector built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
          "Zero-bias base-emitter depletion capacitance at reference temperature";
          parameter Real MJE(min=0.3,max=0.5) = 0.33 
          "Base-emitter junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJE=0.75 
          "Base-emitter built-in potential at reference temperature";
          parameter Real XCJC(min=0,max=1) = 1 
          "Fraction of base-collector depletion capacitance connected to internal base node";
          parameter Real FC(min=0,max=1) = 0.5 
          "Depletion capacitance factor for linearization";
        
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TF(min=0) = 0 
          "Ideal forward transit time";
          parameter Modelica.SIunits.Time TR(min=0) = 0 
          "Ideal reverse transit time";
        
          /* Temperature compensation and area parameters */
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real XTB=0 
          "Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.11 
          "Energy gap for temperature effect on saturation current";
          parameter Real Area=1 "Relative area occupied by device";
          parameter Integer Level(min=1,max=2) = 2 
          "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
        
          /* Numerical parameters */
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (
          Coordsys(extent=[-140,-140; 140,140]),
          Diagram(
            Text(
              extent=[-152,28; -126,12],
              style(color=10, rgbcolor={135,135,135}),
              string="Bint"),
            Text(
              extent=[10,148; 36,130],
              style(color=10, rgbcolor={135,135,135}),
              string="Cint"),
            Text(
              extent=[10,-132; 36,-150],
              style(color=10, rgbcolor={135,135,135}),
              string="Eint"),
            Text(
              extent=[126,-2; 158,-28],
              style(color=10, rgbcolor={135,135,135}),
              string="Heat"),
            Text(
              extent=[-90,22; -78,10],
              style(color=10, rgbcolor={135,135,135}),
              string="vcb"),
            Text(
              extent=[-98,-12; -86,-24],
              style(color=10, rgbcolor={135,135,135}),
              string="veb"),
            Text(
              extent=[-126,-38; -114,-50],
              style(color=10, rgbcolor={135,135,135}),
              string="ieb"),
            Text(
              extent=[-124,122; -112,110],
              style(color=10, rgbcolor={135,135,135}),
              string="icb"),
            Text(
              extent=[92,-124; 110,-140],
              style(color=10, rgbcolor={135,135,135}),
              string="Tdev"),
            Text(
              extent=[136,40; 154,24],
              style(color=10, rgbcolor={135,135,135}),
              string="Tdev"),
            Text(
              extent=[134,-60; 146,-74],
              style(color=10, rgbcolor={135,135,135}),
              string="IB0"),
            Text(
              extent=[-22,-114; -10,-128],
              style(color=10, rgbcolor={135,135,135}),
              string="IB0"),
            Text(
              extent=[100,-20; 112,-34],
              style(color=10, rgbcolor={135,135,135}),
              string="IC0"),
            Text(
              extent=[134,-104; 144,-116],
              style(color=10, rgbcolor={135,135,135}),
              string="qb"),
            Text(
              extent=[-112,-104; -102,-116],
              style(color=10, rgbcolor={135,135,135}),
              string="qb"),
            Text(
              extent=[100,-142; 112,-154],
              style(color=10, rgbcolor={135,135,135}),
              string="C0"),
            Text(
              extent=[-78,-106; -66,-118],
              style(color=10, rgbcolor={135,135,135}),
              string="C0")),
          Icon(
            Text(
              extent=[-80,120; 80,80],
              style(color=3, rgbcolor={0,0,255}),
              string="%name"),
              Line(points=[0,100; 0,-100],
                                         style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[0,100; 20,80],   style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[-20,80; 0,100], style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
            Line(points=[-100,-60; 100,-60], style(color=3, rgbcolor={0,0,255})),
            Line(points=[80,-40; 100,-60], style(color=3, rgbcolor={0,0,255})),
            Line(points=[80,-80; 100,-60], style(color=3, rgbcolor={0,0,255})),
              Text(
                extent=[-112,90; 96,-84],
                style(color=0),
              string="PNP"),
              Text(
                extent=[88,4; 128,-38],
                style(color=0),
              string="int"),
              Text(
                extent=[-156,28; -124,4],
                style(color=9, rgbcolor={149,164,171}),
              string="Bint"),
              Text(
                extent=[10,150; 38,126],
                style(color=9, rgbcolor={149,164,171}),
              string="Cint"),
              Text(
                extent=[122,30; 158,6],
                style(color=9, rgbcolor={149,164,171}),
              string="Heat"),
              Text(
                extent=[8,-126; 38,-152],
                style(color=9, rgbcolor={149,164,171}),
              string="Eint"),
              Text(
                extent=[142,-66; 164,-86],
                style(color=9, rgbcolor={149,164,171}),
              string="qb")),
          Documentation(info="<html>
The PNP element of the Spice bond graph library implements a full-fledges Spice-stye Gummel-Poon model of the PNP bipolar transistor [1-3].  <b>PNPint</b> is a partial model that implements only the internal nodes of the bipolar transistor.  The external parasitic resistances, the external capacitor between base and collector, and the substrate are being added from the outside. <p>

Notice the use of the two internal modulated current sources.  They are connected on the primary side to the <b>Heat</b> port.  As shown in [4,5], these current sources are in fact non-linear resistors, as voltage and current through these sources are always pointing in the same direction.  Thus, they generate heat. <p>

The PNP bipolar transistor is a directed FourPort.  The direction of positive power flow is assumed into the model at the emitter, <b>E</b>, whereas it is assumed out of the model at the base, <b>B</b>, at the collector, <b>C</b>, and at the <b>Heat</b> port. <p>

The causality of the PNP model is free. <p>

<hr>

<pre>
<b>Parameters:</b>

 Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100)

 BR:      Maximum reverse current gain at reference temperature (default value = 1)

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp)

 ISS:     Saturation current for injection (default value = IS Amp)

 NF:      Forward current emission coefficient (default value = 1)

 NR:      Reverse current emission coefficient (default value = 1)

 GminDC:  Leakage conductance (default value = 1e-19 Mho)


<b>Low Current Beta Degradation Effect Parameters:</b>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp)

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp)

 NC:      Low-current base-collector leakage emission coefficient (default value = 2)

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5)


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = 9e30 Volt)

 VAR:     Reverse early voltage (default value = 9e30 Volt)


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = 9e30 Amp)

 IKR:     Corner for reverse beta high-current roll-off (default value = 9e30 Amp)


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (default value = 1e-12 F)

 MJC:     Base-collector junction grading coefficient (default value = 0.33)

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt)

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (default value = 1e-12 F)

 MJE:     Base-emitter junction grading coefficient (default value = 0.33)

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt)

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1)

 FC:      Depletion capacitance factor for linearization (default value = 0.5)


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec)

 TR:      Ideal reverse transit time (default value = 0 sec)


<b>Temperature Compensation and Area Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K)

 XTI:     Saturation current temperature exponent (default value = 3)

 XTB:     Forward and reverse beta temperature coefficient (default value = 0)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 Area:    Relative area occupied by device (default value = 1)


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100)

 EMax:    Maximum exponent for linearization of junction current (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
        BondLib.Interfaces.BondCon Bint "Internal base" 
          annotation (extent=[-150,-10; -130,10]);
        BondLib.Interfaces.BondCon Eint "Internal emitter" 
          annotation (extent=[-10,-150; 10,-130]);
        BondLib.Interfaces.BondCon H "Heat" 
          annotation (extent=[130,-10; 150,10]);
        BondLib.Interfaces.BondCon Cint "Internal collector" 
          annotation (extent=[-10,130; 10,150]);
          BondLib.Junctions.J0p3 J0p3_1 
          annotation (extent=[-10,-10; 10,10], rotation=-90);
          BondLib.Bonds.Bond B1 annotation (extent=[-10,10; 10,70], rotation=
              270);
          BondLib.Junctions.J1p3 J1p3_1 
          annotation (extent=[-10,70; 10,90], rotation=-90);
          BondLib.Bonds.Bond B2 
          annotation (extent=[-10,-70; 10,-10], rotation=90);
          BondLib.Junctions.J1p3 J1p3_2 
          annotation (extent=[-10,-90; 10,-70], rotation=-90);
          BondLib.Bonds.Bond B3 annotation (extent=[-30,70; -10,90], rotation=180);
          BondLib.Bonds.Bond B4 
          annotation (extent=[-30,-90; -10,-70], rotation=180);
          BondLib.Junctions.J0p3 J0p3_2 
          annotation (extent=[-50,70; -30,90], rotation=0);
          BondLib.Bonds.fBond B5 
          annotation (extent=[-50,50; -30,70], rotation=-90);
          BondLib.Sensors.De Vcb 
          annotation (extent=[-50,24; -30,50], rotation=-90);
          BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[-50,-90; -30,-70]);
          BondLib.Bonds.fBond B6 
          annotation (extent=[-50,-70; -30,-50], rotation=90);
          BondLib.Sensors.De Veb 
          annotation (extent=[-50,-50; -30,-24], rotation=90);
          BondLib.Bonds.Bond B7 annotation (extent=[-10,90; 10,110], rotation=
              270);
          BondLib.Bonds.Bond B8 
          annotation (extent=[-10,-110; 10,-90], rotation=90);
          BondLib.Junctions.J0p3 J0p3_3 
          annotation (extent=[10,-130; -10,-110], rotation=-90);
          BondLib.Junctions.J1p3 J1p3_5 
          annotation (extent=[24,10; 44,-10], rotation=90);
          BondLib.Bonds.Bond B9 annotation (extent=[24,10; 44,70], rotation=90);
          BondLib.Bonds.Bond B10 annotation (extent=[24,-70; 44,-10],rotation=
              90);
          BondLib.Bonds.eBond B11 
          annotation (extent=[44,-10; 64,10], rotation=180);
          BondLib.Bonds.Bond B13 
          annotation (extent=[-70,70; -50,90], rotation=180);
          BondLib.Bonds.eBond B14 
          annotation (extent=[-130,70; -110,90], rotation=180);
          BondLib.Bonds.Bond B15 
          annotation (extent=[-70,-90; -50,-70], rotation=180);
          BondLib.Bonds.eBond B16 
          annotation (extent=[-130,-90; -110,-70], rotation=180);
          BondLib.Bonds.eBond B12 
          annotation (extent=[-50,-110; -30,-90], rotation=90);
          BondLib.Junctions.J0p3 J0p3_4 
          annotation (extent=[10,110; -10,130], rotation=-90);
          BondLib.Spice.Utilities.BJTvars BJTvars1(
          Tnom=Tnom,
          BF=BF,
          BR=BR,
          TF=TF,
          EG=EG,
          XTI=XTI,
          VAF=VAF,
          VAR=VAR,
          IKF=IKF,
          IKR=IKR,
          ISC=ISC,
          ISE=ISE,
          NC=NC,
          NE=NE,
          XTB=XTB,
          Area=Area,
          IS=IS,
          ISS=ISS,
          Level=Level) 
          annotation (extent=[60,-120; 120,-60]);
          BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[80,80; 120,120]);
          Modelica.Blocks.Interfaces.RealOutput qb "Base charge" 
          annotation (extent=[140,-106; 160,-86]);
        BondLib.Sources.mSf_int IC0 
                            annotation (extent=[94,-14; 64,14]);
        BondLib.Bonds.fBond B17 
                        annotation (extent=[94,-10; 114,10],  rotation=180);
        BondLib.Sources.mSf_int IB0 
          annotation (extent=[-56,-140; -24,-110], rotation=90);
        BondLib.Bonds.fBond B18 
                        annotation (extent=[-70,-150; -50,-130], rotation=0);
        BondLib.Sensors.De Tdev 
                        annotation (extent=[96,36; 120,60], rotation=-90);
        Modelica.Blocks.Sources.Constant qb1(k=1) 
          annotation (extent=[-118,44; -106,56]);
        Modelica.Blocks.Sources.Constant C0(k=0) 
          annotation (extent=[-62,44; -74,56]);
        Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType = 
                Modelica.SIunits.Current) 
          "External base-collector capacitance current" 
          annotation (extent=[-80,130; -60,150], rotation=90);
        Modelica.Blocks.Math.Gain Gain1(k=-1) 
          annotation (extent=[150,-166; 170,-146], rotation=-90);
        CjS Dbc(
          Tnom=Tnom,
          IS=IS,
          GminDC=GminDC,
          N=NR,
          CJ=CJC,
          MJ=MJC,
          VJ=VJC,
          XCJ=XCJC,
          FC=FC,
          Tau=TR,
          XTI=XTI,
          XTB=XTB,
          EG=EG,
          Area=Area,
          Level=Level,
          BE=false,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-70,66; -110,94]);
        CjS Dbe(
          Tnom=Tnom,
          IS=IS,
          GminDC=GminDC,
          FC=FC,
          XTI=XTI,
          XTB=XTB,
          EG=EG,
          Area=Area,
          Level=Level,
          EMin=EMin,
          EMax=EMax,
          N=NF,
          CJ=CJE,
          MJ=MJE,
          VJ=VJE,
          XCJ=1,
          Tau=TF,
          BE=true) annotation (extent=[-70,-94; -110,-66]);
          BondLib.Bonds.fBond B19 
          annotation (extent=[98,60; 118,80],  rotation=-90);
        equation 
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[-10.2,80;
              -10,80],
                   style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B4.BondCon1,J1p3_2. BondCon3) annotation (points=[-10.2,-80;
              -10,-80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p3_2.BondCon2,B3. BondCon2) annotation (points=[-30,80; -30,
              80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B5.eBondCon1,J0p3_2. BondCon3) annotation (points=[-40,70;
              -40,70],
                   style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(Vcb.BondCon1,B5. fBondCon1) annotation (points=[-40,50; -40,
              50],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p4_1.BondCon2,B4. BondCon2) annotation (points=[-30,-80;
              -30,-80],
                    style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(J0p4_1.BondCon4,B6. eBondCon1) annotation (points=[-40,-70;
              -40,-70],
                    style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B6.fBondCon1,Veb. BondCon1) annotation (points=[-40,-50; -40,
              -50], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B13.BondCon1,J0p3_2. BondCon1) annotation (points=[-50.2,80;
              -50,80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B15.BondCon1,J0p4_1. BondCon1) annotation (points=[-50.2,-80;
              -50,-80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(B12.eBondCon1,J0p4_1. BondCon3) annotation (points=[-40,-90;
              -40,-90], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(Vcb.OutPort1,BJTvars1. vbc) annotation (points=[-40,22.7; -40,
              12; -152,12; -152,-160; 38,-160; 38,-72; 57,-72],       style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(Veb.OutPort1,BJTvars1. vbe) annotation (points=[-40,-22.7;
              -40,-12; -160,-12; -160,-168; 46,-168; 46,-96; 57,-96],   style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
          connect(BJTvars1.qb,qb)  annotation (points=[123,-96; 150,-96],
                                   style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(J0p3_4.BondCon1, Cint)     annotation (points=[6.12303e-016,130;
              0,130; 0,140], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon3, Bint)     annotation (points=[-10,
              -6.12303e-016; -74,-6.12303e-016; -74,0; -140,0], style(color=8,
              rgbcolor={192,192,192}));
        connect(J0p3_3.BondCon2, Eint)     annotation (points=[-6.12303e-016,
              -130; 0,-130; 0,-140], style(color=8, rgbcolor={192,192,192}));
        connect(J0p6_1.BondCon2, H)        annotation (points=[120,100; 128,100;
              128,0; 140,0], style(color=8, rgbcolor={192,192,192}));
        connect(IB0.BondCon2,B12. fBondCon1) annotation (points=[-40,-110; -40,
              -110], style(color=8, rgbcolor={192,192,192}));
        connect(B17.eBondCon1,J0p6_1. BondCon3) annotation (points=[114,
              -1.22461e-015; 114,20; 92,20; 92,80], style(color=8, rgbcolor={
                192,192,192}));
        connect(B18.fBondCon1,IB0. BondCon1) annotation (points=[-50,-140; -40,
              -140], style(color=8, rgbcolor={192,192,192}));
        connect(B16.eBondCon1,J0p6_1. BondCon5) annotation (points=[-130,-80;
              -168,-80; -168,156; 92,156; 92,120], style(color=8, rgbcolor={192,
                192,192}));
        connect(B14.eBondCon1,J0p6_1. BondCon1) annotation (points=[-130,80;
              -164,80; -164,152; 80,152; 80,100], style(color=8, rgbcolor={192,
                192,192}));
        connect(B18.eBondCon1,J0p6_1. BondCon6) annotation (points=[-70,-140;
              -172,-140; -172,160; 108,160; 108,120], style(color=8, rgbcolor={
                192,192,192}));
        connect(Tdev.OutPort1,BJTvars1. Tdev) annotation (points=[108,34.8; 108,
              28; 164,28; 164,-136; 90,-136; 90,-123],   style(color=74,
              rgbcolor={0,0,127}));
        connect(J1p3_1.BondCon1, B7.BondCon2) annotation (points=[-6.12303e-016,
              90; -1.83691e-015,90], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_4.BondCon2, B7.BondCon1) annotation (points=[-6.12303e-016,
              110; 1.80017e-015,110; 1.80017e-015,109.8], style(color=8,
              rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon1, B1.BondCon2) annotation (points=[-6.12303e-016,
              10; -1.83691e-015,10], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon2, B1.BondCon1) annotation (points=[6.12303e-016,
              70; 1.80017e-015,70; 1.80017e-015,69.4], style(color=8, rgbcolor=
                {192,192,192}));
        connect(J0p3_1.BondCon2, B2.BondCon2) annotation (points=[6.12303e-016,
              -10; 6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
        connect(B2.BondCon1, J1p3_2.BondCon1) annotation (points=[-6.00057e-016,
              -69.4; -6.00057e-016,-68.7; -6.12303e-016,-68.7; -6.12303e-016,
              -70], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_2.BondCon2, B8.BondCon2) annotation (points=[6.12303e-016,
              -90; 6.12303e-016,-90], style(color=8, rgbcolor={192,192,192}));
        connect(B8.BondCon1, J0p3_3.BondCon1) annotation (points=[-6.00057e-016,
              -109.8; -6.00057e-016,-107.9; 6.12303e-016,-107.9; 6.12303e-016,
              -110], style(color=8, rgbcolor={192,192,192}));
        connect(B10.BondCon2, J1p3_5.BondCon2) annotation (points=[34,-10; 34,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_3.BondCon3, B10.BondCon1) annotation (points=[10,-120; 34,
              -120; 34,-69.4], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_5.BondCon1, B9.BondCon1) annotation (points=[34,10; 34,
              10.6], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_4.BondCon3, B9.BondCon2) annotation (points=[10,120; 34,
              120; 34,70], style(color=8, rgbcolor={192,192,192}));
        connect(BJTvars1.ib0, Gain1.u) annotation (points=[123,-72; 160,-72;
              160,-144],        style(color=74, rgbcolor={0,0,127}));
        connect(B11.eBondCon1, J1p3_5.BondCon3) annotation (points=[44,
              1.22461e-015; 44,6.12303e-016], style(color=8, rgbcolor={192,192,
                192}));
        connect(IC0.BondCon2, B11.fBondCon1) annotation (points=[64,
              1.77636e-015; 64,-1.22461e-015], style(color=8, rgbcolor={192,192,
                192}));
        connect(IC0.BondCon1, B17.fBondCon1) annotation (points=[94,
              1.77636e-015; 94,1.22461e-015], style(color=8, rgbcolor={192,192,
                192}));
        connect(Dbc.BondCon1, B13.BondCon2) annotation (points=[-70,80; -70,80],
            style(color=8, rgbcolor={192,192,192}));
        connect(B14.fBondCon1, Dbc.BondCon2) annotation (points=[-110,80; -110,
              80], style(color=8, rgbcolor={192,192,192}));
        connect(Dbc.ix, ix) annotation (points=[-80,94; -80,120; -70,120; -70,
              140], style(color=74, rgbcolor={0,0,127}));
        connect(Dbc.id, BJTvars1.ibc) annotation (points=[-100,94; -100,110;
              -156,110; -156,-164; 42,-164; 42,-84; 57,-84], style(color=74,
              rgbcolor={0,0,127}));
        connect(qb1.y, Dbc.qb) annotation (points=[-105.4,50; -100,50; -100,66],
            style(color=74, rgbcolor={0,0,127}));
        connect(C0.y, Dbc.C0) annotation (points=[-74.6,50; -80,50; -80,66],
            style(color=74, rgbcolor={0,0,127}));
        connect(Dbe.BondCon1, B15.BondCon2) annotation (points=[-70,-80; -70,
              -80], style(color=8, rgbcolor={192,192,192}));
        connect(B16.fBondCon1, Dbe.BondCon2) annotation (points=[-110,-80; -110,
              -80], style(color=8, rgbcolor={192,192,192}));
        connect(Dbe.id, BJTvars1.ibe) annotation (points=[-100,-66; -100,-48;
              -164,-48; -164,-172; 50,-172; 50,-108; 57,-108], style(color=74,
              rgbcolor={0,0,127}));
        connect(BJTvars1.C0, Dbe.C0) annotation (points=[123,-108; 128,-108;
              128,-152; -80,-152; -80,-94], style(color=74, rgbcolor={0,0,127}));
        connect(BJTvars1.qb, Dbe.qb) annotation (points=[123,-96; 132,-96; 132,
              -156; -100,-156; -100,-94], style(color=74, rgbcolor={0,0,127}));
        connect(J0p6_1.BondCon4, B19.eBondCon1) annotation (points=[108,80; 108,
              80], style(color=8, rgbcolor={192,192,192}));
        connect(B19.fBondCon1, Tdev.BondCon1) annotation (points=[108,60; 108,
              60], style(color=8, rgbcolor={192,192,192}));
          connect(BJTvars1.ic0, IC0.s) annotation (points=[123,-84; 130,-84;
              130,-30; 79,-30; 79,-12.6], style(color=74, rgbcolor={0,0,127}));
          connect(Gain1.y, IB0.s) annotation (points=[160,-167; 160,-176; -16,
              -176; -16,-125; -25.6,-125], style(color=74, rgbcolor={0,0,127}));
        end PNPint;
      
        function MOSeq1 
        "Computation of injected drain-source current for Levels = 0,1" 
          input Modelica.SIunits.Voltage vds "Drain-source voltage";
          input Modelica.SIunits.Voltage vgs "Gate-source voltage";
          input Modelica.SIunits.Voltage vbs "Bulk-source voltage";
          input Modelica.SIunits.Temperature Tdev "Device temperature";
          input Real sig "sign(vds)";
          input Integer Type "Type=1 for NMOS; Type=-1 for PMOS";
          input Modelica.SIunits.Temperature Tnom "Reference temperature";
          input Modelica.SIunits.Voltage VFB 
          "Flat band voltage at reference temperature";
          input Modelica.SIunits.Transconductance KP 
          "Transconductance parameter at reference temperature";
          input Real GAMMA(unit="V^0.5") "Body-effect parameter";
          input Modelica.SIunits.Voltage PHI 
          "Surface inversion potential at reference temperature";
          input Modelica.SIunits.InversePotential LAMBDA 
          "Channel length modulation";
          input Modelica.SIunits.Length L "Effective hannel length";
          input Modelica.SIunits.Length W "Effective channel width";
          input Modelica.SIunits.Voltage EG 
          "Energy gap for temperature effect on saturation current at 0 K";
          output Modelica.SIunits.Current ids0 "Injected drain-source current";
          output Modelica.SIunits.Voltage Vfb 
          "Flat band voltage at device temperature";
          output Modelica.SIunits.Voltage Vth "Threshold voltage";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
        
      protected 
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          Real RTemp "Temperature quotient";
          Real facTemp "Temperature coefficient";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Energy EGref "Energy gap at reference temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Voltage PHIval 
          "Surface inversion potential at device temperature";
          Modelica.SIunits.Transconductance KPval 
          "Transconductance parameter at device temperature";
          Modelica.SIunits.Transconductance BETA 
          "Transconductance at device temperature";
          Modelica.SIunits.Transconductance Beta 
          "Adjusted transconductance at device temperature";
          Real cdlin(unit="V2") "Linear region current factor";
          Real cdsat(unit="V2") "Saturation region current factor";
        
          /* Auxiliary variables */
          Real PHIroot(unit="V^0.5");
          Real Sarg(unit="V^0.5");
          Modelica.SIunits.Voltage Vgst "vgs - vth";
        annotation (Diagram(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-56,-18; 36,-46],
              style(color=42, rgbcolor={220,0,0}),
              string="Levels = 0,1"),
              Text(
                extent=[-70,8; 50,-22],
                string="of a MOS transistor",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0}))), Icon(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-70,8; 50,-22],
                string="of a MOS transistor",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-56,-18; 36,-46],
              style(color=42, rgbcolor={220,0,0}),
              string="Levels = 0,1")),
          Documentation(info="<html>
This function implements a Spice-style <b>Level-1</b> model of the current flowing between drain and source of a MOSFET in accrodance with the theory of Shichman and Hodges [1].  It is a simple model that computes the current in its linear and saturation regions only.  No channel modulation effects are considered, and the current is set equal to zero in the subthreshold region of the device. <p>

The current injection model is identical for Modelica-Spice <b>Level-0</b> and <b>Level-1</b>. <p>

The model comments the equations by providing the corresponding equation numbers from [2]. <p>

<hr> <p>

<pre>
Input variables:

 vds:     Drain-source voltage (Volt)

 vgs:     Gate-source voltage (Volt)

 vbs:     Bulk-source voltage (Volt)

 Tdev:    Device temperature (K)

 sig:     Indicator of operating mode
            sig = +1:  direct mode
            sig = -1:  reverse mode (source and drain are flipped)


Input parameters:

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)


Output variables:

 isd0:    Drain-source current (Amp)

 Vfb:     Flat band voltage at device temperature (Volt)

 Vth:     Threshold voltage (Volt)

 vdsat:   Saturation voltage (Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Shichman, H. and D.A. Hodges (1968), \"Modeling and Simulation of Insulated-Gate Field-Effector Transistor Switching Circuits,\" <i>IEEE J. Solid-State Circuits</i>, <b>3</b>(3), pp. 285-289.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        algorithm 
          /* Parameter computations and temperature variations (implements TMPUPD) */
          RTemp := Tdev/Tnom;
          facTemp := RTemp*sqrt(RTemp);
          Vt := k*Tdev/q;
          EGref := EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          EGval := EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
          PHIval := RTemp*PHI + EGval - RTemp*EGref - 3*Vt*ln(RTemp);
          Vfb := VFB + Type*(PHIval - PHI)/2 + (EGref - EGval)/2;
          KPval := KP/facTemp;
          BETA := KPval*W/L;
        
          /* Compute drain-source injection current */
          PHIroot := sqrt(PHIval);
          Sarg := if vbs <= 0 then sqrt(PHIval - vbs) else max({0,PHIroot - vbs/(2*PHIroot)});
          Vth := Vfb + Type*(PHIval + GAMMA*Sarg);  /* (4-13) p.172 */
          Vgst := vgs - Type*Vth;
          vdsat := if Vgst > 0 then Vgst else 0;
          Beta := BETA*(1 + LAMBDA*vds);
          cdlin := vds*(Vgst - vds/2);  /* (4-29) p.174 */
          cdsat := Vgst*Vgst/2;  /* (4-31) p.175 */
          ids0 := sig*Beta*(if Vgst > 0 and vds <= vdsat then cdlin else 
                            if Vgst > 0 and vds > vdsat then cdsat else 0);
        end MOSeq1;
      
        function MOSeq2 
        "Computation of injected drain-source current for Level=2" 
          input Modelica.SIunits.Voltage vds "Drain-source voltage";
          input Modelica.SIunits.Voltage vgs "Gate-source voltage";
          input Modelica.SIunits.Voltage vbs "Bulk-source voltage";
          input Modelica.SIunits.Temperature Tdev "Device temperature";
          input Real sig "sign(vds)";
          input Integer Level=2 "Level of MOS model";
          input Integer Type "Type=1 for NMOS; Type=-1 for PMOS";
          input Modelica.SIunits.Temperature Tnom "Reference temperature";
          input Modelica.SIunits.Voltage VFB 
          "Flat band voltage at reference temperature";
          input Modelica.SIunits.Transconductance KP 
          "Transconductance parameter at reference temperature";
          input Real GAMMA(unit="V^0.5") "Body-effect parameter";
          input Modelica.SIunits.Voltage PHI 
          "Surface inversion potential at reference temperature";
          input Modelica.SIunits.InversePotential LAMBDA 
          "Channel length modulation";
          input Real COX(unit="F/m2") "Specific capacitance of SiO2";
          input Real NFS(unit="1/m2") "Surface fast state density";
          input Real NEFF "Total channel charge coefficient";
          input Modelica.SIunits.Length XJ "Metallurgical junction depth";
          input Modelica.SIunits.Length L "Effective hannel length";
          input Modelica.SIunits.Length W "Effective channel width";
          input Real XD(unit="m/V^0.5") "Depletion factor";
          input Real U0(unit="m2/(V.s)") 
          "Surface mobility at reference temperature";
          input Real UEXP "Exponential coefficient for mobility";
          input Real UTRA "Transverse field coefficient";
          input Modelica.SIunits.Velocity VMAX 
          "Maximum drift velocity of carriers";
          input Modelica.SIunits.ElectricFieldStrength ECRIT 
          "Critical electric field for pinch-off";
          input Real DELTA "Width effect on threshold voltage";
          input Modelica.SIunits.Voltage VBP "Gate to channel critical voltage";
          input Modelica.SIunits.Voltage PB 
          "Built-in potential at reference temperature";
          input Modelica.SIunits.Voltage EG 
          "Energy gap for temperature effect on saturation current at 0 K";
          input Real EMin "if x < EMin, the exp(x) function is linearized";
          input Real EMax "if x > EMax, the exp(x) function is linearized";
          output Modelica.SIunits.Current ids0 "Injected drain-source current";
          output Modelica.SIunits.Voltage Vfb 
          "Effective flat band voltage at device temperature";
          output Modelica.SIunits.Voltage Vth "Threshold voltage";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
          output Modelica.SIunits.Voltage vdsat1 "Adjusted saturation voltage";
          output Modelica.SIunits.Voltage vpof "Pinch-off voltage";
        
      protected 
          constant Real pi=Modelica.Constants.pi;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
          "Permittivity of free space";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real ks=11.8 "Dielectric constant of Si";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          constant Integer Scat1[4] = {1,-1,1,-1} "First scattering vector";
          constant Integer Scat2[4] = {1,1,-1,-1} "Second scattering vector";
          parameter Modelica.SIunits.Permittivity es = ks*e0 
          "Permittivity of silicon";
          parameter Real Fn = if Level==2 then es*DELTA*pi/(2*COX*W) else 0;
          parameter Real Fn2 = 1 + Fn;
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Real RTemp "Temperature quotient";
          Real facTemp "Temperature coefficient";
          Modelica.SIunits.Energy EGref "Energy gap at reference temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Voltage PHIval 
          "Surface inversion potential at device temperature";
          Modelica.SIunits.Voltage Von "Adjusted threshold voltage";
          Modelica.SIunits.Voltage Vgst "vgs - vth";
          Modelica.SIunits.Voltage VGSval "Effective vgs";
          Modelica.SIunits.Voltage VDSAT "Initial saturation voltage";
          Modelica.SIunits.Voltage VDSATval "Adjusted saturation voltage";
          Modelica.SIunits.Voltage vdson "Subthreshold voltage";
          Modelica.SIunits.Current ids "Injected drain-source current";
          Real cdlin(unit="V2") "Linear region current factor";
          Real cdsat(unit="V2") "Saturation region current factor";
          Real cdson(unit="V2") "Subthreshold region current coefficient";
          Real cdsub(unit="V2") "Subthreshold region current factor";
          Real GAMMAval(unit="V^0.5") "Adjusted body effect parameter";
          Real GAMMAmod(unit="V^0.5") "Modified body effect parameter";
          Real DGAMMA(unit="1/V^0.5") "der(GAMMAval,vbs)";
          Real et "Subthreshold voltage coefficient";
          Modelica.SIunits.Transconductance KPval 
          "Transconductance parameter at device temperature";
          Modelica.SIunits.Transconductance BETA;
          Real Ueff(unit="m2/(V.s)") 
          "Effective surface mobility at reference temperature";
          Modelica.SIunits.Length WS "Source depletion region width";
          Modelica.SIunits.Length WD "Drain depletion region width";
          Modelica.SIunits.Length WB "Punch-through width";
          Modelica.SIunits.Length Leff "Modulated channel length";
          Modelica.SIunits.Length LEFFval "Effective channel length";
          Modelica.SIunits.Length LD "Channel length modulation";
          Modelica.SIunits.InversePotential LAMBDAval 
          "Effective channel length modulation";
          Real CB(unit="F/m2") "Specific capacitance of depleted region";
          Modelica.SIunits.ElectronNumberDensity N 
          "Intrinsic carrier concentration at device temperature";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vgamma;
          Modelica.SIunits.Voltage Vden;
          Modelica.SIunits.Voltage Vdiff;
          Modelica.SIunits.Voltage Vdiff2;
          Modelica.SIunits.Voltage Vdiff3;
          Modelica.SIunits.Voltage Vdiff4;
          Modelica.SIunits.Voltage Vdiff5;
          Modelica.SIunits.Voltage V1;
          Modelica.SIunits.Voltage V2;
          Modelica.SIunits.Voltage Vd;
          Modelica.SIunits.Voltage X1b;
          Modelica.SIunits.Voltage X2a;
          Modelica.SIunits.Voltage X4;
          Modelica.SIunits.Voltage X5b;
          Modelica.SIunits.Voltage X6b[4];
          Modelica.SIunits.Voltage X7;
          Modelica.SIunits.Voltage VQChan;
          Real VSq(unit="V2");
          Real X1d(unit="V2");
          Real X2b(unit="V2");
          Real X9[8](unit="V2");
          Real X2c(unit="V3");
          Real X3(unit="V3");
          Real Psq(unit="V3");
          Real P(unit="V6");
          Real Pabs(unit="V6");
          Real PHIroot(unit="V^0.5");
          Real Sarg(unit="V^0.5");
          Real Barg(unit="V^0.5");
          Real BARGsat(unit="V^0.5");
          Real gdsat(unit="V^0.5");
          Real LV(unit="V^0.5");
          Real LS(unit="V^0.5");
          Real X1a(unit="V^0.5");
          Real X5a(unit="V^0.5");
          Real X6a[4](unit="V^0.5");
          Real X8[8](unit="V^0.5");
          Real PHI3(unit="V^1.5");
          Real BS3(unit="V^1.5");
          Real BS3sat(unit="V^1.5");
          Real BS3eff(unit="V^1.5");
          Real X1c(unit="V^1.5");
          Real DSarg(unit="1/V^0.5") "der(Sarg,vbs)";
          Real DBarg(unit="1/V^0.5") "der(Barg,vbs)";
          Real Lfact(unit="1/V^1.5");
          Modelica.SIunits.Length DeltaL;
          Modelica.SIunits.Transconductance BETAval;
          Real DV(unit="m/V^0.5");
          Real WSfac;
          Real WDfac;
          Real Ufact;
          Real Lfact2;
          Real LFACTval;
          Integer icnt;
          Integer jcnt;
        annotation (Diagram(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-52,-20; 20,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 2"),
              Text(
                extent=[-70,8; 50,-22],
                string="of a MOS transistor",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0}))), Icon(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-52,-20; 20,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 2"),
              Text(
                extent=[-70,8; 50,-22],
                string="of a MOS transistor",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0}))),
          Documentation(info="<html>
This function implements a Spice-style <b>Level-2</b> model of the current flowing between drain and source of a MOSFET.  It is one of the most complex models currently available that computes the current in its linear, saturation, and subthreshold regions.  Various channel modulation effects are also considered.  The saturation voltage is computed either in accordance with the Grove-Frohman model [1,2] or using Baum's theory of scattering velocity saturation [3]. <p>

The model simulates very slowly, and is probably not useful except for simulating across a single switching event. <p>

The model comments the equations by providing the corresponding equation numbers from [4]. <p>

<hr> <p>

<pre>
Input variables:

 vds:     Drain-source voltage (Volt)

 vgs:     Gate-source voltage (Volt)

 vbs:     Bulk-source voltage (Volt)

 Tdev:    Device temperature (K)

 sig:     Indicator of operating mode
            sig = +1:  direct mode
            sig = -1:  reverse mode (source and drain are flipped)


Input parameters:

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 NEFF:    Total channel charge coefficient (default value = 1)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s))

 UEXP:    Exponential coefficient for mobility (default value = 0)

 UTRA:    Transverse field coefficient (default value = 0)

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s)
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation</b></font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m)

 DELTA:   Width effect on threshold voltage (default value = 0)

 VBP:     Gate to channel critical voltage (default value = 0 Volt)

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-3</font>


Output variables:

 isd0:    Drain-source current (Amp)

 Vfb:     Flat band voltage at device temperature (Volt)

 Vth:     Threshold voltage (Volt)

 vdsat:   Saturation voltage (Volt)

 vdsat1:  Adjusted saturation voltage (Volt)

 vpof:    Pinch-off voltage (Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Frohman-Bentchkowsky, D. (1968), \"On the Effect of Mobility Variation on MOS Device Characteristics,\" <i>Proc. IEEE</i>, <b>56</b>, pp.217-218.
<li>Frohman-Bentchkowsky, D. and A.S. Grove (1969), \"Conductance of MOS Transistors in Saturation,\" <i>IEEE Trans. Electron Devices</i>, <b>ED-16</b>, pp. 108-113.
<li>Baum, G. and H. Beneking (1970), \"Drift Velocity Saturation in MOS Transistors,\" <i>IEEE Trans. Electron Devices</i>, <b>ED-17</b>, pp. 481-482.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        algorithm 
          /* Parameter computations and temperature variations (implements TMPUPD ) */
          RTemp := Tdev/Tnom;
          facTemp := RTemp*sqrt(RTemp);
          Vt := k*Tdev/q;
          EGref := EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          EGval := EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
          PHIval := RTemp*PHI + EGval - RTemp*EGref - 3*Vt*ln(RTemp);
          Vfb := VFB + Type*(PHIval - PHI)/2 + (EGref - EGval)/2;
          KPval := KP/facTemp;
          BETA := KPval*W/L;
        
          /* Compute Sarg and Barg and their derivatives */
          PHIroot := sqrt(PHIval);
          PHI3 := PHIval*PHIroot;
          Sarg := if vbs <= 0 then sqrt(PHIval - vbs) else PHIroot/(1 + vbs/(2*PHIval));
          DSarg := if vbs <= 0 then -0.5/Sarg else -0.5*Sarg*Sarg/PHI3;
          Barg := if vbs < vds then sqrt(PHIval + vds - vbs) else PHIroot/(1 + (vbs - vds)/(2*PHIval));
          DBarg := if vbs < vds then -0.5/Barg else -0.5*Barg*Barg/PHI3;
          BS3 := Barg^3 - Sarg^3;
        
          /* Compute threshold voltage (narrow channel effect) */
          WS := XD*Sarg;  /* (4-57) p.189 */
          WD := XD*Barg;  /* (4-58) p.189 */
          WSfac := if XJ > 0 and Level==2 then sqrt(1 + 2*WS/XJ) else 1;
          WDfac := if XJ > 0 and Level==2 then sqrt(1 + 2*WD/XJ) else 1;
          GAMMAval := GAMMA*(if XJ > 0 then 1 - XJ*(WSfac + WDfac - 2)/(2*L) else 1);  /* (4-56) p.189 */
          DGAMMA := GAMMA*(if XJ > 0 then -XD*(DSarg/WSfac + DBarg/WDfac)/(2*L) else 0);
          CB := Fn - GAMMAval*DSarg - DGAMMA*Sarg;  /* (4-48) p.183 */
          N := if NFS > 0 then 1 + q*NFS/COX + CB else 1;  /* (4-47) p.183 */
          Vth := Vfb + Type*(PHIval + (GAMMA - GAMMAval)*sqrt(PHIval) + GAMMAval*Sarg + Fn*Sarg*Sarg/2);  /* (4-62) p.195 */
          Von := if NFS > 0 then Type*Vth + N*Vt else Type*Vth;  /* (4-46) p.183 */
          Vgst := vgs - Von;
        
          /* Compute effective carrier mobility */
          Vden := vgs - Type*Vth;
          Ufact := if Vden > VBP then exp(UEXP*ln(VBP/Vden)) else 1;  /* (4-45) p.182 */
          Ueff := Ufact*U0;
        
          /* Determine saturation voltage */
          /* Use Grove-Frohman equation */
          GAMMAmod := GAMMAval/Fn2;
          Vgamma := GAMMAmod*GAMMAmod;
          if GAMMAmod > 0 then
            VGSval := if NFS > 0 then max({vgs,Von}) else vgs;
            Vdiff := (VGSval - Type*Vfb - PHIval)/Fn2;
            Vdiff2 := Vdiff + PHIval - vbs;
            if Vdiff2 > 0 then
              Vdiff3 := VGSval - Type*Vfb - PHIval + 0.5*GAMMAval^2*(1 - sqrt(1 + 4/GAMMAval^2*(VGSval - Type*Vfb - vbs)));  /* (4-42) p.181 */
              VDSAT := if Vdiff3 > 0 then Vdiff3 else 0;
              VSq := Vgamma*(vds + PHIval - vbs)/Fn2^2;
              vpof := if VSq < 0 then Type*Vth else Type*Vfb + Fn2*(vds + sqrt(VSq));
              Vdiff3 := (vpof - Type*Vfb - PHIval)/Fn2;
              Vdiff4 := (vpof - Type*Vfb - PHIval)/Fn2 + PHIval - vbs;
              Vdiff5 := Vdiff3 + Vgamma*(1 - sqrt(1 + 2*Vdiff4/Vgamma));
              vdsat1 := if Vdiff4 > 0 and Vdiff5 > 0 then Vdiff5 else 0;
            else
              VDSAT := 0;
              vpof := Type*Vth;
              vdsat1 := 0;
            end if;
          else
            VGSval := vgs;
            VDSAT := if Vdiff > 0 then Vdiff else 0;
            vpof := if Fn2*vds + Type*Vfb > 0 then Fn2*vds + Type*Vfb else 0;
            vdsat1 := if (vpof - Type*Vfb)/Fn2 > 0 then (vpof - Type*Vfb)/Fn2 else 0;
          end if;
          VDSATval := if ECRIT > 0 then VDSAT + ECRIT*L - sqrt(VDSAT*VDSAT + (ECRIT*L)^2) else VDSAT;
        
          /* Use Baum's theory of scattering velocity saturation */
          if VMAX > 0 then
            V1 := (VGSval - Type*Vfb)/Fn2 + PHIval - vbs;
            V2 := PHIval - vbs;
            Vd := VMAX*L/Ueff;
            X1a := 4*GAMMAmod/3;
            X1b := -2*(V1 + Vd);
            X1c := -2*GAMMAmod*Vd;
            X1d := 2*V1*(V2 + Vd) - V2*V2 - X1a*Sarg^3;
            X2a := -X1b;
            X2b := X1a*X1c - 4*X1d;
            X2c := -X1d*(X1a*X1a - 4*X1b) - X1c*X1c;
            X3 := 2*X2a^3/27 - X2a*X2b/3 + X2c;
            P := X3*X3/4 + (X2b - X2a*X2a/3)^3/27;
            Pabs := if P > 0 then P else -P;
            Psq := sqrt(Pabs);
            X4 := (if P < 0 then 2*exp(ln(sqrt(X3*X3/4 + Pabs))/3)*cos(Modelica.Math.atan(-2*Psq/X3)/3) else 
                                 exp(ln(abs(-X3/2 + Psq))/3) + exp(ln(abs(-X3/2 - Psq))/3)) - X2a/3;
            X5a := sqrt(X1a*X1a/4 + X4 - X1b);
            X5b := sqrt(X4*X4/4 - X1d);
            icnt := 0;
            for i in 1:4 loop
              X6a[i] := X1a/2 + Scat1[i]*X5a;
              X6b[i] := X4/2 + Scat2[i]*X5b;
              X7 := X6a[i]*X6a[i]/4 - X6b[i];
              if X7 >= 0 then
                icnt := icnt + 1;
                X8[icnt] := -X6a[i]/2 + sqrt(X7);
                icnt := icnt + 1;
                X8[icnt] := -X6a[i]/2 - sqrt(X7);
              end if;
            end for;
            jcnt := 0;
            for i in 1:icnt loop
              if X8[i] > 0 then
                X9[i] := X1d + X8[i]*(X1c + X8[i]*(X1b + X8[i]*(X1a + X8[i])));
                gdsat := 0;
                if abs(X9[i]) < 1e-6 then
                  jcnt := jcnt + 1;
                  if jcnt < 2 or X8[i] <= gdsat then
                    gdsat := X8[i];
                  end if;
                end if;
              end if;
            end for;
            vdsat := if jcnt > 0 then gdsat*gdsat + vbs - PHIval else VDSATval;
          else
            vdsat := VDSATval;
          end if;
        
          /* Compute channel length modulation */
          if vds > 0 then
            BARGsat := if vbs < vdsat then sqrt(PHIval + vdsat - vbs) else PHIroot/(1 + (vbs - vdsat)/(2*PHIval));
            BS3sat := BARGsat^3 - Sarg^3;
            if LAMBDA == 0 then
              if VMAX == 0 then
                Lfact := XD/(L*vds);
                Vdiff3 := (vds - vdsat)/4;
                LAMBDAval := Lfact*sqrt(Vdiff3 + sqrt(1 + Vdiff3*Vdiff3));  /* (4-54) p.187 */
              else
                DV := XD/sqrt(NEFF);
                LV := VMAX*DV/(2*Ueff);
                VQChan := Vdiff - vdsat - GAMMAval*BARGsat;
                LS := sqrt(LV*LV + (if vds > vdsat then vds - vdsat else 0));
                Lfact := DV/(L*vds);
                LAMBDAval := Lfact*(LS - LV);  /* (4-60) p.192 */
              end if;
            else
              LAMBDAval := LAMBDA;
            end if;
          else
            BS3sat := BS3;
          end if;
          BS3eff := if Vgst <= 0 and vdsat > 0 then BS3sat else BS3;
        
          /* Compute channel shortening at punch-through */
          WB := XD*sqrt(PB);
          LD := L - WB;
          Lfact2 := 1 - LAMBDAval*vds;
          Leff := Lfact2*L;
          DeltaL := LAMBDAval*vds*L;
          if Leff < WB then
            LEFFval := WB/(1 + (DeltaL - LD)/WB);
            LFACTval := LEFFval/L;
          else
            LEFFval := Leff;
            LFACTval := Lfact2;
          end if;
        
          /* Compute effective BETA value */
          BETAval := BETA*Ufact/LFACTval;
        
          /* Compute drain-source current */
          /* Linear region */
          cdlin := vds*(vgs - Type*Vfb - PHIval - Fn2*vds/2) - 2*GAMMAval*BS3eff/3;  /* (4-43) p.182 */
          /* Saturation region */
          cdsat := vdsat*(vgs - Type*Vfb - PHIval - Fn2*vdsat/2) - 2*GAMMAval*BS3sat/3;
          /* Subthreshold region */
          vdson := if vdsat < vds then vdsat else vds;
          cdson := vdson*(Von - Type*Vfb - PHIval - Fn2*vdson/2) - 2*GAMMAval*BS3eff/3;
          et := (vgs - Von)/(N*Vt);
          cdsub := cdson*(if et < EMin then ExMin*(et - EMin + 1) else 
                          if et > EMax then ExMax*(et - EMax + 1) else  exp(et));  /* (4-49) p.183 */
          ids := sig*BETAval*(if Vgst > 0 and vds <= vdsat then cdlin else 
                              if Vgst > 0 and vds > vdsat then cdsat else 
                              if Vgst <= 0 and vdsat > 0 then cdsub else 0);
          ids0 := if vds > 1e-8 then ids else 0;
        end MOSeq2;
      
        function MOSeq3 
        "Computation of injected drain-source current for Level=3" 
          input Modelica.SIunits.Voltage vds "Drain-source voltage";
          input Modelica.SIunits.Voltage vgs "Gate-source voltage";
          input Modelica.SIunits.Voltage vbs "Bulk-source voltage";
          input Modelica.SIunits.Temperature Tdev "Device temperature";
          input Real sig "sign(vds)";
          input Integer Type "Type=1 for NMOS; Type=-1 for PMOS";
          input Modelica.SIunits.Temperature Tnom "Reference temperature";
          input Modelica.SIunits.Voltage VFB 
          "Flat band voltage at reference temperature";
          input Modelica.SIunits.Transconductance KP 
          "Transconductance parameter at reference temperature";
          input Real GAMMA(unit="V^0.5") "Body-effect parameter";
          input Modelica.SIunits.Voltage PHI 
          "Surface inversion potential at reference temperature";
          input Real COX(unit="F/m2") "Specific capacitance of SiO2";
          input Real NFS(unit="1/m2") "Surface fast state density";
          input Modelica.SIunits.Length XJ "Metallurgical junction depth";
          input Modelica.SIunits.Length L "Effective hannel length";
          input Modelica.SIunits.Length W "Effective channel width";
          input Modelica.SIunits.Length LD "Lateral diffusion";
          input Real XD(unit="m/V^0.5") "Depletion factor";
          input Real U0(unit="m2/(V.s)") 
          "Surface mobility at reference temperature";
          input Modelica.SIunits.Velocity VMAX 
          "Maximum drift velocity of carriers";
          input Real DELTA "Width effect on threshold voltage";
          input Real KAPPA "Saturation field factor";
          input Real ETA "Static feedback on threshold voltage";
          input Modelica.SIunits.InversePotential THETA "Mobility modulation";
          input Modelica.SIunits.Voltage EG 
          "Energy gap for temperature effect on saturation current at 0 K";
          input Real EMin "if x < EMin, the exp(x) function is linearized";
          input Real EMax "if x > EMax, the exp(x) function is linearized";
          output Modelica.SIunits.Current ids0 "Injected drain-source current";
          output Modelica.SIunits.Voltage Vfb 
          "Effective flat band voltage at device temperature";
          output Modelica.SIunits.Voltage Vth "Threshold voltage";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
          output Modelica.SIunits.Voltage vpof "Pinch-off voltage";
        
      protected 
          constant Real pi=Modelica.Constants.pi;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
          "Permittivity of free space";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real ks=11.8 "Dielectric constant of Si";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          constant Real c0=0.0631353;
          constant Real c1=0.8013292;
          constant Real c2=-0.01110777;
          parameter Modelica.SIunits.Permittivity es = ks*e0 
          "Permittivity of silicon";
          Real Fn = es*DELTA*pi/(2*COX*W) "Working function";  /* (4-99) p.207 */
          Real SIGMA = 8.15e-22*ETA/(COX*L^3) 
          "Static feedback on threshold voltage";                                      /* (4-94) p.205 */
          Real ExMin = exp(EMin);
          Real ExMax = exp(EMax);
          Real RTemp "Temperature quotient";
          Real facTemp "Temperature coefficient";
          Modelica.SIunits.Energy EGref "Energy gap at reference temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Voltage PHIval 
          "Surface inversion potential at device temperature";
          Modelica.SIunits.Voltage Von "Adjusted threshold voltage";
          Modelica.SIunits.Voltage Vgst "vgs - vth";
          Modelica.SIunits.Current idlin "Linear current";
          Modelica.SIunits.Current idsat "Saturation current";
          Modelica.SIunits.Current ids "Drain-source current";
          Modelica.SIunits.Conductance gdsat "Saturation region conductance";
          Real cdlin(unit="V2") "Linear region current factor";
          Real cdsat(unit="V2") "Saturation region current factor";
          Real cdsub(unit="V2") "Subthreshold region current factor";
          Real et "Subthreshold voltage coefficient";
          Modelica.SIunits.Transconductance KPval 
          "Transconductance parameter at device temperature";
          Modelica.SIunits.Transconductance BETA 
          "Transconductance at device temperature";
          Modelica.SIunits.Transconductance BETAval 
          "Adjusted transconductance at device temperature";
          Modelica.SIunits.Transconductance BETAeff 
          "Adjusted transconductance at device temperature";
          Modelica.SIunits.Transconductance Beta 
          "Final transconductance at device temperature";
          Modelica.SIunits.InversePotential LAMBDAval 
          "Channel length modulation";
          Modelica.SIunits.Length Wp 
          "Width of depleted region in flat source junction";
          Modelica.SIunits.Length Wc 
          "Width of depleted region in source substrate junction";
          Real WpXJ "WP/XJ";
          Real WcXJ "WC/XJ";
          Real Us(unit="m2/(V.s)") 
          "Dependence of mobility on gate electric field";
          Real CB(unit="F/m2") "Specific capacitance of depleted region";
          Modelica.SIunits.ElectronNumberDensity N 
          "Intrinsic carrier concentration at device temperature";
          Modelica.SIunits.ElectricFieldStrength Ep 
          "Lateral field at channel pinch-off";
          Real Fs "Short channel working function";
          Real FB "Body effect working function";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vbody;
          Modelica.SIunits.Voltage vgsx;
          Modelica.SIunits.Voltage vdsx;
          Modelica.SIunits.Voltage Va;
          Modelica.SIunits.Voltage Vb;
          Modelica.SIunits.Length DL;
          Modelica.SIunits.Length DLval;
          Modelica.SIunits.Length DLfac;
          Real ALPHA(unit="m2/V");
          Real Sarg(unit="Volt^0.5");
          Real Lfact;
          Real Fd;
          Real FG;
        annotation (
          Documentation(info="<html>
This function implements a Spice-style <b>Level-3</b> model of the current flowing between drain and source of a MOSFET.  It is is an empirical model that computes the current in its linear, saturation, and weak inversion regions.  Various channel modulation effects are also considered. <p>

The model offers a compromise between the much more sophisticated <b>Level-2</b> model that, however, simulates very slowly, and the often too simple <b>Level-1</b> model that doesn't allow many questions to be answered, such as what is the highest possible switching frequency reachable, how much power is being dissipated at high frequency operation, what is the amount of substrate cross-talk, etc. <p>

The model comments the equations by providing the corresponding equation numbers from [1]. <p>

<hr> <p>

<pre>
Input variables:

 vds:     Drain-source voltage (Volt)

 vgs:     Gate-source voltage (Volt)

 vbs:     Bulk-source voltage (Volt)

 Tdev:    Device temperature (K)

 sig:     Indicator of operating mode
            sig = +1:  direct mode
            sig = -1:  reverse mode (source and drain are flipped)


Input parameters:

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s))

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s)

 DELTA:   Width effect on threshold voltage (default value = 0)

 KAPPA:   Saturation field factor (default value = 0.2)

 ETA:     Static feedback on threshold voltage (default value = 0)

 THETA:   Mobility modulation (default value = 0 1/Volt)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-3</font>


Output variables:

 isd0:    Drain-source current (Amp)

 Vfb:     Flat band voltage at device temperature (Volt)

 Vth:     Threshold voltage (Volt)

 vdsat:   Saturation voltage (Volt)

 vpof:    Pinch-off voltage (Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"),       Icon(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-52,-20; 20,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 3"),
              Text(
                extent=[-70,8; 50,-22],
                string="of a MOS transistor",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0}))),
          Diagram(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-52,-20; 20,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 3"),
              Text(
                extent=[-70,8; 50,-22],
                string="of a MOS transistor",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0}))));
        algorithm 
          /* Parameter computations and temperature variations (Implements TMPUPD) */
          RTemp := Tdev/Tnom;
          facTemp := RTemp*sqrt(RTemp);
          Vt := k*Tdev/q;
          EGref := EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          EGval := EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
          PHIval := RTemp*PHI + EGval - RTemp*EGref - 3*Vt*ln(RTemp);
          Vfb := VFB + Type*((PHIval - PHI)/2 + (EGref - EGval)/2);
          KPval := KP/facTemp;
          BETA := KPval*W/L;
        
          /* Compute short channel effects */
          Sarg := if vbs <= 0 then sqrt(PHIval - vbs) else sqrt(PHIval)/(1 + vbs/(2*PHIval));
          if XJ > 0 then
            Wp := Sarg*XD;  /* (4-96) p.205 */
            WpXJ := Wp/XJ;
            WcXJ := c0 + c1*WpXJ + c2*WpXJ*WpXJ;  /* (4-98) p.206 */
            Wc := WcXJ*XJ;
            Fs := 1 - ((LD + Wc)*sqrt(1 - (Wp/(XJ + Wp))^2) - LD)/L;  /* (4-95) p.205 */
          else
            Fs := 1;
          end if;
        
          /* Compute body effects */
          FB := Fn + (if noEvent(Sarg > 0) then GAMMA*Fs/(4*Sarg) else 0);  /* (4-91) p.202 */
          Vbody := GAMMA*Fs*Sarg + Fn*Sarg*Sarg;
          Vth := Vfb + Type*(PHIval - SIGMA*vds + Vbody);  /* (4-93) p.204 */
          if NFS > 0 then
            CB := if noEvent(Vbody>0 and Sarg>0) then Vbody/(2*Sarg*Sarg) else 0;  /* (4-48) p.183 */
            N := 1 + q*NFS/COX + CB;  /* (4-47) p.183 */
            Von := Type*Vth + N*Vt;  /* (4-46) p.183 */
          else
            N := 1;
            Von := Type*Vth;
          end if;
          Vgst := vgs - Von;
        
          /* Compute mobility modulation by gate voltage */
          vgsx := if vgs > Von then vgs else Von;
          FG := 1/(1 + THETA*(vgsx - Type*Vth));
          Us :=FG*U0;  /* (4-92) p.203 */
          BETAval := FG*BETA;
        
          /* Compute saturation voltage */
          Va := (vgsx - Type*Vth)/(1 + FB);  /* (4-102) p.208 */
          Vb := VMAX*L/Us;  /* (4-103) p.209 */
          vdsat := if VMAX > 0 then Va + Vb - sqrt(Va*Va + Vb*Vb) else Va;  /* (4-101) p.208 */
          vpof := Va;
        
          /* Compute velocity saturation factor */
          vdsx := if vds < vdsat then vds else vdsat;
          Fd := if vdsx > 0 and VMAX > 0 then 1/(1 + vdsx/Vb) else 1;  /* (4-100) p.208 */
          BETAeff := Fd*BETAval;
        
          /* Begin with linear reagion */
          cdlin := vdsx*(vgsx - Type*Vth - (1 + FB)*vdsx/2);
          idlin := BETAeff*cdlin;  /* (4-90) p.202 */
        
          /* Compute channel length modulation */
          if vds > vdsat then
            if VMAX > 0 then
              idsat := idlin;
              gdsat := idsat*(1 - Fd)/Vb;
              Ep := idsat/(gdsat*L);  /* (4-105) p.209 */
              ALPHA := XD*XD;
              DLfac := Ep*ALPHA/2;
              DL := sqrt(DLfac*DLfac + KAPPA*ALPHA*(vds - vdsat)) - DLfac;  /* (4-104) p.209 */
            else
              DL := sqrt(KAPPA*ALPHA*(vds - vdsat));
            end if;
          else
            DL := 0;
          end if;
        
          /* Compute punch-through */
          DLval := if DL > 0.5*L then L - L*L/(4*DL) else DL;
        
          /* Look at saturation region */
          LAMBDAval := DLval/L;
          Lfact := 1/(1 - LAMBDAval);
          Beta := Lfact*BETAeff;
          cdsat := cdlin;
        
          /* Look at weak inversion region */
          et := (vgs - Von)/(N*Vt);
          cdsub := cdlin*(if et < EMin then ExMin*(et - EMin + 1) else 
                          if et > EMax then ExMax*(et - EMax + 1) else  exp(et));  /* (4-49) p.183 */
        
          /* Compute drain-source current */
          ids := sig*Beta*(if vdsx > 0 and vds <= vdsat then cdlin else 
                           if vdsx > 0 and vds > vdsat and Vgst >= 0 then cdsat else 
                           if vdsx > 0 and vds > vdsat and Vgst < 0 then cdsub else 0);
          ids0 := if NFS == 0 and Vgst <= 0 then 0 else ids;
        end MOSeq3;
      
        block nlCM "Non-linear junction capacitance computation of MOSFET" 
          extends Modelica.Blocks.Interfaces.SISO;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Integer Level=3 
          "MOSFET modeling level (check documentation window for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Real CJ(unit="F/m2") = 1e-4 
          "Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real MJ=0.33 "Bulk junction grading coefficient";
          parameter Real CJSW(unit="F/m") = 1e-9 
          "Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Real MJSW=0.33 "Perimeter capacitance grading coefficient";
          parameter Real FC=0.5 
          "Forward-bias depletion capacitance coefficient";
          parameter Modelica.SIunits.Area A=0 "Diffusion area";
          parameter Modelica.SIunits.Length P=0 "Perimeter width";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
        
      protected 
          Modelica.SIunits.Temperature DTemp 
          "Difference between circuit temperature and reference temperature";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Voltage PBval 
          "Built-in potential at device temperature";
          Real CJval(unit="F/m2") 
          "Zero-bias bulk capacitance per square meter at device temperature";
          Real CJSWval(unit="F/m") 
          "Zero-bias perimeter capacitance per meter at device temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Energy EGnom "Energy gap at reference temperature";
          Modelica.SIunits.Voltage v "Voltage across capacitance";
          Modelica.SIunits.Capacitance Cval 
          "Non-linear junction capacitance value";
          Real Cbulk(unit="F/m2") 
          "Bulk capacitance per square meter at device temperature";
          Real Cperi(unit="F/m") 
          "Perimeter capacitance per meter at device temperature";
      public 
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
                Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[-10,-110; 10,-90],  rotation=90);
        annotation (Icon(
                    Text(
                extent=[-78,72; 76,-56],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-82,-40; 82,-64],
                style(color=0, rgbcolor={0,0,0}),
              string="MOSFET")), Diagram(
                 Rectangle(extent=[-100,-100; 100,100],   style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255})),
                    Text(
                extent=[-78,72; 76,-56],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-82,-40; 82,-64],
                style(color=0, rgbcolor={0,0,0}),
              string="MOSFET")));
        equation 
          v=u;
        
          /* Compute thermal voltage as function of temperature */
          Vt = k*Tdev/q;
        
          /* Compute temperature variation on junction capacitance */
          DTemp = Tdev - Tnom;
          RTemp = Tdev/Tnom;
          EGval = EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
          EGnom = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          PBval = PB*RTemp + EGval - EGnom*RTemp - 3*Vt*ln(RTemp);
          CJval = CJ*(1 + MJ*(1 - PBval/PB + 4e-4*DTemp));
          CJSWval = CJSW*(1 + MJSW*(1 - PBval/PB + 4e-4*DTemp));
        
          if Level==0 then
            Cbulk = 0;
            Cperi = 0;
            Cval = 1e-16;
          else
            Cbulk = CJval*(if v > FC*PBval then (1 - FC*(1 + MJ) + MJ*v/PBval)/((1 - FC)^(1 + MJ)) else 
                                                1/noEvent((abs(1 - v/PBval))^MJ));
            Cperi = CJSWval*(if v > FC*PBval then (1 - FC*(1 + MJSW) + MJSW*v/PBval)/((1 - FC)^(1 + MJSW)) else 
                                                  1/noEvent((abs(1 - v/PBval))^MJSW));
            Cval = if A*Cbulk + P*Cperi > 0 then A*Cbulk + P*Cperi else 1e-16;
          end if;
        
          y = Cval;
        end nlCM;
      
        block IDS0 "Injected current between drain and source" 
          extends Modelica.Blocks.Interfaces.SI2SO;
          parameter Integer Level=3 
          "Level of MOS model (check documentation window for details)";
          parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VFB=0 "Flat band voltage";
          parameter Modelica.SIunits.Transconductance KP=0 
          "Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
          "Surface inversion potential at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA=0 
          "Channel length modulation";
          parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
          parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
          parameter Real NEFF=1 "Total channel charge coefficient";
          parameter Modelica.SIunits.Length XJ=0 "Metallurgical junction depth";
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
          parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
          parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
          parameter Real U0(unit="m2/(V.s)") = 0.06 
          "Surface mobility at reference temperature";
          parameter Real UEXP=0 "Exponential coefficient for mobility";
          parameter Real UTRA=0 "Transverse field coefficient";
          parameter Modelica.SIunits.Velocity VMAX=0 
          "Maximum drift velocity of carriers";
          parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
          "Critical electric field for pinch-off";
          parameter Real DELTA=0 "Width effect on threshold voltage";
          parameter Real KAPPA=0.2 "Saturation field factor";
          parameter Real ETA=0 "Static feedback on threshold voltage";
          parameter Modelica.SIunits.InversePotential THETA=0 
          "Mobility modulation";
          parameter Modelica.SIunits.Voltage VBP=0 
          "Gate to channel critical voltage";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
          output Modelica.SIunits.Voltage vds "Drain-source voltage";
          output Modelica.SIunits.Voltage vgs "Gate-source voltage";
          output Modelica.SIunits.Voltage vbs "Bulk-source voltage";
          output Modelica.SIunits.Current ids "Injected drain-source current";
          output Modelica.SIunits.Voltage Vfb "Flat band voltage";
          output Modelica.SIunits.Voltage Vth "Threshold voltage";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
          output Modelica.SIunits.Voltage vdsat1 "Adjusted saturation voltage";
          output Modelica.SIunits.Voltage vpof "Punch-through voltage";
        
      protected 
          Real sig "sign(vds)";
      public 
          Modelica.Blocks.Interfaces.RealInput u3 
          "Connector of Real input signal 3" 
            annotation (extent=[-10,90; 10,110], rotation=-90);
          Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType 
            =   Modelica.SIunits.Temperature) "Device temperature" 
            annotation (extent=[-10,-110; 10,-90], rotation=90);
        annotation (Icon(
              Text(
                extent=[-8,88; 12,76],
                style(color=3, rgbcolor={0,0,255}),
                string="vbs"),
              Text(
                extent=[-96,66; -76,54],
                style(color=3, rgbcolor={0,0,255}),
                string="vds"),
              Text(
                extent=[-96,-54; -76,-66],
                style(color=3, rgbcolor={0,0,255}),
                string="vgs"),
              Text(
                extent=[-10,-78; 10,-90],
                style(color=3, rgbcolor={0,0,255}),
                string="Tdev"),
              Text(
                extent=[-80,26; 84,-14],
                style(color=3, rgbcolor={0,0,255}),
                string="between drain and source"),
              Text(
                extent=[-50,40; 50,12],
                style(color=3, rgbcolor={0,0,255}),
                string="Injected current"),
              Text(
                extent=[-58,-2; 62,-32],
                style(color=3, rgbcolor={0,0,255}),
                string="of a MOS transistor")), Diagram(
                 Rectangle(extent=[-100,-100; 100,100],   style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-8,88; 12,76],
                style(color=3, rgbcolor={0,0,255}),
                string="vbs"),
              Text(
                extent=[-96,66; -76,54],
                style(color=3, rgbcolor={0,0,255}),
                string="vds"),
              Text(
                extent=[-96,-54; -76,-66],
                style(color=3, rgbcolor={0,0,255}),
                string="vgs"),
              Text(
                extent=[-10,-78; 10,-90],
                style(color=3, rgbcolor={0,0,255}),
                string="Tdev"),
              Text(
                extent=[-50,40; 50,12],
                style(color=3, rgbcolor={0,0,255}),
                string="Injected current"),
              Text(
                extent=[-80,26; 84,-14],
                style(color=3, rgbcolor={0,0,255}),
                string="between drain and source"),
              Text(
                extent=[-58,-2; 62,-32],
                style(color=3, rgbcolor={0,0,255}),
                string="of a MOS transistor")),
          Documentation(info="<html>
The <b>IDS0</b> block computes the drain-source current of a MOSFET.  Depending on the modeling level, a different algorithm is being used.  The equations for each of these algorithms are programmed in functions, called <b>MOSeq1</b> .. <b>MOSeq3</b> that implement the particular model in question.  These functions have been carefully documented to make it easier on future developers to add additional models of their own by providing corresponding <b>MOSeq_i</b> functions. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model</b></font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 0-4</font>

 VFB:     Flat band voltage (default value = 0 Volt) <font color=red>Levels 0-4</font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 0-4</font>

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>) <font color=red>Levels 0-4</font>

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt) <font color=red>Levels 0-4</font>

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt) <font color=red>Levels 0-4</font>

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 2-4</font>

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 2-4</font>

 NEFF:    Total channel charge coefficient (default value = 1) <font color=red>Levels 2,4</font>

 XJ:      Metallurgical junction depth (default value = 0 m) <font color=red>Levels 2-4</font>

 L:       Channel length (default value = 1e-4 m) <font color=red>Levels 0-4</font>

 W:       Channel width (default value = 1e-4 m) <font color=red>Levels 0-4</font>

 LD:      Lateral diffusion (default value = 0 m) <font color=red>Level 3</font>

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>) <font color=red>Levels 2-4</font>

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s)) <font color=red>Levels 2-4</font>

 UEXP:    Exponential coefficient for mobility (default value = 0) <font color=red>Levels 2,4</font>

 UTRA:    Transverse field coefficient (default value = 0) <font color=red>Levels 2,4</font>

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s) <font color=red>Levels 2-4</font>
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation (Levels 2,4)</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation (Levels 2,4)</b></font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,4</font>

 DELTA:   Width effect on threshold voltage (default value = 0) <font color=red>Levels 2-4</font>

 KAPPA:   Saturation field factor (default value = 0.2) <font color=red>Level 3</font>

 ETA:     Static feedback on threshold voltage (default value = 0) <font color=red>Level 3</font>

 THETA:   Mobility modulation (default value = 0 1/Volt) <font color=red>Level 3</font>

 VBP:     Gate to channel critical voltage (default value = 0 Volt) <font color=red>Levels 2,4</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 2,4</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt) <font color=red>Levels 0-4</font>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 2-4</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 2-4</font>
</pre>
</html>

"));
          Modelica.Blocks.Interfaces.RealOutput y1[5] 
          "Connector of Real output signals" 
            annotation (extent=[100,-60; 120,-40]);
        equation 
          /* Determine first whether vds > 0 */
          if noEvent(u1 > 0) then
            /* Keep drain and source as they are */
            sig = 1;
            vds = u1;
            vgs = u2;
            vbs = u3;
          else
            /* Switch drain and source */
            sig = -1;
            vds = -u1;
            vgs = u2 - u1;
            vbs = u3 - u1;
          end if;
        
          /* Implement the different levels of drain-source current injection models */
          if Level==0 or Level==1 then
            (ids,Vfb,Vth,vdsat) = MOSeq1(vds=vds,vgs=vgs,vbs=vbs,Tdev=Tdev,sig=sig,
                                         Type=Type,Tnom=Tnom,VFB=VFB,KP=KP,GAMMA=GAMMA,
                                         PHI=PHI,LAMBDA=LAMBDA,L=L,W=W,EG=EG);
            vdsat1 = 0;
            vpof = 0;
          elseif Level==2 or Level==4 then
            (ids,Vfb,Vth,vdsat,vdsat1,vpof) = MOSeq2(vds=vds,vgs=vgs,vbs=vbs,Tdev=Tdev,sig=sig,
                                                     Level=Level,Type=Type,Tnom=Tnom,VFB=VFB,KP=KP,
                                                     GAMMA=GAMMA,PHI=PHI,LAMBDA=LAMBDA,COX=COX,NFS=NFS,
                                                     NEFF=NEFF,XJ=XJ,L=L,W=W,XD=XD,
                                                     U0=U0,UEXP=UEXP,UTRA=UTRA,VMAX=VMAX,ECRIT=ECRIT,
                                                     DELTA=DELTA,VBP=VBP,PB=PB,EG=EG,EMin=EMin,
                                                     EMax=EMax);
          elseif Level==3 then
            (ids,Vfb,Vth,vdsat,vpof) = MOSeq3(vds=vds,vgs=vgs,vbs=vbs,Tdev=Tdev,sig=sig,
                                              Type=Type,Tnom=Tnom,VFB=VFB,KP=KP,GAMMA=GAMMA,
                                              PHI=PHI,COX=COX,NFS=NFS,XJ=XJ,L=L,
                                              W=W,LD=LD,XD=XD,U0=U0,VMAX=VMAX,
                                              DELTA=DELTA,KAPPA=KAPPA,ETA=ETA,THETA=THETA,EG=EG,
                                              EMin=EMin,EMax=EMax);
            vdsat1 = 0;
          end if;
        
          /* Assign outputs */
          y = ids;
          y1[1] = Vfb;
          y1[2] = Vth;
          y1[3] = vdsat;
          y1[4] = vdsat1;
          y1[5] = vpof;
        end IDS0;
      
        model RM "Spice-style resistor model for MOSFETs" 
          extends Interfaces.SpiceTwoPort;
          parameter Modelica.SIunits.Resistance R=0 
          "Resistance at reference temperature";
          parameter Real TR1(unit="Ohm/K") = 0 "Linear temperature coefficient";
          parameter Real TR2(unit="Ohm/K2") = 0 
          "Quadratic temperature coefficient";
          Modelica.SIunits.Resistance Rval 
          "Resistance value at circuit temperature";
          parameter Integer Level=3 
          "MOSFET modeling level (check documentation window for details)";
        annotation (
          Diagram(
              Text(
                extent=[-82,76; 28,-80],
                string="RS",
                style(color=0))),
          Icon(
              Text(
                extent=[-82,76; 28,-80],
                string="RS",
                style(color=0))),
          Documentation(info="<html>
This is the regular Spice-style resistive source <b>RS</b> model, except that it looks at the modeling level.  If the level is equal to 0, the resistance is set equal to zero.
</html>
"));
        equation 
          if Level==0 then
          /* Static injection model */
            Rval = 0;
          else
          /* All other models */
            Rval = R*(1 + TR1*DTemp + TR2*DTempSq);
          end if;
          e1 = Rval*f1;
          e1*f1 = e2*f2;
        end RM;
      
        model CjM "Spice-style junction capacitor model for MOSFETs" 
          extends BondLib.Interfaces.TwoPort;
          parameter Integer Level=3 
          "MOSFET modeling level (check documentation window for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Current IS=0 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Real CJ(unit="F/m2") = 1e-4 
          "Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real MJ=0.33 "Bulk junction grading coefficient";
          parameter Real CJSW(unit="F/m") = 1e-9 
          "Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Real MJSW=0.33 "Perimeter capacitance grading coefficient";
          parameter Real FC=0.5 
          "Forward-bias depletion capacitance coefficient";
          parameter Modelica.SIunits.Area A=0 "Diffusion area";
          parameter Modelica.SIunits.Length P=0 "Perimeter width";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 300 K";
          parameter Modelica.SIunits.Conductance GminDC=1e-12 
          "Leakage conductance";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (
          Icon(
              Text(
                extent=[-84,72; 30,-74],
                style(color=0),
              string="CjM"),
            Text(
              extent=[-52,96; 54,66],
              style(color=3, rgbcolor={0,0,255}),
              string="%name"),
            Text(
              extent=[4,-4; 112,-32],
              string="Spice",
              style(color=0, rgbcolor={0,0,0}))),
          Diagram,
          Documentation(info="<html>
The Spice-style junction capacitance for MOSFETs computes the depletion capacitance value of a junction used in a MOSFET.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 IS:      Transport saturation current (default value = 0 Amp)

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 A:       Diffusion area (default value = 0 m<sup>2</sup>)

 P:       Perimeter width (default value = 0 m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 GminDC:  Leakage conductance (default value = 1e-12 Mho)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.200.
</ol>
</html>
"));
        Junctions.J0p5 J0p5_1 annotation (extent=[-80,-10; -60,10]);
          Bonds.fBond B1 annotation (extent=[-60,-10; -40,10]);
          Sensors.De vd  annotation (extent=[-40,-10; -20,10]);
          Bonds.eBond B2 annotation (extent=[-80,10; -60,30], rotation=90);
          Semiconductors.Utilities.mC2 Cj1 
                  annotation (extent=[-80,30; -60,50], rotation=90);
        nlCM Cval(
          Tnom=Tnom,
          PB=PB,
          CJ=CJ,
          MJ=MJ,
          CJSW=CJSW,
          MJSW=MJSW,
          FC=FC,
          A=A,
          P=P,
          EG=EG,
          Level=Level) annotation (extent=[-30,70; -50,50]);
        Junctions.J0p4 J0p4_1 annotation (extent=[60,-10; 80,10]);
          Bonds.fBond B7 annotation (extent=[60,10; 80,30], rotation=90);
          Sensors.De Tdev 
                         annotation (extent=[60,30; 80,50], rotation=90);
        DjM DjM1(
          Tnom=Tnom,
          IS=IS,
          EG=EG,
          Level=Level,
          EMin=-100,
          EMax=40) annotation (extent=[-20,-40; 20,-20]);
          Bonds.eBond B4 annotation (extent=[20,-40; 40,-20], rotation=0);
          Bonds.fBond B3 annotation (extent=[-40,-40; -20,-20]);
          Bonds.fBond B5 annotation (extent=[-40,-70; -20,-50]);
          Bonds.eBond B6 annotation (extent=[20,-70; 40,-50], rotation=0);
        RS Rd(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-20,-70; 20,-50]);
        equation 
        connect(J0p5_1.BondCon1, BondCon1) annotation (points=[-80,0; -100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon2, B1.eBondCon1) annotation (points=[-60,0; -60,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B1.fBondCon1, vd.BondCon1) annotation (points=[-40,0; -40,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.fBondCon1, J0p5_1.BondCon5) annotation (points=[-70,10; -70,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(Cj1.BondCon1, B2.eBondCon1) annotation (points=[-70,30; -70,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(vd.OutPort1, Cval.u) annotation (points=[-19,0; -10,0; -10,60;
              -28,60], style(color=74, rgbcolor={0,0,127}));
        connect(J0p4_1.BondCon2, BondCon2) annotation (points=[80,0; 100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B7.eBondCon1, J0p4_1.BondCon4) annotation (points=[70,10; 70,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.BondCon1, B7.fBondCon1) annotation (points=[70,30; 70,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.OutPort1, Cval.Tdev) annotation (points=[70,51; 70,80; -40,
              80; -40,70], style(color=74, rgbcolor={0,0,127}));
        connect(DjM1.BondCon2, B4.fBondCon1) annotation (points=[20,-30; 20,-30],
            style(color=8, rgbcolor={192,192,192}));
        connect(B3.fBondCon1, DjM1.BondCon1) annotation (points=[-20,-30; -20,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon4, B3.eBondCon1) annotation (points=[-66,-10; -66,
              -30; -40,-30], style(color=8, rgbcolor={192,192,192}));
        connect(B4.eBondCon1, J0p4_1.BondCon1) annotation (points=[40,-30; 50,
              -30; 50,0; 60,0], style(color=8, rgbcolor={192,192,192}));
        connect(B5.eBondCon1, J0p5_1.BondCon3) annotation (points=[-40,-60; -74,
              -60; -74,-10], style(color=8, rgbcolor={192,192,192}));
        connect(B6.eBondCon1, J0p4_1.BondCon3) annotation (points=[40,-60; 70,
              -60; 70,-10], style(color=8, rgbcolor={192,192,192}));
        connect(Rd.BondCon1, B5.fBondCon1) annotation (points=[-20,-60; -20,-60],
            style(color=8, rgbcolor={192,192,192}));
        connect(Rd.BondCon2, B6.fBondCon1) annotation (points=[20,-60; 20,-60],
            style(color=8, rgbcolor={192,192,192}));
        connect(Cval.y, Cj1.s) annotation (points=[-51,60; -70,60; -70,49],
            style(color=74, rgbcolor={0,0,127}));
        end CjM;
      
        model Cgb 
        "Spice-style gate capacitance between gate and bulk of MOSFETs" 
          extends Interfaces.SpiceTwoPort;
          constant Real pi=Modelica.Constants.pi;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
          "Permittivity of free space";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Modelica.SIunits.ElectronNumberDensity ni=1.45e16 
          "Intrinsic carrier concentration";
          constant Real ks=11.8 "Dielectric constant of Si";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          constant Real c0=0.0631353;
          constant Real c1=0.8013292;
          constant Real c2=-0.01110777;
          parameter Integer Level=3 
          "Level of MOS model (check documentation window for details)";
          parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VFB=0 
          "Flat band voltage at reference temperature";
          parameter Modelica.SIunits.Transconductance KP=0 
          "Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
          "Surface inversion potential at reference temperature";
          parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
          parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
          parameter Modelica.SIunits.Length XJ=0 "Metallurgical junction depth";
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
          parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
          parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
          parameter Real XQC=0 "Coefficient of channel charge share";
          parameter Real DELTA=0 "Width effect on threshold voltage";
          parameter Real ETA=0 "Static feedback on threshold voltage";
          parameter Real CGB0(unit="F/m") = 1e-12 
          "Gate-bulk overlap capacitance per meter";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          Modelica.SIunits.Voltage vds "Drain-source voltage";
          Modelica.SIunits.Voltage vgs "Gate-source voltage";
          Modelica.SIunits.Voltage vbs "Bulk-source voltage";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Energy EGref "Energy gap at reference temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Voltage PHIval 
          "Surface inversion potential at device temperature";
          Modelica.SIunits.Voltage Vfb 
          "Flat band voltage at device temperature";
          Modelica.SIunits.Voltage Vth "Threshold voltage";
          Modelica.SIunits.Voltage Von "Adjusted threshold voltage";
          Modelica.SIunits.Capacitance Cval 
          "Gate capacitance between gate and bulk";
        
      protected 
          parameter Modelica.SIunits.Permittivity es = ks*e0 
          "Permittivity of Si";
          parameter Modelica.SIunits.Capacitance COXval = W*L*COX 
          "Capacitance of SiO2 layer";
          parameter Real SIGMA(unit="1/(F.m)") = 8.15e-22*ETA/(COX*L^3);
          parameter Real Fn = es*DELTA*pi/(2*COX*W);
          Real sig "sign(vds)";
          Real RTemp "Temperature quotient";
          Real facTemp "Temperature coefficient";
          Modelica.SIunits.Length WD "Width of drain depletion region";
          Modelica.SIunits.Length WS "Width of source depletion region";
          Modelica.SIunits.Length Wp 
          "Width of depleted region in flat source junction";
          Modelica.SIunits.Length Wc 
          "Width of depleted region in source substrate junction";
          Modelica.SIunits.Transconductance KPval 
          "Transconductance parameter at device temperature";
          Modelica.SIunits.Transconductance BETA 
          "Transconductance at device temperature";
          Real CB(unit="F/m2") "Specific capacitance of depletion area";
          Real N "Current emission coefficient";
          Real GAMMAval(unit="V^0.5") "Adjusted body effect parameter";
          Real Fs "Short channel working function";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vbody;
          Real PHIroot(unit="V^0.5");
          Real Sarg(unit="V^0.5");
          Real Barg(unit="V^0.5");
          Real PHI3(unit="V^1.5");
          Real DSarg(unit="1/V^0.5") "der(Sarg,vbs)";
          Real DBarg(unit="1/V^0.5") "der(Barg,vbs)";
          Real DGAMMA(unit="1/V^0.5") "der(GAMMAval,vbs)";
          Real WSfac;
          Real WDfac;
          Real WpXJ "WP/XJ";
          Real WcXJ "WC/XJ";
      public 
          Modelica.Blocks.Interfaces.RealInput u2 
          "Connector of Real input signal 2" 
            annotation (extent=[50,-110; 70,-90],rotation=90);
      public 
          Modelica.Blocks.Interfaces.RealInput u1 
          "Connector of Real input signal 1" 
            annotation (extent=[-70,-110; -50,-90],
                                                 rotation=90);
          Modelica.Blocks.Interfaces.RealInput u[5] 
          "Connector of Real input signals" 
            annotation (extent=[-10,90; 10,110],    rotation=270);
        equation 
          /* Parameter computations and temperature variations */
          RTemp = e2/Tnom;
          facTemp = RTemp*sqrt(RTemp);
          Vt = k*e2/q;
          EGref = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          EGval = EG - GapC1*e2*e2/(e2 + GapC2);
          PHIval = RTemp*PHI + EGval - RTemp*EGref - 3*Vt*ln(RTemp);
          Vfb = VFB + Type*((PHIval - PHI)/2 + (EGref - EGval)/2);
          KPval = KP/facTemp;
          BETA = KPval*W/L;
        
          /* Determine first whether vds > 0 */
          if noEvent(u1 > 0) then
            /* Keep drain and source as they are */
            sig = 1;
            vds = u1;
            vgs = e1 + u2;
            vbs = u2;
          else
            /* Switch drain and source */
            sig = -1;
            vds = -u1;
            vgs = e1 - u1 + u2;
            vbs = u2 - u1;
          end if;
        
          /* Compute some useful quantities */
          PHIroot = sqrt(PHIval);
          PHI3 = PHIval*PHIroot;
        
         /* Compute Sarg and Barg and their derivatives */
          if Level==0 then
            Sarg = 0;
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          elseif Level==1 then
            Sarg = if noEvent(vbs <= 0) then sqrt(PHIval - vbs) else max({0,PHIroot - vbs/(2*PHIroot)});
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          elseif Level==2 or Level==4 then
            Sarg = if vbs <= 0 then sqrt(PHIval - vbs) else PHIroot/(1 + vbs/(2*PHIval));
            DSarg = if vbs <= 0 then -0.5/Sarg else -0.5*Sarg*Sarg/PHI3;
            Barg = if vbs < vds then sqrt(PHIval + vds - vbs) else PHIroot/(1 + (vbs - vds)/(2*PHIval));
            DBarg = if vbs < vds then -0.5/Barg else -0.5*Barg*Barg/PHI3;
          elseif Level==3 then
            Sarg = if vbs <= 0 then sqrt(PHIval - vbs) else PHIroot/(1 + vbs/(2*PHIval));
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          end if;
        
          /* For Level=2 and Level=4 */
          WS = XD*Sarg;
          WD = XD*Barg;
          WSfac =  if XJ > 0 then sqrt(1 + 2*WS/XJ) else 0;
          WDfac =  if XJ > 0 then sqrt(1 + 2*WD/XJ) else 0;
          GAMMAval = GAMMA*(if XJ > 0 then 1 - 0.5*XJ*(WSfac + WDfac - 2)/L else 1);
          DGAMMA = GAMMA*(if XJ > 0 then -0.5*XD*(DSarg/WSfac + DBarg/WDfac)/L else 0);
        
          /* For Level=3 */
          Wp = Sarg*XD;
          WpXJ = if XJ > 0 then Wp/XJ else 0;
          WcXJ = c0 + c1*WpXJ + c2*WpXJ*WpXJ;
          Wc = WcXJ*XJ;
          Fs = if XJ > 0 then 1 - ((LD + Wc)*sqrt(1 - (Wp/(XJ + Wp))^2) - LD)/L else 1;
          Vbody =  GAMMA*Fs*Sarg + Fn*Sarg*Sarg;
        
         /* Compute Vth and Von */
          if Level==0 then
            Vth = 0;
            CB = 0;
            N = 0;
            Von = 0;
          elseif Level==1 then
            Vth = Vfb + Type*(PHIval + GAMMA*Sarg);
            CB = 0;
            N = 0;
            Von = Type*Vth;
          elseif Level==2 or Level==4 then
            Vth = Vfb + Type*(PHIval + (GAMMA - GAMMAval)*sqrt(PHIval) + GAMMAval*Sarg + Fn*Sarg*Sarg/2);
            CB = Fn - GAMMAval*DSarg - DGAMMA*Sarg;
            N = if NFS > 0 then 1 + q*NFS/COX + CB else 1;
            Von =  if NFS > 0 then Type*Vth + N*Vt else Type*Vth;
          elseif Level==3 then
            Vth =  Vfb + Type*(PHIval - SIGMA*vds + Vbody);
            CB = if noEvent(Vbody > 0 and Sarg > 0) then Vbody/(2*Sarg*Sarg) else 0;
            N = 1 + q*NFS/COX + CB;
            Von = if NFS > 0 then Type*Vth + N*Vt else Type*Vth;
          end if;
        
          /* Compute gate capacitance between gate and bulk */
          if Level==0 or not CGB0 > 0 then
            Cval = 1e-16;
          else
            Cval = L*CGB0 + (if noEvent(vgs < Von - PHIval) then COXval else 
                             if noEvent(vgs >= Von - PHIval and vgs < Von) then COXval*(Von - vgs)/PHIval else 0);
          end if;
        
          /* Capacitance equation */
          f1 = Cval*der(e1);
        
          /* Computation of heat flow */
          f2 = 0;
        annotation (
          Icon(
              Text(
                extent=[-88,70; 26,-76],
                style(color=0),
              string="Cgb"),
              Text(
                extent=[-70,-76; -50,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="vds"),
              Text(
                extent=[50,-76; 70,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="vbs")),
          Diagram(
              Text(
                extent=[-88,72; 26,-74],
                style(color=0),
              string="Cgb")),
          Documentation(info="<html>
The Spice-style gate capacitances for MOSFETs are non-linear capacitors that use different formulae to compute the capacitance value in the four different regions: <i>accumulation</i>, <i>depletion</i>, <i>saturation</i>, and the <i>linear region</i>.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the gate capacitances at all, but rather compute the electric charges stored in these regions directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the gate capacitances directly, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 XQC:     Coefficient of channel charge share (default value = 0)
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 ETA:     Static feedback on threshold voltage (default value = 1)

 CGB0:    Gate-bulk overlap capacitance per meter (default value = 1e-12 F/m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, pp.196-197.
</ol>
</html>
"));
        end Cgb;
      
        model Cgs 
        "Spice-style gate capacitance between gate and source of MOSFETs" 
          extends Interfaces.SpiceTwoPort;
          constant Real pi=Modelica.Constants.pi;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
          "Permittivity of free space";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Modelica.SIunits.ElectronNumberDensity ni=1.45e16 
          "Intrinsic carrier concentration";
          constant Real ks=11.8 "Dielectric constant of Si";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          constant Real c0=0.0631353;
          constant Real c1=0.8013292;
          constant Real c2=-0.01110777;
          parameter Integer Level=3 
          "Level of MOS model (check documentation window for details)";
          parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VFB=0 
          "Flat band voltage at reference temperature";
          parameter Modelica.SIunits.Transconductance KP=0 
          "Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
          "Surface inversion potential at reference temperature";
          parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
          parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
          parameter Modelica.SIunits.Length XJ=0 "Metallurgical junction depth";
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
          parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
          parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
          parameter Real XQC=0 "Coefficient of channel charge share";
          parameter Real DELTA=0 "Width effect on threshold voltage";
          parameter Real ETA=0 "Static feedback on threshold voltage";
          parameter Real CGS0(unit="F/m") = 1e-12 
          "Gate-source overlap capacitance per meter";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          Modelica.SIunits.Voltage vds "Drain-source voltage";
          Modelica.SIunits.Voltage vgs "Gate-source voltage";
          Modelica.SIunits.Voltage vbs "Bulk-source voltage";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Energy EGref "Energy gap at reference temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Voltage PHIval 
          "Surface inversion potential at device temperature";
          Modelica.SIunits.Voltage Vfb 
          "Flat band voltage at device temperature";
          Modelica.SIunits.Voltage Vth "Threshold voltage";
          Modelica.SIunits.Voltage Von "Adjusted threshold voltage";
          Modelica.SIunits.Capacitance Cval 
          "Gate capacitance between gate and bulk";
        
      protected 
          parameter Modelica.SIunits.Permittivity es = ks*e0 
          "Permittivity of Si";
          parameter Modelica.SIunits.Capacitance COXval = W*L*COX 
          "Capacitance of SiO2 layer";
          parameter Real SIGMA(unit="1/(F.m)") = 8.15e-22*ETA/(COX*L^3);
          parameter Real Fn = es*DELTA*pi/(2*COX*W);
          Real sig "sign(vds)";
          Real RTemp "Temperature quotient";
          Real facTemp "Temperature coefficient";
          Modelica.SIunits.Length WD "Width of drain depletion region";
          Modelica.SIunits.Length WS "Width of source depletion region";
          Modelica.SIunits.Length Wp 
          "Width of depleted region in flat source junction";
          Modelica.SIunits.Length Wc 
          "Width of depleted region in source substrate junction";
          Modelica.SIunits.Transconductance KPval 
          "Transconductance parameter at device temperature";
          Modelica.SIunits.Transconductance BETA 
          "Transconductance at device temperature";
          Real CB(unit="F/m2") "Specific capacitance of depletion area";
          Real N "Current emission coefficient";
          Real GAMMAval(unit="V^0.5") "Adjusted body effect parameter";
          Real Fs "Short channel working function";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vbody;
          Real PHIroot(unit="V^0.5");
          Real Sarg(unit="V^0.5");
          Real Barg(unit="V^0.5");
          Real PHI3(unit="V^1.5");
          Real DSarg(unit="1/V^0.5") "der(Sarg,vbs)";
          Real DBarg(unit="1/V^0.5") "der(Barg,vbs)";
          Real DGAMMA(unit="1/V^0.5") "der(GAMMAval,vbs)";
          Real WSfac;
          Real WDfac;
          Real WpXJ "WP/XJ";
          Real WcXJ "WC/XJ";
      public 
          Modelica.Blocks.Interfaces.RealInput u1 
          "Connector of Real input signal 1" 
            annotation (extent=[-70,-110; -50,-90],
                                                 rotation=90);
      public 
          Modelica.Blocks.Interfaces.RealInput u2 
          "Connector of Real input signal 2" 
            annotation (extent=[50,-110; 70,-90],rotation=90);
        annotation (
          Icon(
              Text(
                extent=[-88,70; 26,-76],
                style(color=0),
              string="Cgs"),
              Text(
                extent=[-70,-76; -50,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="vds"),
              Text(
                extent=[50,-76; 70,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="vbs")),
          Diagram(
              Text(
                extent=[-88,70; 26,-76],
                style(color=0),
              string="Cgs")),
          Documentation(info="<html>
The Spice-style gate capacitances for MOSFETs are non-linear capacitors that use different formulae to compute the capacitance value in the four different regions: <i>accumulation</i>, <i>depletion</i>, <i>saturation</i>, and the <i>linear region</i>.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the gate capacitances at all, but rather compute the electric charges stored in these regions directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the gate capacitances directly, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 XQC:     Coefficient of channel charge share (default value = 0)
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 ETA:     Static feedback on threshold voltage (default value = 1)

 CGS0:    Gate-source overlap capacitance per meter (default value = 1e-12 F/m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, pp.196-197.
</ol>
</html>
"));
          Modelica.Blocks.Interfaces.RealInput u[5] 
          "Connector of Real input signals" 
            annotation (extent=[-10,90; 10,110],    rotation=270);
        equation 
          /* Parameter computations and temperature variations */
          RTemp = e2/Tnom;
          facTemp = RTemp*sqrt(RTemp);
          Vt = k*e2/q;
          EGref = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          EGval = EG - GapC1*e2*e2/(e2 + GapC2);
          PHIval = RTemp*PHI + EGval - RTemp*EGref - 3*Vt*ln(RTemp);
          Vfb = VFB + Type*((PHIval - PHI)/2 + (EGref - EGval)/2);
          KPval = KP/facTemp;
          BETA = KPval*W/L;
        
          /* Determine first whether vds > 0 */
          if noEvent(u1 > 0) then
            /* Keep drain and source as they are */
            sig = 1;
            vds = u1;
            vgs = e1 + u2;
            vbs = u2;
          else
            /* Switch drain and source */
            sig = -1;
            vds = -u1;
            vgs = e1 - u1;
            vbs = u2 - u1;
          end if;
        
          /* Compute some useful quantities */
          PHIroot = sqrt(PHIval);
          PHI3 = PHIval*PHIroot;
        
         /* Compute Sarg and Barg and their derivatives */
          if Level==0 then
            Sarg = 0;
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          elseif Level==1 then
            Sarg = if noEvent(vbs <= 0) then sqrt(PHIval - vbs) else max({0,PHIroot - vbs/(2*PHIroot)});
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          elseif Level==2 or Level==4 then
            Sarg = if vbs <= 0 then sqrt(PHIval - vbs) else PHIroot/(1 + vbs/(2*PHIval));
            DSarg = if vbs <= 0 then -0.5/Sarg else -0.5*Sarg*Sarg/PHI3;
            Barg = if vbs < vds then sqrt(PHIval + vds - vbs) else PHIroot/(1 + (vbs - vds)/(2*PHIval));
            DBarg = if vbs < vds then -0.5/Barg else -0.5*Barg*Barg/PHI3;
          elseif Level==3 then
            Sarg = if vbs <= 0 then sqrt(PHIval - vbs) else PHIroot/(1 + vbs/(2*PHIval));
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          end if;
        
          /* For Level=2 and Level=4 */
          WS = XD*Sarg;
          WD = XD*Barg;
          WSfac =  if XJ > 0 then sqrt(1 + 2*WS/XJ) else 0;
          WDfac =  if XJ > 0 then sqrt(1 + 2*WD/XJ) else 0;
          GAMMAval = GAMMA*(if XJ > 0 then 1 - 0.5*XJ*(WSfac + WDfac - 2)/L else 1);
          DGAMMA = GAMMA*(if XJ > 0 then -0.5*XD*(DSarg/WSfac + DBarg/WDfac)/L else 0);
        
          /* For Level=3 */
          Wp = Sarg*XD;
          WpXJ = if XJ > 0 then Wp/XJ else 0;
          WcXJ = c0 + c1*WpXJ + c2*WpXJ*WpXJ;
          Wc = WcXJ*XJ;
          Fs = if XJ > 0 then 1 - ((LD + Wc)*sqrt(1 - (Wp/(XJ + Wp))^2) - LD)/L else 1;
          Vbody =  GAMMA*Fs*Sarg + Fn*Sarg*Sarg;
        
         /* Compute Vth and Von */
          if Level==0 then
            Vth = 0;
            CB = 0;
            N = 0;
            Von = 0;
          elseif Level==1 then
            Vth = Vfb + Type*(PHIval + GAMMA*Sarg);
            CB = 0;
            N = 0;
            Von = Type*Vth;
          elseif Level==2 or Level==4 then
            Vth = Vfb + Type*(PHIval + (GAMMA - GAMMAval)*sqrt(PHIval) + GAMMAval*Sarg + Fn*Sarg*Sarg/2);
            CB = Fn - GAMMAval*DSarg - DGAMMA*Sarg;
            N = if NFS > 0 then 1 + q*NFS/COX + CB else 1;
            Von =  if NFS > 0 then Type*Vth + N*Vt else Type*Vth;
          elseif Level==3 then
            Vth =  Vfb + Type*(PHIval - SIGMA*vds + Vbody);
            CB = if noEvent(Vbody > 0 and Sarg > 0) then Vbody/(2*Sarg*Sarg) else 0;
            N = 1 + q*NFS/COX + CB;
            Von = if NFS > 0 then Type*Vth + N*Vt else Type*Vth;
          end if;
        
          /* Compute gate capacitance between gate and bulk */
          if Level==0 or not CGS0 > 0 then
            Cval = 1e-16;
          else
               if noEvent(sig > 0) then
                  Cval = W*CGS0 + (if noEvent(vgs < Von - PHIval) then 0 else 
                                   if noEvent(vgs >= Von - PHIval and vgs < Von) then 2*COXval*(1 - (Von - vgs)/PHIval)/3 else 
                                   if noEvent(vgs >= Von) and noEvent(vgs < Von + vds) then 2*COXval/3 else 
                                      2*COXval*(1 - ((vgs - vds - Von)/(2*(vgs - Von) - vds))^2)/3);
            
               else
                  Cval = W*CGS0 + (if noEvent(vgs < Von + vds) then 0 else 
                         2*COXval*(1 - ((vgs - Von)/(2*(vgs - Von) - vds))^2)/3);
               end if;
          end if;
        
          /* Capacitance equation */
          f1 = Cval*der(e1);
        
          /* Computation of heat flow */
          f2 = 0;
        end Cgs;
      
        model Cgd 
        "Spice-style gate capacitance between gate and drain of MOSFETs" 
          extends Interfaces.SpiceTwoPort;
          constant Real pi=Modelica.Constants.pi;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
          "Permittivity of free space";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Modelica.SIunits.ElectronNumberDensity ni=1.45e16 
          "Intrinsic carrier concentration";
          constant Real ks=11.8 "Dielectric constant of Si";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          constant Real c0=0.0631353;
          constant Real c1=0.8013292;
          constant Real c2=-0.01110777;
          parameter Integer Level=3 
          "Level of MOS model (check documentation window for details)";
          parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VFB=0 
          "Flat band voltage at reference temperature";
          parameter Modelica.SIunits.Transconductance KP=0 
          "Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
          "Surface inversion potential at reference temperature";
          parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
          parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
          parameter Modelica.SIunits.Length XJ=0 "Metallurgical junction depth";
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
          parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
          parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
          parameter Real XQC=0 "Coefficient of channel charge share";
          parameter Real DELTA=0 "Width effect on threshold voltage";
          parameter Real ETA=0 "Static feedback on threshold voltage";
          parameter Real CGD0(unit="F/m") = 1e-12 
          "Gate-drain overlap capacitance per meter";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          Modelica.SIunits.Voltage vds "Drain-source voltage";
          Modelica.SIunits.Voltage vgs "Gate-source voltage";
          Modelica.SIunits.Voltage vbs "Bulk-source voltage";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Energy EGref "Energy gap at reference temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Voltage PHIval 
          "Surface inversion potential at device temperature";
          Modelica.SIunits.Voltage Vfb 
          "Flat band voltage at device temperature";
          Modelica.SIunits.Voltage Vth "Threshold voltage";
          Modelica.SIunits.Voltage Von "Adjusted threshold voltage";
          Modelica.SIunits.Capacitance Cval 
          "Gate capacitance between gate and bulk";
        
      protected 
          parameter Modelica.SIunits.Permittivity es = ks*e0 
          "Permittivity of Si";
          parameter Modelica.SIunits.Capacitance COXval = W*L*COX 
          "Capacitance of SiO2 layer";
          parameter Real SIGMA(unit="1/(F.m)") = 8.15e-22*ETA/(COX*L^3);
          parameter Real Fn = es*DELTA*pi/(2*COX*W);
          Real sig "sign(vds)";
          Real RTemp "Temperature quotient";
          Real facTemp "Temperature coefficient";
          Modelica.SIunits.Length WD "Width of drain depletion region";
          Modelica.SIunits.Length WS "Width of source depletion region";
          Modelica.SIunits.Length Wp 
          "Width of depleted region in flat source junction";
          Modelica.SIunits.Length Wc 
          "Width of depleted region in source substrate junction";
          Modelica.SIunits.Transconductance KPval 
          "Transconductance parameter at device temperature";
          Modelica.SIunits.Transconductance BETA 
          "Transconductance at device temperature";
          Real CB(unit="F/m2") "Specific capacitance of depletion area";
          Real N "Current emission coefficient";
          Real GAMMAval(unit="V^0.5") "Adjusted body effect parameter";
          Real Fs "Short channel working function";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vbody;
          Real PHIroot(unit="V^0.5");
          Real Sarg(unit="V^0.5");
          Real Barg(unit="V^0.5");
          Real PHI3(unit="V^1.5");
          Real DSarg(unit="1/V^0.5") "der(Sarg,vbs)";
          Real DBarg(unit="1/V^0.5") "der(Barg,vbs)";
          Real DGAMMA(unit="1/V^0.5") "der(GAMMAval,vbs)";
          Real WSfac;
          Real WDfac;
          Real WpXJ "WP/XJ";
          Real WcXJ "WC/XJ";
        annotation (
          Documentation(info="<html>
The Spice-style gate capacitances for MOSFETs are non-linear capacitors that use different formulae to compute the capacitance value in the four different regions: <i>accumulation</i>, <i>depletion</i>, <i>saturation</i>, and the <i>linear region</i>.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the gate capacitances at all, but rather compute the electric charges stored in these regions directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the gate capacitances directly, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 XQC:     Coefficient of channel charge share (default value = 0)
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 ETA:     Static feedback on threshold voltage (default value = 1)

 CGD0:    Gate-drain overlap capacitance per meter (default value = 1e-12 F/m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, pp.196-197.
</ol>
</html>
"),       Icon(
              Text(
                extent=[-88,70; 26,-76],
                style(color=0),
              string="Cgd"),
              Text(
                extent=[-70,-76; -50,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="vds"),
              Text(
                extent=[50,-76; 70,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="vbs")),
          Diagram(
              Text(
                extent=[-88,70; 26,-76],
                style(color=0),
              string="Cgd")));
      public 
          Modelica.Blocks.Interfaces.RealInput u1 
          "Connector of Real input signal 1" 
            annotation (extent=[-70,-110; -50,-90],
                                                 rotation=90);
      public 
          Modelica.Blocks.Interfaces.RealInput u2 
          "Connector of Real input signal 2" 
            annotation (extent=[50,-110; 70,-90],rotation=90);
          Modelica.Blocks.Interfaces.RealInput u[5] 
          "Connector of Real input signals" 
            annotation (extent=[-10,90; 10,110],    rotation=270);
        equation 
          /* Parameter computations and temperature variations */
          RTemp = e2/Tnom;
          facTemp = RTemp*sqrt(RTemp);
          Vt = k*e2/q;
          EGref = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          EGval = EG - GapC1*e2*e2/(e2 + GapC2);
          PHIval = RTemp*PHI + EGval - RTemp*EGref - 3*Vt*ln(RTemp);
          Vfb = VFB + Type*((PHIval - PHI)/2 + (EGref - EGval)/2);
          KPval = KP/facTemp;
          BETA = KPval*W/L;
        
          /* Determine first whether vds > 0 */
          if noEvent(u1 > 0) then
            /* Keep drain and source as they are */
            sig = 1;
            vds = u1;
            vgs = e1 + u1;
            vbs = u2;
          else
            /* Switch drain and source */
            sig = -1;
            vds = -u1;
            vgs = e1 - u1 + u2;
            vbs = u2 - u1;
          end if;
        
          /* Compute some useful quantities */
          PHIroot = sqrt(PHIval);
          PHI3 = PHIval*PHIroot;
        
         /* Compute Sarg and Barg and their derivatives */
          if Level==0 then
            Sarg = 0;
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          elseif Level==1 then
            Sarg = if noEvent(vbs <= 0) then sqrt(PHIval - vbs) else max({0,PHIroot - vbs/(2*PHIroot)});
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          elseif Level==2 or Level==4 then
            Sarg = if vbs <= 0 then sqrt(PHIval - vbs) else PHIroot/(1 + vbs/(2*PHIval));
            DSarg = if vbs <= 0 then -0.5/Sarg else -0.5*Sarg*Sarg/PHI3;
            Barg = if vbs < vds then sqrt(PHIval + vds - vbs) else PHIroot/(1 + (vbs - vds)/(2*PHIval));
            DBarg = if vbs < vds then -0.5/Barg else -0.5*Barg*Barg/PHI3;
          elseif Level==3 then
            Sarg = if vbs <= 0 then sqrt(PHIval - vbs) else PHIroot/(1 + vbs/(2*PHIval));
            DSarg = 0;
            Barg = 0;
            DBarg = 0;
          end if;
        
          /* For Level=2 and Level=4 */
          WS = XD*Sarg;
          WD = XD*Barg;
          WSfac =  if XJ > 0 then sqrt(1 + 2*WS/XJ) else 0;
          WDfac =  if XJ > 0 then sqrt(1 + 2*WD/XJ) else 0;
          GAMMAval = GAMMA*(if XJ > 0 then 1 - 0.5*XJ*(WSfac + WDfac - 2)/L else 1);
          DGAMMA = GAMMA*(if XJ > 0 then -0.5*XD*(DSarg/WSfac + DBarg/WDfac)/L else 0);
        
          /* For Level=3 */
          Wp = Sarg*XD;
          WpXJ = if XJ > 0 then Wp/XJ else 0;
          WcXJ = c0 + c1*WpXJ + c2*WpXJ*WpXJ;
          Wc = WcXJ*XJ;
          Fs = if XJ > 0 then 1 - ((LD + Wc)*sqrt(1 - (Wp/(XJ + Wp))^2) - LD)/L else 1;
          Vbody =  GAMMA*Fs*Sarg + Fn*Sarg*Sarg;
        
         /* Compute Vth and Von */
          if Level==0 then
            Vth = 0;
            CB = 0;
            N = 0;
            Von = 0;
          elseif Level==1 then
            Vth = Vfb + Type*(PHIval + GAMMA*Sarg);
            CB = 0;
            N = 0;
            Von = Type*Vth;
          elseif Level==2 or Level==4 then
            Vth = Vfb + Type*(PHIval + (GAMMA - GAMMAval)*sqrt(PHIval) + GAMMAval*Sarg + Fn*Sarg*Sarg/2);
            CB = Fn - GAMMAval*DSarg - DGAMMA*Sarg;
            N = if NFS > 0 then 1 + q*NFS/COX + CB else 1;
            Von =  if NFS > 0 then Type*Vth + N*Vt else Type*Vth;
          elseif Level==3 then
            Vth =  Vfb + Type*(PHIval - SIGMA*vds + Vbody);
            CB = if noEvent(Vbody > 0 and Sarg > 0) then Vbody/(2*Sarg*Sarg) else 0;
            N = 1 + q*NFS/COX + CB;
            Von = if NFS > 0 then Type*Vth + N*Vt else Type*Vth;
          end if;
        
          /* Compute gate capacitance between gate and bulk */
          if Level==0 or not CGD0 > 0 then
             Cval = 1e-16;
          else
             if noEvent(sig > 0) then
                   Cval = W*CGD0 + (if noEvent(vgs < Von + vds) then 0 else 
                                    2*COXval*(1 - ((vgs - Von)/(2*(vgs - Von) - vds))^2)/3);
             else
                   Cval = W*CGD0 + (if noEvent(vgs < Von - PHIval) then 0 else 
                                    if noEvent(vgs >= Von - PHIval and vgs < Von) then 2*COXval*(1 - (Von - vgs)/PHIval)/3 else 
                                    if noEvent(vgs >= Von) and noEvent(vgs < Von + vds) then 2*COXval/3 else 
                                    2*COXval*(1 - ((vgs - vds - Von)/(2*(vgs - Von) - vds))^2)/3);
             end if;
          end if;
        
          /* Capacitance equation */
          f1 = Cval*der(e1);
        
          /* Computation of heat flow */
          f2 = 0;
        end Cgd;
      
        model DjM "Spice-style junction diode model for MOSFETs" 
          extends Interfaces.SpiceTwoPort;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Integer Level=3 
          "MOSFET modeling level (check documentation window for details)";
          parameter Modelica.SIunits.Current IS=0 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
          Modelica.SIunits.Energy EGnom 
          "Activation energy at reference temperature";
          Modelica.SIunits.Energy EGval 
          "Activation energy at device temperature";
        
      protected 
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Real RTemp "Temperature quotient";
          Real et;
        annotation (
          Diagram(
              Line(points=[-80,0; 80,0],   style(color=10)),
              Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                    fillColor=10)),
              Line(points=[0,80; 0,-80],   style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[4,82; 14,72],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="f"),
              Text(
                extent=[70,-4; 80,-14],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="e"),
              Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[-66,-6; -36,-14],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="blocking"),
              Text(
                extent=[16,50; 48,42],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="conducting"),
              Line(points=[0,0; 6,10; 12,28; 16,66],            style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1))),
          Icon(
              Text(
                extent=[-86,78; 28,-84],
                style(color=0),
              string="DjM")),
          Documentation(info="<html>
The Spice-style MOSFET junction diode is a regular exponential diode. A separate model was needed to implement the <b>Level</b> parameter of the MOSFET models. <p>

<hr> <p>

<pre>
Parameters:

 Level:  MOSFET modeling level (default value = 3)
           Level=0: Static injection model
           Level=1: Shichman-Hodges model
           Level=2: Grove-Frohman model
           Level=3: Empirical model
           Level=4: Simplified Grove-Frohman model

 IS:     Transport saturation current (default value = 0 Amp)

 EG:     Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 EMin:   if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:   if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre>
</html>
"));
        equation 
          /* Compute thermal voltage as function of temperature */
          Vt = k*e2/q;
        
          /* Compute temperature dependence of saturation current */
          RTemp = e2/Tnom;
          EGval = EG - GapC1*e2*e2/(e2 + GapC2);
          EGnom = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          ISval = IS*exp((EGval - EGnom*RTemp)/Vt);
        
          /* Compute diode characteristic */
          et = e1/Vt;
          if Level==0 then
            /* Static injection model */
            f1 = 0;
          else
            /* All other models */
            f1 = ISval*(if noEvent(et < EMin) then ExMin*(et - EMin + 1) - 1 else 
                        if noEvent(et > EMax) then ExMax*(et - EMax + 1) - 1 else exp(et) - 1);
          end if;
        
          /* Compute heat flow */
          e2*f2 = e1*f1;
        end DjM;
      
        model NMOSint 
        "Spice-style internal NMOS transistor in bond graph technology" 
          parameter Integer Level=3 
          "Level of MOS model (check documentation window for details)";
          parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
          "Zero-bias threshold voltage";
          parameter Modelica.SIunits.Voltage VFB=0 "Flat band voltage";
          parameter Modelica.SIunits.Transconductance KP=0 
          "Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
          "Surface inversion potential at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA=0 
          "Channel length modulation";
          parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
          parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
          parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
          "Substrate doping";
          parameter Real NSS(unit="1/m2") = 0 "Surface state density";
          parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
          parameter Real NEFF = 1 "Total channel charge coefficient";
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
          parameter Modelica.SIunits.Length XJ=0 "Metallurgical junction depth";
          parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
          parameter Modelica.SIunits.Area AD=0 "Drain diffusion area";
          parameter Modelica.SIunits.Length PD=0 "Drain perimeter width";
          parameter Modelica.SIunits.Area AS=0 "Source diffusion area";
          parameter Modelica.SIunits.Length PS=0 "Source perimeter width";
          parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
          parameter Real TPG=1 "Type of gate material";
          parameter Real U0(unit="m2/(V.s)") = 0 
          "Surface mobility at reference temperature";
          parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
          "Critical electric field for mobility";
          parameter Real UEXP=0 "Exponential coefficient for mobility";
          parameter Real UTRA=0 "Transverse field coefficient";
          parameter Modelica.SIunits.Velocity VMAX=0 
          "Maximum drift velocity of carriers";
          parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
          "Critical electric field for pinch-off";
          parameter Real XQC=0 "Coefficient of channel charge share";
          parameter Real DELTA=0 "Width effect on threshold voltage";
          parameter Real KAPPA=0.2 "Saturation field factor";
          parameter Real ETA=0 "Static feedback on threshold voltage";
          parameter Modelica.SIunits.InversePotential THETA=0 
          "Mobility modulation";
          parameter Modelica.SIunits.Voltage VBP=0 
          "Gate to channel critical voltage";
          parameter Modelica.SIunits.Current ISD=0 
          "Drain junction saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=0 
          "Source junction saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Real CJ(unit="F/m2") = 0 
          "Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real MJ=0.33 "Bulk junction grading coefficient";
          parameter Real CJSW(unit="F/m") = 1e-9 
          "Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Real MJSW=0.33 "Perimeter capacitance grading coefficient";
          parameter Real FC=0.5 
          "Forward-bias depletion capacitance coefficient";
          parameter Modelica.SIunits.Conductance GminDC=1e-12 
          "Leakage conductance";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (Coordsys(extent=[-140,-140; 140,140]), Diagram(
            Text(
              extent=[130,20; 152,6],
              string="Bint",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-32,146; -10,132],
              style(color=10, rgbcolor={135,135,135}),
              string="Dint"),
            Text(
              extent=[10,-134; 32,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Sint"),
            Text(
              extent=[-152,-8; -130,-22],
              style(color=10, rgbcolor={135,135,135}),
              string="Gint"),
            Text(
              extent=[-60,-136; -38,-150],
              style(color=10, rgbcolor={135,135,135}),
              string="Heat"),
              Text(
                extent=[-110,94; -96,88],
                string="vds",
                style(color=10, rgbcolor={135,135,135})),
              Text(
                extent=[-110,78; -96,72],
                style(color=10, rgbcolor={135,135,135}),
                string="vgs"),
              Text(
                extent=[-74,108; -60,102],
                style(color=10, rgbcolor={135,135,135}),
                string="vbs"),
              Text(
                extent=[-90,64; -76,58],
                style(color=10, rgbcolor={135,135,135}),
                string="Tdev")),
          Icon(
            Text(
              extent=[-78,128; 82,88],
              style(color=3, rgbcolor={0,0,255}),
              string="%name"),
              Line(points=[0,100; 0,-100],
                                         style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Text(
                extent=[-132,90; 98,-98],
                style(color=0),
              string="NMOS"),
              Line(points=[-20,-80; 0,-100],style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[0,-100; 20,-80],style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Text(
                extent=[94,-2; 134,-44],
                style(color=0),
              string="int"),
            Text(
              extent=[128,22; 150,8],
              string="Bint",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[12,146; 34,132],
              style(color=10, rgbcolor={135,135,135}),
              string="Dint"),
            Text(
              extent=[-152,22; -130,8],
              style(color=10, rgbcolor={135,135,135}),
              string="Gint"),
            Text(
              extent=[-60,-134; -38,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Heat"),
            Text(
              extent=[10,-134; 32,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Sint"),
              Line(points=[22,50; 120,50], style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[20,50; 50,60],  style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[20,50; 50,40],  style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1))),
          Documentation(info="<html>
The Spice-style internal n-channel MOSFET model contains the non-linear current source, the two junction diodes (including the junctions capacitances), as well as leakage conductances everywhere to keep the model together. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Zero-bias threshold voltage (default value = 0 Volt)
            <font color=blue><b>VT0 &gt; 0 for enhancement MOSFET
            VT0 &lt; 0 for depletion MOSFET</b></font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>)

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 NEFF:    Total channel charge coefficient (default value = 1)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>)

 PD:      Drain perimeter width (default value = 0 m)

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>)

 PS:      Source perimeter width (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 TPG:     Type of gate material (default value = 1)

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s))

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m)

 UEXP:    Exponential coefficient for mobility (default value = 0)

 UTRA:    Transverse field coefficient (default value = 0)

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s)
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation</b></font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,3</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2,3</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 KAPPA:   Saturation field factor (default value = 0.2)

 ETA:     Static feedback on threshold voltage (default value = 0)

 THETA:   Mobility modulation (default value = 0 1/Volt)

 VBP:     Gate to channel critical voltage (default value = 0 Volt)

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp)

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp)

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 GminDC:  Leakage conductance (default value = 1e-12 mho)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        BondLib.Interfaces.BondCon Bint "Internal bulk" 
          annotation (extent=[130,-10; 150,10]);
        BondLib.Interfaces.BondCon Gint "Internal gate" 
          annotation (extent=[-150,-10; -130,10]);
        BondLib.Interfaces.BondCon Sint "Internal source" 
          annotation (extent=[-10,-150; 10,-130]);
        BondLib.Interfaces.BondCon Dint "Internal drain" 
          annotation (extent=[-10,130; 10,150]);
        BondLib.Interfaces.BondCon H "Heat" 
          annotation (extent=[-80,-150; -60,-130]);
        BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[-130,-10; -110,10]);
        BondLib.Junctions.J0p4 J0p4_2 annotation (extent=[110,-10; 130,10]);
        BondLib.Junctions.J0p4 J0p4_3 annotation (extent=[-10,110; 10,130]);
        BondLib.Junctions.J0p4 J0p4_4 annotation (extent=[-10,-130; 10,-110]);
        BondLib.Bonds.Bond B1 annotation (extent=[40,20; 60,40], rotation=90);
        BondLib.Junctions.J1p3 J1p3_1 
          annotation (extent=[40,40; 60,60], rotation=90);
        BondLib.Bonds.Bond B2 annotation (extent=[40,60; 60,80], rotation=90);
        BondLib.Bonds.Bond B3 annotation (extent=[60,40; 80,60], rotation=0);
        BondLib.Spice.Utilities.CjM Dbd(
          Level=Level,
          Tnom=Tnom,
          IS=ISD,
          PB=PB,
          CJ=CJ,
          MJ=MJ,
          CJSW=CJSW,
          MJSW=MJSW,
          FC=FC,
          A=AD,
          P=PD,
          EG=EG,
          GminDC=GminDC,
          EMin=EMin,
          EMax=EMax)     annotation (extent=[80,38; 110,62]);
        BondLib.Bonds.Bond B4 annotation (extent=[40,-40; 60,-20], rotation=270);
        BondLib.Junctions.J1p3 J1p3_2 
          annotation (extent=[40,-60; 60,-40], rotation=90);
        BondLib.Bonds.Bond B5 annotation (extent=[40,-80; 60,-60], rotation=270);
        BondLib.Bonds.Bond B6 annotation (extent=[60,-60; 80,-40], rotation=0);
        BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[80,-60; 100,-40]);
        BondLib.Bonds.fBond B7 annotation (extent=[100,-60; 120,-40]);
        BondLib.Sensors.De Vbs annotation (extent=[120,-60; 140,-40]);
        BondLib.Bonds.Bond B8 
          annotation (extent=[80,-80; 100,-60], rotation=270);
        BondLib.Spice.Utilities.CjM Dbs(
          Level=Level,
          Tnom=Tnom,
          PB=PB,
          CJ=CJ,
          MJ=MJ,
          CJSW=CJSW,
          MJSW=MJSW,
          FC=FC,
          EG=EG,
          GminDC=GminDC,
          IS=ISS,
          A=AS,
          P=PS,
          EMin=EMin,
          EMax=EMax) 
                annotation (extent=[78,-110; 102,-80], rotation=-90);
        BondLib.Bonds.eBond B9 annotation (extent=[110,40; 130,60]);
        BondLib.Bonds.eBond B10 
          annotation (extent=[80,-130; 100,-110], rotation=-90);
        BondLib.Junctions.J1p3 J1p3_3 
          annotation (extent=[30,-10; 10,10], rotation=90);
        BondLib.Bonds.Bond B11 annotation (extent=[10,10; 30,30], rotation=270);
        BondLib.Bonds.Bond B12 
          annotation (extent=[10,-30; 30,-10], rotation=270);
        BondLib.Bonds.Bond B13 
          annotation (extent=[-10,-10; 10,10], rotation=180);
        BondLib.Junctions.J0p4 J0p4_5 annotation (extent=[-30,-10; -10,10]);
        BondLib.Bonds.eBond B14 annotation (extent=[-50,-10; -30,10]);
        BondLib.Sources.mSf_int ISD0 
                            annotation (extent=[-80,14; -50,-14]);
        BondLib.Bonds.fBond B15 
          annotation (extent=[-30,10; -10,30], rotation=90);
        BondLib.Sensors.De Vds annotation (extent=[-30,30; -10,50], rotation=90);
        BondLib.Bonds.Bond B16 
          annotation (extent=[-30,-30; -10,-10], rotation=270);
        BondLib.Spice.RS Rds(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-32,-80; -8,-30],  rotation=-90);
        BondLib.Bonds.Bond B18 
          annotation (extent=[-130,50; -110,70], rotation=90);
        BondLib.Junctions.J1p3 J1p3_4 
          annotation (extent=[-130,30; -110,50], rotation=-90);
        BondLib.Bonds.Bond B17 
          annotation (extent=[-120,4; -100,24],  rotation=90);
        BondLib.Bonds.Bond B19 
          annotation (extent=[-150,50; -130,70], rotation=90);
        BondLib.Spice.RS Rdg(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-152,70; -128,120], rotation=90);
        BondLib.Bonds.Bond B20 
          annotation (extent=[-130,-30; -110,-10], rotation=270);
        BondLib.Junctions.J1p3 J1p3_5 
          annotation (extent=[-120,-40; -100,-20], rotation=0);
        BondLib.Bonds.Bond B21 
          annotation (extent=[-100,-40; -80,-20], rotation=0);
        BondLib.Bonds.Bond B22 
          annotation (extent=[-120,-60; -100,-40], rotation=270);
        BondLib.Spice.RS Rgs(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-152,-140; -128,-90], rotation=-90);
        BondLib.Junctions.J0p3 J0p3_2 
          annotation (extent=[-120,-80; -100,-60], rotation=-90);
        BondLib.Bonds.Bond B24 
          annotation (extent=[-150,-90; -130,-70], rotation=270);
        BondLib.Sensors.De Vgs 
          annotation (extent=[-120,-120; -100,-100], rotation=-90);
        BondLib.Bonds.eBond B25 
          annotation (extent=[-30,-100; -10,-80], rotation=-90);
        BondLib.Bonds.eBond B26 annotation (extent=[-130,-150; -110,-130]);
        BondLib.Bonds.eBond B27 
          annotation (extent=[-150,120; -130,140], rotation=90);
        BondLib.Bonds.fBond B28 annotation (extent=[-100,-10; -80,10]);
          BondLib.Spice.Utilities.IDS0 IDS0(
          Tnom=Tnom,
          KP=KP,
          GAMMA=GAMMA,
          LAMBDA=LAMBDA,
          NFS=NFS,
          NEFF=NEFF,
          XJ=XJ,
          U0=U0,
          UEXP=UEXP,
          UTRA=UTRA,
          VMAX=VMAX,
          DELTA=DELTA,
          EG=EG,
          L=L,
          W=W,
          Level=Level,
          EMin=EMin,
          EMax=EMax,
          PHI=PHI,
          Type=Type,
          VFB=VFB,
          COX=COX,
          XD=XD,
          ECRIT=ECRIT,
          VBP=VBP,
          PB=PB,
          KAPPA=KAPPA,
          ETA=ETA,
          THETA=THETA,
          LD=LD)       annotation (extent=[-90,66; -60,94]);
          Modelica.Blocks.Interfaces.RealOutput y1[5] 
          "Connector of Real output signals" 
            annotation (extent=[-80,130; -60,150],rotation=90);
        Modelica.Blocks.Math.Gain Gain1(k=-1) 
          annotation (extent=[-60,40; -40,60],     rotation=-90);
        BondLib.Junctions.J0p6 J0p6_1 
          annotation (extent=[-80,-130; -60,-110], rotation=90);
        BondLib.Bonds.eBond B29 
          annotation (extent=[-80,-110; -60,-90], rotation=-90);
        BondLib.Sensors.De Tdev 
          annotation (extent=[-86,40; -64,60], rotation=90);
        BondLib.Bonds.fBond B30 
          annotation (extent=[-86,20; -64,40], rotation=90);
        BondLib.Spice.RS Rbg(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[90,98; 138,122],    rotation=0);
        BondLib.Bonds.Bond B31 
                              annotation (extent=[70,100; 90,120],
                                                                 rotation=0);
        BondLib.Bonds.eBond B32 
                               annotation (extent=[130,80; 150,100], rotation=
              -90);
        Junctions.J0p5 J0p5_1 
          annotation (extent=[-80,-90; -60,-70], rotation=90);
        BondLib.Junctions.J1p3 J1p3_6 
          annotation (extent=[50,120; 70,140],   rotation=0);
        BondLib.Bonds.Bond B33 
                              annotation (extent=[30,120; 50,140],
                                                                 rotation=0);
        BondLib.Bonds.Bond B34 
                              annotation (extent=[70,120; 90,140],
                                                                 rotation=0);
        BondLib.Bonds.fBond B23 
                               annotation (extent=[-120,-100; -100,-80],
            rotation=-90);
        equation 
        connect(J0p4_1.BondCon1, Gint)     annotation (points=[-130,0; -140,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_2.BondCon2, Bint)     annotation (points=[130,0; 140,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_3.BondCon4, Dint)     annotation (points=[0,130; 0,140],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_4.BondCon3, Sint)     annotation (points=[0,-130; 0,-140],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon1, B1.BondCon2) annotation (points=[50,40; 50,40],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.BondCon1, J1p3_1.BondCon2) annotation (points=[50,60.2; 50,
              60], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[60,50; 60.2,
              50], style(color=8, rgbcolor={192,192,192}));
        connect(B3.BondCon2,Dbd. BondCon1) annotation (points=[80,50; 80,50],
            style(color=8, rgbcolor={192,192,192}));
        connect(B1.BondCon1, J0p4_2.BondCon1) annotation (points=[50,20.2; 50,0;
              110,0], style(color=8, rgbcolor={192,192,192}));
        connect(B4.BondCon2, J1p3_2.BondCon2) annotation (points=[50,-40; 50,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_2.BondCon1, B5.BondCon1) annotation (points=[50,-60; 50,
              -60.2], style(color=8, rgbcolor={192,192,192}));
        connect(B5.BondCon2, J0p4_4.BondCon2) annotation (points=[50,-80; 50,
              -120; 10,-120], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_2.BondCon3, B6.BondCon1) annotation (points=[60,-50; 60.2,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(B6.BondCon2, J0p3_1.BondCon1) annotation (points=[80,-50; 80,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon2, B7.eBondCon1) annotation (points=[100,-50; 100,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(B7.fBondCon1,Vbs. BondCon1) annotation (points=[120,-50; 120,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon3, B8.BondCon1) annotation (points=[90,-60; 90,
              -60.2], style(color=8, rgbcolor={192,192,192}));
        connect(B8.BondCon2,Dbs. BondCon1) annotation (points=[90,-80; 90,-80],
            style(color=8, rgbcolor={192,192,192}));
        connect(B4.BondCon1, J0p4_2.BondCon3) annotation (points=[50,-20.2; 50,
              -10; 120,-10], style(color=8, rgbcolor={192,192,192}));
        connect(Dbd.BondCon2, B9.fBondCon1) annotation (points=[110,50; 110,50],
            style(color=8, rgbcolor={192,192,192}));
        connect(Dbs.BondCon2, B10.fBondCon1) annotation (points=[90,-110; 90,
              -110], style(color=8, rgbcolor={192,192,192}));
        connect(B11.BondCon2, J1p3_3.BondCon2) annotation (points=[20,10; 20,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_3.BondCon1, B12.BondCon1) annotation (points=[20,-10; 20,
              -10.1; 20,-10.2; 20,-10.2], style(color=8, rgbcolor={192,192,192}));
        connect(B11.BondCon1, J0p4_3.BondCon3) annotation (points=[20,29.8; 20,
              80; 0,80; 0,110],   style(color=8, rgbcolor={192,192,192}));
        connect(B12.BondCon2, J0p4_4.BondCon4) annotation (points=[20,-30; 20,
              -100; 0,-100; 0,-110], style(color=8, rgbcolor={192,192,192}));
        connect(B13.BondCon1, J1p3_3.BondCon3) annotation (points=[9.8,
              -1.20011e-015; 2.9,-1.20011e-015; 2.9,-6.12303e-016; 10,
              -6.12303e-016], style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_5.BondCon2, B13.BondCon2) annotation (points=[-10,0; -16,0;
              -16,1.22461e-015; -10,1.22461e-015], style(color=8, rgbcolor={192,
                192,192}));
        connect(B14.eBondCon1, J0p4_5.BondCon1) annotation (points=[-30,0; -30,
              0], style(color=8, rgbcolor={192,192,192}));
        connect(B15.eBondCon1, J0p4_5.BondCon4) annotation (points=[-20,10; -20,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(Vds.BondCon1, B15.fBondCon1) annotation (points=[-20,30; -20,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_5.BondCon3, B16.BondCon1) annotation (points=[-20,-10; -20,
              -10.1; -20,-10.2; -20,-10.2], style(color=8, rgbcolor={192,192,
                192}));
        connect(B16.BondCon2,Rds. BondCon1) annotation (points=[-20,-30; -20,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_4.BondCon2, B17.BondCon2) annotation (points=[-120,30;
              -110,30; -110,24],
                        style(color=8, rgbcolor={192,192,192}));
        connect(B18.BondCon1, J1p3_4.BondCon1) annotation (points=[-120,50.2;
              -120,50], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_4.BondCon3, B19.BondCon1) annotation (points=[-130,40;
              -140,40; -140,50.2], style(color=8, rgbcolor={192,192,192}));
        connect(Rdg.BondCon1, B19.BondCon2) annotation (points=[-140,70; -140,
              70], style(color=8, rgbcolor={192,192,192}));
        connect(B18.BondCon2, J0p4_3.BondCon1) annotation (points=[-120,70;
              -120,120; -10,120], style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_1.BondCon3, B20.BondCon1) annotation (points=[-120,-10;
              -120,-10.2], style(color=8, rgbcolor={192,192,192}));
        connect(B20.BondCon2, J1p3_5.BondCon1) annotation (points=[-120,-30;
              -120,-30], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_5.BondCon3, B22.BondCon1) annotation (points=[-110,-40;
              -110,-40.2], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon1, B22.BondCon2) annotation (points=[-110,-60;
              -110,-60], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon3, B24.BondCon1) annotation (points=[-120,-70;
              -130,-70; -130,-70.2; -140,-70.2], style(color=8, rgbcolor={192,
                192,192}));
        connect(B24.BondCon2, Rgs.BondCon1)  annotation (points=[-140,-90; -140,
              -90], style(color=8, rgbcolor={192,192,192}));
        connect(Rds.BondCon2, B25.fBondCon1) annotation (points=[-20,-80; -20,
              -80], style(color=8, rgbcolor={192,192,192}));
        connect(Rgs.BondCon2, B26.fBondCon1)  annotation (points=[-140,-140;
              -130,-140], style(color=8, rgbcolor={192,192,192}));
        connect(B27.fBondCon1,Rdg. BondCon2) annotation (points=[-140,120; -140,
              120], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_5.BondCon2, B21.BondCon1) annotation (points=[-100,-30;
              -99.8,-30], style(color=8, rgbcolor={192,192,192}));
        connect(B21.BondCon2, J0p4_4.BondCon1) annotation (points=[-80,-30; -34,
              -30; -34,-120; -10,-120], style(color=8, rgbcolor={192,192,192}));
        connect(IDS0.y, Gain1.u)   annotation (points=[-58.5,80; -50,80; -50,62],
            style(color=74, rgbcolor={0,0,127}));
        connect(IDS0.y1, y1)   annotation (points=[-58.5,73; -54,73; -54,120;
              -70,120; -70,140], style(color=74, rgbcolor={0,0,127}));
        connect(Vds.OutPort1, IDS0.u1)   annotation (points=[-20,51; -20,112;
              -110,112; -110,88.4; -93,88.4], style(color=74, rgbcolor={0,0,127}));
        connect(Vgs.OutPort1, IDS0.u2)   annotation (points=[-110,-121; -110,
              -126; -96,-126; -96,40; -110,40; -110,71.6; -93,71.6], style(
              color=74, rgbcolor={0,0,127}));
        connect(ISD0.BondCon2, B14.fBondCon1) annotation (points=[-50,
              -1.77636e-015; -50,0], style(color=8, rgbcolor={192,192,192}));
        connect(ISD0.BondCon1, B28.fBondCon1) annotation (points=[-80,
              -1.77636e-015; -80,0], style(color=8, rgbcolor={192,192,192}));
        connect(J0p6_1.BondCon1, H)        annotation (points=[-70,-130; -70,
              -140], style(color=8, rgbcolor={192,192,192}));
        connect(B29.eBondCon1, J0p6_1.BondCon2) annotation (points=[-70,-110;
              -70,-110], style(color=8, rgbcolor={192,192,192}));
        connect(IDS0.Tdev, Tdev.OutPort1)   annotation (points=[-75,66; -75,61],
            style(color=74, rgbcolor={0,0,127}));
        connect(Tdev.BondCon1, B30.fBondCon1) annotation (points=[-75,40; -75,
              40], style(color=8, rgbcolor={192,192,192}));
        connect(B26.eBondCon1, J0p6_1.BondCon6) annotation (points=[-110,-140;
              -92,-140; -92,-116; -80,-116], style(color=8, rgbcolor={192,192,
                192}));
        connect(B27.eBondCon1, J0p6_1.BondCon5) annotation (points=[-140,140;
              -158,140; -158,-152; -88,-152; -88,-124; -80,-124], style(color=8,
              rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon1, B29.fBondCon1) annotation (points=[-70,-90;
              -70,-90], style(color=8, rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon5, B28.eBondCon1) annotation (points=[-80,-80;
              -92,-80; -92,-16; -104,-16; -104,0; -100,0], style(color=8,
              rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon2, B30.eBondCon1) annotation (points=[-70,-70;
              -70,-26; -44,-26; -44,16; -75,16; -75,20], style(color=8,
              rgbcolor={192,192,192}));
        connect(Vbs.OutPort1, IDS0.u3)   annotation (points=[141,-50; 158,-50;
              158,160; -88,160; -88,108; -75,108; -75,94], style(color=74,
              rgbcolor={0,0,127}));
        connect(B31.BondCon2,Rbg. BondCon1) annotation (points=[90,110; 90,110],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.BondCon2, J0p4_3.BondCon2) annotation (points=[50,80; 50,86;
              16,86; 16,120; 10,120], style(color=8, rgbcolor={192,192,192}));
        connect(Rbg.BondCon2, B32.fBondCon1) annotation (points=[138,110; 140,
              110; 140,100], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_6.BondCon3, B31.BondCon1) annotation (points=[60,120; 60,
              110; 70.2,110], style(color=8, rgbcolor={192,192,192}));
        connect(B33.BondCon2, J1p3_6.BondCon1) annotation (points=[50,130; 50,
              130], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_6.BondCon2, B34.BondCon1) annotation (points=[70,130; 70.2,
              130], style(color=8, rgbcolor={192,192,192}));
        connect(B34.BondCon2, J0p4_2.BondCon4) annotation (points=[90,130; 154,
              130; 154,30; 120,30; 120,10], style(color=8, rgbcolor={192,192,
                192}));
        connect(B17.BondCon1, J0p4_1.BondCon2) annotation (points=[-110,4.2;
              -110,0], style(color=8, rgbcolor={192,192,192}));
        connect(B33.BondCon1, J0p4_1.BondCon4) annotation (points=[30.2,130; 20,
              130; 20,164; -162,164; -162,20; -120,20; -120,10], style(color=8,
              rgbcolor={192,192,192}));
        connect(B25.eBondCon1, J0p5_1.BondCon4) annotation (points=[-20,-100;
              -20,-106; -42,-106; -42,-76; -60,-76], style(color=8, rgbcolor={
                192,192,192}));
        connect(B10.eBondCon1, J0p5_1.BondCon3) annotation (points=[90,-130; 90,
              -154; -16,-154; -16,-124; -46,-124; -46,-84; -60,-84], style(
              color=8, rgbcolor={192,192,192}));
        connect(B9.eBondCon1, J0p6_1.BondCon4) annotation (points=[130,50; 162,
              50; 162,-158; -20,-158; -20,-128; -50,-128; -50,-116; -60,-116],
            style(color=8, rgbcolor={192,192,192}));
        connect(B32.eBondCon1, J0p6_1.BondCon3) annotation (points=[140,80; 142,
              80; 142,66; 166,66; 166,-162; -24,-162; -24,-132; -54,-132; -54,
              -124; -60,-124], style(color=8, rgbcolor={192,192,192}));
        connect(Vgs.BondCon1, B23.fBondCon1) annotation (points=[-110,-100;
              -110,-100], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon2, B23.eBondCon1) annotation (points=[-110,-80;
              -110,-80], style(color=8, rgbcolor={192,192,192}));
        connect(Gain1.y, ISD0.s) annotation (points=[-50,39; -50,30; -65,30;
              -65,12.6], style(color=74, rgbcolor={0,0,127}));
        end NMOSint;
      
        model PMOSint 
        "Spice-style internal PMOS transistor in bond graph technology" 
          parameter Integer Level=3 
          "Level of MOS model (check documentation window for details)";
          parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
          "Zero-bias threshold voltage";
          parameter Modelica.SIunits.Voltage VFB=0 "Flat band voltage";
          parameter Modelica.SIunits.Transconductance KP=0 
          "Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
          "Surface inversion potential at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA=0 
          "Channel length modulation";
          parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
          parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
          parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
          "Substrate doping";
          parameter Real NSS(unit="1/m2") = 0 "Surface state density";
          parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
          parameter Real NEFF = 1 "Total channel charge coefficient";
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
          parameter Modelica.SIunits.Length XJ=0 "Metallurgical junction depth";
          parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
          parameter Modelica.SIunits.Area AD=0 "Drain diffusion area";
          parameter Modelica.SIunits.Length PD=0 "Drain perimeter width";
          parameter Modelica.SIunits.Area AS=0 "Source diffusion area";
          parameter Modelica.SIunits.Length PS=0 "Source perimeter width";
          parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
          parameter Real TPG=1 "Type of gate material";
          parameter Real U0(unit="m2/(V.s)") = 0 
          "Surface mobility at reference temperature";
          parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
          "Critical electric field for mobility";
          parameter Real UEXP=0 "Exponential coefficient for mobility";
          parameter Real UTRA=0 "Transverse field coefficient";
          parameter Modelica.SIunits.Velocity VMAX=0 
          "Maximum drift velocity of carriers";
          parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
          "Critical electric field for pinch-off";
          parameter Real XQC=0 "Coefficient of channel charge share";
          parameter Real DELTA=0 "Width effect on threshold voltage";
          parameter Real KAPPA=0.2 "Saturation field factor";
          parameter Real ETA=0 "Static feedback on threshold voltage";
          parameter Modelica.SIunits.InversePotential THETA=0 
          "Mobility modulation";
          parameter Modelica.SIunits.Voltage VBP=0 
          "Gate to channel critical voltage";
          parameter Modelica.SIunits.Current ISD=0 
          "Drain junction saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=0 
          "Source junction saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Real CJ(unit="F/m2") = 0 
          "Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real MJ=0.33 "Bulk junction grading coefficient";
          parameter Real CJSW(unit="F/m") = 1e-9 
          "Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Real MJSW=0.33 "Perimeter capacitance grading coefficient";
          parameter Real FC=0.5 
          "Forward-bias depletion capacitance coefficient";
          parameter Modelica.SIunits.Conductance GminDC=1e-12 
          "Leakage conductance";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (
          Coordsys(extent=[-140,-140; 140,140]),
          Diagram(
            Text(
              extent=[130,20; 152,6],
              string="Bint",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-32,146; -10,132],
              style(color=10, rgbcolor={135,135,135}),
              string="Dint"),
            Text(
              extent=[10,-134; 32,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Sint"),
            Text(
              extent=[-152,-8; -130,-22],
              style(color=10, rgbcolor={135,135,135}),
              string="Gint"),
            Text(
              extent=[-60,-136; -38,-150],
              style(color=10, rgbcolor={135,135,135}),
              string="Heat"),
              Text(
                extent=[-110,94; -96,88],
                style(color=10, rgbcolor={135,135,135}),
              string="vsd"),
              Text(
                extent=[-110,78; -96,72],
                style(color=10, rgbcolor={135,135,135}),
              string="vsg"),
              Text(
                extent=[-74,108; -60,102],
                style(color=10, rgbcolor={135,135,135}),
              string="vsb"),
              Text(
                extent=[-90,64; -76,58],
                style(color=10, rgbcolor={135,135,135}),
                string="Tdev")),
          Icon(
            Text(
              extent=[-78,128; 82,88],
              style(color=3, rgbcolor={0,0,255}),
              string="%name"),
              Line(points=[0,100; 0,-100],
                                         style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Text(
                extent=[-132,92; 98,-96],
                style(color=0),
              string="PMOS"),
              Line(points=[0,100; 20,80],   style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[-20,80; 0,100], style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[22,50; 120,50], style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[90,40; 120,50], style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[90,60; 120,50], style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
            Text(
              extent=[-152,22; -130,8],
              style(color=10, rgbcolor={135,135,135}),
              string="Gint"),
              Text(
                extent=[94,-2; 134,-44],
                style(color=0),
              string="int"),
            Text(
              extent=[128,22; 150,8],
              string="Bint",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-60,-134; -38,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Heat"),
            Text(
              extent=[10,-134; 32,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Sint"),
            Text(
              extent=[12,146; 34,132],
              style(color=10, rgbcolor={135,135,135}),
              string="Dint")),
          Documentation(info="<html>
The Spice-style internal p-channel MOSFET model contains the non-linear current source, the two junction diodes (including the junctions capacitances), as well as leakage conductances everywhere to keep the model together. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Zero-bias threshold voltage (default value = 0 Volt)
            <font color=blue><b>VT0 &lt; 0 for enhancement MOSFET
            VT0 &gt; 0 for depletion MOSFET</b></font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>)

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 NEFF:    Total channel charge coefficient (default value = 1)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>)

 PD:      Drain perimeter width (default value = 0 m)

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>)

 PS:      Source perimeter width (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 TPG:     Type of gate material (default value = 1)

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s))

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m)

 UEXP:    Exponential coefficient for mobility (default value = 0)

 UTRA:    Transverse field coefficient (default value = 0)

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s)
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation</b></font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,3</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2,3</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 KAPPA:   Saturation field factor (default value = 0.2)

 ETA:     Static feedback on threshold voltage (default value = 0)

 THETA:   Mobility modulation (default value = 0 1/Volt)

 VBP:     Gate to channel critical voltage (default value = 0 Volt)

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp)

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp)

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 GminDC:  Leakage conductance (default value = 1e-12 mho)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        BondLib.Interfaces.BondCon Bint "Internal bulk" 
          annotation (extent=[130,-10; 150,10]);
        BondLib.Interfaces.BondCon Gint "Internal gate" 
          annotation (extent=[-150,-10; -130,10]);
        BondLib.Interfaces.BondCon Sint "Internal source" 
          annotation (extent=[-10,-150; 10,-130]);
        BondLib.Interfaces.BondCon Dint "Internal drain" 
          annotation (extent=[-10,130; 10,150]);
        BondLib.Interfaces.BondCon H "Heat" 
          annotation (extent=[-80,-150; -60,-130]);
        BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[-130,-10; -110,10]);
        BondLib.Junctions.J0p4 J0p4_2 annotation (extent=[110,-10; 130,10]);
        BondLib.Junctions.J0p4 J0p4_3 annotation (extent=[-10,110; 10,130]);
        BondLib.Junctions.J0p4 J0p4_4 annotation (extent=[-10,-130; 10,-110]);
        BondLib.Bonds.Bond B1 annotation (extent=[40,20; 60,40], rotation=270);
        BondLib.Junctions.J1p3 J1p3_1 
          annotation (extent=[40,40; 60,60], rotation=90);
        BondLib.Bonds.Bond B2 annotation (extent=[40,60; 60,80], rotation=270);
        BondLib.Bonds.Bond B3 annotation (extent=[60,40; 80,60], rotation=0);
        BondLib.Spice.Utilities.CjM Ddb(
          Level=Level,
          Tnom=Tnom,
          IS=ISD,
          PB=PB,
          CJ=CJ,
          MJ=MJ,
          CJSW=CJSW,
          MJSW=MJSW,
          FC=FC,
          A=AD,
          P=PD,
          EG=EG,
          GminDC=GminDC,
          EMin=EMin,
          EMax=EMax)     annotation (extent=[80,38; 110,62]);
        BondLib.Bonds.Bond B4 annotation (extent=[40,-40; 60,-20], rotation=90);
        BondLib.Junctions.J1p3 J1p3_2 
          annotation (extent=[40,-60; 60,-40], rotation=90);
        BondLib.Bonds.Bond B5 annotation (extent=[40,-80; 60,-60], rotation=90);
        BondLib.Bonds.Bond B6 annotation (extent=[60,-60; 80,-40], rotation=0);
        BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[80,-60; 100,-40]);
        BondLib.Bonds.fBond B7 annotation (extent=[100,-60; 120,-40]);
        BondLib.Sensors.De Vsb annotation (extent=[120,-60; 140,-40]);
        BondLib.Bonds.Bond B8 
          annotation (extent=[80,-80; 100,-60], rotation=270);
        BondLib.Spice.Utilities.CjM Dsb(
          Level=Level,
          Tnom=Tnom,
          PB=PB,
          CJ=CJ,
          MJ=MJ,
          CJSW=CJSW,
          MJSW=MJSW,
          FC=FC,
          EG=EG,
          GminDC=GminDC,
          IS=ISS,
          A=AS,
          P=PS) annotation (extent=[78,-110; 102,-80], rotation=-90);
        BondLib.Bonds.eBond B9 annotation (extent=[110,40; 130,60]);
        BondLib.Bonds.eBond B10 
          annotation (extent=[80,-130; 100,-110], rotation=-90);
        BondLib.Junctions.J1p3 J1p3_3 
          annotation (extent=[30,-10; 10,10], rotation=90);
        BondLib.Bonds.Bond B11 annotation (extent=[10,10; 30,30], rotation=90);
        BondLib.Bonds.Bond B12 
          annotation (extent=[10,-30; 30,-10], rotation=90);
        BondLib.Bonds.Bond B13 
          annotation (extent=[-10,-10; 10,10], rotation=180);
        BondLib.Junctions.J0p4 J0p4_5 annotation (extent=[-30,-10; -10,10]);
        BondLib.Bonds.eBond B14 annotation (extent=[-50,-10; -30,10]);
        BondLib.Sources.mSf_int ISD0 
                            annotation (extent=[-80,14; -50,-14]);
        BondLib.Bonds.fBond B15 
          annotation (extent=[-30,10; -10,30], rotation=90);
        BondLib.Sensors.De Vsd annotation (extent=[-30,30; -10,50], rotation=90);
        BondLib.Bonds.Bond B16 
          annotation (extent=[-30,-30; -10,-10], rotation=270);
        BondLib.Spice.RS Rsd(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-32,-80; -8,-30],  rotation=-90);
        BondLib.Bonds.Bond B18 
          annotation (extent=[-130,50; -110,70], rotation=90);
        BondLib.Junctions.J1p3 J1p3_4 
          annotation (extent=[-130,30; -110,50], rotation=-90);
        BondLib.Bonds.Bond B17 
          annotation (extent=[-120,4; -100,24],  rotation=90);
        BondLib.Bonds.Bond B19 
          annotation (extent=[-150,50; -130,70], rotation=90);
        BondLib.Spice.RS Rgd(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-152,70; -128,120], rotation=90);
        BondLib.Bonds.Bond B20 
          annotation (extent=[-130,-30; -110,-10], rotation=90);
        BondLib.Junctions.J1p3 J1p3_5 
          annotation (extent=[-120,-40; -100,-20], rotation=0);
        BondLib.Bonds.Bond B21 
          annotation (extent=[-100,-40; -80,-20], rotation=180);
        BondLib.Bonds.Bond B22 
          annotation (extent=[-120,-60; -100,-40], rotation=270);
        BondLib.Spice.RS Rsg(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-152,-140; -128,-90], rotation=-90);
        BondLib.Junctions.J0p3 J0p3_2 
          annotation (extent=[-120,-80; -100,-60], rotation=-90);
        BondLib.Bonds.Bond B23 
          annotation (extent=[-120,-100; -100,-80], rotation=270);
        BondLib.Bonds.Bond B24 
          annotation (extent=[-150,-90; -130,-70], rotation=270);
        BondLib.Sensors.De Vsg 
          annotation (extent=[-120,-120; -100,-100], rotation=-90);
        BondLib.Bonds.eBond B25 
          annotation (extent=[-30,-100; -10,-80], rotation=-90);
        BondLib.Bonds.eBond B26 annotation (extent=[-130,-150; -110,-130]);
        BondLib.Bonds.eBond B27 
          annotation (extent=[-150,120; -130,140], rotation=90);
        BondLib.Bonds.fBond B28 annotation (extent=[-100,-10; -80,10]);
          BondLib.Spice.Utilities.IDS0 IDS0(
          Tnom=Tnom,
          KP=KP,
          GAMMA=GAMMA,
          LAMBDA=LAMBDA,
          NFS=NFS,
          NEFF=NEFF,
          XJ=XJ,
          U0=U0,
          UEXP=UEXP,
          UTRA=UTRA,
          VMAX=VMAX,
          DELTA=DELTA,
          EG=EG,
          L=L,
          W=W,
          Level=Level,
          EMin=EMin,
          EMax=EMax,
          PHI=PHI,
          Type=Type,
          VFB=VFB,
          COX=COX,
          XD=XD,
          ECRIT=ECRIT,
          VBP=VBP,
          PB=PB,
          KAPPA=KAPPA,
          ETA=ETA,
          THETA=THETA,
          LD=LD)       annotation (extent=[-90,66; -60,94]);
          Modelica.Blocks.Interfaces.RealOutput y1[5] 
          "Connector of Real output signals" 
            annotation (extent=[-80,130; -60,150],rotation=90);
        Modelica.Blocks.Math.Gain Gain1(k=-1) 
          annotation (extent=[-60,40; -40,60],     rotation=-90);
        BondLib.Junctions.J0p6 J0p6_1 
          annotation (extent=[-80,-130; -60,-110], rotation=90);
        BondLib.Bonds.eBond B29 
          annotation (extent=[-80,-110; -60,-90], rotation=-90);
        BondLib.Sensors.De Tdev 
          annotation (extent=[-86,40; -64,60], rotation=90);
        BondLib.Bonds.fBond B30 
          annotation (extent=[-86,20; -64,40], rotation=90);
        BondLib.Spice.RS Rgb(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[90,98; 138,122],    rotation=0);
        BondLib.Bonds.Bond B31 
                              annotation (extent=[70,100; 90,120],
                                                                 rotation=0);
        BondLib.Bonds.eBond B32 
                               annotation (extent=[130,80; 150,100], rotation=
              -90);
        BondLib.Junctions.J0p5 J0p5_1 
          annotation (extent=[-80,-90; -60,-70], rotation=90);
        BondLib.Junctions.J1p3 J1p3_6 
          annotation (extent=[50,120; 70,140],   rotation=0);
        BondLib.Bonds.Bond B33 
                              annotation (extent=[30,120; 50,140],
                                                                 rotation=0);
        BondLib.Bonds.Bond B34 
                              annotation (extent=[70,120; 90,140],
                                                                 rotation=0);
        equation 
        connect(J0p4_1.BondCon1, Gint)     annotation (points=[-130,0; -140,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_2.BondCon2, Bint)     annotation (points=[130,0; 140,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_3.BondCon4, Dint)     annotation (points=[0,130; 0,140],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_4.BondCon3, Sint)     annotation (points=[0,-130; 0,-140],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[60,50; 60.2,
              50], style(color=8, rgbcolor={192,192,192}));
        connect(B3.BondCon2,Ddb. BondCon1) annotation (points=[80,50; 80,50],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_2.BondCon3,B6. BondCon1) annotation (points=[60,-50; 60.2,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(B6.BondCon2,J0p3_1. BondCon1) annotation (points=[80,-50; 80,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon2,B7. eBondCon1) annotation (points=[100,-50; 100,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(B7.fBondCon1,Vsb. BondCon1) annotation (points=[120,-50; 120,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon3,B8. BondCon1) annotation (points=[90,-60; 90,
              -60.2], style(color=8, rgbcolor={192,192,192}));
        connect(B8.BondCon2,Dsb. BondCon1) annotation (points=[90,-80; 90,-80],
            style(color=8, rgbcolor={192,192,192}));
        connect(Ddb.BondCon2,B9. fBondCon1) annotation (points=[110,50; 110,50],
            style(color=8, rgbcolor={192,192,192}));
        connect(Dsb.BondCon2,B10. fBondCon1) annotation (points=[90,-110; 90,
              -110], style(color=8, rgbcolor={192,192,192}));
        connect(B13.BondCon1,J1p3_3. BondCon3) annotation (points=[9.8,
              -1.20011e-015; 2.9,-1.20011e-015; 2.9,-6.12303e-016; 10,
              -6.12303e-016], style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_5.BondCon2,B13. BondCon2) annotation (points=[-10,0; -16,0;
              -16,1.22461e-015; -10,1.22461e-015], style(color=8, rgbcolor={192,
                192,192}));
        connect(B14.eBondCon1,J0p4_5. BondCon1) annotation (points=[-30,0; -30,
              0], style(color=8, rgbcolor={192,192,192}));
        connect(B15.eBondCon1,J0p4_5. BondCon4) annotation (points=[-20,10; -20,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(Vsd.BondCon1,B15. fBondCon1) annotation (points=[-20,30; -20,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_5.BondCon3,B16. BondCon1) annotation (points=[-20,-10; -20,
              -10.1; -20,-10.2; -20,-10.2], style(color=8, rgbcolor={192,192,
                192}));
        connect(B16.BondCon2,Rsd. BondCon1) annotation (points=[-20,-30; -20,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_4.BondCon2,B17. BondCon2) annotation (points=[-120,30;
              -110,30; -110,24],
                        style(color=8, rgbcolor={192,192,192}));
        connect(B18.BondCon1,J1p3_4. BondCon1) annotation (points=[-120,50.2;
              -120,50], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_4.BondCon3,B19. BondCon1) annotation (points=[-130,40;
              -140,40; -140,50.2], style(color=8, rgbcolor={192,192,192}));
        connect(Rgd.BondCon1,B19. BondCon2) annotation (points=[-140,70; -140,
              70], style(color=8, rgbcolor={192,192,192}));
        connect(B18.BondCon2,J0p4_3. BondCon1) annotation (points=[-120,70;
              -120,120; -10,120], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_5.BondCon3,B22. BondCon1) annotation (points=[-110,-40;
              -110,-40.2], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon1,B22. BondCon2) annotation (points=[-110,-60;
              -110,-60], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon2,B23. BondCon1) annotation (points=[-110,-80;
              -110,-80.2], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon3,B24. BondCon1) annotation (points=[-120,-70;
              -130,-70; -130,-70.2; -140,-70.2], style(color=8, rgbcolor={192,
                192,192}));
        connect(B24.BondCon2, Rsg.BondCon1)  annotation (points=[-140,-90; -140,
              -90], style(color=8, rgbcolor={192,192,192}));
        connect(B23.BondCon2,Vsg. BondCon1) annotation (points=[-110,-100; -110,
              -100], style(color=8, rgbcolor={192,192,192}));
        connect(Rsd.BondCon2,B25. fBondCon1) annotation (points=[-20,-80; -20,
              -80], style(color=8, rgbcolor={192,192,192}));
        connect(Rsg.BondCon2, B26.fBondCon1)  annotation (points=[-140,-140;
              -130,-140], style(color=8, rgbcolor={192,192,192}));
        connect(B27.fBondCon1,Rgd. BondCon2) annotation (points=[-140,120; -140,
              120], style(color=8, rgbcolor={192,192,192}));
        connect(IDS0.y, Gain1.u)   annotation (points=[-58.5,80; -50,80; -50,62],
            style(color=74, rgbcolor={0,0,127}));
        connect(IDS0.y1, y1)   annotation (points=[-58.5,73; -54,73; -54,120;
              -70,120; -70,140], style(color=74, rgbcolor={0,0,127}));
        connect(Vsd.OutPort1, IDS0.u1)   annotation (points=[-20,51; -20,112;
              -110,112; -110,88.4; -93,88.4], style(color=74, rgbcolor={0,0,127}));
        connect(Vsg.OutPort1, IDS0.u2)   annotation (points=[-110,-121; -110,
              -126; -96,-126; -96,40; -110,40; -110,71.6; -93,71.6], style(
              color=74, rgbcolor={0,0,127}));
        connect(ISD0.BondCon2,B14. fBondCon1) annotation (points=[-50,
              -1.77636e-015; -50,0], style(color=8, rgbcolor={192,192,192}));
        connect(ISD0.BondCon1,B28. fBondCon1) annotation (points=[-80,
              -1.77636e-015; -80,0], style(color=8, rgbcolor={192,192,192}));
        connect(J0p6_1.BondCon1, H)        annotation (points=[-70,-130; -70,
              -140], style(color=8, rgbcolor={192,192,192}));
        connect(B29.eBondCon1,J0p6_1. BondCon2) annotation (points=[-70,-110;
              -70,-110], style(color=8, rgbcolor={192,192,192}));
        connect(IDS0.Tdev, Tdev.OutPort1)   annotation (points=[-75,66; -75,61],
            style(color=74, rgbcolor={0,0,127}));
        connect(Tdev.BondCon1,B30. fBondCon1) annotation (points=[-75,40; -75,
              40], style(color=8, rgbcolor={192,192,192}));
        connect(B26.eBondCon1,J0p6_1. BondCon6) annotation (points=[-110,-140;
              -92,-140; -92,-116; -80,-116], style(color=8, rgbcolor={192,192,
                192}));
        connect(B27.eBondCon1,J0p6_1. BondCon5) annotation (points=[-140,140;
              -158,140; -158,-152; -88,-152; -88,-124; -80,-124], style(color=8,
              rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon1,B29. fBondCon1) annotation (points=[-70,-90;
              -70,-90], style(color=8, rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon5,B28. eBondCon1) annotation (points=[-80,-80;
              -92,-80; -92,-16; -104,-16; -104,0; -100,0], style(color=8,
              rgbcolor={192,192,192}));
        connect(J0p5_1.BondCon2,B30. eBondCon1) annotation (points=[-70,-70;
              -70,-26; -44,-26; -44,16; -75,16; -75,20], style(color=8,
              rgbcolor={192,192,192}));
        connect(Vsb.OutPort1, IDS0.u3)   annotation (points=[141,-50; 158,-50;
              158,160; -88,160; -88,108; -75,108; -75,94], style(color=74,
              rgbcolor={0,0,127}));
        connect(B31.BondCon2,Rgb. BondCon1) annotation (points=[90,110; 90,110],
            style(color=8, rgbcolor={192,192,192}));
        connect(Rgb.BondCon2,B32. fBondCon1) annotation (points=[138,110; 140,
              110; 140,100], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_6.BondCon3,B31. BondCon1) annotation (points=[60,120; 60,
              110; 70.2,110], style(color=8, rgbcolor={192,192,192}));
        connect(B33.BondCon2,J1p3_6. BondCon1) annotation (points=[50,130; 50,
              130], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_6.BondCon2,B34. BondCon1) annotation (points=[70,130; 70.2,
              130], style(color=8, rgbcolor={192,192,192}));
        connect(B34.BondCon2,J0p4_2. BondCon4) annotation (points=[90,130; 154,
              130; 154,30; 120,30; 120,10], style(color=8, rgbcolor={192,192,
                192}));
        connect(B17.BondCon1,J0p4_1. BondCon2) annotation (points=[-110,4.2;
              -110,0], style(color=8, rgbcolor={192,192,192}));
        connect(B33.BondCon1,J0p4_1. BondCon4) annotation (points=[30.2,130; 20,
              130; 20,164; -162,164; -162,20; -120,20; -120,10], style(color=8,
              rgbcolor={192,192,192}));
        connect(B25.eBondCon1,J0p5_1. BondCon4) annotation (points=[-20,-100;
              -20,-106; -42,-106; -42,-76; -60,-76], style(color=8, rgbcolor={
                192,192,192}));
        connect(B10.eBondCon1,J0p5_1. BondCon3) annotation (points=[90,-130; 90,
              -154; -16,-154; -16,-124; -46,-124; -46,-84; -60,-84], style(
              color=8, rgbcolor={192,192,192}));
        connect(B9.eBondCon1,J0p6_1. BondCon4) annotation (points=[130,50; 162,
              50; 162,-158; -20,-158; -20,-128; -50,-128; -50,-116; -60,-116],
            style(color=8, rgbcolor={192,192,192}));
        connect(B32.eBondCon1,J0p6_1. BondCon3) annotation (points=[140,80; 142,
              80; 142,66; 166,66; 166,-162; -24,-162; -24,-132; -54,-132; -54,
              -124; -60,-124], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_2.BondCon1, B5.BondCon2) annotation (points=[50,-60; 50,
              -60], style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_4.BondCon2, B5.BondCon1) annotation (points=[10,-120; 50,
              -120; 50,-79.8], style(color=8, rgbcolor={192,192,192}));
        connect(B4.BondCon1, J1p3_2.BondCon2) annotation (points=[50,-39.8; 50,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B4.BondCon2, J0p4_2.BondCon3) annotation (points=[50,-20; 50,
              -10; 120,-10], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon1, B1.BondCon1) annotation (points=[50,40; 50,
              39.8], style(color=8, rgbcolor={192,192,192}));
        connect(B1.BondCon2, J0p4_2.BondCon1) annotation (points=[50,20; 50,0;
              110,0], style(color=8, rgbcolor={192,192,192}));
        connect(B2.BondCon2, J1p3_1.BondCon2) annotation (points=[50,60; 50,60],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.BondCon1, J0p4_3.BondCon2) annotation (points=[50,79.8; 50,
              86; 16,86; 16,120; 10,120], style(color=8, rgbcolor={192,192,192}));
        connect(B11.BondCon1, J1p3_3.BondCon2) annotation (points=[20,10.2; 20,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(B11.BondCon2, J0p4_3.BondCon3) annotation (points=[20,30; 20,80;
              0,80; 0,110], style(color=8, rgbcolor={192,192,192}));
        connect(B12.BondCon2, J1p3_3.BondCon1) annotation (points=[20,-10; 20,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(B12.BondCon1, J0p4_4.BondCon4) annotation (points=[20,-29.8; 20,
              -100; 0,-100; 0,-110], style(color=8, rgbcolor={192,192,192}));
        connect(B21.BondCon2, J1p3_5.BondCon2) annotation (points=[-100,-30;
              -100,-30], style(color=8, rgbcolor={192,192,192}));
        connect(B21.BondCon1, J0p4_4.BondCon1) annotation (points=[-80.2,-30;
              -34,-30; -34,-120; -10,-120], style(color=8, rgbcolor={192,192,
                192}));
        connect(J0p4_1.BondCon3, B20.BondCon2) annotation (points=[-120,-10;
              -120,-10], style(color=8, rgbcolor={192,192,192}));
        connect(B20.BondCon1, J1p3_5.BondCon1) annotation (points=[-120,-29.8;
              -120,-30], style(color=8, rgbcolor={192,192,192}));
        connect(Gain1.y, ISD0.s) annotation (points=[-50,39; -50,30; -65,30;
              -65,12.6], style(color=74, rgbcolor={0,0,127}));
        end PMOSint;
      
        block nlCD "Non-linear capacitance computation of DIODE" 
          extends Modelica.Blocks.Interfaces.SISO;
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
          "Permittivity of free space";
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          constant Real kox=3.9 "Dielectric constant of SiO2";
          parameter Integer Level=1 
          "DIODE modeling level (check documentation window for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Real Area=1 "Relative device area";
          parameter Real PJ=1 "Relative perimeter width";
          parameter Integer M=1 "Diode multiplier";
          parameter Modelica.SIunits.Current IS=1e-14 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JS=1e-4 
          "Saturation current density at reference temperature";
          parameter Real N=1 "Current emission coefficient";
          parameter Modelica.SIunits.Time TT=0 
          "Transit time at reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in junction potential at reference temperature";
          parameter Modelica.SIunits.Voltage PHP=0.8 
          "Built-in perimeter potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJ=0.5e-12 
          "Zero-bias bulk capacitance at reference temperature";
          parameter Real CJ2(unit="F/m2") = 1e-4 
          "Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real MJ=0.33 "Bulk junction grading coefficient";
          parameter Modelica.SIunits.Capacitance CJSW=0.5e-12 
          "Zero-bias perimeter capacitance at reference temperature";
          parameter Real CJSW2(unit="F/m") = 1e-9 
          "Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Real MJSW=0.33 "Perimeter capacitance grading coefficient";
          parameter Real FC=0.5 
          "Forward-bias depletion capacitance coefficient";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
          "Leakage conductance";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real TTT1(unit="s/K") = 0 
          "Linear temperature coefficient of transit time";
          parameter Real TTT2(unit="s/K2") = 0 
          "Quadratic temperature coefficient of transit time";
          parameter Modelica.SIunits.Current ISW=0 
          "Saturation current of periphery at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JSW=0 
          "Saturation current density of periphery at reference temperature";
          parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
          parameter Modelica.SIunits.Length L=0 "Diode length";
          parameter Modelica.SIunits.Length W=0 "Diode width";
          parameter Modelica.SIunits.Length LP=0 "Polysilicon length";
          parameter Modelica.SIunits.Length WP=0 "Polysilicon width";
          parameter Modelica.SIunits.Length XP=0 
          "Widening of polysilicon due to masking and etching";
          parameter Modelica.SIunits.Length XOI=0 "Polysilicon thickness";
          parameter Modelica.SIunits.Length LM=0 "Metal length";
          parameter Modelica.SIunits.Length WM=0 "Metal width";
          parameter Modelica.SIunits.Length XM=0 
          "Widening of metal due to masking and etching";
          parameter Modelica.SIunits.Length XOM=0 "Metal thickness";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        
      protected 
          parameter Modelica.SIunits.Permittivity eox = kox*e0 
          "Permittivity of SiO2";
          parameter Modelica.SIunits.Area AREAeff = W*L*M;
          parameter Modelica.SIunits.Length PJeff = 2*(W + L)*M;
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Modelica.SIunits.Temperature DTemp 
          "Difference between circuit temperature and reference temperature";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
          Modelica.SIunits.Current ISeff "Effective saturation current";
          Modelica.SIunits.CurrentDensity JSval 
          "Saturation current density at device temperature";
          Modelica.SIunits.Current ISWval 
          "Saturation current of periphery at device temperature";
          Modelica.SIunits.CurrentDensity JSWval 
          "Saturation current density of periphery at device temperature";
          Modelica.SIunits.Voltage PBval 
          "Built-in junction potential at device temperature";
          Modelica.SIunits.Voltage PHPval 
          "Built-in perimeter potential at device temperature";
          Modelica.SIunits.Capacitance CJval 
          "Zero-bias bulk capacitance at device temperature";
          Real CJ2val(unit="F/m2") 
          "Zero-bias bulk capacitance per square meter at device temperature";
          Modelica.SIunits.Capacitance CJSWval 
          "Zero-bias perimeter capacitance at device temperature";
          Real CJSW2val(unit="F/m") 
          "Zero-bias perimeter capacitance per meter at device temperature";
          Modelica.SIunits.Time TTval "Transit time at device temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Energy EGnom "Energy gap at reference temperature";
          Modelica.SIunits.Voltage v "Voltage across capacitance";
          Modelica.SIunits.Capacitance Cval 
          "Non-linear junction capacitance value";
          Modelica.SIunits.Capacitance Cdiff 
          "Diffusion capacitance at device temperature";
          Modelica.SIunits.Capacitance Cbulk 
          "Bulk capacitance at device temperature";
          Real Cbulk2(unit="F/m2") 
          "Bulk capacitance per square meter at device temperature";
          Modelica.SIunits.Capacitance Cperi 
          "Perimeter capacitance at device temperature";
          Real Cperi2(unit="F/m") 
          "Perimeter capacitance per meter at device temperature";
          Modelica.SIunits.Capacitance Cpoly "Polysilicon capacitance";
          Modelica.SIunits.Capacitance Cmetal "Metal capacitance";
          Real et;
        annotation (Icon(
                    Text(
                extent=[-74,70; 80,-58],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-78,-42; 86,-66],
                style(color=0, rgbcolor={0,0,0}),
              string="DIODE")), Diagram(
                 Rectangle(extent=[-100,-100; 100,100],   style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255})),
                    Text(
                extent=[-74,70; 80,-58],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-78,-42; 86,-66],
                style(color=0, rgbcolor={0,0,0}),
              string="DIODE")));
      public 
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
                Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[-10,-110; 10,-90],  rotation=90);
        equation 
          v=u;
        
          /* Compute thermal voltage as function of temperature */
          Vt = k*Tdev/q;
          et = v/(N*Vt);
        
          /* Compute temperature variation on junction capacitance */
          DTemp = Tdev - Tnom;
          RTemp = Tdev/Tnom;
          EGval = EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
          EGnom = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          ISval = IS*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          JSval = JS*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          ISWval = ISW*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          JSWval = JSW*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          ISeff = if Level==1 then Area*ISval + PJ*ISWval else 
                  if Level==3 then AREAeff*JSval + PJeff*JSWval else 0;
          PBval = PB*RTemp + EGval - EGnom*RTemp - 3*Vt*ln(RTemp);
          PHPval = PHP*RTemp + EGval - EGnom*RTemp - 3*Vt*ln(RTemp);
          CJval = CJ*(1 + MJ*(1 - PBval/PB + 4e-4*DTemp));
          CJ2val = CJ2*(1 + MJ*(1 - PBval/PB + 4e-4*DTemp));
          CJSWval = CJSW*(1 + MJSW*(1 - PHPval/PHP + 4e-4*DTemp));
          CJSW2val = CJSW2*(1 + MJSW*(1 - PHPval/PHP + 4e-4*DTemp));
          TTval = TT*(1 + TTT1*DTemp + TTT2*DTemp*DTemp);
        
          if Level==1 then
            /* Non-geometric junction diode */
            Cdiff = TTval*(GminDC + Area*ISeff/(N*Vt)*(if et < EMin then ExMin*(et - EMin + 1) else 
                                                       if et > EMax then ExMax*(et - EMax + 1) else exp(et)));
            Cbulk = CJval*(if v > FC*PBval then (1 - FC*(1 + MJ) + MJ*v/PBval)/((1 - FC)^(1 + MJ)) else 
                                                1/noEvent((abs(1 - v/PBval))^MJ));
            Cbulk2 = 0;
            Cperi = CJSWval*(if v > FC*PHPval then (1 - FC*(1 + MJSW) + MJSW*v/PHPval)/((1 - FC)^(1 + MJSW)) else 
                                                   1/noEvent((abs(1 - v/PHPval))^MJSW));
            Cperi2 = 0;
            Cpoly = 0;
            Cmetal = 0;
            Cval = Cdiff + Area*Cbulk + PJ*Cperi;
          elseif Level==3 then
            /* Geometric junction diode */
            Cdiff = TTval*(GminDC + AREAeff*JSval/(N*Vt)*(if et < EMin then ExMin*(et - EMin + 1) else 
                                                          if et > EMax then ExMax*(et - EMax + 1) else exp(et)));
            Cbulk = 0;
            Cbulk2 = CJ2val*(if v > FC*PBval then (1 - FC*(1 + MJ) + MJ*v/PBval)/((1 - FC)^(1 + MJ)) else 
                                                  1/noEvent((abs(1 - v/PBval))^MJ));
            Cperi = 0;
            Cperi2 = CJSW2val*(if v > FC*PHPval then (1 - FC*(1 + MJSW) + MJSW*v/PHPval)/((1 - FC)^(1 + MJSW)) else 
                                                     1/noEvent((abs(1 - v/PHPval))^MJSW));
            Cpoly = eox*(WP + XP)*(LP + XP)*M/XOI;
            Cmetal = eox*(WM + XM)*(LM + XM)*M/XOM;
            Cval = Cdiff + AREAeff*Cbulk2 + PJeff*Cperi2 + Cpoly + Cmetal;
          else
            /* Fowler-Nordheim diode */
            Cdiff = 0;
            Cbulk = 0;
            Cbulk2 = 0;
            Cperi = 0;
            Cperi2 = 0;
            Cpoly = 0;
            Cmetal = 0;
            Cval = AREAeff*eox/TOX;
          end if;
        
          y = Cval;
        end nlCD;
      
        block ID0 "Injected current of DIODE" 
          extends Modelica.Blocks.Interfaces.SISO;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          parameter Integer Level=1 
          "DIODE modeling level (check documentation window for details)";
          parameter Real N=1 "Current emission coefficient";
          parameter Modelica.SIunits.Current IKF "Forward knee current";
          parameter Modelica.SIunits.Current IKR "Reverse knee current";
          output Modelica.SIunits.Voltage Vt "Thermal voltage";
          output Modelica.SIunits.Current IDstar "Diodic current";
          output Modelica.SIunits.Current ID "Adjusted diodic current";
          output Modelica.SIunits.Current IK "Knee current";
        annotation (Icon(
              Text(
                extent=[-50,36; 50,8],
                style(color=3, rgbcolor={0,0,255}),
                string="Injected current"),
              Text(
                extent=[-36,6; 36,-10],
                style(color=3, rgbcolor={0,0,255}),
              string="of a DIODE")), Diagram(
                 Rectangle(extent=[-100,-100; 100,100],   style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-50,36; 50,8],
                style(color=3, rgbcolor={0,0,255}),
                string="Injected current"),
              Text(
                extent=[-36,6; 36,-10],
                style(color=3, rgbcolor={0,0,255}),
              string="of a DIODE")));
          Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType 
            =   Modelica.SIunits.Temperature) "Device temperature" 
            annotation (extent=[-10,-110; 10,-90], rotation=90);
          Modelica.Blocks.Interfaces.RealInput v(redeclare type SignalType = 
                Modelica.SIunits.Voltage) "Diode voltage" 
            annotation (extent=[-10,90; 10,110],   rotation=270);
        equation 
          Vt = k*Tdev/q;
          IDstar = u;
          IK = if v > -5*N*Vt then IKF else 
               if v > -10*N*Vt then (IKF - IKR)*v/(5*N*Vt) + 2*IKF - IKR else IKR;
          ID = if Level==2 then IDstar else IDstar/(1 + sqrt(IDstar/IK));
          y = ID;
        end ID0;
      
        model CjD "Spice-style capacitor model for diodes" 
          extends BondLib.Interfaces.TwoPort;
          parameter Integer Level=1 
          "DIODE modeling level (check documentation window for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Current IS=1e-14 
          "Saturation current at reference temperature";
          parameter Real N=1 "Current emission coefficient";
          parameter Modelica.SIunits.Time TT=0 
          "Transit time at reference temperature";
          parameter Modelica.SIunits.Capacitance CJ=0.5e-12 
          "Zero-bias bulk capacitance at reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in junction potential at reference temperature";
          parameter Real MJ=0.33 "Bulk junction grading coefficient";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real FC=0.5 
          "Forward-bias depletion capacitance coefficient";
          parameter Modelica.SIunits.Voltage BV=0 
          "Reverse breakdown voltage at reference temperature";
          parameter Modelica.SIunits.Current IBV=0 "Reverse breakdown current";
          parameter Real TCV(unit="V/K")=0 
          "Temperature coefficient of breakdown voltage";
          parameter Real TTT1(unit="s/K") = 0 
          "Linear temperature coefficient of transit time";
          parameter Real TTT2(unit="s/K2") = 0 
          "Quadratic temperature coefficient of transit time";
          parameter Modelica.SIunits.Current ISW=0 
          "Saturation current of periphery at reference temperature";
          parameter Modelica.SIunits.Capacitance CJSW=0.5e-12 
          "Zero-bias perimeter capacitance at reference temperature";
          parameter Modelica.SIunits.Voltage PHP=0.8 
          "Built-in perimeter potential at reference temperature";
          parameter Real MJSW=0.33 "Perimeter capacitance grading coefficient";
          parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
          "Leakage conductance";
          parameter Real Area=1 "Relative device area";
          parameter Real PJ=1 "Relative perimeter width";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Length L=0 "Diode length";
          parameter Modelica.SIunits.Length W=0 "Diode width";
          parameter Integer M=1 "Diode multiplier";
          parameter Modelica.SIunits.CurrentDensity JS=1e-4 
          "Saturation current density at reference temperature";
          parameter Real CJ2(unit="F/m2") = 1e-4 
          "Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JSW=0 
          "Saturation current density of periphery at reference temperature";
          parameter Real CJSW2(unit="F/m") = 1e-9 
          "Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Modelica.SIunits.Length LP=0 "Polysilicon length";
          parameter Modelica.SIunits.Length WP=0 "Polysilicon width";
          parameter Modelica.SIunits.Length XP=0 
          "Widening of polysilicon due to masking and etching";
          parameter Modelica.SIunits.Length XOI=0 "Polysilicon thickness";
          parameter Modelica.SIunits.Length LM=0 "Metal length";
          parameter Modelica.SIunits.Length WM=0 "Metal width";
          parameter Modelica.SIunits.Length XM=0 
          "Widening of metal due to masking and etching";
          parameter Modelica.SIunits.Length XOM=0 "Metal thickness";
          parameter Real JF(unit="A/V2") = 0 "Forward FN current coefficient";
          parameter Real JR(unit="A/V2") = 0 "Reverse FN current coefficient";
          parameter Modelica.SIunits.ElectricFieldStrength EF=0 
          "Forward critical electrical field";
          parameter Modelica.SIunits.ElectricFieldStrength ER=0 
          "Reverse critical electrical field";
        Modelica.Blocks.Interfaces.RealOutput id(redeclare type SignalType = 
                Modelica.SIunits.Current) "Diode current" 
          annotation (extent=[-10,90; 10,110], rotation=90);
          BondLib.Junctions.J0p3 J0p3_1 
                                annotation (extent=[-90,10; -70,-10]);
          BondLib.Bonds.fBond B1 
                         annotation (extent=[-70,-10; -50,10]);
          BondLib.Sensors.De vbi 
                         annotation (extent=[-50,-10; -30,10]);
          BondLib.Sources.mSe Vbi 
                          annotation (extent=[0,-10; -20,10]);
          BondLib.Bonds.fBond B3 
                         annotation (extent=[0,-10; 20,10]);
          BondLib.Junctions.J1p3 J1p3_1 
                                annotation (extent=[20,-10; 40,10], rotation=
              -90);
          BondLib.Bonds.eBond B4 
                         annotation (extent=[20,10; 40,30],  rotation=90);
          BondLib.Sensors.Df id1 
                        annotation (extent=[20,30; 40,50], rotation=90);
          BondLib.Bonds.fBond B5 
                         annotation (extent=[20,-30; 40,-10], rotation=270);
          BondLib.Junctions.J0p3 J0p3_2 
                                annotation (extent=[20,-30; 40,-50]);
          BondLib.Bonds.fBond B6 
                         annotation (extent=[40,-50; 60,-30], rotation=0);
          BondLib.Bonds.fBond B7 
                         annotation (extent=[0,-50; 20,-30], rotation=180);
          BondLib.Bonds.fBond B8 
                         annotation (extent=[50,10; 70,30], rotation=90);
          BondLib.Sensors.De Tdev 
                         annotation (extent=[50,30; 70,50], rotation=90);
          BondLib.Junctions.J0p3 J0p3_3 
                                annotation (extent=[70,-10; 90,10]);
        BondLib.Spice.Utilities.Gj Gj1(
          GminDC=GminDC,
          Area=Area,
          Level=Level) 
               annotation (extent=[0,-50; -28,-30]);
          BondLib.Bonds.eBond B2 
                         annotation (extent=[-90,10; -70,30], rotation=90);
        BondLib.Semiconductors.Utilities.mC2 Cj1 
          annotation (extent=[-70,30; -50,50], rotation=0);
        BondLib.Spice.Utilities.nlCD nlC1(
          Level=Level,
          Tnom=Tnom,
          Area=Area,
          PJ=PJ,
          M=M,
          IS=IS,
          JS=JS,
          N=N,
          TT=TT,
          PB=PB,
          PHP=PHP,
          CJ=CJ,
          CJ2=CJ2,
          MJ=MJ,
          CJSW=CJSW,
          CJSW2=CJSW2,
          MJSW=MJSW,
          FC=FC,
          GminDC=GminDC,
          EG=EG,
          XTI=XTI,
          TTT1=TTT1,
          TTT2=TTT2,
          ISW=ISW,
          JSW=JSW,
          TOX=TOX,
          L=L,
          W=W,
          LP=LP,
          WP=WP,
          XP=XP,
          XOI=XOI,
          LM=LM,
          WM=WM,
          XM=XM,
          XOM=XOM,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-20,50; -40,30]);
        BondLib.Spice.Utilities.DjD DjD1(
          Tnom=Tnom,
          Level=Level,
          IS=IS,
          N=N,
          EG=EG,
          XTI=XTI,
          BV=BV,
          IBV=IBV,
          TCV=TCV,
          ISW=ISW,
          TOX=TOX,
          Area=Area,
          PJ=PJ,
          EMin=EMin,
          EMax=EMax,
          L=L,
          W=W,
          M=M,
          JS=JS,
          JSW=JSW,
          JF=JF,
          JR=JR,
          EF=EF,
          ER=ER) annotation (extent=[60,-52; 100,-28]);
          Bonds.eBond B9 annotation (extent=[90,-36; 110,-16],
                                                             rotation=90);
        equation 
        connect(vbi.BondCon1,B1. fBondCon1) annotation (points=[-50,0; -50,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(Vbi.BondCon1,B3. eBondCon1) annotation (points=[0,0; 0,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(id1.BondCon1,B4. eBondCon1) annotation (points=[30,30; 30,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon3,B3. fBondCon1) annotation (points=[20,
              -6.12303e-016; 20,0], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_1.BondCon1,B4. fBondCon1) annotation (points=[30,10; 30,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(B5.eBondCon1,J1p3_1. BondCon2) annotation (points=[30,-10; 30,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(B5.fBondCon1,J0p3_2. BondCon3) annotation (points=[30,-30; 30,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_2.BondCon2,B6. eBondCon1) annotation (points=[40,-40; 40,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B7.eBondCon1,J0p3_2. BondCon1) annotation (points=[20,-40; 20,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.BondCon1,B8. fBondCon1) annotation (points=[60,30; 60,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_3.BondCon1,B8. eBondCon1) annotation (points=[70,0; 60,0;
              60,10], style(color=8, rgbcolor={192,192,192}));
        connect(Gj1.BondCon1,B7. fBondCon1) annotation (points=[0,-40; 0,-40],
            style(color=8, rgbcolor={192,192,192}));
        annotation (
          Diagram,
          Icon(                  Text(extent=[-72,90; 68,50],   string="%name"),
              Text(
                extent=[-84,78; 26,-78],
                style(color=0),
              string="CjD"),
            Text(
              extent=[12,-2; 100,-30],
              string="Spice",
              style(color=0, rgbcolor={0,0,0}))),
          Documentation(info="<html>
The Spice-style junction capacitance for DIODEs computes the depletion and diffusion capacitance value of a junction used in a DIODE.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the capacitance formulae advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   DIODE modeling level (default value = 1)
            Level=1: Non-geometric junction diode
            Level=2: Fowler-Nordheim diode
            Level=3: Geometric junction diode</b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 IS:      Saturation current at reference temperature (default value = 1e-14 Amp)

 N:       Current mission coefficient (default value = 1)

 TT:      Transit time at reference temperature (default value = 0 sec)

 CJ:      Zero-bias bulk capacitance at reference temperature (default value = 0.5e-12 F)

 PB:      Built-in junction potential at reference temperature (default value = 0.8 Volt)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)

 XTI:     Temperature exponent of saturation current (default value = 3)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 BV:      Reverse breakdown voltage at reference temperature (default value = 0 Volt)

 IBV:     Reverse breakdown current (default value = 0 Amp)

 TCV:     Linear temperature coefficient of breakdown voltage (default value = 0 V/K)

 TTT1:    Linear temperature coefficient of transit time (default value = 0 s/K)
 
 TTT2:    Quadratic temperature coefficient of transit time (default value = 0 s/K2)

 ISW:     Saturation current of perimeter at reference temperature (default value = 0 Amp)

 CJSW:    Zero-bias perimeter capacitance at reference temperature (default value = 0.5e-12 F)

 PHP:     Built-in perimeter potential at reference temperature (default value = 0.8 Volt)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 GminDC:  Leakage conductance (default value = 1e-16 mho)

 Area:    Relative area occupied by the diode (default value = 1)

 PJ:      Relative perimeter width (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)

 L:       Diode length (default value = 0 m)

 W:       Diode width (default value = 0 m)

 M:       Diode multiplier (default value = 1)

 JS:      Saturation current density at reference temperature (default value = 1e-4 Amp/m<sup>2</sup>)

 CJ2:     Zero-bias bulk capacitance per meter square at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 JSW:     Saturation current density of perimeter at reference temperature (default value = 0 Amp/m<sup>2</sup>)

 CJSW2:   Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 LP:       Polysilicon length (default value = 0 m)

 WP:       Polysilicon width (default value = 0 m)

 XP:       Widening of polysilicon due to masking and etching (default value = 0 m)

 XOI:      Polysilicon thickness (default value = 0 m)

 LM:       Metal length (default value = 0 m)

 WM:       Metal width (default value = 0 m)

 XM:       Widening of metal due to masking and etching (default value = 0 m)

 XOM:      Metal thickness (default value = 0 m)

 JF:       Forward Fowler-Nordheim current coefficient (default value = 0 A/V2)

 JR:       Reverse Fowler-Nordheim current coefficient (default value = 0 A/V2)

 EF:       Forward Fowler-Nordheim critical electrical field (default value = 0 V/m)

 ER:       Reverse Fowler-Nordheim critical electrical field (default value = 0 V/m)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.200.
</ol>
</html>
"));
        connect(J0p3_3.BondCon2, BondCon2) annotation (points=[90,0; 100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon1, BondCon1) annotation (points=[-90,0; -100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B1.eBondCon1, J0p3_1.BondCon2) annotation (points=[-70,0; -70,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.fBondCon1, J0p3_1.BondCon3) annotation (points=[-80,10; -80,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(Cj1.BondCon1, B2.eBondCon1) annotation (points=[-70,40; -80,40;
              -80,30], style(color=8, rgbcolor={192,192,192}));
        connect(vbi.OutPort1, nlC1.u) annotation (points=[-29,0; -24,0; -24,20;
              -10,20; -10,40; -18,40], style(color=74, rgbcolor={0,0,127}));
        connect(Tdev.OutPort1, nlC1.Tdev) annotation (points=[60,51; 60,60; -30,
              60; -30,50], style(color=74, rgbcolor={0,0,127}));
        connect(DjD1.BondCon1, B6.fBondCon1) annotation (points=[60,-40; 60,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(id1.OutPort1, id) annotation (points=[30,51; 30,80; 0,80; 0,100],
            style(color=74, rgbcolor={0,0,127}));
        connect(B9.fBondCon1, DjD1.BondCon2) annotation (points=[100,-36; 100,
              -40], style(color=8, rgbcolor={192,192,192}));
        connect(B9.eBondCon1, J0p3_3.BondCon3) annotation (points=[100,-16; 100,
              -14; 80,-14; 80,-10], style(color=8, rgbcolor={192,192,192}));
        connect(Vbi.s, vbi.OutPort1) annotation (points=[-19,0; -29,0], style(
              color=74, rgbcolor={0,0,127}));
        connect(nlC1.y, Cj1.s) annotation (points=[-41,40; -51,40], style(color=
               74, rgbcolor={0,0,127}));
        end CjD;
      
        model DjD "Spice-style DIODE model" 
          extends Interfaces.SpiceTwoPort;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Integer Level=1 
          "DIODE modeling level (check documentation window for details)";
          parameter Modelica.SIunits.Current IS=1e-14 
          "Saturation current at reference temperature";
          parameter Real N=1 "Current emission coefficient";
          parameter Modelica.SIunits.Voltage EG=1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage BV=0 
          "Reverse breakdown voltage at reference temperature";
          parameter Modelica.SIunits.Current IBV=0 "Reverse breakdown current";
          parameter Real TCV(unit="V/K")=0 
          "Temperature coefficient of breakdown voltage";
          parameter Modelica.SIunits.Current ISW=0 
          "Saturation current of periphery at reference temperature";
          parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
          parameter Real Area=1 "Relative device area";
          parameter Real PJ=0 "Relative perimeter width";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Length L=0 "Diode length";
          parameter Modelica.SIunits.Length W=0 "Diode width";
          parameter Integer M=1 "Diode multiplier";
          parameter Modelica.SIunits.CurrentDensity JS=1e-4 
          "Saturation current density at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JSW=0 
          "Saturation current density of periphery at reference temperature";
          parameter Real JF(unit="A/V2") = 0 "Forward FN current coefficient";
          parameter Real JR(unit="A/V2") = 0 "Reverse FN current coefficient";
          parameter Modelica.SIunits.ElectricFieldStrength EF=0 
          "Forward critical electrical field";
          parameter Modelica.SIunits.ElectricFieldStrength ER=0 
          "Reverse critical electrical field";
        
      protected 
          parameter Modelica.SIunits.Voltage Vp = TOX*EF/(6*ln(10));
          parameter Modelica.SIunits.Voltage Vm = -TOX*ER/(6*ln(10));
          parameter Modelica.SIunits.Area AREAeff = W*L*M;
          parameter Modelica.SIunits.Length PJeff = 2*(W + L)*M;
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
          Modelica.SIunits.Current ISeff "Effective saturation current";
          Modelica.SIunits.CurrentDensity JSval 
          "Saturation current density at device temperature";
          Modelica.SIunits.Current ISWval 
          "Saturation current of periphery at device temperature";
          Modelica.SIunits.CurrentDensity JSWval 
          "Saturation current density of periphery at device temperature";
          Modelica.SIunits.Energy EGnom 
          "Activation energy at reference temperature";
          Modelica.SIunits.Voltage BVval 
          "Reverse breakdown voltage at device temperature";
          Modelica.SIunits.Voltage BVeff "Effective reverse breakdown voltage";
          Modelica.SIunits.Current IBVval "Effective reverse breakdown current";
          Modelica.SIunits.Current Ibreak;
          Modelica.SIunits.Current f1a;
          Modelica.SIunits.Current f1b;
          Real Eq(unit="V2/m2");
          Real et;
          Real et2;
        annotation (Diagram(
              Line(points=[-80,0; 80,0],   style(color=10)),
              Line(points=[0,80; 0,-80],   style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[4,82; 14,72],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="f"),
              Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[-66,-6; -36,-14],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="blocking"),
              Line(points=[0,0; 6,10; 12,28; 16,66],            style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[70,-4; 80,-14],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="e"),
              Text(
                extent=[16,50; 48,42],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="conducting"),
              Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                    fillColor=10))), Icon(
              Text(
                extent=[-86,80; 28,-82],
                style(color=0),
              string="DjD")),
          Documentation(info="<html>
This model computes the diodic current of a Spice-style DIODE model.  Spice offers three modeling levels for DIODEs that differ in the formulae they use to compute the diodic current [1].  The <b>Level=1</b> model and the <b>Level=3</b> model represent junction diodes, whereas the <b>Level=2</b> model represents a Fowler-Nordheim diode, essentially a non-linear resistor with a quadratic conductance characteristic. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   DIODE modeling level (default value = 1)
            Level=1: Non-geometric junction diode
            Level=2: Fowler-Nordheim diode
            Level=3: Geometric junction diode</b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 IS:      Saturation current at reference temperature (default value = 1e-14 Amp)

 N:       Current mission coefficient (default value = 1)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)

 XTI:     Temperature exponent of saturation current (default value = 3)

 BV:      Reverse breakdown voltage at reference temperature (default value = 0 Volt)

 IBV:     Reverse breakdown current (default value = 0 Amp)

 TCV:     Linear temperature coefficient of breakdown voltage (default value = 0 V/K)

 ISW:     Saturation current of perimeter at reference temperature (default value = 0 Amp)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 Area:    Relative area occupied by the diode (default value = 1)

 PJ:      Relative perimeter width (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)

 L:       Diode length (default value = 0 m)

 W:       Diode width (default value = 0 m)

 M:       Diode multiplier (default value = 1)

 JS:      Saturation current density at reference temperature (default value = 1e-4 Amp/m<sup>2</sup>)

 JSW:     Saturation current density of perimeter at reference temperature (default value = 0 Amp/m<sup>2</sup>)

 JF:       Forward Fowler-Nordheim current coefficient (default value = 0 A/V2)

 JR:       Reverse Fowler-Nordheim current coefficient (default value = 0 A/V2)

 EF:       Forward Fowler-Nordheim critical electrical field (default value = 0 V/m)

 ER:       Reverse Fowler-Nordheim critical electrical field (default value = 0 V/m)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.200.
</ol>
</html>
"));
        equation 
          /* Compute thermal voltage as function of temperature */
          Vt = k*e2/q;
        
          /* Compute temperature dependence of saturation current and breakdown voltage */
          RTemp = e2/Tnom;
          EGnom = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          ISval = IS*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          JSval = JS*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          ISWval = ISW*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          JSWval = JSW*exp((RTemp - 1)*EGnom/(N*Vt) + (XTI/N)*ln(RTemp));
          ISeff = if Level==1 then Area*ISval + PJ*ISWval else 
                  if Level==3 then AREAeff*JSval + PJeff*JSWval else 0;
          BVval = if BV > 0 then BV - TCV*DTemp else 0;
          Ibreak = -ISeff*(exp(-BV/(N*Vt)) - 1);
          IBVval = if IBV < Ibreak then Ibreak else IBV;
          BVeff = if BV > 0 then BVval - (N*Vt)*ln(IBVval/Ibreak) else 0;
        
          /* Compute diode characteristic */
          et = e1/(N*Vt);
          et2 = -(BVeff + e1)/(N*Vt);
          Eq = (e1/TOX)^2;
          if Level==1 or Level==3 then
            /* Junction diode */
            f1a = ISeff*(if et < EMin then ExMin*(et - EMin + 1) - 1 else 
                         if et > EMax then ExMax*(et - EMax + 1) - 1 else exp(et) - 1);
            f1b = ISeff*(if et2 < EMin then ExMin*(et2 - EMin + 1) - 1 else 
                         if et2 > EMax then ExMax*(et2 - EMax + 1) - 1 else exp(et2) - 1);
            f1 = if BV > 0 then f1a - f1b - IBVval/2 else f1a;
          else
            /* Fowler-Nordheim diode */
            f1a = 0;
            f1b = 0;
            f1 = if e1 > Vp then AREAeff*JF*Eq*exp(-TOX*EF/e1) else 
                 if e1 > 0 then AREAeff*JF*Eq*(1e-6/Vp)*e1 else 
                 if e1 > Vm then -AREAeff*JR*Eq*(1e-6/Vm)*e1 else 
                                 -AREAeff*JR*Eq*exp(TOX*ER/e1);
          end if;
        
          /* Compute heat flow */
          f2 = 0;
        end DjD;
      
        function JFETeq1 
        "Computation of injected drain-source current for Level = 1" 
          input Modelica.SIunits.Voltage vds "Drain-source voltage";
          input Modelica.SIunits.Voltage vgs "Gate-source voltage";
          input Modelica.SIunits.Temperature Tdev "Device temperature";
          input Real sig "sign(vds)";
          input Modelica.SIunits.Temperature Tnom "Reference temperature";
          input Modelica.SIunits.Voltage VT0 
          "Threshold voltage at reference temperature";
          input Modelica.SIunits.Transconductance BETA 
          "Transconductance parameter at reference temperature";
          input Modelica.SIunits.InversePotential LAMBDA 
          "Channel length modulation";
          input Real TCV(unit="V/K") 
          "Linear temperature coefficient of threshold voltage";
          input Real BEX "Mobility temperature exponent";
          output Modelica.SIunits.Current ids0 "Injected drain-source current";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
        
      protected 
          Modelica.SIunits.Temperature DTemp "Temperature difference";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Voltage VT0val 
          "Threshold voltage at device temperature";
          Modelica.SIunits.Transconductance BETAval 
          "Transconductance at device temperature";
          Modelica.SIunits.Transconductance Beta 
          "Adjusted transconductance at device temperature";
          Real cdlin(unit="V2") "Linear region current factor";
          Real cdsat(unit="V2") "Saturation region current factor";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vgst "vgs - VT0";
        annotation (
          Icon(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-48,8; 20,-20],
              style(color=58, rgbcolor={0,111,0}),
              string="of a JFET"),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-50,-18; 24,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 1")),
          Diagram(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-48,8; 20,-20],
              style(color=58, rgbcolor={0,111,0}),
              string="of a JFET"),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-50,-18; 24,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 1")),
          Documentation(info="<html>
This function implements a Spice-style <b>Level-1</b> model of the current flowing between drain and source of a JFET. <p>

The model comments the equations by providing the corresponding equation numbers from [1]. <p>

<hr> <p>

<pre>
Input variables:

 vds:     Drain-source voltage (Volt)

 vgs:     Gate-source voltage (Volt)

 Tdev:    Device temperature (K)

 sig:     Indicator of operating mode
            sig = +1:  direct mode
            sig = -1:  reverse mode (source and drain are flipped)


Input parameters:

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 LAMBDA:  Channel length modulation (default value = 0 1/Volt)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 BEX:     Mobility temperature exponent (default value = 0)


Output variables:

 isd0:    Drain-source current (Amp)

 vdsat:   Saturation voltage (Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        algorithm 
          /* Parameter computations and temperature variations (implements TMPUPD) */
          DTemp := Tdev - Tnom;
          RTemp := Tdev/Tnom;
          VT0val := VT0 - TCV*DTemp;  /* (3-53) p.155 */
          BETAval := BETA*RTemp^BEX;  /* (3-54) p.155 */
        
          /* Compute drain-source injection current */
          Vgst := vgs - VT0val;
          vdsat := if Vgst > 0 then Vgst else 0;
          Beta := BETAval*(1 + LAMBDA*vds);
          cdlin := vds*(2*Vgst - vds);  /* (3-22) p.143 */
          cdsat := Vgst*Vgst;  /* (3-22) p.143 */
          ids0 := sig*Beta*(if Vgst > 0 and vds <= vdsat then cdlin else 
                            if Vgst > 0 and vds > vdsat then cdsat else 0);
        end JFETeq1;
      
        function JFETeq2 
        "Computation of injected drain-source current for Level = 2" 
          input Modelica.SIunits.Voltage vds "Drain-source voltage";
          input Modelica.SIunits.Voltage vgs "Gate-source voltage";
          input Modelica.SIunits.Temperature Tdev "Device temperature";
          input Real sig "sign(vds)";
          input Modelica.SIunits.Temperature Tnom "Reference temperature";
          input Modelica.SIunits.Voltage VT0 
          "Threshold voltage at reference temperature";
          input Modelica.SIunits.Transconductance BETA 
          "Transconductance parameter at reference temperature";
          input Modelica.SIunits.InversePotential LAMBDA 
          "Channel length modulation";
          input Real TCV(unit="V/K") 
          "Linear temperature coefficient of threshold voltage";
          input Real BEX "Mobility temperature exponent";
          input Modelica.SIunits.InversePotential LAM1 
          "Channel length modulation gate voltage parameter";
          output Modelica.SIunits.Current ids0 "Injected drain-source current";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
        
      protected 
          Modelica.SIunits.Temperature DTemp "Temperature difference";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Voltage VT0val 
          "Threshold voltage at device temperature";
          Modelica.SIunits.Transconductance Beta 
          "Adjusted transconductance at device temperature";
          Real cdlin(unit="V2") "Linear region current factor";
          Real cdsat(unit="V2") "Saturation region current factor";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vgst "vgs - VT0";
        annotation (
          Icon(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-48,8; 20,-20],
              style(color=58, rgbcolor={0,111,0}),
              string="of a JFET"),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-50,-18; 24,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 2")),
          Diagram(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-48,8; 20,-20],
              style(color=58, rgbcolor={0,111,0}),
              string="of a JFET"),
              Text(
                extent=[-92,36; 72,-4],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-62,50; 38,22],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-50,-18; 24,-40],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 2")),
          Documentation(info="<html>
This function implements a Spice-style <b>Level-2</b> model of the current flowing between drain and source of a JFET. <p>

The model comments the equations by providing the corresponding equation numbers from [1]. <p>

<hr> <p>

<pre>
Input variables:

 vds:     Drain-source voltage (Volt)

 vgs:     Gate-source voltage (Volt)

 Tdev:    Device temperature (K)

 sig:     Indicator of operating mode
            sig = +1:  direct mode
            sig = -1:  reverse mode (source and drain are flipped)


Input parameters:

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 LAMBDA:  Channel length modulation (default value = 0 1/Volt)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 BEX:     Mobility temperature exponent (default value = 0)

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt)


Output variables:

 isd0:    Drain-source current (Amp)

 vdsat:   Saturation voltage (Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        algorithm 
          /* Parameter computations and temperature variations (implements TMPUPD) */
          DTemp := Tdev - Tnom;
          RTemp := Tdev/Tnom;
          VT0val := VT0 - TCV*DTemp;  /* (3-53) p.155 */
          Beta := BETA*RTemp^BEX;  /* (3-54) p.155 */
        
          /* Compute drain-source injection current */
          Vgst := vgs - VT0val;
          vdsat := if Vgst > 0 then Vgst else 0;
          cdlin := vds*(2*Vgst - vds);  /* (3-49) p.153 */
          cdsat := Vgst*Vgst*(if vgs >= 0 then 1 + LAMBDA*(vds - Vgst)*(1 + LAM1*vgs) else 
                                               1 - LAMBDA*(vds - Vgst)*Vgst/VT0val);  /* (3-49) p.153 */
          ids0 := sig*Beta*(if Vgst > 0 and vds <= vdsat then cdlin else 
                            if Vgst > 0 and vds > vdsat then cdsat else 0);
        end JFETeq2;
      
        function JFETeq3 
        "Computation of injected drain-source current of GaAs MESFET (Level = 3 JFET)" 
          input Modelica.SIunits.Voltage vds "Drain-source voltage";
          input Modelica.SIunits.Voltage vgs "Gate-source voltage";
          input Modelica.SIunits.Temperature Tdev "Device temperature";
          input Real sig "sign(vds)";
          input Modelica.SIunits.Temperature Tnom "Reference temperature";
          input Modelica.SIunits.Voltage VT0 
          "Threshold voltage at reference temperature";
          input Modelica.SIunits.Transconductance BETA 
          "Transconductance parameter at reference temperature";
          input Modelica.SIunits.InversePotential LAMBDA 
          "Channel length modulation";
          input Real TCV(unit="V/K") 
          "Linear temperature coefficient of threshold voltage";
          input Real BEX "Mobility temperature exponent";
          input Modelica.SIunits.InversePotential ALPHA 
          "Hyperbolic tangient fitting parameter";
          input Modelica.SIunits.InversePotential B "Measure of doping profile";
          output Modelica.SIunits.Current ids0 "Injected drain-source current";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
        
      protected 
          Modelica.SIunits.Temperature DTemp "Temperature difference";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Voltage VT0val 
          "Threshold voltage at device temperature";
          Modelica.SIunits.Transconductance BETAval 
          "Transconductance at device temperature";
          Modelica.SIunits.Transconductance Beta 
          "Adjusted transconductance at device temperature";
          Real cdlin(unit="V2") "Linear region current factor";
          Real cdsat(unit="V2") "Saturation region current factor";
        
          /* Auxiliary variables */
          Modelica.SIunits.Voltage Vgst "vgs - VT0";
        annotation (
          Icon(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-50,18; 18,-10],
              style(color=58, rgbcolor={0,111,0}),
              string="of a JFET"),
              Text(
                extent=[-96,48; 68,8],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-68,62; 32,34],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-52,-8; 22,-30],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 3"),
              Text(
                extent=[-70,-30; 40,-62],
              style(color=58, rgbcolor={0,111,0}),
              string="(GaAs MESFET)")),
          Diagram(
            Polygon(points=[-100,100; 20,100; 100,0; 20,-100; -100,-100; -100,
                  100], style(
                color=58,
                rgbcolor={0,111,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-50,18; 18,-10],
              style(color=58, rgbcolor={0,111,0}),
              string="of a JFET"),
              Text(
                extent=[-96,48; 68,8],
                string="between drain and source",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-68,62; 32,34],
                string="Injected current",
              style(color=58, rgbcolor={0,111,0})),
              Text(
                extent=[-52,-8; 22,-30],
              style(color=42, rgbcolor={220,0,0}),
              string="Level = 3"),
              Text(
                extent=[-70,-30; 40,-62],
              style(color=58, rgbcolor={0,111,0}),
              string="(GaAs MESFET)")),
          Documentation(info="<html>
This function implements a Spice-style <b>Level-3</b> model of the current flowing between drain and source of a JFET.  The <b>Level-3</b> model of the JFET describes one type of a GaAs MESFET device.  The theory behind the model is that proposed by Statz, <i>et al</i> [1]. <p>

The model comments the equations by providing the corresponding equation numbers from [2]. <p>

<hr> <p>

<pre>
Input variables:

 vds:     Drain-source voltage (Volt)

 vgs:     Gate-source voltage (Volt)

 Tdev:    Device temperature (K)

 sig:     Indicator of operating mode
            sig = +1:  direct mode
            sig = -1:  reverse mode (source and drain are flipped)


Input parameters:

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 LAMBDA:  Channel length modulation (default value = 0 1/Volt)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 BEX:     Mobility temperature exponent (default value = 0)

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1)

 B:       Measure of doping profile (default value = 0 1/Volt)


Output variables:

 isd0:    Drain-source current (Amp)

 vdsat:   Saturation voltage (Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        algorithm 
          /* Parameter computations and temperature variations (implements TMPUPD) */
          DTemp := Tdev - Tnom;
          RTemp := Tdev/Tnom;
          VT0val := VT0 - TCV*DTemp;  /* (3-53) p.155 */
          BETAval := BETA*RTemp^BEX;  /* (3-54) p.155 */
        
          /* Compute drain-source injection current */
          Vgst := vgs - VT0val;
          vdsat := if Vgst > 0 then Vgst else 0;
          Beta := BETAval*(1 + LAMBDA*vds);
          cdsat := Vgst*Vgst/(1 + B*Vgst);  /* (9-13) p.382 */
          cdlin := cdsat*(1 - (1 - ALPHA*vds/3)^3);  /* (9-13) p.382 */
          ids0 := sig*Beta*(if Vgst > 0 and vds <= 3/ALPHA then cdlin else 
                            if Vgst > 0 and vds > 3/ALPHA then cdsat else 0);
        end JFETeq3;
      
        block nlCJ "Non-linear capacitance computation of JFET and GaAs MESFET" 
          extends Modelica.Blocks.Interfaces.SI2SO;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Integer Level=1 
          "JFET modeling level (check documentation layer for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
          "Threshold voltage at reference temperature";
          parameter Real TCV(unit="V/K") = 0 
          "Linear temperature coefficient of threshold voltage";
          parameter Modelica.SIunits.Current IS=1.e-14 
          "Transport saturation current";
          parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real N=1 "Current emission coefficient";
          parameter Real XTI = if Level==3 then 2 else 3 
          "Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CGS=0.5e-12 
          "Zero-bias gate-source depletion capacitance at reference temperature";
          parameter Modelica.SIunits.Capacitance CGD=0.5e-12 
          "Zero-bias gate-drain depletion capacitance at reference temperature";
          parameter Real M = if Level==3 then 0.5 else 0.33 
          "Junction grading coefficient";
          parameter Real FC=0.5 
          "Factor for linearization of depletion capacitance formula";
          parameter Modelica.SIunits.Time TT=5e-9 "Ideal transit time";
          parameter Modelica.SIunits.InversePotential ALPHA=1 
          "Hyperbolic tangient fitting parameter";
          parameter Modelica.SIunits.Voltage DELTA=0.2 
          "Voltage range for transition";
          parameter Modelica.SIunits.Voltage VMAX=0.5 
          "Limit voltage for carrier velocity saturation";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
          "Leakage conductance";
          parameter Real Area=1 "Relative area occupied by capacitor";
          parameter Boolean GS=true "True if gate-source junction";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
          output Modelica.SIunits.Voltage vgs "Gate-source voltage";
          output Modelica.SIunits.Voltage vgd "Gate-drain voltage";
          output Modelica.SIunits.Voltage v "Diode voltage";
          output Modelica.SIunits.Capacitance Cval "Diode capacitance";
        
      protected 
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Modelica.SIunits.Temperature DTemp 
          "Difference between circuit temperature and reference temperature";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Voltage VT0val 
          "Threshold voltage at device temperature";
          Modelica.SIunits.Energy EGval "Energy gap at device temperature";
          Modelica.SIunits.Energy EGnom "Energy gap at reference temperature";
          Modelica.SIunits.Voltage PBval 
          "Built-in potential at device temperature";
          Modelica.SIunits.Capacitance CGSval 
          "Zero-bias gate-source depletion capacitance at device temperature";
          Modelica.SIunits.Capacitance CGDval 
          "Zero-bias gate-drain depletion capacitance at device temperature";
          Modelica.SIunits.Capacitance CJval 
          "Zero-bias depletion capacitance at device temperature";
          Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
          Modelica.SIunits.Capacitance Cdiff "Diffusion capacitance";
          Modelica.SIunits.Capacitance Cdepl "Depletion capacitance";
        
          /* Auxiliary variables */
          Real et;
          Modelica.SIunits.Voltage Vfac1;
          Modelica.SIunits.Voltage Vfac2;
          Modelica.SIunits.Voltage Ve;
          Modelica.SIunits.Voltage Vn;
          Modelica.SIunits.Voltage VNval;
          Real sig2;
          Real Cfac1;
          Real Cfac2;
          Real Cfac3;
      public 
        Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType = 
                Modelica.SIunits.Temperature) "Device temperature" 
          annotation (extent=[-10,-110; 10,-90],  rotation=90);
        annotation (Icon(
                    Text(
                extent=[-78,68; 76,-60],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-88,-42; 92,-74],
                style(color=0, rgbcolor={0,0,0}),
              string="JFET / GaAs MESFET")), Diagram(
                    Text(
                extent=[-78,68; 76,-60],
                string="nl C",
                style(color=0, rgbcolor={0,0,0})), Text(
                extent=[-88,-42; 92,-74],
                style(color=0, rgbcolor={0,0,0}),
              string="JFET / GaAs MESFET")));
        equation 
          vgs = u1;
          vgd = u2;
          v = if GS then vgs else vgd;
        
          /* Compute thermal voltage as function of temperature */
          Vt = k*Tdev/q;
          et = v/(N*Vt);
        
          /* Compute temperature dependence of saturation current and junction capacitance */
          DTemp = Tdev - Tnom;
          RTemp = Tdev/Tnom;
          VT0val =  VT0 - TCV*DTemp;  /* (3-53) p.155 */
          EGval = EG - GapC1*Tdev*Tdev/(Tdev + GapC2);
          EGnom = EG - GapC1*Tnom*Tnom/(Tnom + GapC2);
          PBval = PB*RTemp + EGval - EGnom*RTemp - 3*Vt*ln(RTemp);
          CGSval = CGS*(1 + M*(1 - PBval/PB + 4e-4*DTemp));
          CGDval = CGD*(1 + M*(1 - PBval/PB + 4e-4*DTemp));
          CJval = if GS then CGSval else CGDval;
          ISval = IS*exp((RTemp - 1)*EGval/Vt + XTI*ln(RTemp));
        
          /* Compute diffusion and depletion capacitances */
          if Level==1 or Level==2 then
            /* pn junction Si diode */
            Cdiff = Area*TT*(GminDC + ISval/(N*Vt)*(if et < EMin then ExMin*(et - EMin + 1) else 
                                                    if et > EMax then ExMax*(et - EMax + 1) else exp(et)));
            Vfac1 = 0;
            Vfac2 = 0;
            Ve = 0;
            Vn = 0;
            VNval = 0;
            sig2 = 0;
            Cfac1 = 0;
            Cfac2 = 0;
            Cfac3 = 0;
            Cdepl = Area*CJval*(if v > FC*PBval then (1 - FC*(1 + M) + M*v/PBval)/((1 - FC)^(1 + M)) else 
                                                     1/noEvent((abs(1 - v/PBval))^M));
            Cval = Cdiff + Cdepl;
          else
            /* Schottky barrier GaAs diode */
            Cdiff = 0;
            Vfac1 = sqrt((Ve - VT0val)^2 + DELTA^2);
            Vfac2 = sqrt((vgs - vgd)^2 + (1/ALPHA)^2);
            Ve = 0.5*(vgs + vgd + Vfac2);  /* (9-15) p.383 */
            Vn = 0.5*(Ve + VT0val + Vfac1);  /* (9-16) p.383 */
            VNval = if Vn <= VMAX then Vn else VMAX;  /* (9-17) p.383 */
            sig2 = if GS then 1 else -1;
            Cfac1 = 0.5*(1 + (Ve - VT0val)/Vfac1);
            Cfac2 = 0.5*(1 + sig2*(vgs - vgd)/Vfac2);
            Cfac3 = 0.5*(1 - sig2*(vgs - vgd)/Vfac2);
            Cdepl = Area*(CGSval*Cfac1*Cfac2/sqrt(1 - Vn/PBval) + CGDval*Cfac3);  /* (9-14) p.383 */
            Cval = Cdiff + Cdepl;
          end if;
        
          y = Cval;
        end nlCJ;
      
        block IDS1 "Injected current between drain and source of a JFET" 
          extends Modelica.Blocks.Interfaces.SI2SO;
          parameter Integer Level=1 
          "JFET modeling level (check documentation layer for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
          "Threshold voltage at reference temperature";
          parameter Modelica.SIunits.Transconductance BETA=0 
          "Transconductance parameter at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA=0 
          "Channel length modulation";
          parameter Real TCV(unit="V/K") = 0 
          "Linear temperature coefficient of threshold voltage";
          parameter Real BEX=0 "Mobility temperature exponent";
          parameter Modelica.SIunits.InversePotential LAM1=0 
          "Channel length modulation gate voltage parameter";
          parameter Modelica.SIunits.InversePotential ALPHA=1 
          "Hyperbolic tangient fitting parameter";
          parameter Modelica.SIunits.InversePotential B=0 
          "Measure of doping profile";
          output Modelica.SIunits.Voltage vds "Drain-source voltage";
          output Modelica.SIunits.Voltage vgs "Gate-source voltage";
          output Modelica.SIunits.Current ids "Injected drain-source current";
          output Modelica.SIunits.Voltage vdsat "Saturation voltage";
        
      protected 
          Real sig "sign(vds)";
        annotation (
          Icon(
              Text(
                extent=[-52,42; 48,14],
                style(color=3, rgbcolor={0,0,255}),
                string="Injected current"),
              Text(
                extent=[-82,26; 82,-14],
                style(color=3, rgbcolor={0,0,255}),
                string="between drain and source"),
              Text(
                extent=[-68,-2; 62,-32],
                style(color=3, rgbcolor={0,0,255}),
              string="of a JFET / MESFET"),
              Text(
                extent=[-96,66; -76,54],
                style(color=3, rgbcolor={0,0,255}),
              string="vds"),
              Text(
                extent=[-96,-54; -76,-66],
                style(color=3, rgbcolor={0,0,255}),
              string="vgs"),
              Text(
                extent=[-10,-76; 10,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="Tdev")),
          Diagram(
                 Rectangle(extent=[-100,-100; 100,100],   style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-52,42; 48,14],
                style(color=3, rgbcolor={0,0,255}),
                string="Injected current"),
              Text(
                extent=[-96,66; -76,54],
                style(color=3, rgbcolor={0,0,255}),
              string="vds"),
              Text(
                extent=[-82,26; 82,-14],
                style(color=3, rgbcolor={0,0,255}),
                string="between drain and source"),
              Text(
                extent=[-68,-2; 62,-32],
                style(color=3, rgbcolor={0,0,255}),
              string="of a JFET / MESFET"),
              Text(
                extent=[-96,-54; -76,-66],
                style(color=3, rgbcolor={0,0,255}),
              string="vgs"),
              Text(
                extent=[-10,-76; 10,-88],
                style(color=3, rgbcolor={0,0,255}),
                string="Tdev")),
          Documentation(info="<html>
The <b>IDS1</b> block computes the drain-source current of JFETs and GaAs MESFETs.  Depending on the modeling level, a different algorithm is being used.  The equations for each of these algorithms are programmed in functions, called <b>JFETeq1</b> .. <b>JFETeq3</b> that implement the particular model in question.  These functions have been carefully documented to make it easier on future developers to add additional models of their own by providing corresponding <b>JFETeq_i</b> functions. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1-3</font>

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt) <font color=red>Levels 1-3</font>

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 1-3</font>

 LAMBDA:  Channel length modulation (default value = 0 1/Volt) <font color=red>Levels 1-3</font>

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K) <font color=red>Levels 1-3</font>

 BEX:     Mobility temperature exponent (default value = 0) <font color=red>Levels 1-3</font>

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt) <font color=red>Level 2</font>

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1) <font color=red>Level 3</font>

 B:       Measure of doping profile (default value = 0 1/Volt) <font color=red>Level 3</font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
      public 
          Modelica.Blocks.Interfaces.RealInput Tdev(redeclare type SignalType 
            =   Modelica.SIunits.Temperature) "Device temperature" 
            annotation (extent=[-10,-110; 10,-90], rotation=90);
        equation 
          /* Determine first whether vds > 0 */
          if noEvent( u1 > 0) then
            /* Keep drain and source as they are */
            sig = 1;
            vds = u1;
            vgs = u2;
          else
            /* Switch drain and source */
            sig = -1;
            vds = -u1;
            vgs = u2 - u1;
          end if;
        
          /* Implement the different levels of drain-source current injection models */
          if Level==0 or Level==1 then
            (ids,vdsat) = JFETeq1(vds=vds,vgs=vgs,Tdev=Tdev,sig=sig,Tnom=Tnom,
                                  VT0=VT0,BETA=BETA,LAMBDA=LAMBDA,TCV=TCV,BEX=BEX);
          elseif Level==2 then
            (ids,vdsat) = JFETeq2(vds=vds,vgs=vgs,Tdev=Tdev,sig=sig,Tnom=Tnom,
                                  VT0=VT0,BETA=BETA,LAMBDA=LAMBDA,TCV=TCV,BEX=BEX,
                                  LAM1=LAM1);
          elseif Level==3 then
            (ids,vdsat) = JFETeq3(vds=vds,vgs=vgs,Tdev=Tdev,sig=sig,Tnom=Tnom,
                                  VT0=VT0,BETA=BETA,LAMBDA=LAMBDA,TCV=TCV,BEX=BEX,
                                  ALPHA=ALPHA,B=B);
          end if;
        
          /* Assign outputs */
          y = ids;
        end IDS1;
      
        model CjJ "Spice-style junction capacitor model for JFETs and MESFETs" 
          extends BondLib.Interfaces.TwoPort;
          parameter Integer Level=1 
          "JFET modeling level (check documentation layer for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
          "Threshold voltage at reference temperature";
          parameter Modelica.SIunits.Current IS=1.e-14 
          "Transport saturation current at reference temperature";
          parameter Real TCV(unit="V/K") = 0 
          "Linear temperature coefficient of threshold voltage";
          parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real N=1 "Current emission coefficient";
          parameter Real XTI = if Level==3 then 2 else 3 
          "Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CGS=0.5e-12 
          "Zero-bias gate-source depletion capacitance at reference temperature";
          parameter Modelica.SIunits.Capacitance CGD=0.5e-12 
          "Zero-bias gate-drain depletion capacitance at reference temperature";
          parameter Real M = if Level==3 then 0.5 else 0.33 
          "Junction grading coefficient";
          parameter Real FC=0.5 
          "Factor for linearization of depletion capacitance formula";
          parameter Modelica.SIunits.Time TT=5e-9 "Ideal transit time";
          parameter Modelica.SIunits.InversePotential ALPHA=0 
          "Hyperbolic tangient fitting parameter";
          parameter Modelica.SIunits.Voltage DELTA=0.2 
          "Voltage range for transition";
          parameter Modelica.SIunits.Voltage VMAX=0.5 
          "Limit voltage for carrier velocity saturation";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
          "Leakage conductance";
          parameter Real Area=1 "Relative area occupied by capacitor";
          parameter Boolean GS=true "True if gate-source junction";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (
          Icon(
              Text(
                extent=[-84,72; 30,-74],
                style(color=0),
              string="CjJ"),
            Text(
              extent=[4,-4; 112,-32],
              string="Spice",
              style(color=0, rgbcolor={0,0,0})),
            Text(
              extent=[-52,96; 54,66],
              style(color=3, rgbcolor={0,0,255}),
              string="%name")),
          Diagram(
            Text(
              extent=[28,58; 38,50],
              string="vds",
              style(color=10, rgbcolor={95,95,95})),
            Text(
              extent=[28,78; 38,70],
              style(color=10, rgbcolor={95,95,95}),
              string="vgs"),
            Text(
              extent=[-26,86; -12,78],
              style(color=10, rgbcolor={95,95,95}),
              string="Tdev")),
          Documentation(info="<html>
The Spice-style junction capacitance for JFETs and GaAs MESFETs computes the depletion and diffusion capacitance values of a junction used in a JFET / MESFET.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [2]
            Level=3: Statz-Curtice (GaAs) MESFET model [3,4]<b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)

 IS:      Transport saturation current (default value = 1e-14 Amp)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 EG:      Energy gap for temperature effect on saturation current at 0 K
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F)

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F)

 M:       Junction grading coefficient
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 TT:      Ideal transit time (default value = 5e-9 sec)

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1)

 DELTA:   Voltage range for transition (default value = 0.2 Volt)

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 GS:      True if gate-source junction (default value = true)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
          BondLib.Bonds.eBond B1 
                         annotation (extent=[-80,10; -60,30], rotation=90);
          BondLib.Semiconductors.Utilities.mC2 Cj1 
                  annotation (extent=[-80,30; -60,50], rotation=90);
        BondLib.Junctions.J0p4 J0p4_1 
                              annotation (extent=[60,-10; 80,10]);
          BondLib.Bonds.fBond B7 
                         annotation (extent=[60,10; 80,30], rotation=90);
          BondLib.Sensors.De Tdev 
                         annotation (extent=[60,30; 80,50], rotation=90);
          BondLib.Bonds.eBond B3 
                         annotation (extent=[20,-10; 40,10],  rotation=0);
          BondLib.Bonds.fBond B2 
                         annotation (extent=[-40,-10; -20,10]);
          BondLib.Bonds.fBond B5 
                         annotation (extent=[-40,-50; -20,-30]);
          BondLib.Bonds.eBond B6 
                         annotation (extent=[20,-50; 40,-30], rotation=0);
        BondLib.Spice.RS Rd(
          Tnom=Tnom,
          R=1/GminDC,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-20,-52; 20,-28]);
        BondLib.Spice.Utilities.nlCJ Cval(
          Level=Level,
          Tnom=Tnom,
          VT0=VT0,
          TCV=TCV,
          IS=IS,
          EG=EG,
          N=N,
          XTI=XTI,
          PB=PB,
          CGS=CGS,
          CGD=CGD,
          M=M,
          FC=FC,
          TT=TT,
          ALPHA=ALPHA,
          DELTA=DELTA,
          VMAX=VMAX,
          GminDC=GminDC,
          Area=Area,
          GS=GS,
          EMin=EMin,
          EMax=EMax) annotation (extent=[16,76; -34,44]);
        BondLib.Junctions.J0p4 J0p4_2 annotation (extent=[-80,-10; -60,10]);
        BondLib.Spice.Utilities.DjJ DjJ1(
          Tnom=Tnom,
          Level=Level,
          IS=IS,
          EG=EG,
          N=N,
          XTI=XTI,
          Area=Area,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-20,-12; 20,12]);
        Modelica.Blocks.Interfaces.RealInput vds(redeclare type SignalType = 
                Modelica.SIunits.Voltage) "Drain-source voltage" 
          annotation (extent=[-50,-110; -30,-90], rotation=90);
        Modelica.Blocks.Interfaces.RealInput vgs(redeclare type SignalType = 
                Modelica.SIunits.Voltage) "Gate-source voltage" 
          annotation (extent=[30,-110; 50,-90], rotation=90);
        equation 
        connect(Cj1.BondCon1,B1. eBondCon1) annotation (points=[-70,30; -70,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(B7.eBondCon1,J0p4_1. BondCon4) annotation (points=[70,10; 70,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.BondCon1,B7. fBondCon1) annotation (points=[70,30; 70,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(B6.eBondCon1,J0p4_1. BondCon3) annotation (points=[40,-40; 70,
              -40; 70,-10], style(color=8, rgbcolor={192,192,192}));
        connect(Rd.BondCon1,B5. fBondCon1) annotation (points=[-20,-40; -20,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(Rd.BondCon2,B6. fBondCon1) annotation (points=[20,-40; 20,-40],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_1.BondCon2, BondCon2) annotation (points=[80,0; 100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_2.BondCon4, B1.fBondCon1) annotation (points=[-70,10; -70,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_2.BondCon1, BondCon1) annotation (points=[-80,0; -100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_2.BondCon2, B2.eBondCon1) annotation (points=[-60,0; -40,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.fBondCon1, DjJ1.BondCon1) annotation (points=[-20,0; -20,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B3.fBondCon1, DjJ1.BondCon2) annotation (points=[20,0; 20,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B3.eBondCon1, J0p4_1.BondCon1) annotation (points=[40,0; 60,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_2.BondCon3, B5.eBondCon1) annotation (points=[-70,-10; -70,
              -40; -40,-40], style(color=8, rgbcolor={192,192,192}));
        connect(Tdev.OutPort1, Cval.Tdev) annotation (points=[70,51; 70,90; -9,
              90; -9,76], style(color=74, rgbcolor={0,0,127}));
        connect(vds, Cval.u1) annotation (points=[-40,-100; -40,-60; 48,-60; 48,
              50.4; 21,50.4], style(color=74, rgbcolor={0,0,127}));
        connect(vgs, Cval.u2) annotation (points=[40,-100; 40,-62; 50,-62; 50,
              69.6; 21,69.6], style(color=74, rgbcolor={0,0,127}));
        connect(Cval.y, Cj1.s) annotation (points=[-36.5,60; -70,60; -70,49],
            style(color=74, rgbcolor={0,0,127}));
        end CjJ;
      
        model DjJ "Spice-style junction diode model for JFETs" 
          extends Interfaces.SpiceTwoPort;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
          "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
          "Electron charge";
          constant Real GapC1=7.02E-4 "First bandgap correction factor Silicon";
          constant Real GapC2=1108.0 "Second bandgap correction factor Silicon";
          parameter Integer Level=1 
          "JFET modeling level (check documentation layer for details)";
          parameter Modelica.SIunits.Current IS=1e-14 
          "Saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real N=1 "Current emission coefficient";
          parameter Real XTI = if Level==3 then 2 else 3 
          "Saturation current temperature exponent";
          parameter Real Area=1 "Relative area occupied by diode";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        
      protected 
          parameter Real ExMin = exp(EMin);
          parameter Real ExMax = exp(EMax);
          Modelica.SIunits.Voltage Vt "Thermal voltage";
          Real RTemp "Temperature quotient";
          Modelica.SIunits.Energy EGval 
          "Activation energy at device temperature";
          Modelica.SIunits.Current ISval 
          "Saturation current at device temperature";
          Real et;
        annotation (
          Icon(
              Text(
                extent=[-84,76; 26,-82],
                style(color=0),
              string="DjJ")),
          Diagram(
              Line(points=[-80,0; 80,0],   style(color=10)),
              Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                    fillColor=10)),
              Line(points=[0,80; 0,-80],   style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
                  color=10,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[4,82; 14,72],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="f"),
              Text(
                extent=[70,-4; 80,-14],
                style(
                  color=10,
                  fillColor=10,
                  fillPattern=1),
                string="e"),
              Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1)),
              Text(
                extent=[-66,-6; -36,-14],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="blocking"),
              Text(
                extent=[16,50; 48,42],
                style(
                  color=10,
                  pattern=3,
                  fillColor=10,
                  fillPattern=1),
                string="conducting"),
              Line(points=[0,0; 6,10; 12,28; 16,66],            style(
                  color=0,
                  thickness=2,
                  fillColor=10,
                  fillPattern=1))),
          Documentation(info="<html>
The Spice-style JFET junction diode is a regular exponential diode. <p>

<hr>

<pre>
Parameters:

 <font color=red><b>Level:  JFET modeling level (default value = 1)
           Level=1: Shichman-Hodges (Si) JFET model
           Level=2: Improved Shichman-Hodges (Si) JFET model [1]
           Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 IS:     Transport saturation current (default value = 1e-14 Amp)

 EG:     Energy gap for temperature effect on saturation current at 0 K
           <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
           default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:      Current emission coefficient (default value = 1)

 XTI:    Saturation current temperature exponent
           <font color=blue><b>default value = 3 for Levels=1,2 (Si)
           default value = 2 for Level=3 (GaAs)</b></font>

 Area:   Relative area occupied by the diode (default value = 1)

 EMin:   if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:   if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
        equation 
          /* Compute thermal voltage as function of temperature */
          Vt = k*e2/q;
          et = e1/(N*Vt);
        
          /* Compute temperature dependence of saturation current */
          RTemp = e2/Tnom;
          EGval = EG - GapC1*e2*e2/(e2 + GapC2);
          ISval = IS*exp((RTemp - 1)*EGval/Vt + XTI*ln(RTemp));
        
          /* Compute diode characteristic */
          f1 = ISval*Area*(if noEvent(et < EMin) then ExMin*(et - EMin + 1) - 1 else 
                           if noEvent(et > EMax) then ExMax*(et - EMax + 1) - 1 else exp(et) - 1);
        
          /* Compute heat flow */
          e2*f2 = e1*f1;
        end DjJ;
      
        model NFETint 
        "Spice-style internal n-channel JFET in bond graph technology" 
          parameter Integer Level=1 
          "JFET modeling level (check documentation layer for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
          "Threshold voltage at reference temperature";
          parameter Modelica.SIunits.Current IS=1.e-14 
          "Transport saturation current at reference temperature";
          parameter Modelica.SIunits.Transconductance BETA=0 
          "Transconductance parameter at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA=0 
          "Channel length modulation";
          parameter Real TCV(unit="V/K") = 0 
          "Linear temperature coefficient of threshold voltage";
          parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real N=1 "Current emission coefficient";
          parameter Real XTI = if Level==3 then 2 else 3 
          "Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CGS=0.5e-12 
          "Zero-bias gate-source depletion capacitance at reference temperature";
          parameter Modelica.SIunits.Capacitance CGD=0.5e-12 
          "Zero-bias gate-drain depletion capacitance at reference temperature";
          parameter Real M = if Level==3 then 0.5 else 0.33 
          "Junction grading coefficient";
          parameter Real FC=0.5 
          "Factor for linearization of depletion capacitance formula";
          parameter Modelica.SIunits.Time TT=5e-9 "Ideal transit time";
          parameter Real BEX=0 "Mobility temperature exponent";
          parameter Modelica.SIunits.InversePotential LAM1=0 
          "Channel length modulation gate voltage parameter";
          parameter Modelica.SIunits.InversePotential ALPHA=1 
          "Hyperbolic tangient fitting parameter";
          parameter Modelica.SIunits.InversePotential B=0 
          "Measure of doping profile";
          parameter Modelica.SIunits.Voltage DELTA=0.2 
          "Voltage range for transition";
          parameter Modelica.SIunits.Voltage VMAX=0.5 
          "Limit voltage for carrier velocity saturation";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
          "Leakage conductance";
          parameter Real Area=1 "Relative area occupied by capacitor";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        annotation (
          Coordsys(extent=[-140,-140; 140,140]),
          Diagram(
            Text(
              extent=[-150,20; -130,6],
              string="Gint",
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1)),
            Text(
              extent=[10,146; 30,132],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Dint"),
            Text(
              extent=[10,-134; 30,-148],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Sint"),
            Text(
              extent=[128,22; 152,4],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Heat")),
          Documentation(info="<html>
The Spice-style internal n-channel JFET model contains the non-linear current source and the two junction diodes (including the junctions capacitances). <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 IS:      Transport saturation current (default value = 1e-14 Amp)

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 LAMBDA:  Channel length modulation (default value = 0 1/Volt)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 EG:      Energy gap for temperature effect on saturation current at 0 K
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F)

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F)

 M:       Junction grading coefficient
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 TT:      Ideal transit time (default value = 5e-9 sec)

 BEX:     Mobility temperature exponent (default value = 0)

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt)

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1)

 B:       Measure of doping profile (default value = 0 1/Volt)

 DELTA:   Voltage range for transition (default value = 0.2 Volt)

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"),       Icon(
              Text(
                extent=[90,-2; 130,-44],
                style(color=0),
              string="int"),
            Text(
              extent=[-78,128; 82,88],
              style(color=3, rgbcolor={0,0,255}),
              string="%name"),
              Line(points=[0,-120; 20,-100],
                                           style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[-20,-100; 0,-120],
                                            style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[0,80; 0,-120],style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
            Text(
              extent=[12,146; 34,132],
              style(color=10, rgbcolor={135,135,135}),
              string="Dint"),
            Text(
              extent=[-152,22; -130,8],
              style(color=10, rgbcolor={135,135,135}),
              string="Gint"),
            Text(
              extent=[10,-134; 32,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Sint"),
            Text(
              extent=[128,24; 152,6],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Heat"),
            Line(points=[-100,-60; 100,-60; 80,-50], style(color=3, rgbcolor={0,
                    0,255})),
            Line(points=[100,-60; 80,-70], style(color=3, rgbcolor={0,0,255})),
              Text(
                extent=[-122,92; 108,-96],
                style(color=0),
              string="NFET")));
        BondLib.Interfaces.BondCon Gint "Internal gate" 
          annotation (extent=[-150,-10; -130,10]);
        BondLib.Interfaces.BondCon H "Heat" 
          annotation (extent=[130,-10; 150,10]);
        BondLib.Interfaces.BondCon Dint "Internal drain" 
          annotation (extent=[-10,130; 10,150]);
        BondLib.Interfaces.BondCon Sint "Internal source" 
          annotation (extent=[-10,-150; 10,-130]);
        BondLib.Junctions.J0p3 J0p3_1 
          annotation (extent=[-120,-10; -100,10], rotation=-90);
        BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[110,10; 130,-10]);
        BondLib.Junctions.J1p3 J1p3_1 
          annotation (extent=[-10,-10; 10,10], rotation=90);
        BondLib.Bonds.Bond B1 annotation (extent=[-10,10; 10,30], rotation=-90);
        BondLib.Bonds.Bond B2 
          annotation (extent=[-10,-30; 10,-10], rotation=-90);
        BondLib.Junctions.J0p3 J0p3_2 
          annotation (extent=[-10,100; 10,120], rotation=-90);
        BondLib.Junctions.J0p3 J0p3_3 
          annotation (extent=[-10,-120; 10,-100], rotation=-90);
        BondLib.Bonds.Bond B3 annotation (extent=[10,-10; 30,10], rotation=0);
        BondLib.Junctions.J0p3 J0p3_4 
          annotation (extent=[30,-10; 50,10], rotation=0);
        BondLib.Bonds.eBond B4 annotation (extent=[50,-10; 70,10], rotation=180);
        BondLib.Sources.mSf_int Ids annotation (extent=[90,-10; 70,10]);
        BondLib.Bonds.fBond B5 
          annotation (extent=[90,-10; 110,10], rotation=180);
        BondLib.Bonds.fBond B6 
          annotation (extent=[110,-30; 130,-10], rotation=270);
        BondLib.Sensors.De Tdev 
          annotation (extent=[110,-50; 130,-30], rotation=-90);
        BondLib.Bonds.fBond B7 
          annotation (extent=[30,-30; 50,-10], rotation=270);
        BondLib.Sensors.De vds 
          annotation (extent=[30,-50; 50,-30], rotation=-90);
        BondLib.Spice.Utilities.IDS1 IDS1(
          Level=Level,
          Tnom=Tnom,
          VT0=VT0,
          BETA=BETA,
          LAMBDA=LAMBDA,
          TCV=TCV,
          BEX=BEX,
          LAM1=LAM1,
          ALPHA=ALPHA,
          B=B) annotation (extent=[64,-116; 96,-84]);
        BondLib.Bonds.Bond B8 
          annotation (extent=[-100,100; -80,120], rotation=0);
        BondLib.Junctions.J1p3 J1p3_2 
          annotation (extent=[-80,100; -60,120], rotation=0);
        BondLib.Bonds.Bond B9 annotation (extent=[-60,100; -40,120], rotation=0);
        BondLib.Bonds.Bond B10 
          annotation (extent=[-80,80; -60,100], rotation=-90);
        BondLib.Spice.Utilities.CjJ Dgd(
          Level=Level,
          Tnom=Tnom,
          VT0=VT0,
          IS=IS,
          TCV=TCV,
          EG=EG,
          N=N,
          XTI=XTI,
          PB=PB,
          CGS=CGS,
          CGD=CGD,
          M=M,
          FC=FC,
          TT=TT,
          ALPHA=ALPHA,
          DELTA=DELTA,
          VMAX=VMAX,
          GminDC=GminDC,
          Area=Area,
          GS=false,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-56,40; -84,80], rotation=-90);
        BondLib.Bonds.eBond B11 annotation (extent=[-60,20; -40,40], rotation=0);
        BondLib.Bonds.Bond B12 
          annotation (extent=[-100,-120; -80,-100], rotation=0);
        BondLib.Junctions.J1p3 J1p3_3 
          annotation (extent=[-80,-100; -60,-120], rotation=0);
        BondLib.Bonds.Bond B13 
          annotation (extent=[-60,-120; -40,-100], rotation=0);
        BondLib.Bonds.Bond B14 
          annotation (extent=[-80,-100; -60,-80], rotation=90);
        BondLib.Junctions.J0p3 J0p3_5 
          annotation (extent=[-60,-80; -80,-60], rotation=-90);
        BondLib.Bonds.Bond B15 
          annotation (extent=[-80,-60; -60,-40], rotation=90);
        BondLib.Spice.Utilities.CjJ Dgs(
          Level=Level,
          Tnom=Tnom,
          VT0=VT0,
          IS=IS,
          TCV=TCV,
          EG=EG,
          N=N,
          XTI=XTI,
          PB=PB,
          CGS=CGS,
          CGD=CGD,
          M=M,
          FC=FC,
          TT=TT,
          ALPHA=ALPHA,
          DELTA=DELTA,
          VMAX=VMAX,
          GminDC=GminDC,
          Area=Area,
          EMin=EMin,
          EMax=EMax,
          GS=true) annotation (extent=[-84,-40; -56,0], rotation=90);
        BondLib.Bonds.eBond B16 annotation (extent=[-60,0; -40,20], rotation=0);
        BondLib.Bonds.fBond B17 
          annotation (extent=[-60,-80; -40,-60], rotation=0);
        BondLib.Sensors.De vgs 
          annotation (extent=[-40,-80; -20,-60], rotation=0);
        Modelica.Blocks.Math.Gain Gain1(k=-1) 
          annotation (extent=[70,-42; 90,-22], rotation=90);
        equation 
        connect(J0p3_1.BondCon3, Gint)     annotation (points=[-120,
              -6.12303e-016; -128,-6.12303e-016; -128,0; -140,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p5_1.BondCon2, H)        annotation (points=[130,0; 140,0],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B1.BondCon2, J1p3_1.BondCon2) annotation (points=[6.12303e-016,
              10; 6.12303e-016,10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-6.12303e-016,
              -10; -6.00057e-016,-10; -6.00057e-016,-10.2], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_2.BondCon1, Dint)     annotation (points=[-6.12303e-016,
              120; 0,120; 0,140], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_2.BondCon2, B1.BondCon1) annotation (points=[6.12303e-016,
              100; -6.00057e-016,100; -6.00057e-016,29.8], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_3.BondCon2, Sint)     annotation (points=[6.12303e-016,
              -120; 0,-120; 0,-140], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_3.BondCon1, B2.BondCon2) annotation (points=[-6.12303e-016,
              -100; 6.12303e-016,-100; 6.12303e-016,-30], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[10,
              -6.12303e-016; 15,-6.12303e-016; 15,0; 10.2,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B3.BondCon2, J0p3_4.BondCon1) annotation (points=[30,0; 30,0],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B4.eBondCon1, J0p3_4.BondCon2) annotation (points=[50,
              1.22461e-015; 57,1.22461e-015; 57,0; 50,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Ids.BondCon2, B4.fBondCon1) annotation (points=[70,0; 75,0; 75,
              -1.22461e-015; 70,-1.22461e-015], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B5.eBondCon1, J0p5_1.BondCon1) annotation (points=[110,
              -1.22461e-015; 110,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B5.fBondCon1, Ids.BondCon1) annotation (points=[90,1.22461e-015;
              90,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p5_1.BondCon5, B6.eBondCon1) annotation (points=[120,-10; 120,
              -10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B6.fBondCon1, Tdev.BondCon1) annotation (points=[120,-30; 120,
              -30], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_4.BondCon3, B7.eBondCon1) annotation (points=[40,-10; 40,
              -10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B7.fBondCon1, vds.BondCon1) annotation (points=[40,-30; 40,-30],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Tdev.OutPort1, IDS1.Tdev) annotation (points=[120,-51; 120,-130;
              80,-130; 80,-116], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vds.OutPort1, IDS1.u1) annotation (points=[40,-51; 40,-90.4;
              60.8,-90.4], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_1.BondCon1, B8.BondCon1) annotation (points=[-110,10; -110,
              110; -99.8,110], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B8.BondCon2, J1p3_2.BondCon1) annotation (points=[-80,110; -80,
              110], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_2.BondCon2, B9.BondCon1) annotation (points=[-60,110;
              -59.8,110], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B9.BondCon2, J0p3_2.BondCon3) annotation (points=[-40,110; -10,
              110], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_2.BondCon3, B10.BondCon1) annotation (points=[-70,100; -70,
              99.8], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B10.BondCon2, Dgd.BondCon1) annotation (points=[-70,80; -70,80],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dgd.BondCon2, B11.fBondCon1) annotation (points=[-70,40; -70,30;
              -60,30], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B12.BondCon1, J0p3_1.BondCon2) annotation (points=[-99.8,-110;
              -110,-110; -110,-10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B12.BondCon2, J1p3_3.BondCon1) annotation (points=[-80,-110;
              -80,-110], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_3.BondCon2, B13.BondCon1) annotation (points=[-60,-110;
              -59.8,-110], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B13.BondCon2, J0p3_3.BondCon3) annotation (points=[-40,-110;
              -10,-110], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B14.BondCon1, J1p3_3.BondCon3) annotation (points=[-70,-99.8;
              -70,-100], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_5.BondCon2, B14.BondCon2) annotation (points=[-70,-80; -70,
              -80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B15.BondCon1, J0p3_5.BondCon1) annotation (points=[-70,-59.8;
              -70,-60], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dgs.BondCon1, B15.BondCon2) annotation (points=[-70,-40; -70,
              -40], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dgs.BondCon2, B16.fBondCon1) annotation (points=[-70,0; -70,10;
              -60,10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_5.BondCon3, B17.eBondCon1) annotation (points=[-60,-70;
              -60,-70], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B17.fBondCon1, vgs.BondCon1) annotation (points=[-40,-70; -40,
              -70], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(IDS1.y, Gain1.u) annotation (points=[97.6,-100; 110,-100; 110,
              -60; 80,-60; 80,-44], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vgs.OutPort1, IDS1.u2) annotation (points=[-19,-70; 20,-70; 20,
              -109.6; 60.8,-109.6], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vds.OutPort1, Dgs.vds) annotation (points=[40,-51; 40,-60; -10,
              -60; -10,-28; -56,-28], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vgs.OutPort1, Dgs.vgs) annotation (points=[-19,-70; -14,-70;
              -14,-12; -56,-12], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dgs.vgs, Dgd.vgs) annotation (points=[-56,-12; -14,-12; -14,52;
              -56,52], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dgs.vds, Dgd.vds) annotation (points=[-56,-28; -10,-28; -10,68;
              -56,68], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B11.eBondCon1, J0p5_1.BondCon4) annotation (points=[-40,30; -30,
              30; -30,44; 124,44; 124,10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B16.eBondCon1, J0p5_1.BondCon3) annotation (points=[-40,10; -26,
              10; -26,40; 116,40; 116,10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Gain1.y, Ids.s) annotation (points=[80,-21; 80,-9], style(color=
               74, rgbcolor={0,0,127}));
        end NFETint;
      
        model PFETint 
        "Spice-style internal p-channel JFET in bond graph technology" 
          parameter Integer Level=1 
          "JFET modeling level (check documentation layer for details)";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
          "Reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
          "Threshold voltage at reference temperature";
          parameter Modelica.SIunits.Current IS=1.e-14 
          "Transport saturation current at reference temperature";
          parameter Modelica.SIunits.Transconductance BETA=0 
          "Transconductance parameter at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA=0 
          "Channel length modulation";
          parameter Real TCV(unit="V/K") = 0 
          "Linear temperature coefficient of threshold voltage";
          parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
          "Energy gap for temperature effect on saturation current at 0 K";
          parameter Real N=1 "Current emission coefficient";
          parameter Real XTI = if Level==3 then 2 else 3 
          "Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage PB=0.8 
          "Built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CGS=0.5e-12 
          "Zero-bias gate-source depletion capacitance at reference temperature";
          parameter Modelica.SIunits.Capacitance CGD=0.5e-12 
          "Zero-bias gate-drain depletion capacitance at reference temperature";
          parameter Real M = if Level==3 then 0.5 else 0.33 
          "Junction grading coefficient";
          parameter Real FC=0.5 
          "Factor for linearization of depletion capacitance formula";
          parameter Modelica.SIunits.Time TT=5e-9 "Ideal transit time";
          parameter Real BEX=0 "Mobility temperature exponent";
          parameter Modelica.SIunits.InversePotential LAM1=0 
          "Channel length modulation gate voltage parameter";
          parameter Modelica.SIunits.InversePotential ALPHA=1 
          "Hyperbolic tangient fitting parameter";
          parameter Modelica.SIunits.InversePotential B=0 
          "Measure of doping profile";
          parameter Modelica.SIunits.Voltage DELTA=0.2 
          "Voltage range for transition";
          parameter Modelica.SIunits.Voltage VMAX=0.5 
          "Limit voltage for carrier velocity saturation";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
          "Leakage conductance";
          parameter Real Area=1 "Relative area occupied by capacitor";
          parameter Real EMin=-100 
          "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
          "if x > EMax, the exp(x) function is linearized";
        BondLib.Interfaces.BondCon Gint "Internal gate" 
          annotation (extent=[-150,-10; -130,10]);
        BondLib.Interfaces.BondCon H "Heat" 
          annotation (extent=[130,-10; 150,10]);
        BondLib.Interfaces.BondCon Dint "Internal drain" 
          annotation (extent=[-10,130; 10,150]);
        BondLib.Interfaces.BondCon Sint "Internal source" 
          annotation (extent=[-10,-150; 10,-130]);
        BondLib.Junctions.J0p3 J0p3_1 
          annotation (extent=[-120,-10; -100,10], rotation=-90);
        BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[110,10; 130,-10]);
        BondLib.Junctions.J1p3 J1p3_1 
          annotation (extent=[-10,-10; 10,10], rotation=90);
        BondLib.Bonds.Bond B1 annotation (extent=[-10,10; 10,30], rotation=90);
        BondLib.Bonds.Bond B2 annotation (extent=[-10,-30; 10,-10], rotation=90);
        BondLib.Junctions.J0p3 J0p3_2 
          annotation (extent=[-10,100; 10,120], rotation=-90);
        BondLib.Junctions.J0p3 J0p3_3 
          annotation (extent=[-10,-120; 10,-100], rotation=-90);
        BondLib.Bonds.Bond B3 annotation (extent=[10,-10; 30,10], rotation=0);
        BondLib.Junctions.J0p3 J0p3_4 
          annotation (extent=[30,-10; 50,10], rotation=0);
        BondLib.Bonds.eBond B4 annotation (extent=[50,-10; 70,10], rotation=180);
        BondLib.Sources.mSf_int Isd annotation (extent=[90,-10; 70,10]);
        BondLib.Bonds.fBond B5 
          annotation (extent=[90,-10; 110,10], rotation=180);
        BondLib.Bonds.fBond B6 
          annotation (extent=[110,-30; 130,-10], rotation=270);
        BondLib.Sensors.De Tdev 
          annotation (extent=[110,-50; 130,-30], rotation=-90);
        BondLib.Bonds.fBond B7 
          annotation (extent=[30,-30; 50,-10], rotation=270);
        BondLib.Sensors.De vsd 
          annotation (extent=[30,-50; 50,-30], rotation=-90);
        BondLib.Spice.Utilities.IDS1 ISD(
          Level=Level,
          Tnom=Tnom,
          VT0=VT0,
          BETA=BETA,
          LAMBDA=LAMBDA,
          TCV=TCV,
          BEX=BEX,
          LAM1=LAM1,
          ALPHA=ALPHA,
          B=B) annotation (extent=[64,-116; 96,-84]);
        BondLib.Bonds.Bond B8 
          annotation (extent=[-100,100; -80,120], rotation=180);
        BondLib.Junctions.J1p3 J1p3_2 
          annotation (extent=[-80,100; -60,120], rotation=0);
        BondLib.Bonds.Bond B9 
          annotation (extent=[-60,100; -40,120], rotation=180);
        BondLib.Bonds.Bond B10 
          annotation (extent=[-80,80; -60,100], rotation=-90);
        BondLib.Spice.Utilities.CjJ Ddg(
          Level=Level,
          Tnom=Tnom,
          VT0=VT0,
          IS=IS,
          TCV=TCV,
          EG=EG,
          N=N,
          XTI=XTI,
          PB=PB,
          CGS=CGS,
          CGD=CGD,
          M=M,
          FC=FC,
          TT=TT,
          ALPHA=ALPHA,
          DELTA=DELTA,
          VMAX=VMAX,
          GminDC=GminDC,
          Area=Area,
          GS=false,
          EMin=EMin,
          EMax=EMax) annotation (extent=[-56,40; -84,80], rotation=-90);
        BondLib.Bonds.eBond B11 annotation (extent=[-60,20; -40,40], rotation=0);
        BondLib.Bonds.Bond B12 
          annotation (extent=[-100,-120; -80,-100], rotation=180);
        BondLib.Junctions.J1p3 J1p3_3 
          annotation (extent=[-80,-100; -60,-120], rotation=0);
        BondLib.Bonds.Bond B13 
          annotation (extent=[-60,-120; -40,-100], rotation=180);
        BondLib.Bonds.Bond B14 
          annotation (extent=[-80,-100; -60,-80], rotation=90);
        BondLib.Junctions.J0p3 J0p3_5 
          annotation (extent=[-60,-80; -80,-60], rotation=-90);
        BondLib.Bonds.Bond B15 
          annotation (extent=[-80,-60; -60,-40], rotation=90);
        BondLib.Spice.Utilities.CjJ Dsg(
          Level=Level,
          Tnom=Tnom,
          VT0=VT0,
          IS=IS,
          TCV=TCV,
          EG=EG,
          N=N,
          XTI=XTI,
          PB=PB,
          CGS=CGS,
          CGD=CGD,
          M=M,
          FC=FC,
          TT=TT,
          ALPHA=ALPHA,
          DELTA=DELTA,
          VMAX=VMAX,
          GminDC=GminDC,
          Area=Area,
          EMin=EMin,
          EMax=EMax,
          GS=true) annotation (extent=[-84,-40; -56,0], rotation=90);
        BondLib.Bonds.eBond B16 annotation (extent=[-60,0; -40,20], rotation=0);
        BondLib.Bonds.fBond B17 
          annotation (extent=[-60,-80; -40,-60], rotation=0);
        BondLib.Sensors.De vsg 
          annotation (extent=[-40,-80; -20,-60], rotation=0);
        Modelica.Blocks.Math.Gain Gain1(k=-1) 
          annotation (extent=[70,-42; 90,-22], rotation=90);
        annotation (
          Diagram(
            Text(
              extent=[-150,20; -130,6],
              string="Gint",
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1)),
            Text(
              extent=[10,146; 30,132],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Dint"),
            Text(
              extent=[10,-134; 30,-148],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Sint"),
            Text(
              extent=[128,22; 152,4],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Heat")),
          Coordsys(extent=[-140,-140; 140,140]),
          Icon(
              Text(
                extent=[90,-2; 130,-44],
                style(color=0),
              string="int"),
            Text(
              extent=[-78,128; 82,88],
              style(color=3, rgbcolor={0,0,255}),
              string="%name"),
              Line(points=[-20,60; 0,80],  style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[0,80; 20,60],    style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
              Line(points=[0,80; 0,-120],style(
                  color=9,
                  rgbcolor={149,164,171},
                  thickness=2,
                  fillColor=74,
                  rgbfillColor={0,0,185},
                  fillPattern=1)),
            Line(points=[-100,-60; 100,-60; 80,-50], style(color=3, rgbcolor={0,
                    0,255})),
            Line(points=[100,-60; 80,-70], style(color=3, rgbcolor={0,0,255})),
            Text(
              extent=[128,24; 152,6],
              style(
                color=10,
                rgbcolor={135,135,135},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Heat"),
            Text(
              extent=[12,146; 34,132],
              style(color=10, rgbcolor={135,135,135}),
              string="Dint"),
            Text(
              extent=[-152,22; -130,8],
              style(color=10, rgbcolor={135,135,135}),
              string="Gint"),
            Text(
              extent=[10,-134; 32,-148],
              style(color=10, rgbcolor={135,135,135}),
              string="Sint"),
              Text(
                extent=[-122,92; 108,-96],
                style(color=0),
              string="PFET")),
          Documentation(info="<html>
The Spice-style internal p-channel JFET model contains the non-linear current source and the two junction diodes (including the junctions capacitances). <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 IS:      Transport saturation current (default value = 1e-14 Amp)

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 LAMBDA:  Channel length modulation (default value = 0 1/Volt)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 EG:      Energy gap for temperature effect on saturation current at 0 K
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F)

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F)

 M:       Junction grading coefficient
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 TT:      Ideal transit time (default value = 5e-9 sec)

 BEX:     Mobility temperature exponent (default value = 0)

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt)

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1)

 B:       Measure of doping profile (default value = 0 1/Volt)

 DELTA:   Voltage range for transition (default value = 0.2 Volt)

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
        
        equation 
        connect(J0p3_1.BondCon3, Gint)     annotation (points=[-120,
              -6.12303e-016; -130,-6.12303e-016; -130,0; -140,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p5_1.BondCon2, H)        annotation (points=[130,0; 140,0],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_2.BondCon1, Dint)     annotation (points=[-6.12303e-016,
              120; -6.12303e-016,130; 0,130; 0,140], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_3.BondCon2, Sint)     annotation (points=[6.12303e-016,
              -120; 6.12303e-016,-130; 0,-130; 0,-140], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[10,
              -6.12303e-016; 10.1,-6.12303e-016; 10.1,0; 10.2,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B3.BondCon2, J0p3_4.BondCon1) annotation (points=[30,0; 30,0],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B4.eBondCon1, J0p3_4.BondCon2) annotation (points=[50,
              1.22461e-015; 50,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Isd.BondCon2, B4.fBondCon1) annotation (points=[70,0; 70,
              -1.22461e-015], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B5.eBondCon1, J0p5_1.BondCon1) annotation (points=[110,
              -1.22461e-015; 110,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B5.fBondCon1, Isd.BondCon1) annotation (points=[90,1.22461e-015;
              90,0], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p5_1.BondCon5, B6.eBondCon1) annotation (points=[120,-10; 120,
              -10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B6.fBondCon1, Tdev.BondCon1) annotation (points=[120,-30; 120,
              -30], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_4.BondCon3, B7.eBondCon1) annotation (points=[40,-10; 40,
              -10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B7.fBondCon1, vsd.BondCon1) annotation (points=[40,-30; 40,-30],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Tdev.OutPort1, ISD.Tdev) annotation (points=[120,-51; 120,-130;
              80,-130; 80,-116], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vsd.OutPort1, ISD.u1) annotation (points=[40,-51; 40,-90.4;
              60.8,-90.4], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_2.BondCon3, B10.BondCon1) annotation (points=[-70,100; -70,
              99.8], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B10.BondCon2, Ddg.BondCon1) annotation (points=[-70,80; -70,80],
            style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Ddg.BondCon2, B11.fBondCon1) annotation (points=[-70,40; -70,30;
              -60,30], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B14.BondCon1, J1p3_3.BondCon3) annotation (points=[-70,-99.8;
              -70,-100], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_5.BondCon2, B14.BondCon2) annotation (points=[-70,-80; -70,
              -80], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B15.BondCon1, J0p3_5.BondCon1) annotation (points=[-70,-59.8;
              -70,-60], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dsg.BondCon1, B15.BondCon2) annotation (points=[-70,-40; -70,
              -40], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dsg.BondCon2, B16.fBondCon1) annotation (points=[-70,0; -70,10;
              -60,10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J0p3_5.BondCon3, B17.eBondCon1) annotation (points=[-60,-70;
              -60,-70], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B17.fBondCon1, vsg.BondCon1) annotation (points=[-40,-70; -40,
              -70], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(ISD.y, Gain1.u) annotation (points=[97.6,-100; 110,-100; 110,
              -60; 80,-60; 80,-44], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vsg.OutPort1, ISD.u2) annotation (points=[-19,-70; 20,-70; 20,
              -109.6; 60.8,-109.6], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vsd.OutPort1, Dsg.vds) annotation (points=[40,-51; 40,-60; -10,
              -60; -10,-28; -56,-28], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(vsg.OutPort1, Dsg.vgs) annotation (points=[-19,-70; -14,-70;
              -14,-12; -56,-12], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dsg.vgs, Ddg.vgs) annotation (points=[-56,-12; -14,-12; -14,52;
              -56,52], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(Dsg.vds, Ddg.vds) annotation (points=[-56,-28; -10,-28; -10,68;
              -56,68], style(
            color=74,
            rgbcolor={0,0,127},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B11.eBondCon1, J0p5_1.BondCon4) annotation (points=[-40,30; -30,
              30; -30,44; 124,44; 124,10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(B16.eBondCon1, J0p5_1.BondCon3) annotation (points=[-40,10; -26,
              10; -26,40; 116,40; 116,10], style(
            color=8,
            rgbcolor={192,192,192},
            fillColor=3,
            rgbfillColor={0,0,255},
            fillPattern=1));
        connect(J1p3_2.BondCon2, B9.BondCon2) annotation (points=[-60,110; -60,
              110], style(color=8, rgbcolor={192,192,192}));
        connect(B9.BondCon1, J0p3_2.BondCon3) annotation (points=[-40.2,110;
              -10,110], style(color=8, rgbcolor={192,192,192}));
        connect(B8.BondCon1, J1p3_2.BondCon1) annotation (points=[-80.2,110;
              -80,110], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon1, B8.BondCon2) annotation (points=[-110,10; -110,
              110; -100,110], style(color=8, rgbcolor={192,192,192}));
        connect(J1p3_3.BondCon2, B13.BondCon2) annotation (points=[-60,-110;
              -60,-110], style(color=8, rgbcolor={192,192,192}));
        connect(B13.BondCon1, J0p3_3.BondCon3) annotation (points=[-40.2,-110;
              -10,-110], style(color=8, rgbcolor={192,192,192}));
        connect(B12.BondCon1, J1p3_3.BondCon1) annotation (points=[-80.2,-110;
              -80,-110], style(color=8, rgbcolor={192,192,192}));
        connect(J0p3_1.BondCon2, B12.BondCon2) annotation (points=[-110,-10;
              -110,-110; -100,-110], style(color=8, rgbcolor={192,192,192}));
        connect(B1.BondCon1, J1p3_1.BondCon2) annotation (points=[-6.00057e-016,
              10.2; -6.00057e-016,19.1; 6.12303e-016,19.1; 6.12303e-016,10],
            style(color=8, rgbcolor={192,192,192}));
        connect(B1.BondCon2, J0p3_2.BondCon2) annotation (points=[6.12303e-016,
              30; 6.12303e-016,100; 6.12303e-016,100], style(color=8, rgbcolor=
                {192,192,192}));
        connect(B2.BondCon2, J1p3_1.BondCon1) annotation (points=[6.12303e-016,
              -10; -6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
        connect(B2.BondCon1, J0p3_3.BondCon1) annotation (points=[-6.00057e-016,
              -29.8; -6.00057e-016,-64.9; -6.12303e-016,-64.9; -6.12303e-016,
              -100], style(color=8, rgbcolor={192,192,192}));
        connect(Gain1.y, Isd.s) annotation (points=[80,-21; 80,-9], style(color=
               74, rgbcolor={0,0,127}));
        end PFETint;
      end Utilities;
    end Spice;
  
  package Switches "Switches of Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30, fillPattern=
                1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
              fillColor=30, fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
              fillColor=30, fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=3)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0.45,
        y=0.01,
        width=0.44,
        height=0.65,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package contains a collecion of switching elements for the Modelica bond graph library. <p>

Switches change their computational causality as a function of the switch position.  For this reason, switches must always be attached to a-causal bonds. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E., M. Otter, and H. Elmqvist (1995), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/VarStruc.pdf\">Bond
     Graph Modeling of Variable Structure Systems</a>,\" <i>Proc. ICBGM'95, 2<sup>nd</sup> SCS Intl. Conf. on Bond Graph
     Modeling and Simulation</i>, Las Vegas, Nevada, pp. 49-55.
<li> Krebs, M. (1997), <i><a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/krebs_ms.pdf\">Modeling of Conditional
     Index Changes</a></i>, MS Thesis, Dept. of Electr. & Comp. Engr., University of Arizona, Tucson, AZ. 
</ol>
</html>
"));
    model Sw "Generic switch element of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      Boolean open "Switch position";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-86,0; 74,0],   style(color=10)),
          Polygon(points=[64,4; 74,0; 64,-4; 64,4],     style(color=10,
                fillColor=10)),
          Line(points=[-6,80; -6,-80], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-10,70; -6,80; -2,70; -10,70], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[0,80; 10,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Text(
            extent=[64,-4; 74,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-82,0; -26,0; 16,0; 60,0],           style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Ellipse(extent=[-8,2; -4,-2], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[-6,-44; 16,-52],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="closed"),
          Text(
            extent=[20,10; 42,2],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="open"),
          Text(
            extent=[-6,0; 14,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s=0"),
          Line(points=[-6,-74; -6,-18; -6,16; -6,62],       style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[-6,38; 16,30],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="closed"),
          Text(
            extent=[-60,10; -38,2],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="open")),
        Icon(Text(
            extent=[-60, 80; 60, -80],
            string="Sw",
            style(color=0)), Text(extent=[-70, 80; 70, 40], string="%name")),
        Documentation(info="<html>
The generic switch element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The switch has an additional modulating Boolean input signal modeling the switch position.  If the modulating input signal has a value of <b>true</b>, the switch is considered <b>open</b>. <p>

The causality of the switch element must be free, i.e., the switch element must be placed inside an algebraic loop. <p>

<hr>

<pre>
Potential variables:

 e:     Bondgraphic effort variable

 f:     Bondgraphic flow variable, normalized positive for flows into the model

 open:  Boolean variable denoting the switch position (modulation input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>0 = if open then f else e</b>
</html>
"));
      Modelica.Blocks.Interfaces.BooleanInput BooleanInPort1 
        "Boolean input signal (true if switch is open)"       annotation (
          extent=[100, -10; 80, 10]);
    equation 
      open = BooleanInPort1;
      0 = if open then f else e;
    end Sw;
    
    model Sw2 "Leaking switch of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      parameter Real Ron(final min=0) = 1.E-5 "Closed switch resistance";
      parameter Real Goff(final min=0) = 1.E-5 "Opened switch conductance";
    protected 
      Real s "Curve parameter";
      Boolean OpenSwitch;
    public 
      Modelica.Blocks.Interfaces.BooleanInput BooleanInPort1 
        "Boolean input signal (true if switch is open)"       annotation (
          extent=[100, -10; 80, 10]);
      annotation (
        Diagram(
          Line(points=[-86,0; 74,0],   style(color=10)),
          Polygon(points=[64,4; 74,0; 64,-4; 64,4],     style(color=10,
                fillColor=10)),
          Line(points=[-6,80; -6,-80], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-10,70; -6,80; -2,70; -10,70], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[0,80; 10,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Text(
            extent=[64,-4; 74,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-80,-38; -26,-10; 14,10; 64,36],     style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[16,68; 36,58],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Ron"),
          Text(
            extent=[-52,-6; -32,-16],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Goff"),
          Ellipse(extent=[-8,2; -4,-2], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[-28,-50; -6,-58],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="closed"),
          Text(
            extent=[28,18; 50,10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="open"),
          Text(
            extent=[-6,0; 14,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s=0"),
          Line(points=[-36,-74; -12,-16; 0,16; 20,76],      style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[6,38; 28,30],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="closed"),
          Text(
            extent=[-54,-22; -32,-30],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="open")),
        Icon(
          Text(
            extent=[-74, 78; 32, -82],
            string="Sw",
            style(color=0)),
          Text(
            extent=[24, -14; 86, -48],
            string="Leak",
            style(color=0, pattern=0)),
          Text(extent=[-70, 100; 70, 60], string="%name")),
        Documentation(info="<html>
The leaking switch element is a passive OnePort element with modulation.  It inherits the effort and flow variables from the passive OnePort.  If the modulating signal has a value of <b>true</b>, the switch is considered <b>open</b>. <p>

The causality of the leaking switch element is free. <p>

<hr>

<pre>
Potential variables:

 e:           Bondgraphic effort variable

 f:           Bondgraphic flow variable, normalized positive for flows into the model

 OpenSwitch:  Boolean variable denoting the switch position (modulation signal)

 s:           Curve parameter


Parameters:

 Ron:         Leakage resistance in <b>on</b> state (default value = 1e-5)

 Goff:        Leakage conductance in <b>off</b> state (default value = 1e-5)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f = s*(if OpenSwitch then Goff else 1)</b> <br>
<b>e = s*(if OpenSwitch then 1 else Ron)</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>

"));
    equation 
      OpenSwitch = BooleanInPort1;
      f = s*(if OpenSwitch then Goff else 1);
      e = s*(if OpenSwitch then 1 else Ron);
    end Sw2;
    
    model Sw3 "Switching between two branches if one is unconnected" 
      
    protected 
      Junctions.J1p3 J1p3_1         annotation (extent=[-10,30; 10,10]);
      Bonds.Bond B1         annotation (extent=[-30,10; -10,30]);
      Bonds.Bond B2         annotation (extent=[10,10; 30,30]);
      Bonds.Bond B3         annotation (extent=[-10,30; 10,50],   rotation=
            90);
      Junctions.J0p3 J0p3_1 
        annotation (extent=[-50,-10; -30,10], rotation=-180);
      Bonds.Bond B4         annotation (extent=[-10,-10; 10,10]);
    public 
      Modelica.Blocks.Interfaces.BooleanInput control 
        "true => p--n2 connected, false => p--n1 connected" annotation (extent=[-20,80;
            20,120],         rotation=-90);
      Interfaces.BondCon p "Positive bond graph connector" 
                                 annotation (extent=[-110,-10; -90,10]);
      Interfaces.BondCon n2 "Second negative bond graph connector" 
                                  annotation (extent=[90,-10; 110,10]);
      Interfaces.BondCon n1 "First negative bond graph connector" 
                                  annotation (extent=[90,40; 110,60]);
    protected 
      Junctions.J0p2 j0p2_1 annotation (extent=[30,-10; 50,10]);
      Junctions.J0p2 j0p2_2 annotation (extent=[30,10; 50,30]);
      Sw Sw1 
            annotation (extent=[-10,50; 10,70], rotation=90);
      annotation (Diagram,
        Icon(Text(
            extent=[-64,80; 56,-80],
            string="Sw",
            style(color=0)), Text(extent=[-70,-46; 70,-86], string="%name"),
          Ellipse(extent=[-44,4; -36,-4], style(color=9, rgbcolor={175,175,175})),
          Line(points=[-94,0; -44,0], style(color=9, rgbcolor={175,175,175})),
          Line(points=[-37,2; 40,50], style(color=9, rgbcolor={175,175,175})),
          Line(points=[40,50; 94,50], style(color=9, rgbcolor={175,175,175})),
          Line(points=[40,0; 94,0], style(color=9, rgbcolor={175,175,175})),
          Line(points=[0,25; 0,80], style(color=5, rgbcolor={255,0,255})),
          Text(extent=[-92,2; -72,24],   string="p",
            style(color=9, rgbcolor={175,175,175})),
          Text(extent=[68,-2; 88,20],  string="n2",
            style(color=9, rgbcolor={175,175,175})),
          Text(extent=[68,48; 88,70],   string="n1",
            style(color=9, rgbcolor={175,175,175}))),
        Documentation(info="<html>
This is a special switch used to check whether anything is connected from the outside to the connector n2.
If this is the case, the switch should be in the p-n2 position.
If this is not the case, the switch should be in the p-n1 position. <p>

This commuting switch must be used instead of the corresponding <b>Sw4</b> commuting switch, because Dymola
sets by default the flow of an unconnected terminal to zero.  Hence we must not set it to zero once more
externally. <p>

<hr>

<pre>
Bond graph connectors:

 p:           Positive connector

 n1:          First negative connector

 n2:          Second negative connector


Control variables:

 control:     Boolean variable denoting the switch position (modulation signal)
</pre> <p>

<hr> <p>

Logic: <p>

<b>contol = true:   p-n2 is connected</b> <br>
<b>contol = false:  p-n1 is connected</b>
</html>"));
    equation 
      connect(J0p3_1.BondCon1, B4.BondCon1) annotation (points=[-30,
            1.22461e-015; -30,0; -9.8,0],
                             style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2, p)       annotation (points=[-50,-1.22461e-015;
            -84,-1.22461e-015; -84,0; -100,0],
                                           style(color=8, rgbcolor={192,192,192}));
      connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[-6.00057e-016,
            30.2; -6.00057e-016,30; 0,30],
                                     style(color=8, rgbcolor={192,192,192}));
      connect(B2.BondCon1, J1p3_1.BondCon2) annotation (points=[10.2,20; 10,20],
          style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-10,20; -10,20],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon3, B1.BondCon1) annotation (points=[-40,10; -40,20;
            -29.8,20], style(color=8, rgbcolor={192,192,192}));
      connect(j0p2_1.BondCon2, n2) 
        annotation (points=[50,0; 100,0], style(color=8, rgbcolor={192,192,192}));
      connect(j0p2_2.BondCon1, B2.BondCon2) 
        annotation (points=[30,20; 30,20], style(color=8, rgbcolor={192,192,192}));
      connect(j0p2_2.BondCon2, n1)       annotation (points=[50,20; 60,20; 60,
            50; 100,50],
                     style(color=8, rgbcolor={192,192,192}));
      connect(Sw1.BondCon1, B3.BondCon2) 
                                        annotation (points=[-6.12303e-016,50;
            6.12303e-016,50], style(color=8, rgbcolor={192,192,192}));
      connect(Sw1.BooleanInPort1, control) 
                                          annotation (points=[5.51073e-016,69;
            5.51073e-016,75.5; 0,75.5; 0,100], style(color=5, rgbcolor={255,0,255}));
      connect(B4.BondCon2, j0p2_1.BondCon1) annotation (points=[10,0; 30,0], style(color=8,
            rgbcolor={192,192,192}));
    end Sw3;
    
    model Sw4 "Switching between two branches with leakage" 
      parameter Real Ron(final min=0) = 1.E-5 "Closed switch resistance";
      parameter Real Goff(final min=0) = 1.E-5 "Opened switch conductance";
    protected 
      Junctions.J1p3 J1p3_1         annotation (extent=[-10,30; 10,10]);
      Bonds.Bond B1         annotation (extent=[-30,10; -10,30]);
      Bonds.Bond B2         annotation (extent=[10,10; 30,30]);
      Bonds.Bond B3         annotation (extent=[-10,30; 10,50],   rotation=
            90);
      Junctions.J1p3 J1p3_2         annotation (extent=[-10,-10; 10,10]);
      Junctions.J0p3 J0p3_1 
        annotation (extent=[-50,-10; -30,10], rotation=-180);
      Bonds.Bond B4         annotation (extent=[-30,-10; -10,10]);
      Bonds.Bond B5         annotation (extent=[10,-10; 30,10]);
      Bonds.Bond B6         annotation (extent=[-10,-30; 10,-10], rotation=
            -90);
      Modelica.Blocks.Logical.Not Not1 
        annotation (extent=[-50,-70; -30,-50]);
      Junctions.J0p2 j0p2_1 annotation (extent=[30,-10; 50,10]);
      Junctions.J0p2 j0p2_2 annotation (extent=[30,10; 50,30]);
    public 
      Modelica.Blocks.Interfaces.BooleanInput control 
        "true => p--n2 connected, false => p--n1 connected" annotation (extent=[-20,80;
            20,120],         rotation=-90);
      Interfaces.BondCon p "Positive bond graph connector" 
                                 annotation (extent=[-110,-10; -90,10]);
      Interfaces.BondCon n2 "Second negative bond graph connector" 
                                  annotation (extent=[90,-10; 110,10]);
      Interfaces.BondCon n1 "First negative bond graph connector" 
                                  annotation (extent=[90,40; 110,60]);
    protected 
      BondLib.Switches.Sw2 Sw1(Ron=Ron, Goff=Goff) 
        annotation (extent=[-10,50; 10,70], rotation=90);
      BondLib.Switches.Sw2 Sw2(Ron=Ron, Goff=Goff) 
        annotation (extent=[-10,-50; 10,-30], rotation=270);
    equation 
      connect(B4.BondCon2,J1p3_2. BondCon1) annotation (points=[-10,0; -10,0],
                      style(color=8, rgbcolor={192,192,192}));
      connect(J1p3_2.BondCon2,B5. BondCon1) annotation (points=[10,0; 10.2,0],
                       style(color=8, rgbcolor={192,192,192}));
      connect(B6.BondCon1,J1p3_2. BondCon3) annotation (points=[-6.00057e-016,
            -10.2; -6.00057e-016,-10; 0,-10],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon1,B4. BondCon1) annotation (points=[-30,
            1.22461e-015; -30,0; -29.8,0],
                             style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon2,p)        annotation (points=[-50,-1.22461e-015;
            -84,-1.22461e-015; -84,0; -100,0],
                                           style(color=8, rgbcolor={192,192,192}));
      connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[-6.00057e-016,
            30.2; -6.00057e-016,30; 0,30],
                                     style(color=8, rgbcolor={192,192,192}));
      connect(B2.BondCon1,J1p3_1. BondCon2) annotation (points=[10.2,20; 10,20],
          style(color=8, rgbcolor={192,192,192}));
      connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-10,20; -10,20],
          style(color=8, rgbcolor={192,192,192}));
      connect(J0p3_1.BondCon3,B1. BondCon1) annotation (points=[-40,10; -40,20;
            -29.8,20], style(color=8, rgbcolor={192,192,192}));
      connect(j0p2_1.BondCon2,n2) 
        annotation (points=[50,0; 100,0], style(color=8, rgbcolor={192,192,192}));
      connect(j0p2_1.BondCon1,B5. BondCon2) 
        annotation (points=[30,0; 30,0], style(color=8, rgbcolor={192,192,192}));
      connect(j0p2_2.BondCon1,B2. BondCon2) 
        annotation (points=[30,20; 30,20], style(color=8, rgbcolor={192,192,192}));
      connect(j0p2_2.BondCon2,n1)        annotation (points=[50,20; 60,20; 60,
            50; 100,50],
                     style(color=8, rgbcolor={192,192,192}));
      connect(Not1.u,control)  annotation (points=[-52,-60; -70,-60; -70,74; 0,
            74; 0,100], style(color=5, rgbcolor={255,0,255}));
      annotation (
        Diagram,
        Documentation(info="<html>
Ideal commuting switch element with leakage. This switch can be used to model a two-way switch.
If <i>control = true</i>, the switch is in the p-n2 connection.
If <i>control = false</i>, the switch is in the p-n1 connection. <p>

<hr>

<pre>
Bond graph connectors:

 p:           Positive connector

 n1:          First negative connector

 n2:          Second negative connector


Control variables:

 control:     Boolean variable denoting the switch position (modulation signal)


Parameters:

 Ron:         Leakage resistance in <b>on</b> state (default value = 1e-5)

 Goff:        Leakage conductance in <b>off</b> state (default value = 1e-5)
</pre> <p>

<hr> <p>

Logic: <p>

<b>contol = true:   p-n2 is connected</b> <br>
<b>contol = false:  p-n1 is connected</b>
</html>"),
        Icon(                Text(extent=[-70,-46; 70,-86], string="%name"),
          Ellipse(extent=[-44,4; -36,-4], style(color=9, rgbcolor={175,175,175})),
          Line(points=[-94,0; -44,0], style(color=9, rgbcolor={175,175,175})),
          Line(points=[-37,2; 40,50], style(color=9, rgbcolor={175,175,175})),
          Line(points=[40,50; 94,50], style(color=9, rgbcolor={175,175,175})),
          Line(points=[40,0; 94,0], style(color=9, rgbcolor={175,175,175})),
          Line(points=[0,25; 0,80], style(color=5, rgbcolor={255,0,255})),
          Text(extent=[-92,2; -72,24],   string="p",
            style(color=9, rgbcolor={175,175,175})),
          Text(extent=[68,-2; 88,20],  string="n2",
            style(color=9, rgbcolor={175,175,175})),
          Text(extent=[68,48; 88,70],   string="n1",
            style(color=9, rgbcolor={175,175,175})),
          Text(
            extent=[-74,82; 32,-78],
            string="Sw",
            style(color=0)),
          Text(
            extent=[24,-10; 86,-44],
            string="Leak",
            style(color=0, pattern=0))));
      connect(Sw1.BondCon1, B3.BondCon2) annotation (points=[-6.12303e-016,50;
            6.12303e-016,50], style(color=8, rgbcolor={192,192,192}));
      connect(Sw1.BooleanInPort1, control) annotation (points=[5.51073e-016,69;
            0,78; 0,100], style(color=5, rgbcolor={255,0,255}));
      connect(Sw2.BondCon1, B6.BondCon2) annotation (points=[1.83691e-015,-30;
            6.12303e-016,-30], style(color=8, rgbcolor={192,192,192}));
      connect(Sw2.BooleanInPort1, Not1.y) annotation (points=[-1.65322e-015,-49;
            -1.65322e-015,-60; -29,-60], style(color=5, rgbcolor={255,0,255}));
    end Sw4;
    
    model D "Electrical ideal diode of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      Real s "Curve parameter";
      Boolean blocking;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                fillColor=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[6,80; 16,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Text(
            extent=[70,-4; 80,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-80,0; -20,0; 0,0; 0,66],            style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Line(points=[20,9; 20,0],   style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Ellipse(extent=[-2,2; 2,-2],  style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[2,48; 36,38],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[-54,-2; -24,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[0,10; 20,0],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s=0")),
        Icon(Text(
            extent=[-82, 70; 80, -74],
            string="D",
            style(color=0)), Text(extent=[-70, 88; 70, 48], string="%name")),
        Documentation(info="<html>
The ideal diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The ideal diode is a switch element with internal modulation.  If the modulating signal has a value of <b>true</b>, the diode is considered <b>blocking</b>. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains. <p>

The causality of the ideal diode element must be free, i.e., the ideal diode element must be placed inside an algebraic loop.  If the causality of the diode is not free, use the leaking diode model, <b>D2</b>, instead. <p>

<hr>

<pre>
Potential variables:

 e:         Bondgraphic effort variable

 f:         Bondgraphic flow variable, normalized positive for flows into the model

 blocking:  Boolean variable denoting the switch position (internal modulation signal)

 s:         Curve parameter
</pre> <p>

<hr> <p>

Equations: <p>

<b>blocking = s &lt; 0</b> <br>
<b>f = if blocking then 0 else s</b> <br>
<b>e = if blocking then s else 0</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>

"));
    equation 
      blocking = s < 0;
      f = if blocking then 0 else s;
      e = if blocking then s else 0;
    end D;
    
    model D2 "Electrical leaking diode of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      parameter Real Ron=1E-5 "Leakage Resistance";
      parameter Real Goff=1E-5 "Leakage Conductance";
      parameter Real e0=0 "Forward threshold effort";
      Real s "Curve parameter";
      Boolean blocking;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                fillColor=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[6,80; 16,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Text(
            extent=[70,-4; 80,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-80,-40; -20,-10; 20,10; 40,70],     style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Line(points=[20,9; 20,0],   style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[12,-2; 32,-12],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Text(
            extent=[20,70; 40,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Ron"),
          Text(
            extent=[-20,10; 0,0],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Goff"),
          Ellipse(extent=[18,12; 22,8], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[32,38; 66,28],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[-52,-28; -22,-36],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[20,14; 40,4],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s=0")),
        Icon(
          Text(
            extent=[-82, 70; 80, -74],
            string="D",
            style(color=0)),
          Text(
            extent=[40, -16; 102, -50],
            string="Leak",
            style(color=0, pattern=0)),
          Text(extent=[-70,94; 70,54],   string="%name")),
        Documentation(info="<html>
The ideal diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The leaking diode is a switch element with internal modulation.  If the modulating signal has a value of <b>true</b>, the diode is considered <b>blocking</b>. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains.  Leaking diodes are a special class of non-linear resistors. <p>

The causality of the leaking diode element is free. <p>

<hr>

<pre>
Potential variables:

 e:         Bondgraphic effort variable

 f:         Bondgraphic flow variable, normalized positive for flows into the model

 blocking:  Boolean variable denoting the switch position (internal modulation signal)

 s:         Curve parameter


Parameters:

 Ron:       Leakage resistance in <b>conducting</b> state (default value = 1e-5 Ohm)

 Goff:      Leakage conductance in <b>blocking</b> state (default value = 1e-5 Mho)

 e0:        Forward threshold effort (default value = 0 Volt)
</pre> <p>

<hr> <p>

Equations: <p>

<b>blocking = s &lt; 0</b> <br>
<b>f = s*(if blocking then Goff else 1) + Goff*e0</b> <br>
<b>e = s*(if blocking then 1 else Ron) + e0</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>

"));
    equation 
      blocking = s < 0;
      f = s*(if blocking then Goff else 1) + Goff*e0;
      e = s*(if blocking then 1 else Ron) + e0;
    end D2;
    
    model Z "Electrical ideal Zener diode of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      constant Integer conducting=1;
      constant Integer blocking=2;
      constant Integer avalanche=3;
      parameter Real e0=1 "Avalanche breakdown Voltage";
      Real s "Curve parameter";
      Integer mode "Operating mode";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[6,80; 16,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Line(points=[0,-66; 0,0; 40,0; 40,70],            style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Ellipse(extent=[-2,2; 2,-2],  style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[2,-30; 34,-40],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[6,8; 36,0],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[0,0; 20,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s=0"),
          Text(
            extent=[42,40; 72,30],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="avalanche"),
          Text(
            extent=[42,10; 62,0],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s=e0"),
          Ellipse(extent=[38,2; 42,-2], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[32,-2; 52,-12],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Polygon(points=[80,4; 90,0; 80,-4; 80,4],     style(color=10,
                fillColor=10)),
          Text(
            extent=[80,-4; 90,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e")),
        Icon(
          Text(
            extent=[-82, 70; 80, -74],
            string="Z",
            style(color=0)),
          Text(extent=[-70, 88; 70, 48], string="%name"),
          Text(extent=[-60, -54; 58, -88], string="e0=%e0")),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The ideal Zener diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The ideal Zener diode is a switch element with internal modulation.  The modulating signal, <b>mode</b>, is a ternary signal with the values <b>conducting</b>, <b>blocking</b>, and <b>avalanche</b>. Zener diodes are usually placed between a circuit node and the ground node.  Thus, the bondgraphic Zener diode is usually attached to a 0-junction via a bond. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains.  Zener diodes are always used as reversed diodes.  Thus, the positive and negative pins have been reversed in the model. <p>

The causality of the ideal diode element must be free, i.e., the ideal diode element must be placed inside an algebraic loop. <p>

<hr>

<pre>
Potential variables:

 e:     Bondgraphic effort variable

 f:     Bondgraphic flow variable, normalized positive for flows into the model

 mode:  Ternary variable denoting the switch position (internal modulation signal)

 s:     Curve parameter


Parameters:

 e0:    Avalanche breakdown voltage (default value = 1 Volt)
</pre> <p>

<hr> <p>

Equations: <p>

<b>mode = if s &lt; 0 then conducting else if s &lt; e0 then blocking else avalanche</b> <br>
<b>e = if mode == conducting then 0 else if mode == blocking then s else e0</b> <br>
<b>f = if mode == conducting then s else if mode == blocking then 0 else s - e0</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>

"));
    equation 
      mode = if s < 0 then conducting else if s < e0 then blocking else avalanche;
      e = if mode == conducting then 0 else if mode == blocking then s else e0;
      f = if mode == conducting then s else if mode == blocking then 0 else s - e0;
    end Z;
    
    model Z2 
      "Electrical ideal leaking Zener diode of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      constant Integer conducting=1;
      constant Integer blocking=2;
      constant Integer avalanche=3;
      parameter Real Ron=1E-5 "Leakage Resistance";
      parameter Real Raval=1E-5 "Leakage Resistance";
      parameter Real Goff=1E-5 "Leakage Conductance";
      parameter Real e0=1 "Avalanche breakdown Voltage";
      Real s1 "Curve parameter";
      Real s2 "Curve parameter";
      Integer mode "Operating mode";
      annotation (
        Icon(
          Text(
            extent=[-80,72; 82,-72],
            string="Z",
            style(color=0)),
          Text(extent=[-68,90; 72,50],   string="%name"),
          Text(extent=[-58,-52; 60,-86], string="e0=%e0"),
          Text(
            extent=[36,-16; 98,-50],
            string="Leak",
            style(color=0, pattern=0))),
        Diagram(
          Line(points=[-70,10; 90,10], style(color=10)),
          Line(points=[10,90; 10,-70], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[6,80; 10,90; 14,80; 6,80],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[16,90; 26,80],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Line(points=[-4,-64; 10,10; 52,22; 64,80],        style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Ellipse(extent=[8,12; 12,8],  style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[-34,-34; -2,-44],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[14,28; 44,20],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[10,10; 30,0],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s1=0"),
          Text(
            extent=[60,56; 90,46],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="avalanche"),
          Text(
            extent=[56,26; 76,16],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s2=0"),
          Ellipse(extent=[50,24; 54,20],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[44,8; 64,-2],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Polygon(points=[90,14; 100,10; 90,6; 90,14],  style(color=10,
                fillColor=10)),
          Text(
            extent=[90,6; 100,-4],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[52,19; 52,10], style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[34,18; 54,8],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Goff"),
          Text(
            extent=[4,-20; 24,-30],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Ron"),
          Text(
            extent=[40,74; 60,64],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="Raval")),
        Documentation(info="<html>
The ideal leaking Zener diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The ideal leaking Zener diode is a switch element with internal modulation.  The modulating signal, <b>mode</b>, is a ternary signal with the values <b>conducting</b>, <b>blocking</b>, and <b>avalanche</b>. Zener diodes are usually placed between a circuit node and the ground node.  Thus, the bondgraphic Zener diode is usually attached to a 0-junction via a bond. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains.  Zener diodes are always used as reversed diodes.  Thus, the positive and negative pins have been reversed in the model.  Leaking diodes are a special class of non-linear resistors. <p>

The causality of the leaking Zener diode element is free. <p>

<hr>

<pre>
Potential variables:

 e:      Bondgraphic effort variable

 f:      Bondgraphic flow variable, normalized positive for flows into the model

 mode:   Ternary variable denoting the switch position (internal modulation signal)

 s1:     Curve parameter

 s2:     Curve parameter


Parameters:

 Ron:    Leakage resistance in <b>conducting</b> state (default value = 1e-5 Ohm)

 Raval:  Leakage resistance in <b>avalanche</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>blocking</b> state (default value = 1e-5 Mho)

 e0:     Avalanche breakdown voltage (default value = 1 Volt)
</pre> <p>

<hr> <p>

Equations: <p>

<b>s2 = s1 - e0</b> <br>
<b>mode = if s1 &lt; 0 then conducting else if s1 &lt; e0 then blocking else avalanche</b> <br>
<b>e = if mode == conducting then Ron*s1 else if mode == blocking then s1 else (s2*Ron + e0)</b> <br>
<b>f = if mode == conducting then s1 else if mode == blocking then Goff*s1 else (s2 + Goff*e0)</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>
"));
    equation 
      s2 = s1 - e0;
      mode = if s1 < 0 then conducting else if s1 < e0 then blocking else avalanche;
      e = if mode == conducting then Ron*s1 else if mode == blocking then s1 else (s2*Ron + e0);
      f = if mode == conducting then s1 else if mode == blocking then Goff*s1 else (s2 + Goff*e0);
    end Z2;
    
    model T "Electrical ideal tunnel diode of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      constant Integer blocking=1;
      constant Integer c1=2;
      constant Integer b1=3;
      constant Integer c2=4;
      constant Integer b2=5;
      constant Integer conducting=6;
      parameter Real em=1 "Intermediate forward voltage";
      parameter Real e0=2 "Maximum forward voltage";
      parameter Real fm=1 "Intermediate bottom current";
      parameter Real f0=2 "Intermediate top current";
      Real s1 "Curve parameter";
      Real s2 "Curve parameter";
      Real s3 "Curve parameter";
      Real s4 "Curve parameter";
      Real s5 "Curve parameter";
      Integer mode "Operating mode";
      annotation (
        Icon(
          Text(
            extent=[-80,70; 82,-74],
            style(color=0),
            string="T"),
          Text(extent=[-70,92; 70,52],   string="%name")),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[6,80; 16,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Ellipse(extent=[-2,2; 2,-2],  style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[42,62; 74,52],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[-52,8; -22,0],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[-14,-2; -2,-8],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s1=0"),
          Text(
            extent=[-12,18; 2,10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="c1"),
          Ellipse(extent=[-2,62; 2,58], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[30,0; 50,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Polygon(points=[80,4; 90,0; 80,-4; 80,4],     style(color=10,
                fillColor=10)),
          Text(
            extent=[80,-4; 90,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-72,0; 0,0; 0,60; 20,60],            style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Line(points=[40,80; 40,30; 20,30; 20,60],         style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[18,50; 32,42],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="c2"),
          Text(
            extent=[22,28; 36,20],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="b2"),
          Text(
            extent=[4,68; 18,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="b1"),
          Text(
            extent=[10,0; 30,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="em"),
          Text(
            extent=[-18,36; 2,26],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="fm"),
          Text(
            extent=[-16,64; 4,54],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="f0"),
          Line(points=[40,28; 40,0],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[20,28; 20,0],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[18,30; 0,30],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[2,60; 14,54],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s2=0"),
          Text(
            extent=[20,68; 32,62],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s3=0"),
          Text(
            extent=[22,36; 34,30],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s4=0"),
          Text(
            extent=[42,30; 54,24],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s5=0"),
          Ellipse(extent=[18,62; 22,58],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Ellipse(extent=[18,32; 22,28],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Ellipse(extent=[38,32; 42,28],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1))),
        Documentation(info="<html>
The ideal tunnel diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The ideal tunnel diode is a switch element with internal modulation.  The modulating signal, <b>mode</b>, is a sextary signal with the values <b>blocking</b>, <b>c1</b>, <b>b1</b>, <b>c2</b>, <b>b2</b>, and <b>blocking</b>. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains. <p>

The causality of the ideal diode element must be free, i.e., the ideal diode element must be placed inside an algebraic loop. <p>

<hr>

<pre>
Potential variables:

 e:     Bondgraphic effort variable

 f:     Bondgraphic flow variable, normalized positive for flows into the model

 mode:  Sextary variable denoting the switch position (internal modulation signal)

 s1:    Curve parameter

 s2:    Curve parameter

 s3:    Curve parameter

 s4:    Curve parameter

 s5:    Curve parameter


Parameters:

 em:    Intermediate forward voltage (default value = 1 Volt)

 e0:    Maximum forward voltage (default value = 2 Volt)

 fm:    Bottom intermediate current (default value = 1 Amp)

 f0:    Top intermediate current (default value = 2 Amp)
</pre> <p>

<hr> <p>

Equations: <p>

<b>s2 = s1 - f0</b> <br>
<b>s3 = s2 - em</b> <br>
<b>s4 = s3 - (f0-fm)</b> <br>
<b>s5 = s4 - (e0-em)</b> <br>
<b>mode = if s1 &lt; 0 then blockinging else
          if s2 &lt; 0 then c1 else 
          if s3 &lt; 0 then b1 else 
          if s4 &lt; 0 then c2 else 
          if s5 &lt; 0 then b2 else conducting</b> <br>
<b>e = if mode == blocking then s1 else 
       if mode == c1 then 0 else 
       if mode == b1 then s2 else 
       if mode == c2 then em else 
       if mode == b2 then (em + s4) else e0</b> <br>
<b>f = if mode == blocking then 0 else 
       if mode == c1 then s1 else 
       if mode == b1 then f0 else 
       if mode == c2 then (f0 - s3) else 
       if mode == b2 then fm else (fm + s5)</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>
"));
    equation 
      s2 = s1 - f0;
      s3 = s2 - em;
      s4 = s3 - (f0-fm);
      s5 = s4 - (e0-em);
      mode = if s1 < 0 then blocking else 
             if s2 < 0 then c1 else 
             if s3 < 0 then b1 else 
             if s4 < 0 then c2 else 
             if s5 < 0 then b2 else conducting;
      e = if mode == blocking then s1 else 
          if mode == c1 then 0 else 
          if mode == b1 then s2 else 
          if mode == c2 then em else 
          if mode == b2 then (em + s4) else e0;
      f = if mode == blocking then 0 else 
          if mode == c1 then s1 else 
          if mode == b1 then f0 else 
          if mode == c2 then (f0 - s3) else 
          if mode == b2 then fm else (fm + s5);
    end T;
    
    model T2 
      "Electrical ideal leaking tunnel diode of the Modelica bond graph library" 
      extends Interfaces.PassiveOnePort;
      constant Integer blocking=1;
      constant Integer c1=2;
      constant Integer b1=3;
      constant Integer c2=4;
      constant Integer b2=5;
      constant Integer conducting=6;
      parameter Real G0=1E-5 "Leakage Conductance";
      parameter Real R1=1E-5 "Leakage Resistance";
      parameter Real R2=1E-5 "Leakage Resistance";
      parameter Real R3=1E-5 "Leakage Resistance";
      parameter Real em1=1 "Intermediate forward voltage";
      parameter Real em2=2 "Intermediate forward voltage";
      parameter Real em3=3 "Intermediate forward voltage";
      parameter Real e0=4 "Maximum forward voltage";
      parameter Real fm=1 "Intermediate bottom current";
      parameter Real f0=2 "Intermediate top current";
      Real s1 "Curve parameter";
      Real s2 "Curve parameter";
      Real s3 "Curve parameter";
      Real s4 "Curve parameter";
      Real s5 "Curve parameter";
      Integer mode "Operating mode";
      annotation (
        Icon(
          Text(
            extent=[-78,70; 84,-74],
            style(color=0),
            string="T"),
          Text(extent=[-68,92; 72,52],   string="%name"),
          Text(
            extent=[22,-18; 84,-52],
            string="Leak",
            style(color=0, pattern=0))),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[6,80; 16,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Ellipse(extent=[-2,2; 2,-2],  style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[46,70; 78,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[-52,-10; -22,-18],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[-6,-2; 6,-8],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s1=0"),
          Text(
            extent=[-4,50; 10,42],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="c1"),
          Ellipse(extent=[10,58; 6,62], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[30,0; 50,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Polygon(points=[80,4; 90,0; 80,-4; 80,4],     style(color=10,
                fillColor=10)),
          Text(
            extent=[80,-4; 90,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-72,-12; 0,0; 8,60; 20,60],          style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Line(points=[48,92; 40,30; 28,30; 20,60],         style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[22,54; 36,46],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="c2"),
          Text(
            extent=[28,30; 42,22],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="b2"),
          Text(
            extent=[8,68; 22,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="b1"),
          Text(
            extent=[4,-6; 14,-16],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="em1"),
          Text(
            extent=[-18,36; 2,26],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="fm"),
          Text(
            extent=[-16,64; 4,54],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="f0"),
          Line(points=[40,28; 40,0],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[20,58; 20,0],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[26,30; 0,30],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[-4,68; 8,62],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s2=0"),
          Text(
            extent=[22,66; 34,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s3=0"),
          Text(
            extent=[14,30; 26,24],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s4=0"),
          Text(
            extent=[42,32; 54,26],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s5=0"),
          Ellipse(extent=[18,62; 22,58],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Ellipse(extent=[26,32; 30,28],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Ellipse(extent=[38,32; 42,28],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Line(points=[6,60; 0,60],   style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[14,0; 24,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="em2"),
          Text(
            extent=[22,-6; 32,-16],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="em3"),
          Line(points=[8,58; 8,-8],   style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[28,28; 28,-8], style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[-60,2; -42,-8],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="G0"),
          Text(
            extent=[-12,16; 6,6],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="R1"),
          Text(
            extent=[10,42; 28,32],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="R2"),
          Text(
            extent=[40,52; 58,42],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="R3")),
        Documentation(info="<html>
The ideal leaking tunnel diode element is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The ideal leaking tunnel diode is a switch element with internal modulation.  The modulating signal, <b>mode</b>, is a sextary signal with the values <b>blocking</b>, <b>c1</b>, <b>b1</b>, <b>c2</b>, <b>b2</b>, and <b>blocking</b>. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains.  Leaking diodes are a special class of non-linear resistors. <p>

The causality of the ideal leaking diode element is free. <p>

<hr>

<pre>
Potential variables:

 e:     Bondgraphic effort variable

 f:     Bondgraphic flow variable, normalized positive for flows into the model

 mode:  Sextary variable denoting the switch position (internal modulation signal)

 s1:    Curve parameter

 s2:    Curve parameter

 s3:    Curve parameter

 s4:    Curve parameter

 s5:    Curve parameter


Parameters:

 G0:    Leakage conductance in blocking state (default value = 1e-5 Mho)

 R1:    Leakage resistance in conducting state (default value = 1e-5 Ohm)

 R2:    Leakage resistance in conducting state (default value = 1e-5 Ohm)

 R3:    Leakage resistance in conducting state (default value = 1e-5 Ohm)

 em1:   First knee voltage (default value = 1 Volt)

 em2:   Second knee voltage (default value = 2 Volt)

 em3:   Third knee voltage (default value = 3 Volt)

 e0:    Fourth knee voltage (default value = 4 Volt)

 fm:    Bottom intermediate current (default value = 1 Amp)

 f0:    Top intermediate current (default value = 2 Amp)
</pre> <p>

<hr> <p>

Equations: <p>

<b>s2 = s1 - f0</b> <br>
<b>s3 = s3 = s2 - (em2-em1)</b> <br>
<b>s4 = s3 - (f0-fm)</b> <br>
<b>s5 = s4 - (e0-em3)</b> <br>
<b>mode = if s1 &lt; 0 then blockinging else
          if s2 &lt; 0 then c1 else 
          if s3 &lt; 0 then b1 else 
          if s4 &lt; 0 then c2 else 
          if s5 &lt; 0 then b2 else conducting</b> <br>
<b>e = if mode == blocking then s1 else 
       if mode == c1 then R1*s1 else 
       if mode == b1 then (em1 + s2) else 
       if mode == c2 then (em2 + R2*s3) else 
       if mode == b2 then (em3 + s4) else (e0 + R3*s5)</b> <br>
<b>f = if mode == blocking then G0*s1 else 
       if mode == c1 then s1 else 
       if mode == b1 then f0 else 
       if mode == c2 then (f0 - s3) else 
       if mode == b2 then fm else (fm + s5)</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>
"));
    equation 
      s2 = s1 - f0;
      s3 = s2 - (em2-em1);
      s4 = s3 - (f0-fm);
      s5 = s4 - (e0-em3);
      mode = if s1 < 0 then blocking else 
             if s2 < 0 then c1 else 
             if s3 < 0 then b1 else 
             if s4 < 0 then c2 else 
             if s5 < 0 then b2 else conducting;
      e = if mode == blocking then s1 else 
          if mode == c1 then R1*s1 else 
          if mode == b1 then (em1 + s2) else 
          if mode == c2 then (em2 + R2*s3) else 
          if mode == b2 then (em3 + s4) else (e0 + R3*s5);
      f = if mode == blocking then G0*s1 else 
          if mode == c1 then s1 else 
          if mode == b1 then f0 else 
          if mode == c2 then (f0 - s3) else 
          if mode == b2 then fm else (fm + s5);
    end T2;
  end Switches;
  
  package Thermal "Thermal models of Modelica bond graph library" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100,-100; 80,50],   style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Text(
          extent=[-85,35; 65,-85],
          string="Library",
          style(color=42)),
        Text(
          extent=[-120,122; 120,73],
          string="%name",
          style(color=1))),
      Window(
        x=0.45,
        y=0.01,
        width=0.44,
        height=0.65,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package contains a number of modeling elements that are used in thermal modeling without convective heat flows, such as in the thermal modeling of buildings. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Nebot, A., F.E. Cellier, and F. Mugica (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/ecoeng_bio2.pdf\">Simulation
     of Heat and Humidity Budget of Biosphere 2 without its Air Conditioning</a>,\" <i>Ecological Engineering</i>,
     <b>13</b>, pp. 333-356.
<li> Weiner, M., and F.E. Cellier (1993), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/Weiner.pdf\">Modeling
     and Simulation of a Solar Energy System by Use of Bond Graphs</a>,\" <i>Proc. SCS 1<sup>st</sup> Intl. Conf. on Bond
     Graph Modeling</i>, San Diego, California, pp.301-306. 
</ol>
</html>"));
    package HeatTransfer "1-dimensional heat transfer with lumped elements" 
      annotation (
      preferedView="info",
      Icon(
          Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30, fillPattern=
                  1)),
          Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                fillColor=30, fillPattern=1)),
          Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                fillColor=30, fillPattern=1)),
          Text(
            extent=[-85,35; 65,-85],
            string="Library",
            style(color=3)),
          Text(
            extent=[-116,124; 124,75],
            string="%name",
            style(color=1))), Documentation(info="<html>
This package offers a partial reimplementation of the HeatTransfer library that forms part of the Modelica standard library using bond graph technology.
</html>"));
      package Interfaces "Interface models of the heat transfer library" 
        annotation (
        preferedView="info",
        Icon(
            Text(
              extent=[-118,112; 122,63],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],   style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=58))), Documentation(info="<html>
Sublibrary of interface models: connectors, wrapper models, and partial models that are being used by the heat transfer library.
</html>"));
        model Heat2BG "Heat to bond graph conversion" 
          
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                              annotation (extent=[90,-10; 110,
                10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_a 
            "Thermal connector" 
            annotation (extent=[-110,-10; -90,10]);
        equation 
          BondCon1.e = port_a.T;
          BondCon1.e*BondCon1.f = port_a.Q_flow;
          
          annotation (
            Icon(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80], style(color=1,
                    rgbcolor={226,0,0})),
              Line(points=[80,80; 80,-80; -80,-80],    style(color=9)),
              Line(points=[94,0; 80,0],   style(color=9)),
              Text(
                extent=[4,-14; 70,-64],
                string="BG",
                style(color=9)),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-80,0; -90,0], style(color=1, rgbcolor={226,0,0})),
              Text(
                extent=[-66,68; 10,18],
                string="Heat",
                style(color=43, rgbcolor={255,85,85}))),
            Documentation(info="<html>
This model converts a heat signal to a bondgraphic power signal.
</html>"),  Diagram(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80], style(color=1,
                    rgbcolor={226,0,0})),
              Line(points=[80,80; 80,-80; -80,-80],    style(color=9)),
              Line(points=[94,0; 80,0],   style(color=9)),
              Text(
                extent=[4,-14; 70,-64],
                string="BG",
                style(color=9)),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-80,0; -90,0], style(color=1, rgbcolor={226,0,0})),
              Text(
                extent=[-66,68; 10,18],
                string="Heat",
                style(color=43, rgbcolor={255,85,85}))));
        end Heat2BG;
        
        model BG2Heat "Bond graph to heat conversion" 
          import BondLib;
          
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port_b 
            "Thermal connector" 
            annotation (extent=[90,-10; 110,10]);
        equation 
          BondCon1.e = port_b.T;
          BondCon1.e*BondCon1.f = -port_b.Q_flow;
          annotation (
            Icon(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80],     style(color=9)),
              Line(points=[-94,0; -80,0],   style(color=9)),
              Line(points=[80,80; 80,-80; -80,-80], style(color=42, rgbcolor={
                      209,0,0})),
              Line(points=[90,0; 80,0], style(color=42, rgbcolor={209,0,0})),
              Text(
                extent=[-64,70; 2,20],
                string="BG",
                style(color=9)),
              Text(
                extent=[-8,-16; 68,-66],
                string="Heat",
                style(color=43, rgbcolor={255,85,85})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255}))),
            Diagram(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80],     style(color=9)),
              Line(points=[-94,0; -80,0],   style(color=9)),
              Line(points=[80,80; 80,-80; -80,-80], style(color=42, rgbcolor={
                      209,0,0})),
              Line(points=[90,0; 80,0], style(color=42, rgbcolor={209,0,0})),
              Text(
                extent=[-64,70; 2,20],
                string="BG",
                style(color=9)),
              Text(
                extent=[-8,-16; 68,-66],
                string="Heat",
                style(color=43, rgbcolor={255,85,85})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255}))),
            Documentation(info="<html>
This model converts a bondgraphic power signal to a heat signal.
</html>"));
        end BG2Heat;
        
        partial model OnePort "OnePort partial model of heat transfer library" 
          Modelica.SIunits.HeatFlowRate Q_flow 
            "Heat flow rate from port_a -> port_b";
          Modelica.SIunits.Temperature dT "port_a.T - port_b.T";
          
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_a 
            "Left thermal connector" 
            annotation (extent=[-120,-10; -100,10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port_b 
            "Right thermal connector" 
            annotation (extent=[100,-10; 120,10]);
        equation 
          dT = port_a.T - port_b.T;
          Q_flow = port_a.Q_flow;
          annotation (Icon, Diagram);
        end OnePort;
        
        partial model TempSensor "Absolute temperature sensor" 
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port 
            "Thermal connector"      annotation (extent=[-110,-10; -90,10]);
          annotation (
            Icon(
              Ellipse(extent=[-20,-98; 20,-60],   style(
                  color=0,
                  thickness=2,
                  fillColor=42)),
              Rectangle(extent=[-12,40; 12,-68],   style(color=42, fillColor=42)),
              Line(points=[12,0; 90,0]),
              Line(points=[-90,0; -12,0],   style(color=42)),
              Polygon(points=[-12,40; -12,80; -10,86; -6,88; 0,90; 6,88; 10,86;
                    12,80; 12,40; -12,40],      style(color=0, thickness=2)),
              Line(points=[-12,40; -12,-64],   style(color=0, thickness=2)),
              Line(points=[12,40; 12,-64],   style(color=0, thickness=2)),
              Line(points=[-40,-20; -12,-20],   style(color=0)),
              Line(points=[-40,20; -12,20],   style(color=0)),
              Line(points=[-40,60; -12,60],   style(color=0)),
              Text(extent=[-132,144; 108,84],   string="%name")),
            Diagram,
            Documentation(info="<html>
Generic temperature sensor.
</html>"));
          Heat2BG Heat2BG1 annotation (extent=[-70,-10; -50,10]);
          BondLib.Sensors.De Tabs annotation (extent=[-10,-10; 10,10]);
          BondLib.Bonds.fBond B1 annotation (extent=[-30,-10; -10,10]);
        equation 
          connect(Heat2BG1.port_a, port) annotation (points=[-70,0; -100,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(B1.fBondCon1, Tabs.BondCon1) annotation (points=[-10,0; -10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B1.eBondCon1, Heat2BG1.BondCon1) annotation (points=[-30,0;
                -50,0], style(color=8, rgbcolor={192,192,192}));
        end TempSensor;
        
        partial model FixedTemp "Generic fixed temperature boundary condition" 
          annotation (
            Icon(
              Rectangle(extent=[-100,100; 101,-100],   style(
                  color=0,
                  pattern=0,
                  fillColor=76,
                  fillPattern=8)),
              Line(points=[-92,0; 66,0],   style(color=42, thickness=2)),
              Polygon(points=[60,-20; 60,20; 100,0; 60,-20],     style(
                  color=42,
                  fillColor=42,
                  fillPattern=1)),
              Text(extent=[-123,160; 117,100],   string="%name"),
              Text(
                extent=[-121,-107; 119,-153],
                string="T=%T",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
Generic temperature source.
</html>"));
        protected 
          BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          BondLib.Sources.mSe Tabs annotation (extent=[10,-10; -10,10]);
          BondLib.Bonds.fBond B1 annotation (extent=[10,-10; 30,10]);
        public 
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port 
            "Thermal connector"      annotation (extent=[100,-10; 120,10]);
        equation 
          connect(B1.fBondCon1, BG2Heat1.BondCon1) annotation (points=[30,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(B1.eBondCon1, Tabs.BondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, port) annotation (points=[70,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
        end FixedTemp;
        
        partial model PrescribedTemp 
          "Generic variable temperature boundary condition" 
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port 
            "Thermal connector"      annotation (extent=[100,-10; 120,10]);
          annotation (
            Icon(
              Rectangle(extent=[-100,100; 100,-100],   style(
                  color=0,
                  pattern=0,
                  fillColor=76,
                  fillPattern=8)),
              Line(points=[-100,0; 64,0],   style(color=42, thickness=2)),
              Polygon(points=[62,-20; 62,20; 99,0; 62,-20],     style(
                  color=42,
                  fillColor=42,
                  fillPattern=1)),
              Text(extent=[-125,162; 115,102],   string="%name")),
            Diagram,
            Documentation(info="<html>
Generic modulated temperature source.
</html>"));
        protected 
          BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          BondLib.Sources.mSe Tabs annotation (extent=[10,-10; -10,10]);
          BondLib.Bonds.fBond B1 annotation (extent=[10,-10; 30,10]);
        equation 
          connect(B1.fBondCon1,BG2Heat1. BondCon1) annotation (points=[30,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(B1.eBondCon1,Tabs. BondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, port) annotation (points=[70,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
        end PrescribedTemp;
      end Interfaces;
      
      package Passive "Passive heat transfer elements" 
        annotation (
        preferedView="info",
        Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))), Documentation(info="<html>
Passive elements of the heat transfer library.
</html>"));
        model ThermalConductor 
          "Lumped thermal element transporting heat without storing it" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.ThermalConductance G = 1 
            "Constant thermal conductance of material";
          annotation (
            Icon(
              Text(extent=[-139,134; 141,74],   string="%name"),
              Rectangle(extent=[-100,70; 100,-70],   style(
                  color=0,
                  pattern=0,
                  fillColor=8,
                  fillPattern=8)),
              Text(
                extent=[-115,-76; 113,-116],
                string="G=%G",
                style(color=0)),
              Line(points=[100,70; 100,-70],   style(color=0, thickness=2)),
              Line(points=[-100,70; -100,-70],   style(color=0, thickness=2))),
            Diagram,
            Documentation(info="<html>
This model looks a bit more complicated than the corresponding model of the Modelica standard library.
The difference is caused by the fact that thermal bond graphs operate on entropy flows rather than
heat flows.  The temperature difference causes additional entropy to be generated, but the power is
proeserved, such that heat in equal heat out.
</html>"));
        protected 
          Interfaces.Heat2BG Heat2BG1 annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          Bonds.fBond B1  annotation (extent=[-30,-10; -10,10]);
          Bonds.eBond B2  annotation (extent=[10,-10; 30,10]);
          HE HE1(theta=1/G) annotation (extent=[-10,-10; 10,10]);
        equation 
          connect(Heat2BG1.port_a, port_a) annotation (points=[-70,0; -110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(BG2Heat1.port_b, port_b) annotation (points=[70,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(B1.fBondCon1, HE1.BondCon1) annotation (points=[-10,0; -10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Heat2BG1.BondCon1, B1.eBondCon1) annotation (points=[-50,0;
                -30,0], style(color=8, rgbcolor={192,192,192}));
          connect(B2.fBondCon1, HE1.BondCon2) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1, BG2Heat1.BondCon1) annotation (points=[30,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
        end ThermalConductor;
        
        model Convection "Lumped thermal element for heat convection" 
          Modelica.SIunits.HeatFlowRate Q_flow 
            "Heat flow rate from solid -> fluid";
          Modelica.SIunits.Temperature dT "= solid.T - fluid.T";
          Modelica.Blocks.Interfaces.RealInput Gc(redeclare type SignalType = 
                Modelica.SIunits.ThermalConductance) 
            "Signal representing the convective thermal conductance in [W/K]" 
            annotation (extent=[-20,80; 20,120],   rotation=270);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a solid 
            "Left thermal connector"  annotation (extent=[-120,-10; -100,10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b fluid 
            "Right thermal connector" annotation (extent=[100,-10; 120,10]);
          annotation (
            Icon(
              Rectangle(extent=[-70,80; 99,-80],   style(color=7, fillColor=7)),
              Rectangle(extent=[-100,80; -70,-80],   style(
                  color=0,
                  fillColor=8,
                  fillPattern=8)),
              Text(extent=[-117,-88; 124,-128],   string="%name"),
              Line(points=[-40,80; -40,-80],   style(color=69, fillColor=47)),
              Line(points=[0,80; 0,-80],   style(color=69, fillColor=47)),
              Line(points=[0,80; 0,-80],   style(color=69, fillColor=47)),
              Line(points=[34,80; 34,-80],   style(color=69, fillColor=47)),
              Line(points=[34,-80; 44,-60],   style(color=69, fillColor=47)),
              Line(points=[34,-80; 24,-60],   style(color=69, fillColor=47)),
              Line(points=[0,-80; 10,-60],   style(color=69, fillColor=47)),
              Line(points=[0,-80; -10,-60],   style(color=69, fillColor=47)),
              Line(points=[-40,-80; -30,-60],   style(color=69, fillColor=47)),
              Line(points=[-40,-80; -50,-60],   style(color=69, fillColor=47)),
              Line(points=[70,-80; 60,-60],   style(color=69, fillColor=47)),
              Line(points=[70,-80; 80,-60],   style(color=69, fillColor=47)),
              Line(points=[70,80; 70,-80],   style(color=69, fillColor=47)),
              Line(points=[100,0; 100,0],   style(color=69, fillColor=47)),
              Line(points=[100,80; 100,-80],   style(color=69, fillColor=47)),
              Line(points=[100,-80; 90,-60],   style(color=69, fillColor=47)),
              Line(points=[100,-80; 110,-60],   style(color=69, fillColor=47)),
              Line(points=[80,30; 100,20],   style(color=42, fillColor=45)),
              Line(points=[80,10; 100,20],   style(color=42, fillColor=45)),
              Line(points=[-59,20; 100,20],   style(color=42, fillColor=45)),
              Line(points=[-60,-20; 100,-20],   style(color=42, fillColor=45)),
              Line(points=[80,-10; 100,-20],   style(color=42, fillColor=45)),
              Line(points=[80,-30; 100,-20],   style(color=42, fillColor=45))),
            Diagram,
            Documentation(info="<html>
This is a highly simplified convection model.  In this model, the flow is caused by temperature difference.
The only effect modeled is that the conductance itself may be variable, caused by some external quantity,
such as wind. <p>

A general convection model needs to take into account mass flow, which is outside the realm of the
BondLib library.  Such a model can be found in the ThermoBondLib library. <p>
</html>"));
        protected 
          Interfaces.Heat2BG Heat2BG1 annotation (extent=[-96,-10; -76,10]);
          Interfaces.BG2Heat BG2Heat1 annotation (extent=[76,-10; 96,10]);
          Junctions.J0p3 J0p3_1 annotation (extent=[-50,-10; -30,-30]);
          Junctions.J0p3 J0p3_2 annotation (extent=[30,-10; 50,-30]);
          Junctions.J1p3 J1p3_1 annotation (extent=[-10,-10; 10,-30]);
          Bonds.fBond B3 annotation (extent=[-10,-10; 10,10],  rotation=90);
          Junctions.J0p3 J0p3_3 annotation (extent=[-10,10; 10,30]);
          Bonds.fBond B4 annotation (extent=[10,10; 30,30]);
          Bonds.fBond B5 annotation (extent=[-30,10; -10,30],   rotation=180);
          Bonds.eBond B2 annotation (extent=[10,-30; 30,-10]);
          Bonds.eBond B6 annotation (extent=[40,-10; 60,10],  rotation=-90);
          Bonds.fBond B1 annotation (extent=[-30,-30; -10,-10]);
          Bonds.eBond B7 annotation (extent=[-60,-10; -40,10],  rotation=-90);
          Bonds.fBond B8 annotation (extent=[-70,-30; -50,-10]);
          Bonds.eBond B9 annotation (extent=[50,-30; 70,-10]);
          Modelica.Blocks.Math.Gain gain(k=0.5) 
            annotation (extent=[-10,48; 10,68], rotation=-90);
          mGSth mGSth1 annotation (extent=[30,30; 50,10]);
          mGSth mGSth2 annotation (extent=[-30,30; -50,10]);
        equation 
          Q_flow = solid.Q_flow;
          dT = mGSth1.e1;
          connect(BG2Heat1.port_b, fluid) annotation (points=[96,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(solid, Heat2BG1.port_a) annotation (points=[-110,0; -96,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,-10;
                -6.12303e-016,-10],
                style(color=8));
          connect(B3.fBondCon1,J0p3_3. BondCon3) annotation (points=[
                6.12303e-016,10; 0,10],
                style(color=8));
          connect(J0p3_3.BondCon2,B4. eBondCon1) annotation (points=[10,20; 10,
                20],
                   style(color=8));
          connect(J0p3_3.BondCon1,B5. eBondCon1) annotation (points=[-10,20;
                -10,20],
                     style(color=8));
          connect(J1p3_1.BondCon2,B2. fBondCon1) annotation (points=[10,-20; 10,
                -20],
                style(color=8));
          connect(B2.eBondCon1,J0p3_2. BondCon1) annotation (points=[30,-20; 30,
                -20],
                style(color=8));
          connect(B6.eBondCon1,J0p3_2. BondCon3) annotation (points=[50,-10; 40,
                -10],
                   style(color=8));
          connect(J0p3_1.BondCon2,B1. eBondCon1) annotation (points=[-30,-20;
                -30,-20],
                   style(color=8));
          connect(B1.fBondCon1,J1p3_1. BondCon1) annotation (points=[-10,-20;
                -10,-20],
                   style(color=8));
          connect(B7.eBondCon1,J0p3_1. BondCon3) annotation (points=[-50,-10;
                -40,-10],
                     style(color=8));
          connect(B8.fBondCon1, J0p3_1.BondCon1) annotation (points=[-50,-20;
                -50,-20], style(color=8, rgbcolor={192,192,192}));
          connect(Heat2BG1.BondCon1, B8.eBondCon1) annotation (points=[-76,0;
                -74,0; -74,-20; -70,-20], style(color=8, rgbcolor={192,192,192}));
          connect(J0p3_2.BondCon2, B9.fBondCon1) annotation (points=[50,-20; 50,
                -20], style(color=8, rgbcolor={192,192,192}));
          connect(B9.eBondCon1, BG2Heat1.BondCon1) annotation (points=[70,-20;
                74,-20; 74,0; 76,0], style(color=8, rgbcolor={192,192,192}));
          connect(gain.u, Gc) annotation (points=[-7.34764e-016,70;
                -7.34764e-016,75; 0,75; 0,100], style(color=74, rgbcolor={0,0,
                  127}));
          connect(mGSth1.BondCon1, B4.fBondCon1) annotation (points=[30,20; 30,
                20], style(color=8, rgbcolor={192,192,192}));
          connect(mGSth1.BondCon2, B6.fBondCon1) annotation (points=[50,20; 50,
                10], style(color=8, rgbcolor={192,192,192}));
          connect(mGSth2.BondCon1, B5.fBondCon1) annotation (points=[-30,20;
                -30,20], style(color=8, rgbcolor={192,192,192}));
          connect(mGSth2.BondCon2, B7.fBondCon1) annotation (points=[-50,20;
                -50,10], style(color=8, rgbcolor={192,192,192}));
          connect(gain.y, mGSth2.s) annotation (points=[6.73533e-016,47;
                6.73533e-016,40; -40,40; -40,29], style(color=74, rgbcolor={0,0,
                  127}));
          connect(gain.y, mGSth1.s) annotation (points=[6.73533e-016,47;
                6.73533e-016,40; 40,40; 40,29], style(color=74, rgbcolor={0,0,
                  127}));
        end Convection;
        
        model BodyRadiation 
          "Lumped thermal element for radiation heat transfer" 
          extends Interfaces.OnePort;
          parameter Real Gr(unit="m2") 
            "Net radiation conductance between two surfaces (see docu)";
          annotation (
            Icon(
              Rectangle(extent=[60,80; 100,-80],   style(
                  color=0,
                  fillColor=8,
                  fillPattern=8)),
              Rectangle(extent=[-100,80; -60,-80],   style(
                  color=0,
                  fillColor=8,
                  fillPattern=8)),
              Line(points=[-40,10; 40,10],   style(color=42, fillColor=45)),
              Line(points=[-40,10; -30,16],   style(color=42, fillColor=45)),
              Line(points=[-40,10; -30,4],   style(color=42, fillColor=45)),
              Line(points=[-40,-10; 40,-10],   style(color=42, fillColor=45)),
              Line(points=[30,-16; 40,-10],   style(color=42, fillColor=45)),
              Line(points=[30,-4; 40,-10],   style(color=42, fillColor=45)),
              Line(points=[-40,-30; 40,-30],   style(color=42, fillColor=45)),
              Line(points=[-40,-30; -30,-24],   style(color=42, fillColor=45)),
              Line(points=[-40,-30; -30,-36],   style(color=42, fillColor=45)),
              Line(points=[-40,30; 40,30],   style(color=42, fillColor=45)),
              Line(points=[30,24; 40,30],   style(color=42, fillColor=45)),
              Line(points=[30,36; 40,30],   style(color=42, fillColor=45)),
              Text(extent=[-132,144; 108,84],   string="%name"),
              Text(
                extent=[-119,-86; 117,-125],
                string="G=%G",
                style(color=0)),
              Rectangle(extent=[-60,80; -54,-80],   style(color=42, fillColor=42)),
              Rectangle(extent=[55,80; 60,-80],   style(color=42, fillColor=42))),
            Diagram,
            Documentation(info="<html>
This is the bond graph model of two bodies radiating against each other.
</html>"));
        protected 
          Interfaces.Heat2BG Heat2BG1 annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          Bonds.fBond B1  annotation (extent=[-30,-10; -10,10]);
          Bonds.eBond B2  annotation (extent=[10,-10; 30,10]);
          RA RA1(A=Gr) annotation (extent=[-10,-10; 10,10]);
        equation 
          connect(Heat2BG1.BondCon1, B1.eBondCon1) annotation (points=[-50,0;
                -30,0], style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1, BG2Heat1.BondCon1) annotation (points=[30,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(Heat2BG1.port_a, port_a) annotation (points=[-70,0; -110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(BG2Heat1.port_b, port_b) annotation (points=[70,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(B1.fBondCon1, RA1.BondCon1) annotation (points=[-10,0; -10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B2.fBondCon1, RA1.BondCon2) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
        end BodyRadiation;
        
        model HeatCapacitor "Lumped thermal element storing heat" 
          parameter Modelica.SIunits.HeatCapacity C=1 
            "Heat capacity of part (= cp*m)";
          Modelica.SIunits.Temperature T(start=Modelica.SIunits.Conversions.from_degC(25.0), stateSelect=StateSelect.prefer) 
            "Temperature of part";
          
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port 
            "Thermal connector"      annotation (extent=[-10,-110; 10,-90],
              rotation=90);
          annotation (
            Icon(
              Text(extent=[-129,121; 131,70],   string="%name"),
              Polygon(points=[0,65; -20,61; -40,55; -52,41; -58,33; -68,23; -72,
                    11; -76,-3; -78,-17; -76,-33; -76,-45; -76,-55; -70,-67;
                    -64,-75; -48,-79; -30,-85; -18,-85; -2,-87; 8,-91; 22,-91;
                    32,-89; 42,-83; 54,-77; 56,-75; 66,-63; 68,-55; 70,-53; 72,
                    -37; 76,-23; 78,-15; 78,1; 74,13; 66,23; 54,31; 44,39; 36,
                    55; 26,63; 0,65],         style(color=9, fillColor=8)),
              Polygon(points=[-58,33; -68,23; -72,11; -76,-3; -78,-17; -76,-33;
                    -76,-45; -76,-55; -70,-67; -64,-75; -48,-79; -30,-85; -18,
                    -85; -2,-87; 8,-91; 22,-91; 32,-89; 42,-83; 54,-77; 42,-79;
                    40,-79; 30,-81; 20,-83; 18,-83; 10,-83; 2,-79; -12,-75; -22,
                    -75; -30,-73; -40,-67; -50,-57; -56,-45; -58,-37; -58,-27;
                    -60,-15; -60,-7; -60,5; -58,15; -56,17; -52,25; -48,33; -44,
                    43; -40,55; -58,33],              style(color=0, fillColor=9)),
              Text(
                extent=[-69,5; 71,-26],
                string="%C",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
Heat capacitance model.
</html>"));
        protected 
          Interfaces.Heat2BG Heat2BG1 
            annotation (extent=[-10,-70; 10,-50], rotation=90);
          Cth Cth1(gamma=C) annotation (extent=[-10,-10; 10,10], rotation=90);
          Bonds.eBond B1  annotation (extent=[-10,-30; 10,-10], rotation=90);
        equation 
          T = port.T;
          connect(Heat2BG1.port_a, port) annotation (points=[-6.12303e-016,-70;
                0,-70; 0,-100], style(color=42, rgbcolor={191,0,0}));
          connect(B1.eBondCon1, Cth1.BondCon1) annotation (points=[6.12303e-016,
                -10; -6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
          connect(Heat2BG1.BondCon1, B1.fBondCon1) annotation (points=[
                6.12303e-016,-50; -6.12303e-016,-50; -6.12303e-016,-30], style(
                color=8, rgbcolor={192,192,192}));
        end HeatCapacitor;
      end Passive;
      
      package Sensors "Sensors of the heat transfer library" 
        annotation (
        preferedView="info",
        Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))), Documentation(info="<html>
Sensor elements of the heat transfer library.
</html>"));
        model TemperatureSensor "Absolute temperature sensor in Kelvin" 
          extends Interfaces.TempSensor;
          Modelica.Blocks.Interfaces.RealOutput T(redeclare type SignalType = 
                Modelica.SIunits.Temperature) "Absolute temperature sensor [K]"
                                annotation (extent=[90,-10; 110,10]);
          annotation (
            Icon(
              Text(
                extent=[126,-20; 26,-120],
                string="K",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
Absolute temperature sensor (in Kelvin).
</html>"));
        equation 
          connect(Tabs.OutPort1, T) annotation (points=[11,0; 100,0], style(
                color=74, rgbcolor={0,0,127}));
        end TemperatureSensor;
        
        model RelTemperatureSensor "Relative Temperature sensor" 
          extends Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealOutput T_rel(redeclare type SignalType
              = Modelica.SIunits.Temperature) "Relative temperature sensor" 
                                annotation (extent=[-10,-80; 10,-100],   rotation=90);
          annotation (
            Icon(
              Rectangle(extent=[-70,-60; 70,20],   style(color=0, fillColor=7)),
              Polygon(points=[0,-40; -10,-16; 10,-16; 0,-40],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Line(points=[0,0; 0,-16],   style(color=0)),
              Line(points=[-70,0; 0,0],   style(color=0)),
              Line(points=[-50,-40; -50,-60],   style(color=0)),
              Line(points=[-30,-40; -30,-60],   style(color=0)),
              Line(points=[-10,-40; -10,-60],   style(color=0)),
              Line(points=[10,-40; 10,-60],   style(color=0)),
              Line(points=[30,-40; 30,-60],   style(color=0)),
              Line(points=[50,-40; 50,-60],   style(color=0)),
              Line(points=[-90,0; -70,0; -70,0],    style(color=42)),
              Line(points=[-100,0; -70,0; -70,0],   style(color=42)),
              Line(points=[70,0; 90,0; 100,0],   style(color=42)),
              Line(points=[0,-30; 0,-80]),
              Text(extent=[-140,94; 144,34],   string="%name"),
              Text(
                extent=[92,-62; 34,-122],
                string="K",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
Relative temperature sensor.  The sensor can be placed between any two heat ports.  There is zero heat flow through the sensor.
</html>"));
        protected 
          Interfaces.Heat2BG Heat2BG1 annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          Bonds.fBond B2 annotation (extent=[-10,-30; 10,-10], rotation=-90);
          BondLib.Sensors.De Trel 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          BondLib.Junctions.J1p3 j1p3_1 annotation (extent=[-10,-10; 10,10]);
          Bonds.fBond B1 annotation (extent=[-30,-10; -10,10], rotation=0);
          Bonds.eBond B3  annotation (extent=[10,-10; 30,10], rotation=0);
        equation 
          connect(Heat2BG1.port_a, port_a) annotation (points=[-70,0; -110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(BG2Heat1.port_b, port_b) annotation (points=[70,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(B2.fBondCon1, Trel.BondCon1) 
                                            annotation (points=[6.12303e-016,
                -30; -6.12303e-016,-30], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j1p3_1.BondCon3, B2.eBondCon1) annotation (points=[0,-10;
                -6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
          connect(Trel.OutPort1, T_rel) annotation (points=[6.73533e-016,-51;
                6.73533e-016,-65.5; 0,-65.5; 0,-90], style(color=74, rgbcolor={
                  0,0,127}));
          connect(B1.fBondCon1, j1p3_1.BondCon1) annotation (points=[-10,0; -10,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(Heat2BG1.BondCon1, B1.eBondCon1) annotation (points=[-50,0;
                -30,0], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_1.BondCon2, B3.fBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B3.eBondCon1, BG2Heat1.BondCon1) annotation (points=[30,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
        end RelTemperatureSensor;
        
        model HeatFlowSensor "Heat flow rate sensor" 
          extends Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealOutput Q_flow1(
                                                       redeclare type 
              SignalType = 
                Modelica.SIunits.HeatFlowRate) 
            "Heat flow from port_a to port_b"                      annotation (extent=[-10,-110;
                10,-90],             rotation=270);
          annotation (
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
              Text(
                extent=[33,-58; 88,-116],
                string="Q_flow",
                style(color=0)),
              Line(points=[70,0; 100,0],  style(color=42)),
              Line(points=[0,-70; 0,-90]),
              Text(extent=[-120,144; 120,84],   string="%name"),
              Line(points=[-100,0; -70,0],style(color=42))),
            Diagram,
            Documentation(info="<html>
Heat flow sensor.  From a bond graph perspective, the heat flow is the power flowing through a bond.
This is being measured using a power bond sensor.
</html>"));
        protected 
          Interfaces.Heat2BG Heat2BG1 annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          BondLib.Sensors.Pbond B1 annotation (extent=[-10,-10; 10,10]);
        equation 
          connect(Heat2BG1.port_a, port_a) annotation (points=[-70,0; -110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(BG2Heat1.port_b, port_b) annotation (points=[70,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(Heat2BG1.BondCon1, B1.BondCon1) annotation (points=[-50,0;
                -9.8,0], style(color=8, rgbcolor={192,192,192}));
          connect(B1.BondCon2, BG2Heat1.BondCon1) annotation (points=[10,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(B1.P, Q_flow1) annotation (points=[0,9; 0,20; 30,20; 30,-40;
                0,-40; 0,-100], style(color=74, rgbcolor={0,0,127}));
        end HeatFlowSensor;
        
        package Celsius "Sensors measuring temperature in degrees Celsius" 
          annotation (
          preferedView="info",
          Icon(
              Text(
                extent=[-120,118; 120,69],
                string="%name",
                style(color=1)),
              Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                     1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=30, fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                    fillColor=30, fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=3))), Documentation(info="<html>
Sensors reporting temperature in degrees Celsius.
</html>"));
          model TemperatureSensor "Absolute temperature sensor in Celsius" 
            extends Interfaces.TempSensor;
            Modelica.Blocks.Interfaces.RealOutput T(redeclare type SignalType 
                = Modelica.SIunits.Conversions.NonSIunits.Temperature_degC) 
              "Absolute temperature sensor [degC]" 
                                          annotation (extent=[90,-10; 110,10]);
            annotation (
              Icon(
                Text(
                  extent=[126,-20; 26,-120],
                  string="C",
                  style(color=0))),
              Diagram,
              Documentation(info="<html>
Absolute temperature sensor (in Celsius).
</html>"));
          protected 
            Utilities.K2C K2C annotation (extent=[40,-10; 60,10]);
          equation 
            connect(Tabs.OutPort1, K2C.Kelvin) annotation (points=[11,0; 38,0],
                style(color=74, rgbcolor={0,0,127}));
            connect(K2C.Celsius, T) annotation (points=[61,0; 100,0], style(
                  color=74, rgbcolor={0,0,127}));
          end TemperatureSensor;
        end Celsius;
        
        package Fahrenheit 
          "Sensors measuring temperature in degrees Fahrenheit" 
          annotation (
          preferedView="info",
          Icon(
              Text(
                extent=[-120,118; 120,69],
                string="%name",
                style(color=1)),
              Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                     1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=30, fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                    fillColor=30, fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=3))), Documentation(info="<html>
Sensors reporting temperature in degrees Fahrenheit.
</html>"));
          model TemperatureSensor "Absolute temperature sensor in Fahrenheit" 
            extends Interfaces.TempSensor;
            Modelica.Blocks.Interfaces.RealOutput T(redeclare type SignalType 
                = Modelica.SIunits.Conversions.NonSIunits.Temperature_degF) 
              "Absolute temperature sensor [degF]" 
                                          annotation (extent=[90,-10; 110,10]);
          protected 
            Utilities.K2F K2F annotation (extent=[40,-10; 60,10]);
          equation 
            
            annotation (
              Icon(
                Text(
                  extent=[126,-20; 26,-120],
                  string="F",
                  style(color=0))),
              Diagram,
              Documentation(info="<html>
Absolute temperature sensor (in Fahrenheit).
</html>"));
            connect(Tabs.OutPort1, K2F.Kelvin) annotation (points=[11,0; 38,0],
                style(color=74, rgbcolor={0,0,127}));
            connect(K2F.Fahrenheit, T) annotation (points=[61,0; 100,0], style(
                  color=74, rgbcolor={0,0,127}));
          end TemperatureSensor;
        end Fahrenheit;
      end Sensors;
      
      package Sources "Sources of the heat transfer library" 
        annotation (
        preferedView="info",
        Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))), Documentation(info="<html>
Source elements of the heat transfer library.
</html>"));
        model FixedTemperature "Fixed temperature boundary condition in Kelvin" 
          extends Interfaces.FixedTemp;
          parameter Modelica.SIunits.Temperature T=298.15 
            "Fixed temperature at port";
          annotation (Icon(
              Text(
                extent=[0,0; -100,-100],
                string="K",
                style(color=0))), Documentation(info="<html>
Absolute temperature source (in Kelvin).
</html>"));
        equation 
          Tabs.s = T;
        end FixedTemperature;
        
        model PrescribedTemperature 
          "Variable temperature boundary condition in Kelvin" 
          extends Interfaces.PrescribedTemp;
          Modelica.Blocks.Interfaces.RealInput T(redeclare type SignalType = 
                Modelica.SIunits.Temperature) "Absolute temperature source [K]"
                                annotation (extent=[-140,-20; -100,20]);
          annotation (
            Icon(
              Text(
                extent=[0,0; -100,-100],
                string="K",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
Absolute modulated temperature source (in Kelvin).
</html>"));
        equation 
          connect(Tabs.s, T) annotation (points=[-9,0; -120,0], style(color=74,
                rgbcolor={0,0,127}));
        end PrescribedTemperature;
        
        model FixedHeatFlow "Fixed heat flow boundary condition" 
          import BondLib;
          parameter Modelica.SIunits.HeatFlowRate Q_flow=0 
            "Fixed heat flow rate at port";
          
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port 
            "Thermal connector"      annotation (extent=[100,-10; 120,10]);
          annotation (
            Icon(
              Text(extent=[-124,120; 142,60],   string="%name"),
              Text(
                extent=[-127,-52; 143,-100],
                string="Q_flow=%Q_flow",
                style(color=0)),
              Line(points=[-90,-20; 58,-20],    style(color=42, thickness=2)),
              Line(points=[-90,20; 56,20],    style(color=42, thickness=2)),
              Polygon(points=[50,0; 50,40; 80,20; 50,0],     style(
                  color=42,
                  fillColor=42,
                  fillPattern=1)),
              Polygon(points=[50,-40; 50,0; 80,-20; 50,-40],     style(
                  color=42,
                  fillColor=42,
                  fillPattern=1)),
              Rectangle(extent=[80,40; 100,-40],  style(color=42, fillColor=42))),
            Diagram,
            Documentation(info="<html>
Source of heat flow.
</html>"));
        protected 
          Interfaces.BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          Bonds.eBond B1 annotation (extent=[10,-10; 30,10]);
          BondLib.Sources.mSP Qflow annotation (extent=[10,-10; -10,10]);
        equation 
          Qflow.s = Q_flow;
          connect(BG2Heat1.port_b, port) annotation (points=[70,0; 110,0],
              style(color=42, rgbcolor={191,0,0}));
          connect(B1.eBondCon1, BG2Heat1.BondCon1) annotation (points=[30,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(Qflow.BondCon1, B1.fBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
        end FixedHeatFlow;
        
        model PrescribedHeatFlow "Prescribed heat flow boundary condition" 
          parameter Modelica.SIunits.Temperature T_ref = Modelica.SIunits.Conversions.from_degC(20) 
            "Reference temperature";
          parameter Real alpha(unit="1/K") = 0 
            "Temperature coefficient of heat flow rate";
          Modelica.Blocks.Interfaces.RealInput Q_flow(
                 redeclare type SignalType = Modelica.SIunits.HeatFlowRate) 
            "Heat flow source" 
                annotation (extent=[-80,-20; -120,20],   rotation=180);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port 
            "Thermal connector"      annotation (extent=[90,-10; 110,10]);
          annotation (
            Icon(
              Line(points=[-60,-20; 40,-20],   style(color=42, thickness=2)),
              Line(points=[-60,20; 40,20],   style(color=42, thickness=2)),
              Line(points=[-80,0; -60,-20],   style(color=42, thickness=2)),
              Line(points=[-80,0; -60,20],   style(color=42, thickness=2)),
              Polygon(points=[40,0; 40,40; 70,20; 40,0],     style(
                  color=42,
                  fillColor=42,
                  fillPattern=1)),
              Polygon(points=[40,-40; 40,0; 70,-20; 40,-40],     style(
                  color=42,
                  fillColor=42,
                  fillPattern=1)),
              Rectangle(extent=[70,40; 90,-40],   style(color=42, fillColor=42)),
              Text(extent=[-134,120; 132,60],   string="%name")),
            Diagram,
            Documentation(info="<html>
Modulated source of heat flow.
</html>"));
        protected 
          Interfaces.BG2Heat BG2Heat1 annotation (extent=[50,-10; 70,10]);
          Bonds.eBond B1 annotation (extent=[10,-10; 30,10]);
          BondLib.Sources.mSP Qflow annotation (extent=[10,-10; -10,10]);
        equation 
          Qflow.s = Q_flow*(1 + alpha*(Qflow.e - T_ref));
          connect(B1.eBondCon1,BG2Heat1. BondCon1) annotation (points=[30,0; 50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(Qflow.BondCon1,B1. fBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, port) annotation (points=[70,0; 100,0],
              style(color=42, rgbcolor={191,0,0}));
        end PrescribedHeatFlow;
        
        package Celsius "Sources specifying temperature in degrees Celsius" 
          annotation (
          preferedView="info",
          Icon(
              Text(
                extent=[-120,118; 120,69],
                string="%name",
                style(color=1)),
              Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                     1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=30, fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                    fillColor=30, fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=3))), Documentation(info="<html>
Sources specifying temperature in degrees Celsius.
</html>"));
          model FixedTemperature 
            "Fixed temperature boundary condition in degree Celsius" 
            extends Interfaces.FixedTemp;
            parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degC 
              T =                                                                25 
              "Fixed Temperature at the port";
            annotation (
              Icon(
                Text(
                  extent=[0,0; -100,-100],
                  string="C",
                  style(color=0))),
              Diagram,
              Documentation(info="<html>
Absolute temperature source (in Celsius).
</html>"));
          protected 
            Utilities.C2K C2K annotation (extent=[-40,-10; -20,10]);
          equation 
            C2K.Celsius = T;
            connect(C2K.Kelvin, Tabs.s) annotation (points=[-19,0; -9,0], style(
                  color=74, rgbcolor={0,0,127}));
          end FixedTemperature;
          
          model PrescribedTemperature 
            "Variable temperature boundary condition in Celsius" 
            extends Interfaces.PrescribedTemp;
            Modelica.Blocks.Interfaces.RealInput T(
               redeclare type SignalType = 
                  Modelica.SIunits.Conversions.NonSIunits.Temperature_degC) 
              "Absolute temperature source [degC]"                       annotation (extent=[-140,-20;
                  -100,20]);
            annotation (Icon(
                Text(
                  extent=[0,0; -100,-100],
                  string="C",
                  style(color=0))), Diagram,
              Documentation(info="<html>
Absolute modulated temperature source (in Celsius).
</html>"));
          protected 
            Utilities.C2K C2K annotation (extent=[-40,-10; -20,10]);
          equation 
            connect(C2K.Celsius, T) annotation (points=[-42,0; -120,0], style(
                  color=74, rgbcolor={0,0,127}));
            connect(C2K.Kelvin, Tabs.s) annotation (points=[-19,0; -9,0], style(
                  color=74, rgbcolor={0,0,127}));
          end PrescribedTemperature;
        end Celsius;
        
        package Fahrenheit 
          "Sources specifying temperature in degrees Fahrenheit" 
          annotation (
          preferedView="info",
          Icon(
              Text(
                extent=[-120,118; 120,69],
                string="%name",
                style(color=1)),
              Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                     1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=30, fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                    fillColor=30, fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=3))), Documentation(info="<html>
Sources specifying temperature in degrees Fahrenheit.
</html>"));
          model FixedTemperature 
            "Fixed temperature boundary condition in Fahrenheit" 
            extends Interfaces.FixedTemp;
            parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degF 
              T =                                                                80 
              "Fixed Temperature at the port";
            annotation (
              Icon(
                Text(
                  extent=[0,0; -100,-100],
                  string="F",
                  style(color=0))),
              Diagram,
              Documentation(info="<html>
Absolute temperature source (in Fahrenheit).
</html>"));
          protected 
            Utilities.F2K F2K annotation (extent=[-40,-10; -20,10]);
          equation 
            F2K.Fahrenheit = T;
            connect(F2K.Kelvin, Tabs.s) annotation (points=[-19,0; -9,0], style(
                  color=74, rgbcolor={0,0,127}));
          end FixedTemperature;
          
          model PrescribedTemperature 
            "Variable temperature boundary condition in Fahrenheit" 
            extends Interfaces.PrescribedTemp;
            Modelica.Blocks.Interfaces.RealInput T(
               redeclare type SignalType = 
                  Modelica.SIunits.Conversions.NonSIunits.Temperature_degF) 
              "Absolute temperature source [degF]" 
              annotation (extent=[-140,-20; -100,20]);
            annotation (
              Icon(
                Text(
                  extent=[0,0; -100,-100],
                  string="F",
                  style(color=0))),
              Documentation(info="<html>
Absolute modulated temperature source (in Celsius).
</html>"),    Diagram);
            Utilities.F2K F2K annotation (extent=[-40,-10; -20,10]);
          equation 
            connect(F2K.Fahrenheit, T) annotation (points=[-42,0; -74,0; -74,0;
                  -120,0], style(color=74, rgbcolor={0,0,127}));
            connect(Tabs.s, F2K.Kelvin) annotation (points=[-9,0; -19,0], style(
                  color=74, rgbcolor={0,0,127}));
          end PrescribedTemperature;
        end Fahrenheit;
      end Sources;
      
      package Switches "Heat switches" 
        annotation (Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))), Documentation(info="<html>
Switches of the heat transfer library.
</html>"));
        model HeatBranch 
          "Branching of heat depending on a whether port n2 is connected" 
          
        protected 
          BondLib.Switches.Sw3 Sw1 
                           annotation (extent=[-20,-20; 20,20]);
          BondLib.Bonds.Bond B1 annotation (extent=[-40,-10; -20,10]);
          BondLib.Bonds.Bond B2 annotation (extent=[20,-10; 40,10]);
          BondLib.Bonds.Bond B3 annotation (extent=[20,0; 40,20]);
        public 
          Modelica.Blocks.Interfaces.BooleanInput control 
            "true => p--n2 connected, false => p--n1 connected" annotation (extent=[-20,80;
                20,120],         rotation=-90);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a p 
            "Positive thermal connector" 
            annotation (extent=[-120,-10; -100,10]);
        protected 
          BondLib.Thermal.HeatTransfer.Interfaces.Heat2BG Heat2BG1 
            annotation (extent=[-80,-10; -60,10]);
        public 
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b n1 
            "First negative thermal connector" 
            annotation (extent=[100,40; 120,60]);
        protected 
          BondLib.Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
            annotation (extent=[60,40; 80,60]);
        public 
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b n2 
            "Second negative thermal connector" 
            annotation (extent=[100,-10; 120,10]);
        protected 
          BondLib.Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat2 
            annotation (extent=[60,-10; 80,10]);
        equation 
          connect(Sw1.control,control)  annotation (points=[0,20; 0,100], style(
                color=5, rgbcolor={255,0,255}));
          connect(B1.BondCon2,Sw1. p) annotation (points=[-20,0; -20,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(Sw1.n2,B2. BondCon1) annotation (points=[20,0; 20.2,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(Sw1.n1,B3. BondCon1) annotation (points=[20,10; 20.2,10],
              style(color=8, rgbcolor={192,192,192}));
          annotation (
            Diagram,
            Icon(
              Ellipse(extent=[-44,4; -36,-4], style(color=42, rgbcolor={213,0,0})),
              Text(extent=[-80,0; -60,22],   string="p",
                style(color=42, rgbcolor={213,0,0})),
              Text(extent=[60,50; 80,72],   string="n1",
                style(color=42, rgbcolor={213,0,0})),
              Text(extent=[60,0; 80,22],   string="n2",
                style(color=42, rgbcolor={213,0,0})),
              Line(points=[-100,0; -44,0], style(color=42, rgbcolor={213,0,0})),
              Line(points=[-37,2; 40,50], style(color=42, rgbcolor={213,0,0})),
              Line(points=[40,50; 100,50], style(color=42, rgbcolor={213,0,0})),
              Line(points=[40,0; 100,0], style(color=42, rgbcolor={213,0,0})),
              Text(extent=[-100,-70; 100,-100],   string="%name"),
              Line(points=[0,25; 0,80], style(color=5, rgbcolor={255,0,255}))),
            Documentation(info="<html>
This is a special heating switch used to check whether anything is connected from the outside to the connector n2.
If this is the case, the switch should be in the p-n2 position.
If this is not the case, the switch should be in the p-n1 position.
</html>"));
          connect(Heat2BG1.port_a, p) annotation (points=[-80,0; -110,0], style(
                color=42, rgbcolor={191,0,0}));
          connect(Heat2BG1.BondCon1, B1.BondCon1) annotation (points=[-60,0;
                -39.8,0], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, n1) annotation (points=[80,50; 110,50],
              style(color=42, rgbcolor={191,0,0}));
          connect(BG2Heat1.BondCon1, B3.BondCon2) annotation (points=[60,50; 50,
                50; 50,10; 40,10], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat2.port_b, n2) annotation (points=[80,0; 110,0], style(
                color=42, rgbcolor={191,0,0}));
          connect(BG2Heat2.BondCon1, B2.BondCon2) annotation (points=[60,0; 40,
                0], style(color=8, rgbcolor={192,192,192}));
        end HeatBranch;
      end Switches;
      
      package Utilities "Auxiliary models of the heat transfer library" 
        annotation (
        preferedView="info",
        Icon(
            Rectangle(extent=[-100,-100; 80,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=42, rgbcolor={127,0,0})),
            Text(
              extent=[-120,122; 120,73],
              string="%name",
              style(color=1))), Documentation(info="<html>
Auxiliary models of the heat transfer library.
</html>"));
        block C2K "Conversion block from Celsius to Kelvin" 
          Modelica.Blocks.Interfaces.RealInput Celsius(
             redeclare type SignalType = 
                Modelica.SIunits.Conversions.NonSIunits.Temperature_degC) 
            "Absolute temperature [degC]" 
             annotation (extent=[-140,-20; -100,20]);
          Modelica.Blocks.Interfaces.RealOutput Kelvin(
             redeclare type SignalType = Modelica.SIunits.Temperature) 
            "Absolute temperature [K]" 
            annotation (extent=[100,-10; 120,10]);
          annotation (
            Icon(
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[112,-40; 32,-120],
                string="K",
                style(color=0)),
              Text(
                extent=[-31,-39; -111,-119],
                string="C",
                style(color=0, thickness=4)),
              Line(points=[-41,0; -100,0]),
              Line(points=[100,0; 40,0]),
              Text(extent=[-137,99; 132,49],   string="%name")),
            Diagram(
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[-40,-50; -99,-99],
                string="C",
                style(color=0, thickness=4)),
              Text(
                extent=[100,-47; 44,-100],
                string="K",
                style(color=0)),
              Line(points=[-100,0; -40,0]),
              Line(points=[41,0; 100,0])),
            Documentation(info="<html>
<p>
This component converts an input signal from Celsius to Kelvin
and provide is as output signal.
</p>
</html>"));
        equation 
          Kelvin = Modelica.SIunits.Conversions.from_degC(Celsius);
        end C2K;
        
        block K2C "Conversion from Kelvin to Celsius" 
          
          Modelica.Blocks.Interfaces.RealInput Kelvin(
             redeclare type SignalType = Modelica.SIunits.Temperature) 
            "Absolute temperature [K]" 
            annotation (extent=[-140,-20; -100,20]);
          Modelica.Blocks.Interfaces.RealOutput Celsius(
             redeclare type SignalType = 
                Modelica.SIunits.Conversions.NonSIunits.Temperature_degC) 
            "Absolute temperature [degC]" 
            annotation (extent=[100,-10; 120,10]);
          annotation (
            Icon(
              Text(extent=[-137,99; 132,49],   string="%name"),
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[-34,-42; -114,-122],
                string="K",
                style(color=0, thickness=4)),
              Text(
                extent=[110,-39; 30,-119],
                string="C",
                style(color=0)),
              Line(points=[-40,0; -100,0]),
              Line(points=[40,0; 100,0])),
            Diagram(
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[-42,-41; -101,-98],
                string="K",
                style(color=0, thickness=4)),
              Text(
                extent=[100,-40; 30,-100],
                string="C",
                style(color=0)),
              Line(points=[-100,0; -40,0]),
              Line(points=[40,0; 100,0])),
            Documentation(info="<html>
<p>
This component converts an input signal from Kelvin to Celsius
and provides is as output signal.
</p>
</html>"));
        equation 
          Celsius = Modelica.SIunits.Conversions.to_degC(Kelvin);
        end K2C;
        
        block F2K "Conversion block from Fahrenheit to Kelvin" 
          
          Modelica.Blocks.Interfaces.RealInput Fahrenheit(
             redeclare type SignalType = 
                Modelica.SIunits.Conversions.NonSIunits.Temperature_degF) 
            "Absolute temperature [degF]"                                 annotation (
              extent=[-140,-20; -100,20]);
          Modelica.Blocks.Interfaces.RealOutput Kelvin(
             redeclare type SignalType = Modelica.SIunits.Temperature) 
            "Absolute temperature [K]" 
              annotation (extent=[100,-10; 120,10]);
          annotation (
            Icon(
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[112,-40; 32,-120],
                string="K",
                style(color=0)),
              Text(
                extent=[-31,-39; -111,-119],
                string="F",
                style(color=0, thickness=4)),
              Line(points=[-41,0; -100,0]),
              Line(points=[100,0; 40,0]),
              Text(extent=[-137,99; 132,49],   string="%name")),
            Diagram(
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[-40,-50; -99,-99],
                string="F",
                style(color=0, thickness=4)),
              Text(
                extent=[100,-47; 44,-100],
                string="K",
                style(color=0)),
              Line(points=[-100,0; -40,0]),
              Line(points=[41,0; 100,0])),
            Documentation(info="<html>
<p>
This component converts a input signal from degree Fahrenheit to Kelvin
and provides is as output signal.
</p>
</html>"));
        equation 
          Kelvin = Modelica.SIunits.Conversions.from_degF(Fahrenheit);
        end F2K;
        
        block K2F "Conversion from Kelvin to Fahrenheit" 
          
          Modelica.Blocks.Interfaces.RealInput Kelvin(
             redeclare type SignalType = Modelica.SIunits.Temperature) 
            "Absolute temperature [K]" 
            annotation (extent=[-140,-20; -100,20]);
          Modelica.Blocks.Interfaces.RealOutput Fahrenheit(
             redeclare type SignalType = 
                Modelica.SIunits.Conversions.NonSIunits.Temperature_degF) 
            "Absolute temperature [degF]" 
         annotation (
              extent=[100,-10; 120,10]);
          annotation (
            Icon(
              Text(extent=[-137,99; 132,49],   string="%name"),
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[-34,-42; -114,-122],
                string="K",
                style(color=0, thickness=4)),
              Text(
                extent=[110,-39; 30,-119],
                string="F",
                style(color=0)),
              Line(points=[-40,0; -100,0]),
              Line(points=[40,0; 100,0])),
            Diagram(
              Ellipse(extent=[-40,40; 40,-40],   style(
                  color=0,
                  thickness=2,
                  fillColor=7)),
              Text(
                extent=[-42,-41; -101,-98],
                string="K",
                style(color=0, thickness=4)),
              Text(
                extent=[100,-40; 30,-100],
                string="F",
                style(color=0)),
              Line(points=[-100,0; -40,0]),
              Line(points=[40,0; 100,0])),
            Documentation(info="<html>
<p>
This component converts all input signals from Kelvin to Fahrenheit
and provides them as output signals.
</p>
</html>"));
        equation 
          Fahrenheit = Modelica.SIunits.Conversions.to_degF(Kelvin);
        end K2F;
      end Utilities;
      
      package Examples "Examples of using the heat transfer library" 
        annotation (
        preferedView="info",
        Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85,71; 65,-49],
              string="Sample",
              style(color=0)),
            Text(
              extent=[-118,114; 122,65],
              string="%name",
              style(color=1)),
            Text(
              extent=[-97,23; 78,-98],
              string="Programs",
              style(color=0))), Documentation(info="<html>
Examples of using the heat transfer library.
</html>"));
        model TwoMasses "Simple conduction demo" 
          parameter Modelica.SIunits.Temperature T_final_K(fixed=false) 
            "Projected final temperature";
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degC 
            T_final_degC(                                                                 fixed=false) 
            "Projected final temperature";
          
          annotation (
            Icon(
              Rectangle(extent=[-100,99; 100,-101],   style(
                  color=0,
                  thickness=2,
                  fillColor=9)),
              Text(
                extent=[-66,46; 70,20],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="Two bodies"),
              Text(
                extent=[-74,28; 82,-14],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="are exchanging"),
              Text(
                extent=[-70,-6; 70,-32],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="heat")),
            Diagram,
            Documentation(info="<html>
<p>
This example demonstrates the thermal response of two masses connected by
a conducting element. The two masses have the same heat capacity but different
initial temperatures (T1=100 [degC], T2= 0 [degC]). The mass with the higher
temperature will cool off while the mass with the lower temperature heats up.
They will each asymptotically approach the calculated temperature <b>T_final_K</b>
(<b>T_final_degC</b>) that results from dividing the total initial energy in the system by the sum
of the heat capacities of each element.
</p>
<p>
Simulate for 5 s and plot the variables<br>
mass1.T, mass2.T, T_final_K or <br>
Tsensor1.T.signal, Tsensor2.T.signal, T_final_degC
</p>
<font color=red><b><p>
This example was copied from an equivalent example of the
corresponding sub-library of the Modelica standard library.
However, the simulation results obtained are slightly different.
</p>
<p>
The reason is that the conductance model used by the standard
library operates on a real resistor.  Yet, this is not a
realistic assumption.  A resistor heats up.  It generates
additional entropy.  This entropy needs to be routed back into
the thermal circuit.
</p>
<p>
In an electrical circuit or in a mechanical system, it may make
sense to ignore the heat generated by the resistors.  However
here, we operate already in the thermal domain.  Hence ignoring
the heat generated by the resistors makes no sense
whatsoever.
</p></b></font>
</html>"),  experiment(StopTime=5),
            experimentSetupOutput);
          Passive.ThermalConductor cunduction(G=10) 
            annotation (extent=[-20,-20; 20,20]);
          Passive.HeatCapacitor mass1(C=15, T(start=
                  Modelica.SIunits.Conversions.from_degC(100.0))) 
            annotation (extent=[-80,20; -40,60]);
          Passive.HeatCapacitor mass2(C=15, T(start=
                  Modelica.SIunits.Conversions.from_degC(0.0))) 
            annotation (extent=[40,20; 80,60]);
          Sensors.Celsius.TemperatureSensor Tsensor1 
            annotation (extent=[-50,-80; -10,-40]);
          Sensors.Celsius.TemperatureSensor Tsensor2 
            annotation (extent=[50,-80; 10,-40]);
        equation 
          connect(cunduction.port_a, mass1.port) annotation (points=[-22,0; -60,
                0; -60,20], style(color=42, rgbcolor={191,0,0}));
          connect(Tsensor1.port, mass1.port) annotation (points=[-50,-60; -60,
                -60; -60,20], style(color=42, rgbcolor={191,0,0}));
          connect(mass2.port, cunduction.port_b) annotation (points=[60,20; 60,
                0; 22,0], style(color=42, rgbcolor={191,0,0}));
          connect(mass2.port, Tsensor2.port) annotation (points=[60,20; 60,-60;
                50,-60], style(color=42, rgbcolor={191,0,0}));
        initial equation 
          T_final_K = (mass1.port.T*mass1.C + mass2.port.T*mass2.C)/(mass1.C +
            mass2.C);
          T_final_degC = Modelica.SIunits.Conversions.to_degC(T_final_K);
        end TwoMasses;
        
        model ControlledTemperature "Control temperature of a resistor" 
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degC 
            TAmb =                                                                20 
            "Ambient Temperature";
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degC 
            TDif =                                                                2 
            "Error in Temperature";
          output Modelica.SIunits.Conversions.NonSIunits.Temperature_degC TRes = Modelica.SIunits.Conversions.to_degC(HeatingResistor1.heatPort.T) 
            "Resulting Temperature";
          Modelica.Blocks.Sources.Ramp Ramp1(
            height=25,
            duration=6,
            offset=25,
            startTime=2) annotation (extent=[40,0; 20,20]);
          Modelica.Blocks.Logical.OnOffController OnOffController1(bandwidth=TDif) 
            annotation (extent=[0,-20; -20,0]);
          Modelica.Blocks.Logical.Not Not1 annotation (extent=[-30,-20; -50,0]);
          Electrical.Analog.Basic.Ground ground 
            annotation (extent=[-100,-100; -80,-80]);
          Electrical.Analog.Sources.ConstantVoltage ConstantVoltage1(V=10) 
            annotation (extent=[-100,-60; -80,-40], rotation=-90);
          Electrical.Analog.Ideal.IdealOpeningSwitch IdealSwitch1 
            annotation (extent=[-70,-50; -50,-30]);
          Electrical.Analog.Basic.HeatingResistor HeatingResistor1(
            R_ref=10,
            T_ref=Modelica.SIunits.Conversions.from_degC(20),
            alpha=1/(235 + 20)) 
            annotation (extent=[-40,-40; -20,-60], rotation=90);
          Sensors.Celsius.TemperatureSensor TemperatureSensor1 
            annotation (extent=[0,-40; 20,-20], rotation=90);
          Passive.HeatCapacitor HeatCapacitor1(C=1, T(start=
                  Modelica.SIunits.Conversions.from_degC(TAmb))) 
            annotation (extent=[0,-60; 20,-80]);
          Passive.ThermalConductor ThermalConductor1(G=0.1) 
            annotation (extent=[40,-60; 60,-40]);
          Sources.Celsius.FixedTemperature FixedTemperature1(T=TAmb) 
            annotation (extent=[100,-60; 80,-40]);
        equation 
          
          connect(Ramp1.y,OnOffController1. reference) annotation (points=[19,10;
                10,10; 10,-4; 2,-4], style(color=74, rgbcolor={0,0,127}));
          annotation (
            Diagram(Ellipse(extent=[8,-48; 12,-52], style(
                  color=42,
                  rgbcolor={209,0,0},
                  fillColor=42,
                  rgbfillColor={207,0,0}))),
            Documentation(info="<html>
<P>
A constant voltage of 10 V is applied to a
temperature dependent resistor of 10*(1+(T-20C)/(235+20C)) Ohms,
whose losses v**2/r are dissipated via a
thermal conductance of 0.1 W/K to ambient temperature 20 degree C.
The resistor is assumed to have a thermal capacity of 1 J/K,
having ambient temparature at the beginning of the experiment.
The temperature of this heating resistor is held by an OnOff-controller
at reference temperature within a given bandwith +/- 1 K
by switching on and off the voltage source.
The reference temperature starts at 25 degree C
and rises between t = 2 and 8 seconds linear to 50 degree C.
An approppriate simulating time would be 10 seconds.
</P>
</html>"),  Icon(
              Rectangle(extent=[-100,100; 100,-100],  style(
                  color=0,
                  thickness=2,
                  fillColor=9)),
              Text(
                extent=[-64,47; 62,22],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="Temperature"),
              Text(
                extent=[-56,27; 54,-10],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="control of an"),
              Text(
                extent=[-78,-5; 78,-34],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="electrical resistor")),
            experiment(StopTime=10),
            experimentSetupOutput);
          connect(ConstantVoltage1.n, ground.p) annotation (points=[-90,-60;
                -90,-80], style(color=3, rgbcolor={0,0,255}));
          connect(IdealSwitch1.p, ConstantVoltage1.p) annotation (points=[-70,
                -40; -90,-40], style(color=3, rgbcolor={0,0,255}));
          connect(HeatingResistor1.p, IdealSwitch1.n) annotation (points=[-30,
                -40; -50,-40], style(color=3, rgbcolor={0,0,255}));
          connect(HeatingResistor1.n, ConstantVoltage1.n) annotation (points=[
                -30,-60; -90,-60], style(color=3, rgbcolor={0,0,255}));
          connect(OnOffController1.y, Not1.u) annotation (points=[-21,-10; -28,
                -10], style(color=5, rgbcolor={255,0,255}));
          connect(Not1.y, IdealSwitch1.control) annotation (points=[-51,-10;
                -60,-10; -60,-31], style(color=5, rgbcolor={255,0,255}));
          connect(TemperatureSensor1.T, OnOffController1.u) annotation (points=
                [10,-20; 10,-16; 2,-16], style(color=74, rgbcolor={0,0,127}));
          connect(TemperatureSensor1.port, HeatCapacitor1.port) annotation (
              points=[10,-40; 10,-60], style(color=42, rgbcolor={191,0,0}));
          connect(TemperatureSensor1.port, HeatingResistor1.heatPort) 
            annotation (points=[10,-40; 10,-50; -20,-50], style(color=42,
                rgbcolor={191,0,0}));
          connect(TemperatureSensor1.port, ThermalConductor1.port_a) 
            annotation (points=[10,-40; 10,-50; 39,-50], style(color=42,
                rgbcolor={191,0,0}));
          connect(FixedTemperature1.port, ThermalConductor1.port_b) annotation (
             points=[79,-50; 61,-50], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
        end ControlledTemperature;
        
        model Motor "Second order thermal model of a motor" 
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degC 
            TAmb =                                                                  20 
            "Ambient temperature";
          Modelica.Blocks.Sources.CombiTimeTable lossTable(extrapolation=Modelica.
                Blocks.Types.Extrapolation.Periodic, table=[0,100,500; 360,100,500;
                360,1000,500; 600,1000,500]) 
                                    annotation (extent=[-50,60; -30,80], rotation=
                -90);
          Modelica.Blocks.Sources.Constant convectionConstant(k=25) 
            annotation (extent=[30,20; 50,40], rotation=-90);
          Passive.Convection convection annotation (extent=[30,-20; 50,0]);
          Sources.Celsius.FixedTemperature environment(T=TAmb) 
            annotation (extent=[90,-20; 70,0]);
          Passive.ThermalConductor winding2core(G=10) 
            annotation (extent=[-50,-20; -30,0]);
          Passive.HeatCapacitor core(T(start=
                  Modelica.SIunits.Conversions.from_degC(TAmb)), C=25000) 
            annotation (extent=[-10,-20; 10,-40]);
          Passive.HeatCapacitor winding(T(start=
                  Modelica.SIunits.Conversions.from_degC(TAmb)), C=2500) 
            annotation (extent=[-90,-20; -70,-40]);
          Sensors.Celsius.TemperatureSensor Tcore 
            annotation (extent=[-30,-60; -10,-40], rotation=270);
          Sensors.Celsius.TemperatureSensor Twinding 
            annotation (extent=[-70,-60; -50,-40], rotation=270);
          Sources.PrescribedHeatFlow coreLosses 
            annotation (extent=[-10,0; 10,20], rotation=-90);
          Sources.PrescribedHeatFlow windingLosses(T_ref=
                Modelica.SIunits.Conversions.from_degC(95), alpha=3.03E-3) 
            annotation (extent=[-90,0; -70,20], rotation=-90);
        equation 
          
          annotation (
            Diagram(Ellipse(extent=[-2,-8; 2,-12], style(
                  color=42,
                  rgbcolor={209,0,0},
                  fillColor=42,
                  rgbfillColor={207,0,0},
                  fillPattern=1))),
            Documentation(info="<html>
<p>
This example contains a simple second order thermal model of a motor. 
The periodic power losses are described by table \"lossTable\":<br>
<table>
<tr><td>time</td><td>winding losses</td><td>core losses</td></tr>
<tr><td>   0</td><td>           100</td><td>        500</td></tr>
<tr><td> 360</td><td>           100</td><td>        500</td></tr>
<tr><td> 360</td><td>          1000</td><td>        500</td></tr>
<tr><td> 600</td><td>          1000</td><td>        500</td></tr>
</table><br>
Since constant speed is assumed, the core losses keep constant 
whereas the winding losses are low for 6 minutes (no-load) and high for 4 minutes (over load).
<br>
The winding losses are corrected by (1 + alpha*(T - T_ref)) because the winding's resistance is temperature dependent whereas the core losses are kept constant (alpha = 0).
</p>
<p>
The power dissipation to the environment is approximated by heat flow through 
a thermal conductance between winding and core, 
partially storage of the heat in the winding's heat capacity 
as well as the core's heat capacity and finally by forced convection to the environment.<br>
Since constant speed is assumed, the cinvective conductance keeps constant.<br>
Using Modelica.Thermal.FluidHeatFlow it would be possible to model the coolant air flow, too 
(instead of simple dissipation to a constant ambient's temperature).
</p>
<p>
Simulate for 7200 s; plot Twinding.T and Tcore.T.
</p>
<font color=red><b><p>
This example was copied from an equivalent example of the
corresponding sub-library of the Modelica standard library.
However, the simulation results obtained are slightly different.
</p>
<p>
The reason is that the conductance and convection models used by
the standard library operate on real resistors.  Yet, this is
not a realistic assumption.  Resistors heat up.  They generate
additional entropy.  This entropy needs to be routed back into
the thermal circuit.
</p>
<p>
In an electrical circuit or in a mechanical system, it may make
sense to ignore the heat generated by the resistors.  However
here, we operate already in the thermal domain.  Hence ignoring
the heat generated by the resistors makes no sense
whatsoever.
</p></b></font>
</html>"),  Icon(
              Rectangle(extent=[-100,100; 100,-100],  style(
                  color=0,
                  thickness=2,
                  fillColor=9)),
              Text(
                extent=[-64,47; 68,22],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="Thermal model"),
              Text(
                extent=[-50,23; 46,-4],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="of a motor")),
            experiment(StopTime=7200),
            experimentSetupOutput);
          connect(convectionConstant.y, convection.Gc) annotation (points=[40,
                19; 40,0], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(environment.port, convection.fluid) annotation (points=[69,
                -10; 51,-10], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(winding2core.port_b, convection.solid) annotation (points=[
                -29,-10; 29,-10], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(core.port, winding2core.port_b) annotation (points=[0,-20; 0,
                -10; -29,-10], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(winding.port, winding2core.port_a) annotation (points=[-80,
                -20; -80,-10; -51,-10], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(winding2core.port_a, Twinding.port) annotation (points=[-51,
                -10; -60,-10; -60,-40], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(Tcore.port, winding2core.port_b) annotation (points=[-20,-40;
                -20,-10; -29,-10], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(coreLosses.port, winding2core.port_b) annotation (points=[
                6.12303e-016,0; 6.12303e-016,-10; -29,-10], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(windingLosses.port, winding.port) annotation (points=[-80,0;
                -80,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(lossTable.y[1], windingLosses.Q_flow) annotation (points=[-40,
                59; -40,40; -80,40; -80,20], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
          connect(lossTable.y[2], coreLosses.Q_flow) annotation (points=[-40,59;
                -40,40; -6.12303e-016,40; -6.12303e-016,20], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=42,
              rgbfillColor={207,0,0},
              fillPattern=1));
        end Motor;
      end Examples;
    end HeatTransfer;
    
    model RS "The bondgraphic linear resistive source element" 
      extends Interfaces.TwoPort;
      parameter Real R=1 "Bondgraphic Resistance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-74, 80; 74, -80],
            string="RS",
            style(color=0))),
        Icon(
          Text(
            extent=[-72, 78; 76, -82],
            string="RS",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-78, -48; 80, -88], string="R=%R")),
        Documentation(info="<html>
The bondgraphic linear resistive source is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The resistance is modeled as a parameter. <p>

The linear resistive source element is to be used as a replacement for resistors in modeling the thermal behavior of non-thermal systems.  The primary side of the linear resistive source element is assumed to not be in the thermal domain. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 R:   Resistance (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e1 = R*f1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      e1 = R*f1;
      e1*f1 = e2*f2;
    end RS;
    
    model mRS "The modulated bondgraphic resistive source element" 
      extends Interfaces.ModTwoPort;
      Real R "Bondgraphic Resistance";
      annotation (
        Diagram(Text(
            extent=[-84,80; 84,-82],
            style(color=0),
            string="mRS")),
        Icon(
          Text(extent=[-70,100; 70,60],  string="%name"),
                Text(
            extent=[-84,80; 84,-82],
            style(color=0),
            string="mRS")),
        Documentation(info="<html>
The bondgraphic modulated resistive source is a directed modulated TwoPort element.  It inherits the effort and flow variables from the directed modulated TwoPort.  The resistance is modeled as a real-valued variable. <p>

The modulated resistive source element is to be used as a replacement for resistors in modeling the thermal behavior of non-thermal systems.  The primary side of the modulated resistive source element is assumed to not be in the thermal domain. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 R:   Resistance value (modulation input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>e1 = R*f1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      R = s;
      e1 = R*f1;
      e1*f1 = e2*f2;
    end mRS;
    
    model RSth "The bondgraphic linear conductive resistive source element" 
      extends Interfaces.TwoPort;
      parameter Modelica.SIunits.ThermalResistance theta=1 "Thermal resistance";
      Real R(unit="K2/W") "Bondgraphic thermal resistance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-74, 80; 36, -82],
            string="RS",
            style(color=0)), Text(
            extent=[34, 42; 78, -60],
            string="th",
            style(color=0))),
        Icon(
          Text(
            extent=[34, 40; 78, -62],
            string="th",
            style(color=0)),
          Text(
            extent=[-74, 80; 36, -82],
            string="RS",
            style(color=0)),
          Text(extent=[-78, -48; 80, -88], string="theta=%theta"),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Documentation(info="<html>
The bondgraphic linear conductive resistive source is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The thermal resistance is modeled as a parameter. <p>

The resistance, <b>R</b>, is proportional to the temperature at the inflow, <b>T1</b>.  Since <b>e1</b> represents the temperature difference, <b>e1 = T1 - T2</b>, and since <b>e2</b> represents the temperature at the outflow, <b>e2 = T2</b>, the temperature at the inflow can be computed as: <b>T1 = e1 + e2</b>. <p>

The linear conductive resistive source element is to be used in linear conduction of thermal systems.  The primary side of the linear resistive source element is assumed to be in the thermal domain. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:      Bondgraphic effort variable of inflow

 f1:      Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:      Bondgraphic effort variable of outflow

 f2:      Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 R:       Bondgraphic thermal resistance


Parameters:

 theta:   Thermal resistance constant (default value = 1 <sup>o</sup>K/W)
</pre> <p>

<hr> <p>

Equations: <p>

<b>R = theta*(e1+e2)</b> <br>
<b>e1 = R*f1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      R = theta*(e1 + e2);
      e1 = R*f1;
      e1*f1 = e2*f2;
    end RSth;
    
    model mRSth "The bondgraphic modulated resistive source element" 
      extends Interfaces.ModTwoPort;
      Modelica.SIunits.ThermalResistance R0 "Thermal resistance";
      Real R(unit="K2/W") "Bondgraphic thermal resistance";
      annotation (
        Icon(
          Text(extent=[-60,90; 80,50],   string="%name"),
          Text(
            extent=[-80,86; 58,-88],
            style(color=0),
            string="mRS"),
          Text(
            extent=[56,30; 90,-64],
            string="th",
            style(color=0))),
        Diagram(
          Text(
            extent=[-80,86; 58,-88],
            style(color=0),
            string="mRS"),
          Text(
            extent=[56,30; 90,-64],
            string="th",
            style(color=0))),
        Documentation(info="<html>
The bondgraphic modulated resistive source is a directed modulated TwoPort element.  It inherits the effort and flow variables from the directed modulated TwoPort.  The thermal resistance is modeled as a real-valued variable. <p>

The resistance, <b>R</b>, is proportional to the temperature at the inflow, <b>T1</b>.  Since <b>e1</b> represents the temperature difference, <b>e1 = T1 - T2</b>, and since <b>e2</b> represents the temperature at the outflow, <b>e2 = T2</b>, the temperature at the inflow can be computed as: <b>T1 = e1 + e2</b>. <p>

The modulated resistive source element is to be used in conduction of thermal systems.  The primary side of the modulated conductive source element is assumed to be in the thermal domain. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:   Bondgraphic effort variable of inflow

 f1:   Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:   Bondgraphic effort variable of outflow

 f2:   Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 R:    Bondgraphic thermal resistance

 R0:   Thermal resistance constant (modulation input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>R = R0*(e1+e2)</b> <br>
<b>e1 = R*f1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      R0 = s;
      R = R0*(e1 + e2);
      e1 = R*f1;
      e1*f1 = e2*f2;
    end mRSth;
    
    model RSrad "The bondgraphic radiative resistive source element" 
      extends Interfaces.TwoPort;
      constant Real sigma(unit="W/(K4.m2)") = Modelica.Constants.sigma 
        "Stefan-Boltzmann constant";
      parameter Modelica.SIunits.Area A=1 "Radiating Surface";
      Real R(unit="K2/W") "Bondgraphic Radiative Resistance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-74, 80; 18, -82],
            string="RS",
            style(color=0)), Text(
            extent=[20, 42; 80, -74],
            string="rad",
            style(color=0))),
        Icon(
          Text(
            extent=[20, 42; 80, -74],
            string="rad",
            style(color=0)),
          Text(
            extent=[-74, 80; 18, -82],
            string="RS",
            style(color=0)),
          Text(extent=[-78,-52; 80,-92], string="A=%A"),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Documentation(info="<html>
The bondgraphic radiative resistive source is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The specific radiation constant and the rediating surface are modeled as parameters. <p>

The resistance, <b>R</b>, is inverse proportional to the square of the temperature at the inflow, <b>T1 = e1</b>. <p>

The radiative resistive source element is to be used in radiation of thermal systems.  The primary side of the radiative resistive source element is assumed to be in the thermal domain. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:      Bondgraphic effort variable of inflow

 f1:      Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:      Bondgraphic effort variable of outflow

 f2:      Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 R:       Bondgraphic radiative resistance


Parameters:

 sigma:   Specific radiation constant (default value = 1 W/(m<sup>2</sup>*<sup>o</sup>K<sup>4</sup>)

 A:       Radiating surface (default value = 1 m<sup>2</sup>)
</pre> <p>

<hr> <p>

Equations: <p>

<b>R = 1/(sigma*A*e1*e1)</b> <br>
<b>e1 = R*f1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      R = 1/(A*sigma*(e1^2));
      e1 = R*f1;
      e1*f1 = e2*f2;
    end RSrad;
    
    model GS "The bondgraphic linear conductive source element" 
      extends Interfaces.TwoPort;
      parameter Real G=1 "Bondgraphic Cunductance";
      annotation (
        Icon(
          Text(extent=[-64,90; 76,50],   string="%name"),
          Text(
            extent=[-66,78; 82,-82],
            style(color=0),
            string="GS"),
          Text(extent=[-72,-48; 86,-88], string="G=%G")),
        Diagram(
          Text(
            extent=[-66,78; 82,-82],
            style(color=0),
            string="GS")),
        Documentation(info="<html>
The bondgraphic linear conductive source is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The conductance is modeled as a parameter. <p>

The linear conductive source element is to be used as a replacement for conductors in modeling the thermal behavior of non-thermal systems.  The primary side of the linear conductive source element is assumed to not be in the thermal domain. <p>

The conductive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 G:   Conductance (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f1 = G*e1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      f1 = G*e1;
      e1*f1 = e2*f2;
    end GS;
    
    model mGS "The modulated bondgraphic conductive source element" 
      extends Interfaces.ModTwoPort;
      Real G "Bondgraphic Conductance";
      annotation (
        Icon(   Text(
            extent=[-82,80; 86,-82],
            style(color=0),
            string="mGS"),
          Text(extent=[-68,100; 72,60],  string="%name")),
        Diagram(Text(
            extent=[-82,80; 86,-82],
            style(color=0),
            string="mGS")),
        Documentation(info="<html>
The bondgraphic modulated conductive source is a directed modulated TwoPort element.  It inherits the effort and flow variables from the modulated directed TwoPort.  The conductance is modeled as a real-valued variable. <p>

The modulated conductive source element is to be used as a replacement for  modulated conductors in modeling the thermal behavior of non-thermal systems.  The primary side of the modulated conductive source element is assumed to not be in the thermal domain. <p>

The conductive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:  Bondgraphic effort variable of inflow

 f1:  Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:  Bondgraphic effort variable of outflow

 f2:  Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 G:   Conductance value (modulation input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>f1 = G*e1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      G = s;
      f1 = G*e1;
      e1*f1 = e2*f2;
    end mGS;
    
    model GSth "The bondgraphic linear conductive source element" 
      extends Interfaces.TwoPort;
      parameter Modelica.SIunits.ThermalConductance G0=1 "Thermal conductance";
      Real G(unit="W/K2") "Bondgraphic thermal conductance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-74, 80; 36, -82],
            style(color=0),
            string="GS"), Text(
            extent=[34, 42; 78, -60],
            string="th",
            style(color=0))),
        Icon(
          Text(
            extent=[34, 40; 78, -62],
            string="th",
            style(color=0)),
          Text(
            extent=[-74, 80; 36, -82],
            style(color=0),
            string="GS"),
          Text(extent=[-78, -48; 80, -88], string="G0=%G0"),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Documentation(info="<html>
The bondgraphic linear conductive source is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The thermal conductance is modeled as a parameter. <p>

The conductance, <b>G</b>, is inverse proportional to the temperature at the inflow, <b>T1</b>.  Since <b>e1</b> represents the temperature difference, <b>e1 = T1 - T2</b>, and since <b>e2</b> represents the temperature at the outflow, <b>e2 = T2</b>, the temperature at the inflow can be computed as: <b>T1 = e1 + e2</b>. <p>

The linear conductive source element is to be used in linear conduction of thermal systems.  The primary side of the linear conductive source element is assumed to be in the thermal domain. <p>

The conductive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:   Bondgraphic effort variable of inflow

 f1:   Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:   Bondgraphic effort variable of outflow

 f2:   Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 G:    Bondgraphic thermal conductance


Parameters:

 G0:   Thermal conductance constant (default value = 1 W/<sup>o</sup>K)
</pre> <p>

<hr> <p>

Equations: <p>

<b>G = G0/(e1+e2)</b> <br>
<b>f1 = G*e1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      G = G0/(e1 + e2);
      f1 = G*e1;
      e1*f1 = e2*f2;
    end GSth;
    
    model mGSth "The bondgraphic modulated conductive source element" 
      extends Interfaces.ModTwoPort;
      Modelica.SIunits.ThermalConductance G0 "Thermal conductance";
      Real G(unit="W/K2") "Bondgraphic thermal conductance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-88, 86; 50, -88],
            style(color=0),
            string="mGS"), Text(
            extent=[52, 30; 86, -64],
            string="th",
            style(color=0))),
        Icon(
          Text(
            extent=[52, 30; 86, -64],
            string="th",
            style(color=0)),
          Text(
            extent=[-88, 86; 50, -88],
            style(color=0),
            string="mGS"),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Documentation(info="<html>
The bondgraphic modulated conductive source is a directed modulated TwoPort element.  It inherits the effort and flow variables from the directed modulated TwoPort.  The thermal conductance is modeled as a real-valued variable. <p>

The conductance, <b>G</b>, is inverse proportional to the temperature at the inflow, <b>T1</b>.  Since <b>e1</b> represents the temperature difference, <b>e1 = T1 - T2</b>, and since <b>e2</b> represents the temperature at the outflow, <b>e2 = T2</b>, the temperature at the inflow can be computed as: <b>T1 = e1 + e2</b>. <p>

The modulated conductive source element is to be used in conduction of thermal systems.  The primary side of the modulated conductive source element is assumed to be in the thermal domain. <p>

The conductive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:   Bondgraphic effort variable of inflow

 f1:   Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:   Bondgraphic effort variable of outflow

 f2:   Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 G:    Bondgraphic thermal conductance

 G0:   Thermal conductance constant (modulation input signal)
</pre> <p>

<hr> <p>

Equations: <p>

<b>G = G0/(e1+e2)</b> <br>
<b>f1 = G*e1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      G0 = s;
      G = G0/(e1 + e2);
      f1 = G*e1;
      e1*f1 = e2*f2;
    end mGSth;
    
    model GSrad "The bondgraphic radiative conductive source element" 
      extends Interfaces.TwoPort;
      constant Real sigma(unit="W/(K4.m2)") = Modelica.Constants.sigma 
        "Stefan-Boltzmann constant";
      parameter Modelica.SIunits.Area A=1 "Radiating surface";
      Real G(unit="W/K2") "Bondgraphic radiative conductance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[-80, 78; 16, -84],
            string="GS",
            style(color=0)), Text(
            extent=[18, 40; 74, -76],
            string="rad",
            style(color=0))),
        Icon(
          Text(
            extent=[18, 40; 74, -76],
            string="rad",
            style(color=0)),
          Text(
            extent=[-80, 78; 16, -84],
            string="GS",
            style(color=0)),
          Text(extent=[-78,-52; 80,-92], string="A=%A"),
          Text(extent=[-70, 90; 70, 50], string="%name")),
        Documentation(info="<html>
The bondgraphic radiative conductive source is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The specific radiation constant and the rediating surface are modeled as parameters. <p>

The conductance, <b>G</b>, is proportional to the square of the temperature at the inflow, <b>T1 = e1</b>. <p>

The radiative conductive source element is to be used in radiation of thermal systems.  The primary side of the radiative conductive source element is assumed to be in the thermal domain. <p>

The conductive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:      Bondgraphic effort variable of inflow

 f1:      Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:      Bondgraphic effort variable of outflow

 f2:      Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 G:       Bondgraphic radiative conductance


Parameters:

 sigma:   Specific radiation constant (default value = 1 W/(m<sup>2</sup>*<sup>o</sup>K<sup>4</sup>)

 A:       Radiating surface (default value = 1 m<sup>2</sup>)
</pre> <p>

<hr> <p>

Equations: <p>

<b>G = sigma*A*e1*e1</b> <br>
<b>f1 = G*e1</b> <br>
<b>f2 = f1*(e1/e2)</b>
</html>
"));
    equation 
      G = A*sigma*(e1^2);
      f1 = G*e1;
      e1*f1 = e2*f2;
    end GSrad;
    
    model Cth "The bondgraphic thermal capacitor element" 
      extends Interfaces.PassiveOnePort;
      parameter Modelica.SIunits.HeatCapacity gamma=1 "Heat capacity";
      Real C(unit="J/K2") "Bondgraphic heat capacity";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Text(
            extent=[26, 38; 90, -80],
            string="th",
            style(color=0)), Text(
            extent=[-66, 74; 30, -78],
            string="C",
            style(color=0))),
        Icon(
          Text(
            extent=[26, 38; 90, -80],
            string="th",
            style(color=0)),
          Text(
            extent=[-66, 74; 30, -78],
            string="C",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-66, -48; 72, -88], string="gamma=%gamma")),
        Documentation(info="<html>
The bondgraphic thermal capacitor is a passive OnePort element.  It inherits the effort and flow variables from the passive OnePort.  The thermal capacitance is modeled as a parameter. <p>

The capacitance, <b>C</b>, is inverse proportional to the temperature. <p>

The capacitor model has a preferred causality away from the element. <p>

<hr>

<pre>
Potential variables:

 e:  Bondgraphic effort variable

 f:  Bondgraphic flow variable, normalized positive for flows into the model

 C:  Bondgraphic thermal capacitance


Parameters:

 gamma:  Thermal capacitance (default value = 1 W*sec/<sup>o</sup>K)
</pre> <p>

<hr> <p>

Equations: <p>

<b>C = gamma/e</b> <br>
<b>f = C*der(e)</b>
</html>
"));
    equation 
      C = gamma/e;
      f = C*der(e);
    end Cth;
    
    model HE "Symmetric linear heat exchange element" 
      extends Interfaces.TwoPort;
      parameter Modelica.SIunits.ThermalResistance theta=1 "Thermal resistance";
      annotation (
        Coordsys(
          extent=[-100,-100; 100,100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram,
        Icon(
          Text(
            extent=[-86, 80; 62, -80],
            string="HE",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-78,-52; 80,-92],   string="theta=%theta"),
          Text(
            extent=[52, 14; 94, -46],
            string="s",
            style(color=0))),
        Documentation(info="<html>
The bondgraphic symmetric linear heat exchange element is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The thermal resistance is modeled as a parameter. <p>

The heat exchange is driven by the temperature difference between the two sides.  The generated entropy is divided evenly to the left and the right. <p>

The heat exchange element has two causality strokes at the element. <p>

<hr>

<pre>
Potential variables:

 e1:      Bondgraphic effort variable of inflow

 f1:      Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:      Bondgraphic effort variable of outflow

 f2:      Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 theta:   Thermal resistance (default value = 1 <sup>o</sup>K/W)
</pre>
</html>
"));
      Junctions.J0p3 J0p3_1 annotation (extent=[-70, 10; -50, -10]);
      Junctions.J0p3 J0p3_2 annotation (extent=[50, 10; 70, -10]);
      Junctions.J1p3 J1p3_1 annotation (extent=[-10, 10; 10, -10]);
      Bonds.fBond B3 annotation (extent=[-10, 10; 10, 30], rotation=90);
      Junctions.J0p3 J0p3_3 annotation (extent=[-10, 30; 10, 50]);
      Bonds.fBond B4 annotation (extent=[10, 30; 30, 50]);
      Bonds.fBond B5 annotation (extent=[-30, 30; -10, 50], rotation=180);
      RSth RSr(theta=2*theta) annotation (extent=[30, 30; 50, 50]);
      Bonds.eBond B2 annotation (extent=[20, -10; 40, 10]);
      Bonds.eBond B6 annotation (extent=[50, 10; 70, 30], rotation=-90);
      Bonds.fBond B1 annotation (extent=[-40, -10; -20, 10]);
      RSth RSl(theta=2*theta) annotation (extent=[-30, 30; -50, 50]);
      Bonds.eBond B7 annotation (extent=[-70, 10; -50, 30], rotation=-90);
    equation 
      connect(J0p3_2.BondCon2, BondCon2) annotation (points=[70,0; 100,0],
          style(color=8));
      connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,10;
            -6.12303e-016,10],
            style(color=8));
      connect(B3.fBondCon1, J0p3_3.BondCon3) annotation (points=[6.12303e-016,
            30; 0,30],
            style(color=8));
      connect(J0p3_3.BondCon2, B4.eBondCon1) annotation (points=[10, 40; 10, 40],
               style(color=8));
      connect(J0p3_3.BondCon1, B5.eBondCon1) annotation (points=[-10, 40; -10,
            40], style(color=8));
      connect(B4.fBondCon1, RSr.BondCon1) annotation (points=[30, 40; 30, 40],
          style(color=8));
      connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[10, 0; 20, 0],
            style(color=8));
      connect(B2.eBondCon1, J0p3_2.BondCon1) annotation (points=[40, 0; 50, 0],
            style(color=8));
      connect(B6.eBondCon1, J0p3_2.BondCon3) annotation (points=[60, 10; 60, 10],
               style(color=8));
      connect(RSr.BondCon2, B6.fBondCon1) annotation (points=[50, 40; 60, 40;
            60, 30], style(color=8));
      connect(J0p3_1.BondCon2, B1.eBondCon1) annotation (points=[-50, 0; -40, 0],
               style(color=8));
      connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-20, 0; -10, 0],
               style(color=8));
      connect(B5.fBondCon1, RSl.BondCon1) annotation (points=[-30, 40; -30, 40],
            style(color=8));
      connect(B7.eBondCon1, J0p3_1.BondCon3) annotation (points=[-60, 10; -60,
            10], style(color=8));
      connect(RSl.BondCon2, B7.fBondCon1) annotation (points=[-50, 40; -60, 40;
              -60, 30], style(color=8));
      connect(BondCon1, J0p3_1.BondCon1) annotation (points=[-100,0; -70,0],
          style(color=8, rgbcolor={192,192,192}));
    end HE;
    
    model HEr "Asymmetric linear heat exchange element" 
      extends Interfaces.TwoPort;
      parameter Modelica.SIunits.ThermalResistance theta=1 "Thermal resistance";
      annotation (
        Coordsys(
          extent=[-100,-100; 100,100],
          grid=[2, 2],
          component=[20, 20]),
        Icon(
          Text(
            extent=[-92, 78; 38, -82],
            string="HE",
            style(color=0)),
          Text(
            extent=[36, 18; 88, -48],
            string="as",
            style(color=0)),
          Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-78, -54; 80, -94], string="theta=%theta")),
        Documentation(info="<html>
The bondgraphic asymmetric linear heat exchange element is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The thermal resistance is modeled as a parameter. <p>

The heat exchange is driven by the temperature difference between the two sides.  The generated entropy is reintroduced at the right. <p>

The heat exchange element has two causality strokes at the element. <p>

<hr>

<pre>
Potential variables:

 e1:      Bondgraphic effort variable of inflow

 f1:      Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:      Bondgraphic effort variable of outflow

 f2:      Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 theta:   Thermal resistance (default value = 1 <sup>o</sup>K/W)
</pre>
</html>
"));
      Junctions.J0p2 J0p2_1 annotation (extent=[-50, -10; -30, 10]);
      Junctions.J1p3 J1p3_1 annotation (extent=[-10, 10; 10, -10]);
      Bonds.fBond B1 annotation (extent=[-30, -10; -10, 10]);
      Bonds.eBond B2 annotation (extent=[10, -10; 30, 10]);
      Junctions.J0p3 J0p3_2 annotation (extent=[30, 10; 50, -10]);
      Bonds.eBond B4 annotation (extent=[30, 10; 50, 30], rotation=-90);
      RSth RS(theta=theta) annotation (extent=[10, 30; 30, 50]);
      Bonds.fBond B3 annotation (extent=[-10, 10; 10, 30], rotation=90);
    equation 
      connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-10, 0; -10, 0],
               style(color=8));
      connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-30, 0; -30, 0],
               style(color=8));
      connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -50,0],
          style(color=8));
      connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[10, 0; 10, 0],
            style(color=8));
      connect(B2.eBondCon1, J0p3_2.BondCon1) annotation (points=[30, 0; 30, 0],
            style(color=8));
      connect(J0p3_2.BondCon2, BondCon2) annotation (points=[50,0; 100,0],
          style(color=8));
      connect(B4.eBondCon1, J0p3_2.BondCon3) annotation (points=[40, 10; 40, 10],
               style(color=8));
      connect(RS.BondCon2, B4.fBondCon1) annotation (points=[30, 40; 40, 40; 40,
              30], style(color=8));
      connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,10;
            -6.12303e-016,10],
            style(color=8));
      connect(B3.fBondCon1, RS.BondCon1) annotation (points=[6.12303e-016,30;
            6.12303e-016,40; 10,40],
                 style(color=8));
    end HEr;
    
    model RA "Radiative heat exchange element" 
      extends Interfaces.TwoPort;
      parameter Modelica.SIunits.Area A=1 "Radiating surface";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Icon(Text(
            extent=[-66, 80; 64, -80],
            string="RA",
            style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name"),
          Text(extent=[-78,-52; 80,-92], string="A=%A")),
        Diagram(Line(points=[-10, 0; -10, 0], style(color=8)), Line(points=[10,
                  0; 10, 0], style(color=8))),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The bondgraphic radiative heat exchange element is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The specific radiation constant and the radiating surface are modeled as parameters. <p>

The radiative heat exchange element has two causality strokes at the element. <p>

<hr>

<pre>
Potential variables:

 e1:      Bondgraphic effort variable of inflow

 f1:      Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:      Bondgraphic effort variable of outflow

 f2:      Bondgraphic flow variable of outflow, normalized positive for flows out of the model


Parameters:

 sigma:   Specific radiation constant (default value = 1 W/(m<sup>2</sup>*<sup>o</sup>K<sup>4</sup>)

 A:       Radiating surface (default value = 1 m<sup>2</sup>)
</pre>
</html>
"));
      Bonds.fBond B1 annotation (extent=[-30, 10; -10, 30]);
      Bonds.eBond B2 annotation (extent=[10, 10; 30, 30]);
      Junctions.J0p3 J0p3_2 annotation (extent=[50, 10; 30, -10], rotation=-90);
      Bonds.eBond B4 annotation (extent=[-30, -30; -10, -10], rotation=-180);
      Bonds.fBond B3 annotation (extent=[10, -30; 30, -10], rotation=180);
      Thermal.GSrad GSrad1(A=A)              annotation (extent=[-10, 10; 10,
            30]);
      Junctions.J0p3 J0p3_1 annotation (extent=[-50, 10; -30, -10], rotation=-
            90);
      Thermal.GSrad GSrad2(A=A)              annotation (extent=[10, -30; -10,
            -10]);
    equation 
      connect(GSrad1.BondCon2, B2.fBondCon1) annotation (points=[10, 20; 10, 20],
               style(color=8));
      connect(B1.fBondCon1, GSrad1.BondCon1) annotation (points=[-10, 20; -10,
            20], style(color=8));
      connect(B3.fBondCon1, GSrad2.BondCon1) annotation (points=[10, -20; 10, -
            20], style(color=8));
      connect(GSrad2.BondCon2, B4.fBondCon1) annotation (points=[-10, -20; -10,
              -20], style(color=8));
      connect(J0p3_1.BondCon2, B1.eBondCon1) annotation (points=[-40, 10; -40,
            20; -30, 20], style(color=8));
      connect(B2.eBondCon1, J0p3_2.BondCon2) annotation (points=[30, 20; 40, 20;
              40, 10], style(color=8));
      connect(J0p3_2.BondCon1, B3.eBondCon1) annotation (points=[40, -10; 40, -
            20; 30, -20], style(color=8));
      connect(B4.eBondCon1, J0p3_1.BondCon1) annotation (points=[-30, -20; -40,
              -20; -40, -10], style(color=8));
      connect(J0p3_1.BondCon3, BondCon1) annotation (points=[-50,6.12303e-016;
            -72,6.12303e-016; -72,0; -100,0], style(color=8, rgbcolor={192,192,
              192}));
      connect(J0p3_2.BondCon3, BondCon2) annotation (points=[50,6.12303e-016;
            72,6.12303e-016; 72,0; 100,0], style(color=8, rgbcolor={192,192,192}));
    end RA;
    
    model DS "Bondgraphic non-linear resistive (diode) source element" 
      extends Interfaces.TwoPort;
      parameter Modelica.SIunits.Resistance Ron=1E-5 "Leakage resistance";
      parameter Modelica.SIunits.Conductance Goff=1E-5 "Leakage conductance";
      parameter Modelica.SIunits.Voltage e0=0 "Forward threshold voltage";
      Real s "Curve parameter";
      Boolean blocking;
      annotation (
        Icon(
          Text(extent=[-70,90; 70,50],   string="%name"),
          Text(
            extent=[-72,78; 76,-82],
            style(color=0),
            string="DS")),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                fillColor=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[6,80; 16,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Text(
            extent=[70,-4; 80,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-80,-40; -20,-10; 20,10; 40,70],     style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Line(points=[20,9; 20,0],   style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[12,-2; 32,-12],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Text(
            extent=[20,70; 40,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Ron"),
          Text(
            extent=[-20,10; 0,0],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Goff"),
          Ellipse(extent=[18,12; 22,8], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[32,38; 66,28],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[-52,-28; -22,-36],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[20,14; 40,4],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s=0")),
        Documentation(info="<html>
The resistive (diode) source element is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The non-linear resistance is modeled by a set of parameters. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains.  Leaking diodes are a special class of non-linear resistors. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>
<hr>

<pre>
Potential variables:

 e1:        Bondgraphic effort variable of inflow

 f1:        Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:        Bondgraphic effort variable of outflow

 f2:        Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 blocking:  Boolean variable denoting the switch position (internal modulation signal)

 s:         Curve parameter


Parameters:

 Ron:       Leakage resistance in <b>conducting</b> state (default value = 1e-5 Ohm)

 Goff:      Leakage conductance in <b>blocking</b> state (default value = 1e-5 Mho)

 e0:        Forward threshold effort (default value = 0 Volt)
</pre> <p>

<hr> <p>

Equations: <p>

<b>blocking = s &lt; 0</b> <br>
<b>f1 = s*(if blocking then Goff else 1) + Goff*e0</b> <br>
<b>e1 = s*(if blocking then 1 else Ron) + e0</b> <br>
<b>f2 = (e1/e2)*f1</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>
"));
    equation 
      blocking = s < 0;
      f1 = s*(if blocking then Goff else 1) + Goff*e0;
      e1 = s*(if blocking then 1 else Ron) + e0;
      e1*f1 = e2*f2;
    end DS;
    
    model DSreal "Electrical real diode of the Modelica bond graph library" 
      extends Interfaces.TwoPort;
      constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
        "Boltzmann's constant";
      constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
        "Electron charge";
      parameter Modelica.SIunits.Current Ids=1.e-6 "Saturation current";
      parameter Modelica.SIunits.Voltage EG=1.11 "Energy gap of Si at 300 K";
      parameter Real N=1 "Emission coefficient";
      parameter Modelica.SIunits.Temperature TNOM=300.15 "Nominal temperature";
      parameter Real XTI=3 "Temperature exponent of saturation current";
      parameter Real Maxexp(final min=Modelica.Constants.small) = 15 
        "Max. exponent for linear continuation";
      parameter Modelica.SIunits.Resistance R=1.e8 "Parallel Ohmic resistance";
      Modelica.SIunits.Voltage Vt "Thermal voltage";
      
    protected 
      Real dchar "Diode characteristic";
      Real tdep "Temperature dependence";
      
      function exlin "Exponential function linearly continued for x > Maxexp" 
        input Real x;
        input Real Maxexp;
        output Real z;
      algorithm 
        z := if x > Maxexp then exp(Maxexp)*(1 + x - Maxexp) else exp(x);
      end exlin;
      
      function pow "Just a helper function for x^y" 
        input Real x;
        input Real y;
        output Real z;
      algorithm 
        z:=x^y;
      end pow;
      annotation (
        Icon(
          Text(
            extent=[-78,68; 44,-72],
            style(color=0),
            string="DS"),
          Text(
            extent=[34,-10; 96,-44],
            style(color=0, pattern=0),
            string="real"),
          Text(extent=[-70,96; 70,56],   string="%name")),
        Diagram(
          Line(points=[-80,0; 80,0],   style(color=10)),
          Polygon(points=[70,4; 80,0; 70,-4; 70,4],     style(color=10,
                fillColor=10)),
          Line(points=[0,80; 0,-80],   style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-4,70; 0,80; 4,70; -4,70],     style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[4,82; 14,72],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Text(
            extent=[70,-4; 80,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-80,-4; -28,-4; -6,-2; 0,0],         style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[-66,-6; -36,-14],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[16,50; 48,42],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Line(points=[0,0; 6,10; 12,28; 16,66],            style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1))),
        Documentation(info="<html>
The real diode source element is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  Real diodes are a special class of non-linear resistors. <p>

The real diode model consists of the diode itself and an parallel Ohmic resistance <b>R</b>. The diode formula essentially is: <p>

<b>f = Ids*(exp(e/Vt) - 1)</b> <p>

However, if the exponent <b>e/Vt</b> reaches the limit <b>Maxexp</b>, the diode characterisic is linearly continued to avoid overflow. <p>

The diode source element is a bit more complex than the diode element, because it not only computes the heat flow, but also the temperature dependence of the diode characteristic. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:      Bondgraphic effort variable of inflow

 f1:      Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:      Bondgraphic effort variable of outflow

 f2:      Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 Vt:      Thermal voltage


Parameters:

 Ids:     Saturation current (default value = 1e-6 Amp)

 Maxexp:  Maximum exponent (default value = 15)

 R:       Parallel Ohmic leakage resistance (default value = 1e8 Ohm)

 EG:      Activation energy (default value = 1.11 J)

 N:       Emission coefficient (default value = 1)

 TNOM:    Nominal temperature (default value = 300.15 K)

 XTI:     Temperature exponent of saturation current (default value = 3)
</pre> <p>

<hr> <p>

Equations: <p>

<b>Vt = k*e2/q</b> <br>
<b>dchar = exlin((v/(N*Vt)), Maxexp) - 1</b> <br>
<b>tdep = exp((e2/TNOM - 1)*EG/(N*Vt))*pow(e2/TNOM, XTI/N)</b> <br>
<b>f1 = Ids*tdep*dchar + e1/R</b> <br>
<b>f2 = (e1/e2)*f1</b>
</html>
"));
    equation 
      /* Compute thermal voltage as function of temperature */
      Vt = k*e2/q;
      
      /* Compute diode characteristic */
      dchar = exlin((e1/(N*Vt)), Maxexp) - 1;
      
      /* Compute temperature dependence */
      tdep = exp((e2/TNOM - 1)*EG/(N*Vt))*pow(e2/TNOM, XTI/N);
      
      /* Compute diode current */
      f1 = Ids*tdep*dchar + e1/R;
      
      /* Compute heat flow */
      f1*e1 = f2*e2;
    end DSreal;
    
    model ZS "Bondgraphic non-linear resistive (Zener diode) source element" 
      extends Interfaces.TwoPort;
      constant Integer conducting=1;
      constant Integer blocking=2;
      constant Integer avalanche=3;
      parameter Modelica.SIunits.Resistance Ron=1E-5 "Leakage resistance";
      parameter Modelica.SIunits.Conductance Goff=1E-5 "Leakage conductance";
      parameter Modelica.SIunits.Voltage e0=1 "Avalanche breakdown voltage";
      Real s1 "Curve parameter";
      Real s2 "Curve parameter";
      Integer mode "Operating mode";
      annotation (
        Icon(
          Text(extent=[-70,90; 70,50],   string="%name"),
          Text(
            extent=[-72,74; 72,-76],
            style(color=0),
            string="ZS"),
          Text(extent=[-62,-56; 56,-90], string="e0=%e0")),
        Diagram(
          Line(points=[-86,0; 74,0],   style(color=10)),
          Line(points=[-6,80; -6,-80], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-10,70; -6,80; -2,70; -10,70], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[0,80; 10,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Line(points=[-20,-74; -6,0; 36,12; 48,70],        style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Ellipse(extent=[-8,2; -4,-2], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[-50,-44; -18,-54],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[-2,18; 28,10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[-6,0; 14,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s1=0"),
          Text(
            extent=[44,46; 74,36],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="avalanche"),
          Text(
            extent=[40,16; 60,6],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s2=0"),
          Ellipse(extent=[34,14; 38,10],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[28,-2; 48,-12],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Polygon(points=[74,4; 84,0; 74,-4; 74,4],     style(color=10,
                fillColor=10)),
          Text(
            extent=[74,-4; 84,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[36,9; 36,0],   style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[18,8; 38,-2],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Goff"),
          Text(
            extent=[-12,-30; 8,-40],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
      string="Ron"),
          Text(
            extent=[24,64; 44,54],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="Raval")),
        Documentation(info="<html>
The non-linear resistive (Zener diode) source element is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The non-linear resistance is modeled by a set of parameters. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains.  Zener diodes are always used as reversed diodes.  Thus, the positive and negative pins have been reversed in the model.  Leaking diodes are a special class of non-linear resistors. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:     Bondgraphic effort variable of inflow

 f1:     Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:     Bondgraphic effort variable of outflow

 f2:     Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 mode:   Ternary variable denoting the switch position (internal modulation signal)

 s1:     Curve parameter

 s2:     Curve parameter


Parameters:

 Ron:    Leakage resistance in <b>conducting</b> state (default value = 1e-5 Ohm)

 Raval:  Leakage resistance in <b>avalanche</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>blocking</b> state (default value = 1e-5 Mho)

 e0:     Avalanche breakdown voltage (default value = 1 Volt)
</pre> <p>

<hr> <p>

Equations: <p>

<b>s2 = s1 - e0</b> <br>
<b>mode = if s1 &lt; 0 then conducting else if s1 &lt; e0 then blocking else avalanche</b> <br>
<b>e1 = if mode == conducting then Ron*s1 else if mode == blocking then s1 else (s2*Ron + e0)</b> <br>
<b>f1 = if mode == conducting then s1 else if mode == blocking then Goff*s1 else (s2 + Goff*e0)</b> <br>
<b>f2 = (e1/e2)*f1</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>
"));
    equation 
      s2 = s1 - e0;
      mode = if s1 < 0 then conducting else if s1 < e0 then blocking else avalanche;
      e1 = if mode == conducting then Ron*s1 else if mode == blocking then s1 else (s2*Ron + e0);
      f1 = if mode == conducting then s1 else if mode == blocking then Goff*s1 else (s2 + Goff*e0);
      e1*f1 = e2*f2;
    end ZS;
    
    model TS "Bondgraphic non-linear resistive (tunnel diode) source element" 
      extends Interfaces.TwoPort;
      constant Integer blocking=1;
      constant Integer c1=2;
      constant Integer b1=3;
      constant Integer c2=4;
      constant Integer b2=5;
      constant Integer conducting=6;
      parameter Modelica.SIunits.Conductance G0=1E-5 "Leakage Conductance";
      parameter Modelica.SIunits.Resistance R1=1E-5 "Leakage Resistance";
      parameter Modelica.SIunits.Resistance R2=1E-5 "Leakage Resistance";
      parameter Modelica.SIunits.Resistance R3=1E-5 "Leakage Resistance";
      parameter Modelica.SIunits.Voltage em1=1 "Intermediate forward voltage";
      parameter Modelica.SIunits.Voltage em2=2 "Intermediate forward voltage";
      parameter Modelica.SIunits.Voltage em3=3 "Intermediate forward voltage";
      parameter Modelica.SIunits.Voltage e0=4 "Maximum forward voltage";
      parameter Modelica.SIunits.Current fm=1 "Intermediate bottom current";
      parameter Modelica.SIunits.Current f0=2 "Intermediate top current";
      Real s1 "Curve parameter";
      Real s2 "Curve parameter";
      Real s3 "Curve parameter";
      Real s4 "Curve parameter";
      Real s5 "Curve parameter";
      Integer mode "Operating mode";
      annotation (
        Icon(
          Text(
            extent=[-68,74; 76,-76],
            style(color=0),
            string="TS"),
          Text(extent=[-66,90; 74,50],   string="%name")),
        Diagram(
          Line(points=[-86,0; 74,0],   style(color=10)),
          Line(points=[-6,80; -6,-80], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Polygon(points=[-10,70; -6,80; -2,70; -10,70], style(
              color=10,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[0,80; 10,70],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="f"),
          Ellipse(extent=[-8,2; -4,-2], style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[40,70; 72,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="conducting"),
          Text(
            extent=[-58,-10; -28,-18],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="blocking"),
          Text(
            extent=[-12,-2; 0,-8],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s1=0"),
          Text(
            extent=[-10,50; 4,42],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="c1"),
          Ellipse(extent=[4,58; 0,62],  style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Text(
            extent=[24,0; 44,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="e0"),
          Polygon(points=[74,4; 84,0; 74,-4; 74,4],     style(color=10,
                fillColor=10)),
          Text(
            extent=[74,-4; 84,-14],
            style(
              color=10,
              fillColor=10,
              fillPattern=1),
            string="e"),
          Line(points=[-78,-12; -6,0; 2,60; 14,60],         style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Line(points=[42,92; 34,30; 22,30; 14,60],         style(
              color=0,
              thickness=2,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[16,54; 30,46],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="c2"),
          Text(
            extent=[22,30; 36,22],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="b2"),
          Text(
            extent=[2,68; 16,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="b1"),
          Text(
            extent=[-2,-6; 8,-16],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="em1"),
          Text(
            extent=[-24,36; -4,26],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="fm"),
          Text(
            extent=[-22,64; -2,54],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="f0"),
          Line(points=[34,28; 34,0],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[14,58; 14,0],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[20,30; -6,30], style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[-10,68; 2,62],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s2=0"),
          Text(
            extent=[16,66; 28,60],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s3=0"),
          Text(
            extent=[8,30; 20,24],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s4=0"),
          Text(
            extent=[36,32; 48,26],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="s5=0"),
          Ellipse(extent=[12,62; 16,58],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Ellipse(extent=[20,32; 24,28],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Ellipse(extent=[32,32; 36,28],style(
              color=0,
              rgbcolor={0,0,0},
              pattern=3,
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1)),
          Line(points=[0,60; -6,60],  style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[8,0; 18,-10],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="em2"),
          Text(
            extent=[16,-6; 26,-16],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="em3"),
          Line(points=[2,58; 2,-8],   style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Line(points=[22,28; 22,-8], style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1)),
          Text(
            extent=[-66,2; -48,-8],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="G0"),
          Text(
            extent=[-18,16; 0,6],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="R1"),
          Text(
            extent=[4,42; 22,32],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="R2"),
          Text(
            extent=[34,52; 52,42],
            style(
              color=10,
              pattern=3,
              fillColor=10,
              fillPattern=1),
            string="R3")),
        Documentation(info="<html>
The non-linear resistive (tunnel diode) source element is a directed TwoPort element.  It inherits the effort and flow variables from the directed TwoPort.  The non-linear resistance is modeled by a set of parameters. <p>

Diodes are switching elements that are primarily being used in electrical circuits, although nothing prevents them from being used also in other energy domains.  Leaking diodes are a special class of non-linear resistors. <p>

The resistive source element has free causality on the primary side, and fixed causality on the secondary side.  The causality stroke is at the element on the secondary side (a source of entropy, rather than a source of temperature). <p>

<hr>

<pre>
Potential variables:

 e1:    Bondgraphic effort variable of inflow

 f1:    Bondgraphic flow variable of inflow, normalized positive for flows into the model

 e2:    Bondgraphic effort variable of outflow

 f2:    Bondgraphic flow variable of outflow, normalized positive for flows out of the model

 mode:  Sextary variable denoting the switch position (internal modulation signal)

 s1:    Curve parameter

 s2:    Curve parameter

 s3:    Curve parameter

 s4:    Curve parameter

 s5:    Curve parameter


Parameters:

 G0:    Leakage conductance in blocking state (default value = 1e-5 Mho)

 R1:    Leakage resistance in conducting state (default value = 1e-5 Ohm)

 R2:    Leakage resistance in conducting state (default value = 1e-5 Ohm)

 R3:    Leakage resistance in conducting state (default value = 1e-5 Ohm)

 em1:   First ntermediate forward voltage (default value = 1 Volt)

 em2:   Second intermediate forward voltage (default value = 2 Volt)

 em3:   Third intermediate forward voltage (default value = 3 Volt)

 e0:    Maximum forward voltage (default value = 4 Volt)

 fm:    Intermediate bottom current (default value = 1 Amp)

 f0:    Intermediate top current (default value = 2 Amp)
</pre> <p>

<hr> <p>

Equations: <p>

<b>s2 = s1 - f0</b> <br>
<b>s3 = s2 - (em2-em1)</b> <br>
<b>s4 = s3 - (f0-fm)</b> <br>
<b>s5 = s4 - (e0-em3)</b> <br>
<b>mode = if s1 &lt; 0 then blockinging else
          if s2 &lt; 0 then c1 else 
          if s3 &lt; 0 then b1 else 
          if s4 &lt; 0 then c2 else 
          if s5 &lt; 0 then b2 else conducting</b> <br>
<b>e1 = if mode == blocking then s1 else 
        if mode == c1 then R1*s1 else 
        if mode == b1 then (em1 + s2) else 
        if mode == c2 then (em2 + R2*s3) else 
        if mode == b2 then (em3 + s4) else (e0 + R3*s5)</b> <br>
<b>f1 = if mode == blocking then G0*s1 else 
        if mode == c1 then s1 else 
        if mode == b1 then f0 else 
        if mode == c2 then (f0 - s3) else 
        if mode == b2 then fm else (fm + s5)</b> <br>
<b>f2 = (e1/e2)*f1</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Otter, M., H. Elmqvist, and S.E. Mattsson (1999), \"<a href=\"http://www.modelica.org/publications/papers/hybrid99.pdf\">Hybrid
     Modeling in Modelica Based on the Synchronous Data Flow Principle</a>,\" <i>Proc. CACSD'99, IEEE Symposium on
     Computer-Aided Control System Design</i>, Hawaii, August 22-27, 1999, pp. 151-157.
</ol>
</html>
"));
    equation 
      s2 = s1 - f0;
      s3 = s2 - (em2-em1);
      s4 = s3 - (f0-fm);
      s5 = s4 - (e0-em3);
      mode = if s1 < 0 then blocking else 
             if s2 < 0 then c1 else 
             if s3 < 0 then b1 else 
             if s4 < 0 then c2 else 
             if s5 < 0 then b2 else conducting;
      e1 = if mode == blocking then s1 else 
           if mode == c1 then R1*s1 else 
           if mode == b1 then (em1 + s2) else 
           if mode == c2 then (em2 + R2*s3) else 
           if mode == b2 then (em3 + s4) else (e0 + R3*s5);
      f1 = if mode == blocking then G0*s1 else 
           if mode == c1 then s1 else 
           if mode == b1 then f0 else 
           if mode == c2 then (f0 - s3) else 
           if mode == b2 then fm else (fm + s5);
    end TS;
  end Thermal;
  
  package Electrical "Electrical circuit diagrams built from bond graphs" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=42)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0,
        y=0.6,
        width=0.4,
        height=0.4,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package offers a partial reimplementation of the standard electrical Modelica library using bond graph technology.
</html>
"));
    package Analog 
      "Analog electircal circuits built from bond graph technology" 
      annotation (preferedView="info",
        Icon(
          Text(
            extent=[-120, 120; 120, 71],
            string="%name",
            style(color=1)),
          Rectangle(extent=[-100, -98; 80, 52], style(fillColor=30, fillPattern=
                 1)),
          Polygon(points=[-100, 52; -80, 72; 100, 72; 80, 52; -100, 52], style(
                fillColor=30, fillPattern=1)),
          Polygon(points=[100, 72; 100, -78; 80, -98; 80, 52; 100, 72], style(
                fillColor=30, fillPattern=1)),
          Text(
            extent=[-85, 37; 65, -83],
            string="Library",
            style(color=3))), Documentation(info="<html>
This package offers a partial reimplementation of the standard analog electrical Modelica library using bond graph technology.
</html>
"));
      package Interfaces "Interfaces of the analog electrical library" 
        annotation (preferedView="info",
          Icon(
            Text(
              extent=[-118, 112; 122, 63],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100, -100; 80, 50], style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85, 35; 65, -85],
              string="Library",
              style(color=58))), Documentation(info="<html>
This package provides some interface models for the analog electrical library built in bond graph technology.
</html>
"));
        model El2BG "Electrical to bond graph conversion" 
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Electrical connector" 
            annotation (extent=[-110, -10; -90, 10]);
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                              annotation (extent=[90, -10; 110,
                  10]);
          annotation (
            Icon(
              Line(points=[-80, -80; -80, 80; 80, 80; -80, -80]),
              Line(points=[80, 80; 80, -80; -80, -80], style(color=9)),
              Line(points=[94, 0; 80, 0], style(color=9)),
              Line(points=[-90, 0; -80, 0], style(color=77)),
              Text(
                extent=[-64, 56; 0, 12],
                string="El",
                style(color=75)),
              Text(
                extent=[4, -14; 70, -64],
                string="BG",
                style(color=9)),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255}))),
            Diagram(
              Line(points=[-80, -80; -80, 80; 80, 80; -80, -80]),
              Line(points=[80, 80; 80, -80; -80, -80], style(color=9)),
              Line(points=[94, 0; 80, 0], style(color=9)),
              Line(points=[-96,0; -80,0],   style(color=77)),
              Text(
                extent=[-64, 56; 0, 12],
                string="El",
                style(color=75)),
              Text(
                extent=[4, -14; 70, -64],
                string="BG",
                style(color=9)),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255}))),
            Documentation(info="<html>
This model converts an electrical wire to a bondgraphic power signal.
</html>
"));
        equation 
          BondCon1.e = p.v;
          BondCon1.f = p.i;
        end El2BG;
        
        model BG2El "Bond graph to electrical conversion" 
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Electrical connector" 
            annotation (extent=[90, -10; 110, 10]);
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                              annotation (extent=[-110, -10; -
                90, 10]);
          annotation (
            Diagram(
              Line(points=[-80, -80; -80, 80; 80, 80; -80, -80], style(color=9)),
              Line(points=[-94, 0; -80, 0], style(color=9)),
              Line(points=[80, 80; 80, -80; -80, -80]),
              Line(points=[96,0; 80,0]),
              Text(
                extent=[-64, 70; 2, 20],
                string="BG",
                style(color=9)),
              Text(
                extent=[0, -16; 64, -60],
                string="El",
                style(color=75)),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255}))),
            Icon(
              Line(points=[-80, -80; -80, 80; 80, 80; -80, -80], style(color=9)),
              Line(points=[-94, 0; -80, 0], style(color=9)),
              Line(points=[80, 80; 80, -80; -80, -80]),
              Line(points=[90, 0; 80, 0]),
              Text(
                extent=[-64, 70; 2, 20],
                string="BG",
                style(color=9)),
              Text(
                extent=[0, -16; 64, -60],
                string="El",
                style(color=75)),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255}))),
            Documentation(info="<html>
This model converts a bondgraphic power signal to an electrical wire.
</html>
"));
        equation 
          BondCon1.e = n.v;
          BondCon1.f = -n.i;
        end BG2El;
        
        partial model OnePort 
          "OnePort partial model of analog electrical library" 
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from pin p to pin n";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Right electrical connector" 
            annotation (extent=[90, -10; 110, 10]);
          annotation (Icon, Diagram(
              Line(points=[-110,18; -85,18],   style(color=9, fillColor=9)),
              Polygon(points=[-95,21; -85,18; -95,15; -95,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[90,18; 115,18],   style(color=9, fillColor=9)),
              Text(
                extent=[-114,17; -94,37],
                string="i",
                style(color=9)),
              Polygon(points=[105,21; 115,18; 105,15; 105,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[98,39; 118,19],
                string="i",
                style(color=9)),
              Line(points=[-127,0; -117,0],   style(color=9)),
              Line(points=[-122,-5; -122,5],   style(color=9)),
              Line(points=[117,0; 127,0],   style(color=9))));
        equation 
          v = p.v - n.v;
          i = p.i;
        end OnePort;
        
        partial model OnePortV 
          "OnePort partial model of analog electrical library for capacitors" 
          parameter Modelica.SIunits.Voltage u0=0 
            "Initial voltage across capacitor";
          parameter Boolean enforceStates = true 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
          Modelica.SIunits.Voltage v(start=u0, stateSelect=if enforceStates then StateSelect.prefer else StateSelect.default) 
            "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from pin p to pin n";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Right electrical connector" 
            annotation (extent=[90, -10; 110, 10]);
          annotation (Icon, Diagram(
              Line(points=[-110,18; -85,18],   style(color=9, fillColor=9)),
              Polygon(points=[-95,21; -85,18; -95,15; -95,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[90,18; 115,18],   style(color=9, fillColor=9)),
              Text(
                extent=[-114,17; -94,37],
                string="i",
                style(color=9)),
              Polygon(points=[105,21; 115,18; 105,15; 105,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[98,39; 118,19],
                string="i",
                style(color=9)),
              Line(points=[-127,0; -117,0],   style(color=9)),
              Line(points=[-122,-5; -122,5],   style(color=9)),
              Line(points=[117,0; 127,0],   style(color=9))));
        equation 
          v = p.v - n.v;
          i = p.i;
        end OnePortV;
        
        partial model OnePortI 
          "OnePort partial model of analog electrical library for inductors" 
          parameter Modelica.SIunits.Current i0=0 
            "Initial current across inductor";
          parameter Boolean enforceStates = true 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i(start=i0, stateSelect=if enforceStates then StateSelect.prefer else StateSelect.default) 
            "Current flowing from pin p to pin n";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Right electrical connector" 
            annotation (extent=[90, -10; 110, 10]);
          annotation (Icon, Diagram(
              Line(points=[-110,18; -85,18],   style(color=9, fillColor=9)),
              Polygon(points=[-95,21; -85,18; -95,15; -95,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[90,18; 115,18],   style(color=9, fillColor=9)),
              Text(
                extent=[-114,17; -94,37],
                string="i",
                style(color=9)),
              Polygon(points=[105,21; 115,18; 105,15; 105,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[98,39; 118,19],
                string="i",
                style(color=9)),
              Line(points=[-127,0; -117,0],   style(color=9)),
              Line(points=[-122,-5; -122,5],   style(color=9)),
              Line(points=[117,0; 127,0],   style(color=9))));
        equation 
          v = p.v - n.v;
          i = p.i;
        end OnePortI;
        
        partial model ModOnePort 
          "Modulated OnePort partial model of analog electrical library" 
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from pin p to pin n";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Richt electrical connector" 
            annotation (extent=[90,-10; 110,10]);
          Modelica.Blocks.Interfaces.RealInput u "Modulation signal" 
            annotation (extent=[-10,80; 10,100], rotation=-90);
          annotation (Icon, Diagram(
              Line(points=[-112,18; -87,18],   style(color=9, fillColor=9)),
              Polygon(points=[-97,21; -87,18; -97,15; -97,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[88,18; 113,18],   style(color=9, fillColor=9)),
              Text(
                extent=[-116,17; -96,37],
                string="i",
                style(color=9)),
              Polygon(points=[103,21; 113,18; 103,15; 103,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[96,39; 116,19],
                string="i",
                style(color=9)),
              Line(points=[-123,0; -113,0],   style(color=9)),
              Line(points=[-118,-5; -118,5],   style(color=9)),
              Line(points=[115,2; 125,2],   style(color=9))));
        equation 
          v = p.v - n.v;
          i = p.i;
        end ModOnePort;
        
        partial model ModOnePortV 
          "Modulated OnePort partial model of analog electrical library for capacitors" 
          parameter Real u0=0 "Initial voltage across capacitor";
          parameter Boolean enforceStates = true 
            "|Advanced||use electrical variables as states instead of bond graph variables";
          Modelica.SIunits.Voltage v(start=u0, stateSelect=if enforceStates then StateSelect.prefer else StateSelect.default) 
            "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from pin p to pin n";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Right electrical connector" 
            annotation (extent=[90,-10; 110,10]);
          Modelica.Blocks.Interfaces.RealInput u "Modulation signal" 
            annotation (extent=[-10,80; 10,100], rotation=-90);
          annotation (Icon, Diagram(
              Line(points=[-112,18; -87,18],   style(color=9, fillColor=9)),
              Polygon(points=[-97,21; -87,18; -97,15; -97,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[88,18; 113,18],   style(color=9, fillColor=9)),
              Text(
                extent=[-116,17; -96,37],
                string="i",
                style(color=9)),
              Polygon(points=[103,21; 113,18; 103,15; 103,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[96,39; 116,19],
                string="i",
                style(color=9)),
              Line(points=[-123,0; -113,0],   style(color=9)),
              Line(points=[-118,-5; -118,5],   style(color=9)),
              Line(points=[115,0; 125,0],   style(color=9))));
        equation 
          v = p.v - n.v;
          i = p.i;
        end ModOnePortV;
        
        partial model ModOnePortV2 
          "Modulated OnePort partial model of analog electrical library for capacitors" 
          parameter Modelica.SIunits.ElectricCharge q0=0 
            "Initial charge across capacitor";
          parameter Boolean enforceStates = true 
            "|Advanced||use electrical variables as states instead of bond graph variables";
          Modelica.SIunits.ElectricCharge q(stateSelect=if enforceStates then StateSelect.prefer else StateSelect.default) 
            "Electrical charge";
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from pin p to pin n";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Right electrical connector" 
            annotation (extent=[90,-10; 110,10]);
          Modelica.Blocks.Interfaces.RealInput u "Modulation signal" 
            annotation (extent=[-10,80; 10,100], rotation=-90);
          annotation (Diagram(
              Line(points=[-118,-5; -118,5],   style(color=9)),
              Polygon(points=[-97,21; -87,18; -97,15; -97,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[115,2; 125,2],   style(color=9)),
              Line(points=[-123,0; -113,0],   style(color=9)),
              Line(points=[-112,18; -87,18],   style(color=9, fillColor=9)),
              Text(
                extent=[-116,17; -96,37],
                string="i",
                style(color=9)),
              Line(points=[88,18; 113,18],   style(color=9, fillColor=9)),
              Polygon(points=[103,21; 113,18; 103,15; 103,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[96,39; 116,19],
                string="i",
                style(color=9))));
        equation 
          v = p.v - n.v;
          i = p.i;
        end ModOnePortV2;
        
        partial model ModOnePortI2 
          "Modulated OnePort partial model of analog electrical library for inductors" 
          parameter Modelica.SIunits.MagneticFlux phi0=0 
            "Initial charge across capacitor";
          parameter Boolean enforceStates = true 
            "|Advanced||use electrical variables as states instead of bond graph variables";
          Modelica.SIunits.MagneticFlux phi(start=phi0, stateSelect=if enforceStates then StateSelect.prefer else StateSelect.default) 
            "Magnetic Flux";
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from pin p to pin n";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Right electrical connector" 
            annotation (extent=[90,-10; 110,10]);
          Modelica.Blocks.Interfaces.RealInput u "Modulation signal" 
            annotation (extent=[-10,80; 10,100], rotation=-90);
          annotation (Diagram(
              Line(points=[-118,-5; -118,5],   style(color=9)),
              Polygon(points=[-97,21; -87,18; -97,15; -97,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[115,2; 125,2],   style(color=9)),
              Line(points=[-123,0; -113,0],   style(color=9)),
              Line(points=[-112,18; -87,18],   style(color=9, fillColor=9)),
              Text(
                extent=[-116,17; -96,37],
                string="i",
                style(color=9)),
              Line(points=[88,18; 113,18],   style(color=9, fillColor=9)),
              Polygon(points=[103,21; 113,18; 103,15; 103,21],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[96,39; 116,19],
                string="i",
                style(color=9))));
        equation 
          v = p.v - n.v;
          i = p.i;
        end ModOnePortI2;
        
        partial model TwoPort 
          "TwoPort partial model of analog electrical library" 
          Modelica.SIunits.Voltage v1 "Voltage drop over the left port";
          Modelica.SIunits.Voltage v2 "Voltage drop over the right port";
          Modelica.SIunits.Current i1 
            "Current flowing from pos. to neg. pin of the left port";
          Modelica.SIunits.Current i2 
            "Current flowing from pos. to neg. pin of the right port";
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 
            "First left electrical connector" 
            annotation (extent=[-110, 40; -90, 60]);
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 
            "Second left electrical connector" 
            annotation (extent=[90, 40; 110, 60]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 
            "First right electrical connector" 
            annotation (extent=[-110, -60; -90, -40]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 
            "Second right electrical connector" 
            annotation (extent=[90, -60; 110, -40]);
          annotation (Diagram(
              Polygon(points=[-120,53; -110,50; -120,47; -120,53],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[-136,50; -111,50],   style(color=9, fillColor=9)),
              Text(
                extent=[-136,53; -119,68],
                string="i1",
                style(color=9, fillColor=9)),
              Polygon(points=[-126,-46; -136,-49; -126,-52; -126,-46],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[-136,-49; -111,-49],   style(color=9, fillColor=9)),
              Text(
                extent=[-127,-46; -110,-31],
                string="i1",
                style(color=9, fillColor=9)),
              Line(points=[111,50; 136,50],   style(
                  color=9,
                  gradient=2,
                  fillColor=9)),
              Polygon(points=[120,53; 110,50; 120,47; 120,53],     style(
                  color=9,
                  gradient=2,
                  fillColor=9,
                  fillPattern=1)),
              Text(
                extent=[118,52; 135,67],
                string="i2",
                style(
                  color=9,
                  gradient=2,
                  fillColor=9)),
              Text(
                extent=[112,-44; 128,-29],
                string="i2",
                style(color=9, fillColor=9)),
              Polygon(points=[127,-47; 137,-50; 127,-53; 127,-47],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[111,-50; 136,-50],   style(color=9, fillColor=9))));
        equation 
          v1 = p1.v - n1.v;
          v2 = p2.v - n2.v;
          i1 = p1.i;
          i2 = p2.i;
        end TwoPort;
      end Interfaces;
      
      package Basic "Basic elements of the analog electrical library" 
        annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30,
                  fillPattern=1)),
            Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50],
                style(fillColor=30, fillPattern=1)),
            Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70],
                style(fillColor=30, fillPattern=1)),
            Text(
              extent=[-85, 35; 65, -85],
              string="Library",
              style(color=3)),
            Text(
              extent=[-120, 114; 120, 65],
              string="%name",
              style(color=1))), Documentation(info="<html>
This package offers a partial reimplementation of the <i>Basic</i> package of the standard analog electrical Modelica library using bond graph technology.
</html>
"));
        model Ground "Electrical ground" 
          Modelica.Electrical.Analog.Interfaces.Pin p "Electrical connector" 
                                                         annotation (extent=[-
                10, 90; 10, 110]);
          annotation (
            Icon(
              Line(points=[-60, 40; 60, 40]),
              Line(points=[-40, 20; 40, 20]),
              Line(points=[-20, 0; 20, 0]),
              Line(points=[0, 90; 0, 40]),
              Text(extent=[-100, -78; 100, -20], string="%name")),
            Diagram,
            Documentation(info="<html>
The electrical ground is modeled as an effort source injecting zero effort.
</html>
"));
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-20, 20; 20, 60], rotation=-90);
          BondLib.Sources.Se V0(e0=0) annotation (extent=[-20, -60; 20, -20],
              rotation=-90);
          BondLib.Bonds.fBond B1 annotation (extent=[-20, -20; 20, 20],
              rotation=90);
        equation 
          connect(El2BG1.BondCon1, B1.fBondCon1) annotation (points=[
                1.22461e-015,20; 1.22461e-015,20],
                                   style(color=8));
          connect(B1.eBondCon1, V0.BondCon1) annotation (points=[-1.22461e-015,
                -20; -1.22461e-015,-20],
                                style(color=8));
          connect(El2BG1.p, p) annotation (points=[-1.22461e-015,60; 0,60; 0,
                100], style(color=3, rgbcolor={0,0,255}));
        end Ground;
        
        model Resistor "Linear resistor of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance R=1 "Resistance";
          annotation (
            Icon(
              Text(extent=[-100,42; 100,102],   string="%name"),
              Rectangle(extent=[-70, 30; 70, -30], style(
                  color=3,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[-90, 0; -70, 0]),
              Line(points=[70, 0; 90, 0]),
              Text(
                extent=[-96, -34; 104, -96],
                string="R=%R",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
The ideal linear resistor is built in bondgraphic technology from a bond graph <b>R</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the resistor in its environment. <p>

<hr>

<pre>
Parameters:

 R:  Resistance (default value = 1 Ohm)
</pre>
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -12; -12, 12]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -12; 38, 12]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, 12; 12, 36], rotation=
                90);
          BondLib.Passive.R R1(R=R) annotation (extent=[-10, 36; 10, 56],
              rotation=90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
        equation 
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12,0; -12,
                0],                    style(color=8));
          connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[12, 0;
                12.26, 0], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(B3.BondCon2, R1.BondCon1) annotation (points=[7.34764e-016,36;
                -6.12303e-016,36],
              style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-38,
                1.77636e-015; -38,1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38,0; 38,
                1.77636e-015], style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end Resistor;
        
        model HeatingResistor "Electrical resistor with heat generation" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance R_ref=1 
            "Resistance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref=300 
            "Reference temperature";
          parameter Real alpha(unit="1/K") = 0 
            "Temperature coefficient of resistance";
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort 
            "Thermal connector"                                        annotation (
              extent=[-10,-90; 10,-110],   rotation=-90);
          annotation (Icon(
              Text(extent=[-142,60; 143,118],  string="%name"),
              Line(points=[-90,0; -70,0]),
              Line(points=[70,0; 90,0]),
              Rectangle(extent=[-70,30; 70,-30],   style(
                  color=3,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[0,-30; 0,-91],   style(color=42)),
              Line(points=[-52,-50; 48,50],   style(color=73, fillColor=73)),
              Polygon(points=[40,52; 50,42; 54,56; 40,52],     style(color=73,
                    fillColor=73))), Diagram);
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[4,58; 28,82]);
          Bonds.Bond B1         annotation (extent=[-22,58; 4,82]);
          Bonds.Bond B2         annotation (extent=[28,58; 54,82]);
          Bonds.Bond B3         annotation (extent=[4,34; 28,58],     rotation=
                270);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -46,56; -22,84]);
          Interfaces.BG2El BG2El1                           annotation (extent=[54,
                56; 80,84]);
          Bonds.eBond B4 annotation (extent=[2,-12; 30,10], rotation=-90);
          Thermal.mRS mRS1 annotation (extent=[4,10; 28,34], rotation=-90);
          Junctions.J0p3 J0p3_1 
            annotation (extent=[6,-32; 26,-12], rotation=-90);
          Bonds.eBond B5 annotation (extent=[2,-54; 30,-32], rotation=-90);
          Bonds.fBond B6 annotation (extent=[-14,-32; 6,-12], rotation=180);
          Modelica.Blocks.Math.Gain Gain1(k=R_ref) 
            annotation (extent=[-12,14; 0,30]);
          Modelica.Blocks.Math.Add Add1 annotation (extent=[-36,12; -20,32]);
          Modelica.Blocks.Math.Gain Gain2(k=alpha) 
            annotation (extent=[-58,-4; -46,12]);
          Modelica.Blocks.Sources.Constant Constant1 
            annotation (extent=[-60,28; -46,44]);
          Modelica.Blocks.Math.Add Add2(k2=-1) 
            annotation (extent=[-44,-38; -60,-18]);
          Modelica.Blocks.Sources.Constant Constant2(k=T_ref) 
            annotation (extent=[-16,-54; -30,-38]);
          BondLib.Sensors.De De1 annotation (extent=[-14,-32; -34,-12]);
          Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
            annotation (extent=[2,-80; 30,-54], rotation=-90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[4,70; 4,70],
                                       style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[28,70;
                28.26,70], style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-22,70;
                -21.74,70],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[54,70; 54,
                70],           style(color=8));
          connect(B3.BondCon2, mRS1.BondCon1) annotation (points=[16,34; 16,34],
              style(color=8, rgbcolor={192,192,192}));
          connect(J0p3_1.BondCon1, B4.eBondCon1) annotation (points=[16,-12; 16,
                -12], style(color=8, rgbcolor={192,192,192}));
          connect(J0p3_1.BondCon2, B5.fBondCon1) annotation (points=[16,-32; 16,
                -32], style(color=8, rgbcolor={192,192,192}));
          connect(B6.eBondCon1, J0p3_1.BondCon3) annotation (points=[6,-22; 6,
                -22], style(color=8, rgbcolor={192,192,192}));
          if cardinality(heatPort) == 0 then
            heatPort.T = T_ref;
          end if;
          connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[16,58; 16,
                57.88; 16,57.76; 16,57.76], style(color=8, rgbcolor={192,192,
                  192}));
          connect(B4.fBondCon1, mRS1.BondCon2) annotation (points=[16,10; 16,10],
              style(color=8, rgbcolor={192,192,192}));
          connect(Add1.y, Gain1.u) annotation (points=[-19.2,22; -13.2,22],
              style(color=74, rgbcolor={0,0,127}));
          connect(Gain2.y, Add1.u2) annotation (points=[-45.4,4; -42,4; -42,16;
                -37.6,16], style(color=74, rgbcolor={0,0,127}));
          connect(Constant1.y, Add1.u1) annotation (points=[-45.3,36; -42,36;
                -42,28; -37.6,28], style(color=74, rgbcolor={0,0,127}));
          connect(Constant2.y, Add2.u2) annotation (points=[-30.7,-46; -36,-46;
                -36,-34; -42.4,-34], style(color=74, rgbcolor={0,0,127}));
          connect(Add2.y, Gain2.u) annotation (points=[-60.8,-28; -70,-28; -70,
                4; -59.2,4], style(color=74, rgbcolor={0,0,127}));
          connect(De1.BondCon1, B6.fBondCon1) annotation (points=[-14,-22; -14,
                -22], style(color=8, rgbcolor={192,192,192}));
          connect(De1.OutPort1, Add2.u1) annotation (points=[-35,-22; -42.4,-22],
              style(color=74, rgbcolor={0,0,127}));
          connect(El2BG1.p, p) annotation (points=[-46,70; -88,70; -88,0; -100,
                0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[80,70; 88,70; 88,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(BG2Heat1.BondCon1, B5.eBondCon1) annotation (points=[16,-54;
                16,-54], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, heatPort) annotation (points=[16,-80; 16,-86;
                0,-86; 0,-100], style(color=42, rgbcolor={191,0,0}));
          connect(Gain1.y, mRS1.s) annotation (points=[0.6,22; 5.2,22], style(
                color=74, rgbcolor={0,0,127}));
        end HeatingResistor;
        
        model Conductor "Linear conductor of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Conductance G=1 "Conductance";
          annotation (
            Icon(
              Text(extent=[-100, 40; 100, 98], string="%name"),
              Line(points=[-90, 0; -70, 0]),
              Rectangle(extent=[-70, 30; 70, -30]),
              Line(points=[70, 0; 90, 0]),
              Text(
                extent=[-100, -38; 100, -100],
                string="G=%G",
                style(color=0, pattern=0))),
            Diagram,
            Documentation(info="<html>
The ideal linear conductor is built in bondgraphic technology from a bond graph <b>R</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the resistor in its environment. <p>

<hr>

<pre>
Parameters:

 G:  Conductance (default value = 1 Mho)
</pre>

</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -12; -12, 12]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -12; 38, 12]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, 12; 12, 36], rotation=
                90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          Passive.G G1(G=G) annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12, 0; -12,
                  0], style(color=8));
          connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[12, 0;
                12.26, 0], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-38,
                1.77636e-015; -38,1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(B3.BondCon2, G1.BondCon1) annotation (points=[7.34764e-016,36;
                -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end Conductor;
        
        model Capacitor "Linear capacitor of analog electrical library" 
          extends Interfaces.OnePortV;
          parameter Modelica.SIunits.Capacitance C=1 "Capacitance";
          annotation (
            Icon(
              Line(points=[14, 0; 90, 0]),
              Line(points=[14, 28; 14, -28], style(thickness=2)),
              Line(points=[-14, 28; -14, -28], style(thickness=2)),
              Line(points=[-90, 0; -14, 0]),
              Text(extent=[-100, 40; 100, 100], string="%name"),
              Text(
                extent=[-100, -40; 100, -100],
                string="C=%C",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
The ideal linear capacitor is built in bondgraphic technology from a bond graph <b>C</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the capacitor in its environment. <p>

<hr>

<pre>
Parameters:

 C:  Capacitance (default value = 1 Farad)
</pre>
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -12; -12, 12]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -12; 38, 12]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, 12; 12, 36], rotation=
                90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Passive.C C1(C=C, e(start=u0)) annotation (extent=[-12, 36;
                12, 60], rotation=90);
        equation 
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12, 0; -12,
                  0], style(color=8));
          connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[12, 0;
                12.26, 0], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-38,
                1.77636e-015; -38,1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(B3.BondCon2, C1.BondCon1) annotation (points=[7.34764e-016,36;
                -7.34764e-016,36],
              style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end Capacitor;
        
        model Inductor "Linear inductor of analog electrical library" 
          extends Interfaces.OnePortI;
          parameter Modelica.SIunits.Inductance L=1 "Inductance";
          annotation (
            Icon(
              Line(points=[-90, 0; -60, 0]),
              Ellipse(extent=[-60, -15; -30, 15]),
              Ellipse(extent=[-30, -15; 0, 15]),
              Ellipse(extent=[0, -15; 30, 15]),
              Ellipse(extent=[30, -15; 60, 15]),
              Line(points=[60, 0; 90, 0]),
              Text(extent=[-100, 40; 100, 100], string="%name"),
              Text(
                extent=[-100, -40; 100, -100],
                string="L=%L",
                style(color=0)),
              Rectangle(extent=[-60, -28; 60, 0], style(color=7, fillColor=7))),
            Diagram,
            Documentation(info="<html>
The ideal linear inductor is built in bondgraphic technology from a bond graph <b>I</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the inductor in its environment. <p>

<hr>

<pre>
Parameters:

 L:  Inductance (default value = 1 Henry)
</pre>
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -12; -12, 12]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -12; 38, 12]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, 12; 12, 36], rotation=
                90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Passive.I L1(I=L, f(start=i0)) annotation (extent=[-12, 36;
                12, 60], rotation=90);
        equation 
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12, 0; -12,
                  0], style(color=8));
          connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[12, 0;
                12.26, 0], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-38,
                1.77636e-015; -38,1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(L1.BondCon1, B3.BondCon2) annotation (points=[-7.34764e-016,
                36; 7.34764e-016,36],
              style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -78,
                1.77636e-015; -78,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end Inductor;
        
        model Transformer "Transformer of analog electrical library" 
          extends Interfaces.TwoPort;
          parameter Modelica.SIunits.Inductance L1=1 "Primary inductance";
          parameter Modelica.SIunits.Inductance L2=1 "Secondary inductance";
          parameter Modelica.SIunits.Inductance M=1 "Coupling inductance";
          annotation (
            Icon(
              Ellipse(extent=[-45, -50; -20, -25]),
              Ellipse(extent=[-45, -25; -20, 0]),
              Ellipse(extent=[-45, 0; -20, 25]),
              Ellipse(extent=[-45, 25; -20, 50]),
              Rectangle(extent=[-72, -60; -33, 60], style(color=7, fillColor=7)),
              Line(points=[-90, 50; -32, 50]),
              Line(points=[-90, -50; -32, -50]),
              Ellipse(extent=[20, -50; 45, -25]),
              Ellipse(extent=[20, -25; 45, 0]),
              Ellipse(extent=[20, 0; 45, 25]),
              Ellipse(extent=[20, 25; 45, 50]),
              Rectangle(extent=[33, -60; 72, 60], style(color=7, fillColor=7)),
              Line(points=[32, 50; 90, 50]),
              Line(points=[32, -50; 90, -50]),
              Text(extent=[-89, 18; -60, -10], string="L1"),
              Text(extent=[64, 18; 90, -10], string="L2"),
              Text(extent=[-18, -70; 20, -98], string="M"),
              Text(extent=[-100, 118; 100, 60], string="%name")),
            Diagram(
              Text(
                extent=[-38, 26; -28, 18],
                string="v1a",
                style(color=9)),
              Text(
                extent=[-38, -14; -28, -22],
                style(color=9),
                string="v1b"),
              Text(
                extent=[-26, 8; -16, 0],
                style(color=9),
                string="v1"),
              Text(
                extent=[-54, 26; -44, 18],
                style(color=9),
                string="i1"),
              Text(
                extent=[-24, -4; -16, -12],
                style(color=9),
                string="i1"),
              Text(
                extent=[-54, -16; -44, -24],
                style(color=9),
                string="i1"),
              Text(
                extent=[42, -14; 52, -22],
                style(color=9),
                string="v2b"),
              Text(
                extent=[42, 26; 52, 18],
                style(color=9),
                string="v2a"),
              Text(
                extent=[16, -2; 28, -10],
                style(color=9),
                string="v2"),
              Text(
                extent=[14, 12; 26, 4],
                style(color=9),
                string="i2"),
              Text(
                extent=[26, 24; 36, 16],
                style(color=9),
                string="i2"),
              Text(
                extent=[26, -16; 36, -24],
                style(color=9),
                string="i2")),
            Documentation(info="<html>
The real transformer is built in bondgraphic technology from a bond graph <b>IF</b> element.  A-causal bonds must be used here, because the causality depends on the embedding of the inductive field in its environment. <p>

<hr>

<pre>
Parameters:

 L1:  Primary inductance (default value = 1 Henry)

 L2:  Secondary inductance (default value = 1 Henry)

 M:   Coupling inductance (default value = 1 Henry)
</pre> <p>

<hr> <p>

Equations: <p>

<b>Det = L1*L2 - M<sup>2</sup></b> <br>
<b>Linv1 = L2/Det</b> <br>
<b>Linv2 = L1/Det</b> <br>
<b>Minv = -M/Det</b> <p>
<b>der(f1) = Linv1*e1 + Minv*e2</b> <br>
<b>der(f2) = Minv*e1 + Linv2*e2</b>
</html>
"));
          
        protected 
          BondLib.Passive.IF TF1(
            I1=L1,
            I2=L2,
            I12=M) annotation (extent=[-10, -10; 10, 10]);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-70, 40; -50, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [-50, -60; -70, -40]);
          BondLib.Bonds.Bond B1 annotation (extent=[-30, -10; -10, 10]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-50, -10; -30, 10],
                rotation=90);
          BondLib.Bonds.Bond B2 annotation (extent=[-50, 10; -30, 30], rotation=
               -90);
          BondLib.Bonds.Bond B3 annotation (extent=[-50, -30; -30, -10],
              rotation=-90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [70, 40; 50, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El2 annotation (extent=
                [50, -60; 70, -40]);
          BondLib.Bonds.Bond B4 annotation (extent=[10, -10; 30, 10], rotation=
                180);
          BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[50, -10; 30, 10],
              rotation=90);
          BondLib.Bonds.Bond B5 annotation (extent=[30, 10; 50, 30], rotation=-
                90);
          BondLib.Bonds.Bond B6 annotation (extent=[30, -30; 50, -10], rotation=
               -90);
        equation 
          connect(J1p3_1.BondCon3, B1.BondCon1) annotation (points=[-30,
                -6.12303e-016; -30,-6.12303e-016; -30,0; -29.8,0],
                      style(color=8));
          connect(B2.BondCon2, J1p3_1.BondCon2) annotation (points=[-40, 10; -
                40, 10], style(color=8));
          connect(J1p3_1.BondCon1, B3.BondCon1) annotation (points=[-40, -10; -
                40, -10.2], style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon1) annotation (points=[-50,50; -40,
                50; -40,29.8],    style(color=8));
          connect(BG2El1.BondCon1, B3.BondCon2) annotation (points=[-50, -50; -
                40, -50; -40, -30], style(color=8));
          connect(B1.BondCon2, TF1.BondCon1) annotation (points=[-10, 0; -10, 0],
                   style(color=8));
          connect(B5.BondCon2, J1p3_2.BondCon2) annotation (points=[40, 10; 40,
                  10], style(color=8));
          connect(J1p3_2.BondCon1, B6.BondCon1) annotation (points=[40, -10; 40,
                  -10.2], style(color=8));
          connect(El2BG2.BondCon1, B5.BondCon1) annotation (points=[50,50; 40,
                50; 40,29.8],  style(color=8));
          connect(BG2El2.BondCon1, B6.BondCon2) annotation (points=[50, -50; 40,
                  -50; 40, -30], style(color=8));
          connect(B4.BondCon1, J1p3_2.BondCon3) annotation (points=[29.8,
                -1.20011e-015; 29.85,-1.20011e-015; 29.85,-1.81241e-015; 29.9,
                -1.81241e-015; 29.9,-6.12303e-016; 30,-6.12303e-016],
                      style(color=8));
          connect(B4.BondCon2, TF1.BondCon2) annotation (points=[10,
                1.22461e-015; 10,0],
                style(color=8));
          connect(El2BG1.p, p1) annotation (points=[-70,50; -100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n1) annotation (points=[-70,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(El2BG2.p, p2) annotation (points=[70,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n2) annotation (points=[70,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
        end Transformer;
        
        model Gyrator "Real gyrator of analog electrical library" 
          extends Interfaces.TwoPort;
          parameter Modelica.SIunits.Conductance G1=1 
            "Gyration forward conductance";
          parameter Modelica.SIunits.Conductance G2=1 
            "Gyration backward conductance";
          annotation (
            Diagram(
              Ellipse(extent=[-62, 52; -58, 48], style(fillColor=77)),
              Ellipse(extent=[58, 52; 62, 48], style(fillColor=77)),
              Ellipse(extent=[-82, -48; -78, -52], style(fillColor=77)),
              Ellipse(extent=[78, -48; 82, -52], style(fillColor=77))),
            Documentation(info="<html>
The real gyrator is a macro element, built from two <b>VCC</b> elements.  The <b>VCC</b> elements are internally built in bondgraphic technology. <p>

<hr>

<pre>
Parameters:

 G1:  Gyration forward conductance (default value = 1 Amp/Volt)

 G2:  Gyration backward conductance (default value = 1 Amp/Volt)

</pre> <p>

<hr> <p>

Equations: <p>

<b>i1 = G*v2</b> <br>
<b>i2 = -G*v1</b>
</html>
"),         Icon(
              Rectangle(extent=[-60, 60; 60, -60]),
              Line(points=[-90, 50; -60, 50]),
              Line(points=[-90, -50; -60, -50]),
              Line(points=[60, 50; 90, 50]),
              Line(points=[60, -50; 90, -50]),
              Text(extent=[-100, 118; 100, 57], string="%name"),
              Line(points=[-40, 30; 40, 30]),
              Line(points=[-20, -20; 20, -20]),
              Polygon(points=[30, 34; 40, 30; 30, 26; 30, 34], style(fillColor=
                      3, fillPattern=1)),
              Polygon(points=[-26, -19; -16, -15; -16, -23; -26, -19], style(
                    fillColor=3, fillPattern=1)),
              Line(points=[-5, 10; -10, -10], style(thickness=2)),
              Line(points=[9, 10; 4, -9], style(thickness=2)),
              Line(points=[-14, 9; 16, 10], style(thickness=2)),
              Text(extent=[-29, 59; 30, 30], string="G1"),
              Text(extent=[-29, -29; 29, -58], string="G2")),
            Coordsys(extent=[-100, -100; 100, 100]));
          
        protected 
          BondLib.Electrical.Analog.Basic.VCC VCC1(transConductance=G1) 
            annotation (extent=[-40, 8; 40, 88]);
          BondLib.Electrical.Analog.Basic.VCC VCC2(transConductance=G2) 
            annotation (extent=[44, -90; -36, -10]);
        equation 
          connect(VCC1.p1, p1) annotation (points=[-40,68; -60,68; -60,50; -100,
                50], style(color=3, rgbcolor={0,0,255}));
          connect(VCC1.n1, n1) annotation (points=[-40,28; -80,28; -80,-50;
                -100,-50], style(color=3, rgbcolor={0,0,255}));
          connect(VCC1.p2, p2) annotation (points=[40,68; 60,68; 60,50; 100,50],
              style(color=3, rgbcolor={0,0,255}));
          connect(VCC1.n2, n2) annotation (points=[40,28; 80,28; 80,-50; 100,
                -50], style(color=3, rgbcolor={0,0,255}));
          connect(VCC1.p1, VCC2.p2) annotation (points=[-40,68; -60,68; -60,-30;
                -36,-30], style(color=3, rgbcolor={0,0,255}));
          connect(VCC1.n1, VCC2.n2) annotation (points=[-40,28; -80,28; -80,-70;
                -36,-70], style(color=3, rgbcolor={0,0,255}));
          connect(VCC1.p2, VCC2.p1) annotation (points=[40,68; 60,68; 60,-30;
                44,-30], style(color=3, rgbcolor={0,0,255}));
          connect(VCC1.n2, VCC2.n1) annotation (points=[40,28; 80,28; 80,-70;
                44,-70], style(color=3, rgbcolor={0,0,255}));
        end Gyrator;
        
        model EMF "Bondgraphic electromotorical force element" 
          parameter Real k(final unit="N.m/A") = 1 "Transformation coefficient";
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from pin p to pin n";
          Modelica.SIunits.AngularVelocity w "Angular velocity of flange_b";
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Left electrical connector" 
            annotation (extent=[-10, 90; 10, 110]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n 
            "Right electrical connector" 
            annotation (extent=[-10, -110; 10, -90]);
          annotation (
            Icon(
              Line(points=[0, 90; 0, 40]),
              Rectangle(extent=[40, 10; 90, -10], style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Ellipse(extent=[-40, 40; 40, -40]),
              Line(points=[0, -90; 0, -40]),
              Text(extent=[20, -40; 100, -100], string="%name"),
              Text(
                extent=[30, 100; 119, 40],
                string="k=%k",
                style(color=9))),
            Documentation(info="<html>
EMF transforms electrical energy into rotational mechanical energy.  It is used as basic building block of an electrical motor. The mechanical connector flange_b can be connected to elements of the Modelica.Mechanics.Rotational library. flange_b.tau is the cut-torque, flange_b.phi is the angle at the rotational connection.
</html>
"),         Diagram(
              Text(
                extent=[2,24; 12,16],
                string="v1a",
                style(color=9)),
              Text(
                extent=[2,-16; 12,-24],
                style(color=9),
                string="v1b"),
              Text(
                extent=[14,8; 24,0],
                style(color=9),
                string="v1"),
              Text(
                extent=[-12,24; -2,16],
                style(color=9),
                string="i1"),
              Text(
                extent=[16,-4; 24,-12],
                style(color=9),
                string="i1"),
              Text(
                extent=[-12,-16; -2,-24],
                style(color=9),
                string="i1"),
              Text(
                extent=[50,-2; 68,-10],
                style(color=9),
                string="omega"),
              Text(
                extent=[52,8; 64,0],
                style(color=9),
                string="tau")));
          
        protected 
          BondLib.Passive.GY GY1(r=k) annotation (extent=[30,-10; 50,10]);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-10, 50; 10, 70], rotation=-90);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [10, -70; -10, -50], rotation=270);
          BondLib.Bonds.Bond B1 annotation (extent=[10,-10; 30,10]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-10,-10; 10,10],
                rotation=90);
          BondLib.Bonds.Bond B2 annotation (extent=[-10,10; 10,30],    rotation=
               -90);
          BondLib.Bonds.Bond B3 annotation (extent=[-10,-30; 10,-10],
              rotation=-90);
          BondLib.Bonds.Bond B4 annotation (extent=[50,-10; 70,10]);
        public 
          Mechanical.Rotational.Interfaces.Flange_b flange_b 
            "Rotational connector" 
            annotation (extent=[90,-10; 110,10]);
          Mechanical.Rotational.Interfaces.BG2Rot bG2Rot 
            annotation (extent=[70,-10; 90,10]);
        equation 
          v = p.v - n.v;
          i = p.i;
          w = GY1.f2;
          connect(J1p3_1.BondCon3, B1.BondCon1) annotation (points=[10,
                -6.12303e-016; 10.2,0], style(color=8));
          connect(B2.BondCon2, J1p3_1.BondCon2) annotation (points=[
                6.12303e-016,10; 6.12303e-016,10],
                         style(color=8));
          connect(J1p3_1.BondCon1, B3.BondCon1) annotation (points=[
                -6.12303e-016,-10; -6.00057e-016,-10; -6.00057e-016,-10.2],
                            style(color=8));
          connect(B1.BondCon2, GY1.BondCon1) annotation (points=[30,0; 30,0],
                style(color=8));
          connect(GY1.BondCon2, B4.BondCon1) annotation (points=[50,0; 50.2,0],
                   style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon1) annotation (points=[
                6.12303e-016,50; 6.12303e-016,40; -6.00057e-016,40;
                -6.00057e-016,29.8],   style(color=8));
          connect(BG2El1.BondCon1, B3.BondCon2) annotation (points=[
                -1.83691e-015,-50; -1.83691e-015,-40; 6.12303e-016,-40;
                6.12303e-016,-30],        style(color=8));
          connect(El2BG1.p, p) annotation (points=[-6.12303e-016,70; 0,70; 0,
                100], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[1.83691e-015,-70; 0,-70; 0,
                -100], style(color=3, rgbcolor={0,0,255}));
          connect(bG2Rot.BondCon1, B4.BondCon2) annotation (points=[70,0; 70,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(bG2Rot.Rot, flange_b) annotation (points=[90,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
        end EMF;
        
        model VCV "Linear voltage-controlled voltage source" 
          extends Interfaces.TwoPort;
          parameter Real gain=1 "Voltage gain";
          annotation (
            Icon(
              Rectangle(extent=[-70, 70; 70, -70], style(fillColor=7)),
              Text(extent=[-99, -59; 100, -109], string="%name"),
              Line(points=[-90, 50; -30, 50]),
              Line(points=[-30, -50; -90, -50]),
              Ellipse(extent=[10, 20; 50, -20]),
              Line(points=[-20, 60; 20, 60]),
              Polygon(points=[20, 60; 10, 63; 10, 57; 20, 60], style(fillColor=
                      3, fillPattern=1)),
              Line(points=[30, 50; 30, -50]),
              Line(points=[30, 50; 90, 50]),
              Line(points=[30, -50; 90, -50])),
            Diagram(
              Text(
                extent=[-68, 26; -58, 18],
                string="v1a",
                style(color=9)),
              Text(
                extent=[-82, 26; -72, 18],
                style(color=9),
                string="i1"),
              Text(
                extent=[-82, -16; -72, -24],
                style(color=9),
                string="i1"),
              Text(
                extent=[-56, -4; -44, -12],
                style(color=9),
                string="i1=0"),
              Text(
                extent=[-68, -14; -58, -22],
                style(color=9),
                string="v1b"),
              Text(
                extent=[-56, 8; -46, 0],
                style(color=9),
                string="v1"),
              Text(
                extent=[42, 8; 54, 0],
                style(color=9),
                string="v2"),
              Text(
                extent=[44, -4; 56, -12],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, 24; 84, 16],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, -16; 84, -24],
                style(color=9),
                string="i2"),
              Text(
                extent=[58, 22; 68, 14],
                style(color=9),
                string="v2a"),
              Text(
                extent=[58, -18; 68, -26],
                style(color=9),
                string="v2b")),
            Documentation(info="<html>
The linear voltage-controlled voltage source is built in bondgraphic technology using an effort detector and a modulated effort source.  A-causal bonds must partially be used here, because the causality of the output port depends on the embedding of the <b>VCV</b> element in its environment. <p>

<hr>

<pre>
Parameters:

 gain:  Voltage gain (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>v2 = gain*v1</b> <br>
<b>i1 = 0</b>
</html>
"));
          
        protected 
          Modelica.Blocks.Math.Gain Gain1(k=gain)   annotation (extent=[-8, -10;
                  12, 10]);
          BondLib.Sensors.De De1 annotation (extent=[-40, -10; -20, 10]);
          BondLib.Bonds.fBond B3 annotation (extent=[-60, -10; -40, 10]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-80, -10; -60, 10],
                rotation=90);
          BondLib.Bonds.fBond B1 annotation (extent=[-80, 10; -60, 30],
              rotation=-90);
          BondLib.Bonds.eBond B2 annotation (extent=[-80, -30; -60, -10],
              rotation=-90);
          BondLib.Sources.mSe mSe1 annotation (extent=[40, -10; 20, 10]);
          BondLib.Bonds.fBond B4 annotation (extent=[40, -10; 60, 10]);
          BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[80, -10; 60, 10],
              rotation=90);
          BondLib.Bonds.Bond B6 annotation (extent=[60, -30; 80, -10], rotation=
               90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-80, 40; -60, 60]);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [80, 40; 60, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [60, -60; 80, -40]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El2 annotation (extent=
                [-60, -60; -80, -40]);
          BondLib.Bonds.Bond B5 annotation (extent=[60, 10; 80, 30], rotation=
                90);
        equation 
          connect(De1.OutPort1,Gain1.u)       annotation (points=[-19,0; -10,0],
                    style(color=3));
          connect(B3.fBondCon1, De1.BondCon1) annotation (points=[-40, 0; -40,
                0], style(color=8));
          connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[-60,
                -6.12303e-016; -60,0],
                        style(color=8));
          connect(B1.fBondCon1, J1p3_1.BondCon2) annotation (points=[-70, 10; -
                70, 10], style(color=8));
          connect(J1p3_1.BondCon1, B2.fBondCon1) annotation (points=[-70, -10;
                -70, -10], style(color=8));
          connect(mSe1.BondCon1, B4.eBondCon1) annotation (points=[40, 0; 40, 0],
                   style(color=8));
          connect(B4.fBondCon1, J1p3_2.BondCon3) annotation (points=[60,0; 60,
                -6.12303e-016],
                      style(color=8));
          connect(El2BG1.BondCon1, B1.eBondCon1) annotation (points=[-60, 50; -
                50, 50; -50, 30; -70, 30], style(color=8));
          connect(BG2El2.BondCon1, B2.eBondCon1) annotation (points=[-60, -50;
                -50, -50; -50, -30; -70, -30], style(color=8));
          connect(B5.BondCon1, J1p3_2.BondCon2) annotation (points=[70, 10.2;
                70, 10], style(color=8));
          connect(B6.BondCon2, J1p3_2.BondCon1) annotation (points=[70, -10; 70,
                  -10], style(color=8));
          connect(El2BG2.BondCon1, B5.BondCon2) annotation (points=[60, 50; 50,
                  50; 50, 30; 70, 30], style(color=8));
          connect(BG2El1.BondCon1, B6.BondCon1) annotation (points=[60,-50; 50,
                -50; 50,-29.8; 70,-29.8], style(color=8));
          connect(El2BG1.p, p1) annotation (points=[-80,50; -100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n1) annotation (points=[-80,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(El2BG2.p, p2) annotation (points=[80,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n2) annotation (points=[80,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gain1.y, mSe1.s) annotation (points=[13,0; 21,0], style(color=
                 74, rgbcolor={0,0,127}));
        end VCV;
        
        model VCC "Linear voltage-controlled current source" 
          extends Interfaces.TwoPort;
          parameter Modelica.SIunits.Conductance transConductance=1 
            "Transconductance";
          annotation (
            Icon(
              Rectangle(extent=[-70, 70; 70, -70], style(fillColor=7)),
              Text(extent=[-99, -60; 100, -109], string="%name"),
              Line(points=[-90, 50; -30, 50]),
              Line(points=[-30, -50; -90, -50]),
              Ellipse(extent=[10, 20; 50, -20]),
              Line(points=[-20, 60; 20, 60]),
              Polygon(points=[20, 60; 10, 63; 10, 57; 20, 60], style(fillColor=
                      3, fillPattern=1)),
              Line(points=[90, 50; 30, 50; 30, 20]),
              Line(points=[91, -50; 30, -50; 30, -20]),
              Line(points=[10, 0; 50, 0])),
            Diagram(
              Text(
                extent=[-68, 26; -58, 18],
                string="v1a",
                style(color=9)),
              Text(
                extent=[-82, 26; -72, 18],
                style(color=9),
                string="i1"),
              Text(
                extent=[-82, -16; -72, -24],
                style(color=9),
                string="i1"),
              Text(
                extent=[-56, -4; -44, -12],
                style(color=9),
                string="i1=0"),
              Text(
                extent=[-68, -14; -58, -22],
                style(color=9),
                string="v1b"),
              Text(
                extent=[-56, 8; -46, 0],
                style(color=9),
                string="v1"),
              Text(
                extent=[42, 8; 54, 0],
                style(color=9),
                string="v2"),
              Text(
                extent=[44, -4; 56, -12],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, 24; 84, 16],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, -16; 84, -24],
                style(color=9),
                string="i2"),
              Text(
                extent=[58, 22; 68, 14],
                style(color=9),
                string="v2a"),
              Text(
                extent=[58, -18; 68, -26],
                style(color=9),
                string="v2b")),
            Documentation(info="<html>
The linear voltage-controlled current source is built in bondgraphic technology using an effort detector and a modulated flow source.  The causalities of all bonds are fixed. <p>

<hr>

<pre>
Parameters:

 transConductance:  Transconductance (default value = 1 Amp/Volt)
</pre> <p>

<hr> <p>

Equations: <p>

<b>i2 = transConductance*v1</b> <br>
<b>i1 = 0</b>
</html>
"));
          
        protected 
          Modelica.Blocks.Math.Gain Gain1(k=transConductance)   annotation (
              extent=[-8, -10; 12, 10]);
          BondLib.Sensors.De De1 annotation (extent=[-40, -10; -20, 10]);
          BondLib.Bonds.fBond B1 annotation (extent=[-60, -10; -40, 10]);
          BondLib.Bonds.fBond B2 annotation (extent=[-80, 10; -60, 30],
              rotation=-90);
          BondLib.Bonds.eBond B3 annotation (extent=[-80, -30; -60, -10],
              rotation=-90);
          BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[80, -10; 60, 10],
              rotation=90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-80, 40; -60, 60]);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [80, 40; 60, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [60, -60; 80, -40]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El2 annotation (extent=
                [-60, -60; -80, -40]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-80, -10; -60, 10],
                rotation=90);
          BondLib.Sources.mSf mSf1 annotation (extent=[40, -10; 20, 10]);
          BondLib.Bonds.eBond B4 annotation (extent=[40, -10; 60, 10]);
          BondLib.Bonds.eBond B6 annotation (extent=[60, 10; 80, 30], rotation=
                90);
          BondLib.Bonds.fBond B5 annotation (extent=[60, -30; 80, -10],
              rotation=90);
        equation 
          connect(De1.OutPort1,Gain1.u)       annotation (points=[-19,0; -10,0],
                    style(color=3));
          connect(B1.fBondCon1, De1.BondCon1) annotation (points=[-40, 0; -40,
                0], style(color=8));
          connect(J1p3_1.BondCon3, B1.eBondCon1) annotation (points=[-60,
                -6.12303e-016; -60,0],
                        style(color=8));
          connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[-70, 10; -
                70, 10], style(color=8));
          connect(J1p3_1.BondCon1, B3.fBondCon1) annotation (points=[-70, -10;
                -70, -10], style(color=8));
          connect(El2BG1.BondCon1, B2.eBondCon1) annotation (points=[-60, 50; -
                50, 50; -50, 30; -70, 30], style(color=8));
          connect(BG2El2.BondCon1, B3.eBondCon1) annotation (points=[-60, -50;
                -50, -50; -50, -30; -70, -30], style(color=8));
          connect(B4.fBondCon1, mSf1.BondCon1) annotation (points=[40, 0; 40, 0],
                   style(color=8));
          connect(B4.eBondCon1, J1p3_2.BondCon3) annotation (points=[60,0; 60,
                -6.12303e-016],
                      style(color=8));
          connect(B6.fBondCon1, J1p3_2.BondCon2) annotation (points=[70, 10; 70,
                  10], style(color=8));
          connect(El2BG2.BondCon1, B6.eBondCon1) annotation (points=[60, 50; 50,
                  50; 50, 30; 70, 30], style(color=8));
          connect(J1p3_2.BondCon1, B5.fBondCon1) annotation (points=[70, -10;
                70, -10], style(color=8));
          connect(BG2El1.BondCon1, B5.eBondCon1) annotation (points=[60, -50;
                50, -50; 50, -30; 70, -30], style(color=8));
          connect(El2BG1.p, p1) annotation (points=[-80,50; -100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n1) annotation (points=[-80,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(El2BG2.p, p2) annotation (points=[80,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n2) annotation (points=[80,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gain1.y, mSf1.s) annotation (points=[13,0; 21,0], style(color=
                 74, rgbcolor={0,0,127}));
        end VCC;
        
        model CCV "Linear current-controlled voltage source" 
          extends Interfaces.TwoPort;
          parameter Modelica.SIunits.Resistance transResistance=1 
            "Transresistance";
          annotation (
            Icon(
              Rectangle(extent=[-70, 70; 70, -70], style(fillColor=7)),
              Text(extent=[-99, -60; 100, -110], string="%name"),
              Ellipse(extent=[10, 20; 50, -20]),
              Line(points=[-20, 60; 20, 60]),
              Polygon(points=[20, 60; 10, 63; 10, 57; 20, 60], style(fillColor=
                      3, fillPattern=1)),
              Line(points=[30, 50; 30, -50]),
              Line(points=[30, 50; 90, 50]),
              Line(points=[30, -50; 90, -50])),
            Diagram(
              Text(
                extent=[-68, 26; -58, 18],
                string="v1a",
                style(color=9)),
              Text(
                extent=[-82, 26; -72, 18],
                style(color=9),
                string="i1"),
              Text(
                extent=[-82, -16; -72, -24],
                style(color=9),
                string="i1"),
              Text(
                extent=[-56, -4; -44, -12],
                style(color=9),
                string="i1"),
              Text(
                extent=[-68, -14; -58, -22],
                style(color=9),
                string="v1b"),
              Text(
                extent=[-58, 12; -46, 2],
                style(color=9),
                string="v1=0"),
              Text(
                extent=[42, 8; 54, 0],
                style(color=9),
                string="v2"),
              Text(
                extent=[44, -4; 56, -12],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, 24; 84, 16],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, -16; 84, -24],
                style(color=9),
                string="i2"),
              Text(
                extent=[58, 22; 68, 14],
                style(color=9),
                string="v2a"),
              Text(
                extent=[58, -18; 68, -26],
                style(color=9),
                string="v2b")),
            Documentation(info="<html>
The linear current-controlled voltage source is built in bondgraphic technology using a flow detector and a modulated effort source.  A-causal bonds must partially be used here, because the causality of the input and output ports depend on the embedding of the <b>CCV</b> element in its environment. <p>

<hr>

<pre>
Parameters:

 transResistance:  Transresistance (default value = 1 Volt/Amp)
</pre> <p>

<hr> <p>

Equations: <p>

<b>v2 = transResistance*i1</b> <br>
<b>v1 = 0</b>
</html>
"));
          
        protected 
          Modelica.Blocks.Math.Gain Gain1(k=transResistance)   annotation (
              extent=[-8, -10; 12, 10]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-80, -10; -60, 10],
                rotation=90);
          BondLib.Sources.mSe mSe1 annotation (extent=[40, -10; 20, 10]);
          BondLib.Bonds.fBond B4 annotation (extent=[40, -10; 60, 10]);
          BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[80, -10; 60, 10],
              rotation=90);
          BondLib.Bonds.Bond B5 annotation (extent=[60, -30; 80, -10], rotation=
               90);
          BondLib.Bonds.Bond B6 annotation (extent=[60, 10; 80, 30], rotation=
                90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-80, 40; -60, 60]);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [80, 40; 60, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [60, -60; 80, -40]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El2 annotation (extent=
                [-60, -60; -80, -40]);
          BondLib.Sensors.Df Df1 annotation (extent=[-40, -10; -20, 10]);
          BondLib.Bonds.Bond B1 annotation (extent=[-80, 10; -60, 30], rotation=
               270);
          BondLib.Bonds.Bond B2 annotation (extent=[-80, -30; -60, -10],
              rotation=270);
          BondLib.Bonds.eBond B3 annotation (extent=[-60, -10; -40, 10]);
        equation 
          connect(mSe1.BondCon1, B4.eBondCon1) annotation (points=[40, 0; 40, 0],
                   style(color=8));
          connect(B4.fBondCon1, J1p3_2.BondCon3) annotation (points=[60,0; 60,
                -6.12303e-016],
                      style(color=8));
          connect(B5.BondCon2, J1p3_2.BondCon1) annotation (points=[70, -10; 70,
                  -10], style(color=8));
          connect(J1p3_2.BondCon2, B6.BondCon1) annotation (points=[70,10; 70,
                10.2], style(color=8));
          connect(Df1.OutPort1,Gain1.u)       annotation (points=[-19,0; -10,0],
                    style(color=3));
          connect(B3.eBondCon1, Df1.BondCon1) annotation (points=[-40, 0; -40,
                0], style(color=8));
          connect(J1p3_1.BondCon3, B3.fBondCon1) annotation (points=[-60,
                -6.12303e-016; -60,0],
                        style(color=8));
          connect(B1.BondCon2, J1p3_1.BondCon2) annotation (points=[-70, 10; -
                70, 10], style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-70, -10; -
                70, -10.2], style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-60,50; -50,
                50; -50,29.8; -70,29.8],   style(color=8));
          connect(BG2El2.BondCon1, B2.BondCon2) annotation (points=[-60, -50; -
                50, -50; -50, -30; -70, -30], style(color=8));
          connect(El2BG2.BondCon1, B6.BondCon2) annotation (points=[60, 50; 50,
                  50; 50, 30; 70, 30], style(color=8));
          connect(BG2El1.BondCon1, B5.BondCon1) annotation (points=[60,-50; 50,
                -50; 50,-29.8; 70,-29.8], style(color=8));
          connect(El2BG1.p, p1) annotation (points=[-80,50; -100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n1) annotation (points=[-80,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(El2BG2.p, p2) annotation (points=[80,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n2) annotation (points=[80,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gain1.y, mSe1.s) annotation (points=[13,0; 21,0], style(color=
                 74, rgbcolor={0,0,127}));
        end CCV;
        
        model CCC "Linear current-controlled current source" 
          extends Interfaces.TwoPort;
          parameter Real gain=1 "Current gain";
          annotation (
            Icon(
              Rectangle(extent=[-70, 70; 70, -70], style(fillColor=7)),
              Text(extent=[-100, -60; 101, -111], string="%name"),
              Ellipse(extent=[10, 20; 50, -20]),
              Line(points=[-20, 60; 20, 60]),
              Polygon(points=[20, 60; 10, 63; 10, 57; 20, 60], style(fillColor=
                      3, fillPattern=1)),
              Line(points=[90, 50; 30, 50; 30, 20]),
              Line(points=[91, -50; 30, -50; 30, -20]),
              Line(points=[10, 0; 50, 0]),
              Line(points=[-30, 50; -30, -50]),
              Line(points=[-30, 50; -90, 50]),
              Line(points=[-30, -50; -90, -50])),
            Diagram(
              Text(
                extent=[-68, 26; -58, 18],
                string="v1a",
                style(color=9)),
              Text(
                extent=[-56, -4; -44, -12],
                style(color=9),
                string="i1"),
              Text(
                extent=[-68, -14; -58, -22],
                style(color=9),
                string="v1b"),
              Text(
                extent=[-58, 12; -46, 2],
                style(color=9),
                string="v1=0"),
              Text(
                extent=[42, 8; 54, 0],
                style(color=9),
                string="v2"),
              Text(
                extent=[44, -4; 56, -12],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, 24; 84, 16],
                style(color=9),
                string="i2"),
              Text(
                extent=[72, -16; 84, -24],
                style(color=9),
                string="i2"),
              Text(
                extent=[58, 22; 68, 14],
                style(color=9),
                string="v2a"),
              Text(
                extent=[58, -18; 68, -26],
                style(color=9),
                string="v2b")),
            Documentation(info="<html>
The linear current-controlled current source is built in bondgraphic technology using a flow detector and a modulated flow source.  A-causal bonds must partially be used here, because the causality of the input port depends on the embedding of the <b>CCC</b> element in its environment. <p>

<hr>

<pre>
Parameters:

 gain:  Current gain (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>i2 = gain*i1</b> <br>
<b>v1 = 0</b>
</html>
"));
          
        protected 
          Modelica.Blocks.Math.Gain Gain1(k=gain)   annotation (extent=[-8, -10;
                  12, 10]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-80, -10; -60, 10],
                rotation=90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-80, 40; -60, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [-60, -60; -80, -40]);
          BondLib.Sensors.Df Df1 annotation (extent=[-40, -10; -20, 10]);
          BondLib.Bonds.Bond B1 annotation (extent=[-80, 10; -60, 30], rotation=
               270);
          BondLib.Bonds.Bond B2 annotation (extent=[-80, -30; -60, -10],
              rotation=270);
          BondLib.Bonds.eBond B3 annotation (extent=[-60, -10; -40, 10]);
          BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[80, -10; 60, 10],
              rotation=90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [80, 40; 60, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El2 annotation (extent=
                [60, -60; 80, -40]);
          BondLib.Sources.mSf mSf1 annotation (extent=[40, -10; 20, 10]);
          BondLib.Bonds.eBond B4 annotation (extent=[40, -10; 60, 10]);
          BondLib.Bonds.eBond B6 annotation (extent=[60, 10; 80, 30], rotation=
                90);
          BondLib.Bonds.fBond B5 annotation (extent=[60, -30; 80, -10],
              rotation=90);
        equation 
          connect(Df1.OutPort1,Gain1.u)       annotation (points=[-19,0; -10,0],
                    style(color=3));
          connect(B3.eBondCon1, Df1.BondCon1) annotation (points=[-40, 0; -40,
                0], style(color=8));
          connect(J1p3_1.BondCon3, B3.fBondCon1) annotation (points=[-60,
                -6.12303e-016; -60,0],
                        style(color=8));
          connect(B1.BondCon2, J1p3_1.BondCon2) annotation (points=[-70, 10; -
                70, 10], style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-70, -10; -
                70, -10.2], style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-60,50; -50,
                50; -50,29.8; -70,29.8],   style(color=8));
          connect(BG2El1.BondCon1, B2.BondCon2) annotation (points=[-60, -50; -
                50, -50; -50, -30; -70, -30], style(color=8));
          connect(B4.fBondCon1, mSf1.BondCon1) annotation (points=[40, 0; 40, 0],
                   style(color=8));
          connect(B6.fBondCon1, J1p3_2.BondCon2) annotation (points=[70, 10; 70,
                  10], style(color=8));
          connect(El2BG2.BondCon1, B6.eBondCon1) annotation (points=[60, 50; 50,
                  50; 50, 30; 70, 30], style(color=8));
          connect(B5.fBondCon1, J1p3_2.BondCon1) annotation (points=[70, -10;
                70, -10], style(color=8));
          connect(BG2El2.BondCon1, B5.eBondCon1) annotation (points=[60, -50;
                50, -50; 50, -30; 70, -30], style(color=8));
          connect(J1p3_2.BondCon3, B4.eBondCon1) annotation (points=[60,
                -6.12303e-016; 60,0],
                      style(color=8));
          connect(El2BG1.p, p1) annotation (points=[-80,50; -100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n1) annotation (points=[-80,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(El2BG2.p, p2) annotation (points=[80,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n2) annotation (points=[80,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gain1.y, mSf1.s) annotation (points=[13,0; 21,0], style(color=
                 74, rgbcolor={0,0,127}));
        end CCC;
        
        model VariableResistor "Modulated linear electrical resistor" 
          extends Interfaces.ModOnePort;
          annotation (
            Icon(
              Text(extent=[-144,-100; 148,-40],   string="%name"),
              Line(points=[-90,0; -70,0]),
              Rectangle(extent=[-70,30; 70,-30],   style(
                  color=3,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[70,0; 90,0]),
              Line(points=[0,80; 0,30],   style(color=73))),
            Diagram,
            Documentation(info="<html>
The modulated resistor is built in bondgraphic technology from a bond graph <b>mR</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the resistor in its environment. <p>

<hr>

<pre>
Potential variables:

 R:  Resistance (modulation input signal)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                90);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Passive.mR mR1 annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],                    style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015], style(color=8));
          connect(mR1.BondCon1, B3.BondCon2) annotation (points=[-7.34764e-016,
                36; 7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(u, mR1.s) annotation (points=[0,90; 0,74.4; 0,58.8; 6.61287e-016,
                58.8], style(color=74, rgbcolor={0,0,127}));
        end VariableResistor;
        
        model VariableConductor "Modulated linear electrical conductor" 
          extends Interfaces.ModOnePort;
          annotation (
            Icon(
              Text(extent=[-144,-100; 148,-40],   string="%name"),
              Line(points=[-90,0; -70,0]),
              Rectangle(extent=[-70,30; 70,-30],   style(
                  color=3,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[70,0; 90,0]),
              Line(points=[0,80; 0,30],   style(color=73))),
            Diagram,
            Documentation(info="<html>
The modulated conductor is built in bondgraphic technology from a bond graph <b>mG</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the conductor in its environment. <p>

<hr>

<pre>
Potential variables:

 G:  Conductance (modulation input signal)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                90);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Passive.mG mG1 annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],                    style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015], style(color=8));
          connect(B3.BondCon2, mG1.BondCon1) annotation (points=[7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(u, mG1.s) annotation (points=[0,90; 0,74.4; 0,58.8; 6.61287e-016,
                58.8], style(color=74, rgbcolor={0,0,127}));
        end VariableConductor;
        
        model VariableCapacitor "Modulated linear electrical capacitor" 
          extends Interfaces.ModOnePortV2;
          annotation (
            Icon(
              Text(extent=[-134,-96; 136,-36],    string="%name"),
              Line(points=[-90,0; -14,0]),
              Line(points=[14,0; 90,0]),
              Line(points=[0,80; 0,28],   style(color=73)),
              Line(points=[-14,28; -14,-28],   style(thickness=2)),
              Line(points=[14,28; 14,-28],   style(thickness=2))),
            Diagram,
            Documentation(info="<html>
The modulated capacitor is built in bondgraphic technology from a bond graph <b>mC</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the capacitor in its environment. <p>

<hr>

<pre>
Potential variables:

 C:  Capacitance (modulation input signal)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                90);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Passive.mC mC1(q(start=q0)) 
                         annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          q = u*v;
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],                    style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015], style(color=8));
          connect(B3.BondCon2, mC1.BondCon1) annotation (points=[7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(mC1.s, u) annotation (points=[6.61287e-016,58.8; 6.61287e-016,72.4;
                0,72.4; 0,90], style(color=74, rgbcolor={0,0,127}));
        end VariableCapacitor;
        
        model VariableInductor "Modulated linear electrical inductor" 
          extends Interfaces.ModOnePortI2;
          annotation (
            Icon(
              Text(extent=[-136,-94; 138,-34],    string="%name"),
              Line(points=[-90,0; -60,0]),
              Line(points=[60,0; 90,0]),
              Line(points=[0,80; 0,6],   style(color=73)),
              Ellipse(extent=[-60,-15; -30,15]),
              Ellipse(extent=[-30,-15; 0,15]),
              Ellipse(extent=[0,-15; 30,15]),
              Ellipse(extent=[30,-15; 60,15]),
              Rectangle(extent=[-60,-30; 60,0],   style(color=7, fillColor=7))),
            Diagram,
            Documentation(info="<html>
The modulated inductor is built in bondgraphic technology from a bond graph <b>mI</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the inductor in its environment. <p>

<hr>

<pre>
Potential variables:

 I:  Inductance (modulation input signal)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                90);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Passive.mI mI1(p(start=phi0)) 
                         annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          phi = u*i;
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],                    style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015], style(color=8));
          connect(B3.BondCon2, mI1.BondCon1) annotation (points=[7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(u, mI1.s) annotation (points=[0,90; 0,74.4; 0,58.8; 6.61287e-016,
                58.8], style(color=74, rgbcolor={0,0,127}));
        end VariableInductor;
      end Basic;
      
      package Ideal "Idealized analog electrical elements" 
        annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30,
                  fillPattern=1)),
            Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50],
                style(fillColor=30, fillPattern=1)),
            Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70],
                style(fillColor=30, fillPattern=1)),
            Text(
              extent=[-85, 35; 65, -85],
              string="Library",
              style(color=3)),
            Text(
              extent=[-118, 114; 122, 65],
              string="%name",
              style(color=1))),
          Documentation(info="<html>
This package offers a partial reimplementation of the <i>Ideal</i> package of the standard analog electrical Modelica library using bond graph technology.
</html>
"));
        
        model IdealCommutingSwitch 
          "Ideal leaking two-way switch of analog electrical library" 
          
        public 
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Positive electrical connector" 
                                   annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 
            "Second negative electrical connector" 
                                    annotation (extent=[90,-10; 110,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 
            "First negative electrical connector" 
                                    annotation (extent=[90,40; 110,60]);
          Modelica.Blocks.Interfaces.BooleanInput control 
            "true => p--n2 connected, false => p--n1 connected" annotation (extent=[
                -20,80; 20,120], rotation=-90);
          annotation (
            Icon(
              Ellipse(extent=[-44,4; -36,-4]),
              Text(extent=[-80,0; -60,22],   string="p"),
              Text(extent=[60,50; 80,72],   string="n1"),
              Text(extent=[60,0; 80,22],   string="n2"),
              Line(points=[-90,0; -44,0]),
              Line(points=[-37,2; 40,50]),
              Line(points=[40,50; 90,50]),
              Line(points=[40,0; 90,0]),
              Text(extent=[-100,-70; 100,-100],   string="%name"),
              Line(points=[0,25; 0,80], style(color=5, rgbcolor={255,0,255}))),
            Diagram,
            Documentation(info="<html>
Ideal commuting electrical switch element with leakage. This switch can be used to model a two-way switch.
If <i>control = true</i>, the switch is in the p-n2 connection.
If <i>control = false</i>, the switch is in the p-n1 connection. <p>

<hr>

<pre>
Control variables:

 control:     Boolean variable denoting the switch position (modulation signal)


Parameters:

 Ron:         Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:        Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre> <p>

<hr> <p>

Logic: <p>

<b>contol = true:   p-n2 is connected</b> <br>
<b>contol = false:  p-n1 is connected</b>
</html>
"));
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[-80,-10;
                -60,10]);
          Interfaces.BG2El BG2El1                           annotation (extent=[60,40;
                80,60]);
          Interfaces.BG2El BG2El2                           annotation (extent=[60,-10;
                80,10]);
          Switches.Sw4 Sw1(Ron=Ron, Goff=Goff) 
            annotation (extent=[-20,-20; 20,20]);
          Bonds.Bond B3         annotation (extent=[20,0; 40,20]);
          Bonds.Bond B2         annotation (extent=[20,-10; 40,10]);
          Bonds.Bond B1         annotation (extent=[-40,-10; -20,10]);
        equation 
          connect(El2BG1.p, p) annotation (points=[-80,0; -100,0],
                                              style(color=3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n2) annotation (points=[80,0; 100,0],
                                            style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n1) annotation (points=[80,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(B1.BondCon2, Sw1.p) annotation (points=[-20,0; -20,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(B1.BondCon1, El2BG1.BondCon1) annotation (points=[-39.8,0;
                -60,0], style(color=8, rgbcolor={192,192,192}));
          connect(B2.BondCon1, Sw1.n2) annotation (points=[20.2,0; 20,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(B2.BondCon2, BG2El2.BondCon1) annotation (points=[40,0; 60,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon1, Sw1.n1) annotation (points=[20.2,10; 20,10],
              style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon2, BG2El1.BondCon1) annotation (points=[40,10; 50,
                10; 50,50; 60,50], style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.control, control) annotation (points=[0,20; 0,100], style(
                color=5, rgbcolor={255,0,255}));
        end IdealCommutingSwitch;
        
        model IdealCommutingSwitch2 
          "Ideal two-way switch of analog electrical library if one branch is unconnected" 
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[-80,-10;
                -60,10]);
          Interfaces.BG2El BG2El1                           annotation (extent=[60,40;
                80,60]);
          Interfaces.BG2El BG2El2                           annotation (extent=[60,-10;
                80,10]);
        public 
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Positive electrical connector" 
                                   annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 
            "Second negative electrical connector" 
                                    annotation (extent=[90,-10; 110,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 
            "First negative electrical connector" 
                                    annotation (extent=[90,40; 110,60]);
          Modelica.Blocks.Interfaces.BooleanInput control 
            "true => p--n2 connected, false => p--n1 connected" annotation (extent=[-20,80;
                20,120],         rotation=-90);
        protected 
          Switches.Sw3 Sw1 annotation (extent=[-20,-20; 20,20]);
          Bonds.Bond B1         annotation (extent=[-40,-10; -20,10]);
          Bonds.Bond B2         annotation (extent=[20,-10; 40,10]);
          Bonds.Bond B3         annotation (extent=[20,0; 40,20]);
        equation 
          connect(El2BG1.p,p)  annotation (points=[-80,0; -100,0],
                                              style(color=3, rgbcolor={0,0,255}));
          connect(BG2El2.n,n2)  annotation (points=[80,0; 100,0],
                                            style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n,n1)  annotation (points=[80,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          annotation (
            Diagram,
            Icon(
              Ellipse(extent=[-44,4; -36,-4]),
              Text(extent=[-80,0; -60,22],   string="p"),
              Text(extent=[60,50; 80,72],   string="n1"),
              Text(extent=[60,0; 80,22],   string="n2"),
              Line(points=[-90,0; -44,0]),
              Line(points=[-37,2; 40,50]),
              Line(points=[40,50; 90,50]),
              Line(points=[40,0; 90,0]),
              Text(extent=[-100,-70; 100,-100],   string="%name"),
              Line(points=[0,25; 0,80], style(color=5, rgbcolor={255,0,255}))),
            Documentation(info="<html>
This is a special electrical switch used to check whether anything is connected from the outside to the connector n2.
If this is the case, the switch should be in the p-n2 position.
If this is not the case, the switch should be in the p-n1 position. <p>

This commuting switch must be used instead of the corresponding <b>IdealCommutingSwitch</b> model, because Dymola
sets by default the flow of an unconnected terminal to zero.  Hence we must not set it to zero once more
externally.
</html>
"));
          connect(Sw1.control, control) annotation (points=[0,20; 0,100], style(
                color=5, rgbcolor={255,0,255}));
          connect(B1.BondCon2, Sw1.p) annotation (points=[-20,0; -20,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(B1.BondCon1, El2BG1.BondCon1) annotation (points=[-39.8,0;
                -60,0], style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.n2, B2.BondCon1) annotation (points=[20,0; 20.2,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(B2.BondCon2, BG2El2.BondCon1) annotation (points=[40,0; 60,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.n1, B3.BondCon1) annotation (points=[20,10; 20.2,10],
              style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon2, BG2El1.BondCon1) annotation (points=[40,10; 50,
                10; 50,50; 60,50], style(color=8, rgbcolor={192,192,192}));
        end IdealCommutingSwitch2;
        
        model IdealIntermediateSwitch "Ideal intermediate switch" 
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
        public 
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 
            "First positive electrical connector" 
                                    annotation (extent=[-110,40; -90,60]);
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 
            "Second positive electrical connector" 
                                    annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 
            "First negative electrical connector" 
                                    annotation (extent=[90,40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 
            "Second negative electrical connector" 
                                    annotation (extent=[90,-10; 110,10]);
          Modelica.Blocks.Interfaces.BooleanInput control 
            "true => p1--n2, p2--n1 connected, otherwise p1--n1, p2--n2  connected"
            annotation (extent=[-20,80; 20,120],rotation=-90);
          annotation (
            Icon(
              Ellipse(extent=[-4,30; 4,22]),
              Text(extent=[-80,50; -60,72],   string="p1"),
              Text(extent=[-80,0; -60,22],   string="p2"),
              Text(extent=[60,50; 80,72],   string="n1"),
              Text(extent=[60,0; 80,22],   string="n2"),
              Line(points=[-90,0; -42,0]),
              Line(points=[-90,50; -42,50]),
              Line(points=[-42,0; 42,50]),
              Line(points=[-42,50; 42,0]),
              Line(points=[42,50; 90,50]),
              Line(points=[0,90; 0,25], style(color=5, rgbcolor={255,0,255})),
              Line(points=[42,0; 90,0]),
              Text(extent=[-100,-56; 100,-86],    string="%name")),
            Diagram(
              Ellipse(extent=[58,52; 62,48], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[78,2; 82,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[-2,2; 2,-2], style(
                  color=5,
                  rgbcolor={255,0,255},
                  fillColor=5,
                  rgbfillColor={255,0,255}))),
            Documentation(info="<html>
The ideal leaking electrical intermediate switch is built as a macro element from two two-way switches. <p>

The logic is such that the two switches are connected in parallel, when the Boolean control signal assumes a value of <b>true</b>.  The are connected over the cross, when the control signal assumes a value of <b>false</b>. <p>

<hr>

<pre>
Parameters:

 Ron:    Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre>
</html>
"));
          IdealCommutingSwitch Sw1(Ron=Ron, Goff=Goff) 
            annotation (extent=[-20,60; 20,20]);
          IdealCommutingSwitch Sw2(Ron=Ron, Goff=Goff) 
            annotation (extent=[-20,-60; 20,-20]);
        equation 
          connect(Sw2.control, control)                   annotation (points=[0,
                -20; 0,0; -40,0; -40,70; 0,70; 0,100], style(color=5, rgbcolor=
                  {255,0,255}));
          connect(Sw1.control, Sw2.control) 
            annotation (points=[0,20; 0,-20], style(color=5, rgbcolor={255,0,
                  255}));
          connect(Sw1.p, p1)                   annotation (points=[-20,40; -60,
                40; -60,50; -100,50], style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.p, p2)                   annotation (points=[-20,-40; -60,
                -40; -60,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(n1, Sw2.n1)                   annotation (points=[100,50; 60,
                50; 60,-30; 20,-30], style(color=3, rgbcolor={0,0,255}));
          connect(n2, Sw1.n1)                   annotation (points=[100,0; 40,0;
                40,30; 20,30], style(color=3, rgbcolor={0,0,255}));
          connect(Sw1.n2, n1)                   annotation (points=[20,40; 40,
                40; 40,50; 100,50], style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.n2, n2)                   annotation (points=[20,-40; 80,
                -40; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end IdealIntermediateSwitch;
        
        model ControlledIdealCommutingSwitch "Controlled ideal two-way switch" 
          parameter Modelica.SIunits.Voltage level=0.5 "Switch level";
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
        public 
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "Positive electrical connector" 
                                   annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 
            "Second negative electrical connector" 
                                    annotation (extent=[90,-10; 110,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 
            "First negative electrical connector" 
                                    annotation (extent=[90,40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin control 
            "Control pin: if control.v > level p--n2 connected, otherwise p--n1 connected"
             annotation (extent=[-10,90; 10,110],   rotation=90);
          annotation (
            Icon(
              Ellipse(extent=[-44,4; -36,-4]),
              Text(extent=[-80,0; -60,22],   string="p"),
              Text(extent=[60,50; 80,72],   string="n1"),
              Text(extent=[60,0; 80,22],   string="n2"),
              Line(points=[-90,0; -44,0]),
              Line(points=[-37,2; 40,50]),
              Line(points=[40,50; 90,50]),
              Line(points=[0,90; 0,25]),
              Line(points=[40,0; 90,0]),
              Text(extent=[-100,-50; 100,-80],    string="%name")),
            Diagram(Ellipse(extent=[-82,2; -78,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)), Ellipse(extent=[-2,82; 2,78], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1))),
            Documentation(info="<html>
The ideal leaking controlled electrical commuting switch is built as a macro element from an ideal controlled opening switch and an ideal controlled closing switch. <p>

The logic is such that the top switch is considered <b>open</b> and the bottom switch is considered <b>closed</b>, when the real-valued control signal assumes a value larger than a threshold value, called <b>level</b>. <p>

<hr>

<pre>
Parameters:

 Ron:    Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre>
</html>
"));
          ControlledIdealOpeningSwitch Sw1(
            level=level,
            Ron=Ron,
            Goff=Goff) 
            annotation (extent=[-20,20; 20,60]);
          ControlledIdealClosingSwitch Sw2(
            level=level,
            Ron=Ron,
            Goff=Goff) 
            annotation (extent=[-20,-60; 20,-20]);
        equation 
          connect(Sw2.control, control)                           annotation (
              points=[0,-20; 0,10; 40,10; 40,80; 0,80; 0,100], style(color=3,
                rgbcolor={0,0,255}));
          connect(Sw1.control, control)                           annotation (
              points=[0,60; 0,100], style(color=3, rgbcolor={0,0,255}));
          connect(Sw1.n, n1)                           annotation (points=[20,
                40; 60,40; 60,50; 100,50], style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.n, n2)                           annotation (points=[20,
                -40; 60,-40; 60,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Sw1.p, p)                           annotation (points=[-20,
                40; -80,40; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.p, p)                           annotation (points=[-20,
                -40; -80,-40; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
        end ControlledIdealCommutingSwitch;
        
        model ControlledIdealIntermediateSwitch 
          "Controlled ideal intermediate switch" 
          parameter Modelica.SIunits.Voltage level=0.5 "Switch level";
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
        public 
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 
            "First positive electrical connector" 
                                    annotation (extent=[-110,40; -90,60]);
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 
            "Second positive electrical connector" 
                                    annotation (extent=[-110,-10; -90,10]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 
            "First negative electrical connector" 
                                    annotation (extent=[90,40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 
            "Second negative electrical connector" 
                                    annotation (extent=[90,-10; 110,10]);
          Modelica.Electrical.Analog.Interfaces.Pin control "Control pin: if control.v > level p1--n2, p2--n1 connected,
         otherwise p1--n1, p2--n2  connected"       annotation (extent=[-10,90;
                10,110],
                      rotation=90);
          annotation (
            Icon(
              Ellipse(extent=[-4,30; 4,22]),
              Text(extent=[-80,50; -60,72],   string="p1"),
              Text(extent=[-80,0; -60,22],   string="p2"),
              Text(extent=[60,50; 80,72],   string="n1"),
              Text(extent=[60,0; 80,22],   string="n2"),
              Line(points=[-90,0; -40,0]),
              Line(points=[-90,50; -39,50]),
              Line(points=[-40,0; 40,50]),
              Line(points=[-40,50; 40,0]),
              Line(points=[40,50; 90,50]),
              Line(points=[0,90; 0,25]),
              Line(points=[40,0; 90,0]),
              Text(extent=[-100,-54; 100,-84],    string="%name")),
            Diagram(
              Ellipse(extent=[78,2; 82,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[58,52; 62,48], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[-2,2; 2,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255}))),
            Documentation(info="<html>
The ideal leaking controlled electrical intermediate switch is built as a macro element from two controlled two-way switches. <p>

The logic is such that the two switches are connected in parallel, when the Boolean control signal assumes a value of <b>true</b>.  The are connected over the cross, when the control signal assumes a value of <b>false</b>. <p>

<hr>

<pre>
Parameters:

 Ron:    Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre>
</html>
"));
          ControlledIdealCommutingSwitch Sw1(
            level=level,
            Ron=Ron,
            Goff=Goff) 
            annotation (extent=[-20,60; 20,20]);
          ControlledIdealCommutingSwitch Sw2(
            level=level,
            Ron=Ron,
            Goff=Goff) 
            annotation (extent=[-20,-60; 20,-20]);
        equation 
          connect(control, Sw2.control)                             annotation (
             points=[0,100; 0,70; -40,70; -40,0; 0,0; 0,-20], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Sw1.control, Sw2.control)          annotation (points=[0,20;
                0,-20], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Sw1.p, p1)                             annotation (points=[
                -20,40; -60,40; -60,50; -100,50], style(color=3, rgbcolor={0,0,
                  255}));
          connect(Sw2.p, p2)                             annotation (points=[
                -20,-40; -60,-40; -60,0; -100,0], style(color=3, rgbcolor={0,0,
                  255}));
          connect(Sw1.n2, n1)                             annotation (points=[
                20,40; 40,40; 40,50; 100,50], style(color=3, rgbcolor={0,0,255}));
          connect(Sw1.n1, n2)                             annotation (points=[
                20,30; 40,30; 40,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.n1, n1)                             annotation (points=[
                20,-30; 60,-30; 60,50; 100,50], style(color=3, rgbcolor={0,0,
                  255}));
          connect(Sw2.n2, n2)                             annotation (points=[
                20,-40; 80,-40; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end ControlledIdealIntermediateSwitch;
        
        model IdealDiode "Ideal leaking diode of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
          parameter Modelica.SIunits.Voltage Vknee(final min=0) = 0 
            "Forward threshold voltage";
          annotation (
            Icon(
              Polygon(points=[30, 0; -30, 40; -30, -40; 30, 0], style(
                  color=3,
                  gradient=0,
                  fillColor=7)),
              Line(points=[-90, 0; 40, 0]),
              Line(points=[40, 0; 90, 0]),
              Line(points=[30, 40; 30, -40], style(color=3)),
              Text(extent=[-102, 94; 98, 64], string="%name")),
            Diagram,
            Documentation(info="<html>
The ideal leaking electrical diode is built in bondgraphic technology from a bond graph <b>D2</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the diode in its environment.  A leaking diode has been used to ensure that the diode will not lead to a division by zero in either switch position, irrespective of the circuit topology. <p>

<hr>

<pre>
Parameters:

 Ron:     Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:    Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)

 Vknee:   Forward threshold voltage (default = 0 Volt)
</pre>
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -12; -12, 12]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -12; 38, 12]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, 12; 12, 36], rotation=
                90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Switches.D2 D1(Ron=Ron, Goff=Goff,
            e0=Vknee)                                annotation (extent=[-16,
                36; 16, 68], rotation=90);
        equation 
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12, 0; -12,
                  0], style(color=8));
          connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[12, 0;
                12.26, 0], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-38,
                1.77636e-015; -38,1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(D1.BondCon1, B3.BondCon2) annotation (points=[-9.79685e-016,
                36; 7.34764e-016,36],
              style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -78,
                1.77636e-015; -78,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end IdealDiode;
        
        model IdealZenerDiode 
          "Ideal leaking Zener diode of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Resistance Raval(final min=0) = 1.E-5 
            "Avalanche switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
          parameter Modelica.SIunits.Voltage v0(final min=0) = 1 
            "Avalanche breakdown voltage";
          annotation (
            Icon(
              Polygon(points=[-20,0; 30,40; 30,-40; -20,0],     style(
                  color=3,
                  gradient=0,
                  fillColor=7)),
              Line(points=[-90,0; 40,0]),
              Line(points=[40,0; 90,0]),
              Line(points=[-20,40; -20,-40], style(color=3)),
              Text(extent=[-102,94; 98,64],   string="%name"),
              Line(points=[-20,-40; 0,-60],  style(color=3)),
              Line(points=[-40,60; -20,40],  style(color=3)),
              Text(extent=[-58,-62; 60,-96], string="v0=%v0")),
            Diagram,
            Documentation(info="<html>
The ideal leaking electrical Zener diode is built in bondgraphic technology from a bond graph <b>Z2</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the diode in its environment.  A leaking diode has been used to ensure that the diode will not lead to a division by zero in either switch position, irrespective of the circuit topology. <p>

Zener diodes are always used as reversed diodes.  Thus, the positive and negative pins have been reversed in the model. <p>

<hr>

<pre>
Parameters:

 Ron:    Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)

 Raval:  Leakage resistance in <b>breakdown</b> state (default value = 1e-5 Ohm)

 v0:     Avalanche breakdown voltage (default = 1 Volt)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                90);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Switches.Z2 Z1(
            Ron=Ron,
            Raval=Raval,
            Goff=Goff,
            e0=v0) annotation (extent=[-12,36; 12,68], rotation=90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],   style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(B3.BondCon2, Z1.BondCon1) annotation (points=[7.34764e-016,36;
                -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
        end IdealZenerDiode;
        
        model IdealTunnelDiode 
          "Ideal leaking tunnel diode of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance R1(final min=0) = 1.E-5 
            "First closed switch resistance";
          parameter Modelica.SIunits.Resistance R2(final min=0) = 1.E-5 
            "Second closed switch resistance";
          parameter Modelica.SIunits.Resistance R3(final min=0) = 1.E-5 
            "Third closed switch resistance";
          parameter Modelica.SIunits.Conductance G0(final min=0) = 1.E-5 
            "Opened switch conductance";
          parameter Modelica.SIunits.Voltage em1(final min=0) = 1 
            "First knee voltage";
          parameter Modelica.SIunits.Voltage em2(final min=em1) = 2 
            "Second knee voltage";
          parameter Modelica.SIunits.Voltage em3(final min=em2) = 3 
            "Third knee voltage";
          parameter Modelica.SIunits.Voltage e0(final min=em3) = 4 
            "Fourth knee voltage";
          parameter Modelica.SIunits.Current fm(final min=0) = 1 
            "Bottom intermediate current";
          parameter Modelica.SIunits.Current f0(final min=fm) = 2 
            "Top intermediate current";
          annotation (
            Icon(
              Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                  color=3,
                  gradient=0,
                  fillColor=7)),
              Line(points=[-90,0; 40,0]),
              Line(points=[40,0; 90,0]),
              Line(points=[30,40; 30,-40],   style(color=3)),
              Text(extent=[-102,94; 98,64],   string="%name"),
              Line(points=[10,60; 30,40],    style(color=3)),
              Line(points=[30,-40; 10,-60],  style(color=3))),
            Diagram,
            Documentation(info="<html>
The ideal leaking electrical tunnel diode is built in bondgraphic technology from a bond graph <b>T2</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the diode in its environment.  A leaking diode has been used to ensure that the diode will not lead to a division by zero in either switch position, irrespective of the circuit topology. <p>

<hr>

<pre>
Parameters:

 G0:    Leakage conductance in blocking state (default value = 1e-5 Mho)

 R1:    Leakage resistance in first conducting state (default value = 1e-5 Ohm)

 R2:    Leakage resistance in second conducting state (default value = 1e-5 Ohm)

 R3:    Leakage resistance in final conducting state (default value = 1e-5 Ohm)

 em1:   First knee voltage (default value = 1 Volt)

 em2:   Second knee voltage (default value = 2 Volt)

 em3:   Third knee voltage (default value = 3 Volt)

 e0:    Fourth knee voltage (default value = 4 Volt)

 fm:    Bottom intermediate current (default value = 1 Amp)

 f0:    Top intermediate current (default value = 2 Amp)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                90);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Switches.T2 T1(
            G0=G0,
            R1=R1,
            R2=R2,
            R3=R3,
            em1=em1,
            em2=em2,
            em3=em3,
            e0=e0,
            fm=fm,
            f0=f0) annotation (extent=[-12,36; 12,68], rotation=90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],   style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 82,
                1.77636e-015; 82,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(B3.BondCon2, T1.BondCon1) annotation (points=[7.34764e-016,36;
                -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
        end IdealTunnelDiode;
        
        model IdealTransformer 
          "Ideal linear transformer of analog electrical library" 
          extends Interfaces.TwoPort;
          parameter Real n=1 "Turns ratio";
          annotation (
            Icon(
              Ellipse(extent=[-45, -50; -20, -25]),
              Ellipse(extent=[-45, -25; -20, 0]),
              Ellipse(extent=[-45, 0; -20, 25]),
              Ellipse(extent=[-45, 25; -20, 50]),
              Rectangle(extent=[-72, -60; -33, 60], style(color=7, fillColor=7)),
              Line(points=[-90, 50; -32, 50]),
              Line(points=[-90, -50; -32, -50]),
              Ellipse(extent=[20, -50; 45, -25]),
              Ellipse(extent=[20, -25; 45, 0]),
              Ellipse(extent=[20, 0; 45, 25]),
              Ellipse(extent=[20, 25; 45, 50]),
              Rectangle(extent=[33, -60; 72, 60], style(color=7, fillColor=7)),
              Line(points=[32, 50; 90, 50]),
              Line(points=[32, -50; 90, -50]),
              Text(extent=[-100, -68; 100, -88], string="n=%n"),
              Text(extent=[-100, 92; 100, 72], string="%name")),
            Diagram(
              Text(
                extent=[-38, 26; -28, 18],
                string="v1a",
                style(color=9)),
              Text(
                extent=[-38, -14; -28, -22],
                style(color=9),
                string="v1b"),
              Text(
                extent=[42, -14; 52, -22],
                style(color=9),
                string="v2b"),
              Text(
                extent=[42, 26; 52, 18],
                style(color=9),
                string="v2a"),
              Text(
                extent=[-26, 8; -16, 0],
                style(color=9),
                string="v1"),
              Text(
                extent=[12, 8; 24, 0],
                style(color=9),
                string="-v2"),
              Text(
                extent=[-54, 26; -44, 18],
                style(color=9),
                string="i1"),
              Text(
                extent=[-24, -4; -16, -12],
                style(color=9),
                string="i1"),
              Text(
                extent=[-54, -16; -44, -24],
                style(color=9),
                string="i1"),
              Text(
                extent=[14, -4; 26, -12],
                style(color=9),
                string="i2"),
              Text(
                extent=[26, 24; 36, 16],
                style(color=9),
                string="i2"),
              Text(
                extent=[26, -16; 36, -24],
                style(color=9),
                string="i2")),
            Documentation(info="<html>
The ideal transformer is built in bondgraphic technology from a bond graph <b>TF</b> elements.  A-causal bonds must be used here, because the causality depends on the embedding of the transformer in its environment. <p>

<hr>

<pre>
Parameters:

 n:  Transformation constant (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>v1 = -n*v2</b> <br>
<b>i2 = n*11</b>
</html>
"));
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-70, 40; -50, 60]);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [70, 40; 50, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [50, -60; 70, -40]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El2 annotation (extent=
                [-50, -60; -70, -40]);
          BondLib.Passive.TF TF1(m=n) annotation (extent=[-10, -10; 10, 10]);
          BondLib.Bonds.Bond B1 annotation (extent=[-30, -10; -10, 10]);
          BondLib.Bonds.Bond B2 annotation (extent=[10, -10; 30, 10]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-50, -10; -30, 10],
                rotation=90);
          BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[50, -10; 30, 10],
              rotation=90);
          BondLib.Bonds.Bond B3 annotation (extent=[-50, 10; -30, 30], rotation=
               -90);
          BondLib.Bonds.Bond B4 annotation (extent=[-50, -30; -30, -10],
              rotation=-90);
          BondLib.Bonds.Bond B5 annotation (extent=[30, 10; 50, 30], rotation=-
                90);
          BondLib.Bonds.Bond B6 annotation (extent=[30, -30; 50, -10], rotation=
               -90);
        equation 
          connect(B1.BondCon2, TF1.BondCon1) annotation (points=[-10, 0; -10, 0],
                   style(color=8));
          connect(TF1.BondCon2, B2.BondCon1) annotation (points=[10, 0; 10.2, 0],
                   style(color=8));
          connect(J1p3_1.BondCon3, B1.BondCon1) annotation (points=[-30,
                -6.12303e-016; -30,-6.12303e-016; -30,0; -29.8,0],
                      style(color=8));
          connect(B2.BondCon2, J1p3_2.BondCon3) annotation (points=[30,0; 30,
                -6.12303e-016],
                    style(color=8));
          connect(B3.BondCon2, J1p3_1.BondCon2) annotation (points=[-40, 10; -
                40, 10], style(color=8));
          connect(B5.BondCon2, J1p3_2.BondCon2) annotation (points=[40, 10; 40,
                  10], style(color=8));
          connect(J1p3_2.BondCon1, B6.BondCon1) annotation (points=[40, -10; 40,
                  -10.2], style(color=8));
          connect(J1p3_1.BondCon1, B4.BondCon1) annotation (points=[-40, -10; -
                40, -10.2], style(color=8));
          connect(El2BG1.BondCon1, B3.BondCon1) annotation (points=[-50,50; -40,
                50; -40,29.8],    style(color=8));
          connect(BG2El2.BondCon1, B4.BondCon2) annotation (points=[-50, -50; -
                40, -50; -40, -30], style(color=8));
          connect(El2BG2.BondCon1, B5.BondCon1) annotation (points=[50,50; 40,
                50; 40,29.8],  style(color=8));
          connect(BG2El1.BondCon1, B6.BondCon2) annotation (points=[50, -50; 40,
                  -50; 40, -30], style(color=8));
          connect(El2BG1.p, p1) annotation (points=[-70,50; -100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n1) annotation (points=[-70,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(El2BG2.p, p2) annotation (points=[70,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n2) annotation (points=[70,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
        end IdealTransformer;
        
        model IdealGyrator "Ideal linear gyrator of analog electrical library" 
          extends Interfaces.TwoPort;
          parameter Modelica.SIunits.Conductance G=1 "Gyration conductance";
          annotation (
            Icon(
              Ellipse(extent=[-70, -30; -10, 30]),
              Rectangle(extent=[-80, -40; -41, 40], style(
                  color=7,
                  pattern=1,
                  thickness=1,
                  arrow=0,
                  fillColor=7)),
              Line(points=[-90, 50; -40, 50; -40, -50; -90, -50]),
              Text(extent=[-100, 98; 100, 68], string="%name"),
              Line(points=[-30, 60; 20, 60]),
              Polygon(points=[20, 63; 30, 60; 20, 57; 20, 63], style(
                  pattern=1,
                  thickness=1,
                  arrow=0,
                  fillColor=3,
                  fillPattern=1)),
              Ellipse(extent=[10, -30; 70, 30]),
              Rectangle(extent=[80, -40; 40, 40], style(
                  color=7,
                  pattern=1,
                  thickness=1,
                  arrow=0,
                  fillColor=7)),
              Line(points=[90, 50; 40, 50; 40, -50; 90, -50])),
            Diagram(
              Text(
                extent=[-38, 26; -28, 18],
                string="v1a",
                style(color=9)),
              Text(
                extent=[-38, -14; -28, -22],
                style(color=9),
                string="v1b"),
              Text(
                extent=[-26, 8; -16, 0],
                style(color=9),
                string="v1"),
              Text(
                extent=[-54, 26; -44, 18],
                style(color=9),
                string="i1"),
              Text(
                extent=[-24, -4; -16, -12],
                style(color=9),
                string="i1"),
              Text(
                extent=[-54, -16; -44, -24],
                style(color=9),
                string="i1"),
              Text(
                extent=[42, -14; 52, -22],
                style(color=9),
                string="v2b"),
              Text(
                extent=[42, 26; 52, 18],
                style(color=9),
                string="v2a"),
              Text(
                extent=[14, -4; 26, -12],
                style(color=9),
                string="i2"),
              Text(
                extent=[26, 24; 36, 16],
                style(color=9),
                string="i2"),
              Text(
                extent=[26, -16; 36, -24],
                style(color=9),
                string="i2"),
              Text(
                extent=[12, 8; 24, 0],
                style(color=9),
                string="-v2")),
            Documentation(info="<html>
The ideal gyrator is built in bondgraphic technology from a bond graph <b>GY</b> elements.  A-causal bonds must be used here, because the causality depends on the embedding of the gyrator in its environment. <p>

<hr>

<pre>
Parameters:

 r:  Gyration constant (default value = 1)
</pre> <p>

<hr> <p>

Equations: <p>

<b>v1 = r*i2</b> <br>
<b>v2 = -r*i1</b>
</html>
"));
          
        protected 
          BondLib.Passive.GY GY1(r=-G) annotation (extent=[-10, -10; 10, 10]);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-70, 40; -50, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [-50, -60; -70, -40]);
          BondLib.Bonds.Bond B1 annotation (extent=[-30, -10; -10, 10]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-50, -10; -30, 10],
                rotation=90);
          BondLib.Bonds.Bond B2 annotation (extent=[-50, 10; -30, 30], rotation=
               -90);
          BondLib.Bonds.Bond B3 annotation (extent=[-50, -30; -30, -10],
              rotation=-90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [70, 40; 50, 60]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El2 annotation (extent=
                [50, -60; 70, -40]);
          BondLib.Bonds.Bond B4 annotation (extent=[10, -10; 30, 10]);
          BondLib.Junctions.J1p3 J1p3_2 annotation (extent=[50, -10; 30, 10],
              rotation=90);
          BondLib.Bonds.Bond B5 annotation (extent=[30, 10; 50, 30], rotation=-
                90);
          BondLib.Bonds.Bond B6 annotation (extent=[30, -30; 50, -10], rotation=
               -90);
        equation 
          connect(J1p3_1.BondCon3, B1.BondCon1) annotation (points=[-30,
                -6.12303e-016; -30,-6.12303e-016; -30,0; -29.8,0],
                      style(color=8));
          connect(B2.BondCon2, J1p3_1.BondCon2) annotation (points=[-40, 10; -
                40, 10], style(color=8));
          connect(J1p3_1.BondCon1, B3.BondCon1) annotation (points=[-40, -10; -
                40, -10.2], style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon1) annotation (points=[-50,50; -40,
                50; -40,29.8],    style(color=8));
          connect(BG2El1.BondCon1, B3.BondCon2) annotation (points=[-50, -50; -
                40, -50; -40, -30], style(color=8));
          connect(B1.BondCon2, GY1.BondCon1) annotation (points=[-10, 0; -10, 0],
                   style(color=8));
          connect(B4.BondCon2, J1p3_2.BondCon3) annotation (points=[30,0; 30,
                -6.12303e-016],
                    style(color=8));
          connect(B5.BondCon2, J1p3_2.BondCon2) annotation (points=[40, 10; 40,
                  10], style(color=8));
          connect(J1p3_2.BondCon1, B6.BondCon1) annotation (points=[40, -10; 40,
                  -10.2], style(color=8));
          connect(El2BG2.BondCon1, B5.BondCon1) annotation (points=[50,50; 40,
                50; 40,29.8],  style(color=8));
          connect(BG2El2.BondCon1, B6.BondCon2) annotation (points=[50, -50; 40,
                  -50; 40, -30], style(color=8));
          connect(GY1.BondCon2, B4.BondCon1) annotation (points=[10, 0; 10.2, 0],
                   style(color=8));
          connect(El2BG1.p, p1) annotation (points=[-70,50; -100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n1) annotation (points=[-70,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(El2BG2.p, p2) annotation (points=[70,50; 100,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(BG2El2.n, n2) annotation (points=[70,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
        end IdealGyrator;
        
        model Idle "Open circuit of analog electrical library" 
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Rectangle(extent=[-60, 60; 60, -60]),
              Line(points=[-90, 0; -40, 0]),
              Line(points=[89, 0; 40, 0]),
              Text(extent=[-100, 98; 100, 68], string="%name")),
            Diagram,
            Documentation(info="<html>
The open circuit element sets the current in the branch equal to zero.  This is accomplished using a flow source with zero flow.
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[12, -12; 38, 12], rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38, -12; -12, 12],
              rotation=180);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Sources.Sf Sf1(f0=0) annotation (extent=[-12, 36; 12, 62],
              rotation=90);
          BondLib.Bonds.eBond B3 annotation (extent=[-12, 12; 12, 36], rotation=
               -90);
        equation 
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,
                1.77636e-015; 38,1.77636e-015; 38,-1.44014e-015; 37.74,
                -1.44014e-015], style(color=8));
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                1.46953e-015], style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                                 style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0; -12,
                -7.20069e-016; -12,-1.44014e-015; -12.26,-1.44014e-015],
                                  style(color=8));
          connect(B3.fBondCon1, Sf1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36],
                              style(color=8));
          connect(B3.eBondCon1, J1p3_1.BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],   style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end Idle;
        
        model Short "Short circuit of analog electrical library" 
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Rectangle(extent=[-60, 60; 60, -60]),
              Line(points=[91, 0; -90, 0]),
              Text(extent=[-100, 98; 100, 68], string="%name")),
            Diagram,
            Documentation(info="<html>
The closed circuit element sets the voltage in the branch equal to zero.  This is accomplished using an effort source with zero effort.
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[12, -12; 38, 12], rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38, -12; -12, 12],
              rotation=180);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Sources.Se Se1(e0=0) annotation (extent=[-14, 36; 14, 64],
              rotation=90);
          BondLib.Bonds.fBond B3 annotation (extent=[-12, 12; 12, 36], rotation=
               -90);
        equation 
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,
                1.77636e-015; 38,1.77636e-015; 38,-1.44014e-015; 37.74,
                -1.44014e-015],
                    style(color=8));
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.eBondCon1, Se1.BondCon1) annotation (points=[-7.34764e-016,
                36; 9.19132e-016,36],
                              style(color=8));
          connect(B3.fBondCon1, J1p3_1.BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],   style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end Short;
        
        model IdealOpeningSwitch 
          "Ideal leaking switch of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
        public 
          Modelica.Blocks.Interfaces.BooleanInput control(start=false) 
            "true/false opened/closed switch"                     annotation (
              extent=[-10, 80; 10, 100], rotation=-90);
          annotation (
            Diagram,
            Icon(
              Ellipse(extent=[-44, 4; -36, -4]),
              Line(points=[-90, 0; -44, 0]),
              Line(points=[-37, 2; 40, 50]),
              Line(points=[40, 0; 90, 0]),
              Text(extent=[-100, -48; 100, -78], string="%name"),
              Line(points=[0, 80; 0, 26], style(color=83)),
              Line(points=[40,0; 40,20], style(color=3, rgbcolor={0,0,255}))),
            Documentation(info="<html>
The ideal leaking electrical opening switch is built in bondgraphic technology from a bond graph <b>Sw2</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the switch in its environment.  A leaking switch has been used to ensure that the switch will not lead to a division by zero in either switch position, irrespective of the circuit topology. <p>

The logic is such that the switch is considered <b>open</b>, when the Boolean control signal assumes a value of <b>true</b>. <p>

<hr>

<pre>
Parameters:

 Ron:    Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre>
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -12; -12, 12]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -12; 38, 12]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, 12; 12, 36], rotation=
                90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Switches.Sw2 Sw1(Ron=Ron, Goff=Goff) annotation (extent=[-14,
                  36; 14, 66], rotation=90);
        equation 
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12, 0; -12,
                  0], style(color=8));
          connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[12, 0;
                12.26, 0], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-38,
                1.77636e-015; -38,1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(Sw1.BondCon1, B3.BondCon2) annotation (points=[9.19132e-016,
                36; 7.34764e-016,36],
                style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -82,
                1.77636e-015; -82,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Sw1.BooleanInPort1, control) annotation (points=[2.54786e-015,
                64.5; 2.54786e-015,73.25; 0,73.25; 0,90], style(color=5,
                rgbcolor={255,0,255}));
        end IdealOpeningSwitch;
        
        model IdealClosingSwitch 
          "Ideal leaking switch of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
        public 
          Modelica.Blocks.Interfaces.BooleanInput control(start=false) 
            "true/false opened/closed switch"                     annotation (
              extent=[-10, 80; 10, 100], rotation=-90);
          annotation (
            Diagram,
            Icon(
              Ellipse(extent=[-44, 4; -36, -4]),
              Line(points=[-90, 0; -44, 0]),
              Line(points=[-37, 2; 40, 50]),
              Line(points=[40, 0; 90, 0]),
              Text(extent=[-100, -48; 100, -78], string="%name"),
              Line(points=[0, 80; 0, 26], style(color=83))),
            Documentation(info="<html>
The ideal leaking electrical closing switch is built in bondgraphic technology from a bond graph <b>Sw2</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the switch in its environment.  A leaking switch has been used to ensure that the switch will not lead to a division by zero in either switch position, irrespective of the circuit topology. <p>

The logic is such that the switch is considered <b>closed</b>, when the Boolean control signal assumes a value of <b>true</b>. <p>

<hr>

<pre>
Parameters:

 Ron:    Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:   Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre>
</html>
"));
          
        protected 
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,-8; 12,-32]);
          BondLib.Bonds.Bond B1 annotation (extent=[-38,-32; -12,-8]);
          BondLib.Bonds.Bond B2 annotation (extent=[12,-32; 38,-8]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12,-8; 12,16],   rotation=
                90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                -62,-34; -38,-6]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                -34; 64,-6]);
          BondLib.Switches.Sw2 Sw1(Ron=Ron, Goff=Goff) annotation (extent=[-14,
                16; 14,46],    rotation=90);
          Modelica.Blocks.Logical.Not Not1 
            annotation (extent=[-10,50; 10,70], rotation=-90);
        equation 
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12,-20;
                -12,-20],
                      style(color=8));
          connect(J1p3_1.BondCon2, B2.BondCon1) annotation (points=[12,-20;
                12.26,-20],style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                -7.20069e-016,-7.76; -7.20069e-016,12.12; 0,12.12; 0,-8],
                       style(color=8));
          connect(El2BG1.BondCon1, B1.BondCon1) annotation (points=[-38,-20;
                -37.74,-20],
                      style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38,-20; 38,
                -20],
                    style(color=8));
          connect(Sw1.BondCon1, B3.BondCon2) annotation (points=[9.19132e-016,
                16; 7.34764e-016,16],
                style(color=8));
          connect(Sw1.BooleanInPort1, Not1.y) annotation (points=[2.54786e-015,
                44.5; 2.54786e-015,47.25; 6.73533e-016,47.25; 6.73533e-016,49],
              style(color=5, rgbcolor={255,0,255}));
          connect(Not1.u, control) annotation (points=[-7.34764e-016,72; 0,72;
                0,90], style(color=5, rgbcolor={255,0,255}));
          connect(El2BG1.p, p) annotation (points=[-62,-20; -80,-20; -80,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,-20; 80,-20; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
        end IdealClosingSwitch;
        
        model ControlledIdealOpeningSwitch 
          "Controlled ideal leaking switch of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage level=0.5 "Switch level";
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
          Modelica.Electrical.Analog.Interfaces.PositivePin control 
            "Switch is open if control signal > switch level" 
            annotation (extent=[-10, 90; 10, 110]);
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-10, 60; 10, 80], rotation=-90);
          
          annotation (
            Diagram,
            Icon(
              Ellipse(extent=[-44, 4; -36, -4]),
              Line(points=[-90, 0; -44, 0]),
              Line(points=[-37, 2; 40, 50]),
              Line(points=[40, 0; 90, 0]),
              Line(points=[0, 90; 0, 25]),
              Text(extent=[-100, -44; 100, -74], string="%name"),
              Line(points=[40,0; 40,20], style(color=3, rgbcolor={0,0,255}))),
            Documentation(info="<html>
The controlled ideal leaking opening electrical switch is built in bondgraphic technology from a bond graph <b>Sw2</b> element, a 1-junction, and three bonds.  In addition, an effort detector senses the potential of the control input.  If that signal is larger than a threshold, called <i>level</i>, the switch is open.  Otherwise, it is closed.  This element is an idealized version of a transistor switch. <p>

A-causal bonds must mostly be used here, because the causality depends on the embedding of the switch in its environment.  A leaking switch has been used to ensure that the switch will not lead to a division by zero in either switch position, irrespective of the circuit topology. <p>

<hr>

<pre>
Parameters:

 level:   Potential threshold for switch control (default value = 0.5 Volt)

 Ron:     Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:    Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre>
</html>
"));
          
        protected 
          BondLib.Bonds.fBond B4 annotation (extent=[-10, 40; 10, 60], rotation=
               -90);
          BondLib.Sensors.De De1 annotation (extent=[-10, 20; 10, 40], rotation=
               -90);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -92; -12, -68]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -92; 38, -68]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, -68; 12, -44],
              rotation=90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [-62, -94; -38, -66]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -94; 64, -66]);
          BondLib.Switches.Sw2 Sw1(Ron=Ron, Goff=Goff) annotation (extent=[-14,
                  -44; 14, -14], rotation=90);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, -68; 12, -92]);
          Modelica.Blocks.Logical.GreaterThreshold GreaterThreshold1(threshold=level) 
            annotation (extent=[-10,-10; 10,10], rotation=-90);
        equation 
          connect(El2BG1.BondCon1, B4.eBondCon1) annotation (points=[
                6.12303e-016,60; -6.12303e-016,60],
                     style(color=8));
          connect(B4.fBondCon1, De1.BondCon1) annotation (points=[6.12303e-016,
                40; -6.12303e-016,40],
                style(color=8));
          connect(Sw1.BondCon1, B3.BondCon2) annotation (points=[9.19132e-016,
                -44; 7.34764e-016,-44],  style(color=8));
          connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[0,-68; 0,
                -67.88; 0,-67.76; -7.20069e-016,-67.76],
                         style(color=8));
          connect(B2.BondCon1, J1p3_1.BondCon2) annotation (points=[12.26,-80;
                12,-80],style(color=8));
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12, -80; -
                12, -80], style(color=8));
          connect(El2BG2.BondCon1, B1.BondCon1) annotation (points=[-38,-80;
                -37.74,-80],
                          style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38, -80; 38,
                  -80], style(color=8));
          connect(GreaterThreshold1.y, Sw1.BooleanInPort1) annotation (points=[
                6.73533e-016,-11; 6.73533e-016,-9.5; 2.54786e-015,-9.5;
                2.54786e-015,-15.5], style(color=5, rgbcolor={255,0,255}));
          connect(De1.OutPort1, GreaterThreshold1.u) annotation (points=[
                6.73533e-016,19; 6.73533e-016,14.5; -7.34764e-016,14.5;
                -7.34764e-016,12], style(color=74, rgbcolor={0,0,127}));
          connect(El2BG2.p, p) annotation (points=[-62,-80; -80,-80; -80,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,-80; 80,-80; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(El2BG1.p, control) annotation (points=[-6.12303e-016,80;
                -6.12303e-016,88; 0,88; 0,100], style(color=3, rgbcolor={0,0,
                  255}));
        end ControlledIdealOpeningSwitch;
        
        model ControlledIdealClosingSwitch 
          "Controlled ideal leaking switch of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage level=0.5 "Switch level";
          parameter Modelica.SIunits.Resistance Ron(final min=0) = 1.E-5 
            "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min=0) = 1.E-5 
            "Opened switch conductance";
          Modelica.Electrical.Analog.Interfaces.PositivePin control 
            "Switch is open if control signal < switch level" 
            annotation (extent=[-10, 90; 10, 110]);
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-10, 60; 10, 80], rotation=-90);
          
          annotation (
            Diagram,
            Icon(
              Ellipse(extent=[-44, 4; -36, -4]),
              Line(points=[-90, 0; -44, 0]),
              Line(points=[-37, 2; 40, 50]),
              Line(points=[40, 0; 90, 0]),
              Line(points=[0, 90; 0, 25]),
              Text(extent=[-100, -44; 100, -74], string="%name")),
            Documentation(info="<html>
The controlled ideal leaking closing electrical switch is built in bondgraphic technology from a bond graph <b>Sw2</b> element, a 1-junction, and three bonds.  In addition, an effort detector senses the potential of the control input.  If that signal is smaller than a threshold, called <i>level</i>, the switch is open.  Otherwise, it is closed.  This element is an idealized version of a transistor switch. <p>

A-causal bonds must mostly be used here, because the causality depends on the embedding of the switch in its environment.  A leaking switch has been used to ensure that the switch will not lead to a division by zero in either switch position, irrespective of the circuit topology. <p>

<hr>

<pre>
Parameters:

 level:   Potential threshold for switch control (default value = 0.5 Volt)

 Ron:     Leakage resistance in <b>on</b> state (default value = 1e-5 Ohm)

 Goff:    Leakage conductance in <b>off</b> state (default value = 1e-5 Mho)
</pre>
</html>
"));
          
        protected 
          BondLib.Bonds.fBond B4 annotation (extent=[-10, 40; 10, 60], rotation=
               -90);
          BondLib.Sensors.De De1 annotation (extent=[-10, 20; 10, 40], rotation=
               -90);
          Modelica.Blocks.Logical.LessThreshold LessThan1(    threshold=level) 
            annotation (extent=[-10, -8; 10, 12], rotation=-90);
          BondLib.Bonds.Bond B1 annotation (extent=[-38, -92; -12, -68]);
          BondLib.Bonds.Bond B2 annotation (extent=[12, -92; 38, -68]);
          BondLib.Bonds.Bond B3 annotation (extent=[-12, -68; 12, -44],
              rotation=90);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG2 annotation (extent=
                [-62, -94; -38, -66]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -94; 64, -66]);
          BondLib.Switches.Sw2 Sw1(Ron=Ron, Goff=Goff) annotation (extent=[-14,
                  -44; 14, -14], rotation=90);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, -68; 12, -92]);
        equation 
          connect(El2BG1.BondCon1, B4.eBondCon1) annotation (points=[
                6.12303e-016,60; -6.12303e-016,60],
                     style(color=8));
          connect(B4.fBondCon1, De1.BondCon1) annotation (points=[6.12303e-016,
                40; -6.12303e-016,40],
                style(color=8));
          connect(De1.OutPort1,LessThan1.u)       annotation (points=[
                6.73533e-016,19; 6.73533e-016,17.75; -6.12308e-017,17.75;
                -6.12308e-017,16.5; -7.34764e-016,16.5; -7.34764e-016,14],
                       style(color=3));
          connect(Sw1.BondCon1, B3.BondCon2) annotation (points=[9.19132e-016,
                -44; 7.34764e-016,-44],  style(color=8));
          connect(LessThan1.y,       Sw1.BooleanInPort1) annotation (points=[
                6.73533e-016,-9; 6.73533e-016,-10.625; 3.22139e-015,-10.625;
                3.22139e-015,-12.25; 2.54786e-015,-12.25; 2.54786e-015,-15.5],
                                 style(color=5));
          connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[0,-68; 0,
                -67.88; 0,-67.76; -7.20069e-016,-67.76],
                         style(color=8));
          connect(B2.BondCon1, J1p3_1.BondCon2) annotation (points=[12.26,-80;
                12,-80],style(color=8));
          connect(B1.BondCon2, J1p3_1.BondCon1) annotation (points=[-12, -80; -
                12, -80], style(color=8));
          connect(El2BG2.BondCon1, B1.BondCon1) annotation (points=[-38,-80;
                -37.74,-80],
                          style(color=8));
          connect(B2.BondCon2, BG2El1.BondCon1) annotation (points=[38, -80; 38,
                  -80], style(color=8));
          connect(El2BG2.p, p) annotation (points=[-62,-80; -80,-80; -80,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,-80; 80,-80; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(El2BG1.p, control) annotation (points=[-6.12303e-016,80; 0,80;
                0,100], style(color=3, rgbcolor={0,0,255}));
        end ControlledIdealClosingSwitch;
      end Ideal;
      
      package Semiconductors 
        "Semiconductor models of the analog electrical library" 
        model Diode "Real diode of the analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current Ids=1.e-6 "Saturation current";
          parameter Modelica.SIunits.Voltage Vt=0.04 
            "Thermal voltage equivalent of temperature (kT/qn)";
          parameter Real Maxexp(final min=Modelica.Constants.small) = 15 
            "Max. exponent for linear continuation";
          parameter Modelica.SIunits.Resistance R=1.e8 
            "Parallel Ohmic leakage resistance";
          annotation (
            Icon(
              Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                  color=3,
                  gradient=0,
                  fillColor=7)),
              Line(points=[-90,0; 40,0]),
              Line(points=[40,0; 90,0]),
              Line(points=[30,40; 30,-40],   style(color=3)),
              Text(extent=[-100,100; 100,70],   string="%name"),
              Text(
                extent=[-98,-58; 102,-100],
                string="Vt=%Vt",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
The real electrical diode is built in bondgraphic technology from a bond graph <b>Dreal</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the diode in its environment. <p>

<hr>

<pre>
Parameters:

 Ids:     Saturation current (default value = 1e-6 Amp)

 Vt:      Thermal voltage (default value = 0.04 Volt)

 Maxexp:  Maximum exponent (default value = 15)

 R:       Parallel Ohmic leakage resistance (default value = 1e8 Ohm)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                90);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          BondLib.Semiconductors.Dreal D1(
            Ids=Ids,
            Vt=Vt,
            Maxexp=Maxexp,
            R=R) annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],   style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015],
                    style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          connect(B3.BondCon2, D1.BondCon1) annotation (points=[7.34764e-016,36;
                -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
        end Diode;
        
        model NPN "NPN bipolar Ebers-Moll transistor" 
          parameter Real Bf=50 "Forward beta coefficient";
          parameter Real Br=0.1 "Reverse beta coefficient";
          parameter Modelica.SIunits.Current Is=1.e-16 
            "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak=0.02 
            "Inverse early voltage";
          parameter Modelica.SIunits.Time Tauf=0.12e-9 
            "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur=5e-9 
            "Ideal reverse transit time";
          parameter Modelica.SIunits.Capacitance Ccs=1e-12 
            "Collector-substrate capacitance";
          parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
            "Base-emitter zero bias depletion capacitance";
          parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
            "Base-collector zero bias depletion capacitance";
          parameter Modelica.SIunits.Voltage Phie=0.8 
            "Base-emitter diffusion voltage";
          parameter Real Me=0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic=0.8 
            "Base-collector diffusion voltage";
          parameter Real Mc=0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Conductance Gbc=1e-15 
            "Base-collector leakage conductance";
          parameter Modelica.SIunits.Conductance Gbe=1e-15 
            "Base-emitter leakage conductance";
          parameter Modelica.SIunits.Voltage Vt=0.02585 "Thermal voltage";
          parameter Real EMin=-100 
            "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "if x > EMax, the exp(x) function is linearized";
          parameter Boolean enforceStates = true 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          annotation (
            Icon(
              Text(extent=[-110,74; 94,100],    string="%name"),
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[30,-50; 24,-36; 16,-44; 30,-50],     style(fillColor=3,
                      fillPattern=1))),
            Diagram(
              Ellipse(extent=[-42,2; -38,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[-2,2; 2,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[26,-84; 30,-88], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Ellipse(extent=[-96,-32; -92,-36], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[14,50; 28,42],
                string="ibc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[-100,-24; -86,-32],
                string="ibc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[86,14; 100,6],
                string="ibc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[86,2; 100,-6],
                string="ibe",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[-88,-52; -74,-60],
                string="ibe",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[14,-30; 28,-38],
                string="ibe",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[-68,50; -54,42],
                string="vbc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[70,-14; 84,-22],
                string="vbc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Ellipse(extent=[-2,82; 2,78], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Ellipse(extent=[-2,-78; 2,-82], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Ellipse(extent=[38,52; 42,48], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Ellipse(extent=[38,-48; 42,-52], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1))),
            Documentation(info="<html>
The NPN element of the analog electrical library built on bond graph technology implements the Ebers-Moll model of a NPN bipolar transistor.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"));
          Utilities.JunctionCapacitor Dbc(
            Is=Is,
            Tau=Taur,
            Cj=Cjc,
            Phi=Phic,
            M=Mc,
            Gb=Gbc,
            Vt=Vt,
            EMin=EMin,
            EMax=EMax,
            enforceStates=enforceStates) 
                       annotation (extent=[-20,60; 20,20], rotation=-90);
          Utilities.JunctionCapacitor Dbe(
            Is=Is,
            Vt=Vt,
            EMin=EMin,
            EMax=EMax,
            Tau=Tauf,
            Cj=Cje,
            Phi=Phie,
            M=Me,
            Gb=Gbe,
            enforceStates=enforceStates) 
                    annotation (extent=[-20,-60; 20,-20], rotation=-90);
          Sensors.VoltageSensor Vbc 
            annotation (extent=[-60,60; -20,20], rotation=-90);
          Sources.SignalCurrent IB0 
            annotation (extent=[-20,-60; -60,-20], rotation=-90);
          Sources.SignalCurrent IC0 
            annotation (extent=[20,-20; 60,20], rotation=-90);
          BondLib.Semiconductors.Utilities.IC0 IC0_1(
            Br=Br,
            Vak=Vak) annotation (extent=[80,-10; 60,10]);
          BondLib.Semiconductors.Utilities.IB0 IB0_1(Bf=Bf, Br=Br) 
            annotation (extent=[-80,-50; -60,-30]);
          BondLib.Electrical.Analog.Basic.Capacitor Ccs1(C=Ccs, enforceStates=
                enforceStates) 
            annotation (extent=[40,66; 68,94]);
          BondLib.Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[74,70; 94,90], rotation=90);
        equation 
          connect(IC0_1.y, IC0.InPort1) annotation (points=[59,0; 58.5,0; 58.5,
                7.34764e-016; 52,7.34764e-016], style(color=74, rgbcolor={0,0,
                  127}));
          connect(IB0_1.y, IB0.InPort1) annotation (points=[-59,-40; -52,-40],
              style(color=74, rgbcolor={0,0,127}));
          connect(Ccs1.n, Gnd1.p)    annotation (points=[68,80; 74,80], style(
                color=3, rgbcolor={0,0,255}));
          connect(IC0.p, Ccs1.p) annotation (points=[40,20; 40,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(IB0.p, B) annotation (points=[-40,-20; -40,0; -100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(IC0.p, C) annotation (points=[40,20; 40,50; 100,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(IB0.n,Dbe. n) annotation (points=[-40,-60; -40,-80;
                1.22461e-015,-80; 1.22461e-015,-60], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(IC0.n,Dbe. n) annotation (points=[40,-20; 40,-80;
                1.22461e-015,-80; 1.22461e-015,-60], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(IC0.n, E) annotation (points=[40,-20; 40,-50; 100,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(IB0_1.u2, IC0_1.u2) annotation (points=[-82,-46; -88,-46; -88,
                -86; 110,-86; 110,-6; 82,-6], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(IB0_1.u1, IC0_1.u1) annotation (points=[-82,-34; -94,-34; -94,
                -92; 116,-92; 116,6; 82,6], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Dbc.y, IB0_1.u1) annotation (points=[20,40; 28,40; 28,88;
                -110,88; -110,-34; -82,-34], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Dbe.y, IB0_1.u2) annotation (points=[20,-40; 28,-40; 28,-86;
                -88,-86; -88,-46; -82,-46], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Vbc.V, IC0_1.u3) annotation (points=[-60,40; -116,40; -116,
                -98; 70,-98; 70,-10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=74,
              rgbfillColor={0,0,156},
              fillPattern=1));
          connect(Vbc.p, B) annotation (points=[-40,20; -40,0; -100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vbc.n, Ccs1.p) annotation (points=[-40,60; -40,80; 40,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(Dbe.p, B) annotation (points=[-1.22461e-015,-20;
                -1.22461e-015,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Dbc.p, B) annotation (points=[-1.22461e-015,20; 0,20; 0,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Dbc.n, Ccs1.p) annotation (points=[1.22461e-015,60;
                1.22461e-015,80; 40,80], style(color=3, rgbcolor={0,0,255}));
        end NPN;
        
        model PNP "PNP bipolar Ebers-Moll transistor" 
          parameter Real Bf=50 "Forward beta coefficient";
          parameter Real Br=0.1 "Reverse beta coefficient";
          parameter Modelica.SIunits.Current Is=1.e-16 
            "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak=0.02 
            "Inverse early voltage";
          parameter Modelica.SIunits.Time Tauf=0.12e-9 
            "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur=5e-9 
            "Ideal reverse transit time";
          parameter Modelica.SIunits.Capacitance Ccs=1e-12 
            "Collector-substrate capacitance";
          parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
            "Base-emitter zero bias depletion capacitance";
          parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
            "Base-collector zero bias depletion capacitance";
          parameter Modelica.SIunits.Voltage Phie=0.8 
            "Base-emitter diffusion voltage";
          parameter Real Me=0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic=0.8 
            "Base-collector diffusion voltage";
          parameter Real Mc=0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Conductance Gbc=1e-15 
            "Base-collector leakage conductance";
          parameter Modelica.SIunits.Conductance Gbe=1e-15 
            "Base-emitter leakage conductance";
          parameter Modelica.SIunits.Voltage Vt=0.02585 "Thermal voltage";
          parameter Real EMin=-100 
            "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "if x > EMax, the exp(x) function is linearized";
          parameter Boolean enforceStates = true 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          annotation (
            Icon(
              Text(extent=[-114,76; 100,100],   string="%name"),
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[-10,-10; 5,-17; -3,-25; -10,-10],     style(fillColor=
                      3, fillPattern=1))),
            Diagram(
              Ellipse(extent=[-42,2; -38,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[-2,2; 2,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[26,-84; 30,-88], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Ellipse(extent=[-96,-32; -92,-36], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[14,50; 28,42],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="icb"),
              Text(
                extent=[-100,-24; -86,-32],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="icb"),
              Text(
                extent=[86,14; 100,6],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="icb"),
              Text(
                extent=[86,2; 100,-6],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="ieb"),
              Text(
                extent=[-88,-52; -74,-60],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="ieb"),
              Text(
                extent=[14,-30; 28,-38],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="ieb"),
              Text(
                extent=[-68,50; -54,42],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="vcb"),
              Text(
                extent=[70,-14; 84,-22],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="vcb"),
              Ellipse(extent=[-2,82; 2,78], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Ellipse(extent=[-2,-78; 2,-82], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Ellipse(extent=[38,52; 42,48], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Ellipse(extent=[38,-48; 42,-52], style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1))),
            Documentation(info="<html>
The PNP element of the analog electrical library built on bond graph technology implements the Ebers-Moll model of a PNP bipolar transistor.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"));
          Utilities.JunctionCapacitor Dcb(
            Is=Is,
            Tau=Taur,
            Cj=Cjc,
            Phi=Phic,
            M=Mc,
            Gb=Gbc,
            Vt=Vt,
            EMin=EMin,
            EMax=EMax,
            enforceStates=enforceStates) 
                       annotation (extent=[-20,20; 20,60], rotation=-90);
          Utilities.JunctionCapacitor Deb(
            Is=Is,
            Vt=Vt,
            EMin=EMin,
            EMax=EMax,
            Tau=Tauf,
            Cj=Cje,
            Phi=Phie,
            M=Me,
            Gb=Gbe,
            enforceStates=enforceStates) 
                    annotation (extent=[-20,-20; 20,-60], rotation=-90);
          Sensors.VoltageSensor Vcb 
            annotation (extent=[-60,20; -20,60], rotation=-90);
          Sources.SignalCurrent IB0 
            annotation (extent=[-20,-20; -60,-60], rotation=-90);
          Sources.SignalCurrent IC0 
            annotation (extent=[20,20; 60,-20], rotation=-90);
          BondLib.Semiconductors.Utilities.IC0 IC0_1(
            Br=Br,
            Vak=Vak) annotation (extent=[80,-10; 60,10]);
          BondLib.Semiconductors.Utilities.IB0 IB0_1(Bf=Bf, Br=Br) 
            annotation (extent=[-80,-50; -60,-30]);
          BondLib.Electrical.Analog.Basic.Capacitor Ccs1(C=Ccs, enforceStates=
                enforceStates) 
            annotation (extent=[40,66; 68,94]);
          BondLib.Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[74,70; 94,90], rotation=90);
        equation 
          connect(IC0_1.y,IC0. InPort1) annotation (points=[59,0; 55.5,0; 55.5,
                -7.34764e-016; 52,-7.34764e-016],
                                                style(color=74, rgbcolor={0,0,
                  127}));
          connect(IB0_1.y,IB0. InPort1) annotation (points=[-59,-40; -52,-40],
              style(color=74, rgbcolor={0,0,127}));
          connect(Ccs1.n,Gnd1. p)    annotation (points=[68,80; 74,80], style(
                color=3, rgbcolor={0,0,255}));
          connect(IB0_1.u2,IC0_1. u2) annotation (points=[-82,-46; -88,-46; -88,
                -86; 110,-86; 110,-6; 82,-6], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(IB0_1.u1,IC0_1. u1) annotation (points=[-82,-34; -94,-34; -94,
                -92; 116,-92; 116,6; 82,6], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Dcb.y,IB0_1. u1) annotation (points=[20,40; 28,40; 28,88;
                -110,88; -110,-34; -82,-34], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Deb.y,IB0_1. u2) annotation (points=[20,-40; 28,-40; 28,-86;
                -88,-86; -88,-46; -82,-46], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Vcb.V,IC0_1. u3) annotation (points=[-60,40; -116,40; -116,
                -98; 70,-98; 70,-10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=74,
              rgbfillColor={0,0,156},
              fillPattern=1));
          connect(IC0.n, Ccs1.p) annotation (points=[40,20; 40,80], style(color=
                 3, rgbcolor={0,0,255}));
          connect(IC0.n, C) annotation (points=[40,20; 40,50; 100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(IC0.p, E) annotation (points=[40,-20; 40,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(IB0.n, B) annotation (points=[-40,-20; -40,0; -100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(IB0.p,Deb. p) annotation (points=[-40,-60; -40,-80;
                -1.22461e-015,-80; -1.22461e-015,-60], style(color=3, rgbcolor=
                  {0,0,255}));
          connect(Dcb.p, Ccs1.p) annotation (points=[-1.22461e-015,60; 0,60; 0,
                80; 40,80], style(color=3, rgbcolor={0,0,255}));
          connect(Dcb.n, B) annotation (points=[1.22461e-015,20; 1.22461e-015,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Vcb.n, B) annotation (points=[-40,20; -40,0; -100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vcb.p, Ccs1.p) annotation (points=[-40,60; -40,80; 40,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(Deb.n, B) annotation (points=[1.22461e-015,-20; 1.22461e-015,
                0; -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Deb.p, IC0.p) annotation (points=[-1.22461e-015,-60; 0,-60; 0,
                -80; 40,-80; 40,-20], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
        end PNP;
        
        model HeatingDiode 
          "Real heating diode of the analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current Ids=1.e-6 "Saturation current";
          parameter Real EG=1.11 "activation energy";
          parameter Real N=1 "Emission coefficient";
          parameter Modelica.SIunits.Temperature TNOM=300.15 
            "Nominal temperature";
          parameter Real XTI=3 "Temperature exponent of saturation current";
          parameter Real Maxexp(final min=Modelica.Constants.small) = 15 
            "Max. exponent for linear continuation";
          parameter Modelica.SIunits.Resistance R=1.e8 
            "Parallel Ohmic resistance";
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "Thermal connector" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          annotation (
            Icon(
              Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                  color=3,
                  gradient=0,
                  fillColor=1,
                  fillPattern=0)),
              Line(points=[30,40; 30,-40],   style(color=3)),
              Line(points=[0,-20; 0,-91],   style(color=42)),
              Line(points=[-90,0; 90,0], style(color=3, rgbcolor={0,0,255})),
              Text(extent=[-140,109; 145,59],   string="%name")),
            Diagram,
            Documentation(info="<html>
The real electrical heating diode is built in bondgraphic technology from a bond graph <b>DSreal</b> element.  A-causal bonds must be used here, because the causality depends on the embedding of the diode in its environment. <p>

<hr>

<pre>
Parameters:

 Ids:     Saturation current (default value = 1e-6 Amp)

 Maxexp:  Maximum exponent (default value = 15)

 R:       Parallel Ohmic leakage resistance (default value = 1e8 Ohm)

 EG:      Activation energy (default value = 1.11 J)

 N:       Emission coefficient (default value = 1)

 TNOM:    Nominal temperature (default value = 300.15 K)

 XTI:     Temperature exponent of saturation current (default value = 3)
</pre>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
          Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
          Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
          Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                270);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,26; -38,54]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                26; 64,54]);
          Thermal.DSreal D1(
            Ids=Ids,
            EG=EG,
            N=N,
            TNOM=TNOM,
            XTI=XTI,
            Maxexp=Maxexp,
            R=R) annotation (extent=[-12,-20; 12,4], rotation=-90);
          Bonds.eBond B4 annotation (extent=[-12,-44; 12,-20], rotation=-90);
          Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
            annotation (extent=[-14,-72; 14,-44], rotation=-90);
        equation 
          assert( heatPort.T > 0,"temperature must be positive");
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40; -12,
                40],  style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                12.26,40], style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                -37.74,40],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                40],style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0; -100,
                0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(D1.BondCon2, B4.fBondCon1) annotation (points=[7.34764e-016,
                -20; 7.34764e-016,-24; 0,-20; -7.34764e-016,-20],
              style(color=8, rgbcolor={192,192,192}));
          connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[0,28; 0,
                27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                  192}));
          connect(B3.BondCon2, D1.BondCon1) annotation (points=[-2.20429e-015,4;
                -7.34764e-016,4], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                9.19132e-016,-44; 7.34764e-016,-44], style(color=8, rgbcolor={
                  192,192,192}));
          connect(BG2Heat1.port_b, heatPort) annotation (points=[2.63358e-015,
                -72; 2.63358e-015,-82; 0,-82; 0,-100], style(color=42, rgbcolor=
                 {191,0,0}));
        end HeatingDiode;
        
        model HeatingNPN 
          "NPN bipolar Ebers-Moll transistor with temperature variation" 
          parameter Real Bf=50 "Forward beta coefficient";
          parameter Real Br=0.1 "Reverse beta coefficient";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "Reference temperature";
          parameter Modelica.SIunits.Current Is=1.e-16 
            "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak=0.02 
            "Inverse early voltage";
          parameter Modelica.SIunits.Time Tauf=0.12e-9 
            "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur=5e-9 
            "Ideal reverse transit time";
          parameter Real NF=1 "Forward current emission coefficient";
          parameter Real NR=1 "Reverse current emission coefficient";
          parameter Modelica.SIunits.Capacitance Ccs=1e-12 
            "Collector-substrate capacitance";
          parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
            "Base-emitter zero bias depletion capacitance";
          parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
            "Base-collector zero bias depletion capacitance";
          parameter Modelica.SIunits.Voltage Phie=0.8 
            "Base-emitter diffusion voltage";
          parameter Real Me=0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic=0.8 
            "Base-collector diffusion voltage";
          parameter Real Mc=0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Voltage EG=1.11 
            "Energy gap for temperature effect on saturation current";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real XTB=0 
            "Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Conductance Gbc=1e-15 
            "Base-collector leakage conductance";
          parameter Modelica.SIunits.Conductance Gbe=1e-15 
            "Base-emitter leakage conductance";
          parameter Real EMin=-100 
            "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "if x > EMax, the exp(x) function is linearized";
          parameter Boolean enforceStates = true 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          Sensors.VoltageSensor Vbc 
            annotation (extent=[-70,60; -30,20], rotation=-90);
          BondLib.Electrical.Analog.Basic.Capacitor Ccs1(C=Ccs, enforceStates=
                enforceStates) 
            annotation (extent=[40,66; 68,94]);
          BondLib.Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[74,70; 94,90], rotation=90);
          annotation (
            Diagram(
              Ellipse(extent=[-52,2; -48,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(
                extent=[14,50; 28,42],
                string="ibc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[-106,-26; -92,-34],
                string="ibc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[90,10; 104,2],
                string="ibc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[90,-4; 104,-12],
                string="ibe",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[-106,-46; -92,-54],
                string="ibe",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[14,-30; 28,-38],
                string="ibe",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[-82,48; -68,40],
                string="vbc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[76,-16; 90,-24],
                string="vbc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1)),
              Text(
                extent=[50,-14; 64,-22],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="Tdev"),
              Text(
                extent=[-82,-54; -68,-62],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="Tdev"),
              Text(
                extent=[64,-72; 78,-80],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="Tdev")),
            Icon(
              Text(extent=[-100,80; 100,100],   string="%name"),
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[30,-50; 24,-36; 16,-44; 30,-50],     style(
                    fillColor=3, fillPattern=1)),
              Line(points=[0,-91; 0,-30],   style(color=42))),
            Documentation(info="<html>
The NPN element of the analog electrical library built on bond graph technology implements the Ebers-Moll model of a NPN bipolar transistor.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 NF:    Forward current emission coefficient (default value = 1)

 NR:    Reverse current emission coefficient (default value = 1)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 XTI:   Saturation current temperature exponent (default value = 3)

 XTB:   Forward and reverse beta temperature coefficient (default value = 0)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "Thermal connector" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          BondLib.Semiconductors.Utilities.IB02 IB0_1(
            Tnom=Tnom,
            Bf=Bf,
            Br=Br,
            XTB=XTB) annotation (extent=[-92,-50; -72,-30]);
          BondLib.Electrical.Analog.Spice.Utilities.Isignal IB0 
            annotation (extent=[-30,-60; -70,-20], rotation=-90);
          BondLib.Semiconductors.Utilities.IC02 IC0_1(
            Tnom=Tnom,
            Br=Br,
            Vak=Vak,
            XTB=XTB) annotation (extent=[80,-10; 60,10]);
          BondLib.Electrical.Analog.Spice.Utilities.Isignal IC0 
            annotation (extent=[20,-20; 60,20], rotation=-90);
          BondLib.Electrical.Analog.Semiconductors.Utilities.HeatingJunctionCapacitor
            Dbe(
            Tnom=Tnom,
            Is=Is,
            Tau=Tauf,
            Cj=Cje,
            Phi=Phie,
            M=Me,
            EG=EG,
            N=NF,
            XTI=XTI,
            Gb=Gbe,
            EMin=EMin,
            EMax=EMax,
            enforceStates=enforceStates) 
                       annotation (extent=[-20,-60; 20,-20], rotation=-90);
          BondLib.Electrical.Analog.Semiconductors.Utilities.HeatingJunctionCapacitor
            Dbc(
            Tnom=Tnom,
            Is=Is,
            EG=EG,
            XTI=XTI,
            EMin=EMin,
            EMax=EMax,
            Tau=Taur,
            Cj=Cjc,
            Phi=Phic,
            M=Mc,
            N=NR,
            Gb=Gbc,
            enforceStates=enforceStates) 
                    annotation (extent=[-20,60; 20,20], rotation=-90);
          Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev 
            annotation (extent=[50,-90; 70,-70]);
        equation 
          connect(Ccs1.n,Gnd1. p)    annotation (points=[68,80; 74,80], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vbc.p,B)  annotation (points=[-50,20; -50,0; -100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vbc.n,Ccs1. p) annotation (points=[-50,60; -50,80; 40,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(IB0_1.y, IB0.InPort1) annotation (points=[-71,-40; -62,-40],
              style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(IB0.p, B) annotation (points=[-50,-20; -50,0; -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(IB0.n, E) annotation (points=[-50,-60; -50,-72; 40,-72; 40,
                -50; 100,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(IC0_1.y, IC0.InPort1) annotation (points=[59,0; 58.5,0; 58.5,
                7.34764e-016; 52,7.34764e-016], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(IC0.p, C) annotation (points=[40,20; 40,50; 100,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(IC0.n, E) annotation (points=[40,-20; 40,-50; 100,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Dbe.p, B) annotation (points=[-1.22461e-015,-20; 0,-20; 0,0;
                -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Dbe.n, E) annotation (points=[1.22461e-015,-60; 0,-60; 0,-72;
                40,-72; 40,-50; 100,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Dbc.n, Ccs1.p) annotation (points=[1.22461e-015,60;
                1.22461e-015,80; 40,80], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(heatPort, Tdev.port) annotation (points=[0,-100; 0,-80; 50,
                -80], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Tdev.T, IC0_1.Tdev) annotation (points=[70,-80; 80,-80; 80,
                -30; 64,-30; 64,-10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Tdev.T, IB0_1.Tdev) annotation (points=[70,-80; 80,-80; 80,
                -106; -82,-106; -82,-50], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Ccs1.p, C) annotation (points=[40,80; 40,50; 100,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(IC0.heatPort, heatPort) annotation (points=[20,-1.22461e-015;
                20,-20; 12,-20; 12,-80; 0,-80; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbe.heatPort, IB0.heatPort) annotation (points=[-20,-40; -30,
                -40], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbe.heatPort, Dbc.heatPort) annotation (points=[-20,-40; -20,
                40], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbe.heatPort, heatPort) annotation (points=[-20,-40; -20,-80;
                0,-80; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbc.p, B) annotation (points=[-1.22461e-015,20; 0,20; 0,0;
                -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Vbc.V, IC0_1.u3) annotation (points=[-70,40; -108,40; -108,
                -108; 82,-108; 82,-24; 76,-24; 76,-10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbc.y, IC0_1.u1) annotation (points=[20,40; 90,40; 90,6; 82,6],
              style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbe.y, IC0_1.u2) annotation (points=[20,-40; 90,-40; 90,-6;
                82,-6], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbe.y, IB0_1.u2) annotation (points=[20,-40; 30,-40; 30,-110;
                -110,-110; -110,-46; -94,-46], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
          connect(Dbc.y, IB0_1.u1) annotation (points=[20,40; 30,40; 30,70;
                -110,70; -110,-34; -94,-34], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=74,
              rgbfillColor={0,0,168},
              fillPattern=1));
        end HeatingNPN;
        
        model HeatingPNP 
          "PNP bipolar Ebers-Moll transistor with temperature variation" 
          parameter Real Bf=50 "Forward beta coefficient";
          parameter Real Br=0.1 "Reverse beta coefficient";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "Reference temperature";
          parameter Modelica.SIunits.Current Is=1.e-16 
            "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak=0.02 
            "Inverse early voltage";
          parameter Modelica.SIunits.Time Tauf=0.12e-9 
            "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur=5e-9 
            "Ideal reverse transit time";
          parameter Real NF=1 "Forward current emission coefficient";
          parameter Real NR=1 "Reverse current emission coefficient";
          parameter Modelica.SIunits.Capacitance Ccs=1e-12 
            "Collector-substrate capacitance";
          parameter Modelica.SIunits.Capacitance Cje=0.4e-12 
            "Base-emitter zero bias depletion capacitance";
          parameter Modelica.SIunits.Capacitance Cjc=0.5e-12 
            "Base-collector zero bias depletion capacitance";
          parameter Modelica.SIunits.Voltage Phie=0.8 
            "Base-emitter diffusion voltage";
          parameter Real Me=0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic=0.8 
            "Base-collector diffusion voltage";
          parameter Real Mc=0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Voltage EG=1.11 
            "Energy gap for temperature effect on saturation current";
          parameter Real XTI=3 "Saturation current temperature exponent";
          parameter Real XTB=0 
            "Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Conductance Gbc=1e-15 
            "Base-collector leakage conductance";
          parameter Modelica.SIunits.Conductance Gbe=1e-15 
            "Base-emitter leakage conductance";
          parameter Real EMin=-100 
            "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "if x > EMax, the exp(x) function is linearized";
          parameter Boolean enforceStates = true 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          Sensors.VoltageSensor Vcb 
            annotation (extent=[-70,20; -30,60], rotation=-90);
          Basic.Capacitor Ccs1(                          C=Ccs, enforceStates=
                enforceStates) 
            annotation (extent=[40,66; 68,94]);
          Basic.Ground Gnd1 
            annotation (extent=[74,70; 94,90], rotation=90);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "Thermal connector" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          BondLib.Semiconductors.Utilities.IB02 IB0_1(
            Tnom=Tnom,
            Bf=Bf,
            Br=Br,
            XTB=XTB) annotation (extent=[-90,-50; -70,-30]);
          BondLib.Electrical.Analog.Spice.Utilities.Isignal IB0 
            annotation (extent=[-30,-20; -70,-60], rotation=-90);
          BondLib.Electrical.Analog.Semiconductors.Utilities.HeatingJunctionCapacitor
            Deb(
            Tnom=Tnom,
            Is=Is,
            Tau=Tauf,
            Cj=Cje,
            Phi=Phie,
            M=Me,
            EG=EG,
            N=NF,
            XTI=XTI,
            Gb=Gbe,
            EMin=EMin,
            EMax=EMax,
            enforceStates=enforceStates) 
                       annotation (extent=[-20,-20; 20,-60], rotation=-90);
          BondLib.Semiconductors.Utilities.IC02 IC0_1(
            Tnom=Tnom,
            Br=Br,
            Vak=Vak,
            XTB=XTB) annotation (extent=[80,-10; 60,10]);
          BondLib.Electrical.Analog.Spice.Utilities.Isignal IC0 
            annotation (extent=[20,20; 60,-20], rotation=-90);
          BondLib.Electrical.Analog.Semiconductors.Utilities.HeatingJunctionCapacitor
            Dcb(
            Tnom=Tnom,
            Is=Is,
            EG=EG,
            XTI=XTI,
            EMin=EMin,
            EMax=EMax,
            Tau=Taur,
            Cj=Cjc,
            Phi=Phic,
            M=Mc,
            N=NR,
            Gb=Gbc,
            enforceStates=enforceStates) 
                    annotation (extent=[-20,20; 20,60], rotation=-90);
          Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev 
            annotation (extent=[50,-90; 70,-70]);
          annotation (
            Diagram(
              Ellipse(extent=[-52,2; -48,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(
                extent=[14,50; 28,42],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="icb"),
              Text(
                extent=[-106,-26; -92,-34],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="icb"),
              Text(
                extent=[90,10; 104,2],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="icb"),
              Text(
                extent=[90,-2; 104,-10],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="ieb"),
              Text(
                extent=[-106,-48; -92,-56],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="ieb"),
              Text(
                extent=[14,-30; 28,-38],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="ieb"),
              Text(
                extent=[-78,50; -64,42],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="vcb"),
              Text(
                extent=[76,-14; 90,-22],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="vcb"),
              Text(
                extent=[64,-72; 78,-80],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="Tdev"),
              Text(
                extent=[48,-12; 62,-20],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="Tdev"),
              Text(
                extent=[-78,-52; -64,-60],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=74,
                  rgbfillColor={0,0,156},
                  fillPattern=1),
                string="Tdev")),
            Icon(
              Text(extent=[-100,70; 100,90],    string="%name"),
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[-10,-10; 5,-17; -3,-25; -10,-10],     style(
                    fillColor=3, fillPattern=1)),
              Line(points=[0,-91; 0,-30],   style(color=42))),
            Documentation(info="<html>
The PNP element of the analog electrical library built on bond graph technology implements the Ebers-Moll model of a PNP bipolar transistor.  The substrate is not shown on the icon layer, as it is assumed grounded. The transistor is assumed to be vertically diffused, as the substrate is connected to the collector, rather than to the base. <p>

<hr>

<pre>
Parameters:

 Bf:    Forward beta coefficient (default value = 50)

 Br:    Reverse beta coefficient (default value = 0.1)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vak:   Inverse early voltage (default value = 0.02 /Volt)

 Tauf:  Ideal forward transit time (default value = 0.12e-9 sec)

 Taur:  Ideal reverse transit time (default value = 5e-9 sec)

 NF:    Forward current emission coefficient (default value = 1)

 NR:    Reverse current emission coefficient (default value = 1)

 Ccs:   Collector-substrate capacitance (default value = 1e-12 F)

 Cje:   Base-emitter zero bias depletion capacitance (default value = 0.4e-12 F)

 Cjc:   Base-collector zero bias depletion capacitance (default value = 0.5e-12 F)

 Phie:  Base-emitter diffusion voltage (default value = 0.8 Volt)

 Me:    Base-emitter gradation exponent (default value = 0.4)

 Phic:  Base-collector diffusion voltage (default value = 0.8 Volt)

 Mc:    Base-collector gradation exponent (default value = 0.333)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 XTI:   Saturation current temperature exponent (default value = 3)

 XTB:   Forward and reverse beta temperature coefficient (default value = 0)

 Gbc:   Base-collector leakage conductance (default value = 1e-15 Mho)

 Gbe:   Base-emitter leakage conductance (default value = 1e-15 Mho)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre> <p>

<hr> <p>

References: <p>

Vlach, J. and K. Singal (1983), <I>Computer methods for circuit analysis and design</I>, Van Nostrand Reinhold, New York,
on page 317 ff.
</html>
"));
        equation 
          connect(Ccs1.n,Gnd1. p)    annotation (points=[68,80; 74,80], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vcb.n,B)  annotation (points=[-50,20; -50,0; -100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vcb.p,Ccs1. p) annotation (points=[-50,60; -50,80; 40,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(IB0_1.y, IB0.InPort1) annotation (points=[-69,-40; -62,-40],
              style(color=74, rgbcolor={0,0,127}));
          connect(IB0.n, B) annotation (points=[-50,-20; -50,0; -100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(IB0.p, Deb.p) annotation (points=[-50,-60; -50,-70;
                -1.22461e-015,-70; -1.22461e-015,-60], style(color=3, rgbcolor=
                  {0,0,255}));
          connect(Deb.n, B) annotation (points=[1.22461e-015,-20; 1.22461e-015,
                0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Deb.p, E) annotation (points=[-1.22461e-015,-60; 0,-60; 0,-70;
                40,-70; 40,-50; 100,-50], style(color=3, rgbcolor={0,0,255}));
          connect(IC0_1.y, IC0.InPort1) annotation (points=[59,0; 55.5,0; 55.5,
                -7.34764e-016; 52,-7.34764e-016], style(color=74, rgbcolor={0,0,
                  127}));
          connect(IC0.p, E) annotation (points=[40,-20; 40,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(IC0.n, C) annotation (points=[40,20; 40,50; 100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Ccs1.p, C) annotation (points=[40,80; 40,50; 100,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Dcb.p, Ccs1.p) annotation (points=[-1.22461e-015,60;
                -1.22461e-015,80; 40,80], style(color=3, rgbcolor={0,0,255}));
          connect(Dcb.n, B) annotation (points=[1.22461e-015,20; 1.22461e-015,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(Tdev.port, heatPort) annotation (points=[50,-80; 0,-80; 0,
                -100], style(color=42, rgbcolor={191,0,0}));
          connect(Deb.heatPort, heatPort) annotation (points=[-20,-40; -20,-80;
                0,-80; 0,-100], style(color=42, rgbcolor={191,0,0}));
          connect(IB0.heatPort, Deb.heatPort) annotation (points=[-30,-40; -20,
                -40], style(color=42, rgbcolor={191,0,0}));
          connect(IC0.heatPort, Tdev.port) annotation (points=[20,1.22461e-015;
                20,-20; 10,-20; 10,-80; 50,-80], style(color=42, rgbcolor={191,
                  0,0}));
          connect(Tdev.T, IC0_1.Tdev) annotation (points=[70,-80; 80,-80; 80,
                -30; 64,-30; 64,-10], style(color=74, rgbcolor={0,0,127}));
          connect(Tdev.T, IB0_1.Tdev) annotation (points=[70,-80; 80,-80; 80,
                -106; -80,-106; -80,-50], style(color=74, rgbcolor={0,0,127}));
          connect(Deb.y, IC0_1.u2) annotation (points=[20,-40; 90,-40; 90,-6;
                82,-6], style(color=74, rgbcolor={0,0,127}));
          connect(Vcb.V, IC0_1.u3) annotation (points=[-70,40; -110,40; -110,
                -108; 82,-108; 82,-22; 76,-22; 76,-10], style(color=74,
                rgbcolor={0,0,127}));
          connect(Deb.y, IB0_1.u2) annotation (points=[20,-40; 30,-40; 30,-110;
                -112,-110; -112,-46; -92,-46], style(color=74, rgbcolor={0,0,
                  127}));
          connect(Dcb.y, IC0_1.u1) annotation (points=[20,40; 90,40; 90,6; 82,6],
              style(color=74, rgbcolor={0,0,127}));
          connect(Dcb.y, IB0_1.u1) annotation (points=[20,40; 30,40; 30,70;
                -112,70; -112,-34; -92,-34], style(color=74, rgbcolor={0,0,127}));
          connect(Dcb.heatPort, Deb.heatPort) annotation (points=[-20,40; -20,
                -40], style(color=42, rgbcolor={191,0,0}));
        end HeatingPNP;
        
        package Utilities 
          "Utilities for semiconductor models of analog electrical library" 
          annotation (preferedView="info",Icon(
              Rectangle(extent=[-100,-100; 80,50], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=42, rgbcolor={127,0,0})),
              Text(
                extent=[-120,122; 120,73],
                string="%name",
                style(color=1))), Documentation(info="<html>
This package contains some utility models used inside some of the semiconductor models of the analog electrical library built from bond graph technology.
</html>
"));
          model VariableCapacitor2 "Modulated electrical capacitor" 
            extends Interfaces.ModOnePortV;
            annotation (
              Icon(
                Line(points=[-90,0; -14,0]),
                Line(points=[14,0; 90,0]),
                Line(points=[0,80; 0,28],   style(color=73)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Text(extent=[-134,-96; 136,-36],    string="%name")),
              Diagram,
              Documentation(info="<html>
This modulated capacitor is built in bondgraphic technology from a bond graph <b>mC2</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the capacitor in its environment. <p>

It is not recommended to use this model.  The regular <b>VariableCapacitor</b> model should be used always.  For this reason, this model has been hidden inside the Utilities package.  However, since the Ebers-Moll bipolar transistor models make use of this model, it had to be constructed. <p>

<hr>

<pre>
Potential variables:

 C:  Capacitance (modulation input signal)
</pre>
</html>
"));
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
            Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
            Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
            Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                  90);
            Interfaces.El2BG El2BG1                           annotation (extent=[
                  -62,-14; -38,14]);
            Interfaces.BG2El BG2El1                           annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Semiconductors.Utilities.mC2 mC1(e(start=u0)) 
              annotation (extent=[-12,36; 12,60], rotation=90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0;
                  -12,0],                style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0;
                  12.26,0],  style(color=8));
            connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                  -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                         style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                  1.77636e-015; -38,0; -37.74,0],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                  1.77636e-015], style(color=8));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                  1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(B3.BondCon2, mC1.BondCon1) annotation (points=[7.34764e-016,
                  36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
            connect(u, mC1.s) annotation (points=[0,90; 0,74.4; 0,58.8; 6.61287e-016,
                  58.8], style(color=74, rgbcolor={0,0,127}));
          end VariableCapacitor2;
          
          model JunctionDiode "Junction diode of the analog electrical library" 
            extends Interfaces.OnePort;
            parameter Modelica.SIunits.Current Is=1.e-16 
              "Transport saturation current";
            parameter Modelica.SIunits.Voltage Vt=0.02585 
              "Voltage equivalent of temperature (kT/qn)";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            annotation (
              Icon(
                Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                    color=3,
                    gradient=0,
                    fillColor=7)),
                Line(points=[-90,0; -30,0]),
                Line(points=[30,0; 90,0]),
                Line(points=[30,40; 30,-40],   style(color=3)),
                Text(extent=[-100,100; 100,70],   string="%name"),
                Text(
                  extent=[-24,12; 8,-16],
                  string="J",
                  style(color=3, rgbcolor={0,0,255}))),
              Diagram,
              Documentation(info="<html>
The junction diode is built in bondgraphic technology from a bond graph <b>Dj</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the diode in its environment. <p>

The junction diode model is a simple diode with an exponential characteristic. The diode formula essentially is: <p>

<b>f = Is*(exp(e/Vt) - 1)</b> <p>

However, if the exponent <b>e/Vt</b> reaches either an upper limit <b>EMax</b> or a lower limit <b>Emin</b>, the diode characterisic is linearly continued to avoid overflow or underflow, respectively. <p>

<hr>

<pre>
Parameters:

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre>
</html>
"));
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
            Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
            Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
            Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                  90);
            Interfaces.El2BG El2BG1                           annotation (extent=[
                  -62,-14; -38,14]);
            Interfaces.BG2El BG2El1                           annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Semiconductors.Utilities.Dj D1(
              Is=Is,
              Vt=Vt,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-12,36; 12,60], rotation=90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0;
                  -12,0],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0;
                  12.26,0],  style(color=8));
            connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                  -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                         style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                  1.77636e-015; -38,0; -37.74,0],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                  1.77636e-015],
                      style(color=8));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(B3.BondCon2, D1.BondCon1) annotation (points=[7.34764e-016,
                  36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          end JunctionDiode;
          
          model JunctionCapacitor "Non-linear electrical junction capacitor" 
            extends Interfaces.OnePort;
            parameter Modelica.SIunits.Current Is=1.e-16 
              "Transport saturation current";
            parameter Modelica.SIunits.Time Tau=5e-9 "Ideal transit time";
            parameter Modelica.SIunits.Capacitance Cj=0.5e-12 
              "Zero bias depletion capacitance";
            parameter Modelica.SIunits.Voltage Phi=0.8 "Diffusion voltage";
            parameter Real M=0.333 "Gradation exponent";
            parameter Modelica.SIunits.Conductance Gb=1e-15 
              "Leakage conductance";
            parameter Modelica.SIunits.Voltage Vt=0.02585 "Thermal voltage";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Blocks.Interfaces.RealOutput y(redeclare type SignalType 
                = Modelica.SIunits.Current) "Diode current" 
              annotation (extent=[-10,90; 10,110],   rotation=90);
            annotation (
              Icon(
                Polygon(points=[30,0; -30,20; -30,-20; 30,0], style(
                    color=10,
                    rgbcolor={135,135,135},
                    gradient=0,
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[30,20; 30,-20], style(color=10, rgbcolor={135,135,
                        135})),
                Line(points=[-90,0; -14,0]),
                Line(points=[14,0; 90,0]),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Text(extent=[-100,-88; 96,-32],     string="%name"),
                Line(points=[0,90; 0,28],   style(color=74, rgbcolor={0,0,156}))),
              Diagram,
              Documentation(info="<html>
Junction capacitances are used in semiconductor technology to model the charge accumulated in the vicinity of a pn-junction. <p>

The model also computes the diodic current through the junction and puts it out as an output signal. <p>

<hr>

<pre>
Parameters:

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Tau:   Ideal transit time (default value = 5e-9 sec)

 Cj:    Zero bias depletion capacitance (default value = 0.5e-12 F)

 Phi:   Diffusion voltage (default value = 0.8 Volt)

 M:     Gradation exponent (default value = 0.333)

 Gb:    Leakage conductance (default value = 1e-15 Mho)

 Vt:    Thermal voltage (default value = 0.02585 Volt)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre>
</html>
"));
            
            VariableCapacitor2 VariableCapacitor1(enforceStates=enforceStates) 
              annotation (extent=[-70,12; -90,-12], rotation=90);
            BondLib.Semiconductors.Utilities.nlC nlC1(
              Is=Is,
              Tau=Tau,
              Phi=Phi,
              Cj=Cj,
              M=M,
              Vt=Vt,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-44,-10; -64,10],
                                                             rotation=0);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vbi 
              annotation (extent=[-36,-12; -10,12], rotation=-90);
            BondLib.Electrical.Analog.Basic.VCV VCV1 
              annotation (extent=[26,22; 0,-22],  rotation=-180);
            BondLib.Electrical.Analog.Sensors.CurrentSensor Id 
              annotation (extent=[32,40; 52,20],     rotation=0);
            BondLib.Electrical.Analog.Basic.Conductor Conductor1(G=Gb) 
              annotation (extent=[70,-20; 90,20], rotation=-90);
            BondLib.Electrical.Analog.Semiconductors.Utilities.JunctionDiode 
              JunctionDiode1(
              Is=Is,
              Vt=Vt,
              EMin=EMin,
              EMax=EMax) annotation (extent=[50,-20; 70,20], rotation=-90);
            Basic.Ground Gnd1    annotation (extent=[50,-50; 70,-30]);
          equation 
            connect(nlC1.y, VariableCapacitor1.u) annotation (points=[-65,0;
                  -61,0; -61,-6.61287e-016; -71,-6.61287e-016], style(color=74,
                  rgbcolor={0,0,127}));
            connect(Vbi.V, nlC1.u) annotation (points=[-36,-7.34764e-016; -24,
                  -7.34764e-016; -22,0; -42,0], style(color=74, rgbcolor={0,0,
                    127}));
            connect(VariableCapacitor1.p, Vbi.p) annotation (points=[-80,12;
                  -80,30; -23,30; -23,12], style(color=3, rgbcolor={0,0,255}));
            connect(VCV1.p1, Vbi.p) annotation (points=[0,11; 0,30; -23,30; -23,
                  12], style(color=3, rgbcolor={0,0,255}));
            connect(VariableCapacitor1.n, Vbi.n) annotation (points=[-80,-12;
                  -80,-30; -23,-30; -23,-12], style(color=3, rgbcolor={0,0,255}));
            connect(VCV1.n1, Vbi.n) annotation (points=[0,-11; 0,-30; -23,-30;
                  -23,-12], style(color=3, rgbcolor={0,0,255}));
            connect(Id.p, VCV1.p2) annotation (points=[32,30; 26,30; 26,11],
                style(color=3, rgbcolor={0,0,255}));
            connect(Id.n, JunctionDiode1.p) annotation (points=[52,30; 60,30;
                  60,20], style(color=3, rgbcolor={0,0,255}));
            connect(Conductor1.p, Id.n) annotation (points=[80,20; 80,30; 52,30],
                style(color=3, rgbcolor={0,0,255}));
            connect(Id.I, y) annotation (points=[42,40; 42,60; 0,60; 0,100],
                style(color=74, rgbcolor={0,0,127}));
            connect(VCV1.n2, Gnd1.p)    annotation (points=[26,-11; 26,-30; 60,
                  -30], style(color=3, rgbcolor={0,0,255}));
            connect(JunctionDiode1.n, Gnd1.p)    annotation (points=[60,-20; 60,
                  -30], style(color=3, rgbcolor={0,0,255}));
            connect(Conductor1.n, Gnd1.p)    annotation (points=[80,-20; 80,-30;
                  60,-30], style(color=3, rgbcolor={0,0,255}));
            connect(p, VariableCapacitor1.p) annotation (points=[-100,0; -92,0;
                  -92,30; -80,30; -80,12], style(color=3, rgbcolor={0,0,255}));
            connect(n, VariableCapacitor1.n) annotation (points=[100,0; 90,0;
                  90,-60; -80,-60; -80,-12], style(color=3, rgbcolor={0,0,255}));
          end JunctionCapacitor;
          
          model HeatingJunctionDiode 
            "Heating junction diode of the analog electrical library" 
            extends Interfaces.OnePort;
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Current Is=1e-16 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage EG=1.11 
              "Energy gap for temperature effect on saturation current";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            Interfaces.El2BG El2BG1                           annotation (extent=[
                  -62,26; -38,54]);
            Interfaces.BG2El BG2El1                           annotation (extent=[38,
                  26; 64,54]);
            Bonds.eBond B4 annotation (extent=[-12,-44; 12,-20], rotation=-90);
            BondLib.Semiconductors.Utilities.DjS DjS1(
              Tnom=Tnom,
              Is=Is,
              EG=EG,
              N=N,
              XTI=XTI,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-12,-20; 12,4], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-14,-72; 14,-44], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Icon(
                Polygon(points=[30,0; -30,40; -30,-40; 30,0], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[30,40; 30,-40],   style(color=3)),
                Line(points=[0,-20; 0,-91],   style(color=42)),
                Text(extent=[-140,109; 145,59],   string="%name"),
                Line(points=[-90,0; -30,0], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=7,
                    rgbfillColor={255,255,255},
                    fillPattern=1)),
                Line(points=[30,0; 90,0], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=7,
                    rgbfillColor={255,255,255},
                    fillPattern=1)),
                Text(
                  extent=[-22,12; 10,-16],
                  string="J",
                  style(color=3, rgbcolor={0,0,255}))),
              Documentation(info="<html>
The heating junction diode is built in bondgraphic technology from a bond graph <b>DjS</b> element, a 1-junction, and three bonds.  A-causal bonds must be used here, because the causality depends on the embedding of the diode in its environment. <p>

The junction diode model is a simple diode with an exponential characteristic. The diode formula essentially is: <p>

<b>f = Is*(exp(e/Vt) - 1)</b> <p>

However, if the exponent <b>e/Vt</b> reaches either an upper limit <b>EMax</b> or a lower limit <b>Emin</b>, the diode characterisic is linearly continued to avoid overflow or underflow, respectively. <p>

<hr>

<pre>
Parameters:

 Tnom:  Reference temperature (default value = 300.15 K)

 Is:    Transport saturation current (default value = 1e-16 Amp)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 N:     Current emission coefficient (default value = 1)

 XTI:   Saturation current temperature exponent (default value = 3)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre>
</html>
"));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(DjS1.BondCon1, B3.BondCon2) annotation (points=[
                  -7.34764e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(DjS1.BondCon2, B4.fBondCon1) annotation (points=[
                  7.34764e-016,-20; -7.34764e-016,-20], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  9.19132e-016,-44; 7.34764e-016,-44], style(color=8, rgbcolor=
                    {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[2.63358e-015,
                  -72; 2.63358e-015,-83; 0,-83; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end HeatingJunctionDiode;
          
          model HeatingJunctionCapacitor 
            "Non-linear electrical heating junction capacitor" 
            extends Interfaces.OnePort;
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Current Is=1.e-16 
              "Transport saturation current";
            parameter Modelica.SIunits.Time Tau=5e-9 "Ideal transit time";
            parameter Modelica.SIunits.Capacitance Cj=0.5e-12 
              "Zero bias depletion capacitance";
            parameter Modelica.SIunits.Voltage Phi=0.8 "Diffusion voltage";
            parameter Real M=0.333 "Gradation exponent";
            parameter Modelica.SIunits.Voltage EG=1.11 
              "Energy gap for temperature effect on saturation current";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Modelica.SIunits.Conductance Gb=1e-15 
              "Leakage conductance";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Blocks.Interfaces.RealOutput y(redeclare type SignalType 
                = Modelica.SIunits.Current) "Diode current" 
              annotation (extent=[-10,90; 10,110],   rotation=90);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vbi 
              annotation (extent=[-32,-12; -6,12],  rotation=-90);
            BondLib.Electrical.Analog.Basic.VCV VCV1 
              annotation (extent=[26,22; 0,-22],  rotation=-180);
            BondLib.Electrical.Analog.Sensors.CurrentSensor Id 
              annotation (extent=[32,40; 52,20],     rotation=0);
            BondLib.Electrical.Analog.Basic.Conductor Conductor1(G=Gb) 
              annotation (extent=[70,-20; 90,20], rotation=-90);
            BondLib.Electrical.Analog.Basic.Ground Gnd1 
                                 annotation (extent=[50,-50; 70,-30]);
            BondLib.Electrical.Analog.Semiconductors.Utilities.VariableCapacitor2
              VariableCapacitor1(enforceStates=enforceStates) 
              annotation (extent=[-70,12; -90,-12], rotation=90);
            BondLib.Semiconductors.Utilities.nlC2 nlC1(
              Tnom=Tnom,
              Is=Is,
              Tau=Tau,
              Phi=Phi,
              Cj=Cj,
              M=M,
              EG=EG,
              N=N,
              XTI=XTI,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-40,-10; -60,10]);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            BondLib.Electrical.Analog.Semiconductors.Utilities.HeatingJunctionDiode
              HeatingJunctionDiode1(
              Tnom=Tnom,
              Is=Is,
              EG=EG,
              N=N,
              XTI=XTI,
              EMin=EMin,
              EMax=EMax) annotation (extent=[50,-20; 70,20], rotation=-90);
            Modelica.Thermal.HeatTransfer.TemperatureSensor TemperatureSensor1 
              annotation (extent=[-20,-80; -40,-60]);
          equation 
            connect(VCV1.p1,Vbi. p) annotation (points=[0,11; 0,30; -20,30; -20,
                  12; -19,12],
                       style(color=3, rgbcolor={0,0,255}));
            connect(VCV1.n1,Vbi. n) annotation (points=[0,-11; 0,-30; -20,-30;
                  -20,-12; -19,-12],
                            style(color=3, rgbcolor={0,0,255}));
            connect(Id.p,VCV1. p2) annotation (points=[32,30; 26,30; 26,11],
                style(color=3, rgbcolor={0,0,255}));
            connect(Conductor1.p,Id. n) annotation (points=[80,20; 80,30; 52,30],
                style(color=3, rgbcolor={0,0,255}));
            connect(Id.I,y)  annotation (points=[42,40; 42,60; 0,60; 0,100],
                style(color=74, rgbcolor={0,0,127}));
            connect(VCV1.n2,Gnd1. p)    annotation (points=[26,-11; 26,-30; 60,
                  -30], style(color=3, rgbcolor={0,0,255}));
            connect(Conductor1.n,Gnd1. p)    annotation (points=[80,-20; 80,-30;
                  60,-30], style(color=3, rgbcolor={0,0,255}));
            annotation (
              Diagram,
              Icon(
                Polygon(points=[30,0; -30,20; -30,-20; 30,0], style(
                    color=10,
                    rgbcolor={135,135,135},
                    gradient=0,
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[30,20; 30,-20], style(color=10, rgbcolor={135,135,
                        135})),
                Line(points=[-90,0; -14,0]),
                Line(points=[14,0; 90,0]),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Text(extent=[-100,-88; 96,-32],     string="%name"),
                Line(points=[0,90; 0,28],   style(color=74, rgbcolor={0,0,156})),
                Line(points=[0,-28; 0,-90], style(color=42, rgbcolor={206,0,0}))),
              Documentation(info="<html>
Junction capacitances are used in semiconductor technology to model the charge accumulated in the vicinity of a pn-junction. The model also computes the diodic current through the junction and puts it out as an output signal. <p>

The model requires a heat port in spite of the fact that it doesn't heat anything.  The heat port is needed to read the device temperature.

<hr>

<pre>
Parameters:

 Is:    Transport saturation current (default value = 1e-16 Amp)

 Tau:   Ideal transit time (default value = 5e-9 sec)

 Cj:    Zero bias depletion capacitance (default value = 0.5e-12 F)

 Phi:   Diffusion voltage (default value = 0.8 Volt)

 M:     Gradation exponent (default value = 0.333)

 EG:    Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 N:     Current emission coefficient (default value = 1)

 XTI:   Saturation current temperature exponent (default value = 3)

 Gb:    Leakage conductance (default value = 1e-15 Mho)

 EMin:  Minimum exponent (default value = -100)

 EMax:  Maximum exponent (default value = 40)
</pre>
</html>
"));
            connect(Vbi.p, VariableCapacitor1.p) annotation (points=[-19,12;
                  -20,12; -20,30; -80,30; -80,12], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(Vbi.n, VariableCapacitor1.n) annotation (points=[-19,-12;
                  -20,-12; -20,-30; -80,-30; -80,-12], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(Vbi.V, nlC1.u) annotation (points=[-32,-7.34764e-016; -38,
                  -7.34764e-016; -38,0], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(nlC1.y, VariableCapacitor1.u) annotation (points=[-61,0;
                  -65,0; -65,-6.61287e-016; -71,-6.61287e-016], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(HeatingJunctionDiode1.n, Gnd1.p) annotation (points=[60,-20;
                  60,-30], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(HeatingJunctionDiode1.p, Id.n) annotation (points=[60,20;
                  60,30; 52,30], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(TemperatureSensor1.port, heatPort) annotation (points=[-20,
                  -70; 0,-70; 0,-100], style(
                color=42,
                rgbcolor={191,0,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(HeatingJunctionDiode1.heatPort, TemperatureSensor1.port) 
              annotation (points=[50,-1.22461e-015; 50,-70; -20,-70], style(
                color=42,
                rgbcolor={191,0,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(nlC1.Tdev, TemperatureSensor1.T) annotation (points=[-50,
                  -10; -50,-70; -40,-70], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(VariableCapacitor1.p, p) annotation (points=[-80,12; -80,30;
                  -92,30; -92,0; -100,0], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            connect(VariableCapacitor1.n, n) annotation (points=[-80,-12; -80,
                  -50; 92,-50; 92,0; 100,0], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1));
            
          end HeatingJunctionCapacitor;
        end Utilities;
        annotation (preferedView="info",Documentation(info="<html>
This package offers a partial reimplementation of the <i>Semiconductors</i> package of the standard analog electrical Modelica library using bond graph technology.
</html>
"));
      end Semiconductors;
      
      package Sensors "Sensors of analog electrical library" 
        annotation (preferedView="info",Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30,
                  fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],
                style(fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],
                style(fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3)),
            Text(
              extent=[-118,114; 122,65],
              string="%name",
              style(color=1))), Documentation(info="<html>
This package offers a reimplementation of the <i>Sensors</i> package of the standard analog electrical Modelica library using bond graph technology.
</html>
"));
        model PotentialSensor "Sensor of electrical potential" 
          Modelica.Electrical.Analog.Interfaces.PositivePin p 
            "pin to be measured"                        annotation (extent=[
                -110,-10; -90,10]);
          annotation (
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[-29,-11; 30,-70],
                string="V",
                style(color=0)),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[100,0; 70,0]),
              Text(extent=[-150,72; 150,112],   string="%name")),
            Diagram,
            Documentation(info="<html>
Sensor of electrical potential built in bond graph technology using an effort sensor.
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1 annotation (extent=[-60,-20; -20,20]);
          Bonds.fBond B1 annotation (extent=[-20,-20; 20,20]);
          BondLib.Sensors.De De1 annotation (extent=[20,-20; 60,20]);
          
        public 
          Modelica.Blocks.Interfaces.RealOutput phi(redeclare type SignalType 
              = Modelica.SIunits.ElectricPotential) "Potential sensor" 
            annotation (extent=[100,-10; 120,10]);
        equation 
          connect(El2BG1.BondCon1, B1.eBondCon1) annotation (points=[-20,0; -20,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(De1.BondCon1, B1.fBondCon1) annotation (points=[20,0; 20,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(De1.OutPort1, phi) annotation (points=[62,0; 110,0], style(
                color=74, rgbcolor={0,0,127}));
          connect(El2BG1.p, p) annotation (points=[-60,0; -100,0], style(color=
                  3, rgbcolor={0,0,255}));
        end PotentialSensor;
        
        model VoltageSensor "Sensor of electrical voltage" 
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[-29,-11; 30,-70],
                string="V",
                style(color=0)),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0],   style(color=0)),
              Line(points=[0,-90; 0,-70]),
              Text(extent=[-150,72; 150,112],   string="%name")),
            Diagram,
            Documentation(info="<html>
Sensor of electrical voltage built in bond graph technology using an effort sensor.
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,-12; 12,12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,-36; 12,-12], rotation=
                270);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          BondLib.Sensors.De De1 
            annotation (extent=[-12,-60; 12,-36], rotation=-90);
          
        public 
          Modelica.Blocks.Interfaces.RealOutput V(redeclare type SignalType = 
                Modelica.SIunits.Voltage) "Voltage sensor" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],                    style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                2.16021e-015,-12.24; 2.16021e-015,-18.12; 0,-18.12; 0,-12],
              style(color=8, rgbcolor={192,192,192}));
          connect(De1.BondCon1, B3.BondCon2) annotation (points=[-7.34764e-016,
                -36; -2.20429e-015,-36], style(color=8, rgbcolor={192,192,192}));
          connect(De1.OutPort1, V) annotation (points=[8.0824e-016,-61.2;
                8.0824e-016,-74.6; 0,-74.6; 0,-100],
                                         style(color=74, rgbcolor={0,0,127}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end VoltageSensor;
        
        model CurrentSensor "Sensor of electrical current" 
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[-29,-11; 30,-70],
                string="A",
                style(color=0)),
              Text(extent=[-150,72; 150,112],   string="%name"),
              Line(points=[70,0; 90,0],   style(color=0)),
              Line(points=[0,-90; 0,-70]),
              Line(points=[-70,0; -90,0],   style(color=0))),
            Diagram,
            Documentation(info="<html>
Sensor of electrical current built in bond graph technology using a flow sensor.
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,-12; 12,12]);
          Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
          Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
          Bonds.Bond B3         annotation (extent=[-12,-36; 12,-12], rotation=
                270);
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          BondLib.Sensors.Df Df1 
            annotation (extent=[-12,-60; 12,-36], rotation=-90);
          
        public 
          Modelica.Blocks.Interfaces.RealOutput I(redeclare type SignalType = 
                Modelica.SIunits.Current) "Current sensor" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
        equation 
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0; -12,
                0],                    style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0; 12.26,
                0],        style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                1.77636e-015; -38,0; -37.74,0],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                1.77636e-015], style(color=8));
          connect(B3.BondCon1, J1p3_1.BondCon3) annotation (points=[
                2.16021e-015,-12.24; 2.16021e-015,-18.12; 0,-18.12; 0,-12],
              style(color=8, rgbcolor={192,192,192}));
          connect(Df1.BondCon1, B3.BondCon2) annotation (points=[-7.34764e-016,
                -36; -2.20429e-015,-36], style(color=8, rgbcolor={192,192,192}));
          connect(Df1.OutPort1, I) annotation (points=[8.0824e-016,-61.2;
                8.0824e-016,-74.6; 0,-74.6; 0,-100],
                                         style(color=74, rgbcolor={0,0,127}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end CurrentSensor;
      end Sensors;
      
      package Sources "Sources of analog electrical library" 
        annotation (preferedView="info",Icon(
            Rectangle(extent=[-100, -100; 80, 50], style(fillColor=30,
                  fillPattern=1)),
            Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50],
                style(fillColor=30, fillPattern=1)),
            Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70],
                style(fillColor=30, fillPattern=1)),
            Text(
              extent=[-85, 35; 65, -85],
              string="Library",
              style(color=3)),
            Text(
              extent=[-118, 114; 122, 65],
              string="%name",
              style(color=1))), Documentation(info="<html>
This package offers a reimplementation of the <i>Sources</i> package of the standard analog electrical Modelica library using bond graph technology.
</html>
"));
        model SignalVoltage 
          "Modulated voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Line(points=[-90, 0; -50, 0]),
              Line(points=[50, 0; 90, 0]),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[-50,0; 50,0],   style(color=0, rgbcolor={0,0,0})),
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-100,-120; 100,-80],   string="%name")),
            Diagram,
            Documentation(info="<html>
The modulated voltage source is built in bondgraphic technology from a bond graph <b>mSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment.
</html>
"));
          Modelica.Blocks.Interfaces.RealInput InPort1(redeclare type 
              SignalType = 
                Modelica.SIunits.Voltage) "Voltage source" 
                                                    annotation (extent=[-10, 70;
                  10, 50], rotation=90);
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -34; -38, -6]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -34; 64, -6]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, -8; 12, -32]);
          BondLib.Bonds.Bond B1 annotation (extent=[12, -32; 38, -8], rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38, -32; -12, -8],
              rotation=180);
          BondLib.Sources.mSe V1 annotation (extent=[-12, 16; 12, 40], rotation=
               90);
          BondLib.Bonds.fBond B3 annotation (extent=[-12, -8; 12, 16], rotation=
               -90);
        equation 
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12, -20; 12,
                  -20], style(color=8));
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,-20;
                37.74,-20],
                        style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38, -20; -
                38, -20], style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,-20;
                -12.26,-20],
                          style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                16; -7.34764e-016,16],
                              style(color=8));
          connect(B3.fBondCon1, J1p3_1.BondCon3) annotation (points=[
                7.34764e-016,-8; 0,-8],   style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,-20; -80,-20; -80,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,-20; 80,-20; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(V1.s, InPort1) annotation (points=[6.61287e-016,38.8;
                6.61287e-016,45.4; 0,45.4; 0,60], style(color=74, rgbcolor={0,0,
                  127}));
        end SignalVoltage;
        
        model ConstantVoltage 
          "Constant voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Value of constant voltage";
          annotation (
            Icon(
              Line(points=[-10,60; -10,-60],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,30; 0,-30],   style(color=0, rgbcolor={0,0,0})),
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Line(points=[-90,0; -10,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-100,-120; 100,-80],   string="%name=%V")),
            Diagram,
            Documentation(info="<html>
The constant voltage source is built in bondgraphic technology from a bond graph <b>Se</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:  Constant source voltage (default value = 1 Volt)
</pre>
</html>
"));
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[12, -12; 38, 12], rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38, -12; -12, 12],
              rotation=180);
          BondLib.Bonds.fBond B3 annotation (extent=[-12, 12; 12, 36], rotation=
               -90);
          BondLib.Sources.Se V1(e0=V) annotation (extent=[-12, 36; 12, 60],
              rotation=90);
        equation 
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,
                1.77636e-015; 38,1.77636e-015; 38,-1.44014e-015; 37.74,
                -1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1, J1p3_1.BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36],
                style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end ConstantVoltage;
        
        model StepVoltage "Step voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Height of step";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
                 Line(points=[-70,-70; 0,-70; 0,70; 69,70],     style(color=8)),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The step voltage source is built in bondgraphic technology from a bond graph <b>stepSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Height of source voltage step (default value = 1 Volt)

 offset:     Offset of step to be generated (default value = 0 Volt)

 startTime:  Start time of step to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.stepSe V1(
            e0=V,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end StepVoltage;
        
        model RampVoltage "Ramp voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Height of ramp";
          parameter Modelica.SIunits.Time duration(min=Modelica.Constants.small) = 2 
            "Duration of ramp";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
                 Line(points=[-80,-60; -50,-60; 50,60; 80,60],     style(color=8)),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The ramp voltage source is built in bondgraphic technology from a bond graph <b>rampSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Height of source voltage ramp (default value = 1 Volt)

 duration:   Duration of source voltage ramp (default value = 2 sec)

 offset:     Offset of ramp to be generated (default value = 0 Volt)

 startTime:  Start time of ramp to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.rampSe V1(
            e0=V,
            duration=duration,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -78,
                1.77636e-015; -78,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end RampVoltage;
        
        model SineVoltage 
          "Sinusoidal voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Amplitude of sine wave";
          parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
          parameter Modelica.SIunits.Frequency freqHz=1 
            "Frequency of sine wave";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                 Line(points=[-66,0; -56.2,29.9; -49.8,46.5; -44.2,58.1; -39.3,
                    65.2; -34.3,69.2; -29.4,69.8; -24.5,67; -19.6,61; -14.6,52;
                    -9,38.6; -1.98,18.6; 12.79,-26.9; 19.1,-44; 24.8,-56.2;
                    29.7,-64; 34.6,-68.6; 39.5,-70; 44.5,-67.9; 49.4,-62.5;
                    54.3,-54.1; 59.9,-41.3; 67,-21.7; 74,0],
                                                          style(color=8)),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The sinusoidal voltage source is built in bondgraphic technology from a bond graph <b>sinSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Amplitude of source voltage (default value = 1 Volt)

 phase:      Phase of sine wave to be generated (default value = 0 rad)

 freqHz:     Frequency of sine wave to be generated (default value = 1 Hz)

 offset:     Offset of sine wave to be generated (default value = 0 Volt)

 startTime:  Start time of sine wave to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[12, -12; 38, 12], rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38, -12; -12, 12],
              rotation=180);
          BondLib.Bonds.fBond B3 annotation (extent=[-12, 12; 12, 36], rotation=
               -90);
          BondLib.Sources.sinSe V1(
            e0=V,
            phase=phase,
            freqHz=freqHz,
            offset=offset,
            startTime=startTime) annotation (extent=[-12, 36; 12, 60], rotation=
               90);
        equation 
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,
                1.77636e-015; 38,1.77636e-015; 38,-1.44014e-015; 37.74,
                -1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1, J1p3_1.BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36],
                style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 82,
                1.77636e-015; 82,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end SineVoltage;
        
        model ExpSineVoltage 
          "Damped sinusoidal voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Amplitude of sine wave";
          parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
          parameter Modelica.SIunits.Frequency freqHz=2 
            "Frequency of sine wave";
          parameter Modelica.SIunits.Damping damping=1 
            "Damping coefficient of sine wave";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
                 Line(points=[-64,-14; -59.2,18.3; -56,36.3; -52.7,50.5; -49.5,
                    60.2; -46.3,65.3; -43.1,65.6; -39.9,61.3; -36.7,53.1; -32.6,
                    38.2; -27,11.8; -19,-27.9; -14.2,-47.7; -10.1,-59.9; -6.1,
                    -67.2; -2.1,-69.3; 1.9,-66.5; 5.9,-59.3; 10.77,-46.1; 24.44,
                    -0.3; 29.3,12.4; 34.1,20.8; 38.1,24; 42.9,23.2; 47.8,17.8;
                    54.2,5.4; 67.1,-24.5; 73.5,-35.2; 79.1,-39.9; 84.7,-39.9;
                    91.2,-34.5; 96,-27.8],         style(color=8)),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The damped sinusoidal voltage source is built in bondgraphic technology from a bond graph <b>expsinSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Amplitude of source voltage (default value = 1 Volt)

 phase:      Phase of damped sine wave to be generated (default value = 0 rad)

 freqHz:     Frequency of damped sine wave to be generated (default value = 2 Hz)

 damping:    Damping coefficient of damped sine wave to be generated (default value = 1 Hz)

 offset:     Offset of sine wave to be generated (default value = 0 Volt)

 startTime:  Start time of sine wave to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.expsinSe V1(
            e0=V,
            phase=phase,
            freqHz=freqHz,
            damping=damping,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end ExpSineVoltage;
        
        model ExponentialsVoltage 
          "Step response voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Amplitude of sine wave";
          parameter Modelica.SIunits.Time riseTime(min=0) = 0.5 "Rise time";
          parameter Modelica.SIunits.Time riseTimeConst(min=Modelica.Constants.small) = 0.1 
            "Rise time constant";
          parameter Modelica.SIunits.Time fallTimeConst(min=Modelica.Constants.small) = riseTimeConst 
            "Fall time constant";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                 Line(points=[-76,-59; -73.2,-44.3; -70.3,-31.1; -66.8,-16.6;
                    -63.3,-4; -59.7,6.92; -55.5,18.18; -51.3,27.7; -46.3,37;
                    -40.6,45.5; -34.3,53.1; -27.2,59.6; -18.7,65.3; -8.1,70.2;
                    -6,71; -3.88,58.5; -1.05,43.7; 1.78,30.8; 4.606,19.45; 8.14,
                    7.3; 11.68,-3; 15.9,-13.2; 20.2,-21.6; 25.1,-29.5; 30.8,
                    -36.4; 37.1,-42.3; 44.9,-47.5; 54.8,-51.8; 64,-54.4],    style(
                    color=8)),
              Text(extent=[-150,80; 150,120], string="%name"),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The step response voltage source is built in bondgraphic technology from a bond graph <b>expSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:              Height of step response source voltage after infinite time (default value = 1 Volt)

 riseTime:       Rise time of step response to be generated (default value = 0.5 sec)

 riseTimeConst:  Rise time constant of step response to be generated (default value = 0.1)

 fallTimeConst:  Fall time constant of step response to be generated (default value = riseTimeConst)

 offset:         Offset of step response to be generated (default value = 0 Volt)

 startTime:      Start time of step response to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.expSe V1(
            riseTime=riseTime,
            riseTimeConst=riseTimeConst,
            fallTimeConst=fallTimeConst,
            offset=offset,
            startTime=startTime,
            e0=V) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(V1.BondCon1, B3.eBondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end ExponentialsVoltage;
        
        model PulseVoltage "Pulse voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Amplitude of pulse";
          parameter Real width(
            final min=Modelica.Constants.small,
            final max=100) = 50 "Width of pulse in % of period";
          parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
            "Time for one period";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
                 Line(points=[-70,-70; -40,-70; -40,70; 0,70; 0,-70; 40,-70; 40,
                    70; 80,70],    style(color=8)),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The pulse voltage source is built in bondgraphic technology from a bond graph <b>pulseSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Amplitude of source voltage pulse (default value = 1 Volt)

 width:      Percentage of period in on state (default value = 50%)

 period:     Length of period of pulse to be generated (default value = 1 sec)

 offset:     Offset of pulse to be generated (default value = 0 Volt)

 startTime:  Start time of pulse to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.pulseSe V1(
            width=width,
            period=period,
            offset=offset,
            startTime=startTime,
            e0=V) annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 79,
                1.77636e-015; 79,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end PulseVoltage;
        
        model SawToothVoltage 
          "Saw tooth voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 "Amplitude of saw tooth";
          parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
            "Time for one period";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
                 Line(points=[-85,-70; -65,-70; -5,71; -5,-70; 55,71; 55,-70],
                  style(color=8)),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The saw tooth voltage source is built in bondgraphic technology from a bond graph <b>sawSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Amplitude of source voltage saw tooth (default value = 1 Volt)

 period:     Length of period of saw tooth to be generated (default value = 1 sec)

 offset:     Offset of saw tooth to be generated (default value = 0 Volt)

 startTime:  Start time of saw tooth to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.sawSe V1(
            e0=V,
            period=period,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end SawToothVoltage;
        
        model TrapezoidVoltage 
          "Trapezoidal voltage source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Voltage V=1 
            "Amplitude of trapezoidal function";
          parameter Modelica.SIunits.Time rising(final min=0) = 0 
            "Rising duration of trapezoid";
          parameter Modelica.SIunits.Time width(final min=0) = 0.5 
            "Width duration of trapezoid";
          parameter Modelica.SIunits.Time falling(final min=0) = 0 
            "Falling duration of trapezoid";
          parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
            "Time for one period";
          parameter Integer nperiod=-1 
            "Number of periods (< 0 means infinite number of periods)";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0})),
                 Line(points=[-81,-70; -60,-70; -30,70; 1,70; 30,-70; 51,-70;
                    80,70],  style(color=8))),
            Diagram,
            Documentation(info="<html>
The trapezoidal voltage source is built in bondgraphic technology from a bond graph <b>trapSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Amplitude of source voltage trapezoid (default value = 1 Volt)

 rising:     Length of rising time of trapezoid to be generated (default value = 0 sec)

 width :     Length of on state of trapezoid to be generated (default value = 0.5 sec)

 falling:    Length of falling time of trapezoid to be generated (default value = 0 sec)

 period:     Length of period of trapezoid to be generated (default value = 1 sec)

 nperiod:    Number of periods of trapezoid to be generated
             an infinite number of periods is being generated if nperiod &lt; 0 (default value = -1)

 offset:     Offset of trapezoid to be generated (default value = 0 Volt)

 startTime:  Start time of trapezoid to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.trapSe V1(
            e0=V,
            rising=rising,
            width=width,
            falling=falling,
            period=period,
            nperiod=nperiod,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end TrapezoidVoltage;
        
        model TableVoltage 
          "Tabular voltage source of analog electrical library" 
          parameter Real table[:, :]=[0, 0; 1, 1; 2, 4] 
            "Table matrix (time = first column, voltage = second column)";
          parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Text(
                extent=[-120,50; -20,0],
                style(color=3, rgbcolor={0,0,255}),
                string="+"),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[20,50; 120,0],
                style(color=3, rgbcolor={0,0,255}),
                string="-"),
              Text(extent=[-150,80; 150,120], string="%name"),
                 Line(points=[-58,-36; -58,84; 42,84; 42,-36; -58,-36; -58,-6;
                    42,-6; 42,24; -58,24; -58,54; 42,54; 42,84; -8,84; -8,-37],
                         style(color=8)),
              Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
            Diagram,
            Documentation(info="<html>
The tabular voltage source is built in bondgraphic technology from a bond graph <b>tableSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the voltage source in its environment. <p>

<hr>

<pre>
Parameters:

 table:      Two-dimensional table with supporting values (default value = [0, 0; 1, 1; 2, 4])

 offset:     Offset of time function to be generated (default value = 0 Volt)

 startTime:  Start time of time function to be generated (default value = 0 sec)
</pre> <p>

<hr> <p>

The model generates a voltage source by <b>linear interpolation</b> in
a table. The time points and voltage values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
time points, and the second column contains the voltage to be interpolated.
The table interpolation has the following proporties: <p>

<ul>
<li>The time points need to be <b>monotonically increasing</b>.
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table.
<li>Values <b>outside</b> of the table range, are computed by
    <b>extrapolation</b> through the last or first two points of the
    table.
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the voltage value is just returned independantly of the
    actual time instant, i.e., this is a constant voltage source.
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined
    by the table can be shifted both in time and in the voltage.
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries,
    in order to not integrate over a discontinuous or not differentiable
    points.
</ul> <p>

<hr> <p>

<b>Example:</b> <p>

<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the voltage v =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the voltage v =  2.5,
    e.g., time = 2.0, the voltage v =  4.0,
    e.g., time = 5.0, the voltage v = 23.0 (i.e. extrapolation).
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.fBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.tableSe V1(
            table=table,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.eBondCon1, V1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end TableVoltage;
        
        model SignalCurrent 
          "Modulated current source of analog electrical library" 
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-70; 60,-70], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-70; 30,-60; 30,-80; 60,-70], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-100,-120; 100,-80],   string="%name")),
            Diagram,
            Documentation(info="<html>
The modulated current source is built in bondgraphic technology from a bond graph <b>mSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment.
</html>
"));
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                -62,-34; -38,-6]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                -34; 64,-6]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,-8; 12,-32]);
          BondLib.Bonds.Bond B1 annotation (extent=[12,-32; 38,-8],   rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38,-32; -12,-8],
              rotation=180);
          
        public 
          Modelica.Blocks.Interfaces.RealInput InPort1(redeclare type 
              SignalType = 
                Modelica.SIunits.Current) "Current source" 
                                                    annotation (extent=[-10, 50;
                  10, 70], rotation=-90);
          
        protected 
          BondLib.Sources.mSf I1 annotation (extent=[-12,16; 12,40],   rotation=
               90);
          BondLib.Bonds.eBond B3 annotation (extent=[-12,-8; 12,16],   rotation=
               -90);
        equation 
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,-20; 12,
                -20],   style(color=8));
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,-20;
                37.74,-20],
                        style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38,-20;
                -38,-20], style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,-20;
                -12.26,-20],
                          style(color=8));
          connect(B3.eBondCon1, J1p3_1.BondCon3) annotation (points=[
                7.34764e-016,-8; 0,-8],   style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                16; -7.34764e-016,16],
                              style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,-20; -80,-20; -80,0;
                -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,-20; 80,-20; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(I1.s, InPort1) annotation (points=[6.61287e-016,38.8;
                6.61287e-016,45.4; 0,45.4; 0,60], style(color=74, rgbcolor={0,0,
                  127}));
        end SignalCurrent;
        
        model ConstantCurrent 
          "Constant current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Value of constant current";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-100,80; 100,120],     string="%name=%I")),
            Diagram,
            Documentation(info="<html>
The constant current source is built in bondgraphic technology from a bond graph <b>Sf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 I:  Constant source current (default value = 1 Amp)
</pre>

</html>
"));
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[12, -12; 38, 12], rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38, -12; -12, 12],
              rotation=180);
          BondLib.Bonds.eBond B3 annotation (extent=[-12, 12; 12, 36], rotation=
               -90);
          BondLib.Sources.Sf I1(f0=I) annotation (extent=[-12, 36; 12, 60],
              rotation=90);
        equation 
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,
                1.77636e-015; 38,1.77636e-015; 38,-1.44014e-015; 37.74,
                -1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(B3.eBondCon1, J1p3_1.BondCon3) annotation (points=[
                7.34764e-016,12; 0,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36],
                style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end ConstantCurrent;
        
        model StepCurrent "Step current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Height of step";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-86,-70; -14,-70; -14,70; 57,70],     style(color=8))),
            Diagram,
            Documentation(info="<html>
The step current source is built in bondgraphic technology from a bond graph <b>stepSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Height of step current (default value = 1 Amp)

 offset:     Offset of step to be generated (default value = 0 Amp)

 startTime:  Start time of step to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.stepSf I1(
            f0=I,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 79,
                1.77636e-015; 79,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end StepCurrent;
        
        model RampCurrent "Ramp current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Height of ramp";
          parameter Modelica.SIunits.Time duration(min=Modelica.Constants.small) = 2 
            "Duration of ramp";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-80,-60; -50,-60; 50,60; 80,60],     style(color=8))),
            Diagram,
            Documentation(info="<html>
The ramp current source is built in bondgraphic technology from a bond graph <b>rampSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 I:          Height of source current ramp (default value = 1 Amp)

 duration:   Duration of source current ramp (default value = 2 sec)

 offset:     Offset of ramp to be generated (default value = 0 Amp)

 startTime:  Start time of ramp to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.rampSf I1(
            f0=I,
            duration=duration,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end RampCurrent;
        
        model SineCurrent 
          "Sinusoidal current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Amplitude of sine wave";
          parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
          parameter Modelica.SIunits.Frequency freqHz=1 
            "Frequency of sine wave";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-70,0; -60.2,29.9; -53.8,46.5; -48.2,58.1; -43.3,
                    65.2; -38.3,69.2; -33.4,69.8; -28.5,67; -23.6,61; -18.6,52;
                    -13,38.6; -5.98,18.6; 8.79,-26.9; 15.1,-44; 20.8,-56.2;
                    25.7,-64; 30.6,-68.6; 35.5,-70; 40.5,-67.9; 45.4,-62.5;
                    50.3,-54.1; 55.9,-41.3; 63,-21.7; 70,0],
                                                          style(color=8))),
            Diagram,
            Documentation(info="<html>
The sinusoidal current source is built in bondgraphic technology from a bond graph <b>sinSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 V:          Amplitude of source current (default value = 1 Amp)

 phase:      Phase of sine wave to be generated (default value = 0 rad)

 freqHz:     Frequency of sine wave to be generated (default value = 1 Hz)

 offset:     Offset of sine wave to be generated (default value = 0 Amp)

 startTime:  Start time of sine wave to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=
                [-62, -14; -38, 14]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=
                [38, -14; 64, 14]);
          BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12, 12; 12, -12]);
          BondLib.Bonds.Bond B1 annotation (extent=[12, -12; 38, 12], rotation=
                180);
          BondLib.Bonds.Bond B2 annotation (extent=[-38, -12; -12, 12],
              rotation=180);
          BondLib.Bonds.eBond B3 annotation (extent=[-12, 12; 12, 36], rotation=
               -90);
          BondLib.Sources.sinSf I1(
            f0=I,
            phase=phase,
            freqHz=freqHz,
            offset=offset,
            startTime=startTime) annotation (extent=[-12, 36; 12, 60], rotation=
               90);
        equation 
          connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1, B1.BondCon1) annotation (points=[38,
                1.77636e-015; 38,1.77636e-015; 38,-1.44014e-015; 37.74,
                -1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1, B2.BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36],
                style(color=8));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end SineCurrent;
        
        model ExpSineCurrent 
          "Damped sinusoidal current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Amplitude of sine wave";
          parameter Modelica.SIunits.Angle phase=0 "Phase of sine wave";
          parameter Modelica.SIunits.Frequency freqHz=2 
            "Frequency of sine wave";
          parameter Modelica.SIunits.Damping damping=1 
            "Damping coefficient of sine wave";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-80,-14; -75.2,18.3; -72,36.3; -68.7,50.5; -65.5,
                    60.2; -62.3,65.3; -59.1,65.6; -55.9,61.3; -52.7,53.1; -48.6,
                    38.2; -43,11.8; -35,-27.9; -30.2,-47.7; -26.1,-59.9; -22.1,
                    -67.2; -18.1,-69.3; -14.1,-66.5; -10.1,-59.3; -5.23,-46.1;
                    8.44,-0.3; 13.3,12.4; 18.1,20.8; 22.1,24; 26.9,23.2; 31.8,
                    17.8; 38.2,5.4; 51.1,-24.5; 57.5,-35.2; 63.1,-39.9; 68.7,
                    -39.9; 75.2,-34.5; 80,-27.8],  style(color=8))),
            Diagram,
            Documentation(info="<html>
The damped sinusoidal current source is built in bondgraphic technology from a bond graph <b>expsinSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 I:          Amplitude of source current (default value = 1 Amp)

 phase:      Phase of damped sine wave to be generated (default value = 0 rad)

 freqHz:     Frequency of damped sine wave to be generated (default value = 2 Hz)

 damping:    Damping coefficient of damped sine wave to be generated (default value = 1 Hz)

 offset:     Offset of sine wave to be generated (default value = 0 Amp)

 startTime:  Start time of sine wave to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.expsinSf I1(
            f0=I,
            phase=phase,
            freqHz=freqHz,
            damping=damping,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end ExpSineCurrent;
        
        model ExponentialsCurrent 
          "Step response current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Amplitude of step response";
          parameter Modelica.SIunits.Time riseTime(min=0) = 0.5 "Rise time";
          parameter Modelica.SIunits.Time riseTimeConst(min=Modelica.Constants.small) = 0.1 
            "Rise time constant";
          parameter Modelica.SIunits.Time fallTimeConst(min=Modelica.Constants.small) = riseTimeConst 
            "Fall time constant";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-76,-59; -73.2,-44.3; -70.3,-31.1; -66.8,-16.6;
                    -63.3,-4; -59.7,6.92; -55.5,18.18; -51.3,27.7; -46.3,37;
                    -40.6,45.5; -34.3,53.1; -27.2,59.6; -18.7,65.3; -8.1,70.2;
                    -6,71; -3.88,58.5; -1.05,43.7; 1.78,30.8; 4.606,19.45; 8.14,
                    7.3; 11.68,-3; 15.9,-13.2; 20.2,-21.6; 25.1,-29.5; 30.8,
                    -36.4; 37.1,-42.3; 44.9,-47.5; 54.8,-51.8; 64,-54.4],    style(
                    color=8))),
            Diagram,
            Documentation(info="<html>
The step response current source is built in bondgraphic technology from a bond graph <b>expSe</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 I:              Height of step response source current after infinite time (default value = 1 Amp)

 riseTime:       Rise time of step response to be generated (default value = 0.5 sec)

 riseTimeConst:  Rise time constant of step response to be generated (default value = 0.1)

 fallTimeConst:  Fall time constant of step response to be generated (default value = riseTimeConst)

 offset:         Offset of step response to be generated (default value = 0 Amp)

 startTime:      Start time of step response to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.expSf I1(
            riseTime=riseTime,
            riseTimeConst=riseTimeConst,
            fallTimeConst=fallTimeConst,
            offset=offset,
            startTime=startTime,
            f0=I) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -81,
                1.77636e-015; -81,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end ExponentialsCurrent;
        
        model PulseCurrent "Pulse current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Amplitude of pulse";
          parameter Real width(
            final min=Modelica.Constants.small,
            final max=100) = 50 "Width of pulse in % of period";
          parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
            "Time for one period";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
                 Line(points=[-70,-67; -40,-67; -40,73; 0,73; 0,-67; 40,-67; 40,
                    73; 80,73],    style(color=8))),
            Diagram,
            Documentation(info="<html>
The pulse current source is built in bondgraphic technology from a bond graph <b>pulseSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 I:          Amplitude of source current pulse (default value = 1 Amp)

 width:      Percentage of period in on state (default value = 50%)

 period:     Length of period of pulse to be generated (default value = 1 sec)

 offset:     Offset of pulse to be generated (default value = 0 Amp)

 startTime:  Start time of pulse to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.pulseSf I1(
            f0=I,
            width=width,
            period=period,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 82,
                1.77636e-015; 82,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end PulseCurrent;
        
        model SawToothCurrent 
          "Saw tooth current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 "Amplitude of saw tooth";
          parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
            "Time for one period";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-70,-71; -50,-71; 10,70; 10,-71; 70,70; 70,-71],
                  style(color=8))),
            Diagram,
            Documentation(info="<html>
The saw tooth current source is built in bondgraphic technology from a bond graph <b>sawSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 I:          Amplitude of source current saw tooth (default value = 1 Amp)

 period:     Length of period of saw tooth to be generated (default value = 1 sec)

 offset:     Offset of saw tooth to be generated (default value = 0 Amp)

 startTime:  Start time of saw tooth to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.sawSf I1(
            f0=I,
            period=period,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -79,
                1.77636e-015; -79,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end SawToothCurrent;
        
        model TrapezoidCurrent 
          "Trapezoidal current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Current I=1 
            "Amplitude of trapezoidal function";
          parameter Modelica.SIunits.Time rising(final min=0) = 0 
            "Rising duration of trapezoid";
          parameter Modelica.SIunits.Time width(final min=0) = 0.5 
            "Width duration of trapezoid";
          parameter Modelica.SIunits.Time falling(final min=0) = 0 
            "Falling duration of trapezoid";
          parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
            "Time for one period";
          parameter Integer nperiod=-1 
            "Number of periods (< 0 means infinite number of periods)";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-81,-66; -60,-66; -30,74; 1,74; 30,-66; 51,-66;
                    80,74],  style(color=8))),
            Diagram,
            Documentation(info="<html>
The trapezoidal current source is built in bondgraphic technology from a bond graph <b>trapSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 I:          Amplitude of source current trapezoid (default value = 1 Amp)

 rising:     Length of rising time of trapezoid to be generated (default value = 0 sec)

 width :     Length of on state of trapezoid to be generated (default value = 0.5 sec)

 falling:    Length of falling time of trapezoid to be generated (default value = 0 sec)

 period:     Length of period of trapezoid to be generated (default value = 1 sec)

 nperiod:    Number of periods of trapezoid to be generated
             an infinite number of periods is being generated if nperiod &lt; 0 (default value = -1)

 offset:     Offset of trapezoid to be generated (default value = 0 Amp)

 startTime:  Start time of trapezoid to be generated (default value = 0 sec)
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.trapSf I1(
            f0=I,
            rising=rising,
            width=width,
            falling=falling,
            period=period,
            nperiod=nperiod,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end TrapezoidCurrent;
        
        model TableCurrent 
          "Tabular current source of analog electrical library" 
          extends Interfaces.OnePort;
          parameter Real table[:, :]=[0, 0; 1, 1; 2, 4] 
            "Table matrix (time = first column, current = second column)";
          parameter Modelica.SIunits.Current offset=0 "Current offset";
          parameter Modelica.SIunits.Time startTime=0 "Time offset";
          annotation (
            Icon(
              Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-50,50; 50,-50],   style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
              Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
              Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(extent=[-150,120; 150,80],   string="%name"),
                 Line(points=[-66,-36; -66,84; 34,84; 34,-36; -66,-36; -66,-6;
                    34,-6; 34,24; -66,24; -66,54; 34,54; 34,84; -16,84; -16,-37],
                         style(color=8))),
            Diagram,
            Documentation(info="<html>
The tabular current source is built in bondgraphic technology from a bond graph <b>tableSf</b> element, a 1-junction, and three bonds.  A-causal bonds must be used except at the source, because the causality depends on the embedding of the current source in its environment. <p>

<hr>

<pre>
Parameters:

 table:      Two-dimensional table with supporting values (default value = [0, 0; 1, 1; 2, 4])

 offset:     Offset of time function to be generated (default value = 0 Volt)

 startTime:  Start time of time function to be generated (default value = 0 sec)
</pre> <p>

<hr> <p>

The model generates a current source by <b>linear interpolation</b> in
a table. The time points and current values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
time points, and the second column contains the current to be interpolated.
The table interpolation has the following proporties: <p>

<ul>
<li>The time points need to be <b>monotonically increasing</b>.
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table.
<li>Values <b>outside</b> of the table range, are computed by
    <b>extrapolation</b> through the last or first two points of the
    table.
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the voltage value is just returned independantly of the
    actual time instant, i.e., this is a constant voltage source.
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined
    by the table can be shifted both in time and in the voltage.
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries,
    in order to not integrate over a discontinuous or not differentiable
    points.
</ul> <p>

<hr> <p>

<b>Example:</b> <p>

<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the current i =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the current i =  2.5,
    e.g., time = 2.0, the current i =  4.0,
    e.g., time = 5.0, the current i = 23.0 (i.e. extrapolation).
</pre>
</html>
"));
          
        protected 
          Interfaces.El2BG El2BG1                           annotation (extent=[
                -62,-14; -38,14]);
          Interfaces.BG2El BG2El1                           annotation (extent=[38,
                -14; 64,14]);
          Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
          Bonds.Bond B1         annotation (extent=[12,-12; 38,12],   rotation=
                180);
          Bonds.Bond B2         annotation (extent=[-38,-12; -12,12],
              rotation=180);
          Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
               -90);
          BondLib.Sources.tableSf I1(
            table=table,
            offset=offset,
            startTime=startTime) 
            annotation (extent=[-12,36; 12,60], rotation=90);
        equation 
          connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                1.46953e-015],
                    style(color=8));
          connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                    style(color=8));
          connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                1.77636e-015; -38,1.46953e-015],
                      style(color=8));
          connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                      style(color=8));
          connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                7.34764e-016,12],
                     style(color=8));
          connect(B3.fBondCon1, I1.BondCon1) annotation (points=[-7.34764e-016,
                36; -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                1.77636e-015; -80,0; -100,0],
                                style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end TableCurrent;
      end Sources;
      
      package Spice "Spice-style models of electronic circuits" 
        annotation (preferedView="info",Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30,
                  fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],
                style(fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],
                style(fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3)),
            Text(
              extent=[-118,114; 122,65],
              string="%name",
              style(color=1))), Documentation(info="<html>
This package contains Spice-style models of electronic circuit elements.  The library implements facets of a dialect of <b>HSpice</b>, called <b>BBSpice</b>, further developed from HSpice at <i>Burr Brown, Inc.</i> in Tucson Arizona, a company that has meanwhile been acquired by <i>Texas Instruments, Inc.</i>  The semiconductor models are further decomposed into equivalent circuits at the graphic circuit diagram layer to make the models more easily understandable. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, Chapter 6.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        package Interfaces "Common interfaces of Spice-style models" 
          annotation (preferedView="info",Icon(
              Text(
                extent=[-118,112; 122,63],
                string="%name",
                style(color=1)),
              Rectangle(extent=[-100,-100; 80,50],   style(
                  color=58,
                  fillColor=30,
                  fillPattern=1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],
                  style(
                  color=58,
                  fillColor=30,
                  fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],
                  style(
                  color=58,
                  fillColor=30,
                  fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=58))), Documentation(info="<html>
This package contains common interfaces of Spice-style electronic circuit element models.
</html>
"));
          connector SPin "Differently colored pin for substrate node" 
            Modelica.SIunits.Voltage v "Potential at the pin";
            flow Modelica.SIunits.Current i "Current flowing into the pin";
            annotation (Diagram(
                      Rectangle(extent=[-40,40; 40,-40], style(
                    color=2,
                    rgbcolor={0,255,0},
                    fillColor=2,
                    rgbfillColor={0,255,0})),
                            Text(
                  extent=[-170,108; 30,48],
                  string="%name",
                  style(
                    color=2,
                    rgbcolor={0,255,0},
                    fillColor=2,
                    rgbfillColor={0,255,0}))), Icon(
                   Rectangle(extent=[-100,100; 100,-100], style(
                    color=2,
                    rgbcolor={0,255,0},
                    fillColor=2,
                    rgbfillColor={0,255,0}))));
          end SPin;
          
          partial model SpiceTwoPort "TwoPort interface for Spice-style models" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(
                  start=300.15)) "may remain unconnected" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            annotation (Icon(
                Text(extent=[-40,86; 40,56],   string="%name")), Documentation(
                  info="<html>
All Spice-style OnePorts must have an additional HeatPort, even capacitors and inductors.  The reason is that they need the current circuit temperature, which they import through the HeatPort.
</html>
"),           Diagram);
          end SpiceTwoPort;
          
          partial model SpiceTwoPortV 
            "TwoPort interface for Spice-style models" 
            extends BondLib.Electrical.Analog.Interfaces.OnePortV;
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(
                  start=300.15)) "may remain unconnected" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            annotation (Icon(
                Text(extent=[-40,86; 40,56],   string="%name")), Documentation(
                  info="<html>
All Spice-style OnePorts must have an additional HeatPort, even capacitors and inductors.  The reason is that they need the current circuit temperature, which they import through the HeatPort.
</html>
"),           Diagram);
          end SpiceTwoPortV;
          
          partial model SpiceTwoPortI 
            "TwoPort interface for Spice-style models" 
            extends BondLib.Electrical.Analog.Interfaces.OnePortI;
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(
                  start=300.15)) "may remain unconnected" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            annotation (Icon(
                Text(extent=[-40,86; 40,56],   string="%name")), Documentation(
                  info="<html>
All Spice-style OnePorts must have an additional HeatPort, even capacitors and inductors.  The reason is that they need the current circuit temperature, which they import through the HeatPort.
</html>
"),           Diagram);
          end SpiceTwoPortI;
        end Interfaces;
        
        package RampingSources "Sources with ramping for Spice" 
          annotation (preferedView="info",Icon(
              Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30,
                    fillPattern=1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],
                  style(fillColor=30, fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],
                  style(fillColor=30, fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=3)),
              Text(
                extent=[-118,114; 122,65],
                string="%name",
                style(color=1))), Documentation(info="<html>
Complex semiconductor circuits with realistic device parameters cannot be simulated in Spice without first computing a <b>DC OP-point</b>, i.e., an initial <b>nodeset</b> that is close to steady-state conditions.  The Spice library of Modelica doesn't currently offer DC analysis.  Furthermore, the DC analysis also frequently fails due to numerical non-linearities in the transistor models. <p>

Among circuit analysts, a technique has been developed to circumvent these difficulties, called <i>ramping</i> [1].  One OP-point that can always be computed easily is the one where all sources are set equal to zero.  In that case, all node potentials must evidently be zero as well.  Thus, rather than working with the regular <b>startTime</b> and <b>offset</b> parameters of Modelica sources, it may make sense to start always from zero, ramp all sources up to their <b>offset</b> during the <b>rampTime</b>, then wait with the actual analysis during a period, called <b>settleTime</b>. <p>

Thus in Spice ramping sources, the <b>startTime</b> parameter of regular Modelica sources is replaced by two parameters: <b>rampTime</b> and <b>settleTime</b>.  To prevent numerical problems, the <b>phase</b> parameter of the sinusoidal sources has also been removed.  If a sine wave signal is supposed to be off by a certain fraction of a period from another, you need to adjust the <b>settleTime</b> of one of them accordingly. <p>

For example, a ramping trapezoidal voltage source with the parameters: <p>

<img src=\"../Images/BG_Fig28.png\"> <p>

produces the following voltage characteristic: <p>

<img src=\"../Images/BG_Fig29.png\"> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp.213-215.
</ol> 

</html>
"));
          model V "Constant voltage ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Voltage V=1 "Constant voltage";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping time";
            annotation (
              Icon(
                Line(points=[-90,0; -10,0],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[-10,60; -10,-60],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[0,30; 0,-30],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[0,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
                Text(
                  extent=[-120,50; -20,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+"),
                Text(
                  extent=[20,50; 120,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(extent=[-100,-120; 100,-80],   string="%name=%V")),
              Diagram,
              Documentation(info="<html>
The constant voltage ramping source ramps the voltage up (or down) from zero to its final value during <b>rampTime</b>. <p>

<hr>

<pre>
Parameters:

 V:         Constant source voltage (default value = 1 Volt)

 rampTime:  Initial ramping period (default value = 0 sec)
</pre>
</html>
"));
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,12; 12,-12]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Bonds.fBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Sources.rampSe V1(
              e0=V,
              offset=0,
              startTime=0,
              duration=rampTime) 
              annotation (extent=[-12,36; 12,60], rotation=90);
          equation 
            connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015],
                      style(color=8));
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                  -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                        style(color=8));
            connect(B3.fBondCon1,J1p3_1. BondCon3) annotation (points=[
                  7.34764e-016,12; 0,12],
                       style(color=8));
            connect(B3.eBondCon1,V1. BondCon1) annotation (points=[
                  -7.34764e-016,36; -7.34764e-016,36],
                                         style(color=8, rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0],
                                  style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          end V;
          
          model sinV "Sinusoidal voltage ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Voltage V=1 "Amplitude of sine wave";
            parameter Modelica.SIunits.Frequency freqHz=1 
              "Frequency of sine wave";
            parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            annotation (
              Icon(
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Text(
                  extent=[-120,50; -20,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+"),
                Text(
                  extent=[20,50; 120,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(extent=[-150,80; 150,120], string="%name"),
                   Line(points=[-66,0; -56.2,29.9; -49.8,46.5; -44.2,58.1;
                      -39.3,65.2; -34.3,69.2; -29.4,69.8; -24.5,67; -19.6,61;
                      -14.6,52; -9,38.6; -1.98,18.6; 12.79,-26.9; 19.1,-44;
                      24.8,-56.2; 29.7,-64; 34.6,-68.6; 39.5,-70; 44.5,-67.9;
                      49.4,-62.5; 54.3,-54.1; 59.9,-41.3; 67,-21.7; 74,0],
                                                            style(color=8)),
                Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
              Diagram,
              Documentation(info="<html>
The sinusoidal voltage ramping source for Spice is built from two superposed sources, a sinusoidal source and a ramping source placed in series. <p>

<hr>

<pre>
Parameters:

 V:           Amplitude of source voltage (default value = 1 Volt)

 freqHz:      Frequency of sine wave to be generated (default value = 1 Hz)

 offset:      Offset of sine wave to be generated (default value = 0 Volt)

 rampTime:    Initial ramping period (default value = 0 sec)

 settleTime:  Initial settling period (default value = 0 sec)
</pre>
</html>
"));
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Bonds.fBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Sources.sinSe V1(
              e0=V,
              freqHz=freqHz,
              phase=0,
              offset=0,
              startTime=rampTime + settleTime) 
                                   annotation (extent=[-12,36; 12,60],   rotation=
                 90);
            BondLib.Junctions.J1p4 J1p4_1 annotation (extent=[-12,-12; 12,12]);
            BondLib.Bonds.fBond B4 annotation (extent=[-12,-36; 12,-12], rotation=
                  90);
            BondLib.Sources.rampSe Vramp(
              offset=0,
              startTime=0,
              e0=offset,
              duration=rampTime) 
              annotation (extent=[-12,-60; 12,-36], rotation=-90);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(B3.eBondCon1,V1. BondCon1) annotation (points=[
                  -7.34764e-016,36; -7.34764e-016,36],
                  style(color=8));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(J1p4_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon4, B3.fBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(B4.fBondCon1, J1p4_1.BondCon3) annotation (points=[
                  7.34764e-016,-12; 0,-12], style(color=8, rgbcolor={192,192,
                    192}));
            connect(Vramp.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-36; 1.87707e-022,-36; 0,-38; -7.34764e-016,-36],
                style(color=8, rgbcolor={192,192,192}));
          end sinV;
          
          model expsinV "Damped sinusoidal voltage ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Voltage V=1 "Amplitude of sine wave";
            parameter Modelica.SIunits.Frequency freqHz=2 
              "Frequency of sine wave";
            parameter Modelica.SIunits.Damping damping=1 
              "Damping coefficient of sine wave";
            parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            annotation (
              Icon(
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Text(
                  extent=[-120,50; -20,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+"),
                Text(
                  extent=[20,50; 120,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(extent=[-150,80; 150,120], string="%name"),
                   Line(points=[-64,-14; -59.2,18.3; -56,36.3; -52.7,50.5;
                      -49.5,60.2; -46.3,65.3; -43.1,65.6; -39.9,61.3; -36.7,
                      53.1; -32.6,38.2; -27,11.8; -19,-27.9; -14.2,-47.7; -10.1,
                      -59.9; -6.1,-67.2; -2.1,-69.3; 1.9,-66.5; 5.9,-59.3;
                      10.77,-46.1; 24.44,-0.3; 29.3,12.4; 34.1,20.8; 38.1,24;
                      42.9,23.2; 47.8,17.8; 54.2,5.4; 67.1,-24.5; 73.5,-35.2;
                      79.1,-39.9; 84.7,-39.9; 91.2,-34.5; 96,-27.8],
                                                     style(color=8)),
                Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
              Diagram,
              Documentation(info="<html>
The damped sinusoidal voltage ramping source for Spice is built from two superposed sources, a damped sinusoidal source and a ramping source placed in series. <p>

<hr>

<pre>
Parameters:

 V:           Amplitude of source voltage (default value = 1 Volt)

 freqHz:      Frequency of damped sine wave to be generated (default value = 2 Hz)

 damping:     Damping coefficient of damped sine wave to be generated (default value = 1 Hz)

 offset:      Offset of sine wave to be generated (default value = 0 Volt)

 rampTime:    Initial ramping period (default value = 0 sec)

 settleTime:  Initial settling period (default value = 0 sec)
</pre>
</html>
"));
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Bonds.fBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Junctions.J1p4 J1p4_1 annotation (extent=[-12,-12; 12,12]);
            BondLib.Bonds.fBond B4 annotation (extent=[-12,-36; 12,-12], rotation=
                  90);
            BondLib.Sources.rampSe Vramp(
              offset=0,
              startTime=0,
              e0=offset,
              duration=rampTime) 
              annotation (extent=[-12,-60; 12,-36], rotation=-90);
            BondLib.Sources.expsinSe V1(
              e0=V,
              freqHz=freqHz,
              damping=damping,
              phase=0,
              offset=0,
              startTime=rampTime + settleTime) 
              annotation (extent=[-12,36; 12,60], rotation=90);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p4_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon4, B3.fBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(B4.fBondCon1, J1p4_1.BondCon3) annotation (points=[
                  7.34764e-016,-12; 0,-12], style(color=8, rgbcolor={192,192,
                    192}));
            connect(Vramp.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-36; -7.34764e-016,-36], style(color=8,
                  rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -82,
                  1.77636e-015; -82,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(V1.BondCon1, B3.eBondCon1) annotation (points=[
                  -7.34764e-016,36; -7.34764e-016,36], style(color=8, rgbcolor=
                    {192,192,192}));
          end expsinV;
          
          model expV "Step response voltage ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Voltage V=1 "Amplitude of step response";
            parameter Modelica.SIunits.Time riseTime(min=0) = 0.5 "Rise time";
            parameter Modelica.SIunits.Time riseTimeConst(min=Modelica.Constants.small) = 0.1 
              "Rise time constant";
            parameter Modelica.SIunits.Time fallTimeConst(min=Modelica.Constants.small) = riseTimeConst 
              "Fall time constant";
            parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            annotation (
              Icon(
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Text(
                  extent=[-120,50; -20,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+"),
                Text(
                  extent=[20,50; 120,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(extent=[-150,80; 150,120], string="%name"),
                   Line(points=[-76,-59; -73.2,-44.3; -70.3,-31.1; -66.8,-16.6;
                      -63.3,-4; -59.7,6.92; -55.5,18.18; -51.3,27.7; -46.3,37;
                      -40.6,45.5; -34.3,53.1; -27.2,59.6; -18.7,65.3; -8.1,70.2;
                      -6,71; -3.88,58.5; -1.05,43.7; 1.78,30.8; 4.606,19.45;
                      8.14,7.3; 11.68,-3; 15.9,-13.2; 20.2,-21.6; 25.1,-29.5;
                      30.8,-36.4; 37.1,-42.3; 44.9,-47.5; 54.8,-51.8; 64,-54.4],
                                                                               style(
                      color=8)),
                Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0}))),
              Diagram,
              Documentation(info="<html>
The step response voltage ramping source for Spice is built from two superposed sources, a step response source and a ramping source placed in series. <p>

<hr>

<pre>
Parameters:

 V:              Height of step response source voltage after infinite time (default value = 1 Volt)

 riseTime:       Rise time of step response to be generated (default value = 0.5 sec)

 riseTimeConst:  Rise time constant of step response to be generated (default value = 0.1)

 fallTimeConst:  Fall time constant of step response to be generated (default value = riseTimeConst)

 offset:         Offset of step response to be generated (default value = 0 Volt)

 rampTime:       Initial ramping period (default value = 0 sec)

 settleTime:     Initial settling period (default value = 0 sec)
</pre>
</html>
"));
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Bonds.fBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Junctions.J1p4 J1p4_1 annotation (extent=[-12,-12; 12,12]);
            BondLib.Bonds.fBond B4 annotation (extent=[-12,-36; 12,-12], rotation=
                  90);
            BondLib.Sources.rampSe Vramp(
              offset=0,
              startTime=0,
              e0=offset,
              duration=rampTime) 
              annotation (extent=[-12,-60; 12,-36], rotation=-90);
            BondLib.Sources.expSe V1(
              riseTime=riseTime,
              riseTimeConst=riseTimeConst,
              fallTimeConst=fallTimeConst,
              e0=V,
              offset=0,
              startTime=rampTime + settleTime) 
              annotation (extent=[-12,36; 12,60], rotation=90);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p4_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon4, B3.fBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(B4.fBondCon1, J1p4_1.BondCon3) annotation (points=[
                  7.34764e-016,-12; 0,-12], style(color=8, rgbcolor={192,192,
                    192}));
            connect(Vramp.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-36; -7.34764e-016,-36], style(color=8,
                  rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(V1.BondCon1, B3.eBondCon1) annotation (points=[
                  -7.34764e-016,36; -7.34764e-016,36], style(color=8, rgbcolor=
                    {192,192,192}));
          end expV;
          
          model trapV "Trapezoidal voltage ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Voltage V=1 
              "Amplitude of trapezoidal function";
            parameter Modelica.SIunits.Time rising(final min=0) = 0 
              "Rising duration of trapezoid";
            parameter Modelica.SIunits.Time width(final min=0) = 0.5 
              "Width duration of trapezoid";
            parameter Modelica.SIunits.Time falling(final min=0) = 0 
              "Falling duration of trapezoid";
            parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
              "Time for one period";
            parameter Integer nperiod=-1 
              "Number of periods (< 0 means infinite number of periods)";
            parameter Modelica.SIunits.Voltage offset=0 "Voltage offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Bonds.fBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Junctions.J1p4 J1p4_1 annotation (extent=[-12,-12; 12,12]);
            BondLib.Bonds.fBond B4 annotation (extent=[-12,-36; 12,-12], rotation=
                  90);
            BondLib.Sources.rampSe Vramp(
              offset=0,
              startTime=0,
              e0=offset,
              duration=rampTime) 
              annotation (extent=[-12,-60; 12,-36], rotation=-90);
            BondLib.Sources.trapSe V1(
              e0=V,
              rising=rising,
              width=width,
              falling=falling,
              period=period,
              nperiod=nperiod,
              offset=0,
              startTime=rampTime + settleTime) 
              annotation (extent=[-12,36; 12,60], rotation=90);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p4_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J1p4_1.BondCon4, B3.fBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(B4.fBondCon1, J1p4_1.BondCon3) annotation (points=[
                  7.34764e-016,-12; 0,-12], style(color=8, rgbcolor={192,192,
                    192}));
            connect(Vramp.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-36; -7.34764e-016,-36], style(color=8,
                  rgbcolor={192,192,192}));
            annotation (
              Diagram,
              Icon(
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[-90,0; 90,0], style(color=0, rgbcolor={0,0,0})),
                Text(
                  extent=[-120,50; -20,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+"),
                Text(
                  extent=[20,50; 120,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(extent=[-150,80; 150,120], string="%name"),
                   Line(points=[-81,-70; -60,-70; -30,70; 1,70; 30,-70; 51,-70;
                      80,70],  style(color=8))),
              Documentation(info="<html>
The trapezoidal voltage ramping source for Spice is built from two superposed sources, a trapezoidal source and a ramping source placed in series. <p>

<hr>

<pre>
Parameters:

 V:           Amplitude of source voltage trapezoid (default value = 1 Volt)

 rising:      Length of rising time of trapezoid to be generated (default value = 0 sec)

 width :      Length of on state of trapezoid to be generated (default value = 0.5 sec)

 falling:     Length of falling time of trapezoid to be generated (default value = 0 sec)

 period:      Length of period of trapezoid to be generated (default value = 1 sec)

 nperiod:     Number of periods of trapezoid to be generated
              an infinite number of periods is being generated if nperiod &lt; 0 (default value = -1)

 offset:      Offset of trapezoid to be generated (default value = 0 Volt)

 rampTime:    Initial ramping period (default value = 0 sec)

 settleTime:  Initial settling period (default value = 0 sec)
</html>
"));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(V1.BondCon1, B3.eBondCon1) annotation (points=[
                  -7.34764e-016,36; -7.34764e-016,36], style(color=8, rgbcolor=
                    {192,192,192}));
          end trapV;
          
          model I "Constant current ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Current I=1 "Constant current";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping time";
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,12; 12,-12]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Bonds.eBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Sources.rampSf I1(
              f0=I,
              duration=rampTime,
              offset=0,
              startTime=0) 
              annotation (extent=[-12,36; 12,60], rotation=90);
          equation 
            connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015],
                      style(color=8));
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,0;
                  -12.13,0; -12.13,-1.44014e-015; -12.26,-1.44014e-015],
                        style(color=8));
            connect(J1p3_1.BondCon3,B3. eBondCon1) annotation (points=[0,12;
                  7.34764e-016,12],
                       style(color=8));
            connect(B3.fBondCon1,I1. BondCon1) annotation (points=[
                  -7.34764e-016,36; -7.34764e-016,36],
                                         style(color=8, rgbcolor={192,192,192}));
            annotation (
              Diagram,
              Icon(
                Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
                Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(extent=[-100,80; 100,120],     string="%name=%I")),
              Documentation(info="<html>
The constant current ramping source ramps the current up (or down) from zero to its final value during <b>rampTime</b>. <p>

<hr>

<pre>
Parameters:

 I:         Constant source current (default value = 1 Amp)

 rampTime:  Initial ramping period (default value = 0 sec)
</pre>
</html>
"));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                  1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          end I;
          
          model sinI "Sinusoidal current ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Current I=1 "Amplitude of sine wave";
            parameter Modelica.SIunits.Frequency freqHz=1 
              "Frequency of sine wave";
            parameter Modelica.SIunits.Current offset=0 "Current offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            annotation (
              Icon(
                Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
                Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(extent=[-150,120; 150,80],   string="%name"),
                   Line(points=[-70,0; -60.2,29.9; -53.8,46.5; -48.2,58.1;
                      -43.3,65.2; -38.3,69.2; -33.4,69.8; -28.5,67; -23.6,61;
                      -18.6,52; -13,38.6; -5.98,18.6; 8.79,-26.9; 15.1,-44;
                      20.8,-56.2; 25.7,-64; 30.6,-68.6; 35.5,-70; 40.5,-67.9;
                      45.4,-62.5; 50.3,-54.1; 55.9,-41.3; 63,-21.7; 70,0],
                                                            style(color=8))),
              Diagram,
              Documentation(info="<html>
The sinusoidal current ramping source for Spice is built from two superposed sources, a sinusoidal source and a ramping source placed in parallel. <p>

<hr>

<pre>
Parameters:

 I:           Amplitude of source current (default value = 1 Amp)

 freqHz:      Frequency of sine wave to be generated (default value = 1 Hz)

 offset:      Offset of sine wave to be generated (default value = 0 Amp)

 rampTime:    Initial ramping period (default value = 0 sec)

 settleTime:  Initial settling period (default value = 0 sec)
</pre>
</html>
"));
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Sources.sinSf I1(
              f0=I,
              freqHz=freqHz,
              phase=0,
              offset=0,
              startTime=rampTime + settleTime) 
                                   annotation (extent=[-36,36; -60,60],  rotation=0);
            BondLib.Sources.rampSf Iramp(
              f0=offset,
              duration=rampTime,
              offset=0,
              startTime=0) annotation (extent=[36,36; 60,60], rotation=0);
            Bonds.eBond B3         annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            Bonds.eBond B4         annotation (extent=[-36,36; -12,60],  rotation=0);
            Junctions.J1p3 J1p3_1 annotation (extent=[-12,12; 12,-12]);
            Junctions.J0p3 J0p3_1 annotation (extent=[-12,36; 12,60]);
            Bonds.eBond B5         annotation (extent=[12,36; 36,60],    rotation=
                  180);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                  1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon3, B3.fBondCon1) annotation (points=[0,36;
                  -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
            connect(B4.eBondCon1, J0p3_1.BondCon1) annotation (points=[-12,48;
                  -12,48], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon2, B5.eBondCon1) annotation (points=[12,48;
                  12,48], style(color=8, rgbcolor={192,192,192}));
            connect(I1.BondCon1, B4.fBondCon1) annotation (points=[-36,48; -36,
                  48], style(color=8, rgbcolor={192,192,192}));
            connect(B5.fBondCon1, Iramp.BondCon1) annotation (points=[36,48; 36,
                  48], style(color=8, rgbcolor={192,192,192}));
          end sinI;
          
          model expsinI "Damped sinusoidal current ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Current I=1 "Amplitude of sine wave";
            parameter Modelica.SIunits.Frequency freqHz=2 
              "Frequency of sine wave";
            parameter Modelica.SIunits.Damping damping=1 
              "Damping coefficient of sine wave";
            parameter Modelica.SIunits.Current offset=0 "Current offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            
          protected 
            BondLib.Sources.expsinSf I1(
              f0=I,
              freqHz=freqHz,
              damping=damping,
              offset=offset,
              phase=0,
              startTime=rampTime + settleTime) 
              annotation (extent=[-36,36; -60,60],rotation=0);
            
            annotation (
              Diagram,
              Icon(
                Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
                Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(extent=[-150,120; 150,80],   string="%name"),
                   Line(points=[-80,-14; -75.2,18.3; -72,36.3; -68.7,50.5;
                      -65.5,60.2; -62.3,65.3; -59.1,65.6; -55.9,61.3; -52.7,
                      53.1; -48.6,38.2; -43,11.8; -35,-27.9; -30.2,-47.7; -26.1,
                      -59.9; -22.1,-67.2; -18.1,-69.3; -14.1,-66.5; -10.1,-59.3;
                      -5.23,-46.1; 8.44,-0.3; 13.3,12.4; 18.1,20.8; 22.1,24;
                      26.9,23.2; 31.8,17.8; 38.2,5.4; 51.1,-24.5; 57.5,-35.2;
                      63.1,-39.9; 68.7,-39.9; 75.2,-34.5; 80,-27.8],
                                                     style(color=8))),
              Documentation(info="<html>
The damped sinusoidal current ramping source for Spice is built from two superposed sources, a damped sinusoidal source and a ramping source placed in parallel. <p>

<hr>

<pre>
Parameters:

 I:           Amplitude of source current (default value = 1 Amp)

 freqHz:      Frequency of damped sine wave to be generated (default value = 2 Hz)

 damping:     Damping coefficient of damped sine wave to be generated (default value = 1 Hz)

 offset:      Offset of sine wave to be generated (default value = 0 Amp)

 rampTime:    Initial ramping period (default value = 0 sec)

 settleTime:  Initial settling period (default value = 0 sec)
</pre>
</html>
"));
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Sources.rampSf Iramp(
              f0=offset,
              duration=rampTime,
              offset=0,
              startTime=0) annotation (extent=[36,36; 60,60], rotation=0);
            BondLib.Bonds.eBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Bonds.eBond B4 annotation (extent=[-36,36; -12,60],  rotation=0);
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,12; 12,-12]);
            BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[-12,36; 12,60]);
            BondLib.Bonds.eBond B5 annotation (extent=[12,36; 36,60],    rotation=
                  180);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon3, B3.fBondCon1) annotation (points=[0,36;
                  -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
            connect(B4.eBondCon1, J0p3_1.BondCon1) annotation (points=[-12,48;
                  -12,48], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon2, B5.eBondCon1) annotation (points=[12,48;
                  12,48], style(color=8, rgbcolor={192,192,192}));
            connect(B5.fBondCon1, Iramp.BondCon1) annotation (points=[36,48; 36,
                  48], style(color=8, rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 81,
                  1.77636e-015; 81,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(I1.BondCon1, B4.fBondCon1) annotation (points=[-36,48; -36,
                  48], style(color=8, rgbcolor={192,192,192}));
          end expsinI;
          
          model expI "Step response current ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Current I=1 "Amplitude of step response";
            parameter Modelica.SIunits.Time riseTime(min=0) = 0.5 "Rise time";
            parameter Modelica.SIunits.Time riseTimeConst(min=Modelica.Constants.small) = 0.1 
              "Rise time constant";
            parameter Modelica.SIunits.Time fallTimeConst(min=Modelica.Constants.small) = riseTimeConst 
              "Fall time constant";
            parameter Modelica.SIunits.Current offset=0 "Current offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            annotation (
              Icon(
                Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
                Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(extent=[-150,120; 150,80],   string="%name"),
                   Line(points=[-76,-59; -73.2,-44.3; -70.3,-31.1; -66.8,-16.6;
                      -63.3,-4; -59.7,6.92; -55.5,18.18; -51.3,27.7; -46.3,37;
                      -40.6,45.5; -34.3,53.1; -27.2,59.6; -18.7,65.3; -8.1,70.2;
                      -6,71; -3.88,58.5; -1.05,43.7; 1.78,30.8; 4.606,19.45;
                      8.14,7.3; 11.68,-3; 15.9,-13.2; 20.2,-21.6; 25.1,-29.5;
                      30.8,-36.4; 37.1,-42.3; 44.9,-47.5; 54.8,-51.8; 64,-54.4],
                                                                               style(
                      color=8))),
              Diagram,
              Documentation(info="<html>
The step response current ramping source for Spice is built from two superposed sources, a step response source and a ramping source placed in parallel. <p>

<hr>

<pre>
Parameters:

 I:              Height of step response source current after infinite time (default value = 1 Amp)

 riseTime:       Rise time of step response to be generated (default value = 0.5 sec)

 riseTimeConst:  Rise time constant of step response to be generated (default value = 0.1)

 fallTimeConst:  Fall time constant of step response to be generated (default value = riseTimeConst)

 offset:         Offset of step response to be generated (default value = 0 Amp)

 rampTime:       Initial ramping period (default value = 0 sec)

 settleTime:     Initial settling period (default value = 0 sec)
</pre>
</html>
"));
            
          protected 
            BondLib.Sources.expSf I1(
              riseTime=riseTime,
              riseTimeConst=riseTimeConst,
              fallTimeConst=fallTimeConst,
              f0=I,
              offset=0,
              startTime=rampTime + settleTime) 
              annotation (extent=[-36,36; -60,60],rotation=0);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Sources.rampSf Iramp(
              f0=offset,
              duration=rampTime,
              offset=0,
              startTime=0) annotation (extent=[36,36; 60,60], rotation=0);
            BondLib.Bonds.eBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Bonds.eBond B4 annotation (extent=[-36,36; -12,60],  rotation=0);
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,12; 12,-12]);
            BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[-12,36; 12,60]);
            BondLib.Bonds.eBond B5 annotation (extent=[12,36; 36,60],    rotation=
                  180);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon3, B3.fBondCon1) annotation (points=[0,36;
                  -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
            connect(B4.eBondCon1, J0p3_1.BondCon1) annotation (points=[-12,48;
                  -12,48], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon2, B5.eBondCon1) annotation (points=[12,48;
                  12,48], style(color=8, rgbcolor={192,192,192}));
            connect(B5.fBondCon1, Iramp.BondCon1) annotation (points=[36,48; 36,
                  48], style(color=8, rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -82,
                  1.77636e-015; -82,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(I1.BondCon1, B4.fBondCon1) annotation (points=[-36,48; -36,
                  48], style(color=8, rgbcolor={192,192,192}));
          end expI;
          
          model trapI "Trapezoidal current ramping source for Spice" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Current I=1 
              "Amplitude of trapezoidal function";
            parameter Modelica.SIunits.Time rising(final min=0) = 0 
              "Rising duration of trapezoid";
            parameter Modelica.SIunits.Time width(final min=0) = 0.5 
              "Width duration of trapezoid";
            parameter Modelica.SIunits.Time falling(final min=0) = 0 
              "Falling duration of trapezoid";
            parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small) = 1 
              "Time for one period";
            parameter Integer nperiod=-1 
              "Number of periods (< 0 means infinite number of periods)";
            parameter Modelica.SIunits.Current offset=0 "Current offset";
            parameter Modelica.SIunits.Time rampTime=0 "Ramping period";
            parameter Modelica.SIunits.Time settleTime=0 "Settling period";
            annotation (
              Icon(
                Line(points=[-90,0; -50,0],   style(color=0, rgbcolor={0,0,0})),
                Ellipse(extent=[-50,50; 50,-50],   style(
                    color=0,
                    rgbcolor={0,0,0},
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[50,0; 90,0],   style(color=0, rgbcolor={0,0,0})),
                Line(points=[-60,-80; 60,-80], style(color=3, rgbcolor={0,0,255})),
                Polygon(points=[60,-80; 30,-70; 30,-90; 60,-80], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(extent=[-150,120; 150,80],   string="%name"),
                   Line(points=[-81,-66; -60,-66; -30,74; 1,74; 30,-66; 51,-66;
                      80,74],  style(color=8))),
              Diagram,
              Documentation(info="<html>
The trapezoidal current ramping source for Spice is built from two superposed sources, a trapezoidal source and a ramping source placed in parallel. <p>

<hr>

<pre>
Parameters:

 I:           Amplitude of source current trapezoid (default value = 1 Amp)

 rising:      Length of rising time of trapezoid to be generated (default value = 0 sec)

 width :      Length of on state of trapezoid to be generated (default value = 0.5 sec)

 falling:     Length of falling time of trapezoid to be generated (default value = 0 sec)

 period:      Length of period of trapezoid to be generated (default value = 1 sec)

 nperiod:     Number of periods of trapezoid to be generated
              an infinite number of periods is being generated if nperiod &lt; 0 (default value = -1)

 offset:      Offset of trapezoid to be generated (default value = 0 Amp)

 rampTime:    Initial ramping period (default value = 0 sec)

 settleTime:  Initial settling period (default value = 0 sec)
</html>
"));
            
          protected 
            BondLib.Sources.trapSf I1(
              f0=I,
              rising=rising,
              width=width,
              falling=falling,
              period=period,
              nperiod=nperiod,
              offset=0,
              startTime=rampTime + settleTime) 
              annotation (extent=[-36,36; -60,60],rotation=0);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-12; 38,12],   rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-12; -12,12],
                rotation=180);
            BondLib.Sources.rampSf Iramp(
              f0=offset,
              duration=rampTime,
              offset=0,
              startTime=0) annotation (extent=[36,36; 60,60], rotation=0);
            BondLib.Bonds.eBond B3 annotation (extent=[-12,12; 12,36],   rotation=
                 -90);
            BondLib.Bonds.eBond B4 annotation (extent=[-36,36; -12,60],  rotation=0);
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,12; 12,-12]);
            BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[-12,36; 12,60]);
            BondLib.Bonds.eBond B5 annotation (extent=[12,36; 36,60],    rotation=
                  180);
          equation 
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,
                  1.77636e-015; 38,-1.44014e-015; 37.74,-1.44014e-015],
                      style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,
                  1.77636e-015; -38,1.46953e-015],
                        style(color=8));
            connect(J1p3_1.BondCon1, B2.BondCon1) annotation (points=[-12,0;
                  -12,-1.44014e-015; -12.26,-1.44014e-015], style(color=8,
                  rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,12;
                  7.34764e-016,12], style(color=8, rgbcolor={192,192,192}));
            connect(J1p3_1.BondCon2, B1.BondCon2) annotation (points=[12,0; 12,
                  1.46953e-015], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon3, B3.fBondCon1) annotation (points=[0,36;
                  -7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
            connect(B4.eBondCon1, J0p3_1.BondCon1) annotation (points=[-12,48;
                  -12,48], style(color=8, rgbcolor={192,192,192}));
            connect(J0p3_1.BondCon2, B5.eBondCon1) annotation (points=[12,48;
                  12,48], style(color=8, rgbcolor={192,192,192}));
            connect(B5.fBondCon1, Iramp.BondCon1) annotation (points=[36,48; 36,
                  48], style(color=8, rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 83,
                  1.77636e-015; 83,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(I1.BondCon1, B4.fBondCon1) annotation (points=[-36,48; -36,
                  48], style(color=8, rgbcolor={192,192,192}));
          end trapI;
        end RampingSources;
        
        model R "Spice-style resistor of the analog electrical library" 
          extends Interfaces.SpiceTwoPort;
          parameter Modelica.SIunits.Resistance R=1 
            "Resistance at reference temperature";
          parameter Real TR1(unit="Ohm/K") = 0 "Linear temperature coefficient";
          parameter Real TR2(unit="Ohm/K2") = 0 
            "Quadratic temperature coefficient";
          parameter Real Area=1 "Relative area occupied by resistor";
          annotation (Icon(
              Rectangle(extent=[-70,30; 70,-30],   style(
                  color=3,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[-90,0; -70,0]),
              Line(points=[70,0; 90,0]),
              Line(points=[0,-30; 0,-91],   style(color=42)),
              Text(
                extent=[-100,-32; 100,-94],
                string="R=%R",
                style(color=0))),                              Diagram(
        Rectangle(extent=[-60,0; -40,-20],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-60,0; -40,-20],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-40,-10; -30,-10], style(color=83, rgbcolor={255,85,
                      255})),
              Line(points=[-30,-10; -30,-30], style(color=83, rgbcolor={255,85,
                      255})),
              Line(points=[-30,-30; -14,-30], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
The Spice-style resistor has a reference resistance value computed at the reference temperature, <b>Tnom</b>, and for a relative area of <b>Area=1</b>.  It changes its value as a function of the circuit temperature and the relative area occupied by the resistor. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
Parameters:

 R:     Reference resistance (default value = 1 Ohm)

 Tnom:  Reference temperature (default value = 300.15 <sup>o</sup>K)

 TR1:   Linear temperature variation coefficient (default value = 0 Ohm/<sup>o</sup>K)

 TR2:   Quadratic temperature variation coefficient (default value = 0 Ohm/<sup>o</sup>K<sup>2</sup>)

 Area:  Relative area occupied by resistor (default value = 1)
</pre> <p>
</html>
"));
          
        protected 
          Junctions.J1p3 J1p3_1         annotation (extent=[-10,50; 10,70]);
          Bonds.Bond B1         annotation (extent=[-30,50; -10,70]);
          Bonds.Bond B2         annotation (extent=[10,50; 30,70]);
          Bonds.Bond B3         annotation (extent=[-10,30; 10,50],   rotation=
                270);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[-60,50;
                -40,70]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[40,50;
                60,70]);
          BondLib.Bonds.eBond B4 
                         annotation (extent=[-10,-20; 10,0],   rotation=-90);
          BondLib.Spice.RS R1(
            Tnom=Tnom,
            R=R,
            TR1=TR1,
            TR2=TR2,
            Area=Area) annotation (extent=[-10,0; 10,30],  rotation=-90);
          Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Switches.Sw3 Sw1 annotation (extent=[10,-40; -10,-20], rotation=-90);
          BondLib.Bonds.eBond B5 
                         annotation (extent=[-10,-60; 10,-40], rotation=-90);
          Bonds.fBond B6 annotation (extent=[-40,-60; -20,-40]);
          BondLib.Sources.Se T_nom(e0=Tnom) 
            annotation (extent=[-40,-60; -60,-40]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-10,60; -10,
                60],  style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[10,60; 10.2,
                60],       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-40,60;
                -29.8,60],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[30,60; 40,
                60],style(color=8));
          connect(J1p3_1.BondCon3, B3.BondCon1) annotation (points=[0,50; 0,
                49.8; 1.80017e-015,49.8],   style(color=8, rgbcolor={192,192,
                  192}));
          connect(B3.BondCon2, R1.BondCon1)  annotation (points=[-1.83691e-015,
                30; -6.12303e-016,30],
                                     style(color=8, rgbcolor={192,192,192}));
          connect(R1.BondCon2, B4.fBondCon1)  annotation (points=[6.12303e-016,
                0; -6.12303e-016,0],     style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p, p) annotation (points=[-60,60; -80,60; -80,0; -100,
                0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[60,60; 80,60; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Sw1.p, B4.eBondCon1) annotation (points=[6.12303e-016,-20;
                6.12303e-016,-20], style(color=8, rgbcolor={192,192,192}));
          connect(B5.fBondCon1, Sw1.n2) annotation (points=[-6.12303e-016,-40;
                -6.12303e-016,-40], style(color=8, rgbcolor={192,192,192}));
          connect(B5.eBondCon1, BG2Heat1.BondCon1) annotation (points=[
                6.12303e-016,-60; -6.12303e-016,-60], style(color=8, rgbcolor={
                  192,192,192}));
          connect(Sw1.n1, B6.fBondCon1) annotation (points=[-5,-40; -10,-40;
                -10,-50; -20,-50], style(color=8, rgbcolor={192,192,192}));
          connect(T_nom.BondCon1, B6.eBondCon1) annotation (points=[-40,-50;
                -40,-50], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, heatPort) annotation (points=[6.12303e-016,
                -80; 0,-80; 0,-100], style(color=42, rgbcolor={191,0,0}));
        end R;
        
        model C "Spice-style capacitor of the analog electrical library" 
          extends Interfaces.SpiceTwoPortV;
          parameter Modelica.SIunits.Capacitance C=1 
            "Capacitance at reference temperature";
          parameter Real TC1(unit="F/K") = 0 "Linear temperature coefficient";
          parameter Real TC2(unit="F/K2") = 0 
            "Quadratic temperature coefficient";
          parameter Real Area=1 "Relative area occupied by capacitor";
          annotation (
            Icon(
              Line(points=[14,0; 90,0]),
              Line(points=[14,28; 14,-28],   style(thickness=2)),
              Line(points=[-14,28; -14,-28],   style(thickness=2)),
              Line(points=[-90,0; -14,0]),
              Line(points=[0,-28; 0,-91],   style(color=42)),
              Text(
                extent=[-100,-34; 100,-94],
                string="C=%C",
                style(color=0))),
            Diagram(
        Rectangle(extent=[-60,0; -40,-20],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-60,0; -40,-20],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-40,-10; -30,-10], style(color=83, rgbcolor={255,85,
                      255})),
              Line(points=[-30,-10; -30,-30], style(color=83, rgbcolor={255,85,
                      255})),
              Line(points=[-30,-30; -14,-30], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
The Spice-style capacitor has a reference capacitance value computed at the reference temperature, <b>Tnom</b>, and for a relative area of <b>Area=1</b>.  It changes its value as a function of the circuit temperature and the relative area occupied by the capacitor. <p>

Although the capacitor does not produce any heat (heat flow = 0), it requires a heat port, because it imports the current device temperature through that port. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
Parameters:

 C:     Reference capacitance (default value = 1 F)

 Tnom:  Reference temperature (default value = 300.15 <sup>o</sup>K)

 TC1:   Linear temperature variation coefficient (default value = 0 F/<sup>o</sup>K)

 TC2:   Quadratic temperature variation coefficient (default value = 0 F/<sup>o</sup>K<sup>2</sup>)

 Area:  Relative area occupied by capacitor (default value = 1)


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true)
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltage as state variable
                   enforceStates = false:  Use (internal) bond graph effort as state variable</b></font>
</pre> <p>
</html>
"));
          
        protected 
          BondLib.Spice.CS C1(
            Tnom=Tnom,
            C=C,
            TC1=TC1,
            TC2=TC2,
            Area=Area,
            e1(start=u0)) 
                       annotation (extent=[-10,0; 10,30],  rotation=-90);
          Junctions.J1p3 J1p3_1         annotation (extent=[-10,50; 10,70]);
          Bonds.Bond B1         annotation (extent=[-30,50; -10,70]);
          Bonds.Bond B2         annotation (extent=[10,50; 30,70]);
          Bonds.Bond B3         annotation (extent=[-10,30; 10,50],   rotation=
                270);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[-60,50;
                -40,70]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[40,50;
                60,70]);
          BondLib.Bonds.eBond B4 
                         annotation (extent=[-10,-20; 10,0],   rotation=-90);
          BondLib.Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          BondLib.Switches.Sw3 Sw1 
            annotation (extent=[10,-40; -10,-20], rotation=-90);
          BondLib.Bonds.eBond B5 
                         annotation (extent=[-10,-60; 10,-40], rotation=-90);
          BondLib.Bonds.fBond B6 annotation (extent=[-40,-60; -20,-40]);
          BondLib.Sources.Se T_nom(e0=Tnom) 
            annotation (extent=[-40,-60; -60,-40]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-10,60; -10,
                60],  style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[10,60; 10.2,
                60],       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-40,60;
                -29.8,60],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[30,60; 40,
                60],style(color=8));
          connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,50; 0,
                49.8; 1.80017e-015,49.8],   style(color=8, rgbcolor={192,192,
                  192}));
          connect(Sw1.p,B4. eBondCon1) annotation (points=[6.12303e-016,-20;
                6.12303e-016,-20], style(color=8, rgbcolor={192,192,192}));
          connect(B5.fBondCon1,Sw1. n2) annotation (points=[-6.12303e-016,-40;
                -6.12303e-016,-40], style(color=8, rgbcolor={192,192,192}));
          connect(B5.eBondCon1,BG2Heat1. BondCon1) annotation (points=[
                6.12303e-016,-60; -6.12303e-016,-60], style(color=8, rgbcolor={
                  192,192,192}));
          connect(Sw1.n1,B6. fBondCon1) annotation (points=[-5,-40; -10,-40;
                -10,-50; -20,-50], style(color=8, rgbcolor={192,192,192}));
          connect(T_nom.BondCon1,B6. eBondCon1) annotation (points=[-40,-50;
                -40,-50], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, heatPort) annotation (points=[6.12303e-016,
                -80; 6.12303e-016,-86; 0,-86; 0,-100], style(color=42, rgbcolor=
                 {191,0,0}));
          connect(El2BG1.p, p) annotation (points=[-60,60; -80,60; -80,0; -100,
                0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[60,60; 80,60; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(C1.BondCon1, B3.BondCon2) annotation (points=[-6.12303e-016,
                30; -1.83691e-015,30], style(color=8, rgbcolor={192,192,192}));
          connect(C1.BondCon2, B4.fBondCon1) annotation (points=[6.12303e-016,0;
                -6.12303e-016,0], style(color=8, rgbcolor={192,192,192}));
        end C;
        
        model L "Spice-style inductor of the analog electrical library" 
          extends Interfaces.SpiceTwoPortI;
          parameter Modelica.SIunits.Inductance L=1 
            "Inductance at reference temperature";
          parameter Real TL1(unit="H/K") = 0 "Linear temperature coefficient";
          parameter Real TL2(unit="H/K2") = 0 
            "Quadratic temperature coefficient";
          annotation (
            Icon(
              Line(points=[-90,0; -60,0]),
              Ellipse(extent=[-60,-15; -30,15]),
              Ellipse(extent=[-30,-15; 0,15]),
              Ellipse(extent=[0,-15; 30,15]),
              Ellipse(extent=[30,-15; 60,15]),
              Line(points=[60,0; 90,0]),
              Rectangle(extent=[-60,-28; 60,0],   style(color=7, fillColor=7)),
              Line(points=[0,0; 0,-91],     style(color=42)),
              Text(
                extent=[-100,-30; 100,-90],
                string="L=%L",
                style(color=0))),
            Diagram(
        Rectangle(extent=[-60,0; -40,-20],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-60,0; -40,-20],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-40,-10; -30,-10], style(color=83, rgbcolor={255,85,
                      255})),
              Line(points=[-30,-10; -30,-30], style(color=83, rgbcolor={255,85,
                      255})),
              Line(points=[-30,-30; -14,-30], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
The Spice-style inductor has a reference inductance value computed at the reference temperature, <b>Tnom</b>.  It changes its value as a function of the circuit temperature. <p>

Although the inductor does not produce any heat (heat flow = 0), it requires a heat port, because it imports the current device temperature through that port. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
Parameters:

 L:     Reference inductance (default value = 1 H)

 Tnom:  Reference temperature (default value = 300.15 <sup>o</sup>K)

 TL1:   Linear temperature variation coefficient (default value = 0 H/<sup>o</sup>K)

 TL2:   Quadratic temperature variation coefficient (default value = 0 H/<sup>o</sup>K<sup>2</sup>)


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true)
                   <font color=blue><b>enforceStates = true:   Use (external) inductive current as state variable
                   enforceStates = false:  Use (internal) bond graph flow as state variable</b></font>
</pre> <p>

</html>
"));
          
        protected 
          BondLib.Spice.LS L1(
            Tnom=Tnom,
            L=L,
            TL1=TL1,
            TL2=TL2,
            f1(start=i0)) 
                     annotation (extent=[-10,0; 10,30],  rotation=-90);
          Junctions.J1p3 J1p3_1         annotation (extent=[-10,50; 10,70]);
          Bonds.Bond B1         annotation (extent=[-30,50; -10,70]);
          Bonds.Bond B2         annotation (extent=[10,50; 30,70]);
          Bonds.Bond B3         annotation (extent=[-10,30; 10,50],   rotation=
                270);
          BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[-60,50;
                -40,70]);
          BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[40,50;
                60,70]);
          BondLib.Bonds.eBond B4 
                         annotation (extent=[-10,-20; 10,0],   rotation=-90);
          BondLib.Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          BondLib.Switches.Sw3 Sw1 
            annotation (extent=[10,-40; -10,-20], rotation=-90);
          BondLib.Bonds.eBond B5 
                         annotation (extent=[-10,-60; 10,-40], rotation=-90);
          BondLib.Bonds.fBond B6 annotation (extent=[-40,-60; -20,-40]);
          BondLib.Sources.Se T_nom(e0=Tnom) 
            annotation (extent=[-40,-60; -60,-40]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-10,60; -10,
                60],  style(color=8));
          connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[10,60; 10.2,
                60],       style(color=8));
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-40,60;
                -29.8,60],
                      style(color=8));
          connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[30,60; 40,
                60],style(color=8));
          connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,50; 0,
                49.8; 1.80017e-015,49.8],   style(color=8, rgbcolor={192,192,
                  192}));
          connect(Sw1.p,B4. eBondCon1) annotation (points=[6.12303e-016,-20;
                6.12303e-016,-20], style(color=8, rgbcolor={192,192,192}));
          connect(B5.fBondCon1,Sw1. n2) annotation (points=[-6.12303e-016,-40;
                -6.12303e-016,-40], style(color=8, rgbcolor={192,192,192}));
          connect(B5.eBondCon1,BG2Heat1. BondCon1) annotation (points=[
                6.12303e-016,-60; -6.12303e-016,-60], style(color=8, rgbcolor={
                  192,192,192}));
          connect(Sw1.n1,B6. fBondCon1) annotation (points=[-5,-40; -10,-40;
                -10,-50; -20,-50], style(color=8, rgbcolor={192,192,192}));
          connect(T_nom.BondCon1,B6. eBondCon1) annotation (points=[-40,-50;
                -40,-50], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Heat1.port_b, heatPort) annotation (points=[6.12303e-016,
                -80; 6.12303e-016,-85; 0,-85; 0,-100], style(color=42, rgbcolor=
                 {191,0,0}));
          connect(El2BG1.p, p) annotation (points=[-60,60; -80,60; -80,0; -100,
                0], style(color=3, rgbcolor={0,0,255}));
          connect(BG2El1.n, n) annotation (points=[60,60; 80,60; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(L1.BondCon1, B3.BondCon2) annotation (points=[-6.12303e-016,
                30; -1.83691e-015,30], style(color=8, rgbcolor={192,192,192}));
          connect(L1.BondCon2, B4.fBondCon1) annotation (points=[6.12303e-016,0;
                -6.12303e-016,0], style(color=8, rgbcolor={192,192,192}));
        end L;
        
        model D "Spice-style diode of the analog electrical library" 
          extends Interfaces.SpiceTwoPort;
          constant Real inf=Modelica.Constants.inf;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
            "Boltzmann's constant";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
            "Electron charge";
          
          /* General parameters */
          parameter Integer Level(min=1,max=3) = 1 
            "DIODE modeling level (check documentation window for details)";
          parameter Real Area(min=Modelica.Constants.small) = 1 
            "Relative device area";
          
          /* Process parameters */
          parameter Modelica.SIunits.Length TOX(min=Modelica.Constants.small) = 1e-7 
            "|Process parameters|Thin oxide thickness";
          
          /* Basic Electrical parameters */
          parameter Modelica.SIunits.Voltage PB(min=0) = 0.8 
            "|Electrical parameters|Built-in junction potential at reference temperature";
          parameter Modelica.SIunits.Voltage BV(min=0) = 0 
            "|Electrical parameters|Reverse breakdown voltage at reference temperature";
          parameter Modelica.SIunits.Current IBV(min=0) = 0 
            "|Electrical parameters|Reverse breakdown current";
          parameter Modelica.SIunits.Voltage PHP(min=0) = 0.8 
            "|Electrical parameters|Built-in perimeter potential at reference temperature";
          parameter Real N=1 
            "|Electrical parameters|Current emission coefficient";
          parameter Modelica.SIunits.Current IS(min=Modelica.Constants.small) = 1e-14 
            "|Electrical parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISW(min=0) = 0 
            "|Electrical parameters|Saturation current of perimeter at reference temperature";
          parameter Real PJ(min=0) = 1 
            "|Electrical parameters|Relative perimeter width";
          
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF(min=Modelica.Constants.small) = inf 
            "|High current beta degradation effect|Forward knee current";
          parameter Modelica.SIunits.Current IKR(min=Modelica.Constants.small) = inf 
            "|High current beta degradation effect|Reverse knee current";
          
          /* Parasitic resistor parameters */
          parameter Modelica.SIunits.Resistance RS(min=0) = 0 
            "|Parasitic resistor parameters|Resistance at reference temperature";
          
          /* Junction capacitance parameters */
          parameter Modelica.SIunits.Capacitance CJ(min=0) = 0.5e-12 
            "|Junction capacitance parameters|Zero-bias bulk capacitance at reference temperature";
          parameter Real MJ(min=0.1,max=0.9) = 0.33 
            "|Junction capacitance parameters|Bulk junction grading coefficient";
          parameter Modelica.SIunits.Capacitance CJSW(min=0) = 0.5e-12 
            "|Junction capacitance parameters|Zero-bias perimeter capacitance at reference temperature";
          parameter Real MJSW(min=0.1,max=0.9) = 0.33 
            "|Junction capacitance parameters|Perimeter capacitance grading coefficient";
          parameter Real FC(min=0,max=0.95) = 0.5 
            "|Junction capacitance parameters|Forward-bias depletion capacitance coefficient";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TT(min=0) = 0 
            "|Transit time parameters|Transit time at reference temperature";
          
          /* Temperature compensation parameters */
          parameter Real TRS(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of resistance";
          parameter Real TCV(unit="V/K")=0 
            "|Temperature compensation parameters|Linear temperature coefficient of breakdown voltage";
          parameter Real TTT1(unit="s/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of transit time";
          parameter Real TTT2(unit="s/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of transit time";
          parameter Real XTI(min=0) = 3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage EG(min=0.1) = 1.16 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
          
          /* Level=2 General parameters */
          parameter Integer M(min=1) = 1 "|Level=2||Diode multiplier";
          
          /* Level=2 Geometric parameters */
          parameter Modelica.SIunits.Length L(min=0) = 0 
            "|Level=2|Geometric parameters|Diode length";
          parameter Modelica.SIunits.Length W(min=0) = 0 
            "|Level=2|Geometric parameters|Diode width";
          
          /* Level=2 Electrical parameters */
          parameter Real JF(min=0,unit="A/V2") = 0 
            "|Level=2|Electrical parameters|Forward Fowler-Nordheim current coefficient";
          parameter Real JR(min=0,unit="A/V2") = 0 
            "|Level=2|Electrical parameters|Reverse Fowler-Nordheim current coefficient";
          parameter Modelica.SIunits.ElectricFieldStrength EF(min=0) = 0 
            "|Level=2|Electrical parameters|Forward Fowler-Nordheim critical electrical field";
          parameter Modelica.SIunits.ElectricFieldStrength ER(min=0) = 0 
            "|Level=2|Electrical parameters|Reverse Fowler-Nordheim critical electrical field";
          
          /* Level=3 Geometric parameters */
          parameter Modelica.SIunits.Length LP(min=0) = 0 
            "|Level=3|Geometric parameters|Polysilicon length";
          parameter Modelica.SIunits.Length WP(min=0) = 0 
            "|Level=3|Geometric parameters|Polysilicon width";
          parameter Modelica.SIunits.Length XP(min=0) = 0 
            "|Level=3|Geometric parameters|Widening of polysilicon due to masking and etching";
          parameter Modelica.SIunits.Length XOI(min=0) = 0 
            "|Level=3|Geometric parameters|Polysilicon thickness";
          parameter Modelica.SIunits.Length LM(min=0) = 0 
            "|Level=3|Geometric parameters|Metal length";
          parameter Modelica.SIunits.Length WM(min=0) = 0 
            "|Level=3|Geometric parameters|Metal width";
          parameter Modelica.SIunits.Length XM(min=0) = 0 
            "|Level=3|Geometric parameters|Widening of metal due to masking and etching";
          parameter Modelica.SIunits.Length XOM(min=0) = 0 
            "|Level=3|Geometric parameters|Metal thickness";
          
          /* Level=3 Electrical parameters */
          parameter Modelica.SIunits.CurrentDensity JS(min=Modelica.Constants.small) = 1e-4 
            "|Level=3|Electrical parameters|Saturation current density at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JSW(min=0) = 0 
            "|Level=3|Electrical parameters|Saturation current density of perimeter at reference temperature";
          
          /* Level=3 Junction capacitance parameters */
          parameter Real CJ2(min=Modelica.Constants.small,unit="F/m2") = 1e-4 
            "|Level=3|Junction capacitance parameters|Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real CJSW2(min=0,unit="F/m") = 1e-9 
            "|Level=3|Junction capacitance parameters|Zero-bias perimeter capacitance per meter at reference temperature";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=Modelica.Constants.small) = 1e-16 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
        protected 
          parameter Modelica.SIunits.Area AREAeff = W*L*M;
          parameter Modelica.SIunits.Length PJeff = 2*(W + L)*M;
          parameter Modelica.SIunits.Voltage Vt = k*Tnom/q;
          parameter Modelica.SIunits.Current ISeff = if Level==1 then Area*IS + PJ*ISW else 
                                                     if Level==3 then AREAeff*JS + PJeff*JSW else 0;
          parameter Modelica.SIunits.Current Ibreak = -ISeff*(exp(-BV/(N*Vt)) - 1);
          parameter Modelica.SIunits.Current IBVval = if IBV < Ibreak then Ibreak else IBV;
          parameter Modelica.SIunits.Voltage BVval = if BV > 0 then BVval - (N*Vt)*ln(IBVval/Ibreak) else 0;
          annotation (
            Diagram(
        Rectangle(extent=[-100,-30; -80,-50],
                                            style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-100,-30; -80,-50],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-80,-40; -74,-40], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                  color=3,
                  gradient=0,
                  fillColor=7)),
              Line(points=[-90,0; 40,0]),
              Line(points=[40,0; 90,0]),
              Line(points=[30,40; 30,-40],   style(color=3)),
              Line(points=[0,-20; 0,-91],   style(color=42))),
            Documentation(info="<html>
The Spice-style diode offers three modeling levels [1].  The <b>Level=1</b> model characterizes a junction diode by its electrical properties.  This model is similar to the models of the junction diodes used within the BJT and MOSFET models, but it is more accurate.  In particular, it includes a description of avalanche breakdown behavior.  The <b>Level=3</b> model is similar to the <b>Level=1</b> model, but here, the diode is characterized by its physical parameters, such as length and width of the junction.  Also, this model is yet more accurate as it allows the modeler to specify a polysilicon and a metal layer, in which case the additional capacitances associated with these layers will be computed as well.  The <b>Level=2</b> model is an entirely different device.  It represents essentially a non-linear resistor with a quadratic characteristic on both sides.  Thus, the Fowler-Nordheim diode is a symmetric device. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   DIODE modeling level (default value = 1)
            Level=1: Non-geometric junction diode
            Level=2: Fowler-Nordheim diode
            Level=3: Geometric junction diode</b></font>

 Area:    Relative area occupied by the diode (default value = 1) <font color=red>Level 1</font>


<b>Process Parameters:</b>

 TOX:     Thin oxide thickness (default value = 1e-7 m) <font color=red>Levels 1-3</font>


<b>Basic Electrical Parameters:</b>

 PB:      Built-in junction potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1,3</font>

 BV:      Reverse breakdown voltage at reference temperature (default value = 0 Volt) <font color=red>Levels 1,3</font>
            <font color=blue><b>BV must be specified as a positive number.  A value of 0 indicates that the breakdown voltage is infinite.</b></font>

 IBV:     Reverse breakdown current (default value = 0 Amp) <font color=red>Levels 1,3</font>
            <font color=blue><b>IBV must be specified as a positive number.  A value of 0 indicates that the breakdown voltage is infinite.
            Either BV or IBV (the diodic current at breakdown) can be specified.  If both are specified and
            the values are inconsistent with each other, they will be suitably adjusted to match.</b></font>

 PHP:     Built-in perimeter potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1,3</font>

 N:       Current emission coefficient (default value = 1) <font color=red>Levels 1,3</font>

 IS:      Saturation current at reference temperature (default value = 1e-14 Amp) <font color=red>Level 1</font>

 ISW:     Saturation current of perimeter at reference temperature (default value = 0 Amp) <font color=red>Level 1</font>

 PJ:      Relative perimeter width (default value = 1) <font color=red>Level 1</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Forward knee current (default value = &infin; Amp) <font color=red>Levels 1,3</font>

 IKR:     Reverse knee current (default value = &infin; Amp) <font color=red>Levels 1,3</font>


<b>Parasitic Resistor Parameters:</b>

 RS:      Resistance at reference temperature (default value = 0 Ohm) <font color=red>Levels 1-3</font>


<b>Junction Capacitance Parameters:</b>

 CJ:      Zero-bias bulk capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Level 1</font>

 MJ:      Bulk junction grading coefficient (default value = 0.33) <font color=red>Levels 1,3</font>

 CJSW:    Zero-bias perimeter capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Level 1</font>

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33) <font color=red>Levels 1,3</font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1,3</font>


<b>Transit Time Parameters:</b>

 TT:      Transit time at reference temperature (default value = 0 sec) <font color=red>Levels 1,3</font>


<b>Temperature Compensation Parameters:</b>

 TRS:     Linear temperature coefficient of resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TCV:     Linear temperature coefficient of breakdown voltage (default value = 0 V/K) <font color=red>Levels 1,3</font>

 TTT1:    Linear temperature coefficient of transit time (default value = 0 s/K) <font color=red>Levels 1,3</font>
 
 TTT2:    Quadratic temperature coefficient of transit time (default value = 0 s/K2) <font color=red>Levels 1,3</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1-3</font>

 XTI:     Temperature exponent of saturation current (default value = 3) <font color=red>Levels 1,3</font>
            <font color=blue><b>XTI = 3 for <i>pn</i>-junction
            XTI = 2 for Schottky barrier diode</b></font>

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt) <font color=red>Levels 1,3</font>
            <font color=blue><b>EG = 1.16 for Si
            EG = 0.73 for Schottky barrier diode
            EG = 0.71 for Ge</b></font>


<b>Level=2 General Parameters:</b>

 M:       Diode multiplier (default value = 1) <font color=red>Levels 2,3</font>


<b>Level=2 Geometric Parameters:</b>

 L:       Diode length (default value = 0 m) <font color=red>Levels 2,3</font>

 W:       Diode width (default value = 0 m) <font color=red>Levels 2,3</font>


<b>Level=2 Electrical Parameters:</b>

 JF:       Forward Fowler-Nordheim current coefficient (default value = 0 A/V<sup>2</sup>) <font color=red>Level 2</font>

 JR:       Reverse Fowler-Nordheim current coefficient (default value = 0 A/V<sup>2</sup>) <font color=red>Level 2</font>

 EF:       Forward Fowler-Nordheim critical electrical field (default value = 0 V/m) <font color=red>Level 2</font>

 ER:       Reverse Fowler-Nordheim critical electrical field (default value = 0 V/m) <font color=red>Level 2</font>


<b>Level=3 Geometric Parameters:</b>

 LP:       Polysilicon length (default value = 0 m) <font color=red>Level 3</font>

 WP:       Polysilicon width (default value = 0 m) <font color=red>Level 3</font>

 XP:       Widening of polysilicon due to masking and etching (default value = 0 m) <font color=red>Level 3</font>

 XOI:      Polysilicon thickness (default value = 0 m) <font color=red>Level 3</font>

 LM:       Metal length (default value = 0 m) <font color=red>Level 3</font>

 WM:       Metal width (default value = 0 m) <font color=red>Level 3</font>

 XM:       Widening of metal due to masking and etching (default value = 0 m) <font color=red>Level 3</font>

 XOM:      Metal thickness (default value = 0 m) <font color=red>Level 3</font>


<b>Level=3 Electrical Parameters:</b>

 JS:      Saturation current density at reference temperature (default value = 1e-4 Amp/m<sup>2</sup>) <font color=red>Level 3</font>

 JSW:     Saturation current density of perimeter at reference temperature (default value = 0 Amp/m<sup>2</sup>) <font color=red>Level 3</font>


<b>Level=2 Junction Capacitance Parameters:</b>

 CJ2:     Zero-bias bulk capacitance per meter square at reference temperature (default value = 1e-4 F/m<sup>2</sup>) <font color=red>Level 3</font>

 CJSW2:   Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1,3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1,3</font>

 GminDC:  Leakage conductance (default value = 1e-16 mho) <font color=red>Levels 1,3</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 0-4</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
        public 
          R RS1(
            Tnom=Tnom,
            R=RS,
            TR1=TRS,
            TR2=0,
            Area=Area) annotation (extent=[-80,-10; -40,10]);
          BondLib.Spice.Utilities.ID0 ID0(
            Level=Level,
            N=N,
            IKF=IKF,
            IKR=IKR) annotation (extent=[4,-16; 36,16], rotation=-90);
          BondLib.Electrical.Analog.Spice.Utilities.CjD Dd(
            Tnom=Tnom,
            Level=Level,
            Area=Area,
            PJ=PJ,
            M=M,
            IS=IS,
            JS=JS,
            N=N,
            TT=TT,
            PB=PB,
            PHP=PHP,
            CJ=CJ,
            CJ2=CJ2,
            MJ=MJ,
            CJSW=CJSW,
            CJSW2=CJSW2,
            MJSW=MJSW,
            FC=FC,
            GminDC=GminDC,
            EG=EG,
            XTI=XTI,
            TCV=TCV,
            TTT1=TTT1,
            TTT2=TTT2,
            ISW=ISW,
            JSW=JSW,
            TOX=TOX,
            L=L,
            W=W,
            LP=LP,
            WP=WP,
            XP=XP,
            XOI=XOI,
            LM=LM,
            WM=WM,
            XM=XM,
            XOM=XOM,
            JF=JF,
            JR=JR,
            EF=EF,
            ER=ER,
            EMin=EMin,
            EMax=EMax,
            BV=BVval,
            IBV=IBVval,
            enforceStates=enforceStates) 
                       annotation (extent=[0,58; 40,30]);
          BondLib.Electrical.Analog.Spice.Utilities.Isignal Id 
            annotation (extent=[0,-60; 40,-20]);
          BondLib.Electrical.Analog.Sensors.VoltageSensor vd 
            annotation (extent=[10,90; 30,70]);
          Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
            annotation (extent=[-40,-70; -20,-50]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-50,-50; -70,-30], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[-100,-90; -80,-70]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(Dd.id, ID0.u)   annotation (points=[20,28.6; 20,19.2], style(
                color=74, rgbcolor={0,0,127}));
          connect(ID0.y, Id.InPort1) annotation (points=[20,-17.6; 20,-28],
              style(color=74, rgbcolor={0,0,127}));
          connect(RS1.p, p) annotation (points=[-80,0; -100,0], style(color=3,
                rgbcolor={0,0,255}));
          connect(RS1.n, Dd.p)   annotation (points=[-40,0; -20,0; -20,44; 0,44],
              style(color=3, rgbcolor={0,0,255}));
          connect(Dd.p, vd.p)   annotation (points=[0,44; -20,44; -20,80; 10,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(RS1.n, Id.p) annotation (points=[-40,0; -20,0; -20,-40; 0,-40],
              style(color=3, rgbcolor={0,0,255}));
          connect(Id.n, n) annotation (points=[40,-40; 70,-40; 70,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Dd.n, n)   annotation (points=[40,44; 70,44; 70,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(vd.n, Dd.n)   annotation (points=[30,80; 70,80; 70,44; 40,44],
              style(color=3, rgbcolor={0,0,255}));
          connect(vd.V, ID0.v) annotation (points=[20,90; 20,94; 80,94; 80,20;
                50,20; 50,9.79685e-016; 36,9.79685e-016], style(color=74,
                rgbcolor={0,0,127}));
          connect(Dd.heatPort, RS1.heatPort)   annotation (points=[20,58; -30,
                58; -30,-20; -60,-20; -60,-10],
                                       style(color=42, rgbcolor={191,0,0}));
          connect(Tdev1.T, ID0.Tdev) annotation (points=[-20,-60; -10,-60; -10,
                -9.79685e-016; 4,-9.79685e-016], style(color=74, rgbcolor={0,0,
                  127}));
          connect(Id.heatPort, Dd.heatPort) annotation (points=[20,-60; 60,-60;
                60,58; 20,58], style(color=42, rgbcolor={191,0,0}));
          connect(Tdev1.port, RS1.heatPort) annotation (points=[-40,-60; -40,
                -20; -60,-20; -60,-10], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.p, RS1.heatPort) annotation (points=[-60,-29; -60,-10],
              style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[-60,-51; -60,-80; 0,-80;
                0,-100], style(color=42, rgbcolor={191,0,0}));
          connect(T_nom.port, Sw1.n1) annotation (points=[-79,-80; -72,-80; -72,
                -51; -65,-51], style(color=42, rgbcolor={191,0,0}));
        end D;
        
        model Z "Spice-style Zener diode of the analog electrical library" 
          extends Interfaces.SpiceTwoPort;
          constant Real inf=Modelica.Constants.inf;
          
          /* General parameters */
          parameter Integer Level(min=1,max=2) = 1 
            "ZENER diode modeling level (check documentation window for details)";
          parameter Real Area(min=Modelica.Constants.small) = 1 
            "Relative device area";
          
          /* Process parameters */
          parameter Modelica.SIunits.Length TOX(min=Modelica.Constants.small) = 1e-7 
            "|Process parameters|Thin oxide thickness";
          
          /* Basic Electrical parameters */
          parameter Modelica.SIunits.Voltage PB(min=0) = 0.8 
            "|Electrical parameters|Built-in junction potential at reference temperature";
          parameter Modelica.SIunits.Voltage BV(min=0) = 10 
            "|Electrical parameters|Reverse Zener breakdown voltage";
          parameter Modelica.SIunits.Voltage PHP(min=0) = 0.8 
            "|Electrical parameters|Built-in perimeter potential at reference temperature";
          parameter Modelica.SIunits.Voltage V0(min=0)=0 
            "|Electrical parameters|Forward threshold voltage";
          parameter Real N=1 
            "|Electrical parameters|Current emission coefficient";
          parameter Modelica.SIunits.Current IS(min=Modelica.Constants.small) = 1e-14 
            "|Electrical parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISW(min=0) = 0 
            "|Electrical parameters|Saturation current of perimeter at reference temperature";
          parameter Real PJ(min=0) = 1 
            "|Electrical parameters|Relative perimeter width";
          
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF(min=Modelica.Constants.small) = inf 
            "|High current beta degradation effect|Forward knee current";
          parameter Modelica.SIunits.Current IKR(min=Modelica.Constants.small) = inf 
            "|High current beta degradation effect|Reverse knee current";
          
          /* Parasitic resistor parameters */
          parameter Modelica.SIunits.Resistance RS(min=0) = 0 
            "|Parasitic resistor parameters|Series resistance at reference temperature";
          parameter Modelica.SIunits.Resistance RB(min=0)=0 
            "|Parasitic resistor parameters|Zener breakdown resistance at reference temperature";
          
          /* Junction capacitance parameters */
          parameter Modelica.SIunits.Capacitance CJ(min=0) = 0.5e-12 
            "|Junction capacitance parameters|Zero-bias bulk capacitance at reference temperature";
          parameter Real MJ(min=0.1,max=0.9) = 0.33 
            "|Junction capacitance parameters|Bulk junction grading coefficient";
          parameter Modelica.SIunits.Capacitance CJSW(min=0) = 0.5e-12 
            "|Junction capacitance parameters|Zero-bias perimeter capacitance at reference temperature";
          parameter Real MJSW(min=0.1,max=0.9) = 0.33 
            "|Junction capacitance parameters|Perimeter capacitance grading coefficient";
          parameter Real FC(min=0,max=0.95) = 0.5 
            "|Junction capacitance parameters|Forward-bias depletion capacitance coefficient";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TT(min=0) = 0 
            "|Transit time parameters|Transit time at reference temperature";
          
          /* Temperature compensation parameters */
          parameter Real TRS(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of series resistance";
          parameter Real TRB(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of Zener breakdown resistance";
          parameter Real TTT1(unit="s/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of transit time";
          parameter Real TTT2(unit="s/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of transit time";
          parameter Real XTI(min=0) = 3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Modelica.SIunits.Voltage EG(min=0.1) = 1.16 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
          
          /* Level=2 General parameters */
          parameter Integer M(min=1) = 1 "|Level=2||Diode multiplier";
          
          /* Level=2 Geometric parameters */
          parameter Modelica.SIunits.Length L(min=0) = 0 
            "|Level=2|Geometric parameters|Diode length";
          parameter Modelica.SIunits.Length W(min=0) = 0 
            "|Level=2|Geometric parameters|Diode width";
          parameter Modelica.SIunits.Length LP(min=0) = 0 
            "|Level=2|Geometric parameters|Polysilicon length";
          parameter Modelica.SIunits.Length WP(min=0) = 0 
            "|Level=2|Geometric parameters|Polysilicon width";
          parameter Modelica.SIunits.Length XP(min=0) = 0 
            "|Level=2|Geometric parameters|Widening of polysilicon due to masking and etching";
          parameter Modelica.SIunits.Length XOI(min=0) = 0 
            "|Level=2|Geometric parameters|Polysilicon thickness";
          parameter Modelica.SIunits.Length LM(min=0) = 0 
            "|Level=2|Geometric parameters|Metal length";
          parameter Modelica.SIunits.Length WM(min=0) = 0 
            "|Level=2|Geometric parameters|Metal width";
          parameter Modelica.SIunits.Length XM(min=0) = 0 
            "|Level=2|Geometric parameters|Widening of metal due to masking and etching";
          parameter Modelica.SIunits.Length XOM(min=0) = 0 
            "|Level=2|Geometric parameters|Metal thickness";
          
          /* Level=2 Electrical parameters */
          parameter Modelica.SIunits.CurrentDensity JS(min=Modelica.Constants.small) = 1e-4 
            "|Level=2|Electrical parameters|Saturation current density at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JSW(min=0) = 0 
            "|Level=2|Electrical parameters|Saturation current density of perimeter at reference temperature";
          
          /* Level=2 junction capacitance parameters */
          parameter Real CJ2(min=Modelica.Constants.small,unit="F/m2") = 1e-4 
            "|Level=2|Junction capacitance parameters|Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real CJSW2(min=0,unit="F/m") = 1e-9 
            "|Level=2|Junction capacitance parameters|Zero-bias perimeter capacitance per meter at reference temperature";
          
          /* Initialization parameters */
          parameter Modelica.SIunits.Time rampTime = 0 
            "|Initialization||Ramping time for sources";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=Modelica.Constants.small) = 1e-16 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
        protected 
          parameter Integer LEV = if Level==1 then 1 else 3;
        public 
          D D1(
            Tnom=Tnom,
            IS=IS,
            RS=RS,
            N=N,
            TT=TT,
            CJ=CJ,
            PB=PB,
            MJ=MJ,
            EG=EG,
            XTI=XTI,
            FC=FC,
            BV=0,
            IBV=0,
            TRS=TRS,
            TCV=0,
            TTT1=TTT1,
            TTT2=TTT2,
            ISW=ISW,
            CJSW=CJSW,
            PHP=PHP,
            MJSW=MJSW,
            IKF=IKF,
            IKR=IKR,
            TOX=TOX,
            GminDC=GminDC,
            Area=Area,
            PJ=PJ,
            EMin=EMin,
            EMax=EMax,
            L=L,
            W=W,
            M=M,
            JS=JS,
            CJ2=CJ2,
            JSW=JSW,
            CJSW2=CJSW2,
            LP=LP,
            WP=WP,
            XP=XP,
            XOI=XOI,
            LM=LM,
            WM=WM,
            XM=XM,
            XOM=XOM,
            JF=0,
            JR=0,
            EF=0,
            ER=0,
            Level=LEV,
            enforceStates=enforceStates) 
                       annotation (extent=[20,30; 40,50]);
          D D2(
            Tnom=Tnom,
            IS=IS,
            RS=RS,
            N=N,
            TT=TT,
            CJ=CJ,
            PB=PB,
            MJ=MJ,
            EG=EG,
            XTI=XTI,
            FC=FC,
            IBV=0,
            TRS=TRS,
            TCV=0,
            TTT1=TTT1,
            TTT2=TTT2,
            ISW=ISW,
            CJSW=CJSW,
            PHP=PHP,
            MJSW=MJSW,
            IKF=IKF,
            IKR=IKR,
            TOX=TOX,
            GminDC=GminDC,
            Area=Area,
            PJ=PJ,
            EMin=EMin,
            EMax=EMax,
            L=L,
            W=W,
            M=M,
            JS=JS,
            CJ2=CJ2,
            JSW=JSW,
            CJSW2=CJSW2,
            LP=LP,
            WP=WP,
            XP=XP,
            XOI=XOI,
            LM=LM,
            WM=WM,
            XM=XM,
            XOM=XOM,
            JF=0,
            JR=0,
            EF=0,
            ER=0,
            Level=LEV,
            BV=0,
            enforceStates=enforceStates) 
                       annotation (extent=[60,-10; 40,10]);
          annotation (
            Diagram(
        Rectangle(extent=[-40,-40; -20,-60],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-40,-40; -20,-60],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-20,-50; -14,-50], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Polygon(points=[28,0; -20,40; -20,-40; 28,0],     style(
                  color=3,
                  gradient=0,
                  fillColor=7)),
              Line(points=[-90,0; 40,0]),
              Line(points=[40,0; 90,0]),
              Line(points=[28,40; 28,-40],   style(color=3)),
              Line(points=[48,60; 28,40],    style(color=3)),
              Line(points=[28,-40; 8,-60],   style(color=3)),
              Text(extent=[-58,-62; 60,-96], string="BV=%BV"),
              Line(points=[0,-24; 0,-92], style(color=42, rgbcolor={166,0,0}))),
            Documentation(info="<html>
The Spice-style Zener diode model has been built from two diodes as a macro model [1].  It offers two modeling levels.  The <b>Level=1</b> model characterizes the internal junction diodes by their electrical properties.  The <b>Level=2</b> model is similar to the <b>Level=1</b> model, but here, the diodes are characterized by their physical parameters, such as length and width of the junction.  Also, this model is yet more accurate as it allows the modeler to specify a polysilicon and a metal layer, in which case the additional capacitances associated with these layers will be computed as well. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   DIODE modeling level (default value = 1)
            Level=1: Non-geometric Zener diode
            Level=2: Geometric Zener diode</b></font>

 Area:    Relative area occupied by the diode (default value = 1)


<b>Process Parameters:</b>

 TOX:     Thin oxide thickness (default value = 1e-7 m)


<b>Basic Electrical Parameters:</b>

 PB:      Built-in junction potential at reference temperature (default value = 0.8 Volt)

 BV:      Zener breakdown voltage (default value = 10 Volt)
          <font color=blue><b>BV must be specified as a positive number.</b></font>

 PHP:     Built-in perimeter potential at reference temperature (default value = 0.8 Volt)

 V0:      Forward threshold voltage (default value = 0 Volt)

 N:       Current emission coefficient (default value = 1)

 IS:      Saturation current at reference temperature (default value = 1e-14 Amp)

 ISW:     Saturation current of perimeter at reference temperature (default value = 0 Amp)

 PJ:      Relative perimeter width (default value = 1)


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Forward knee current (default value = &infin; Amp)

 IKR:     Reverse knee current (default value = &infin; Amp)


<b>Parasitic Resistor Parameters:</b>

 RS:      Series resistance at reference temperature (default value = 0 Ohm)

 RB:      Zener breakdown resistance at reference temperature (default value = 0 Ohm)


<b>Junction Capacitance Parameters:</b>

 CJ:      Zero-bias bulk capacitance at reference temperature (default value = 0.5e-12 F)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 CJSW:    Zero-bias perimeter capacitance at reference temperature (default value = 0.5e-12 F)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)


<b>Transit Time Parameters:</b>

 TT:      Transit time at reference temperature (default value = 0 sec)


<b>Temperature Compensation Parameters:</b>

 TRS:     Linear temperature coefficient of series resistance (default value = 0 Ohm/K)

 TRB:     Linear temperature coefficient of Zener resistance (default value = 0 Ohm/K)

 TTT1:    Linear temperature coefficient of transit time (default value = 0 s/K)
 
 TTT2:    Quadratic temperature coefficient of transit time (default value = 0 s/K2)

 Tnom:    Reference temperature (default value = 300.15 K)

 XTI:     Temperature exponent of saturation current (default value = 3)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)


<b>Level=2 General Parameters:</b>

 M:       Diode multiplier (default value = 1)


<b>Level=2 Geometric Parameters:</b>

 L:       Diode length (default value = 0 m)

 W:       Diode width (default value = 0 m)

 LP:      Polysilicon length (default value = 0 m)

 WP:      Polysilicon width (default value = 0 m)

 XP:      Widening of polysilicon due to masking and etching (default value = 0 m)

 XOI:     Polysilicon thickness (default value = 0 m)

 LM:      Metal length (default value = 0 m)

 WM:      Metal width (default value = 0 m)

 XM:      Widening of metal due to masking and etching (default value = 0 m)

 XOM:     Metal thickness (default value = 0 m)


<b>Level=2 Electrical Parameters:</b>

 JS:      Saturation current density at reference temperature (default value = 1e-4 Amp/m<sup>2</sup>)

 JSW:     Saturation current density of perimeter at reference temperature (default value = 0 Amp/m<sup>2</sup>)


<b>Level=2 Junction Capacitance Parameters:</b>

 CJ2:     Zero-bias bulk capacitance per meter square at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 CJSW2:   Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)


<b>Initialization Parameters:</b>

 rampTime: Initial ramping time for sources (default value = 0 s)


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)

 GminDC:  Leakage conductance (default value = 1e-16 mho)


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 0-4</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.18.
</ol>
</html>
"));
          
          RampingSources.V V1(V=V0, rampTime=rampTime) 
            annotation (extent=[-40,30; -20,50]);
          RampingSources.V V2(rampTime=rampTime, V=BV) 
            annotation (extent=[10,-10; -10,10]);
          R R1(
            Tnom=Tnom,
            R=RB,
            TR1=TRB,
            TR2=0,
            Area=Area) annotation (extent=[-40,-10; -60,10]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[10,-60; -10,-40], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[-70,-90; -50,-70]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(D1.n, n) annotation (points=[40,40; 80,40; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(D2.p, n) annotation (points=[60,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(V1.n, D1.p) annotation (points=[-20,40; 20,40], style(color=3,
                rgbcolor={0,0,255}));
          connect(V1.p, p) annotation (points=[-40,40; -80,40; -80,0; -100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(V2.p, D2.n) annotation (points=[10,0; 40,0],     style(color=
                  3, rgbcolor={0,0,255}));
          connect(R1.p, V2.n) annotation (points=[-40,0; -10,0],     style(
                color=3, rgbcolor={0,0,255}));
          connect(R1.n, p) annotation (points=[-60,0; -100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(D2.heatPort, R1.heatPort) annotation (points=[50,-10; 50,-20;
                -50,-20; -50,-10], style(color=42, rgbcolor={191,0,0}));
          connect(D1.heatPort, D2.heatPort) annotation (points=[30,30; 30,-20;
                50,-20; 50,-10], style(color=42, rgbcolor={191,0,0}));
          connect(T_nom.port, Sw1.n1) annotation (points=[-49,-80; -12,-80; -12,
                -61; -5,-61], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.p, R1.heatPort) annotation (points=[6.73533e-016,-39;
                6.73533e-016,-20; -50,-20; -50,-10], style(color=42, rgbcolor={
                  191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[-6.73533e-016,-61;
                -6.73533e-016,-75.5; 0,-75.5; 0,-100], style(color=42, rgbcolor=
                 {191,0,0}));
        end Z;
        
        model Qnl "Spice-style laterally diffused NPN bipolar transistor" 
          constant Real inf=Modelica.Constants.inf;
          
          /* General parameters */
          parameter Integer Level(min=1,max=2) = 2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Real Area=1 "Relative area occupied by device";
          
          /* DC model parameters */
          parameter Real BF=100 
            "|DC model parameters|Maximum forward current gain at reference temperature";
          parameter Real BR=1 
            "|DC model parameters|Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
            "|DC model parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
            "|DC model parameters|Saturation current used for current injection";
          parameter Real NF=1 
            "|DC model parameters|Forward current emission coefficient";
          parameter Real NR=1 
            "|DC model parameters|Reverse current emission coefficient";
          parameter Modelica.SIunits.Voltage VSUB=0 
            "|DC model parameters|Substrate potential (if unconnected)";
          
          /* Low current beta degradation effect parameters */
          parameter Real C4(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
          parameter Modelica.SIunits.Current ISC(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Real C2(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
          parameter Modelica.SIunits.Current ISE(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
            "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
            "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
          
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=inf 
            "|High current beta degradation effect|Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=inf 
            "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
          
          /* Base width modulation parameters */
          parameter Modelica.SIunits.Voltage VAF=inf 
            "|Base width modulation parameters|Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=inf 
            "|Base width modulation parameters|Reverse early voltage";
          
          /* Parasitic resistor parameters */
          parameter Modelica.SIunits.Current IRB=inf 
            "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
          parameter Modelica.SIunits.Resistance RB(min=0) = 0 
            "|Parasitic resistor parameters|Zero-bias base resistance";
          parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
            "|Parasitic resistor parameters|Minimum base resistance at high currents";
          parameter Modelica.SIunits.Resistance RC(min=0) = 0 
            "|Parasitic resistor parameters|Collector resistance";
          parameter Modelica.SIunits.Resistance RE(min=0) = 0 
            "|Parasitic resistor parameters|Emitter resistance";
          
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
          parameter Real MJC(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-collector junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJC=0.75 
            "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
          parameter Real MJE(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-emitter junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJE=0.75 
            "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
          parameter Real MJS(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Substrate junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJS=0.75 
            "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
          parameter Real XCJC(min=0,max=1) = 1 
            "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
          parameter Real FC(min=0,max=1) = 0.5 
            "|Junction capacitor parameters|Depletion capacitance factor for linearization";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TF(min=0) = 0 
            "|Transit time parameters|Ideal forward transit time";
          parameter Modelica.SIunits.Time TR(min=0) = 0 
            "|Transit time parameters|Ideal reverse transit time";
          
          /* Temperature compensation parameters */
          parameter Real TRB1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
          parameter Real TRB2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
          parameter Real TRM1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
          parameter Real TRM2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
          parameter Real TRC1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
          parameter Real TRC2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
          parameter Real TRE1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
          parameter Real TRE2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI=3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Real XTB=0 
            "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.11 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
          
          /* Initialization parameters */
          parameter Modelica.SIunits.Time rampTime=0 
            "|Advanced|Initialization parameters|Substrate potential ramping time (if unconnected)";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
        protected 
          parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
          parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Utilities.Qni Qni(
            BF=BF,
            BR=BR,
            IS=IS,
            NF=NF,
            NR=NR,
            GminDC=GminDC,
            NC=NC,
            NE=NE,
            VAF=VAF,
            VAR=VAR,
            IKF=IKF,
            IKR=IKR,
            CJC=CJC,
            MJC=MJC,
            VJC=VJC,
            CJE=CJE,
            MJE=MJE,
            VJE=VJE,
            XCJC=XCJC,
            TF=TF,
            TR=TR,
            Tnom=Tnom,
            XTI=XTI,
            XTB=XTB,
            EG=EG,
            Area=Area,
            EMin=EMin,
            EMax=EMax,
            FC=FC,
            Level=Level,
            ISC=ISCeff,
            ISE=ISEeff,
            ISS=ISS,
            enforceStates=enforceStates) 
                     annotation (extent=[-20,-20; 20,20]);
          annotation (
            Diagram(
              Text(
                extent=[32,8; 42,0],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"),
              Text(
                extent=[-60,20; -50,12],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"),
              Text(
                extent=[-18,40; -8,32],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
              Text(
                extent=[-30,70; -20,62],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
        Rectangle(extent=[20,-60; 40,-80],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-60; 40,-80],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-70; 20,-70],   style(color=83, rgbcolor={255,85,
                      255})),
              Ellipse(extent=[-2,-48; 2,-52], style(
                  color=42,
                  rgbcolor={211,0,0},
                  fillColor=42,
                  rgbfillColor={209,0,0})),
        Rectangle(extent=[-80,-70; -60,-90],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-70; -60,-90],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-86,-80; -80,-80], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[30,-50; 24,-36; 16,-44; 30,-50],     style(fillColor=3,
                      fillPattern=1)),
              Line(points=[0,-40; 0,-91],   style(color=42)),
            Text(
              extent=[12,30; 120,-34],
              string="%Area x",
              style(color=3, rgbcolor={0,0,255})),
              Text(extent=[-100,64; 100,98],    string="%name")),
            Documentation(info="<html>
The <b>Qnl</b> element of the Spice analog electrical library implements a full-fledges Spice-stye Gummel-Poon model of the laterally diffused NPN bipolar transistor [1-3]. The model contains a <b>Qni</b> model implementing the inner parts of the NPN transistor up to the internal nodes. <p>

The substrate node is an electrical node, but we decided to paint it green so that the substrate connections wouldn't clutter up the circuit diagram unnecessarily. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

The substrate port may remain unconnected, in which case the substrate is assumed to be at a fixed potential of <b>VSUB</b>. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 VSUB:    Substrate potential if unconnected (default value = 0 V) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Initialization Parameters:</b>

 rampTime:Substrate potential ramping time if unconnected (default value = 0 s) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 1,2</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
          Utilities.Rb Rb(
            Tnom=Tnom,
            RB=RB,
            RBM=RBM,
            IRB=IRB,
            TRB1=TRB1,
            TRB2=TRB2,
            TRM1=TRM1,
            TRM2=TRM2,
            Area=Area,
            Level=Level) 
                       annotation (extent=[-80,-10; -40,10]);
        Modelica.Blocks.Sources.Constant C0(k=0) 
          annotation (extent=[-56,-46; -44,-34]);
        Modelica.Blocks.Sources.Constant qb(k=1) 
          annotation (extent=[-56,-66; -44,-54]);
          Utilities.Cbx Cbx(Tnom=Tnom) 
                             annotation (extent=[-50,40; -10,60]);
          Utilities.Rce Rc(
            Tnom=Tnom,
            R=RC,
            TR1=TRC1,
            TR2=TRC2,
            Area=Area,
            Level=Level) annotation (extent=[40,40; 80,60]);
          Utilities.Rce Re(
            Tnom=Tnom,
            Area=Area,
            Level=Level,
            R=RE,
            TR1=TRE1,
            TR2=TRE2) annotation (extent=[40,-50; 80,-30]);
          Utilities.Cj Dbs(
            Tnom=Tnom,
            EG=EG,
            XTI=XTI,
            MJ=MJS,
            VJ=VJS,
            XCJ=1,
            Tau=0,
            GminDC=0,
            Area=Area,
            Level=Level,
            BE=false,
            EMin=EMin,
            EMax=EMax,
            IS=0,
            CJ=CJS,
            FC=0,
            N=1,
            enforceStates=enforceStates) 
                       annotation (extent=[-10,-70; -30,-30], rotation=-90);
          Interfaces.SPin S "Substrate (may remain unconnected)" 
                            annotation (extent=[-110,-60; -90,-40]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
          Ideal.IdealCommutingSwitch2 Sw2 
            annotation (extent=[-90,-90; -110,-70], rotation=90);
          Basic.Ground grd annotation (extent=[-90,-16; -70,-34]);
          RampingSources.V Vsub(V=VSUB, rampTime=rampTime) 
            annotation (extent=[-90,-60; -70,-40], rotation=90);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          
          if cardinality(S) <= 1 then
            Sw2.control = false;
          else
            Sw2.control = true;
          end if;
          connect(Rb.heatPort, Qni.heatPort)   annotation (points=[-60,-10; -60,
                -20; 0,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rb.n, Qni.Bint)   annotation (points=[-40,0; -20,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rb.p, B)  annotation (points=[-80,0; -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Qni.qb, Rb.qb)   annotation (points=[22,0; 30,0; 30,30; -60,
                30; -60,10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cbx.p, Rb.p)   annotation (points=[-50,50; -80,50; -80,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Cbx.heatPort, Qni.heatPort)   annotation (points=[-30,40; -30,
                -20; 0,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Qni.ix, Cbx.ix)   annotation (points=[-10,22; -10,40; 0,40; 0,
                70; -30,70; -30,60], style(color=74, rgbcolor={0,0,127}));
          connect(Rc.n, C) annotation (points=[80,50; 100,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rc.p, Cbx.n) annotation (points=[40,50; -10,50], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Rc.heatPort, Qni.heatPort) annotation (points=[60,40; 60,-20;
                0,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Re.n, E) annotation (points=[80,-40; 90,-40; 90,-50; 100,-50],
                                                                 style(color=3,
                rgbcolor={0,0,255}));
          connect(Re.p, Qni.Eint) annotation (points=[40,-40; 20,-40; 20,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(Dbs.p, Qni.Bint) annotation (points=[-20,-30; -20,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(qb.y, Dbs.qb) annotation (points=[-43.4,-60; -30,-60], style(
                color=74, rgbcolor={0,0,127}));
          connect(C0.y, Dbs.C0) annotation (points=[-43.4,-40; -30,-40], style(
                color=74, rgbcolor={0,0,127}));
          connect(Qni.Cint, Rc.p) annotation (points=[20,10; 20,50; 40,50],
              style(color=3, rgbcolor={0,0,255}));
          connect(Dbs.heatPort, Qni.heatPort) annotation (points=[-10,-50; 0,
                -50; 0,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Re.heatPort, Dbs.heatPort) annotation (points=[60,-50; -10,
                -50], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.p, Qni.heatPort) annotation (points=[-6.73533e-016,-59;
                -6.73533e-016,-40.5; 0,-40.5; 0,-20], style(color=42, rgbcolor=
                  {191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-87.5; 0,-87.5; 0,-100], style(color=42, rgbcolor=
                  {191,0,0}));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-81; 12,-81; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw2.p, Dbs.n) annotation (points=[-100,-90; -100,-100; -20,
                -100; -20,-70], style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.n2, S) annotation (points=[-100,-70; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vsub.p, Sw2.n1) annotation (points=[-80,-60; -95,-60; -95,-70],
              style(color=3, rgbcolor={0,0,255}));
          connect(Vsub.n, grd.p) annotation (points=[-80,-40; -80,-34], style(
                color=3, rgbcolor={0,0,255}));
        end Qnl;
        
        model Qnv "Spice-style vertically diffused NPN bipolar transistor" 
          constant Real inf=Modelica.Constants.inf;
          
          /* General parameters */
          parameter Integer Level(min=1,max=2) = 2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Real Area=1 "Relative area occupied by device";
          
          /* DC model parameters */
          parameter Real BF=100 
            "|DC model parameters|Maximum forward current gain at reference temperature";
          parameter Real BR=1 
            "|DC model parameters|Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
            "|DC model parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
            "|DC model parameters|Saturation current used for current injection";
          parameter Real NF=1 
            "|DC model parameters|Forward current emission coefficient";
          parameter Real NR=1 
            "|DC model parameters|Reverse current emission coefficient";
          parameter Modelica.SIunits.Voltage VSUB=0 
            "|DC model parameters|Substrate potential (if unconnected)";
          
          /* Low current beta degradation effect parameters */
          parameter Real C4(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
          parameter Modelica.SIunits.Current ISC(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Real C2(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
          parameter Modelica.SIunits.Current ISE(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
            "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
            "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
          
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=inf 
            "|High current beta degradation effect|Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=inf 
            "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
          
          /* Base width modulation parameters */
          parameter Modelica.SIunits.Voltage VAF=inf 
            "|Base width modulation parameters|Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=inf 
            "|Base width modulation parameters|Reverse early voltage";
          
          /* Parasitic Resistor Parameters */
          parameter Modelica.SIunits.Current IRB=inf 
            "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
          parameter Modelica.SIunits.Resistance RB(min=0) = 0 
            "|Parasitic resistor parameters|Zero-bias base resistance";
          parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
            "|Parasitic resistor parameters|Minimum base resistance at high currents";
          parameter Modelica.SIunits.Resistance RC(min=0) = 0 
            "|Parasitic resistor parameters|Collector resistance";
          parameter Modelica.SIunits.Resistance RE(min=0) = 0 
            "|Parasitic resistor parameters|Emitter resistance";
          
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
          parameter Real MJC(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-collector junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJC=0.75 
            "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
          parameter Real MJE(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-emitter junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJE=0.75 
            "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
          parameter Real MJS(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Substrate junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJS=0.75 
            "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
          parameter Real XCJC(min=0,max=1) = 1 
            "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
          parameter Real FC(min=0,max=1) = 0.5 
            "|Junction capacitor parameters|Depletion capacitance factor for linearization";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TF(min=0) = 0 
            "|Transit time parameters|Ideal forward transit time";
          parameter Modelica.SIunits.Time TR(min=0) = 0 
            "|Transit time parameters|Ideal reverse transit time";
          
          /* Temperature compensation and area parameters */
          parameter Real TRB1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
          parameter Real TRB2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
          parameter Real TRM1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
          parameter Real TRM2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
          parameter Real TRC1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
          parameter Real TRC2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
          parameter Real TRE1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
          parameter Real TRE2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI=3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Real XTB=0 
            "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.11 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
          
          /* Initialization parameters */
          parameter Modelica.SIunits.Time rampTime=0 
            "|Advanced|Initialization parameters|Substrate potential ramping time (if unconnected)";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
        protected 
          parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
          parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Utilities.Qni Qni(
            BF=BF,
            BR=BR,
            IS=IS,
            NF=NF,
            NR=NR,
            GminDC=GminDC,
            NC=NC,
            NE=NE,
            VAF=VAF,
            VAR=VAR,
            IKF=IKF,
            IKR=IKR,
            CJC=CJC,
            MJC=MJC,
            VJC=VJC,
            CJE=CJE,
            MJE=MJE,
            VJE=VJE,
            XCJC=XCJC,
            TF=TF,
            TR=TR,
            Tnom=Tnom,
            XTI=XTI,
            XTB=XTB,
            EG=EG,
            Area=Area,
            EMin=EMin,
            EMax=EMax,
            FC=FC,
            Level=Level,
            ISC=ISCeff,
            ISE=ISEeff,
            ISS=ISS,
            enforceStates=enforceStates) 
                     annotation (extent=[-40,-20; 0,20]);
          Utilities.Rb Rb(
            Tnom=Tnom,
            RB=RB,
            RBM=RBM,
            IRB=IRB,
            TRB1=TRB1,
            TRB2=TRB2,
            TRM1=TRM1,
            TRM2=TRM2,
            Area=Area,
            Level=Level) 
                       annotation (extent=[-88,-10; -48,10]);
        Modelica.Blocks.Sources.Constant C0(k=0) 
          annotation (extent=[80,-6; 68,6]);
        Modelica.Blocks.Sources.Constant qb(k=1) 
          annotation (extent=[80,14; 68,26]);
          annotation (
            Diagram(Text(
                extent=[2,8; 12,0],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"), Text(
                extent=[-68,20; -58,12],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"),
                    Text(
                extent=[-24,42; -14,32],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
                    Text(
                extent=[-42,72; -32,62],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
        Rectangle(extent=[20,-60; 40,-80],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-60; 40,-80],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-70; 20,-70],   style(color=83, rgbcolor={255,85,
                      255})),
        Rectangle(extent=[-60,-70; -40,-90],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-60,-70; -40,-90],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-50,-70; -50,-64], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[30,-50; 24,-36; 16,-44; 30,-50],     style(fillColor=3,
                      fillPattern=1)),
              Line(points=[0,-40; 0,-91],   style(color=42)),
            Text(
              extent=[20,30; 112,-34],
              string="%Area x",
              style(color=3, rgbcolor={0,0,255})),
              Text(extent=[-102,64; 102,100],   string="%name")),
            Documentation(info="<html>
The <b>Qnv</b> element of the Spice analog electrical library implements a full-fledges Spice-stye Gummel-Poon model of the vertically diffused NPN bipolar transistor [1-3]. The model contains a <b>Qni</b> model implementing the inner parts of the NPN transistor up to the internal nodes. <p>

The substrate node is an electrical node, but we decided to paint it green so that the substrate connections wouldn't clutter up the circuit diagram unnecessarily. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

The substrate port may remain unconnected, in which case the substrate is assumed to be at a fixed potential of <b>VSUB</b>. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 VSUB:    Substrate potential if unconnected (default value = 0 V) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Initialization Parameters:</b>

 rampTime:Substrate potential ramping time if unconnected (default value = 0 s) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 1,2</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
          Utilities.Cbx Cbx(Tnom=Tnom) 
                             annotation (extent=[-54,40; -14,60]);
          Utilities.Rce Rc(
            Tnom=Tnom,
            R=RC,
            TR1=TRC1,
            TR2=TRC2,
            Area=Area,
            Level=Level) annotation (extent=[40,40; 80,60]);
          Utilities.Cj Dsc(
            Tnom=Tnom,
            EG=EG,
            XTI=XTI,
            MJ=MJS,
            VJ=VJS,
            XCJ=1,
            Tau=0,
            GminDC=0,
            Area=Area,
            Level=Level,
            BE=false,
            EMin=EMin,
            EMax=EMax,
            IS=0,
            CJ=CJS,
            FC=0,
            N=1,
            enforceStates=enforceStates) 
                       annotation (extent=[30,30; 50,-10], rotation=-90);
          Utilities.Rce Re(
            Tnom=Tnom,
            Area=Area,
            Level=Level,
            R=RE,
            TR1=TRE1,
            TR2=TRE2) annotation (extent=[40,-50; 80,-30]);
          Interfaces.SPin S "Substrate (may remain unconnected)" 
                            annotation (extent=[-110,-60; -90,-40]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
          Ideal.IdealCommutingSwitch2 Sw2 
            annotation (extent=[-40,-40; -60,-60], rotation=0);
          RampingSources.V Vsub(V=VSUB, rampTime=rampTime) 
            annotation (extent=[-90,-80; -70,-60], rotation=270);
          Basic.Ground grd annotation (extent=[-90,-102; -70,-84]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          
          if cardinality(S) <= 1 then
            Sw2.control = false;
          else
            Sw2.control = true;
          end if;
          connect(Rb.heatPort, Qni.heatPort)   annotation (points=[-68,-10; -68,
                -20; -20,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rb.n, Qni.Bint)   annotation (points=[-48,0; -40,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rb.p, B)  annotation (points=[-88,0; -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Qni.qb, Rb.qb)   annotation (points=[2,0; 12,0; 12,30; -68,30;
                -68,10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cbx.p, Rb.p)   annotation (points=[-54,50; -88,50; -88,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Qni.heatPort, Cbx.heatPort)   annotation (points=[-20,-20;
                -34,-20; -34,40], style(color=42, rgbcolor={191,0,0}));
          connect(Qni.ix, Cbx.ix)   annotation (points=[-30,22; -30,34; -24,34;
                -24,66; -34,66; -34,60], style(color=74, rgbcolor={0,0,127}));
          connect(Rc.n, C) annotation (points=[80,50; 100,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rc.p, Cbx.n) annotation (points=[40,50; -14,50], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Dsc.n, Rc.p) annotation (points=[40,30; 40,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(qb.y, Dsc.qb) annotation (points=[67.4,20; 50,20], style(
                color=74, rgbcolor={0,0,127}));
          connect(C0.y, Dsc.C0) annotation (points=[67.4,0; 58.7,0; 58.7,
                -1.77636e-015; 50,-1.77636e-015], style(color=74, rgbcolor={0,0,
                  127}));
          connect(Rc.heatPort, Dsc.heatPort) annotation (points=[60,40; 20,40;
                20,10; 30,10], style(color=42, rgbcolor={191,0,0}));
          connect(Re.n, E) annotation (points=[80,-40; 90,-40; 90,-50; 100,-50],
                                                                 style(color=3,
                rgbcolor={0,0,255}));
          connect(Re.p, Qni.Eint) annotation (points=[40,-40; 0,-40; 0,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(Dsc.heatPort, Re.heatPort) annotation (points=[30,10; 20,10;
                20,-50; 60,-50], style(color=42, rgbcolor={191,0,0}));
          connect(Qni.Cint, Rc.p) annotation (points=[0,10; 0,50; 40,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Re.heatPort, Qni.heatPort) annotation (points=[60,-50; -20,
                -50; -20,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-81; 12,-81; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.p, Qni.heatPort) annotation (points=[-6.73533e-016,-59;
                -6.73533e-016,-50; -20,-50; -20,-20], style(color=42, rgbcolor=
                  {191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-85.5; 0,-85.5; 0,-100], style(color=42, rgbcolor=
                  {191,0,0}));
          connect(Sw2.n2, S) annotation (points=[-60,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Sw2.p, Dsc.p) annotation (points=[-40,-50; -30,-50; -30,-30;
                40,-30; 40,-10], style(color=3, rgbcolor={0,0,255}));
          connect(Vsub.p, Sw2.n1) annotation (points=[-80,-60; -80,-55; -60,-55],
              style(color=3, rgbcolor={0,0,255}));
          connect(grd.p, Vsub.n) annotation (points=[-80,-84; -80,-80], style(
                color=3, rgbcolor={0,0,255}));
        end Qnv;
        
        model Qpl "Spice-style laterally diffused PNP bipolar transistor" 
          constant Real inf=Modelica.Constants.inf;
          
          /* General parameters */
          parameter Integer Level(min=1,max=2) = 2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Real Area=1 "Relative area occupied by device";
          
          /* DC model parameters */
          parameter Real BF=100 
            "|DC model parameters|Maximum forward current gain at reference temperature";
          parameter Real BR=1 
            "|DC model parameters|Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
            "|DC model parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
            "|DC model parameters|Saturation current used for current injection";
          parameter Real NF=1 
            "|DC model parameters|Forward current emission coefficient";
          parameter Real NR=1 
            "|DC model parameters|Reverse current emission coefficient";
          parameter Modelica.SIunits.Voltage VSUB=0 
            "|DC model parameters|Substrate potential (if unconnected)";
          
          /* Low current beta degradation effect parameters */
          parameter Real C4(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
          parameter Modelica.SIunits.Current ISC(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Real C2(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
          parameter Modelica.SIunits.Current ISE(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
            "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
            "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
          
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=inf 
            "|High current beta degradation effect|Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=inf 
            "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
          
          /* Base width modulation parameters */
          parameter Modelica.SIunits.Voltage VAF=inf 
            "|Base width modulation parameters|Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=inf 
            "|Base width modulation parameters|Reverse early voltage";
          
          /* Parasitic Resistor Parameters */
          parameter Modelica.SIunits.Current IRB=inf 
            "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
          parameter Modelica.SIunits.Resistance RB(min=0) = 0 
            "|Parasitic resistor parameters|Zero-bias base resistance";
          parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
            "|Parasitic resistor parameters|Minimum base resistance at high currents";
          parameter Modelica.SIunits.Resistance RC(min=0) = 0 
            "|Parasitic resistor parameters|Collector resistance";
          parameter Modelica.SIunits.Resistance RE(min=0) = 0 
            "|Parasitic resistor parameters|Emitter resistance";
          
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
          parameter Real MJC(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-collector junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJC=0.75 
            "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
          parameter Real MJE(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-emitter junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJE=0.75 
            "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
          parameter Real MJS(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Substrate junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJS=0.75 
            "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
          parameter Real XCJC(min=0,max=1) = 1 
            "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
          parameter Real FC(min=0,max=1) = 0.5 
            "|Junction capacitor parameters|Depletion capacitance factor for linearization";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TF(min=0) = 0 
            "|Transit time parameters|Ideal forward transit time";
          parameter Modelica.SIunits.Time TR(min=0) = 0 
            "|Transit time parameters|Ideal reverse transit time";
          
          /* Temperature compensation and area parameters */
          parameter Real TRB1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
          parameter Real TRB2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
          parameter Real TRM1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
          parameter Real TRM2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
          parameter Real TRC1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
          parameter Real TRC2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
          parameter Real TRE1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
          parameter Real TRE2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI=3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Real XTB=0 
            "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.11 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
          
          /* Initialization parameters */
          parameter Modelica.SIunits.Time rampTime=0 
            "|Advanced|Initialization parameters|Substrate potential ramping time (if unconnected)";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
        protected 
          parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
          parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Utilities.Rb Rb(
            Tnom=Tnom,
            RB=RB,
            RBM=RBM,
            IRB=IRB,
            TRB1=TRB1,
            TRB2=TRB2,
            TRM1=TRM1,
            TRM2=TRM2,
            Area=Area,
            Level=Level) 
                       annotation (extent=[-80,-10; -40,10]);
        Modelica.Blocks.Sources.Constant C0(k=0) 
          annotation (extent=[-56,-66; -44,-54]);
        Modelica.Blocks.Sources.Constant qb(k=1) 
          annotation (extent=[-56,-46; -44,-34]);
          annotation (
            Diagram(
              Text(
                extent=[32,8; 42,0],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"),
              Text(
                extent=[-60,20; -50,12],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"),
              Text(
                extent=[-30,70; -20,62],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
              Text(
                extent=[-18,40; -8,32],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
        Rectangle(extent=[20,-60; 40,-80],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-60; 40,-80],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-70; 20,-70],   style(color=83, rgbcolor={255,85,
                      255})),
              Ellipse(extent=[-2,-48; 2,-52], style(
                  color=42,
                  rgbcolor={211,0,0},
                  fillColor=42,
                  rgbfillColor={209,0,0})),
        Rectangle(extent=[-80,-70; -60,-90],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-70; -60,-90],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-86,-80; -80,-80], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
The <b>Qpl</b> element of the Spice analog electrical library implements a full-fledges Spice-stye Gummel-Poon model of the laterally diffused PNP bipolar transistor [1-3]. The model contains a <b>Qpi</b> model implementing the inner parts of the PNP transistor up to the internal nodes. <p>

The substrate node is an electrical node, but we decided to paint it green so that the substrate connections wouldn't clutter up the circuit diagram unnecessarily. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

The substrate port may remain unconnected, in which case the substrate is assumed to be at a fixed potential of <b>VSUB</b>. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 VSUB:    Substrate potential if unconnected (default value = 0 V) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Initialization Parameters:</b>

 rampTime:Substrate potential ramping time if unconnected (default value = 0 s) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 1,2</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"),         Icon(
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[-10,-10; 5,-17; -3,-25; -10,-10],     style(fillColor=
                      3, fillPattern=1)),
              Text(extent=[-100,64; 100,100],   string="%name"),
              Line(points=[0,-40; 0,-91],   style(color=42)),
            Text(
              extent=[14,30; 118,-34],
              string="%Area x",
              style(color=3, rgbcolor={0,0,255}))));
          Utilities.Qpi Qpi(
            BF=BF,
            BR=BR,
            IS=IS,
            NF=NF,
            NR=NR,
            GminDC=GminDC,
            NC=NC,
            NE=NE,
            VAF=VAF,
            VAR=VAR,
            IKF=IKF,
            IKR=IKR,
            CJC=CJC,
            MJC=MJC,
            VJC=VJC,
            CJE=CJE,
            MJE=MJE,
            VJE=VJE,
            XCJC=XCJC,
            TF=TF,
            TR=TR,
            Tnom=Tnom,
            XTI=XTI,
            XTB=XTB,
            EG=EG,
            Area=Area,
            EMin=EMin,
            EMax=EMax,
            FC=FC,
            Level=Level,
            ISC=ISCeff,
            ISE=ISEeff,
            ISS=ISS,
            enforceStates=enforceStates) 
                             annotation (extent=[-20,-20; 20,20]);
          Utilities.Cbx Cbx(Tnom=Tnom) 
                             annotation (extent=[-50,40; -10,60]);
          Utilities.Rce Rc(
            Tnom=Tnom,
            R=RC,
            TR1=TRC1,
            TR2=TRC2,
            Area=Area,
            Level=Level) annotation (extent=[40,40; 80,60]);
          Utilities.Rce Re(
            Tnom=Tnom,
            Area=Area,
            Level=Level,
            R=RE,
            TR1=TRE1,
            TR2=TRE2) annotation (extent=[40,-50; 80,-30]);
          Utilities.Cj Dsb(
            Tnom=Tnom,
            EG=EG,
            XTI=XTI,
            MJ=MJS,
            VJ=VJS,
            XCJ=1,
            Tau=0,
            GminDC=0,
            Area=Area,
            Level=Level,
            BE=false,
            EMin=EMin,
            EMax=EMax,
            IS=0,
            CJ=CJS,
            FC=0,
            N=1,
            enforceStates=enforceStates) 
                       annotation (extent=[-10,-30; -30,-70], rotation=-90);
          Interfaces.SPin S "Substrate (may remain unconnected)" 
                            annotation (extent=[-110,-60; -90,-40]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
          Ideal.IdealCommutingSwitch2 Sw2 
            annotation (extent=[-90,-90; -110,-70], rotation=90);
          RampingSources.V Vsub(V=VSUB, rampTime=rampTime) 
            annotation (extent=[-90,-60; -70,-40], rotation=90);
          Basic.Ground grd annotation (extent=[-90,-16; -70,-34]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          
          if cardinality(S) <= 1 then
            Sw2.control = false;
          else
            Sw2.control = true;
          end if;
          connect(Rb.p, B)  annotation (points=[-80,0; -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Qpi.Bint, Rb.n)   annotation (points=[-20,0; -40,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={188,0,0},
              fillPattern=1));
          connect(Qpi.qb, Rb.qb)   annotation (points=[22,0; 30,0; 30,30; -60,
                30; -60,10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=42,
              rgbfillColor={188,0,0},
              fillPattern=1));
          connect(Qpi.heatPort, Rb.heatPort)   annotation (points=[0,-20; -60,
                -20; -60,-10], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={188,0,0},
              fillPattern=1));
          connect(Cbx.p, Rb.p)   annotation (points=[-50,50; -80,50; -80,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Qpi.heatPort, Cbx.heatPort)   annotation (points=[0,-20; -30,
                -20; -30,40], style(color=42, rgbcolor={191,0,0}));
          connect(Qpi.ix, Cbx.ix)   annotation (points=[-10,22; -10,40; 0,40; 0,
                70; -30,70; -30,60], style(color=74, rgbcolor={0,0,127}));
          connect(Rc.n, C) annotation (points=[80,50; 100,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rc.p, Cbx.n) annotation (points=[40,50; -10,50], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Rc.heatPort, Qpi.heatPort) annotation (points=[60,40; 60,-20;
                0,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Re.n, E) annotation (points=[80,-40; 90,-40; 90,-50; 100,-50],
                                                                 style(color=3,
                rgbcolor={0,0,255}));
          connect(Re.p, Qpi.Eint) annotation (points=[40,-40; 20,-40; 20,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(C0.y, Dsb.C0) annotation (points=[-43.4,-60; -30,-60], style(
                color=74, rgbcolor={0,0,127}));
          connect(qb.y, Dsb.qb) annotation (points=[-43.4,-40; -30,-40], style(
                color=74, rgbcolor={0,0,127}));
          connect(Dsb.n, Qpi.Bint) annotation (points=[-20,-30; -20,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(Qpi.Cint, Rc.p) annotation (points=[20,10; 20,50; 40,50],
              style(color=3, rgbcolor={0,0,255}));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-81; 12,-81; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-87.5; 0,-87.5; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Dsb.heatPort, Sw1.p) annotation (points=[-10,-50;
                -6.73533e-016,-50; -6.73533e-016,-59], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.p, Re.heatPort) annotation (points=[-6.73533e-016,-59;
                -6.73533e-016,-50; 60,-50], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.p, Qpi.heatPort) annotation (points=[-6.73533e-016,-59;
                -6.73533e-016,-39.5; 0,-39.5; 0,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw2.n2, S) annotation (points=[-100,-70; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vsub.p, Sw2.n1) annotation (points=[-80,-60; -95,-60; -95,-70],
              style(color=3, rgbcolor={0,0,255}));
          connect(Vsub.n, grd.p) annotation (points=[-80,-40; -80,-34], style(
                color=3, rgbcolor={0,0,255}));
          connect(Sw2.p, Dsb.p) annotation (points=[-100,-90; -100,-100; -20,
                -100; -20,-70], style(color=3, rgbcolor={0,0,255}));
        end Qpl;
        
        model Qpv "Spice-style vertically diffused PNP bipolar transistor" 
          constant Real inf=Modelica.Constants.inf;
          
          /* General parameters */
          parameter Integer Level(min=1,max=2) = 2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Real Area=1 "Relative area occupied by device";
          
          /* DC model parameters */
          parameter Real BF=100 
            "|DC model parameters|Maximum forward current gain at reference temperature";
          parameter Real BR=1 
            "|DC model parameters|Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
            "|DC model parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
            "|DC model parameters|Saturation current used for current injection";
          parameter Real NF=1 
            "|DC model parameters|Forward current emission coefficient";
          parameter Real NR=1 
            "|DC model parameters|Reverse current emission coefficient";
          parameter Modelica.SIunits.Voltage VSUB=0 
            "|DC model parameters|Substrate potential (if unconnected)";
          
          /* Low current beta degradation effect parameters */
          parameter Real C4(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
          parameter Modelica.SIunits.Current ISC(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Real C2(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
          parameter Modelica.SIunits.Current ISE(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
            "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
            "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
          
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=inf 
            "|High current beta degradation effect|Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=inf 
            "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
          
          /* Base width modulation parameters */
          parameter Modelica.SIunits.Voltage VAF=inf 
            "|Base width modulation parameters|Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=inf 
            "|Base width modulation parameters|Reverse early voltage";
          
          /* Parasitic Resistor Parameters */
          parameter Modelica.SIunits.Current IRB=inf 
            "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
          parameter Modelica.SIunits.Resistance RB(min=0) = 0 
            "|Parasitic resistor parameters|Zero-bias base resistance";
          parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
            "|Parasitic resistor parameters|Minimum base resistance at high currents";
          parameter Modelica.SIunits.Resistance RC(min=0) = 0 
            "|Parasitic resistor parameters|Collector resistance";
          parameter Modelica.SIunits.Resistance RE(min=0) = 0 
            "|Parasitic resistor parameters|Emitter resistance";
          
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
          parameter Real MJC(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-collector junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJC=0.75 
            "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
          parameter Real MJE(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-emitter junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJE=0.75 
            "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
          parameter Real MJS(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Substrate junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJS=0.75 
            "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
          parameter Real XCJC(min=0,max=1) = 1 
            "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
          parameter Real FC(min=0,max=1) = 0.5 
            "|Junction capacitor parameters|Depletion capacitance factor for linearization";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TF(min=0) = 0 
            "|Transit time parameters|Ideal forward transit time";
          parameter Modelica.SIunits.Time TR(min=0) = 0 
            "|Transit time parameters|Ideal reverse transit time";
          
          /* Temperature compensation and area parameters */
          parameter Real TRB1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
          parameter Real TRB2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
          parameter Real TRM1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
          parameter Real TRM2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
          parameter Real TRC1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
          parameter Real TRC2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
          parameter Real TRE1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
          parameter Real TRE2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI=3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Real XTB=0 
            "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.11 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
          
          /* Initialization parameters */
          parameter Modelica.SIunits.Time rampTime=0 
            "|Advanced|Initialization parameters|Substrate potential ramping time (if unconnected)";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
        protected 
          parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
          parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Utilities.Rb Rb(
            Tnom=Tnom,
            RB=RB,
            RBM=RBM,
            IRB=IRB,
            TRB1=TRB1,
            TRB2=TRB2,
            TRM1=TRM1,
            TRM2=TRM2,
            Area=Area,
            Level=Level) 
                       annotation (extent=[-88,-10; -48,10]);
        Modelica.Blocks.Sources.Constant C0(k=0) 
          annotation (extent=[80,14; 68,26]);
        Modelica.Blocks.Sources.Constant qb(k=1) 
          annotation (extent=[80,-6; 68,6]);
          annotation (
            Diagram(Text(
                extent=[2,8; 12,0],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"), Text(
                extent=[-68,20; -58,12],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="qb"),
                    Text(
                extent=[-24,42; -14,32],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
                    Text(
                extent=[-42,72; -32,62],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="ix"),
        Rectangle(extent=[20,-60; 40,-80],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-60; 40,-80],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-70; 20,-70],   style(color=83, rgbcolor={255,85,
                      255})),
        Rectangle(extent=[-60,-70; -40,-90],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-60,-70; -40,-90],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-50,-70; -50,-64], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[-10,-10; 5,-17; -3,-25; -10,-10],     style(fillColor=
                      3, fillPattern=1)),
              Text(extent=[-100,64; 100,100],   string="%name"),
              Line(points=[0,-40; 0,-91],   style(color=42)),
            Text(
              extent=[24,30; 108,-34],
              string="%Area x",
              style(color=3, rgbcolor={0,0,255}))),
            Documentation(info="<html>
The <b>Qpv</b> element of the Spice analog electrical library implements a full-fledges Spice-stye Gummel-Poon model of the vertically diffused PNP bipolar transistor [1-3]. The model contains a <b>Qpi</b> model implementing the inner parts of the PNP transistor up to the internal nodes. <p>

The substrate node is an electrical node, but we decided to paint it green so that the substrate connections wouldn't clutter up the circuit diagram unnecessarily. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

The substrate port may remain unconnected, in which case the substrate is assumed to be at a fixed potential of <b>VSUB</b>. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 VSUB:    Substrate potential if unconnected (default value = 0 V) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Initialization Parameters:</b>

 rampTime:Substrate potential ramping time if unconnected (default value = 0 s) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 1,2</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
          Utilities.Qpi Qpi(
            BF=BF,
            BR=BR,
            IS=IS,
            NF=NF,
            NR=NR,
            GminDC=GminDC,
            NC=NC,
            NE=NE,
            VAF=VAF,
            VAR=VAR,
            IKF=IKF,
            IKR=IKR,
            CJC=CJC,
            MJC=MJC,
            VJC=VJC,
            CJE=CJE,
            MJE=MJE,
            VJE=VJE,
            XCJC=XCJC,
            TF=TF,
            TR=TR,
            Tnom=Tnom,
            XTI=XTI,
            XTB=XTB,
            EG=EG,
            Area=Area,
            EMin=EMin,
            EMax=EMax,
            FC=FC,
            Level=Level,
            ISC=ISCeff,
            ISE=ISEeff,
            ISS=ISS,
            enforceStates=enforceStates) 
                             annotation (extent=[-40,-20; 0,20]);
          Utilities.Cbx Cbx(Tnom=Tnom) 
                             annotation (extent=[-54,40; -14,60]);
          Utilities.Rce Rc(
            Tnom=Tnom,
            R=RC,
            TR1=TRC1,
            TR2=TRC2,
            Area=Area,
            Level=Level) annotation (extent=[40,40; 80,60]);
          Utilities.Rce Re(
            Tnom=Tnom,
            Area=Area,
            Level=Level,
            R=RE,
            TR1=TRE1,
            TR2=TRE2) annotation (extent=[40,-50; 80,-30]);
          Utilities.Cj Dbs(
            Tnom=Tnom,
            EG=EG,
            XTI=XTI,
            MJ=MJS,
            VJ=VJS,
            XCJ=1,
            Tau=0,
            GminDC=0,
            Area=Area,
            Level=Level,
            BE=false,
            EMin=EMin,
            EMax=EMax,
            IS=0,
            CJ=CJS,
            FC=0,
            N=1,
            enforceStates=enforceStates) 
                       annotation (extent=[30,-10; 50,30], rotation=-90);
          Interfaces.SPin S "Substrate (may remain unconnected)" 
                            annotation (extent=[-110,-60; -90,-40]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
          Basic.Ground grd annotation (extent=[-90,-102; -70,-84]);
          RampingSources.V Vsub(V=VSUB, rampTime=rampTime) 
            annotation (extent=[-90,-80; -70,-60], rotation=270);
          Ideal.IdealCommutingSwitch2 Sw2 
            annotation (extent=[-40,-40; -60,-60], rotation=0);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          
          if cardinality(S) <= 1 then
            Sw2.control = false;
          else
            Sw2.control = true;
          end if;
          connect(Rb.p, B)  annotation (points=[-88,0; -100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Qpi.Bint, Rb.n)   annotation (points=[-40,0; -48,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={188,0,0},
              fillPattern=1));
          connect(Qpi.qb, Rb.qb)   annotation (points=[2,0; 12,0; 12,30; -68,30;
                -68,10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=42,
              rgbfillColor={188,0,0},
              fillPattern=1));
          connect(Rb.heatPort, Qpi.heatPort)   annotation (points=[-68,-10; -68,
                -20; -20,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={188,0,0},
              fillPattern=1));
          connect(Cbx.p, Rb.p)   annotation (points=[-54,50; -88,50; -88,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Qpi.heatPort, Cbx.heatPort)   annotation (points=[-20,-20;
                -34,-20; -34,40], style(color=42, rgbcolor={191,0,0}));
          connect(Qpi.ix, Cbx.ix)   annotation (points=[-30,22; -30,34; -24,34;
                -24,66; -34,66; -34,60], style(color=74, rgbcolor={0,0,127}));
          connect(Rc.n, C) annotation (points=[80,50; 100,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rc.p, Cbx.n) annotation (points=[40,50; -14,50], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Re.n, E) annotation (points=[80,-40; 90,-40; 90,-50; 100,-50],
                                                                 style(color=3,
                rgbcolor={0,0,255}));
          connect(Re.p, Qpi.Eint) annotation (points=[40,-40; 0,-40; 0,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(Dbs.p, Rc.p) annotation (points=[40,30; 40,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(C0.y, Dbs.C0) annotation (points=[67.4,20; 50,20], style(
                color=74, rgbcolor={0,0,127}));
          connect(qb.y, Dbs.qb) annotation (points=[67.4,0; 58.7,0; 58.7,
                1.77636e-015; 50,1.77636e-015], style(color=74, rgbcolor={0,0,
                  127}));
          connect(Rc.heatPort, Dbs.heatPort) annotation (points=[60,40; 22,40;
                22,10; 30,10], style(color=42, rgbcolor={191,0,0}));
          connect(Dbs.heatPort, Re.heatPort) annotation (points=[30,10; 22,10;
                22,-50; 60,-50], style(color=42, rgbcolor={191,0,0}));
          connect(Qpi.Cint, Rc.p) annotation (points=[0,10; 0,50; 40,50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Re.heatPort, Qpi.heatPort) annotation (points=[60,-50; -20,
                -50; -20,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-81; 12,-81; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-85.5; 0,-85.5; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.p, Re.heatPort) annotation (points=[-6.73533e-016,-59;
                -6.73533e-016,-50; 60,-50], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw2.n2, S) annotation (points=[-60,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vsub.p, Sw2.n1) annotation (points=[-80,-60; -80,-55; -60,-55],
              style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.p, Dbs.n) annotation (points=[-40,-50; -30,-50; -30,-30;
                40,-30; 40,-10], style(color=3, rgbcolor={0,0,255}));
          connect(Vsub.n, grd.p) annotation (points=[-80,-80; -80,-84], style(
                color=3, rgbcolor={0,0,255}));
        end Qpv;
        
        model Mn "Spice-style NMOS transistor" 
          constant Real pi=Modelica.Constants.pi;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
            "Boltzmann's constant";
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
            "Permittivity of free space";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
            "Electron charge";
          constant Modelica.SIunits.ElectronNumberDensity ni=1.45e16 
            "Intrinsic carrier concentration at 300 K";
          constant Real kox=3.9 "Dielectric constant of SiO2";
          constant Real ks=11.8 "Dielectric constant of Si";
          constant Real GapC1=7.02E-4 
            "Sze's first bandgap correction factor Silicon";
          constant Real GapC2=1108.0 
            "Sze's second bandgap correction factor Silicon";
          constant Modelica.SIunits.Voltage Fn0=3.25 
            "Work function constant for computation of flat band voltage";
          constant Real TOXdef = 1e-7 "Default oxide thickness";
          
          /* General parameters */
          parameter Integer Level(min=0,max=4) = 3 
            "Level of MOS model (check documentation window for details)";
          
          /* Basic Geometric parameters */
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 
            "|Geometric parameters|Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 
            "|Geometric parameters|Channel width";
          
          /* Basic Process parameters */
          parameter Real TPG=1 "|Process parameters|Type of gate material";
          parameter Modelica.SIunits.Length TOX=TOXdef 
            "|Process parameters|Thin oxide thickness";
          parameter Real COX(unit="F/m2") = 0 
            "|Process parameters|Specific capacitance of SiO2";
          parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
            "|Process parameters|Substrate doping";
          
          /* Basic Electrical parameters */
          parameter Real U0(unit="m2/(V.s)") = 0 
            "|Electrical parameters|Surface mobility at reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
            "|Electrical parameters|Zero-bias threshold voltage";
          parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
            "|Electrical parameters|Channel length modulation";
          parameter Modelica.SIunits.Length LD(min=0) = 0 
            "|Electrical parameters|Lateral diffusion";
          parameter Modelica.SIunits.Length WD(min=0) = 0 
            "|Electrical parameters|Width diffusion";
          parameter Modelica.SIunits.Transconductance KP=0 
            "|Electrical parameters|Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 
            "|Electrical parameters|Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
            "|Electrical parameters|Surface inversion potential at reference temperature";
          parameter Real NSS(unit="1/m2") = 0 
            "|Electrical parameters|Surface state density";
          
          /* Basic temperature compensation parameters */
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Modelica.SIunits.Voltage EG=1.16 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
          
          /* Level=1 electrical parameters */
          parameter Modelica.SIunits.Area AD=0 
            "|Level=1|Electrical parameters|Drain diffusion area";
          parameter Modelica.SIunits.Length PD=0 
            "|Level=1|Electrical parameters|Drain perimeter width";
          parameter Modelica.SIunits.Area AS=0 
            "|Level=1|Electrical parameters|Source diffusion area";
          parameter Modelica.SIunits.Length PS=0 
            "|Level=1|Electrical parameters|Source perimeter width";
          parameter Modelica.SIunits.Current ISD(min=0) = 0 
            "|Level=1|Electrical parameters|Drain junction saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS(min=0) = 0 
            "|Level=1|Electrical parameters|Source junction saturation current at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JS(min=0) = 0 
            "|Level=1|Electrical parameters|Saturation current density at reference temperature";
          parameter Modelica.SIunits.Voltage PB(min=0.2) = 0.8 
            "|Level=1|Electrical parameters|Built-in potential at reference temperature";
          
          /* Level=1 parasitic resistance parameters */
          parameter Modelica.SIunits.Resistance RD(min=0) = 1 
            "|Level=1|Parasitic resistance parameters|Drain Ohmic resistance";
          parameter Modelica.SIunits.Resistance RS(min=0) = 1 
            "|Level=1|Parasitic resistance parameters|Source Ohmic resistance";
          
          /* Level=1 junction capacitance parameters */
          parameter Real CJ(unit="F/m2",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real MJ(min=0.3,max=0.5) = 0.33 
            "|Level=1|Junction capacitance parameters|Bulk junction grading coefficient";
          parameter Real CJSW(unit="F/m",min=0) = 1e-9 
            "|Level=1|Junction capacitance parameters|Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Real MJSW(min=0.3,max=0.5) = 0.33 
            "|Level=1|Junction capacitance parameters|Perimeter capacitance grading coefficient";
          parameter Real FC(min=0,max=0.95)= 0.5 
            "|Level=1|Junction capacitance parameters|Forward-bias depletion capacitance coefficient";
          parameter Real CGB0(unit="F/m",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Gate-bulk overlap capacitance per meter";
          parameter Real CGD0(unit="F/m",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Gate-drain overlap capacitance per meter";
          parameter Real CGS0(unit="F/m",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Gate-source overlap capacitance per meter";
          
          /* Level=1 temperature compensation parameters */
          parameter Real TRD1(unit="Ohm/degree") = 0 
            "|Level=1|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
          parameter Real TRD2(unit="Ohm/degree2") = 0 
            "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of drain resistance";
          parameter Real TRS1(unit="Ohm/degree") = 0 
            "|Level=1|Temperature compensation parameters|Linear temperature coefficient of source resistance";
          parameter Real TRS2(unit="Ohm/degree2") = 0 
            "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of source resistance";
          
          /* Level=2 process parameters */
          parameter Modelica.SIunits.Length XJ=0 
            "|Level=2|Process parameters|Metallurgical junction depth";
          
          /* Level=2 electrical parameters */
          parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
            "|Level=2|Electrical parameters|Critical electric field for mobility";
          parameter Real UEXP=0 
            "|Level=2|Electrical parameters|Exponential coefficient for mobility";
          parameter Real UTRA(min=0,max=0.5) = 0 
            "|Level=2|Electrical parameters|Transverse field coefficient";
          parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
            "|Level=2|Electrical parameters|Critical electric field for pinch-off";
          parameter Real DELTA=0 
            "|Level=2|Electrical parameters|Width effect on threshold voltage";
          parameter Modelica.SIunits.Velocity VMAX=0 
            "|Level=2|Electrical parameters|Maximum drift velocity of carriers";
          parameter Real NFS(unit="1/m2") = 0 
            "|Level=2|Electrical parameters|Surface fast state density";
          parameter Real NEFF(min=Modelica.Constants.small) = 1 
            "|Level=2|Electrical parameters|Total channel charge coefficient";
          parameter Real XQC=0 
            "|Level=2|Electrical parameters|Coefficient of channel charge share";
          
          /* Level=3 electrical parameters */
          parameter Real KAPPA(min=Modelica.Constants.small) = 0.2 
            "|Level=3|Electrical parameters|Saturation field factor";
          parameter Real ETA=0 
            "|Level=3|Electrical parameters|Static feedback on threshold voltage";
          parameter Modelica.SIunits.InversePotential THETA=0 
            "|Level=3|Electrical parameters|Mobility modulation";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC=1e-12 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
          /* Implement Spice-style MODCHK */
        protected 
          parameter Integer Type = 1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Permittivity es = ks*e0 
            "Permittivity of silicon";
          parameter Modelica.SIunits.Permittivity eox = kox*e0 
            "Permittivity of SiO2";
          parameter Modelica.SIunits.Length LDeff = LD;
          parameter Modelica.SIunits.Length Leff = L - 2*LDeff;
          parameter Modelica.SIunits.Length Weff = W - 2*WD;
          parameter Modelica.SIunits.Length TOXeff = if TOX == TOXdef and COX > 0 then eox/COX else TOX;
          parameter Real COXeff(unit="F/m2") = if COX > 0 then COX else eox/TOX;
          parameter Modelica.SIunits.Capacitance CGB0eff = CGB0;
          parameter Modelica.SIunits.Capacitance CGD0eff = CGD0;
          parameter Modelica.SIunits.Capacitance CGS0eff = CGS0;
          parameter Real U0def(unit="m2/(V.s)") = 0.06;
          parameter Real U0tmp(unit="m2/(V.s)") = if KP == 0 and U0 == 0 then U0def else U0;
          parameter Modelica.SIunits.Transconductance KPeff = if KP > 0 then KP else U0tmp*COXeff;
          parameter Real U0eff(unit="m2/(V.s)") = if KP > 0 and U0 == 0 then KP/COXeff else U0tmp;
          parameter Real GAMMAtmp(unit="V^0.5") = if GAMMA == 0 and NSUB == 0 then 0.35 else GAMMA;
          parameter Modelica.SIunits.ElectronNumberDensity NSUBeff = if NSUB > 0 then NSUB else (GAMMAtmp*COXeff)^2/(2*es*q);
          parameter Real GAMMAeff(unit="V^0.5") = if GAMMA == 0 and NSUB > 0 then sqrt(2*es*q*NSUB)/COXeff else GAMMAtmp;
          parameter Modelica.SIunits.InversePotential LAMBDAeff = if LAMBDA > 0 then LAMBDA else 
                                                                  if Level <> 2 then sqrt(2*es/(q*NSUBeff)) else LAMBDA;
          parameter Modelica.SIunits.Voltage EGref = EG - GapC1*Tnom*Tnom/(Tnom + GapC2) 
            "Energy gap for temperature effect on saturation current at reference temperature";
          parameter Modelica.SIunits.ElectronNumberDensity XNI = ni*(Tnom/300)^1.5 * exp(EGref*q/(2*k)*(1/300 - 1/Tnom)) 
            "Intrinsic carrier concentration at reference temperature";
          parameter Modelica.SIunits.Voltage VTref = k*Tnom/q 
            "Thermal voltage at reference temperature";
          parameter Modelica.SIunits.Voltage PHIeff = if PHI > 0 then PHI else if NSUB > 0 then max({2*VTref*ln(NSUB/XNI),0.1}) else 0.6;
          parameter Modelica.SIunits.Voltage FermiG = Type*TPG*EGref/2;
          parameter Modelica.SIunits.Voltage Fn1 = if TPG == 0 then 3.2 else Fn0 + EGref/2 - FermiG;
          parameter Modelica.SIunits.Voltage FermiS = Type*PHIeff/2;
          parameter Modelica.SIunits.Voltage Fn2 = Fn1 - Fn0 - EGref/2 - FermiS;
          parameter Modelica.SIunits.Voltage VFB = Fn2 - NSS*q/COXeff;
          parameter Modelica.SIunits.Voltage Vcorr = Type*(GAMMAeff*sqrt(PHIeff) + PHIeff);
          parameter Modelica.SIunits.Voltage VT0eff = if Type*VT0 > 0 then VT0 else VFB + Vcorr;
          parameter Modelica.SIunits.Voltage VFBeff = if Type*VT0 > 0 then VT0 - Vcorr else VFB;
          parameter Real XD(unit="m/V^0.5") = sqrt(2*es/(q*NSUBeff)) 
            "Depletion factor";
          parameter Real CJeff(unit="F/m2") = if CJ > 0 then CJ else sqrt(es*q*NSUBeff/(2*PB));
          parameter Modelica.SIunits.Voltage EGeff = if Level == 3 and EG == 0 then 0.2 else EGref;
          parameter Modelica.SIunits.Voltage VBP = UCRIT*es/COXeff 
            "Gate to channel critical voltage";
          parameter Modelica.SIunits.Current ISDeff = if ISD > 0 then ISD else if AD*JS > 0 then AD*JS else 1e-15;
          parameter Modelica.SIunits.Current ISSeff = if ISS > 0 then ISS else if AS*JS > 0 then AS*JS else 1e-15;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
            annotation (extent=[90,40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
            annotation (extent=[-90,-40; -110,-60]);
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
            annotation (extent=[90,-40; 110,-60]);
          Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" 
            annotation (extent=[90,-10; 110,10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          annotation (Icon(
              Text(extent=[-100,80; 100,100],   string="%name"),
              Line(points=[-90,-50; -10,-50]),
              Line(points=[-10,-50; -10,50]),
              Line(points=[10,70; 10,29]),
              Line(points=[10,20; 10,-21]),
              Line(points=[10,-30; 10,-71]),
              Line(points=[10,50; 91,50]),
              Line(points=[10,0; 90,0]),
              Line(points=[10,-50; 90,-50]),
              Polygon(points=[40,0; 60,5; 60,-5; 40,0],     style(fillColor=3,
                    fillPattern=1)),
              Line(points=[0,-91; 0,-50],   style(color=42))), Diagram(
              Ellipse(extent=[-2,-58; 2,-62], style(
                  color=42,
                  rgbcolor={179,0,0},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1)),
              Text(
                extent=[-54,-34; -40,-40],
                string="vbs",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1)),
              Text(
                extent=[-54,68; -40,62],
                string="vbs",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1)),
              Text(
                extent=[-16,94; -2,88],
                string="vbs",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1)),
              Text(
                extent=[20,96; 34,90],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vds"),
              Text(
                extent=[-30,68; -16,62],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vds"),
              Text(
                extent=[-30,-34; -16,-40],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vds"),
              Ellipse(extent=[78,2; 82,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
        Rectangle(extent=[20,-66; 40,-86],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-66; 40,-86],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-76; 20,-76],   style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
The Spice-style n-channel MOSFET model contains the internal MOSFET plus the three external gate capacitances and the drain and source resistances. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model</b></font>


<b>Basic Geometric Parameters:</b>

 L:       Channel length (default value = 1e-4 m) <font color=red>Levels 0-4</font>

 W:       Channel width (default value = 1e-4 m) <font color=red>Levels 0-4</font>


<b>Basic Process Parameters:</b>

 TPG:     Type of gate material (default value = 1) <font color=red>Levels 0-4</font>
            <font color=blue><b>TPG = 0:  Aluminum gate</b></font>
            <font color=blue><b>TPG = 1:  Polysilicon gate</b></font>

 TOX:     Thin oxide thickness (default value = 1e-7 m) <font color=red>Levels 0-4</font>

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 0-4</font>

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>) <font color=red>Levels 0-4</font>


<b>Basic Electrical Parameters:</b>

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s)) <font color=red>Levels 0-4</font>

 VT0:     Zero-bias threshold voltage (default value = 0 Volt) <font color=red>Levels 0,1</font>
            <font color=blue><b>VT0 &gt; 0 for enhancement MOSFET
            VT0 &lt; 0 for depletion MOSFET</b></font>

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt) <font color=red>Levels 0-4</font>

 LD:      Lateral diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 WD:      Width diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 0-4</font>

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>) <font color=red>Levels 0-4</font>

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt) <font color=red>Levels 0-4</font>

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 0-4</font>


<b>Basic Temperature Compensation Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 0-4</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt) <font color=red>Levels 0-4</font>


<b>Level=1 Electrical Parameters:</b>

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PD:      Drain perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PS:      Source perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 JS:      Transport saturation current density (default value = 0 Amp/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-4</font>


<b>Level=1 Parasitic Resistor Parameters:</b>

 RD:      Drain Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>

 RS:      Source Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>


<b>Level=1 Junction Capacitance Parameters:</b>

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 MJ:      Bulk junction grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m) <font color=red>Levels 1-4</font>

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1-4</font>

 CGB0:    Gate-bulk overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGD0:    Gate-drain overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGS0:    Gate-source overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>


<b>Level=1 Temperature Compensation Parameters:</b>

 TRD1:    Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRD2:    Quadratic temperature coefficient of drain resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>

 TRS1:    Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRS2:    Quadratic temperature coefficient of source resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>


<b>Level=2 Process Parameters:</b>

 XJ:      Metallurgical Junction Depth (default value = 0 m) <font color=red>Levels 2-4</font>


<b>Level=2 Electrical Parameters:</b>

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m) <font color=red>Levels 2,4</font>

 UEXP:    Exponential coefficient for mobility (default value = 0) <font color=red>Levels 2,4</font>

 UTRA:    Transverse field coefficient (default value = 0) <font color=red>Levels 2,4</font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,4</font>

 DELTA:   Width effect on threshold voltage (default value = 0) <font color=red>Levels 2-4</font>


 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s) <font color=red>Levels 2-4</font>
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation (Levels 2,4)</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation (Levels 2,4)</b></font>

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 2-4</font>

 NEFF:    Total channel charge coefficient (default value = 1) <font color=red>Levels 2,4</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2-4</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>


<b>Level=3 Electrical Parameters:</b>

 KAPPA:   Saturation field factor (default value = 0.2) <font color=red>Level 3</font>

 ETA:     Static feedback on threshold voltage (default value = 0) <font color=red>Level 3</font>

 THETA:   Mobility modulation (default value = 0 1/Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-4</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-4</font>

 GminDC:  Leakage conductance (default value = 1e-12 mho) <font color=red>Levels 0-4</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 0-4</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
          Utilities.Mni Mni(
            Tnom=Tnom,
            PHI=PHIeff,
            NSS=NSS,
            NFS=NFS,
            NEFF=NEFF,
            TPG=TPG,
            UEXP=UEXP,
            UTRA=UTRA,
            VMAX=VMAX,
            THETA=THETA,
            PB=PB,
            MJ=MJ,
            MJSW=MJSW,
            FC=FC,
            GminDC=GminDC,
            Level=Level,
            EMin=EMin,
            EMax=EMax,
            GAMMA=GAMMAeff,
            KP=KPeff,
            ISD=ISDeff,
            ISS=ISSeff,
            VT0=VT0eff,
            LAMBDA=LAMBDAeff,
            TOX=TOXeff,
            NSUB=NSUBeff,
            LD=LDeff,
            U0=U0eff,
            CJ=CJeff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            COX=COXeff,
            VFB=VFBeff,
            XJ=XJ,
            AD=AD,
            PD=PD,
            AS=AS,
            PS=PS,
            CJSW=CJSW,
            XD=XD,
            UCRIT=UCRIT,
            VBP=VBP,
            XQC=XQC,
            ECRIT=ECRIT,
            Type=1,
            KAPPA=KAPPA,
            DELTA=DELTA,
            ETA=ETA,
            enforceStates=enforceStates) 
                             annotation (extent=[-20,-20; 20,20]);
          Utilities.Cgs Cgs1(
            Tnom=Tnom,
            Level=Level,
            PHI=PHIeff,
            GAMMA=GAMMAeff,
            LD=LDeff,
            CGS0=CGS0eff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            XJ=XJ,
            DELTA=DELTA,
            ETA=ETA,
            VFB=VFBeff,
            KP=KPeff,
            COX=COXeff,
            NFS=NFS,
            XD=XD,
            XQC=XQC,
            Type=1,
            enforceStates=enforceStates,
            u0=0)      annotation (extent=[-50,-60; -10,-40]);
          Utilities.Cgd Cgd1(
            Tnom=Tnom,
            PHI=PHIeff,
            GAMMA=GAMMAeff,
            LD=LDeff,
            CGD0=CGD0eff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            XJ=XJ,
            XQC=XQC,
            DELTA=DELTA,
            ETA=ETA,
            VFB=VFBeff,
            KP=KPeff,
            COX=COXeff,
            NFS=NFS,
            XD=XD,
            Level=Level,
            Type=1,
            enforceStates=enforceStates,
            u0=0)      annotation (extent=[-50,40; -10,60]);
          Utilities.Cgb Cgb1(
            Tnom=Tnom,
            PHI=PHIeff,
            GAMMA=GAMMAeff,
            LD=LDeff,
            CGB0=CGB0eff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            XJ=XJ,
            XQC=XQC,
            DELTA=DELTA,
            ETA=ETA,
            VFB=VFBeff,
            KP=KPeff,
            COX=COXeff,
            NFS=NFS,
            XD=XD,
            Level=Level,
            Type=1,
            enforceStates=enforceStates,
            u0=0)      annotation (extent=[-10,70; 30,90]);
          Sensors.VoltageSensor Vds 
            annotation (extent=[128,-8; 108,12], rotation=-90);
          Sensors.VoltageSensor Vbs 
            annotation (extent=[90,-14; 70,-34], rotation=90);
          Utilities.RM Rd(
            Tnom=Tnom,
            R=RD,
            TR1=TRD1,
            TR2=TRD2,
            Level=Level) annotation (extent=[30,40; 70,60]);
          Utilities.RM Rs(
            Tnom=Tnom,
            Level=Level,
            R=RS,
            TR1=TRS1,
            TR2=TRS2) annotation (extent=[28,-60; 68,-40]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-86; 10,-66], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
        algorithm 
          assert(NSUBeff > XNI, "NSUB must be larger than intrinsic carrier concentration");
          assert(LAMBDAeff < 0.2, "LAMBDA value too large, leads to numerical instability");
          assert(Leff > 0, "The effective channel length must be larger than zero");
          
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(Mni.B, B)  annotation (points=[20,0; 100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(G, Mni.G)  annotation (points=[-100,-50; -60,-50; -60,-10;
                -20,-10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgs1.p, G) annotation (points=[-50,-50; -100,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgd1.p, G) annotation (points=[-50,50; -60,50; -60,-50; -100,
                -50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgb1.p, Cgd1.p) annotation (points=[-10,80; -60,80; -60,50;
                -50,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgb1.n, B) annotation (points=[30,80; 80,80; 80,0; 100,0],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Vds.p, D) annotation (points=[118,12; 118,50; 100,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(S, Vds.n) annotation (points=[100,-50; 118,-50; 118,-8],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Vds.V, Cgb1.vds) annotation (points=[128,2; 132,2; 132,96; 22,
                96; 22,90], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgb1.vds, Cgd1.vds) annotation (points=[22,90; 22,96; -18,96;
                -18,60], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgd1.vds, Cgs1.vds) annotation (points=[-18,60; -18,70; -14,
                70; -14,-28; -18,-28; -18,-40], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Vbs.n, S) annotation (points=[80,-34; 80,-50; 100,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Vbs.p, B) annotation (points=[80,-14; 80,0; 100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Vbs.V, Cgs1.vbs) annotation (points=[70,-24; -42,-24; -42,-40],
              style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgs1.vbs, Cgd1.vbs) annotation (points=[-42,-40; -42,26; -56,
                26; -56,70; -42,70; -42,60], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Cgd1.vbs, Cgb1.vbs) annotation (points=[-42,60; -42,94; -2,94;
                -2,90], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.n, D) annotation (points=[70,50; 100,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rd.p, Cgd1.n) annotation (points=[30,50; -10,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Rd.p, Mni.D)  annotation (points=[30,50; 20,50; 20,10], style(
                color=3, rgbcolor={0,0,255}));
          connect(Rs.n, S) annotation (points=[68,-50; 100,-50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rs.p, Cgs1.n) annotation (points=[28,-50; -10,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Rs.p, Mni.S)  annotation (points=[28,-50; 20,-50; 20,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(Rd.heatPort, Mni.heatPort)  annotation (points=[50,40; 50,-20;
                0,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Rd.heatPort, Cgd1.heatPort) annotation (points=[50,40; -30,40],
              style(color=42, rgbcolor={191,0,0}));
          connect(Cgb1.heatPort, Rd.heatPort) annotation (points=[10,70; 10,40;
                50,40], style(color=42, rgbcolor={191,0,0}));
          connect(Mni.y1, Cgb1.u)  annotation (points=[10,22; 10,32; -2,32; -2,
                70], style(color=74, rgbcolor={0,0,127}));
          connect(Mni.y1, Cgd1.u)  annotation (points=[10,22; 10,32; -42,32;
                -42,40], style(color=74, rgbcolor={0,0,127}));
          connect(Mni.y1, Cgs1.u)  annotation (points=[10,22; 10,32; -70,32;
                -70,-70; -42,-70; -42,-60], style(color=74, rgbcolor={0,0,127}));
          connect(Cgs1.heatPort, Mni.heatPort) annotation (points=[-30,-60; 0,-60; 0,
                -20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Mni.heatPort, Rs.heatPort) annotation (points=[0,-20; 0,-60;
                48,-60], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-87; 12,-87; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-87;
                6.73533e-016,-89.5; 0,-89.5; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.p, Cgs1.heatPort) annotation (points=[-6.73533e-016,-65;
                -6.73533e-016,-60; -30,-60], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
        end Mn;
        
        model Mp "Spice-style PMOS transistor" 
          constant Real pi=Modelica.Constants.pi;
          constant Modelica.SIunits.Entropy k=Modelica.Constants.k 
            "Boltzmann's constant";
          constant Modelica.SIunits.Permittivity e0=Modelica.Constants.epsilon_0 
            "Permittivity of free space";
          constant Modelica.SIunits.ElectricCharge q=1.6021892e-19 
            "Electron charge";
          constant Modelica.SIunits.ElectronNumberDensity ni=1.45e16 
            "Intrinsic carrier concentration at 300 K";
          constant Real kox=3.9 "Dielectric constant of SiO2";
          constant Real ks=11.8 "Dielectric constant of Si";
          constant Real GapC1=7.02E-4 
            "Sze's first bandgap correction factor Silicon";
          constant Real GapC2=1108.0 
            "Sze's second bandgap correction factor Silicon";
          constant Modelica.SIunits.Voltage Fn0=3.25 
            "Work function constant for computation of flat band voltage";
          constant Real TOXdef = 1e-7 "Default oxide thickness";
          
          /* General parameters */
          parameter Integer Level(min=0,max=4) = 3 
            "Level of MOS model (check documentation window for details)";
          
          /* Basic Geometric parameters */
          parameter Modelica.SIunits.Length L(min=0) = 1e-4 
            "|Geometric parameters|Channel length";
          parameter Modelica.SIunits.Length W(min=0) = 1e-4 
            "|Geometric parameters|Channel width";
          
          /* Basic Process parameters */
          parameter Real TPG=1 "|Process parameters|Type of gate material";
          parameter Modelica.SIunits.Length TOX=TOXdef 
            "|Process parameters|Thin oxide thickness";
          parameter Real COX(unit="F/m2") = 0 
            "|Process parameters|Specific capacitance of SiO2";
          parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
            "|Process parameters|Substrate doping";
          
          /* Basic Electrical parameters */
          parameter Real U0(unit="m2/(V.s)") = 0 
            "|Electrical parameters|Surface mobility at reference temperature";
          parameter Modelica.SIunits.Voltage VT0=0 
            "|Electrical parameters|Zero-bias threshold voltage";
          parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
            "|Electrical parameters|Channel length modulation";
          parameter Modelica.SIunits.Length LD(min=0) = 0 
            "|Electrical parameters|Lateral diffusion";
          parameter Modelica.SIunits.Length WD(min=0) = 0 
            "|Electrical parameters|Width diffusion";
          parameter Modelica.SIunits.Transconductance KP=0 
            "|Electrical parameters|Transconductance parameter at reference temperature";
          parameter Real GAMMA(unit="V^0.5") = 0 
            "|Electrical parameters|Body-effect parameter";
          parameter Modelica.SIunits.Voltage PHI=0 
            "|Electrical parameters|Surface inversion potential at reference temperature";
          parameter Real NSS(unit="1/m2") = 0 
            "|Electrical parameters|Surface state density";
          
          /* Basic temperature compensation parameters */
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Modelica.SIunits.Voltage EG=1.16 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
          
          /* Level=1 electrical parameters */
          parameter Modelica.SIunits.Area AD=0 
            "|Level=1|Electrical parameters|Drain diffusion area";
          parameter Modelica.SIunits.Length PD=0 
            "|Level=1|Electrical parameters|Drain perimeter width";
          parameter Modelica.SIunits.Area AS=0 
            "|Level=1|Electrical parameters|Source diffusion area";
          parameter Modelica.SIunits.Length PS=0 
            "|Level=1|Electrical parameters|Source perimeter width";
          parameter Modelica.SIunits.Current ISD(min=0) = 0 
            "|Level=1|Electrical parameters|Drain junction saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS(min=0) = 0 
            "|Level=1|Electrical parameters|Source junction saturation current at reference temperature";
          parameter Modelica.SIunits.CurrentDensity JS(min=0) = 0 
            "|Level=1|Electrical parameters|Saturation current density at reference temperature";
          parameter Modelica.SIunits.Voltage PB(min=0.2) = 0.8 
            "|Level=1|Electrical parameters|Built-in potential at reference temperature";
          
          /* Level=1 parasitic resistance parameters */
          parameter Modelica.SIunits.Resistance RD(min=0) = 1 
            "|Level=1|Parasitic resistance parameters|Drain Ohmic resistance";
          parameter Modelica.SIunits.Resistance RS(min=0) = 1 
            "|Level=1|Parasitic resistance parameters|Source Ohmic resistance";
          
          /* Level=1 junction capacitance parameters */
          parameter Real CJ(unit="F/m2",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Zero-bias bulk capacitance per square meter at reference temperature";
          parameter Real MJ(min=0.3,max=0.5) = 0.33 
            "|Level=1|Junction capacitance parameters|Bulk junction grading coefficient";
          parameter Real CJSW(unit="F/m",min=0) = 1e-9 
            "|Level=1|Junction capacitance parameters|Zero-bias perimeter capacitance per meter at reference temperature";
          parameter Real MJSW(min=0.3,max=0.5) = 0.33 
            "|Level=1|Junction capacitance parameters|Perimeter capacitance grading coefficient";
          parameter Real FC(min=0,max=0.95)= 0.5 
            "|Level=1|Junction capacitance parameters|Forward-bias depletion capacitance coefficient";
          parameter Real CGB0(unit="F/m",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Gate-bulk overlap capacitance per meter";
          parameter Real CGD0(unit="F/m",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Gate-drain overlap capacitance per meter";
          parameter Real CGS0(unit="F/m",min=0) = 0 
            "|Level=1|Junction capacitance parameters|Gate-source overlap capacitance per meter";
          
          /* Level=1 temperature compensation parameters */
          parameter Real TRD1(unit="Ohm/degree") = 0 
            "|Level=1|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
          parameter Real TRD2(unit="Ohm/degree2") = 0 
            "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of drain resistance";
          parameter Real TRS1(unit="Ohm/degree") = 0 
            "|Level=1|Temperature compensation parameters|Linear temperature coefficient of source resistance";
          parameter Real TRS2(unit="Ohm/degree2") = 0 
            "|Level=1|Temperature compensation parameters|Quadratic temperature coefficient of source resistance";
          
          /* Level=2 process parameters */
          parameter Modelica.SIunits.Length XJ=0 
            "|Level=2|Process parameters|Metallurgical junction depth";
          
          /* Level=2 electrical parameters */
          parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
            "|Level=2|Electrical parameters|Critical electric field for mobility";
          parameter Real UEXP=0 
            "|Level=2|Electrical parameters|Exponential coefficient for mobility";
          parameter Real UTRA(min=0,max=0.5) = 0 
            "|Level=2|Electrical parameters|Transverse field coefficient";
          parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
            "|Level=2|Electrical parameters|Critical electric field for pinch-off";
          parameter Real DELTA=0 
            "|Level=2|Electrical parameters|Width effect on threshold voltage";
          parameter Modelica.SIunits.Velocity VMAX=0 
            "|Level=2|Electrical parameters|Maximum drift velocity of carriers";
          parameter Real NFS(unit="1/m2") = 0 
            "|Level=2|Electrical parameters|Surface fast state density";
          parameter Real NEFF(min=Modelica.Constants.small) = 1 
            "|Level=2|Electrical parameters|Total channel charge coefficient";
          parameter Real XQC=0 
            "|Level=2|Electrical parameters|Coefficient of channel charge share";
          
          /* Level=3 electrical parameters */
          parameter Real KAPPA(min=Modelica.Constants.small) = 0.2 
            "|Level=3|Electrical parameters|Saturation field factor";
          parameter Real ETA=0 
            "|Level=3|Electrical parameters|Static feedback on threshold voltage";
          parameter Modelica.SIunits.InversePotential THETA=0 
            "|Level=3|Electrical parameters|Mobility modulation";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC=1e-12 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
          /* Implement Spice-style MODCHK */
        protected 
          parameter Integer Type = -1 "Type=1 for NMOS; Type=-1 for PMOS";
          parameter Modelica.SIunits.Permittivity es = ks*e0 
            "Permittivity of silicon";
          parameter Modelica.SIunits.Permittivity eox = kox*e0 
            "Permittivity of SiO2";
          parameter Modelica.SIunits.Length LDeff = LD;
          parameter Modelica.SIunits.Length Leff = L - 2*LDeff;
          parameter Modelica.SIunits.Length Weff = W - 2*WD;
          parameter Modelica.SIunits.Length TOXeff = if TOX == TOXdef and COX > 0 then eox/COX else TOX;
          parameter Real COXeff(unit="F/m2") = if COX > 0 then COX else eox/TOX;
          parameter Modelica.SIunits.Capacitance CGB0eff = CGB0;
          parameter Modelica.SIunits.Capacitance CGD0eff = CGD0;
          parameter Modelica.SIunits.Capacitance CGS0eff = CGS0;
          parameter Real U0def(unit="m2/(V.s)") = 0.06;
          parameter Real U0tmp(unit="m2/(V.s)") = if KP == 0 and U0 == 0 then U0def else U0;
          parameter Modelica.SIunits.Transconductance KPeff = if KP > 0 then KP else U0tmp*COXeff;
          parameter Real U0eff(unit="m2/(V.s)") = if KP > 0 and U0 == 0 then KP/COXeff else U0tmp;
          parameter Real GAMMAtmp(unit="V^0.5") = if GAMMA == 0 and NSUB == 0 then 0.35 else GAMMA;
          parameter Modelica.SIunits.ElectronNumberDensity NSUBeff = if NSUB > 0 then NSUB else (GAMMAtmp*COXeff)^2/(2*es*q);
          parameter Real GAMMAeff(unit="V^0.5") = if GAMMA == 0 and NSUB > 0 then sqrt(2*es*q*NSUB)/COXeff else GAMMAtmp;
          parameter Modelica.SIunits.InversePotential LAMBDAeff = if LAMBDA > 0 then LAMBDA else 
                                                                  if Level <> 2 then sqrt(2*es/(q*NSUBeff)) else LAMBDA;
          parameter Modelica.SIunits.Voltage EGref = EG - GapC1*Tnom*Tnom/(Tnom + GapC2) 
            "Energy gap for temperature effect on saturation current at reference temperature";
          parameter Modelica.SIunits.ElectronNumberDensity XNI = ni*(Tnom/300)^1.5 * exp(EGref*q/(2*k)*(1/300 - 1/Tnom)) 
            "Intrinsic carrier concentration at reference temperature";
          parameter Modelica.SIunits.Voltage VTref = k*Tnom/q 
            "Thermal voltage at reference temperature";
          parameter Modelica.SIunits.Voltage PHIeff = if PHI > 0 then PHI else if NSUB > 0 then max({2*VTref*ln(NSUB/XNI),0.1}) else 0.6;
          parameter Modelica.SIunits.Voltage FermiG = Type*TPG*EGref/2;
          parameter Modelica.SIunits.Voltage Fn1 = if TPG == 0 then 3.2 else Fn0 + EGref/2 - FermiG;
          parameter Modelica.SIunits.Voltage FermiS = Type*PHIeff/2;
          parameter Modelica.SIunits.Voltage Fn2 = Fn1 - Fn0 - EGref/2 - FermiS;
          parameter Modelica.SIunits.Voltage VFB = Fn2 - NSS*q/COXeff;
          parameter Modelica.SIunits.Voltage Vcorr = Type*(GAMMAeff*sqrt(PHIeff) + PHIeff);
          parameter Modelica.SIunits.Voltage VT0eff = if Type*VT0 > 0 then VT0 else VFB + Vcorr;
          parameter Modelica.SIunits.Voltage VFBeff = if Type*VT0 > 0 then VT0 - Vcorr else VFB;
          parameter Real XD(unit="m/V^0.5") = sqrt(2*es/(q*NSUBeff)) 
            "Depletion factor";
          parameter Real CJeff(unit="F/m2") = if CJ > 0 then CJ else sqrt(es*q*NSUBeff/(2*PB));
          parameter Modelica.SIunits.Voltage EGeff = if Level == 3 and EG == 0 then 0.2 else EGref;
          parameter Modelica.SIunits.Voltage VBP = UCRIT*es/COXeff 
            "Gate to channel critical voltage";
          parameter Modelica.SIunits.Current ISDeff = if ISD > 0 then ISD else if AD*JS > 0 then AD*JS else 1e-15;
          parameter Modelica.SIunits.Current ISSeff = if ISS > 0 then ISS else if AS*JS > 0 then AS*JS else 1e-15;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
            annotation (extent=[90,40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
            annotation (extent=[-90,-40; -110,-60]);
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
            annotation (extent=[90,-40; 110,-60]);
          Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" 
            annotation (extent=[90,-10; 110,10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Utilities.Cgs Csg1(
            Tnom=Tnom,
            Level=Level,
            PHI=PHIeff,
            GAMMA=GAMMAeff,
            LD=LDeff,
            CGS0=CGS0eff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            XJ=XJ,
            XQC=XQC,
            DELTA=DELTA,
            ETA=ETA,
            Type=Type,
            VFB=VFBeff,
            KP=KPeff,
            COX=COXeff,
            NFS=NFS,
            XD=XD,
            enforceStates=enforceStates,
            u0=0)      annotation (extent=[-10,-50; -50,-30]);
          Utilities.Cgd Cdg1(
            Tnom=Tnom,
            Level=Level,
            PHI=PHIeff,
            GAMMA=GAMMAeff,
            LD=LDeff,
            CGD0=CGD0eff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            XJ=XJ,
            XQC=XQC,
            DELTA=DELTA,
            ETA=ETA,
            Type=Type,
            VFB=VFBeff,
            KP=KPeff,
            COX=COXeff,
            NFS=NFS,
            XD=XD,
            enforceStates=enforceStates,
            u0=0)      annotation (extent=[-10,40; -50,60]);
          Utilities.Cgb Cbg1(
            Tnom=Tnom,
            Level=Level,
            PHI=PHIeff,
            GAMMA=GAMMAeff,
            LD=LDeff,
            CGB0=CGB0eff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            XJ=XJ,
            XQC=XQC,
            DELTA=DELTA,
            ETA=ETA,
            Type=Type,
            VFB=VFBeff,
            KP=KPeff,
            COX=COXeff,
            NFS=NFS,
            XD=XD,
            enforceStates=enforceStates,
            u0=0)      annotation (extent=[30,70; -10,90]);
          Sensors.VoltageSensor Vsd 
            annotation (extent=[128,12; 108,-8], rotation=-90);
          Sensors.VoltageSensor Vsb 
            annotation (extent=[90,-34; 70,-14], rotation=90);
          Utilities.RM Rd(
            Tnom=Tnom,
            R=RD,
            TR1=TRD1,
            TR2=TRD2,
            Level=Level) annotation (extent=[30,40; 70,60]);
          Utilities.RM Rs(
            Tnom=Tnom,
            Level=Level,
            R=RS,
            TR1=TRS1,
            TR2=TRS2) annotation (extent=[28,-50; 68,-30]);
          annotation (
            Diagram(
              Ellipse(extent=[-2,-58; 2,-62], style(
                  color=42,
                  rgbcolor={179,0,0},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1)),
              Text(
                extent=[20,96; 34,90],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vsb"),
              Text(
                extent=[-14,96; 0,90],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vsd"),
              Ellipse(extent=[78,2; 82,-2], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Text(
                extent=[-20,66; -6,60],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vsb"),
              Text(
                extent=[-54,66; -40,60],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vsd"),
              Text(
                extent=[-18,-24; -4,-30],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vsb"),
              Text(
                extent=[-56,-24; -42,-30],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=42,
                  rgbfillColor={168,0,0},
                  fillPattern=1),
                string="vsd"),
        Rectangle(extent=[20,-66; 40,-86],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-66; 40,-86],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-76; 20,-76],   style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
The Spice-style p-channel MOSFET model contains the internal MOSFET plus the three external gate capacitances and the drain and source resistances. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model</b></font>


<b>Basic Geometric Parameters:</b>

 L:       Channel length (default value = 1e-4 m) <font color=red>Levels 0-4</font>

 W:       Channel width (default value = 1e-4 m) <font color=red>Levels 0-4</font>


<b>Basic Process Parameters:</b>

 TPG:     Type of gate material (default value = 1) <font color=red>Levels 0-4</font>
            <font color=blue><b>TPG = 0:  Aluminum gate</b></font>
            <font color=blue><b>TPG = 1:  Polysilicon gate</b></font>

 TOX:     Thin oxide thickness (default value = 1e-7 m) <font color=red>Levels 0-4</font>

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 0-4</font>

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>) <font color=red>Levels 0-4</font>


<b>Basic Electrical Parameters:</b>

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s)) <font color=red>Levels 0-4</font>

 VT0:     Zero-bias threshold voltage (default value = 0 Volt) <font color=red>Levels 0,1</font>
            <font color=blue><b>VT0 &lt; 0 for enhancement MOSFET
            VT0 &gt; 0 for depletion MOSFET</b></font>

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt) <font color=red>Levels 0-4</font>

 LD:      Lateral diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 WD:      Width diffusion (default value = 0 m) <font color=red>Levels 0-4</font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 0-4</font>

 GAMMA:   Body-effect parameter (default value = 0 Volt<sup>0.5</sup>) <font color=red>Levels 0-4</font>

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt) <font color=red>Levels 0-4</font>

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 0-4</font>


<b>Basic Temperature Compensation Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 0-4</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt) <font color=red>Levels 0-4</font>


<b>Level=1 Electrical Parameters:</b>

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PD:      Drain perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PS:      Source perimeter width (default value = 0 m) <font color=red>Levels 1-4</font>

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 1-4</font>

 JS:      Transport saturation current density (default value = 0 Amp/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-4</font>


<b>Level=1 Parasitic Resistor Parameters:</b>

 RD:      Drain Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>

 RS:      Source Ohmic resistance (default value = 1 Ohm) <font color=red>Levels 1-4</font>


<b>Level=1 Junction Capacitance Parameters:</b>

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 0 F/m<sup>2</sup>) <font color=red>Levels 1-4</font>

 MJ:      Bulk junction grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m) <font color=red>Levels 1-4</font>

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33) <font color=red>Levels 1-4</font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1-4</font>

 CGB0:    Gate-bulk overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGD0:    Gate-drain overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>

 CGS0:    Gate-source overlap capacitance per meter (default value = 0 F/m) <font color=red>Levels 1-4</font>


<b>Level=1 Temperature Compensation Parameters:</b>

 TRD1:    Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRD2:    Quadratic temperature coefficient of drain resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>

 TRS1:    Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-4</font>

 TRS2:    Quadratic temperature coefficient of source resistance (default value = 0 Ohm/K<sup>2</sup>) <font color=red>Levels 1-4</font>


<b>Level=2 Process Parameters:</b>

 XJ:      Metallurgical Junction Depth (default value = 0 m) <font color=red>Levels 2-4</font>


<b>Level=2 Electrical Parameters:</b>

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m) <font color=red>Levels 2,4</font>

 UEXP:    Exponential coefficient for mobility (default value = 0) <font color=red>Levels 2,4</font>

 UTRA:    Transverse field coefficient (default value = 0) <font color=red>Levels 2,4</font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,4</font>

 DELTA:   Width effect on threshold voltage (default value = 0) <font color=red>Levels 2-4</font>

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s) <font color=red>Levels 2-4</font>
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation (Levels 2,4)</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation (Levels 2,4)</b></font>

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>) <font color=red>Levels 2-4</font>

 NEFF:    Total channel charge coefficient (default value = 1) <font color=red>Levels 2,4</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2-4</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>


<b>Level=3 Electrical Parameters:</b>

 KAPPA:   Saturation field factor (default value = 0.2) <font color=red>Level 3</font>

 ETA:     Static feedback on threshold voltage (default value = 0) <font color=red>Level 3</font>

 THETA:   Mobility modulation (default value = 0 1/Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-4</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-4</font>

 GminDC:  Leakage conductance (default value = 1e-12 mho) <font color=red>Levels 0-4</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 0-4</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"),         Icon(
              Text(extent=[-100,80; 100,100],   string="%name"),
              Line(points=[-90,-50; -10,-50]),
              Line(points=[-10,-50; -10,50]),
              Line(points=[10,70; 10,29]),
              Line(points=[10,20; 10,-21]),
              Line(points=[10,-30; 10,-71]),
              Line(points=[10,50; 91,50]),
              Line(points=[10,0; 90,0]),
              Line(points=[10,-50; 90,-50]),
              Polygon(points=[60,0; 40,5; 40,-5; 60,0],     style(fillColor=3,
                    fillPattern=1)),
              Line(points=[0,-91; 0,-50],   style(color=42))));
          Utilities.Mpi Mpi(
            Tnom=Tnom,
            GAMMA=GAMMAeff,
            KP=KPeff,
            PHI=PHIeff,
            NSS=NSS,
            NFS=NFS,
            NEFF=NEFF,
            TPG=TPG,
            UEXP=UEXP,
            UTRA=UTRA,
            VMAX=VMAX,
            THETA=THETA,
            ISD=ISDeff,
            ISS=ISSeff,
            PB=PB,
            MJ=MJ,
            MJSW=MJSW,
            FC=FC,
            GminDC=GminDC,
            Level=Level,
            EMin=EMin,
            EMax=EMax,
            VT0=VT0eff,
            LAMBDA=LAMBDAeff,
            TOX=TOXeff,
            NSUB=NSUBeff,
            LD=LDeff,
            U0=U0eff,
            CJ=CJeff,
            EG=EGeff,
            L=Leff,
            W=Weff,
            COX=COXeff,
            VFB=VFBeff,
            XJ=XJ,
            AD=AD,
            PD=PD,
            AS=AS,
            PS=PS,
            CJSW=CJSW,
            XD=XD,
            UCRIT=UCRIT,
            VBP=VBP,
            XQC=XQC,
            ECRIT=ECRIT,
            Type=-1,
            KAPPA=KAPPA,
            DELTA=DELTA,
            ETA=ETA,
            enforceStates=enforceStates) 
                       annotation (extent=[-20,-20; 20,20]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-86; 10,-66], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
        algorithm 
          assert(NSUBeff > XNI, "NSUB must be larger than intrinsic carrier concentration");
          assert(LAMBDAeff < 0.2, "LAMBDA value too large, leads to numerical instability");
          assert(Leff > 0, "The effective channel length must be larger than zero");
          
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(Rd.n,D)  annotation (points=[70,50; 100,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rs.n,S)  annotation (points=[68,-40; 80,-40; 80,-50; 100,-50],
                                                                 style(color=3,
                rgbcolor={0,0,255}));
          connect(Cbg1.heatPort,Rd. heatPort) annotation (points=[10,70; 10,34;
                50,34; 50,40],
                        style(color=42, rgbcolor={191,0,0}));
          connect(Rd.heatPort, Mpi.heatPort)  annotation (points=[50,40; 50,-20;
                0,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Mpi.B, B)  annotation (points=[20,0; 100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.p, Mpi.D)  annotation (points=[30,50; 20,50; 20,10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rs.p, Mpi.S)  annotation (points=[28,-40; 20,-40; 20,-10],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Mpi.G, G)  annotation (points=[-20,-10; -76,-10; -76,-50;
                -100,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Csg1.n, G) annotation (points=[-50,-40; -76,-40; -76,-50;
                -100,-50],                                           style(
                color=3, rgbcolor={0,0,255}));
          connect(Csg1.p, Rs.p) annotation (points=[-10,-40; 28,-40], style(
                color=3, rgbcolor={0,0,255}));
          connect(Cdg1.n, G) annotation (points=[-50,50; -76,50; -76,-50; -100,
                -50], style(color=3, rgbcolor={0,0,255}));
          connect(Cdg1.p, Rd.p) annotation (points=[-10,50; 30,50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Cbg1.p, B) annotation (points=[30,80; 80,80; 80,0; 100,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Cbg1.n, G) annotation (points=[-10,80; -76,80; -76,-50; -100,
                -50], style(color=3, rgbcolor={0,0,255}));
          connect(Vsb.n, B) annotation (points=[80,-14; 80,0; 100,0], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vsb.p, S) annotation (points=[80,-34; 80,-50; 100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vsd.p, S) annotation (points=[118,-8; 118,-50; 100,-50],
              style(color=3, rgbcolor={0,0,255}));
          connect(D, Vsd.n) annotation (points=[100,50; 118,50; 118,12], style(
                color=3, rgbcolor={0,0,255}));
          connect(Csg1.vbs, Vsb.V) annotation (points=[-18,-30; -18,-24; 70,-24],
              style(color=74, rgbcolor={0,0,127}));
          connect(Vsd.V, Cbg1.vds) annotation (points=[128,2; 132,2; 132,98; -2,
                98; -2,90], style(color=74, rgbcolor={0,0,127}));
          connect(Cbg1.vds, Cdg1.vds) annotation (points=[-2,90; -2,98; -42,98;
                -42,60], style(color=74, rgbcolor={0,0,127}));
          connect(Cdg1.vds, Csg1.vds) annotation (points=[-42,60; -42,66; -60,
                66; -60,-24; -42,-24; -42,-30], style(color=74, rgbcolor={0,0,
                  127}));
          connect(Csg1.vbs, Cdg1.vbs) annotation (points=[-18,-30; -18,-24; -10,
                -24; -10,30; 0,30; 0,66; -18,66; -18,60], style(color=74,
                rgbcolor={0,0,127}));
          connect(Cbg1.vbs, Cdg1.vbs) annotation (points=[22,90; 22,96; 38,96;
                38,66; -18,66; -18,60], style(color=74, rgbcolor={0,0,127}));
          connect(Cdg1.heatPort, Cbg1.heatPort) annotation (points=[-30,40; -30,34;
                10,34; 10,70], style(color=42, rgbcolor={191,0,0}));
          connect(Mpi.y1, Cdg1.u)  annotation (points=[10,22; 10,26; -18,26;
                -18,40], style(color=74, rgbcolor={0,0,127}));
          connect(Mpi.y1, Cbg1.u)  annotation (points=[10,22; 10,26; 16,26; 16,
                60; 22,60; 22,70], style(color=74, rgbcolor={0,0,127}));
          connect(Mpi.y1, Csg1.u)  annotation (points=[10,22; 10,26; 24,26; 24,
                -54; -18,-54; -18,-50], style(color=74, rgbcolor={0,0,127}));
          connect(Rs.heatPort, Csg1.heatPort) annotation (points=[48,-50; 48,-60;
                -30,-60; -30,-50], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Mpi.heatPort, Csg1.heatPort) annotation (points=[0,-20; 0,-60;
                -30,-60; -30,-50], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-87; 12,-87; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-87;
                6.73533e-016,-89.5; 0,-89.5; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.p, Csg1.heatPort) annotation (points=[-6.73533e-016,-65;
                -6.73533e-016,-60; -30,-60; -30,-50], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
        end Mp;
        
        model Jn "Spice-style n-channel JFET" 
          
          /* General parameters */
          parameter Integer Level(min=1,max=3) = 1 
            "JFET modeling level (check documentation layer for details)";
          parameter Real Area=1 "Relative area occupied by capacitor";
          
          /* Basic Electrical parameters */
          parameter Modelica.SIunits.Voltage VT0=0 
            "|Electrical parameters|Threshold voltage at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
            "|Electrical parameters|Channel length modulation";
          parameter Modelica.SIunits.Transconductance BETA=0 
            "|Electrical parameters|Transconductance parameter at reference temperature";
          parameter Real N=1 
            "|Electrical parameters|Current emission coefficient";
          parameter Modelica.SIunits.Current IS(min=Modelica.Constants.small) = 1.e-14 
            "|Electrical parameters|Transport saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
            "|Electrical parameters|Built-in potential at reference temperature";
          
          /* Parasitic resistor parameters */
          parameter Modelica.SIunits.Resistance RS(min=0) = 0 
            "|Parasitic resistor parameters|Source resistance";
          parameter Modelica.SIunits.Resistance RD(min=0) = 0 
            "|Parasitic resistor parameters|Drain resistance";
          parameter Modelica.SIunits.Resistance RG(min=0) = 0 
            "|Parasitic resistor parameters|Gate resistance";
          
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CGS(min=Modelica.Constants.small) = 0.5e-12 
            "|Junction capacitor parameters|Zero-bias gate-source depletion capacitance at reference temperature";
          parameter Modelica.SIunits.Capacitance CGD(min=Modelica.Constants.small) = 0.5e-12 
            "|Junction capacitor parameters|Zero-bias gate-drain depletion capacitance at reference temperature";
          parameter Real M(min=0.1,max=0.9) = if Level==3 then 0.5 else 0.33 
            "|Junction capacitor parameters|Junction grading coefficient";
          parameter Real FC(min=0,max=0.95)= 0.5 
            "|Junction capacitor parameters|Factor for linearization of depletion capacitance formula";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TT(min=0) = 5e-9 
            "|Transit time parameters|Ideal transit time";
          
          /* Temperature compensation parameters */
          parameter Real TCV(unit="V/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of threshold voltage";
          parameter Real TRS(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of source resistance";
          parameter Real TRD(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
          parameter Real TRG(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of gate resistance";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI = if Level==3 then 2 else 3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Real BEX=0 
            "|Temperature compensation parameters|Mobility temperature exponent";
          parameter Modelica.SIunits.Voltage EG(min=0.1) = if Level==3 then 0.73 else 1.16 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
          
          /* Level=2 electrical parameters */
          parameter Modelica.SIunits.InversePotential LAM1=0 
            "|Level=2|Electrical parameters|Channel length modulation gate voltage parameter";
          
          /* Level=3 electrical parameters */
          parameter Modelica.SIunits.InversePotential ALPHA=1 
            "|Level=3|Electrical parameters|Hyperbolic tangient fitting parameter";
          parameter Modelica.SIunits.InversePotential B=0 
            "|Level=3|Electrical parameters|Measure of doping profile";
          parameter Modelica.SIunits.Voltage DELTA=0.2 
            "|Level=3|Electrical parameters|Voltage range for transition";
          parameter Modelica.SIunits.Voltage VMAX=0.5 
            "|Level=3|Electrical parameters|Limit voltage for carrier velocity saturation";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
            annotation (extent=[-90,-40; -110,-60]);
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
            annotation (extent=[90,40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
            annotation (extent=[90,-40; 110,-60]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Utilities.Jni Jni1(
            Level=Level,
            Tnom=Tnom,
            VT0=VT0,
            IS=IS,
            BETA=BETA,
            LAMBDA=LAMBDA,
            TCV=TCV,
            EG=EG,
            N=N,
            XTI=XTI,
            PB=PB,
            CGS=CGS,
            CGD=CGD,
            M=M,
            FC=FC,
            TT=TT,
            BEX=BEX,
            LAM1=LAM1,
            ALPHA=ALPHA,
            B=B,
            DELTA=DELTA,
            VMAX=VMAX,
            GminDC=GminDC,
            Area=Area,
            EMin=EMin,
            EMax=EMax,
            enforceStates=enforceStates) 
                       annotation (extent=[-20,-20; 20,20]);
          annotation (
            Diagram(
        Rectangle(extent=[20,-60; 40,-80],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-60; 40,-80],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-70; 20,-70],   style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
The Spice-style n-channel JFET model contains an internal n-channel JFET together with the three external resistances. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Area:    Relative area occupied by the diode (default value = 1) <font color=red>Levels 1-3</font>


<b>Basic Electrical Parameters:</b>

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt) <font color=red>Levels 1-3</font>
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 LAMBDA:  Channel length modulation (default value = 0 1/Volt) <font color=red>Levels 1-3</font>

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 1-3</font>

 N:       Current emission coefficient (default value = 1) <font color=red>Levels 1-3</font>

 IS:      Transport saturation current (default value = 1e-14 Amp) <font color=red>Levels 1-3</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-3</font>


<b>Parasitic Resistor Parameters:</b>

 RS:      Source resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RD:      Drain resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RG:      Gate resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>


<b>Junction Capacitance Parameters:</b>

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 M:       Junction grading coefficient <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1,2</font>


<b>Transit Time Parameters:</b>

 TT:      Ideal transit time (default value = 5e-9 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation Parameters:</b>

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K) <font color=red>Levels 1-3</font>

 TRS:     Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRD:     Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRG:     Linear temperature coefficient of gate resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1-3</font>

 XTI:     Saturation current temperature exponent <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 BEX:     Mobility temperature exponent (default value = 0) <font color=red>Levels 1-3</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>


<b>Level=2 Electrical Parameters:</b>

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt) <font color=red>Level 2</font>


<b>Level=3 Electrical Parameters:</b>

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1) <font color=red>Level 3</font>

 B:       Measure of doping profile (default value = 0 1/Volt) <font color=red>Level 3</font>

 DELTA:   Voltage range for transition (default value = 0.2 Volt) <font color=red>Level 3</font>

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-3</font>

 GminDC:  Leakage conductance (default value = 1e-15 Mho) <font color=red>Levels 1-3</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 0-4</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"),         Icon(
              Line(points=[0,50; 91,50]),
              Line(points=[0,-50; 90,-50]),
              Line(points=[-90,-50; 0,-50]),
              Line(points=[0,-70; 0,70]),
              Polygon(points=[0,-50; -20,-46; -20,-54; 0,-50],
                                                            style(fillColor=3,
                    fillPattern=1)),
              Line(points=[0,-91; 0,-80],   style(color=42)),
              Text(extent=[-100,80; 100,100],   string="%name")));
          R Rg(
            Tnom=Tnom,
            R=RG,
            TR1=TRG,
            TR2=0,
            Area=Area) annotation (extent=[-80,-22; -40,2]);
          R Rd(
            Tnom=Tnom,
            TR2=0,
            Area=Area,
            R=RD,
            TR1=TRD) annotation (extent=[40,38; 80,62]);
          R Rs(
            Tnom=Tnom,
            TR2=0,
            Area=Area,
            R=RS,
            TR1=TRS) annotation (extent=[40,-52; 80,-28]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(Rg.p, G) annotation (points=[-80,-10; -90,-10; -90,-50; -100,
                -50],                                          style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rg.n, Jni1.G) annotation (points=[-40,-10; -20,-10],
                                                                   style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.n, D) annotation (points=[80,50; 100,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.p, Jni1.D) annotation (points=[40,50; 20,50; 20,10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rs.n, S) annotation (points=[80,-40; 90,-40; 90,-50; 100,-50],
                                                                 style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rs.p, Jni1.S) annotation (points=[40,-40; 20,-40; 20,-10],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.heatPort, Jni1.heatPort) annotation (points=[60,38; 60,-20;
                0,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rg.heatPort, Jni1.heatPort) annotation (points=[-60,-22; -60,
                -30; 0,-30; 0,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Rs.heatPort, Jni1.heatPort) annotation (points=[60,-52; 0,-52;
                0,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-81; 12,-81; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.p, Rs.heatPort) annotation (points=[-6.73533e-016,-59;
                -6.73533e-016,-52; 60,-52], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-86.5; 0,-86.5; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
        end Jn;
        
        model Jp "Spice-style p-channel JFET" 
          
          /* General parameters */
          parameter Integer Level(min=1,max=3) = 1 
            "JFET modeling level (check documentation layer for details)";
          parameter Real Area=1 "Relative area occupied by capacitor";
          
          /* Basic Electrical parameters */
          parameter Modelica.SIunits.Voltage VT0=0 
            "|Electrical parameters|Threshold voltage at reference temperature";
          parameter Modelica.SIunits.InversePotential LAMBDA(min=0) = 0 
            "|Electrical parameters|Channel length modulation";
          parameter Modelica.SIunits.Transconductance BETA=0 
            "|Electrical parameters|Transconductance parameter at reference temperature";
          parameter Real N=1 
            "|Electrical parameters|Current emission coefficient";
          parameter Modelica.SIunits.Current IS(min=Modelica.Constants.small) = 1.e-14 
            "|Electrical parameters|Transport saturation current at reference temperature";
          parameter Modelica.SIunits.Voltage PB=0.8 
            "|Electrical parameters|Built-in potential at reference temperature";
          
          /* Parasitic resistor parameters */
          parameter Modelica.SIunits.Resistance RS(min=0) = 0 
            "|Parasitic resistor parameters|Source resistance";
          parameter Modelica.SIunits.Resistance RD(min=0) = 0 
            "|Parasitic resistor parameters|Drain resistance";
          parameter Modelica.SIunits.Resistance RG(min=0) = 0 
            "|Parasitic resistor parameters|Gate resistance";
          
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CGS(min=Modelica.Constants.small) = 0.5e-12 
            "|Junction capacitor parameters|Zero-bias gate-source depletion capacitance at reference temperature";
          parameter Modelica.SIunits.Capacitance CGD(min=Modelica.Constants.small) = 0.5e-12 
            "|Junction capacitor parameters|Zero-bias gate-drain depletion capacitance at reference temperature";
          parameter Real M(min=0.1,max=0.9) = if Level==3 then 0.5 else 0.33 
            "|Junction capacitor parameters|Junction grading coefficient";
          parameter Real FC(min=0,max=0.95)= 0.5 
            "|Junction capacitor parameters|Factor for linearization of depletion capacitance formula";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TT(min=0) = 5e-9 
            "|Transit time parameters|Ideal transit time";
          
          /* Temperature compensation parameters */
          parameter Real TCV(unit="V/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of threshold voltage";
          parameter Real TRS(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of source resistance";
          parameter Real TRD(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of drain resistance";
          parameter Real TRG(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of gate resistance";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI = if Level==3 then 2 else 3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Real BEX=0 
            "|Temperature compensation parameters|Mobility temperature exponent";
          parameter Modelica.SIunits.Voltage EG(min=0.1) = if Level==3 then 0.73 else 1.16 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current at 0 K";
          
          /* Level=2 electrical parameters */
          parameter Modelica.SIunits.InversePotential LAM1=0 
            "|Level=2|Electrical parameters|Channel length modulation gate voltage parameter";
          
          /* Level=3 electrical parameters */
          parameter Modelica.SIunits.InversePotential ALPHA=1 
            "|Level=3|Electrical parameters|Hyperbolic tangient fitting parameter";
          parameter Modelica.SIunits.InversePotential B=0 
            "|Level=3|Electrical parameters|Measure of doping profile";
          parameter Modelica.SIunits.Voltage DELTA=0.2 
            "|Level=3|Electrical parameters|Voltage range for transition";
          parameter Modelica.SIunits.Voltage VMAX=0.5 
            "|Level=3|Electrical parameters|Limit voltage for carrier velocity saturation";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC=1e-16 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
            annotation (extent=[-90,-40; -110,-60]);
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
            annotation (extent=[90,40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
            annotation (extent=[90,-40; 110,-60]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          R Rg(
            Tnom=Tnom,
            R=RG,
            TR1=TRG,
            TR2=0,
            Area=Area) annotation (extent=[-80,-22; -40,2]);
          R Rd(
            Tnom=Tnom,
            TR2=0,
            Area=Area,
            R=RD,
            TR1=TRD) annotation (extent=[40,38; 80,62]);
          R Rs(
            Tnom=Tnom,
            TR2=0,
            Area=Area,
            R=RS,
            TR1=TRS) annotation (extent=[40,-52; 80,-28]);
          Utilities.Jpi Jpi1(
            Level=Level,
            Tnom=Tnom,
            VT0=VT0,
            IS=IS,
            BETA=BETA,
            LAMBDA=LAMBDA,
            TCV=TCV,
            EG=EG,
            N=N,
            XTI=XTI,
            PB=PB,
            CGS=CGS,
            CGD=CGD,
            M=M,
            FC=FC,
            TT=TT,
            BEX=BEX,
            LAM1=LAM1,
            ALPHA=ALPHA,
            B=B,
            DELTA=DELTA,
            VMAX=VMAX,
            GminDC=GminDC,
            Area=Area,
            EMin=EMin,
            EMax=EMax,
            enforceStates=enforceStates) 
                       annotation (extent=[-20,-20; 20,20]);
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
        equation 
          connect(Rg.p, G) annotation (points=[-80,-10; -90,-10; -90,-50; -100,
                -50],                                          style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.n, D) annotation (points=[80,50; 100,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rs.n, S) annotation (points=[80,-40; 90,-40; 90,-50; 100,-50],
                                                                 style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          annotation (
            Diagram(
        Rectangle(extent=[20,-60; 40,-80],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-60; 40,-80],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-70; 20,-70],   style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Line(points=[0,50; 91,50]),
              Line(points=[0,-50; 90,-50]),
              Line(points=[-90,-50; 0,-50]),
              Line(points=[0,-70; 0,70]),
              Polygon(points=[-60,-50; -40,-46; -40,-54; -60,-50],
                                                            style(fillColor=3,
                    fillPattern=1)),
              Line(points=[0,-91; 0,-80],   style(color=42)),
              Text(extent=[-100,80; 100,100],   string="%name")),
            Documentation(info="<html>
The Spice-style p-channel JFET model contains an internal p-channel JFET together with the three external resistances. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

<hr> <p>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Area:    Relative area occupied by the diode (default value = 1) <font color=red>Levels 1-3</font>


<b>Basic Electrical Parameters:</b>

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt) <font color=red>Levels 1-3</font>
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 LAMBDA:  Channel length modulation (default value = 0 1/Volt) <font color=red>Levels 1-3</font>

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>) <font color=red>Levels 1-3</font>

 N:       Current emission coefficient (default value = 1) <font color=red>Levels 1-3</font>

 IS:      Transport saturation current (default value = 1e-14 Amp) <font color=red>Levels 1-3</font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt) <font color=red>Levels 1-3</font>


<b>Parasitic Resistor Parameters:</b>

 RS:      Source resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RD:      Drain resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>

 RG:      Gate resistance (default value = 0 Ohm) <font color=red>Levels 1-3</font>


<b>Junction Capacitance Parameters:</b>

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F) <font color=red>Levels 1-3</font>

 M:       Junction grading coefficient <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5) <font color=red>Levels 1,2</font>


<b>Transit Time Parameters:</b>

 TT:      Ideal transit time (default value = 5e-9 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation Parameters:</b>

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K) <font color=red>Levels 1-3</font>

 TRS:     Linear temperature coefficient of source resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRD:     Linear temperature coefficient of drain resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 TRG:     Linear temperature coefficient of gate resistance (default value = 0 Ohm/K) <font color=red>Levels 1-3</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1-3</font>

 XTI:     Saturation current temperature exponent <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 BEX:     Mobility temperature exponent (default value = 0) <font color=red>Levels 1-3</font>

 EG:      Energy gap for temperature effect on saturation current at 0 K <font color=red>Levels 1-3</font>
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>


<b>Level=2 Electrical Parameters:</b>

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt) <font color=red>Level 2</font>


<b>Level=3 Electrical Parameters:</b>

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1) <font color=red>Level 3</font>

 B:       Measure of doping profile (default value = 0 1/Volt) <font color=red>Level 3</font>

 DELTA:   Voltage range for transition (default value = 0.2 Volt) <font color=red>Level 3</font>

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt) <font color=red>Level 3</font>


<b>Numerical Parameters:</b>

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100) <font color=red>Levels 1-3</font>

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40) <font color=red>Levels 1-3</font>

 GminDC:  Leakage conductance (default value = 1e-15 Mho) <font color=red>Levels 1-3</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 0-4</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          connect(Rg.n, Jpi1.G) annotation (points=[-40,-10; -20,-10],
                                                                   style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.heatPort, Jpi1.heatPort) annotation (points=[60,38; 60,-20;
                0,-20], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rd.p, Jpi1.D) annotation (points=[40,50; 20,50; 20,10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Jpi1.S, Rs.p) annotation (points=[20,-10; 20,-40; 40,-40],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
          connect(Rg.heatPort, Jpi1.heatPort) annotation (points=[-60,-22; -60,
                -30; 0,-30; 0,-20], style(color=42, rgbcolor={191,0,0}));
          connect(Jpi1.heatPort, Rs.heatPort) annotation (points=[0,-20; 0,-52;
                60,-52], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-81; 12,-81; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.p, Rs.heatPort) annotation (points=[-6.73533e-016,-59;
                -6.73533e-016,-52; 60,-52], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-86.5; 0,-86.5; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={209,0,0},
              fillPattern=1));
        end Jp;
        
        package Utilities "Utility models of Spice circuit element models" 
          annotation (preferedView="info",Icon(
              Rectangle(extent=[-100,-100; 80,50], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=42, rgbcolor={127,0,0})),
              Text(
                extent=[-120,122; 120,73],
                string="%name",
                style(color=1))), Documentation(info="<html>
This package contains some utility models used within the Spice-style circuit element models.
</html>
"));
          model Rce 
            "Spice-style bipolar collector/emitter resistor of the analog electrical library" 
            extends Interfaces.SpiceTwoPort;
            parameter Modelica.SIunits.Resistance R=0 
              "Resistance at reference temperature";
            parameter Real TR1(unit="Ohm/degree") = 0 
              "Linear temperature coefficient";
            parameter Real TR2(unit="Ohm/degree2") = 0 
              "Quadratic temperature coefficient";
            parameter Real Area=1 "Relative area occupied by resistor";
            parameter Integer Level=2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            
            annotation (Diagram, Documentation(info="<html>
The collector and emitter resistances are regular Spice-style resistors.  However, a separate model had to be developed, in order to account for the <b>Level</b> parameter of Spice. <p>

<hr> <p>

<pre>
Parameters:

 R :     Resistance at reference temperature (default value = 0 Ohm)

 TR1:    Linear temperature variation coefficient (default value = 0 Ohm/<sup>o</sup>K)

 TR2:    Quadratic temperature variation coefficient (default value = 0 Ohm/<sup>o</sup>K<sup>2</sup>)

 Area:   Relative area occupied by resistor (default value = 1)

 Level:  Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)
</pre>
</html>
"),           Icon(
                Rectangle(extent=[-70,30; 70,-30],   style(
                    color=3,
                    fillColor=7,
                    fillPattern=1)),
                Text(
                  extent=[-50,-40; 46,-80],
                  style(color=0),
                  string="R=%R"),
                Line(points=[0,-30; 0,-91],   style(color=42)),
                Line(points=[-90,0; -70,0]),
                Line(points=[70,0; 90,0])));
          protected 
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,28; 12,52]);
            BondLib.Bonds.Bond B1 annotation (extent=[-38,28; -12,52]);
            BondLib.Bonds.Bond B2 annotation (extent=[12,28; 38,52]);
            BondLib.Bonds.Bond B3 annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[-62,26;
                  -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,26;
                  64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            BondLib.Spice.Utilities.RS2 Rce1(
              Tnom=Tnom,
              R=R,
              TR1=TR1,
              TR2=TR2,
              Area=Area,
              Level=Level) annotation (extent=[-10,-32; 10,4], rotation=-90);
            BondLib.Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            connect(B3.BondCon2,Rce1. BondCon1) annotation (points=[
                  -2.20429e-015,4; -6.12303e-016,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(Rce1.BondCon2,B4. fBondCon1) annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.BondCon1,B4. eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-86; 0,-86; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end Rce;
          
          model Rb 
            "Spice-style bipolar base resistor of the analog electrical library" 
            extends Interfaces.SpiceTwoPort;
            parameter Modelica.SIunits.Resistance RB=0 
              "Zero-bias base resistance at reference temperature";
            parameter Modelica.SIunits.Resistance RBM=0 
              "Minimum base resistance at reference temperature";
            parameter Modelica.SIunits.Current IRB=0 
              "Reference base current at reference temperature";
            parameter Real TRB1(unit="Ohm/degree") = 0 
              "Linear temperature coefficient [Ohm/degree]";
            parameter Real TRB2(unit="Ohm/degree2") = 0 
              "Quadratic temperature coefficient [Ohm/(degree*degree)]";
            parameter Real TRM1(unit="Ohm/degree") = 0 
              "Linear temperature coefficient [Ohm/degree]";
            parameter Real TRM2(unit="Ohm/degree2") = 0 
              "Quadratic temperature coefficient [Ohm/(degree*degree)]";
            parameter Real Area=1 "Relative area occupied by resistor";
            parameter Integer Level=2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            BondLib.Spice.Utilities.RBS Rb1(
              Tnom=Tnom,
              RB=RB,
              RBM=RBM,
              IRB=IRB,
              TRB1=TRB1,
              TRB2=TRB2,
              TRM1=TRM1,
              TRM2=TRM2,
              Area=Area,
              Level=Level) 
                         annotation (extent=[-10,-32; 10,4], rotation=-90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput qb "Base charge" 
              annotation (extent=[-10,90; 10,110], rotation=-90);
          protected 
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Icon(
                Rectangle(extent=[-70,30; 70,-30],   style(
                    color=3,
                    fillColor=7,
                    fillPattern=1)),
                Line(points=[-90,0; -70,0]),
                Line(points=[70,0; 90,0]),
                Text(
                  extent=[-50,-40; 46,-80],
                  style(color=0),
                  string="R=%RB"),
                Line(points=[0,-30; 0,-91],   style(color=42))),
              Documentation(info="<html>
The base resistor is the most important of the parasitic resistors of a bipolar transistor.  For this reason, Spice models this resistor using a more sophisticated model with additional parameters. <p>

<hr> <p>

<pre>
Parameters:

 RB:     Reference zero-bias base resistance (default value = 0 Ohm)

 RBM:    Reference minimum base resistance (default value = 0 Ohm)

 IRB:    Current where base resistance falls halfway to minimum value (default value = 0 Amp)

 Tnom:   Reference temperature (default value = 300.15 <sup>o</sup>K)

 TRB1:   Linear temperature variation coefficient of RB (default value = 0 Ohm/<sup>o</sup>K)

 TRB2:   Quadratic temperature variation coefficient of RB (default value = 0 Ohm/<sup>o</sup>K<sup>2</sup>)

 TRM1:   Linear temperature variation coefficient of RBM (default value = 0 Ohm/<sup>o</sup>K)

 TRM2:   Quadratic temperature variation coefficient of RBM (default value = 0 Ohm/<sup>o</sup>K<sup>2</sup>)

 Area:   Relative area occupied by resistor (default value = 1)

 Level:  Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)
</pre>
</html>
"));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Rb1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(Rb1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(Rb1.qb, qb) annotation (points=[10,-14; 20,-14; 20,80; 0,80;
                  0,100], style(color=74, rgbcolor={0,0,127}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-87; 0,-87; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end Rb;
          
          model Gj "Spice-style leakage conductance model" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Conductance GminDC=1e-16 
              "Leakage conductance";
            parameter Real Area=1 "Relative area occupied by device";
            parameter Integer Level=2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            
          protected 
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,-12; 12,12]);
            BondLib.Bonds.Bond B1 annotation (extent=[-38,-12; -12,12]);
            BondLib.Bonds.Bond B2 annotation (extent=[12,-12; 38,12]);
            BondLib.Bonds.Bond B3 annotation (extent=[-12,-36; 12,-12], rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Spice.Utilities.Gj Gj1(
              GminDC=GminDC,
              Area=Area,
              Level=Level) annotation (extent=[-10,-68; 10,-36], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0;
                  -12,0],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0;
                  12.26,0],  style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                  1.77636e-015; -38,0; -37.74,0],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                  1.77636e-015],
                      style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,-12; 0,
                  -12.24; 2.16021e-015,-12.24],
                                              style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Icon(
                Text(extent=[-40,86; 40,56],   string="%name"),
                Rectangle(extent=[-70,30; 70,-30],   style(
                    color=3,
                    fillColor=7,
                    fillPattern=1)),
                Line(points=[-90,0; -70,0]),
                Line(points=[70,0; 90,0]),
                Text(
                  extent=[-50,-40; 46,-80],
                  style(color=0),
                  string="G=%GminDC")),
              Documentation(info="<html>
The Spice-style leakage conductance model is a conductive source without temperature variation.  The <b>Level</b> parameter is used to acknowledge the additional <b>Area</b> parameter of the Gummel-Poon transistor model. No heat port is needed here, as the model is used only as a mathematical construct to compute the currents of the non-linear current sources. The heat generation is accounted for in the non-linear sources. <p>

<hr> <p>

<pre>
Parameters:

 GminDC:   Leakage conductance (default value = 1e-16 Mho)

 Area:     Relative area occupied by resistor (default value = 1)

 Level:    Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)
</pre>
</html>
"));
            connect(Gj1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,-36; -2.20429e-015,-36], style(color=8,
                  rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
          end Gj;
          
          model varC "Modulated electrical capacitor" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            parameter Real XCJ=1 
              "Fraction of depletion capacitance considered inside the junction";
            parameter Integer Level=2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,12; 12,-12]);
            Bonds.Bond B1         annotation (extent=[-38,-12; -12,12]);
            Bonds.Bond B2         annotation (extent=[12,-12; 38,12]);
            Bonds.Bond B3         annotation (extent=[-12,12; 12,36],   rotation=
                  90);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-14; -38,14]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -14; 64,14]);
            BondLib.Spice.Utilities.mC3 mC1(Level=Level, XCJ=XCJ) 
              annotation (extent=[-10,36; 10,60], rotation=90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput Cdepl(redeclare type 
                SignalType = 
                  Modelica.SIunits.Capacitance) "Depletion capacitance" 
              annotation (extent=[-60,90; -40,110], rotation=-90);
            Modelica.Blocks.Interfaces.RealInput Cdiff(redeclare type 
                SignalType = 
                  Modelica.SIunits.Capacitance) "Diffusion capacitance" 
              annotation (extent=[40,90; 60,110], rotation=-90);
            Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType 
                = Modelica.SIunits.Current) 
              "External base-collector capacitance current" 
              annotation (extent=[-10,100; 10,120], rotation=90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,0;
                  -12,0],                style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,0;
                  12.26,0],  style(color=8));
            connect(B3.BondCon1,J1p3_1. BondCon3) annotation (points=[
                  -7.20069e-016,12.24; -7.20069e-016,12.12; 0,12.12; 0,12],
                         style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,
                  1.77636e-015; -38,0; -37.74,0],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,0; 38,
                  1.77636e-015], style(color=8));
            annotation (
              Diagram,
              Icon(
                Text(extent=[-134,-96; 136,-36],    string="%name"),
                Line(points=[-90,0; -14,0]),
                Line(points=[14,0; 90,0]),
                Line(points=[0,100; 0,28],  style(color=73)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[14,28; 14,-28],   style(thickness=2))),
              Documentation(info="<html>
This modulated capacitor is built in bondgraphic technology from a bond graph <b>mC3</b> element. It implements the variable junction capacitor of an np-junction in a Spice-style bipolar transistor model. <p>

<hr>

<pre>
Potential variables:

 Cdepl:  Depletion capacitance (modulation input signal)

 Cdiff:  Diffusion capacitance (modulation input signal)


Parameters:

 XCJ:    Fraction of depletion capacitance considered inside the junction (default value = 1)

 Level:  Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)
</pre>
</html>
"));
            connect(mC1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,36; -6.12303e-016,38; 7.34764e-016,40;
                  7.34764e-016,36], style(color=8, rgbcolor={192,192,192}));
            connect(El2BG1.p, p) annotation (points=[-62,1.77636e-015; -80,
                  1.77636e-015; -80,0; -100,0], style(color=3, rgbcolor={0,0,
                    255}));
            connect(BG2El1.n, n) annotation (points=[64,1.77636e-015; 80,
                  1.77636e-015; 80,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(mC1.ix, ix) annotation (points=[-10,48; -20,48; -20,80; 0,
                  80; 0,110], style(color=74, rgbcolor={0,0,127}));
            connect(mC1.Cdepl, Cdepl) annotation (points=[10,54; 20,54; 20,60;
                  -50,60; -50,100], style(color=74, rgbcolor={0,0,127}));
            connect(mC1.Cdiff, Cdiff) annotation (points=[10,42; 50,42; 50,100],
                style(color=74, rgbcolor={0,0,127}));
          end varC;
          
          model Cj 
            "Spice-style junction capacitor model for bipolar transistors" 
            extends Interfaces.SpiceTwoPort;
            parameter Modelica.SIunits.Current IS=1e-16 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Modelica.SIunits.Capacitance CJ=0.5e-12 
              "Zero-bias depletion capacitance at reference temperature";
            parameter Real MJ=0.333 "Junction grading coefficient";
            parameter Modelica.SIunits.Voltage VJ=0.8 
              "Built-in potential at reference temperature";
            parameter Real XCJ=1 
              "Fraction of depletion capacitance considered inside the junction";
            parameter Real FC=0.5 
              "Depletion capacitance factor for linearization";
            parameter Modelica.SIunits.Time Tau=1e-9 "Ideal transit time";
            parameter Modelica.SIunits.Conductance GminDC=1e-15 
              "Leakage conductance";
            parameter Real Area=1 "Relative area occupied by capacitor";
            parameter Integer Level=2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            parameter Boolean BE=true "True if base-emitter junction";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Blocks.Interfaces.RealInput C0(redeclare type SignalType 
                = Modelica.SIunits.Capacitance) "Base capacitance" 
              annotation (extent=[-60,90; -40,110], rotation=-90);
            Modelica.Blocks.Interfaces.RealInput qb "Base charge" 
              annotation (extent=[40,90; 60,110], rotation=-90);
            Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType 
                = Modelica.SIunits.Current) 
              "External base-collector capacitance current" 
              annotation (extent=[-60,-110; -40,-90],
                                                   rotation=270);
            annotation (
              Diagram,
              Icon(
                Polygon(points=[30,0; -30,20; -30,-20; 30,0], style(
                    color=10,
                    rgbcolor={135,135,135},
                    gradient=0,
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[30,20; 30,-20], style(color=10, rgbcolor={135,135,
                        135})),
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42))),
              Documentation(info="<html>
The Spice-style junction capacitance computes both the depletion capacitance value and the diffusion capacitance value of a junction used in a bipolar transistor.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model (just as in the case of the simpler Ebers-Moll model).  The numerical error should remain small, as the time constants associated with temperature change are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2].  Since: <p>

<b>f = der(q)</b> <p>

the space charge, <b>q</b>, was symbolically differentiated.  In the differentiation, only electrical signals, i.e., the voltage, <b>e</b>, were considered time-varying.  The temperature gradients were assumed to be negligible.  This produced an equation of the form: <p>

<b>f = C(e)*der(e)</b> <p>

which is the formula that was used to compute the non-linear capacitance <b>C(e)</b> in the model. <p>

Notice that the junction capacitance assumes the function of the usually drawn junction diode.  However in Spice, the so-called junction diode only carries the capacitive current.  It computes the diodic current mathematically, but passes this information on as a modulating signal to the two non-linear current sources.  Thus, the entire diodic (Ohmic) current really flows through the two non-linear current sources, which truly are non-linear resistors. <p>

To reflect the mathematical reality of the model, a capacitor symbol was used to represent the junction capacitance in the icon layer.  However, to make the circuit diagram look more familiar to Spice users, a diode was drawn as a shadow over the capacitance. <p>

<hr>

<pre>
Parameters:

 Is:      Transport saturation current (default value = 1e-16 Amp)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent (default value = 3)

 CJ:      Zero-bias depletion capacitance at reference temperature (default value = 0.5e-12 F)

 MJ:      Junction grading coefficient (default value = 0.333)

 VJ:      Built-in potential at reference temperature (default value = 0.75 Volt)

 XCJ:     Fraction of base-collector depletion capacitance connected to internal base node (default value = 1)

 FC:      Depletion capacitance factor for linearization (default value = 0.5)

 Tau:     Ideal transit time (default value = 1e-9 sec)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 Level:   Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)

 BE:      True if base-emitter junction (default value = true)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.69.
</ol>
</html>"));
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vbi 
              annotation (extent=[-32,-12; -6,12],  rotation=-90);
            BondLib.Electrical.Analog.Basic.VCV VCV1(gain=1) 
              annotation (extent=[26,22; 0,-22],  rotation=-180);
            BondLib.Electrical.Analog.Sensors.CurrentSensor Id 
              annotation (extent=[32,20; 52,40],     rotation=0);
            BondLib.Electrical.Analog.Basic.Ground Gnd1 
                                 annotation (extent=[50,-50; 70,-30]);
            BondLib.Electrical.Analog.Spice.Utilities.varC varC1(XCJ=XCJ, Level=
                 Level) annotation (extent=[-90,-10; -70,10], rotation=-90);
            BondLib.Spice.Utilities.nlC nlC2(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              N=N,
              XTI=XTI,
              VJ=VJ,
              CJ=CJ,
              MJ=MJ,
              FC=FC,
              Tau=Tau,
              GminDC=GminDC,
              Area=Area,
              Level=Level,
              BE=BE,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-40,10; -60,-10]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-20,-80; -40,-60]);
            Modelica.Blocks.Interfaces.RealOutput id(redeclare type SignalType 
                = Modelica.SIunits.Current) "Diode current" 
              annotation (extent=[40,-110; 60,-90],rotation=270);
            BondLib.Electrical.Analog.Spice.Utilities.Dj Dj1(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              N=N,
              XTI=XTI,
              Area=Area,
              Level=Level,
              EMin=EMin,
              EMax=EMax) annotation (extent=[50,-20; 70,20], rotation=-90);
            BondLib.Electrical.Analog.Spice.Utilities.Gj Gj1(
              GminDC=GminDC,
              Area=Area,
              Level=Level) annotation (extent=[70,-20; 90,20], rotation=-90);
          equation 
            connect(VCV1.p1,Vbi. p) annotation (points=[0,11; 0,30; -19,30; -19,
                  12], style(color=3, rgbcolor={0,0,255}));
            connect(VCV1.n1,Vbi. n) annotation (points=[0,-11; 0,-30; -19,-30;
                  -19,-12], style(color=3, rgbcolor={0,0,255}));
            connect(VCV1.n2,Gnd1. p)    annotation (points=[26,-11; 26,-30; 60,
                  -30], style(color=3, rgbcolor={0,0,255}));
            connect(varC1.p, Vbi.p) annotation (points=[-80,10; -80,30; -19,30;
                  -19,12], style(color=3, rgbcolor={0,0,255}));
            connect(varC1.n, Vbi.n) annotation (points=[-80,-10; -80,-30; -19,
                  -30; -19,-12], style(color=3, rgbcolor={0,0,255}));
            connect(Vbi.V, nlC2.v) annotation (points=[-32,-7.34764e-016; -36,
                  -7.34764e-016; -36,0; -40,0], style(color=74, rgbcolor={0,0,
                    127}));
            connect(nlC2.C0, C0) annotation (points=[-55,10; -55,80; -50,80;
                  -50,100], style(color=74, rgbcolor={0,0,127}));
            connect(nlC2.qb, qb) annotation (points=[-45,10; -46,10; -46,72; 50,
                  72; 50,100], style(color=74, rgbcolor={0,0,127}));
            connect(Tdev1.port, heatPort)              annotation (points=[-20,
                  -70; 0,-70; 0,-100], style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.T, nlC2.Tdev)              annotation (points=[-40,
                  -70; -50,-70; -50,-10], style(color=74, rgbcolor={0,0,127}));
            connect(varC1.ix, ix) annotation (points=[-69,6.73533e-016; -66,
                  6.73533e-016; -66,-80; -50,-80; -50,-100], style(color=74,
                  rgbcolor={0,0,127}));
            connect(Id.I, id) annotation (points=[42,20; 42,-80; 50,-80; 50,
                  -100], style(color=74, rgbcolor={0,0,127}));
            connect(Dj1.n, Gnd1.p) annotation (points=[60,-20; 60,-30], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Dj1.heatPort, heatPort) annotation (points=[50,
                  -1.22461e-015; 50,-70; 0,-70; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
            connect(Gj1.n, Gnd1.p) annotation (points=[80,-20; 80,-30; 60,-30],
                style(color=3, rgbcolor={0,0,255}));
            connect(Id.p, VCV1.p2) annotation (points=[32,30; 26,30; 26,11],
                style(color=3, rgbcolor={0,0,255}));
            connect(Id.n, Dj1.p) annotation (points=[52,30; 60,30; 60,20],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dj1.p, Gj1.p) annotation (points=[60,20; 60,30; 80,30; 80,
                  20], style(color=3, rgbcolor={0,0,255}));
            connect(p, varC1.p) annotation (points=[-100,0; -92,0; -92,30; -80,
                  30; -80,10], style(color=3, rgbcolor={0,0,255}));
            connect(n, varC1.n) annotation (points=[100,0; 92,0; 92,-50; -80,
                  -50; -80,-10], style(color=3, rgbcolor={0,0,255}));
            connect(nlC2.Cdepl, varC1.Cdepl) annotation (points=[-61,5; -65.5,5;
                  -65.5,5; -70,5], style(color=74, rgbcolor={0,0,127}));
            connect(nlC2.Cdiff, varC1.Cdiff) annotation (points=[-61,-5; -65.5,
                  -5; -65.5,-5; -70,-5], style(color=74, rgbcolor={0,0,127}));
          end Cj;
          
          model Cbx 
            "Spice-style external base-collector capacitance model for bipolar transistors" 
            extends Interfaces.SpiceTwoPort;
            annotation (
              Icon(
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42)),
                Line(points=[0,20; 0,-20], style(pattern=2, thickness=2)),
                Text(
                  extent=[-42,30; -16,2],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+"),
                Text(
                  extent=[18,32; 44,4],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-")),
              Diagram,
              Documentation(info="<html>
The Spice-style external base-collector capacitance computes the external depletion capacitance value of a bipolar transistor.  The model is built as a non-linear current source to prevent an unnecessary higher index DAE system.
</html>
"));
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput ix(redeclare type SignalType 
                = Modelica.SIunits.Current) 
              "External base-collector capacitance current" 
              annotation (extent=[-10,90; 10,110], rotation=270);
            
          protected 
            BondLib.Spice.Utilities.CbxS Cbx 
              annotation (extent=[-10,-32; 10,4], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Cbx.BondCon1, B3.BondCon2)   annotation (points=[
                  -6.12303e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(Cbx.BondCon2, B4.fBondCon1)   annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(Cbx.ix, ix)   annotation (points=[-10,-14; -20,-14; -20,70;
                  0,70; 0,100], style(color=74, rgbcolor={0,0,127}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-87; 0,-87; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end Cbx;
          
          model Dj "Spice-style junction diode model for bipolar transistors" 
            extends Interfaces.SpiceTwoPort;
            parameter Modelica.SIunits.Current IS=1e-16 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Real Area=1 "Relative area occupied by diode";
            parameter Integer Level=2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            annotation (
              Diagram,
              Documentation(info="<html>
The internal junction diode is used inside the junction capacitance model to compute the diodic current, which however is only used as a mathematical construct, i.e., doesn't carry any power.  The diodic current, once computed, is then passed on as a modulating signal to the two non-linear current sources that actually carry the entire diodic (Ohmic) current of the bipolar junction transistor. <p>

<hr>

<pre>
Parameters:

 Is:      Transport saturation current (default value = 1e-16 Amp)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent (default value = 3)

 Area:    Relative area occupied by the diode (default value = 1)

 Level:   Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2) (default value = 2)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre>
</html>
"),           Icon(
                Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                    color=3,
                    gradient=0,
                    fillColor=7)),
                Line(points=[-90,0; -30,0]),
                Line(points=[30,0; 90,0]),
                Line(points=[30,40; 30,-40],   style(color=3)),
                Line(points=[0,-20; 0,-91],   style(color=42)),
                Text(
                  extent=[-24,14; 4,-14],
                  style(color=3, rgbcolor={0,0,255}),
                  string="J")));
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            BondLib.Spice.Utilities.DjS DjS1(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              N=N,
              XTI=XTI,
              Area=Area,
              Level=Level,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-10,-32; 10,4], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(B3.BondCon2, DjS1.BondCon1) annotation (points=[
                  -2.20429e-015,4; -6.12303e-016,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(DjS1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-87; 0,-87; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end Dj;
          
          model Isignal 
            "Modulated internal current source with heat computation" 
            extends BondLib.Electrical.Analog.Interfaces.OnePort;
            annotation (
              Icon(
                Ellipse(extent=[-50,50; 50,-50],   style(fillColor=7)),
                Line(points=[-90,0; -50,0]),
                Line(points=[0,-50; 0,50]),
                Line(points=[50,0; 90,0]),
                Text(extent=[-100,-88; 100,-48],    string="%name")),
              Diagram,
              Documentation(info="<html>
The internal modulated current source of Spice-style transistors is built in bondgraphic technology from a bond graph <b>mSf_Int</b> element.  The internal modulated current source of Spice-style transistors is in fact a non-linear resistor, rather than a source, since voltage and current are always pointing in the same direction. <p>

The internal modulated current source also has a heat port to put out the heat that is being generated by the resistor.
</html>
"));
            Modelica.Blocks.Interfaces.RealInput InPort1(redeclare type 
                SignalType = 
                  Modelica.SIunits.Current) "Current source" 
                                                      annotation (extent=[-10,
                  50; 10,70],rotation=-90);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            
          protected 
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,-74; -38,-46]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  -74; 64,-46]);
            BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-12,-48; 12,-72]);
            BondLib.Bonds.Bond B1 annotation (extent=[12,-72; 38,-48],  rotation=
                  180);
            BondLib.Bonds.Bond B2 annotation (extent=[-38,-72; -12,-48],
                rotation=180);
            BondLib.Bonds.eBond B3 annotation (extent=[-12,-48; 12,-24], rotation=
                 -90);
            BondLib.Sources.mSf_int I1 
              annotation (extent=[-10,-24; 10,0], rotation=270);
            BondLib.Bonds.fBond B4 
              annotation (extent=[-10,0; 10,20], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[38,16; 64,44], rotation=0);
          equation 
            connect(J1p3_1.BondCon2,B1. BondCon2) annotation (points=[12,-60;
                  12,-60],style(color=8));
            connect(BG2El1.BondCon1,B1. BondCon1) annotation (points=[38,-60;
                  37.74,-60],
                          style(color=8));
            connect(El2BG1.BondCon1,B2. BondCon2) annotation (points=[-38,-60;
                  -38,-60], style(color=8));
            connect(J1p3_1.BondCon1,B2. BondCon1) annotation (points=[-12,-60;
                  -12.26,-60],
                            style(color=8));
            connect(B3.eBondCon1,J1p3_1. BondCon3) annotation (points=[
                  7.34764e-016,-48; 0,-48], style(color=8));
            connect(El2BG1.p, p) annotation (points=[-62,-60; -80,-60; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,-60; 80,-60; 80,0; 100,
                  0], style(color=3, rgbcolor={0,0,255}));
            connect(I1.BondCon2, B3.fBondCon1) annotation (points=[
                  -1.83691e-015,-24; -7.34764e-016,-24], style(color=8,
                  rgbcolor={192,192,192}));
            connect(B4.fBondCon1, I1.BondCon1) annotation (points=[6.12303e-016,
                  0; 1.83691e-015,0], style(color=8, rgbcolor={192,192,192}));
            connect(B4.eBondCon1, BG2Heat1.BondCon1) annotation (points=[
                  -6.12303e-016,20; 0,20; 0,30; 38,30], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[64,30; 70,30;
                  70,-80; 0,-80; 0,-100], style(color=42, rgbcolor={191,0,0}));
            connect(InPort1, I1.s) annotation (points=[0,60; 0,40; -20,40; -20,
                  -12; -9,-12], style(color=74, rgbcolor={0,0,127}));
          end Isignal;
          
          model Qni 
            "Spice-style NPN transistor model without parasitic resistors and substrate diode" 
            /* DC model parameters */
            constant Real inf=Modelica.Constants.inf;
            parameter Real BF=100 
              "Maximum forward current gain at reference temperature";
            parameter Real BR=1 
              "Maximum reverse current gain at reference temperature";
            parameter Modelica.SIunits.Current IS=1e-16 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Current ISS=IS 
              "Saturation current used for current injection";
            parameter Real NF=1 "Forward current emission coefficient";
            parameter Real NR=1 "Reverse current emission coefficient";
            parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
              "Leakage conductance";
            
            /* Low current beta degradation effect parameters */
            parameter Modelica.SIunits.Current ISC(min=0) = 0 
              "Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
            parameter Modelica.SIunits.Current ISE(min=0) = 0 
              "Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
            parameter Real NC=2 
              "Low-current base-collector leakage emission coefficient";
            parameter Real NE=1.5 
              "Low-current base-emitter leakage emission coefficient";
            
              /* Base width modulation parameters */
            parameter Modelica.SIunits.Voltage VAF=inf "Forward early voltage";
            parameter Modelica.SIunits.Voltage VAR=inf "Reverse early voltage";
            
            /* High current beta degradation effect parameters */
            parameter Modelica.SIunits.Current IKF=inf 
              "Corner for forward beta high-current roll-off";
            parameter Modelica.SIunits.Current IKR=inf 
              "Corner for reverse beta high-current roll-off";
            
            /* Junction capacitor parameters */
            parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
              "Zero-bias base-collector depletion capacitance at reference temperature";
            parameter Real MJC(min=0.3,max=0.5) = 0.33 
              "Base-collector junction grading coefficient";
            parameter Modelica.SIunits.Voltage VJC=0.8 
              "Base-collector built-in potential at reference temperature";
            parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
              "Zero-bias base-emitter depletion capacitance at reference temperature";
            parameter Real MJE(min=0.3,max=0.5) = 0.33 
              "Base-emitter junction grading coefficient";
            parameter Modelica.SIunits.Voltage VJE=0.75 
              "Base-emitter built-in potential at reference temperature";
            parameter Real XCJC(min=0,max=1) = 1 
              "Fraction of base-collector depletion capacitance connected to internal base node";
            parameter Real FC(min=0,max=1) = 0.5 
              "Depletion capacitance factor for linearization";
            
            /* Transit time parameters */
            parameter Modelica.SIunits.Time TF(min=0) = 0 
              "Ideal forward transit time";
            parameter Modelica.SIunits.Time TR(min=0) = 0 
              "Ideal reverse transit time";
            
            /* Temperature compensation and area parameters */
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Real XTB=0 
              "Forward and reverse beta temperature coefficient";
            parameter Modelica.SIunits.Voltage EG=1.11 
              "Energy gap for temperature effect on saturation current";
            parameter Real Area=1 "Relative area occupied by device";
            parameter Integer Level(min=1,max=2) = 2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            
            /* Numerical parameters */
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Electrical.Analog.Interfaces.Pin Bint "Internal base node"
                                                               annotation (extent=[
                  -90,-10; -110,10]);
          public 
            Modelica.Electrical.Analog.Interfaces.Pin Cint 
              "Internal collector node"                             annotation (extent=[90,
                  40; 110,60]);
            annotation (
              Diagram(
                Ellipse(extent=[-12,2; -8,-2], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(
                  extent=[-98,78; -86,72],
                  string="vbc",
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1)),
                Text(
                  extent=[-98,70; -86,64],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="ibc"),
                Text(
                  extent=[-98,62; -86,56],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="vbe"),
                Text(
                  extent=[-98,54; -86,48],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="ibe"),
                Text(
                  extent=[-38,78; -26,72],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="IB0"),
                Text(
                  extent=[-38,70; -26,64],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="IC0"),
                Text(
                  extent=[-38,62; -26,56],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="qb"),
                Text(
                  extent=[-38,54; -26,48],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="C0"),
                Text(
                  extent=[-60,22; -48,16],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="Tdev"),
                Text(
                  extent=[-50,96; -38,90],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="ix")),
              Icon(
                Line(points=[-10,40; -10,-40]),
                Line(points=[-10,0; -90,0]),
                Line(points=[91,50; 30,50]),
                Line(points=[30,50; -10,10]),
                Line(points=[-10,-10; 30,-50]),
                Line(points=[30,-50; 91,-50]),
                Polygon(points=[30,-50; 24,-36; 16,-44; 30,-50],     style(fillColor=3,
                        fillPattern=1)),
                Text(extent=[-100,66; 100,94],    string="%name"),
                Line(points=[0,-40; 0,-91],   style(color=42))),
              Documentation(info="<html>
The NPN element of the Spice analog electrical library implements a full-fledges Spice-stye Gummel-Poon model of the NPN bipolar transistor [1-3].  <b>NPNint</b> is a partial model that implements only the internal nodes of the bipolar transistor.  The external parasitic resistances, the external capacitor between base and collector, and the substrate are being added from the outside. <p>

<hr>

<pre>
<b>Parameters:</b>

 Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100)

 BR:      Maximum reverse current gain at reference temperature (default value = 1)

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp)

 ISS:     Saturation current for injection (default value = IS Amp)

 NF:      Forward current emission coefficient (default value = 1)

 NR:      Reverse current emission coefficient (default value = 1)

 GminDC:  Leakage conductance (default value = 1e-19 Mho)


<b>Low Current Beta Degradation Effect Parameters:</b>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp)

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp)

 NC:      Low-current base-collector leakage emission coefficient (default value = 2)

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5)


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = 9e30 Volt)

 VAR:     Reverse early voltage (default value = 9e30 Volt)


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = 9e30 Amp)

 IKR:     Corner for reverse beta high-current roll-off (default value = 9e30 Amp)


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (default value = 1e-12 F)

 MJC:     Base-collector junction grading coefficient (default value = 0.33)

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt)

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (default value = 1e-12 F)

 MJE:     Base-emitter junction grading coefficient (default value = 0.33)

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt)

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1)

 FC:      Depletion capacitance factor for linearization (default value = 0.5)


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec)

 TR:      Ideal reverse transit time (default value = 0 sec)


<b>Temperature Compensation and Area Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K)

 XTI:     Saturation current temperature exponent (default value = 3)

 XTB:     Forward and reverse beta temperature coefficient (default value = 0)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 Area:    Relative area occupied by device (default value = 1)


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100)

 EMax:    Maximum exponent for linearization of junction current (default value = 40)


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true)
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
            Modelica.Electrical.Analog.Interfaces.Pin Eint 
              "Internal emitter node"                             annotation (extent=[90,
                  -40; 110,-60]);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            BondLib.Spice.Utilities.BJTvars BJTvars1(
            Tnom=Tnom,
            BF=BF,
            BR=BR,
            TF=TF,
            EG=EG,
            XTI=XTI,
            VAF=VAF,
            VAR=VAR,
            IKF=IKF,
            IKR=IKR,
            ISC=ISC,
            ISE=ISE,
            NC=NC,
            NE=NE,
            XTB=XTB,
            Area=Area,
              IS=IS,
              ISS=ISS,
              Level=Level) 
            annotation (extent=[-80,40; -40,80]);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vbc 
              annotation (extent=[0,20; -20,40], rotation=90);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vbe 
              annotation (extent=[-30,-20; -50,-40], rotation=90);
          Modelica.Blocks.Sources.Constant qb1(k=1) 
            annotation (extent=[20,60; 32,72]);
          Modelica.Blocks.Sources.Constant C0(k=0) 
            annotation (extent=[20,80; 32,92]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev 
              annotation (extent=[-30,-80; -50,-60]);
            Modelica.Blocks.Interfaces.RealOutput qb "Base charge" 
              annotation (extent=[100,-10; 120,10]);
            Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType 
                = Modelica.SIunits.Current) 
              "External base-collector capacitance current" 
              annotation (extent=[-60,100; -40,120], rotation=90);
            Isignal IC0 annotation (extent=[50,-10; 70,10], rotation=-90);
            Isignal IB0 annotation (extent=[-20,-40; 0,-20], rotation=-90);
            Cj Dbc(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              N=NR,
              XTI=XTI,
              CJ=CJC,
              MJ=MJC,
              VJ=VJC,
              XCJ=XCJC,
              FC=FC,
              Tau=TR,
              GminDC=GminDC,
              Area=Area,
              Level=Level,
              BE=false,
              EMin=EMin,
              EMax=EMax,
              enforceStates=enforceStates) 
                         annotation (extent=[30,10; 10,50], rotation=90);
            Cj Dbe(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              XTI=XTI,
              FC=FC,
              GminDC=GminDC,
              Area=Area,
              Level=Level,
              EMin=EMin,
              EMax=EMax,
              N=NF,
              CJ=CJE,
              MJ=MJE,
              VJ=VJE,
              XCJ=1,
              Tau=TF,
              BE=true,
              enforceStates=enforceStates) 
                       annotation (extent=[30,-10; 10,-50], rotation=90);
          equation 
            connect(Vbc.p, Bint) annotation (points=[-10,20; -10,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Vbe.p, Bint) annotation (points=[-40,-20; -40,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Vbc.V, BJTvars1.vbc) annotation (points=[-20,30; -100,30;
                  -100,72; -82,72], style(color=74, rgbcolor={0,0,127}));
            connect(Tdev.port, heatPort) annotation (points=[-30,-70; 0,-70; 0,
                  -100], style(
                color=42,
                rgbcolor={191,0,0},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Tdev.T, BJTvars1.Tdev) annotation (points=[-50,-70; -60,-70;
                  -60,38], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(BJTvars1.qb, qb) annotation (points=[-38,56; 80,56; 80,0;
                  110,0], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(IC0.p, Cint) annotation (points=[60,10; 60,50; 100,50],
                style(color=3, rgbcolor={0,0,255}));
            connect(IC0.n, Eint) annotation (points=[60,-10; 60,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(BJTvars1.ic0, IC0.InPort1) annotation (points=[-38,64; -20,
                  64; -20,98; 76,98; 76,0; 72,0; 72,3.67382e-016; 66,
                  3.67382e-016],                                        style(
                  color=74, rgbcolor={0,0,127}));
            connect(IB0.p, Bint) annotation (points=[-10,-20; -10,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(BJTvars1.ib0, IB0.InPort1) annotation (points=[-38,72; -22,
                  72; -22,100; 78,100; 78,-60; 2,-60; 2,-30; -4,-30], style(
                  color=74, rgbcolor={0,0,127}));
            connect(Dbc.p, Bint) annotation (points=[20,10; 20,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dbc.n, Cint) annotation (points=[20,50; 100,50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Vbc.n, Dbc.n) annotation (points=[-10,40; -10,50; 20,50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dbc.ix, ix) annotation (points=[10,20; 2,20; 2,90; -50,90;
                  -50,110], style(color=74, rgbcolor={0,0,127}));
            connect(Dbc.id, BJTvars1.ibc) annotation (points=[10,40; 4,40; 4,88;
                  -98,88; -98,64; -82,64], style(color=74, rgbcolor={0,0,127}));
            connect(C0.y, Dbc.C0) annotation (points=[32.6,86; 42,86; 42,20; 30,
                  20], style(color=74, rgbcolor={0,0,127}));
            connect(qb1.y, Dbc.qb) annotation (points=[32.6,66; 40,66; 40,40;
                  30,40], style(color=74, rgbcolor={0,0,127}));
            connect(Dbe.p, Bint) annotation (points=[20,-10; 20,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dbe.n, Eint) annotation (points=[20,-50; 100,-50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(IB0.n, Dbe.n) annotation (points=[-10,-40; -10,-50; 20,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dbe.C0, BJTvars1.C0) annotation (points=[30,-20; 36,-20; 36,
                  4; -26,4; -26,48; -38,48], style(color=74, rgbcolor={0,0,127}));
            connect(Dbe.qb, BJTvars1.qb) annotation (points=[30,-40; 38,-40; 38,
                  6; -24,6; -24,56; -38,56], style(color=74, rgbcolor={0,0,127}));
            connect(Dbe.id, BJTvars1.ibe) annotation (points=[10,-40; 4,-40; 4,
                  -10; -70,-10; -70,32; -96,32; -96,48; -82,48], style(color=74,
                  rgbcolor={0,0,127}));
            connect(Dbe.heatPort, Tdev.port) annotation (points=[10,-30; 6,-30;
                  6,-70; -30,-70], style(color=42, rgbcolor={191,0,0}));
            connect(Dbc.heatPort, Dbe.heatPort) annotation (points=[10,30; 6,30;
                  6,-30; 10,-30], style(color=42, rgbcolor={191,0,0}));
            connect(Vbe.n, IB0.n) annotation (points=[-40,-40; -40,-50; -10,-50;
                  -10,-40], style(color=3, rgbcolor={0,0,255}));
            connect(IB0.heatPort, Tdev.port) annotation (points=[-20,-30; -24,
                  -30; -24,-70; -30,-70], style(color=42, rgbcolor={191,0,0}));
            connect(IC0.heatPort, heatPort) annotation (points=[50,
                  -6.12303e-016; 48,-6.12303e-016; 48,0; 46,0; 46,-70; 0,-70; 0,
                  -100], style(color=42, rgbcolor={191,0,0}));
            connect(Vbe.V, BJTvars1.vbe) annotation (points=[-50,-30; -72,-30;
                  -72,28; -98,28; -98,56; -82,56], style(color=74, rgbcolor={0,
                    0,127}));
          end Qni;
          
          model Qpi 
            "Spice-style PNP transistor model without parasitic resistors and substrate diode" 
            /* DC model parameters */
            constant Real inf=Modelica.Constants.inf;
            parameter Real BF=100 
              "Maximum forward current gain at reference temperature";
            parameter Real BR=1 
              "Maximum reverse current gain at reference temperature";
            parameter Modelica.SIunits.Current IS=1e-16 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Current ISS=IS 
              "Saturation current used for current injection";
            parameter Real NF=1 "Forward current emission coefficient";
            parameter Real NR=1 "Reverse current emission coefficient";
            parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
              "Leakage conductance";
            
            /* Low current beta degradation effect parameters */
            parameter Modelica.SIunits.Current ISC(min=0) = 0 
              "Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
            parameter Modelica.SIunits.Current ISE(min=0) = 0 
              "Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
            parameter Real NC=2 
              "Low-current base-collector leakage emission coefficient";
            parameter Real NE=1.5 
              "Low-current base-emitter leakage emission coefficient";
            
              /* Base width modulation parameters */
            parameter Modelica.SIunits.Voltage VAF=inf "Forward early voltage";
            parameter Modelica.SIunits.Voltage VAR=inf "Reverse early voltage";
            
            /* High current beta degradation effect parameters */
            parameter Modelica.SIunits.Current IKF=inf 
              "Corner for forward beta high-current roll-off";
            parameter Modelica.SIunits.Current IKR=inf 
              "Corner for reverse beta high-current roll-off";
            
            /* Junction capacitor parameters */
            parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
              "Zero-bias base-collector depletion capacitance at reference temperature";
            parameter Real MJC(min=0.3,max=0.5) = 0.33 
              "Base-collector junction grading coefficient";
            parameter Modelica.SIunits.Voltage VJC=0.8 
              "Base-collector built-in potential at reference temperature";
            parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
              "Zero-bias base-emitter depletion capacitance at reference temperature";
            parameter Real MJE(min=0.3,max=0.5) = 0.33 
              "Base-emitter junction grading coefficient";
            parameter Modelica.SIunits.Voltage VJE=0.75 
              "Base-emitter built-in potential at reference temperature";
            parameter Real XCJC(min=0,max=1) = 1 
              "Fraction of base-collector depletion capacitance connected to internal base node";
            parameter Real FC(min=0,max=1) = 0.5 
              "Depletion capacitance factor for linearization";
            
            /* Transit time parameters */
            parameter Modelica.SIunits.Time TF(min=0) = 0 
              "Ideal forward transit time";
            parameter Modelica.SIunits.Time TR(min=0) = 0 
              "Ideal reverse transit time";
            
            /* Temperature compensation and area parameters */
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Real XTB=0 
              "Forward and reverse beta temperature coefficient";
            parameter Modelica.SIunits.Voltage EG=1.11 
              "Energy gap for temperature effect on saturation current";
            parameter Real Area=1 "Relative area occupied by device";
            parameter Integer Level(min=1,max=2) = 2 
              "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
            
            /* Numerical parameters */
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Electrical.Analog.Interfaces.Pin Bint "Internal base node"
                                                               annotation (extent=[
                  -90,-10; -110,10]);
          public 
            Modelica.Electrical.Analog.Interfaces.Pin Cint 
              "Internal collector node"                             annotation (extent=[90,
                  40; 110,60]);
            Modelica.Electrical.Analog.Interfaces.Pin Eint 
              "Internal emitter node"                             annotation (extent=[90,
                  -40; 110,-60]);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            BondLib.Spice.Utilities.BJTvars BJTvars1(
            Tnom=Tnom,
            BF=BF,
            BR=BR,
            TF=TF,
            EG=EG,
            XTI=XTI,
            VAF=VAF,
            VAR=VAR,
            IKF=IKF,
            IKR=IKR,
            ISC=ISC,
            ISE=ISE,
            NC=NC,
            NE=NE,
            XTB=XTB,
            Area=Area,
              IS=IS,
              ISS=ISS,
              Level=Level) 
            annotation (extent=[-80,40; -40,80]);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Veb 
              annotation (extent=[-30,-40; -50,-20], rotation=90);
          Modelica.Blocks.Sources.Constant qb1(k=1) 
            annotation (extent=[20,80; 32,92]);
          Modelica.Blocks.Sources.Constant C0(k=0) 
            annotation (extent=[20,60; 32,72]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev 
              annotation (extent=[-30,-80; -50,-60]);
            annotation (
              Diagram(
                Ellipse(extent=[-12,2; -8,-2], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(
                  extent=[-98,78; -86,72],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="vcb"),
                Text(
                  extent=[-98,70; -86,64],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="icb"),
                Text(
                  extent=[-98,62; -86,56],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="veb"),
                Text(
                  extent=[-98,54; -86,48],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="ieb"),
                Text(
                  extent=[-38,78; -26,72],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="IB0"),
                Text(
                  extent=[-38,70; -26,64],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="IC0"),
                Text(
                  extent=[-38,62; -26,56],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="qb"),
                Text(
                  extent=[-38,54; -26,48],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="C0"),
                Text(
                  extent=[-60,22; -48,16],
                  style(
                    color=10,
                    rgbcolor={95,95,95},
                    fillColor=3,
                    rgbfillColor={0,0,255},
                    fillPattern=1),
                  string="Tdev")),
              Icon(
                Line(points=[-10,40; -10,-40]),
                Line(points=[-10,0; -90,0]),
                Line(points=[91,50; 30,50]),
                Line(points=[30,50; -10,10]),
                Line(points=[-10,-10; 30,-50]),
                Line(points=[30,-50; 91,-50]),
                Polygon(points=[-10,-10; 5,-17; -3,-25; -10,-10],     style(fillColor=
                        3, fillPattern=1)),
                Line(points=[0,-40; 0,-91],   style(color=42)),
                Text(extent=[-102,66; 102,94],    string="%name")),
              Documentation(info="<html>
The PNP element of the Spice bond graph library implements a full-fledges Spice-stye Gummel-Poon model of the PNP bipolar transistor [1-3].  <b>PNPint</b> is a partial model that implements only the internal nodes of the bipolar transistor.  The external parasitic resistances, the external capacitor between base and collector, and the substrate are being added from the outside. <p>

<hr>

<pre>
<b>Parameters:</b>

 Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100)

 BR:      Maximum reverse current gain at reference temperature (default value = 1)

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp)

 ISS:     Saturation current for injection (default value = IS Amp)

 NF:      Forward current emission coefficient (default value = 1)

 NR:      Reverse current emission coefficient (default value = 1)

 GminDC:  Leakage conductance (default value = 1e-19 Mho)


<b>Low Current Beta Degradation Effect Parameters:</b>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp)

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp)

 NC:      Low-current base-collector leakage emission coefficient (default value = 2)

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5)


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = 9e30 Volt)

 VAR:     Reverse early voltage (default value = 9e30 Volt)


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = 9e30 Amp)

 IKR:     Corner for reverse beta high-current roll-off (default value = 9e30 Amp)


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (default value = 1e-12 F)

 MJC:     Base-collector junction grading coefficient (default value = 0.33)

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt)

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (default value = 1e-12 F)

 MJE:     Base-emitter junction grading coefficient (default value = 0.33)

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt)

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1)

 FC:      Depletion capacitance factor for linearization (default value = 0.5)


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec)

 TR:      Ideal reverse transit time (default value = 0 sec)


<b>Temperature Compensation and Area Parameters:</b>

 Tnom:    Reference temperature (default value = 300.15 K)

 XTI:     Saturation current temperature exponent (default value = 3)

 XTB:     Forward and reverse beta temperature coefficient (default value = 0)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt)

 Area:    Relative area occupied by device (default value = 1)


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100)

 EMax:    Maximum exponent for linearization of junction current (default value = 40)


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true)
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
            Modelica.Blocks.Interfaces.RealOutput qb "Base charge" 
              annotation (extent=[100,-10; 120,10]);
            Modelica.Blocks.Interfaces.RealOutput ix(redeclare type SignalType 
                = Modelica.SIunits.Current) 
              "External base-collector capacitance current" 
              annotation (extent=[-60,100; -40,120], rotation=90);
            Isignal IC0 annotation (extent=[50,10; 70,-10], rotation=-90);
            Isignal IB0 annotation (extent=[-20,-20; 0,-40], rotation=-90);
            Cj Dcb(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              N=NR,
              XTI=XTI,
              CJ=CJC,
              MJ=MJC,
              VJ=VJC,
              XCJ=XCJC,
              FC=FC,
              Tau=TR,
              GminDC=GminDC,
              Area=Area,
              Level=Level,
              BE=false,
              EMin=EMin,
              EMax=EMax,
              enforceStates=enforceStates) 
                         annotation (extent=[30,50; 10,10], rotation=90);
            Cj Deb(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              XTI=XTI,
              FC=FC,
              GminDC=GminDC,
              Area=Area,
              Level=Level,
              EMin=EMin,
              EMax=EMax,
              N=NF,
              CJ=CJE,
              MJ=MJE,
              VJ=VJE,
              XCJ=1,
              Tau=TF,
              BE=true,
              enforceStates=enforceStates) 
                       annotation (extent=[30,-50; 10,-10], rotation=90);
            Sensors.VoltageSensor Vcb 
              annotation (extent=[0,40; -20,20], rotation=90);
          equation 
            connect(Tdev.port, heatPort) annotation (points=[-30,-70; 0,-70; 0,
                  -100], style(
                color=42,
                rgbcolor={191,0,0},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Tdev.T, BJTvars1.Tdev) annotation (points=[-50,-70; -60,-70;
                  -60,38], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Veb.n, Bint) annotation (points=[-40,-20; -40,0; -100,0],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(BJTvars1.qb, qb) annotation (points=[-38,56; 80,56; 80,0;
                  110,0], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(IC0.n, Cint) annotation (points=[60,10; 60,50; 100,50],
                style(color=3, rgbcolor={0,0,255}));
            connect(IC0.p, Eint) annotation (points=[60,-10; 60,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(BJTvars1.ic0, IC0.InPort1) annotation (points=[-38,64; -20,
                  64; -20,98; 74,98; 74,-3.67382e-016; 66,-3.67382e-016], style(
                  color=74, rgbcolor={0,0,127}));
            connect(IB0.n, Bint) annotation (points=[-10,-20; -10,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(BJTvars1.ib0, IB0.InPort1) annotation (points=[-38,72; -22,
                  72; -22,100; 76,100; 76,-60; 0,-60; 0,-30; -4,-30], style(
                  color=74, rgbcolor={0,0,127}));
            connect(IB0.heatPort, Tdev.port) annotation (points=[-20,-30; -24,
                  -30; -24,-70; -30,-70], style(color=42, rgbcolor={191,0,0}));
            connect(Deb.heatPort, heatPort) annotation (points=[10,-30; 6,-30;
                  6,-70; 0,-70; 0,-100], style(color=42, rgbcolor={191,0,0}));
            connect(Dcb.heatPort, Deb.heatPort) annotation (points=[10,30; 6,30;
                  6,-30; 10,-30], style(color=42, rgbcolor={191,0,0}));
            connect(IC0.heatPort, heatPort) annotation (points=[50,6.12303e-016;
                  48,6.12303e-016; 48,0; 46,0; 46,-70; 0,-70; 0,-100], style(
                  color=42, rgbcolor={191,0,0}));
            connect(Dcb.p, Cint) annotation (points=[20,50; 100,50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Vcb.p, Dcb.p) annotation (points=[-10,40; -10,50; 20,50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Vcb.n, Bint) annotation (points=[-10,20; -10,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dcb.n, Bint) annotation (points=[20,10; 20,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Deb.n, Bint) annotation (points=[20,-10; 20,0; -100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Deb.p, Eint) annotation (points=[20,-50; 100,-50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(IB0.p, Deb.p) annotation (points=[-10,-40; -10,-50; 20,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Veb.p, IB0.p) annotation (points=[-40,-40; -40,-50; -10,-50;
                  -10,-40], style(color=3, rgbcolor={0,0,255}));
            connect(Vcb.V, BJTvars1.vbc) annotation (points=[-20,30; -100,30;
                  -100,72; -82,72], style(color=74, rgbcolor={0,0,127}));
            connect(Dcb.ix, ix) annotation (points=[10,40; 4,40; 4,90; -50,90;
                  -50,110], style(color=74, rgbcolor={0,0,127}));
            connect(Dcb.id, BJTvars1.ibc) annotation (points=[10,20; 2,20; 2,88;
                  -98,88; -98,64; -82,64], style(color=74, rgbcolor={0,0,127}));
            connect(C0.y, Dcb.C0) annotation (points=[32.6,66; 40,66; 40,40; 30,
                  40], style(color=74, rgbcolor={0,0,127}));
            connect(qb1.y, Dcb.qb) annotation (points=[32.6,86; 42,86; 42,20;
                  30,20], style(color=74, rgbcolor={0,0,127}));
            connect(Deb.id, BJTvars1.ibe) annotation (points=[10,-20; 2,-20; 2,
                  -10; -70,-10; -70,32; -96,32; -96,48; -82,48], style(color=74,
                  rgbcolor={0,0,127}));
            connect(Veb.V, BJTvars1.vbe) annotation (points=[-50,-30; -72,-30;
                  -72,28; -98,28; -98,56; -82,56], style(color=74, rgbcolor={0,
                    0,127}));
            connect(Deb.qb, BJTvars1.qb) annotation (points=[30,-20; 36,-20; 36,
                  4; -26,4; -26,56; -38,56], style(color=74, rgbcolor={0,0,127}));
            connect(Deb.C0, BJTvars1.C0) annotation (points=[30,-40; 38,-40; 38,
                  6; -28,6; -28,48; -38,48], style(color=74, rgbcolor={0,0,127}));
          end Qpi;
          
          model RM "Spice-style resistor model for MOSFETs" 
            extends Interfaces.SpiceTwoPort;
            parameter Modelica.SIunits.Resistance R=0 
              "Resistance at reference temperature";
            parameter Real TR1(unit="Ohm/K") = 0 
              "Linear temperature coefficient";
            parameter Real TR2(unit="Ohm/K2") = 0 
              "Quadratic temperature coefficient";
            parameter Integer Level=1 
              "MOSFET modeling level (check documentation window for details)";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            BondLib.Spice.Utilities.RM RM1(
              Tnom=Tnom,
              R=R,
              TR1=TR1,
              TR2=TR2,
              Level=Level) annotation (extent=[-10,-32; 10,4], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Documentation(info="<html>
This is the regular Spice-style resistive source <b>RS</b> model, except that it looks at the modeling level.  If the level is equal to 0, the resistance is set equal to zero.
</html>
"),           Icon(
                Rectangle(extent=[-70,30; 70,-30],   style(
                    color=3,
                    fillColor=7,
                    fillPattern=1)),
                Text(
                  extent=[-50,-40; 46,-80],
                  style(color=0),
                  string="R=%R"),
                Line(points=[0,-30; 0,-91],   style(color=42)),
                Line(points=[70,0; 90,0]),
                Line(points=[-90,0; -70,0])));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(RM1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(RM1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 0,-80; 0,-100], style(color=42, rgbcolor={191,0,0}));
          end RM;
          
          model CjM "Spice-style junction capacitor model for MOSFETs" 
            extends Interfaces.SpiceTwoPort;
            parameter Integer Level=3 
              "MOSFET modeling level (check documentation window for details)";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Current IS=0 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage PB=0.8 
              "Built-in potential at reference temperature";
            parameter Real CJ(unit="F/m2") = 1e-4 
              "Zero-bias bulk capacitance per square meter at reference temperature";
            parameter Real MJ=0.33 "Bulk junction grading coefficient";
            parameter Real CJSW(unit="F/m") = 1e-9 
              "Zero-bias perimeter capacitance per meter at reference temperature";
            parameter Real MJSW=0.33 
              "Perimeter capacitance grading coefficient";
            parameter Real FC=0.5 
              "Forward-bias depletion capacitance coefficient";
            parameter Modelica.SIunits.Area A=0 "Diffusion area";
            parameter Modelica.SIunits.Length P=0 "Perimeter width";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 300 K";
            parameter Modelica.SIunits.Conductance GminDC=1e-12 
              "Leakage conductance";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Semiconductors.Utilities.VariableCapacitor2 VarC1(enforceStates=
                  enforceStates) 
              annotation (extent=[-80,-20; -60,20], rotation=-90);
            BondLib.Spice.Utilities.nlCM nlCM1(
              Tnom=Tnom,
              PB=PB,
              CJ=CJ,
              MJ=MJ,
              CJSW=CJSW,
              MJSW=MJSW,
              FC=FC,
              A=A,
              P=P,
              EG=EG,
              Level=Level) annotation (extent=[-30,-10; -50,10]);
            annotation (
              Diagram,
              Icon(
                Polygon(points=[30,0; -30,20; -30,-20; 30,0], style(
                    color=10,
                    rgbcolor={135,135,135},
                    gradient=0,
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[30,20; 30,-20], style(color=10, rgbcolor={135,135,
                        135})),
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42))),
              Documentation(info="<html>
The Spice-style junction capacitance for MOSFETs computes the depletion capacitance value of a junction used in a MOSFET.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the capacitance formula advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 IS:      Transport saturation current (default value = 0 Amp)

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 A:       Diffusion area (default value = 0 m<sup>2</sup>)

 P:       Perimeter width (default value = 0 m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 GminDC:  Leakage conductance (default value = 1e-12 Mho)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.200.
</ol>
</html>
"));
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vd 
              annotation (extent=[-14,-14; 14,14],  rotation=-90);
            BondLib.Electrical.Analog.Spice.Utilities.DjM DjM1(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              Level=Level,
              EMin=EMin,
              EMax=EMax) annotation (extent=[30,-20; 50,20], rotation=-90);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-10,-80; -30,-60]);
            R Rd(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[60,-20; 80,20],rotation=-90);
          equation 
            connect(nlCM1.y, VarC1.u) annotation (points=[-51,0; -51,
                  1.10215e-015; -61,1.10215e-015], style(color=74, rgbcolor={0,
                    0,127}));
            connect(Vd.V, nlCM1.u) annotation (points=[-14,9.19132e-016; -24,
                  9.19132e-016; -24,0; -28,0], style(color=74, rgbcolor={0,0,
                    127}));
            connect(VarC1.p, Vd.p) annotation (points=[-70,20; -70,40;
                  9.19132e-016,40; 9.19132e-016,14], style(color=3, rgbcolor={0,
                    0,255}));
            connect(Vd.p, DjM1.p) annotation (points=[9.19132e-016,14; 0,14; 0,
                  40; 40,40; 40,20], style(color=3, rgbcolor={0,0,255}));
            connect(VarC1.n, Vd.n) annotation (points=[-70,-20; -70,-40;
                  2.63358e-015,-40; 2.63358e-015,-14], style(color=3, rgbcolor=
                    {0,0,255}));
            connect(Vd.n, DjM1.n) annotation (points=[2.63358e-015,-14; 0,-14;
                  0,-40; 40,-40; 40,-20], style(color=3, rgbcolor={0,0,255}));
            connect(p, VarC1.p) annotation (points=[-100,0; -84,0; -84,40; -70,
                  40; -70,20], style(color=3, rgbcolor={0,0,255}));
            connect(Tdev1.port, heatPort) annotation (points=[-10,-70; 0,-70; 0,
                  -100], style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.T, nlCM1.Tdev) annotation (points=[-30,-70; -40,-70;
                  -40,-10], style(color=74, rgbcolor={0,0,127}));
            connect(DjM1.heatPort, Tdev1.port) annotation (points=[30,
                  -1.22461e-015; 30,-70; -10,-70], style(color=42, rgbcolor={
                    191,0,0}));
            connect(DjM1.n, n) annotation (points=[40,-20; 40,-40; 88,-40; 88,0;
                  100,0], style(color=3, rgbcolor={0,0,255}));
            connect(Rd.n, DjM1.n) annotation (points=[70,-20; 70,-40; 40,-40;
                  40,-20], style(color=3, rgbcolor={0,0,255}));
            connect(Rd.p, DjM1.p) annotation (points=[70,20; 70,40; 40,40; 40,
                  20], style(color=3, rgbcolor={0,0,255}));
            connect(Rd.heatPort, Tdev1.port) annotation (points=[60,
                  -1.22461e-015; 60,-70; -10,-70], style(color=42, rgbcolor={
                    191,0,0}));
          end CjM;
          
          model Cgb 
            "Spice-style gate capacitance between gate and bulk of MOSFETs" 
            extends Interfaces.SpiceTwoPortV;
            parameter Integer Level=3 
              "Level of MOS model (check documentation window for details)";
            parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VFB=0 
              "Flat band voltage at reference temperature";
            parameter Modelica.SIunits.Transconductance KP=0 
              "Transconductance parameter at reference temperature";
            parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
            parameter Modelica.SIunits.Voltage PHI=0 
              "Surface inversion potential at reference temperature";
            parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
            parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
            parameter Modelica.SIunits.Length XJ=0 
              "Metallurgical junction depth";
            parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
            parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
            parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
            parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
            parameter Real XQC=0 "Coefficient of channel charge share";
            parameter Real DELTA=0 "Width effect on threshold voltage";
            parameter Real ETA=0 "Static feedback on threshold voltage";
            parameter Real CGB0(unit="F/m") = 1e-12 
              "Gate-bulk overlap capacitance per meter";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,6; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-34], rotation=-90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput vds(redeclare type SignalType 
                = Modelica.SIunits.Voltage) "Drain-source voltage" 
              annotation (extent=[50,90; 70,110],  rotation=270);
            Modelica.Blocks.Interfaces.RealInput vbs(redeclare type SignalType 
                = Modelica.SIunits.Voltage) "Base-source voltage" 
              annotation (extent=[-70,90; -50,110],rotation=270);
            Modelica.Blocks.Interfaces.RealInput u[5] 
              "Connector of Real input signals" 
              annotation (extent=[-70,-110; -50,-90], rotation=90);
            
          protected 
            BondLib.Spice.Utilities.Cgb Cgb1(
              Tnom=Tnom,
              Level=Level,
              Type=Type,
              VFB=VFB,
              KP=KP,
              GAMMA=GAMMA,
              PHI=PHI,
              COX=COX,
              NFS=NFS,
              XJ=XJ,
              L=L,
              W=W,
              LD=LD,
              XD=XD,
              DELTA=DELTA,
              ETA=ETA,
              CGB0=CGB0,
              EG=EG,
              XQC=XQC,
              e1(start=u0)) 
                     annotation (extent=[-10,-34; 10,6], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.78; 2.16021e-015,27.78], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Icon(
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42)),
                Text(
                  extent=[18,30; 44,2],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(
                  extent=[-42,28; -16,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+")),
              Documentation(info="<html>
The Spice-style gate capacitances for MOSFETs are non-linear capacitors that use different formulae to compute the capacitance value in the four different regions: <i>accumulation</i>, <i>depletion</i>, <i>saturation</i>, and the <i>linear region</i>.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the gate capacitances at all, but rather compute the electric charges stored in these regions directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the gate capacitances directly, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 XQC:     Coefficient of channel charge share (default value = 0)
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 ETA:     Static feedback on threshold voltage (default value = 1)

 CGB0:    Gate-bulk overlap capacitance per meter (default value = 1e-12 F/m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, pp.196-197.
</ol>
</html>
"));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Cgb1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,6; -2.20429e-015,6],
                                       style(color=8, rgbcolor={192,192,192}));
            connect(Cgb1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-34; -7.34764e-016,-34], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(Cgb1.u1, vds) annotation (points=[-10,-2; -30,-2; -30,70;
                  60,70; 60,100],                     style(color=74, rgbcolor=
                    {0,0,127}));
            connect(Cgb1.u2, vbs) annotation (points=[-10,-26; -70,-26; -70,70;
                  -60,70; -60,100],                       style(color=74,
                  rgbcolor={0,0,127}));
            connect(u, Cgb1.u) annotation (points=[-60,-100; -60,-46; 20,-46;
                  20,-14; 10,-14], style(color=74, rgbcolor={0,0,127}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-86; 0,-86; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end Cgb;
          
          model Cgs 
            "Spice-style gate capacitance between gate and source of MOSFETs" 
            extends Interfaces.SpiceTwoPortV;
            parameter Integer Level=3 
              "Level of MOS model (check documentation window for details)";
            parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VFB=0 
              "Flat band voltage at reference temperature";
            parameter Modelica.SIunits.Transconductance KP=0 
              "Transconductance parameter at reference temperature";
            parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
            parameter Modelica.SIunits.Voltage PHI=0 
              "Surface inversion potential at reference temperature";
            parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
            parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
            parameter Modelica.SIunits.Length XJ=0 
              "Metallurgical junction depth";
            parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
            parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
            parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
            parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
            parameter Real XQC=0 "Coefficient of channel charge share";
            parameter Real DELTA=0 "Width effect on threshold voltage";
            parameter Real ETA=0 "Static feedback on threshold voltage";
            parameter Real CGS0(unit="F/m") = 1e-12 
              "Gate-source overlap capacitance per meter";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput vds(redeclare type SignalType 
                = Modelica.SIunits.Voltage) "Drain-source voltage" 
              annotation (extent=[50,90; 70,110],  rotation=270);
            Modelica.Blocks.Interfaces.RealInput vbs(redeclare type SignalType 
                = Modelica.SIunits.Voltage) "Base-source voltage" 
              annotation (extent=[-70,90; -50,110],rotation=270);
            
          protected 
            BondLib.Spice.Utilities.Cgs Cgs1(
              Tnom=Tnom,
              GAMMA=GAMMA,
              PHI=PHI,
              XJ=XJ,
              LD=LD,
              DELTA=DELTA,
              ETA=ETA,
              CGS0=CGS0,
              EG=EG,
              L=L,
              W=W,
              Level=Level,
              Type=Type,
              VFB=VFB,
              KP=KP,
              COX=COX,
              XD=XD,
              NFS=NFS,
              XQC=XQC,
              e1(start=u0)) 
                         annotation (extent=[-10,-32; 10,4], rotation=-90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput u[5] 
              "Connector of Real input signals" 
              annotation (extent=[-70,-110; -50,-90], rotation=90);
          protected 
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Icon(
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42)),
                Text(
                  extent=[18,28; 44,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(
                  extent=[-42,26; -16,-2],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+")),
              Documentation(info="<html>
The Spice-style gate capacitances for MOSFETs are non-linear capacitors that use different formulae to compute the capacitance value in the four different regions: <i>accumulation</i>, <i>depletion</i>, <i>saturation</i>, and the <i>linear region</i>.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the gate capacitances at all, but rather compute the electric charges stored in these regions directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the gate capacitances directly, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 XQC:     Coefficient of channel charge share (default value = 0)
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 ETA:     Static feedback on threshold voltage (default value = 1)

 CGS0:    Gate-source overlap capacitance per meter (default value = 1e-12 F/m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, pp.196-197.
</ol>
</html>
"));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Cgs1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(Cgs1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(vds, Cgs1.u1) annotation (points=[60,100; 60,70; -30,70;
                  -30,-3.2; -10,-3.2], style(color=74, rgbcolor={0,0,127}));
            connect(vbs, Cgs1.u2) annotation (points=[-60,100; -60,70; -70,70;
                  -70,-24.8; -10,-24.8], style(color=74, rgbcolor={0,0,127}));
            connect(u, Cgs1.u) annotation (points=[-60,-100; -60,-40; 20,-40;
                  20,-14; 10,-14],
                           style(color=74, rgbcolor={0,0,127}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 0,-80; 0,-100], style(color=42, rgbcolor={191,0,0}));
          end Cgs;
          
          model Cgd 
            "Spice-style gate capacitance between gate and drain of MOSFETs" 
            extends Interfaces.SpiceTwoPortV;
            parameter Integer Level=3 
              "Level of MOS model (check documentation window for details)";
            parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VFB=0 
              "Flat band voltage at reference temperature";
            parameter Modelica.SIunits.Transconductance KP=0 
              "Transconductance parameter at reference temperature";
            parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
            parameter Modelica.SIunits.Voltage PHI=0 
              "Surface inversion potential at reference temperature";
            parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
            parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
            parameter Modelica.SIunits.Length XJ=0 
              "Metallurgical junction depth";
            parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
            parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
            parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
            parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
            parameter Real XQC=0 "Coefficient of channel charge share";
            parameter Real DELTA=0 "Width effect on threshold voltage";
            parameter Real ETA=0 "Static feedback on threshold voltage";
            parameter Real CGD0(unit="F/m") = 1e-12 
              "Gate-drain overlap capacitance per meter";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput vds(redeclare type SignalType 
                = Modelica.SIunits.Voltage) "Drain-source voltage" 
              annotation (extent=[50,90; 70,110],  rotation=270);
            Modelica.Blocks.Interfaces.RealInput vbs(redeclare type SignalType 
                = Modelica.SIunits.Voltage) "Base-source voltage" 
              annotation (extent=[-70,90; -50,110],rotation=270);
            
          protected 
            BondLib.Spice.Utilities.Cgd Cgd1(
              Tnom=Tnom,
              GAMMA=GAMMA,
              PHI=PHI,
              XJ=XJ,
              LD=LD,
              DELTA=DELTA,
              ETA=ETA,
              CGD0=CGD0,
              EG=EG,
              L=L,
              W=W,
              Level=Level,
              Type=Type,
              VFB=VFB,
              KP=KP,
              COX=COX,
              XD=XD,
              NFS=NFS,
              XQC=XQC,
              e1(start=u0)) 
                         annotation (extent=[-10,-32; 10,4], rotation=-90);
            
          public 
            Modelica.Blocks.Interfaces.RealInput u[5] 
              "Connector of Real input signals" 
              annotation (extent=[-70,-110; -50,-90], rotation=90);
          protected 
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Documentation(info="<html>
The Spice-style gate capacitances for MOSFETs are non-linear capacitors that use different formulae to compute the capacitance value in the four different regions: <i>accumulation</i>, <i>depletion</i>, <i>saturation</i>, and the <i>linear region</i>.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the gate capacitances at all, but rather compute the electric charges stored in these regions directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the gate capacitances directly, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the space charge formula advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Type:    Type of MOSFET (default value = 1)
            Type = +1:  NMOS
            Type = -1:  PMOS

 Tnom:    Reference temperature (default value = 300.15 K)

 VFB:     Flat band voltage at reference temperature (default value = 0 Volt)

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 XQC:     Coefficient of channel charge share (default value = 0)
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 ETA:     Static feedback on threshold voltage (default value = 1)

 CGD0:    Gate-drain overlap capacitance per meter (default value = 1e-12 F/m)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, pp.196-197.
</ol>
</html>
"),           Icon(
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42)),
                Text(
                  extent=[18,28; 44,0],
                  style(color=3, rgbcolor={0,0,255}),
                  string="-"),
                Text(
                  extent=[-42,26; -16,-2],
                  style(color=3, rgbcolor={0,0,255}),
                  string="+")));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Cgd1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(Cgd1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(vds, Cgd1.u1) annotation (points=[60,100; 60,70; -30,70;
                  -30,-3.2; -10,-3.2], style(color=74, rgbcolor={0,0,127}));
            connect(vbs, Cgd1.u2) annotation (points=[-60,100; -60,70; -70,70;
                  -70,-24.8; -10,-24.8], style(color=74, rgbcolor={0,0,127}));
            connect(u, Cgd1.u) annotation (points=[-60,-100; -60,-40; 20,-40;
                  20,-14; 10,-14],
                           style(color=74, rgbcolor={0,0,127}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-86; 0,-86; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end Cgd;
          
          model DjM "Spice-style junction diode model for MOSFETs" 
            extends Interfaces.SpiceTwoPort;
            parameter Integer Level=3 
              "MOSFET modeling level (check documentation window for details)";
            parameter Modelica.SIunits.Current IS=0 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            BondLib.Spice.Utilities.DjM DjM1(
              Tnom=Tnom,
              IS=IS,
              EG=EG,
              Level=Level,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-10,-32; 10,4], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Documentation(info="<html>
The Spice-style MOSFET junction diode is a regular exponential diode. A separate model was needed to implement the <b>Level</b> parameter of the MOSFET models. <p>

<hr> <p>

<pre>
Parameters:

 Level:  MOSFET modeling level (default value = 3)
           Level=0: Static injection model
           Level=1: Shichman-Hodges model
           Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 IS:     Transport saturation current (default value = 0 Amp)

 JS:     Transport saturation current density (default value = 0 Amp/m<sup>2</sup>)

 A :     Diffusion area (default value = 0 m<sup>2</sup>)

 EG:     Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 EMin:   if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:   if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre>
</html>
"),           Icon(
                Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                    color=3,
                    gradient=0,
                    fillColor=7)),
                Line(points=[-90,0; -30,0]),
                Line(points=[30,0; 90,0]),
                Line(points=[30,40; 30,-40],   style(color=3)),
                Line(points=[0,-20; 0,-91],   style(color=42)),
                Text(
                  extent=[-24,14; 4,-14],
                  style(color=3, rgbcolor={0,0,255}),
                  string="J")));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(DjM1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(DjM1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-86; 0,-86; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end DjM;
          
          model Mni "Spice-style internal NMOS transistor" 
            parameter Integer Level=3 
              "Level of MOS model (check documentation window for details)";
            parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VT0=0 
              "Zero-bias threshold voltage";
            parameter Modelica.SIunits.Voltage VFB=0 "Flat band voltage";
            parameter Modelica.SIunits.Transconductance KP=0 
              "Transconductance parameter at reference temperature";
            parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
            parameter Modelica.SIunits.Voltage PHI=0 
              "Surface inversion potential at reference temperature";
            parameter Modelica.SIunits.InversePotential LAMBDA=0 
              "Channel length modulation";
            parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
            parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
            parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
              "Substrate doping";
            parameter Real NSS(unit="1/m2") = 0 "Surface state density";
            parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
            parameter Real NEFF = 1 "Total channel charge coefficient";
            parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
            parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
            parameter Modelica.SIunits.Length XJ=0 
              "Metallurgical junction depth";
            parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
            parameter Modelica.SIunits.Area AD=0 "Drain diffusion area";
            parameter Modelica.SIunits.Length PD=0 "Drain perimeter width";
            parameter Modelica.SIunits.Area AS=0 "Source diffusion area";
            parameter Modelica.SIunits.Length PS=0 "Source perimeter width";
            parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
            parameter Real TPG=1 "Type of gate material";
            parameter Real U0(unit="m2/(V.s)") = 0 
              "Surface mobility at reference temperature";
            parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
              "Critical electric field for mobility";
            parameter Real UEXP=0 "Exponential coefficient for mobility";
            parameter Real UTRA=0 "Transverse field coefficient";
            parameter Modelica.SIunits.Velocity VMAX=0 
              "Maximum drift velocity of carriers";
            parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
              "Critical electric field for pinch-off";
            parameter Real XQC=0 "Coefficient of channel charge share";
            parameter Real DELTA=0 "Width effect on threshold voltage";
            parameter Real KAPPA=0.2 "Saturation field factor";
            parameter Real ETA=0 "Static feedback on threshold voltage";
            parameter Modelica.SIunits.InversePotential THETA=0 
              "Mobility modulation";
            parameter Modelica.SIunits.Voltage VBP=0 
              "Gate to channel critical voltage";
            parameter Modelica.SIunits.Current ISD=0 
              "Drain junction saturation current at reference temperature";
            parameter Modelica.SIunits.Current ISS=0 
              "Source junction saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage PB=0.8 
              "Built-in potential at reference temperature";
            parameter Real CJ(unit="F/m2") = 0 
              "Zero-bias bulk capacitance per square meter at reference temperature";
            parameter Real MJ=0.33 "Bulk junction grading coefficient";
            parameter Real CJSW(unit="F/m") = 1e-9 
              "Zero-bias perimeter capacitance per meter at reference temperature";
            parameter Real MJSW=0.33 
              "Perimeter capacitance grading coefficient";
            parameter Real FC=0.5 
              "Forward-bias depletion capacitance coefficient";
            parameter Modelica.SIunits.Conductance GminDC=1e-12 
              "Leakage conductance";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
              annotation (extent=[90,40; 110,60]);
            Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
              annotation (extent=[-90,-40; -110,-60]);
            Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
              annotation (extent=[90,-40; 110,-60]);
            Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" 
              annotation (extent=[90,-10; 110,10]);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            annotation (Icon(
                Text(extent=[-100,80; 100,100],   string="%name"),
                Line(points=[-90,-50; -10,-50]),
                Line(points=[-10,-50; -10,50]),
                Line(points=[10,70; 10,29]),
                Line(points=[10,20; 10,-21]),
                Line(points=[10,-30; 10,-71]),
                Line(points=[10,50; 91,50]),
                Line(points=[10,0; 90,0]),
                Line(points=[10,-50; 90,-50]),
                Polygon(points=[40,0; 60,5; 60,-5; 40,0],     style(fillColor=3,
                      fillPattern=1)),
                Line(points=[0,-91; 0,-50],   style(color=42))), Diagram(
                Text(
                  extent=[-88,14; -74,8],
                  string="vds",
                  style(color=10, rgbcolor={135,135,135})),
                Text(
                  extent=[-86,-2; -72,-8],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vgs"),
                Text(
                  extent=[-66,30; -52,24],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vbs"),
                Text(
                  extent=[-66,-24; -52,-30],
                  style(color=10, rgbcolor={135,135,135}),
                  string="Tdev"),
                Ellipse(extent=[48,2; 52,-2], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255}))),
              Documentation(info="<html>
The Spice-style internal n-channel MOSFET model contains the non-linear current source, the two junction diodes (including the junctions capacitances), as well as leakage conductances everywhere to keep the model together. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Zero-bias threshold voltage (default value = 0 Volt)
            <font color=blue><b>VT0 &gt; 0 for enhancement MOSFET
            VT0 &lt; 0 for depletion MOSFET</b></font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>)

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 NEFF:    Total channel charge coefficient (default value = 1)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>)

 PD:      Drain perimeter width (default value = 0 m)

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>)

 PS:      Source perimeter width (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 TPG:     Type of gate material (default value = 1)

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s))

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m)

 UEXP:    Exponential coefficient for mobility (default value = 0)

 UTRA:    Transverse field coefficient (default value = 0)

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s)
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation</b></font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,3</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2,3</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 KAPPA:   Saturation field factor (default value = 0.2)

 ETA:     Static feedback on threshold voltage (default value = 0)

 THETA:   Mobility modulation (default value = 0 1/Volt)

 VBP:     Gate to channel critical voltage (default value = 0 Volt)

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp)

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp)

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 GminDC:  Leakage conductance (default value = 1e-12 mho)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"));
            Isignal Ids annotation (extent=[-6,-14; -34,14], rotation=-90);
            CjM Dbd(
              Tnom=Tnom,
              IS=ISD,
              PB=PB,
              CJ=CJ,
              MJ=MJ,
              CJSW=CJSW,
              MJSW=MJSW,
              FC=FC,
              A=AD,
              P=PD,
              EG=EG,
              GminDC=GminDC,
              Level=Level,
              EMin=EMin,
              EMax=EMax,
              enforceStates=enforceStates) 
                           annotation (extent=[64,8; 36,44], rotation=90);
            CjM Dbs(
              Tnom=Tnom,
              PB=PB,
              CJ=CJ,
              MJ=MJ,
              CJSW=CJSW,
              MJSW=MJSW,
              FC=FC,
              EG=EG,
              GminDC=GminDC,
              Level=Level,
              IS=ISS,
              A=AS,
              P=PS,
              EMin=EMin,
              EMax=EMax,
              enforceStates=enforceStates) 
                    annotation (extent=[64,-8; 36,-44], rotation=90);
            BondLib.Spice.Utilities.IDS0 IDS0(
              Tnom=Tnom,
              KP=KP,
              GAMMA=GAMMA,
              LAMBDA=LAMBDA,
              NFS=NFS,
              NEFF=NEFF,
              XJ=XJ,
              U0=U0,
              UEXP=UEXP,
              UTRA=UTRA,
              VMAX=VMAX,
              DELTA=DELTA,
              EG=EG,
              L=L,
              W=W,
              Level=Level,
              EMin=EMin,
              EMax=EMax,
              PHI=PHI,
              Type=Type,
              VFB=VFB,
              COX=COX,
              XD=XD,
              ECRIT=ECRIT,
              VBP=VBP,
              PB=PB,
              LD=LD,
              KAPPA=KAPPA,
              ETA=ETA,
              THETA=THETA) 
                         annotation (extent=[-68,-14; -38,14]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-20,-90; -40,-70]);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vds 
              annotation (extent=[128,-8; 108,12], rotation=-90);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vgs 
              annotation (extent=[60,-90; 80,-70], rotation=0);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vbs 
              annotation (extent=[94,-14; 74,-34], rotation=90);
            R Rgb(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[-70,70; -30,90]);
            R Rgd(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[-70,40; -30,60]);
            R Rds(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[0,-20; 20,20], rotation=-90);
            R Rgs(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[-70,-60; -30,-40]);
            Modelica.Blocks.Interfaces.RealOutput y1[5] 
              "Connector of Real output signals" 
              annotation (extent=[40,100; 60,120],  rotation=90);
          equation 
            connect(B, Dbd.p) annotation (points=[100,0; 50,0; 50,8], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Dbd.n, D) annotation (points=[50,44; 50,50; 100,50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Dbs.n, S) annotation (points=[50,-44; 50,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(IDS0.y, Ids.InPort1)   annotation (points=[-36.5,
                  1.77636e-015; -13.7,1.77636e-015; -13.7,2.29069e-015; -28.4,
                  2.29069e-015], style(color=74, rgbcolor={0,0,127}));
            connect(Ids.p, D) annotation (points=[-20,14; -20,50; 100,50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Ids.n, S) annotation (points=[-20,-14; -20,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Tdev1.port, heatPort) annotation (points=[-20,-80; 0,-80; 0,
                  -100], style(color=42, rgbcolor={191,0,0}));
            connect(Dbs.heatPort, Tdev1.port) annotation (points=[36,-26; 36,
                  -80; -20,-80], style(color=42, rgbcolor={191,0,0}));
            connect(Dbd.heatPort, Dbs.heatPort) annotation (points=[36,26; 36,
                  -26], style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.port, Ids.heatPort) annotation (points=[-20,-80; -6,
                  -80; -6,9.19132e-016], style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.T, IDS0.Tdev)   annotation (points=[-40,-80; -80,-80;
                  -80,-30; -53,-30; -53,-14], style(color=74, rgbcolor={0,0,127}));
            connect(Vds.p, D) annotation (points=[118,12; 118,50; 100,50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Vds.n, S) annotation (points=[118,-8; 118,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(IDS0.u1, Vds.V)   annotation (points=[-71,8.4; -92,8.4; -92,
                  92; 132,92; 132,2; 128,2], style(color=74, rgbcolor={0,0,127}));
            connect(Vgs.n, S) annotation (points=[80,-80; 100,-80; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Vgs.p, G) annotation (points=[60,-80; 50,-80; 50,-104; -100,
                  -104; -100,-50], style(color=3, rgbcolor={0,0,255}));
            connect(Vgs.V, IDS0.u2)   annotation (points=[70,-90; 70,-106; -82,
                  -106; -82,-8.4; -71,-8.4], style(color=74, rgbcolor={0,0,127}));
            connect(IDS0.u3, Vbs.V)   annotation (points=[-53,14; -54,14; -54,
                  34; 70,34; 70,-24; 74,-24], style(color=74, rgbcolor={0,0,127}));
            connect(Vbs.p, B) annotation (points=[84,-14; 84,0; 100,0], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Vbs.n, S) annotation (points=[84,-34; 84,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Rgb.p, G) annotation (points=[-70,80; -90,80; -90,-50; -100,
                  -50], style(color=3, rgbcolor={0,0,255}));
            connect(Rgb.n, B) annotation (points=[-30,80; 32,80; 32,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dbs.p, B) annotation (points=[50,-8; 50,0; 100,0], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Rgb.heatPort, Dbd.heatPort) annotation (points=[-50,70; 36,
                  70; 36,26], style(color=42, rgbcolor={191,0,0}));
            connect(Rgd.p, G) annotation (points=[-70,50; -90,50; -90,-50; -100,
                  -50], style(color=3, rgbcolor={0,0,255}));
            connect(Rgd.n, D) annotation (points=[-30,50; 100,50], style(color=
                    3, rgbcolor={0,0,255}));
            connect(Dbd.heatPort, Rgd.heatPort) annotation (points=[36,26; 36,
                  40; -50,40], style(color=42, rgbcolor={191,0,0}));
            connect(Rds.p, D) annotation (points=[10,20; 10,50; 100,50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Rds.n, S) annotation (points=[10,-20; 10,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Rds.heatPort, Ids.heatPort) annotation (points=[0,
                  -1.22461e-015; -4,-1.22461e-015; -4,9.19132e-016; -6,
                  9.19132e-016], style(color=42, rgbcolor={191,0,0}));
            connect(Rgs.p, G) annotation (points=[-70,-50; -100,-50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Rgs.n, S) annotation (points=[-30,-50; 100,-50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Rgs.heatPort, Tdev1.port) annotation (points=[-50,-60; -6,
                  -60; -6,-80; -20,-80], style(color=42, rgbcolor={191,0,0}));
            connect(IDS0.y1, y1)   annotation (points=[-36.5,-7; -32,-7; -32,
                  -30; 28,-30; 28,76; 50,76; 50,110],
                                              style(color=74, rgbcolor={0,0,127}));
          end Mni;
          
          model Mpi "Spice-style internal PMOS transistor" 
            parameter Integer Level=3 
              "Level of MOS model (check documentation window for details)";
            parameter Integer Type=1 "Type=1 for NMOS; Type=-1 for PMOS";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VT0=0 
              "Zero-bias threshold voltage";
            parameter Modelica.SIunits.Voltage VFB=0 "Flat band voltage";
            parameter Modelica.SIunits.Transconductance KP=0 
              "Transconductance parameter at reference temperature";
            parameter Real GAMMA(unit="V^0.5") = 0 "Body-effect parameter";
            parameter Modelica.SIunits.Voltage PHI=0 
              "Surface inversion potential at reference temperature";
            parameter Modelica.SIunits.InversePotential LAMBDA=0 
              "Channel length modulation";
            parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
            parameter Real COX(unit="F/m2") = 0 "Specific capacitance of SiO2";
            parameter Modelica.SIunits.ElectronNumberDensity NSUB= 0 
              "Substrate doping";
            parameter Real NSS(unit="1/m2") = 0 "Surface state density";
            parameter Real NFS(unit="1/m2") = 0 "Surface fast state density";
            parameter Real NEFF = 1 "Total channel charge coefficient";
            parameter Modelica.SIunits.Length L(min=0) = 1e-4 "Channel length";
            parameter Modelica.SIunits.Length W(min=0) = 1e-4 "Channel width";
            parameter Modelica.SIunits.Length XJ=0 
              "Metallurgical junction depth";
            parameter Modelica.SIunits.Length LD=0 "Lateral diffusion";
            parameter Modelica.SIunits.Area AD=0 "Drain diffusion area";
            parameter Modelica.SIunits.Length PD=0 "Drain perimeter width";
            parameter Modelica.SIunits.Area AS=0 "Source diffusion area";
            parameter Modelica.SIunits.Length PS=0 "Source perimeter width";
            parameter Real XD(unit="m/V^0.5") = 0 "Depletion factor";
            parameter Real TPG=1 "Type of gate material";
            parameter Real U0(unit="m2/(V.s)") = 0 
              "Surface mobility at reference temperature";
            parameter Modelica.SIunits.ElectricFieldStrength UCRIT=1e6 
              "Critical electric field for mobility";
            parameter Real UEXP=0 "Exponential coefficient for mobility";
            parameter Real UTRA=0 "Transverse field coefficient";
            parameter Modelica.SIunits.Velocity VMAX=0 
              "Maximum drift velocity of carriers";
            parameter Modelica.SIunits.ElectricFieldStrength ECRIT=0 
              "Critical electric field for pinch-off";
            parameter Real XQC=0 "Coefficient of channel charge share";
            parameter Real DELTA=0 "Width effect on threshold voltage";
            parameter Real KAPPA=0.2 "Saturation field factor";
            parameter Real ETA=0 "Static feedback on threshold voltage";
            parameter Modelica.SIunits.InversePotential THETA=0 
              "Mobility modulation";
            parameter Modelica.SIunits.Voltage VBP=0 
              "Gate to channel critical voltage";
            parameter Modelica.SIunits.Current ISD=0 
              "Drain junction saturation current at reference temperature";
            parameter Modelica.SIunits.Current ISS=0 
              "Source junction saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage PB=0.8 
              "Built-in potential at reference temperature";
            parameter Real CJ(unit="F/m2") = 0 
              "Zero-bias bulk capacitance per square meter at reference temperature";
            parameter Real MJ=0.33 "Bulk junction grading coefficient";
            parameter Real CJSW(unit="F/m") = 1e-9 
              "Zero-bias perimeter capacitance per meter at reference temperature";
            parameter Real MJSW=0.33 
              "Perimeter capacitance grading coefficient";
            parameter Real FC=0.5 
              "Forward-bias depletion capacitance coefficient";
            parameter Modelica.SIunits.Conductance GminDC=1e-12 
              "Leakage conductance";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
              annotation (extent=[90,40; 110,60]);
            Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
              annotation (extent=[-90,-40; -110,-60]);
            Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
              annotation (extent=[90,-40; 110,-60]);
            Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" 
              annotation (extent=[90,-10; 110,10]);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            Isignal Isd annotation (extent=[-6,14; -34,-14], rotation=-90);
            CjM Ddb(
              Tnom=Tnom,
              IS=ISD,
              PB=PB,
              CJ=CJ,
              MJ=MJ,
              CJSW=CJSW,
              MJSW=MJSW,
              FC=FC,
              A=AD,
              P=PD,
              EG=EG,
              GminDC=GminDC,
              Level=Level,
              enforceStates=enforceStates) 
                           annotation (extent=[64,44; 36,8], rotation=90);
            CjM Dsb(
              Tnom=Tnom,
              PB=PB,
              CJ=CJ,
              MJ=MJ,
              CJSW=CJSW,
              MJSW=MJSW,
              FC=FC,
              EG=EG,
              GminDC=GminDC,
              Level=Level,
              IS=ISS,
              A=AS,
              P=PS,
              enforceStates=enforceStates) 
                    annotation (extent=[64,-44; 36,-8], rotation=90);
            BondLib.Spice.Utilities.IDS0 IDS0(
              Tnom=Tnom,
              KP=KP,
              GAMMA=GAMMA,
              LAMBDA=LAMBDA,
              NFS=NFS,
              NEFF=NEFF,
              XJ=XJ,
              U0=U0,
              UEXP=UEXP,
              UTRA=UTRA,
              VMAX=VMAX,
              DELTA=DELTA,
              EG=EG,
              L=L,
              W=W,
              Level=Level,
              EMin=EMin,
              EMax=EMax,
              PHI=PHI,
              Type=Type,
              VFB=VFB,
              COX=COX,
              XD=XD,
              ECRIT=ECRIT,
              VBP=VBP,
              PB=PB,
              LD=LD,
              KAPPA=KAPPA,
              ETA=ETA,
              THETA=THETA) 
                         annotation (extent=[-68,-14; -38,14]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-20,-90; -40,-70]);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vsd 
              annotation (extent=[128,12; 108,-8], rotation=-90);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vsg 
              annotation (extent=[80,-90; 60,-70], rotation=0);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vsb 
              annotation (extent=[94,-34; 74,-14], rotation=90);
            annotation (
              Diagram(
                Text(
                  extent=[-88,14; -74,8],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsd"),
                Text(
                  extent=[-86,-2; -72,-8],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsg"),
                Text(
                  extent=[-66,30; -52,24],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsb"),
                Text(
                  extent=[-66,-24; -52,-30],
                  style(color=10, rgbcolor={135,135,135}),
                  string="Tdev"),
                Ellipse(extent=[48,2; 52,-2], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255}))),
              Documentation(info="<html>
The Spice-style internal p-channel MOSFET model contains the non-linear current source, the two junction diodes (including the junctions capacitances), as well as leakage conductances everywhere to keep the model together. <p>

<hr> <p>

<pre>
Parameters:

 Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Zero-bias threshold voltage (default value = 0 Volt)
            <font color=blue><b>VT0 &lt; 0 for enhancement MOSFET
            VT0 &gt; 0 for depletion MOSFET</b></font>

 KP:      Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 GAMMA:   Body-effect parameter (default value = Volt<sup>0.5</sup>)

 PHI:     Surface inversion potential at reference temperature (default value = 0 Volt)

 LAMBDA:  Channel length modulation  (default value = 0 1/Volt)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 COX:     Specific capacitance of SiO2 (default value = 0 F/m<sup>2</sup>)

 NSUB:    Substrate doping (default value = 0 1/m<sup>3</sup>)

 NSS:     Surface state density (default value = 0 1/m<sup>2</sup>)

 NFS:     Surface fast state density (default value = 0 1/m<sup>2</sup>)

 NEFF:    Total channel charge coefficient (default value = 1)

 XJ:      Metallurgical junction depth (default value = 0 m)

 L:       Channel length (default value = 1e-4 m)

 W:       Channel width (default value = 1e-4 m)

 LD:      Lateral diffusion (default value = 0 m)

 AD:      Drain diffusion area (default value = 0 m<sup>2</sup>)

 PD:      Drain perimeter width (default value = 0 m)

 AS:      Source diffusion area (default value = 0 m<sup>2</sup>)

 PS:      Source perimeter width (default value = 0 m)

 XD:      Depletion factor (default value = 0 m/Volt<sup>0.5</sup>)

 TPG:     Type of gate material (default value = 1)

 U0:      Surface mobility at reference temperature (default value = 0 m<sup>2</sup>/(V*s))

 UCRIT:   Critical electric field for mobility (default value = 1e6 V/m)

 UEXP:    Exponential coefficient for mobility (default value = 0)

 UTRA:    Transverse field coefficient (default value = 0)

 VMAX:    Maximum drift velocity of carriers (default value = 0 m/s)
            <font color=blue><b>VMAX = 0: Frohman-Grove model of drain current computation</b></font>
            <font color=blue><b>VMAX &gt; 0: Baum-Beneking model of drain current computation</b></font>

 ECRIT:   Critical electric field for pinch-off (default value = 0 V/m) <font color=red>Levels 2,3</font>

 XQC:     Coefficient of channel charge share (default value = 0) <font color=red>Levels 2,3</font>
            <font color=blue><b>XQC &lt;= 0.5: Ward-Dutton model of gate capacitance computation</b></font>
            <font color=blue><b>XQC &gt;  0.5: Meyer model of gate capacitance computation</b></font>

 DELTA:   Width effect on threshold voltage (default value = 0)

 KAPPA:   Saturation field factor (default value = 0.2)

 ETA:     Static feedback on threshold voltage (default value = 0)

 THETA:   Mobility modulation (default value = 0 1/Volt)

 VBP:     Gate to channel critical voltage (default value = 0 Volt)

 ISD:     Drain junction saturation current at reference temperature (default value = 0 Amp)

 ISS:     Source junction saturation current at reference temperature (default value = 0 Amp)

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CJ:      Zero-bias bulk capacitance per square meter at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 CJSW:    Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 GminDC:  Leakage conductance (default value = 1e-12 mho)

 EG:      Energy gap for temperature effect on saturation current (default value = 1.16 Volt)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
</ol>
</html>
"),           Icon(
                Text(extent=[-100,80; 100,100],   string="%name"),
                Line(points=[-90,-50; -10,-50]),
                Line(points=[-10,-50; -10,50]),
                Line(points=[10,70; 10,29]),
                Line(points=[10,20; 10,-21]),
                Line(points=[10,-30; 10,-71]),
                Line(points=[10,50; 91,50]),
                Line(points=[10,0; 90,0]),
                Line(points=[10,-50; 90,-50]),
                Polygon(points=[60,0; 40,5; 40,-5; 60,0],     style(fillColor=3,
                      fillPattern=1)),
                Line(points=[0,-91; 0,-50],   style(color=42))));
            R Rbg(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[-70,70; -30,90]);
            R Rdg(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[-70,40; -30,60]);
            R Rsd(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[0,-20; 20,20], rotation=-90);
            R Rsg(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[-70,-60; -30,-40]);
            Modelica.Blocks.Interfaces.RealOutput y1[5] 
              "Connector of Real output signals" 
              annotation (extent=[40,100; 60,120],  rotation=90);
          equation 
            connect(IDS0.y, Isd.InPort1)   annotation (points=[-36.5,
                  1.77636e-015; -13.7,1.77636e-015; -13.7,-2.29069e-015; -28.4,
                  -2.29069e-015],style(color=74, rgbcolor={0,0,127}));
            connect(Tdev1.port,heatPort)  annotation (points=[-20,-80; 0,-80; 0,
                  -100], style(color=42, rgbcolor={191,0,0}));
            connect(Dsb.heatPort,Tdev1. port) annotation (points=[36,-26; 36,
                  -80; -20,-80], style(color=42, rgbcolor={191,0,0}));
            connect(Ddb.heatPort,Dsb. heatPort) annotation (points=[36,26; 36,
                  -26], style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.port,Isd. heatPort) annotation (points=[-20,-80; -6,
                  -80; -6,-9.19132e-016],style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.T, IDS0.Tdev)   annotation (points=[-40,-80; -80,-80;
                  -80,-30; -53,-30; -53,-14], style(color=74, rgbcolor={0,0,127}));
            connect(IDS0.u1, Vsd.V)   annotation (points=[-71,8.4; -92,8.4; -92,
                  92; 132,92; 132,2; 128,2], style(color=74, rgbcolor={0,0,127}));
            connect(Vsg.V, IDS0.u2)   annotation (points=[70,-90; 70,-106; -82,
                  -106; -82,-8.4; -71,-8.4], style(color=74, rgbcolor={0,0,127}));
            connect(IDS0.u3, Vsb.V)   annotation (points=[-53,14; -54,14; -54,
                  34; 70,34; 70,-24; 74,-24], style(color=74, rgbcolor={0,0,127}));
            connect(Vsg.p, S) annotation (points=[80,-80; 100,-80; 100,-50], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Vsg.n, G) annotation (points=[60,-80; 50,-80; 50,-104; -100,-104;
                  -100,-50], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Vsb.n, B) annotation (points=[84,-14; 84,0; 100,0], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Vsb.p, S) annotation (points=[84,-34; 84,-50; 100,-50],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(D, Vsd.n) annotation (points=[100,50; 118,50; 118,12],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(S, Vsd.p) annotation (points=[100,-50; 118,-50; 118,-8],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Ddb.p, D) annotation (points=[50,44; 50,50; 100,50], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Ddb.n, B) annotation (points=[50,8; 50,0; 100,0], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Dsb.p, S) annotation (points=[50,-44; 50,-50; 100,-50],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Dsb.n, Ddb.n) annotation (points=[50,-8; 50,8], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Isd.n, D) annotation (points=[-20,14; -20,50; 100,50],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Isd.p, S) annotation (points=[-20,-14; -20,-50; 100,-50],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Rbg.p, G) annotation (points=[-70,80; -90,80; -90,-50; -100,
                  -50], style(color=3, rgbcolor={0,0,255}));
            connect(Rbg.n, B) annotation (points=[-30,80; 30,80; 30,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Rbg.heatPort, Ddb.heatPort) annotation (points=[-50,70; 36,
                  70; 36,26], style(color=42, rgbcolor={191,0,0}));
            connect(Rdg.n, D) annotation (points=[-30,50; 100,50], style(color=
                    3, rgbcolor={0,0,255}));
            connect(Rdg.p, G) annotation (points=[-70,50; -90,50; -90,-50; -100,
                  -50], style(color=3, rgbcolor={0,0,255}));
            connect(Rdg.heatPort, Ddb.heatPort) annotation (points=[-50,40; 36,
                  40; 36,26], style(color=42, rgbcolor={191,0,0}));
            connect(Rsd.heatPort, Isd.heatPort) annotation (points=[0,
                  -1.22461e-015; -4,-1.22461e-015; -4,-9.19132e-016; -6,
                  -9.19132e-016], style(color=42, rgbcolor={191,0,0}));
            connect(Rsd.p, D) annotation (points=[10,20; 10,50; 100,50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Rsd.n, S) annotation (points=[10,-20; 10,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Rsg.n, S) annotation (points=[-30,-50; 100,-50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Rsg.p, G) annotation (points=[-70,-50; -100,-50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Rsg.heatPort, Tdev1.port) annotation (points=[-50,-60; -6,
                  -60; -6,-80; -20,-80], style(color=42, rgbcolor={191,0,0}));
            connect(IDS0.y1, y1)   annotation (points=[-36.5,-7; -32,-7; -32,
                  -30; 26,-30; 26,76; 50,76; 50,110],
                                              style(color=74, rgbcolor={0,0,127}));
          end Mpi;
          
          model CjD "Spice-style junction capacitor model for DIODEs" 
            extends Interfaces.SpiceTwoPort;
            parameter Integer Level=1 
              "DIODE modeling level (check documentation window for details)";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Current IS=1e-14 
              "Saturation current at reference temperature";
            parameter Real N=1 "Current emission coefficient";
            parameter Modelica.SIunits.Time TT=0 
              "Transit time at reference temperature";
            parameter Modelica.SIunits.Capacitance CJ=0.5e-12 
              "Zero-bias bulk capacitance at reference temperature";
            parameter Modelica.SIunits.Voltage PB=0.8 
              "Built-in junction potential at reference temperature";
            parameter Real MJ=0.33 "Bulk junction grading coefficient";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Real FC=0.5 
              "Forward-bias depletion capacitance coefficient";
            parameter Modelica.SIunits.Voltage BV=0 
              "Reverse breakdown voltage at reference temperature";
            parameter Modelica.SIunits.Current IBV=0 
              "Reverse breakdown current";
            parameter Real TCV(unit="V/K")=0 
              "Temperature coefficient of breakdown voltage";
            parameter Real TTT1(unit="s/K") = 0 
              "Linear temperature coefficient of transit time";
            parameter Real TTT2(unit="s/K2") = 0 
              "Quadratic temperature coefficient of transit time";
            parameter Modelica.SIunits.Current ISW=0 
              "Saturation current of periphery at reference temperature";
            parameter Modelica.SIunits.Capacitance CJSW=0.5e-12 
              "Zero-bias perimeter capacitance at reference temperature";
            parameter Modelica.SIunits.Voltage PHP=0.8 
              "Built-in perimeter potential at reference temperature";
            parameter Real MJSW=0.33 
              "Perimeter capacitance grading coefficient";
            parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
            parameter Modelica.SIunits.Conductance GminDC=1e-16 
              "Leakage conductance";
            parameter Real Area=1 "Relative device area";
            parameter Real PJ=1 "Relative perimeter width";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Modelica.SIunits.Length L=0 "Diode length";
            parameter Modelica.SIunits.Length W=0 "Diode width";
            parameter Integer M=1 "Diode multiplier";
            parameter Modelica.SIunits.CurrentDensity JS=1e-4 
              "Saturation current density at reference temperature";
            parameter Real CJ2(unit="F/m2") = 1e-4 
              "Zero-bias bulk capacitance per square meter at reference temperature";
            parameter Modelica.SIunits.CurrentDensity JSW=0 
              "Saturation current density of periphery at reference temperature";
            parameter Real CJSW2(unit="F/m") = 1e-9 
              "Zero-bias perimeter capacitance per meter at reference temperature";
            parameter Modelica.SIunits.Length LP=0 "Polysilicon length";
            parameter Modelica.SIunits.Length WP=0 "Polysilicon width";
            parameter Modelica.SIunits.Length XP=0 
              "Widening of polysilicon due to masking and etching";
            parameter Modelica.SIunits.Length XOI=0 "Polysilicon thickness";
            parameter Modelica.SIunits.Length LM=0 "Metal length";
            parameter Modelica.SIunits.Length WM=0 "Metal width";
            parameter Modelica.SIunits.Length XM=0 
              "Widening of metal due to masking and etching";
            parameter Modelica.SIunits.Length XOM=0 "Metal thickness";
            parameter Real JF(unit="A/V2") = 0 "Forward FN current coefficient";
            parameter Real JR(unit="A/V2") = 0 "Reverse FN current coefficient";
            parameter Modelica.SIunits.ElectricFieldStrength EF=0 
              "Forward critical electrical field";
            parameter Modelica.SIunits.ElectricFieldStrength ER=0 
              "Reverse critical electrical field";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Sensors.VoltageSensor Vd 
              annotation (extent=[-32,-12; -6,12],  rotation=-90);
            Basic.VCV VCV1(                          gain=1) 
              annotation (extent=[26,22; 0,-22],  rotation=-180);
            Sensors.CurrentSensor Id 
              annotation (extent=[32,40; 52,20],     rotation=0);
            Basic.Ground Gnd1    annotation (extent=[50,-50; 70,-30]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-20,-80; -40,-60]);
            BondLib.Electrical.Analog.Spice.Utilities.Gj Gj1(
              GminDC=GminDC,
              Area=Area,
              Level=Level) annotation (extent=[70,-20; 90,20], rotation=-90);
            BondLib.Spice.Utilities.nlCD nlCD1(
              Level=Level,
              Tnom=Tnom,
              Area=Area,
              PJ=PJ,
              M=M,
              IS=IS,
              JS=JS,
              N=N,
              TT=TT,
              PB=PB,
              PHP=PHP,
              CJ=CJ,
              CJ2=CJ2,
              MJ=MJ,
              CJSW=CJSW,
              CJSW2=CJSW2,
              MJSW=MJSW,
              FC=FC,
              GminDC=GminDC,
              EG=EG,
              XTI=XTI,
              TTT1=TTT1,
              TTT2=TTT2,
              ISW=ISW,
              JSW=JSW,
              TOX=TOX,
              L=L,
              W=W,
              LP=LP,
              WP=WP,
              XP=XP,
              XOI=XOI,
              LM=LM,
              WM=WM,
              XM=XM,
              XOM=XOM,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-40,-10; -60,10]);
            BondLib.Electrical.Analog.Semiconductors.Utilities.VariableCapacitor2
              VarC1(enforceStates=enforceStates, u0=0) 
              annotation (extent=[-90,-20; -70,20], rotation=-90);
            BondLib.Electrical.Analog.Spice.Utilities.DjD DjD1(
              Tnom=Tnom,
              Level=Level,
              Area=Area,
              PJ=PJ,
              M=M,
              IS=IS,
              JS=JS,
              N=N,
              EG=EG,
              XTI=XTI,
              BV=BV,
              IBV=IBV,
              ISW=ISW,
              JSW=JSW,
              TCV=TCV,
              TOX=TOX,
              L=L,
              W=W,
              JF=JF,
              JR=JR,
              EF=EF,
              ER=ER,
              EMin=EMin,
              EMax=EMax) annotation (extent=[50,-20; 70,20], rotation=-90);
            Modelica.Blocks.Interfaces.RealOutput id(redeclare type SignalType 
                = Modelica.SIunits.Current) "Diode current" 
              annotation (extent=[-10,100; 10,120],rotation=90);
          equation 
            connect(VCV1.p1, Vd.p)  annotation (points=[0,11; 0,30; -19,30; -19,
                  12], style(color=3, rgbcolor={0,0,255}));
            connect(VCV1.n1, Vd.n)  annotation (points=[0,-11; 0,-30; -19,-30;
                  -19,-12], style(color=3, rgbcolor={0,0,255}));
            connect(VCV1.n2,Gnd1. p)    annotation (points=[26,-11; 26,-30; 60,
                  -30], style(color=3, rgbcolor={0,0,255}));
            connect(Tdev1.port, heatPort)              annotation (points=[-20,
                  -70; 0,-70; 0,-100], style(color=42, rgbcolor={191,0,0}));
            connect(Gj1.n,Gnd1. p) annotation (points=[80,-20; 80,-30; 60,-30],
                style(color=3, rgbcolor={0,0,255}));
            connect(Id.p,VCV1. p2) annotation (points=[32,30; 26,30; 26,11],
                style(color=3, rgbcolor={0,0,255}));
            annotation (Diagram, Icon(
                Polygon(points=[30,0; -30,20; -30,-20; 30,0], style(
                    color=10,
                    rgbcolor={135,135,135},
                    gradient=0,
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[30,20; 30,-20], style(color=10, rgbcolor={135,135,
                        135})),
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42))),
              Documentation(info="<html>
The Spice-style junction capacitance for DIODEs computes the depletion and diffusion capacitance value of a junction used in a DIODE.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

This particular model uses the capacitance formulae advocated in [2]. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   DIODE modeling level (default value = 1)
            Level=1: Non-geometric junction diode
            Level=2: Fowler-Nordheim diode
            Level=3: Geometric junction diode</b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 IS:      Saturation current at reference temperature (default value = 1e-14 Amp)

 N:       Current mission coefficient (default value = 1)

 TT:      Transit time at reference temperature (default value = 0 sec)

 CJ:      Zero-bias bulk capacitance at reference temperature (default value = 0.5e-12 F)

 PB:      Built-in junction potential at reference temperature (default value = 0.8 Volt)

 MJ:      Bulk junction grading coefficient (default value = 0.33)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)

 XTI:     Temperature exponent of saturation current (default value = 3)

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 BV:      Reverse breakdown voltage at reference temperature (default value = 0 Volt)

 IBV:     Reverse breakdown current (default value = 0 Amp)

 TCV:     Linear temperature coefficient of breakdown voltage (default value = 0 V/K)

 TTT1:    Linear temperature coefficient of transit time (default value = 0 s/K)
 
 TTT2:    Quadratic temperature coefficient of transit time (default value = 0 s/K2)

 ISW:     Saturation current of perimeter at reference temperature (default value = 0 Amp)

 CJSW:    Zero-bias perimeter capacitance at reference temperature (default value = 0.5e-12 F)

 PHP:     Built-in perimeter potential at reference temperature (default value = 0.8 Volt)

 MJSW:    Perimeter capacitance grading coefficient (default value = 0.33)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 GminDC:  Leakage conductance (default value = 1e-16 mho)

 Area:    Relative area occupied by the diode (default value = 1)

 PJ:      Relative perimeter width (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)

 L:       Diode length (default value = 0 m)

 W:       Diode width (default value = 0 m)

 M:       Diode multiplier (default value = 1)

 JS:      Saturation current density at reference temperature (default value = 1e-4 Amp/m<sup>2</sup>)

 CJ2:     Zero-bias bulk capacitance per meter square at reference temperature (default value = 1e-4 F/m<sup>2</sup>)

 JSW:     Saturation current density of perimeter at reference temperature (default value = 0 Amp/m<sup>2</sup>)

 CJSW2:   Zero-bias perimeter capacitance per meter at reference temperature (default value = 1e-9 F/m)

 LP:       Polysilicon length (default value = 0 m)

 WP:       Polysilicon width (default value = 0 m)

 XP:       Widening of polysilicon due to masking and etching (default value = 0 m)

 XOI:      Polysilicon thickness (default value = 0 m)

 LM:       Metal length (default value = 0 m)

 WM:       Metal width (default value = 0 m)

 XM:       Widening of metal due to masking and etching (default value = 0 m)

 XOM:      Metal thickness (default value = 0 m)

 JF:       Forward Fowler-Nordheim current coefficient (default value = 0 A/V2)

 JR:       Reverse Fowler-Nordheim current coefficient (default value = 0 A/V2)

 EF:       Forward Fowler-Nordheim critical electrical field (default value = 0 V/m)

 ER:       Reverse Fowler-Nordheim critical electrical field (default value = 0 V/m)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.200.
</ol>
</html>
"));
            connect(nlCD1.u, Vd.V) annotation (points=[-38,0; -37,0; -37,
                  -7.34764e-016; -32,-7.34764e-016], style(color=74, rgbcolor={
                    0,0,127}));
            connect(Tdev1.T, nlCD1.Tdev) annotation (points=[-40,-70; -50,-70;
                  -50,-10], style(color=74, rgbcolor={0,0,127}));
            connect(VarC1.u, nlCD1.y) annotation (points=[-71,1.10215e-015;
                  -64.5,1.10215e-015; -64.5,0; -61,0], style(color=74, rgbcolor=
                   {0,0,127}));
            connect(VarC1.p, Vd.p) annotation (points=[-80,20; -80,30; -19,30;
                  -19,12], style(color=3, rgbcolor={0,0,255}));
            connect(VarC1.n, Vd.n) annotation (points=[-80,-20; -80,-30; -19,
                  -30; -19,-12], style(color=3, rgbcolor={0,0,255}));
            connect(VarC1.p, p) annotation (points=[-80,20; -80,30; -92,30; -92,
                  0; -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(VarC1.n, n) annotation (points=[-80,-20; -80,-50; 92,-50;
                  92,0; 100,0], style(color=3, rgbcolor={0,0,255}));
            connect(DjD1.n, Gnd1.p) annotation (points=[60,-20; 60,-30], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Id.n, DjD1.p) annotation (points=[52,30; 60,30; 60,20],
                style(color=3, rgbcolor={0,0,255}));
            connect(DjD1.p, Gj1.p) annotation (points=[60,20; 60,30; 80,30; 80,
                  20], style(color=3, rgbcolor={0,0,255}));
            connect(DjD1.heatPort, Tdev1.port) annotation (points=[50,
                  -1.22461e-015; 46,-1.22461e-015; 46,0; 40,0; 40,-70; -20,-70],
                style(color=42, rgbcolor={191,0,0}));
            connect(Id.I, id) annotation (points=[42,40; 42,70; 0,70; 0,110],
                style(color=74, rgbcolor={0,0,127}));
          end CjD;
          
          model DjD "Spice-style DIODE model" 
            extends Interfaces.SpiceTwoPort;
            parameter Integer Level=1 
              "DIODE modeling level (check documentation window for details)";
            parameter Modelica.SIunits.Current IS=1e-14 
              "Saturation current at reference temperature";
            parameter Real N=1 "Current emission coefficient";
            parameter Modelica.SIunits.Voltage EG=1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real XTI=3 "Saturation current temperature exponent";
            parameter Modelica.SIunits.Voltage BV=0 
              "Reverse breakdown voltage at reference temperature";
            parameter Modelica.SIunits.Current IBV=0 
              "Reverse breakdown current";
            parameter Real TCV(unit="V/K")=0 
              "Temperature coefficient of breakdown voltage";
            parameter Modelica.SIunits.Current ISW=0 
              "Saturation current of periphery at reference temperature";
            parameter Modelica.SIunits.Length TOX=1e-7 "Thin oxide thickness";
            parameter Real Area=1 "Relative device area";
            parameter Real PJ=0 "Relative perimeter width";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Modelica.SIunits.Length L=0 "Diode length";
            parameter Modelica.SIunits.Length W=0 "Diode width";
            parameter Integer M=1 "Diode multiplier";
            parameter Modelica.SIunits.CurrentDensity JS=1e-4 
              "Saturation current density at reference temperature";
            parameter Modelica.SIunits.CurrentDensity JSW=0 
              "Saturation current density of periphery at reference temperature";
            parameter Real JF(unit="A/V2") = 0 "Forward FN current coefficient";
            parameter Real JR(unit="A/V2") = 0 "Reverse FN current coefficient";
            parameter Modelica.SIunits.ElectricFieldStrength EF=0 
              "Forward critical electrical field";
            parameter Modelica.SIunits.ElectricFieldStrength ER=0 
              "Reverse critical electrical field";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            BondLib.Spice.Utilities.DjD DjD1(
              Tnom=Tnom,
              Level=Level,
              Area=Area,
              PJ=PJ,
              M=M,
              IS=IS,
              JS=JS,
              N=N,
              EG=EG,
              XTI=XTI,
              BV=BV,
              IBV=IBV,
              ISW=ISW,
              JSW=JSW,
              TCV=TCV,
              TOX=TOX,
              L=L,
              W=W,
              JF=JF,
              JR=JR,
              EF=EF,
              ER=ER,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-10,-32; 10,4], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (Diagram, Icon(
                Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                    color=3,
                    gradient=0,
                    fillColor=7)),
                Line(points=[-90,0; -30,0]),
                Line(points=[30,0; 90,0]),
                Line(points=[30,40; 30,-40],   style(color=3)),
                Line(points=[0,-20; 0,-91],   style(color=42))),
              Documentation(info="<html>
This model computes the diodic current of a Spice-style DIODE model.  Spice offers three modeling levels for DIODEs that differ in the formulae they use to compute the diodic current [1].  The <b>Level=1</b> model and the <b>Level=3</b> model represent junction diodes, whereas the <b>Level=2</b> model represents a Fowler-Nordheim diode, essentially a non-linear resistor with a quadratic conductance characteristic. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   DIODE modeling level (default value = 1)
            Level=1: Non-geometric junction diode
            Level=2: Fowler-Nordheim diode
            Level=3: Geometric junction diode</b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 IS:      Saturation current at reference temperature (default value = 1e-14 Amp)

 N:       Current mission coefficient (default value = 1)

 EG:      Energy gap for temperature effect on saturation current at 0 K (default value = 1.16 Volt)

 XTI:     Temperature exponent of saturation current (default value = 3)

 BV:      Reverse breakdown voltage at reference temperature (default value = 0 Volt)

 IBV:     Reverse breakdown current (default value = 0 Amp)

 TCV:     Linear temperature coefficient of breakdown voltage (default value = 0 V/K)

 ISW:     Saturation current of perimeter at reference temperature (default value = 0 Amp)

 TOX:     Thin oxide thickness (default value = 1e-7 m)

 Area:    Relative area occupied by the diode (default value = 1)

 PJ:      Relative perimeter width (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)

 L:       Diode length (default value = 0 m)

 W:       Diode width (default value = 0 m)

 M:       Diode multiplier (default value = 1)

 JS:      Saturation current density at reference temperature (default value = 1e-4 Amp/m<sup>2</sup>)

 JSW:     Saturation current density of perimeter at reference temperature (default value = 0 Amp/m<sup>2</sup>)

 JF:       Forward Fowler-Nordheim current coefficient (default value = 0 A/V2)

 JR:       Reverse Fowler-Nordheim current coefficient (default value = 0 A/V2)

 EF:       Forward Fowler-Nordheim critical electrical field (default value = 0 V/m)

 ER:       Reverse Fowler-Nordheim critical electrical field (default value = 0 V/m)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.200.
</ol>
</html>
"));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(DjD1.BondCon1, B3.BondCon2) annotation (points=[
                  -6.12303e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(DjD1.BondCon2, B4.fBondCon1) annotation (points=[
                  6.12303e-016,-32; 6.12303e-016,-37; -7.34764e-016,-37;
                  -7.34764e-016,-32],
                style(color=8, rgbcolor={192,192,192}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 7.34764e-016,-86; 0,-86; 0,-100], style(color=42,
                  rgbcolor={191,0,0}));
          end DjD;
          
          model CjJ 
            "Spice-style junction capacitor model for JFETs and MESFETs" 
            extends Interfaces.SpiceTwoPort;
            parameter Integer Level=1 
              "JFET modeling level (check documentation layer for details)";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VT0=0 
              "Threshold voltage at reference temperature";
            parameter Modelica.SIunits.Current IS=1.e-14 
              "Transport saturation current at reference temperature";
            parameter Real TCV(unit="V/K") = 0 
              "Linear temperature coefficient of threshold voltage";
            parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI = if Level==3 then 2 else 3 
              "Saturation current temperature exponent";
            parameter Modelica.SIunits.Voltage PB=0.8 
              "Built-in potential at reference temperature";
            parameter Modelica.SIunits.Capacitance CGS=0.5e-12 
              "Zero-bias gate-source depletion capacitance at reference temperature";
            parameter Modelica.SIunits.Capacitance CGD=0.5e-12 
              "Zero-bias gate-drain depletion capacitance at reference temperature";
            parameter Real M = if Level==3 then 0.5 else 0.33 
              "Junction grading coefficient";
            parameter Real FC=0.5 
              "Factor for linearization of depletion capacitance formula";
            parameter Modelica.SIunits.Time TT=5e-9 "Ideal transit time";
            parameter Modelica.SIunits.InversePotential ALPHA=1 
              "Hyperbolic tangient fitting parameter";
            parameter Modelica.SIunits.Voltage DELTA=0.2 
              "Voltage range for transition";
            parameter Modelica.SIunits.Voltage VMAX=0.5 
              "Limit voltage for carrier velocity saturation";
            parameter Modelica.SIunits.Conductance GminDC=1e-16 
              "Leakage conductance";
            parameter Real Area=1 "Relative area occupied by capacitor";
            parameter Boolean GS=true "True if gate-source junction";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            annotation (
              Icon(
                Polygon(points=[30,0; -30,20; -30,-20; 30,0], style(
                    color=10,
                    rgbcolor={135,135,135},
                    gradient=0,
                    fillColor=7,
                    rgbfillColor={255,255,255})),
                Line(points=[30,20; 30,-20], style(color=10, rgbcolor={135,135,
                        135})),
                Line(points=[14,0; 90,0]),
                Line(points=[14,28; 14,-28],   style(thickness=2)),
                Line(points=[-14,28; -14,-28],   style(thickness=2)),
                Line(points=[-90,0; -14,0]),
                Line(points=[0,-28; 0,-91],   style(color=42))),
              Diagram(
              Text(
                extent=[0,20; 10,12],
                string="vds",
                style(color=10, rgbcolor={95,95,95})),
              Text(
                extent=[0,2; 10,-6],
                style(color=10, rgbcolor={95,95,95}),
                string="vgs"),
              Text(
                extent=[-42,-18; -28,-26],
                style(color=10, rgbcolor={95,95,95}),
                string="Tdev")),
              Documentation(info="<html>
The Spice-style junction capacitance for JFETs and GaAs MESFETs computes the depletion and diffusion capacitance values of a junction used in a JFET / MESFET.  Different Spice dialects vary in the formulae they use for this purpose.  Many Spice dialects actually don't use a formula for the junction capacitance at all, but rather compute the electric charge stored in the junction directly, which is conceptually cleaner.  However, that approach is computationally cumbersome, as it leads to an awkward algebraic loop [1].  Thus, we chose to compute the junction capacitance, and use a (physically incorrect) approximate non-linear capacitor model.  The numerical error should remain small, as the time constants associated with temperature variation are much larger than those associated with electrical phenomena. <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [2]
            Level=3: Statz-Curtice (GaAs) MESFET model [3,4]<b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)

 IS:      Transport saturation current (default value = 1e-14 Amp)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 EG:      Energy gap for temperature effect on saturation current at 0 K
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F)

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F)

 M:       Junction grading coefficient
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 TT:      Ideal transit time (default value = 5e-9 sec)

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1)

 DELTA:   Voltage range for transition (default value = 0.2 Volt)

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 GS:      True if gate-source junction (default value = true)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 224-225.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
            Semiconductors.Utilities.VariableCapacitor2 VarC1(enforceStates=
                  enforceStates, u0=0) 
              annotation (extent=[-80,-20; -60,20], rotation=-90);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-10,-80; -30,-60]);
            BondLib.Electrical.Analog.Spice.R Rd(
              Tnom=Tnom,
              R=1/GminDC,
              TR1=0,
              TR2=0,
              Area=1) annotation (extent=[60,-20; 80,20],rotation=-90);
            BondLib.Spice.Utilities.nlCJ nlCJ1(
              Level=Level,
              Tnom=Tnom,
              VT0=VT0,
              TCV=TCV,
              IS=IS,
              EG=EG,
              N=N,
              XTI=XTI,
              PB=PB,
              CGS=CGS,
              CGD=CGD,
              M=M,
              FC=FC,
              TT=TT,
              ALPHA=ALPHA,
              DELTA=DELTA,
              VMAX=VMAX,
              GminDC=GminDC,
              Area=Area,
              GS=GS,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-2,-16; -50,16]);
            BondLib.Electrical.Analog.Spice.Utilities.DjJ DjJ1(
              Tnom=Tnom,
              Level=Level,
              IS=IS,
              EG=EG,
              N=N,
              XTI=XTI,
              Area=Area,
              EMin=EMin,
              EMax=EMax) annotation (extent=[30,-20; 50,20], rotation=-90);
          Modelica.Blocks.Interfaces.RealInput vds(redeclare type SignalType = 
                  Modelica.SIunits.Voltage) "Drain-source voltage" 
            annotation (extent=[-50,90; -30,110],   rotation=270);
          Modelica.Blocks.Interfaces.RealInput vgs(redeclare type SignalType = 
                  Modelica.SIunits.Voltage) "Gate-source voltage" 
            annotation (extent=[30,90; 50,110],   rotation=270);
          equation 
            connect(Rd.heatPort,Tdev1. port) annotation (points=[60,
                  -1.22461e-015; 60,-70; -10,-70], style(color=42, rgbcolor={
                    191,0,0}));
            connect(Tdev1.T, nlCJ1.Tdev) annotation (points=[-30,-70; -40,-70;
                  -40,-30; -26,-30; -26,-16], style(color=74, rgbcolor={0,0,127}));
            connect(nlCJ1.y, VarC1.u) annotation (points=[-52.4,0; -56.7,0;
                  -56.7,1.10215e-015; -61,1.10215e-015], style(color=74,
                  rgbcolor={0,0,127}));
            connect(Tdev1.port, heatPort) annotation (points=[-10,-70; 0,-70; 0,
                  -100], style(color=42, rgbcolor={191,0,0}));
            connect(Rd.n, n) annotation (points=[70,-20; 70,-40; 88,-40; 88,0;
                  100,0], style(color=3, rgbcolor={0,0,255}));
            connect(DjJ1.n, Rd.n) annotation (points=[40,-20; 40,-40; 70,-40;
                  70,-20], style(color=3, rgbcolor={0,0,255}));
            connect(VarC1.n, DjJ1.n) annotation (points=[-70,-20; -70,-40; 40,
                  -40; 40,-20], style(color=3, rgbcolor={0,0,255}));
            connect(Rd.p, DjJ1.p) annotation (points=[70,20; 70,40; 40,40; 40,
                  20], style(color=3, rgbcolor={0,0,255}));
            connect(DjJ1.p, VarC1.p) annotation (points=[40,20; 40,40; -70,40;
                  -70,20], style(color=3, rgbcolor={0,0,255}));
            connect(VarC1.p, p) annotation (points=[-70,20; -70,40; -86,40; -86,
                  0; -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(DjJ1.heatPort, Tdev1.port) annotation (points=[30,
                  -1.22461e-015; 30,-70; -10,-70], style(color=42, rgbcolor={
                    191,0,0}));
            connect(nlCJ1.u1, vds) annotation (points=[2.8,9.6; 12,9.6; 12,70;
                  -40,70; -40,100], style(color=74, rgbcolor={0,0,127}));
            connect(nlCJ1.u2, vgs) annotation (points=[2.8,-9.6; 14,-9.6; 14,70;
                  40,70; 40,100], style(color=74, rgbcolor={0,0,127}));
          end CjJ;
          
          model DjJ "Spice-style diode model for JFETs and MESFETs" 
            extends Interfaces.SpiceTwoPort;
            parameter Integer Level=1 
              "JFET modeling level (check documentation layer for details)";
            parameter Modelica.SIunits.Current IS=1e-14 
              "Saturation current at reference temperature";
            parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI = if Level==3 then 2 else 3 
              "Saturation current temperature exponent";
            parameter Real Area=1 "Relative area occupied by diode";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            
          protected 
            Junctions.J1p3 J1p3_1         annotation (extent=[-12,28; 12,52]);
            Bonds.Bond B1         annotation (extent=[-38,28; -12,52]);
            Bonds.Bond B2         annotation (extent=[12,28; 38,52]);
            Bonds.Bond B3         annotation (extent=[-12,4; 12,28],    rotation=
                  270);
            BondLib.Electrical.Analog.Interfaces.El2BG El2BG1 annotation (extent=[
                  -62,26; -38,54]);
            BondLib.Electrical.Analog.Interfaces.BG2El BG2El1 annotation (extent=[38,
                  26; 64,54]);
            BondLib.Bonds.eBond B4 
                           annotation (extent=[-12,-56; 12,-32], rotation=-90);
            BondLib.Spice.Utilities.DjJ DjJ1(
              Tnom=Tnom,
              Level=Level,
              IS=IS,
              EG=EG,
              N=N,
              XTI=XTI,
              Area=Area,
              EMin=EMin,
              EMax=EMax) annotation (extent=[-12,-32; 12,4], rotation=-90);
            Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
              annotation (extent=[-12,-80; 12,-56], rotation=-90);
          equation 
            connect(B1.BondCon2,J1p3_1. BondCon1) annotation (points=[-12,40;
                  -12,40],
                        style(color=8));
            connect(J1p3_1.BondCon2,B2. BondCon1) annotation (points=[12,40;
                  12.26,40], style(color=8));
            connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-38,40;
                  -37.74,40],
                        style(color=8));
            connect(B2.BondCon2,BG2El1. BondCon1) annotation (points=[38,40; 38,
                  40],style(color=8));
            connect(J1p3_1.BondCon3,B3. BondCon1) annotation (points=[0,28; 0,
                  27.76; 2.16021e-015,27.76], style(color=8, rgbcolor={192,192,
                    192}));
            annotation (
              Diagram,
              Documentation(info="<html>
The Spice-style JFET junction diode is a regular exponential diode. <p>

<hr>

<pre>
Parameters:

 <font color=red><b>Level:  JFET modeling level (default value = 1)
           Level=1: Shichman-Hodges (Si) JFET model
           Level=2: Improved Shichman-Hodges (Si) JFET model [1]
           Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 IS:     Transport saturation current (default value = 1e-14 Amp)

 EG:     Energy gap for temperature effect on saturation current at 0 K
           <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
           default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:      Current emission coefficient (default value = 1)

 XTI:    Saturation current temperature exponent
           <font color=blue><b>default value = 3 for Levels=1,2 (Si)
           default value = 2 for Level=3 (GaAs)</b></font>

 Area:   Relative area occupied by the diode (default value = 1)

 EMin:   if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:   if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"),           Icon(
                Polygon(points=[30,0; -30,40; -30,-40; 30,0],     style(
                    color=3,
                    gradient=0,
                    fillColor=7)),
                Line(points=[-90,0; -30,0]),
                Line(points=[30,0; 90,0]),
                Line(points=[30,40; 30,-40],   style(color=3)),
                Line(points=[0,-20; 0,-91],   style(color=42)),
                Text(
                  extent=[-20,14; 8,-14],
                  style(color=3, rgbcolor={0,0,255}),
                  string="J")));
            connect(El2BG1.p, p) annotation (points=[-62,40; -80,40; -80,0;
                  -100,0], style(color=3, rgbcolor={0,0,255}));
            connect(BG2El1.n, n) annotation (points=[64,40; 80,40; 80,0; 100,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(DjJ1.BondCon1, B3.BondCon2) annotation (points=[
                  -7.34764e-016,4; -2.20429e-015,4], style(color=8, rgbcolor={
                    192,192,192}));
            connect(DjJ1.BondCon2, B4.fBondCon1) annotation (points=[
                  7.34764e-016,-32; -7.34764e-016,-32], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.BondCon1, B4.eBondCon1) annotation (points=[
                  -7.34764e-016,-56; 7.34764e-016,-56], style(color=8, rgbcolor=
                   {192,192,192}));
            connect(BG2Heat1.port_b, heatPort) annotation (points=[7.34764e-016,
                  -80; 0,-80; 0,-100], style(color=42, rgbcolor={191,0,0}));
          end DjJ;
          
          model Jni "Spice-style internal n-channel JFET" 
            parameter Integer Level=1 
              "JFET modeling level (check documentation layer for details)";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VT0=0 
              "Threshold voltage at reference temperature";
            parameter Modelica.SIunits.Current IS=1.e-14 
              "Transport saturation current at reference temperature";
            parameter Modelica.SIunits.Transconductance BETA=0 
              "Transconductance parameter at reference temperature";
            parameter Modelica.SIunits.InversePotential LAMBDA=0 
              "Channel length modulation";
            parameter Real TCV(unit="V/K") = 0 
              "Linear temperature coefficient of threshold voltage";
            parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI = if Level==3 then 2 else 3 
              "Saturation current temperature exponent";
            parameter Modelica.SIunits.Voltage PB=0.8 
              "Built-in potential at reference temperature";
            parameter Modelica.SIunits.Capacitance CGS=0.5e-12 
              "Zero-bias gate-source depletion capacitance at reference temperature";
            parameter Modelica.SIunits.Capacitance CGD=0.5e-12 
              "Zero-bias gate-drain depletion capacitance at reference temperature";
            parameter Real M = if Level==3 then 0.5 else 0.33 
              "Junction grading coefficient";
            parameter Real FC=0.5 
              "Factor for linearization of depletion capacitance formula";
            parameter Modelica.SIunits.Time TT=5e-9 "Ideal transit time";
            parameter Real BEX=0 "Mobility temperature exponent";
            parameter Modelica.SIunits.InversePotential LAM1=0 
              "Channel length modulation gate voltage parameter";
            parameter Modelica.SIunits.InversePotential ALPHA=1 
              "Hyperbolic tangient fitting parameter";
            parameter Modelica.SIunits.InversePotential B=0 
              "Measure of doping profile";
            parameter Modelica.SIunits.Voltage DELTA=0.2 
              "Voltage range for transition";
            parameter Modelica.SIunits.Voltage VMAX=0.5 
              "Limit voltage for carrier velocity saturation";
            parameter Modelica.SIunits.Conductance GminDC=1e-16 
              "Leakage conductance";
            parameter Real Area=1 "Relative area occupied by capacitor";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
              annotation (extent=[-90,-40; -110,-60]);
            Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
              annotation (extent=[90,40; 110,60]);
            Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
              annotation (extent=[90,-40; 110,-60]);
            annotation (
              Diagram(
                Ellipse(extent=[84,-48; 88,-52], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(
                  extent=[-40,18; -28,12],
                  string="vds",
                  style(color=10, rgbcolor={135,135,135})),
                Text(
                  extent=[-22,66; -8,60],
                  string="vds",
                  style(color=10, rgbcolor={135,135,135})),
                Text(
                  extent=[-20,-62; -8,-68],
                  string="vds",
                  style(color=10, rgbcolor={135,135,135})),
                Text(
                  extent=[10,-62; 22,-68],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vgs"),
                Text(
                  extent=[-40,-6; -28,-12],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vgs"),
                Text(
                  extent=[10,66; 22,60],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vgs"),
                Text(
                  extent=[2,-22; 14,-28],
                  style(color=10, rgbcolor={135,135,135}),
                  string="Tdev")),
              Icon(
                Line(points=[0,50; 91,50]),
                Line(points=[0,-50; 90,-50]),
                Line(points=[-90,-50; 0,-50]),
                Line(points=[0,-70; 0,70]),
                Polygon(points=[0,-50; -20,-46; -20,-54; 0,-50],
                                                              style(fillColor=3,
                      fillPattern=1)),
                Line(points=[0,-91; 0,-80],   style(color=42)),
                Text(extent=[-100,80; 100,100],   string="%name")),
              Documentation(info="<html>
The Spice-style internal n-channel JFET model contains the non-linear current source and the two junction diodes (including the junctions capacitances). <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 IS:      Transport saturation current (default value = 1e-14 Amp)

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 LAMBDA:  Channel length modulation (default value = 0 1/Volt)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 EG:      Energy gap for temperature effect on saturation current at 0 K
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F)

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F)

 M:       Junction grading coefficient
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 TT:      Ideal transit time (default value = 5e-9 sec)

 BEX:     Mobility temperature exponent (default value = 0)

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt)

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1)

 B:       Measure of doping profile (default value = 0 1/Volt)

 DELTA:   Voltage range for transition (default value = 0.2 Volt)

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            Sensors.VoltageSensor Vds 
              annotation (extent=[96,-10; 76,10],  rotation=-90);
            Isignal Ids annotation (extent=[64,-14; 36,14],  rotation=-90);
            BondLib.Spice.Utilities.IDS1 IDS(
              Level=Level,
              Tnom=Tnom,
              VT0=VT0,
              BETA=BETA,
              LAMBDA=LAMBDA,
              TCV=TCV,
              BEX=BEX,
              LAM1=LAM1,
              ALPHA=ALPHA,
              B=B) annotation (extent=[-20,-20; 20,20]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-20,-80; -40,-60]);
            BondLib.Electrical.Analog.Spice.Utilities.CjJ Dgd(
              Tnom=Tnom,
              Level=Level,
              VT0=VT0,
              IS=IS,
              TCV=TCV,
              EG=EG,
              N=N,
              XTI=XTI,
              PB=PB,
              CGS=CGS,
              CGD=CGD,
              M=M,
              FC=FC,
              TT=TT,
              ALPHA=ALPHA,
              DELTA=DELTA,
              VMAX=VMAX,
              GminDC=GminDC,
              Area=Area,
              GS=false,
              EMin=EMin,
              EMax=EMax,
              enforceStates=enforceStates) 
                         annotation (extent=[-20,36; 20,64]);
            BondLib.Electrical.Analog.Spice.Utilities.CjJ Dgs(
              Tnom=Tnom,
              Level=Level,
              VT0=VT0,
              IS=IS,
              TCV=TCV,
              EG=EG,
              N=N,
              XTI=XTI,
              PB=PB,
              CGS=CGS,
              CGD=CGD,
              M=M,
              FC=FC,
              TT=TT,
              ALPHA=ALPHA,
              DELTA=DELTA,
              VMAX=VMAX,
              GminDC=GminDC,
              Area=Area,
              EMin=EMin,
              EMax=EMax,
              GS=true,
              enforceStates=enforceStates) 
                       annotation (extent=[-20,-36; 20,-64]);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vgs 
              annotation (extent=[40,-94; 60,-74], rotation=0);
          equation 
            connect(IDS.y, Ids.InPort1) annotation (points=[22,0; 36.8,0; 36.8,
                  2.29069e-015; 41.6,2.29069e-015], style(color=74, rgbcolor={0,
                    0,127}));
            connect(Vds.n, S) annotation (points=[86,-10; 86,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Vds.p, D) annotation (points=[86,10; 86,50; 100,50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Ids.p, D) annotation (points=[50,14; 50,50; 100,50], style(
                  color=3, rgbcolor={0,0,255}));
            connect(Ids.n, S) annotation (points=[50,-14; 50,-50; 100,-50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Ids.heatPort, heatPort) annotation (points=[64,9.19132e-016;
                  64,0; 68,0; 68,-70; 0,-70; 0,-100], style(color=42, rgbcolor=
                    {191,0,0}));
            connect(Tdev1.port, heatPort) annotation (points=[-20,-70; 0,-70; 0,
                  -100], style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.T, IDS.Tdev) annotation (points=[-40,-70; -50,-70;
                  -50,-28; 0,-28; 0,-20], style(color=74, rgbcolor={0,0,127}));
            connect(Dgd.heatPort, Ids.heatPort) annotation (points=[0,36; 68,36;
                  68,0; 64,0; 64,9.19132e-016], style(color=42, rgbcolor={191,0,
                    0}));
            connect(Dgd.n, D) annotation (points=[20,50; 100,50], style(color=3,
                  rgbcolor={0,0,255}));
            connect(Dgd.p, G) annotation (points=[-20,50; -80,50; -80,-50; -100,
                  -50],
                style(color=3, rgbcolor={0,0,255}));
            connect(Dgs.n, S) annotation (points=[20,-50; 100,-50], style(color=
                   3, rgbcolor={0,0,255}));
            connect(Dgs.p, G) annotation (points=[-20,-50; -100,-50],
                      style(color=3, rgbcolor={0,0,255}));
            connect(Dgs.heatPort, Ids.heatPort) annotation (points=[0,-36; 68,
                  -36; 68,0; 64,0; 64,9.19132e-016], style(color=42, rgbcolor={
                    191,0,0}));
            connect(Vgs.n, S) annotation (points=[60,-84; 86,-84; 86,-50; 100,
                  -50], style(color=3, rgbcolor={0,0,255}));
            connect(Vgs.p, G) annotation (points=[40,-84; -80,-84; -80,-50;
                  -100,-50],
                      style(color=3, rgbcolor={0,0,255}));
            connect(Vds.V, Dgd.vds) annotation (points=[96,-6.12303e-016; 100,
                  -6.12303e-016; 100,0; 108,0; 108,80; -8,80; -8,64], style(
                  color=74, rgbcolor={0,0,127}));
            connect(Dgd.vds, IDS.u1) annotation (points=[-8,64; -8,80; -60,80;
                  -60,12; -24,12], style(color=74, rgbcolor={0,0,127}));
            connect(IDS.u1, Dgs.vds) annotation (points=[-24,12; -60,12; -60,
                  -92; -8,-92; -8,-64], style(color=74, rgbcolor={0,0,127}));
            connect(Vgs.V, Dgd.vgs) annotation (points=[50,-94; 50,-100; 32,
                  -100; 32,72; 8,72; 8,64], style(color=74, rgbcolor={0,0,127}));
            connect(Vgs.V, Dgs.vgs) annotation (points=[50,-94; 50,-100; 32,
                  -100; 32,-74; 8,-74; 8,-64], style(color=74, rgbcolor={0,0,
                    127}));
            connect(Dgd.vgs, IDS.u2) annotation (points=[8,64; 8,72; -50,72;
                  -50,-12; -24,-12], style(color=74, rgbcolor={0,0,127}));
          end Jni;
          
          model Jpi "Spice-style internal p-channel JFET" 
            parameter Integer Level=1 
              "JFET modeling level (check documentation layer for details)";
            parameter Modelica.SIunits.Temperature Tnom=300.15 
              "Reference temperature";
            parameter Modelica.SIunits.Voltage VT0=0 
              "Threshold voltage at reference temperature";
            parameter Modelica.SIunits.Current IS=1.e-14 
              "Transport saturation current at reference temperature";
            parameter Modelica.SIunits.Transconductance BETA=0 
              "Transconductance parameter at reference temperature";
            parameter Modelica.SIunits.InversePotential LAMBDA=0 
              "Channel length modulation";
            parameter Real TCV(unit="V/K") = 0 
              "Linear temperature coefficient of threshold voltage";
            parameter Modelica.SIunits.Voltage EG = if Level==3 then 0.73 else 1.16 
              "Energy gap for temperature effect on saturation current at 0 K";
            parameter Real N=1 "Current emission coefficient";
            parameter Real XTI = if Level==3 then 2 else 3 
              "Saturation current temperature exponent";
            parameter Modelica.SIunits.Voltage PB=0.8 
              "Built-in potential at reference temperature";
            parameter Modelica.SIunits.Capacitance CGS=0.5e-12 
              "Zero-bias gate-source depletion capacitance at reference temperature";
            parameter Modelica.SIunits.Capacitance CGD=0.5e-12 
              "Zero-bias gate-drain depletion capacitance at reference temperature";
            parameter Real M = if Level==3 then 0.5 else 0.33 
              "Junction grading coefficient";
            parameter Real FC=0.5 
              "Factor for linearization of depletion capacitance formula";
            parameter Modelica.SIunits.Time TT=5e-9 "Ideal transit time";
            parameter Real BEX=0 "Mobility temperature exponent";
            parameter Modelica.SIunits.InversePotential LAM1=0 
              "Channel length modulation gate voltage parameter";
            parameter Modelica.SIunits.InversePotential ALPHA=1 
              "Hyperbolic tangient fitting parameter";
            parameter Modelica.SIunits.InversePotential B=0 
              "Measure of doping profile";
            parameter Modelica.SIunits.Voltage DELTA=0.2 
              "Voltage range for transition";
            parameter Modelica.SIunits.Voltage VMAX=0.5 
              "Limit voltage for carrier velocity saturation";
            parameter Modelica.SIunits.Conductance GminDC=1e-16 
              "Leakage conductance";
            parameter Real Area=1 "Relative area occupied by capacitor";
            parameter Real EMin=-100 
              "if x < EMin, the exp(x) function is linearized";
            parameter Real EMax=40 
              "if x > EMax, the exp(x) function is linearized";
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            Modelica.Electrical.Analog.Interfaces.Pin G "Gate" 
              annotation (extent=[-90,-40; -110,-60]);
            Modelica.Electrical.Analog.Interfaces.Pin D "Drain" 
              annotation (extent=[90,40; 110,60]);
            Modelica.Electrical.Analog.Interfaces.Pin S "Source" 
              annotation (extent=[90,-40; 110,-60]);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                    300.15)) "Thermal connector" 
              annotation (extent=[-10,-90; 10,-110],   rotation=-90);
            Sensors.VoltageSensor Vsd 
              annotation (extent=[96,10; 76,-10],  rotation=-90);
            Isignal Isd annotation (extent=[64,14; 36,-14],  rotation=-90);
            BondLib.Spice.Utilities.IDS1 ISD(
              Level=Level,
              Tnom=Tnom,
              VT0=VT0,
              BETA=BETA,
              LAMBDA=LAMBDA,
              TCV=TCV,
              BEX=BEX,
              LAM1=LAM1,
              ALPHA=ALPHA,
              B=B) annotation (extent=[-20,-20; 20,20]);
            Modelica.Thermal.HeatTransfer.TemperatureSensor Tdev1 
              annotation (extent=[-20,-80; -40,-60]);
            BondLib.Electrical.Analog.Spice.Utilities.CjJ Ddg(
              Tnom=Tnom,
              Level=Level,
              VT0=VT0,
              IS=IS,
              TCV=TCV,
              EG=EG,
              N=N,
              XTI=XTI,
              PB=PB,
              CGS=CGS,
              CGD=CGD,
              M=M,
              FC=FC,
              TT=TT,
              ALPHA=ALPHA,
              DELTA=DELTA,
              VMAX=VMAX,
              GminDC=GminDC,
              Area=Area,
              GS=false,
              EMin=EMin,
              EMax=EMax,
              enforceStates=enforceStates) 
                         annotation (extent=[20,36; -20,64]);
            BondLib.Electrical.Analog.Spice.Utilities.CjJ Dsg(
              Tnom=Tnom,
              Level=Level,
              VT0=VT0,
              IS=IS,
              TCV=TCV,
              EG=EG,
              N=N,
              XTI=XTI,
              PB=PB,
              CGS=CGS,
              CGD=CGD,
              M=M,
              FC=FC,
              TT=TT,
              ALPHA=ALPHA,
              DELTA=DELTA,
              VMAX=VMAX,
              GminDC=GminDC,
              Area=Area,
              EMin=EMin,
              EMax=EMax,
              GS=true,
              enforceStates=enforceStates) 
                       annotation (extent=[20,-36; -20,-64]);
            BondLib.Electrical.Analog.Sensors.VoltageSensor Vsg 
              annotation (extent=[60,-94; 40,-74], rotation=0);
            annotation (
              Diagram(
                Ellipse(extent=[84,-48; 88,-52], style(
                    color=3,
                    rgbcolor={0,0,255},
                    fillColor=3,
                    rgbfillColor={0,0,255})),
                Text(
                  extent=[-40,18; -28,12],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsd"),
                Text(
                  extent=[-22,66; -8,60],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsg"),
                Text(
                  extent=[-20,-62; -8,-68],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsg"),
                Text(
                  extent=[10,-62; 22,-68],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsd"),
                Text(
                  extent=[-40,-6; -28,-12],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsg"),
                Text(
                  extent=[10,66; 22,60],
                  style(color=10, rgbcolor={135,135,135}),
                  string="vsd"),
                Text(
                  extent=[2,-22; 14,-28],
                  style(color=10, rgbcolor={135,135,135}),
                  string="Tdev")),
              Icon(
                Line(points=[0,50; 91,50]),
                Line(points=[0,-50; 90,-50]),
                Line(points=[-90,-50; 0,-50]),
                Line(points=[0,-70; 0,70]),
                Polygon(points=[-60,-50; -40,-46; -40,-54; -60,-50],
                                                              style(fillColor=3,
                      fillPattern=1)),
                Line(points=[0,-91; 0,-80],   style(color=42)),
                Text(extent=[-100,80; 100,100],   string="%name")),
              Documentation(info="<html>
The Spice-style internal p-channel JFET model contains the non-linear current source and the two junction diodes (including the junctions capacitances). <p>

<hr> <p>

<pre>
Parameters:

 <font color=red><b>Level:   JFET modeling level (default value = 1)
            Level=1: Shichman-Hodges (Si) JFET model
            Level=2: Improved Shichman-Hodges (Si) JFET model [1]
            Level=3: Statz-Curtice (GaAs) MESFET model [2,3]<b></font>

 Tnom:    Reference temperature (default value = 300.15 K)

 VT0:     Threshold voltage at reference temperature (default value = 0 Volt)
            <font color=blue><b>VT0 &gt; 0 for enhancement JFET
            VT0 &lt; 0 for depletion JFET</b></font>

 IS:      Transport saturation current (default value = 1e-14 Amp)

 BETA:    Transconductance parameter at reference temperature (default value = 0 Amp/Volt<sup>2</sup>)

 LAMBDA:  Channel length modulation (default value = 0 1/Volt)

 TCV:     Linear temperature coefficient of threshold voltage (default value = 0 Volt/K)

 EG:      Energy gap for temperature effect on saturation current at 0 K
            <font color=blue><b>default value = 1.16 Volt for Levels=1,2 (Si)
            default value = 0.73 Volt for Level=3 (GaAs)</b></font>

 N:       Current emission coefficient (default value = 1)

 XTI:     Saturation current temperature exponent
            <font color=blue><b>default value = 3 for Levels=1,2 (Si)
            default value = 2 for Level=3 (GaAs)</b></font>

 PB:      Built-in potential at reference temperature (default value = 0.8 Volt)

 CGS:     Zero-bias gate-source depletion capacitance at reference temperature (default value = 0.5e-12 F)

 CGD:     Zero-bias gate-drain depletion capacitance at reference temperature (default value = 0.5e-12 F)

 M:       Junction grading coefficient
            <font color=blue><b>default value = 0.33 for Levels=1,2 (Si)
            default value = 0.5  for Level=3 (GaAs)</b></font>

 FC:      Forward-bias depletion capacitance coefficient (default value = 0.5)

 TT:      Ideal transit time (default value = 5e-9 sec)

 BEX:     Mobility temperature exponent (default value = 0)

 LAM1:    Channel length modulation gate voltage parameter (default value = 0 1/Volt)

 ALPHA:   Hyperbolic tangient fitting parameter (default value = 1)

 B:       Measure of doping profile (default value = 0 1/Volt)

 DELTA:   Voltage range for transition (default value = 0.2 Volt)

 VMAX:    Limit voltage for carrier velocity saturation (default value = 0.5 Volt)

 GminDC:  Leakage conductance (default value = 1e-15 Mho)

 Area:    Relative area occupied by the diode (default value = 1)

 EMin:    if x &lt; EMin, the exp(x) function is linearized (default value = -100)

 EMax:    if x &gt; EMax, the exp(x) function is linearized (default value = 40)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.153.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York, p.382.
<li> Statz, H., P. Newman, I.W. Smith, R.A. Pucel, and H.A. Hans (1987), \"GaAs FET Device and Circuit Simulation in Spice,\"
     <i>IEEE Trans. Electron Devices</i>, <b>34</b>(2), pp. 160-169.
</ol>
</html>
"));
          equation 
            connect(ISD.y, Isd.InPort1) annotation (points=[22,0; 36.8,0; 36.8,
                  -2.29069e-015; 41.6,-2.29069e-015], style(color=74, rgbcolor=
                    {0,0,127}));
            connect(Isd.heatPort, heatPort) annotation (points=[64,
                  -9.19132e-016; 64,0; 68,0; 68,-70; 0,-70; 0,-100], style(
                  color=42, rgbcolor={191,0,0}));
            connect(Tdev1.port, heatPort) annotation (points=[-20,-70; 0,-70; 0,
                  -100], style(color=42, rgbcolor={191,0,0}));
            connect(Tdev1.T, ISD.Tdev) annotation (points=[-40,-70; -50,-70;
                  -50,-28; 0,-28; 0,-20], style(color=74, rgbcolor={0,0,127}));
            connect(Ddg.heatPort, Isd.heatPort) annotation (points=[0,36; 68,36;
                  68,0; 64,0; 64,-9.19132e-016], style(color=42, rgbcolor={191,
                    0,0}));
            connect(Dsg.heatPort, Isd.heatPort) annotation (points=[0,-36; 68,
                  -36; 68,0; 64,0; 64,-9.19132e-016], style(color=42, rgbcolor=
                    {191,0,0}));
            connect(Vsd.V, Ddg.vds) annotation (points=[96,6.12303e-016; 100,
                  6.12303e-016; 100,0; 108,0; 108,72; 8,72; 8,64], style(color=
                    74, rgbcolor={0,0,127}));
            connect(Ddg.vds, ISD.u1) annotation (points=[8,64; 8,72; -50,72;
                  -50,12; -24,12], style(color=74, rgbcolor={0,0,127}));
            connect(Vsg.V, Ddg.vgs) annotation (points=[50,-94; 50,-98; 32,-98;
                  32,80; -8,80; -8,64], style(color=74, rgbcolor={0,0,127}));
            connect(Ddg.vgs, ISD.u2) annotation (points=[-8,64; -8,80; -54,80;
                  -54,-12; -24,-12], style(color=74, rgbcolor={0,0,127}));
            connect(D, Vsd.n) annotation (points=[100,50; 86,50; 86,10], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(S, Vsd.p) annotation (points=[100,-50; 86,-50; 86,-10],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Vsg.p, S) annotation (points=[60,-84; 86,-84; 86,-50; 100,
                  -50], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Vsg.n, G) annotation (points=[40,-84; -80,-84; -80,-50;
                  -100,-50],
                      style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Isd.n, D) annotation (points=[50,14; 50,50; 100,50], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Isd.p, S) annotation (points=[50,-14; 50,-50; 100,-50],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Ddg.p, D) annotation (points=[20,50; 100,50], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Ddg.n, G) annotation (points=[-20,50; -80,50; -80,-50; -100,
                  -50],
                style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Dsg.p, S) annotation (points=[20,-50; 100,-50], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Dsg.n, G) annotation (points=[-20,-50; -100,-50],
                      style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(ISD.u2, Dsg.vgs) annotation (points=[-24,-12; -54,-12; -54,
                  -88; -8,-88; -8,-64], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
            connect(Vsd.V, Dsg.vds) annotation (points=[96,6.12303e-016; 108,
                  6.12303e-016; 108,-100; 20,-100; 20,-76; 8,-76; 8,-64], style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1));
          end Jpi;
        end Utilities;
      end Spice;
      
      package Examples 
        "Examples of analog electrical circuits copied from standard library" 
        annotation (preferedView="info",Icon(
            Rectangle(extent=[-100,-100; 80,50], style(
                color=0,
                rgbcolor={0,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                color=0,
                rgbcolor={0,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                color=0,
                rgbcolor={0,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Text(
              extent=[-118,114; 122,65],
              string="%name",
              style(color=1)),
            Text(
              extent=[-85,63; 65,-57],
              string="Sample",
              style(color=0)),
            Text(
              extent=[-97,15; 78,-106],
              string="Programs",
              style(color=0))), Documentation(info="<html>
This package contains a number of examples copied from the standard analog electrical library, except that each of the modeling elements has been replaced by the corresponding modeling element from the analog electrical library built on bond graph technology.  These models were placed here to verify the correctness of the bond graph library.
</html>
"));
        model CharacteristicIdealDiodes "Characteristic of ideal diodes" 
          annotation (preferedView="info",Documentation(info="<html>
<P>
Three examples of ideal diodes are shown:<br><br>
the <b>totally ideal diode</b> (Ideal) with all parameters to be zero <br>
the <b>nearly ideal diode</b> with <i>Ron=0.1</i> and <i>Goff=0.1</i> <br>
the nearly ideal but <b>displaced diode</b> with <i>Vknee=5</i> and <i>Ron=0.1</i> and 
<i>Goff=0.1</i> <br><br>
The resistance and conductance are chosen untypically high since the slopes should
be seen in the graphics.
</P>
<P>
Simulate until T=1 s.
</P>
<P>
Plot in separate windows:<br> <br>
Ideal.i versus Ideal.v <br>
With_Ron_Goff.i versus With_Ron_Goff.v<br>
With_Ron_Goff_Vknee.i versus With_Ron_Goff_Vknee.v<br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>
"),         Diagram(Text(extent=[-90,96; 90,42], string=
                  "Characteristic Ideal Diodes")),
            Icon(
              Rectangle(extent=[-100,-100; 80,50],   style(fillColor=7)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=7)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                    fillColor=7)),
              Text(
                extent=[-96,3; 77,-49],
                string="Example",
                style(color=3)),
              Text(
                extent=[-120,132; 120,73],
                string="%name",
                style(color=1))),
            experiment,
            experimentSetupOutput);
          Modelica.Electrical.Analog.Basic.Ground Ground1 
          annotation (extent=[-52,-86; -32,-66]);
          Sources.SineVoltage SineVoltage1(V=10, offset=-9) 
            annotation (extent=[-32,4; -52,-16], rotation=90);
          Sources.SineVoltage SineVoltage2(V=10, offset=0) 
            annotation (extent=[-52,44; -72,24], rotation=90);
          Sources.SineVoltage SineVoltage3(V=10, offset=0) 
            annotation (extent=[-12,-36; -32,-56], rotation=90);
          Basic.Resistor R1(R=1e-3) annotation (extent=[58,34; 78,54]);
          Basic.Resistor R2(R=1e-3) annotation (extent=[58,-6; 78,14]);
          Basic.Resistor R3(R=1e-3) annotation (extent=[58,-46; 78,-26]);
          BondLib.Electrical.Analog.Ideal.IdealDiode Ideal(Ron=0, Goff=0) 
            annotation (extent=[-2,34; 18,54]);
          BondLib.Electrical.Analog.Ideal.IdealDiode With_Ron_Goff(Ron=0.1, Goff=
               0.1) annotation (extent=[-2,-6; 18,14]);
          BondLib.Electrical.Analog.Ideal.IdealDiode With_Ron_Goff_Vknee(
            Ron=0.2,
            Goff=0.2,
            Vknee=5) annotation (extent=[-2,-46; 18,-26]);
        equation 
          connect(SineVoltage2.n, Ground1.p) annotation (points=[-62,24; -62,
                -66; -42,-66], style(color=3, rgbcolor={0,0,255}));
          connect(Ideal.p, SineVoltage2.p) annotation (points=[-2,44; -62,44],
              style(color=3, rgbcolor={0,0,255}));
          connect(Ideal.n, R1.p) annotation (points=[18,44; 58,44], style(color=
                 3, rgbcolor={0,0,255}));
          connect(R1.n, R2.n) annotation (points=[78,44; 78,4], style(color=3,
                rgbcolor={0,0,255}));
          connect(SineVoltage1.n, Ground1.p) annotation (points=[-42,-16; -42,
                -66; -42,-66], style(color=3, rgbcolor={0,0,255}));
          connect(SineVoltage1.p, With_Ron_Goff.p) annotation (points=[-42,4;
                -2,4], style(color=3, rgbcolor={0,0,255}));
          connect(With_Ron_Goff.n, R2.p) annotation (points=[18,4; 58,4], style(
                color=3, rgbcolor={0,0,255}));
          connect(R2.n, R3.n) annotation (points=[78,4; 78,-36], style(color=3,
                rgbcolor={0,0,255}));
          connect(SineVoltage3.n, Ground1.p) annotation (points=[-22,-56; -22,
                -66; -42,-66], style(color=3, rgbcolor={0,0,255}));
          connect(SineVoltage3.p, With_Ron_Goff_Vknee.p) annotation (points=[
                -22,-36; -2,-36], style(color=3, rgbcolor={0,0,255}));
          connect(With_Ron_Goff_Vknee.n, R3.p) annotation (points=[18,-36; 58,
                -36], style(color=3, rgbcolor={0,0,255}));
          connect(R3.n, Ground1.p) annotation (points=[78,-36; 78,-66; -42,-66],
              style(color=3, rgbcolor={0,0,255}));
        end CharacteristicIdealDiodes;
        
        model DifferenceAmplifier "Differential amplifier" 
          annotation (preferedView="info",
            Icon(
              Rectangle(extent=[-100,-100; 80,50],   style(fillColor=7)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=7)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                    fillColor=7)),
              Text(
                extent=[-96,3; 77,-49],
                string="Example",
                style(color=3)),
              Text(
                extent=[-120,132; 120,73],
                string="%name",
                style(color=1))),
            Documentation(info="<html>
<p>
It is a simple NPN transistor amplifier circuit. The voltage difference between R1.p and R3.n is amplified. The output signal is the voltage between R2.n and R4.n. In this example the voltage at V1 is amplified because R3.n is grounded.
<p>
The simulation end time should be set to 1e- 8. Please plot the input voltage V1.v, and the output voltages R2.n.v, and R4.n.v.
<p>
<dl>
<dt>
<b>Reference:</b>
<dd>
Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 59
<p>
<dt>
<b>Main Authors:</b>
<dd>
<a href=\"http://people.eas.iis.fhg.de/Christoph.Clauss/\">Christoph Clau&szlig;</a>
    &lt;<a href=\"mailto:clauss@eas.iis.fhg.de\">clauss@eas.iis.fhg.de</a>&gt;<br>
    <a href=\"http://people.eas.iis.fhg.de/Andre.Schneider/\">Andr&eacute; Schneider</a>
    &lt;<a href=\"mailto:schneider@eas.iis.fhg.de\">schneider@eas.iis.fhg.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2002, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>
"),         Diagram,
            experiment(StopTime=1e-008),
            experimentSetupOutput);
          Basic.Ground Gnd1 annotation (extent=[-96,-80; -84,-70]);
          Basic.Ground Gnd2 annotation (extent=[-66,-80; -54,-70]);
          Basic.Ground Gnd3 annotation (extent=[-16,-20; -4,-10]);
          Basic.Ground Gnd4 annotation (extent=[84,-40; 96,-30]);
          Basic.Ground Gnd5 annotation (extent=[54,-80; 66,-70]);
          Basic.Ground Gnd6 annotation (extent=[4,-20; 16,-10]);
          Basic.Ground Gnd7 annotation (extent=[-16,-80; -4,-70]);
          Basic.Ground Gnd8 annotation (extent=[4,-80; 16,-70]);
          Basic.Ground Gnd9 annotation (extent=[74,20; 86,30]);
          Sources.ExpSineVoltage V1(
            V=0.2,
            freqHz=0.2e9,
            damping=0.1e8) annotation (extent=[-100,-60; -80,-40], rotation=-90);
          Basic.Resistor R1(R=0.0001) annotation (extent=[-90,-20; -70,0]);
          Basic.Resistor R2(R=100) 
            annotation (extent=[-50,20; -30,40], rotation=-90);
          Basic.Resistor R3(R=0.0001) annotation (extent=[70,-20; 90,0]);
          Basic.Resistor R4(R=100) 
            annotation (extent=[30,20; 50,40], rotation=-90);
          Basic.Capacitor C1(C=1e-10) 
            annotation (extent=[-70,-60; -50,-40], rotation=-90);
          Basic.Capacitor C2(C=1e-10) 
            annotation (extent=[-30,-14; -10,6], rotation=0);
          Basic.Capacitor C3(C=1e-10) 
            annotation (extent=[50,-60; 70,-40], rotation=-90);
          Basic.Capacitor C4(C=1e-10) 
            annotation (extent=[10,-14; 30,6], rotation=0);
          Basic.Capacitor C5(C=1e-10) 
            annotation (extent=[0,-60; 20,-40], rotation=-90);
          Sources.RampVoltage V2(V=15, duration=1e-9) 
            annotation (extent=[70,40; 90,60], rotation=-90);
          Sources.RampCurrent I1(I=0.16, duration=1e-9) 
            annotation (extent=[-20,-60; 0,-40], rotation=-90);
          Utilities.Transistor Transistor1 annotation (extent=[-60,-20; -40,0]);
          Utilities.Transistor Transistor2 annotation (extent=[60,-20; 40,0]);
        equation 
          connect(Gnd1.p, V1.n) annotation (points=[-90,-70; -90,-60], style(
                color=3, rgbcolor={0,0,255}));
          connect(V1.p, R1.p) annotation (points=[-90,-40; -90,-10], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gnd4.p, R3.n) annotation (points=[90,-30; 90,-10], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gnd2.p, C1.n) annotation (points=[-60,-70; -60,-60], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gnd3.p, C2.n) annotation (points=[-10,-10; -10,-4], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gnd5.p, C3.n) annotation (points=[60,-70; 60,-60], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gnd6.p, C4.p) annotation (points=[10,-10; 10,-4], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Gnd8.p, C5.n) annotation (points=[10,-70; 10,-60], style(
                color=3, rgbcolor={0,0,255}));
          connect(Gnd9.p, V2.n) annotation (points=[80,30; 80,40], style(color=
                  3, rgbcolor={0,0,255}));
          connect(V2.p, R4.p) annotation (points=[80,60; 80,70; 40,70; 40,40],
              style(color=3, rgbcolor={0,0,255}));
          connect(R2.p, V2.p) annotation (points=[-40,40; -40,70; 80,70; 80,60],
              style(color=3, rgbcolor={0,0,255}));
          connect(Gnd7.p, I1.n) annotation (points=[-10,-70; -10,-60], style(
                color=3, rgbcolor={0,0,255}));
          connect(C5.p, I1.p) annotation (points=[10,-40; -10,-40], style(color=
                 3, rgbcolor={0,0,255}));
          connect(R1.n, Transistor1.b) annotation (points=[-70,-10; -60,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(C1.p, Transistor1.b) annotation (points=[-60,-40; -60,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(I1.p, Transistor1.e) annotation (points=[-10,-40; -30,-40;
                -30,-16; -40,-16], style(color=3, rgbcolor={0,0,255}));
          connect(C2.p, Transistor1.c) annotation (points=[-30,-4; -40,-4],
              style(color=3, rgbcolor={0,0,255}));
          connect(R2.n, Transistor1.c) annotation (points=[-40,20; -40,-4],
              style(color=3, rgbcolor={0,0,255}));
          connect(C4.n, Transistor2.c) annotation (points=[30,-4; 40,-4], style(
                color=3, rgbcolor={0,0,255}));
          connect(Transistor2.c, R4.n) annotation (points=[40,-4; 40,20], style(
                color=3, rgbcolor={0,0,255}));
          connect(C5.p, Transistor2.e) annotation (points=[10,-40; 30,-40; 30,
                -16; 40,-16], style(color=3, rgbcolor={0,0,255}));
          connect(Transistor2.b, R3.p) annotation (points=[60,-10; 70,-10],
              style(color=3, rgbcolor={0,0,255}));
          connect(C3.p, Transistor2.b) annotation (points=[60,-40; 60,-10],
              style(color=3, rgbcolor={0,0,255}));
        end DifferenceAmplifier;
        
        model HeatingRectifier "Heating rectifier" 
          annotation (preferedView="info",
            Icon(
              Rectangle(extent=[-100,-100; 80,50],   style(fillColor=7)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=7)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                    fillColor=7)),
              Text(
                extent=[-96,3; 77,-49],
                string="Example",
                style(color=3)),
              Text(
                extent=[-120,132; 120,73],
                string="%name",
                style(color=1))),
            Diagram(        Text(extent=[-74,88; 20,60],   string=
                  "HeatingRectifier")),
            Documentation(info="<html>
<P>
The heating rectifier shows a heat flow always if the electrical capacitor is loaded.
</P>
<P>
Simulate until T=5 s.
</P>
<P>
Plot in separate windows:<br> <br>
SineVoltage1.v  and  Capacitor1.p.v <br>
HeatCapacitor1.port.T and HeatingDiode1.heatPort.T<br>
HeatingDiode1.heatPort.Q_flow<br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>
"),       experiment(StopTime=5));
          Modelica.Electrical.Analog.Basic.Ground G 
          annotation (extent=[-60,-14; -40,6]);
          Basic.Resistor R annotation (extent=[40,60; 60,80]);
          Basic.Capacitor Capacitor1 annotation (extent=[40,26; 60,46]);
          Sources.SineVoltage SineVoltage1 
            annotation (extent=[-60,16; -40,36], rotation=-90);
          Semiconductors.HeatingDiode HeatingDiode1 
            annotation (extent=[0,26; 20,46]);
          Thermal.HeatTransfer.Passive.ThermalConductor ThermalConductor1(G=10) 
            annotation (extent=[0,-34; 20,-14], rotation=-90);
          Thermal.HeatTransfer.Passive.HeatCapacitor HeatCapacitor1(C=1) 
            annotation (extent=[0,-54; 20,-74]);
        equation 
          connect(R.n, Capacitor1.n) annotation (points=[60,70; 60,36], style(
                color=3, rgbcolor={0,0,255}));
          connect(R.p, Capacitor1.p) annotation (points=[40,70; 40,36], style(
                color=3, rgbcolor={0,0,255}));
          connect(Capacitor1.n, G.p) annotation (points=[60,36; 60,6; -50,6],
              style(color=3, rgbcolor={0,0,255}));
          connect(SineVoltage1.n, G.p) annotation (points=[-50,16; -50,6],
              style(color=3, rgbcolor={0,0,255}));
          connect(HeatingDiode1.n, Capacitor1.p) annotation (points=[20,36; 40,
                36], style(color=3, rgbcolor={0,0,255}));
          connect(HeatingDiode1.p, SineVoltage1.p) annotation (points=[0,36;
                -50,36], style(color=3, rgbcolor={0,0,255}));
          connect(ThermalConductor1.port_a, HeatingDiode1.heatPort) annotation (
             points=[10,-13; 10,26], style(color=42, rgbcolor={191,0,0}));
          connect(HeatCapacitor1.port, ThermalConductor1.port_b) annotation (
              points=[10,-54; 10,-35], style(color=42, rgbcolor={191,0,0}));
        end HeatingRectifier;
        
        model Rectifier "B6 diode bridge" 
          parameter Modelica.SIunits.Voltage VAC=400 "RMS line-to-line";
          parameter Modelica.SIunits.Frequency f=50 "line frequency";
          parameter Modelica.SIunits.Inductance LAC=60E-6 "line inductor";
          parameter Modelica.SIunits.Resistance Ron=1E-3 
            "diode forward resistance";
          parameter Modelica.SIunits.Conductance Goff=1E-3 
            "diode backward conductance";
          parameter Modelica.SIunits.Voltage Vknee=2 "diode threshold voltage";
          parameter Modelica.SIunits.Capacitance CDC=15E-3 "DC capacitance";
          parameter Modelica.SIunits.Current IDC=500 "load current";
          parameter Modelica.SIunits.Voltage vC10 = VAC*sqrt(2)/2;
          parameter Modelica.SIunits.Voltage vC20 = VAC*sqrt(2)/2;
          output Modelica.SIunits.Voltage uDC;
          output Modelica.SIunits.Current iAC[3];
          output Modelica.SIunits.Voltage uAC[3];
          output Modelica.SIunits.Power Losses;
          annotation (preferedView="info",
            Icon(
              Rectangle(extent=[-100,-100; 80,50],   style(fillColor=7)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=7)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                    fillColor=7)),
              Text(
                extent=[-96,3; 77,-49],
                string="Example",
                style(color=3)),
              Text(
                extent=[-120,132; 120,73],
                string="%name",
                style(color=1))),
            Diagram(
            Text(
              extent=[-82,80; 78,60],
              string="Rectifier",
              style(color=0, rgbcolor={0,0,0})),
            Line(points=[-18,8; -18,-8; -20,-4; -16,-4; -18,-8],   style(
                  color=0, rgbcolor={0,0,0})),
            Line(points=[-32,12; -28,10; -32,8; -32,12],      style(color=0,
                  rgbcolor={0,0,0})),
            Line(points=[30,20; 30,-40; 28,-36; 32,-36; 30,-40],      style(
                  color=0, rgbcolor={0,0,0})),
            Text(
              extent=[-40,6; -24,-2],
              string="iAC",
              style(color=0, rgbcolor={0,0,0})),
            Text(
              extent=[-16,-2; 0,-10],
              string="uAC",
              style(color=0, rgbcolor={0,0,0})),
            Text(
              extent=[20,-26; 36,-34],
              string="uDC",
              style(color=0, rgbcolor={0,0,0}))),
            Documentation(info="<html>
<P>
The rectifier example shows a B6 diode bridge fed by a three phase sinusoidal voltage, loaded by a DC current.<br>
DC capacitors start at ideal no-load voltage, thus making easier initial transient.
</P>
<P>
Simulate until T=0.1 s,<br>
Set communication interval to 1e-5, and<br>
Limit output storage to defined output variables only.<p>

Plot in separate windows:<br><br>
uDC ... DC-voltage<br>
iAC ... AC-currents 1..3<br>
uAC ... AC-voltages 1..3 (distorted)<br>
Try different load currents iDC = 0..approximately 500 A.
</P>
<p>
You may watch Losses (of the whole diode bridge) trying different diode parameters.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Anton Haumer<br> realized<br>
       </li>
</ul>
</html>
"),         experiment(StopTime=0.1, Interval=1e-005),
            experimentSetupOutput(
              states=false,
              derivatives=false,
              inputs=false,
              auxiliaries=false));
          Modelica.Blocks.Sources.Constant Constant1(k=IDC) 
          annotation (extent=[98,-20; 78,0],     rotation=0);
          Basic.Ground Ground1 annotation (extent=[42,-88; 62,-68]);
          Basic.Capacitor Capacitor1(C=2*CDC, u0=vC10) 
            annotation (extent=[28,20; 48,40], rotation=-90);
          Basic.Capacitor Capacitor2(C=2*CDC, u0=vC20) 
            annotation (extent=[28,-60; 48,-40], rotation=-90);
          Sources.SignalCurrent SignalCurrent1 
            annotation (extent=[48,-20; 68,0], rotation=-90);
          Sources.SineVoltage SineVoltage1(V=VAC*sqrt(2/3), freqHz=f) 
            annotation (extent=[-92,20; -72,0], rotation=180);
          Sources.SineVoltage SineVoltage2(
            V=VAC*sqrt(2/3),
            freqHz=f,
            phase=-2/3*Modelica.Constants.pi) 
            annotation (extent=[-92,0; -72,-20], rotation=180);
          Sources.SineVoltage SineVoltage3(
            V=VAC*sqrt(2/3),
            freqHz=f,
            phase=-4/3*Modelica.Constants.pi) 
            annotation (extent=[-92,-20; -72,-40], rotation=180);
          Basic.Inductor Inductor1(L=LAC) annotation (extent=[-62,0; -42,20]);
          Basic.Inductor Inductor2(L=LAC) annotation (extent=[-62,-20; -42,0]);
          Basic.Inductor Inductor3(L=LAC) annotation (extent=[-62,-40; -42,-20]);
          Ideal.IdealDiode IdealDiode1(
            Ron=Ron,
            Goff=Goff,
            Vknee=Vknee) annotation (extent=[-32,20; -12,40], rotation=90);
          Ideal.IdealDiode IdealDiode2(
            Ron=Ron,
            Goff=Goff,
            Vknee=Vknee) annotation (extent=[-12,20; 8,40], rotation=90);
          Ideal.IdealDiode IdealDiode3(
            Ron=Ron,
            Goff=Goff,
            Vknee=Vknee) annotation (extent=[8,20; 28,40], rotation=90);
          Ideal.IdealDiode IdealDiode4(
            Ron=Ron,
            Goff=Goff,
            Vknee=Vknee) annotation (extent=[-32,-60; -12,-40], rotation=90);
          Ideal.IdealDiode IdealDiode5(
            Ron=Ron,
            Goff=Goff,
            Vknee=Vknee) annotation (extent=[-12,-60; 8,-40], rotation=90);
          Ideal.IdealDiode IdealDiode6(
            Ron=Ron,
            Goff=Goff,
            Vknee=Vknee) annotation (extent=[8,-60; 28,-40], rotation=90);
        equation 
          uDC = Capacitor1.v + Capacitor2.v;
          iAC = {Inductor1.i,Inductor2.i,Inductor3.i};
          uAC[1] = Inductor1.n.v - Inductor2.n.v;
          uAC[2] = Inductor2.n.v - Inductor3.n.v;
          uAC[3] = Inductor3.n.v - Inductor1.n.v;
          Losses = IdealDiode1.v*IdealDiode1.i + IdealDiode2.v*IdealDiode2.i +
            IdealDiode3.v*IdealDiode3.i + IdealDiode4.v*IdealDiode4.i +
            IdealDiode5.v*IdealDiode5.i + IdealDiode6.v*IdealDiode6.i;
          connect(Constant1.y, SignalCurrent1.InPort1) annotation (points=[77,
                -10; 64,-10], style(color=74, rgbcolor={0,0,127}));
          connect(SineVoltage1.n, SineVoltage2.n) annotation (points=[-92,10;
                -92,-10], style(color=3, rgbcolor={0,0,255}));
          connect(SineVoltage2.n, SineVoltage3.n) annotation (points=[-92,-10;
                -92,-30], style(color=3, rgbcolor={0,0,255}));
          connect(SineVoltage1.p, Inductor1.p) annotation (points=[-72,10; -62,
                10],                 style(color=3, rgbcolor={0,0,255}));
          connect(SineVoltage2.p, Inductor2.p) annotation (points=[-72,-10; -67,
                -10; -67,-10; -62,-10], style(color=3, rgbcolor={0,0,255}));
          connect(SineVoltage3.p, Inductor3.p) annotation (points=[-72,-30; -62,
                -30], style(color=3, rgbcolor={0,0,255}));
          connect(Inductor1.n, IdealDiode4.n) annotation (points=[-42,10; -22,
                10; -22,-40], style(color=3, rgbcolor={0,0,255}));
          connect(Inductor2.n, IdealDiode5.n) annotation (points=[-42,-10; -2,
                -10; -2,-40], style(color=3, rgbcolor={0,0,255}));
          connect(Inductor3.n, IdealDiode3.p) annotation (points=[-42,-30; 18,
                -30; 18,20], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode1.p, IdealDiode4.n) annotation (points=[-22,20; -22,
                -40], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode2.p, IdealDiode5.n) annotation (points=[-2,20; -2,5;
                -2,5; -2,-10; -2,-40; -2,-40], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode6.n, IdealDiode3.p) annotation (points=[18,-40; 18,
                20], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode1.n, IdealDiode2.n) annotation (points=[-22,40; -2,
                40], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode2.n, IdealDiode3.n) annotation (points=[-2,40; 18,
                40], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode3.n, Capacitor1.p) annotation (points=[18,40; 38,40],
              style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode4.p, IdealDiode5.p) annotation (points=[-22,-60; -2,
                -60], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode5.p, IdealDiode6.p) annotation (points=[-2,-60; 18,
                -60], style(color=3, rgbcolor={0,0,255}));
          connect(IdealDiode6.p, Capacitor2.n) annotation (points=[18,-60; 38,
                -60], style(color=3, rgbcolor={0,0,255}));
          connect(Capacitor1.n, Capacitor2.p) annotation (points=[38,20; 38,-40],
              style(color=3, rgbcolor={0,0,255}));
          connect(Ground1.p, Capacitor2.p) annotation (points=[52,-68; 52,-30;
                38,-30; 38,-40], style(color=3, rgbcolor={0,0,255}));
          connect(Capacitor1.p, SignalCurrent1.p) annotation (points=[38,40; 58,
                40; 58,0], style(color=3, rgbcolor={0,0,255}));
          connect(Capacitor2.n, SignalCurrent1.n) annotation (points=[38,-60;
                58,-60; 58,-20], style(color=3, rgbcolor={0,0,255}));
        end Rectifier;
        
        model ShowVariableResistor "Simple demo of a VariableResistor model" 
          annotation (preferedView="info",
            Icon(
              Rectangle(extent=[-100,-100; 80,50],   style(fillColor=7)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                    fillColor=7)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                    fillColor=7)),
              Text(
                extent=[-96,3; 77,-49],
                string="Example",
                style(color=3)),
              Text(
                extent=[-120,132; 120,73],
                string="%name",
                style(color=1))),
            Diagram(Text(
                extent=[-90,106; 90,34],
                style(color=3, rgbcolor={0,0,255}),
                string="Example VariableResistor")),
            Documentation(info="<html>
<P>
It is a simple test circuit for the VariableResistor. The VariableResistor
sould be compared with R2.
</P>
<P>
Simulate until T=1 s.
</P>
<P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Teresa Schlegel<br> realized<br>
       </li>
</ul>
</html>
"),         experiment,
            experimentSetupOutput);
          Modelica.Blocks.Sources.Ramp Ramp1(height=5, offset=2) 
          annotation(extent=[-10,4; 10,24], rotation=270);
          Basic.Ground Ground1 annotation (extent=[-90,-84; -70,-64]);
          Basic.Ground Ground2 annotation (extent=[70,-50; 90,-30]);
          Basic.Resistor R1 annotation (extent=[-60,34; -40,54]);
          Basic.Resistor R2 annotation (extent=[-10,34; 10,54]);
          Basic.Resistor R3 annotation (extent=[30,34; 50,54]);
          Basic.Resistor R4 annotation (extent=[-60,-26; -40,-6]);
          Basic.Resistor R5 annotation (extent=[30,-26; 50,-6]);
          Basic.VariableResistor VariableResistor1 
            annotation (extent=[-10,-26; 10,-6]);
          Sources.SineVoltage SineVoltage1 
            annotation (extent=[-90,-50; -70,-30], rotation=90);
        equation 
          connect(Ramp1.y, VariableResistor1.u) annotation (points=[
                -2.0206e-015,3; -2.0206e-015,-1.5; 0,-1.5; 0,-7], style(color=
                  74, rgbcolor={0,0,127}));
          connect(SineVoltage1.n, R4.p) annotation (points=[-80,-30; -80,-16;
                -60,-16], style(color=3, rgbcolor={0,0,255}));
          connect(R1.p, R4.p) annotation (points=[-60,44; -80,44; -80,-16; -60,
                -16], style(color=3, rgbcolor={0,0,255}));
          connect(R1.n, R2.p) annotation (points=[-40,44; -10,44], style(color=
                  3, rgbcolor={0,0,255}));
          connect(R4.n, VariableResistor1.p) annotation (points=[-40,-16; -10,
                -16], style(color=3, rgbcolor={0,0,255}));
          connect(R2.n, R3.p) annotation (points=[10,44; 30,44], style(color=3,
                rgbcolor={0,0,255}));
          connect(VariableResistor1.n, R5.p) annotation (points=[10,-16; 30,-16],
              style(color=3, rgbcolor={0,0,255}));
          connect(R5.n, R3.n) annotation (points=[50,-16; 80,-16; 80,44; 50,44],
              style(color=3, rgbcolor={0,0,255}));
          connect(Ground2.p, R3.n) annotation (points=[80,-30; 80,44; 50,44],
              style(color=3, rgbcolor={0,0,255}));
          connect(Ground1.p, SineVoltage1.p) annotation (points=[-80,-64; -80,
                -50], style(color=3, rgbcolor={0,0,255}));
        end ShowVariableResistor;
        
        package Utilities 
          "Some utility models used in the examples of the analog electrical library" 
          annotation (preferedView="info",Icon(
              Rectangle(extent=[-100,-100; 80,50], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                  color=42,
                  rgbcolor={127,0,0},
                  fillColor=30,
                  rgbfillColor={235,235,235},
                  fillPattern=1)),
              Text(
                extent=[-85,35; 65,-85],
                string="Library",
                style(color=42, rgbcolor={127,0,0})),
              Text(
                extent=[-120,122; 120,73],
                string="%name",
                style(color=1))), Documentation(info="<html>
This package contains some utility models used by some of the examples of the analog electrical library.
</html>
"));
          model Transistor "NPN transistor based amplifier" 
            parameter Boolean enforceStates = true 
              "|Advanced||Use electrical variables as states instead of bond graph variables";
            
            annotation (
              Documentation(info="<html>
This model presents a simple amplifier built from a single NPN bipolar transistor.
</html>
"),           Icon(
                Rectangle(extent=[-80,80; 80,-80]),
                Line(points=[80,60; 100,60]),
                Line(points=[80,-60; 100,-60]),
                Line(points=[-100,0; -80,0]),
                Line(points=[-60,0; -10,0]),
                Line(points=[-10,40; -10,-40]),
                Line(points=[60,60; 40,60; -10,10]),
                Line(points=[-10,-10; 40,-60; 60,-60])),
              Diagram);
            Modelica.Electrical.Analog.Interfaces.Pin c "Collector" 
                             annotation (extent=[90,50; 110,70]);
            Modelica.Electrical.Analog.Interfaces.Pin b "Base" 
                             annotation (extent=[-110,-10; -90,10]);
            Modelica.Electrical.Analog.Interfaces.Pin e "Emitter" 
                             annotation (extent=[90,-70; 110,-50]);
            Basic.Resistor Rtb(R=0.05) annotation (extent=[-70,-10; -50,10]);
            Basic.Capacitor Ct(C=1e-10, enforceStates=enforceStates) 
              annotation (extent=[-40,-40; -20,-20], rotation=-90);
            Semiconductors.NPN Tr(
              Bf=50,
              Br=0.1,
              Is=1e-16,
              Vak=0.02,
              Tauf=0.12e-9,
              Taur=5e-9,
              Ccs=1e-12,
              Cje=0.4e-12,
              Cjc=0.5e-12,
              Phie=0.8,
              Me=0.4,
              Phic=0.8,
              Mc=0.333,
              Gbc=1e-15,
              Gbe=1e-15,
              Vt=0.02585,
              enforceStates=enforceStates) 
                         annotation (extent=[-10,-20; 30,20]);
            Basic.Resistor Rtc(R=0.1) annotation (extent=[50,0; 70,20]);
            Basic.Ground Gnd1    annotation (extent=[-40,-80; -20,-60]);
          equation 
            connect(Tr.C, Rtc.p) annotation (points=[30,10; 50,10], style(color=
                   3, rgbcolor={0,0,255}));
            connect(Rtc.n, c) annotation (points=[70,10; 80,10; 80,60; 100,60],
                style(color=3, rgbcolor={0,0,255}));
            connect(Tr.E, e) annotation (points=[30,-10; 80,-10; 80,-60; 100,
                  -60], style(color=3, rgbcolor={0,0,255}));
            connect(Ct.n, Gnd1.p)    annotation (points=[-30,-40; -30,-60],
                style(color=3, rgbcolor={0,0,255}));
            connect(Rtb.n, Tr.B) annotation (points=[-50,0; -10,0], style(color=
                   3, rgbcolor={0,0,255}));
            connect(Ct.p, Tr.B) annotation (points=[-30,-20; -30,0; -10,0],
                style(color=3, rgbcolor={0,0,255}));
            connect(Rtb.p, b) annotation (points=[-70,0; -100,0], style(color=3,
                  rgbcolor={0,0,255}));
          end Transistor;
        end Utilities;
      end Examples;
    end Analog;
  end Electrical;
  
  package Mechanical "1D Mechanical Systems built from bond graphs" 
    annotation (
      preferedView="info",
      Icon(
        Rectangle(extent=[-100,-100; 80,50],   style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Text(
          extent=[-85,35; 65,-85],
          string="Library",
          style(color=42)),
        Text(
          extent=[-120,122; 120,73],
          string="%name",
          style(color=1))),
      Documentation(info="<html>
This library reimplements in parts the 1D mechanical library of the Modelica standard library.  All models are internally mapped onto a bond graph representation in order to minimize the need for equation modeling. <p>

Due to the underlying bond graph technology, the connectors of this library are incompatible with the connectors of the corresponding mechanical sub-library of the Modelica standard library.  The bond graph models require velocity information to be transferred across the connectors beside from positional information.  Consequently, the connectors of this library contain one additional variable. <p>

For this reason, models from this library cannot be mixed with the models of the corresponding sub-library of the Modelica standard library.
</html>"));
    package Translational "Translational 1D systems built from bond graphs" 
      annotation (
        preferedView="info",
        Icon(
          Text(
            extent=[-120,118; 120,69],
            string="%name",
            style(color=1)),
          Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                 1)),
          Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                fillColor=30, fillPattern=1)),
          Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                fillColor=30, fillPattern=1)),
          Text(
            extent=[-85,35; 65,-85],
            string="Library",
            style(color=3))),
        Documentation(info="<html>
This library reimplements parts of the 1D translational library of the Modelica standard library.  All models are internally mapped onto a bond graph representation in order to minimize the need for equation modeling. <p>

Due to the underlying bond graph technology, the connectors of this library are incompatible with the connectors of the corresponding mechanical sub-library of the Modelica standard library.  The bond graph models require velocity information to be transferred across the connectors beside from positional information.  Consequently, the connectors of this library contain one additional variable. <p>

For this reason, models from this library cannot be mixed with the models of the corresponding sub-library of the Modelica standard library.
</html>"));
      package Interfaces "Wrapping models and partial models" 
        annotation (
          preferedView="info",
          Icon(
            Text(
              extent=[-118,112; 122,63],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],   style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=58))),
          Documentation(info="<html>
Sublibrary of interface models: connectors, wrapper models, and partial models that are being used by the 1D translational library.
</html>"));
        connector Flange_a "Translational bond graph connector" 
          
          annotation (
            Icon(Rectangle(extent=[-100,-100; 100,100],   style(
                  color=58,
                  rgbcolor={0,127,0},
                  fillColor=58,
                  rgbfillColor={0,127,0}))),
            Diagram(Rectangle(extent=[-40,-40; 40,40],   style(
                  color=58,
                  rgbcolor={0,127,0},
                  fillColor=58,
                  rgbfillColor={0,127,0})),
                                    Text(
                extent=[-160,110; 40,50],
                string="%name",
                style(color=58, rgbcolor={0,127,0}))),
            Documentation(info="<html>
\"Left\" translational connector. <p>

This connector is incompatible with the corresponding connector of the standard library because of the underlying bond graph technology that requires velocity information beside from positional information to be transferred.
</html>"));
          Modelica.SIunits.Position s "Absolute position of flange";
          Modelica.SIunits.Velocity v "Absolute velocity of flange";
          flow Modelica.SIunits.Force f "Cut force directed into flange";
        end Flange_a;
        
        connector Flange_b "Translational bond graph connector" 
          
          annotation (
            Icon(Rectangle(extent=[-100,-100; 100,100],   style(
                  color=58,
                  rgbcolor={0,127,0},
                  fillColor=7,
                  rgbfillColor={255,255,255}))),
            Diagram(Rectangle(extent=[-40,-40; 40,40],   style(
                  color=58,
                  rgbcolor={0,127,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                                   Text(
                extent=[-40,110; 160,50],
                string="%name",
                style(color=58, rgbcolor={0,127,0}))),
            Documentation(info="<html>
\"Right\" translational connector. <p>

This connector is incompatible with the corresponding connector of the standard library because of the underlying bond graph technology that requires velocity information beside from positional information to be transferred.
</html>"));
          Modelica.SIunits.Position s "Absolute position of flange";
          Modelica.SIunits.Velocity v "Absolute velocity of flange";
          flow Modelica.SIunits.Force f "Cut force directed into flange";
        end Flange_b;
        
        connector RealSignal "Bi-directional real signal" 
          extends Modelica.Blocks.Interfaces.RealSignal;
          annotation (Icon(Polygon(points=[0,80; -80,0; 0,-80; 80,0; 0,80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)), Text(
                extent=[-80,28; 80,-26],
                style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1),
                string="%name")), Diagram(
                           Polygon(points=[0,80; -80,0; 0,-80; 80,0; 0,80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1))),
            Documentation(info="<html>
Bi-directional real signal used to connect activated bonds of which the direction of information flow is not pre-determined.
</html>"));
        end RealSignal;
        
        model Tr2BG "Translational to bond graph conversion" 
          
          Flange_a Tr "Translational connector" 
            annotation (extent=[-111,-11; -91,9]);
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
            annotation (extent=[90,-10; 110,10]);
          RealSignal s(redeclare type SignalType = 
                Modelica.SIunits.Position) "Positional signal" 
            annotation (extent=[-10,-110; 10,-90]);
          annotation (
            Diagram(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80], style(color=58,
                    rgbcolor={0,220,0})),
              Line(points=[80,80; 80,-80; -80,-80],    style(color=9)),
              Line(points=[94,0; 80,0],   style(color=9)),
              Line(points=[-96,0; -80,0], style(color=58, rgbcolor={0,220,0})),
              Text(
                extent=[-64,56; 0,12],
                string="Tr",
                style(
                  color=58,
                  rgbcolor={0,220,0},
                  fillColor=58,
                  rgbfillColor={0,127,0})),
              Text(
                extent=[4,-14; 70,-64],
                string="BG",
                style(color=9)),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Icon(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80], style(color=58,
                    rgbcolor={0,220,0})),
              Line(points=[80,80; 80,-80; -80,-80],    style(color=9)),
              Line(points=[94,0; 80,0],   style(color=9)),
              Line(points=[-96,0; -80,0], style(color=58, rgbcolor={0,220,0})),
              Text(
                extent=[-64,56; 0,12],
                string="Tr",
                style(
                  color=58,
                  rgbcolor={0,220,0},
                  fillColor=58,
                  rgbfillColor={0,127,0})),
              Text(
                extent=[4,-14; 70,-64],
                string="BG",
                style(color=9)),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Documentation(info="<html>
Wrapper model translating mechanical (external) connectors to bond graph (internal) connectors.
</html>"));
        equation 
          BondCon1.e = Tr.f;
          BondCon1.f = Tr.v;
          s = Tr.s;
        end Tr2BG;
        
        model BG2Tr "Bond graph to translational conversion" 
          Flange_b Tr "Translational connector" 
            annotation (extent=[89,-11; 109,9]);
          RealSignal s(redeclare type SignalType = 
                Modelica.SIunits.Position) "Positional signal" 
            annotation (extent=[-10,-110; 10,-90]);
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
            annotation (extent=[-110,-10; -90,10]);
          annotation (
            Diagram(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80],     style(color=9)),
              Line(points=[-94,0; -80,0],   style(color=9)),
              Line(points=[80,80; 80,-80; -80,-80], style(color=2, rgbcolor={0,
                      255,0})),
              Line(points=[96,0; 80,0], style(color=2, rgbcolor={0,255,0})),
              Text(
                extent=[-64,70; 2,20],
                string="BG",
                style(color=9)),
              Text(
                extent=[0,-16; 64,-60],
                string="Tr",
                style(color=2, rgbcolor={0,255,0})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Icon(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80],     style(color=9)),
              Line(points=[-94,0; -80,0],   style(color=9)),
              Line(points=[80,80; 80,-80; -80,-80], style(color=2, rgbcolor={0,
                      255,0})),
              Line(points=[96,0; 80,0], style(color=2, rgbcolor={0,255,0})),
              Text(
                extent=[-64,70; 2,20],
                string="BG",
                style(color=9)),
              Text(
                extent=[0,-16; 64,-60],
                string="Tr",
                style(color=2, rgbcolor={0,255,0})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Documentation(info="<html>
Wrapper model translating bond graph (internal) connectors to mechanical (external) connectors.
</html>"));
        equation 
          Tr.f = -BondCon1.e;
          Tr.v = BondCon1.f;
          Tr.s = s;
        end BG2Tr;
        
        partial model OnePort "OnePort partial model of translational library" 
          Flange_a flange_a "Left translational connector" 
            annotation (extent=[-110,-10; -90,10]);
          Flange_b flange_b "Right translational connector" 
            annotation (extent=[89,-10; 109,10]);
          annotation (Documentation(info="<html>
Partial \"OnePort\" model of the translational library based on bond graph technology.
</html>"), Diagram);
        end OnePort;
      end Interfaces;
      
      package Passive "Passive translational elements" 
        annotation (
          preferedView="info",
          Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))),
          Documentation(info="<html>
Passive elements of the 1D translational library.
</html>"));
        model Fixed "Fixed flange" 
          parameter Modelica.SIunits.Position s0=0 
            "Fixed offset position of housing";
          annotation (
            Icon(
              Line(points=[-80,-40; 80,-40],   style(color=0)),
              Line(points=[80,-40; 40,-80],   style(color=0)),
              Line(points=[40,-40; 0,-80],   style(color=0)),
              Line(points=[0,-40; -40,-80],   style(color=0)),
              Line(points=[-40,-40; -80,-80],   style(color=0)),
              Line(points=[0,-40; 0,-10],   style(color=0)),
              Text(extent=[0,-90; 0,-150],   string="%name")),
            Diagram(
              Line(points=[-80,-40; 80,-40],   style(color=0)),
              Line(points=[80,-40; 40,-80],   style(color=0)),
              Line(points=[40,-40; 0,-80],   style(color=0)),
              Line(points=[0,-40; -40,-80],   style(color=0)),
              Line(points=[-40,-40; -80,-80],   style(color=0)),
              Line(points=[0,-40; 0,-4],    style(color=0)),
              Polygon(points=[50,-90; 20,-80; 20,-100; 50,-90],     style(color=10,
                    fillColor=10)),
              Line(points=[-60,-90; 20,-90],   style(color=0, fillColor=10))),
            Documentation(info="<html>
Translational inertial (ground) node.
</html>"));
          Interfaces.Flange_b flange_b "Translational connector" 
                                       annotation (extent=[-10,-10; 10,10]);
        equation 
          flange_b.s = s0;
          flange_b.v = 0;
        end Fixed;
        
        model Damper "Linear 1D translational damper" 
          extends Interfaces.OnePort;
          parameter Real d(
            final unit="N/ (m/s)",
            final min=0) = 0 "Damping constant [N/ (m/s)]";
          Modelica.SIunits.Distance s_rel "Relative width of damper";
          Modelica.SIunits.Velocity v_rel "Relative velocity of damper";
          Modelica.SIunits.Force f "Damping force";
        protected 
          Interfaces.Tr2BG Tr2BG 
            annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Tr BG2Tr 
            annotation (extent=[50,-10; 70,10]);
          Bonds.eBond B1  annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2  annotation (extent=[10,-10; 30,10]);
          Bonds.eBond B3 
            annotation (extent=[-10,-30; 10,-10],
                                                rotation=270);
          BondLib.Passive.R R(R=d) 
            annotation (extent=[-10,-50; 10,-30],rotation=270);
          BondLib.Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
        equation 
          connect(B1.eBondCon1,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0],      style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(Tr2BG.BondCon1,B1. fBondCon1) annotation (points=[-50,0; -30,
                0],                 style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B2. eBondCon1) annotation (points=[10,0; 10,0],
                     style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(B2.fBondCon1,BG2Tr. BondCon1) annotation (points=[30,0; 50,0],
                                 style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(j0p3_1.BondCon3,B3. fBondCon1) annotation (points=[0,-10;
                1.83691e-015,-10],style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(R.BondCon1,B3. eBondCon1) annotation (points=[1.83691e-015,
                -30; -1.83691e-015,-30], style(color=8, rgbcolor={192,192,192}));
          annotation (
            Diagram,
            Icon(
              Line(points=[-90,0; -60,0],   style(color=0)),
              Line(points=[-60,-30; -60,30],   style(color=0)),
              Line(points=[-60,-30; 60,-30],   style(color=0)),
              Line(points=[-60,30; 60,30],   style(color=0)),
              Rectangle(extent=[-60,30; 30,-30],   style(color=0, fillColor=8)),
              Line(points=[30,0; 90,0],   style(color=0)),
              Polygon(points=[50,-90; 20,-80; 20,-100; 50,-90],     style(color=10,
                    fillColor=10)),
              Line(points=[-60,-90; 20,-90],   style(color=0, fillColor=10)),
              Text(extent=[0,106; 0,46],   string="%name")),
            Documentation(info="<html>
Wrapped 1D linear translational damper model.
</html>"));
          s_rel = BG2Tr.s - Tr2BG.s;
          v_rel = -R.f;
          f = -R.e;
          connect(Tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -85.05,
                -0.1; -85.05,0; -100,0], style(color=58, rgbcolor={0,127,0}));
          connect(BG2Tr.Tr, flange_b) annotation (points=[69.9,-0.1; 82.95,-0.1;
                82.95,0; 99,0], style(color=58, rgbcolor={0,127,0}));
        end Damper;
        
        model Spring "Linear 1D translational spring" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Distance s_rel0=0 
            "Unstretched spring length";
          parameter Real c(
            final unit="N/m",
            final min=0) = 1 "Spring constant ";
          Modelica.SIunits.Distance s_rel "Relative width of spring";
          Modelica.SIunits.Velocity v_rel "Relative velocity of spring";
          Modelica.SIunits.Force f "Spring force";
        protected 
          Interfaces.Tr2BG Tr2BG 
            annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Tr BG2Tr 
            annotation (extent=[50,-10; 70,10]);
          Bonds.eBond B1  annotation (extent=[-30,30; -10,50]);
          Bonds.fBond B2  annotation (extent=[10,30; 30,50]);
          Junctions.J0p3 j0p3_1  annotation (extent=[-10,30; 10,50]);
          BondLib.Sources.mSe Fx 
            annotation (extent=[-10,-10; 10,10], rotation=-90);
          Bonds.fBond B3  annotation (extent=[-10,10; 10,30], rotation=
                90);
          Modelica.Blocks.Math.Gain G1(k=c) 
            annotation (extent=[-10,-40; 10,-20], rotation=90);
          Modelica.Blocks.Math.Add3 A1(k3=-1) 
            annotation (extent=[-30,-70; -10,-50]);
          Modelica.Blocks.Sources.Constant C1(k=s_rel0) 
            annotation (extent=[-80,-70; -60,-50]);
        equation 
          connect(j0p3_1.BondCon1,B1. eBondCon1) annotation (points=[-10,40;
                -10,40],
                    style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_1.BondCon2,B2. eBondCon1) annotation (points=[10,40; 10,
                40],
              style(color=8, rgbcolor={192,192,192}));
          connect(Tr2BG.BondCon1,B1. fBondCon1) annotation (points=[-50,0; -40,
                0; -40,40; -30,40], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Tr.BondCon1,B2. fBondCon1) annotation (points=[50,0; 40,0;
                40,40; 30,40], style(color=8, rgbcolor={192,192,192}));
          connect(Fx.BondCon1,B3. eBondCon1) annotation (points=[-6.12303e-016,
                10; -6.12303e-016,10], style(color=8, rgbcolor={192,192,192}));
          connect(B3.fBondCon1,j0p3_1. BondCon3) annotation (points=[
                6.12303e-016,30; 0,30], style(color=8, rgbcolor={192,192,192}));
          connect(A1.y,G1. u) annotation (points=[-9,-60; -7.34764e-016,-60;
                -7.34764e-016,-42], style(color=74, rgbcolor={0,0,127}));
          connect(Tr2BG.s,A1. u1) annotation (points=[-60,-10; -60,-30; -44,-30;
                -44,-52; -32,-52], style(color=3, rgbcolor={0,0,255}));
          connect(C1.y,A1. u2) annotation (points=[-59,-60; -32,-60], style(
                color=74, rgbcolor={0,0,127}));
          connect(BG2Tr.s,A1. u3) annotation (points=[60,-10; 60,-80; -44,-80;
                -44,-68; -32,-68], style(color=3, rgbcolor={0,0,255}));
          annotation (
            Diagram,
            Icon(
              Line(points=[-60,-90; 20,-90],   style(color=0, fillColor=10)),
              Polygon(points=[50,-90; 20,-80; 20,-100; 50,-90],     style(color=10,
                    fillColor=10)),
              Text(extent=[0,110; 0,50],   string="%name"),
              Line(points=[-90,0; -60,0; -44,-30; -16,30; 14,-30; 44,30; 60,0;
                    90,0],    style(color=0))),
            Documentation(info="<html>
Wrapped 1D linear translational spring model.
</html>"));
          s_rel = BG2Tr.s - Tr2BG.s;
          v_rel = Fx.f;
          f = -G1.y;
          connect(Tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -85.05,
                -0.1; -85.05,0; -100,0], style(color=58, rgbcolor={0,127,0}));
          connect(BG2Tr.Tr, flange_b) annotation (points=[69.9,-0.1; 82.95,-0.1;
                82.95,0; 99,0], style(color=58, rgbcolor={0,127,0}));
          connect(G1.y, Fx.s) annotation (points=[6.73533e-016,-19;
                6.73533e-016,-14.5; 5.51073e-016,-14.5; 5.51073e-016,-9], style(
                color=74, rgbcolor={0,0,127}));
        end Spring;
        
        model SlidingMass "Sliding mass with inertia" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Mass m=1 "Mass of the sliding mass";
          parameter Modelica.SIunits.Length L=0 
            "Length of component from left flange to right flange (= flange_b.s - flange_a.s)";
          parameter Modelica.SIunits.Length s0=0 
            "Initial position of mass center";
          Modelica.SIunits.Length s(stateSelect=StateSelect.prefer) 
            "Absolute position of mass";
          Modelica.SIunits.Velocity v(stateSelect=StateSelect.prefer) 
            "Absolute velocity of mass";
          Modelica.SIunits.Acceleration a "Absolute acceleration of mass";
        protected 
          Interfaces.Tr2BG Tr2BG 
            annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Tr BG2Tr 
            annotation (extent=[50,-10; 70,10]);
          Bonds.eBond B2  annotation (extent=[10,30; 30,50]);
          Bonds.fBond B1  annotation (extent=[-30,30; -10,50]);
          Bonds.eBond B4 
            annotation (extent=[-10,10; 10,30], rotation=270);
          Bonds.fBond B3 
            annotation (extent=[-10,50; 10,70], rotation=90);
          Junctions.J1p4 j1p4_1  annotation (extent=[-10,30; 10,50]);
          BondLib.Passive.I I(I=m) 
            annotation (extent=[-10,70; 10,90], rotation=90);
          Modelica.Blocks.Math.Add A1(k2=-1) 
            annotation (extent=[-20,-40; -40,-20]);
          Modelica.Blocks.Math.Add A2 annotation (extent=[20,-40; 40,-20]);
          Modelica.Blocks.Sources.Constant C1(k=L/2) 
            annotation (extent=[-40,-80; -20,-60]);
          BondLib.Sensors.Dq sAbs(q0=s0) 
            annotation (extent=[-10,-10; 10,10], rotation=-90);
        equation 
          connect(B1.fBondCon1,j1p4_1. BondCon1) annotation (points=[-10,40;
                -10,40], style(color=8, rgbcolor={192,192,192}));
          connect(j1p4_1.BondCon2,B2. fBondCon1) annotation (points=[10,40; 10,
                40], style(color=8, rgbcolor={192,192,192}));
          connect(Tr2BG.BondCon1,B1. eBondCon1) annotation (points=[-50,0; -40,
                0; -40,40; -30,40], style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1,BG2Tr. BondCon1) annotation (points=[30,40; 40,
                40; 40,0; 50,0], style(color=8, rgbcolor={192,192,192}));
          connect(j1p4_1.BondCon4,B3. eBondCon1) annotation (points=[0,50;
                -6.12303e-016,50], style(color=8, rgbcolor={192,192,192}));
          connect(I.BondCon1,B3. fBondCon1) annotation (points=[-6.12303e-016,
                70; 6.12303e-016,70], style(color=8, rgbcolor={192,192,192}));
          connect(A1.y,Tr2BG. s) annotation (points=[-41,-30; -60,-30; -60,-10],
              style(color=74, rgbcolor={0,0,127}));
          connect(A2.y,BG2Tr. s) annotation (points=[41,-30; 60,-30; 60,-10],
              style(color=74, rgbcolor={0,0,127}));
          connect(C1.y,A1. u2) annotation (points=[-19,-70; 0,-70; 0,-36; -18,
                -36], style(color=74, rgbcolor={0,0,127}));
          connect(C1.y,A2. u2) annotation (points=[-19,-70; 0,-70; 0,-36; 18,
                -36], style(color=74, rgbcolor={0,0,127}));
          connect(B4.fBondCon1,j1p4_1. BondCon3) annotation (points=[
                1.83691e-015,30; 0,30], style(color=8, rgbcolor={192,192,192}));
          connect(sAbs.BondCon1,B4. eBondCon1) annotation (points=[
                -6.12303e-016,10; -1.83691e-015,10], style(color=8, rgbcolor={
                  192,192,192}));
          connect(sAbs.OutPort1,A1. u1) annotation (points=[6.73533e-016,-11;
                6.73533e-016,-24; -18,-24], style(color=74, rgbcolor={0,0,127}));
          connect(sAbs.OutPort1,A2. u1) annotation (points=[6.73533e-016,-11;
                6.73533e-016,-24; 18,-24], style(color=74, rgbcolor={0,0,127}));
          annotation (
            Diagram,
            Icon(
              Line(points=[-100,0; -55,0],   style(color=58, rgbcolor={0,127,0})),
              Line(points=[55,0; 100,0],   style(color=58, rgbcolor={0,127,0})),
              Rectangle(extent=[-55,-30; 56,30],   style(
                  color=0,
                  gradient=3,
                  fillColor=7,
                  fillPattern=1)),
              Polygon(points=[50,-90; 20,-80; 20,-100; 50,-90],     style(color=10,
                    fillColor=10)),
              Line(points=[-60,-90; 20,-90],   style(color=0, fillColor=10)),
              Text(extent=[0,100; 0,40],   string="%name")),
            Documentation(info="<html>
Wrapped 1D linear translational sliding mass model.
</html>"));
          s = sAbs.OutPort1;
          v = I.f;
          a = I.e/m;
          connect(Tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -83.05,
                -0.1; -83.05,0; -100,0], style(color=58, rgbcolor={0,127,0}));
          connect(BG2Tr.Tr, flange_b) annotation (points=[69.9,-0.1; 82.95,-0.1;
                82.95,0; 99,0], style(color=58, rgbcolor={0,127,0}));
        end SlidingMass;
        
        model Rod "Rod without inertia" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Length L=0 
            "Length of component from left flange to right flange (= flange_b.s - flange_a.s)";
          parameter Modelica.SIunits.Length s0=0 
            "Initial position of rod center";
          Modelica.SIunits.Length s "Absolute positon of mass";
        protected 
          Interfaces.Tr2BG Tr2BG 
            annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Tr BG2Tr 
            annotation (extent=[50,-10; 70,10]);
          Bonds.eBond B2  annotation (extent=[10,30; 30,50]);
          Bonds.fBond B1  annotation (extent=[-30,30; -10,50]);
          Bonds.eBond B4 
            annotation (extent=[-10,10; 10,30], rotation=270);
          Modelica.Blocks.Math.Add A1(k2=-1) 
            annotation (extent=[-20,-40; -40,-20]);
          Modelica.Blocks.Math.Add A2 annotation (extent=[20,-40; 40,-20]);
          Modelica.Blocks.Sources.Constant C1(k=L/2) 
            annotation (extent=[-40,-80; -20,-60]);
          BondLib.Sensors.Dq sAbs(q0=s0) 
            annotation (extent=[-10,-10; 10,10], rotation=-90);
          BondLib.Junctions.J1p3 j1p3_1 annotation (extent=[-10,30; 10,50]);
        equation 
          connect(Tr2BG.BondCon1,B1. eBondCon1) annotation (points=[-50,0; -40,
                0; -40,40; -30,40], style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1,BG2Tr. BondCon1) annotation (points=[30,40; 40,
                40; 40,0; 50,0], style(color=8, rgbcolor={192,192,192}));
          connect(A1.y,Tr2BG. s) annotation (points=[-41,-30; -60,-30; -60,-10],
              style(color=74, rgbcolor={0,0,127}));
          connect(A2.y,BG2Tr. s) annotation (points=[41,-30; 60,-30; 60,-10],
              style(color=74, rgbcolor={0,0,127}));
          connect(C1.y,A1. u2) annotation (points=[-19,-70; 0,-70; 0,-36; -18,
                -36], style(color=74, rgbcolor={0,0,127}));
          connect(C1.y,A2. u2) annotation (points=[-19,-70; 0,-70; 0,-36; 18,
                -36], style(color=74, rgbcolor={0,0,127}));
          connect(sAbs.BondCon1,B4. eBondCon1) annotation (points=[
                -6.12303e-016,10; -1.83691e-015,10], style(color=8, rgbcolor={
                  192,192,192}));
          connect(sAbs.OutPort1,A1. u1) annotation (points=[6.73533e-016,-11;
                6.73533e-016,-24; -18,-24], style(color=74, rgbcolor={0,0,127}));
          connect(sAbs.OutPort1,A2. u1) annotation (points=[6.73533e-016,-11;
                6.73533e-016,-24; 18,-24], style(color=74, rgbcolor={0,0,127}));
          annotation (Diagram, Icon(
              Line(points=[-100,0; -55,0],   style(color=58, rgbcolor={0,127,0})),
              Line(points=[53,0; 99,0], style(color=58, rgbcolor={0,127,0})),
              Polygon(points=[50,-90; 20,-80; 20,-100; 50,-90],     style(color=10,
                    fillColor=10)),
              Line(points=[-60,-90; 20,-90],   style(color=0, fillColor=10)),
              Rectangle(extent=[-55,10; 53,-10],   style(
                  color=9,
                  fillColor=8,
                  fillPattern=1)),
              Text(extent=[0,100; 0,40],   string="%name")));
          s = sAbs.OutPort1;
          connect(Tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -83.05,
                -0.1; -83.05,0; -100,0], style(
              color=58,
              rgbcolor={0,127,0},
              fillPattern=1));
          connect(BG2Tr.Tr, flange_b) annotation (points=[69.9,-0.1; 81.95,-0.1;
                81.95,0; 99,0], style(
              color=58,
              rgbcolor={0,127,0},
              fillPattern=1));
          connect(B1.fBondCon1, j1p3_1.BondCon1) annotation (points=[-10,40;
                -10,40], style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(B2.fBondCon1, j1p3_1.BondCon2) annotation (points=[10,40; 10,
                40], style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(B4.fBondCon1, j1p3_1.BondCon3) annotation (points=[
                1.83691e-015,30; 0,30], style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
        end Rod;
        
        model SpringDamper 
          "Linear 1D translational spring and damper in parallel" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Position s_rel0=0 
            "unstretched spring length";
          parameter Real c(
            final unit="N/m",
            final min=0) = 1 "spring constant";
          parameter Real d(
            final unit="N/(m/s)",
            final min=0) = 1 "damping constant";
          Modelica.SIunits.Velocity v_rel 
            "relative velocity between flange_a and flange_b";
          Spring spring(s_rel0=s_rel0, c=c) annotation (extent=[-20,20; 20,60]);
          Damper damper(d=d) annotation (extent=[-20,-60; 20,-20]);
          annotation (
            Diagram,
            Icon(
              Line(points=[-80,40; -60,40; -45,10; -15,70; 15,10; 45,70; 60,40;
                    80,40],    style(color=0)),
              Line(points=[-80,40; -80,-70],   style(color=0)),
              Line(points=[-80,-70; -52,-70],   style(color=0)),
              Rectangle(extent=[-52,-49; 38,-91],   style(color=0, fillColor=8)),
              Line(points=[-52,-49; 68,-49],   style(color=0)),
              Line(points=[-51,-91; 69,-91],   style(color=0)),
              Line(points=[38,-70; 80,-70],   style(color=0)),
              Line(points=[80,40; 80,-70],   style(color=0)),
              Line(points=[-90,0; -80,0],   style(color=0)),
              Line(points=[80,0; 90,0],   style(color=0)),
              Polygon(points=[53,-18; 23,-8; 23,-28; 53,-18],     style(color=10,
                    fillColor=10)),
              Line(points=[-57,-18; 23,-18],   style(color=0, fillColor=10)),
              Text(extent=[1,140; 1,80],   string="%name")),
            Documentation(info="<html>
A spring and a damper connected in parallel.
Since this is such a frequently used combination, a separate model was created for it.
</html>"));
        equation 
          v_rel = flange_b.v - flange_a.v;
          connect(spring.flange_a, flange_a) annotation (points=[-20,40; -60,40;
                -60,0; -100,0], style(color=58, rgbcolor={0,127,0}));
          connect(damper.flange_a, flange_a) annotation (points=[-20,-40; -60,
                -40; -60,0; -100,0], style(color=58, rgbcolor={0,127,0}));
          connect(spring.flange_b, flange_b) annotation (points=[19.8,40; 60,40;
                60,0; 99,0], style(color=58, rgbcolor={0,127,0}));
          connect(damper.flange_b, flange_b) annotation (points=[19.8,-40; 60,
                -40; 60,0; 99,0], style(color=58, rgbcolor={0,127,0}));
        end SpringDamper;
      end Passive;
      
      package Sensors "Sensors of the 1D translational library" 
        annotation (
          preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30, fillPattern=
                    1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3)),
            Text(
              extent=[-120,122; 120,73],
              string="%name",
              style(color=1))), Documentation(info="<html>
Sensors of the 1D translational library.
</html>"));
        model ForceSensor 
          "Ideal sensor to measure the force between two flanges" 
          extends Interfaces.OnePort;
          annotation (
            Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Line(points=[70,0; 90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[0,-60; 0,-90], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
                 Text(
                extent=[38,-68; 118,-118],
                string="f",
                style(color=0)),
              Text(extent=[0,100; 0,40],   string="%name")),
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the force between two flanges.
This element should be placed in series with another element to measure the force within a connection.
</html>"));
          
        protected 
          Interfaces.Tr2BG tr2BG annotation (extent=[-70,10; -50,-10]);
          Interfaces.BG2Tr bG2Tr annotation (extent=[50,10; 70,-10]);
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,-30; 10,-10], rotation=-90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.De F 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
        public 
          Modelica.Blocks.Interfaces.RealOutput f(redeclare type SignalType = 
                Modelica.SIunits.Force) "Absolute force" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
        equation 
          connect(tr2BG.Tr, flange_a) annotation (points=[-70.1,0.1; -83.05,0.1;
                -83.05,0; -100,0],       style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(bG2Tr.Tr, flange_b) annotation (points=[69.9,0.1; 82.95,0.1;
                82.95,0; 99,0], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.BondCon2, j0p3_1.BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.BondCon1, tr2BG.BondCon1) annotation (points=[-29.8,0; -50,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.eBondCon1, j0p3_1.BondCon3) annotation (points=[
                -6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2, B3.BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B3.BondCon2, bG2Tr.BondCon1) annotation (points=[30,0; 50,0],
              style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.fBondCon1, F.BondCon1) annotation (points=[6.12303e-016,
                -30; -6.12303e-016,-30], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(F.OutPort1, f) annotation (points=[6.73533e-016,-51;
                6.73533e-016,-70.5; 0,-70.5; 0,-100], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(tr2BG.s, bG2Tr.s) annotation (points=[-60,10; -60,40; 60,40;
                60,10], style(color=3, rgbcolor={0,0,255}));
        end ForceSensor;
        
        model PositionSensor "Ideal sensor to measure the absolute position" 
          annotation (
            Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Text(
                extent=[80,-28; 114,-62],
                string="s",
                style(color=0)),
              Text(extent=[0,100; 0,40],   string="%name"),
              Line(points=[70,0; 90,0], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1))),
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the absolute position.
This sensor element can be connected to any flange.
</html>"));
          
          Interfaces.Flange_a flange_a "Translational connector" 
                                       annotation (extent=[-110,-10; -90,10]);
          Modelica.Blocks.Interfaces.RealOutput s(redeclare type SignalType = 
                Modelica.SIunits.Position) "Absolute position" 
            annotation (extent=[90,-10; 110,10]);
        protected 
          Interfaces.Tr2BG tr2BG annotation (extent=[-70,-10; -50,10]);
          Bonds.eBond B1 annotation (extent=[-40,-10; -20,10]);
        public 
          BondLib.Sensors.Df V annotation (extent=[-20,-10; 0,10]);
        equation 
          connect(tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -83.05,
                -0.1; -83.05,0; -100,0], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.fBondCon1, tr2BG.BondCon1) annotation (points=[-40,0; -50,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(tr2BG.s, s) annotation (points=[-60,-10; -60,-20; 60,-20; 60,
                0; 100,0], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(V.BondCon1, B1.eBondCon1) annotation (points=[-20,0; -20,0],
              style(color=8, rgbcolor={192,192,192}));
        end PositionSensor;
        
        model SpeedSensor "Ideal sensor to measure the absolute velocity" 
        protected 
          Interfaces.Tr2BG tr2BG annotation (extent=[-70,-10; -50,10]);
          Bonds.eBond B1 annotation (extent=[-40,-10; -20,10]);
        public 
          BondLib.Mechanical.Translational.Interfaces.Flange_a flange_a 
            "Translational connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Blocks.Interfaces.RealOutput v(redeclare type SignalType = 
                Modelica.SIunits.Velocity) "Absolute velocity" 
            annotation (extent=[90,-10; 110,10]);
          BondLib.Sensors.Df V annotation (extent=[-20,-10; 0,10]);
        equation 
          connect(tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -83.05,
                -0.1; -83.05,0; -100,0], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.fBondCon1, tr2BG.BondCon1) annotation (points=[-40,0; -50,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          annotation (
            Diagram,
            Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Text(extent=[0,100; 0,40],   string="%name"),
              Line(points=[70,0; 90,0], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Text(
                extent=[80,-28; 114,-62],
                style(color=0),
                string="v")),
            Documentation(info="<html>
Ideal sensor to measure the absolute velocity.
This sensor element can be connected to any flange.
</html>"));
          connect(V.BondCon1, B1.eBondCon1) annotation (points=[-20,0; -20,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(V.OutPort1, v) annotation (points=[1,0; 100,0], style(color=
                  74, rgbcolor={0,0,127}));
          
        end SpeedSensor;
        
        model AccSensor "Ideal sensor to measure the absolute acceleration" 
        protected 
          Interfaces.Tr2BG tr2BG annotation (extent=[-70,-10; -50,10]);
          Bonds.eBond B1 annotation (extent=[-40,-10; -20,10]);
        public 
          BondLib.Mechanical.Translational.Interfaces.Flange_a flange_a 
            "Translational connector" 
            annotation (extent=[-110,-10; -90,10]);
          Modelica.Blocks.Interfaces.RealOutput a(redeclare type SignalType = 
                Modelica.SIunits.Acceleration) "Absolute acceleration" 
            annotation (extent=[90,-10; 110,10]);
          Modelica.Blocks.Continuous.Der A   annotation (extent=[30,-10; 50,10]);
          BondLib.Sensors.Df V annotation (extent=[-20,-10; 0,10]);
        equation 
          connect(tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -83.05,
                -0.1; -83.05,0; -100,0], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.fBondCon1, tr2BG.BondCon1) annotation (points=[-40,0; -50,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          annotation (
            Diagram,
            Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Text(extent=[0,100; 0,40],   string="%name"),
              Line(points=[70,0; 90,0], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Text(
                extent=[80,-28; 114,-62],
                style(color=0),
                string="a")),
            Documentation(info="<html>
Ideal sensor to measure the absolute acceleration.
This sensor element can be connected to any flange.
</html>"));
          connect(A.y, a)   annotation (points=[51,0; 100,0], style(color=74,
                rgbcolor={0,0,127}));
          connect(B1.eBondCon1, V.BondCon1) annotation (points=[-20,0; -20,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(V.OutPort1, A.u)   annotation (points=[1,0; 28,0], style(
                color=74, rgbcolor={0,0,127}));
          
        end AccSensor;
        
        model RelPositionSensor "Ideal sensor to measure the relative position" 
          extends Interfaces.OnePort;
        public 
          Modelica.Blocks.Interfaces.RealOutput s_rel(redeclare type SignalType
              = Modelica.SIunits.Position) "Relative position" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
          annotation (
            Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Line(points=[70,0; 90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[0,-60; 0,-90], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
                 Text(
                extent=[38,-68; 118,-118],
                style(color=0),
                string="s_rel"),
              Text(extent=[0,100; 0,40],   string="%name")),
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the relative position.
This sensor element can be connected between any two flanges.
</html>"));
        protected 
          Interfaces.Tr2BG tr2BG annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Tr bG2Tr annotation (extent=[50,-10; 70,10]);
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,10; 10,-10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,10; 10,30],   rotation=90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.Df V_rel 
            annotation (extent=[-10,30; 10,50], rotation=90);
          Modelica.Blocks.Math.Add A1(k1=+1, k2=-1) 
            annotation (extent=[-10,-60; 10,-40], rotation=-90);
        equation 
          connect(B1.BondCon2,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.BondCon1,tr2BG. BondCon1) annotation (points=[-29.8,0; -50,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B3. BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B3.BondCon2,bG2Tr. BondCon1) annotation (points=[30,0; 50,0],
              style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -83.05,
                -0.1; -83.05,0; -100,0], style(color=58, rgbcolor={0,127,0}));
          connect(bG2Tr.Tr, flange_b) annotation (points=[69.9,-0.1; 82.95,-0.1;
                82.95,0; 99,0], style(color=58, rgbcolor={0,127,0}));
          connect(j0p3_1.BondCon3, B2.eBondCon1) annotation (points=[0,10;
                -6.12303e-016,10], style(color=8, rgbcolor={192,192,192}));
          connect(V_rel.BondCon1, B2.fBondCon1) annotation (points=[
                -6.12303e-016,30; 6.12303e-016,30], style(color=8, rgbcolor={
                  192,192,192}));
          connect(A1.y, s_rel) annotation (points=[6.73533e-016,-61;
                6.73533e-016,-75.5; 0,-75.5; 0,-100], style(color=74, rgbcolor=
                  {0,0,127}));
          connect(tr2BG.s, A1.u2) annotation (points=[-60,-10; -60,-20; -6,-20;
                -6,-38], style(color=3, rgbcolor={0,0,255}));
          connect(bG2Tr.s, A1.u1) annotation (points=[60,-10; 60,-20; 6,-20; 6,
                -38], style(color=3, rgbcolor={0,0,255}));
        end RelPositionSensor;
        
        model RelSpeedSensor "Ideal sensor to measure the relative velocity" 
          import BondLib;
          extends Interfaces.OnePort;
        public 
          Modelica.Blocks.Interfaces.RealOutput v_rel(redeclare type SignalType
              = Modelica.SIunits.Velocity) "Relative velocity" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
          annotation (
            Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Line(points=[70,0; 90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[0,-60; 0,-90], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
                 Text(
                extent=[38,-68; 118,-118],
                style(color=0),
                string="v_rel"),
              Text(extent=[0,100; 0,40],   string="%name")),
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the relative velocity.
This sensor element can be connected between any two flanges.
</html>"));
        protected 
          Interfaces.Tr2BG tr2BG annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Tr bG2Tr annotation (extent=[50,-10; 70,10]);
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,-30; 10,-10], rotation=-90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.Df V_rel 
            annotation (extent=[-10,-50; 10,-30], rotation=270);
        equation 
          connect(B1.BondCon2,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.BondCon1,tr2BG. BondCon1) annotation (points=[-29.8,0; -50,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.eBondCon1,j0p3_1. BondCon3) annotation (points=[
                -6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B3. BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B3.BondCon2,bG2Tr. BondCon1) annotation (points=[30,0; 50,0],
              style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -83.05,
                -0.1; -83.05,0; -100,0],
                                   style(color=58, rgbcolor={0,127,0}));
          connect(bG2Tr.Tr, flange_b) annotation (points=[69.9,-0.1; 82.95,-0.1;
                82.95,0; 99,0], style(color=58, rgbcolor={0,127,0}));
          connect(V_rel.BondCon1, B2.fBondCon1) annotation (points=[
                1.83691e-015,-30; 6.12303e-016,-30], style(color=8, rgbcolor={
                  192,192,192}));
          connect(V_rel.OutPort1, v_rel) annotation (points=[-2.0206e-015,-51;
                -2.0206e-015,-70.5; 0,-70.5; 0,-100], style(color=74, rgbcolor=
                  {0,0,127}));
        end RelSpeedSensor;
        
        model RelAccSensor "Ideal sensor to measure the relative acceleration" 
          import BondLib;
          extends Interfaces.OnePort;
        protected 
          Interfaces.Tr2BG tr2BG annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Tr bG2Tr annotation (extent=[48,-10; 68,10]);
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,-30; 10,-10], rotation=-90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.Df V_rel 
            annotation (extent=[-10,-50; 10,-30], rotation=270);
        public 
          Modelica.Blocks.Interfaces.RealOutput a_rel(redeclare type SignalType
              = Modelica.SIunits.Acceleration) "Relative acceleration" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
        protected 
          Modelica.Blocks.Continuous.Der A_rel 
                                             annotation (extent=[-10,-80; 10,
                -60], rotation=-90);
        equation 
          connect(B1.BondCon2,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B1.BondCon1,tr2BG. BondCon1) annotation (points=[-29.8,0; -50,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.eBondCon1,j0p3_1. BondCon3) annotation (points=[
                -6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B3. BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B3.BondCon2,bG2Tr. BondCon1) annotation (points=[30,0; 48,0],
              style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(V_rel.BondCon1, B2.fBondCon1) annotation (points=[
                1.83691e-015,-30; 6.12303e-016,-30], style(color=8, rgbcolor={
                  192,192,192}));
          annotation (
            Diagram,
            Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Line(points=[70,0; 90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[0,-60; 0,-90], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Text(extent=[0,100; 0,40],   string="%name"),
                 Text(
                extent=[38,-68; 118,-118],
                style(color=0),
                string="a_rel")),
            Documentation(info="<html>
Ideal sensor to measure the relative acceleration.
This sensor element can be connected between any two flanges.
</html>"));
          connect(A_rel.y, a_rel) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-86.5; 0,-86.5; 0,-100], style(color=74, rgbcolor=
                  {0,0,127}));
          connect(A_rel.u, V_rel.OutPort1) annotation (points=[-7.34764e-016,
                -58; -7.34764e-016,-55; -2.0206e-015,-55; -2.0206e-015,-51],
              style(color=74, rgbcolor={0,0,127}));
          connect(tr2BG.Tr, flange_a) annotation (points=[-70.1,-0.1; -84.05,
                -0.1; -84.05,0; -100,0],
                                   style(color=58, rgbcolor={0,127,0}));
          connect(bG2Tr.Tr, flange_b) annotation (points=[67.9,-0.1; 82.95,-0.1;
                82.95,0; 99,0], style(color=58, rgbcolor={0,127,0}));
        end RelAccSensor;
      end Sensors;
      
      package Sources "Sources of the 1D translational library" 
        
        annotation (
          preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(fillColor=30, fillPattern=
                    1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3)),
            Text(
              extent=[-120,122; 120,73],
              string="%name",
              style(color=1))), Documentation(info="<html>
Active (external) sources of the 1D translational library.
</html>"));
        model Force 
          "External force acting on a drive train element as input signal" 
          Modelica.Blocks.Interfaces.RealInput f(redeclare type SignalType = 
                Modelica.SIunits.Force) "External force" 
            annotation (extent=[-100,-10; -80,10]);
          Interfaces.Flange_b flange_b "Translational connector" 
                                       annotation (extent=[90,-10; 110,10]);
        protected 
           BondLib.Sources.mSe F   annotation (extent=[-10,-10; -30,10]);
           Interfaces.BG2Tr BG2Tr 
             annotation (extent=[10,-10; 30,10]);
           BondLib.Bonds.fBond B1 annotation (extent=[-10,-10; 10,10]);
        equation 
          connect(F.BondCon1, B1.eBondCon1)    annotation (points=[-10,0; -10,0],
               style(color=8, rgbcolor={192,192,192}));
           connect(B1.fBondCon1,BG2Tr. BondCon1) annotation (points=[10,0; 10,0],
               style(color=8, rgbcolor={192,192,192}));
          annotation (
            Diagram,
            Icon(
              Polygon(points=[-80,10; 20,10; 20,40; 90,0; 20,-40; 20,-10; -80,
                    -10; -80,10],     style(
                  color=58,
                  rgbcolor={0,127,0},
                  fillColor=58,
                  rgbfillColor={0,127,0})),
              Text(
                extent=[-82,-32; -29,-80],
                string="f",
                style(color=0)),
              Text(extent=[-2,113; -2,53], string="%name")),
            Documentation(info="<html>
External force acting on a drive train element as input signal.
</html>"));
          connect(BG2Tr.Tr, flange_b) annotation (points=[29.9,-0.1; 62.95,-0.1;
                62.95,0; 100,0], style(color=58, rgbcolor={0,127,0}));
          connect(F.s, f) annotation (points=[-29,0; -90,0], style(color=74,
                rgbcolor={0,0,127}));
        end Force;
        
        model Speed 
          "Forced movement of a flange according to a velocity signal" 
          parameter Modelica.SIunits.Position s_start=0 "Start position";
          Modelica.SIunits.Position s(final start=s_start, final fixed=true) 
            "absolute position of flange_b";
          Modelica.Blocks.Interfaces.RealInput v(redeclare type SignalType = 
                Modelica.SIunits.Velocity) "External velocity" 
            annotation (extent=[-100,-10; -80,10]);
          Interfaces.Flange_b flange_b "Translational connector" 
                                       annotation (extent=[90,-10; 110,10]);
        protected 
          BondLib.Sources.mSf V   annotation (extent=[-10,-10; -30,10]);
          annotation (
            Diagram,
            Documentation(info="<html>
External velocity acting on a drive train element as input signal.
</html>"),  Icon(
              Line(points=[-80,0; 90,0],   style(color=58, rgbcolor={0,127,0})),
              Text(
                extent=[-55,-18; -104,-58],
                style(color=0),
                string="v"),
              Text(extent=[20,80; 20,20], string="%name")));
          BondLib.Mechanical.Translational.Interfaces.BG2Tr bG2Tr 
            annotation (extent=[10,-10; 30,10]);
          BondLib.Bonds.eBond B1 annotation (extent=[-10,-10; 10,10]);
          Modelica.Blocks.Continuous.Integrator S(y_start=s_start) 
            annotation (extent=[-20,-50; 0,-30]);
        equation 
          s = S.y;
          connect(bG2Tr.Tr, flange_b) annotation (points=[29.9,-0.1; 62.95,-0.1;
                62.95,0; 100,0], style(color=58, rgbcolor={0,127,0}));
          connect(bG2Tr.BondCon1, B1.eBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(V.BondCon1, B1.fBondCon1)   annotation (points=[-10,0; -10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(S.u, v) annotation (points=[-22,-40; -50,-40; -50,0; -90,0],
              style(color=74, rgbcolor={0,0,127}));
          connect(S.y, bG2Tr.s) annotation (points=[1,-40; 20,-40; 20,-10],
              style(color=74, rgbcolor={0,0,127}));
          connect(V.s, v) annotation (points=[-29,0; -90,0], style(color=74,
                rgbcolor={0,0,127}));
        end Speed;
        
        model Accelerate 
          "Forced movement of a flange according to an acceleration signal" 
          parameter Modelica.SIunits.Position s_start=0 "Start position";
          parameter Modelica.SIunits.Velocity v_start=0 "Start velocity";
          Modelica.SIunits.Velocity v(final start=v_start, final fixed=true) 
            "Absolute velocity of flange_b";
          Modelica.SIunits.Position s(final start=s_start, final fixed=true) 
            "Absolute position of flange_b";
        protected 
          BondLib.Sources.mSf mSf annotation (extent=[-10,-10; -30,10]);
          BondLib.Mechanical.Translational.Interfaces.BG2Tr bG2Tr 
            annotation (extent=[10,-10; 30,10]);
          BondLib.Bonds.eBond B1 annotation (extent=[-10,-10; 10,10]);
        public 
          Modelica.Blocks.Interfaces.RealInput a(redeclare type SignalType = 
                Modelica.SIunits.Acceleration) "External acceleration" 
            annotation (extent=[-100,-10; -80,10]);
          BondLib.Mechanical.Translational.Interfaces.Flange_b flange_b 
            "Translational connector" 
            annotation (extent=[90,-10; 110,10]);
        protected 
          Modelica.Blocks.Continuous.Integrator V(y_start=v_start) 
            annotation (extent=[-60,-10; -40,10]);
          Modelica.Blocks.Continuous.Integrator S(y_start=s_start) 
            annotation (extent=[-20,-50; 0,-30]);
        equation 
          connect(bG2Tr.Tr, flange_b) annotation (points=[29.9,-0.1; 62.95,-0.1;
                62.95,0; 100,0], style(color=58, rgbcolor={0,127,0}));
          connect(bG2Tr.BondCon1, B1.eBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(mSf.BondCon1, B1.fBondCon1) annotation (points=[-10,0; -10,0],
              style(color=8, rgbcolor={192,192,192}));
          annotation (
            Diagram,
            Documentation(info="<html>
External acceleration acting on a drive train element as input signal.
</html>"),  Icon(
              Line(points=[-80,0; 90,0],   style(color=58, rgbcolor={0,127,0})),
              Text(
                extent=[-55,-18; -104,-58],
                style(color=0),
                string="a"),
              Text(extent=[20,80; 20,20], string="%name")));
          v = V.y;
          s = S.y;
          connect(a, V.u) annotation (points=[-90,0; -62,0], style(color=74,
                rgbcolor={0,0,127}));
          connect(S.u, V.y) annotation (points=[-22,-40; -34,-40; -34,0; -39,0],
              style(color=74, rgbcolor={0,0,127}));
          connect(S.y, bG2Tr.s) annotation (points=[1,-40; 20,-40; 20,-10],
              style(color=74, rgbcolor={0,0,127}));
          connect(mSf.s, V.y) annotation (points=[-29,0; -39,0], style(color=74,
                rgbcolor={0,0,127}));
        end Accelerate;
        
        model Position 
          "Forced movement of a flange according to a reference position" 
          Modelica.SIunits.Velocity v "Absolute velocity of flange_b";
        protected 
          BondLib.Sources.mSf mSf annotation (extent=[-10,-10; -30,10]);
          BondLib.Mechanical.Translational.Interfaces.BG2Tr bG2Tr 
            annotation (extent=[10,-10; 30,10]);
          BondLib.Bonds.eBond B1 annotation (extent=[-10,-10; 10,10]);
        public 
          Modelica.Blocks.Interfaces.RealInput s(redeclare type SignalType = 
                Modelica.SIunits.Position) "External position" 
            annotation (extent=[-100,-10; -80,10]);
          BondLib.Mechanical.Translational.Interfaces.Flange_b flange_b 
            "Translational connector" 
            annotation (extent=[90,-10; 110,10]);
        protected 
          Modelica.Blocks.Continuous.Der V annotation (extent=[-60,-10; -40,10]);
        equation 
          connect(bG2Tr.Tr, flange_b) annotation (points=[29.9,-0.1; 62.95,-0.1;
                62.95,0; 100,0], style(color=58, rgbcolor={0,127,0}));
          connect(bG2Tr.BondCon1, B1.eBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(mSf.BondCon1, B1.fBondCon1) annotation (points=[-10,0; -10,0],
              style(color=8, rgbcolor={192,192,192}));
          annotation (Diagram, Documentation(info="<html>
External position acting on a drive train element as input signal.
</html>"),  Icon(
              Line(points=[-80,0; 90,0],   style(color=58, rgbcolor={0,127,0})),
              Text(
                extent=[-55,-18; -104,-58],
                style(color=0),
                string="s"),
              Text(extent=[20,80; 20,20], string="%name")));
          v = V.y;
          connect(s, V.u) annotation (points=[-90,0; -62,0], style(color=74,
                rgbcolor={0,0,127}));
          connect(bG2Tr.s, s) annotation (points=[20,-10; 20,-20; -72,-20; -72,
                0; -90,0], style(color=3, rgbcolor={0,0,255}));
          connect(mSf.s, V.y) annotation (points=[-29,0; -39,0], style(color=74,
                rgbcolor={0,0,127}));
        end Position;
      end Sources;
      
      package Utilities "Auxiliary models of the 1D translational library" 
        annotation (
          preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=42, rgbcolor={127,0,0})),
            Text(
              extent=[-120,122; 120,73],
              string="%name",
              style(color=1))),
          Documentation(info="<html>
Auxiliary models of the 1D translational library.
</html>"));
        model RelativeStates "Definition of relative state variables" 
          extends Interfaces.OnePort;
          Modelica.SIunits.Position s_rel(stateSelect=StateSelect.prefer) 
            "Relative position used as state variable";
          Modelica.SIunits.Velocity v_rel(stateSelect=StateSelect.prefer) 
            "Relative velocity used as state variable";
          Modelica.SIunits.Acceleration a_rel "Relative acceleration";
          annotation (
            Icon(
              Ellipse(extent=[-38,40; 42,-40],   style(color=4, fillColor=4)),
              Text(extent=[-38,40; 42,-40],   string="S"),
              Line(points=[-90,0; -40,0],   style(color=0, pattern=3)),
              Line(points=[42,0; 92,0],   style(
                  color=0,
                  pattern=3,
                  fillColor=4)),
              Text(extent=[2,110; 2,50],   string="%name")),
            Diagram(
              Ellipse(extent=[-40,40; 40,-40],   style(color=4, fillColor=4)),
              Text(extent=[-40,40; 40,-40],   string="S"),
              Line(points=[40,0; 90,0],   style(
                  color=0,
                  pattern=2,
                  fillColor=4)),
              Line(points=[-100,-60; 80,-60],   style(color=9)),
              Text(extent=[-30,-70; 30,-90], string="v_rel"),
              Line(points=[-76,80; -5,80],   style(color=10, fillColor=10)),
              Polygon(points=[14,80; -6,85; -6,75; 14,80],     style(color=10,
                    fillColor=10)),
              Text(
                extent=[14,93; 94,68],
                style(color=10),
                string="translation axis"),
              Line(points=[-90,0; -40,0],   style(
                  color=0,
                  pattern=2,
                  fillColor=4)),
              Line(points=[100,-10; 100,-80],   style(color=9)),
              Polygon(points=[80,-65; 80,-55; 100,-60; 80,-65],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Line(points=[-100,-10; -100,-80],   style(color=9))),
            Documentation(info="<html>
<p>
Usually, the absolute position and the absolute velocity of
Modelica.Mechanics.Translational.Inertia models are used as state variables.
In some circumstances, relative quantities are better suited, e.g.,
because it may be easier to supply initial values.
In such cases, model <b>RelativeStates</b> allows the definition of state variables
in the following way:
</p>
<ul>
<li> Connect an instance of this model between two flange connectors.</li>
<li> The <b>relative position</b> and the <b>relative velocity</b>
     between the two connectors are used as <b>state variables</b>.
</ul>
<p>
An example is given in the next figure
</p>
<IMG SRC=\"../Images/BG_Fig30.png\">
<p>
Here, the relative position and the relative velocity between
the two masses are used as state variables. Additionally, the
simulator selects either the absolute position and absolute
velocity of model mass1 or of model mass2 as state variables.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 19, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</html>"));
        equation 
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          a_rel = der(v_rel);
          flange_a.f = 0;
          flange_b.f = 0;
        end RelativeStates;
        
        model Add "Bidirectional adder" 
          
          parameter Real k1=+1 "Gain of upper input";
          parameter Real k2=+1 "Gain of lower input";
          annotation (
            Icon(Rectangle(extent=[-100,-100; 100,100],   style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-98,-52; 7,-92],
                string="%k2",
                style(color=0)),
              Text(
                extent=[-100,90; 5,50],
                string="%k1",
                style(color=0)),
              Line(points=[-100,60; -40,60; -30,40]),
              Ellipse(extent=[-50,50; 50,-50]),
              Line(points=[-100,-60; -40,-60; -30,-40]),
              Line(points=[-15,-25.99; 15,25.99],   style(color=0)),
              Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
              Line(points=[50,0; 100,0]),
              Line(points=[-100,60; -74,24; -44,24],    style(color=74, rgbcolor=
                  {0,0,127})),
              Line(points=[-100,-60; -74,-28; -42,-28],    style(color=74,
                rgbcolor={0,0,127})),
              Ellipse(extent=[-50,50; 50,-50],   style(color=74, rgbcolor={0,0,
                  127})),
              Line(points=[50,0; 100,0],   style(color=74, rgbcolor={0,0,127})),
              Text(
                extent=[-38,34; 38,-34],
                string="+",
                style(color=0)),
              Text(
                extent=[-100,52; 5,92],
                string="%k1",
                style(color=0)),
              Text(
                extent=[-100,-52; 5,-92],
                string="%k2",
                style(color=0)),
              Text(extent=[-154,148; 146,108],   string="%name")),
            Diagram(
              Rectangle(extent=[-100,-100; 100,100],   style(color=3, fillColor=7)),
              Text(
                extent=[-98,-52; 7,-92],
                string="%k2",
                style(color=0)),
              Text(
                extent=[-100,90; 5,50],
                string="%k1",
                style(color=0)),
              Line(points=[-100,60; -40,60; -30,40]),
              Ellipse(extent=[-50,50; 50,-50]),
              Line(points=[-100,-60; -40,-60; -30,-40]),
              Line(points=[-15,-25.99; 15,25.99],   style(color=0)),
              Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
              Line(points=[50,0; 100,0]),
              Line(points=[-100,60; -74,24; -44,24],    style(color=74, rgbcolor=
                  {0,0,127})),
              Line(points=[-100,-60; -74,-28; -42,-28],    style(color=74,
                rgbcolor={0,0,127})),
              Ellipse(extent=[-50,50; 50,-50],   style(color=74, rgbcolor={0,0,
                  127})),
              Line(points=[50,0; 100,0],   style(color=74, rgbcolor={0,0,127})),
              Text(
                extent=[-38,34; 38,-34],
                string="+",
                style(color=0)),
              Text(
                extent=[-100,52; 5,92],
                string="k1",
                style(color=0)),
              Text(
                extent=[-100,-52; 5,-92],
                string="k2",
                style(color=0))),
            Documentation(info="<html>
Bidirectional adder representing a translational 0-junction at positional level.
</html>"));
          Interfaces.RealSignal u1(redeclare type SignalType = 
                Modelica.SIunits.Position) "First positional signal" 
            annotation (extent=[-118,50; -98,70]);
          Interfaces.RealSignal u2(redeclare type SignalType = 
                Modelica.SIunits.Position) "Second positional signal" 
            annotation (extent=[-118,-70; -98,-50]);
          Interfaces.RealSignal y(redeclare type SignalType = 
                Modelica.SIunits.Position) "Third positional signal" 
            annotation (extent=[98,-10; 118,10]);
        equation 
          y = k1*u1 + k2*u2;
        end Add;
      end Utilities;
      
      package Examples "Examples of using the 1D translational library" 
        annotation (
          preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85,71; 65,-49],
              string="Sample",
              style(color=0)),
            Text(
              extent=[-118,114; 122,65],
              string="%name",
              style(color=1)),
            Text(
              extent=[-97,23; 78,-98],
              string="Programs",
              style(color=0))),
          Documentation(info="<html>
Examples of using the 1D translational library.
</html>"));
        model ThreeSlidingMasses "Three masses sliding on each other" 
          Modelica.SIunits.Length x1 "Absolute positon of mass 1";
          Modelica.SIunits.Length x2 "Absolute positon of mass 2";
          Modelica.SIunits.Length x3 "Absolute positon of mass 3";
          annotation (
            Documentation(info="<html>
Three masses are sliding on each other. <p>

This example is taken from the book <a href=\"http://www.inf.ethz.ch/personal/fcellier/Pubs/BG/springer.html\">
                                       Continuous System Modeling</a> by F.E. Cellier.
It is the example of Fig. 7.18 (page 267): <p>

<img src=\"../Images/BG_Ex17.png\"> <p>

</html>"),  Coordsys(extent=[-100,-40; 100,60]),
            Diagram,
            Icon(Bitmap(extent=[-100,60; 100,-40], name="Images/BG_Ex17.png")),
            experiment(StopTime=10),
            experimentSetupOutput);
          Passive.Spring spring1(s_rel0=0.5, c=15) 
            annotation (extent=[-68,-30; -48,-10]);
          Passive.Fixed wall1 
            annotation (extent=[-84,-30; -64,-10], rotation=-90);
          Passive.Damper damp12(d=20) annotation (extent=[-8,-30; 12,-10]);
          Passive.Spring spring2(s_rel0=0.5, c=25) 
            annotation (extent=[48,-30; 68,-10]);
          Passive.Fixed wall2(s0=3) 
            annotation (extent=[84,-30; 64,-10],   rotation=270);
          Passive.Damper damp13(d=20) annotation (extent=[-40,-10; -20,10]);
          Passive.Damper damp23(d=10) annotation (extent=[20,-10; 40,10]);
          Sources.Force force 
            annotation (extent=[-8,20; 12,40]);
          Modelica.Blocks.Sources.Constant Constant(k=10) 
            annotation (extent=[-40,20; -20,40]);
          Passive.SlidingMass mass1(m=3, s0=0.5) 
            annotation (extent=[-40,-30; -20,-10]);
          Passive.SlidingMass mass2(m=6, s0=2.5) 
            annotation (extent=[20,-30; 40,-10]);
          Passive.SlidingMass mass3(m=10, s0=0) 
            annotation (extent=[-8,-10; 12,10]);
        equation 
          x1 = mass1.s;
          x2 = mass2.s;
          x3 = mass3.s;
          connect(wall1.flange_b,spring1. flange_a) annotation (points=[-74,-20;
                -70.05,-20; -70.05,-20; -68,-20],          style(color=58,
                rgbcolor={0,127,0}));
          connect(spring2.flange_b,wall2. flange_b) annotation (points=[67.9,-20;
                71,-20; 71,-20; 74,-20],            style(color=58, rgbcolor={0,
                  127,0}));
          connect(mass1.flange_a,spring1. flange_b) annotation (points=[-40,-20;
                -48.1,-20], style(color=58, rgbcolor={0,127,0}));
          connect(mass1.flange_b,damp12. flange_a) annotation (points=[-20.1,
                -20; -8,-20], style(color=58, rgbcolor={0,127,0}));
          connect(damp13.flange_a,mass1. flange_a) annotation (points=[-40,0;
                -40,-20], style(color=58, rgbcolor={0,127,0}));
          connect(mass2.flange_a,damp12. flange_b) annotation (points=[20,-20;
                11.9,-20], style(color=58, rgbcolor={0,127,0}));
          connect(mass2.flange_b,spring2. flange_a) annotation (points=[39.9,-20;
                48,-20],      style(color=58, rgbcolor={0,127,0}));
          connect(damp23.flange_b,mass2. flange_b) annotation (points=[39.9,0;
                39.9,-20], style(color=58, rgbcolor={0,127,0}));
          connect(mass3.flange_b,damp23. flange_a) annotation (points=[11.9,0;
                20,0], style(color=58, rgbcolor={0,127,0}));
          connect(mass3.flange_a,damp13. flange_b) annotation (points=[-8,0;
                -20.1,0], style(color=58, rgbcolor={0,127,0}));
          connect(force.flange_b,mass3. flange_b) annotation (points=[12,30; 12,
                0; 11.9,0], style(color=58, rgbcolor={0,127,0}));
          connect(Constant.y, force.f) annotation (points=[-19,30; -7,30],
              style(color=74, rgbcolor={0,0,127}));
        end ThreeSlidingMasses;
        
        model Sensors 
          "Example demonstarting the use of 1D translational sensors" 
          import BondLib;
          Modelica.Blocks.Sources.Sine Sine1(
                             amplitude=10, freqHz=4) annotation (extent=[-90,50;
                -70,70]);
          Sources.Force Force1 annotation (extent=[-50,50; -30,70]);
          annotation (
            Diagram,
            Documentation(info="These sensors measure
   force f in N
   position s in m
   velocity v in m/s
   acceleration a in m/s2
Dhe measured velocity and acceleration is independent on
the flange the sensor is connected to. The position
depends on the flange (flange_a or flange_b) and the
length L of the component.
Plot PositionSensor1.s, PositionSensor2.s and SlidingMass1.s
to see the difference.
<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>First Version from December 10, 1999 by P. Beater </i> </li>
</ul>
</html>"),  Icon(
              Line(points=[0,0; -90,0], style(color=0, rgbcolor={0,0,0})),
              Line(points=[70,0; 90,0], style(color=0, rgbcolor={0,0,0})),
              Rectangle(extent=[-70,20; 70,-60], style(color=0, rgbcolor={0,0,0})),
              Line(points=[0,0; 0,-16], style(color=0, rgbcolor={0,0,0})),
              Polygon(points=[-10,-16; 0,-40; 10,-16; -10,-16], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0})),
              Line(points=[-50,-40; -50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-30,-40; -30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[-10,-40; -10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[10,-40; 10,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[30,-40; 30,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[50,-40; 50,-60], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1)),
              Line(points=[0,-60; 0,-90], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=0,
                  rgbfillColor={0,0,0},
                  fillPattern=1))));
          BondLib.Mechanical.Translational.Passive.SlidingMass slidingMass(L=1) 
            annotation (extent=[30,50; 50,70]);
          BondLib.Mechanical.Translational.Sensors.PositionSensor 
            PositionSensor1 annotation (extent=[30,10; 50,30]);
          BondLib.Mechanical.Translational.Sensors.SpeedSensor SpeedSensor1 
            annotation (extent=[30,-30; 50,-10]);
          BondLib.Mechanical.Translational.Sensors.PositionSensor 
            PositionSensor2 annotation (extent=[70,50; 90,70]);
          BondLib.Mechanical.Translational.Sensors.AccSensor AccSensor1 
            annotation (extent=[30,-70; 50,-50]);
          BondLib.Mechanical.Translational.Sensors.ForceSensor ForceSensor1 
            annotation (extent=[-10,50; 10,70]);
        equation 
          connect(PositionSensor1.flange_a, slidingMass.flange_a) annotation (
              points=[30,20; 30,60], style(color=58, rgbcolor={0,127,0}));
          connect(SpeedSensor1.flange_a, PositionSensor1.flange_a) annotation (
              points=[30,-20; 30,20], style(color=58, rgbcolor={0,127,0}));
          connect(slidingMass.flange_b, PositionSensor2.flange_a) annotation (
              points=[49.9,60; 70,60], style(color=58, rgbcolor={0,127,0}));
          connect(AccSensor1.flange_a, SpeedSensor1.flange_a) annotation (
              points=[30,-60; 30,-20], style(color=58, rgbcolor={0,127,0}));
          connect(Force1.flange_b, ForceSensor1.flange_a) annotation (points=[
                -30,60; -10,60], style(color=58, rgbcolor={0,127,0}));
          connect(ForceSensor1.flange_b, slidingMass.flange_a) annotation (
              points=[9.9,60; 30,60], style(color=58, rgbcolor={0,127,0}));
          connect(Sine1.y, Force1.f) annotation (points=[-69,60; -49,60], style(
                color=74, rgbcolor={0,0,127}));
        end Sensors;
      end Examples;
    end Translational;
    
    package Rotational "Rotational 1D systems built from bond graphs" 
      
      annotation (
        preferedView="info",
        Icon(
          Text(
            extent=[-120,118; 120,69],
            string="%name",
            style(color=1)),
          Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                 1)),
          Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                fillColor=30, fillPattern=1)),
          Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                fillColor=30, fillPattern=1)),
          Text(
            extent=[-85,35; 65,-85],
            string="Library",
            style(color=3))), Documentation(info="<html>
This library reimplements parts of the 1D rotational library of the Modelica standard library.  All models are internally mapped onto a bond graph representation in order to minimize the need for equation modeling. <p>

Due to the underlying bond graph technology, the connectors of this library are incompatible with the connectors of the corresponding mechanical sub-library of the Modelica standard library.  The bond graph models require velocity information to be transferred across the connectors beside from positional information.  Consequently, the connectors of this library contain one additional variable. <p>

For this reason, models from this library cannot be mixed with the models of the corresponding sub-library of the Modelica standard library.
</html>"));
      package Interfaces "Wrapping models and partial models" 
        
        annotation (
          preferedView="info",
          Icon(
            Text(
              extent=[-118,112; 122,63],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],   style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],
                style(
                color=58,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=58))), Documentation(info="<html>
Sublibrary of interface models: connectors, wrapper models, and partial models that are being used by the 1D rotational library.
</html>"));
        connector Flange_a "Rotational bond graph connector" 
          
          annotation (
            Icon(Ellipse(extent=[-100,100; 100,-100], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=10,
                  rgbfillColor={95,95,95},
                  fillPattern=1))),
            Diagram(                Text(
                extent=[-160,110; 40,50],
                string="%name",
                style(color=0)), Ellipse(extent=[-40,40; 40,-40], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=10,
                  rgbfillColor={135,135,135}))),
            Documentation(info="<html>
\"Left\" rotational connector. <p>

This connector is incompatible with the corresponding connector of the standard library because of the underlying bond graph technology that requires velocity information beside from positional information to be transferred.
</html>"));
          Modelica.SIunits.Angle phi "Absolute rotation angle of flange";
          Modelica.SIunits.AngularVelocity w 
            "Absolute angular velocity of flange";
          flow Modelica.SIunits.Torque tau "Cut torque in the flange";
        end Flange_a;
        
        connector Flange_b "Rotational bond graph connector" 
          
          annotation (
            Icon(Ellipse(extent=[-100,100; 100,-100],style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1))),
            Diagram(             Ellipse(extent=[-40,40; 40,-40], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                                    Text(
                extent=[-40,110; 160,50],
                string="%name",
                style(color=0))),
            Documentation(info="<html>
\"Right\" rotational connector. <p>

This connector is incompatible with the corresponding connector of the standard library because of the underlying bond graph technology that requires velocity information beside from positional information to be transferred.
</html>"));
          Modelica.SIunits.Angle phi "Absolute rotation angle of flange";
          Modelica.SIunits.AngularVelocity w 
            "Absolute angular velocity of flange";
          flow Modelica.SIunits.Torque tau "Cut torque in the flange";
        end Flange_b;
        
        model Rot2BG "Rotational to bond graph conversion" 
          
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
            annotation (extent=[90,-10; 110,10]);
          BondLib.Mechanical.Rotational.Interfaces.Flange_a Rot 
            "Rotational connector" 
            annotation (extent=[-110,-10; -90,10]);
          BondLib.Mechanical.Translational.Interfaces.RealSignal phi(redeclare 
              type SignalType = 
                           Modelica.SIunits.Angle) "Angular position" 
            annotation (extent=[-10,-110; 10,-90]);
          annotation (
            Diagram(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80], style(color=0,
                    rgbcolor={0,0,0})),
              Line(points=[80,80; 80,-80; -80,-80],    style(color=9)),
              Line(points=[94,0; 80,0],   style(color=9)),
              Line(points=[-96,0; -80,0], style(color=0, rgbcolor={0,0,0})),
              Text(
                extent=[-64,56; 0,12],
                string="Rot",
                style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=58,
                  rgbfillColor={0,127,0})),
              Text(
                extent=[4,-14; 70,-64],
                string="BG",
                style(color=9)),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Icon(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80], style(color=0,
                    rgbcolor={0,0,0})),
              Line(points=[80,80; 80,-80; -80,-80],    style(color=9)),
              Line(points=[94,0; 80,0],   style(color=9)),
              Line(points=[-96,0; -80,0], style(color=0, rgbcolor={0,0,0})),
              Text(
                extent=[-64,56; 0,12],
                string="Rot",
                style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=58,
                  rgbfillColor={0,127,0})),
              Text(
                extent=[4,-14; 70,-64],
                string="BG",
                style(color=9)),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Documentation(info="<html>
Wrapper model translating mechanical (external) connectors to bond graph (internal) connectors.
</html>"));
        equation 
          BondCon1.e = Rot.tau;
          BondCon1.f = Rot.w;
          phi = Rot.phi;
        end Rot2BG;
        
        model BG2Rot "Bond graph to rotational conversion" 
          
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
            annotation (extent=[-110,-10; -90,10]);
          BondLib.Mechanical.Rotational.Interfaces.Flange_b Rot 
            "Rotational connector" 
            annotation (extent=[90,-10; 110,10]);
          BondLib.Mechanical.Translational.Interfaces.RealSignal phi(redeclare 
              type SignalType = 
                           Modelica.SIunits.Angle) "Angular position" 
            annotation (extent=[-10,-110; 10,-90]);
          annotation (
            Diagram(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80],     style(color=9)),
              Line(points=[-94,0; -80,0],   style(color=9)),
              Line(points=[80,80; 80,-80; -80,-80], style(color=0, rgbcolor={0,
                      0,0})),
              Line(points=[96,0; 80,0], style(color=0, rgbcolor={0,0,0})),
              Text(
                extent=[-64,70; 2,20],
                string="BG",
                style(color=9)),
              Text(
                extent=[0,-16; 64,-60],
                string="Rot",
                style(color=0, rgbcolor={0,0,0})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Icon(
              Line(points=[-80,-80; -80,80; 80,80; -80,-80],     style(color=9)),
              Line(points=[-94,0; -80,0],   style(color=9)),
              Line(points=[80,80; 80,-80; -80,-80], style(color=0, rgbcolor={0,
                      0,0})),
              Line(points=[96,0; 80,0], style(color=0, rgbcolor={0,0,0})),
              Text(
                extent=[-64,70; 2,20],
                string="BG",
                style(color=9)),
              Text(
                extent=[0,-16; 64,-60],
                string="Rot",
                style(color=0, rgbcolor={0,0,0})),
              Line(points=[52,0; 40,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[52,0; 40,10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-50,0; 52,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[0,-92; 0,-80], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=58,
                  rgbfillColor={0,127,0},
                  fillPattern=1))),
            Documentation(info="<html>
Wrapper model translating bond graph (internal) connectors to mechanical (external) connectors.
</html>"));
        equation 
          Rot.tau = -BondCon1.e;
          Rot.w = BondCon1.f;
          Rot.phi = phi;
        end BG2Rot;
        
        partial model OnePort "OnePort partial model of rotational library" 
          Flange_a flange_a "Left rotational connector" 
                            annotation (extent=[-110,-10; -90,10]);
          Flange_b flange_b "Right rotational connector" 
                            annotation (extent=[90,-10; 110,10]);
          annotation (Documentation(info="<html>
Partial \"OnePort\" model of the rotational library based on bond graph technology.
</html>"), Diagram);
        end OnePort;
        
        partial model FrictionBase "Base class of Coulomb friction elements" 
          // parameter Modelica.SIunits.AngularVelocity w_small=1 "Relative angular velocity near zero";
          parameter Modelica.SIunits.AngularVelocity w_small=1e10 
            "Relative angular velocity near zero" 
             annotation(Dialog(tab="Advanced"));
          
          // Equations to define the following variables have to be defined in subclasses
          Modelica.SIunits.AngularVelocity w_relfric 
            "Relative angular velocity between frictional surfaces";
          Modelica.SIunits.AngularAcceleration a_relfric 
            "Relative angular acceleration between frictional surfaces";
          Modelica.SIunits.Torque tau 
            "Friction torque (positive, if directed in opposite direction of w_rel)";
          Modelica.SIunits.Torque tau0 
            "Friction torque for w=0 and forward sliding";
          Modelica.SIunits.Torque tau0_max 
            "Maximum friction torque for w=0 and locked";
          Boolean free "true, if frictional element is not active";
          
          // Equations to define the following variables are given in this class
          Real sa 
            "Path parameter of friction characteristic tau = f(a_relfric)";
          
          Boolean startForward(start=false, fixed=true) 
            "true, if w_rel=0 and start of forward sliding or w_rel > w_small";
          Boolean startBackward(start=false, fixed=true) 
            "true, if w_rel=0 and start of backward sliding or w_rel < -w_small";
          Boolean locked "true, if w_rel=0 and not sliding";
          
          constant Integer Unknown=3 "Value of mode is not known";
          constant Integer Free=2 "Element is not active";
          constant Integer Forward=1 "w_rel > 0 (forward sliding)";
          constant Integer Stuck=0 
            "w_rel = 0 (forward sliding, locked or backward sliding)";
          constant Integer Backward=-1 "w_rel < 0 (backward sliding)";
          Integer mode(
            final min=Backward,
            final max=Unknown,
            start=Unknown,
            fixed=true);
          
        equation 
          /* Friction characteristic
      (locked is introduced to help the Modelica translator determining
      the different structural configurations, if for each configuration
      special code shall be generated) */
          
          startForward = pre(mode) == Stuck and (sa > tau0_max or pre(startForward)
             and sa > tau0) or pre(mode) == Backward and w_relfric > w_small or 
            initial() and (w_relfric > 0);
          startBackward = pre(mode) == Stuck and (sa < -tau0_max or pre(
            startBackward) and sa < -tau0) or pre(mode) == Forward and w_relfric <
            -w_small or initial() and (w_relfric < 0);
          locked = not free and not (pre(mode) == Forward or startForward or pre(
            mode) == Backward or startBackward);
          
          a_relfric = if locked then 0 else if free then sa else if startForward then 
                  sa - tau0 else if startBackward then sa + tau0 else if pre(mode)
             == Forward then sa - tau0 else sa + tau0;
          
          /* Friction torque has to be defined in a subclass. Example for a clutch:
       tau = if locked then sa else if free then 0 else cgeo*fn*
            (if startForward then   Math.tempInterpol1( w_relfric, mue_pos, 2) else
             if startBackward then -Math.tempInterpol1(-w_relfric, mue_pos, 2) else
             if pre(mode) == Forward then  Math.tempInterpol1( w_relfric, mue_pos, 2)
                                     else -Math.tempInterpol1(-w_relfric, mue_pos, 2)); */
          
          // finite state machine to determine configuration
          mode = if free then Free else (if (pre(mode) == Forward or pre(mode) ==
            Free or startForward) and w_relfric > 0 then Forward else if (pre(mode)
             == Backward or pre(mode) == Free or startBackward) and w_relfric < 0 then 
                  Backward else Stuck);
          annotation (Documentation(info="<html>
This partial model is copied over one-to-one from the rotational library featured in the Modelica standard library.
The bond graph notation doesn't change this model at all as it only determines the internal operation of friction elements.
</html>"));
        end FrictionBase;
      end Interfaces;
      
      package Passive "Passive rotational elements" 
        annotation (
          preferedView="info",
          Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))), Documentation(info="<html>
Passive elements of the 1D rotational library.
</html>"));
        model Fixed "Flange fixed in housing at a given angle" 
          parameter Modelica.SIunits.Angle phi0=0 
            "Fixed offset angle of housing";
          
          Interfaces.Flange_b flange_b "Rotational connector" 
                                       annotation (extent=[-10,-10; 10,10]);
          annotation (
            Icon(
              Text(extent=[0,-92; 0,-152],   string="%name=%phi0"),
              Line(points=[-80,-40; 80,-40],   style(color=0)),
              Line(points=[80,-40; 40,-80],   style(color=0)),
              Line(points=[40,-40; 0,-80],   style(color=0)),
              Line(points=[0,-40; -40,-80],   style(color=0)),
              Line(points=[-40,-40; -80,-80],   style(color=0)),
              Line(points=[0,-40; 0,-10],   style(color=0))),
            Diagram(
              Line(points=[-80,-40; 80,-40],   style(color=0)),
              Line(points=[80,-40; 40,-80],   style(color=0)),
              Line(points=[40,-40; 0,-80],   style(color=0)),
              Line(points=[0,-40; -40,-80],   style(color=0)),
              Line(points=[-40,-40; -80,-80],   style(color=0)),
              Line(points=[0,-40; 0,-4],    style(color=0)),
              Line(points=[-82,46; -11,46],   style(color=10, fillColor=10)),
              Polygon(points=[8,46; -12,51; -12,41; 8,46],     style(color=10,
                    fillColor=10)),
              Text(
                extent=[12,53; 80,40],
                string="rotation axis",
                style(color=10))),
            Documentation(info="<html>
Rotational inertial (ground) node.
</html>"));
        equation 
          flange_b.phi = phi0;
          flange_b.w = 0;
        end Fixed;
        
        model Damper "Linear 1D rotational damper" 
          extends Interfaces.OnePort;
          parameter Real d(
            final unit="N.m.s/rad",
            final min=0) = 0 "Damping constant";
          Modelica.SIunits.Distance phi_rel "Relative angle of damper";
          Modelica.SIunits.AngularVelocity w_rel 
            "Relative angular velocity of damper";
          Modelica.SIunits.Torque tau "Damping torque";
          annotation (
            Icon(
              Line(points=[-90,0; -60,0],   style(color=0)),
              Line(points=[-60,-30; -60,30],   style(color=0)),
              Line(points=[-60,-30; 60,-30],   style(color=0)),
              Line(points=[-60,30; 60,30],   style(color=0)),
              Rectangle(extent=[-60,30; 30,-30],   style(color=0, fillColor=8)),
              Line(points=[30,0; 90,0],   style(color=0)),
              Text(extent=[0,100; 0,40],   string="%name"),
              Text(
                extent=[0,-50; 0,-100],
                string="d=%d",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
Wrapped 1D linear rotational damper model.
</html>"));
        protected 
          Bonds.eBond B1  annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2  annotation (extent=[10,-10; 30,10]);
          Bonds.eBond B3 
            annotation (extent=[-10,-30; 10,-10],
                                                rotation=270);
          BondLib.Passive.R R(R=d) 
            annotation (extent=[-10,-50; 10,-30],rotation=270);
          BondLib.Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
          BondLib.Mechanical.Rotational.Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,-10; -50,10]);
          BondLib.Mechanical.Rotational.Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,-10; 70,10]);
        equation 
          phi_rel = BG2Rot.phi - Rot2BG.phi;
          w_rel = -R.f;
          tau = -R.e;
          connect(B1.eBondCon1,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0],      style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B2. eBondCon1) annotation (points=[10,0; 10,0],
                     style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(j0p3_1.BondCon3,B3. fBondCon1) annotation (points=[0,-10;
                1.83691e-015,-10],style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(R.BondCon1,B3. eBondCon1) annotation (points=[1.83691e-015,
                -30; -1.83691e-015,-30], style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.BondCon1, B1.fBondCon1) annotation (points=[-50,0; -30,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(B2.fBondCon1, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
        end Damper;
        
        model Spring "Linear 1D rotational spring" 
          extends Interfaces.OnePort;
          parameter Real c(final unit="N.m/rad", final min=0) "Spring constant";
          parameter Modelica.SIunits.Angle phi_rel0=0 
            "Unstretched spring angle";
          Modelica.SIunits.Angle phi_rel "Relative angle of spring";
          Modelica.SIunits.AngularVelocity w_rel 
            "Relative angular velocity of spring";
          Modelica.SIunits.Torque tau "Spring torque";
          annotation (
            Icon(
              Text(extent=[0,100; 0,40],   string="%name"),
              Text(
                extent=[10,-40; 10,-90],
                string="c=%c",
                style(color=0)),
              Line(points=[-100,0; -58,0; -43,-30; -13,30; 17,-30; 47,30; 62,0;
                    100,0],   style(
                  color=0,
                  pattern=1,
                  thickness=1,
                  arrow=0))),
            Diagram,
            Documentation(info="<html>
Wrapped 1D linear rotational spring model.
</html>"));
        protected 
          Bonds.eBond B1  annotation (extent=[-30,30; -10,50]);
          Bonds.fBond B2  annotation (extent=[10,30; 30,50]);
          Junctions.J0p3 j0p3_1  annotation (extent=[-10,30; 10,50]);
          BondLib.Sources.mSe Tx 
            annotation (extent=[-10,-10; 10,10], rotation=-90);
          Bonds.fBond B3  annotation (extent=[-10,10; 10,30], rotation=
                90);
          Modelica.Blocks.Math.Gain G1(k=c) 
            annotation (extent=[-10,-40; 10,-20], rotation=90);
          Modelica.Blocks.Math.Add3 A1(k3=-1) 
            annotation (extent=[-30,-70; -10,-50]);
          Modelica.Blocks.Sources.Constant C1(k=phi_rel0) 
            annotation (extent=[-80,-70; -60,-50]);
          Interfaces.Rot2BG Rot2BG annotation (extent=[-70,-10; -50,10]);
          Interfaces.BG2Rot BG2Rot annotation (extent=[50,-10; 70,10]);
        equation 
          phi_rel = BG2Rot.phi - Rot2BG.phi;
          w_rel = Tx.f;
          tau = -G1.y;
          connect(j0p3_1.BondCon1,B1. eBondCon1) annotation (points=[-10,40;
                -10,40],
                    style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_1.BondCon2,B2. eBondCon1) annotation (points=[10,40; 10,
                40],
              style(color=8, rgbcolor={192,192,192}));
          connect(Tx.BondCon1,B3. eBondCon1) annotation (points=[-6.12303e-016,
                10; -6.12303e-016,10], style(color=8, rgbcolor={192,192,192}));
          connect(B3.fBondCon1,j0p3_1. BondCon3) annotation (points=[
                6.12303e-016,30; 0,30], style(color=8, rgbcolor={192,192,192}));
          connect(A1.y,G1. u) annotation (points=[-9,-60; -7.34764e-016,-60;
                -7.34764e-016,-42], style(color=74, rgbcolor={0,0,127}));
          connect(C1.y,A1. u2) annotation (points=[-59,-60; -32,-60], style(
                color=74, rgbcolor={0,0,127}));
          connect(Rot2BG.phi, A1.u1) annotation (points=[-60,-10; -60,-30; -44,
                -30; -44,-52; -32,-52], style(color=3, rgbcolor={0,0,255}));
          connect(Rot2BG.BondCon1, B1.fBondCon1) annotation (points=[-50,0; -40,
                0; -40,40; -30,40], style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.phi, A1.u3) annotation (points=[60,-10; 60,-80; -44,
                -80; -44,-68; -32,-68], style(color=3, rgbcolor={0,0,255}));
          connect(BG2Rot.BondCon1, B2.fBondCon1) annotation (points=[50,0; 40,0;
                40,40; 30,40], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(G1.y, Tx.s) annotation (points=[6.73533e-016,-19;
                6.73533e-016,-14.5; 5.51073e-016,-14.5; 5.51073e-016,-9], style(
                color=74, rgbcolor={0,0,127}));
        end Spring;
        
        model Inertia "1D-rotational component with inertia" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Inertia J=1 "Moment of inertia";
          parameter Modelica.SIunits.Length phi0=0 
            "Initial angle of Inertia center";
          Modelica.SIunits.Angle phi(stateSelect=StateSelect.prefer) 
            "Absolute angle of inertia";
          Modelica.SIunits.AngularVelocity w(stateSelect=StateSelect.prefer) 
            "Absolute angular velocity of inertia";
          Modelica.SIunits.AngularAcceleration a 
            "Absolute angular acceleration of inertia";
        protected 
          Bonds.eBond B2  annotation (extent=[10,-10; 30,10]);
          Bonds.fBond B1  annotation (extent=[-30,-10; -10,10]);
          Bonds.eBond B4 
            annotation (extent=[-10,-30; 10,-10],
                                                rotation=270);
          Bonds.fBond B3 
            annotation (extent=[-10,10; 10,30], rotation=90);
          Junctions.J1p4 j1p4_1  annotation (extent=[-10,-10; 10,10]);
          BondLib.Passive.I I(I=J) 
            annotation (extent=[-10,30; 10,50], rotation=90);
          BondLib.Sensors.Dq sAbs(q0=phi0) 
            annotation (extent=[-10,-50; 10,-30],rotation=-90);
          BondLib.Mechanical.Rotational.Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,-10; -50,10]);
          BondLib.Mechanical.Rotational.Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,-10; 70,10]);
        equation 
          connect(B1.fBondCon1,j1p4_1. BondCon1) annotation (points=[-10,0; -10,
                0],      style(color=8, rgbcolor={192,192,192}));
          connect(j1p4_1.BondCon2,B2. fBondCon1) annotation (points=[10,0; 10,0],
                     style(color=8, rgbcolor={192,192,192}));
          connect(j1p4_1.BondCon4,B3. eBondCon1) annotation (points=[0,10;
                -6.12303e-016,10], style(color=8, rgbcolor={192,192,192}));
          connect(I.BondCon1,B3. fBondCon1) annotation (points=[-6.12303e-016,
                30; 6.12303e-016,30], style(color=8, rgbcolor={192,192,192}));
          connect(B4.fBondCon1,j1p4_1. BondCon3) annotation (points=[
                1.83691e-015,-10; 0,-10],
                                        style(color=8, rgbcolor={192,192,192}));
          connect(sAbs.BondCon1,B4. eBondCon1) annotation (points=[
                -6.12303e-016,-30; -1.83691e-015,-30],
                                                     style(color=8, rgbcolor={
                  192,192,192}));
          annotation (
            Diagram,
            Icon(
              Rectangle(extent=[-100,10; -50,-10],  style(
                  color=0,
                  gradient=2,
                  fillColor=8)),
              Rectangle(extent=[50,10; 100,-10],  style(
                  color=0,
                  gradient=2,
                  fillColor=8)),
              Line(points=[-80,-25; -60,-25],   style(color=0)),
              Line(points=[60,-25; 80,-25],   style(color=0)),
              Line(points=[-70,-25; -70,-70],   style(color=0)),
              Line(points=[70,-25; 70,-70],   style(color=0)),
              Line(points=[-80,25; -60,25],   style(color=0)),
              Line(points=[60,25; 80,25],   style(color=0)),
              Line(points=[-70,45; -70,25],   style(color=0)),
              Line(points=[70,45; 70,25],   style(color=0)),
              Line(points=[-70,-70; 70,-70],   style(color=0)),
              Rectangle(extent=[-50,50; 50,-50],   style(
                  color=0,
                  gradient=2,
                  fillColor=8)),
              Text(extent=[0,120; 0,60],   string="%name"),
              Text(
                extent=[0,-80; 0,-130],
                string="J=%J",
                style(color=0))),
            Documentation(info="<html>
Wrapped 1D linear rotational sliding mass model.
</html>"));
          phi = sAbs.OutPort1;
          w = I.f;
          a = I.e/J;
          connect(Rot2BG.BondCon1, B1.eBondCon1) annotation (points=[-50,0; -30,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.BondCon1, B2.eBondCon1) annotation (points=[50,0; 30,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(sAbs.OutPort1, Rot2BG.phi) annotation (points=[6.73533e-016,
                -51; 6.73533e-016,-70; -60,-70; -60,-10], style(color=74,
                rgbcolor={0,0,127}));
          connect(sAbs.OutPort1, BG2Rot.phi) annotation (points=[6.73533e-016,
                -51; 6.73533e-016,-70; 60,-70; 60,-10], style(color=74,
                rgbcolor={0,0,127}));
        end Inertia;
        
        model SpringDamper "Linear 1D rotational spring and damper in parallel" 
          extends Interfaces.OnePort;
          parameter Modelica.SIunits.Angle phi_rel0=0 
            "Unstretched spring angle";
          parameter Real c(final unit="N.m/rad", final min=0) "Spring constant";
          parameter Real d(
            final unit="N.m.s/rad",
            final min=0) = 0 "Damping constant";
          Modelica.SIunits.AngularVelocity w_rel 
            "Relative angular velocity between flange_a and flange_b";
          Modelica.SIunits.Torque tau 
            "Friction torque (positive, if directed in opposite direction of w_rel)";
          Spring spring(c=c, phi_rel0=phi_rel0) 
            annotation (extent=[-20,20; 20,60]);
          Damper damper(d=d) annotation (extent=[-20,-60; 20,-20]);
          annotation (
            Diagram,
            Documentation(info="<html>
A spring and a damper connected in parallel.
Since this is such a frequently used combination, a separate model was created for it.
</html>"),  Icon(
              Line(points=[-80,40; -60,40; -45,10; -15,70; 15,10; 45,70; 60,40;
                    80,40],   style(color=0)),
              Line(points=[-80,40; -80,-70],   style(color=0)),
              Line(points=[-80,-70; -52,-70],   style(color=0)),
              Rectangle(extent=[-52,-40; 38,-100],   style(color=0, fillColor=8)),
              Line(points=[-52,-40; 68,-40],   style(color=0)),
              Line(points=[-52,-100; 68,-100],   style(color=0)),
              Line(points=[38,-70; 80,-70],   style(color=0)),
              Line(points=[80,40; 80,-70],   style(color=0)),
              Line(points=[-90,0; -80,0],   style(color=0)),
              Line(points=[80,0; 90,0],   style(color=0)),
              Text(extent=[0,130; 0,70],   string="%name=%c"),
              Text(
                extent=[-107,-149; 92,-109],
                string="d=%d",
                style(color=0))));
        equation 
          w_rel = flange_b.w - flange_a.w;
          tau = flange_b.tau;
          connect(spring.flange_a, flange_a) annotation (points=[-20,40; -60,40;
                -60,0; -100,0], style(color=0, rgbcolor={0,0,0}));
          connect(damper.flange_a, flange_a) annotation (points=[-20,-40; -60,
                -40; -60,0; -100,0], style(color=0, rgbcolor={0,0,0}));
          connect(spring.flange_b, flange_b) annotation (points=[20,40; 60,40;
                60,0; 100,0], style(color=0, rgbcolor={0,0,0}));
          connect(damper.flange_b, flange_b) annotation (points=[20,-40; 60,-40;
                60,0; 100,0], style(color=0, rgbcolor={0,0,0}));
        end SpringDamper;
        
        model IdealGear "Ideal gear without inertia" 
          extends Interfaces.OnePort;
          parameter Real ratio=1 
            "Transmission ratio (flange_a.phi/flange_b.phi)";
          annotation (
            Diagram(
        Rectangle(extent=[-80,-30; -60,-50],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-30; -60,-50],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-60,-40; -52,-40], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=8)),
              Rectangle(extent=[-40,20; -20,-20],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-40,140; -20,20],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[20,100; 40,60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[20,60; 40,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[40,10; 100,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-20,90; 20,70],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-100,10; -40,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[-80,20; -60,20],   style(color=0)),
              Line(points=[-80,-20; -60,-20],   style(color=0)),
              Line(points=[-70,-20; -70,-70],   style(color=0)),
              Line(points=[0,60; 0,-90],   style(color=0)),
              Line(points=[-10,60; 10,60],   style(color=0)),
              Line(points=[-10,100; 10,100],   style(color=0)),
              Line(points=[60,-20; 80,-20],   style(color=0)),
              Line(points=[60,20; 80,20],   style(color=0)),
              Line(points=[70,-20; 70,-70],   style(color=0)),
              Line(points=[70,-70; -70,-70],   style(color=0)),
              Text(extent=[10,200; 10,140], string="%name=%ratio")),
            Documentation(info="<html>
<p>
This element characterices any type of gear box which is fixed in the
ground and which has one driving shaft and one driven shaft.
The gear is <b>ideal</b>, i.e., it does not have inertia, elasticity, damping
or backlash. If these effects have to be considered, the gear has to be
connected to other elements in an appropriate way.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</html>"));
          Interfaces.Flange_a bearing "Bearing rotational connector" 
                            annotation (extent=[-10,-90; 10,-110]);
        protected 
          Interfaces.Rot2BG Rot2BG annotation (extent=[-90,40; -70,20]);
          Interfaces.Rot2BG Rot2BG1 
            annotation (extent=[-10,-90; 10,-70], rotation=90);
          Interfaces.BG2Rot BG2Rot annotation (extent=[70,40; 90,20]);
          Bonds.Bond B1 annotation (extent=[-70,20; -50,40]);
          Junctions.J0p3 j0p3_1 annotation (extent=[-50,20; -30,40]);
          Bonds.Bond B2 annotation (extent=[-30,20; -10,40]);
          BondLib.Passive.TF TF1(m=1/ratio) annotation (extent=[-10,20; 10,40]);
          Bonds.Bond B3 annotation (extent=[10,20; 30,40]);
          Junctions.J0p3 j0p3_2 annotation (extent=[30,20; 50,40]);
          Bonds.Bond B4 annotation (extent=[50,20; 70,40]);
          BondLib.Junctions.J1p3 j1p3_1 annotation (extent=[-10,-10; 10,10]);
          BondLib.Bonds.fBond B5 annotation (extent=[-30,-10; -10,10]);
          BondLib.Bonds.eBond B7 
            annotation (extent=[-10,-30; 10,-10], rotation=90);
          BondLib.Bonds.eBond B6 
            annotation (extent=[10,-10; 30,10],  rotation=0);
          BondLib.Mechanical.Rotational.Utilities.TFangle TF2(m=1/ratio) 
            annotation (extent=[-10,60; 10,80]);
          BondLib.Mechanical.Rotational.Utilities.Add Add1(k2=-1) 
            annotation (extent=[-50,60; -30,80]);
          BondLib.Mechanical.Rotational.Utilities.Add Add2 
            annotation (extent=[30,60; 50,80]);
        public 
          Junctions.J1p3 j1p3_2 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          Bonds.Bond B8 annotation (extent=[-30,-50; -10,-30]);
          Bonds.Bond B9 annotation (extent=[-10,-70; 10,-50], rotation=90);
          Switches.Sw Sw1 annotation (extent=[-30,-50; -50,-30]);
        equation 
          if cardinality(bearing) <= 1 then
            bearing.phi = 0;
            Sw1.BooleanInPort1 = true;
          else
            Sw1.BooleanInPort1 = false;
          end if;
          connect(Rot2BG.Rot, flange_a) annotation (points=[-90,30; -94,30; -94,
                0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[90,30; 94,30; 94,0;
                100,0],                                                   style(
                color=0, rgbcolor={0,0,0}));
          connect(Rot2BG1.Rot, bearing) annotation (points=[-6.12303e-016,-90;
                0,-90; 0,-100], style(color=0, rgbcolor={0,0,0}));
          connect(B1.BondCon1, Rot2BG.BondCon1) annotation (points=[-69.8,30;
                -70,30],style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_1.BondCon1, B1.BondCon2) annotation (points=[-50,30; -50,
                30],style(color=8, rgbcolor={192,192,192}));
          connect(B2.BondCon1, j0p3_1.BondCon2) annotation (points=[-29.8,30;
                -30,30],style(color=8, rgbcolor={192,192,192}));
          connect(TF1.BondCon1, B2.BondCon2) annotation (points=[-10,30; -10,30],
              style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon1, TF1.BondCon2) annotation (points=[10.2,30; 10,30],
              style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_2.BondCon1, B3.BondCon2) annotation (points=[30,30; 30,
                30],
              style(color=8, rgbcolor={192,192,192}));
          connect(B4.BondCon1, j0p3_2.BondCon2) annotation (points=[50.2,30; 50,
                30],style(color=8, rgbcolor={192,192,192}));
          connect(B4.BondCon2, BG2Rot.BondCon1) annotation (points=[70,30; 70,
                30],
              style(color=8, rgbcolor={192,192,192}));
          connect(B5.fBondCon1, j1p3_1.BondCon1) annotation (points=[-10,0; -10,
                0],       style(color=8, rgbcolor={192,192,192}));
          connect(B5.eBondCon1, j0p3_1.BondCon3) annotation (points=[-30,0; -40,
                0; -40,20],        style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_1.BondCon3,B7. eBondCon1) annotation (points=[0,-10;
                6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_1.BondCon2,B6. fBondCon1) annotation (points=[10,0; 10,0],
                      style(color=8, rgbcolor={192,192,192}));
          connect(B6.eBondCon1, j0p3_2.BondCon3) annotation (points=[30,0; 40,0;
                40,20],       style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.phi, Add1.u1) annotation (points=[-80,40; -80,76;
                -50.8,76], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Add1.y, TF2.phi_a) annotation (points=[-29.2,70; -10.8,70],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(TF2.phi_b, Add2.u1) annotation (points=[10.8,70; 20,70; 20,76;
                29.2,76], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Add2.y, BG2Rot.phi) annotation (points=[50.8,70; 80,70; 80,40],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Rot2BG1.phi, Add2.u2) annotation (points=[10,-80; 60,-80; 60,
                50; 20,50; 20,64; 29.2,64], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Add1.u2, Add2.u2) annotation (points=[-50.8,64; -60,64; -60,
                50; 20,50; 20,64; 29.2,64], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B8.BondCon2,j1p3_2. BondCon3) annotation (points=[-10,-40;
                -10,-40], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_2.BondCon2,B9. BondCon2) annotation (points=[
                6.12303e-016,-50; 6.12303e-016,-50], style(color=8, rgbcolor={
                  192,192,192}));
          connect(Sw1.BondCon1,B8. BondCon1) annotation (points=[-30,-40; -29.8,
                -40], style(color=8, rgbcolor={192,192,192}));
          connect(B7.fBondCon1, j1p3_2.BondCon1) annotation (points=[
                -6.12303e-016,-30; -6.12303e-016,-30], style(color=8, rgbcolor=
                  {192,192,192}));
          connect(B9.BondCon1, Rot2BG1.BondCon1) annotation (points=[
                -6.00057e-016,-69.8; 0,-69.8; 0,-70; 6.12303e-016,-70], style(
                color=8, rgbcolor={192,192,192}));
        end IdealGear;
        
        model IdealPlanetary "Ideal planetary gear box" 
          parameter Real ratio=100/50 
            "number of ring_teeth/sun_teeth (e.g. ratio=100/50)";
          Interfaces.Flange_a sun 
            "sun flange (flange axis directed INTO cut plane)" 
            annotation (extent=[-110,-10; -90,10]);
          Interfaces.Flange_a carrier 
            "carrier flange (flange axis directed INTO cut plane)" 
            annotation (extent=[-110,30; -90,50]);
          Interfaces.Flange_b ring 
            "ring flange (flange axis directed OUT OF cut plane)" 
            annotation (extent=[90,-10; 110,10]);
          annotation (
            Icon(
              Rectangle(extent=[50,100; 10,-100],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-10,45; -50,85],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-10,30; -50,-30],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-50,10; -100,-10],  style(
                  color=8,
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[100,10; 50,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[50,100; -50,105],   style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Rectangle(extent=[50,-100; -50,-105],   style(
                  color=9,
                  fillColor=9,
                  fillPattern=1)),
              Rectangle(extent=[-80,70; -50,60],   style(
                  color=0,
                  gradient=2,
                  arrow=1,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[-90,40; -70,40],   style(color=0)),
              Line(points=[-80,50; -60,50],   style(color=0)),
              Line(points=[-70,50; -70,40],   style(color=0)),
              Line(points=[-80,80; -59,80],   style(color=0)),
              Line(points=[-70,100; -70,80],   style(color=0)),
              Text(extent=[0,168; 0,108],   string="%name"),
              Text(
                extent=[0,-112; 0,-163],
                string="ratio=%ratio",
                style(color=0))),
            Diagram,
            Documentation(info="<html>
<p>
The IdealPlanetary gear box is an ideal gear without inertia,
elasticity, damping or backlash consisting
of an inner <b>sun</b> wheel, an outer <b>ring</b> wheel and a
<b>planet</b> wheel located between sun and ring wheel. The bearing
of the planet wheel shaft is fixed in the planet <b>carrier</b>.
The component can be connected to other elements at the
sun, ring and/or carrier flanges. It is not possible to connect
to the planet wheel. If inertia shall not be neglected,
the sun, ring and carrier inertias can be easily added by attaching
inertias (= model Inertia) to the corresponding connectors.
The inertias of the planet wheels are always neglected.
</p>
<p>
The icon of the planetary gear signals that the sun and carrier
flanges are on the left side and the ring flange is on the right side
of the gear box. However, this component is generic and is valid
independantly how the flanges are actually placed (e.g. sun wheel
may be placed on the right side instead on the left side in reality).
</p>
<p>
The ideal planetary gearbox is uniquely defined by the ratio
of the number of ring teeth zr with respect to the number of
sun teeth zs. For example, if there are 100 ring teeth and
50 sun teeth then ratio = zr/zs = 2. The number of planet teeth
zp has to fulfill the following relationship:
</p>
<pre>
   <b>zp := (zr - zs) / 2</b>
</pre>
<p>
Therefore, in the above example zp = 25 is required.
</p>
<p>
According to the overall convention, the positive direction of all
vectors, especially the absolute angular velocities and cut-torques
in the flanges, are along the axis vector displayed in the icon.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 5, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</html>"));
        protected 
          Interfaces.Rot2BG Rot2BG annotation (extent=[-90,10; -70,-10]);
          Interfaces.Rot2BG Rot2BG1 
                                   annotation (extent=[-90,50; -70,30]);
          Interfaces.BG2Rot BG2Rot annotation (extent=[70,50; 90,30]);
          BondLib.Passive.TF TF1(m=1 + ratio) 
                                            annotation (extent=[-50,30; -30,50]);
          BondLib.Bonds.Bond B1 annotation (extent=[-70,30; -50,50]);
          BondLib.Bonds.Bond B2 annotation (extent=[-30,30; -10,50]);
          BondLib.Junctions.J0p3 j0p3_1 
                                annotation (extent=[-10,30; 10,50]);
          BondLib.Bonds.Bond B3 annotation (extent=[10,30; 30,50]);
          BondLib.Passive.TF TF2(m=1/ratio) annotation (extent=[30,30; 50,50]);
          BondLib.Bonds.Bond B4 annotation (extent=[50,30; 70,50]);
          BondLib.Bonds.Bond B5 annotation (extent=[-10,10; 10,30], rotation=90);
          BondLib.Mechanical.Rotational.Utilities.TFangle TF4(m=1/ratio) 
            annotation (extent=[30,60; 50,80]);
          BondLib.Mechanical.Rotational.Utilities.TFangle TF3(m=1 + ratio) 
            annotation (extent=[-50,60; -30,80]);
          BondLib.Mechanical.Rotational.Utilities.Add Add1 
            annotation (extent=[-10,60; 10,80]);
        equation 
          connect(Rot2BG.Rot, sun) annotation (points=[-90,0; -100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(Rot2BG1.Rot, carrier) annotation (points=[-90,40; -100,40],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, ring) annotation (points=[90,40; 94,40; 94,0; 100,
                0], style(color=0, rgbcolor={0,0,0}));
          connect(B1.BondCon1, Rot2BG1.BondCon1) annotation (points=[-69.8,40;
                -70,40], style(color=8, rgbcolor={192,192,192}));
          connect(TF1.BondCon1, B1.BondCon2) annotation (points=[-50,40; -50,40],
              style(color=8, rgbcolor={192,192,192}));
          connect(B2.BondCon1, TF1.BondCon2) annotation (points=[-29.8,40; -30,
                40], style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_1.BondCon1, B2.BondCon2) annotation (points=[-10,40; -10,
                40], style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon1, j0p3_1.BondCon2) annotation (points=[10.2,40; 10,
                40], style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon2, TF2.BondCon1) annotation (points=[30,40; 30,40],
              style(color=8, rgbcolor={192,192,192}));
          connect(B4.BondCon2, BG2Rot.BondCon1) annotation (points=[70,40; 70,
                40], style(color=8, rgbcolor={192,192,192}));
          connect(B4.BondCon1, TF2.BondCon2) annotation (points=[50.2,40; 50,40],
              style(color=8, rgbcolor={192,192,192}));
          connect(B5.BondCon2, j0p3_1.BondCon3) annotation (points=[
                6.12303e-016,30; 0,30], style(color=8, rgbcolor={192,192,192}));
          connect(B5.BondCon1, Rot2BG.BondCon1) annotation (points=[
                -6.00057e-016,10.2; -6.00057e-016,0; -70,0], style(color=8,
                rgbcolor={192,192,192}));
          connect(Rot2BG1.phi, TF3.phi_a) annotation (points=[-80,50; -80,70;
                -50.8,70], style(color=3, rgbcolor={0,0,255}));
          connect(TF3.phi_b, Add1.u1) annotation (points=[-29.2,70; -20,70; -20,
                76; -10.8,76], style(color=3, rgbcolor={0,0,255}));
          connect(Rot2BG.phi, Add1.u2) annotation (points=[-80,10; -80,20; -20,
                20; -20,64; -10.8,64], style(color=3, rgbcolor={0,0,255}));
          connect(Add1.y, TF4.phi_a) annotation (points=[10.8,70; 29.2,70],
              style(color=3, rgbcolor={0,0,255}));
          connect(TF4.phi_b, BG2Rot.phi) annotation (points=[50.8,70; 80,70; 80,
                50], style(color=3, rgbcolor={0,0,255}));
        end IdealPlanetary;
        
        model IdealGearR2T 
          "Gearbox transforming rotational into translational motion" 
          parameter Real ratio(final unit="rad/m") = 1 
            "transmission ratio (flange_a.phi/flange_b.s)";
          
          Interfaces.Flange_a flange_a "Rotational connector" 
                                       annotation (extent=[-110,-10; -90,10]);
          annotation (
            Diagram(
        Rectangle(extent=[-10,-80; 10,-100],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-10,-80; 10,-100],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[0,-80; 0,-72], style(color=83, rgbcolor={255,85,255})),
        Rectangle(extent=[70,-80; 90,-100], style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[70,-80; 90,-100],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[80,-80; 80,-72], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Ellipse(extent=[-70,40; 10,-40],   style(
                  color=0,
                  fillColor=7,
                  fillPattern=1)),
              Ellipse(extent=[-40,10; -20,-10],   style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Rectangle(extent=[-100,10; -70,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Text(extent=[0,110; 0,50],   string="%name=%ratio"),
              Polygon(points=[-74,-60; -54,-40; -34,-60; -14,-40; 6,-60; 26,-40;
                    46,-60; 66,-40; 86,-60; -74,-60],      style(
                  color=0,
                  gradient=0,
                  fillColor=9,
                  fillPattern=1)),
              Rectangle(extent=[-74,-60; 106,-80],   style(
                  color=0,
                  gradient=0,
                  fillColor=9,
                  fillPattern=1)),
              Rectangle(extent=[95,-10; 106,-60],   style(color=0, fillColor=9)),
              Rectangle(extent=[-80,-80; -40,-120],   style(color=8, fillColor=8)),
              Rectangle(extent=[40,-80; 80,-120],   style(color=8, fillColor=8))),
            Documentation(info="<html>
This is an ideal mass- and inertialess gearbox which transforms a
1D-rotational into a 1D-translational motion. If elasticity, damping
or backlash has to be considered, this ideal gearbox has to be
connected with corresponding elements.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>May 16, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</html>"));
          Interfaces.Flange_a bearingR "Bearing rotational connector" 
                                       annotation (extent=[-70,-90; -50,-110]);
          Translational.Interfaces.Flange_b flange_b "Translational connector" 
            annotation (extent=[90,-10; 110,10]);
          Translational.Interfaces.Flange_a bearingT 
            "Bearing translational connector" 
            annotation (extent=[50,-90; 70,-110]);
        protected 
          Interfaces.Rot2BG Rot2BG annotation (extent=[-90,10; -70,-10]);
          Bonds.Bond B1 annotation (extent=[-70,-10; -50,10]);
          Junctions.J0p3 j0p3_1 annotation (extent=[-50,-10; -30,10]);
          Bonds.Bond B2 annotation (extent=[-30,-10; -10,10]);
          BondLib.Passive.TF TF1(m=1/ratio) annotation (extent=[-10,-10; 10,10]);
          BondLib.Bonds.Bond B3 
                        annotation (extent=[10,-10; 30,10]);
          BondLib.Junctions.J0p3 j0p3_2 
                                annotation (extent=[30,-10; 50,10]);
          BondLib.Bonds.Bond B4 
                        annotation (extent=[50,-10; 70,10]);
          BondLib.Mechanical.Translational.Interfaces.BG2Tr BG2Tr 
            annotation (extent=[70,10; 90,-10]);
          BondLib.Bonds.eBond B6 
            annotation (extent=[30,-30; 50,-10],  rotation=90);
          BondLib.Bonds.eBond B5 
            annotation (extent=[-50,-30; -30,-10],rotation=90);
          Interfaces.Rot2BG Rot2BG1 
                                   annotation (extent=[-50,-70; -30,-90],
              rotation=-90);
          BondLib.Mechanical.Translational.Interfaces.Tr2BG Tr2BG 
            annotation (extent=[50,-90; 30,-70], rotation=90);
          BondLib.Mechanical.Rotational.Utilities.Add Add1 
            annotation (extent=[-50,30; -30,50]);
          Utilities.TFangpos TF2(m=1/ratio) annotation (extent=[-10,30; 10,50]);
          Translational.Utilities.Add Add2 annotation (extent=[30,30; 50,50]);
        public 
          Junctions.J1p3 j1p3_2 
            annotation (extent=[-50,-50; -30,-30],rotation=90);
          Bonds.Bond B7 annotation (extent=[-50,-70; -30,-50],rotation=90);
          Bonds.Bond B8 annotation (extent=[-30,-50; -10,-30],rotation=180);
          Switches.Sw Sw1 annotation (extent=[10,-50; -10,-70], rotation=90);
        public 
          Junctions.J1p3 j1p3_1 
            annotation (extent=[30,-50; 50,-30],  rotation=90);
          Bonds.Bond B9 annotation (extent=[30,-70; 50,-50],  rotation=90);
          Bonds.Bond B10 
                        annotation (extent=[50,-50; 70,-30],  rotation=180);
          Switches.Sw Sw2 annotation (extent=[90,-50; 70,-70], rotation=90);
        equation 
          if cardinality(bearingR) <= 1 then
            bearingR.phi = 0;
            Sw1.BooleanInPort1 = true;
          else
            Sw1.BooleanInPort1 = false;
          end if;
          
          if cardinality(bearingT) <= 1 then
            bearingT.s = 0;
            Sw2.BooleanInPort1 = true;
          else
            Sw2.BooleanInPort1 = false;
          end if;
          connect(B1.BondCon1,Rot2BG. BondCon1) annotation (points=[-69.8,0;
                -70,0], style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_1.BondCon1,B1. BondCon2) annotation (points=[-50,0; -50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(B2.BondCon1,j0p3_1. BondCon2) annotation (points=[-29.8,0;
                -30,0], style(color=8, rgbcolor={192,192,192}));
          connect(TF1.BondCon1,B2. BondCon2) annotation (points=[-10,0; -10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon1,TF1. BondCon2) annotation (points=[10.2,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_2.BondCon1,B3. BondCon2) annotation (points=[30,0; 30,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-90,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(j0p3_2.BondCon2, B4.BondCon1) annotation (points=[50,0; 50.2,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(BG2Tr.BondCon1, B4.BondCon2) annotation (points=[70,0; 70,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Tr.Tr, flange_b) annotation (points=[89.9,0.1; 92.95,0.1;
                92.95,0; 100,0], style(color=58, rgbcolor={0,127,0}));
          connect(B6.eBondCon1, j0p3_2.BondCon3) annotation (points=[40,-10; 40,
                -10], style(color=8, rgbcolor={192,192,192}));
          connect(B5.eBondCon1, j0p3_1.BondCon3) annotation (points=[-40,-10;
                -40,-10], style(color=8, rgbcolor={192,192,192}));
          connect(bearingR, Rot2BG1.Rot) annotation (points=[-60,-100; -60,-94;
                -40,-94; -40,-90], style(color=0, rgbcolor={0,0,0}));
          connect(bearingT, Tr2BG.Tr) annotation (points=[60,-100; 60,-94; 40,
                -94; 40,-90.1; 39.9,-90.1],
                                  style(color=58, rgbcolor={0,127,0}));
          connect(Rot2BG.phi, Add1.u1) annotation (points=[-80,10; -80,46;
                -50.8,46], style(color=3, rgbcolor={0,0,255}));
          connect(Rot2BG1.phi, Add1.u2) annotation (points=[-50,-80; -60,-80;
                -60,34; -50.8,34], style(color=3, rgbcolor={0,0,255}));
          connect(Add1.y, TF2.phi_a) annotation (points=[-29.2,40; -10.8,40],
              style(color=3, rgbcolor={0,0,255}));
          connect(TF2.s_b, Add2.u1) annotation (points=[10.8,40; 20,40; 20,46;
                29.2,46], style(color=3, rgbcolor={0,0,255}));
          connect(Tr2BG.s, Add2.u2) annotation (points=[30,-80; 20,-80; 20,34;
                29.2,34], style(color=3, rgbcolor={0,0,255}));
          connect(Add2.y, BG2Tr.s) annotation (points=[50.8,40; 80,40; 80,10],
              style(color=3, rgbcolor={0,0,255}));
          connect(j1p3_2.BondCon2, B5.fBondCon1) annotation (points=[-40,-30;
                -40,-30], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_2.BondCon1, B7.BondCon2) annotation (points=[-40,-50;
                -40,-50], style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG1.BondCon1, B7.BondCon1) annotation (points=[-40,-70;
                -40,-69.8], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_2.BondCon3, B8.BondCon2) annotation (points=[-30,-40;
                -30,-40], style(color=8, rgbcolor={192,192,192}));
          connect(B8.BondCon1, Sw1.BondCon1) annotation (points=[-10.2,-40;
                6.12303e-016,-40; 6.12303e-016,-50], style(color=8, rgbcolor={
                  192,192,192}));
          connect(j1p3_1.BondCon1, B9.BondCon2) annotation (points=[40,-50; 40,
                -50], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_1.BondCon3, B10.BondCon2) annotation (points=[50,-40; 50,
                -40], style(color=8, rgbcolor={192,192,192}));
          connect(B6.fBondCon1, j1p3_1.BondCon2) annotation (points=[40,-30; 40,
                -30], style(color=8, rgbcolor={192,192,192}));
          connect(B9.BondCon1, Tr2BG.BondCon1) annotation (points=[40,-69.8; 40,
                -70], style(color=8, rgbcolor={192,192,192}));
          connect(Sw2.BondCon1, B10.BondCon1) annotation (points=[80,-50; 80,
                -40; 69.8,-40], style(color=8, rgbcolor={192,192,192}));
        end IdealGearR2T;
        
        model Clutch "Clutch based on Coulomb friction" 
          extends Interfaces.OnePort;
          parameter Real mue_pos[:, :]=[0, 0.5] 
            "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min=1) = 1 
            "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Modelica.SIunits.Position cgeo(final min=0) = 1 
            "Geometry constant containing friction distribution assumption";
          parameter Modelica.SIunits.Force fn_max(final min=0) = 1 
            "Maximum normal force";
          Modelica.SIunits.Distance phi_rel 
            "Relative rotation angle (= flange_b.phi - flange_a.phi)";
          Modelica.SIunits.AngularVelocity w_rel 
            "Relative angular velocity (flange_b.w - flange_a.w)";
          Modelica.SIunits.AngularAcceleration a_rel 
            "Relative angular acceleration (flange_b.a - flange_a.a)";
          Modelica.SIunits.Torque tau 
            "Friction torque (positive, if directed in opposite direction of w_rel)";
          
          Modelica.Blocks.Interfaces.RealInput f_normalized 
            "Normalized force signal 0..1 (normal force = fn_max*f_normalized; clutch is engaged if > 0)"
                                                    annotation (extent=[10,90;
                -10,110], rotation=-90);
          annotation (
            Icon(
              Polygon(points=[-30,40; -60,50; -60,30; -30,40],     style(fillColor=3,
                     fillPattern=1)),
              Line(points=[0,90; -90,70; -90,40; -30,40],     style(
                  pattern=1,
                  thickness=1,
                  arrow=0)),
              Line(points=[0,90; 90,70; 90,40; 30,40]),
              Polygon(points=[30,40; 60,50; 60,30; 30,40],     style(fillColor=3,
                    fillPattern=1)),
              Rectangle(extent=[10,60; 30,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-30,60; -10,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[30,10; 100,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-100,10; -30,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Text(extent=[0,-130; 0,-70],   string="%name")),
            Diagram,
            Documentation(info="<html>
<p>
This component models a <b>clutch</b>, i.e., a component with
two flanges where friction is present between the two flanges
and these flanges are pressed together via a normal force.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter. Friction in the
clutch is modelled in the following way:
</p>
<p>
When the relative angular velocity is not zero, the friction torque is a
function of the velocity dependent friction coefficient  mue(w_rel) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w_rel) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w_rel),
    w_rel >= 0, is defined via table mue_pos (first column = w_rel,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the relative angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the relative
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the relative acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w_rel=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>
<br>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       realized first version.
</li>
</ul>
</html>"));
        protected 
          Bonds.eBond B1  annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2  annotation (extent=[10,-10; 30,10]);
          Bonds.eBond B3 
            annotation (extent=[-10,10; 10,30], rotation=90);
          BondLib.Junctions.J0p3 j0p3_1 annotation (extent=[-10,10; 10,-10]);
          BondLib.Mechanical.Rotational.Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,-10; -50,10]);
          BondLib.Mechanical.Rotational.Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,-10; 70,10]);
        public 
          BondLib.Mechanical.Rotational.Utilities.R_Fric R(
            mue_pos=mue_pos,
            peak=peak,
            cgeo=cgeo,
            fn_max=fn_max) annotation (extent=[-10,30; 10,50], rotation=90);
        equation 
          phi_rel = BG2Rot.phi - Rot2BG.phi;
          w_rel = -R.w_relfric;
          a_rel = -R.a_relfric;
          a_rel = der(w_rel);
          tau = -R.tau;
          connect(B1.eBondCon1,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0],      style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B2. eBondCon1) annotation (points=[10,0; 10,0],
                     style(
              color=8,
              rgbcolor={192,192,192},
              fillPattern=1));
          connect(Rot2BG.BondCon1,B1. fBondCon1) annotation (points=[-50,0; -30,
                0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B2.fBondCon1,BG2Rot. BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B3.fBondCon1, j0p3_1.BondCon3) annotation (points=[
                -6.12303e-016,10; 0,10], style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(R.BondCon1, B3.eBondCon1) annotation (points=[-6.12303e-016,
                30; 6.12303e-016,30], style(color=8, rgbcolor={192,192,192}));
          connect(R.f_normalized, f_normalized) annotation (points=[5.51073e-016,49;
                5.51073e-016,69.5; 0,69.5; 0,100], style(color=74, rgbcolor={0,
                  0,127}));
        end Clutch;
        
        model Brake "Brake based on Coulomb friction" 
          extends Interfaces.OnePort;
          parameter Real mue_pos[:, :]=[0, 0.5] 
            "[w,mue] positive sliding friction coefficient (w>=0)";
          parameter Real peak(final min=1) = 1 
            "peak*mue_pos[1,2] = maximum value of mue for w==0";
          parameter Modelica.SIunits.Position cgeo(final min=0) = 1 
            "Geometry constant containing friction distribution assumption";
          parameter Modelica.SIunits.Force fn_max(final min=0) = 1 
            "Maximum normal force";
          Modelica.SIunits.Distance phi 
            "Absolute rotation angle of flange_a and flange_b";
          Modelica.SIunits.AngularVelocity w 
            "Absolute angular velocity of flange_a and flange_b";
          Modelica.SIunits.AngularAcceleration a 
            "Absolute angular acceleration of flange_a and flange_b";
          Modelica.SIunits.Torque tau "Friction torque";
          Interfaces.Flange_a bearing "Bearing rotational connector" 
                            annotation (extent=[-10,-90; 10,-110]);
          Modelica.Blocks.Interfaces.RealInput f_normalized 
            "Normalized force signal 0..1 (normal force = fn_max*f_normalized; clutch is engaged if > 0)"
                                                    annotation (extent=[10,90;
                -10,110], rotation=-90);
          annotation (
            Diagram(
        Rectangle(extent=[-80,-30; -60,-50],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-30; -60,-50],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-60,-40; -52,-40], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(   Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=
                     8)),
              Polygon(points=[-37,-55; -37,-90; 37,-90; 37,-55; 33,-55; 33,-86;
                    -33,-86; -33,-55; -37,-55],     style(color=8, fillColor=8)),
              Text(
                extent=[14,-63; 82,-76],
                string="rotation axis",
                style(color=10)),
              Polygon(points=[0,-70; -20,-65; -20,-75; 0,-70],       style(color=10,
                    fillColor=10)),
              Line(points=[-90,-70; -19,-70],  style(color=10, fillColor=10)),
              Rectangle(extent=[-100,10; -20,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[20,10; 100,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[0,90; -80,70; -80,-40; -70,-40],     style(
                  pattern=1,
                  thickness=1,
                  arrow=0)),
              Rectangle(extent=[-20,60; 20,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[0,90; 80,70; 80,-40; 70,-40]),
              Polygon(points=[40,-40; 70,-30; 70,-50; 40,-40],     style(fillColor=3,
                     fillPattern=1)),
              Polygon(points=[-40,-40; -70,-30; -70,-50; -40,-40],     style(
                    fillColor=3, fillPattern=1)),
              Rectangle(extent=[-40,-25; -30,-55],   style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Rectangle(extent=[30,-25; 40,-55],   style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Text(extent=[10,-190; 10,-130], string="%name")),
            Documentation(info="<html>
<p>
This component models a <b>brake</b>, i.e., a component where a frictional
torque is acting between the housing and a flange and a controlled normal
force presses the flange to the housing in order to increase friction.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter.
Friction in the brake is modelled in the following way:
</p>
<p>
When the absolute angular velocity \"w\" is not zero, the friction torque
is a function of the velocity dependent friction coefficient  mue(w) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w),
    w >= 0, is defined via table mue_pos (first column = w,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the absolute angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the absolute
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the absolute acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>
<br>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       realized first version.
</li>
</ul>
</html>"));
        protected 
          Interfaces.Rot2BG Rot2BG annotation (extent=[-90,30; -70,10]);
          Interfaces.Rot2BG Rot2BG1 
            annotation (extent=[-10,-90; 10,-70], rotation=90);
          Interfaces.BG2Rot BG2Rot annotation (extent=[70,30; 90,10]);
          Bonds.Bond B1 annotation (extent=[-70,10; -50,30]);
          Junctions.J0p3 j0p3_1 annotation (extent=[-50,10; -30,30]);
          Bonds.Bond B2 annotation (extent=[-30,10; -10,30]);
          BondLib.Bonds.Bond B3 
                        annotation (extent=[10,10; 30,30]);
          BondLib.Junctions.J0p3 j0p3_2 
                                annotation (extent=[30,10; 50,30]);
          BondLib.Bonds.Bond B4 
                        annotation (extent=[50,10; 70,30]);
          BondLib.Junctions.J1p3 j1p3_1 annotation (extent=[-10,-10; 10,10]);
          BondLib.Bonds.fBond B5 annotation (extent=[-30,-10; -10,10]);
          BondLib.Bonds.eBond B7 
            annotation (extent=[-10,-30; 10,-10], rotation=90);
          BondLib.Bonds.eBond B6 
            annotation (extent=[10,-10; 30,10],  rotation=0);
        public 
          BondLib.Junctions.J1p3 j1p3_2 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          BondLib.Bonds.Bond B8 
                        annotation (extent=[-30,-50; -10,-30]);
          BondLib.Bonds.Bond B9 
                        annotation (extent=[-10,-70; 10,-50], rotation=90);
          BondLib.Switches.Sw Sw1 
                          annotation (extent=[-30,-50; -50,-30]);
        protected 
          BondLib.Junctions.J1p3 j1p3_3 annotation (extent=[-10,30; 10,10]);
          BondLib.Bonds.eBond B10 
            annotation (extent=[-10,30; 10,50], rotation=90);
        public 
          BondLib.Mechanical.Rotational.Utilities.R_Fric R(
            mue_pos=mue_pos,
            peak=peak,
            cgeo=cgeo,
            fn_max=fn_max) annotation (extent=[-10,50; 10,70], rotation=90);
        equation 
          phi = BG2Rot.phi;
          w = -R.w_relfric;
          a = -R.a_relfric;
          a = der(w);
          tau = R.tau;
          
          if cardinality(bearing) <= 1 then
            bearing.phi = 0;
            Sw1.BooleanInPort1 = true;
          else
            Sw1.BooleanInPort1 = false;
          end if;
          connect(B1.BondCon1,Rot2BG. BondCon1) annotation (points=[-69.8,20;
                -70,20],style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_1.BondCon1,B1. BondCon2) annotation (points=[-50,20; -50,
                20],style(color=8, rgbcolor={192,192,192}));
          connect(B2.BondCon1,j0p3_1. BondCon2) annotation (points=[-29.8,20;
                -30,20],style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_2.BondCon1,B3. BondCon2) annotation (points=[30,20; 30,
                20],
              style(color=8, rgbcolor={192,192,192}));
          connect(B4.BondCon1,j0p3_2. BondCon2) annotation (points=[50.2,20; 50,
                20],style(color=8, rgbcolor={192,192,192}));
          connect(B4.BondCon2,BG2Rot. BondCon1) annotation (points=[70,20; 70,
                20],
              style(color=8, rgbcolor={192,192,192}));
          connect(B5.fBondCon1,j1p3_1. BondCon1) annotation (points=[-10,0; -10,
                0],       style(color=8, rgbcolor={192,192,192}));
          connect(B5.eBondCon1,j0p3_1. BondCon3) annotation (points=[-30,0; -40,
                0; -40,10],        style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_1.BondCon3,B7. eBondCon1) annotation (points=[0,-10;
                6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_1.BondCon2,B6. fBondCon1) annotation (points=[10,0; 10,0],
                      style(color=8, rgbcolor={192,192,192}));
          connect(B6.eBondCon1,j0p3_2. BondCon3) annotation (points=[30,0; 40,0;
                40,10],       style(color=8, rgbcolor={192,192,192}));
          connect(B8.BondCon2,j1p3_2. BondCon3) annotation (points=[-10,-40;
                -10,-40], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_2.BondCon2,B9. BondCon2) annotation (points=[
                6.12303e-016,-50; 6.12303e-016,-50], style(color=8, rgbcolor={
                  192,192,192}));
          connect(Sw1.BondCon1,B8. BondCon1) annotation (points=[-30,-40; -29.8,
                -40], style(color=8, rgbcolor={192,192,192}));
          connect(B7.fBondCon1,j1p3_2. BondCon1) annotation (points=[
                -6.12303e-016,-30; -6.12303e-016,-30], style(color=8, rgbcolor=
                  {192,192,192}));
          connect(B9.BondCon1,Rot2BG1. BondCon1) annotation (points=[
                -6.00057e-016,-69.8; 0,-69.8; 0,-70; 6.12303e-016,-70], style(
                color=8, rgbcolor={192,192,192}));
          connect(Rot2BG1.Rot, bearing) annotation (points=[-6.12303e-016,-90;
                0,-90; 0,-100], style(color=0, rgbcolor={0,0,0}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-90,20; -94,20; -94,
                0; -100,0], style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[90,20; 94,20; 94,0;
                100,0], style(color=0, rgbcolor={0,0,0}));
          connect(j1p3_3.BondCon1, B2.BondCon2) annotation (points=[-10,20; -10,
                20], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_3.BondCon2, B3.BondCon1) annotation (points=[10,20; 10.2,
                20], style(color=8, rgbcolor={192,192,192}));
          connect(R.BondCon1, B10.eBondCon1) 
                                            annotation (points=[-6.12303e-016,
                50; 6.12303e-016,50], style(color=8, rgbcolor={192,192,192}));
          connect(B10.fBondCon1, j1p3_3.BondCon3) annotation (points=[
                -6.12303e-016,30; 0,30],
                           style(color=8, rgbcolor={192,192,192}));
          connect(R.f_normalized, f_normalized) annotation (points=[
                5.51073e-016,69; 5.51073e-016,84.5; 0,84.5; 0,100], style(color=
                 74, rgbcolor={0,0,127}));
          connect(Rot2BG.phi, BG2Rot.phi) annotation (points=[-80,30; -80,80;
                80,80; 80,30], style(color=3, rgbcolor={0,0,255}));
        end Brake;
      end Passive;
      
      package Sensors "Sensors of the 1D rotational library" 
        annotation (
          preferedView="info",
          Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))), Documentation(info="<html>
Sensors of the 1D rotational library.
</html>"));
        model TorqueSensor 
          "Ideal sensor to measure the torque between two flanges" 
          extends Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealOutput tau(redeclare type SignalType 
              = Modelica.SIunits.Torque) "Absolute torque" 
        annotation (extent=[-90,-100; -70,-120],   rotation=90);
          annotation (
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
              Text(
                extent=[-51,-78; 49,-119],
                string="tau",
                style(color=0)),
              Line(points=[-80,-100; -80,0]),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0],   style(color=0)),
              Text(extent=[150,80; -150,120],
                                           string="%name")),
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the torque between two flanges.
This element should be placed in series with another element to measure the torque within a connection.
</html>"));
        protected 
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,-30; 10,-10], rotation=-90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.De T 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          BondLib.Mechanical.Rotational.Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,10; -50,-10]);
          BondLib.Mechanical.Rotational.Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,10; 70,-10]);
        equation 
          connect(B1.BondCon2,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.eBondCon1,j0p3_1. BondCon3) annotation (points=[
                -6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B3. BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.fBondCon1,T. BondCon1) annotation (points=[6.12303e-016,
                -30; -6.12303e-016,-30], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(T.OutPort1, tau) annotation (points=[6.73533e-016,-51;
                6.73533e-016,-70; -80,-70; -80,-110], style(color=74, rgbcolor=
                  {0,0,127}));
          connect(Rot2BG.BondCon1, B1.BondCon1) annotation (points=[-50,0;
                -29.8,0], style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(B3.BondCon2, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(Rot2BG.phi, BG2Rot.phi) annotation (points=[-60,10; -60,40;
                60,40; 60,10], style(color=3, rgbcolor={0,0,255}));
        end TorqueSensor;
        
        model AngleSensor "Ideal sensor to measure the absolute flange angle" 
          
          annotation (
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[70,-30; 120,-80],
                string="phi",
                style(color=0)),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0]),
              Text(extent=[150,80; -150,120],
                                           string="%name")),
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the absolute angle.
This sensor element can be connected to any flange.
</html>"));
          Modelica.Blocks.Interfaces.RealOutput phi(redeclare type SignalType 
              = Modelica.SIunits.Angle) "Absolute angle" 
            annotation (extent=[90,-10; 110,10]);
          Interfaces.Flange_a flange_a "Rotational connector" 
                                       annotation (extent=[-110,-10; -90,10]);
        protected 
          Bonds.eBond B1 annotation (extent=[-40,-10; -20,10]);
          BondLib.Sensors.Df W annotation (extent=[-20,-10; 0,10]);
          BondLib.Mechanical.Rotational.Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,-10; -50,10]);
        equation 
          connect(W.BondCon1,B1. eBondCon1) annotation (points=[-20,0; -20,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.BondCon1, B1.fBondCon1) annotation (points=[-50,0; -40,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(Rot2BG.phi, phi) annotation (points=[-60,-10; -60,-20; 60,-20;
                60,0; 100,0], style(color=3, rgbcolor={0,0,255}));
        end AngleSensor;
        
        model SpeedSensor 
          "Ideal sensor to measure the absolute flange angular velocity" 
          
        protected 
          Interfaces.Rot2BG Rot2BG annotation (extent=[-70,-10; -50,10]);
        public 
          Interfaces.Flange_a flange_a "Rotational connector" 
                                       annotation (extent=[-110,-10; -90,10]);
          Modelica.Blocks.Interfaces.RealOutput w(redeclare type SignalType = 
                Modelica.SIunits.AngularVelocity) "Absolute angular velocity" 
            annotation (extent=[90,-10; 110,10]);
        protected 
          Bonds.eBond B1 annotation (extent=[-40,-10; -20,10]);
          BondLib.Sensors.Df W annotation (extent=[-20,-10; 0,10]);
        equation 
          connect(Rot2BG.Rot,flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          annotation (
            Diagram,
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[70,-30; 120,-80],
                style(color=0),
                string="w"),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0]),
              Text(extent=[150,80; -150,120],
                                           string="%name")),
            Documentation(info="<html>
Ideal sensor to measure the absolute angular velocity.
This sensor element can be connected to any flange.
</html>"));
          connect(W.BondCon1,B1. eBondCon1) annotation (points=[-20,0; -20,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B1.fBondCon1, Rot2BG.BondCon1) annotation (points=[-40,0; -50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(W.OutPort1, w) annotation (points=[1,0; 100,0], style(color=
                  74, rgbcolor={0,0,127}));
        end SpeedSensor;
        
        model AccSensor 
          "Ideal sensor to measure the absolute flange angular acceleration" 
          
        protected 
          Interfaces.Rot2BG Rot2BG annotation (extent=[-70,-10; -50,10]);
        public 
          Interfaces.Flange_a flange_a "Rotational connector" 
                                       annotation (extent=[-110,-10; -90,10]);
          Modelica.Blocks.Interfaces.RealOutput a(redeclare type SignalType = 
                Modelica.SIunits.AngularAcceleration) 
            "Absolute angular acceleration" 
            annotation (extent=[90,-10; 110,10]);
        protected 
          Bonds.eBond B1 annotation (extent=[-40,-10; -20,10]);
          Modelica.Blocks.Continuous.Der A   annotation (extent=[30,-10; 50,10]);
          BondLib.Sensors.Df W annotation (extent=[-20,-10; 0,10]);
        equation 
          connect(Rot2BG.Rot,flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          annotation (
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the absolute angular acceleration.
This sensor element can be connected to any flange.
</html>"),  Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[70,-30; 120,-80],
                style(color=0),
                string="a"),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0]),
              Text(extent=[150,80; -150,120],
                                           string="%name")));
          connect(B1.eBondCon1,W. BondCon1) annotation (points=[-20,0; -20,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(W.OutPort1,A. u)   annotation (points=[1,0; 28,0], style(
                color=74, rgbcolor={0,0,127}));
          connect(Rot2BG.BondCon1, B1.fBondCon1) annotation (points=[-50,0; -40,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(A.y, a) annotation (points=[51,0; 100,0], style(color=74,
                rgbcolor={0,0,127}));
        end AccSensor;
        
        model RelAngleSensor 
          "Ideal sensor to measure the relative angle between two flanges" 
          extends Interfaces.OnePort;
        protected 
          Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,-10; 70,10]);
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,-10; -50,10]);
          annotation (
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the relative angle.
This sensor element can be connected between any two flanges.
</html>"),  Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[20,-70; 160,-100],
                string="phi_rel",
                style(color=0)),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0],   style(color=0)),
              Line(points=[0,-100; 0,-70]),
              Text(extent=[150,80; -150,120],
                                           string="%name")));
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,10; 10,-10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,10; 10,30],   rotation=90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.Df W_rel 
            annotation (extent=[-10,30; 10,50], rotation=90);
          Modelica.Blocks.Math.Add A1(k1=+1, k2=-1) 
            annotation (extent=[-10,-60; 10,-40], rotation=-90);
        public 
          Modelica.Blocks.Interfaces.RealOutput phi_rel(redeclare type 
              SignalType = 
                Modelica.SIunits.Angle) "Relative angle" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
        equation 
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(B1.BondCon2,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B3. BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon3,B2. eBondCon1) annotation (points=[0,10;
                -6.12303e-016,10], style(color=8, rgbcolor={192,192,192}));
          connect(W_rel.BondCon1,B2. fBondCon1) annotation (points=[
                -6.12303e-016,30; 6.12303e-016,30], style(color=8, rgbcolor={
                  192,192,192}));
          connect(B1.BondCon1, Rot2BG.BondCon1) annotation (points=[-29.8,0;
                -50,0], style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon2, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(A1.y, phi_rel) 
                               annotation (points=[6.73533e-016,-61;
                6.73533e-016,-75.5; 0,-75.5; 0,-100], style(color=74, rgbcolor=
                  {0,0,127}));
          connect(A1.u2, Rot2BG.phi) annotation (points=[-6,-38; -6,-20; -60,
                -20; -60,-10], style(color=74, rgbcolor={0,0,127}));
          connect(A1.u1, BG2Rot.phi) annotation (points=[6,-38; 6,-20; 60,-20;
                60,-10], style(color=74, rgbcolor={0,0,127}));
        end RelAngleSensor;
        
        model RelSpeedSensor 
          "Ideal sensor to measure the relative angular velocity between two flanges" 
          extends Interfaces.OnePort;
        protected 
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,-10; -50,10]);
        protected 
          Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,-10; 70,10]);
          annotation (
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the relative angular velocity.
This sensor element can be connected between any two flanges.
</html>"),  Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[20,-70; 160,-100],
                style(color=0),
                string="w_rel"),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0],   style(color=0)),
              Line(points=[0,-100; 0,-70]),
              Text(extent=[150,80; -150,120],
                                           string="%name")));
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,-30; 10,-10], rotation=-90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.Df W_rel 
            annotation (extent=[-10,-50; 10,-30], rotation=270);
        public 
          Modelica.Blocks.Interfaces.RealOutput w_rel(redeclare type SignalType
              = Modelica.SIunits.AngularVelocity) "Relative angular velocity" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
        equation 
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(B1.BondCon2,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.eBondCon1,j0p3_1. BondCon3) annotation (points=[
                -6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B3. BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(W_rel.BondCon1,B2. fBondCon1) annotation (points=[
                1.83691e-015,-30; 6.12303e-016,-30], style(color=8, rgbcolor={
                  192,192,192}));
          connect(W_rel.OutPort1,w_rel)  annotation (points=[-2.0206e-015,-51;
                -2.0206e-015,-70.5; 0,-70.5; 0,-100], style(color=74, rgbcolor=
                  {0,0,127}));
          connect(B1.BondCon1, Rot2BG.BondCon1) annotation (points=[-29.8,0;
                -50,0], style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon2, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
        end RelSpeedSensor;
        
        model RelAccSensor 
          "Ideal sensor to measure the relative angular acceleration between two flanges" 
          extends Interfaces.OnePort;
        protected 
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,-10; -50,10]);
        protected 
          Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,-10; 70,10]);
          annotation (
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the relative angular acceleration.
This sensor element can be connected between any two flanges.
</html>"),  Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
                 Text(
                extent=[20,-70; 160,-100],
                style(color=0),
                string="a_rel"),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0],   style(color=0)),
              Line(points=[0,-100; 0,-70]),
              Text(extent=[150,80; -150,120],
                                           string="%name")));
          Junctions.J0p3 j0p3_1 annotation (extent=[-10,-10; 10,10]);
          Bonds.Bond B1 annotation (extent=[-30,-10; -10,10]);
          Bonds.fBond B2 annotation (extent=[-10,-30; 10,-10], rotation=-90);
          Bonds.Bond B3 annotation (extent=[10,-10; 30,10]);
          BondLib.Sensors.Df W_rel 
            annotation (extent=[-10,-50; 10,-30], rotation=270);
          Modelica.Blocks.Continuous.Der A_rel 
                                             annotation (extent=[-10,-80; 10,
                -60], rotation=-90);
        public 
          Modelica.Blocks.Interfaces.RealOutput a_rel(redeclare type SignalType
              = Modelica.SIunits.AngularAcceleration) 
            "Relative angular acceleration" 
            annotation (extent=[-10,-110; 10,-90], rotation=-90);
        equation 
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(B1.BondCon2,j0p3_1. BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(B2.eBondCon1,j0p3_1. BondCon3) annotation (points=[
                -6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(j0p3_1.BondCon2,B3. BondCon1) annotation (points=[10,0; 10.2,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=0,
              rgbfillColor={0,0,0},
              fillPattern=1));
          connect(W_rel.BondCon1,B2. fBondCon1) annotation (points=[
                1.83691e-015,-30; 6.12303e-016,-30], style(color=8, rgbcolor={
                  192,192,192}));
          connect(A_rel.y,a_rel)  annotation (points=[6.73533e-016,-81;
                6.73533e-016,-86.5; 0,-86.5; 0,-100], style(color=74, rgbcolor=
                  {0,0,127}));
          connect(A_rel.u,W_rel. OutPort1) annotation (points=[-7.34764e-016,
                -58; -7.34764e-016,-55; 0,-51; -2.0206e-015,-51],
              style(color=74, rgbcolor={0,0,127}));
          connect(B1.BondCon1, Rot2BG.BondCon1) annotation (points=[-29.8,0;
                -50,0], style(color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon2, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
        end RelAccSensor;
        
        model PowerSensor 
          "Ideal sensor to measure the power between two flanges" 
         extends Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealOutput power( redeclare type 
              SignalType = 
                Modelica.SIunits.Power) "Power in flange flange_a" 
        annotation (extent=[-90,-100; -70,-120],   rotation=90);
          annotation (
            Icon(
              Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
              Line(points=[0,70; 0,40],   style(color=0)),
              Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
              Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
              Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
              Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
              Line(points=[0,0; 9.02,28.6],   style(color=0)),
              Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-5,5; 5,-5],   style(
                  color=0,
                  gradient=0,
                  fillColor=0,
                  fillPattern=1)),
              Line(points=[-80,-100; -80,0],   style(color=74, rgbcolor={0,0,127})),
              Line(points=[-70,0; -90,0],   style(color=0)),
              Line(points=[70,0; 90,0],   style(color=0)),
              Text(extent=[150,80; -150,120],
                                           string="%name"),
              Text(
                extent=[-50,-80; 100,-120],
                style(color=0),
                string="power")),
            Diagram,
            Documentation(info="<html>
Ideal sensor to measure the power between two flanges.
This element should be placed in series with another element to measure the power within a connection.
</html>"));
        protected 
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-70,10; -50,-10]);
          Interfaces.BG2Rot BG2Rot 
            annotation (extent=[50,10; 70,-10]);
        public 
          BondLib.Sensors.Pbond B1 annotation (extent=[-10,-10; 10,10]);
        equation 
          connect(Rot2BG.phi,BG2Rot. phi) annotation (points=[-60,10; -60,40;
                60,40; 60,10], style(color=3, rgbcolor={0,0,255}));
          connect(Rot2BG.Rot, flange_a) annotation (points=[-70,0; -100,0],
              style(color=0, rgbcolor={0,0,0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(B1.BondCon1, Rot2BG.BondCon1) annotation (points=[-9.8,0; -50,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(B1.BondCon2, BG2Rot.BondCon1) annotation (points=[10,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B1.P, power) annotation (points=[0,9; 0,20; -30,20; -30,-60;
                -80,-60; -80,-110], style(color=74, rgbcolor={0,0,127}));
        end PowerSensor;
      end Sensors;
      
      package Sources "Sources of the 1D rotational library" 
        annotation (
          preferedView="info",
          Icon(
            Text(
              extent=[-120,118; 120,69],
              string="%name",
              style(color=1)),
            Rectangle(extent=[-100,-100; 80,50],  style(fillColor=30, fillPattern=
                   1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                  fillColor=30, fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],     style(
                  fillColor=30, fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=3))), Documentation(info="<html>
Active (external) sources of the 1D rotational library.
</html>"));
        model Torque "Input signal acting as external torque on a flange" 
          
          Modelica.Blocks.Interfaces.RealInput tau(redeclare type SignalType = 
                Modelica.SIunits.Torque) "External torque" 
            annotation (extent=[-120,-10; -100,10]);
          Interfaces.Flange_a bearing "Bearing rotational connector" 
                                      annotation (extent=[-10,-90; 10,-110]);
          Interfaces.Flange_b flange_b "Rotational connector" 
                                       annotation (extent=[90,-10; 110,10]);
          annotation (
            Icon(
              Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=8)),
              Text(extent=[0,130; 0,70],   string="%name"),
              Text(
                extent=[-62,-29; -141,-70],
                string="tau",
                style(color=0)),
              Line(points=[-88,0; -64,30; -36,52; -2,62; 28,56; 48,44; 64,28;
                    76,14; 86,0],   style(
                  color=0,
                  thickness=2,
                  fillColor=0)),
              Polygon(points=[86,0; 66,58; 37,27; 86,0],     style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Line(points=[-30,-30; 30,-30],   style(color=0)),
              Line(points=[0,-30; 0,-90],   style(color=0)),
              Line(points=[-30,-50; -10,-30],   style(color=0)),
              Line(points=[-10,-50; 10,-30],   style(color=0)),
              Line(points=[10,-50; 30,-30],   style(color=0))),
            Diagram(
        Rectangle(extent=[-80,-30; -60,-50],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-30; -60,-50],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-60,-40; -52,-40], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
<p>
The input signal <b>tau</b> defines an external
torque in [Nm] which acts (with negative sign) at
a flange connector, i.e., the component connected to this
flange is driven by torque <b>tau</b>.</p>
<p>
The input signal can be provided from one of the signal generator
blocks of Modelica.Blocks.Sources.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</html>"));
        protected 
          Interfaces.BG2Rot BG2Rot annotation (extent=[50,-10; 70,10]);
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-10,-90; 10,-70], rotation=90);
          Junctions.J0p3 j0p3_2 annotation (extent=[-10,-10; 10,10]);
          Bonds.eBond B3 annotation (extent=[-10,-30; 10,-10], rotation=90);
          Bonds.fBond B2 annotation (extent=[10,-10; 30,10]);
          Bonds.fBond B1 annotation (extent=[-30,-10; -10,10]);
          BondLib.Sources.mSe Tau  annotation (extent=[-30,-10; -50,10]);
        public 
          Junctions.J1p3 j1p3_1 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          Bonds.Bond B4 annotation (extent=[-30,-50; -10,-30]);
          Bonds.Bond B5 annotation (extent=[-10,-70; 10,-50], rotation=90);
          Switches.Sw Sw1 annotation (extent=[-30,-50; -50,-30]);
        equation 
          if cardinality(bearing) <= 1 then
            bearing.phi = 0;
            Sw1.BooleanInPort1 = true;
          else
            Sw1.BooleanInPort1 = false;
          end if;
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(bearing, Rot2BG.Rot) annotation (points=[0,-100; 0,-90;
                -6.12303e-016,-90], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B3.eBondCon1, j0p3_2.BondCon3) annotation (points=[
                6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B2.eBondCon1, j0p3_2.BondCon2) annotation (points=[10,0; 10,0],
              style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B2.fBondCon1, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B1.fBondCon1, j0p3_2.BondCon1) annotation (points=[-10,0; -10,
                0], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Tau.BondCon1, B1.eBondCon1) annotation (points=[-30,0; -30,0],
              style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(j1p3_1.BondCon1, B3.fBondCon1) annotation (points=[
                -6.12303e-016,-30; -6.12303e-016,-30], style(color=8, rgbcolor=
                  {192,192,192}));
          connect(B4.BondCon2, j1p3_1.BondCon3) annotation (points=[-10,-40;
                -10,-40], style(color=8, rgbcolor={192,192,192}));
          connect(j1p3_1.BondCon2, B5.BondCon2) annotation (points=[
                6.12303e-016,-50; 6.12303e-016,-50], style(color=8, rgbcolor={
                  192,192,192}));
          connect(Rot2BG.BondCon1, B5.BondCon1) annotation (points=[
                6.12303e-016,-70; -6.00057e-016,-70; -6.00057e-016,-69.8],
              style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.BondCon1, B4.BondCon1) annotation (points=[-30,-40; -29.8,
                -40], style(color=8, rgbcolor={192,192,192}));
          connect(Tau.s, tau) annotation (points=[-49,0; -110,0], style(color=
                  74, rgbcolor={0,0,127}));
        end Torque;
        
        model Speed 
          "Forced movement of a flange according to a velocity signal" 
          parameter Modelica.SIunits.Angle phi_start=0 "Start angle";
          Modelica.SIunits.Angle phi(final start=phi_start, final fixed=true) 
            "absolute angle of flange_b";
          
          annotation (
            Icon(
              Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=8)),
              Rectangle(extent=[-100,20; 100,-20],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[-30,-32; 30,-32],   style(color=0)),
              Line(points=[0,52; 0,32],   style(color=0)),
              Line(points=[-29,32; 30,32],   style(color=0)),
              Line(points=[0,-32; 0,-90],   style(color=0)),
              Line(points=[-10,-32; -30,-52],   style(color=0)),
              Line(points=[0,-32; -20,-52],   style(color=0)),
              Line(points=[10,-32; -10,-52],   style(color=0)),
              Line(points=[20,-32; 0,-52],   style(color=0)),
              Line(points=[-20,-32; -30,-42],   style(color=0)),
              Line(points=[30,-32; 10,-52],   style(color=0)),
              Line(points=[30,-42; 20,-52],   style(color=0)),
              Text(
                extent=[-54,-44; -158,-78],
                style(color=0),
                string="w_ref"),
              Text(extent=[0,120; 0,60],   string="%name")),
            Diagram(
        Rectangle(extent=[-80,-30; -60,-50],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-30; -60,-50],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-60,-40; -52,-40], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
External angular velocity acting on a drive train element as input signal.
</html>"));
          Modelica.Blocks.Interfaces.RealInput w_ref(redeclare type SignalType 
              = Modelica.SIunits.AngularVelocity) "External angular velocity" 
            annotation (extent=[-120,-10; -100,10]);
          Interfaces.Flange_b flange_b "Rotational connector" 
                                       annotation (extent=[90,-10; 110,10]);
          Interfaces.Flange_a bearing "Bearing rotational connector" 
                                      annotation (extent=[-10,-90; 10,-110]);
        protected 
          Interfaces.BG2Rot BG2Rot annotation (extent=[50,-10; 70,10]);
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-10,-90; 10,-70], rotation=90);
          Junctions.J0p3 j0p3_2 annotation (extent=[-10,-10; 10,10]);
          Bonds.eBond B3 annotation (extent=[-10,-30; 10,-10], rotation=90);
          Bonds.eBond B1 annotation (extent=[-30,-10; -10,10], rotation=0);
          Bonds.eBond B2 annotation (extent=[10,-10; 30,10],   rotation=0);
          BondLib.Sources.mSf W   annotation (extent=[-30,-10; -50,10]);
          Modelica.Blocks.Continuous.Integrator Phi(y_start=phi_start) 
            annotation (extent=[-50,30; -30,50]);
          Modelica.Blocks.Math.Add Add1 annotation (extent=[30,-50; 50,-30]);
        public 
          Junctions.J1p3 j1p3_1 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          Bonds.Bond B4 annotation (extent=[-30,-50; -10,-30]);
          Bonds.Bond B5 annotation (extent=[-10,-70; 10,-50], rotation=90);
          Switches.Sw Sw1 annotation (extent=[-30,-50; -50,-30]);
        equation 
          phi = Phi.y;
          
          if cardinality(bearing) <= 1 then
            bearing.phi = 0;
            Sw1.BooleanInPort1 = true;
          else
            Sw1.BooleanInPort1 = false;
          end if;
          connect(B3.eBondCon1,j0p3_2. BondCon3) annotation (points=[
                6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B1.eBondCon1, j0p3_2.BondCon1) annotation (points=[-10,0; -10,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_2.BondCon2,B2. fBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(Rot2BG.Rot, bearing) annotation (points=[-6.12303e-016,-90;
                -6.12303e-016,-83; 0,-83; 0,-100], style(color=0, rgbcolor={0,0,
                  0}));
          connect(W.BondCon1, B1.fBondCon1) annotation (points=[-30,0; -30,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Phi.u, w_ref) annotation (points=[-52,40; -70,40; -70,0; -110,
                0],      style(color=74, rgbcolor={0,0,127}));
          connect(Phi.y, Add1.u1) annotation (points=[-29,40; 20,40; 20,-34; 28,
                -34],    style(color=74, rgbcolor={0,0,127}));
          connect(Rot2BG.phi, Add1.u2) annotation (points=[10,-80; 20,-80; 20,
                -46; 28,-46], style(color=3, rgbcolor={0,0,255}));
          connect(B4.BondCon2,j1p3_1. BondCon3) annotation (points=[-10,-40;
                -10,-40], style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.BondCon1,B4. BondCon1) annotation (points=[-30,-40; -29.8,
                -40], style(color=8, rgbcolor={192,192,192}));
          connect(B3.fBondCon1, j1p3_1.BondCon1) annotation (points=[
                -6.12303e-016,-30; -6.12303e-016,-30], style(color=8, rgbcolor=
                  {192,192,192}));
          connect(B5.BondCon2, j1p3_1.BondCon2) annotation (points=[
                6.12303e-016,-50; 6.12303e-016,-50], style(color=8, rgbcolor={
                  192,192,192}));
          connect(B5.BondCon1, Rot2BG.BondCon1) annotation (points=[
                -6.00057e-016,-69.8; 0,-69.8; 0,-70; 6.12303e-016,-70], style(
                color=8, rgbcolor={192,192,192}));
          connect(Add1.y, BG2Rot.phi) annotation (points=[51,-40; 60,-40; 60,
                -10], style(color=74, rgbcolor={0,0,127}));
          connect(W.s, w_ref) annotation (points=[-49,0; -110,0], style(color=
                  74, rgbcolor={0,0,127}));
        end Speed;
        
        model Accelerate 
          "Forced movement of a flange according to an acceleration signal" 
          parameter Modelica.SIunits.Angle phi_start=0 "Start angle";
          parameter Modelica.SIunits.AngularVelocity w_start=0 
            "Start angular velocity";
          Modelica.SIunits.AngularVelocity w(final start=w_start, final fixed=true) 
            "Absolute angular velocity of flange_b";
          Modelica.SIunits.Angle phi(final start=phi_start, final fixed=true) 
            "Absolute angle of flange_b";
          
          Modelica.Blocks.Interfaces.RealInput a(redeclare type SignalType = 
                Modelica.SIunits.AngularAcceleration) 
            "External angular acceleration" 
            annotation (extent=[-120,-10; -100,10]);
          Interfaces.Flange_b flange_b "Rotational connector" 
                                       annotation (extent=[90,-10; 110,10]);
          Interfaces.Flange_a bearing "Bearing rotational connector" 
                                      annotation (extent=[-10,-90; 10,-110]);
          annotation (
            Icon(
              Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=8)),
              Rectangle(extent=[-100,20; 100,-20],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[-30,-32; 30,-32],   style(color=0)),
              Line(points=[0,52; 0,32],   style(color=0)),
              Line(points=[-29,32; 30,32],   style(color=0)),
              Line(points=[0,-32; 0,-90],   style(color=0)),
              Line(points=[-10,-32; -30,-52],   style(color=0)),
              Line(points=[0,-32; -20,-52],   style(color=0)),
              Line(points=[10,-32; -10,-52],   style(color=0)),
              Line(points=[20,-32; 0,-52],   style(color=0)),
              Line(points=[-20,-32; -30,-42],   style(color=0)),
              Line(points=[30,-32; 10,-52],   style(color=0)),
              Line(points=[30,-42; 20,-52],   style(color=0)),
              Text(
                extent=[-54,-44; -158,-78],
                style(color=0),
                string="a"),
              Text(extent=[0,120; 0,60],   string="%name")),
            Diagram(
        Rectangle(extent=[-80,-30; -60,-50],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-30; -60,-50],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-60,-40; -52,-40], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
External angular acceleration acting on a drive train element as input signal.
</html>"));
        protected 
          Interfaces.BG2Rot BG2Rot annotation (extent=[50,-10; 70,10]);
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-10,-90; 10,-70], rotation=90);
          Junctions.J0p3 j0p3_2 annotation (extent=[-10,-10; 10,10]);
          Bonds.eBond B3 annotation (extent=[-10,-30; 10,-10], rotation=90);
          Bonds.eBond B1 annotation (extent=[-30,-10; -10,10], rotation=0);
          Bonds.eBond B2 annotation (extent=[10,-10; 30,10],   rotation=0);
          BondLib.Sources.mSf mSF annotation (extent=[-30,-10; -50,10]);
          Modelica.Blocks.Continuous.Integrator Phi(y_start=phi_start) 
            annotation (extent=[-50,30; -30,50]);
          Modelica.Blocks.Math.Add Add1 annotation (extent=[30,-50; 50,-30]);
          Modelica.Blocks.Continuous.Integrator W(y_start=w_start) 
            annotation (extent=[-90,-10; -70,10]);
        public 
          Junctions.J1p3 j1p3_1 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          Bonds.Bond B4 annotation (extent=[-30,-50; -10,-30]);
          Bonds.Bond B5 annotation (extent=[-10,-70; 10,-50], rotation=90);
          Switches.Sw Sw1 annotation (extent=[-30,-50; -50,-30]);
        equation 
          w = W.y;
          phi = Phi.y;
          
          if cardinality(bearing) <= 1 then
            bearing.phi = 0;
            Sw1.BooleanInPort1 = true;
          else
            Sw1.BooleanInPort1 = false;
          end if;
          connect(B3.eBondCon1,j0p3_2. BondCon3) annotation (points=[
                6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B1.eBondCon1, j0p3_2.BondCon1) annotation (points=[-10,0; -10,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_2.BondCon2,B2. fBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(mSF.BondCon1, B1.fBondCon1) annotation (points=[-30,0; -30,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Phi.y, Add1.u1) annotation (points=[-29,40; 20,40; 20,-34; 28,
                -34],    style(color=74, rgbcolor={0,0,127}));
          connect(Rot2BG.phi, Add1.u2) annotation (points=[10,-80; 20,-80; 20,
                -46; 28,-46], style(color=3, rgbcolor={0,0,255}));
          connect(Add1.y, BG2Rot.phi) annotation (points=[51,-40; 60,-40; 60,
                -10], style(color=74, rgbcolor={0,0,127}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(Rot2BG.Rot, bearing) annotation (points=[-6.12303e-016,-90;
                -6.12303e-016,-83; 0,-83; 0,-100], style(color=0, rgbcolor={0,0,
                  0}));
          connect(Phi.u, W.y) annotation (points=[-52,40; -60,40; -60,0; -69,0],
                    style(color=74, rgbcolor={0,0,127}));
          connect(W.u, a) annotation (points=[-92,0; -110,0], style(color=74,
                rgbcolor={0,0,127}));
          connect(B4.BondCon2,j1p3_1. BondCon3) annotation (points=[-10,-40;
                -10,-40], style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.BondCon1,B4. BondCon1) annotation (points=[-30,-40; -29.8,
                -40], style(color=8, rgbcolor={192,192,192}));
          connect(B5.BondCon2, j1p3_1.BondCon2) annotation (points=[
                6.12303e-016,-50; 6.12303e-016,-50], style(color=8, rgbcolor={
                  192,192,192}));
          connect(B3.fBondCon1, j1p3_1.BondCon1) annotation (points=[
                -6.12303e-016,-30; -6.12303e-016,-30], style(color=8, rgbcolor=
                  {192,192,192}));
          connect(B5.BondCon1, Rot2BG.BondCon1) annotation (points=[
                -6.00057e-016,-69.8; 0,-69.8; 0,-70; 6.12303e-016,-70], style(
                color=8, rgbcolor={192,192,192}));
          connect(mSF.s, W.y) annotation (points=[-49,0; -69,0], style(color=74,
                rgbcolor={0,0,127}));
        end Accelerate;
        
        model Position 
          "Forced movement of a flange according to a reference angle signal" 
          Modelica.SIunits.AngularVelocity w 
            "Absolute angular velocity of flange_b";
          
          Modelica.Blocks.Interfaces.RealInput phi_ref(redeclare type 
              SignalType = 
                Modelica.SIunits.Angle) "External angle" 
            annotation (extent=[-120,-10; -100,10]);
          Interfaces.Flange_b flange_b "Rotational connector" 
                                       annotation (extent=[90,-10; 110,10]);
          Interfaces.Flange_a bearing "Bearing rotational connector" 
                                      annotation (extent=[-10,-90; 10,-110]);
          annotation (
            Icon(
              Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=8)),
              Rectangle(extent=[-100,20; 100,-20],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[-30,-32; 30,-32],   style(color=0)),
              Line(points=[0,52; 0,32],   style(color=0)),
              Line(points=[-29,32; 30,32],   style(color=0)),
              Line(points=[0,-32; 0,-90],   style(color=0)),
              Line(points=[-10,-32; -30,-52],   style(color=0)),
              Line(points=[0,-32; -20,-52],   style(color=0)),
              Line(points=[10,-32; -10,-52],   style(color=0)),
              Line(points=[20,-32; 0,-52],   style(color=0)),
              Line(points=[-20,-32; -30,-42],   style(color=0)),
              Line(points=[30,-32; 10,-52],   style(color=0)),
              Line(points=[30,-42; 20,-52],   style(color=0)),
              Text(
                extent=[-54,-44; -158,-78],
                style(color=0),
                string="phi_ref"),
              Text(extent=[0,120; 0,60],   string="%name")),
            Diagram(
        Rectangle(extent=[-80,-30; -60,-50],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-80,-30; -60,-50],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-60,-40; -52,-40], style(color=83, rgbcolor={255,85,
                      255}))),
            Documentation(info="<html>
External angle acting on a drive train element as input signal.
</html>"));
        protected 
          Interfaces.BG2Rot BG2Rot annotation (extent=[50,-10; 70,10]);
          Interfaces.Rot2BG Rot2BG 
            annotation (extent=[-10,-90; 10,-70], rotation=90);
          Junctions.J0p3 j0p3_2 annotation (extent=[-10,-10; 10,10]);
          Bonds.eBond B3 annotation (extent=[-10,-30; 10,-10], rotation=90);
          Bonds.eBond B1 annotation (extent=[-30,-10; -10,10], rotation=0);
          Bonds.eBond B2 annotation (extent=[10,-10; 30,10],   rotation=0);
          BondLib.Sources.mSf mSF annotation (extent=[-30,-10; -50,10]);
          Modelica.Blocks.Math.Add Add1 annotation (extent=[30,-50; 50,-30]);
          Modelica.Blocks.Continuous.Der W annotation (extent=[-80,-10; -60,10]);
        public 
          Junctions.J1p3 j1p3_1 
            annotation (extent=[-10,-50; 10,-30], rotation=-90);
          Bonds.Bond B4 annotation (extent=[-30,-50; -10,-30]);
          Bonds.Bond B5 annotation (extent=[-10,-70; 10,-50], rotation=90);
          Switches.Sw Sw1 annotation (extent=[-30,-50; -50,-30]);
        equation 
          w = W.y;
          
          if cardinality(bearing) <= 1 then
            bearing.phi = 0;
            Sw1.BooleanInPort1 = true;
          else
            Sw1.BooleanInPort1 = false;
          end if;
          connect(B3.eBondCon1,j0p3_2. BondCon3) annotation (points=[
                6.12303e-016,-10; 0,-10], style(
              color=8,
              rgbcolor={192,192,192},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(B1.eBondCon1, j0p3_2.BondCon1) annotation (points=[-10,0; -10,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(j0p3_2.BondCon2,B2. fBondCon1) annotation (points=[10,0; 10,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1, BG2Rot.BondCon1) annotation (points=[30,0; 50,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(mSF.BondCon1, B1.fBondCon1) annotation (points=[-30,0; -30,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(Rot2BG.phi, Add1.u2) annotation (points=[10,-80; 20,-80; 20,
                -46; 28,-46], style(color=3, rgbcolor={0,0,255}));
          connect(Add1.y, BG2Rot.phi) annotation (points=[51,-40; 60,-40; 60,
                -10], style(color=74, rgbcolor={0,0,127}));
          connect(Rot2BG.Rot, bearing) annotation (points=[-6.12303e-016,-90;
                -6.12303e-016,-83; 0,-83; 0,-100], style(color=0, rgbcolor={0,0,
                  0}));
          connect(BG2Rot.Rot, flange_b) annotation (points=[70,0; 100,0], style(
                color=0, rgbcolor={0,0,0}));
          connect(W.u, phi_ref) annotation (points=[-82,0; -110,0], style(color=
                 74, rgbcolor={0,0,127}));
          connect(Add1.u1, phi_ref) annotation (points=[28,-34; 20,-34; 20,40;
                -90,40; -90,0; -110,0],  style(color=74, rgbcolor={0,0,127}));
          connect(B4.BondCon2,j1p3_1. BondCon3) annotation (points=[-10,-40;
                -10,-40], style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.BondCon1,B4. BondCon1) annotation (points=[-30,-40; -29.8,
                -40], style(color=8, rgbcolor={192,192,192}));
          connect(B5.BondCon2, j1p3_1.BondCon2) annotation (points=[
                6.12303e-016,-50; 6.12303e-016,-50], style(color=8, rgbcolor={
                  192,192,192}));
          connect(B3.fBondCon1, j1p3_1.BondCon1) annotation (points=[
                -6.12303e-016,-30; -6.12303e-016,-30], style(color=8, rgbcolor=
                  {192,192,192}));
          connect(B5.BondCon1, Rot2BG.BondCon1) annotation (points=[
                -6.00057e-016,-69.8; 0,-69.8; 0,-70; 6.12303e-016,-70], style(
                color=8, rgbcolor={192,192,192}));
          connect(mSF.s, W.y) annotation (points=[-49,0; -59,0], style(color=74,
                rgbcolor={0,0,127}));
        end Position;
      end Sources;
      
      package Utilities "Auxiliary models of 1D rotational library" 
        annotation (
          preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70], style(
                color=42,
                rgbcolor={127,0,0},
                fillColor=30,
                rgbfillColor={235,235,235},
                fillPattern=1)),
            Text(
              extent=[-85,35; 65,-85],
              string="Library",
              style(color=42, rgbcolor={127,0,0})),
            Text(
              extent=[-120,122; 120,73],
              string="%name",
              style(color=1))), Documentation(info="<html>
Auxiliary models of the 1D rotational library.
</html>"));
        model TFangle "Transformation of angles" 
          parameter Real m=1 "Transformation constant";
          
          annotation (
            Diagram(
              Rectangle(extent=[-100,100; 100,-100], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                 Line(points=[44,-42; 70,-48; 44,-54],    style(color=0)), Line(
                  points=[-68,-48; 70,-48],   style(color=0)),
              Text(
                extent=[-58,82; 62,-78],
                string="TF",
                style(color=0))),
            Icon(
              Rectangle(extent=[-100,100; 100,-100], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                 Line(points=[44,-42; 70,-48; 44,-54],    style(color=0)), Line(
                  points=[-68,-48; 70,-48],   style(color=0)),
              Text(
                extent=[-58,82; 62,-78],
                string="TF",
                style(color=0)),
              Text(extent=[-72,146; 68,106], string="%name"),
              Text(extent=[-68,-52; 72,-92],   string="m=%m")),
            Documentation(info="<html>
Transformation of positional information across a rotational transformer.
</html>"));
          Translational.Interfaces.RealSignal phi_a(redeclare type SignalType 
              = Modelica.SIunits.Angle) "First angular position" 
            annotation (extent=[-118,-10; -98,10]);
          Translational.Interfaces.RealSignal phi_b(redeclare type SignalType 
              = Modelica.SIunits.Angle) "Second angular position" 
            annotation (extent=[98,-10; 118,10]);
        equation 
          phi_b = m*phi_a;
        end TFangle;
        
        model Add "Bidirectional adder" 
          
          parameter Real k1=+1 "Gain of upper input";
          parameter Real k2=+1 "Gain of lower input";
          annotation (
            Icon(Rectangle(extent=[-100,-100; 100,100],   style(
                color=74,
                rgbcolor={0,0,127},
                fillColor=7,
                rgbfillColor={255,255,255})),
              Text(
                extent=[-98,-52; 7,-92],
                string="%k2",
                style(color=0)),
              Text(
                extent=[-100,90; 5,50],
                string="%k1",
                style(color=0)),
              Line(points=[-100,60; -40,60; -30,40]),
              Ellipse(extent=[-50,50; 50,-50]),
              Line(points=[-100,-60; -40,-60; -30,-40]),
              Line(points=[-15,-25.99; 15,25.99],   style(color=0)),
              Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
              Line(points=[50,0; 100,0]),
              Line(points=[-100,60; -74,24; -44,24],    style(color=74, rgbcolor=
                  {0,0,127})),
              Line(points=[-100,-60; -74,-28; -42,-28],    style(color=74,
                rgbcolor={0,0,127})),
              Ellipse(extent=[-50,50; 50,-50],   style(color=74, rgbcolor={0,0,
                  127})),
              Line(points=[50,0; 100,0],   style(color=74, rgbcolor={0,0,127})),
              Text(
                extent=[-38,34; 38,-34],
                string="+",
                style(color=0)),
              Text(
                extent=[-100,52; 5,92],
                string="%k1",
                style(color=0)),
              Text(
                extent=[-100,-52; 5,-92],
                string="%k2",
                style(color=0)),
              Text(extent=[-150,148; 150,108],   string="%name")),
            Diagram(
              Rectangle(extent=[-100,-100; 100,100],   style(color=3, fillColor=7)),
              Text(
                extent=[-98,-52; 7,-92],
                string="%k2",
                style(color=0)),
              Text(
                extent=[-100,90; 5,50],
                string="%k1",
                style(color=0)),
              Line(points=[-100,60; -40,60; -30,40]),
              Ellipse(extent=[-50,50; 50,-50]),
              Line(points=[-100,-60; -40,-60; -30,-40]),
              Line(points=[-15,-25.99; 15,25.99],   style(color=0)),
              Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
              Line(points=[50,0; 100,0]),
              Line(points=[-100,60; -74,24; -44,24],    style(color=74, rgbcolor=
                  {0,0,127})),
              Line(points=[-100,-60; -74,-28; -42,-28],    style(color=74,
                rgbcolor={0,0,127})),
              Ellipse(extent=[-50,50; 50,-50],   style(color=74, rgbcolor={0,0,
                  127})),
              Line(points=[50,0; 100,0],   style(color=74, rgbcolor={0,0,127})),
              Text(
                extent=[-38,34; 38,-34],
                string="+",
                style(color=0)),
              Text(
                extent=[-100,52; 5,92],
                string="k1",
                style(color=0)),
              Text(
                extent=[-100,-52; 5,-92],
                string="k2",
                style(color=0))),
            Documentation(info="<html>
Bidirectional adder representing a rotational 0-junction at positional level.
</html>"));
          Translational.Interfaces.RealSignal u1(redeclare type SignalType = 
                Modelica.SIunits.Angle) "First angular position" 
            annotation (extent=[-118,50; -98,70]);
          Translational.Interfaces.RealSignal u2(redeclare type SignalType = 
                Modelica.SIunits.Angle) "Second angular position" 
            annotation (extent=[-118,-70; -98,-50]);
          Translational.Interfaces.RealSignal y(redeclare type SignalType = 
                Modelica.SIunits.Angle) "Third angular position" 
            annotation (extent=[98,-10; 118,10]);
        equation 
          y = k1*u1 + k2*u2;
        end Add;
        
        model TFangpos "Transformation from angle to position" 
          parameter Real m(final unit="m/rad", final min=0)=1 
            "Transformation constant";
          
          Translational.Interfaces.RealSignal phi_a(redeclare type SignalType 
              = Modelica.SIunits.Angle) "First angular position" 
            annotation (extent=[-118,-10; -98,10]);
          Translational.Interfaces.RealSignal s_b(redeclare type SignalType = 
                Modelica.SIunits.Position) "Second angular position" 
            annotation (extent=[98,-10; 118,10]);
          annotation (
            Icon(
              Rectangle(extent=[-100,100; 100,-100], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                 Line(points=[44,-42; 70,-48; 44,-54],    style(color=0)), Line(
                  points=[-68,-48; 70,-48],   style(color=0)),
              Text(
                extent=[-58,82; 62,-78],
                string="TF",
                style(color=0)),
              Text(extent=[-72,146; 68,106], string="%name"),
              Text(extent=[-68,-52; 72,-92],   string="m=%m")),
            Diagram(
              Rectangle(extent=[-100,100; 100,-100], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
                 Line(points=[44,-42; 70,-48; 44,-54],    style(color=0)), Line(
                  points=[-68,-48; 70,-48],   style(color=0)),
              Text(
                extent=[-58,82; 62,-78],
                string="TF",
                style(color=0))),
            Documentation(info="<html>
Transformation of positional information across a rotational to translational transformer.
</html>"));
        equation 
          s_b = m*phi_a;
        end TFangpos;
        
        model RelativeStates "Definition of relative state variables" 
          extends Interfaces.OnePort;
          Modelica.SIunits.Angle phi_rel(stateSelect=StateSelect.prefer) 
            "Relative angle used as state variable";
          Modelica.SIunits.AngularVelocity w_rel(stateSelect=StateSelect.prefer) 
            "Relative angular velocity used as state variable";
          Modelica.SIunits.AngularAcceleration a_rel 
            "Relative angular acceleration";
          annotation (
            Diagram(
              Ellipse(extent=[-40,40; 40,-40],   style(color=4, fillColor=4)),
              Text(extent=[-40,40; 40,-40],   string="S"),
              Line(points=[40,0; 90,0],   style(
                  color=0,
                  pattern=2,
                  fillColor=4)),
              Line(points=[-100,-60; 80,-60],   style(color=9)),
              Text(extent=[-30,-70; 30,-90],   string="w_rel"),
              Line(points=[-76,80; -5,80],   style(color=10, fillColor=10)),
              Polygon(points=[14,80; -6,85; -6,75; 14,80],     style(color=10,
                    fillColor=10)),
              Text(
                extent=[18,87; 86,74],
                string="rotation axis",
                style(color=10)),
              Line(points=[-90,0; -40,0],   style(
                  color=0,
                  pattern=2,
                  fillColor=4)),
              Polygon(points=[80,-65; 80,-55; 100,-60; 80,-65],     style(
                  color=9,
                  fillColor=9,
                  fillPattern=1))),
            Icon(
              Ellipse(extent=[-40,40; 40,-40],   style(color=4, fillColor=4)),
              Text(extent=[-40,40; 40,-40],   string="S"),
              Line(points=[-92,0; -42,0],   style(color=0, pattern=3)),
              Line(points=[40,0; 90,0],   style(
                  color=0,
                  pattern=3,
                  fillColor=4)),
              Text(extent=[0,110; 0,50],   string="%name")),
            Documentation(info="<html>
<p>
Usually, the absolute angle and the absolute angular velocity of
Modelica.Mechanics.Rotational.Inertia models are used as state variables.
In some circumstances, relative quantities are better suited, e.g.,
because it may be easier to supply initial values.
In such cases, model <b>RelativeStates</b> allows the definition of state variables
in the following way:
</p>
<ul>
<li> Connect an instance of this model between two flange connectors.</li>
<li> The <b>relative rotation angle</b> and the <b>relative angular velocity</b>
     between the two connectors are used as <b>state variables</b>.
</ul>
<p>
An example is given in the next figure
</p>
<IMG SRC=\"../Images/BG_Fig31.png\">
<p>
Here, the relative angle and the relative angular velocity between
the two inertias are used as state variables. Additionally, the
simulator selects either the absolute angle and absolute angular
velocity of model inertia1 or of model inertia2 as state variables.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 19, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</html>"));
        equation 
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          flange_a.tau = 0;
          flange_b.tau = 0;
        end RelativeStates;
        
        model R_Fric "Rotational resistance describing friction" 
          extends Interfaces.FrictionBase;
          parameter Real mue_pos[:, :]=[0, 0.5] 
            "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min=1) = 1 
            "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Modelica.SIunits.Position cgeo(final min=0) = 1 
            "Geometry constant containing friction distribution assumption";
          parameter Modelica.SIunits.Force fn_max(final min=0) = 1 
            "Maximum normal force";
          Real mue0 "Friction coefficient for w=0 and forward sliding";
          Modelica.SIunits.Force fn "Normal force (fn=fn_max*inPort.signal)";
          BondLib.Interfaces.BondCon BondCon1 "Bond graph connector" 
                                              annotation (extent=[-110,-10; -90,
                10]);
          Modelica.Blocks.Interfaces.RealInput f_normalized 
            "Normalized force signal 0..1 (normal force = fn_max*f_normalized; clutch is engaged if > 0)"
                                                    annotation (extent=[100,-10;
                80,10]);
          annotation (
            Diagram(Text(
                extent=[-84,58; -10,-62],
                style(color=0),
                string="R"), Text(
                extent=[-34,4; 94,-50],
                style(color=0, rgbcolor={0,0,0}),
                string="Fric")),
            Icon(                Text(extent=[-72,80; 68,40],   string="%name"),
                    Text(
                extent=[-84,58; -10,-62],
                style(color=0),
                string="R"), Text(
                extent=[-34,4; 94,-50],
                style(color=0, rgbcolor={0,0,0}),
                string="Fric")),
            Documentation(info="<html>
A bondgraphic modulated resistor describing the torque characteristic of mechanical (rotational) friction.
</html>"));
        equation 
          // Constant auxiliary variable
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          
          // Normal force and friction torque for w_rel=0
          fn = fn_max*f_normalized;
          free = fn <= 0;
          tau0 = mue0*cgeo*fn;
          tau0_max = peak*tau0;
          
          // Friction torque
          tau = if locked then sa else if free then 0 else cgeo*fn*(if startForward then 
                  Modelica.Math.tempInterpol1(w_relfric, mue_pos, 2) else if 
            startBackward then -Modelica.Math.tempInterpol1(-w_relfric, mue_pos, 2) else 
            if pre(mode) == Forward then Modelica.Math.tempInterpol1(w_relfric, mue_pos,
            2) else -Modelica.Math.tempInterpol1(-w_relfric, mue_pos, 2));
          
          // Bond graph connection
          tau = BondCon1.e;
          w_relfric = BondCon1.d*BondCon1.f;
        end R_Fric;
      end Utilities;
      
      package Examples "Examples of using the 1D rotational library" 
        annotation (
          preferedView="info",
          Icon(
            Rectangle(extent=[-100,-100; 80,50],   style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[-100,50; -80,70; 100,70; 80,50; -100,50],      style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Polygon(points=[100,70; 100,-80; 80,-100; 80,50; 100,70],      style(
                color=0,
                fillColor=30,
                fillPattern=1)),
            Text(
              extent=[-85,71; 65,-49],
              string="Sample",
              style(color=0)),
            Text(
              extent=[-118,114; 122,65],
              string="%name",
              style(color=1)),
            Text(
              extent=[-97,23; 78,-98],
              string="Programs",
              style(color=0))), Documentation(info="<html>
Examples of using the 1D rotational library.
</html>"));
        model ElasticBearing "Example to show possible usage of bearing flange" 
          annotation (Icon(
              Rectangle(extent=[-100,10; -40,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-40,20; -20,-20],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-40,140; -20,20],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-20,90; 20,70],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[20,100; 40,60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[20,60; 40,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[40,10; 100,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[-80,20; -60,20],   style(color=0)),
              Line(points=[60,20; 80,20],   style(color=0)),
              Line(points=[-10,60; 10,60],   style(color=0)),
              Line(points=[0,60; 0,-90],   style(color=0)),
              Line(points=[70,-70; -70,-70],   style(color=0)),
              Line(points=[-70,-20; -70,-70],   style(color=0)),
              Line(points=[-80,-20; -60,-20],   style(color=0)),
              Line(points=[70,-20; 70,-70],   style(color=0)),
              Line(points=[60,-20; 80,-20],   style(color=0)),
                 Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=8)),
              Line(points=[0,60; 0,-90], style(color=0, rgbcolor={0,0,0})),
              Ellipse(extent=[-110,10; -90,-10], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=10,
                  rgbfillColor={95,95,95})),
              Ellipse(extent=[-10,-90; 10,-110], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=10,
                  rgbfillColor={95,95,95})),
              Ellipse(extent=[90,10; 110,-10], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255}))), Diagram,
            Documentation(info="<html>
<p>
This model demonstrates the usage of the bearing flange.
The gearbox is not connected rigidly to the ground, but by
a spring-damper-system. This allows examination of the gearbox
housing dynamics.</p>
<p>
Simulate for about 10 seconds and plot the angular velocities of the inertias <tt>housing.w</tt>,
<tt>shaft.w</tt> and <tt>load.w</tt>.</p>
</html>"),  experiment(StopTime=10),
            experimentSetupOutput);
          Modelica.Blocks.Sources.Ramp ramp(              duration=5, height=100) 
            annotation (extent=[-90,40; -70,60]);
          Sources.Torque torque annotation (extent=[-50,40; -30,60]);
          Passive.Inertia shaft annotation (extent=[-20,40; 0,60]);
          Passive.IdealGear idealGear(ratio=3) 
            annotation (extent=[10,40; 30,60]);
          Passive.Spring spring(c=1e3) annotation (extent=[40,40; 60,60]);
          Passive.Inertia load(J=50) annotation (extent=[70,40; 90,60]);
          Passive.Inertia housing(J=5) 
            annotation (extent=[10,0; 30,20], rotation=90);
          Passive.SpringDamper springDamper(c=1e5, d=5) 
            annotation (extent=[10,-40; 30,-20], rotation=90);
          Passive.Fixed fixed annotation (extent=[10,-80; 30,-60]);
        equation 
          connect(ramp.y, torque.tau) annotation (points=[-69,50; -51,50],
              style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(torque.flange_b, shaft.flange_a) annotation (points=[-30,50;
                -20,50], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(shaft.flange_b, idealGear.flange_a) annotation (points=[0,50;
                10,50], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(idealGear.flange_b, spring.flange_a) annotation (points=[30,
                50; 40,50], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(spring.flange_b, load.flange_a) annotation (points=[60,50; 70,
                50], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(housing.flange_b, idealGear.bearing) annotation (points=[20,
                20; 20,40], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(springDamper.flange_b, housing.flange_a) annotation (points=[
                20,-20; 20,0], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(fixed.flange_b, springDamper.flange_a) annotation (points=[20,
                -70; 20,-40], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
        end ElasticBearing;
        
        model CoupledClutches "Drive train with 3 dynamically coupled clutches" 
          parameter Modelica.SIunits.Frequency freqHz=0.2 
            "frequency of sine function to invoke clutch1";
          parameter Modelica.SIunits.Time T2=0.4 "time when clutch2 is invoked";
          parameter Modelica.SIunits.Time T3=0.9 "time when clutch3 is invoked";
          
          annotation (
            Icon(
              Polygon(points=[-30,40; -60,50; -60,30; -30,40],     style(fillColor=3,
                     fillPattern=1)),
              Line(points=[0,90; -90,70; -90,40; -30,40],     style(
                  pattern=1,
                  thickness=1,
                  arrow=0)),
              Line(points=[0,90; 90,70; 90,40; 30,40]),
              Polygon(points=[30,40; 60,50; 60,30; 30,40],     style(fillColor=3,
                    fillPattern=1)),
              Rectangle(extent=[10,60; 30,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-30,60; -10,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[30,10; 100,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[-100,10; -30,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Ellipse(extent=[-110,10; -90,-10], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=10,
                  rgbfillColor={135,135,135})),
              Ellipse(extent=[90,10; 110,-10], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255}))),
            Diagram,
            Coordsys(extent=[-120,-100; 180,100]),
            Documentation(info="<html>
<p>This example demonstrates how variable structure
drive trains are handeled. The drive train consists
of 4 inertias and 3 clutches, where the clutches
are controlled by input signals. The system has
2^3=8 different configurations and 3^3 = 27
different states (every clutch may be in forward
sliding, backward sliding or locked mode when the
relative angular velocity is zero). By invoking the
clutches at different time instances, the switching
of the configurations can be studied.</p>
<p>Simulate the system for 1.2 seconds with the
following initial values:<br>
J1.w = 10.</p>
<p>Plot the following variables:<br>
angular velocities of inertias (J1.w, J2.w, J3.w,
J4.w), frictional torques of clutches (clutchX.tau),
frictional mode of clutches (clutchX.mode) where
mode = -1/0/+1 means backward sliding,
locked, forward sliding.</p>
<p><b>Release Notes:</b></p>
<ul><li><i>June 30, 1999</i>
        by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
        Realized.</li>
</ul>
</html>"),  experiment(StopTime=1.2),
            experimentSetupOutput);
          Modelica.Blocks.Sources.Step step2(
                             startTime=T3) 
            annotation (extent=[110,20; 130,40], rotation=-90);
          Modelica.Blocks.Sources.Step step1(
                             startTime=T2) 
            annotation (extent=[50,20; 70,40],   rotation=-90);
          Modelica.Blocks.Sources.Sine sin2(
            amplitude=1,
            freqHz=freqHz,
            phase=1.57)   annotation (extent=[-10,20; 10,40],    rotation=-90);
          Modelica.Blocks.Sources.Sine sin1(
                            amplitude=10, freqHz=5) 
            annotation (extent=[-100,-10; -80,10]);
          Sources.Torque torque annotation (extent=[-70,-10; -50,10]);
          Passive.Inertia J1(w(start=10)) annotation (extent=[-40,-10; -20,10]);
          Passive.Clutch clutch1(peak=1.1, fn_max=20) 
            annotation (extent=[-10,-10; 10,10]);
          Passive.Clutch clutch2(peak=1.1, fn_max=20) 
            annotation (extent=[50,-10; 70,10]);
          Passive.Inertia J2 annotation (extent=[20,-10; 40,10]);
          Passive.Inertia J3 annotation (extent=[80,-10; 100,10]);
          Passive.Clutch clutch3(peak=1.1, fn_max=20) 
            annotation (extent=[110,-10; 130,10]);
          Passive.Inertia J4 annotation (extent=[140,-10; 160,10]);
        equation 
          connect(torque.tau, sin1.y) annotation (points=[-71,0; -79,0], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(J1.flange_a, torque.flange_b) annotation (points=[-40,0; -50,
                0], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(J1.flange_b, clutch1.flange_a) annotation (points=[-20,0; -10,
                0], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(sin2.y, clutch1.f_normalized) annotation (points=[
                6.73533e-016,19; 6.73533e-016,14.5; 0,14.5; 0,10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(J2.flange_a, clutch1.flange_b) annotation (points=[20,0; 10,0],
              style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(J2.flange_b, clutch2.flange_a) annotation (points=[40,0; 50,0],
              style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(J3.flange_a, clutch2.flange_b) annotation (points=[80,0; 70,0],
              style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(J3.flange_b, clutch3.flange_a) annotation (points=[100,0; 110,
                0], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(step2.y, clutch3.f_normalized) annotation (points=[120,19;
                120,10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(clutch3.flange_b, J4.flange_a) annotation (points=[130,0; 140,
                0], style(
              color=0,
              rgbcolor={0,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(step1.y, clutch2.f_normalized) annotation (points=[60,19; 60,
                10], style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
        end CoupledClutches;
        
        model Friction "Drive train with clutch and brake" 
          import Modelica.Constants.pi;
          parameter Modelica.SIunits.Time startTime=0.5 "Start time of step";
          
          output Modelica.SIunits.Torque tMotor "driving torque of inertia3";
          output Modelica.SIunits.Torque tClutch "friction torque of clutch";
          output Modelica.SIunits.Torque tBrake "friction torque of brake";
          output Modelica.SIunits.Torque tSpring "spring torque";
          
          Modelica.Blocks.Sources.Step step2(
            height=-1,
            offset=1,
            startTime=startTime)   annotation (extent=[-100,-30; -80,-10]);
          Modelica.Blocks.Sources.Sine sine(
                            amplitude=200, freqHz=50/pi) 
            annotation (extent=[-100,10; -80,30]);
          Modelica.Blocks.Math.Product product 
                               annotation (extent=[-60,-10; -40,10]);
          Modelica.Blocks.Sources.Constant const(
                                 k=1) 
            annotation (extent=[30,20; 50,40],  rotation=-90);
          Modelica.Blocks.Sources.Step step(
                            startTime=startTime) 
            annotation (extent=[120,20; 140,40], rotation=-90);
          annotation (
            Diagram,
            Icon(   Rectangle(extent=[-20,-80; 20,-120],   style(color=8, fillColor=
                     8)),
              Polygon(points=[-37,-55; -37,-90; 37,-90; 37,-55; 33,-55; 33,-86;
                    -33,-86; -33,-55; -37,-55],     style(color=8, fillColor=8)),
              Text(
                extent=[14,-63; 82,-76],
                string="rotation axis",
                style(color=10)),
              Polygon(points=[0,-70; -20,-65; -20,-75; 0,-70],       style(color=10,
                    fillColor=10)),
              Line(points=[-90,-70; -19,-70],  style(color=10, fillColor=10)),
              Rectangle(extent=[-100,10; -20,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Rectangle(extent=[20,10; 100,-10],  style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[0,90; -80,70; -80,-40; -70,-40],     style(
                  pattern=1,
                  thickness=1,
                  arrow=0)),
              Rectangle(extent=[-20,60; 20,-60],   style(
                  gradient=2,
                  fillColor=8,
                  fillPattern=1)),
              Line(points=[0,90; 80,70; 80,-40; 70,-40]),
              Polygon(points=[40,-40; 70,-30; 70,-50; 40,-40],     style(fillColor=3,
                     fillPattern=1)),
              Polygon(points=[-40,-40; -70,-30; -70,-50; -40,-40],     style(
                    fillColor=3, fillPattern=1)),
              Rectangle(extent=[-40,-25; -30,-55],   style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Rectangle(extent=[30,-25; 40,-55],   style(
                  color=0,
                  fillColor=0,
                  fillPattern=1)),
              Ellipse(extent=[-110,10; -90,-10], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=10,
                  rgbfillColor={135,135,135})),
              Ellipse(extent=[-10,-90; 10,-110], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=10,
                  rgbfillColor={135,135,135})),
              Ellipse(extent=[90,10; 110,-10], style(
                  color=0,
                  rgbcolor={0,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255}))),
            Coordsys(extent=[-120,-100; 180,100]),
            Documentation(info="<html>
<p>
This drive train contains a frictional <b>clutch</b> and a <b>brake</b>.
Simulate the system for 1 second using the following initial
values (defined already in the model):</p>
<pre>   inertia1.w =  90 (or brake.w)
   inertia2.w =  90
   inertia3.w = 100
</pre>
<p>Plot the output signals</p>
<pre>   tMotor      Torque of motor
   tClutch     Torque in clutch
   tBrake      Torque in brake
   tSpring     Torque in spring
</pre>
<p>as well as the absolute angular velocities of the three inertia components
(inertia1.w, inertia2.w, inertia3.w).</p>
<p><b>Release Notes:</b></p>
<ul><li><i>June 30, 1999</i>
    by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
    Realized.</li>
</ul>
</html>"),  experiment,
            experimentSetupOutput);
          Sources.Torque torque annotation (extent=[-30,-10; -10,10]);
          Passive.Inertia inertia3(w(start=100)) 
            annotation (extent=[0,-10; 20,10]);
          Passive.Clutch clutch(fn_max=160) annotation (extent=[30,-10; 50,10]);
          Passive.Inertia inertia2(w(start=90), J=0.05) 
            annotation (extent=[60,-10; 80,10]);
          Passive.SpringDamper spring(c=160, d=1) 
            annotation (extent=[90,-10; 110,10]);
          Passive.Brake brake(fn_max=1600) annotation (extent=[120,-10; 140,10]);
          Passive.Inertia inertia1(w(start=90)) 
            annotation (extent=[150,-10; 170,10]);
        equation 
          tMotor = torque.tau;
          tClutch = clutch.tau;
          tBrake = brake.tau;
          tSpring = spring.tau;
          connect(sine.y, product.u1) annotation (points=[-79,20; -70,20; -70,6;
                -62,6], style(color=74, rgbcolor={0,0,127}));
          connect(step2.y, product.u2) annotation (points=[-79,-20; -70,-20;
                -70,-6; -62,-6], style(color=74, rgbcolor={0,0,127}));
          connect(product.y, torque.tau) annotation (points=[-39,0; -31,0],
              style(color=74, rgbcolor={0,0,127}));
          connect(torque.flange_b, inertia3.flange_a) 
            annotation (points=[-10,0; 0,0], style(color=0, rgbcolor={0,0,0}));
          connect(clutch.flange_a, inertia3.flange_b) 
            annotation (points=[30,0; 20,0], style(color=0, rgbcolor={0,0,0}));
          connect(const.y, clutch.f_normalized) annotation (points=[40,19; 40,
                10; 40,10], style(color=74, rgbcolor={0,0,127}));
          connect(inertia2.flange_a, clutch.flange_b) 
            annotation (points=[60,0; 50,0], style(color=0, rgbcolor={0,0,0}));
          connect(spring.flange_a, inertia2.flange_b) 
            annotation (points=[90,0; 80,0], style(color=0, rgbcolor={0,0,0}));
          connect(brake.flange_a, spring.flange_b) annotation (points=[120,0;
                110,0], style(color=0, rgbcolor={0,0,0}));
          connect(step.y, brake.f_normalized) annotation (points=[130,19; 130,
                10], style(color=74, rgbcolor={0,0,127}));
          connect(inertia1.flange_a, brake.flange_b) annotation (points=[150,0;
                140,0], style(color=0, rgbcolor={0,0,0}));
        end Friction;
      end Examples;
    end Rotational;
  end Mechanical;
  
  package Hydraulic "Hydraulic components built from bond graphs" 
    annotation (preferedView="info",
      Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
            color=42,
            fillColor=30,
            fillPattern=1)),
        Text(
          extent=[-85, 35; 65, -85],
          string="Library",
          style(color=42)),
        Text(
          extent=[-120, 122; 120, 73],
          string="%name",
          style(color=1))),
      Window(
        x=0.02,
        y=0,
        width=0.44,
        height=0.65,
        library=1,
        autolayout=1),
      Documentation(info="<html>
This package contains a collection of components used in hydraulic systems that are built from bond graph technology.  The set of elements is by no means exhaustive.  It currently contains only one type of hydraulic motor together with its servo valve.
</html>"));
    block Modulator "Modulator needed by the servo valve" 
      parameter Real k(unit="m^3.5/kg^0.5") = 1 "Hydraulic valve flow constant";
      parameter Real x0=0 "Percentage of mechanical valve underlap";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-116, 18; -90, 18], style(color=0)),
          Line(points=[-96, 22; -90, 18; -96, 14], style(color=0)),
          Text(
            extent=[-110, 34; -98, 20],
            string="x",
            style(color=0)),
          Text(
            extent=[4, 70; 16, 62],
            string="x0+x",
            style(color=9)),
          Line(points=[84, 78; 110, 78], style(color=0)),
          Line(points=[104, 82; 110, 78; 104, 74], style(color=0)),
          Line(points=[84, -44; 110, -44], style(color=0)),
          Line(points=[104, -40; 110, -44; 104, -48], style(color=0)),
          Text(
            extent=[76, 94; 118, 82],
            string="k*(x0+x)",
            style(color=0)),
          Text(
            extent=[76, -26; 118, -38],
            string="k*(x0-x)",
            style(color=0)),
          Text(
            extent=[-26, 8; -10, -8],
            string="x0",
            style(color=9))),
        Icon(
          Line(points=[-90, 0; -40, 0], style(color=74, rgbcolor={0,0,135})),
          Text(
            extent=[-28, 10; -10, -10],
            string="k",
            style(color=9)),
          Polygon(points=[-40, 40; -40, -40; 20, 0; -40, 40], style(color=74,
                rgbcolor={0,0,135})),
          Line(points=[20, 0; 40, 0; 40, 60; 90, 60], style(color=74, rgbcolor=
                  {0,0,135})),
          Line(points=[20, 0; 40, 0; 40, -60; 90, -60], style(color=74,
                rgbcolor={0,0,135})),
          Text(
            extent=[80, 100; 120, 62],
            string="+",
            style(color=9)),
          Text(
            extent=[80, -20; 120, -60],
            string="-",
            style(color=9)),
          Text(extent=[-74, 122; 72, 80], string="%name")),
        Documentation(info="<html>
The <b>Modulator</b> block is needed by the servo valve.  Each of the four valve flows, <b>q<sub>1</sub></b>, <b>q<sub>2</sub></b>, <b>q<sub>3</sub></b>, and <b>q<sub>4</sub></b>, is characterized by an underlap, <b>x<sub>0</sub></b>.  Each of the four flows is proportional to either <b>x<sub>0</sub> + x</b> or <b>x<sub>0</sub> - x</b>, depending on the topology.  These two proportionality factors are computed by the <b>Modulator</b> block.  They are used as modulation signals in the four hydraulic conductor models representing the turbulent valve flows.
</html>
"));
      Modelica.Blocks.Interfaces.RealInput u    annotation (extent=[-110, -10;
            -90, 10]);
      Modelica.Blocks.Interfaces.RealOutput y1    annotation (extent=[90, 50;
            110, 70]);
      Modelica.Blocks.Interfaces.RealOutput y2    annotation (extent=[90, -70;
            110, -50]);
      Modelica.Blocks.Math.Gain k1(k=k)   annotation (extent=[60, 50; 80, 70]);
      Modelica.Blocks.Math.Add A1 annotation (extent=[-20, 50; 0, 70]);
      Modelica.Blocks.Math.Gain k2(k=k)   annotation (extent=[60, -70; 80, -50]);
      Modelica.Blocks.Math.Add A2(k2=-1) annotation (extent=[-20, -70; 0, -50]);
      Modelica.Blocks.Sources.Constant X0(k=x0)   annotation (extent=[-60, -10;
              -40, 10]);
      Modelica.Blocks.Nonlinear.Limiter L1(uMin=0)   annotation (extent=[20, 50;
              40, 70]);
      Modelica.Blocks.Nonlinear.Limiter L2(uMin=0)   annotation (extent=[20, -
            70; 40, -50]);
    equation 
      connect(u, A1.u1)            annotation (points=[-100,0; -80,0; -80,66;
            -22,66], style(color=74, rgbcolor={0,0,135}));
      connect(u, A2.u2)            annotation (points=[-100,0; -80,0; -80,-66;
            -22,-66], style(color=74, rgbcolor={0,0,135}));
      connect(X0.y,A1.u2)             annotation (points=[-39,0; -30,0; -30,54;
            -22,54], style(color=74, rgbcolor={0,0,135}));
      connect(X0.y,A2.u1)             annotation (points=[-39,0; -30,0; -30,-54;
            -22,-54], style(color=74, rgbcolor={0,0,135}));
      connect(A1.y,L1.u)             annotation (points=[1,60; 18,60], style(
            color=74, rgbcolor={0,0,135}));
      connect(L1.y,k1.u)             annotation (points=[41,60; 58,60], style(
            color=74, rgbcolor={0,0,135}));
      connect(k1.y, y1)             annotation (points=[81,60; 100,60], style(
            color=74, rgbcolor={0,0,135}));
      connect(A2.y,L2.u)             annotation (points=[1,-60; 18,-60], style(
            color=74, rgbcolor={0,0,135}));
      connect(L2.y,k2.u)             annotation (points=[41,-60; 58,-60], style(
            color=74, rgbcolor={0,0,135}));
      connect(k2.y, y2)             annotation (points=[81,-60; 100,-60], style(
            color=74, rgbcolor={0,0,135}));
    end Modulator;
    
    model Servo "Servo mechanism built from bond graph technology" 
      extends Modelica.Blocks.Interfaces.SISO;
      parameter Modelica.SIunits.Resistance rs=1 "Servo resistance";
      parameter Modelica.SIunits.Inductance ls=1 "Servo inductance";
      parameter Real bs(unit="kg/s") = 1 "Servo damper";
      parameter Modelica.SIunits.Mass ms=1 "Servo mass";
      parameter Real ks(unit="kg/s2") = 1 "Servo spring";
      parameter Real psis(unit="V.s") = 1 "Servo induction constant";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram,
        Icon(
          Rectangle(extent=[-100, -100; 100, 100], style(color=3, fillColor=7)),
          Rectangle(extent=[100, 6; -48, -6], style(fillColor=73)),
          Line(points=[12, 14; -16, -14], style(color=41, thickness=2)),
          Line(points=[16, 14; -12, -14], style(color=41, thickness=2)),
          Line(points=[20, 14; -8, -14], style(color=41, thickness=2)),
          Line(points=[24, 14; -4, -14], style(color=41, thickness=2)),
          Line(points=[28, 14; 0, -14], style(color=41, thickness=2)),
          Line(points=[28, 16; -60, 16], style(color=41, thickness=2)),
          Line(points=[28, 16; 28, 14], style(color=41, thickness=2)),
          Line(points=[-16, -14; -60, -14], style(color=41, thickness=2)),
          Ellipse(extent=[-62, 18; -58, 14], style(fillColor=49)),
          Ellipse(extent=[-62, -12; -58, -16], style(fillColor=49)),
          Line(points=[-68, 10; -68, -10]),
          Line(points=[-70, -6; -68, -10; -66, -6]),
          Text(extent=[-78, 4; -68, -4], string="u"),
          Line(points=[-138, 26; -108, 26], style(color=0)),
          Line(points=[-112, 30; -108, 26; -112, 22], style(color=0)),
          Text(
            extent=[-130, 36; -120, 28],
            string="u",
            style(color=0)),
          Line(points=[92, 16; 122, 16], style(color=0)),
          Line(points=[118, 20; 122, 16; 118, 12], style(color=0)),
          Text(
            extent=[102, 26; 112, 18],
            string="x",
            style(color=0)),
          Rectangle(extent=[-34, 20; -16, 12], style(fillColor=49)),
          Text(extent=[-28, 30; -20, 22], string="R"),
          Text(extent=[-10, -18; -2, -26], string="L"),
          Rectangle(extent=[-50, 48; -48, -48], style(fillColor=73)),
          Line(points=[-50, 38; -62, 38], style(thickness=2)),
          Rectangle(extent=[-62, 44; -68, 32], style(pattern=0, fillColor=55)),
          Line(points=[-62, 42; -62, 34], style(thickness=2)),
          Line(points=[-58, 44; -68, 44; -68, 32; -58, 32], style(thickness=2)),
          Line(points=[-68, 38; -100, 38], style(thickness=2)),
          Text(extent=[-68, 54; -60, 46], string="B"),
          Line(points=[-50, -30; -60, -30; -62, -34; -64, -26; -66, -34; -68, -
                26; -70, -34; -72, -30; -100, -30], style(thickness=2)),
          Text(extent=[-70, -36; -62, -44], string="k")),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The servo mechanism described here has a signal input, the armature voltage, <b>u</b>, and a signal output, the position of the tongue of the servo valve, <b>x</b>. <p>

The input signal is converted to a bond graphic power signal using a modulated effort source.  The spring force is measured in the bond graph using an effort sensor.  It is then converted to the tongue position using the amplifier model of Modelica's block diagram library.
</html>
"));
      Sources.mSe mSe1 annotation (extent=[-70, -10; -90, 10]);
      Bonds.fBond B1 annotation (extent=[-70, -10; -50, 10]);
      Junctions.J1p4 J1p4_1 annotation (extent=[-50, -10; -30, 10]);
      Bonds.eBond B2 annotation (extent=[-50, 10; -30, 30], rotation=90);
      Passive.R Rs(R=rs) annotation (extent=[-50, 30; -30, 50], rotation=90);
      Bonds.fBond B3 annotation (extent=[-50, -30; -30, -10], rotation=-90);
      Passive.I Ls(I=ls) annotation (extent=[-50, -50; -30, -30], rotation=-90);
      Passive.GY GY1(r=psis) annotation (extent=[-10, -10; 10, 10]);
      Bonds.eBond B4 annotation (extent=[-30, -10; -10, 10]);
      Bonds.fBond B5 annotation (extent=[10, -10; 30, 10]);
      Junctions.J1p4 J1p4_2 annotation (extent=[30, -10; 50, 10]);
      Bonds.eBond B6 annotation (extent=[30, 10; 50, 30], rotation=90);
      Passive.R Bs(R=bs) annotation (extent=[30, 30; 50, 50], rotation=90);
      Bonds.fBond B7 annotation (extent=[30, -30; 50, -10], rotation=-90);
      Passive.I Ms(I=ms) annotation (extent=[30, -50; 50, -30], rotation=-90);
      Bonds.eBond B8 annotation (extent=[50, -10; 70, 10]);
      Passive.C Ks(C=1/ks) annotation (extent=[70, -50; 90, -30], rotation=-90);
      Modelica.Blocks.Math.Gain Gain1(k=1/ks)   annotation (extent=[70, 70; 90,
              90]);
      Junctions.J0p3 J0p3_1 annotation (extent=[70, -10; 90, 10], rotation=-90);
      Bonds.eBond B9 annotation (extent=[70, -30; 90, -10], rotation=-90);
      Bonds.fBond B10 annotation (extent=[70, 10; 90, 30], rotation=90);
      Sensors.De De1 annotation (extent=[70, 30; 90, 50], rotation=90);
    equation 
      connect(mSe1.BondCon1, B1.eBondCon1) annotation (points=[-70, 0; -70, 0],
            style(color=8));
      connect(B1.fBondCon1, J1p4_1.BondCon1) annotation (points=[-50, 0; -50, 0],
               style(color=8));
      connect(J1p4_1.BondCon4, B2.fBondCon1) annotation (points=[-40, 10; -40,
            10], style(color=8));
      connect(B2.eBondCon1, Rs.BondCon1) annotation (points=[-40, 30; -40, 30],
            style(color=8));
      connect(J1p4_1.BondCon3, B3.eBondCon1) annotation (points=[-40, -10; -40,
              -10], style(color=8));
      connect(B3.fBondCon1, Ls.BondCon1) annotation (points=[-40, -30; -40, -30],
               style(color=8));
      connect(J1p4_1.BondCon2, B4.fBondCon1) annotation (points=[-30, 0; -30, 0],
               style(color=8));
      connect(B4.eBondCon1, GY1.BondCon1) annotation (points=[-10, 0; -10, 0],
          style(color=8));
      connect(GY1.BondCon2, B5.eBondCon1) annotation (points=[10, 0; 10, 0],
          style(color=8));
      connect(B5.fBondCon1, J1p4_2.BondCon1) annotation (points=[30, 0; 30, 0],
            style(color=8));
      connect(J1p4_2.BondCon4, B6.fBondCon1) annotation (points=[40, 10; 40, 10],
               style(color=8));
      connect(B6.eBondCon1, Bs.BondCon1) annotation (points=[40, 30; 40, 30],
          style(color=8));
      connect(J1p4_2.BondCon3, B7.eBondCon1) annotation (points=[40, -10; 40, -
            10], style(color=8));
      connect(B7.fBondCon1, Ms.BondCon1) annotation (points=[40, -30; 40, -30],
            style(color=8));
      connect(J1p4_2.BondCon2, B8.fBondCon1) annotation (points=[50, 0; 50, 0],
            style(color=8));
      connect(Gain1.y,y)              annotation (points=[91,80; 96,80; 96,0;
            110,0], style(color=74, rgbcolor={0,0,135}));
      connect(B8.eBondCon1, J0p3_1.BondCon3) annotation (points=[70,0; 70,
            -6.12303e-016],
            style(color=8));
      connect(J0p3_1.BondCon2, B9.fBondCon1) annotation (points=[80, -10; 80, -
            10], style(color=8));
      connect(B9.eBondCon1, Ks.BondCon1) annotation (points=[80, -30; 80, -30],
            style(color=8));
      connect(J0p3_1.BondCon1, B10.eBondCon1) annotation (points=[80, 10; 80,
            10], style(color=8));
      connect(B10.fBondCon1, De1.BondCon1) annotation (points=[80, 30; 80, 30],
            style(color=8));
      connect(De1.OutPort1,Gain1.u)       annotation (points=[80,51; 80,60; 60,
            60; 60,80; 68,80], style(color=74, rgbcolor={0,0,135}));
      connect(mSe1.s, u) annotation (points=[-89,0; -120,0], style(color=74,
            rgbcolor={0,0,127}));
    end Servo;
    
    model Servovalve "Servo valve built from bond graph technology" 
      parameter Real k(unit="m^3.5/kg^0.5") = 1 "Hydraulic valve flow constant";
      parameter Real x0=0 "Percentage of mechanical valve underlap";
      parameter Modelica.SIunits.Pressure ps=1 "System pressure";
      parameter Modelica.SIunits.Pressure p0=0 "Environmental pressure";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Line(points=[-108, 14; -88, 14], style(color=0)),
          Text(
            extent=[-104, 24; -94, 14],
            string="x",
            style(color=0)),
          Line(points=[-92, 16; -88, 14; -92, 12], style(color=0)),
          Text(
            extent=[-78, -74; -72, -82],
            string="p1",
            style(color=9)),
          Text(
            extent=[-90, -80; -82, -88],
            string="qL1",
            style(color=9)),
          Text(
            extent=[-62, -60; -56, -68],
            string="p1",
            style(color=9)),
          Text(
            extent=[-62, -48; -56, -56],
            string="q1",
            style(color=9)),
          Text(
            extent=[-22, -60; -14, -70],
            style(color=9),
            string="ps"),
          Text(
            extent=[-22, -48; -16, -56],
            string="q1",
            style(color=9)),
          Text(
            extent=[14, -50; 22, -60],
            style(color=9),
            string="ps"),
          Text(
            extent=[16, -62; 22, -70],
            string="q4",
            style(color=9)),
          Text(
            extent=[56, -52; 62, -60],
            string="p2",
            style(color=9)),
          Text(
            extent=[56, -62; 62, -70],
            string="q4",
            style(color=9)),
          Text(
            extent=[72, -78; 78, -86],
            string="p2",
            style(color=9)),
          Text(
            extent=[82, -80; 90, -88],
            string="qL2",
            style(color=9)),
          Text(
            extent=[-54, -36; -40, -42],
            style(color=9),
            string="ps-p1"),
          Text(
            extent=[-36, -36; -30, -44],
            string="q1",
            style(color=9)),
          Text(
            extent=[2, -32; 10, -42],
            style(color=9),
            string="ps"),
          Text(
            extent=[-18, -32; -4, -42],
            string="q1+q4",
            style(color=9)),
          Text(
            extent=[44, -36; 50, -44],
            string="q4",
            style(color=9)),
          Text(
            extent=[24, -38; 38, -44],
            style(color=9),
            string="ps-p2"),
          Text(
            extent=[-64, 68; -58, 60],
            string="p1",
            style(color=9)),
          Text(
            extent=[-64, 58; -58, 50],
            string="q2",
            style(color=9)),
          Text(
            extent=[-50, 44; -44, 36],
            string="q2",
            style(color=9)),
          Text(
            extent=[-38, 46; -24, 40],
            style(color=9),
            string="p1-p0"),
          Text(
            extent=[-26, 70; -18, 60],
            style(color=9),
            string="p0"),
          Text(
            extent=[-24, 58; -18, 50],
            string="q2",
            style(color=9)),
          Text(
            extent=[2, 48; 10, 38],
            style(color=9),
            string="p0"),
          Text(
            extent=[-18, 48; -4, 38],
            string="q2+q3",
            style(color=9)),
          Text(
            extent=[18, 60; 26, 50],
            style(color=9),
            string="p0"),
          Text(
            extent=[16, 72; 22, 64],
            string="q3",
            style(color=9)),
          Text(
            extent=[30, 44; 36, 36],
            string="q3",
            style(color=9)),
          Text(
            extent=[42, 46; 56, 40],
            style(color=9),
            string="p2-p0"),
          Text(
            extent=[58, 60; 64, 52],
            string="p2",
            style(color=9)),
          Text(
            extent=[58, 72; 64, 64],
            string="q3",
            style(color=9))),
        Icon(
          Polygon(points=[-80, -90; -32, -60; -32, -42; -40, -42; -40, -2; -8,
                -2; -8, -38; 8, -38; 8, -2; 40, -2; 40, -42; 32, -42; 32, -60;
                80, -90; 74, -94; 20, -60; 20, -42; -20, -42; -20, -60; -72, -
                94; -80, -90], style(pattern=0, fillColor=65)),
          Polygon(points=[-56, 66; -56, 38; -40, 38; -40, 2; -8, 2; -8, 38; 8,
                38; 8, 2; 40, 2; 40, 38; 56, 38; 56, 66; 40, 66; 40, 42; 8, 42;
                  8, 66; -8, 66; -8, 42; -40, 42; -40, 66; -56, 66], style(
                pattern=0, fillColor=65)),
          Line(points=[-108, 14; -88, 14], style(color=0)),
          Text(
            extent=[-104, 24; -94, 14],
            string="x",
            style(color=0)),
          Rectangle(extent=[-88, 2; 100, -2], style(fillColor=73)),
          Rectangle(extent=[-8, 38; 8, -38], style(fillColor=73)),
          Rectangle(extent=[-56, 38; -40, -38], style(fillColor=73)),
          Rectangle(extent=[40, 38; 56, -38], style(fillColor=73)),
          Line(points=[-80, 4; -68, 4; -68, 42; -56, 42; -56, 66]),
          Line(points=[-40, 66; -40, 42; -8, 42; -8, 66]),
          Line(points=[8, 66; 8, 42; 40, 42; 40, 66]),
          Line(points=[56, 66; 56, 42; 68, 42; 68, 4; 80, 4]),
          Line(points=[-80, -4; -68, -4; -68, -42; -32, -42; -32, -60; -80, -90]),
          Line(points=[80, -4; 68, -4; 68, -42; 32, -42; 32, -60; 80, -90]),
          Line(points=[-72, -94; -20, -60; -20, -42; 20, -42; 20, -60; 74, -94]),
          Line(points=[-48, 48; -48, 72], style(color=0)),
          Line(points=[48, 48; 48, 72], style(color=0)),
          Line(points=[0, 72; 0, 48], style(color=0)),
          Text(
            extent=[-56, 86; -40, 74],
            string="p0",
            style(color=0)),
          Text(
            extent=[40, 86; 56, 74],
            string="p0",
            style(color=0)),
          Text(
            extent=[-8, 86; 8, 74],
            string="ps",
            style(color=0)),
          Line(points=[-34, 36; -46, 46], style(color=0)),
          Line(points=[-4, 44; -14, 36], style(color=0)),
          Line(points=[4, 44; 14, 36], style(color=0)),
          Line(points=[34, 36; 46, 46], style(color=0)),
          Text(
            extent=[-38, 36; -28, 28],
            string="q2",
            style(color=0)),
          Text(
            extent=[-22, 36; -12, 28],
            string="q1",
            style(color=0)),
          Text(
            extent=[12, 36; 22, 28],
            string="q4",
            style(color=0)),
          Text(
            extent=[28, 36; 38, 28],
            string="q3",
            style(color=0)),
          Line(points=[26, -40; 26, -58], style(color=0)),
          Line(points=[-26, -58; -26, -40], style(color=0)),
          Text(
            extent=[20, -32; 32, -40],
            string="qL2",
            style(color=0)),
          Text(
            extent=[-32, -32; -20, -40],
            string="qL1",
            style(color=0)),
          Line(points=[-50, 68; -48, 72; -46, 68], style(color=0)),
          Line(points=[-2, 52; 0, 48; 2, 52], style(color=0)),
          Line(points=[46, 68; 48, 72; 50, 68], style(color=0)),
          Line(points=[-44, 42; -46, 46; -42, 44], style(color=0)),
          Line(points=[-14, 40; -14, 36; -10, 36], style(color=0)),
          Line(points=[12, 40; 14, 36; 10, 36], style(color=0)),
          Line(points=[42, 44; 46, 46; 44, 42], style(color=0)),
          Line(points=[24, -44; 26, -40; 28, -44], style(color=0)),
          Line(points=[-28, -54; -26, -58; -24, -54], style(color=0)),
          Text(extent=[-80, 132; 80, 84], string="%name"),
          Line(points=[-92, 16; -88, 14; -92, 12], style(color=0)),
          Text(
            extent=[-30, -8; -14, -20],
            string="p1",
            style(color=0)),
          Text(
            extent=[18, -8; 34, -20],
            string="p2",
            style(color=0))),
        Window(
          x=0.45,
          y=0.01,
          width=0.44,
          height=0.65),
        Documentation(info="<html>
The servo valve model described here has a signal input, the position of the tongue, <b>x</b>, and two bondgraphic outputs: the two load flows, <b>q<sub>L1</sub></b> and <b>q<sub>L2</sub></b>.  The bondgraphic interface is designed such that the servo valve can be directly connected to the hydraulic motor without either a set of bonds or a set of junctions in between. <p>

The two chambers of the servo valve are at the pressures <b>p<sub>1</sub></b> and <b>p<sub>2</sub></b>, respectively. <p>

The four valve flows, <b>q<sub>1</sub></b>, <b>q<sub>2</sub></b>, <b>q<sub>3</sub></b>, and <b>q<sub>4</sub></b>, are assumed to be turbulent flows.  The valve flows are modulated by the tongue position.  The modulation of the four hydraulic conductors representing the turbulent flows is computed by the <b>Modulator</b> block.
</html>

"));
      Modelica.Blocks.Interfaces.RealInput x    annotation (extent=[-108, -10;
            -88, 10]);
      Interfaces.BondCon BondCon1 annotation (extent=[-90, -110; -70, -90]);
      Interfaces.BondCon BondCon2 annotation (extent=[70, -110; 90, -90]);
      Modulator M1(k=k, x0=x0) annotation (extent=[-80, 10; -60, -10]);
      Bonds.eBond B6 annotation (extent=[-90, -90; -70, -70], rotation=-90);
      Junctions.J0p3 J0p3_1 annotation (extent=[-90, -70; -70, -50]);
      Bonds.eBond B2 annotation (extent=[-70, -70; -50, -50], rotation=180);
      Junctions.J1p3 J1p3_1 annotation (extent=[-50, -50; -30, -70]);
      Bonds.fBond B3 annotation (extent=[-30, -70; -10, -50], rotation=180);
      Junctions.J0p3 J0p3_2 annotation (extent=[-10, -50; 10, -70]);
      Bonds.fBond B4 annotation (extent=[10, -70; 30, -50]);
      Junctions.J1p3 J1p3_2 annotation (extent=[30, -50; 50, -70]);
      Bonds.eBond B5 annotation (extent=[50, -70; 70, -50]);
      Bonds.fBond B1 annotation (extent=[70, -90; 90, -70], rotation=90);
      Junctions.J0p3 J0p3_3 annotation (extent=[70, -70; 90, -50]);
      Bonds.fBond B7 annotation (extent=[-50, -50; -30, -30], rotation=90);
      Bonds.fBond B8 annotation (extent=[-10, -50; 10, -30], rotation=-90);
      Bonds.fBond B9 annotation (extent=[30, -50; 50, -30], rotation=90);
      Sources.Se PS(e0=ps) annotation (extent=[-10, -30; 10, -10], rotation=90);
      Passive.HydromG mG1 annotation (extent=[-50, -30; -30, -10], rotation=90);
      Passive.HydromG mG4 annotation (extent=[30, -30; 50, -10], rotation=90);
      Bonds.fBond B15 annotation (extent=[-70, 50; -50, 70]);
      Junctions.J1p3 J1p3_3 annotation (extent=[-50, 50; -30, 70]);
      Bonds.fBond B12 annotation (extent=[-50, 30; -30, 50], rotation=-90);
      Passive.HydromG mG2 annotation (extent=[-50, 10; -30, 30], rotation=-90);
      Bonds.eBond B11 annotation (extent=[-30, 50; -10, 70]);
      Junctions.J0p3 J0p3_4 annotation (extent=[-10, 50; 10, 70]);
      Bonds.eBond B10 annotation (extent=[-10, 30; 10, 50], rotation=-90);
      Sources.Se P0(e0=p0) annotation (extent=[-10, 10; 10, 30], rotation=-90);
      Bonds.eBond B13 annotation (extent=[10, 50; 30, 70], rotation=180);
      Junctions.J1p3 J1p3_4 annotation (extent=[30, 50; 50, 70]);
      Bonds.fBond B14 annotation (extent=[30, 30; 50, 50], rotation=-90);
      Passive.HydromG mG3 annotation (extent=[30, 10; 50, 30], rotation=-90);
      Bonds.fBond B16 annotation (extent=[50, 50; 70, 70], rotation=180);
    equation 
      connect(B6.eBondCon1, BondCon1) annotation (points=[-80,-90; -80,-100;
            -80,-100],style(color=8));
      connect(J0p3_1.BondCon3, B6.fBondCon1) annotation (points=[-80, -70; -80,
              -70], style(color=8));
      connect(B2.eBondCon1, J0p3_1.BondCon2) annotation (points=[-70, -60; -70,
              -60], style(color=8));
      connect(J1p3_1.BondCon1, B2.fBondCon1) annotation (points=[-50, -60; -50,
              -60], style(color=8));
      connect(B3.fBondCon1, J1p3_1.BondCon2) annotation (points=[-30, -60; -30,
              -60], style(color=8));
      connect(J0p3_2.BondCon1, B3.eBondCon1) annotation (points=[-10, -60; -10,
              -60], style(color=8));
      connect(J0p3_2.BondCon2, B4.eBondCon1) annotation (points=[10, -60; 10, -
            60], style(color=8));
      connect(B4.fBondCon1, J1p3_2.BondCon1) annotation (points=[30, -60; 30, -
            60], style(color=8));
      connect(J1p3_2.BondCon2, B5.fBondCon1) annotation (points=[50, -60; 50, -
            60], style(color=8));
      connect(BondCon2, B1.eBondCon1) annotation (points=[80,-100; 80,-90],
          style(color=8));
      connect(B1.fBondCon1, J0p3_3.BondCon3) annotation (points=[80, -70; 80, -
            70], style(color=8));
      connect(B5.eBondCon1, J0p3_3.BondCon1) annotation (points=[70, -60; 70, -
            60], style(color=8));
      connect(J1p3_1.BondCon3, B7.eBondCon1) annotation (points=[-40, -50; -40,
              -50], style(color=8));
      connect(B8.fBondCon1, J0p3_2.BondCon3) annotation (points=[6.12303e-016,
            -50; 0,-50],
               style(color=8));
      connect(J1p3_2.BondCon3, B9.eBondCon1) annotation (points=[40, -50; 40, -
            50], style(color=8));
      connect(PS.BondCon1, B8.eBondCon1) annotation (points=[-6.12303e-016,-30;
            -6.12303e-016,-30],
          style(color=8));
      connect(B7.fBondCon1, mG1.BondCon1) annotation (points=[-40, -30; -40, -
            30], style(color=8));
      connect(B9.fBondCon1, mG4.BondCon1) annotation (points=[40, -30; 40, -30],
            style(color=8));
      connect(J0p3_1.BondCon1, B15.eBondCon1) annotation (points=[-90, -60; -90,
              60; -70, 60], style(color=8));
      connect(B15.fBondCon1, J1p3_3.BondCon1) annotation (points=[-50, 60; -50,
              60], style(color=8));
      connect(J1p3_3.BondCon3, B12.eBondCon1) annotation (points=[-40, 50; -40,
              50], style(color=8));
      connect(B12.fBondCon1, mG2.BondCon1) annotation (points=[-40, 30; -40, 30],
               style(color=8));
      connect(J1p3_3.BondCon2, B11.fBondCon1) annotation (points=[-30, 60; -30,
              60], style(color=8));
      connect(B11.eBondCon1, J0p3_4.BondCon1) annotation (points=[-10, 60; -10,
              60], style(color=8));
      connect(J0p3_4.BondCon3, B10.fBondCon1) annotation (points=[0,50;
            -6.12303e-016,50],
            style(color=8));
      connect(B10.eBondCon1, P0.BondCon1) annotation (points=[6.12303e-016,30;
            -6.12303e-016,30],
          style(color=8));
      connect(B13.eBondCon1, J0p3_4.BondCon2) annotation (points=[10, 60; 10,
            60], style(color=8));
      connect(J1p3_4.BondCon1, B13.fBondCon1) annotation (points=[30, 60; 30,
            60], style(color=8));
      connect(J1p3_4.BondCon3, B14.eBondCon1) annotation (points=[40, 50; 40,
            50], style(color=8));
      connect(B14.fBondCon1, mG3.BondCon1) annotation (points=[40, 30; 40, 30],
            style(color=8));
      connect(B16.fBondCon1, J1p3_4.BondCon2) annotation (points=[50, 60; 50,
            60], style(color=8));
      connect(J0p3_3.BondCon2, B16.eBondCon1) annotation (points=[90, -60; 90,
            60; 70, 60], style(color=8));
      connect(M1.u, x) annotation (points=[-80,0; -98,0], style(color=74,
            rgbcolor={0,0,127}));
      connect(mG2.s, M1.y2) annotation (points=[-40,11; -40,6; -60,6], style(
            color=74, rgbcolor={0,0,127}));
      connect(mG1.s, M1.y1) annotation (points=[-40,-11; -40,-6; -60,-6], style(
            color=74, rgbcolor={0,0,127}));
      connect(mG3.s, M1.y1) annotation (points=[40,11; 40,6; 10,6; 10,-6; -60,
            -6], style(color=74, rgbcolor={0,0,127}));
      connect(mG4.s, M1.y2) annotation (points=[40,-11; 40,0; -10,0; -10,6; -60,
            6], style(color=74, rgbcolor={0,0,127}));
    end Servovalve;
    
    model Hydromotor "Hydraulic motor model built from bond graph technology" 
      parameter Real ce(unit="kg/(m4.s2)") = 1 "External leakage";
      parameter Real ci(unit="kg/(m4.s2)") = 1 "Internal leakage";
      parameter Real c1(unit="kg/(m4.s2)") = 1 "Compression coefficient";
      parameter Real psi(unit="kg/(m.s)") = 1 "Motor induction constant";
      parameter Real rhom(unit="kg.m2/s") = 1 "Motor friction constant";
      parameter Modelica.SIunits.Inertia jm=1 "Motor inertia";
      parameter Modelica.SIunits.Pressure ps=1 "Service pressure";
      parameter Modelica.SIunits.Pressure p0=0 "Environment pressure";
      annotation (
        Coordsys(
          extent=[-100, -140; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Text(
            extent=[-40, 36; -28, 28],
            string="p1",
            style(color=9)),
          Text(
            extent=[52, 18; 64, 10],
            string="p2",
            style(color=9)),
          Text(
            extent=[40, -4; 52, -12],
            string="p2",
            style(color=9)),
          Text(
            extent=[-28, 18; -16, 10],
            string="p1",
            style(color=9)),
          Text(
            extent=[-28, 58; -16, 50],
            string="p1",
            style(color=9)),
          Text(
            extent=[12, 58; 24, 50],
            string="p2",
            style(color=9)),
          Text(
            extent=[28, 32; 40, 24],
            string="p2",
            style(color=9)),
          Text(
            extent=[-64, 10; -52, 2],
            string="p1",
            style(color=9)),
          Text(
            extent=[12, 18; 24, 10],
            string="p2",
            style(color=9)),
          Text(
            extent=[-40, -4; -28, -12],
            string="p1",
            style(color=9)),
          Text(
            extent=[0, -4; 12, -12],
            string="pL",
            style(color=9)),
          Text(
            extent=[-12, 72; 0, 64],
            string="pL",
            style(color=9)),
          Text(
            extent=[0, 72; 12, 64],
            string="qi",
            style(color=9)),
          Text(
            extent=[-28, 48; -16, 40],
            string="qi",
            style(color=9)),
          Text(
            extent=[12, 48; 24, 40],
            string="qi",
            style(color=9)),
          Text(
            extent=[-16, -6; -4, -14],
            string="qind",
            style(color=9)),
          Text(
            extent=[-28, 8; -16, 0],
            string="qind",
            style(color=9)),
          Text(
            extent=[12, 8; 24, 0],
            string="qind",
            style(color=9)),
          Text(
            extent=[-16, -44; -4, -52],
            string="wm",
            style(color=9)),
          Text(
            extent=[14, -74; 26, -82],
            string="wm",
            style(color=9)),
          Text(
            extent=[-26, -58; -14, -66],
            string="wm",
            style(color=9)),
          Text(
            extent=[54, 8; 66, 0],
            string="qe2",
            style(color=9)),
          Text(
            extent=[-66, 22; -54, 14],
            string="qe1",
            style(color=9)),
          Text(
            extent=[-56, -4; -44, -12],
            string="qc1",
            style(color=9)),
          Text(
            extent=[24, -4; 36, -12],
            string="qc2",
            style(color=9)),
          Text(
            extent=[12, -60; 24, -68],
            string="Tb",
            style(color=9)),
          Text(
            extent=[-24, -70; -12, -78],
            string="Tj",
            style(color=9)),
          Text(
            extent=[0, -44; 12, -52],
            string="Tm",
            style(color=9)),
          Text(
            extent=[94, 4; 104, -4],
            string="wm",
            style(color=0)),
          Line(points=[92, -4; 112, -4], style(color=0)),
          Line(points=[106, 0; 112, -4; 106, -8], style(color=0)),
          Line(points=[40, 20; 40, 20], style(color=8)),
          Line(points=[30, 10; 30, 10], style(color=8)),
          Line(points=[40, 0; 40, 0], style(color=8)),
          Line(points=[30, 10; 30, 10], style(color=8)),
          Line(points=[40, 20; 40, 20], style(color=8)),
          Line(points=[40, 0; 40, 0], style(color=8)),
          Line(points=[0, -80; 0, -80], style(color=8)),
          Line(points=[0, -60; 0, -60], style(color=8)),
          Line(points=[10, -70; 10, -70], style(color=8)),
          Line(points=[-10, -70; -10, -70], style(color=8)),
          Line(points=[-40, 60; -40, 60], style(color=8)),
          Line(points=[40, 60; 40, 60], style(color=8)),
          Line(points=[0, -60; 0, -60], style(color=8)),
          Line(points=[10, -70; 10, -70], style(color=8)),
          Line(points=[-10, -70; -10, -70], style(color=8))),
        Icon(
          Rectangle(extent=[-80, 40; 80, -80], style(gradient=2, fillColor=41)),
          Polygon(points=[-80, -120; -80, -100; -60, -100; -42, -20; 38, -20;
                60, -100; 80, -100; 80, -120; -80, -120], style(color=0,
                fillColor=0)),
          Rectangle(extent=[-68, 20; 70, -60], style(fillColor=7)),
          Rectangle(extent=[-10, 16; 12, -56], style(fillColor=73)),
          Rectangle(extent=[-68, -16; -80, -24], style(pattern=0, fillColor=7)),
          Rectangle(extent=[80, -16; 68, -24], style(pattern=0, fillColor=7)),
          Rectangle(extent=[94, -18; -86, -22], style(fillColor=73)),
          Rectangle(extent=[-64, 74; -56, 18], style(pattern=0, fillColor=7)),
          Rectangle(extent=[56, 74; 64, 18], style(pattern=0, fillColor=7)),
          Polygon(points=[-64, 86; -64, 20; -68, 20; -68, -16; -80, -16; -80, -
                18; -10, -18; -10, 16; 12, 16; 12, -18; 80, -18; 80, -16; 70, -
                16; 70, 20; 64, 20; 64, 86; 56, 86; 56, 20; -56, 20; -56, 86; -
                64, 86], style(pattern=0, fillColor=65)),
          Polygon(points=[-80, -22; -80, -24; -68, -24; -68, -60; 70, -60; 70,
                -24; 80, -24; 80, -22; 12, -22; 12, -56; -10, -56; -10, -22; -
                80, -22], style(pattern=0, fillColor=65)),
          Line(points=[-64, 86; -64, 20; -68, 20; -68, -16; -80, -16]),
          Line(points=[-80, -24; -68, -24; -68, -60; 70, -60; 70, -24; 80, -24]),
          Line(points=[-56, 80; -56, 20; 56, 20; 56, 80]),
          Line(points=[64, 80; 64, 20; 70, 20; 70, -16; 80, -16]),
          Line(points=[22, 8; 14, 18], style(color=0)),
          Text(
            extent=[20, 10; 30, 2],
            string="qi",
            style(color=0)),
          Line(points=[-12, 18; -20, 10], style(color=0)),
          Text(
            extent=[-28, 10; -18, 2],
            string="qi",
            style(color=0)),
          Line(points=[-58, -8; -66, -16], style(color=0)),
          Line(points=[-66, -12; -66, -16; -62, -16], style(color=0)),
          Text(
            extent=[-60, -8; -48, -16],
            string="qe1",
            style(color=0)),
          Line(points=[58, -8; 68, -16], style(color=0)),
          Line(points=[68, -12; 68, -16; 64, -16], style(color=0)),
          Text(
            extent=[48, -8; 60, -16],
            string="qe2",
            style(color=0)),
          Line(points=[60, 38; 60, 18], style(color=0)),
          Line(points=[-62, 22; -60, 18; -58, 22], style(color=0)),
          Text(
            extent=[54, 18; 66, 10],
            string="qL2",
            style(color=0)),
          Line(points=[-60, 18; -60, 36], style(color=0)),
          Line(points=[58, 34; 60, 38; 62, 34], style(color=0)),
          Text(
            extent=[-66, 18; -54, 10],
            string="qL1",
            style(color=0)),
          Text(extent=[-56, 138; 56, 100], string="%name"),
          Line(points=[92, -4; 112, -4], style(color=0)),
          Line(points=[106, 0; 112, -4; 106, -8], style(color=0)),
          Text(
            extent=[94, 6; 104, -2],
            string="wm",
            style(color=0)),
          Line(points=[-16, 18; -12, 18; -12, 14], style(color=0)),
          Line(points=[22, 12; 22, 8; 18, 8], style(color=0)),
          Polygon(points=[-86, 100; -86, 86; -56, 86; -56, 94; -74, 94; -74,
                100; -86, 100], style(pattern=0, fillColor=65)),
          Polygon(points=[74, 100; 74, 94; 56, 94; 56, 86; 86, 86; 86, 100; 74,
                  100], style(pattern=0, fillColor=65)),
          Line(points=[-64, 86; -86, 86; -86, 100]),
          Line(points=[-56, 80; -56, 94; -74, 94; -74, 100]),
          Line(points=[74, 100; 74, 94; 56, 94; 56, 80]),
          Line(points=[86, 100; 86, 86; 64, 86; 64, 80]),
          Text(
            extent=[-46, -36; -34, -44],
            style(color=0),
            string="p1"),
          Text(
            extent=[36, -36; 48, -44],
            style(color=0),
            string="p2")),
        Window(
          x=0.44,
          y=0.06,
          width=0.47,
          height=0.64),
        Documentation(info="<html>
The hydraulic motor model described here has two bondgraphic inputs: the two load flows, <b>q<sub>L1</sub></b> and <b>q<sub>L2</sub></b>.  It has a signal output: the angular velocity, <b>omega<sub>m</sub></b>, of the motor.  The bondgraphic interface is designed such that the hydraulic motor can be directly connected to the servo valve without either a set of bonds or a set of junctions in between. <p>

The two chambers of the hydraulic motor are at the pressures <b>p<sub>1</sub></b> and <b>p<sub>2</sub></b>, respectively. <p>

The model also contains an internal leakage flow, <b>q<sub>i</sub></b>, and two external leakage flows, <b>q<sub>e1</sub></b> and <b>q<sub>e2</sub></b>.  The leakage flows are assumed to be laminar flows.
</html>

"));
      Interfaces.BondCon BondCon1 annotation (extent=[-90, 90; -70, 110]);
      Interfaces.BondCon BondCon2 annotation (extent=[70, 90; 90, 110]);
      Junctions.J0p3 J0p3_1 annotation (extent=[30, 40; 50, 60], rotation=-90);
      Bonds.eBond B4 annotation (extent=[-50, 20; -30, 40], rotation=-90);
      Bonds.fBond B9 annotation (extent=[50, 0; 70, 20]);
      Passive.R Re2(R=1/ce) annotation (extent=[70, 0; 90, 20]);
      Bonds.fBond B1 annotation (extent=[-30, 40; -10, 60]);
      Junctions.J1p3 J1p3_1 annotation (extent=[-10, 60; 10, 40]);
      Bonds.fBond B3 annotation (extent=[-10, 60; 10, 80], rotation=90);
      Passive.R Ri(R=1/ci) annotation (extent=[-10, 80; 10, 100], rotation=90);
      Bonds.eBond B2 annotation (extent=[10, 40; 30, 60]);
      Junctions.J0p3 J0p3_2 annotation (extent=[-50, 40; -30, 60], rotation=90);
      Bonds.fBond B11 annotation (extent=[30, 20; 50, 40], rotation=90);
      Junctions.J0p4 J0p4_2 annotation (extent=[-50, 0; -30, 20]);
      Bonds.fBond B7 annotation (extent=[-30, 0; -10, 20]);
      Junctions.J1p3 J1p3_2 annotation (extent=[-10, 0; 10, 20]);
      Bonds.eBond B8 annotation (extent=[10, 0; 30, 20]);
      Bonds.fBond B5 annotation (extent=[-70, 0; -50, 20], rotation=180);
      Passive.R Re1(R=1/ce) annotation (extent=[-70, 0; -90, 20]);
      Bonds.eBond B10 annotation (extent=[30, -20; 50, 0], rotation=-90);
      Passive.C C2(C=1/c1, e(start=(ps + p0)/2)) annotation (extent=[30, -40;
            50, -20], rotation=-90);
      Bonds.eBond B6 annotation (extent=[-50, -20; -30, 0], rotation=-90);
      Passive.C C1(C=1/c1, e(start=(ps + p0)/2)) annotation (extent=[-50, -40;
            -30, -20], rotation=-90);
      Bonds.fBond B12 annotation (extent=[-10, -20; 10, 0], rotation=-90);
      Passive.TF TF1(m=1/psi) annotation (extent=[-10, -40; 10, -20], rotation=
            -90);
      Bonds.fBond B13 annotation (extent=[-10, -60; 10, -40], rotation=-90);
      Bonds.eBond B14 annotation (extent=[10, -80; 30, -60]);
      Passive.R Bm(R=rhom) annotation (extent=[30, -80; 50, -60]);
      Bonds.fBond B15 annotation (extent=[-30, -80; -10, -60], rotation=180);
      Passive.I Jm(I=jm) annotation (extent=[-30, -80; -50, -60]);
      Junctions.J0p4 J0p4_1 annotation (extent=[30, 0; 50, 20]);
      Modelica.Blocks.Interfaces.RealOutput wm    annotation (extent=[92, -30;
            112, -10]);
      Sensors.Df Df1 annotation (extent=[-10, -120; 10, -100], rotation=-90);
      Bonds.eBond B16 annotation (extent=[-10, -100; 10, -80], rotation=-90);
      Junctions.J1p4 J1p4_1 annotation (extent=[-10, -80; 10, -60]);
    equation 
      connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,60;
            -6.12303e-016,60], style(color=8));
      connect(J0p4_2.BondCon1, B5.eBondCon1) annotation (points=[-50,10; -50,10],
                 style(color=8));
      connect(B5.fBondCon1, Re1.BondCon1) annotation (points=[-70,10; -70,10],
            style(color=8));
      connect(B10.eBondCon1, C2.BondCon1) annotation (points=[40, -20; 40, -20],
            style(color=8));
      connect(J0p4_2.BondCon3, B6.fBondCon1) annotation (points=[-40, 0; -40, 0],
               style(color=8));
      connect(B6.eBondCon1, C1.BondCon1) annotation (points=[-40, -20; -40, -20],
               style(color=8));
      connect(J1p3_2.BondCon3, B12.eBondCon1) annotation (points=[0,0;
            -6.12303e-016,0],
          style(color=8));
      connect(B12.fBondCon1, TF1.BondCon1) annotation (points=[6.12303e-016,-20;
            -6.12303e-016,-20], style(color=8));
      connect(TF1.BondCon2, B13.eBondCon1) annotation (points=[6.12303e-016,-40;
            -6.12303e-016,-40],
                         style(color=8));
      connect(B14.eBondCon1, Bm.BondCon1) annotation (points=[30, -70; 30, -70],
            style(color=8));
      connect(B15.fBondCon1, Jm.BondCon1) annotation (points=[-30, -70; -30, -
            70], style(color=8));
      connect(J0p4_1.BondCon3, B10.fBondCon1) annotation (points=[40, 0; 40, 0],
            style(color=8));
      connect(J0p4_1.BondCon2, B9.eBondCon1) annotation (points=[50, 10; 50, 10],
               style(color=8));
      connect(B9.fBondCon1, Re2.BondCon1) annotation (points=[70, 10; 70, 10],
          style(color=8));
      connect(J0p3_2.BondCon1, B4.fBondCon1) annotation (points=[-40, 40; -40,
            40], style(color=8));
      connect(B4.eBondCon1, J0p4_2.BondCon4) annotation (points=[-40, 20; -40,
            20], style(color=8));
      connect(J0p4_1.BondCon4, B11.eBondCon1) annotation (points=[40, 20; 40,
            20], style(color=8));
      connect(B11.fBondCon1, J0p3_1.BondCon2) annotation (points=[40, 40; 40,
            40], style(color=8));
      connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[10, 50; 10, 50],
               style(color=8));
      connect(B2.eBondCon1, J0p3_1.BondCon3) annotation (points=[30, 50; 30, 50],
               style(color=8));
      connect(J0p3_2.BondCon3, B1.eBondCon1) annotation (points=[-30, 50; -30,
            50], style(color=8));
      connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-10, 50; -10,
            50], style(color=8));
      connect(J1p3_2.BondCon2, B8.fBondCon1) annotation (points=[10, 10; 10, 10],
               style(color=8));
      connect(B8.eBondCon1, J0p4_1.BondCon1) annotation (points=[30, 10; 30, 10],
               style(color=8));
      connect(J0p4_2.BondCon2, B7.eBondCon1) annotation (points=[-30, 10; -30,
            10], style(color=8));
      connect(B7.fBondCon1, J1p3_2.BondCon1) annotation (points=[-10, 10; -10,
            10], style(color=8));
      connect(B3.fBondCon1, Ri.BondCon1) annotation (points=[6.12303e-016,80;
            -6.12303e-016,80],
                    style(color=8));
      connect(J0p3_1.BondCon1, BondCon2) annotation (points=[40,60; 40,80; 80,
            80; 80,100],   style(color=8));
      connect(BondCon1, J0p3_2.BondCon2) annotation (points=[-80,100; -80,80;
            -40,80; -40,60],   style(color=8));
      connect(J1p4_1.BondCon1, B15.eBondCon1) annotation (points=[-10, -70; -10,
              -70], style(color=8));
      connect(B13.fBondCon1, J1p4_1.BondCon4) annotation (points=[6.12303e-016,
            -60; 0,-60],
                 style(color=8));
      connect(J1p4_1.BondCon2, B14.fBondCon1) annotation (points=[10, -70; 10,
            -70], style(color=8));
      connect(J1p4_1.BondCon3, B16.fBondCon1) annotation (points=[0,-80;
            -6.12303e-016,-80], style(color=8));
      connect(B16.eBondCon1, Df1.BondCon1) annotation (points=[6.12303e-016,
            -100; -6.12303e-016,-100],
                           style(color=8));
      connect(Df1.OutPort1, wm)       annotation (points=[6.73533e-016,-121;
            6.73533e-016,-130; 80,-130; 80,-20; 102,-20], style(color=74,
            rgbcolor={0,0,135}));
    end Hydromotor;
  end Hydraulic;
  
  package Examples "Examples of use of bond graph library" 
    annotation (preferedView="info",Icon(
        Rectangle(extent=[-100, -100; 80, 50], style(
            color=0,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[-100, 50; -80, 70; 100, 70; 80, 50; -100, 50], style(
            color=0,
            fillColor=30,
            fillPattern=1)),
        Polygon(points=[100, 70; 100, -80; 80, -100; 80, 50; 100, 70], style(
            color=0,
            fillColor=30,
            fillPattern=1)),
        Text(
          extent=[-85, 71; 65, -49],
          string="Sample",
          style(color=0)),
        Text(
          extent=[-118, 114; 122, 65],
          string="%name",
          style(color=1)),
        Text(
          extent=[-97, 23; 78, -98],
          string="Programs",
          style(color=0))), Documentation(info="<html>
This package contains a number of examples demonstrating the use of the bond graph library: <p>

<ol>
<li> a simple bond graph of an electrical circuit,
<li> the same circuit modeled using the electrical library built in bond graph technology,
<li> a hydraulic control system,
<li> a thermal building model,
<li> a thermal model of Biosphere 2,
<li> a set of models testing <font color=red size=5><b><i>ModelicaSpice</i></b></font>.
</ol>
</html>
"));
    model SimpleBondGraph "A simple bond graph of an electrical circuit" 
      constant Real pi=4*Modelica.Math.atan(1);
      annotation (preferedView="info",
        Documentation(info="<html>
This is a simple bond graph of an electrical circuit.  The development of the bond graph of this circuit was demonstrated in: <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. and A. Nebot (2005), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/modelica2005_cellier.pdf\">The
     Modelica Bond Graph Library</a>,\" <i>Proc. 4<sup>th</sup> Modelica Conference</i>, Hamburg, Germany.
</ol> <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate the bond graph during 0.002 seconds. <p>
 
<hr> <p>

<b>Simulation Results:</b> <p>

<img src=\"../Images/BG_Ex1.png\"> 
</html>
"),     Diagram,
        Coordsys(extent=[-100, -120; 300, 120]),
        Icon(
          Rectangle(extent=[0, 100; 200, -100], style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[24, 44; 178, 4],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="A simple bond graph"),
          Text(
            extent=[22, 20; 180, -20],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="of an electrical circuit")),
        experiment(StopTime=0.002),
        experimentSetupOutput);
      BondLib.Sources.sinSe U0(e0=10, freqHz=2500/pi) annotation (extent=[-40,
            -20; -80, 20]);
      BondLib.Bonds.fBond B1 annotation (extent=[-40, -20; 0, 20]);
      BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[0, -20; 40, 20]);
      BondLib.Bonds.fBond B2 annotation (extent=[0, -60; 40, -20], rotation=-90);
      BondLib.Passive.I L(I=0.0015, f(start=2)) annotation (extent=[0, -100; 40,
              -60], rotation=-90);
      BondLib.Bonds.fBond B3 annotation (extent=[40, -20; 80, 20]);
      BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[80, 20; 120, -20]);
      BondLib.Bonds.fBond B4 annotation (extent=[80, 20; 120, 60], rotation=90);
      BondLib.Passive.R R1(R=100) annotation (extent=[80, 60; 120, 100],
          rotation=90);
      BondLib.Bonds.eBond B5 annotation (extent=[120, -20; 160, 20]);
      BondLib.Junctions.J0p3 J0p3_2 annotation (extent=[160, -20; 200, 20]);
      BondLib.Bonds.eBond B6 annotation (extent=[160, -60; 200, -20], rotation=
            -90);
      BondLib.Passive.C C1(C=1e-6, e(start=1)) annotation (extent=[160, -100;
            200, -60], rotation=-90);
      BondLib.Bonds.fBond B7 annotation (extent=[200, -20; 240, 20]);
      BondLib.Passive.R R2(R=20) annotation (extent=[240, -20; 280, 20],
          rotation=0);
    equation 
      connect(U0.BondCon1, B1.eBondCon1) annotation (points=[-40, 0; -40, 0],
          style(color=8));
      connect(B1.fBondCon1, J0p3_1.BondCon1) annotation (points=[0, 0; 0, 0],
          style(color=8));
      connect(J0p3_1.BondCon3, B2.eBondCon1) annotation (points=[20,-20; 20,-20],
                 style(color=8));
      connect(B2.fBondCon1, L.BondCon1) annotation (points=[20,-60; 20,-60],
          style(color=8));
      connect(J0p3_1.BondCon2, B3.eBondCon1) annotation (points=[40, 0; 40, 0],
            style(color=8));
      connect(B3.fBondCon1, J1p3_1.BondCon1) annotation (points=[80,0; 80,0],
            style(color=8));
      connect(B4.eBondCon1, J1p3_1.BondCon3) annotation (points=[100,20; 100,20],
                 style(color=8));
      connect(R1.BondCon1, B4.fBondCon1) annotation (points=[100,60; 100,60],
            style(color=8));
      connect(J1p3_1.BondCon2, B5.fBondCon1) annotation (points=[120,0; 120,0],
               style(color=8));
      connect(B5.eBondCon1, J0p3_2.BondCon1) annotation (points=[160,0; 160,0],
               style(color=8));
      connect(J0p3_2.BondCon3, B6.fBondCon1) annotation (points=[180, -20; 180,
              -20], style(color=8));
      connect(B6.eBondCon1, C1.BondCon1) annotation (points=[180, -60; 180, -60],
               style(color=8));
      connect(J0p3_2.BondCon2, B7.eBondCon1) annotation (points=[200,0; 200,0],
               style(color=8));
      connect(B7.fBondCon1, R2.BondCon1) annotation (points=[240,0; 240,0],
          style(color=8));
    end SimpleBondGraph;
    
    model ElectricalCircuit 
      "A simple  electrical circuit built in bond graph technology" 
      constant Real pi=4*Modelica.Math.atan(1);
      BondLib.Electrical.Analog.Sources.SineVoltage U0(V=10, freqHz=2500/pi) 
        annotation (extent=[-80, -20; -40, 20], rotation=-90);
      BondLib.Electrical.Analog.Basic.Resistor R1(R=100) annotation (extent=[-
            40, 20; 0, 60], rotation=-90);
      BondLib.Electrical.Analog.Basic.Resistor R2(R=20) annotation (extent=[0,
            -20; 40, 20], rotation=0);
      BondLib.Electrical.Analog.Basic.Inductor L(L=0.0015, i0=2) annotation (
          extent=[40, 20; 80, 60], rotation=-90);
      BondLib.Electrical.Analog.Basic.Capacitor C(C=1e-6, u0=1) annotation (
          extent=[-40, -60; 0, -20], rotation=-90);
      annotation (preferedView="info",
        Coordsys(extent=[-100, -120; 100, 100]),
        Diagram,
        Icon(
          Rectangle(extent=[-100, 89; 100, -111], style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-70, 40; 76, 7],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="An electrical circuit"),
          Text(
            extent=[-48, 11; 52, -8],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="built in"),
          Text(
            extent=[-78, 0; 80, -40],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="bond graph technology")),
        Documentation(info="<html>
This is a circuit diagram of a simple electrical circuit.  In fact, it is the same circuit as the one used in the previous example.  The circuit diagram was built using models from the analog electrical library contained within the bond graph library, i.e., each model has built in bond graph technology.  Yet from the outside, the circuit elements are indistinguishable from those in Modelica's standard analog electrical library. <p>

The circuit was elaborated on in: <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. and A. Nebot (2005), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/modelica2005_cellier.pdf\">The
     Modelica Bond Graph Library</a>,\" <i>Proc. 4<sup>th</sup> Modelica Conference</i>, Hamburg, Germany.
</ol> <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate the bond graph during 0.002 seconds. <p>
 
<hr> <p>

<b>Simulation Results:</b> <p>

<img src=\"../Images/BG_Ex2.png\"> 
</html>
"),     experiment(StopTime=0.002),
        experimentSetupOutput);
      BondLib.Electrical.Analog.Basic.Ground G annotation (extent=[-20, -120;
            20, -80]);
    equation 
      connect(U0.p, R1.p) annotation (points=[-60,20; -60,80; -20,80; -20,60],
          style(color=3, rgbcolor={0,0,255}));
      connect(L.p, R1.p) annotation (points=[60,60; 60,80; -20,80; -20,60],
          style(color=3, rgbcolor={0,0,255}));
      connect(U0.n, G.p) annotation (points=[-60,-20; -60,-80; 0,-80], style(
            color=3, rgbcolor={0,0,255}));
      connect(C.n, G.p) annotation (points=[-20,-60; -20,-80; 0,-80], style(
            color=3, rgbcolor={0,0,255}));
      connect(R1.n, C.p) annotation (points=[-20,20; -20,-20], style(color=3,
            rgbcolor={0,0,255}));
      connect(R2.p, C.p) annotation (points=[0,0; -20,0; -20,-20], style(color=
              3, rgbcolor={0,0,255}));
      connect(L.n, G.p) annotation (points=[60,20; 60,-80; 0,-80], style(color=
              3, rgbcolor={0,0,255}));
      connect(R2.n, L.n) annotation (points=[40,0; 60,0; 60,20], style(color=3,
            rgbcolor={0,0,255}));
    end ElectricalCircuit;
    
    package OperationalAmplifierCircuit "Operational amplifier circuit" 
      annotation (preferedView="info",
        Icon(
          Rectangle(extent=[-99.85,98.65; 100.15,-101.35],   style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-57.85,52.65; 54,24],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Electronic circuit"),
          Text(
            extent=[-63.85,2.65; 66,-44],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="operational amplifier"),
          Text(
            extent=[-71.7,35.3; 74,4],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="containing an inverter"),
          Text(
            extent=[-45.55,13.95; 38,-16],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="built from an")),
        Diagram(
          Rectangle(extent=[-99.85,100.65; 100.15,-99.35],   style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-57.85,54.65; 54,26],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Electronic circuit"),
          Text(
            extent=[-63.85,4.65; 66,-42],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="operational amplifier"),
          Text(
            extent=[-71.7,37.3; 74,6],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="containing an inverter"),
          Text(
            extent=[-45.55,15.95; 38,-14],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="built from an")),
        Documentation(info="<html>
This package contains an electronic circuit representing an analog inverter built up from a commercial style operational amplifier circuit containing 12 bipolar transistors, 6 NPNs and 6 PNPs. <p>

The design of this model has been described in [1]. <p>

<hr> <p>

<b>Experimentation:</b> <p>

This model uses seconds as time units.  You should simulate the model during 0.0001 seconds.  Plot the input voltage and the output voltage of the inverter together on one plot. <p>

<hr> <p>

<b>Simulation Results:</b> <p>

<img src=\"../Images/BG_Ex6.png\"> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 216 ff.
</ol>
</html>
"));
      model OpAmp "Operational amplifier" 
        parameter Boolean enforceStates = true 
          "|Advanced||Use electrical variables as states instead of bond graph variables";
        annotation (
          Coordsys(extent=[-140,-100; 100,100]),
          Icon(
            Ellipse(extent=[-98,64; -30,-56], style(
                color=3,
                rgbcolor={0,0,255},
                thickness=2)),
            Rectangle(extent=[-54,66; 60,-60], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[60,0; -56,62], style(
                color=3,
                rgbcolor={0,0,255},
                thickness=2,
                fillPattern=1)),
            Line(points=[60,0; -54,-54], style(
                color=3,
                rgbcolor={0,0,255},
                thickness=2,
                fillPattern=1)),
            Line(points=[-94,30; -130,30], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[-92,-30; -130,-30], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[-20,44; -20,90], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[-20,-38; -20,-90], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[60,0; 90,0], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Text(
              extent=[-96,60; -42,6],
              style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="-"),
            Text(
              extent=[-84,-12; -52,-48],
              style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="+")),
          Diagram(
            Ellipse(extent=[38,-18; 42,-22], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255})),
            Ellipse(extent=[38,48; 42,44], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1)),
            Text(
              extent=[-36,86; -22,80],
              string="Vcc",
              style(
                color=10,
                rgbcolor={95,95,95},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1)),
            Text(
              extent=[-8,-84; 6,-90],
              style(
                color=10,
                rgbcolor={95,95,95},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="Vee"),
            Text(
              extent=[-98,14; -82,8],
              style(
                color=10,
                rgbcolor={95,95,95},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="-VI"),
            Text(
              extent=[-8,14; 8,8],
              style(
                color=10,
                rgbcolor={95,95,95},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="+VI"),
            Text(
              extent=[66,14; 82,8],
              style(
                color=10,
                rgbcolor={95,95,95},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="VO"),
            Ellipse(extent=[-22,-88; -18,-92], style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255}))),
          Documentation(info="<html>
This circuit represents a commercial high-gain operational amplifier built from 12 individual bipolar transistors, six of which are NPN transistors, whereas the other six are PNP transistors. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 216 ff.
</ol>
</html>
"));
        Modelica.Electrical.Analog.Interfaces.Pin vi_n 
          annotation (extent=[-150,20; -130,40]);
        Modelica.Electrical.Analog.Interfaces.Pin vi_p 
          annotation (extent=[-150,-40; -130,-20]);
        Modelica.Electrical.Analog.Interfaces.Pin vo 
          annotation (extent=[90,-10; 110,10]);
        Modelica.Electrical.Analog.Interfaces.Pin vcc 
          annotation (extent=[-30,90; -10,110]);
        Modelica.Electrical.Analog.Interfaces.Pin vee 
          annotation (extent=[-30,-110; -10,-90]);
        Electrical.Analog.Semiconductors.PNP PNP1(enforceStates=enforceStates) 
          annotation (extent=[-100,70; -120,50]);
        Electrical.Analog.Basic.Resistor R1(R=1.5e6) 
          annotation (extent=[-130,-60; -110,-40], rotation=-90);
        Electrical.Analog.Semiconductors.PNP PNP2(enforceStates=enforceStates) 
          annotation (extent=[-66,70; -46,50]);
        Electrical.Analog.Semiconductors.PNP PNP3(enforceStates=enforceStates) 
          annotation (extent=[20,70; 40,50]);
        Electrical.Analog.Basic.Capacitor C1(C=10e-12) 
          annotation (extent=[0,-30; 20,-10]);
        Electrical.Analog.Semiconductors.NPN NPN4(enforceStates=enforceStates) 
          annotation (extent=[60,36; 80,56]);
        Electrical.Analog.Semiconductors.NPN NPN5(enforceStates=enforceStates) 
          annotation (extent=[20,20; 40,40]);
        Electrical.Analog.Semiconductors.PNP PNP6(enforceStates=enforceStates) 
          annotation (extent=[-90,16; -70,-4]);
        Electrical.Analog.Semiconductors.PNP PNP7(enforceStates=enforceStates) 
          annotation (extent=[0,16; -20,-4]);
        Electrical.Analog.Semiconductors.NPN NPN8(enforceStates=enforceStates) 
          annotation (extent=[20,-10; 40,10]);
        Electrical.Analog.Semiconductors.PNP PNP9(enforceStates=enforceStates) 
          annotation (extent=[60,-10; 80,-30]);
        Electrical.Analog.Semiconductors.NPN NPN10(enforceStates=enforceStates) 
          annotation (extent=[20,-60; 40,-40]);
        Electrical.Analog.Semiconductors.NPN NPN11(enforceStates=enforceStates) 
          annotation (extent=[-50,-80; -70,-60]);
        Electrical.Analog.Semiconductors.NPN NPN12(enforceStates=enforceStates) 
          annotation (extent=[-40,-80; -20,-60]);
      equation 
        connect(PNP1.C, R1.p) annotation (points=[-120,55; -120,-40], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP6.B, vi_n) annotation (points=[-90,6; -100,6; -100,30; -140,
              30], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP6.E, PNP7.E) annotation (points=[-70,11; -70,20; -20,20; -20,
              11], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP2.C, PNP7.E) annotation (points=[-46,55; -46,20; -20,20; -20,
              11], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP7.B, vi_p) annotation (points=[0,6; 0,-10; -100,-10; -100,
              -30; -140,-30], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(C1.p, NPN10.B) annotation (points=[0,-20; 0,-50; 20,-50], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP6.C, NPN11.C) annotation (points=[-70,1; -70,-65], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(NPN11.C, NPN11.B) annotation (points=[-70,-65; -70,-54; -46,-54;
              -46,-70; -50,-70], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(R1.n, NPN11.E) annotation (points=[-120,-60; -120,-90; -70,-90;
              -70,-75], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(NPN12.B, NPN11.B) annotation (points=[-40,-70; -50,-70], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(NPN10.E, NPN12.E) annotation (points=[40,-55; 40,-90; -20,-90;
              -20,-75], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(NPN11.E, NPN12.E) annotation (points=[-70,-75; -70,-90; -20,-90;
              -20,-75], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(NPN12.E, vee) annotation (points=[-20,-75; -20,-100], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP7.C, NPN12.C) annotation (points=[-20,1; -20,-65], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(NPN12.C, NPN10.B) annotation (points=[-20,-65; -20,-50; 20,-50],
            style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP9.C, NPN10.E) annotation (points=[80,-25; 80,-90; 40,-90; 40,
              -55], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(PNP1.E, PNP2.E) annotation (points=[-120,65; -120,80; -46,80;
              -46,65], style(color=3, rgbcolor={0,0,255}));
        connect(PNP2.E, vcc) annotation (points=[-46,65; -46,80; -20,80; -20,
              100], style(color=3, rgbcolor={0,0,255}));
        connect(NPN4.C, PNP3.E) annotation (points=[80,51; 80,80; 40,80; 40,65],
            style(color=3, rgbcolor={0,0,255}));
        connect(PNP2.E, PNP3.E) annotation (points=[-46,65; -46,80; 40,80; 40,
              65], style(color=3, rgbcolor={0,0,255}));
        connect(PNP3.C, NPN5.C) annotation (points=[40,55; 40,35], style(color=
                3, rgbcolor={0,0,255}));
        connect(NPN5.C, NPN4.B) annotation (points=[40,35; 40,46; 60,46], style(
              color=3, rgbcolor={0,0,255}));
        connect(NPN5.B, NPN5.C) annotation (points=[20,30; 20,46; 40,46; 40,35],
            style(color=3, rgbcolor={0,0,255}));
        connect(NPN5.E, NPN8.C) annotation (points=[40,25; 40,5], style(color=3,
              rgbcolor={0,0,255}));
        connect(NPN8.B, NPN8.C) annotation (points=[20,0; 20,16; 40,16; 40,5],
            style(color=3, rgbcolor={0,0,255}));
        connect(NPN4.E, PNP9.E) annotation (points=[80,41; 80,-15], style(color=
               3, rgbcolor={0,0,255}));
        connect(PNP9.E, vo) annotation (points=[80,-15; 80,0; 100,0], style(
              color=3, rgbcolor={0,0,255}));
        connect(NPN8.E, NPN10.C) annotation (points=[40,-5; 40,-45], style(
              color=3, rgbcolor={0,0,255}));
        connect(C1.n, NPN10.C) annotation (points=[20,-20; 40,-20; 40,-45],
            style(color=3, rgbcolor={0,0,255}));
        connect(PNP9.B, NPN10.C) annotation (points=[60,-20; 40,-20; 40,-45],
            style(color=3, rgbcolor={0,0,255}));
        connect(PNP1.B, PNP2.B) annotation (points=[-100,60; -66,60], style(
              color=3, rgbcolor={0,0,255}));
        connect(PNP1.B, PNP1.C) annotation (points=[-100,60; -90,60; -90,46;
              -120,46; -120,55], style(color=3, rgbcolor={0,0,255}));
        connect(PNP2.B, PNP3.B) annotation (points=[-66,60; 20,60], style(color=
               3, rgbcolor={0,0,255}));
      end OpAmp;
      
      model Inverter 
        "Analog inverter circuitCommerical high gain inverter circuit build from Spice technology" 
        output Modelica.SIunits.Voltage vin "Input voltage";
        output Modelica.SIunits.Voltage vout "Output voltage";
        annotation (
          Icon(
            Line(points=[-60,60; -60,-60], style(
                thickness=2,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[-60,60; 60,0], style(
                thickness=2,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[-60,-60; 60,0], style(
                thickness=2,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[-60,0; -100,0], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[60,0; 100,0], style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Text(
              extent=[-66,30; -12,-24],
              style(
                pattern=0,
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1),
              string="-")),
          Diagram,
          Documentation(info="<html>
The analog inverter is built from a commercial high-gain operational amplifier. <p>

<hr> <p>

<b>Experimentation:</b> <p>

This model uses seconds as time units.  You should simulate the model during 0.0001 seconds.  Plot the input voltage and the output voltage of the inverter together in one plot. <p>

<hr> <p>

<b>Simulation Results:</b> <p>

<img src=\"../Images/BG_Ex6.png\"> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 216 ff.
</ol>
</html>
"),     experiment(StopTime=0.0001));
        Electrical.Analog.Basic.Resistor Rf(R=1e6) 
          annotation (extent=[20,60; 40,80]);
        Electrical.Analog.Basic.Resistor R1(R=1e6) 
          annotation (extent=[-40,0; -20,20]);
        Electrical.Analog.Basic.Ground Gnd1 annotation (extent=[-6,-26; 6,-16]);
        Electrical.Analog.Basic.Ground Gnd2 
          annotation (extent=[-66,-66; -54,-56]);
        Electrical.Analog.Basic.Ground Gnd3 annotation (extent=[34,14; 46,24]);
        Electrical.Analog.Basic.Ground Gnd4 annotation (extent=[14,-66; 26,-56]);
        Electrical.Analog.Basic.Ground Gnd5 annotation (extent=[54,-66; 66,-56]);
        Electrical.Analog.Sources.ConstantVoltage Vcc(V=5) 
          annotation (extent=[20,30; 40,50]);
        Electrical.Analog.Sources.ConstantVoltage Vee(V=-5) 
          annotation (extent=[10,-46; 30,-26], rotation=-90);
        Electrical.Analog.Sensors.VoltageSensor Vo 
          annotation (extent=[70,-46; 50,-26], rotation=-90);
        Electrical.Analog.Sources.TrapezoidVoltage Vin(
          V=4,
          rising=1e-6,
          width=24e-6,
          falling=1e-6,
          period=50e-6,
          nperiod=2,
          offset=-2,
          startTime=5e-6) annotation (extent=[-70,-46; -50,-26], rotation=-90);
        BondLib.Examples.OperationalAmplifierCircuit.OpAmp OpAmp1 
          annotation (extent=[0,-16; 40,24]);
      equation 
        vin = Vin.v;
        vout = Vo.v;
        connect(Vcc.n, Gnd3.p) annotation (points=[40,40; 40,24], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Vee.n, Gnd4.p) annotation (points=[20,-46; 20,-56], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Vo.n, Gnd5.p) annotation (points=[60,-46; 60,-56], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Vin.n, Gnd2.p) 
                              annotation (points=[-60,-46; -60,-56], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(R1.p, Vin.p) 
                            annotation (points=[-40,10; -60,10; -60,-26], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(R1.n, OpAmp1.vi_n) annotation (points=[-20,10; 0,10], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Vcc.p, OpAmp1.vcc) annotation (points=[20,40; 20,24], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Rf.p, OpAmp1.vi_n) annotation (points=[20,70; 0,70; 0,10],
            style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(OpAmp1.vi_p, Gnd1.p) annotation (points=[0,-2; 0,-16], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Vee.p, OpAmp1.vee) annotation (points=[20,-26; 20,-16], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Rf.n, OpAmp1.vo) annotation (points=[40,70; 60,70; 60,4; 40,4],
            style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
        connect(Vo.p, OpAmp1.vo) annotation (points=[60,-26; 60,4; 40,4], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=0,
            fillColor=7,
            rgbfillColor={255,255,255},
            fillPattern=1));
      end Inverter;
    end OperationalAmplifierCircuit;
    
    model HydraulicControl "A hydraulic control system" 
      parameter Modelica.SIunits.Pressure ps=0.137e8 "Service pressure";
      parameter Modelica.SIunits.Pressure p0=1.0132e5 "Environment pressure";
      Modelica.Blocks.Sources.Constant ThSet annotation (extent=[-80, 0; -40,
            40]);
      Modelica.Blocks.Math.Feedback Feedback1 annotation (extent=[-20, 0; 20,
            40]);
      annotation (preferedView="info",
        Diagram,
        Coordsys(extent=[-100, -100; 300, 100]),
        Icon(
          Rectangle(extent=[0, 100; 200, -100], style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[56, 46; 152, 18],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="A hydraulic"),
          Text(
            extent=[16, 32; 184, -14],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="position control circuit")),
        Documentation(info="<html>
This is a hydraulic position control circuit.  From the outside, the system is described as a simple block diagram.  Yet, three of the blocks have been modeled internally using bond graph technology.  The development of this control circuit has been described in: <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. and A. Nebot (2005), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/modelica2005_cellier.pdf\">The
     Modelica Bond Graph Library</a>,\" <i>Proc. 4<sup>th</sup> Modelica Conference</i>, Hamburg, Germany.
</ol> <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate the bond graph during 0.3 second. <p>
 
<hr> <p>

<b>Simulation Results:</b> <p>

<img src=\"../Images/BG_Ex3.png\"> 
</html>
"),     experiment(StopTime=0.3),
        experimentSetupOutput);
      Modelica.Blocks.Nonlinear.Limiter Limiter1 annotation (extent=[40, 0; 80,
              40]);
      BondLib.Hydraulic.Servo Servo1(
        ls=1e-12,
        bs=2,
        ms=0.01,
        ks=400,
        psis=0.005,
        rs=1.25e-5) annotation (extent=[100, 0; 140, 40]);
      BondLib.Hydraulic.Servovalve Servovalve1(
        k=0.248e-6,
        x0=0.05,
        ps=ps,
        p0=p0) annotation (extent=[160, 0; 200, 40]);
      BondLib.Hydraulic.Hydromotor Hydromotor1(
        ce=0.737e-12,
        ci=0.737e-13,
        c1=5.857e13,
        psi=0.575e-5,
        rhom=1.5,
        jm=0.08,
        ps=ps,
        p0=p0) annotation (extent=[160, -40; 200, 0]);
      Modelica.Blocks.Continuous.Integrator Integrator1 annotation (extent=[220,
              -40; 260, 0]);
    equation 
      connect(ThSet.y,Feedback1.u1)             annotation (points=[-38,20; -16,
            20],     style(color=3));
      connect(Servovalve1.BondCon1, Hydromotor1.BondCon1) annotation (points=[164,0;
            164,-3.55271e-015],           style(color=8));
      connect(Servovalve1.BondCon2, Hydromotor1.BondCon2) annotation (points=[196,0;
            196,-3.55271e-015],          style(color=8));
      connect(Integrator1.y,Feedback1.u2)             annotation (points=[262,
            -20; 280,-20; 280,-80; 0,-80; 0,4], style(color=74, rgbcolor={0,0,
              156}));
      connect(Servo1.y, Servovalve1.x) annotation (points=[142,20; 160.4,20],
          style(color=74, rgbcolor={0,0,127}));
      connect(Feedback1.y, Limiter1.u) annotation (points=[18,20; 36,20], style(
            color=74, rgbcolor={0,0,127}));
      connect(Limiter1.y, Servo1.u) annotation (points=[82,20; 96,20], style(
            color=74, rgbcolor={0,0,127}));
      connect(Hydromotor1.wm, Integrator1.u) annotation (points=[200.4,-20; 216,
            -20], style(color=74, rgbcolor={0,0,127}));
    end HydraulicControl;
    
    package SolarHouse "A passive solar heating model" 
      annotation (preferedView="info",
        Window(
          x=0.01,
          y=0.31,
          width=0.44,
          height=0.65,
          library=1,
          autolayout=1),
        Icon(
          Rectangle(extent=[-85.85, 100.65; 114.15, -99.35], style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-45.85, 60.65; 74, 32],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Thermal model of"),
          Text(
            extent=[-69.85, 12.65; 98.15, -33.35],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="with passive solar heating"),
          Text(
            extent=[-57.7, 43.3; 88, 12],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="an experimental house"),
          Text(
            extent=[-57.55, 23.95; 88.15, -7.35],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="located in Tucson, AZ")),
        Diagram(
          Rectangle(extent=[-95.7, 99.3; 104.3, -100.7], style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-55.7, 59.3; 64.15, 30.65],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Thermal model of"),
          Text(
            extent=[-79.7, 29.3; 88.3, -16.7],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="with passive solar heating"),
          Text(
            extent=[-67.55, 41.95; 78.15, 10.65],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="an experimental house"),
          Text(
            extent=[-69.4, 2.6; 76.3, -28.7],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="located in Tucson, AZ")),
        Documentation(info="<html>
This package contains a thermal building model of an experimental house located near the airport in Tucson, Arizona, with a passive solar heating system.  The house contains four rooms, one of which is a solarium that collects heat during the winter and releases it during the summer. <p>

The code is collected into individual models describing the walls, the windows, etc.  The main model to be simulated is the last model of the set, called <b>House</b>. <p>

The design of this model has been described in [1,2]. <p>

The model makes use of six data tables.  Although these tables are not very large, they are used by many of the models.  In order to avoid an unnecessary explosion of table lookup in the model, the six tables were collected into a single block, called <i>TabularFunctions</i>, that is being invoked by the top level model.  The <i>TabularFunctions</i> block collects the outputs of the six tables into a single vector output signal, which gets connected down through various layers of the model hierarchy to the models that actually need the tabular data. <p>

The data items themselves are stored on an external binary file (<b>Solar_tables.mat</b>), rather than in memory, for the purpose of speed-up. <p>

I don't have pictures available from the time, when the research was done that is presented in this code (around 1991).  By now, this research unit is no longer in use.  The pictures below were taken in 2005. <p>

<img src=\"../Images/BG_Ex14.jpg\"> <p>

This is a view from the front (South side).  The solarium has meanwhile been taken out. <p>

<img src=\"../Images/BG_Ex15.jpg\"> <p>

This is a view from the back (North side). <p>

<img src=\"../Images/BG_Ex16.jpg\"> <p>

This is a view from the West side.  The house is built as an adobe structure.  In the background, there is another (more modern) research unit that is still being used for experimentation. <p>

<hr> <p>

<b>Experimentation:</b> <p>

This model uses hours as time units.  If you wish to only experiment with this model, you can simulate it over 4 days by setting the final time to 96 hours.  However, if you wish to reproduce the results obtained by Markus Weiner in his MS Thesis [1], you need to simulate the model for a full year, i.e., during 8600 hours. <p>

The model uses <b>SIunits</b> throughout for its computations.  The <b>display units</b> of <i>hours</i> instead of <i>seconds</i> for time are achieved by a <i>time scaling procedure</i>.  To this end, the input of each integrator in the model needs to be multiplied explicitly by a factor of 3600.  Since in a bondgraphic thermal model, all integrators are associated with capacitors, it suffices to divide each of the final capacitor values explicitly by the conversion factor of 3600.  This is the approach that was taken in the model. <p>

Also, the temperature values on the top-level output variables are displayed in <i>Celsius degrees</i> rather than in <i>Kelvin</i>.  The conversion is accomplished by manually converting the temperature values at the top level for the output variables, whereas internally, the model uses absolute temperature values throughout. <p>

Unfortunately, this model is rather large.  It produces far too much I/O, when all variables are being stored.  Dymola by default stores each and every variable.  Thus, if you wish to simulate over an extended period of time, you should turn off the automatic storage of all variables, and selectively store only those variables that you wish to look at.  This will speed up the simulation dramatically.  At least, you should turn off the storage of the auxiliary variables. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Weiner, M. (1992), <i>Bond Graph Model of a Passive Solar Heating System</i>, MS Thesis, Dept. of Electr. &amp;
     Comp. Engr., University of Arizona, Tucson, Arizona.
<li> Weiner, M., and F.E. Cellier (1993), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/weiner.pdf\">Modeling
     and Simulation of a Solar Energy System by Use of Bond Graphs</a>,\" <i>Proc. 1<sup>st</sup> SCS Intl. Conf. on Bond Graph
     Modeling</i>, San Diego, California, pp.301-306.
</ol>
</html>
"));
      
      block TabularFunctions "Tabular time-dependent functions" 
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Text(
              extent=[-28,-92; 8,-100],
              string="Day in the year",
              style(color=9)),
            Text(
              extent=[14,-92; 50,-100],
              string="Hour of the day",
              style(color=9)),
            Text(
              extent=[-8,-42; 44,-50],
              string="Average temperature",
              style(color=9)),
            Text(
              extent=[2,-50; 34,-58],
              string="over the year",
              style(color=9))),
          Icon(Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
            Rectangle(extent=[-36,60; 0,40],    style(color=0, fillColor=49)),
            Rectangle(extent=[-36,40; 0,20],    style(color=0, fillColor=49)),
            Rectangle(extent=[-36,20; 0,-2],    style(color=0, fillColor=49)),
            Rectangle(extent=[-36,0; 0,-22],    style(color=0, fillColor=49)),
            Rectangle(extent=[-36,-20; 0,-42],    style(color=0, fillColor=49)),
            Rectangle(extent=[-36,-40; 0,-60],    style(color=0, fillColor=49)),
            Rectangle(extent=[0,60; 36,40],    style(color=0, fillColor=7)),
            Rectangle(extent=[0,40; 36,20],    style(color=0, fillColor=7)),
            Rectangle(extent=[0,20; 36,-2],    style(color=0, fillColor=7)),
            Rectangle(extent=[0,0; 36,-20],    style(color=0, fillColor=7)),
            Rectangle(extent=[0,-20; 36,-40],    style(color=0, fillColor=7)),
            Rectangle(extent=[0,-40; 36,-60],    style(color=0, fillColor=7)),
            Line(points=[36,0; 100,0], style(color=0, rgbcolor={0,0,0})),
            Text(
              extent=[-80,98; 80,80],
              string="%name",
              style(color=3, rgbcolor={0,0,255}))),
          Documentation(info="<html>
This block computes all of the time-dependent tabular functions that the SolarHouse model requires.  It makes them available to the models that need them using a multi-signal output port.
</html>
"));
        Modelica.Blocks.Math.Gain K1(k=1/24)   annotation (extent=[-40,-90; -20,
              -70]);
        DayHour DayHour1 annotation (extent=[0,-90; 20,-70]);
        Modelica.Blocks.Tables.CombiTable1Ds TempVariation(
          tableOnFile=true,
          tableName="tempvar_table",
          fileName="Solar_tables.mat") annotation (extent=[40,-90; 60,-70]);
        Modelica.Blocks.Tables.CombiTable1Ds AvgTemperature(
          tableOnFile=true,
          tableName="avgtemp_table",
          fileName="Solar_tables.mat") annotation (extent=[-40,-60; -20,-40]);
        Modelica.Blocks.Interfaces.RealOutput y[6] 
          "Connector of Real output signals" 
          annotation (extent=[100,-10; 120,10]);
        Modelica.Blocks.Sources.Clock Clock1 annotation (extent=[-90,-10; -70,
              10]);
        Modelica.Blocks.Tables.CombiTable1Ds AT1(
          tableOnFile=true,
          tableName="AT1_table",
          fileName="Solar_tables.mat") annotation (extent=[-40,-30; -20,-10]);
        Modelica.Blocks.Tables.CombiTable1Ds BT1(
          tableOnFile=true,
          tableName="BT1_table",
          fileName="Solar_tables.mat") annotation (extent=[-40,0; -20,20]);
        Modelica.Blocks.Tables.CombiTable1Ds CT1(
          tableOnFile=true,
          tableName="CT1_table",
          fileName="Solar_tables.mat") annotation (extent=[-40,30; -20,50]);
        Modelica.Blocks.Tables.CombiTable1Ds ET1(
          tableOnFile=true,
          tableName="ET1_table",
          fileName="Solar_tables.mat") annotation (extent=[-40,60; -20,80]);
      equation 
        connect(K1.y,DayHour1.u)             annotation (points=[-19,-80; -2,
              -80],style(color=3));
        connect(DayHour1.y,TempVariation. u) annotation (points=[21,-80; 38,-80],
            style(color=74, rgbcolor={0,0,127}));
        connect(Clock1.y, K1.u) annotation (points=[-69,0; -60,0; -60,-80; -42,
              -80], style(color=74, rgbcolor={0,0,127}));
        connect(AvgTemperature.u, Clock1.y) annotation (points=[-42,-50; -60,
              -50; -60,0; -69,0], style(color=74, rgbcolor={0,0,127}));
        connect(AT1.u, Clock1.y) annotation (points=[-42,-20; -60,-20; -60,0;
              -69,0], style(color=74, rgbcolor={0,0,127}));
        connect(BT1.u, Clock1.y) annotation (points=[-42,10; -60,10; -60,0; -69,
              0], style(color=74, rgbcolor={0,0,127}));
        connect(CT1.u, Clock1.y) annotation (points=[-42,40; -60,40; -60,0; -69,
              0], style(color=74, rgbcolor={0,0,127}));
        connect(ET1.u, Clock1.y) annotation (points=[-42,70; -60,70; -60,0; -69,
              0], style(color=74, rgbcolor={0,0,127}));
        connect(TempVariation.y[1], y[1]) annotation (points=[61,-80; 80,-80;
              80,-8.33333; 110,-8.33333], style(color=74, rgbcolor={0,0,127}));
        connect(AvgTemperature.y[1], y[2]) annotation (points=[-19,-50; 78,-50;
              78,-5; 110,-5], style(color=74, rgbcolor={0,0,127}));
        connect(AT1.y[1], y[3]) annotation (points=[-19,-20; 76,-20; 76,
              -1.66667; 110,-1.66667], style(color=74, rgbcolor={0,0,127}));
        connect(BT1.y[1], y[4]) annotation (points=[-19,10; 76,10; 76,1.66667;
              110,1.66667], style(color=74, rgbcolor={0,0,127}));
        connect(CT1.y[1], y[5]) annotation (points=[-19,40; 78,40; 78,5; 110,5],
            style(color=74, rgbcolor={0,0,127}));
        connect(ET1.y[1], y[6]) annotation (points=[-19,70; 80,70; 80,8.33333;
              110,8.33333], style(color=74, rgbcolor={0,0,127}));
      end TabularFunctions;
      
      block DayHour "Conversion of fractional days to hours" 
        extends Modelica.Blocks.Interfaces.SISO;
        output Real day(unit="d") "day of the year";
        output Real hour(unit="h") "hour of the day";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Rectangle(extent=[-100, 100; 100, -100]),
            Text(extent=[-80, 60; 80, 20], string="Convert the"),
            Text(extent=[-80, 20; 80, -20], string="day of the year"),
            Text(extent=[-80, -20; 80, -60], string="to the hour of the day")),
          Icon(
            Text(extent=[-80, 72; 16, 32], string="Day of the year"),
            Rectangle(extent=[-84, 72; 20, 32]),
            Text(extent=[-22, -42; 86, -76], string="Hour of the day"),
            Rectangle(extent=[-22, -42; 86, -78]),
            Line(points=[-12, 12; 26, -26]),
            Line(points=[26, -14; 26, -26; 14, -26])),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This auxiliary block converts fractional days to hours of the day. <p>

The input variable is the simulation clock (in hours) divided by 24.  This is the day of the year, whereby the fraction indirectly contains the information about the hour of the day.  The fraction is extracted and converted back to hours.
</html>
"));
      equation 
        day = u;
        hour = 24*(day - floor(day));
        y = hour;
      end DayHour;
      
      block Temperature "Ambient temperature" 
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Text(
              extent=[-48,-32; 12,-40],
              style(color=9),
              string="Daily temperature variation"),
            Text(
              extent=[62,22; 96,16],
              style(color=9),
              string="temperature"),
            Text(
              extent=[-44,28; 8,20],
              string="Average temperature",
              style(color=9)),
            Text(
              extent=[-34,20; -2,12],
              string="over the year",
              style(color=9)),
            Text(
              extent=[62,28; 96,22],
              style(color=9),
              string="Ambient")),
          Icon(
               Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
            Rectangle(extent=[-30, 76; 26, -78]),
            Rectangle(extent=[-6, 60; 2, -54]),
            Ellipse(extent=[-8, -52; 4, -64], style(fillColor=73)),
            Rectangle(extent=[-6, -10; 2, -58], style(fillColor=73)),
            Line(points=[-8, -44; -18, -44]),
            Line(points=[-8, -42; -12, -42]),
            Line(points=[-8, -40; -12, -40]),
            Line(points=[-8, -38; -12, -38]),
            Line(points=[-8, -36; -12, -36]),
            Line(points=[-8, 36; -18, 36]),
            Line(points=[-8, 38; -12, 38]),
            Line(points=[-8, 40; -12, 40]),
            Line(points=[-8, 42; -12, 42]),
            Line(points=[-8, 44; -12, 44]),
            Line(points=[-8, -34; -18, -34]),
            Line(points=[-8, -32; -12, -32]),
            Line(points=[-8, -30; -12, -30]),
            Line(points=[-8, -28; -12, -28]),
            Line(points=[-8, -26; -12, -26]),
            Line(points=[-8, -24; -18, -24]),
            Line(points=[-8, -22; -12, -22]),
            Line(points=[-8, -20; -12, -20]),
            Line(points=[-8, -18; -12, -18]),
            Line(points=[-8, -16; -12, -16]),
            Line(points=[-8, -14; -18, -14]),
            Line(points=[-8, -12; -12, -12]),
            Line(points=[-8, -10; -12, -10]),
            Line(points=[-8, -8; -12, -8]),
            Line(points=[-8, -6; -12, -6]),
            Line(points=[-8, -4; -18, -4]),
            Line(points=[-8, -2; -12, -2]),
            Line(points=[-8, 0; -12, 0]),
            Line(points=[-8, 2; -12, 2]),
            Line(points=[-8, 4; -12, 4]),
            Line(points=[-8, 6; -18, 6]),
            Line(points=[-8, 8; -12, 8]),
            Line(points=[-8, 10; -12, 10]),
            Line(points=[-8, 12; -12, 12]),
            Line(points=[-8, 14; -12, 14]),
            Line(points=[-8, 16; -18, 16]),
            Line(points=[-8, 18; -12, 18]),
            Line(points=[-8, 20; -12, 20]),
            Line(points=[-8, 22; -12, 22]),
            Line(points=[-8, 24; -12, 24]),
            Line(points=[-8, 26; -18, 26]),
            Line(points=[-8, 28; -12, 28]),
            Line(points=[-8, 30; -12, 30]),
            Line(points=[-8, 32; -12, 32]),
            Line(points=[-8, 34; -12, 34]),
                                 Text(extent=[-148,150; 152,110],   string=
                  "%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This block computes the ambient temperature at the experimental house for any day of the year and any hour of the day.  Rather than operating on a huge data file with observed data for the location of Tucson for an entire year, the model makes use of two lookup tables. <p>

One table contains the average temperature for each day of the year measured in <sup>o</sup>K.  The table is small, as data values are stored only once per month (roughly).  Interpolation is used to interpolate between these storage data points. <p>

The other table contains the average temperature variation during one day.  Also this table is small, as data values are stored once per hour.  The temperature variation is stored as a multiplicative factor of the average temperature of the day. <p>

This model is good enough for our purposes.
</html>
"));
        Modelica.Blocks.Math.Product P1 annotation (extent=[60, -10; 80, 10]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[-140,-20; -100,20]);
        Modelica.Blocks.Interfaces.RealOutput y 
          "Connector of Real output signal" 
          annotation (extent=[100,-10; 120,10]);
      equation 
        connect(P1.u1, u[2]) annotation (points=[58,6; 40,6; 40,20; -80,20; -80,
              -10; -120,-10], style(color=74, rgbcolor={0,0,127}));
        connect(P1.y, y) annotation (points=[81,0; 110,0], style(color=74,
              rgbcolor={0,0,127}));
        connect(P1.u2, u[1]) annotation (points=[58,-6; 40,-6; 40,-40; -80,-40;
              -80,-16.6667; -120,-16.6667], style(color=74, rgbcolor={0,0,127}));
      end Temperature;
      
      block Temperature2 "Ground temperature" 
        constant Real pi=Modelica.Constants.pi;
        output Real corr "Temperature correction factor";
        output Modelica.SIunits.Temperature AvgTemp "Average temperature";
        output Modelica.SIunits.Temperature AvgTemp2 
          "Average temperature corrected";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Text(
              extent=[-52,-30; 4,-42],
              style(color=9),
              string="Daily temperature variation"),
            Text(
              extent=[60,28; 90,22],
              style(color=9),
              string="Ambient"),
            Text(
              extent=[60,24; 92,16],
              style(color=9),
              string="temperature"),
            Text(
              extent=[-82,54; -30,46],
              string="Average temperature",
              style(color=9)),
            Text(
              extent=[-72,48; -40,40],
              string="over the year",
              style(color=9)),
            Polygon(points=[-30,50; -30,30; -10,40; -30,50], style(color=2,
                  rgbcolor={0,255,0})),
            Text(
              extent=[-32,60; -8,56],
              string="Temperature",
              style(color=2, rgbcolor={0,255,0})),
            Text(
              extent=[-30,56; -10,52],
              string="Correction",
              style(color=2, rgbcolor={0,255,0})),
            Line(points=[-10,40; 40,40; 40,6; 56,6], style(color=2, rgbcolor={0,
                    255,0})),
            Line(points=[-120,-10; -80,-10; -80,40; -30,40], style(color=2,
                  rgbcolor={0,255,0}))),
          Icon(
               Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
            Rectangle(extent=[-30, 76; 26, -78]),
            Rectangle(extent=[-6, 60; 2, -54]),
            Ellipse(extent=[-8, -52; 4, -64], style(fillColor=73)),
            Rectangle(extent=[-6, -10; 2, -58], style(fillColor=73)),
            Line(points=[-8, -44; -18, -44]),
            Line(points=[-8, -42; -12, -42]),
            Line(points=[-8, -40; -12, -40]),
            Line(points=[-8, -38; -12, -38]),
            Line(points=[-8, -36; -12, -36]),
            Line(points=[-8, 36; -18, 36]),
            Line(points=[-8, 38; -12, 38]),
            Line(points=[-8, 40; -12, 40]),
            Line(points=[-8, 42; -12, 42]),
            Line(points=[-8, 44; -12, 44]),
            Line(points=[-8, -34; -18, -34]),
            Line(points=[-8, -32; -12, -32]),
            Line(points=[-8, -30; -12, -30]),
            Line(points=[-8, -28; -12, -28]),
            Line(points=[-8, -26; -12, -26]),
            Line(points=[-8, -24; -18, -24]),
            Line(points=[-8, -22; -12, -22]),
            Line(points=[-8, -20; -12, -20]),
            Line(points=[-8, -18; -12, -18]),
            Line(points=[-8, -16; -12, -16]),
            Line(points=[-8, -14; -18, -14]),
            Line(points=[-8, -12; -12, -12]),
            Line(points=[-8, -10; -12, -10]),
            Line(points=[-8, -8; -12, -8]),
            Line(points=[-8, -6; -12, -6]),
            Line(points=[-8, -4; -18, -4]),
            Line(points=[-8, -2; -12, -2]),
            Line(points=[-8, 0; -12, 0]),
            Line(points=[-8, 2; -12, 2]),
            Line(points=[-8, 4; -12, 4]),
            Line(points=[-8, 6; -18, 6]),
            Line(points=[-8, 8; -12, 8]),
            Line(points=[-8, 10; -12, 10]),
            Line(points=[-8, 12; -12, 12]),
            Line(points=[-8, 14; -12, 14]),
            Line(points=[-8, 16; -18, 16]),
            Line(points=[-8, 18; -12, 18]),
            Line(points=[-8, 20; -12, 20]),
            Line(points=[-8, 22; -12, 22]),
            Line(points=[-8, 24; -12, 24]),
            Line(points=[-8, 26; -18, 26]),
            Line(points=[-8, 28; -12, 28]),
            Line(points=[-8, 30; -12, 30]),
            Line(points=[-8, 32; -12, 32]),
            Line(points=[-8, 34; -12, 34]),
                                 Text(extent=[-146,152; 154,112],   string=
                  "%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This block computes the ground temperature at the experimental house for any day of the year and any hour of the day.  Rather than operating on a huge data file with observed data for the location of Tucson for an entire year, the model makes use of two lookup tables. <p>

One table contains the average air temperature for each day of the year measured in <sup>o</sup>K.  The table is small, as data values are stored only once per month (roughly).  Interpolation is used to interpolate between these storage data points. <p>

The other table contains the average temperature variation during one day.  Also this table is small, as data values are stored once per hour.  The temperature variation is stored as a multiplicative factor of the average temperature of the day. <p>

The average ground temperature is computed by a correction equation applied to the air temperature. <p>
</html>
"));
        Modelica.Blocks.Math.Product P1 annotation (extent=[60, -10; 80, 10]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[-140,-20; -100,20]);
        Modelica.Blocks.Interfaces.RealOutput y 
          "Connector of Real output signal" 
          annotation (extent=[100,-10; 120,10]);
      equation 
        AvgTemp = u[2];
        corr = 1 + 0.004*sin(pi*time/60) + 0.0025*cos(pi*(time - 60)/120) +
          0.0002*sin(pi*(time - 30)/180);
        AvgTemp2 = corr*AvgTemp;
        P1.u1 = AvgTemp2;
        connect(P1.u2, u[1]) annotation (points=[58,-6; 40,-6; 40,-40; -80,-40;
              -80,-16.6667; -120,-16.6667], style(color=74, rgbcolor={0,0,127}));
        connect(P1.y, y) annotation (points=[81,0; 110,0], style(color=74,
              rgbcolor={0,0,127}));
      end Temperature2;
      
      block TFilter "Auxiliary block for air conditioning" 
        extends Modelica.Blocks.Interfaces.SISO;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Rectangle(extent=[-100, 100; 100, -100]),
            Text(extent=[-80, 60; 80, 20], string="Temperature"),
            Text(extent=[-80, 20; 80, -20], string="Filter")),
          Icon(Text(extent=[-80, 20; 80, -20], string="Filter"), Text(extent=[-
                  80, 60; 80, 20], string="Temperature")),
          Documentation(info="<html>
This auxiliary block represents a \"temperature filter\" used inside the air conditioning unit for control purposes.
</html>
"));
      equation 
        y = 1 - 2/(1 + exp(-u));
      end TFilter;
      
      block AirConditioning "Air conditioning unit" 
        extends Modelica.Blocks.Interfaces.SISO;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-76, 38; 24, -58]),
            Line(points=[-76, 38; -20, 64]),
            Line(points=[24, 38; 80, 64]),
            Line(points=[24, -58; 80, -28]),
            Line(points=[-20, 64; 80, 64; 80, -28]),
            Rectangle(extent=[-70, 32; -28, -52]),
            Rectangle(extent=[-24, 32; 18, -52]),
            Rectangle(extent=[-68, 30; -30, 26]),
            Rectangle(extent=[-22, -46; 16, -50]),
            Rectangle(extent=[-68, 24; -30, 20]),
            Rectangle(extent=[-68, 18; -30, 14]),
            Rectangle(extent=[-68, 12; -30, 8]),
            Rectangle(extent=[-68, 4; -30, 0]),
            Rectangle(extent=[-68, -2; -30, -6]),
            Rectangle(extent=[-68, -8; -30, -12]),
            Rectangle(extent=[-68, -14; -30, -18]),
            Rectangle(extent=[-68, -22; -30, -26]),
            Rectangle(extent=[-68, -28; -30, -32]),
            Rectangle(extent=[-68, -34; -30, -38]),
            Rectangle(extent=[-68, -40; -30, -44]),
            Rectangle(extent=[-68, -46; -30, -50]),
            Rectangle(extent=[-22, 30; 16, 26]),
            Rectangle(extent=[-22, 24; 16, 20]),
            Rectangle(extent=[-22, 18; 16, 14]),
            Rectangle(extent=[-22, 12; 16, 8]),
            Rectangle(extent=[-22, 4; 16, 0]),
            Rectangle(extent=[-22, -2; 16, -6]),
            Rectangle(extent=[-22, -8; 16, -12]),
            Rectangle(extent=[-22, -14; 16, -18]),
            Rectangle(extent=[-22, -22; 16, -26]),
            Rectangle(extent=[-22, -28; 16, -32]),
            Rectangle(extent=[-22, -34; 16, -38]),
            Rectangle(extent=[-22, -40; 16, -44])),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The air conditioning unit uses a control algorithm to determine the amount of active air conditioning to be applied to the living room.
</html>
"),       Diagram);
        Modelica.Blocks.Sources.Constant C1(k=300)   annotation (extent=[-80,20;
              -60,40]);
        Modelica.Blocks.Sources.Constant C2(k=295)   annotation (extent=[-80,
              -30; -60,-10]);
        TFilter TF1 annotation (extent=[-20,40; 0,60]);
        TFilter TF2 annotation (extent=[-20,-10; 0,10]);
        Modelica.Blocks.Nonlinear.Limiter L1(uMax=0, uMin=-10)     annotation (
            extent=[10, 40; 30, 60]);
        Modelica.Blocks.Nonlinear.Limiter L2(uMax=10, uMin=0)     annotation (
            extent=[10, -10; 30, 10]);
        Modelica.Blocks.Math.Add S1(k2=-1) annotation (extent=[-50, 40; -30, 60]);
        Modelica.Blocks.Math.Add S2(k2=-1) annotation (extent=[-50, -10; -30,
              10]);
        Modelica.Blocks.Math.Add A1 annotation (extent=[40,14; 60,34]);
        Modelica.Blocks.Math.Division Division1 annotation (extent=[40, -80; 60,
                -60]);
        Modelica.Blocks.Math.Gain G1(k=10000)   annotation (extent=[0, -60; 20,
                -40]);
      equation 
        connect(S1.u1, u) annotation (points=[-52,56; -90,56; -90,0; -120,0],
            style(color=74, rgbcolor={0,0,127}));
        connect(S2.u1, u) annotation (points=[-52,6; -56,6; -56,0; -120,0],
            style(color=74, rgbcolor={0,0,127}));
        connect(S2.u2, C2.y) annotation (points=[-52,-6; -56,-6; -56,-20; -59,
              -20], style(color=74, rgbcolor={0,0,127}));
        connect(S1.u2, C1.y) annotation (points=[-52,44; -56,44; -56,30; -59,30],
            style(color=74, rgbcolor={0,0,127}));
        connect(S1.y, TF1.u) annotation (points=[-29,50; -22,50], style(color=
                74, rgbcolor={0,0,127}));
        connect(TF1.y, L1.u) annotation (points=[1,50; 8,50], style(color=74,
              rgbcolor={0,0,127}));
        connect(TF2.u, S2.y) annotation (points=[-22,0; -29,0], style(color=74,
              rgbcolor={0,0,127}));
        connect(TF2.y, L2.u) 
          annotation (points=[1,0; 8,0], style(color=74, rgbcolor={0,0,127}));
        connect(L1.y, A1.u1) annotation (points=[31,50; 34,50; 34,30; 38,30],
            style(color=74, rgbcolor={0,0,127}));
        connect(L2.y, A1.u2) annotation (points=[31,0; 34,0; 34,18; 38,18],
            style(color=74, rgbcolor={0,0,127}));
        connect(A1.y, G1.u) annotation (points=[61,24; 70,24; 70,-30; -10,-30;
              -10,-50; -2,-50], style(color=74, rgbcolor={0,0,127}));
        connect(G1.y, Division1.u1) annotation (points=[21,-50; 30,-50; 30,-64;
              38,-64], style(color=74, rgbcolor={0,0,127}));
        connect(Division1.u2, u) annotation (points=[38,-76; -90,-76; -90,0;
              -120,0], style(color=74, rgbcolor={0,0,127}));
        connect(Division1.y, y) annotation (points=[61,-70; 84,-70; 84,0; 110,0],
            style(color=74, rgbcolor={0,0,127}));
      end AirConditioning;
      
      block SolarPosition "Solar position in the sky" 
        constant Real pi=Modelica.Constants.pi;
        parameter Real lon(unit="deg") = 111 "Longitude";
        parameter Real lat(unit="deg") = 32 "Latitude";
        parameter Real lsm(unit="deg") = 105;
        parameter Real sfa(unit="deg") "Orientation of surface";
        parameter Real tilt(unit="deg") "Tilt of surface";
        parameter Real latr(unit="rad") = pi*lat/180 "Latitude";
        parameter Real sfar(unit="rad") = pi*sfa/180 "Orientation of surface";
        parameter Real tiltr(unit="rad") = pi*tilt/180 "Tilt of surface";
        output Real h "Height";
        output Real dec "Declination";
        output Real inc1;
        output Real inc2;
        output Real inc3;
        output Real Time(unit="h");
        output Real ETime;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
            Ellipse(extent=[-24, 24; 26, -26]),
            Polygon(points=[-66, 32; -18, 16; -36, 66; -6, 24; -6, 22; 2, 76; 8,
                    22; 44, 64; 20, 14; 70, 36; 26, 4; 78, -2; 26, -8; 70, -36;
                    18, -18; 38, -68; 8, -26; 2, -78; -6, -24; -38, -66; -18, -
                  20; -64, -38; -22, -8; -74, -2; -24, 2; -66, 32], style(color=
                   41, fillColor=49)),
            Text(extent=[102,-30; 142,-54],  string="alt"),
            Text(extent=[102,54; 142,30],  string="inc"),
                                 Text(extent=[-144,154; 156,114],   string=
                  "%name")),
          Diagram(
            Line(points=[118,46; 128,40; 118,34],    style(color=9)),
            Line(points=[82,40; 128,40],   style(color=9)),
            Line(points=[118, -34; 128, -40; 118, -46], style(color=9)),
            Line(points=[82, -40; 128, -40], style(color=9)),
            Text(
              extent=[78,52; 118,40],
              string="Inclination",
              style(color=9)),
            Text(
              extent=[84,-30; 118,-40],
              style(color=9),
              string="Altitude"),
            Rectangle(extent=[-20,80; 80,-80], style(color=2, rgbcolor={0,255,0})),
            Text(
              extent=[-2,20; 62,2],
              string="Spherical",
              style(color=2, rgbcolor={0,255,0})),
            Text(
              extent=[-4,-2; 66,-18],
              string="Trigonometry",
              style(color=2, rgbcolor={0,255,0})),
            Line(points=[-100,0; -20,0], style(color=2, rgbcolor={0,255,0})),
            Line(points=[-40,-40; -40,-40; -20,-40], style(color=2, rgbcolor={0,
                    255,0})),
            Line(points=[80, -60; 100, -60], style(color=2, rgbcolor={0,255,0})),
            Text(
              extent=[-54,10; -20,0],
              string="ETime",
              style(color=2, rgbcolor={0,255,0})),
            Line(points=[80,60; 100,60], style(color=2, rgbcolor={0,255,0}))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This block computes the postion of the sun in the sky using spherical trigonometry.
</html>
"));
        Modelica.Blocks.Interfaces.RealOutput alt(redeclare type SignalType = 
              Modelica.SIunits.Angle) "Altitude"    annotation (extent=[100, -
              70; 120, -50]);
        Modelica.Blocks.Sources.Clock Clock1 annotation (extent=[-60,-50; -40,
              -30]);
        Modelica.Blocks.Interfaces.RealOutput inc(redeclare type SignalType = 
              Modelica.SIunits.Angle) "Inclination" annotation (extent=[100,50;
              120,70]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[-140,-20; -100,20]);
      equation 
        Time = Clock1.y;
        ETime = u[6];
        h = (pi/12)*(Time - 12 + ETime/60 + (lsm - lon)/15);
        dec = (pi/180)*23.45*sin(2*pi*(284 + Time/24)/365);
        inc1 = sin(dec)*sin(latr)*cos(tiltr) - sin(dec)*cos(latr)*sin(tiltr)*
          cos(sfar);
        inc2 = cos(dec)*cos(latr)*cos(tiltr)*cos(h) + cos(dec)*sin(latr)*sin(
          tiltr)*cos(sfar)*cos(h);
        inc3 = cos(dec)*sin(tiltr)*sin(sfar)*sin(h);
        inc = inc1 + inc2 + inc3;
        alt = cos(latr)*cos(dec)*cos(h) + sin(latr)*sin(dec);
      end SolarPosition;
      
      block SolarRadiation "Solar radiation" 
        constant Real pi=Modelica.Constants.pi;
        parameter Real sc=1353;
        parameter Real tilt(unit="deg") "Tilt of surface";
        parameter Real tiltr(unit="rad") = pi*tilt/180 "Tilt of surface";
        parameter Real ro=0.33;
        parameter Real cr=1 "Direct light gain factor";
        parameter Real cf=1 "Diffuse light gain factor";
        parameter Real absor "Absorption coefficient";
        parameter Modelica.SIunits.Area area "Area of exposed surface";
        parameter Real OH;
        parameter Real W;
        output Real altL "Limited altitude value";
        output Real sou;
        output Real idn;
        output Real dir "Direct light";
        output Real dif "Diffuse light";
        output Real Time(unit="h");
        output Real BTime;
        output Real CTime;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Line(points=[90, 20; 126, 20], style(color=9)),
            Line(points=[116, 26; 126, 20; 116, 14], style(color=9)),
            Text(
              extent=[88, 32; 114, 20],
              string="Entropy",
              style(color=9)),
            Text(
              extent=[32,-12; 50,-16],
              string="Entropy",
              style(color=9)),
            Text(
              extent=[31,-16; 49,-20],
              string="Output",
              style(color=9)),
            Text(
              extent=[-1,10; 17,6],
              string="Heat",
              style(color=9)),
            Text(
              extent=[1,6; 19,2],
              string="Output",
              style(color=9)),
            Text(
              extent=[-82,-96; -52,-100],
              string="Direct Light",
              style(color=9)),
            Text(
              extent=[-50,-96; -20,-100],
              string="Diffuse Light",
              style(color=9))),
          Icon(Rectangle(extent=[-100,-100; 100,100],   style(
              color=74,
              rgbcolor={0,0,127},
              fillColor=7,
              rgbfillColor={255,255,255})),
            Ellipse(extent=[-24, 24; 26, -26]),
            Polygon(points=[-66, 32; -18, 16; -36, 66; -6, 24; -6, 22; 2, 76; 8,
                    22; 44, 64; 20, 14; 70, 36; 26, 4; 78, -2; 26, -8; 70, -36;
                    18, -18; 38, -68; 8, -26; 2, -78; -6, -24; -38, -66; -18, -
                  20; -64, -38; -22, -8; -74, -2; -24, 2; -66, 32], style(color=
                   41, fillColor=49)),
            Line(points=[16, -52; 22, -92], style(color=41, thickness=2)),
            Line(points=[44, -42; 78, -78], style(color=41, thickness=2)),
            Line(points=[62, -16; 92, -24], style(color=41, thickness=2)),
            Line(points=[60, -22; 90, -34], style(color=41, thickness=2)),
            Line(points=[48, -36; 88, -62], style(color=41, thickness=2)),
            Line(points=[38, -52; 64, -90], style(color=41, thickness=2)),
            Line(points=[22, -54; 36, -90], style(color=41, thickness=2)),
            Text(
              extent=[-164,-54; -138,-70],
              string="alt",
              style(color=3, rgbcolor={0,0,255})),
            Text(
              extent=[-166,68; -140,52],
              string="inc",
              style(color=3, rgbcolor={0,0,255})),
            Text(
              extent=[-36,-82; -6,-100],
              string="Temp",
              style(color=9)),
            Text(
              extent=[88, 32; 114, 20],
              string="Entropy",
              style(color=9)),
            Line(points=[116, 26; 126, 20; 116, 14], style(color=9)),
            Line(points=[126, 20; 86, 20], style(color=9)),
                                 Text(extent=[-150,152; 150,112],   string=
                  "%name")),
          Window(
            x=0.01,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This block computes the solar radiation available to any sun-exposed surface on the basis of the position of the sun in the sky and the relative angle at which the exposed surface faces the sun.  Also used are the color of the exposed surface (absorption coefficient). <p>

The model is fairly complex.  It calculates not only the direct radiation from the sun, but also the diffuse radiation received through the atmosphere.
</html>
"));
        Modelica.Blocks.Interfaces.RealInput alt(redeclare type SignalType = 
              Modelica.SIunits.Angle) "Altitude"  annotation (extent=[-140, -80;
                -100, -40]);
        Modelica.Blocks.Interfaces.RealInput Temp(redeclare type SignalType = 
              Modelica.SIunits.Temperature) "Temperature" 
                                                   annotation (extent=[-10,-110;
              10,-90],       rotation=90);
        Modelica.Blocks.Nonlinear.Limiter L2(uMax=1E10, uMin=0)     annotation (
           extent=[70,-10; 90,10]);
        Modelica.Blocks.Nonlinear.Limiter altLim(uMax=10, uMin=0.05) 
          annotation (extent=[-80, 60; -60, 80]);
        Modelica.Blocks.Nonlinear.Limiter Ldir(uMax=1E10, uMin=0) 
          annotation (extent=[-76,-90; -56,-70],   rotation=90);
        Modelica.Blocks.Nonlinear.Limiter Ldif(uMax=1E10, uMin=0) 
          annotation (extent=[-46,-90; -26,-70],   rotation=90);
        Modelica.Blocks.Math.Product P1 annotation (extent=[40,-10; 60,10]);
        Modelica.Blocks.Math.Division D1 annotation (extent=[10, -20; 30, 0]);
        Modelica.Blocks.Math.Add A1 annotation (extent=[-60,-30; -40,-10],
            rotation=90);
        Modelica.Blocks.Math.Gain Absorption(k=absor)   annotation (extent=[-20,-14;
              0,6]);
        Modelica.Blocks.Math.Gain Gdif(k=cf)   annotation (extent=[-46,-60; -26,
              -40],     rotation=90);
        Modelica.Blocks.Math.Gain Gdir(k=cr)   annotation (extent=[-76,-60; -56,
              -40],     rotation=90);
        Modelica.Blocks.Math.Sign altSign annotation (extent=[-80, 20; -60, 40]);
        Modelica.Blocks.Sources.Clock Clock1 annotation (extent=[50,-70; 70,-50]);
        Modelica.Blocks.Interfaces.RealInput inc(redeclare type SignalType = 
              Modelica.SIunits.Angle) "Inclination" 
                                                  annotation (extent=[-140,40;
              -100,80]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[-140,-20; -100,20]);
        Modelica.Blocks.Interfaces.RealOutput y 
          "Connector of Real output signal" 
          annotation (extent=[100,-10; 120,10]);
      equation 
        Time = Clock1.y;
        BTime = u[4];
        CTime = u[5];
        altL = altLim.y;
        sou = sc*(1 + 0.033*cos(2*pi*(Time/24)/365.));
        idn = sou/exp(BTime/altL);
        dir = idn*inc*(area - OH*W*(alt/sqrt(1 - alt*alt)));
        dif = idn*area*(CTime*(1 + cos(tiltr))/2 + (CTime + alt)*ro*(1 - cos(
          tiltr))/2);
        Ldir.u = dir;
        Ldif.u = dif;
        connect(D1.y, P1.u2) annotation (points=[31,-10; 34,-10; 34,-6; 38,-6],
            style(color=74, rgbcolor={0,0,127}));
        connect(altSign.y, P1.u1) annotation (points=[-59,30; 34,30; 34,6; 38,6],
            style(color=74, rgbcolor={0,0,127}));
        connect(P1.y, L2.u) annotation (points=[61,0; 68,0], style(color=74,
              rgbcolor={0,0,127}));
        connect(D1.u2, Temp) annotation (points=[8,-16; 0,-16; 0,-100], style(
              color=74, rgbcolor={0,0,127}));
        connect(Absorption.y, D1.u1) annotation (points=[1,-4; 8,-4], style(
              color=74, rgbcolor={0,0,127}));
        connect(Gdif.y, A1.u2) annotation (points=[-36,-39; -36,-36; -44,-36;
              -44,-32], style(color=74, rgbcolor={0,0,127}));
        connect(Gdir.y, A1.u1) annotation (points=[-66,-39; -66,-36; -56,-36;
              -56,-32], style(color=74, rgbcolor={0,0,127}));
        connect(Ldif.y, Gdif.u) annotation (points=[-36,-69; -36,-62], style(
              color=74, rgbcolor={0,0,127}));
        connect(Ldir.y, Gdir.u) annotation (points=[-66,-69; -66,-62], style(
              color=74, rgbcolor={0,0,127}));
        connect(altLim.u, alt) annotation (points=[-82,70; -90,70; -90,-60;
              -120,-60], style(color=74, rgbcolor={0,0,127}));
        connect(altSign.u, alt) annotation (points=[-82,30; -90,30; -90,-60;
              -120,-60], style(color=74, rgbcolor={0,0,127}));
        connect(L2.y, y) annotation (points=[91,0; 110,0], style(color=74,
              rgbcolor={0,0,127}));
        connect(A1.y, Absorption.u) annotation (points=[-50,-9; -50,-4; -22,-4],
            style(color=74, rgbcolor={0,0,127}));
      end SolarRadiation;
      
      block SolarGlass "Solar radiation through glass" 
        constant Real pi=Modelica.Constants.pi;
        parameter Real sc;
        parameter Real tilt(unit="deg") "Tilt of surface";
        parameter Real tiltr(unit="rad") = pi*tilt/180 "Tilt of surface";
        parameter Real ro=0.33;
        parameter Real cr=1 "Direct light gain factor";
        parameter Real cf=1 "Diffuse light gain factor";
        parameter Real hi=8.3;
        parameter Real ho=22.7;
        parameter Modelica.SIunits.Area area "Area of exposed surface";
        parameter Real OH;
        parameter Real W;
        output Real altL "Limited altitude value";
        output Real idn;
        output Real dir "Direct light";
        output Real dif "Diffuse light";
        output Real Time(unit="h");
        output Real ATime;
        output Real BTime;
        output Real CTime;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[1, 1],
            component=[10, 10]),
          Icon(
            Rectangle(extent=[-100, -100; 100, 100], style(color=3, fillColor=7)),
            Ellipse(extent=[-24, 24; 26, -26]),
            Polygon(points=[-66, 32; -18, 16; -36, 66; -6, 24; -6, 22; 2, 76; 8,
                    22; 44, 64; 20, 14; 70, 36; 26, 4; 78, -2; 26, -8; 70, -36;
                    18, -18; 38, -68; 8, -26; 2, -78; -6, -24; -38, -66; -18, -
                  20; -64, -38; -22, -8; -74, -2; -24, 2; -66, 32], style(color=
                   41, fillColor=49)),
            Line(points=[16, -52; 22, -92], style(color=41, thickness=2)),
            Line(points=[44, -42; 78, -78], style(color=41, thickness=2)),
            Line(points=[62, -16; 92, -24], style(color=41, thickness=2)),
            Line(points=[60, -22; 90, -34], style(color=41, thickness=2)),
            Line(points=[48, -36; 88, -62], style(color=41, thickness=2)),
            Line(points=[38, -52; 64, -90], style(color=41, thickness=2)),
            Line(points=[22, -54; 36, -90], style(color=41, thickness=2)),
            Text(
              extent=[-166,-51; -140,-67],
              string="alt",
              style(color=3, rgbcolor={0,0,255})),
            Text(
              extent=[88, 32; 114, 20],
              string="Entropy",
              style(color=9)),
            Line(points=[116, 26; 126, 20; 116, 14], style(color=9)),
            Line(points=[126, 20; 86, 20], style(color=9)),
            Text(
              extent=[-42, -80; -12, -98],
              string="Temp",
              style(color=9)),
            Line(points=[38, 2; 38, 8; 96, -2; 96, -100], style(color=57,
                  thickness=2)),
            Line(points=[38, -6; 38, -16], style(color=53, thickness=2)),
            Line(points=[38, -26; 38, -88; 96, -100], style(color=53, thickness=
                   2)),
            Text(
              extent=[-167,70; -141,54],
              string="inc",
              style(color=3, rgbcolor={0,0,255})),
                                 Text(extent=[-146,156; 154,116],   string=
                  "%name")),
          Diagram(
            Text(
              extent=[-88, -1; -80, -4],
              string="direct",
              style(color=9)),
            Text(
              extent=[-88, -4; -80, -7],
              string="light",
              style(color=9)),
            Text(
              extent=[44, -6; 55, -9],
              string="diffuse",
              style(color=9)),
            Text(
              extent=[45, -10; 53, -13],
              string="light",
              style(color=9))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This block computes the solar radiation available to any sun-exposed glass surface on the basis of the position of the sun in the sky and the relative angle at which the exposed surface faces the sun.  Glass windows require a special model, because some of the incoming radiation is directly transmitted through the glass. <p>

The model is fairly complex.  It calculates not only the direct radiation from the sun, but also the diffuse radiation received through the atmosphere.
</html>
"));
        Modelica.Blocks.Interfaces.RealInput alt(redeclare type SignalType = 
              Modelica.SIunits.Angle) "Altitude"  annotation (extent=[-140,-80;
              -100,-40],    layer="icon");
        Modelica.Blocks.Interfaces.RealInput Temp(redeclare type SignalType = 
              Modelica.SIunits.Temperature) "Temperature" 
                                                   annotation (extent=[-10,-110;
              10,-90],       rotation=90);
        DirectGlassTransmission DirectGlassTransmission1 annotation (extent=[
              -80,23; -60,43]);
        DirectGlassAbsorption DirectGlassAbsorption1 annotation (extent=[-80, -
              60; -60, -40]);
        Modelica.Blocks.Nonlinear.Limiter Ldir(uMax=1E10, uMin=0) 
          annotation (extent=[-76,-10; -66,0]);
        Modelica.Blocks.Math.Gain Gdir(k=cr)   annotation (extent=[-60, -10; -
              50, 0]);
        Modelica.Blocks.Math.Product P1 annotation (extent=[-40, 25; -30, 35]);
        Modelica.Blocks.Math.Product P2 annotation (extent=[-40,-56; -30,-46]);
        Modelica.Blocks.Math.Add A1 annotation (extent=[-20, 22; -10, 32]);
        Modelica.Blocks.Math.Add A2 annotation (extent=[-20,-53; -10,-43]);
        Modelica.Blocks.Math.Product P3 annotation (extent=[-10, 0; -20, 10]);
        Modelica.Blocks.Math.Product P4 annotation (extent=[-10, -30; -20, -20]);
        DiffuseGlassTransmission DiffuseGlassTransmission1 annotation (extent=[
              20, -2; 0, 18]);
        DiffuseGlassAbsorption DiffuseGlassAbsorption1 annotation (extent=[20,
              -38; 0, -18]);
        Modelica.Blocks.Math.Gain Gdif(k=2*cf)   annotation (extent=[40, -15;
              30, -5]);
        Modelica.Blocks.Math.Gain G1(k=hi/(hi + ho))   annotation (extent=[26,
              -55; 40,-41]);
        Modelica.Blocks.Math.Add A3 annotation (extent=[50,-50; 60,-40]);
        Modelica.Blocks.Math.Division Division1 annotation (extent=[5, -75; 15,
                -65]);
        Modelica.Blocks.Math.Gain G2(k=sc)   annotation (extent=[-15, -75; -5,
              -65]);
        Modelica.Blocks.Nonlinear.Limiter L1(uMax=1E10, uMin=0)     annotation (
           extent=[25, -75; 35, -65]);
        Modelica.Blocks.Math.Product P5 annotation (extent=[45, -78; 55, -68]);
        Modelica.Blocks.Math.Sign altSig annotation (extent=[-70, -95; -60, -85]);
        Modelica.Blocks.Nonlinear.Limiter L2(uMax=1E10, uMin=0)     annotation (
           extent=[65, -78; 75, -68]);
        Modelica.Blocks.Nonlinear.Limiter altLim(uMax=10, uMin=0.05) 
          annotation (extent=[-70, -75; -60, -65]);
        Modelica.Blocks.Interfaces.RealInput inc(redeclare type SignalType = 
              Modelica.SIunits.Angle) "Inclination" 
                                                  annotation (extent=[-140,40;
              -100,80],     layer="icon");
        Modelica.Blocks.Sources.Clock Clock1 annotation (extent=[-80,70; -60,90]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[-140,-20; -100,20]);
        Modelica.Blocks.Interfaces.RealOutput y 
          "Connector of Real output signal" 
          annotation (extent=[100,-10; 120,10]);
      equation 
        Time = Clock1.y;
        ATime = u[3];
        BTime = u[4];
        CTime = u[5];
        altL = altLim.y;
        idn = ATime/exp(BTime/altL);
        dir = idn*inc*(area - OH*W*(alt/sqrt(1 - alt*alt)));
        dif = idn*area*(CTime*(1 + cos(tiltr))/2 + (CTime + alt)*ro*(1 - cos(
          tiltr))/2);
        Ldir.u = dir;
        Gdif.u = dif;
        connect(DirectGlassTransmission1.u, inc) annotation (points=[-82,33;
              -90,33; -90,60; -120,60], style(color=74, rgbcolor={0,0,127}));
        connect(DirectGlassAbsorption1.u, inc) annotation (points=[-82,-50; -90,
              -50; -90,60; -120,60], style(color=74, rgbcolor={0,0,127}));
        connect(DirectGlassTransmission1.y, P1.u1) annotation (points=[-59,33;
              -41,33], style(color=74, rgbcolor={0,0,127}));
        connect(altSig.y, P5.u2) annotation (points=[-59.5,-90; -35,-90; -35,
              -85; 40,-85; 40,-76; 44,-76], style(color=74, rgbcolor={0,0,127}));
        connect(Division1.u2, Temp) annotation (points=[4,-73; 0,-73; 0,-100],
            style(color=74, rgbcolor={0,0,127}));
        connect(Division1.u1, G2.y) annotation (points=[4,-67; 0,-67; 0,-70;
              -4.5,-70], style(color=74, rgbcolor={0,0,127}));
        connect(Division1.y, L1.u) annotation (points=[15.5,-70; 24,-70], style(
              color=74, rgbcolor={0,0,127}));
        connect(L1.y, P5.u1) annotation (points=[35.5,-70; 44,-70], style(color=
               74, rgbcolor={0,0,127}));
        connect(P5.y, L2.u) annotation (points=[55.5,-73; 64,-73], style(color=
                74, rgbcolor={0,0,127}));
        connect(altSig.u, alt) annotation (points=[-71,-90; -90,-90; -90,-60;
              -120,-60], style(color=74, rgbcolor={0,0,127}));
        connect(altLim.u, alt) annotation (points=[-71,-70; -90,-70; -90,-60;
              -120,-60], style(color=74, rgbcolor={0,0,127}));
        connect(DirectGlassAbsorption1.y, P2.u2) annotation (points=[-59,-50;
              -50,-50; -50,-54; -41,-54], style(color=74, rgbcolor={0,0,127}));
        connect(Gdir.y, P2.u1) annotation (points=[-49.5,-5; -45,-5; -45,-48;
              -41,-48], style(color=74, rgbcolor={0,0,127}));
        connect(Gdir.y, P1.u2) annotation (points=[-49.5,-5; -45,-5; -45,27;
              -41,27], style(color=74, rgbcolor={0,0,127}));
        connect(P3.y, A1.u2) annotation (points=[-20.5,5; -25,5; -25,24; -21,24],
            style(color=74, rgbcolor={0,0,127}));
        connect(DiffuseGlassTransmission1.y, P3.u1) annotation (points=[-1,8;
              -9,8], style(color=74, rgbcolor={0,0,127}));
        connect(A1.y, A3.u1) annotation (points=[-9.5,27; 55,27; 55,-24; 45,-24;
              45,-42; 49,-42], style(color=74, rgbcolor={0,0,127}));
        connect(G1.y, A3.u2) annotation (points=[40.7,-48; 49,-48], style(color=
               74, rgbcolor={0,0,127}));
        connect(A2.y, G1.u) annotation (points=[-9.5,-48; 24.6,-48], style(
              color=74, rgbcolor={0,0,127}));
        connect(DiffuseGlassAbsorption1.y, P4.u2) annotation (points=[-1,-28;
              -9,-28], style(color=74, rgbcolor={0,0,127}));
        connect(Gdif.y, P4.u1) annotation (points=[29.5,-10; -6,-10; -6,-22; -9,
              -22], style(color=74, rgbcolor={0,0,127}));
        connect(Gdif.y, P3.u2) annotation (points=[29.5,-10; -6,-10; -6,2; -9,2],
            style(color=74, rgbcolor={0,0,127}));
        connect(A3.y, G2.u) annotation (points=[60.5,-45; 65,-45; 65,-60; -20,
              -60; -20,-70; -16,-70], style(color=74, rgbcolor={0,0,127}));
        connect(Ldir.y, Gdir.u) annotation (points=[-65.5,-5; -61,-5], style(
              color=74, rgbcolor={0,0,127}));
        connect(P4.y, A2.u1) annotation (points=[-20.5,-25; -25,-25; -25,-45;
              -21,-45], style(color=74, rgbcolor={0,0,127}));
        connect(P2.y, A2.u2) annotation (points=[-29.5,-51; -21,-51], style(
              color=74, rgbcolor={0,0,127}));
        connect(P1.y, A1.u1) annotation (points=[-29.5,30; -21,30], style(color=
               74, rgbcolor={0,0,127}));
        connect(L2.y, y) annotation (points=[75.5,-73; 85,-73; 85,0; 110,0],
            style(color=74, rgbcolor={0,0,127}));
      end SolarGlass;
      
      block DirectGlassAbsorption 
        "Absorption coefficient of glass for direct solar radiation" 
        extends Modelica.Blocks.Interfaces.SISO;
        output Real inc "Inclination";
        output Real absor "Absorption";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Text(
              extent=[-146,10; -120,-8],
              string="inc",
              style(color=3, rgbcolor={0,0,255})),
            Text(extent=[-60, 78; 60, 38], string="Absorption of"),
            Text(extent=[-80, 38; 82, -2], string="direct solar radiation"),
            Text(extent=[-60, -2; 60, -42], string="for DSA glass")),
          Icon(
            Text(extent=[-60, 0; 60, -40], string="for DSA glass"),
            Text(extent=[-80, 40; 82, 0], string="direct solar radiation"),
            Text(extent=[-60, 80; 60, 40], string="Absorption of"),
            Text(
              extent=[-140, 8; -114, -10],
              string="inc",
              style(color=49))),
          Documentation(info="<html>
This auxiliary block computes the absorption coefficient for glass with respect to direct solar radiation.
</html>
"));
      equation 
        inc = u;
        absor = 0.01154 + 0.77674*inc - 3.94657*inc*inc + 8.57881*inc^3 -
          8.38135*inc^4 + 3.01188*inc^5;
        y = absor;
      end DirectGlassAbsorption;
      
      block DiffuseGlassAbsorption 
        "Absorption coefficient of glass for diffuse solar radiation" 
        extends Modelica.Blocks.Interfaces.SO;
        output Real absor "Absorption";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Text(extent=[-60, 80; 60, 40], string="Absorption of"),
            Text(extent=[-80, 40; 82, 0], string="diffuse solar radiation"),
            Text(extent=[-60, 0; 60, -40], string="for DSA glass")),
          Icon(
            Text(extent=[-60, 80; 60, 40], string="Absorption of"),
            Text(extent=[-80, 40; 82, 0], string="diffuse solar radiation"),
            Text(extent=[-60, 0; 60, -40], string="for DSA glass")),
          Documentation(info="<html>
This auxiliary block computes the absorption coefficient for glass with respect to diffuse solar radiation.
</html>
"));
      equation 
        absor = 0.01154/2 + 0.77674/3 - 3.94657/4 + 8.57881/5 - 8.38135/6 +
          3.01188/7;
        y = absor;
      end DiffuseGlassAbsorption;
      
      block DirectGlassTransmission 
        "Transmission coefficient of glass for direct solar radiation" 
        extends Modelica.Blocks.Interfaces.SISO;
        output Real inc "Inclination";
        output Real trans "Transmission";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Text(extent=[-60, 78; 60, 38], string="Transmission of"),
            Text(extent=[-80, 38; 82, -2], string="direct solar radiation"),
            Text(extent=[-60, -2; 60, -42], string="for DSA glass"),
            Text(
              extent=[-144,10; -118,-8],
              string="inc",
              style(color=3, rgbcolor={0,0,255}))),
          Icon(
            Text(extent=[-60, 78; 60, 38], string="Transmission of"),
            Text(extent=[-80, 38; 82, -2], string="direct solar radiation"),
            Text(extent=[-60, -2; 60, -42], string="for DSA glass"),
            Text(
              extent=[-140, 8; -114, -10],
              string="inc",
              style(color=49))),
          Documentation(info="<html>
This auxiliary block computes the transmission coefficient for glass with respect to direct solar radiation.
</html>
"));
      equation 
        inc = u;
        trans = -0.00885 + 2.71235*inc - 0.62062*inc*inc - 7.07329*inc^3 +
          9.75995*inc^4 - 3.89922*inc^5;
        y = trans;
      end DirectGlassTransmission;
      
      block DiffuseGlassTransmission 
        "Transmission coefficient of glass for diffuse solar radiation" 
        extends Modelica.Blocks.Interfaces.SO;
        output Real trans "Transmission";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Text(extent=[-60, 78; 60, 38], string="Transmission of"),
            Text(extent=[-80, 38; 82, -2], string="diffuse solar radiation"),
            Text(extent=[-60, -2; 60, -42], string="for DSA glass")),
          Icon(
            Text(extent=[-60, 78; 60, 38], string="Transmission of"),
            Text(extent=[-80, 38; 82, -2], string="diffuse solar radiation"),
            Text(extent=[-60, -2; 60, -42], string="for DSA glass")),
          Documentation(info="<html>
This auxiliary block computes the transmission coefficient for glass with respect to diffuse solar radiation.
</html>
"));
      equation 
        trans = -0.00885/2 + 2.71235/3 - 0.62062/4 - 7.07329/5 + 9.75995/6 -
          3.89922/7;
        y = trans;
      end DiffuseGlassTransmission;
      
      model C1D "Heat conduction through wall" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.ThermalResistance theta "Thermal resistance";
        parameter Modelica.SIunits.HeatCapacity gamma "Heat capacity";
        annotation (
          Coordsys(
            extent=[-100,-100; 100,100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-78, 52; 78, -54],
              string="C1D",
              style(color=0, pattern=0)), Text(extent=[-70, 90; 70, 50], string=
                 "%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The heat conduction model used here is the directed (why not) heat exchanger model of the bond graph library with an additional heat capacity built in at the output node.  In this way, walls can be subdivided into layers, possibly consisting of different material, with heat conduction models cascaded without a need for attaching heat capacitors externally, except at the very first node. <p>

Heat capacitors were attached only at one side, because otherwise, higher index models result when such models get cascaded.  Although Modelica is perfectly capable of reducing the perturbation index on its own, this would only have slowed down the compilation of the model unnecessarily.
</html>
"));
        BondLib.Bonds.fBond B1 annotation (extent=[-60, -10; -40, 10]);
        BondLib.Bonds.fBond B3 annotation (extent=[-20, 30; 0, 50]);
        BondLib.Bonds.eBond B2 annotation (extent=[0, -10; 20, 10]);
        BondLib.Bonds.eBond B4 annotation (extent=[20, 30; 40, 50]);
        BondLib.Bonds.eBond B5 annotation (extent=[40, -30; 60, -10], rotation=
              -90);
        BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-40, 10; -20, -10]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-80, -10; -60, 10]);
        BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[40, -10; 60, 10]);
        BondLib.Thermal.RSth RSth1(theta=theta) annotation (extent=[0, 30; 20,
              50]);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=288)) annotation (extent=
              [40, -50; 60, -30], rotation=-90);
      equation 
        connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -80,0],
              style(color=8));
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-60, 0; -60,
                0], style(color=8));
        connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-40, 0; -40,
                0], style(color=8));
        connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[-30, 10; -30,
                40; -20, 40], style(color=8));
        connect(B3.fBondCon1, RSth1.BondCon1) annotation (points=[0, 40; 0, 40],
              style(color=8));
        connect(RSth1.BondCon2, B4.fBondCon1) annotation (points=[20, 40; 20,
              40], style(color=8));
        connect(B4.eBondCon1, J0p4_1.BondCon4) annotation (points=[40, 40; 50,
              40; 50, 10], style(color=8));
        connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[-20, 0; 0, 0],
                 style(color=8));
        connect(B2.eBondCon1, J0p4_1.BondCon1) annotation (points=[20, 0; 40, 0],
                 style(color=8));
        connect(J0p4_1.BondCon3, B5.fBondCon1) annotation (points=[50, -10; 50,
                -10], style(color=8));
        connect(B5.eBondCon1, Cth1.BondCon1) annotation (points=[50, -30; 50, -
              30], style(color=8));
        connect(J0p4_1.BondCon2, BondCon2) annotation (points=[60,0; 100,0],
            style(color=8));
      end C1D;
      
      model C1V "Heat convection in boundary layer" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.ThermalResistance theta "Thermal Resistance";
        annotation (
          Coordsys(
            extent=[-100,-100; 100,100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-78, 52; 78, -54],
              string="C1V",
              style(color=0, pattern=0)), Text(extent=[-70, 90; 70, 50], string=
                 "%name")),
          Window(
            x=0.03,
            y=0.02,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This model is the regular directed heat exchanger model, <b>HEr</b>, of the bond graph library.  However, we have duplicated the model here under a different name, in order to be able to provide meaningful documentation for its use. <p>

In thermal building models, it is customary to assume that there is a special convection layer in the air neighboring each wall.  Our model follows this tradition, and offers convection in the boundary layer of air next to each wall.
</html>
"));
        BondLib.Bonds.fBond B1 annotation (extent=[-60, -10; -40, 10]);
        BondLib.Bonds.fBond B3 annotation (extent=[-20, 30; 0, 50]);
        BondLib.Bonds.eBond B2 annotation (extent=[0, -10; 20, 10]);
        BondLib.Bonds.eBond B4 annotation (extent=[20, 30; 40, 50]);
        BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-40, 10; -20, -10]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-80, -10; -60, 10]);
        BondLib.Thermal.RSth RSth1(theta=theta) annotation (extent=[0, 30; 20,
              50]);
        BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[40, 10; 60, -10]);
      equation 
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-60, 0; -60,
                0], style(color=8));
        connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-40, 0; -40,
                0], style(color=8));
        connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[-30, 10; -30,
                40; -20, 40], style(color=8));
        connect(B3.fBondCon1, RSth1.BondCon1) annotation (points=[0, 40; 0, 40],
              style(color=8));
        connect(RSth1.BondCon2, B4.fBondCon1) annotation (points=[20, 40; 20,
              40], style(color=8));
        connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[-20, 0; 0, 0],
                 style(color=8));
        connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -80,0],
              style(color=8));
        connect(B4.eBondCon1, J0p3_1.BondCon3) annotation (points=[40, 40; 50,
              40; 50, 10], style(color=8));
        connect(B2.eBondCon1, J0p3_1.BondCon1) annotation (points=[20, 0; 40, 0],
                 style(color=8));
        connect(J0p3_1.BondCon2, BondCon2) annotation (points=[60,0; 100,0],
            style(color=8));
      end C1V;
      
      model ExWall "Exterior wall" 
        extends BondLib.Interfaces.ActiveOnePort;
        parameter Modelica.SIunits.ThermalConductivity lv1=22.7 
          "Specific thermal conductivity of convection outside";
        parameter Modelica.SIunits.Length dxv1=1 
          "Width of convective layer outside";
        parameter Modelica.SIunits.Area area "Area of external wall";
        parameter Modelica.SIunits.ThermalResistance thv1=dxv1/(lv1*area) 
          "Thermal resistance of convection outside";
        parameter Modelica.SIunits.ThermalConductivity ld=1.7 
          "Specific thermal conductivity of dry wall";
        parameter Modelica.SIunits.Length dxd=0.14 
          "Thickness of conduction layer";
        parameter Modelica.SIunits.ThermalResistance theta=dxd/(ld*area) 
          "Thermal resistance of dry wall layer";
        parameter Modelica.SIunits.ThermalConductivity lv2=8.3 
          "Specific thermal conductivity of convection inside";
        parameter Modelica.SIunits.Length dxv2=1 
          "Width of convective layer inside";
        parameter Modelica.SIunits.ThermalResistance thv2=dxv2/(lv2*area) 
          "Thermal resistance of convection inside";
        parameter Modelica.SIunits.SpecificHeatCapacity c=840 
          "Specific heat capacity of dry wall";
        parameter Modelica.SIunits.Density dens=1762 "Density of external wall";
        parameter Modelica.SIunits.Length thk=0.41 "Thickness of external wall";
        parameter Modelica.SIunits.Mass m=area*thk*dens/4 
          "Mass of external wall segment";
        parameter Real gamma(unit="W.h/K") = c*m/3600 
          "Heat capacity of external wall segment";
        parameter Real sfa(unit="deg") "Orientation of wall";
        parameter Real tilt(unit="deg") = 90 "Tilt of wall";
        parameter Real OH;
        parameter Real W;
        parameter Real absor=0.5 "Absorption coefficient of wall";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Window(
            x=0.02,
            y=0.01,
            width=0.44,
            height=0.65),
          Icon(Text(
              extent=[-74, 68; 86, -66],
              string="ExWall",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram,
          Documentation(info="<html>
The exterior wall model consists of a boundary convection layer on the outside, three conduction layers inside the wall, and another boundary convection layer on the inside. <p>

The model computes its own solar position, which is computationally wasteful, but reduces the number of connections on the higher level models.  It then computes the radiation through the wall.  Since the wall also radiates to the outside, the radiation model, which computes the balance radiation, requires information of the temperature of the outside of the wall (the 0-junction between the outside convection layer, <b>C1V</b>, and the first conduction layer, <b>C1D</b>.  This temperature is sensed using an effort detector, <b>De</b>. <p>

The solar radiation model computes the entropy flow to the wall from both direct and diffuse radiation.  This enters the model through a modulated flow source, which is attached to the 0-junction representing the outside of the wall. <p>

The ambient air temperature is also computed inside the model, which is again computationally wasteful.  The ambient air temperature is connected by means of a modulated effort source to the outside of the air convection layer.  Since the ambient air temperature is simply an observed quantity, rather than a physically computed quantity, no heat capacity is added to that node.
</html>
"));
        Temperature Temperature1 annotation (extent=[-60,30; -40,50]);
        SolarPosition SolarPosition1(sfa=sfa, tilt=tilt) annotation (extent=[
              -60,60; -40,80]);
        SolarRadiation SolarRadiation1(
          tilt=tilt,
          absor=absor,
          area=area,
          OH=OH,
          W=W) annotation (extent=[-20, 60; 0, 80]);
        C1D C1D1(theta=theta, gamma=gamma) annotation (extent=[40, -10; 60, 10]);
        C1D C1D3(theta=theta, gamma=gamma) annotation (extent=[10, -90; -10, -
              70]);
        C1D C1D2(theta=theta, gamma=gamma) annotation (extent=[70,-70; 90,-50],
                 rotation=-90);
        C1V C1V1(theta=thv1) annotation (extent=[-40, -10; -20, 10]);
        C1V C1V2(theta=thv2) annotation (extent=[-70,-90; -90,-70]);
        BondLib.Sources.mSe mSe1 annotation (extent=[0, 30; -20, 50]);
        BondLib.Sources.mSf mSf1 annotation (extent=[30, 60; 10, 80]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-80, -10; -60, 10]);
        BondLib.Bonds.fBond B2 annotation (extent=[-60, -10; -40, 10]);
        BondLib.Bonds.fBond B1 annotation (extent=[0, 30; 20, 50]);
        BondLib.Bonds.eBond B3 annotation (extent=[-20, -10; 0, 10]);
        BondLib.Bonds.eBond B4 annotation (extent=[30, 60; 50, 80]);
        BondLib.Bonds.eBond B5 annotation (extent=[0, -30; 20, -10], rotation=-
              90);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=288)) annotation (extent=
              [0, -50; 20, -30], rotation=-90);
        BondLib.Bonds.fBond B6 annotation (extent=[20, -10; 40, 10]);
        BondLib.Bonds.fBond B7 annotation (extent=[60, -10; 80, 10]);
        BondLib.Junctions.J0p2 J0p2_2 annotation (extent=[70,-30; 90,-10],
            rotation=90);
        BondLib.Bonds.fBond B8 annotation (extent=[70,-50; 90,-30],   rotation=
              270);
        BondLib.Bonds.fBond B9 annotation (extent=[50, -90; 70, -70], rotation=
              180);
        BondLib.Junctions.J0p2 J0p2_3 annotation (extent=[30, -90; 50, -70]);
        BondLib.Bonds.fBond B10 annotation (extent=[10, -90; 30, -70], rotation=
             180);
        BondLib.Junctions.J0p2 J0p2_4 annotation (extent=[-50, -90; -30, -70]);
        BondLib.Bonds.fBond B12 annotation (extent=[-70, -90; -50, -70],
            rotation=180);
        BondLib.Bonds.eBond B13 annotation (extent=[-110, -70; -90, -50],
            rotation=90);
        BondLib.Junctions.J0p2 J0p2_5 annotation (extent=[-110, -50; -90, -30],
              rotation=90);
        BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[0, 10; 20, -10]);
        BondLib.Sensors.De De1 annotation (extent=[20, 44; 0, 64]);
        BondLib.Bonds.fBond B14 annotation (extent=[20, 44; 40, 64], rotation=
              180);
        BondLib.Bonds.fBond B11 annotation (extent=[-30, -90; -10, -70],
            rotation=180);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[100,-10; 120,10], rotation=180);
      equation 
        connect(J0p2_1.BondCon2, B2.eBondCon1) annotation (points=[-60, 0; -60,
                0], style(color=8));
        connect(mSe1.BondCon1, B1.eBondCon1) annotation (points=[0, 40; 0, 40],
              style(color=8));
        connect(B1.fBondCon1, J0p2_1.BondCon1) annotation (points=[20, 40; 26,
              40; 26, 22; -86, 22; -86, 0; -80, 0], style(color=8));
        connect(B2.fBondCon1, C1V1.BondCon1) annotation (points=[-40,0; -40,0],
                 style(color=8));
        connect(C1V1.BondCon2, B3.fBondCon1) annotation (points=[-20,0; -20,0],
                 style(color=8));
        connect(mSf1.BondCon1, B4.fBondCon1) annotation (points=[30, 70; 30, 70],
                 style(color=8));
        connect(B5.eBondCon1, Cth1.BondCon1) annotation (points=[10, -30; 10, -
              30], style(color=8));
        connect(B6.fBondCon1, C1D1.BondCon1) annotation (points=[40, 0; 40, 0],
              style(color=8));
        connect(C1D1.BondCon2, B7.eBondCon1) annotation (points=[60, 0; 60, 0],
              style(color=8));
        connect(B7.fBondCon1, J0p2_2.BondCon2) annotation (points=[80,0; 80,-10],
                   style(color=8));
        connect(B9.fBondCon1, J0p2_3.BondCon2) annotation (points=[50, -80; 50,
                -80], style(color=8));
        connect(J0p2_3.BondCon1, B10.eBondCon1) annotation (points=[30, -80; 30,
                -80], style(color=8));
        connect(B10.fBondCon1, C1D3.BondCon1) annotation (points=[10, -80; 10,
              -80], style(color=8));
        connect(J0p2_4.BondCon1, B12.eBondCon1) annotation (points=[-50, -80; -
              50, -80], style(color=8));
        connect(B12.fBondCon1, C1V2.BondCon1) annotation (points=[-70,-80; -70,
              -80],   style(color=8));
        connect(C1V2.BondCon2, B13.fBondCon1) annotation (points=[-90,-80; -100,
              -80; -100,-70],       style(color=8));
        connect(B13.eBondCon1, J0p2_5.BondCon1) annotation (points=[-100, -50;
              -100, -50], style(color=8));
        connect(J0p2_5.BondCon2, BondCon1) annotation (points=[-100,-30; -100,0],
                     style(color=8));
        connect(B3.eBondCon1, J0p5_1.BondCon1) annotation (points=[0, 0; 0, 0],
              style(color=8));
        connect(J0p5_1.BondCon2, B6.eBondCon1) annotation (points=[20, 0; 20, 0],
                 style(color=8));
        connect(J0p5_1.BondCon5, B5.fBondCon1) annotation (points=[10, -10; 10,
                -10], style(color=8));
        connect(B14.fBondCon1, De1.BondCon1) annotation (points=[20, 54; 20, 54],
                 style(color=8));
        connect(J0p5_1.BondCon3, B14.eBondCon1) annotation (points=[6, 10; 6,
              18; 46, 18; 46, 54; 40, 54], style(color=8));
        connect(B4.eBondCon1, J0p5_1.BondCon4) annotation (points=[50, 70; 60,
              70; 60, 14; 14, 14; 14, 10], style(color=8));
        connect(J0p2_2.BondCon1, B8.eBondCon1) annotation (points=[80,-30; 80,
              -30],   style(color=8));
        connect(B8.fBondCon1, C1D2.BondCon1) annotation (points=[80,-50; 80,-50],
                   style(color=8));
        connect(C1D2.BondCon2, B9.eBondCon1) annotation (points=[80,-70; 80,-80;
              70,-80],      style(color=8));
        connect(C1D3.BondCon2, B11.eBondCon1) annotation (points=[-10, -80; -10,
                -80], style(color=8));
        connect(B11.fBondCon1, J0p2_4.BondCon2) annotation (points=[-30, -80; -
              30, -80], style(color=8));
        connect(SolarRadiation1.u, SolarPosition1.u) annotation (points=[-22,70;
              -30,70; -30,90; -70,90; -70,70; -62,70], style(color=74, rgbcolor=
               {0,0,127}));
        connect(SolarPosition1.u, Temperature1.u) annotation (points=[-62,70;
              -70,70; -70,40; -62,40], style(color=74, rgbcolor={0,0,127}));
        connect(De1.OutPort1, SolarRadiation1.Temp) annotation (points=[-1,54;
              -10,54; -10,60], style(color=74, rgbcolor={0,0,127}));
        connect(SolarRadiation1.u, u) annotation (points=[-22,70; -30,70; -30,
              90; 90,90; 90,0; 110,0], style(color=74, rgbcolor={0,0,127}));
        connect(SolarPosition1.alt, SolarRadiation1.alt) annotation (points=[-39,
              64; -22,64], style(color=74, rgbcolor={0,0,127}));
        connect(SolarPosition1.inc, SolarRadiation1.inc) annotation (points=[
              -39,76; -22,76], style(color=74, rgbcolor={0,0,127}));
        connect(mSf1.s, SolarRadiation1.y) annotation (points=[11,70; 1,70],
            style(color=74, rgbcolor={0,0,127}));
        connect(mSe1.s, Temperature1.y) annotation (points=[-19,40; -39,40],
            style(color=74, rgbcolor={0,0,127}));
      end ExWall;
      
      model IntWall "Interior wall" 
        extends BondLib.Interfaces.TwoPort;
        
        parameter Modelica.SIunits.ThermalConductivity lv=8.3 
          "Specific thermal conductance of convection";
        parameter Modelica.SIunits.Length dxv=1 "Width of convective layer";
        parameter Modelica.SIunits.Area area "Area of internal wall";
        parameter Modelica.SIunits.ThermalResistance thv=dxv/(lv*area) 
          "Thermal resistance of convection";
        parameter Modelica.SIunits.ThermalConductivity ld=1.7 
          "Specific thermal conductance of dry wall";
        parameter Modelica.SIunits.Length dxd=0.1 
          "Thickness of conduction layer";
        parameter Modelica.SIunits.ThermalResistance theta=dxd/(ld*area) 
          "Thermal resistance of dry wall layer";
        parameter Modelica.SIunits.SpecificHeatCapacity c=840 
          "Specific heat capacity of dry wall";
        parameter Modelica.SIunits.Density dens=1762 "Density of internal wall";
        parameter Modelica.SIunits.Length thk=0.3 "Thickness of internal wall";
        parameter Modelica.SIunits.Mass m=area*thk*dens/4 
          "Mass of internal wall segment";
        parameter Real gamma(unit="W.h/K") = c*m/3600 
          "Heat capacity of internal wall segment";
        annotation (
          Coordsys(
            extent=[-100,-100; 100,100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-74, 68; 86, -66],
              string="IntWall",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram(
            Line(points=[0, 0; 0, 0], style(color=8)),
            Line(points=[10, -10; 10, -10], style(color=8)),
            Line(points=[20, 0; 20, 0], style(color=8))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
Interior walls have the same structure as exterior walls, i.e., a boundary convection layer on one side, followed by three conduction layers inside the wall, followed by another boundary convection layer on the other side. <p>

However, there is no solar radiation to be taken into account for interior walls.  Thus, the inerior wall model is considerably simpler than the exterior wall model.
</html>
"));
        C1D C1D1(theta=theta, gamma=gamma) annotation (extent=[40, -10; 60, 10]);
        C1D C1D3(theta=theta, gamma=gamma) annotation (extent=[-10, -70; -30, -
              50]);
        C1D C1D2(theta=theta, gamma=gamma) annotation (extent=[70, -70; 50, -50]);
        C1V C1V1(theta=thv) annotation (extent=[-40, -10; -20, 10]);
        C1V C1V2(theta=thv) annotation (extent=[-50, -90; -30, -70]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-80, -10; -60, 10]);
        BondLib.Bonds.fBond B1 annotation (extent=[-60, -10; -40, 10]);
        BondLib.Bonds.eBond B2 annotation (extent=[-20, -10; 0, 10]);
        BondLib.Bonds.eBond B3 annotation (extent=[0, -30; 20, -10], rotation=-
              90);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=288)) annotation (extent=
              [0, -50; 20, -30], rotation=-90);
        BondLib.Bonds.fBond B4 annotation (extent=[20, -10; 40, 10]);
        BondLib.Bonds.fBond B5 annotation (extent=[60, -10; 80, 10]);
        BondLib.Junctions.J0p2 J0p2_2 annotation (extent=[70, -30; 90, -10],
            rotation=90);
        BondLib.Bonds.fBond B6 annotation (extent=[70, -50; 90, -30], rotation=
              270);
        BondLib.Bonds.fBond B7 annotation (extent=[30, -70; 50, -50], rotation=
              180);
        BondLib.Junctions.J0p2 J0p2_3 annotation (extent=[10, -70; 30, -50]);
        BondLib.Bonds.fBond B8 annotation (extent=[-10, -70; 10, -50], rotation=
             180);
        BondLib.Junctions.J0p2 J0p2_4 annotation (extent=[-70, -70; -50, -50]);
        BondLib.Bonds.fBond B9 annotation (extent=[-50, -70; -30, -50],
            rotation=180);
        BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[0, -10; 20, 10]);
        BondLib.Bonds.eBond B10 annotation (extent=[-70, -90; -50, -70]);
        BondLib.Junctions.J0p2 J0p2_5 annotation (extent=[-10, -90; 10, -70]);
        BondLib.Bonds.eBond B11 annotation (extent=[-30, -90; -10, -70]);
      equation 
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-60, 0; -60,
                0], style(color=8));
        connect(B1.fBondCon1, C1V1.BondCon1) annotation (points=[-40, 0; -40, 0],
                 style(color=8));
        connect(C1V1.BondCon2, B2.fBondCon1) annotation (points=[-20, 0; -20, 0],
                 style(color=8));
        connect(B3.eBondCon1, Cth1.BondCon1) annotation (points=[10, -30; 10, -
              30], style(color=8));
        connect(B4.fBondCon1, C1D1.BondCon1) annotation (points=[40, 0; 40, 0],
              style(color=8));
        connect(C1D1.BondCon2, B5.eBondCon1) annotation (points=[60, 0; 60, 0],
              style(color=8));
        connect(B5.fBondCon1, J0p2_2.BondCon2) annotation (points=[80, 0; 80, -
              10], style(color=8));
        connect(B2.eBondCon1, J0p3_1.BondCon1) annotation (points=[0, 0; 0, 0],
              style(color=8));
        connect(J0p3_1.BondCon3, B3.fBondCon1) annotation (points=[10, -10; 10,
                -10], style(color=8));
        connect(J0p3_1.BondCon2, B4.eBondCon1) annotation (points=[20, 0; 20, 0],
                 style(color=8));
        connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -80,0],
              style(color=8));
        connect(J0p2_2.BondCon1, B6.eBondCon1) annotation (points=[80, -30; 80,
                -30], style(color=8));
        connect(B6.fBondCon1, C1D2.BondCon1) annotation (points=[80, -50; 80, -
              60; 70, -60], style(color=8));
        connect(C1D2.BondCon2, B7.eBondCon1) annotation (points=[50, -60; 50, -
              60], style(color=8));
        connect(B7.fBondCon1, J0p2_3.BondCon2) annotation (points=[30, -60; 30,
                -60], style(color=8));
        connect(J0p2_3.BondCon1, B8.eBondCon1) annotation (points=[10, -60; 10,
                -60], style(color=8));
        connect(B8.fBondCon1, C1D3.BondCon1) annotation (points=[-10, -60; -10,
                -60], style(color=8));
        connect(C1D3.BondCon2, B9.eBondCon1) annotation (points=[-30, -60; -30,
                -60], style(color=8));
        connect(B9.fBondCon1, J0p2_4.BondCon2) annotation (points=[-50, -60; -
              50, -60], style(color=8));
        connect(J0p2_4.BondCon1, B10.fBondCon1) annotation (points=[-70, -60; -
              80, -60; -80, -80; -70, -80], style(color=8));
        connect(B10.eBondCon1, C1V2.BondCon1) annotation (points=[-50, -80; -50,
                -80], style(color=8));
        connect(C1V2.BondCon2, B11.fBondCon1) annotation (points=[-30, -80; -30,
                -80], style(color=8));
        connect(B11.eBondCon1, J0p2_5.BondCon1) annotation (points=[-10, -80; -
              10, -80], style(color=8));
        connect(J0p2_5.BondCon2, BondCon2) annotation (points=[10,-80; 100,-80;
              100,0],       style(color=8));
      end IntWall;
      
      model Roof "Roof" 
        extends BondLib.Interfaces.ActiveOnePort;
        parameter Modelica.SIunits.ThermalConductivity lv1=22.7 
          "Specific thermal conductivity of convection outside";
        parameter Modelica.SIunits.Length dxv1=1 
          "Width of convective layer outside";
        parameter Modelica.SIunits.Area area "Area of roof";
        parameter Modelica.SIunits.ThermalResistance thv1=dxv1/(lv1*area) 
          "Thermal resistance of convection outside";
        parameter Modelica.SIunits.ThermalConductivity ld1=45.3 
          "Specific thermal conductivity of outmost roof layer";
        parameter Modelica.SIunits.Length dxd1=0.038 
          "Thickness of outmost roof layer";
        parameter Modelica.SIunits.ThermalResistance theta1=dxd1/(ld1*area) 
          "Thermal resistance of outmost layer";
        parameter Modelica.SIunits.ThermalConductivity ld2=0.0385 
          "Specific thermal conductivity of center roof layer";
        parameter Modelica.SIunits.Length dxd2=0.2 
          "Thickness of center roof layer";
        parameter Modelica.SIunits.ThermalResistance theta2=dxd2/(ld2*area) 
          "Thermal resistance of center layer";
        parameter Modelica.SIunits.ThermalConductivity ld3=0.1143 
          "Specific thermal conductivity of inner roof layer";
        parameter Modelica.SIunits.Length dxd3=0.016 
          "Thickness of inner roof layer";
        parameter Modelica.SIunits.ThermalResistance theta3=dxd3/(ld3*area) 
          "Thermal resistance of inner layer";
        parameter Modelica.SIunits.ThermalConductivity lv2=7.4 
          "Specific thermal conductivity of convection inside";
        parameter Modelica.SIunits.Length dxv2=1 
          "Width of convective layer inside";
        parameter Modelica.SIunits.ThermalResistance thv2=dxv2/(lv2*area) 
          "Thermal resistance of convection inside";
        parameter Modelica.SIunits.SpecificHeatCapacity c1=500 
          "Specific heat capacity of outmost roof layer";
        parameter Modelica.SIunits.Density dens1=7850 
          "Density of outmost roof layer";
        parameter Modelica.SIunits.Mass m1=area*ld1*dens1 
          "Mass of outmost roof layer";
        parameter Real gamma1(unit="W.h/K") = c1*m1/3600 
          "Heat capacity of outmost roof layer";
        parameter Modelica.SIunits.SpecificHeatCapacity c2=657 
          "Specific heat capacity of center roof layer";
        parameter Modelica.SIunits.Density dens2=19.2 
          "Density of center roof layer";
        parameter Modelica.SIunits.Mass m2=0.5*area*ld2*dens2 
          "Mass of center roof layer";
        parameter Real gamma2(unit="W.h/K") = c2*m2/3600 
          "Heat capacity of center roof layer";
        parameter Modelica.SIunits.SpecificHeatCapacity c3=1214 
          "Specific heat capacity of inner roof layer";
        parameter Modelica.SIunits.Density dens3=544 
          "Density of inner roof layer";
        parameter Modelica.SIunits.Mass m3=area*ld3*dens3 
          "Mass of inner roof layer";
        parameter Real gamma3(unit="W.h/K") = c3*m3/3600 
          "Heat capacity of inner roof layer";
        parameter Real sfa(unit="deg") = 180 "Orientation of wall";
        parameter Real tilt(unit="deg") = 7 "Tilt of wall";
        parameter Real OH;
        parameter Real W;
        parameter Real absor=0.2 "Absorption coefficient of wall";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-52,54; 46,-58],
              string="Roof",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram,
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
It would not have been necessary to create a separate roof model, as this model is exactly the same physical model as the exterior wall model (only with different physical parameter values), yet cut and paste is cheap, and since the readability of the higher level models seemed improved by offering a separate roof model, such a model was created.
</html>
"));
        Temperature Temperature1 annotation (extent=[-60,30; -40,50]);
        SolarPosition SolarPosition1(sfa=sfa, tilt=tilt) annotation (extent=[
              -60,60; -40,80]);
        SolarRadiation SolarRadiation1(
          tilt=tilt,
          absor=absor,
          area=area,
          OH=OH,
          W=W) annotation (extent=[-20, 60; 0, 80]);
        C1D C1D1(theta=theta1, gamma=gamma2) annotation (extent=[40, -10; 60,
              10]);
        C1V C1V1(theta=thv1) annotation (extent=[-40, -10; -20, 10]);
        BondLib.Sources.mSe mSe1 annotation (extent=[0, 30; -20, 50]);
        BondLib.Sources.mSf mSf1 annotation (extent=[30, 60; 10, 80]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-80, -10; -60, 10]);
        BondLib.Bonds.fBond B2 annotation (extent=[-60, -10; -40, 10]);
        BondLib.Bonds.fBond B1 annotation (extent=[0, 30; 20, 50]);
        BondLib.Bonds.eBond B3 annotation (extent=[-20, -10; 0, 10]);
        BondLib.Bonds.eBond B4 annotation (extent=[30, 60; 50, 80]);
        BondLib.Bonds.eBond B5 annotation (extent=[0, -30; 20, -10], rotation=-
              90);
        BondLib.Thermal.Cth Cth1(gamma=gamma1, e(start=288)) annotation (extent=
             [0, -50; 20, -30], rotation=-90);
        BondLib.Bonds.fBond B6 annotation (extent=[20, -10; 40, 10]);
        BondLib.Bonds.fBond B7 annotation (extent=[60, -10; 80, 10]);
        BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[0, 10; 20, -10]);
        BondLib.Sensors.De De1 annotation (extent=[20, 44; 0, 64]);
        BondLib.Bonds.fBond B14 annotation (extent=[20, 44; 40, 64], rotation=
              180);
        C1D C1D3(theta=theta3, gamma=gamma3) annotation (extent=[10, -90; -10,
              -70]);
        C1D C1D2(theta=theta2, gamma=gamma2) annotation (extent=[70, -70; 90, -
              50], rotation=-90);
        C1V C1V2(theta=thv2) annotation (extent=[-70, -90; -90, -70]);
        BondLib.Bonds.fBond B9 annotation (extent=[50, -90; 70, -70], rotation=
              180);
        BondLib.Junctions.J0p2 J0p2_3 annotation (extent=[30, -90; 50, -70]);
        BondLib.Bonds.fBond B10 annotation (extent=[10, -90; 30, -70], rotation=
             180);
        BondLib.Junctions.J0p2 J0p2_4 annotation (extent=[-50, -90; -30, -70]);
        BondLib.Bonds.fBond B12 annotation (extent=[-70, -90; -50, -70],
            rotation=180);
        BondLib.Bonds.eBond B13 annotation (extent=[-110, -70; -90, -50],
            rotation=90);
        BondLib.Bonds.fBond B11 annotation (extent=[-30, -90; -10, -70],
            rotation=180);
        BondLib.Junctions.J0p2 J0p2_5 annotation (extent=[-110, -50; -90, -30],
              rotation=90);
        BondLib.Bonds.fBond B8 annotation (extent=[70, -50; 90, -30], rotation=
              270);
        BondLib.Junctions.J0p2 J0p2_2 annotation (extent=[70, -30; 90, -10],
            rotation=90);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[100,-10; 120,10], rotation=180);
      equation 
        connect(J0p2_1.BondCon2, B2.eBondCon1) annotation (points=[-60, 0; -60,
                0], style(color=8));
        connect(mSe1.BondCon1, B1.eBondCon1) annotation (points=[0, 40; 0, 40],
              style(color=8));
        connect(B1.fBondCon1, J0p2_1.BondCon1) annotation (points=[20, 40; 26,
              40; 26, 22; -86, 22; -86, 0; -80, 0], style(color=8));
        connect(B2.fBondCon1, C1V1.BondCon1) annotation (points=[-40, 0; -40, 0],
                 style(color=8));
        connect(C1V1.BondCon2, B3.fBondCon1) annotation (points=[-20, 0; -20, 0],
                 style(color=8));
        connect(mSf1.BondCon1, B4.fBondCon1) annotation (points=[30, 70; 30, 70],
                 style(color=8));
        connect(B5.eBondCon1, Cth1.BondCon1) annotation (points=[10, -30; 10, -
              30], style(color=8));
        connect(B6.fBondCon1, C1D1.BondCon1) annotation (points=[40, 0; 40, 0],
              style(color=8));
        connect(C1D1.BondCon2, B7.eBondCon1) annotation (points=[60, 0; 60, 0],
              style(color=8));
        connect(B3.eBondCon1, J0p5_1.BondCon1) annotation (points=[0, 0; 0, 0],
              style(color=8));
        connect(J0p5_1.BondCon5, B5.fBondCon1) annotation (points=[10, -10; 10,
                -10], style(color=8));
        connect(J0p5_1.BondCon2, B6.eBondCon1) annotation (points=[20, 0; 20, 0],
                 style(color=8));
        connect(B14.fBondCon1, De1.BondCon1) annotation (points=[20, 54; 20, 54],
                 style(color=8));
        connect(J0p5_1.BondCon3, B14.eBondCon1) annotation (points=[6, 10; 6,
              18; 46, 18; 46, 54; 40, 54], style(color=8));
        connect(B4.eBondCon1, J0p5_1.BondCon4) annotation (points=[50, 70; 60,
              70; 60, 14; 14, 14; 14, 10], style(color=8));
        connect(B9.fBondCon1, J0p2_3.BondCon2) annotation (points=[50, -80; 50,
                -80], style(color=8));
        connect(J0p2_3.BondCon1, B10.eBondCon1) annotation (points=[30, -80; 30,
                -80], style(color=8));
        connect(B10.fBondCon1, C1D3.BondCon1) annotation (points=[10, -80; 10,
              -80], style(color=8));
        connect(J0p2_4.BondCon1, B12.eBondCon1) annotation (points=[-50, -80; -
              50, -80], style(color=8));
        connect(B12.fBondCon1, C1V2.BondCon1) annotation (points=[-70, -80; -70,
                -80], style(color=8));
        connect(C1V2.BondCon2, B13.fBondCon1) annotation (points=[-90, -80; -
              100, -80; -100, -70], style(color=8));
        connect(C1D2.BondCon2, B9.eBondCon1) annotation (points=[80, -70; 80, -
              80; 70, -80], style(color=8));
        connect(C1D3.BondCon2, B11.eBondCon1) annotation (points=[-10, -80; -10,
                -80], style(color=8));
        connect(B11.fBondCon1, J0p2_4.BondCon2) annotation (points=[-30, -80; -
              30, -80], style(color=8));
        connect(B13.eBondCon1, J0p2_5.BondCon1) annotation (points=[-100, -50;
              -100, -50], style(color=8));
        connect(J0p2_5.BondCon2, BondCon1) annotation (points=[-100,-30; -100,0],
                     style(color=8));
        connect(B8.fBondCon1, C1D2.BondCon1) annotation (points=[80, -50; 80, -
              50], style(color=8));
        connect(J0p2_2.BondCon1, B8.eBondCon1) annotation (points=[80, -30; 80,
                -30], style(color=8));
        connect(B7.fBondCon1, J0p2_2.BondCon2) annotation (points=[80, 0; 80, -
              10], style(color=8));
        connect(SolarPosition1.inc, SolarRadiation1.inc) annotation (points=[
              -39,76; -22,76], style(color=74, rgbcolor={0,0,127}));
        connect(SolarPosition1.alt, SolarRadiation1.alt) annotation (points=[
              -39,64; -22,64], style(color=74, rgbcolor={0,0,127}));
        connect(SolarRadiation1.Temp, De1.OutPort1) annotation (points=[-10,60;
              -10,54; -1,54], style(color=74, rgbcolor={0,0,127}));
        connect(SolarRadiation1.u, SolarPosition1.u) annotation (points=[-22,70;
              -30,70; -30,90; -70,90; -70,70; -62,70], style(color=74, rgbcolor=
               {0,0,127}));
        connect(SolarPosition1.u, Temperature1.u) annotation (points=[-62,70;
              -70,70; -70,40; -62,40], style(color=74, rgbcolor={0,0,127}));
        connect(SolarRadiation1.u, u) annotation (points=[-22,70; -30,70; -30,
              90; 90,90; 90,0; 110,0], style(color=74, rgbcolor={0,0,127}));
        connect(mSf1.s, SolarRadiation1.y) annotation (points=[11,70; 1,70], style(
              color=74, rgbcolor={0,0,127}));
        connect(mSe1.s, Temperature1.y) annotation (points=[-19,40; -39,40],
            style(color=74, rgbcolor={0,0,127}));
      end Roof;
      
      model Window "Window" 
        extends BondLib.Interfaces.ActiveOnePort;
        parameter Modelica.SIunits.ThermalConductivity lv=3.4 
          "Specific thermal conductivity of convection outside";
        parameter Modelica.SIunits.Length dxv=1 
          "Width of convective layer outside";
        parameter Modelica.SIunits.Area area "Area of window";
        parameter Modelica.SIunits.ThermalResistance thv=dxv/(lv*area) 
          "Thermal resistance of convection outside";
        parameter Real sfa(unit="deg") "Orientation of window";
        parameter Real tilt(unit="deg") = 90 "Tilt of window";
        parameter Real OH;
        parameter Real W;
        parameter Real sc=0.85;
        annotation (
          Coordsys(
            extent=[-100,-100; 100,100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-74, 68; 86, -66],
              string="Window",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram,
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The window model has an outside convection layer, but no conduction layers, as the glass is considered thin and homogeneous.  It would have been possible (and probably meaningful!), to provide an inside convection layer as well, but the student didn't do so.  Thus, the glass doesn't have its own temperature.  The 0-junction on the inside of the convection layer simply represents the temperature of the room behind the window. <p>

The solar input is similar to the solar input used in the exterior wall model, except that the general <i>solar radiation</i> model was replaced here by the <i>solar radiation through glass</i> model.
</html>
"));
        Temperature Temperature1 annotation (extent=[-60,30; -40,50]);
        SolarPosition SolarPosition1(sfa=sfa, tilt=tilt) annotation (extent=[
              -60,60; -40,80]);
        C1V C1V1(theta=thv) annotation (extent=[-40,-10; -20,10]);
        BondLib.Sources.mSe mSe1 annotation (extent=[0, 30; -20, 50]);
        BondLib.Sources.mSf mSf1 annotation (extent=[30, 60; 10, 80]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-80, -10; -60, 10]);
        BondLib.Bonds.fBond B2 annotation (extent=[-60, -10; -40, 10]);
        BondLib.Bonds.fBond B1 annotation (extent=[0, 30; 20, 50]);
        BondLib.Bonds.eBond B3 annotation (extent=[-20, -10; 0, 10]);
        BondLib.Bonds.eBond B4 annotation (extent=[30, 60; 50, 80]);
        SolarGlass SolarGlass1(
          sc=sc,
          tilt=tilt,
          area=area,
          OH=OH,
          W=W) annotation (extent=[-20, 60; 0, 80]);
        BondLib.Sensors.De De1 annotation (extent=[20, 44; 0, 64]);
        BondLib.Bonds.fBond B5 annotation (extent=[20, 44; 40, 64], rotation=
              180);
        BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[0, -10; 20, 10]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[100,-10; 120,10], rotation=180);
      equation 
        connect(J0p2_1.BondCon2, B2.eBondCon1) annotation (points=[-60, 0; -60,
                0], style(color=8));
        connect(mSe1.BondCon1, B1.eBondCon1) annotation (points=[0, 40; 0, 40],
              style(color=8));
        connect(B1.fBondCon1, J0p2_1.BondCon1) annotation (points=[20,40; 40,40;
              40,20; -86,20; -86,0; -80,0],         style(color=8));
        connect(B2.fBondCon1, C1V1.BondCon1) annotation (points=[-40,0; -40,0],
                 style(color=8));
        connect(C1V1.BondCon2, B3.fBondCon1) annotation (points=[-20,0; -20,0],
                 style(color=8));
        connect(mSf1.BondCon1, B4.fBondCon1) annotation (points=[30, 70; 30, 70],
                 style(color=8));
        connect(B5.fBondCon1, De1.BondCon1) annotation (points=[20, 54; 20, 54],
              style(color=8));
        connect(B3.eBondCon1, J0p4_1.BondCon1) annotation (points=[0, 0; 0, 0],
              style(color=8));
        connect(J0p4_1.BondCon4, B5.eBondCon1) annotation (points=[10,10; 10,16;
              50,16; 50,54; 40,54],        style(color=8));
        connect(B4.eBondCon1, J0p4_1.BondCon2) annotation (points=[50, 70; 60,
              70; 60, 0; 20, 0], style(color=8));
        connect(J0p4_1.BondCon3, BondCon1) annotation (points=[10,-10; 10,-20;
              -100,-20; -100,0],      style(color=8));
        connect(SolarGlass1.inc, SolarPosition1.inc) annotation (points=[-22,76;
              -39,76], style(color=74, rgbcolor={0,0,127}));
        connect(SolarGlass1.alt, SolarPosition1.alt) annotation (points=[-22,64;
              -39,64], style(color=74, rgbcolor={0,0,127}));
        connect(SolarGlass1.Temp, De1.OutPort1) annotation (points=[-10,60; -10,
              54; -1,54], style(color=74, rgbcolor={0,0,127}));
        connect(SolarGlass1.u, SolarPosition1.u) annotation (points=[-22,70;
              -30,70; -30,90; -70,90; -70,70; -62,70], style(color=74, rgbcolor=
               {0,0,127}));
        connect(SolarPosition1.u, Temperature1.u) annotation (points=[-62,70;
              -70,70; -70,40; -62,40], style(color=74, rgbcolor={0,0,127}));
        connect(SolarGlass1.u, u) annotation (points=[-22,70; -30,70; -30,90;
              80,90; 80,0; 110,0], style(color=74, rgbcolor={0,0,127}));
        connect(mSf1.s, SolarGlass1.y) annotation (points=[11,70; 1,70], style(
              color=74, rgbcolor={0,0,127}));
        connect(mSe1.s, Temperature1.y) annotation (points=[-19,40; -39,40],
            style(color=74, rgbcolor={0,0,127}));
      end Window;
      
      model ExDoor "Exterior door" 
        extends Interfaces.ActiveOnePort;
        parameter Modelica.SIunits.ThermalConductivity lv=0.189 
          "Specific thermal conductivity of convection outside";
        parameter Modelica.SIunits.Length dxv=1 
          "Width of convective layer outside";
        parameter Modelica.SIunits.Area area "Area of door";
        parameter Modelica.SIunits.ThermalResistance thv=dxv/(lv*area) 
          "Thermal resistance of convection outside";
        parameter Real sfa(unit="deg") "Orientation of door";
        parameter Real tilt(unit="deg") = 90 "Tilt of door";
        parameter Real OH;
        parameter Real W;
        parameter Real absor=0.2 "Absorption coefficient of door";
        Temperature Temperature1 annotation (extent=[-60,30; -40,50]);
        SolarPosition SolarPosition1(sfa=sfa, tilt=tilt) annotation (extent=[
              -60,60; -40,80]);
        C1V C1V1(theta=thv) annotation (extent=[-40,-10; -20,10]);
        Sources.mSe mSe1         annotation (extent=[0,30; -20,50]);
        Sources.mSf mSf1         annotation (extent=[30,60; 10,80]);
        Junctions.J0p2 J0p2_1         annotation (extent=[-80,-10; -60,10]);
        Bonds.fBond B2         annotation (extent=[-60,-10; -40,10]);
        Bonds.fBond B1         annotation (extent=[0,30; 20,50]);
        Bonds.eBond B3         annotation (extent=[-20,-10; 0,10]);
        Bonds.eBond B4         annotation (extent=[30,60; 50,80]);
        Sensors.De De1         annotation (extent=[20,44; 0,64]);
        Bonds.fBond B5         annotation (extent=[20,44; 40,64],   rotation=
              180);
        Junctions.J0p4 J0p4_1         annotation (extent=[0,-10; 20,10]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[100,-10; 120,10], rotation=180);
        SolarRadiation SolarRadiation1(
          tilt=tilt,
          absor=absor,
          area=area,
          OH=OH,
          W=W) annotation (extent=[-20,60; 0,80]);
      equation 
        connect(J0p2_1.BondCon2,B2. eBondCon1) annotation (points=[-60,0; -60,0],
                    style(color=8));
        connect(mSe1.BondCon1,B1. eBondCon1) annotation (points=[0,40; 0,40],
              style(color=8));
        connect(B1.fBondCon1,J0p2_1. BondCon1) annotation (points=[20,40; 40,40;
              40,20; -86,20; -86,0; -80,0],         style(color=8));
        connect(B2.fBondCon1,C1V1. BondCon1) annotation (points=[-40,0; -40,0],
                 style(color=8));
        connect(C1V1.BondCon2,B3. fBondCon1) annotation (points=[-20,0; -20,0],
                 style(color=8));
        connect(mSf1.BondCon1,B4. fBondCon1) annotation (points=[30,70; 30,70],
                 style(color=8));
        connect(B5.fBondCon1,De1. BondCon1) annotation (points=[20,54; 20,54],
              style(color=8));
        connect(B3.eBondCon1,J0p4_1. BondCon1) annotation (points=[0,0; 0,0],
              style(color=8));
        connect(J0p4_1.BondCon4,B5. eBondCon1) annotation (points=[10,10; 10,16;
              50,16; 50,54; 40,54],        style(color=8));
        connect(B4.eBondCon1,J0p4_1. BondCon2) annotation (points=[50,70; 60,70;
              60,0; 20,0],       style(color=8));
        annotation (
          Diagram,
          Icon(Text(
              extent=[-76,62; 80,-66],
              style(color=0),
              string="ExDoor"),Text(extent=[-74,88; 66,48],   string="%name")),
          Documentation(info="<html>
The exterior doors are similar to windows, in that they are thin and homogeneous.  Thus the model contains an outside convection layer, no conduction layers, and (unfortunately) no inside convection layer either.  This is the same error that the student made in modeling the windows.
</html>
"));
        connect(J0p4_1.BondCon3, BondCon1) annotation (points=[10,-10; 10,-20; -100,
              -20; -100,0; -100,0], style(color=8, rgbcolor={192,192,192}));
        connect(SolarPosition1.u, Temperature1.u) annotation (points=[-62,70; -70,
              70; -70,40; -62,40], style(color=74, rgbcolor={0,0,127}));
        connect(SolarRadiation1.u, u) annotation (points=[-22,70; -30,70; -30,
              90; 80,90; 80,0; 110,0], style(color=74, rgbcolor={0,0,127}));
        connect(SolarRadiation1.u, SolarPosition1.u) annotation (points=[-22,70;
              -30,70; -30,90; -70,90; -70,70; -62,70], style(color=74, rgbcolor=
               {0,0,127}));
        connect(SolarPosition1.inc, SolarRadiation1.inc) annotation (points=[
              -39,76; -22,76], style(color=74, rgbcolor={0,0,127}));
        connect(SolarPosition1.alt, SolarRadiation1.alt) annotation (points=[
              -39,64; -22,64], style(color=74, rgbcolor={0,0,127}));
        connect(SolarRadiation1.Temp, De1.OutPort1) annotation (points=[-10,60;
              -10,54; -1,54], style(color=74, rgbcolor={0,0,127}));
        connect(mSf1.s, SolarRadiation1.y) annotation (points=[11,70; 1,70],
            style(color=74, rgbcolor={0,0,127}));
        connect(mSe1.s, Temperature1.y) annotation (points=[-19,40; -39,40],
            style(color=74, rgbcolor={0,0,127}));
      end ExDoor;
      
      model IntDoor "Interior door" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.ThermalConductivity lv=0.189 
          "Specific thermal conductivity of convection outside";
        parameter Modelica.SIunits.Length dxv=1 
          "Width of convective layer outside";
        parameter Modelica.SIunits.Area area "Area of door";
        parameter Modelica.SIunits.ThermalResistance thv=dxv/(lv*area) 
          "Thermal resistance of convection outside";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Line(points=[10, -10; 10, -10], style(color=8)),
            Line(points=[60, 0; 60, 0], style(color=8)),
            Line(points=[40, 0; 40, 0], style(color=8)),
            Line(points=[20, 0; 20, 0], style(color=8)),
            Line(points=[0, 0; 0, 0], style(color=8))),
          Icon(Text(
              extent=[-84,66; 86,-70],
              style(color=0),
              string="IntDoor"),  Text(extent=[-70, 90; 70, 50], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The internal doors are similar to windows, in that they are thin and homogeneous.  Thus the model contains an outside convection layer, no conduction layers, and (unfortunately) no inside convection layer either.  This is the same error that the student made in modeling the windows.
</html>

"));
        C1V C1V1(theta=thv) annotation (extent=[-10,-10; 10,10]);
        BondLib.Bonds.fBond B1 annotation (extent=[-30,-10; -10,10]);
        BondLib.Bonds.eBond B2 annotation (extent=[10,-10; 30,10]);
        Junctions.J0p2 J0p2_1 annotation (extent=[-50,-10; -30,10]);
        Junctions.J0p2 J0p2_2 annotation (extent=[30,-10; 50,10]);
      equation 
        connect(B1.fBondCon1, C1V1.BondCon1) annotation (points=[-10,0; -10,0],
            style(color=8));
        connect(C1V1.BondCon2, B2.fBondCon1) annotation (points=[10,0; 10,0],
              style(color=8));
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-30,0; -30,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p2_1.BondCon1, BondCon1) annotation (points=[-50,0; -100,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.eBondCon1, J0p2_2.BondCon1) annotation (points=[30,0; 30,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p2_2.BondCon2, BondCon2) annotation (points=[50,0; 100,0],
            style(color=8, rgbcolor={192,192,192}));
      end IntDoor;
      
      model Slab "Slab below the experimental house" 
        extends BondLib.Interfaces.ActiveOnePort;
        parameter Modelica.SIunits.ThermalConductivity ld=5.11 
          "Specific thermal conductivity of slab";
        parameter Modelica.SIunits.Length dxd=1 
          "Thickness of slab for conduction";
        parameter Modelica.SIunits.Length thk=0.2 "Thickness of slab";
        parameter Modelica.SIunits.Length perim "Area of floor for conduction";
        parameter Modelica.SIunits.Area aread=perim*thk 
          "Effective area of floor for conduction";
        parameter Modelica.SIunits.ThermalResistance theta=dxd/(ld*aread) 
          "Thermal resistance of slab";
        parameter Modelica.SIunits.ThermalConductivity lv=7.4 
          "Specific thermal conductivity of convection";
        parameter Modelica.SIunits.Length dxv=1 "Width of convective layer";
        parameter Modelica.SIunits.Area area "Area of floor for convection";
        parameter Modelica.SIunits.ThermalResistance thv=dxv/(lv*area) 
          "Thermal resistance of convection";
        parameter Modelica.SIunits.SpecificHeatCapacity c=835 
          "Specific heat capacity of slab";
        parameter Modelica.SIunits.Density dens=2083 "Density of slab";
        parameter Modelica.SIunits.Mass m=area*thk*dens "Mass of slab";
        parameter Real gamma(unit="W.h/K") = c*m/3600 "Heat capacity of slab";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Line(points=[0, 0; 0, 0], style(color=8)),
            Line(points=[10, -10; 10, -10], style(color=8)),
            Line(points=[20, 0; 20, 0], style(color=8)),
            Line(points=[10, -10; 10, -10], style(color=8)),
            Line(points=[0, 0; 0, 0], style(color=8)),
            Line(points=[20, 0; 20, 0], style(color=8))),
          Icon(Text(
              extent=[-52,58; 52,-58],
              string="Slab",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Documentation(info="<html>
In Tucson, houses are built on sand.  The house is not thermally insulated from the ground, thus, the thermal building model ought to take into account the exchange of heat between the house and the slab underneath it. <p>

The slab is modeled with a single conduction layer connecting the temperature of the slab to the temperature of the floor.  Above the floor, there is a boundary convection layer.
</html>
"));
        C1D C1D1(theta=theta, gamma=gamma) annotation (extent=[-20,10; 0,30]);
        C1V C1V1(theta=thv) annotation (extent=[0,-30; -20,-10]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-70,30; -50,50],
            rotation=90);
        BondLib.Bonds.fBond B1 annotation (extent=[-40,10; -20,30]);
        BondLib.Bonds.eBond B4 annotation (extent=[-40,-30; -20,-10], rotation=
              180);
        BondLib.Bonds.fBond B3 annotation (extent=[0,-30; 20,-10], rotation=180);
        BondLib.Bonds.fBond B2 annotation (extent=[0,10; 20,30]);
        BondLib.Junctions.J0p2 J0p2_2 annotation (extent=[30,-10; 50,10],
            rotation=90);
        BondLib.Junctions.J0p2 J0p2_3 annotation (extent=[-60,-30; -40,-10]);
        Temperature2 Temperature2_1 annotation (extent=[40,50; 20,70]);
        Sources.mSe mSe1         annotation (extent=[-20,50; 0,70]);
        Bonds.fBond B5          annotation (extent=[-40,50; -20,70], rotation=
              180);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[100,-10; 120,10], rotation=180);
      equation 
        connect(J0p2_3.BondCon2, B4.eBondCon1) annotation (points=[-40,-20; -40,
              -20], style(color=8, rgbcolor={192,192,192}));
        connect(B4.fBondCon1, C1V1.BondCon2) annotation (points=[-20,-20; -20,
              -20], style(color=8, rgbcolor={192,192,192}));
        connect(J0p2_3.BondCon1, BondCon1) annotation (points=[-60,-20; -80,-20;
              -80,0; -100,0], style(color=8, rgbcolor={192,192,192}));
        connect(C1V1.BondCon1, B3.fBondCon1) annotation (points=[0,-20; 0,-20],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p2_2.BondCon1, B3.eBondCon1) annotation (points=[40,-10; 40,
              -20; 20,-20], style(color=8, rgbcolor={192,192,192}));
        connect(B2.fBondCon1, J0p2_2.BondCon2) annotation (points=[20,20; 40,20;
              40,10], style(color=8, rgbcolor={192,192,192}));
        connect(C1D1.BondCon2, B2.eBondCon1) annotation (points=[0,20; 0,20],
            style(color=8, rgbcolor={192,192,192}));
        connect(B1.fBondCon1, C1D1.BondCon1) annotation (points=[-20,20; -20,20],
            style(color=8, rgbcolor={192,192,192}));
        connect(J0p2_1.BondCon1, B1.eBondCon1) annotation (points=[-60,30; -60,
              20; -40,20], style(color=8, rgbcolor={192,192,192}));
        connect(B5.fBondCon1, J0p2_1.BondCon2) annotation (points=[-40,60; -60,
              60; -60,50], style(color=8, rgbcolor={192,192,192}));
        connect(B5.eBondCon1, mSe1.BondCon1) annotation (points=[-20,60; -20,60],
            style(color=8, rgbcolor={192,192,192}));
        connect(Temperature2_1.u, u) annotation (points=[42,60; 80,60; 80,0;
              110,0], style(color=74, rgbcolor={0,0,127}));
        connect(Temperature2_1.y, mSe1.s) annotation (points=[19,60; -1,60],
            style(color=74, rgbcolor={0,0,127}));
      end Slab;
      
      model LivingRoom "Living room" 
        parameter Modelica.SIunits.SpecificHeatCapacity c=1000 
          "Specific heat capacity of living room";
        parameter Modelica.SIunits.Mass m=265 "Mass of living room";
        parameter Real gamma(unit="W.h/K") = c*m/3600 
          "Heat capacity of living room";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[1, 1],
            component=[10, 10]),
          Diagram(
            Line(points=[-90, 90; -90, -40; 90, -40; 90, 20; -30, 20; -30, 90;
                  -90, 90], style(color=41, thickness=2)),
            Line(points=[-29, 90; 90, 90; 90, 20], style(color=57, thickness=2)),
            Line(points=[30, 90; 30, 20], style(color=53, thickness=2)),
            Line(points=[10, -40; 10, -80; 90, -80; 90, -41], style(color=53,
                  thickness=2)),
            Rectangle(extent=[60, -38; 80, -42], style(color=41)),
            Rectangle(extent=[-30, -38; -10, -42], style(color=41)),
            Rectangle(extent=[-70, -38; -50, -42], style(color=41)),
            Rectangle(extent=[-84, 92; -64, 88], style(color=41)),
            Polygon(points=[-56, 90; -41, 75; -41, 90; -56, 90], style(color=41)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=41)),
            Polygon(points=[40, 20; 40, 40; 56, 20; 40, 20], style(color=41)),
            Polygon(points=[19, -40; 19, -20; 34, -40; 19, -40], style(color=41)),
            Rectangle(extent=[-10, 92; 10, 88], style(color=53)),
            Rectangle(extent=[50, 92; 70, 88], style(color=53)),
            Rectangle(extent=[10, -78; 30, -82], style(color=53)),
            Rectangle(extent=[30, -78; 50, -82], style(color=53)),
            Rectangle(extent=[50, -78; 70, -82], style(color=53)),
            Rectangle(extent=[70, -78; 90, -82], style(color=53)),
            Polygon(points=[10, -50; -10, -50; 10, -65; 10, -50], style(color=
                    53)),
            Text(
              extent=[-19, 69; 17, 54],
              string="Bedroom 2",
              style(color=53)),
            Text(
              extent=[41, 70; 76, 54],
              string="Bedroom 1",
              style(color=53)),
            Text(
              extent=[33, -58; 64, -70],
              string="Sunspace",
              style(color=53)),
            Line(points=[50, -44; 50, -44], style(color=8)),
            Line(points=[41, -64; 41, -64], style(color=8)),
            Line(points=[70, -61; 70, -60], style(color=8))),
          Icon(
            Line(points=[-90, 90; -90, -40; 90, -40; 90, 20; -30, 20; -30, 90;
                  -90, 90], style(color=9, thickness=2)),
            Line(points=[-29, 90; 90, 90; 90, 20], style(color=57, thickness=2)),
            Line(points=[30, 90; 30, 20], style(color=53, thickness=2)),
            Line(points=[10, -40; 10, -80; 90, -80; 90, -41], style(color=53,
                  thickness=2)),
            Rectangle(extent=[60, -38; 80, -42], style(color=9)),
            Rectangle(extent=[-30, -38; -10, -42], style(color=9)),
            Rectangle(extent=[-70, -38; -50, -42], style(color=9)),
            Rectangle(extent=[-84, 92; -64, 88], style(color=9)),
            Polygon(points=[-55, 90; -40, 75; -40, 90; -55, 90], style(color=9)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=9)),
            Polygon(points=[40, 20; 40, 40; 56, 20; 40, 20], style(color=9)),
            Polygon(points=[20, -40; 20, -20; 35, -40; 20, -40], style(color=9)),
            Rectangle(extent=[-10, 92; 10, 88], style(color=53)),
            Rectangle(extent=[50, 92; 70, 88], style(color=53)),
            Rectangle(extent=[10, -78; 30, -82], style(color=53)),
            Rectangle(extent=[30, -78; 50, -82], style(color=53)),
            Rectangle(extent=[50, -78; 70, -82], style(color=53)),
            Rectangle(extent=[70, -78; 90, -82], style(color=53)),
            Polygon(points=[10, -50; -10, -50; 10, -65; 10, -50], style(color=
                    53)),
            Text(
              extent=[-19, 69; 17, 54],
              string="Bedroom 2",
              style(color=53)),
            Text(
              extent=[41, 70; 76, 54],
              string="Bedroom 1",
              style(color=53)),
            Text(
              extent=[35, -54; 66, -66],
              string="Sunspace",
              style(color=53)),
            Text(
              extent=[-66, 3; 59, -17],
              string="Living Room",
              style(color=9))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
We are now ready to assemble the house from its components.  At this point, the modeling task has become quite simple. <p>

The living room is assumed to be hmogeneous with respect to its temperature.  Thus, there is a single 0-junction with a heat capacity attached representing the air of the living room.  SInce the living room interacts with lots of modeling elements, there are many connections to the single 0-junction.  Thus, the 0-junction was split into 5 individual 0-junctions connected by bonds. <p>

The living room has four exterior walls facing North, West, South, and East.  It has three interior walls connecting it with the other three rooms. <p>

The living room has three exterior windows facing North and South.  It has three interior doors connecting it with the other three rooms.  There is also an interior window to the solarium.  Since interior windows use the same model as interior doors, the student chose to represent that window using the interior door model. There is also an exterior door. <p>

Other than that, there is the roof, the slab, and the air conditioning unit, and that's it. <p>

Of course, the devil is in the detail.  Each of these models contains a substantial number of parameters that need to be set, and getting the correct parameter values together for such a complex model is no easy task.
</html>
"));
        Window NWin(
          area=1.5,
          OH=0,
          W=0,
          sfa=180) annotation (extent=[-85, 68; -66, 88], rotation=90);
        ExWall NWall(
          area=6.6,
          sfa=180,
          OH=0.3,
          W=12) annotation (extent=[-70, 68; -50, 88], rotation=90);
        ExWall WWall(
          area=20.7,
          sfa=90,
          OH=0.3,
          W=8) annotation (extent=[-80, 20; -100, 40]);
        BondLib.Bonds.eBond NB1 annotation (extent=[-85, 48; -65, 68], rotation=
             -90);
        BondLib.Bonds.eBond NB2 annotation (extent=[-70, 48; -50, 68], rotation=
             -90);
        BondLib.Bonds.eBond NB3 annotation (extent=[-55, 48; -35, 68], rotation=
             -90);
        BondLib.Bonds.eBond WB1 annotation (extent=[-79,22.5; -64,37.5]);
        BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[-63, 40; -43, 20]);
        Window SWin1(
          area=2.23,
          OH=0,
          W=0,
          sfa=0) annotation (extent=[-70, -62; -50, -42], rotation=-90);
        Window SWin2(
          area=2.23,
          OH=0,
          W=0,
          sfa=0) annotation (extent=[-30,-62; -10,-42],  rotation=-90);
        ExWall SWall(
          area=15.7,
          sfa=0,
          OH=0.3,
          W=12) annotation (extent=[-50, -62; -30, -42], rotation=-90);
        BondLib.Bonds.eBond SB1 annotation (extent=[-70, -42; -50, -22],
            rotation=90);
        BondLib.Bonds.eBond SB2 annotation (extent=[-50, -42; -30, -22],
            rotation=90);
        BondLib.Bonds.eBond SB3 annotation (extent=[-30, -42; -10, -22],
            rotation=90);
        BondLib.Bonds.eBond CB1 annotation (extent=[-60.5,5; -45.5,20],
                                                                      rotation=
              -90);
        BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[-13, -10; 7, 10]);
        BondLib.Bonds.eBond CB2 annotation (extent=[-29,-7.5; -14,7.5]);
        BondLib.Bonds.eBond CB3 annotation (extent=[7, -7; 22, 8], rotation=180);
        BondLib.Junctions.J0p5 J0p5_3 annotation (extent=[22, 10; 42, -10]);
        BondLib.Bonds.eBond CB4 annotation (extent=[42, -7; 57, 8], rotation=
              180);
        BondLib.Junctions.J0p6 J0p6_2 annotation (extent=[57, -10; 77, 10]);
        IntDoor SsWin(
          lv=3.4,
          area=1.1) annotation (extent=[60, -60; 80, -40], rotation=-90);
        IntDoor SsDoor(
          lv=3.57,
          area=2.17) annotation (extent=[15, -60; 35, -40], rotation=-90);
        IntWall SsWall(
          area=12.9,
          dxd=0.14,
          thk=0.41) annotation (extent=[42, -35; 57, -19], rotation=-90);
        BondLib.Bonds.fBond SB4 annotation (extent=[52, -15; 62, -5], rotation=
              180);
        BondLib.Bonds.eBond SB5 annotation (extent=[40, -55; 60, -35], rotation=
             -90);
        BondLib.Bonds.fBond NB4 annotation (extent=[2, 20; 17, 34], rotation=90);
        BondLib.Bonds.fBond NB5 annotation (extent=[17, 20; 32, 34], rotation=
              90);
        BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[44.5,-65; 55.5,-55]);
        BondLib.Bonds.fBond SB6 annotation (extent=[17, -40; 32, -25], rotation=
             -90);
        BondLib.Bonds.fBond SB7 annotation (extent=[63, -40; 78, -25], rotation=
             -90);
        BondLib.Bonds.eBond SB8 annotation (extent=[31,-66.5; 44,-53.5]);
        BondLib.Bonds.eBond SB9 annotation (extent=[55,-66.5; 68,-53.5],
                                                                       rotation=
             180);
        BondLib.Interfaces.BondCon BondCon1 "Bond graph connector to sun space"
                                            annotation (extent=[45, -77; 55, -
              67]);
        IntWall WallL2(area=15.2) annotation (extent=[15, 34; 35, 54], rotation=
             90);
        IntDoor DoorL2(
          lv=3.57,
          area=2.17) annotation (extent=[-1, 34; 19, 54], rotation=90);
        BondLib.Bonds.eBond NB7 annotation (extent=[2.5,55; 15.5,68],
                                                                    rotation=90);
        BondLib.Bonds.eBond NB8 annotation (extent=[19, 55; 32, 68], rotation=
              90);
        BondLib.Junctions.J0p3 J0p3_1 annotation (extent=[9,74.5; 25,61.5]);
        BondLib.Interfaces.BondCon BondCon2 "Bond graph connector to bedroom 2"
                                            annotation (extent=[12, 78; 22, 88]);
        BondLib.Bonds.fBond NB9 annotation (extent=[42, 20; 57, 34], rotation=
              90);
        BondLib.Bonds.fBond NB10 annotation (extent=[63, 20; 78, 34], rotation=
              90);
        IntDoor DoorL1(
          lv=3.57,
          area=2.17) annotation (extent=[40, 34; 60, 54], rotation=90);
        IntWall WallL1(area=6.6) annotation (extent=[60, 34; 80, 54], rotation=
              90);
        BondLib.Bonds.eBond NB11 annotation (extent=[43.5,55.5; 56.5,68.5],
                                                                      rotation=
              90);
        BondLib.Bonds.eBond NB12 annotation (extent=[63.5,55.5; 76.5,68.5],
                                                                      rotation=
              90);
        BondLib.Junctions.J0p3 J0p3_2 annotation (extent=[52, 75; 68, 62]);
        BondLib.Interfaces.BondCon BondCon3 "Bond graph connector to bedroom 1"
                                            annotation (extent=[55, 78; 65, 88]);
        BondLib.Bonds.eBond EB1 annotation (extent=[77,-7.5; 92,7.5],
                                                                    rotation=
              180);
        ExWall EWall(
          area=11.3,
          sfa=270,
          OH=0.3,
          W=8) annotation (extent=[92, -10; 112, 10]);
        BondLib.Bonds.eBond AB1 annotation (extent=[-30, 16; -16, 30], rotation=
             -90);
        BondLib.Bonds.eBond RB1 annotation (extent=[-14, 16; 0, 30], rotation=-
              90);
        Roof Roof1(
          area=72.5,
          OH=0,
          W=0) annotation (extent=[-17, 30; 3, 50], rotation=90);
        BondLib.Sources.mSf mSf1 annotation (extent=[-30.5,30; -15.5,45],
            rotation=90);
        AirConditioning AirConditioning1 annotation (extent=[-28,50; -18,60],
              rotation=-90);
        BondLib.Bonds.eBond CCB1 annotation (extent=[-14, -29; 0, -15],
            rotation=-90);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=288)) annotation (extent=
              [-14, -45; 1, -30], rotation=-90);
        Slab Slab1(perim=23.7, area=72.5) annotation (extent=[-4,-62; 16,-42],
              rotation=270);
        BondLib.Bonds.eBond SLB2 annotation (extent=[-4,-42; 16,-22],
            rotation=90);
        BondLib.Junctions.J0p6 J0p6_3 annotation (extent=[-50, -10; -30, 10]);
        BondLib.Sensors.De De1 annotation (extent=[-28, 67; -18, 77], rotation=
              -90);
        BondLib.Bonds.fBond AB2 annotation (extent=[-31, 77; -16, 92], rotation=
             -90);
        ExDoor NDoor(
          lv=3.57,
          area=2.17,
          sfa=180,
          OH=0,
          W=0,
          absor=0.6) annotation (extent=[-52,68; -38,88], rotation=90);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[-100,-5; -90,5]);
      equation 
        connect(NWin.BondCon1, NB1.fBondCon1) annotation (points=[-75.5,68; -75,
              68],   style(color=8));
        connect(NWall.BondCon1, NB2.fBondCon1) annotation (points=[-60, 68; -60,
                68], style(color=8));
        connect(WWall.BondCon1, WB1.fBondCon1) annotation (points=[-80,30; -79,
              30],   style(color=8));
        connect(WB1.eBondCon1, J0p5_1.BondCon1) annotation (points=[-64,30; -63,
              30],     style(color=8));
        connect(NB1.eBondCon1, J0p5_1.BondCon3) annotation (points=[-75,48; -75,
              40; -57,40],      style(color=8));
        connect(NB2.eBondCon1, J0p5_1.BondCon4) annotation (points=[-60,48; -49,
              48; -49,40],      style(color=8));
        connect(NB3.eBondCon1, J0p5_1.BondCon2) annotation (points=[-45,48; -45,
              40; -40,40; -40,30; -43,30],        style(color=8));
        connect(SWin1.BondCon1, SB1.fBondCon1) annotation (points=[-60, -42; -
              60, -42], style(color=8));
        connect(SWall.BondCon1, SB2.fBondCon1) annotation (points=[-40, -42; -
              40, -42], style(color=8));
        connect(SWin2.BondCon1, SB3.fBondCon1) annotation (points=[-20,-42; -20,
              -42],     style(color=8));
        connect(J0p5_1.BondCon5, CB1.fBondCon1) annotation (points=[-53,20; -53,
              20],     style(color=8));
        connect(CB2.eBondCon1, J0p6_1.BondCon1) annotation (points=[-14,0; -13,
              0],   style(color=8));
        connect(CB3.eBondCon1, J0p6_1.BondCon2) annotation (points=[7,0.5; 7,0],
              style(color=8));
        connect(J0p5_3.BondCon1, CB3.fBondCon1) annotation (points=[22,0; 22,
              0.5],
                  style(color=8));
        connect(CB4.eBondCon1, J0p5_3.BondCon2) annotation (points=[42,0.5;
              42.25,0.5; 42.25,-0.5; 42.5,-0.5; 42.5,0; 42,0],
                  style(color=8));
        connect(J0p6_2.BondCon1, CB4.fBondCon1) annotation (points=[57,0; 57,
              0.5],
                  style(color=8));
        connect(J0p5_3.BondCon3, NB4.eBondCon1) annotation (points=[28, 10; 28,
                15; 9.5, 15; 9.5, 20], style(color=8));
        connect(J0p5_3.BondCon4, NB5.eBondCon1) annotation (points=[36,10; 36,
              18; 24.5,18; 24.5,20], style(color=8));
        connect(SsWall.BondCon2, SB5.fBondCon1) annotation (points=[49.5,-35;
              50,-35],style(color=8));
        connect(SB6.fBondCon1, SsDoor.BondCon1) annotation (points=[24.5,-40;
              25,-40],style(color=8));
        connect(SB7.fBondCon1, SsWin.BondCon1) annotation (points=[70.5,-40; 70,
              -40],   style(color=8));
        connect(J0p6_2.BondCon4, SB7.eBondCon1) annotation (points=[71,-10; 71,
              -17.5; 71,-25; 70.5,-25],
                      style(color=8));
        connect(SB5.eBondCon1, J0p4_1.BondCon4) annotation (points=[50,-55; 50,
              -55],   style(color=8));
        connect(SB8.eBondCon1, J0p4_1.BondCon1) annotation (points=[44,-60;
              44.5,-60],
                      style(color=8));
        connect(SsDoor.BondCon2, SB8.fBondCon1) annotation (points=[25,-60; 31,
              -60],   style(color=8));
        connect(SB9.eBondCon1, J0p4_1.BondCon2) annotation (points=[55,-60;
              55.5,-60],
                      style(color=8));
        connect(SsWin.BondCon2, SB9.fBondCon1) annotation (points=[70,-60; 68,
              -60],   style(color=8));
        connect(J0p4_1.BondCon3, BondCon1) annotation (points=[50,-65; 50,-72],
                 style(color=8));
        connect(NB5.fBondCon1, WallL2.BondCon1) annotation (points=[24.5,34; 25,
              34],   style(color=8));
        connect(NB4.fBondCon1, DoorL2.BondCon1) annotation (points=[9.5,34; 9,
              34], style(color=8));
        connect(DoorL2.BondCon2, NB7.fBondCon1) annotation (points=[9,54; 9,55],
                   style(color=8));
        connect(WallL2.BondCon2, NB8.fBondCon1) annotation (points=[25,54; 25,
              54.5; 25,55; 25.5,55],
                     style(color=8));
        connect(NB7.eBondCon1, J0p3_1.BondCon1) annotation (points=[9,68; 9,68],
                     style(color=8));
        connect(NB8.eBondCon1, J0p3_1.BondCon2) annotation (points=[25.5,68; 25,
              68],     style(color=8));
        connect(J0p3_1.BondCon3, BondCon2) annotation (points=[17,74.5; 17,83],
              style(color=8));
        connect(J0p6_2.BondCon5, NB9.eBondCon1) annotation (points=[63,10; 63,
              16; 49.5,16; 49.5,20], style(color=8));
        connect(J0p6_2.BondCon6, NB10.eBondCon1) annotation (points=[71,10; 71,
              15; 71,20; 70.5,20],
                     style(color=8));
        connect(NB9.fBondCon1, DoorL1.BondCon1) annotation (points=[49.5,34; 50,
              34],   style(color=8));
        connect(NB10.fBondCon1, WallL1.BondCon1) annotation (points=[70.5,34;
              70,34],style(color=8));
        connect(DoorL1.BondCon2, NB11.fBondCon1) annotation (points=[50,54; 50,
              55.5], style(color=8));
        connect(WallL1.BondCon2, NB12.fBondCon1) annotation (points=[70,54; 70,
              55.5], style(color=8));
        connect(NB11.eBondCon1, J0p3_2.BondCon1) annotation (points=[50,68.5;
              52,68.5],
                     style(color=8));
        connect(NB12.eBondCon1, J0p3_2.BondCon2) annotation (points=[70,68.5;
              68,68.5],
                     style(color=8));
        connect(J0p3_2.BondCon3, BondCon3) annotation (points=[60,75; 60,83],
              style(color=8));
        connect(EB1.eBondCon1, J0p6_2.BondCon2) annotation (points=[77,
              9.18455e-016; 77,0],
                  style(color=8));
        connect(EWall.BondCon1, EB1.fBondCon1) annotation (points=[92,0; 92,
              -9.18455e-016],
                    style(color=8));
        connect(RB1.eBondCon1, J0p6_1.BondCon6) annotation (points=[-7,16; -7,
              13; 1,13; 1,10],     style(color=8));
        connect(AB1.eBondCon1, J0p6_1.BondCon5) annotation (points=[-23, 16; -
              23, 10; -7, 10], style(color=8));
        connect(Roof1.BondCon1, RB1.fBondCon1) annotation (points=[-7,30; -7,30],
                   style(color=8));
        connect(mSf1.BondCon1, AB1.fBondCon1) annotation (points=[-23,30; -23,
              30],   style(color=8));
        connect(J0p6_1.BondCon3, CCB1.fBondCon1) annotation (points=[-7,-10; -7,
              -12.5; -7,-15; -7,-15],
                       style(color=8));
        connect(CCB1.eBondCon1, Cth1.BondCon1) annotation (points=[-7,-29; -7,
              -29.5; -6.5,-29.5; -6.5,-30],     style(color=8));
        connect(CB1.eBondCon1, J0p6_3.BondCon5) annotation (points=[-53,5; -53,
              2; -49,2; -49,10; -44,10],      style(color=8));
        connect(J0p6_3.BondCon2, CB2.fBondCon1) annotation (points=[-30,0; -29,
              0],   style(color=8));
        connect(SB1.eBondCon1, J0p6_3.BondCon1) annotation (points=[-60, -22; -
              60, 0; -50, 0], style(color=8));
        connect(SB2.eBondCon1, J0p6_3.BondCon3) annotation (points=[-40, -22; -
              40, -16; -44, -16; -44, -10], style(color=8));
        connect(SB3.eBondCon1, J0p6_3.BondCon4) annotation (points=[-20,-22;
              -20,-10; -36,-10],  style(color=8));
        connect(De1.OutPort1,AirConditioning1.u)       annotation (points=[-23,66.5;
              -23,61],        style(color=3));
        connect(AB2.fBondCon1, De1.BondCon1) annotation (points=[-23.5,77; -23,
              77], style(color=8));
        connect(J0p6_3.BondCon6, AB2.eBondCon1) annotation (points=[-36,10; -36,
              95; -23.5,95; -23.5,92],   style(color=8));
        connect(SB4.eBondCon1, J0p6_2.BondCon3) annotation (points=[62, -10; 63,
                -10], style(color=8));
        connect(SB4.fBondCon1, SsWall.BondCon1) annotation (points=[52,-10;
              49.5,-10; 49.5,-19],
                               style(color=8));
        connect(J0p5_3.BondCon5, SB6.eBondCon1) annotation (points=[32,-10; 32,
              -16; 24.5,-16; 24.5,-25], style(color=8));
        connect(NDoor.BondCon1, NB3.fBondCon1) annotation (points=[-45,68; -45,
              68], style(color=8, rgbcolor={192,192,192}));
        connect(Slab1.BondCon1, SLB2.fBondCon1) annotation (points=[6,-42; 6,
              -42], style(color=8, rgbcolor={192,192,192}));
        connect(SLB2.eBondCon1, J0p6_1.BondCon4) annotation (points=[6,-22; 6,
              -16; 1,-16; 1,-10], style(color=8, rgbcolor={192,192,192}));
        connect(SWin1.u, u) annotation (points=[-60,-63; -60,-70; -80,-70; -80,
              0; -95,0],
                       style(color=74, rgbcolor={0,0,127}));
        connect(WWall.u, u) annotation (points=[-101,30; -105,30; -105,20; -80,
              20; -80,0; -95,0],  style(color=74, rgbcolor={0,0,127}));
        connect(SWin1.u, SWall.u) annotation (points=[-60,-63; -60,-70; -40,-70;
              -40,-63], style(color=74, rgbcolor={0,0,127}));
        connect(SWall.u, SWin2.u) annotation (points=[-40,-63; -40,-70; -20,-70;
              -20,-63], style(color=74, rgbcolor={0,0,127}));
        connect(SWin2.u, Slab1.u) annotation (points=[-20,-63; -20,-70; 6,-70;
              6,-63], style(color=74, rgbcolor={0,0,127}));
        connect(WWall.u, NWin.u) annotation (points=[-101,30; -105,30; -105,96;
              -75.5,96; -75.5,89], style(color=74, rgbcolor={0,0,127}));
        connect(NWin.u, NWall.u) annotation (points=[-75.5,89; -75.5,96; -60,96;
              -60,89], style(color=74, rgbcolor={0,0,127}));
        connect(NWall.u, NDoor.u) annotation (points=[-60,89; -60,96; -45,96;
              -45,89], style(color=74, rgbcolor={0,0,127}));
        connect(NDoor.u, Roof1.u) annotation (points=[-45,89; -45,96; -7,96; -7,
              51], style(color=74, rgbcolor={0,0,127}));
        connect(Roof1.u, EWall.u) annotation (points=[-7,51; -7,96; 120,96; 120,
              0; 113,0], style(color=74, rgbcolor={0,0,127}));
        connect(AirConditioning1.y, mSf1.s) annotation (points=[-23,49.5; -23,
              44.25], style(color=74, rgbcolor={0,0,127}));
      end LivingRoom;
      
      model BedRoom1 "Bed room #1" 
        parameter Modelica.SIunits.SpecificHeatCapacity c=1000 
          "Specific heat capacity of first bedroom";
        parameter Modelica.SIunits.Mass m=54 "Mass of first bedroom";
        parameter Real gamma(unit="W.h/K") = c*m/3600 
          "Heat capacity of first bedroom";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[1, 1],
            component=[10, 10]),
          Icon(
            Line(points=[-90, 90; -90, -40; 90, -40; 90, 20; -30, 20; -30, 90;
                  -90, 90], style(color=53, thickness=2)),
            Line(points=[-29, 90; 90, 90; 90, 20], style(color=57, thickness=2)),
            Line(points=[30, 90; 30, 20], style(color=53, thickness=2)),
            Line(points=[10, -40; 10, -80; 90, -80; 90, -41], style(color=53,
                  thickness=2)),
            Rectangle(extent=[60, -38; 80, -42], style(color=53)),
            Rectangle(extent=[-30, -38; -10, -42], style(color=53)),
            Rectangle(extent=[-70, -38; -50, -42], style(color=53)),
            Rectangle(extent=[-84, 92; -64, 88], style(color=53)),
            Polygon(points=[-55, 90; -40, 75; -40, 90; -55, 90], style(color=53)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=53)),
            Polygon(points=[40, 20; 40, 40; 56, 20; 40, 20], style(color=9)),
            Polygon(points=[20, -40; 20, -20; 35, -40; 20, -40], style(color=53)),
            Rectangle(extent=[-10, 92; 10, 88], style(color=53)),
            Rectangle(extent=[50, 92; 70, 88], style(color=9)),
            Rectangle(extent=[10, -78; 30, -82], style(color=53)),
            Rectangle(extent=[30, -78; 50, -82], style(color=53)),
            Rectangle(extent=[50, -78; 70, -82], style(color=53)),
            Rectangle(extent=[70, -78; 90, -82], style(color=53)),
            Polygon(points=[10, -50; -10, -50; 10, -65; 10, -50], style(color=
                    53)),
            Text(
              extent=[-19, 69; 17, 54],
              string="Bedroom 2",
              style(color=53)),
            Text(
              extent=[34, 74; 86, 40],
              string="Bedroom 1",
              style(color=9)),
            Text(
              extent=[35, -54; 66, -66],
              string="Sunspace",
              style(color=53)),
            Text(
              extent=[-36, 0; 44, -10],
              string="Living Room",
              style(color=53)),
            Rectangle(extent=[30, 90; 90, 20], style(color=9, thickness=2))),
          Diagram(
            Line(points=[-90, 90; -90, -40; 90, -40; 90, 20; -30, 20; -30, 90;
                  -90, 90], style(color=53, thickness=2)),
            Line(points=[-29, 90; 90, 90; 90, 20], style(color=57, thickness=2)),
            Line(points=[30, 90; 30, 20], style(color=53, thickness=2)),
            Line(points=[10, -40; 10, -80; 90, -80; 90, -41], style(color=53,
                  thickness=2)),
            Rectangle(extent=[60, -38; 80, -42], style(color=53)),
            Rectangle(extent=[-30, -38; -10, -42], style(color=53)),
            Rectangle(extent=[-70, -38; -50, -42], style(color=53)),
            Rectangle(extent=[-84, 92; -64, 88], style(color=53)),
            Polygon(points=[-55, 90; -40, 75; -40, 90; -55, 90], style(color=53)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=53)),
            Polygon(points=[40, 20; 40, 40; 56, 20; 40, 20], style(color=41)),
            Polygon(points=[20, -40; 20, -20; 35, -40; 20, -40], style(color=53)),
            Rectangle(extent=[-10, 92; 10, 88], style(color=53)),
            Rectangle(extent=[50, 92; 70, 88], style(color=41)),
            Rectangle(extent=[10, -78; 30, -82], style(color=53)),
            Rectangle(extent=[30, -78; 50, -82], style(color=53)),
            Rectangle(extent=[50, -78; 70, -82], style(color=53)),
            Rectangle(extent=[70, -78; 90, -82], style(color=53)),
            Polygon(points=[10, -50; -10, -50; 10, -65; 10, -50], style(color=
                    53)),
            Text(
              extent=[-19, 69; 17, 54],
              string="Bedroom 2",
              style(color=53)),
            Text(
              extent=[34, 72; 86, 38],
              string="Bedroom 1",
              style(color=41)),
            Text(
              extent=[35, -54; 66, -66],
              string="Sunspace",
              style(color=53)),
            Text(
              extent=[-36, -7; 44, -17],
              string="Living Room",
              style(color=53)),
            Rectangle(extent=[30, 90; 90, 20], style(color=41, thickness=2)),
            Line(points=[56, 29; 56, 29], style(color=8))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This model uses the same technology as the more complex <i>living room</i>.  Please, read the documeentation of the living room model.
</html>
"));
        BondLib.Junctions.J0p5 J0p5_1 annotation (extent=[50, 30; 70, 50]);
        BondLib.Junctions.J0p5 J0p5_2 annotation (extent=[50, 84; 70, 64]);
        BondLib.Bonds.eBond CB1 annotation (extent=[53, 50; 67, 64], rotation=-
              90);
        BondLib.Bonds.eBond EB1 annotation (extent=[71, 33; 86, 48], rotation=
              180);
        ExWall EWall(
          area=9.4,
          sfa=270,
          OH=0.3,
          W=8) annotation (extent=[86, 30; 106, 50]);
        BondLib.Bonds.eBond NB1 annotation (extent=[49, 84; 63, 99], rotation=-
              90);
        BondLib.Bonds.eBond NB2 annotation (extent=[70, 84; 84, 99], rotation=-
              90);
        Window NWin(
          area=1.5,
          OH=0,
          W=0,
          sfa=180) annotation (extent=[47, 89; 28, 109]);
        ExWall NWall(
          area=9,
          sfa=180,
          OH=0.3,
          W=12) annotation (extent=[87, 89; 107, 109]);
        BondLib.Bonds.eBond RB1 annotation (extent=[70, 67; 84, 81], rotation=-
              180);
        Roof Roof1(
          area=15,
          OH=0,
          W=0) annotation (extent=[84, 64; 104, 84]);
        BondLib.Bonds.fBond WB1 annotation (extent=[35, 67; 50, 81], rotation=
              180);
        IntWall Wall12(area=6.6) annotation (extent=[35, 64; 15, 84]);
        BondLib.Bonds.eBond WB2 annotation (extent=[2, 68; 15, 81], rotation=
              180);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-13, 67; 2, 82]);
        BondLib.Interfaces.BondCon BondCon1 "Bond graph connector to bedroom 2"
                                            annotation (extent=[-30,69; -20,80]);
        BondLib.Bonds.eBond CCB1 annotation (extent=[73, 6; 87, 20], rotation=-
              90);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=288)) annotation (extent=
              [72, -9; 87, 6], rotation=-90);
        BondLib.Bonds.eBond CB2 annotation (extent=[48, 5; 63, 20], rotation=90);
        BondLib.Interfaces.BondCon BondCon2 
          "Bond graph connector to living room" 
                                            annotation (extent=[50,-20; 61,-10]);
        BondLib.Bonds.eBond SLB2 annotation (extent=[10, 15; 30, 35], rotation=
              90);
        Slab Slab1(perim=7.9, area=15) annotation (extent=[10,-5; 30,15],
            rotation=270);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[90,55; 100,65], rotation=180);
      equation 
        connect(CB1.eBondCon1, J0p5_1.BondCon5) annotation (points=[60, 50; 60,
                50], style(color=8));
        connect(J0p5_2.BondCon5, CB1.fBondCon1) annotation (points=[60, 64; 60,
                64], style(color=8));
        connect(EB1.eBondCon1, J0p5_1.BondCon2) annotation (points=[71,40.5; 71,
              40.25; 71,40; 70,40],
                     style(color=8));
        connect(EWall.BondCon1, EB1.fBondCon1) annotation (points=[86,40; 86,
              40.5],
                   style(color=8));
        connect(NB1.eBondCon1, J0p5_2.BondCon3) annotation (points=[56, 84; 56,
                84], style(color=8));
        connect(NB2.eBondCon1, J0p5_2.BondCon4) annotation (points=[77, 84; 64,
                84], style(color=8));
        connect(NWall.BondCon1, NB2.fBondCon1) annotation (points=[87, 99; 77,
              99], style(color=8));
        connect(RB1.eBondCon1, J0p5_2.BondCon2) annotation (points=[70,74; 70,
              74],   style(color=8));
        connect(Roof1.BondCon1, RB1.fBondCon1) annotation (points=[84, 74; 84,
              74], style(color=8));
        connect(NWin.BondCon1, NB1.fBondCon1) annotation (points=[47, 99; 56,
              99], style(color=8));
        connect(J0p5_2.BondCon1, WB1.eBondCon1) annotation (points=[50, 74; 50,
                74], style(color=8));
        connect(WB1.fBondCon1, Wall12.BondCon1) annotation (points=[35, 74; 35,
                74], style(color=8));
        connect(Wall12.BondCon2, WB2.fBondCon1) annotation (points=[15,74; 15,
              74.5], style(color=8));
        connect(WB2.eBondCon1, J0p2_1.BondCon2) annotation (points=[2,74.5; 2,
              74.5], style(color=8));
        connect(J0p2_1.BondCon1, BondCon1) annotation (points=[-13,74.5; -25,
              74.5],
                 style(color=8));
        connect(J0p5_1.BondCon4, CCB1.fBondCon1) annotation (points=[64, 30; 80,
                30; 80, 20], style(color=8));
        connect(CCB1.eBondCon1, Cth1.BondCon1) annotation (points=[80,6; 79.75,
              6; 79.75,6; 79.5,6],
                 style(color=8));
        connect(CB2.eBondCon1, J0p5_1.BondCon3) annotation (points=[55.5,20;
              55.5,25; 56,25; 56,30],
                     style(color=8));
        connect(CB2.fBondCon1, BondCon2) annotation (points=[55.5,5; 55.5,-15],
            style(color=8));
        connect(SLB2.eBondCon1, J0p5_1.BondCon1) annotation (points=[20, 35; 20,
                40; 50, 40], style(color=8));
        connect(SLB2.fBondCon1, Slab1.BondCon1) annotation (points=[20,15; 20,
              15], style(color=8, rgbcolor={192,192,192}));
        connect(Roof1.u, u) annotation (points=[105,74; 113,74; 113,67; 86,67;
              86,60; 95,60], style(color=74, rgbcolor={0,0,127}));
        connect(Roof1.u, EWall.u) annotation (points=[105,74; 113,74; 113,40;
              107,40], style(color=74, rgbcolor={0,0,127}));
        connect(Roof1.u, NWall.u) annotation (points=[105,74; 113,74; 113,99;
              108,99], style(color=74, rgbcolor={0,0,127}));
        connect(NWall.u, NWin.u) annotation (points=[108,99; 113,99; 113,112;
              21,112; 21,99; 27.05,99], style(color=74, rgbcolor={0,0,127}));
        connect(EWall.u, Slab1.u) annotation (points=[107,40; 113,40; 113,-20;
              20,-20; 20,-6], style(color=74, rgbcolor={0,0,127}));
      end BedRoom1;
      
      model BedRoom2 "Bed room #2" 
        parameter Modelica.SIunits.SpecificHeatCapacity c=1000 
          "Specific heat capacity of second bedroom";
        parameter Modelica.SIunits.Mass m=54 "Mass of second bedroom";
        parameter Real gamma(unit="W.h/K") = c*m/3600 
          "Heat capacity of second bedroom";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[1, 1],
            component=[10, 10]),
          Icon(
            Line(points=[-90, 90; -90, -40; 90, -40; 90, 20; -30, 20; -30, 90;
                  -90, 90], style(color=53, thickness=2)),
            Line(points=[-29, 90; 90, 90; 90, 20], style(color=57, thickness=2)),
            Line(points=[30, 90; 30, 20], style(color=53, thickness=2)),
            Line(points=[10, -40; 10, -80; 90, -80; 90, -41], style(color=53,
                  thickness=2)),
            Rectangle(extent=[60, -38; 80, -42], style(color=53)),
            Rectangle(extent=[-30, -38; -10, -42], style(color=53)),
            Rectangle(extent=[-70, -38; -50, -42], style(color=53)),
            Rectangle(extent=[-84, 92; -64, 88], style(color=53)),
            Polygon(points=[-55, 90; -40, 75; -40, 90; -55, 90], style(color=53)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=9)),
            Polygon(points=[40, 20; 40, 40; 56, 20; 40, 20], style(color=53)),
            Polygon(points=[20, -40; 20, -20; 35, -40; 20, -40], style(color=53)),
            Rectangle(extent=[-10, 92; 10, 88], style(color=9)),
            Rectangle(extent=[50, 92; 70, 88], style(color=53)),
            Rectangle(extent=[10, -78; 30, -82], style(color=53)),
            Rectangle(extent=[30, -78; 50, -82], style(color=53)),
            Rectangle(extent=[50, -78; 70, -82], style(color=53)),
            Rectangle(extent=[70, -78; 90, -82], style(color=53)),
            Polygon(points=[10, -50; -10, -50; 10, -65; 10, -50], style(color=
                    53)),
            Text(
              extent=[41, 69; 77, 54],
              string="Bedroom 1",
              style(color=53)),
            Text(
              extent=[-26, 76; 26, 42],
              string="Bedroom 2",
              style(color=9)),
            Text(
              extent=[35, -54; 66, -66],
              string="Sunspace",
              style(color=53)),
            Text(
              extent=[-36, 0; 44, -10],
              string="Living Room",
              style(color=53)),
            Rectangle(extent=[-30, 90; 30, 20], style(color=9, thickness=2))),
          Diagram(
            Line(points=[-90, 90; -90, -40; 90, -40; 90, 20; -30, 20; -30, 90;
                  -90, 90], style(color=53, thickness=2)),
            Line(points=[-29, 90; 90, 90; 90, 20], style(color=57, thickness=2)),
            Line(points=[30, 90; 30, 20], style(color=53, thickness=2)),
            Line(points=[10, -40; 10, -80; 90, -80; 90, -41], style(color=53,
                  thickness=2)),
            Rectangle(extent=[60, -38; 80, -42], style(color=53)),
            Rectangle(extent=[-30, -38; -10, -42], style(color=53)),
            Rectangle(extent=[-70, -38; -50, -42], style(color=53)),
            Rectangle(extent=[-84, 92; -64, 88], style(color=53)),
            Polygon(points=[-55, 90; -40, 75; -40, 90; -55, 90], style(color=53)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=41)),
            Polygon(points=[40, 20; 40, 40; 56, 20; 40, 20], style(color=53)),
            Polygon(points=[20, -40; 20, -20; 35, -40; 20, -40], style(color=53)),
            Rectangle(extent=[-10, 92; 10, 88], style(color=41)),
            Rectangle(extent=[50, 92; 70, 88], style(color=53)),
            Rectangle(extent=[10, -78; 30, -82], style(color=53)),
            Rectangle(extent=[30, -78; 50, -82], style(color=53)),
            Rectangle(extent=[50, -78; 70, -82], style(color=53)),
            Rectangle(extent=[70, -78; 90, -82], style(color=53)),
            Polygon(points=[10, -50; -10, -50; 10, -65; 10, -50], style(color=
                    53)),
            Text(
              extent=[41, 69; 77, 54],
              string="Bedroom 1",
              style(color=53)),
            Text(
              extent=[-26, 76; 26, 42],
              string="Bedroom 2",
              style(color=41)),
            Text(
              extent=[35, -54; 66, -66],
              string="Sunspace",
              style(color=53)),
            Text(
              extent=[-27, -1; 53, -11],
              string="Living Room",
              style(color=53)),
            Rectangle(extent=[-30, 90; 30, 20], style(color=41, thickness=2)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=53))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This model uses the same technology as the more complex <i>living room</i>.  Please, read the documeentation of the living room model.
</html>
"));
        BondLib.Junctions.J0p5 J0p5_2 annotation (extent=[-10, 84; 10, 64]);
        BondLib.Bonds.eBond CB1 annotation (extent=[-7, 50; 7, 64], rotation=-
              90);
        BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[-10, 30; 10, 50]);
        BondLib.Bonds.eBond NB1 annotation (extent=[-11, 84; 3, 99], rotation=-
              90);
        BondLib.Bonds.eBond NB2 annotation (extent=[13, 84; 27, 99], rotation=-
              90);
        ExWall NWall(
          area=9,
          sfa=180,
          OH=0.3,
          W=12) annotation (extent=[30, 89; 50, 109]);
        Window NWin(
          area=1.5,
          OH=0,
          W=0,
          sfa=180) annotation (extent=[-14, 89; -33, 109]);
        BondLib.Bonds.eBond RB1 annotation (extent=[-24, 67; -10, 81]);
        Roof Roof1(
          area=15,
          OH=0,
          W=0) annotation (extent=[-24, 64; -44, 84]);
        BondLib.Bonds.eBond CCB1 annotation (extent=[13, 16; 27, 30], rotation=
              -90);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=288)) annotation (extent=
              [13, 1; 28, 16], rotation=-90);
        BondLib.Bonds.eBond CB2 annotation (extent=[-7.5,3; 7.5,18],
                                                                   rotation=90);
        BondLib.Interfaces.BondCon BondCon1 
          "Bond graph connector to living room" 
                                            annotation (extent=[-5, -20; 5, -10]);
        BondLib.Bonds.eBond SLB2 annotation (extent=[-30, 10; -10, 30],
            rotation=90);
        Slab Slab1(perim=4.2, area=15) annotation (extent=[-30,-10; -10,10],
            rotation=270);
        BondLib.Bonds.eBond CB3 annotation (extent=[22,66; 37,82],   rotation=
              180);
        BondLib.Interfaces.BondCon BondCon2 "Bond graph connector to bedroom 1"
                                            annotation (extent=[50,69; 60,79]);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[-25,90; -15,100], rotation=270);
      equation 
        connect(J0p5_2.BondCon5, CB1.fBondCon1) annotation (points=[0,64;
              4.59566e-016,64],
                   style(color=8));
        connect(CB1.eBondCon1, J0p4_1.BondCon4) annotation (points=[
              1.31679e-015,50; 0,50],
                   style(color=8));
        connect(NB1.eBondCon1, J0p5_2.BondCon3) annotation (points=[-4,84; -4,
              84],   style(color=8));
        connect(NB2.eBondCon1, J0p5_2.BondCon4) annotation (points=[20, 84; 4,
              84], style(color=8));
        connect(NWall.BondCon1, NB2.fBondCon1) annotation (points=[30, 99; 20,
              99], style(color=8));
        connect(NWin.BondCon1, NB1.fBondCon1) annotation (points=[-14,99; -4,99],
                   style(color=8));
        connect(RB1.eBondCon1, J0p5_2.BondCon1) annotation (points=[-10, 74; -
              10, 74], style(color=8));
        connect(Roof1.BondCon1, RB1.fBondCon1) annotation (points=[-24, 74; -24,
                74], style(color=8));
        connect(J0p4_1.BondCon2, CCB1.fBondCon1) annotation (points=[10, 40; 20,
                40; 20, 30], style(color=8));
        connect(CCB1.eBondCon1, Cth1.BondCon1) annotation (points=[20,16; 20.5,
              16], style(color=8));
        connect(CB2.eBondCon1, J0p4_1.BondCon3) annotation (points=[
              4.59227e-016,18; 4.59227e-016,24; 0,24; 0,30],
                   style(color=8));
        connect(CB2.fBondCon1, BondCon1) annotation (points=[-4.59227e-016,3;
              -4.59227e-016,-8; 0,-8; 0,-15],
            style(color=8));
        connect(SLB2.eBondCon1, J0p4_1.BondCon1) annotation (points=[-20, 30; -
              20, 40; -10, 40], style(color=8));
        connect(CB3.eBondCon1, J0p5_2.BondCon2) annotation (points=[22,74; 10,
              74],   style(color=8));
        connect(SLB2.fBondCon1, Slab1.BondCon1) annotation (points=[-20,10; -20,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(CB3.fBondCon1, BondCon2) annotation (points=[37,74; 55,74],
            style(color=8, rgbcolor={192,192,192}));
        connect(NWin.u, u) annotation (points=[-33.95,99; -50,99; -50,85; -20,
              85; -20,95], style(color=74, rgbcolor={0,0,127}));
        connect(NWin.u, Roof1.u) annotation (points=[-33.95,99; -50,99; -50,74;
              -45,74], style(color=74, rgbcolor={0,0,127}));
        connect(Roof1.u, Slab1.u) annotation (points=[-45,74; -50,74; -50,-16;
              -20,-16; -20,-11], style(color=74, rgbcolor={0,0,127}));
        connect(NWin.u, NWall.u) annotation (points=[-33.95,99; -50,99; -50,113;
              56,113; 56,99; 51,99], style(color=74, rgbcolor={0,0,127}));
      end BedRoom2;
      
      model SunSpace "Solarium" 
        parameter Modelica.SIunits.SpecificHeatCapacity c=1000 
          "Specific heat capacity of solarium";
        parameter Modelica.SIunits.Mass m=48.6 "Mass of solarium";
        parameter Real gamma(unit="W.h/K") = c*m/3600 
          "Heat capacity of solarium";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[1, 1],
            component=[10, 10]),
          Icon(
            Line(points=[-90, 90; -90, -40; 90, -40; 90, 20; -30, 20; -30, 90;
                  -90, 90], style(color=53, thickness=2)),
            Line(points=[-29, 90; 90, 90; 90, 20], style(color=57, thickness=2)),
            Line(points=[30, 90; 30, 20], style(color=53, thickness=2)),
            Line(points=[10, -40; 10, -80; 90, -80; 90, -41], style(color=53,
                  thickness=2)),
            Rectangle(extent=[60, -38; 80, -42], style(color=9)),
            Rectangle(extent=[-30, -38; -10, -42], style(color=53)),
            Rectangle(extent=[-70, -38; -50, -42], style(color=53)),
            Rectangle(extent=[-84, 92; -64, 88], style(color=53)),
            Polygon(points=[-55, 90; -40, 75; -40, 90; -55, 90], style(color=53)),
            Polygon(points=[20, 20; 20, 40; 5, 20; 20, 20], style(color=53)),
            Polygon(points=[40, 20; 40, 40; 56, 20; 40, 20], style(color=53)),
            Polygon(points=[20, -40; 20, -20; 35, -40; 20, -40], style(color=9)),
            Rectangle(extent=[-10, 92; 10, 88], style(color=53)),
            Rectangle(extent=[50, 92; 70, 88], style(color=53)),
            Rectangle(extent=[10, -78; 30, -82], style(color=9)),
            Rectangle(extent=[30, -78; 50, -82], style(color=9)),
            Rectangle(extent=[50, -78; 70, -82], style(color=9)),
            Rectangle(extent=[70, -78; 90, -82], style(color=9)),
            Polygon(points=[10, -50; -10, -50; 10, -65; 10, -50], style(color=9)),
            Text(
              extent=[-19, 69; 17, 54],
              string="Bedroom 2",
              style(color=53)),
            Text(
              extent=[24, -50; 76, -70],
              string="Sunspace",
              style(color=9)),
            Text(
              extent=[-36, 0; 44, -10],
              string="Living Room",
              style(color=53)),
            Text(
              extent=[41, 69; 77, 54],
              string="Bedroom 1",
              style(color=53)),
            Rectangle(extent=[10, -40; 90, -80], style(color=9, thickness=2))),
          Diagram(
            Rectangle(extent=[30, 2; 70, -2], style(color=41)),
            Polygon(points=[-60, 0; -60, 40; -30, 0; -60, 0], style(color=41)),
            Rectangle(extent=[-70, -78; -30, -82], style(color=41)),
            Polygon(points=[-70, -20; -110, -20; -70, -48; -70, -20], style(
                  color=41)),
            Text(
              extent=[-6, -22; 26, -40],
              string="Sunspace",
              style(color=41)),
            Rectangle(extent=[-70, 0; 90, -80], style(color=41, thickness=2)),
            Rectangle(extent=[-30, -78; 10, -82], style(color=41)),
            Rectangle(extent=[10, -78; 50, -82], style(color=41)),
            Rectangle(extent=[50, -78; 90, -82], style(color=41)),
            Line(points=[90, 0; 90, 100], style(color=53, thickness=2)),
            Line(points=[-70, 0; -100, 0], style(color=53, thickness=2)),
            Text(
              extent=[-86, 74; 14, 60],
              string="Living Room",
              style(color=53))),
          Documentation(info="<html>
This model uses the same technology as the more complex <i>living room</i>.  Please, read the documeentation of the living room model.
</html>
"));
        BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[20, -55; 50, -25]);
        BondLib.Junctions.J0p6 J0p6_2 annotation (extent=[-30, -55; 0, -25]);
        BondLib.Bonds.eBond CB1 annotation (extent=[0, -50; 20, -30]);
        BondLib.Bonds.eBond EB1 annotation (extent=[50, -50; 70, -30], rotation=
             180);
        Window EWin(
          lv=4.9,
          area=6.2,
          OH=0,
          W=0,
          sfa=270) annotation (extent=[70, -50; 89, -30]);
        BondLib.Bonds.eBond EB2 annotation (extent=[49,-27; 69,-7],    rotation=
             180);
        BondLib.Bonds.eBond CB2 annotation (extent=[19, -17; 39, 3], rotation=
              270);
        BondLib.Interfaces.BondCon BondCon1 
          "Bond graph connector to living room" 
                                            annotation (extent=[24, 8; 34, 18]);
        BondLib.Bonds.eBond WB1 annotation (extent=[-50, -50; -30, -30]);
        Window WWin(
          lv=4.9,
          area=6.2,
          OH=0,
          W=0,
          sfa=90) annotation (extent=[-50, -50; -69, -30]);
        BondLib.Bonds.eBond WB2 annotation (extent=[-50,-27; -30,-7]);
        BondLib.Bonds.eBond RB1 annotation (extent=[-19,-17; 1,3],   rotation=
              270);
        BondLib.Bonds.eBond CCB1 annotation (extent=[19, -75; 39, -55],
            rotation=-90);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=288)) annotation (extent=
              [19, -95; 39, -75], rotation=-90);
        BondLib.Bonds.eBond SB1 annotation (extent=[-19, -75; 1, -55], rotation=
             90);
        Window SWin(
          lv=4.9,
          area=10.3,
          OH=0,
          W=0,
          sfa=0) annotation (extent=[-19, -95; 1, -75], rotation=-90);
        BondLib.Bonds.eBond SB2 annotation (extent=[-40, -75; -20, -55],
            rotation=90);
        BondLib.Bonds.eBond SLB2 annotation (extent=[40, -75; 60, -55],
            rotation=90);
        Slab Slab1(
          thk=0.1,
          perim=11.7,
          area=17.2,
          c=880,
          dens=2300) annotation (extent=[40,-95; 60,-75],   rotation=270);
        ExDoor SWall(
          lv=5.11,
          area=1,
          sfa=0,
          OH=0,
          W=0) annotation (extent=[-40,-95; -20,-75], rotation=-90);
        ExDoor EWall(
          lv=5.11,
          OH=0,
          W=0,
          area=3,
          sfa=270) annotation (extent=[69,-27; 89,-7], rotation=0);
        ExDoor WWall(
          lv=5.11,
          OH=0,
          W=0,
          area=3,
          sfa=90) annotation (extent=[-50,-27; -70,-7], rotation=0);
        ExDoor SsRoof(
          sfa=0,
          OH=0,
          W=0,
          lv=0.23,
          area=17.2,
          tilt=15,
          absor=0.3) annotation (extent=[-19,23; 1,3], rotation=-90);
        Modelica.Blocks.Interfaces.RealInput u[6] 
          "Connector of Real input signals" 
          annotation (extent=[65,-90; 75,-80], rotation=90);
      equation 
        connect(J0p6_2.BondCon2, CB1.fBondCon1) annotation (points=[0, -40; 0,
              -40], style(color=8));
        connect(CB1.eBondCon1, J0p6_1.BondCon1) annotation (points=[20, -40; 20,
                -40], style(color=8));
        connect(EB1.eBondCon1, J0p6_1.BondCon2) annotation (points=[50, -40; 50,
                -40], style(color=8));
        connect(EWin.BondCon1, EB1.fBondCon1) annotation (points=[70, -40; 70,
              -40], style(color=8));
        connect(CB2.eBondCon1, J0p6_1.BondCon5) annotation (points=[29,-17; 29,
              -21; 29,-21; 29,-25],
                      style(color=8));
        connect(WB1.eBondCon1, J0p6_2.BondCon1) annotation (points=[-30, -40; -
              30, -40], style(color=8));
        connect(WWin.BondCon1, WB1.fBondCon1) annotation (points=[-50, -40; -50,
                -40], style(color=8));
        connect(J0p6_2.BondCon5, WB2.eBondCon1) annotation (points=[-21,-25;
              -21,-17; -30,-17],
                        style(color=8));
        connect(RB1.eBondCon1, J0p6_2.BondCon6) annotation (points=[-9,-17; -9,
              -21; -9,-21; -9,-25],
                      style(color=8));
        connect(SB1.eBondCon1, J0p6_2.BondCon4) annotation (points=[-9, -55; -9,
                -55], style(color=8));
        connect(J0p6_1.BondCon3, CCB1.fBondCon1) annotation (points=[29, -55;
              29, -55], style(color=8));
        connect(CCB1.eBondCon1, Cth1.BondCon1) annotation (points=[29, -75; 29,
                -75], style(color=8));
        connect(SWin.BondCon1, SB1.fBondCon1) annotation (points=[-9, -75; -9,
              -75], style(color=8));
        connect(SB2.eBondCon1, J0p6_2.BondCon3) annotation (points=[-30,-55;
              -21,-55], style(color=8));
        connect(SLB2.eBondCon1, J0p6_1.BondCon4) annotation (points=[50,-55; 41,
              -55],     style(color=8));
        connect(SLB2.fBondCon1, Slab1.BondCon1) annotation (points=[50,-75; 50,
              -75], style(color=8, rgbcolor={192,192,192}));
        connect(SB2.fBondCon1, SWall.BondCon1) annotation (points=[-30,-75; -30,
              -75], style(color=8, rgbcolor={192,192,192}));
        connect(EB2.fBondCon1, EWall.BondCon1) annotation (points=[69,-17; 69,
              -17], style(color=8, rgbcolor={192,192,192}));
        connect(EB2.eBondCon1, J0p6_1.BondCon6) annotation (points=[49,-17; 41,
              -17; 41,-25], style(color=8, rgbcolor={192,192,192}));
        connect(CB2.fBondCon1, BondCon1) annotation (points=[29,3; 29,13; 29,13],
            style(color=8, rgbcolor={192,192,192}));
        connect(WWall.BondCon1, WB2.fBondCon1) annotation (points=[-50,-17; -50,
              -17], style(color=8, rgbcolor={192,192,192}));
        connect(SsRoof.BondCon1, RB1.fBondCon1) annotation (points=[-9,3; -9,3],
            style(color=8, rgbcolor={192,192,192}));
        connect(Slab1.u, SWin.u) annotation (points=[50,-96; 50,-103; -9,-103;
              -9,-96], style(color=74, rgbcolor={0,0,127}));
        connect(SWin.u, SWall.u) annotation (points=[-9,-96; -9,-103; -30,-103;
              -30,-96], style(color=74, rgbcolor={0,0,127}));
        connect(Slab1.u, EWin.u) annotation (points=[50,-96; 50,-103; 96,-103;
              96,-40; 89.95,-40], style(color=74, rgbcolor={0,0,127}));
        connect(EWin.u, EWall.u) annotation (points=[89.95,-40; 96,-40; 96,-17;
              90,-17], style(color=74, rgbcolor={0,0,127}));
        connect(SWall.u, WWin.u) annotation (points=[-30,-96; -30,-103; -77,
              -103; -77,-40; -69.95,-40], style(color=74, rgbcolor={0,0,127}));
        connect(WWin.u, WWall.u) annotation (points=[-69.95,-40; -77,-40; -77,
              -17; -71,-17], style(color=74, rgbcolor={0,0,127}));
        connect(EWall.u, SsRoof.u) annotation (points=[90,-17; 96,-17; 96,30;
              -9,30; -9,24], style(color=74, rgbcolor={0,0,127}));
        connect(EWin.u, u) annotation (points=[89.95,-40; 96,-40; 96,-70; 70,
              -70; 70,-85], style(color=74, rgbcolor={0,0,127}));
      end SunSpace;
      
      model House "Experimental house" 
        constant Modelica.SIunits.CelsiusTemperature T0 = Modelica.Constants.T_zero 
          "Conversion between Centigrade and Kelvin";
        output Modelica.SIunits.CelsiusTemperature T_lr 
          "Temperature of living room";
        output Modelica.SIunits.CelsiusTemperature T_br1 
          "Temperature of bedroom 1";
        output Modelica.SIunits.CelsiusTemperature T_br2 
          "Temperature of bedroom 2";
        output Modelica.SIunits.CelsiusTemperature T_sol 
          "Temperature of solarium";
        output Modelica.SIunits.CelsiusTemperature T_amb "Ambient temperature";
        output Modelica.SIunits.CelsiusTemperature T_ew1 
          "Temperature at outer surface of eastern wall";
        output Modelica.SIunits.CelsiusTemperature T_ew2 
          "Outer internal temperature of eastern wall";
        output Modelica.SIunits.CelsiusTemperature T_ew3 
          "Inner internal temperature of eastern wall";
        output Modelica.SIunits.CelsiusTemperature T_ew4 
          "Temperature at inner surface of eastern wall";
        output Modelica.SIunits.HeatFlowRate Q_lr "Heat flow into living room";
        output Modelica.SIunits.HeatFlowRate Q_br1 "Heat flow into bedroom 1";
        output Modelica.SIunits.HeatFlowRate Q_br2 "Heat flow into bedroom 2";
        output Modelica.SIunits.HeatFlowRate Q_sol "Heat flow into solarium";
        output Modelica.SIunits.HeatFlowRate Q_ac 
          "Cooling flow from air conditioning";
        output Modelica.SIunits.HeatFlowRate Q_nwin 
          "Radiation heat flow through North exposed window";
        output Modelica.SIunits.HeatFlowRate Q_ewall 
          "Radiation heat flow into living room through East exposed wall";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-82, 88; 86, 82], style(color=9)),
            Rectangle(extent=[-74, 82; 78, 74], style(color=9)),
            Rectangle(extent=[-74, 74; 78, 38], style(color=9)),
            Rectangle(extent=[-50, 68; -30, 50], style(color=9)),
            Line(points=[-40, 68; -40, 50], style(color=9)),
            Rectangle(extent=[-52, 68; -28, 66], style(color=0, fillColor=9)),
            Rectangle(extent=[-8, 68; 12, 50], style(color=9)),
            Line(points=[2, 68; 2, 50], style(color=9)),
            Rectangle(extent=[-10, 68; 14, 66], style(color=0, fillColor=9)),
            Rectangle(extent=[52, 68; 72, 50], style(color=9)),
            Line(points=[62, 68; 62, 50], style(color=9)),
            Rectangle(extent=[46, 68; 30, 38], style(color=9)),
            Rectangle(extent=[28, 68; 74, 66], style(color=9, fillColor=9)),
            Rectangle(extent=[28, 40; 48, 38], style(color=9, fillColor=9)),
            Ellipse(extent=[42, 54; 44, 52], style(color=9)),
            Line(points=[-86, 38; 90, 38], style(color=9)),
            Text(
              extent=[-90, 38; -2, 30],
              string="North Elevation",
              style(color=9)),
            Rectangle(extent=[-38, -20; 78, 10], style(color=9)),
            Line(points=[78, 10; 78, 18; -38, 12], style(color=9)),
            Line(points=[-38, -20; -38, 12], style(color=9)),
            Line(points=[78, 18; -46, 12; -46, 18; 78, 26; 78, 18], style(color=
                   9)),
            Line(points=[-38, 10; -76, 2; -76, -2; -38, 6], style(color=9)),
            Rectangle(extent=[-74, -2; -38, -20], style(color=9)),
            Rectangle(extent=[-72, -20; -40, -2], style(color=9)),
            Rectangle(extent=[-60, -20; -50, -2], style(color=9)),
            Rectangle(extent=[-62, -20; -52, -2], style(color=9)),
            Line(points=[-40, -2; -40, 6], style(color=9)),
            Line(points=[-42, -2; -42, 6], style(color=9)),
            Line(points=[-44, -2; -44, 4], style(color=9)),
            Line(points=[-46, -2; -46, 4], style(color=9)),
            Line(points=[-48, -2; -48, 4], style(color=9)),
            Line(points=[-50, -2; -50, 4], style(color=9)),
            Line(points=[-52, -2; -52, 4], style(color=9)),
            Line(points=[-54, -2; -54, 2], style(color=9)),
            Line(points=[-56, -2; -56, 2], style(color=9)),
            Line(points=[-58, -2; -58, 2], style(color=9)),
            Line(points=[-60, -4; -60, 2], style(color=9)),
            Line(points=[-62, -4; -62, 0], style(color=9)),
            Line(points=[-64, -2; -64, 0], style(color=9)),
            Line(points=[-66, -2; -66, 0], style(color=9)),
            Line(points=[-68, -2; -68, 0], style(color=9)),
            Text(
              extent=[4, -20; 92, -28],
              string="East Elevation",
              style(color=9)),
            Line(points=[-86, -20; 88, -20], style(color=9)),
            Rectangle(extent=[-80, -40; 84, -46], style(color=9)),
            Rectangle(extent=[-80, -46; 84, -54], style(color=9)),
            Rectangle(extent=[-72, -54; 76, -90], style(color=9)),
            Rectangle(extent=[-72, -90; 76, -56], style(color=9)),
            Rectangle(extent=[-62, -64; -34, -82], style(color=9)),
            Line(points=[-48, -64; -48, -82], style(color=9)),
            Rectangle(extent=[-64, -62; -32, -64], style(color=9, fillColor=9)),
            Rectangle(extent=[-24, -64; 4, -82], style(color=9)),
            Line(points=[-10, -64; -10, -82], style(color=9)),
            Rectangle(extent=[-26, -62; 6, -64], style(color=9, fillColor=9)),
            Rectangle(extent=[10, -56; 82, -64], style(color=9, fillColor=7)),
            Rectangle(extent=[10, -64; 82, -66], style(color=9, fillColor=7)),
            Rectangle(extent=[10, -66; 82, -68], style(color=9, fillColor=7)),
            Rectangle(extent=[14, -68; 76, -90], style(color=9)),
            Rectangle(extent=[16, -68; 74, -90], style(color=9)),
            Rectangle(extent=[20, -68; 34, -90], style(color=9)),
            Ellipse(extent=[30, -76; 32, -80], style(color=9)),
            Rectangle(extent=[36, -68; 50, -90], style(color=9)),
            Rectangle(extent=[52, -68; 72, -90], style(color=9)),
            Rectangle(extent=[52, -78; 64, -68], style(color=9)),
            Line(points=[56, -78; 56, -68], style(color=9)),
            Line(points=[-86, -90; 88, -90], style(color=9)),
            Text(
              extent=[-88, -90; 0, -98],
              string="South Elevation",
              style(color=9)),
            Rectangle(extent=[12, -90; 78, -92], style(color=9, fillColor=9))),
          Window(
            x=0.55,
            y=0.19,
            width=0.44,
            height=0.65),
          experiment(StopTime=8600),
          experimentSetupOutput(
            doublePrecision=true,
            states=false,
            derivatives=false,
            inputs=false,
            auxiliaries=false),
          Documentation(info="<html>
This is the top-level model of the experimental house.  This is the program that needs to be compiled and simulated. <p>
 
The iconic representation shows schematics of the North, East, and South elevations of the house. <p>

The diagram representation is interesting too.  Here, we overlaid four different models, the models of the four rooms onto the same space on the screen.  The four models have transparent backgrounds, thus they are visible to the top layer, except in places, where different models have lines at the same locations.  The living room model is the top layer.  Therefore, the walls of the living room are grey, whereas those of the other rooms are green (they actually come from the living room model). <p>

The actual \"model\" here consists of the four connections in the colors red, lilac, yellow, and blue, as well as the connections from the six tabular functions. <p>

<font size=6 color=red><b>Enjoy!</b></font>
</html>
"),       Diagram);
        BedRoom1 BedRoom1_1 annotation (extent=[-100, -100; 100, 100]);
        BedRoom2 BedRoom2_1 annotation (extent=[-100, -100; 100, 100]);
        SunSpace SunSpace1 annotation (extent=[-100, -100; 100, 100]);
        LivingRoom LivingRoom1 annotation (extent=[-100, -100; 100, 100]);
        TabularFunctions TabularFunctions1 
          annotation (extent=[-90,-90; -70,-70]);
      equation 
        T_lr = LivingRoom1.Cth1.e + T0;
        T_br1 = BedRoom1_1.Cth1.e + T0;
        T_br2 = BedRoom2_1.Cth1.e + T0;
        T_sol = SunSpace1.Cth1.e + T0;
        T_amb = LivingRoom1.NWin.Temperature1.y + T0;
        T_ew1 = LivingRoom1.EWall.Cth1.e + T0;
        T_ew2 = LivingRoom1.EWall.C1D1.Cth1.e + T0;
        T_ew3 = LivingRoom1.EWall.C1D2.Cth1.e + T0;
        T_ew4 = LivingRoom1.EWall.C1D3.Cth1.e + T0;
        Q_lr = LivingRoom1.Cth1.e * LivingRoom1.Cth1.f;
        Q_br1 = BedRoom1_1.Cth1.e * BedRoom1_1.Cth1.f;
        Q_br2 = BedRoom2_1.Cth1.e * BedRoom2_1.Cth1.f;
        Q_sol = SunSpace1.Cth1.e * SunSpace1.Cth1.f;
        Q_ac = LivingRoom1.Cth1.e * LivingRoom1.AirConditioning1.y;
        Q_nwin = LivingRoom1.Cth1.e * LivingRoom1.NWin.SolarGlass1.y;
        Q_ewall = LivingRoom1.EWall.Cth1.e * LivingRoom1.EWall.SolarRadiation1.y;
        connect(BedRoom2_1.BondCon2, BedRoom1_1.BondCon1) annotation (points=[55,
              74; 34.25,74; 34.25,74; 13.5,74; 13.5,74.5; -25,74.5],
                                style(color=41, thickness=4));
        connect(LivingRoom1.BondCon3, BedRoom1_1.BondCon2) annotation (points=[60,
              83; 60,34; 60,-15; 55.5,-15],
                                style(color=77, thickness=4));
        connect(BedRoom2_1.BondCon1, LivingRoom1.BondCon2) annotation (points=[0,
              -15; 0,-17; 14,84; 17,83],       style(color=81, thickness=4));
        connect(LivingRoom1.BondCon1, SunSpace1.BondCon1) annotation (points=[50,
              -72; 50,13; 29,13],       style(color=49, thickness=4));
        connect(TabularFunctions1.y, LivingRoom1.u) annotation (points=[-69,-80;
              -60,-80; -60,-60; -106,-60; -106,0; -95,0], style(color=74,
              rgbcolor={0,0,127}));
        connect(TabularFunctions1.y, SunSpace1.u) annotation (points=[-69,-80;
              -60,-80; -60,-96; 70,-96; 70,-85], style(color=74, rgbcolor={0,0,
                127}));
        connect(SunSpace1.u, BedRoom1_1.u) annotation (points=[70,-85; 70,-96;
              106,-96; 106,60; 95,60], style(color=74, rgbcolor={0,0,127}));
        connect(BedRoom1_1.u, BedRoom2_1.u) annotation (points=[95,60; 106,60;
              106,106; -20,106; -20,95], style(color=74, rgbcolor={0,0,127}));
      end House;
    end SolarHouse;
    
    package Biosphere "Biosphere 2" 
      annotation (preferedView="info",
        Diagram(
          Rectangle(extent=[-100.4, 100.9; 99.6, -99.1], style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-60.4, 60.9; 59.45, 32.25],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Thermal model of"),
          Text(
            extent=[-74.4, 10.9; 77.65, -34.2],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="without air conditioning"),
          Text(
            extent=[-40.25, 41.55; 41.65, 11.8],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Biosphere 2"),
          Text(
            extent=[-72.1, 24.2; 73.6, -7.1],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="located in Tucson, AZ")),
        Icon(
          Rectangle(extent=[-100.05, 99.1; 99.95, -100.9], style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-60.05, 59.1; 59.8, 30.45],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Thermal model of"),
          Text(
            extent=[-74.05, 9.1; 78, -36],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="without air conditioning"),
          Text(
            extent=[-39.9, 39.75; 42, 10],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="Biosphere 2"),
          Text(
            extent=[-71.75, 22.4; 73.95, -8.9],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="located in Tucson, AZ")),
        Documentation(info="<html>
This package contains a model of the thermal budget without air conditioning of Biosphere 2, an experimental research facility located 35 miles (50 km) north of Tucson, Arizona.  The structure is modeled in its original use, as a <i>materially closed, but energetically open system</i>.  Biosphere 2 is a glass building on 3 acres (12,000 m<sup>2</sup>).  It contains a number of different biomes: <p>

<ol>
<li> Amazonas rain forest biome,
<li> upper and lower Savannah biomes,
<li> salt-water lake biome,
<li> mangrove marsh biome,
<li> Southwestern desert biome, and
<li> agricultural biome.
</ol> <p>

<img src=\"../Images/BG_Ex4.jpg\"> <p>

<img src=\"../Images/BG_Ex5.jpg\"> <p>

Eight people lived in this structure for two years, growing their own food and regenerating their own air.  The purpose of the exercise was to test the feasibility of space colonies with current technology. <p>

The model makes use of five large data tables containing observations of: <p>

<ol>
<li> temperature,
<li> humidity,
<li> cloud cover,
<li> solar radiation, and
<li> wind velocity,
</ol> <p>

stored hourly for an entire year for the location of Oracle, AZ, i.e., the location of the Biospere 2 project. <p>

These data items are stored on an external binary file (<b>Bio_tables.mat</b>) rather than in memory.  This is essential, as Dymola isn't geared to dealing with large data tables in memory.  Dymola essentially converts all matrices to sets of individual scalar variables in the process of compilation.  This is necessary, as the most flexible automatic causality assignment depends on this feature.  Yet, this is highly inefficient, when dealing with large data tables containing many thousands of data points.  We must thus hide these tables from Dymola, and can do so by placing them in external data files. <p>

The model of the hemodynamics had originally been developed by Francisco Luttmann in his Ph.D. dissertation [1].  The original Fortran code was converted to bond graphs and an early version of Dymola  by &Agrave;ngela Nebot during one of her postdoctoral stays in Tucson.  It was converted to graphical form and a modern version of Dymola/Modelica by Fran&ccedil;ois Cellier.  The overall model has been published in [2,3]. <p>

<hr> <p>

<b>Experimentation:</b> <p>

This model uses hours as time units.  If you wish to only experiment with this model, you can simulate it over 4 days by setting the final time to 96 hours.  However, if you wish to reproduce the results published in the 1999 paper [1], you need to simulate the model for a full year, i.e., during 8600 hours. <p>

Contrary to the <i>SolarHouse</i> example that computes internally using <i>seconds</i> as <i>computing time units</i>, accomplishing the <i>display units</i> of <i>hours</i> by means of time scaling, this model uses <i>hours</i> as units of time throughout. <p>

This is a typical American model.  Instead of consistently using <b>SIunits</b> for its computations, it uses units at will, converting between incompatible units by use of numerical conversion factors. <p>

Unfortunately, this model is rather large.  It produces far too much I/O, when all variables are being stored.  Dymola by default stores each and every variable.  Thus, if you wish to simulate over an extended period of time, you should turn off the automatic storage of all variables, and selectively store only those variables that you wish to look at.  This will speed up the simulation dramatically.  At least, you should turn off the storage of the auxiliary variables. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Luttmann, F. (1990), <i>A Dynamic Thermal Model of a Self-sustaining Closed Environment Life Support System</i>, Ph.D. dissertation, Department of Nuclear and Energy Engineering, University of Arizona.
<li> Nebot, A., F.E. Cellier, and F. Mugica (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/ecoeng_bio2.pdf\">Simulation
     of Heat and Humidity Budget of Biosphere 2 without its Air Conditioning</a>,\" <i>Ecological Engineering</i>,
     <b>13</b>, pp. 333-356.
<li> Cellier, F.E. and A. Nebot (2005), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/modelica2005_cellier.pdf\">The
     Modelica Bond Graph Library</a>,\" <i>Proc. 4<sup>th</sup> Modelica Conference</i>, Hamburg, Germany.
</ol>
</html>
"),     uses(Modelica(version="2.1"), BondLib(version="1")),
        version="2",
        conversion(from(version="", script="ConvertFromBiosphere_.mos"), from(
              version="1", script="ConvertFromBiosphere_1.mos")));
      block AmbientTemp "Ambient temperature" 
        extends Modelica.Blocks.Interfaces.SISO;
        constant Real m=0.556 
          "Multiplicative factor for conversion of temperature from Fahrenheit to Centigrade";
        constant Modelica.SIunits.CelsiusTemperature a=-17.78 
          "Additive factor for conversion of temperature from Fahrenheit to Centigrade";
        constant Modelica.SIunits.Temperature T0 = -Modelica.Constants.T_zero 
          "Conversion between Centigrade and Kelvin";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-30, 76; 26, -78]),
            Rectangle(extent=[-6, 60; 2, -54]),
            Ellipse(extent=[-8, -52; 4, -64], style(fillColor=73)),
            Rectangle(extent=[-6, -10; 2, -58], style(fillColor=73)),
            Line(points=[-8, -44; -18, -44]),
            Line(points=[-8, -42; -12, -42]),
            Line(points=[-8, -40; -12, -40]),
            Line(points=[-8, -38; -12, -38]),
            Line(points=[-8, -36; -12, -36]),
            Line(points=[-8, 36; -18, 36]),
            Line(points=[-8, 38; -12, 38]),
            Line(points=[-8, 40; -12, 40]),
            Line(points=[-8, 42; -12, 42]),
            Line(points=[-8, 44; -12, 44]),
            Line(points=[-8, -34; -18, -34]),
            Line(points=[-8, -32; -12, -32]),
            Line(points=[-8, -30; -12, -30]),
            Line(points=[-8, -28; -12, -28]),
            Line(points=[-8, -26; -12, -26]),
            Line(points=[-8, -24; -18, -24]),
            Line(points=[-8, -22; -12, -22]),
            Line(points=[-8, -20; -12, -20]),
            Line(points=[-8, -18; -12, -18]),
            Line(points=[-8, -16; -12, -16]),
            Line(points=[-8, -14; -18, -14]),
            Line(points=[-8, -12; -12, -12]),
            Line(points=[-8, -10; -12, -10]),
            Line(points=[-8, -8; -12, -8]),
            Line(points=[-8, -6; -12, -6]),
            Line(points=[-8, -4; -18, -4]),
            Line(points=[-8, -2; -12, -2]),
            Line(points=[-8, 0; -12, 0]),
            Line(points=[-8, 2; -12, 2]),
            Line(points=[-8, 4; -12, 4]),
            Line(points=[-8, 6; -18, 6]),
            Line(points=[-8, 8; -12, 8]),
            Line(points=[-8, 10; -12, 10]),
            Line(points=[-8, 12; -12, 12]),
            Line(points=[-8, 14; -12, 14]),
            Line(points=[-8, 16; -18, 16]),
            Line(points=[-8, 18; -12, 18]),
            Line(points=[-8, 20; -12, 20]),
            Line(points=[-8, 22; -12, 22]),
            Line(points=[-8, 24; -12, 24]),
            Line(points=[-8, 26; -18, 26]),
            Line(points=[-8, 28; -12, 28]),
            Line(points=[-8, 30; -12, 30]),
            Line(points=[-8, 32; -12, 32]),
            Line(points=[-8, 34; -12, 34])),
          Diagram(
            Text(
              extent=[4, -18; 84, -24],
              string="Temperature in Centigrade",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[70, -62; 102, -68],
              string="Temperature",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[70, -68; 102, -74],
              string="in Kelvin",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-54,-22; -22,-28],
              string="Temperature",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-54,-28; -22,-34],
              string="in Fahrenheit",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-114,10; -82,4],
              string="Time",
              style(color=10, rgbcolor={135,135,135}))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This block computes the ambient temperature as a function of time for the location of Oracle, Arizona, i.e., for the location of Biosphere 2.  The model operates on a huge table containing temperature values of an entire year stored hourly in <sup>o</sup>F.  The model then converts the interpolated temperature value to <sup>o</sup>C, and finally to <sup>o</sup>K, which are the unit needed by the model. <p>

The model reads the data from an external file.  This is essential, as Dymola isn't geared to dealing with large data tables in memory.  Dymola essentially converts all matrices to sets of individual scalar variables in the process of compilation.  This is necessary, as the most flexible automatic causality assignment depends on this feature.  Yet, this is highly inefficient, when dealing with large data tables containing many thousands of data points.  We must hide these tables from Dymola, and can do so by placing them in external data files.
</html>
"));
      public 
        Modelica.Blocks.Math.Gain CelsiusConversion(k=m)    annotation (extent=[-30,
                -20; 10, 20]);
        Modelica.Blocks.Math.Add Add1 annotation (extent=[30, -8; 70, 32]);
        Modelica.Blocks.Sources.Constant Offset(k=a)   annotation (extent=[-30, 40;
                10, 80]);
        Modelica.Blocks.Math.Add Add2 annotation (extent=[30, -80; 70, -40]);
        Modelica.Blocks.Sources.Constant KelvinConversion(k=T0) 
                                                            annotation (extent=[-40,
                -92; 0, -52]);
        Modelica.Blocks.Tables.CombiTable1Ds TempTable(
          tableOnFile=true,
          tableName="temp_table",
          fileName="Bio_tables.mat") 
                                 annotation (extent=[-88,-20; -48,20]);
      equation 
        connect(CelsiusConversion.y, Add1.u2)             annotation (points=[12,0;
              26,0], style(color=74, rgbcolor={0,0,135}));
        connect(Offset.y,Add1.u1)             annotation (points=[12,60; 18,60; 18,
              24; 26,24], style(color=74, rgbcolor={0,0,135}));
        connect(Add1.y,Add2.u1)             annotation (points=[72,12; 80,12; 80,
              -26; 10,-26; 10,-48; 26,-48], style(color=74, rgbcolor={0,0,135}));
        connect(KelvinConversion.y, Add2.u2)      annotation (points=[2,-72; 26,-72], style(
              color=74, rgbcolor={0,0,135}));
        connect(Add2.y,y)              annotation (points=[72,-60; 90,-60; 90,0;
              110,0], style(color=74, rgbcolor={0,0,135}));
        connect(TempTable.u, u) 
          annotation (points=[-92,0; -120,0], style(color=74, rgbcolor={0,0,127}));
        connect(TempTable.y[1], CelsiusConversion.u) 
          annotation (points=[-46,0; -34,0], style(color=74, rgbcolor={0,0,127}));
      end AmbientTemp;
      
      block NightSkyTemp "Apparent night sky temperature" 
        extends Modelica.Blocks.Interfaces.SISO;
        constant Real m=0.556 
          "Multiplicative factor for conversion of temperature from Fahrenheit to Centigrade";
        constant Modelica.SIunits.CelsiusTemperature a=-17.78 
          "Additive factor for conversion of temperature from Fahrenheit to Centigrade";
        constant Modelica.SIunits.Temperature T0 = -Modelica.Constants.T_zero 
          "Conversion between Centigrade and Kelvin";
        output Modelica.SIunits.CelsiusTemperature tdp 
          "Wet bulb (dew point) temperature";
        output Real ccf(unit="Btu/(h.sqft)") "Diffuse radiation";
        output Real eps1 "Emissivity of clear sky";
        output Real eps2 "Emissivity of sky with cloud cover";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-30, 76; 26, -78]),
            Rectangle(extent=[-6, 60; 2, -54]),
            Ellipse(extent=[-8, -52; 4, -64], style(fillColor=73)),
            Rectangle(extent=[-6, -10; 2, -58], style(fillColor=73)),
            Line(points=[-8, -44; -18, -44]),
            Line(points=[-8, -42; -12, -42]),
            Line(points=[-8, -40; -12, -40]),
            Line(points=[-8, -38; -12, -38]),
            Line(points=[-8, -36; -12, -36]),
            Line(points=[-8, 36; -18, 36]),
            Line(points=[-8, 38; -12, 38]),
            Line(points=[-8, 40; -12, 40]),
            Line(points=[-8, 42; -12, 42]),
            Line(points=[-8, 44; -12, 44]),
            Line(points=[-8, -34; -18, -34]),
            Line(points=[-8, -32; -12, -32]),
            Line(points=[-8, -30; -12, -30]),
            Line(points=[-8, -28; -12, -28]),
            Line(points=[-8, -26; -12, -26]),
            Line(points=[-8, -24; -18, -24]),
            Line(points=[-8, -22; -12, -22]),
            Line(points=[-8, -20; -12, -20]),
            Line(points=[-8, -18; -12, -18]),
            Line(points=[-8, -16; -12, -16]),
            Line(points=[-8, -14; -18, -14]),
            Line(points=[-8, -12; -12, -12]),
            Line(points=[-8, -10; -12, -10]),
            Line(points=[-8, -8; -12, -8]),
            Line(points=[-8, -6; -12, -6]),
            Line(points=[-8, -4; -18, -4]),
            Line(points=[-8, -2; -12, -2]),
            Line(points=[-8, 0; -12, 0]),
            Line(points=[-8, 2; -12, 2]),
            Line(points=[-8, 4; -12, 4]),
            Line(points=[-8, 6; -18, 6]),
            Line(points=[-8, 8; -12, 8]),
            Line(points=[-8, 10; -12, 10]),
            Line(points=[-8, 12; -12, 12]),
            Line(points=[-8, 14; -12, 14]),
            Line(points=[-8, 16; -18, 16]),
            Line(points=[-8, 18; -12, 18]),
            Line(points=[-8, 20; -12, 20]),
            Line(points=[-8, 22; -12, 22]),
            Line(points=[-8, 24; -12, 24]),
            Line(points=[-8, 26; -18, 26]),
            Line(points=[-8, 28; -12, 28]),
            Line(points=[-8, 30; -12, 30]),
            Line(points=[-8, 32; -12, 32]),
            Line(points=[-8, 34; -12, 34])),
          Diagram(
            Rectangle(extent=[-30,-36; 20,-86],   style(color=61)),
            Line(points=[-58,-60; -30,-60],   style(color=61)),
            Line(points=[2,4; 10,4; 10,-36],  style(color=61)),
            Line(points=[20,-60; 28,-60; 28,0; 36,0],     style(color=57)),
            Text(
              extent=[-20,-46; 8,-54],
              string="Night Sky",
              style(color=57)),
            Text(
              extent=[-24,-56; 14,-64],
              string="Temperature",
              style(color=57)),
            Text(
              extent=[-24,-66; 14,-74],
              string="Computation",
              style(color=57)),
            Text(
              extent=[-84,48; -54,42],
              string="Dry bulb",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-84,-12; -54,-18],
              style(color=10, rgbcolor={135,135,135}),
              string="Wet bulb"),
            Text(
              extent=[-84,-18; -54,-24],
              style(color=10, rgbcolor={135,135,135}),
              string="temperature"),
            Text(
              extent=[-84,42; -54,36],
              style(color=10, rgbcolor={135,135,135}),
              string="temperature"),
            Text(
              extent=[-58,-54; -28,-60],
              style(color=10, rgbcolor={135,135,135}),
              string="Diffuse"),
            Text(
              extent=[-58,-60; -28,-66],
              style(color=10, rgbcolor={135,135,135}),
              string="radiation"),
            Text(
              extent=[-84,36; -52,30],
              string="in Fahrenheit",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-84,-24; -52,-30],
              string="in Fahrenheit",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[16,54; 46,48],
              string="Dry bulb",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[20,48; 50,42],
              style(color=10, rgbcolor={135,135,135}),
              string="temperature"),
            Text(
              extent=[20,42; 52,36],
              style(color=10, rgbcolor={135,135,135}),
              string="in Centigrade"),
            Text(
              extent=[-16,-12; 14,-18],
              style(color=10, rgbcolor={135,135,135}),
              string="Wet bulb"),
            Text(
              extent=[-20,-18; 10,-24],
              style(color=10, rgbcolor={135,135,135}),
              string="temperature"),
            Text(
              extent=[-22,-24; 10,-30],
              style(color=10, rgbcolor={135,135,135}),
              string="in Centigrade"),
            Text(
              extent=[26,-50; 56,-56],
              style(color=10, rgbcolor={135,135,135}),
              string="Emissivity"),
            Text(
              extent=[22,-56; 52,-62],
              style(color=10, rgbcolor={135,135,135}),
              string="factor"),
            Text(
              extent=[92,-14; 124,-20],
              style(color=10, rgbcolor={135,135,135}),
              string="temperature"),
            Text(
              extent=[88,-20; 120,-26],
              string="in Kelvin",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[88,-8; 120,-14],
              style(color=10, rgbcolor={135,135,135}),
              string="Night sky"),
            Text(
              extent=[-114,10; -82,4],
              string="Time",
              style(color=10, rgbcolor={135,135,135}))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The model also requires the apparent temperature of the night sky to compute the radiation of the structure to the sky during the night.  The night sky temperature depends on the ambient temperature, the humidity, and the cloud cover.  All three are retrieved from huge data tables stored on an external binary file. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Berdahl, P. and R. Fromberg (1982), \"The Thermal Radiance of Clear Skies,\" <i>Solar Energy</i>, <b>29</b>(4), pp. 299-314.
<li>Kimbal, B.A. and S.B. Idso (1982), \"A Model of Thermal Radiation from Partly Claudy Skies,\" <i>Water Resources Research</i>, <b>18</b>(4), pp. 931-936.
</ol>
</html>
"));
      public 
        Modelica.Blocks.Math.Gain CelsiusConversion2(k=m)   annotation (extent=[-50,
                -10; -30, 10]);
        Modelica.Blocks.Math.Gain CelsiusConversion1(k=m)    annotation (extent=[-
              50, 50; -30, 70]);
        Modelica.Blocks.Sources.Constant Offset(k=a)   annotation (extent=[-50, 20;
                -30, 40]);
        Modelica.Blocks.Math.Add Add1 annotation (extent=[-20, 44; 0, 64]);
        Modelica.Blocks.Math.Add Add2 annotation (extent=[-20, -6; 0, 14]);
        Modelica.Blocks.Math.Add Add3 annotation (extent=[70, -10; 90, 10]);
        Modelica.Blocks.Sources.Constant KelvinConversion(k=T0) 
                                                            annotation (extent=[40,
                -40; 60, -20]);
        Modelica.Blocks.Math.Product Product1 annotation (extent=[40, -4; 60, 16]);
        Modelica.Blocks.Tables.CombiTable1Ds TempTable(
          tableOnFile=true,
          tableName="temp_table",
          fileName="Bio_tables.mat") 
                                 annotation (extent=[-80,50; -60,70]);
        Modelica.Blocks.Tables.CombiTable1Ds WetTable(
          tableOnFile=true,
          tableName="wet_table",
          fileName="Bio_tables.mat") 
                                annotation (extent=[-80,-10; -60,10]);
        Modelica.Blocks.Tables.CombiTable1Ds CloudTable(
          tableOnFile=true,
          tableName="cloud_table",
          fileName="Bio_tables.mat") 
                                  annotation (extent=[-80,-70; -60,-50]);
      equation 
        connect(CelsiusConversion1.y, Add1.u1)             annotation (points=[-29,
              60; -22,60], style(color=74, rgbcolor={0,0,135}));
        connect(Offset.y,Add1.u2)             annotation (points=[-29,30; -26,30;
              -26,48; -22,48], style(color=74, rgbcolor={0,0,135}));
        connect(CelsiusConversion2.y,Add2.u2)             annotation (points=[-29,0;
              -25.5,0; -25.5,-2; -22,-2], style(color=74, rgbcolor={0,0,135}));
        connect(Offset.y,Add2.u1)             annotation (points=[-29,30; -26,30;
              -26,10; -22,10], style(color=74, rgbcolor={0,0,135}));
        connect(Product1.y,Add3.u1)             annotation (points=[61,6; 68,6], style(
              color=74, rgbcolor={0,0,135}));
        connect(Add1.y,Product1.u1)             annotation (points=[1,54; 20,54; 20,
              12; 38,12], style(color=74, rgbcolor={0,0,135}));
        connect(KelvinConversion.y, Add3.u2)      annotation (points=[61,-30; 64,
              -30; 64,-6; 68,-6], style(color=74, rgbcolor={0,0,135}));
        connect(Add3.y,y)              annotation (points=[91,0; 110,0], style(
              color=74, rgbcolor={0,0,135}));
        tdp = Add2.y;
        ccf = CloudTable.y[1];
        eps1 = 0.8 + 0.00366*tdp;
        eps2 = eps1 + (1 - eps1)*ccf;
        Product1.u2 = eps2^(1/4);
        connect(TempTable.y[1], CelsiusConversion1.u)  annotation (points=[-59,60;
              -52,60], style(color=74, rgbcolor={0,0,127}));
        connect(u, TempTable.u) annotation (points=[-120,0; -90,0; -90,60; -82,60],
            style(color=74, rgbcolor={0,0,127}));
        connect(WetTable.u, u) 
          annotation (points=[-82,0; -120,0], style(color=74, rgbcolor={0,0,127}));
        connect(WetTable.y[1],CelsiusConversion2. u) 
          annotation (points=[-59,0; -52,0], style(color=74, rgbcolor={0,0,127}));
        connect(CloudTable.u, u) annotation (points=[-82,-60; -90,-60; -90,0; -120,
              0], style(color=74, rgbcolor={0,0,127}));
      end NightSkyTemp;
      
      model SolarInput "Solar radiation" 
        extends Modelica.Blocks.Interfaces.SISO;
        constant Real m=11.247 
          "Multiplicative factor for conversion of radiation from Btu/(h.sqft) to kJ/(h.m2)";
        constant Real a=0 
          "Additive factor for conversion of radiation from Btu/(h.sqft) to kJ/(h.m2)";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Ellipse(extent=[-24, 24; 26, -26]),
            Polygon(points=[-66, 32; -18, 16; -36, 66; -6, 24; -6, 22; 2, 76; 8, 22;
                    44, 64; 20, 14; 70, 36; 26, 4; 78, -2; 26, -8; 70, -36; 18, -18;
                    38, -68; 8, -26; 2, -78; -6, -24; -38, -66; -18, -20; -64, -38;
                    -22, -8; -74, -2; -24, 2; -66, 32], style(color=41, fillColor=
                    49)),
            Line(points=[16, -52; 22, -92], style(color=41, thickness=2)),
            Line(points=[44, -42; 78, -78], style(color=41, thickness=2)),
            Line(points=[62, -16; 92, -24], style(color=41, thickness=2)),
            Line(points=[60, -22; 90, -34], style(color=41, thickness=2)),
            Line(points=[48, -36; 88, -62], style(color=41, thickness=2)),
            Line(points=[38, -52; 64, -90], style(color=41, thickness=2)),
            Line(points=[22, -54; 36, -90], style(color=41, thickness=2))),
          Documentation(info="<html>
The model also requires the total solar input available to the Biosphere 2 structure.  Rather than using spherical trigonometry to compute the available radiation, also this information is simply pulled out of a huge table, where data have been stored for an entire year.
</html>
"),       Diagram(
            Text(
              extent=[-110,10; -78,4],
              string="Time",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-58,-24; -22,-30],
              style(color=10, rgbcolor={135,135,135}),
              string="Global horizontal"),
            Text(
              extent=[-56,-30; -26,-36],
              style(color=10, rgbcolor={135,135,135}),
              string="radiation"),
            Text(
              extent=[-58,-36; -22,-42],
              style(color=10, rgbcolor={135,135,135}),
              string="[Btu/(h.sqft)]"),
            Text(
              extent=[72,-6; 108,-12],
              style(color=10, rgbcolor={135,135,135}),
              string="Global horizontal"),
            Text(
              extent=[74,-12; 104,-18],
              style(color=10, rgbcolor={135,135,135}),
              string="radiation"),
            Text(
              extent=[72,-18; 108,-24],
              style(color=10, rgbcolor={135,135,135}),
              string="[kJ/(h.m2)]")));
      public 
        Modelica.Blocks.Math.Gain UnitConversion(k=m)       annotation (extent=[-30,
                -20; 10, 20]);
        Modelica.Blocks.Math.Add Add1 annotation (extent=[30, -8; 70, 32]);
        Modelica.Blocks.Sources.Constant Offset(k=a)   annotation (extent=[-30, 40;
                10, 80]);
        Modelica.Blocks.Tables.CombiTable1Ds RadiationTable(
          tableOnFile=true,
          tableName="rad_table",
          fileName="Bio_tables.mat") 
                                annotation (extent=[-86,-20; -46,20]);
      equation 
        connect(UnitConversion.y, Add1.u2)                annotation (points=[12,0;
              26,0], style(color=74, rgbcolor={0,0,135}));
        connect(Offset.y,Add1.u1)             annotation (points=[12,60; 18,60; 18,
              24; 26,24], style(color=74, rgbcolor={0,0,135}));
        connect(Add1.y,y)              annotation (points=[72,12; 80,12; 80,0; 110,
              0], style(color=74, rgbcolor={0,0,135}));
        connect(RadiationTable.y[1], UnitConversion.u) 
          annotation (points=[-44,0; -34,0], style(color=74, rgbcolor={0,0,135}));
        connect(RadiationTable.u, u) 
          annotation (points=[-90,0; -120,0], style(color=74, rgbcolor={0,0,127}));
      end SolarInput;
      
      model WindVelocity "Wind velocity" 
        extends Modelica.Blocks.Interfaces.SISO;
        constant Real m=3600 
          "Multiplicative factor for converting from m/s to m/h";
        constant Real a=0 "Additive factor for converting from m/s to m/h";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-100, -54; 100, -100], style(color=9, fillColor=53)),
            Polygon(points=[-24, -68; -14, -44; -10, -30; -10, -20; -2, -20; -2, -
                  30; 2, -44; 12, -70; -24, -70; -24, -68], style(color=9,
                  fillColor=46)),
            Polygon(points=[-20, -20; -22, 6; -20, 32; -10, 52; 6, 74; 30, 90; 24,
                  62; 16, 36; 14, 2; 8, -20; -20, -20], style(color=9, fillColor=62)),
            Line(points=[-76, 64; -14, 64], style(thickness=2)),
            Line(points=[-56, 56; -34, 56], style(thickness=2)),
            Line(points=[-68, 48; -22, 48], style(thickness=2))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The model also requires the wind velocity to compute the convection of the boundary layer of air surrounding the dome of the Biosphere 2 structure.  Also this information has been stored for an entire year, and is simply pulled out of a huge data table.
</html>
"),       Diagram(
            Text(
              extent=[-110,8; -78,2],
              string="Time",
              style(color=10, rgbcolor={135,135,135})),
            Text(
              extent=[-56,-34; -20,-40],
              style(color=10, rgbcolor={135,135,135}),
              string="Wind velocity"),
            Text(
              extent=[-58,-40; -22,-46],
              style(color=10, rgbcolor={135,135,135}),
              string="[m/s]"),
            Text(
              extent=[70,-8; 106,-14],
              style(color=10, rgbcolor={135,135,135}),
              string="Wind velocity"),
            Text(
              extent=[70,-14; 106,-20],
              style(color=10, rgbcolor={135,135,135}),
              string="[m/h]")));
      public 
        Modelica.Blocks.Math.Gain UnitConversion(k=m)       annotation (extent=[-30,
              -32; 10,8]);
        Modelica.Blocks.Math.Add Add1 annotation (extent=[30,-20; 70,20]);
        Modelica.Blocks.Sources.Constant Offset(k=a)   annotation (extent=[-30, 40;
                10, 80]);
        Modelica.Blocks.Tables.CombiTable1Ds VelocityTable(
          tableOnFile=true,
          tableName="vel_table",
          fileName="Bio_tables.mat") 
                                annotation (extent=[-86,-32; -46,8]);
      equation 
        connect(Offset.y,Add1.u1)             annotation (points=[12,60; 18,60; 18,
              12; 26,12], style(color=74, rgbcolor={0,0,135}));
        connect(Add1.y,y)              annotation (points=[72,0; 110,0], style(
              color=74, rgbcolor={0,0,135}));
        connect(UnitConversion.y, Add1.u2)                annotation (points=[12,
              -12; 26,-12], style(color=74, rgbcolor={0,0,135}));
        connect(VelocityTable.y[1], UnitConversion.u)     annotation (points=[-44,
              -12; -34,-12], style(color=74, rgbcolor={0,0,135}));
        connect(VelocityTable.u, u) annotation (points=[-90,-12; -96,-12; -96,0;
              -120,0], style(color=74, rgbcolor={0,0,127}));
      end WindVelocity;
      
      block Glass 
        "Absorption, transmission, reflection of solar input at glass panels" 
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Real alph=0.6 "Portion of absorbed power";
        parameter Real tau=0.2 "Portion of transmitted power";
        parameter Real refl = 1 - alph - tau "Portion of reflected power";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Line(points=[-46, -52; -46, 24], style(color=57, thickness=2)),
            Line(points=[-46, 24; 32, 56], style(color=57, thickness=2)),
            Line(points=[-46, -52; 32, -20], style(color=61, thickness=2)),
            Line(points=[32, -20; 32, 56], style(color=57, thickness=2)),
            Line(points=[-8, 76; -22, 42], style(color=41, thickness=2)),
            Line(points=[-4, 72; -10, 46], style(color=41, thickness=2)),
            Line(points=[2, 76; 2, 52], style(color=41, thickness=2)),
            Line(points=[0, -34; 0, -100], style(color=41, thickness=2)),
            Line(points=[32, 0; 100, 0], style(color=41, thickness=2)),
            Line(points=[-46, 0; -100, 0], style(color=41, thickness=2)),
            Text(
              extent=[4,-54; 50,-74],
              string="Transmission",
              style(color=41)),
            Text(
              extent=[48, 14; 84, 0],
              string="Absorption",
              style(color=41)),
            Text(
              extent=[-88, 14; -54, 0],
              string="Reflection",
              style(color=41))),
          Diagram(
            Line(points=[32, -20; 32, 56], style(color=57, thickness=2)),
            Line(points=[-46, -52; 32, -20], style(color=61, thickness=2)),
            Line(points=[-46, -52; -46, 24], style(color=57, thickness=2)),
            Line(points=[-46, 24; 32, 56], style(color=57, thickness=2)),
            Line(points=[-8, 76; -22, 42], style(color=41, thickness=2)),
            Line(points=[-4, 72; -10, 46], style(color=41, thickness=2)),
            Line(points=[2, 76; 2, 52], style(color=41, thickness=2)),
            Text(
              extent=[48, 14; 84, 0],
              string="Absorption",
              style(color=41)),
            Line(points=[32, 0; 100, 0], style(color=41, thickness=2)),
            Text(
              extent=[4, -54; 46, -70],
              string="Transmission",
              style(color=41)),
            Line(points=[0, -34; 0, -100], style(color=41, thickness=2)),
            Text(
              extent=[-88, 14; -54, 0],
              string="Reflection",
              style(color=41)),
            Line(points=[-46, 0; -100, 0], style(color=41, thickness=2))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This model computes in a global fashion, how much of the solar input gets absorbed by the glass panes, how much gets transmitted through, and how much gets reflected. <p>

Since the glass panes have many different orientations, a physical model would be rather computation intensive.  Thus, the model goes another route.  It estimaes the three quantities globally and on average. <p>

<hr> <p>

<pre>
Parameters:

  alph:  Portion of absorbed power (default value = 0.2)

  tau:   Portion of transmitted power (default value = 0.6)

  refl:  Portion of reflected power (default value = 1 - alph - tau)
</pre>
</html>
"));
        Modelica.Blocks.Interfaces.RealInput Pin(redeclare type SignalType = 
              Modelica.SIunits.Power) "Incoming power" 
                                                  annotation (extent=[-10, 80; 10,
              100], rotation=-90);
        Modelica.Blocks.Interfaces.RealOutput Prefl(redeclare type SignalType 
            = Modelica.SIunits.Power) "Reflected power" 
                                                    annotation (extent=[-100, -10;
              -120, 10]);
        Modelica.Blocks.Interfaces.RealOutput Ptrans(redeclare type SignalType 
            = Modelica.SIunits.Power) "Transmitted power" 
                                                    annotation (extent=[10, -120; -
              10, -100], rotation=-90);
        Modelica.Blocks.Interfaces.RealOutput Pabs(redeclare type SignalType = 
              Modelica.SIunits.Power) "Absorbed power" 
                                                    annotation (extent=[100, -10;
              120, 10]);
      equation 
        Pabs = alph*Pin;
        Ptrans = tau*Pin;
        Prefl = refl*Pin;
      end Glass;
      
      block Absorption "Distribution of solar input" 
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Real apix=0.4 "Portion of Biosphere covered by pond";
        parameter Real avix=0.4 "Portion of Biosphere covered by vegetation";
        parameter Real laix=2 "Leaf area index";
        parameter Real refl=0.05 "Reflectance of pond";
        parameter Real vkx=0.6 "Canopy light extinction coefficient";
        parameter Real rhos=0.13 "Reflectance of soil";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Line(points=[-46, 78; -60, 44], style(color=41, thickness=2)),
            Line(points=[-42, 74; -48, 48], style(color=41, thickness=2)),
            Line(points=[-36, 78; -36, 54], style(color=41, thickness=2)),
            Line(points=[-30, 84; -26, 62], style(color=41, thickness=2)),
            Line(points=[-60, 36; -60, 0; -100, 0], style(color=41, thickness=2)),
            Line(points=[-48, 36; -48, -40; 0, -40; 0, -100], style(color=41,
                  thickness=2)),
            Line(points=[-36, 36; -36, 0; 100, 0], style(
                color=1,
                rgbcolor={255,0,0},
                thickness=2)),
            Line(points=[-26, 36; -26, 20; 40, 20; 40, 100], style(color=41,
                  thickness=2)),
            Text(
              extent=[-96, 12; -70, 2],
              string="Soil",
              style(color=41)),
            Text(
              extent=[4, -66; 40, -78],
              string="Vegetation",
              style(color=1, rgbcolor={255,0,0})),
            Text(
              extent=[58, 12; 84, 2],
              string="Pond",
              style(color=41)),
            Text(
              extent=[42, 76; 78, 64],
              string="Reflected",
              style(color=1, rgbcolor={255,0,0}))),
          Icon(
            Line(points=[-46, 78; -60, 44], style(color=41, thickness=2)),
            Line(points=[-42, 74; -48, 48], style(color=41, thickness=2)),
            Line(points=[-36, 78; -36, 54], style(color=41, thickness=2)),
            Line(points=[-30, 84; -26, 62], style(color=41, thickness=2)),
            Line(points=[-60, 36; -60, 0; -100, 0], style(color=41, thickness=2)),
            Line(points=[-48, 36; -48, -40; 0, -40; 0, -100], style(color=41,
                  thickness=2)),
            Line(points=[-36, 36; -36, 0; 100, 0], style(color=45, thickness=2)),
            Line(points=[-26, 36; -26, 20; 40, 20; 40, 100], style(color=41,
                  thickness=2)),
            Text(
              extent=[-96, 12; -70, 2],
              string="Soil",
              style(color=41)),
            Text(
              extent=[4, -66; 40, -78],
              string="Vegetation",
              style(color=45)),
            Text(
              extent=[58, 12; 84, 2],
              string="Pond",
              style(color=41)),
            Text(
              extent=[42, 76; 78, 64],
              string="Reflected",
              style(color=45))),
          Documentation(info="<html>
This model computes, how much of the transmitted solar input gets absorbed by the soil, by the vegetation, and by the pond, and how much gets reflected back out. <p>

<hr> <p>

<pre>
Parameters:

  apix:  Portion of Biosphere covered by pond (default value = 0.4)

  avix:  Portion of Biosphere covered by vegetation (default value = 0.4)

  laix:  Leaf area index (default value = 2)

  refl:  Reflectance of pond (default value = 0.05)

  vkx:   Canopy light extinction coefficient (default value = 0.6)

  rhos:  Reflectance of soil (default value = 0.13)
</pre>
</html>
"));
        Modelica.Blocks.Interfaces.RealOutput Psoil "Power to soil" 
                                                    annotation (extent=[-100, -10;
              -120, 10]);
        Modelica.Blocks.Interfaces.RealOutput Ppond "Power to pond" 
                                                    annotation (extent=[100, -10;
              120, 10]);
        Modelica.Blocks.Interfaces.RealOutput Pveg "Power to vegetation" 
                                                    annotation (extent=[10, -120; -
              10, -100], rotation=-90);
        Modelica.Blocks.Interfaces.RealInput Pin "Incoming power" 
                                                  annotation (extent=[-50, 80; -30,
                100], rotation=-90);
        Modelica.Blocks.Interfaces.RealOutput Prefl "Reflected power" 
                                                    annotation (extent=[50, 120; 30,
                100], rotation=-90);
      equation 
        Ppond = apix*(1 - refl)*Pin;
        Pveg = avix*(1 - exp(-vkx*laix))*Pin;
        Psoil = (1 - rhos)*(avix*exp(-vkx*laix) + (1 - avix - apix))*Pin;
        Prefl = Pin - Ppond - Pveg - Psoil;
      end Absorption;
      
      model CV "Convection" 
        extends BondLib.Interfaces.TwoPort;
        parameter Real G0(unit="kJ/(h.K)") = 1 "Thermal conductance";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CV",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Documentation(info="<html>
The convection model is the regular directed heat exchanger model, <b>HEr</b>, of the bond graph library.  However, we have duplicated the model here under a different name, in order to be able to provide meaningful documentation for its use. <p>

Convection, in general, requires a much more complex model, because mass and entropy get moved around together.  However, in the current implementation of the model, we ignored the air conditioning.  Thus air doesn't get moved around, except for the purpose of potential equilibration.  In that case, a much simplified version of the model, namely that of a simple heat exchanger can be used. <p>

<hr> <p>

<pre>
Parameters:

  G0:  Thermal conductance (default value = 1 kJ/(h.K))
</pre>
</html>
"));
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-50, -10; -30, 10]);
        BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-10, 10; 10, -10]);
        BondLib.Bonds.fBond B1 annotation (extent=[-30, -10; -10, 10]);
        BondLib.Bonds.eBond B2 annotation (extent=[10, -10; 30, 10]);
        BondLib.Junctions.J0p3 J0p3_2 annotation (extent=[30, 10; 50, -10]);
        BondLib.Bonds.eBond B4 annotation (extent=[30, 10; 50, 30], rotation=-90);
        BondLib.Bonds.fBond B3 annotation (extent=[-10, 10; 10, 30], rotation=90);
        BondLib.Thermal.GSth GSth1(G0=G0) annotation (extent=[10, 30; 30, 50]);
      equation 
        connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-10, 0; -10, 0],
              style(color=8));
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-30, 0; -30, 0],
              style(color=8));
        connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[10, 0; 10, 0],
            style(color=8));
        connect(B2.eBondCon1, J0p3_2.BondCon1) annotation (points=[30, 0; 30, 0],
            style(color=8));
        connect(B4.eBondCon1, J0p3_2.BondCon3) annotation (points=[40, 10; 40, 10],
              style(color=8));
        connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,10;
              -6.12303e-016,10],
            style(color=8));
        connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -50,0],
            style(color=8));
        connect(J0p3_2.BondCon2, BondCon2) annotation (points=[50,0; 100,0],  style(
             color=8));
        connect(B3.fBondCon1, GSth1.BondCon1) annotation (points=[6.12303e-016,
              30; 6.12303e-016,40; 10,40],
                     style(color=8));
        connect(GSth1.BondCon2, B4.fBondCon1) annotation (points=[30, 40; 40, 40;
              40, 30], style(color=8));
      end CV;
      
      model CD "Heat conduction in the soil" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.Area ag "Ground area of Biosphere";
        parameter Real apix "Portion of Biosphere covered by pond";
        parameter Real lambda(unit="kJ/(h.m2.K)") 
          "Specific thermal conductance of ground";
        parameter Real G0(unit="kJ/(h.K)") = ag*(1 - apix)*lambda 
          "Thermal conductance of ground layer";
        parameter Real gamma0(unit="kJ/(m2.K)") 
          "Specific heat capacity of ground";
        parameter Real gamma(unit="kJ/K") = ag*(1 - apix)*gamma0 
          "Heat capacity of ground layer";
        parameter Modelica.SIunits.Temperature T0 "Initial temperature";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CD",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The heat conduction model used here is the directed heat exchanger model of the bond graph library with an additional heat capacity built in at the output node.  In this way, the soil can be subdivided into layers, with heat conduction models being cascaded without a need for attaching heat capacitors externally, except at the very first node. <p>

Heat capacitors were attached only at one side, because otherwise, higher index models result when such models get cascaded.  Although Modelica is perfectly capable of reducing the perturbation index on its own, this would only have slowed down the compilation of the model unnecessarily. <p>

<hr> <p>

<pre>
Parameters:

  ag:      Ground area of Biosphere (default value = 2368.9 m<sup>2</sup>)

  apix:    Portion of Biosphere covered by pond (default value = 0.4)

  lambda:  Specific thermal conductance of ground (no default value) [kJ/(h.m<sup>2</sup>.K)]

  G0:      Thermal conductance of ground layer (default value = ag*(1 - apix)*lambda kJ/(h.K))

  gamma0:  Specific heat capacity of ground (no default value) [kJ/(m<sup>2</sup>.K)]

  gamma:   Heat capacity of ground layer (default value = ag*(1 - apix)*gamma0 kJ/K))

  T0:      Initial temperature (no default value) [K]
</pre>
</html>
"));
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-50, -10; -30, 10]);
        BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-10, 10; 10, -10]);
        BondLib.Bonds.fBond B1 annotation (extent=[-30, -10; -10, 10]);
        BondLib.Bonds.eBond B2 annotation (extent=[10, -10; 30, 10]);
        BondLib.Bonds.eBond B4 annotation (extent=[30, 10; 50, 30], rotation=-90);
        BondLib.Bonds.fBond B3 annotation (extent=[-10, 10; 10, 30], rotation=90);
        BondLib.Thermal.GSth GSth1(G0=G0) annotation (extent=[10, 30; 30, 50]);
        BondLib.Junctions.J0p4 J0p4_1 annotation (extent=[30, -10; 50, 10]);
        BondLib.Bonds.eBond B5 annotation (extent=[30, -30; 50, -10], rotation=-90);
        BondLib.Thermal.Cth Cth1(gamma=gamma, e(start=T0)) annotation (extent=[30,
              -50; 50, -30], rotation=-90);
      equation 
        connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-10, 0; -10, 0],
              style(color=8));
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-30, 0; -30, 0],
              style(color=8));
        connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[10, 0; 10, 0],
            style(color=8));
        connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,10;
              -6.12303e-016,10],
            style(color=8));
        connect(B3.fBondCon1, GSth1.BondCon1) annotation (points=[6.12303e-016,
              30; 6.12303e-016,40; 10,40],
                     style(color=8));
        connect(GSth1.BondCon2, B4.fBondCon1) annotation (points=[30, 40; 40, 40;
              40, 30], style(color=8));
        connect(B4.eBondCon1, J0p4_1.BondCon4) annotation (points=[40, 10; 40, 10],
              style(color=8));
        connect(B2.eBondCon1, J0p4_1.BondCon1) annotation (points=[30, 0; 30, 0],
            style(color=8));
        connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -50,0],
            style(color=8));
        connect(J0p4_1.BondCon2, BondCon2) annotation (points=[50,0; 100,0],  style(
             color=8));
        connect(J0p4_1.BondCon3, B5.fBondCon1) annotation (points=[40, -10; 40, -10],
                 style(color=8));
        connect(B5.eBondCon1, Cth1.BondCon1) annotation (points=[40, -30; 40, -30],
              style(color=8));
      end CD;
      
      model CVext "External boundary layer convection" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.Area ac = 5667.0 "Cover area of Biosphere";
        parameter Real hai(unit="kJ/(h.m2.K)") = 39.6 
          "Convection coefficient equation intercept";
        parameter Real hai2(unit="kJ/(m3.K)") = 0.0034 
          "Convection coefficient equation slope";
        parameter Real ucx=1 
          "Cover normal specific thermal conductance adjustment factor";
        parameter Real ucp(unit="kJ/(h.m2.K)") = 72 
          "Cover normal specific thermal conductance";
        parameter Real R2(unit="h.m2.K/kJ") = 1/(ucx*ucp) 
          "Specific resistance due to conduction";
        Real uao(unit="m/h") "Wind velocity";
        Real R1(unit="h.m2.K/kJ") "Specific resistance due to convection";
        Real G0(unit="kJ/(h.K)") "Thermal conductance";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Rectangle(extent=[-26, -54; 6, -86], style(color=57)),
            Line(points=[-38, -70; -26, -70], style(color=57)),
            Line(points=[6, -70; 20, -70; 20, -50], style(color=57)),
            Text(
              extent=[-22, -58; 4, -68],
              string="Computation of",
              style(color=57)),
            Text(
              extent=[-22, -66; 4, -76],
              string="convection",
              style(color=57)),
            Text(
              extent=[-22, -74; 4, -84],
              string="due to wind",
              style(color=57))),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CVext",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
The external boundary layer convection requires a slightly more complex model than the internal convection, since air circulation around the dome removes hot air from the surface, thereby seemingly enhancing the convecting area. <p>

The model is essentially the same model as used for internal convection, but the conductance of convection is here not a constant, but rather a function of the wind velocity. <p>

The wind velocity is retrieved from a large lookup table. <p>

<hr> <p>

<pre>
Parameters:

  ac:    Cover area of Biosphere (default value = 5667.0 m<sup>2</sup>)

  hai:   Convection coefficient equation intercept (default value = 39.6 kJ/(h.m<sup>2</sup>.K))

  hai2:  Convection coefficient equation slope (default value = 0.0034 kJ/(m<sup>3</sup>.K))

  ucx:   Cover normal specific thermal conductance adjustment factor (default value = 1)

  ucp:   Cover normal specific thermal conductance (default value = 72 kJ/(h.m<sup>2</sup>.K))

  R2:    Specific resistance due to conduction (default value = 1/(ucx*ucp) h.m<sup>2</sup>.K/kJ)
</pre>
</html>
"));
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-50, -10; -30, 10]);
        BondLib.Junctions.J1p3 J1p3_1 annotation (extent=[-10, -10; 10, 10]);
        BondLib.Bonds.fBond B1 annotation (extent=[-30, -10; -10, 10]);
        BondLib.Bonds.eBond B2 annotation (extent=[10, -10; 30, 10]);
        BondLib.Junctions.J0p3 J0p3_2 annotation (extent=[30, -10; 50, 10]);
        BondLib.Bonds.eBond B4 annotation (extent=[30, -30; 50, -10], rotation=90);
        BondLib.Bonds.fBond B3 annotation (extent=[-10, -30; 10, -10], rotation=270);
        BondLib.Thermal.mGSth mGSth1 annotation (extent=[10, -50; 30, -30]);
        WindVelocity WindVelocity1 annotation (extent=[-60, -80; -40, -60]);
        Modelica.Blocks.Sources.Clock Clock1 annotation (extent=[-100,-80; -80,-60]);
      equation 
        connect(B1.fBondCon1, J1p3_1.BondCon1) annotation (points=[-10, 0; -10, 0],
              style(color=8));
        connect(J0p2_1.BondCon2, B1.eBondCon1) annotation (points=[-30, 0; -30, 0],
              style(color=8));
        connect(J1p3_1.BondCon2, B2.fBondCon1) annotation (points=[10, 0; 10, 0],
            style(color=8));
        connect(B2.eBondCon1, J0p3_2.BondCon1) annotation (points=[30, 0; 30, 0],
            style(color=8));
        connect(J1p3_1.BondCon3, B3.eBondCon1) annotation (points=[0,-10;
              1.83691e-015,-10],
              style(color=8));
        connect(B4.eBondCon1, J0p3_2.BondCon3) annotation (points=[40, -10; 40, -10],
                 style(color=8));
        connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -50,0],
            style(color=8));
        connect(J0p3_2.BondCon2, BondCon2) annotation (points=[50,0; 100,0],  style(
             color=8));
        connect(B3.fBondCon1, mGSth1.BondCon1) annotation (points=[
              -1.83691e-015,-30; -1.83691e-015,-40; 10,-40],
                        style(color=8));
        connect(mGSth1.BondCon2, B4.fBondCon1) annotation (points=[30, -40; 40, -40;
                40, -30], style(color=8));
        uao = WindVelocity1.y;
        R1 = 1/(hai + hai2*uao);
        G0 = ac/(R1 + R2);
        mGSth1.s = G0;
        connect(Clock1.y, WindVelocity1.u) annotation (points=[-79,-70; -62,-70], style(
              color=74, rgbcolor={0,0,135}));
      end CVext;
      
      model mGSev "Evaporation" 
        extends BondLib.Interfaces.ModTwoPort;
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        Real G(unit="kJ.kg_air/(h.kg_water)") 
          "Bondgraphic evaporation conductance";
        Real svp(unit="kPa") "Saturation vapor pressure";
        Real hum(unit="kg_water/kg_air") "Humidity ratio";
        Real P(unit="kJ/h") "Power flow into evaporation";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(Text(
              extent=[-78, 70; 76, -70],
              string="mGSev",
              style(color=0))),
          Icon(Text(
              extent=[-78, 70; 76, -70],
              string="mGSev",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
Evaporation is modeled in this program using Teten's law [1,2], an experimental equation that avoids the need of using steam tables. <p>

In the process of evaporation, <i>sensible heat</i> gets converted to <i>latent heat</i>. <p>

Water vapor is at a higher energy level than liquid water.  In order to evaporate, the activation energy must be overcome that is needed to provide the water molecules with the additional energy.  This energy is taken out of the thermal domain. <p>

In our model, humidity is used as an effort variable.  It gets multiplied with a quantity that corresponds to the power converted in the process. <p>

The humidity generated in the process of evaporation is proportional to the difference between the current humidity and the humidity at saturation vapor pressure, i.e., the humidity at the dew point. <p>

<hr> <p>

<pre>
Parameters:

  pao:  Atmospheric pressure (default value = 87.848 kPa)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Murray, F.W. (1967), \"On the Computation of Saturation Vapor Pressure,\" <i>L. Appl. Meteorol.</i>, <b>6</b>, pp. 203-204.
<li>Ot&aacute;rola A., M. Holdaway, L.-&Aring;. Nyman, S.J.E. Radford, and B.J. Butler (2005), \"ALMA Memo #512 - Atmospheric Transparency at Chajnantor: 1973-2003,\" <a href=\"http://alma.nrao.edu/memos/index.html\">Atacama Large Millimeter Array Memos</a>.
</ol>
</html>
"));
      equation 
        G = s;
        svp = 0.61078*exp(17.269*(e1 - 273.15)/(e1 - 35.85));
        hum = 0.62198/((pao/svp) - 1);
        P = G*(hum - e2);
        e1*f1 = P;
        e2*f2 = P;
      end mGSev;
      
      model mGSco "Condensation" 
        extends BondLib.Interfaces.ModTwoPort;
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        Real G(unit="kJ.kg_air/(h.kg_water)") 
          "Bondgraphic condensation conductance";
        Real svp(unit="kPa") "Saturation vapor pressure";
        Real hum(unit="kg_water/kg_air") "Humidity ratio";
        Real P(unit="kJ/h") "Power flow into condensation";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(Text(
              extent=[-78, 70; 76, -70],
              string="mGSco",
              style(color=0))),
          Icon(Text(
              extent=[-78, 70; 76, -70],
              string="mGSco",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Documentation(info="<html>
Condensation is the reverse process of evaporation.  It is also modeled here using Teten's law, an experimental equation that avoids the need of using steam tables. <p>

In the process of condensation, <i>latent heat</i> gets converted to <i>sensible heat</i>. <p>

Liquid water is at a lower energy level than water vapor.  In the process of condensation, the surplus energy is given back to the thermal domain, i.e., gets converted to entropy. <p>

In our model, humidity is used as an effort variable.  It gets multiplied with a quantity that corresponds to the power converted in the process. <p>

The humidity comsumed in the process of condensation is proportional to the difference between the current humidity and the humidity at saturation vapor pressure, i.e., the humidity at the dew point. <p>

No condensation takes place, until the humidty reaches the dew point.  Once it passes this point, water vapor is being liquified, such that the relative humidity remains at exactly 100%. <p>

Condensation takes place in our model both at the glass panes of the dome, and also in the bulk of the air.  The dome isn't tall enough to produce rain, but it most certainly can produce very dense fog. <p>

<hr> <p>

<pre>
Parameters:

  pao:  Atmospheric pressure (default value = 87.848 kPa)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Murray, F.W. (1967), \"On the Computation of Saturation Vapor Pressure,\" <i>L. Appl. Meteorol.</i>, <b>6</b>, pp. 203-204.
<li>Ot&aacute;rola A., M. Holdaway, L.-&Aring;. Nyman, S.J.E. Radford, and B.J. Butler (2005), \"ALMA Memo #512 - Atmospheric Transparency at Chajnantor: 1973-2003,\" <a href=\"http://alma.nrao.edu/memos/index.html\">Atacama Large Millimeter Array Memos</a>.
</ol>
</html>
"));
      equation 
        G = s;
        svp = 0.61078*exp(17.269*(e2 - 273.15)/(e2 - 35.85));
        hum = 0.62198/((pao/svp) - 1);
        P = G*(e1 - hum);
        e1*f1 = P;
        e2*f2 = P;
      end mGSco;
      
      model CWpond "Evaporation of the pond" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.Area ag = 2368.9 "Ground area of Biosphere";
        parameter Real hai(unit="kJ/(h.m2.K)") = 39.6 
          "Convection coefficient equation intercept";
        parameter Real apix = 0.4 "Percentage of Biosphere covered by pond";
        parameter Real lambda(unit="kJ/kg_water") = 2501.0 
          "Latent heat of vaporization";
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        Real ca(unit="kJ/(kg_air.K)") "Specific heat capacity of moist air";
        Real G0(unit="kg_air/(h.m2)") 
          "Specific boundary layer conductance due to evaporation";
        Real G(unit="kJ.kg_air/(h.kg_water)") 
          "Boundary layer conductance of evaporation";
        Real G2(unit="kJ.kg_air^2/(h.kg_water^2)") "Bondgraphic conductance";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CWpond",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram(
            Rectangle(extent=[-46, -44; -14, -76], style(color=57)),
            Line(points=[-14, -60; 0, -60; 0, -20], style(color=57)),
            Text(
              extent=[-42, -48; -16, -58],
              string="Computation of",
              style(color=57)),
            Text(
              extent=[-42, -56; -16, -66],
              string="convection due",
              style(color=57)),
            Text(
              extent=[-42, -64; -16, -74],
              string="to evaporation",
              style(color=57))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This model computes the evaporation of the pond.  It uses the generic evaporation model together with an internal modulation computing the conductance of evaporation. <p>

<hr> <p>

<pre>
Parameters:

  ag:      Ground area of Biosphere (default value = 2368.9 m<sup>2</sup>)

  hai:     Convection coefficient equation intercept (default value = 39.6 kJ/(h.m<sup>2</sup>.K))

  apix:    Portion of Biosphere covered by pond (default value = 0.4)

  lambda:  Latent heat of vaporization (default value = 2501 kJ/kg_water)

  pao:     Atmospheric pressure (default value = 87.848 kPa)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>American Society of Heating, Refrigerating and Air-Conditioning Engineers (1997), <i>ASHRAE Handbook of Fundamentals</i>.
<li><a href=\"http://www.engineeringtoolbox.com/enthalpy-moist-air-8_683.html\">Air Psychrometrics</a>
</ol>
</html>
"));
        mGSev mGSev1(pao=pao) annotation (extent=[-20, -20; 20, 20]);
      equation 
        connect(BondCon1, mGSev1.BondCon1) annotation (points=[-100,0; -20,0],
            style(color=8));
        connect(mGSev1.BondCon2, BondCon2) annotation (points=[20,0; 100,0],  style(
             color=8));
        ca = 1.005 + 1.859*e2;
        G0 = hai/ca;
        G = ag*apix*lambda*G0;
        G2 = G/e2;
        mGSev1.s = G2;
      end CWpond;
      
      model CWsoil "Evaporation of the soil" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.Area ag = 2368.9 "Ground area of Biosphere";
        parameter Real hai(unit="kJ/(h.m2.K)") = 39.6 
          "Convection coefficient equation intercept";
        parameter Real apix = 0.4 "Percentage of Biosphere covered by pond";
        parameter Real lambda(unit="kJ/kg_water") = 2501.0 
          "Latent heat of vaporization";
        parameter Real rg(unit="h.m2/kg_air") = 0.0278 
          "Specific ground resistance of evaporation to vapor diffusion";
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        Real ca(unit="kJ/(kg_air.K)") "Specific heat capacity of moist air";
        Real R0(unit="h.m2/kg_air") 
          "Specific boundary layer resistance of soil due to evaporation";
        Real G0(unit="kg_air/(h.m2)") 
          "Specific boundary layer conductance due to evaporation";
        Real G(unit="kJ.kg_air/(h.kg_water)") 
          "Boundary layer conductance of evaporation";
        Real G2(unit="kJ.kg_air^2/(h.kg_water^2)") "Bondgraphic conductance";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CWsoil",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram(
            Rectangle(extent=[-46, -44; -14, -76], style(color=57)),
            Line(points=[-14, -60; 0, -60; 0, -20], style(color=57)),
            Text(
              extent=[-42, -48; -16, -58],
              string="Computation of",
              style(color=57)),
            Text(
              extent=[-42, -56; -16, -66],
              string="convection due",
              style(color=57)),
            Text(
              extent=[-42, -64; -16, -74],
              string="to evaporation",
              style(color=57))),
          Documentation(info="<html>
This model computes the evaporation of the soil.  It uses the generic evaporation model together with an internal modulation computing the conductance of evaporation. <p>

<hr> <p>

<pre>
Parameters:

  ag:      Ground area of Biosphere (default value = 2368.9 m<sup>2</sup>)

  hai:     Convection coefficient equation intercept (default value = 39.6 kJ/(h.m<sup>2</sup>.K))

  apix:    Portion of Biosphere covered by pond (default value = 0.4)

  lambda:  Latent heat of vaporization (default value = 2501 kJ/kg_water)

  rg:      Specific ground resistance of evaporation to vapor diffusion (default value = 0.0278 h.m<sup>2</sup>/kg_air)

  pao:     Atmospheric pressure (default value = 87.848 kPa)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>American Society of Heating, Refrigerating and Air-Conditioning Engineers (1997), <i>ASHRAE Handbook of Fundamentals</i>.
<li><a href=\"http://www.engineeringtoolbox.com/enthalpy-moist-air-8_683.html\">Air Psychrometrics</a>
<li>Kimball, B.A. (1986), <i>A Modular Energy Balance Program Including Subroutines for Greenhouses and Other Latent Heat Devices</i>, United States Department of Agriculture, ARS-33.
</ol>
</html>
"));
        mGSev mGSev1(pao=pao) annotation (extent=[-20, -20; 20, 20]);
      equation 
        connect(BondCon1, mGSev1.BondCon1) annotation (points=[-100,0; -20,0],
            style(color=8));
        connect(mGSev1.BondCon2, BondCon2) annotation (points=[20,0; 100,0],  style(
             color=8));
        ca = 1.005 + 1.859*e2;
        R0 = ca/hai;
        G0 = 1/(R0 + rg);
        G = ag*(1 - apix)*lambda*G0;
        G2 = G/e2;
        mGSev1.s = G2;
      end CWsoil;
      
      model CWveg "Evaporation of the vegetation" 
        extends BondLib.Interfaces.TwoPort;
        parameter Modelica.SIunits.Area ag = 2368.9 "Ground area of Biosphere";
        parameter Real hai(unit="kJ/(h.m2.K)") = 39.6 
          "Convection coefficient equation intercept";
        parameter Real avix = 0.4 
          "Percetage of Biosphere covered by vegetation";
        parameter Real laix = 2 "Leaf area index";
        parameter Real lambda(unit="kJ/kg_water") = 2501.0 
          "Latent heat of vaporization";
        parameter Real rg(unit="h.m2/kg_air") = 0.0278 
          "Specific ground resistance of evaporation to vapor diffusion";
        parameter Real b1(unit="kJ/kg_air") = 20.3 
          "Stomatal resistance coefficient";
        parameter Real b2(unit="kJ/(h.m2)") = 61.2 
          "Specific canopy light reflection";
        parameter Real taux=1 "Light adjustment factor";
        parameter Real taup=0.6 "Canopy light extinction coefficient";
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        Real ca(unit="kJ/(kg_air.K)") "Specific heat capacity of moist air";
        Real so(unit="kJ/(h.m2)") "Specific solar input to canopy";
        Real R0(unit="h.m2/kg_air") 
          "Specific boundary layer resistance of vegetation due to evaporation";
        Real R1(unit="h.m2/kg_air") 
          "Specific stomatal resistance of evaporation of vegetation";
        Real G0(unit="kg_air/(h.m2)") 
          "Specific boundary layer conductance due to evaporation";
        Real G(unit="kJ.kg_air/(h.kg_water)") 
          "Boundary layer conductance of evaporation";
        Real G2(unit="kJ.kg_air^2/(h.kg_water^2)") "Bondgraphic conductance";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CWveg",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram(
            Rectangle(extent=[-16, -44; 16, -76], style(color=57)),
            Text(
              extent=[-12, -48; 14, -58],
              string="Computation of",
              style(color=57)),
            Text(
              extent=[-12, -56; 14, -66],
              string="convection due",
              style(color=57)),
            Text(
              extent=[-12, -64; 14, -74],
              string="to evaporation",
              style(color=57)),
            Line(points=[-28, -60; -16, -60], style(color=61)),
            Line(points=[0, -44; 0, -20], style(color=57))),
          Documentation(info="<html>
This model computes the evaporation of the vegetation.  It uses the generic evaporation model together with an internal modulation computing the conductance of evaporation. <p>

<hr> <p>

<pre>
Parameters:

  ag:      Ground area of Biosphere (default value = 2368.9 m<sup>2</sup>)

  hai:     Convection coefficient equation intercept (default value = 39.6 kJ/(h.m<sup>2</sup>.K))

  avix:    Portion of Biosphere covered by vegetation (default value = 0.4)

  laix:    Leaf area index (default value = 2)

  lambda:  Latent heat of vaporization (default value = 2501 kJ/kg_water)

  rg:      Specific ground resistance of evaporation to vapor diffusion (default value = 0.0278 h.m<sup>2</sup>/kg_air)

  b1:      Stomatal resistance coefficient (default value = 20.3 kJ/kg_air)

  b2:      Specific canopy light reflection (default value = 61.2 kJ/(h.m2))

  taux:    Light adjustment factor (default value = 1)

  taup:    Canopy light extinction coefficient (default value = 0.6)

  pao:     Atmospheric pressure (default value = 87.848 kPa)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>American Society of Heating, Refrigerating and Air-Conditioning Engineers (1997), <i>ASHRAE Handbook of Fundamentals</i>.
<li><a href=\"http://www.engineeringtoolbox.com/enthalpy-moist-air-8_683.html\">Air Psychrometrics</a>
<li>Kimball, B.A. (1986), <i>A Modular Energy Balance Program Including Subroutines for Greenhouses and Other Latent Heat Devices</i>, United States Department of Agriculture, ARS-33.
</ol>

</html>
"));
        mGSev mGSev1(pao=pao) annotation (extent=[-20, -20; 20, 20]);
        SolarInput SolarInput1 annotation (extent=[-50, -70; -30, -50]);
        Modelica.Blocks.Sources.Clock Clock1 annotation (extent=[-90,-70; -70,-50]);
      equation 
        connect(BondCon1, mGSev1.BondCon1) annotation (points=[-100,0; -20,0],
            style(color=8));
        connect(mGSev1.BondCon2, BondCon2) annotation (points=[20,0; 100,0],  style(
             color=8));
        ca = 1.005 + 1.859*e2;
        R0 = ca/hai;
        so = SolarInput1.y;
        R1 = b1/(b2 + (so*taux*taup));
        G0 = 1/(R0 + R1 + rg);
        G = ag*avix*laix*lambda*G0;
        G2 = G/e2;
        mGSev1.s = G2;
        connect(Clock1.y, SolarInput1.u) annotation (points=[-69,-60; -52,-60], style(
              color=74, rgbcolor={0,0,135}));
      end CWveg;
      
      model CWcov "Condensation at the dome" 
        extends BondLib.Interfaces.TwoPort;
        constant Real lambda(unit="kJ/kg_water") = 2501 
          "Latent heat of vaporization";
        parameter Modelica.SIunits.Area ac = 5667.0 "Cover area of Biosphere";
        parameter Real hai(unit="kJ/(h.m2.K)") = 39.6 
          "Convection coefficient equation intercept";
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        Real ca(unit="kJ/(kg_air.K)") "Specific heat capacity of moist air";
        Real G0(unit="kg_air/(h.m2)") 
          "Specific boundary layer conductance of condensation on cover";
        Real G(unit="kJ.kg_air/(h.kg_water)") 
          "Boundary layer conductance of condensation";
        Real G2(unit="kJ.kg_air^2/(h.kg_water^2)") "Bondgraphic conductance";
        Real svp(unit="kPa") "Saturation vapor pressure of cover";
        Real hum(unit="kg_water/kg_air") 
          "Humidity ratio of cover at saturation";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CWcov",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram(
            Rectangle(extent=[-46, -44; -14, -76], style(color=57)),
            Line(points=[-14, -60; 0, -60; 0, -20], style(color=57)),
            Text(
              extent=[-42, -48; -16, -58],
              string="Computation of",
              style(color=57)),
            Text(
              extent=[-42, -56; -16, -66],
              string="convection due",
              style(color=57)),
            Text(
              extent=[-42, -64; -16, -74],
              string="to condensation",
              style(color=57))),
          Documentation(info="<html>
This model computes the condensation at the dome.  It uses the generic condensation model together with an internal modulation computing the conductance of condensation. <p>

An if statement is used to check whether the current humidity is above that of the dew point.  If this is not the case, no condensation takes place. <p>

<hr> <p>

<pre>
Parameters:

  ac:      Cover area of Biosphere (default value = 5667.0 m<sup>2</sup>)

  hai:     Convection coefficient equation intercept (default value = 39.6 kJ/(h.m<sup>2</sup>.K))

  pao:     Atmospheric pressure (default value = 87.848 kPa)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>American Society of Heating, Refrigerating and Air-Conditioning Engineers (1997), <i>ASHRAE Handbook of Fundamentals</i>.
<li><a href=\"http://www.engineeringtoolbox.com/enthalpy-moist-air-8_683.html\">Air Psychrometrics</a>
</ol>
</html>
"));
        mGSco mGSco1(pao=pao) annotation (extent=[-20, -20; 20, 20]);
      equation 
        connect(BondCon1, mGSco1.BondCon1) annotation (points=[-100,0; -20,0],
            style(color=8));
        connect(mGSco1.BondCon2, BondCon2) annotation (points=[20,0; 100,0],  style(
             color=8));
        ca = 1.005 + 1.859*e2;
        G0 = hai/ca;
        svp = 0.61078*exp(17.269*(e2 - 273.15)/(e2 - 35.85));
        hum = 0.62198/((pao/svp) - 1);
        G = if e1 < hum then ac*G0*lambda else 0;
        G2 = G/e1;
        mGSco1.s = G2;
      end CWcov;
      
      model CWbulk "Condensation in the bulk" 
        extends BondLib.Interfaces.TwoPort;
        constant Real lambda(unit="kJ/kg_water") = 2501 
          "Latent heat of vaporization";
        parameter Modelica.SIunits.Volume vai = 35000.0 
          "Air volume of Biosphere";
        parameter Real hai(unit="kJ/(h.m2.K)") = 39.6 
          "Convection coefficient equation intercept";
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        Real ca(unit="kJ/(kg_air.K)") "Specific heat capacity of moist air";
        Real G0(unit="kg_air/(h.m2)") 
          "Specific boundary layer conductance of condensation in the bulk";
        Real G(unit="kJ.kg_air/(h.kg_water)") 
          "Boundary layer conductance of condensation";
        Real G2(unit="kJ.kg_air^2/(h.kg_water^2)") "Bondgraphic conductance";
        Real svp(unit="kPa") "Saturation vapor pressure of air";
        Real hum(unit="kg_water/kg_air") "Humidity ratio of air at saturation";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-66, 80; 64, -80],
              string="CWbulk",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Diagram(
            Rectangle(extent=[-46, -44; -14, -76], style(color=57)),
            Line(points=[-14, -60; 0, -60; 0, -20], style(color=57)),
            Text(
              extent=[-42, -48; -16, -58],
              string="Computation of",
              style(color=57)),
            Text(
              extent=[-42, -56; -16, -66],
              string="convection due",
              style(color=57)),
            Text(
              extent=[-42, -64; -16, -74],
              string="to condensation",
              style(color=57))),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This model computes the condensation in the bulk of the air.  It uses the generic condensation model together with an internal modulation computing the conductance of condensation. <p>

An if statement is used to check whether the current humidity is above that of the dew point.  If this is not the case, no condensation takes place. <p>

<hr> <p>

<pre>
Parameters:

  vai:     Air volume of Biosphere (default value = 35000 m<sup>3</sup>)

  hai:     Convection coefficient equation intercept (default value = 39.6 kJ/(h.m<sup>2</sup>.K))

  pao:     Atmospheric pressure (default value = 87.848 kPa)
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>American Society of Heating, Refrigerating and Air-Conditioning Engineers (1997), <i>ASHRAE Handbook of Fundamentals</i>.
<li><a href=\"http://www.engineeringtoolbox.com/enthalpy-moist-air-8_683.html\">Air Psychrometrics</a>
</ol>
</html>
"));
        mGSco mGSco1(pao=pao) annotation (extent=[-20, -20; 20, 20]);
      equation 
        connect(BondCon1, mGSco1.BondCon1) annotation (points=[-100,0; -20,0],
            style(color=8));
        connect(mGSco1.BondCon2, BondCon2) annotation (points=[20,0; 100,0],  style(
             color=8));
        ca = 1.005 + 1.859*e2;
        G0 = hai/ca;
        svp = 0.61078*exp(17.269*(e2 - 273.15)/(e2 - 35.85));
        hum = 0.62198/((pao/svp) - 1);
        G = if e1 < hum then vai*G0*lambda else 0;
        G2 = G/e1;
        mGSco1.s = G2;
      end CWbulk;
      
      model Cmoist "Capacity of moisture" 
        extends BondLib.Interfaces.ModPassiveOnePort;
        constant Real R(unit="kJ/(kg.K)") = 0.287 "Gas constant";
        constant Real lambda(unit="kJ/kg_water") = 2501 
          "Latent heat of vaporization";
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        parameter Modelica.SIunits.Volume vai = 35000.0 
          "Air volume of biosphere";
        Modelica.SIunits.Temperature Tair "Air temperature inside Biosphere";
        Real da(unit="kg_air/m3") "Air density";
        Real gamma(unit="kJ*kg_air/kg_water") "Capacitance of moisture";
        Real C(unit="kJ*kg_air^2/(kg_water^2)") "Bondgraphic capacitance";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(Text(
              extent=[-76, 54; 66, -56],
              string="Cmoist",
              style(color=0))),
          Icon(Text(
              extent=[-76, 54; 66, -56],
              string="Cmoist",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Documentation(info="<html>
This model computes the capacity of the moisture, i.e., represents the capacitor attached to the 0-junction representing the humidity. <p>

<hr> <p>

<pre>
Parameters:

  pao:     Atmospheric pressure (default value = 87.848 kPa)

  vai:     Air volume of Biosphere (default value = 35000 m<sup>3</sup>)
</pre> <p>

<hr> <p>

<b>Comments:</b> <p>

This model presents maybe one of the nicest examples emonstrating the usefulness of bond graph modeling. <p>

If you check the literature, you will notice that <b>evaporation</b> and <b>condensation</b> are not an area of physics that is very well ploughed.  For example, our model uses <b>Teten's law</b> to compute the saturation vapor pressure.  However, different authors either use <a href=\"http://cires.colorado.edu/~voemel/vp.html\">different formulae</a> or at least slightly different numerical parameter values.  The simulation results obtained in this fashion differ by a few percent one from another. <p>

Hence the student, who created the model, Francisco Luttmann, had a hard time finding good equations to mathematically capture the phenomena of evaporation and condensation.  His model was pretty good, but he still made mistakes that he didn't find.  In particular, his latent heat capacitance model was totally wrong.  He had forgotten to multiply the capacitance value by lambda, i.e., his capacitance value was off by a factor of 2500. <p>

When using bond graphs to describe evaporation and condensation, we first need to select the two adjugate variables.  In fact, we only have one choice to make. <p>

In accordance with Luttmann's model, I chose the <i>humidity ratio</i> as the effort variable, thus: <p>

<b>[e] = kg_water/kg_air</b> <p>

The units of effort are <i>kg_water</i> per <i>kg_air</i>. <p>

Since the product of effort and flow must be power, and since we know that power is being expressed in this model as <i>kJ/h</i>, it follows that: <p>

<b>[f] = [P]/[e] = kJ.kg_air/(h.kg_water)</b> <p>

Since the <i>bondgraphic resistance</i> is defined by the equation: <p>

<b>e = R*f</b> <p>

it follows that: <p>

<b>[R] = [e]/[f] = h.kg_water<sup>2</sup>/(kJ.kg_air<sup>2</sup>)</b> <p>

Comparing these units with those used by Luttmann, we see that Luttmann's units were off by <i>kg_water/kg_air</i>.  Hence the bondgraphic latent heat resistance must be multiplied by the humidity ratio, i.e., the effort variable, which is not very surprising, as the same happens in the thermal domain. <p>

Since the <i>bondgraphic capacitance</i> is defined by the equation: <p>

<b>f = C*der(e)</b> <p>

it follows that: <p>

<b>[C] = [f].h/[e] = kJ.kg_air<sup>2</sup>/kg_water<sup>2</sup></b> <p>

Since the bondgraphic resistor is being multiplied by <i>e</i>, the bondgraphic capacitance must be divided by <i>e</i>, such that the product of resistance times capacitance remains a time constant.  Thus the units of the physical capacitance are: <p>

<b>[gamma] = [C]*[e] = kJ.kg_air/kg_water</b> <p>

However, Luttmann let his capacitance be described as air mass, i.e., the product of air density, <b>da</b>, and air volume, <b>vai</b>, thus: <p>

<b>[gamma] = [da]*[vai] = kg_air</b> <p>

Consequently, his units were off by <i>kJ/kg_water</i>, which are exactly the units of <b>lambda</b>, the latent heat of vaporization.  Hence the equation was corrected to be: <p>

<b>gamma = lambda*da*vai</b> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Luttmann, F. (1990), <i>A Dynamic Thermal Model of a Self-sustaining Closed Environment Life Support System</i>, Ph.D. dissertation, Department of Nuclear and Energy Engineering, University of Arizona.
</ol></html>
"));
      equation 
        Tair = s;
        da = pao/(R*Tair);
        gamma = lambda*da*vai;
        C = gamma/e;
        C*der(e) = f;
      end Cmoist;
      
      model Soil "Soil" 
        extends BondLib.Interfaces.PassiveOnePort;
        parameter Modelica.SIunits.Area ag = 2368.9 "Ground area of Biosphere";
        parameter Real apix = 0.4 "Portion of Biosphere covered by pond";
        parameter Real lambda1(unit="kJ/(h.m2.K)") = 90 
          "Specific thermal conductance of first ground layer";
        parameter Real gamma1(unit="kJ/(m2.K)") = 200 
          "Specific heat capacity of first ground layer";
        parameter Real lambda2(unit="kJ/(h.m2.K)") = 45 
          "Specific thermal conductance of second ground layer";
        parameter Real gamma2(unit="kJ/(m2.K)") = 300 
          "Specific heat capacity of second ground layer";
        parameter Real lambda3(unit="kJ/(h.m2.K)") = 22.5 
          "Specific thermal conductance of third ground layer";
        parameter Real gamma3(unit="kJ/(m2.K)") = 600 
          "Specific heat capacity of third ground layer";
        parameter Modelica.SIunits.Temperature T01=285 
          "Initial temperature of first ground layer";
        parameter Modelica.SIunits.Temperature T02=286 
          "Initial temperature of second ground layer";
        parameter Modelica.SIunits.Temperature T03=288 
          "Initial temperature of third ground layer";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Text(
              extent=[-74, 56; 70, -62],
              string="Soil",
              style(color=0)), Text(extent=[-70, 90; 70, 50], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          Diagram(Line(points=[-60, 0; -60, 0], style(color=8))),
          Documentation(info="<html>
This model calculates the heat conduction in the soil.  The soil is modeled as having three layers.  The bottom of the soil is assumed to be thermally insulated.  Therefore, the model uses three <b>CD</b> elements followed by a flow source of zero flow. <p>

<hr> <p>

<pre>
Parameters:

  ag:       Ground area of Biosphere (default value = 2368.9 m<sup>2</sup>)

  apix:     Portion of Biosphere covered by pond (default value = 0.4)

  lambda1:  Specific thermal conductance of first ground layer (default value = 90 kJ/(h.m<sup>2</sup>.K))

  gamma1:   Specific heat capacity of first ground layer (default value = 200 kJ/(m<sup>2</sup>.K))

  lambda2:  Specific thermal conductance of second ground layer (default value = 45 kJ/(h.m<sup>2</sup>.K))

  gamma2:   Specific heat capacity of second ground layer (default value = 300 kJ/(m<sup>2</sup>.K))

  lambda3:  Specific thermal conductance of third ground layer (default value = 22.5 kJ/(h.m<sup>2</sup>.K))

  gamma3:   Specific heat capacity of third ground layer (default value = 600 kJ/(m<sup>2</sup>.K))

  T01:      Initial temperature of first ground layer (default value = 285 K)

  T02:      Initial temperature of second ground layer (default value = 286 K)

  T03:      Initial temperature of third ground layer (default value = 288 K)
</pre>
</html>
"));
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[-60, -10; -40, 10]);
        CD CD1(
          ag=ag,
          apix=apix,
          lambda=lambda1,
          gamma0=gamma1,
          T0=T01) annotation (extent=[-20, -10; 0, 10]);
        BondLib.Bonds.fBond B2 annotation (extent=[-40, -10; -20, 10]);
        BondLib.Bonds.fBond B3 annotation (extent=[0, -10; 20, 10]);
        BondLib.Junctions.J0p2 J0p2_2 annotation (extent=[20, -10; 40, 10]);
        BondLib.Bonds.fBond B4 annotation (extent=[40, -10; 60, 10]);
        CD CD2(
          ag=ag,
          apix=apix,
          lambda=lambda2,
          gamma0=gamma2,
          T0=T02) annotation (extent=[60, -10; 80, 10]);
        BondLib.Bonds.fBond B5 annotation (extent=[60, -50; 80, -30], rotation=180);
        BondLib.Junctions.J0p2 J0p2_3 annotation (extent=[40, -50; 60, -30]);
        BondLib.Bonds.fBond B6 annotation (extent=[20, -50; 40, -30], rotation=180);
        CD CD3(
          ag=ag,
          apix=apix,
          lambda=lambda2,
          gamma0=gamma2,
          T0=T03) annotation (extent=[20, -50; 0, -30]);
        BondLib.Bonds.fBond B7 annotation (extent=[-20, -50; 0, -30], rotation=180);
        BondLib.Sources.Sf Sf1(f0=0) annotation (extent=[-60, -50; -80, -30]);
        BondLib.Junctions.J0p2 J0p2_4 annotation (extent=[-40, -50; -20, -30]);
        BondLib.Bonds.fBond B8 annotation (extent=[-60, -50; -40, -30], rotation=
              180);
      equation 
        connect(J0p2_1.BondCon2, B2.eBondCon1) annotation (points=[-40, 0; -40, 0],
              style(color=8));
        connect(B2.fBondCon1, CD1.BondCon1) annotation (points=[-20, 0; -20, 0],
            style(color=8));
        connect(CD1.BondCon2, B3.eBondCon1) annotation (points=[0, 0; 0, 0], style(
              color=8));
        connect(B3.fBondCon1, J0p2_2.BondCon1) annotation (points=[20, 0; 20, 0],
            style(color=8));
        connect(J0p2_2.BondCon2, B4.eBondCon1) annotation (points=[40, 0; 40, 0],
            style(color=8));
        connect(B4.fBondCon1, CD2.BondCon1) annotation (points=[60, 0; 60, 0],
            style(color=8));
        connect(CD2.BondCon2, B5.eBondCon1) annotation (points=[80, 0; 90, 0; 90, -
              40; 80, -40], style(color=8));
        connect(B5.fBondCon1, J0p2_3.BondCon2) annotation (points=[60, -40; 60, -40],
                 style(color=8));
        connect(J0p2_3.BondCon1, B6.eBondCon1) annotation (points=[40, -40; 40, -40],
                 style(color=8));
        connect(B6.fBondCon1, CD3.BondCon1) annotation (points=[20, -40; 20, -40],
            style(color=8));
        connect(CD3.BondCon2, B7.eBondCon1) annotation (points=[0, -40; 0, -40],
            style(color=8));
        connect(B7.fBondCon1, J0p2_4.BondCon2) annotation (points=[-20, -40; -20, -
              40], style(color=8));
        connect(J0p2_4.BondCon1, B8.eBondCon1) annotation (points=[-40, -40; -40, -
              40], style(color=8));
        connect(B8.fBondCon1, Sf1.BondCon1) annotation (points=[-60, -40; -60, -40],
              style(color=8));
        connect(BondCon1, J0p2_1.BondCon1) annotation (points=[-100,0; -60,0],
            style(color=8));
      end Soil;
      
      model Biosphere "Biosphere 2 main model" 
        constant Modelica.SIunits.CelsiusTemperature T0 = Modelica.Constants.T_zero 
          "Conversion between Centigrade and Kelvin";
        constant Real sigconv = 3.6 
          "Conversion factor of Stephan-Boltzmann constant to appropriate units";
        constant Real lambda(unit="kJ/kg_water") = 2501 
          "Latent heat of vaporization";
        parameter Modelica.SIunits.Area ag=2368.9 "Ground area of Biosphere";
        parameter Modelica.SIunits.Area ac=5667.0 "Cover area of Biosphere";
        parameter Real pao(unit="kPa") = 87.848 "Atmospheric pressure";
        parameter Real alph=0.2 "Portion of absorbed power";
        parameter Real tau=0.6 "Portion of transmitted power";
        parameter Real avix=0.4 "Portion of Biosphere covered by vegetation";
        parameter Real apix=0.4 "Portion of Biosphere covered by pond";
        parameter Real laix=2 "Leaf area index";
        parameter Real vkx=0.6 "Canopy light extinction coefficient";
        parameter Real rhos=0.13 "Reflectance of soil for solar radiation";
        parameter Modelica.SIunits.Volume vai=35000 "Air volume of Biosphere";
        parameter Real refl=0.05 "Reflectance of pond";
        parameter Real hai(unit="kJ/(h.m2.K)") = 39.6 
          "Convection coefficient equation intercept";
        parameter Real hai2(unit="kJ/(m3.K)") = 0.0034 
          "Convection coefficient equation slope";
        parameter Real Ct(unit="kJ/K") = 2842 "Heat capacity of air";
        parameter Modelica.SIunits.Length hpd=5 "Average depth of pond";
        parameter Real Cp(unit="kJ/K") = 4190*ag*apix*hpd 
          "Heat capacity of pond";
        parameter Real fcsky=0.67 "Sky radiation view factory";
        parameter Real epsc=1 "Sky radiation correction factor";
        output Modelica.SIunits.CelsiusTemperature Tdome 
          "Temperature of the dome";
        output Modelica.SIunits.CelsiusTemperature Tlake 
          "Temperature of the lake";
        output Modelica.SIunits.CelsiusTemperature Tveg 
          "Temperature of the vegetatione";
        output Modelica.SIunits.CelsiusTemperature Tsoil 
          "Temperature of the top layer of the soil";
        output Modelica.SIunits.CelsiusTemperature Tgr1 
          "Temperature of the topmost layer inside the soil";
        output Modelica.SIunits.CelsiusTemperature Tgr2 
          "Temperature of the middle layer inside the soil";
        output Modelica.SIunits.CelsiusTemperature Tgr3 
          "Temperature of the lowermost layer inside the soil";
        output Modelica.SIunits.CelsiusTemperature Tair 
          "Temperature of the air in degrees Centigrade";
        output Real Hair(unit="kg_water/kg_air") "Inside air humidity ratio";
        output Real Hsat(unit="kg_water/kg_air") 
          "Humidity ratio of saturated air";
        output Real Hrel(unit="%") "Relative humidity of the air";
        annotation (
          Coordsys(
            extent=[-200, -150; 200, 150],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Polygon(points=[-190, -140; 190, -140; 190, 20; 80, 80; -80, 80; -190,
                  20; -190, -140], style(color=81, thickness=2)),
            Polygon(points=[2, -64; 2, -48; -2, -48; -2, -64; 2, -64], style(
                  pattern=0, fillColor=42)),
            Polygon(points=[-8, -48; 8, -48; 0, -18; -8, -48], style(pattern=0,
                  fillColor=58)),
            Polygon(points=[-18, -64; -18, -48; -22, -48; -22, -64; -18, -64],
                style(pattern=0, fillColor=42)),
            Polygon(points=[-28, -48; -12, -48; -20, -18; -28, -48], style(pattern=
                    0, fillColor=58)),
            Line(points=[-190, -64; 40, -64; 80, -104; 190, -104], style(color=81,
                  thickness=2)),
            Line(points=[48, -72; 190, -72], style(color=81, thickness=2)),
            Line(points=[-80, -60; -80, -60], style(color=8)),
            Line(points=[40, -10; 40, -10], style(color=8)),
            Line(points=[-80, 20; -80, 20], style(color=8)),
            Line(points=[150, 20; 150, 20], style(color=8))),
          Icon(
            Text(extent=[-162,134; 158,94],    string="%name"), Bitmap(extent=[-202,
                  60; 202,-140], name="Images/BG_Ex4.jpg")),
          Window(
            x=0.45,
            y=0.01,
            width=0.44,
            height=0.65),
          experiment(StopTime=8600, Interval=1),
          experimentSetupOutput(
            doublePrecision=true,
            states=false,
            derivatives=false,
            inputs=false,
            auxiliaries=false),
          Documentation(info="<html>
This is the top-level model of the Biosphere 2 model.  This is the program that needs to be compiled and simulated. <p>
 
The iconic representation represents Biosphere 2, modeled as a single biome, i.e., we assume that the entire air inside Biosphere 2 is at the same temperature.  This is certainly not realistic, but doesn't truly matter, as long as the air conditioning is left out of the model. <p>

We are now ready to assemble the Biosphere 2 model from its components.  At this point, the modeling task has become quite simple. <p>

Each of the five modeling elements: <p>

<ol>
<li>the inside air,
<li>the dome,
<li>the pond,
<li>the vegetation, and
<li>the top layer of the soil
</ol> <p>

is represented by a 0-junction with a heat capacity attached to it that calculates the temperature of that modeling element.  The dome 0-junction was split into two 0-junctions connected by a bond due to the large number of connections to it. <p>

A sixth 0-junction is used to represent the moisture of the air.  It has its own special moisture capacitor element attached to it. <p>

The six modeling elements are connected among each other by convection, radiation, condensation, and evaporation as appropriate. <p>

Each modeling element has furthermore a modulated power source attached to it, which represents the solar input to that modeling element.  Rather than modeling the distribution of solar energy in any detail, the model uses a fairly global approach to railroad the available solar input to the five modeling elements. <p>


At the top of the model, we have the usual computation of ambient temperature (for external convection), night sky temperature (for radiation), solar radiation, and that's it. <p>

Of course, the devil is in the detail.  Each of these models contains a substantial number of parameters that need to be set, and getting the correct parameter values together for such a complex model is no easy task. <p>

<font size=6 color=red><b>Enjoy!</b></font>
</html>
"));
        BondLib.Junctions.J0p6 J0p6_1 annotation (extent=[-140, -70; -120, -50]);
        BondLib.Junctions.J0p6 J0p6_2 annotation (extent=[-30, 70; -10, 90]);
        BondLib.Junctions.J0p6 J0p6_3 annotation (extent=[10, 70; 30, 90]);
        BondLib.Junctions.J0p5 J0p5_3 annotation (extent=[120, -58; 140, -78]);
        BondLib.Junctions.J0p5 J0p5_4 annotation (extent=[-20, 0; 0, -20]);
        BondLib.Junctions.J0p2 J0p2_1 annotation (extent=[90, 120; 110, 140]);
        BondLib.Junctions.J0p2 J0p2_2 annotation (extent=[-110, 120; -90, 140]);
        BondLib.Bonds.fBond B1 annotation (extent=[-10, 70; 10, 90], rotation=180);
        BondLib.Bonds.fBond B2 annotation (extent=[-130, 120; -110, 140]);
        BondLib.Sources.mSe mSe1 annotation (extent=[-130, 120; -150, 140]);
        NightSkyTemp NightSkyTemp1 annotation (extent=[-180, 120; -160, 140]);
        BondLib.Bonds.fBond B3 annotation (extent=[-90, 120; -70, 140]);
        BondLib.Thermal.RA RA1(A=ac*fcsky*epsc*sigconv)      annotation (extent=[-
              70, 120; -50, 140]);
        BondLib.Bonds.eBond B4 annotation (extent=[-50, 120; -30, 140]);
        AmbientTemp AmbientTemp1 annotation (extent=[180, 120; 160, 140]);
        BondLib.Sources.mSe mSe2 annotation (extent=[130, 120; 150, 140]);
        BondLib.Bonds.fBond B5 annotation (extent=[110, 120; 130, 140], rotation=
              180);
        BondLib.Bonds.fBond B6 annotation (extent=[70, 120; 90, 140], rotation=180);
        CVext CVext1(
          ac=ac,
          hai=hai,
          hai2=hai2) annotation (extent=[70, 120; 50, 140]);
        BondLib.Bonds.eBond B7 annotation (extent=[30, 120; 50, 140], rotation=180);
        BondLib.Bonds.eBond B8 annotation (extent=[40, 80; 60, 100]);
        BondLib.Thermal.Cth Cth1(gamma=Ct, e(start=276)) annotation (extent=[60, 80;
                80, 100]);
        BondLib.Bonds.eBond B9 annotation (extent=[-50, 80; -30, 100]);
        BondLib.Sources.mSP mSP1 annotation (extent=[-50, 80; -70, 100]);
        Glass Glass1(alph=alph, tau=tau) annotation (extent=[-110, 80; -90, 100]);
        SolarInput SolarInput1 annotation (extent=[-180, 80; -160, 100]);
        Modelica.Blocks.Math.Gain Gain1(k=ag)   annotation (extent=[-148, 80; -128,
                100]);
        Absorption Absorption1(
          apix=apix,
          avix=avix,
          laix=laix,
          refl=refl,
          vkx=vkx,
          rhos=rhos) annotation (extent=[-180, -20; -160, 0]);
        BondLib.Bonds.eBond B10 annotation (extent=[-50, 60; -30, 80]);
        BondLib.Sources.mSP mSP2 annotation (extent=[-50, 58; -70, 78]);
        Glass Glass2(alph=alph, tau=tau) annotation (extent=[-150,42; -130,62]);
        BondLib.Bonds.eBond B11 annotation (extent=[-160, -70; -140, -50]);
        BondLib.Sources.mSP mSP3 annotation (extent=[-160, -70; -180, -50]);
        BondLib.Bonds.eBond B12 annotation (extent=[-40, -20; -20, 0]);
        BondLib.Sources.mSP mSP4 annotation (extent=[-40, -20; -60, 0]);
        BondLib.Bonds.eBond B13 annotation (extent=[100, -78; 120, -58]);
        BondLib.Sources.mSP mSP5 annotation (extent=[100, -78; 80, -58]);
        BondLib.Bonds.eBond B14 annotation (extent=[120, -100; 140, -80], rotation=
              -90);
        BondLib.Thermal.Cth Cth2(gamma=Cp, e(start=288)) annotation (extent=[120, -
              120; 140, -100], rotation=-90);
        BondLib.Bonds.eBond B15 annotation (extent=[-150, -100; -130, -80],
            rotation=-90);
        BondLib.Thermal.Cth Cth3(gamma=Ct, e(start=285)) annotation (extent=[-150,
              -120; -130, -100], rotation=-90);
        BondLib.Bonds.fBond B16 annotation (extent=[-130, -100; -110, -80],
            rotation=-90);
        Soil Soil1(ag=ag, apix=apix) annotation (extent=[-130, -120; -110, -100],
            rotation=-90);
        BondLib.Bonds.eBond B17 annotation (extent=[-20, -80; 0, -60], rotation=-90);
        BondLib.Thermal.Cth Cth4(gamma=Ct, e(start=280)) annotation (extent=[-20, -
              100; 0, -80], rotation=-90);
        BondLib.Bonds.eBond B18 annotation (extent=[-120, -70; -100, -50], rotation=
             180);
        BondLib.Thermal.RA RA2(A=ag*(1 - avix - apix)*sigconv)      annotation (
            extent=[-80, -70; -100, -50]);
        BondLib.Bonds.eBond B20 annotation (extent=[0, -20; 20, 0], rotation=180);
        BondLib.Thermal.RA RA3(A=ag*avix*sigconv)      annotation (extent=[40, -20;
                20, 0]);
        BondLib.Bonds.eBond B22 annotation (extent=[140, -78; 160, -58], rotation=
              180);
        BondLib.Thermal.RA RA4(A=ag*apix*sigconv)      annotation (extent=[180, -78;
                160, -58]);
        BondLib.Bonds.fBond B19 annotation (extent=[-80, -70; -60, -50], rotation=
              180);
        BondLib.Bonds.fBond B21 annotation (extent=[40, -20; 60, 0], rotation=180);
        BondLib.Bonds.fBond B23 annotation (extent=[170, -52; 190, -32], rotation=-
              90);
        CV CV1(G0=ag*(1 - apix)*hai) annotation (extent=[-100, -10; -120, 10]);
        BondLib.Bonds.fBond B24 annotation (extent=[-100, -10; -80, 10], rotation=
              180);
        BondLib.Bonds.eBond B25 annotation (extent=[-140, -10; -120, 10], rotation=
              180);
        BondLib.Bonds.eBond B26 annotation (extent=[-100, 10; -80, 30], rotation=
              180);
        BondLib.Thermal.Cth Cth5(gamma=Ct, e(start=280)) annotation (extent=[
              -160,10; -140,30],
                             rotation=180);
        BondLib.Bonds.fBond B27 annotation (extent=[-70, 40; -50, 60]);
        CV CV2(G0=ac*hai) annotation (extent=[-50, 40; -30, 60]);
        BondLib.Bonds.eBond B28 annotation (extent=[-30, 40; -10, 60]);
        BondLib.Bonds.fBond B29 annotation (extent=[-60, 20; -40, 40]);
        CV CV3(G0=ag*avix*laix*2*hai) annotation (extent=[-40, 20; -20, 40]);
        BondLib.Bonds.eBond B30 annotation (extent=[-20, 20; 0, 40]);
        BondLib.Bonds.fBond B31 annotation (extent=[-60, 0; -40, 20]);
        CV CV4(G0=ag*apix*hai) annotation (extent=[-40, 0; -20, 20]);
        BondLib.Bonds.eBond B32 annotation (extent=[-20, 0; 0, 20]);
        BondLib.Junctions.J0p6 J0p6_4 annotation (extent=[-80, 10; -60, 30]);
        BondLib.Junctions.J0p6 J0p6_5 annotation (extent=[110, 10; 130, 30]);
        BondLib.Bonds.eBond B33 annotation (extent=[130, 10; 150, 30]);
        BondLib.Bonds.fBond B34 annotation (extent=[80, 40; 100, 60], rotation=180);
        CWcov CWcov1(
          ac=ac,
          hai=hai,
          lambda=lambda,
          pao=pao) annotation (extent=[80, 40; 60, 60]);
        BondLib.Bonds.eBond B35 annotation (extent=[40, 40; 60, 60], rotation=180);
        BondLib.Bonds.eBond B36 annotation (extent=[80, 20; 100, 40]);
        CWveg CWveg1(
          ag=ag,
          hai=hai,
          avix=avix,
          laix=laix,
          lambda=lambda,
          pao=pao) annotation (extent=[60, 20; 80, 40]);
        BondLib.Bonds.fBond B37 annotation (extent=[40, 20; 60, 40]);
        BondLib.Bonds.eBond B38 annotation (extent=[80, 0; 100, 20]);
        CWpond CWpond1(
          ag=ag,
          hai=hai,
          apix=apix,
          lambda=lambda,
          pao=pao) annotation (extent=[60, 0; 80, 20]);
        BondLib.Bonds.fBond B39 annotation (extent=[40, 0; 60, 20]);
        BondLib.Bonds.eBond B40 annotation (extent=[80, -40; 100, -20]);
        CWsoil CWsoil1(
          ag=ag,
          hai=hai,
          apix=apix,
          lambda=lambda,
          pao=pao) annotation (extent=[60, -40; 80, -20]);
        BondLib.Bonds.fBond B41 annotation (extent=[40, -40; 60, -20]);
        CWbulk CWbulk1(
          vai=vai,
          hai=hai,
          lambda=lambda,
          pao=pao) annotation (extent=[0, -130; -20, -110]);
        BondLib.Bonds.fBond B42 annotation (extent=[0, -130; 20, -110], rotation=
              180);
        BondLib.Bonds.eBond B43 annotation (extent=[-40, -130; -20, -110], rotation=
             180);
        Cmoist Cmoist1(
          pao=pao,
          vai=vai,
          e(start=0.015)) annotation (extent=[150, 10; 170, 30]);
        Modelica.Blocks.Sources.Clock Clock1 annotation (extent=[160,80; 180,100]);
        Modelica.Blocks.Nonlinear.Limiter Lim1(uMin=0) 
          annotation (extent=[-182,-132; -170,-120]);
        Junctions.J0p3 J0p3_1 annotation (extent=[-120,30; -100,10]);
        BondLib.Bonds.eBond B44 annotation (extent=[-140,10; -120,30],  rotation=
              180);
        BondLib.Bonds.fBond B45 annotation (extent=[-120,31; -100,51], rotation=
             90);
        Sensors.De tair annotation (extent=[-100,46; -80,66]);
      equation 
        connect(J0p6_3.BondCon1, B1.eBondCon1) annotation (points=[10, 80; 10, 80],
              style(color=8));
        connect(B1.fBondCon1, J0p6_2.BondCon2) annotation (points=[-10, 80; -10, 80],
                 style(color=8));
        connect(B2.fBondCon1, J0p2_2.BondCon1) annotation (points=[-110, 130; -110,
                130], style(color=8));
        connect(mSe1.BondCon1, B2.eBondCon1) annotation (points=[-130, 130; -130,
              130], style(color=8));
        connect(J0p2_2.BondCon2, B3.eBondCon1) annotation (points=[-90, 130; -90,
              130], style(color=8));
        connect(B3.fBondCon1, RA1.BondCon1) annotation (points=[-70, 130; -70, 130],
              style(color=8));
        connect(RA1.BondCon2, B4.fBondCon1) annotation (points=[-50, 130; -50, 130],
              style(color=8));
        connect(mSe2.BondCon1, B5.eBondCon1) annotation (points=[130, 130; 130, 130],
                 style(color=8));
        connect(B5.fBondCon1, J0p2_1.BondCon2) annotation (points=[110, 130; 110,
              130], style(color=8));
        connect(J0p2_1.BondCon1, B6.eBondCon1) annotation (points=[90, 130; 90, 130],
                 style(color=8));
        connect(B6.fBondCon1, CVext1.BondCon1) annotation (points=[70, 130; 70, 130],
                 style(color=8));
        connect(CVext1.BondCon2, B7.fBondCon1) annotation (points=[50, 130; 50, 130],
                 style(color=8));
        connect(B4.eBondCon1, J0p6_2.BondCon6) annotation (points=[-30, 130; -16,
              130; -16, 90], style(color=8));
        connect(B7.eBondCon1, J0p6_3.BondCon5) annotation (points=[30, 130; 16, 130;
                16, 90], style(color=8));
        connect(J0p6_3.BondCon6, B8.fBondCon1) annotation (points=[24, 90; 40, 90],
              style(color=8));
        connect(B8.eBondCon1, Cth1.BondCon1) annotation (points=[60, 90; 60, 90],
            style(color=8));
        connect(B9.eBondCon1, J0p6_2.BondCon5) annotation (points=[-30, 90; -24, 90],
                 style(color=8));
        connect(mSP1.BondCon1, B9.fBondCon1) annotation (points=[-50, 90; -50, 90],
              style(color=8));
        connect(SolarInput1.y,Gain1.u)             annotation (points=[-159,90;
              -150,90], style(color=74, rgbcolor={0,0,135}));
        connect(mSP2.BondCon1, B10.fBondCon1) annotation (points=[-50, 68; -50, 70],
              style(color=8));
        connect(B11.eBondCon1, J0p6_1.BondCon1) annotation (points=[-140, -60; -140,
                -60], style(color=8));
        connect(mSP3.BondCon1, B11.fBondCon1) annotation (points=[-160, -60; -160,
              -60], style(color=8));
        connect(B12.eBondCon1, J0p5_4.BondCon1) annotation (points=[-20, -10; -20,
              -10], style(color=8));
        connect(mSP4.BondCon1, B12.fBondCon1) annotation (points=[-40, -10; -40, -
              10], style(color=8));
        connect(B13.eBondCon1, J0p5_3.BondCon1) annotation (points=[120, -68; 120,
              -68], style(color=8));
        connect(mSP5.BondCon1, B13.fBondCon1) annotation (points=[100, -68; 100, -
              68], style(color=8));
        connect(J0p5_3.BondCon5, B14.fBondCon1) annotation (points=[130, -78; 130,
              -80], style(color=8));
        connect(B14.eBondCon1, Cth2.BondCon1) annotation (points=[130, -100; 130, -
              100], style(color=8));
        connect(B15.eBondCon1, Cth3.BondCon1) annotation (points=[-140, -100; -140,
                -100], style(color=8));
        connect(J0p6_1.BondCon3, B15.fBondCon1) annotation (points=[-134, -70; -140,
                -70; -140, -80], style(color=8));
        connect(J0p6_1.BondCon4, B16.eBondCon1) annotation (points=[-126, -70; -120,
                -70; -120, -80], style(color=8));
        connect(B16.fBondCon1, Soil1.BondCon1) annotation (points=[-120, -100; -120,
                -100], style(color=8));
        connect(J0p5_4.BondCon5, B17.fBondCon1) annotation (points=[-10, -20; -10,
              -60], style(color=8));
        connect(B17.eBondCon1, Cth4.BondCon1) annotation (points=[-10, -80; -10, -
              80], style(color=8));
        connect(B18.eBondCon1, J0p6_1.BondCon2) annotation (points=[-120, -60; -120,
                -60], style(color=8));
        connect(RA2.BondCon2, B18.fBondCon1) annotation (points=[-100, -60; -100, -
              60], style(color=8));
        connect(B20.eBondCon1, J0p5_4.BondCon2) annotation (points=[0,-10; 0,
              -10],
              style(color=8));
        connect(RA3.BondCon2, B20.fBondCon1) annotation (points=[20,-10; 20,-10],
              style(color=8));
        connect(B10.eBondCon1, J0p6_2.BondCon1) annotation (points=[-30, 70; -30,
              80], style(color=8));
        connect(B22.eBondCon1, J0p5_3.BondCon2) annotation (points=[140, -68; 140,
              -68], style(color=8));
        connect(RA4.BondCon2, B22.fBondCon1) annotation (points=[160, -68; 160, -68],
                 style(color=8));
        connect(B19.fBondCon1, RA2.BondCon1) annotation (points=[-80, -60; -80, -60],
                 style(color=8));
        connect(J0p6_2.BondCon3, B19.eBondCon1) annotation (points=[-24, 70; -24,
              60; -36, 60; -36, -60; -60, -60], style(color=8));
        connect(B21.fBondCon1, RA3.BondCon1) annotation (points=[40,-10; 40,-10],
              style(color=8));
        connect(J0p6_3.BondCon4, B21.eBondCon1) annotation (points=[24,70; 24,
              60; 88,60; 88,-10; 60,-10],  style(color=8));
        connect(B23.fBondCon1, RA4.BondCon1) annotation (points=[180, -52; 180, -68],
                 style(color=8));
        connect(B23.eBondCon1, J0p6_3.BondCon2) annotation (points=[180, -32; 180,
              70; 30, 70; 30, 80], style(color=8));
        connect(B24.fBondCon1, CV1.BondCon1) annotation (points=[-100,
              1.22461e-015; -100,0],
              style(color=8));
        connect(CV1.BondCon2, B25.fBondCon1) annotation (points=[-120,0; -120,
              -1.22461e-015],
              style(color=8));
        connect(B25.eBondCon1, J0p6_1.BondCon5) annotation (points=[-140,
              1.22461e-015; -148,1.22461e-015; -148,-34; -134,-34; -134,-50],
                                                   style(color=8));
        connect(B27.fBondCon1, CV2.BondCon1) annotation (points=[-50, 50; -50, 50],
              style(color=8));
        connect(CV2.BondCon2, B28.fBondCon1) annotation (points=[-30, 50; -30, 50],
              style(color=8));
        connect(B28.eBondCon1, J0p6_2.BondCon4) annotation (points=[-10, 50; -4, 50;
                -4, 60; -16, 60; -16, 70], style(color=8));
        connect(B29.fBondCon1, CV3.BondCon1) annotation (points=[-40, 30; -40, 30],
              style(color=8));
        connect(CV3.BondCon2, B30.fBondCon1) annotation (points=[-20, 30; -20, 30],
              style(color=8));
        connect(B30.eBondCon1, J0p5_4.BondCon3) annotation (points=[0, 30; 8, 30; 8,
                4; -14, 4; -14, 0], style(color=8));
        connect(B31.fBondCon1, CV4.BondCon1) annotation (points=[-40, 10; -40, 10],
              style(color=8));
        connect(CV4.BondCon2, B32.fBondCon1) annotation (points=[-20, 10; -20, 10],
              style(color=8));
        connect(B32.eBondCon1, J0p5_3.BondCon3) annotation (points=[0, 10; 26, 10;
              26, -44; 126, -44; 126, -58], style(color=8));
        connect(J0p6_4.BondCon1, B26.fBondCon1) annotation (points=[-80, 20; -80,
              20], style(color=8));
        connect(J0p6_4.BondCon3, B24.eBondCon1) annotation (points=[-74,10; -74,
              -1.22461e-015; -80,-1.22461e-015],
                         style(color=8));
        connect(J0p6_4.BondCon4, B31.eBondCon1) annotation (points=[-66, 10; -60,
              10], style(color=8));
        connect(J0p6_4.BondCon2, B29.eBondCon1) annotation (points=[-60, 20; -60,
              30], style(color=8));
        connect(J0p6_4.BondCon6, B27.eBondCon1) annotation (points=[-66, 30; -66,
              40; -74, 40; -74, 50; -70, 50], style(color=8));
        connect(J0p6_5.BondCon2, B33.fBondCon1) annotation (points=[130, 20; 130,
              20], style(color=8));
        connect(J0p6_5.BondCon6, B34.eBondCon1) annotation (points=[124, 30; 124,
              50; 100, 50], style(color=8));
        connect(B34.fBondCon1, CWcov1.BondCon1) annotation (points=[80, 50; 80, 50],
              style(color=8));
        connect(CWcov1.BondCon2, B35.fBondCon1) annotation (points=[60, 50; 60, 50],
              style(color=8));
        connect(B35.eBondCon1, J0p6_3.BondCon3) annotation (points=[40, 50; 16, 50;
                16, 70], style(color=8));
        connect(B36.eBondCon1, J0p6_5.BondCon5) annotation (points=[100, 30; 116,
              30], style(color=8));
        connect(CWveg1.BondCon2, B36.fBondCon1) annotation (points=[80, 30; 80, 30],
              style(color=8));
        connect(B37.fBondCon1, CWveg1.BondCon1) annotation (points=[60, 30; 60, 30],
              style(color=8));
        connect(J0p5_4.BondCon4, B37.eBondCon1) annotation (points=[-6, 0; 14, 0;
              14, 30; 40, 30], style(color=8));
        connect(B38.eBondCon1, J0p6_5.BondCon1) annotation (points=[100, 10; 106,
              10; 106, 20; 110, 20], style(color=8));
        connect(CWpond1.BondCon2, B38.fBondCon1) annotation (points=[80, 10; 80, 10],
                 style(color=8));
        connect(B39.fBondCon1, CWpond1.BondCon1) annotation (points=[60, 10; 60, 10],
                 style(color=8));
        connect(J0p5_3.BondCon4, B39.eBondCon1) annotation (points=[134, -58; 134,
              -36; 34, -36; 34, 10; 40, 10], style(color=8));
        connect(B40.eBondCon1, J0p6_5.BondCon3) annotation (points=[100, -30; 116,
              -30; 116, 10], style(color=8));
        connect(CWsoil1.BondCon2, B40.fBondCon1) annotation (points=[80, -30; 80, -
              30], style(color=8));
        connect(B41.fBondCon1, CWsoil1.BondCon1) annotation (points=[60, -30; 60, -
              30], style(color=8));
        connect(J0p6_1.BondCon6, B41.eBondCon1) annotation (points=[-126, -50; -126,
                -34; 18, -34; 18, -30; 40, -30], style(color=8));
        connect(B42.fBondCon1, CWbulk1.BondCon1) annotation (points=[0, -120; 0, -
              120], style(color=8));
        connect(J0p6_5.BondCon4, B42.eBondCon1) annotation (points=[124, 10; 124, -
              30; 150, -30; 150, -128; 60, -128; 60, -120; 20, -120], style(color=8));
        connect(CWbulk1.BondCon2, B43.fBondCon1) annotation (points=[-20, -120; -20,
                -120], style(color=8));
        connect(B43.eBondCon1, J0p6_4.BondCon5) annotation (points=[-40, -120; -92,
                -120; -92, 34; -74, 34; -74, 30], style(color=8));
        connect(B33.eBondCon1, Cmoist1.BondCon1) annotation (points=[150, 20; 150,
              20], style(color=8));
        Tdome = Cth1.e + T0;
        Tlake = Cth2.e + T0;
        Tveg = Cth4.e + T0;
        Tsoil = Cth3.e + T0;
        Tgr1 = Soil1.CD1.Cth1.e + T0;
        Tgr2 = Soil1.CD2.Cth1.e + T0;
        Tgr3 = Soil1.CD3.Cth1.e + T0;
        Tair = Cth5.e + T0;
        Hair = Cmoist1.e;
        Hsat = CWbulk1.hum;
        Lim1.u = Hair/Hsat;
        Hrel = 100*Lim1.y;
        connect(Clock1.y, AmbientTemp1.u) annotation (points=[181,90; 190,90; 190,
              130; 182,130], style(color=74, rgbcolor={0,0,135}));
        connect(Clock1.y, NightSkyTemp1.u) annotation (points=[181,90; 190,90; 190,
              114; -190,114; -190,130; -182,130], style(color=74, rgbcolor={0,0,
                135}));
        connect(SolarInput1.u, NightSkyTemp1.u) annotation (points=[-182,90; -190,
              90; -190,130; -182,130], style(color=74, rgbcolor={0,0,127}));
        connect(Glass2.Pin, Absorption1.Prefl) annotation (points=[-140,61;
              -140,68; -166,68; -166,1], style(color=74, rgbcolor={0,0,127}));
        connect(Absorption1.Pin, Glass1.Ptrans) annotation (points=[-174,-1;
              -176,-1; -176,72; -100,72; -100,79], style(color=74, rgbcolor={0,
                0,127}));
        connect(Glass1.Pin, Gain1.y) annotation (points=[-100,99; -100,110;
              -120,110; -120,90; -127,90], style(color=74, rgbcolor={0,0,127}));
        connect(J0p3_1.BondCon2, B26.eBondCon1) annotation (points=[-100,20;
              -100,20], style(color=8, rgbcolor={192,192,192}));
        connect(B44.fBondCon1, J0p3_1.BondCon1) annotation (points=[-120,20;
              -120,20], style(color=8, rgbcolor={192,192,192}));
        connect(Cth5.BondCon1, B44.eBondCon1) annotation (points=[-140,20; -140,
              20], style(color=8, rgbcolor={192,192,192}));
        connect(B45.eBondCon1, J0p3_1.BondCon3) annotation (points=[-110,31;
              -110,30], style(color=8, rgbcolor={192,192,192}));
        connect(tair.BondCon1, B45.fBondCon1) annotation (points=[-100,56; -110,
              56; -110,51], style(color=8, rgbcolor={192,192,192}));
        connect(NightSkyTemp1.y, mSe1.s) annotation (points=[-159,130; -149,130],
            style(color=74, rgbcolor={0,0,127}));
        connect(AmbientTemp1.y, mSe2.s) annotation (points=[159,130; 149,130],
            style(color=74, rgbcolor={0,0,127}));
        connect(Glass1.Pabs, mSP1.s) annotation (points=[-89,90; -69,90], style(
              color=74, rgbcolor={0,0,127}));
        connect(Glass2.Pabs, mSP2.s) annotation (points=[-129,52; -120,52; -120,
              68; -69,68], style(color=74, rgbcolor={0,0,127}));
        connect(Absorption1.Ppond, mSP5.s) annotation (points=[-159,-10; -100,
              -10; -100,-50; 60,-50; 60,-68; 81,-68], style(color=74, rgbcolor=
                {0,0,127}));
        connect(Absorption1.Psoil, mSP3.s) annotation (points=[-181,-10; -186,
              -10; -186,-60; -179,-60], style(color=74, rgbcolor={0,0,127}));
        connect(Absorption1.Pveg, mSP4.s) annotation (points=[-170,-21; -170,
              -26; -70,-26; -70,-10; -59,-10], style(color=74, rgbcolor={0,0,
                127}));
        connect(tair.OutPort1, Cmoist1.s) annotation (points=[-79,56; -60,56;
              -60,64; 180,64; 180,20; 169,20], style(color=74, rgbcolor={0,0,
                127}));
      end Biosphere;
    end Biosphere;
    
    package ModelicaSpice 
      "A set of examples demonstrating the use of Modelica-Spice" 
      annotation (preferedView="info",
        Icon(
          Rectangle(extent=[-99.85,100.65; 100,-100],        style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-57.85,54.65; 54,26],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="A set of examples"),
          Text(
            extent=[-76,36; 74,4],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="demonstrating the use of"),
          Text(
            extent=[-73.55,11.95; 76,-34],
            string="Modelica-Spice",
            style(
              color=42,
              rgbcolor={216,0,0},
              thickness=2,
              fillColor=9,
              rgbfillColor={160,160,164},
              fillPattern=1))),
        Diagram(
          Rectangle(extent=[-99.85,100.65; 100,-100],        style(
              color=0,
              thickness=2,
              fillColor=9)),
          Text(
            extent=[-57.85,54.65; 54,26],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="A set of examples"),
          Text(
            extent=[-78,36; 72,4],
            style(
              color=0,
              thickness=2,
              fillColor=9,
              fillPattern=1),
            string="demonstrating the use of"),
          Text(
            extent=[-75.55,11.95; 74,-34],
            string="Modelica-Spice",
            style(
              color=42,
              rgbcolor={216,0,0},
              thickness=2,
              fillColor=9,
              rgbfillColor={160,160,164},
              fillPattern=1))),
        Documentation(info="<html>
This package contains a set of examples demonstrating the use of <font color=red size=5><b><i>Modelica-Spice</i></b></font>. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
      package InverterCircuit 
        "Simple inverter circuit built from a single NPN bipolar junction transistor" 
        annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-67.85,54.65; 70,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog inverter circuit"),
            Text(
              extent=[-64,36; 64,4],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built using a single"),
            Text(
              extent=[-68,16; 68,-16],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="NPN bipolar transistor")),
          Diagram(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-67.85,54.65; 70,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog inverter circuit"),
            Text(
              extent=[-64,36; 64,4],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built using a single"),
            Text(
              extent=[-68,16; 68,-16],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="NPN bipolar transistor")),
          Documentation(info="<html>
This package contains two separate simple analog electronic inverter circuits built each from a single NPN bipolar junction transistor.  They only differ in the type of transistor being used.  One of them uses a <i>laterally diffused NPN</i>, the other uses a <i>vertically diffused NPN</i>. <p>

On the lateral NPN, the substrate node should be tied to the highest voltage.  On the vertical NPN, it should be tied to the lowest voltage. <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate both circuits during 4e-7 seconds with a communication interval of 5e-9 seconds.  Plot the input and output voltages, <b>vin</b> and <b>vout</b>, together on one graph.  Plot the power dissipated by the two non-linear current sources, <b>Pel</b>, in a separate plot window. <p>

<hr> <p>

<b>Simulation Results:</b> <p>

You should obtain the following graph: <p>

<img src=\"../Images/BG_Ex7.png\"> <p>

The red curve shows the output voltage for the circuit using the laterally diffused NPN, the green curve shows the same voltage for the circuit using the vertically diffused NPN.  They differ primarily during their initialization.  Their analog inversion characteristics are almost identical. <p>

Notice the use of the ramping voltage sources, used also on the two DC supplies.  Without the use of ramping sources, both circuits would exhibit violent high-frequency transient responses initially. <p>

The power dissipated by the two non-linear (internally modulated) current sources is indeed always positive: <p>

<img src=\"../Images/BG_Ex8.png\"> <p>

as proven in [1,2]. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
        model NPNlateral 
          "Simple inverter circuit using a laterally diffused NPN transistor" 
          output Modelica.SIunits.Voltage vin "Input voltage";
          output Modelica.SIunits.Voltage vout "Output voltage";
          output Modelica.SIunits.ActivePower Pel 
            "Electric power dissipated through the two internal current sources";
          output Modelica.SIunits.HeatFlowRate Pth 
            "Total heat dissipated by transistor";
          annotation (
            Icon(
              Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                  color=0,
                  thickness=2,
                  fillColor=9)),
              Text(
                extent=[-67.85,54.65; 70,24],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="Analog inverter circuit"),
              Text(
                extent=[-64,36; 64,4],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="built using a single"),
              Text(
                extent=[-60,16; 58,-16],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="laterally diffused"),
              Text(
                extent=[-68,-4; 68,-36],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="NPN bipolar transistor")),
            Documentation(info="<html>
Analog electronic inverter circuit built from a single laterally diffused NPN bipolar junction transistor.  Details of experimentation and expected simulation results are provided in the documentation of the <b>InverterCircuit</b> package.
</html>
"),         experiment(StopTime=4e-007, Interval=5e-009),
            experimentSetupOutput,
            Diagram);
          Electrical.Analog.Spice.Qnl Q1(
            NF=1,
            NR=1,
            RC=750,
            VJS=0.75,
            Area=1,
            EMin=-100,
            EMax=40,
            VAF=9e30,
            VAR=9e30,
            IS=0.11e-9,
            ISS=0.11e-9,
            GminDC=1e-12,
            IKF=9e30,
            IKR=9e30,
            CJC=3.6e-12,
            CJE=5.7e-12,
            CJS=11e-12,
            XTI=3,
            XTB=0,
            EG=1.16,
            Tnom=298.15,
            XCJC=1,
            RB=1000,
            RBM=500,
            RE=13.3,
            MJC=0.5,
            VJC=0.64,
            MJE=0.5,
            VJE=0.77,
            MJS=0.5,
            TRB1=0.005,
            TRB2=0.0005,
            TRM1=0.005,
            TRM2=0.0005,
            TRC1=0.005,
            TRC2=0.0005,
            TRE1=0.005,
            TRE2=0.0005,
            FC=0.5,
            Level=2,
            BF=50,
            BR=0.1,
            ISC=1e-16,
            ISE=1e-16,
            NC=2,
            NE=1.5,
            TF=1e-12,
            TR=1e-12,
            IRB=9e30,
            VSUB=6,
            rampTime=60e-9) 
                     annotation (extent=[20,-12; 40,12]);
          Electrical.Analog.Spice.R Rin(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=5.6e3) 
                    annotation (extent=[-20,-10; 0,10]);
          Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[-36,-60; -24,-50]);
          Electrical.Analog.Spice.R Rbias(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=10e3) annotation (extent=[0,-40; 20,-12],  rotation=-90);
          Electrical.Analog.Basic.Ground Gnd2 
            annotation (extent=[4,-90; 16,-80]);
          Electrical.Analog.Basic.Ground Gnd3 
            annotation (extent=[34,-26; 46,-16]);
          Electrical.Analog.Spice.R Rload(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=1e3)  annotation (extent=[30,16; 50,44], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd4 
            annotation (extent=[14,60; 26,70]);
          Electrical.Analog.Sensors.VoltageSensor Vo 
            annotation (extent=[80,-40; 60,-20], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd5 
            annotation (extent=[64,-60; 76,-50]);
          Electrical.Analog.Spice.RampingSources.trapV Vin(
            V=6,
            rising=10e-9,
            width=180e-9,
            falling=10e-9,
            period=750e-9,
            nperiod=1,
            offset=0,
            rampTime=60e-9,
            settleTime=40e-9) annotation (extent=[-40,-40; -20,-20], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vbias(        V=-6, rampTime=60e-9) 
            annotation (extent=[0,-70; 20,-50], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vload(        V=6, rampTime=60e-9) 
            annotation (extent=[30,50; 50,70], rotation=90);
          Thermal.HeatTransfer.Sensors.HeatFlowSensor Qtot 
            annotation (extent=[-32,40; -52,60]);
          Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
                298.15) annotation (extent=[-80,40; -60,60]);
        equation 
          vin = Vin.v;
          vout = Vo.v;
          Pel = Q1.Qni.IB0.v*Q1.Qni.IB0.i + Q1.Qni.IC0.v*Q1.Qni.IC0.i;
          Pth = Qtot.Q_flow;
          connect(Rin.n,Rbias. p) annotation (points=[0,0; 10,0; 10,-12],     style(
                color=3, rgbcolor={0,0,255}));
          connect(Q1.B,Rin. n) 
            annotation (points=[20,0; 0,0],  style(color=3, rgbcolor={0,0,255}));
          connect(Q1.E,Gnd3. p) 
            annotation (points=[40,-6; 40,-16], style(color=3, rgbcolor={0,0,255}));
          connect(Rload.n,Q1. C) 
            annotation (points=[40,16; 40,6], style(color=3, rgbcolor={0,0,255}));
          connect(Vo.n,Gnd5. p) 
            annotation (points=[70,-40; 70,-50], style(color=3, rgbcolor={0,0,255}));
          connect(Vo.p,Q1. C) annotation (points=[70,-20; 70,12; 40,12; 40,6], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vin.n,Gnd1. p) annotation (points=[-30,-40; -30,-50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rin.p,Vin. p) annotation (points=[-20,0; -30,0; -30,-20], style(
                color=3, rgbcolor={0,0,255}));
          connect(Rbias.n,Vbias. p) 
            annotation (points=[10,-40; 10,-50], style(color=3, rgbcolor={0,0,255}));
          connect(Vbias.n,Gnd2. p) 
            annotation (points=[10,-70; 10,-80], style(color=3, rgbcolor={0,0,255}));
          connect(Vload.p,Rload. p) 
            annotation (points=[40,50; 40,44], style(color=3, rgbcolor={0,0,255}));
          connect(Vload.n,Gnd4. p) annotation (points=[40,70; 40,80; 20,80; 20,
                70],
              style(color=3, rgbcolor={0,0,255}));
          connect(FixedTemperature1.port, Qtot.port_b) annotation (points=[-59,
                50; -53,50], style(color=42, rgbcolor={191,0,0}));
          connect(Rbias.heatPort, Rin.heatPort) annotation (points=[0,-26; -10,
                -26; -10,-10], style(color=42, rgbcolor={191,0,0}));
          connect(Rin.heatPort, Qtot.port_a) annotation (points=[-10,-10; -26,
                -10; -26,50; -31,50], style(color=42, rgbcolor={191,0,0}));
          connect(Rload.heatPort, Qtot.port_a) annotation (points=[30,30; -26,
                30; -26,50; -31,50], style(color=42, rgbcolor={191,0,0}));
          connect(Q1.heatPort, Rload.heatPort) annotation (points=[30,-12; 16,
                -12; 16,30; 30,30], style(color=42, rgbcolor={191,0,0}));
          connect(Q1.S, Vload.p) annotation (points=[20,-6; 24,-6; 24,50; 40,50],
              style(color=2, rgbcolor={0,255,0}));
        end NPNlateral;
        
        model NPNvertical 
          "Simple inverter circuit using a vertically diffused NPN transistor" 
          output Modelica.SIunits.Voltage vin "Input voltage";
          output Modelica.SIunits.Voltage vout "Output voltage";
          output Modelica.SIunits.ActivePower Pel 
            "Electric power dissipated through the two internal current sources";
          output Modelica.SIunits.HeatFlowRate Pth 
            "Total heat dissipated by transistor";
          annotation (
            Icon(
              Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                  color=0,
                  thickness=2,
                  fillColor=9)),
              Text(
                extent=[-67.85,54.65; 70,24],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="Analog inverter circuit"),
              Text(
                extent=[-64,36; 64,4],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="built using a single"),
              Text(
                extent=[-60,16; 58,-16],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="vertically diffused"),
              Text(
                extent=[-68,-4; 68,-36],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="NPN bipolar transistor")),
            Documentation(info="<html>
Analog electronic inverter circuit built from a single vertically diffused NPN bipolar junction transistor.  Details of experimentation and expected simulation results are provided in the documentation of the <b>InverterCircuit</b> package.
</html>
"),         experiment(StopTime=4e-007, Interval=5e-009),
            experimentSetupOutput,
            Diagram);
          Electrical.Analog.Spice.R Rin(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=5.6e3) 
                    annotation (extent=[-20,-10; 0,10]);
          Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[-36,-60; -24,-50]);
          Electrical.Analog.Spice.R Rbias(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=10e3) annotation (extent=[0,-40; 20,-12],  rotation=-90);
          Electrical.Analog.Basic.Ground Gnd2 
            annotation (extent=[4,-90; 16,-80]);
          Electrical.Analog.Basic.Ground Gnd3 
            annotation (extent=[34,-26; 46,-16]);
          Electrical.Analog.Spice.R Rload(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=1e3)  annotation (extent=[30,16; 50,44], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd4 
            annotation (extent=[14,60; 26,70]);
          Electrical.Analog.Sensors.VoltageSensor Vo 
            annotation (extent=[80,-40; 60,-20], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd5 
            annotation (extent=[64,-60; 76,-50]);
          Electrical.Analog.Spice.Qnv Q1(
            VAF=1e30,
            VAR=1e30,
            IKF=1e30,
            IKR=1e30,
            IS=0.11e-9,
            ISS=0.11e-9,
            NF=1,
            NR=1,
            GminDC=1e-12,
            IRB=0,
            RB=1000,
            RBM=500,
            RC=750,
            RE=13.3,
            CJC=3.6e-12,
            MJC=0.5,
            VJC=0.64,
            CJE=5.7e-12,
            MJE=0.5,
            VJE=0.77,
            CJS=11e-12,
            MJS=0.5,
            VJS=0.75,
            XCJC=1,
            TRB1=0.005,
            TRB2=0.0005,
            TRM1=0.005,
            TRM2=0.0005,
            TRC1=0.005,
            TRC2=0.0005,
            TRE1=0.005,
            TRE2=0.0005,
            Tnom=298.15,
            XTI=3,
            XTB=0,
            EG=1.16,
            Area=1,
            EMin=-100,
            EMax=40,
            BF=50,
            BR=0.1,
            ISC=1e-16,
            ISE=1e-16,
            NC=2,
            NE=1.5,
            FC=0.5,
            TF=1e-12,
            TR=1e-12,
            Level=2,
            VSUB=-6,
            rampTime=60e-9) 
                     annotation (extent=[20,-12; 40,12]);
          Electrical.Analog.Spice.RampingSources.trapV Vin(
            V=6,
            rising=10e-9,
            width=180e-9,
            falling=10e-9,
            period=750e-9,
            nperiod=1,
            offset=0,
            rampTime=60e-9,
            settleTime=40e-9) annotation (extent=[-40,-40; -20,-20], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vbias(        V=-6, rampTime=60e-9) 
            annotation (extent=[0,-70; 20,-50], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vload(        V=6, rampTime=60e-9) 
            annotation (extent=[30,50; 50,70], rotation=90);
          Thermal.HeatTransfer.Sensors.HeatFlowSensor Qtot 
            annotation (extent=[-32,40; -52,60]);
          Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
                298.15) annotation (extent=[-80,40; -60,60]);
        equation 
          vin = Vin.v;
          vout = Vo.v;
          Pel = Q1.Qni.IB0.v*Q1.Qni.IB0.i + Q1.Qni.IC0.v*Q1.Qni.IC0.i;
          Pth = Qtot.Q_flow;
          connect(Rin.n,Rbias. p) annotation (points=[0,0; 10,0; 10,-12],     style(
                color=3, rgbcolor={0,0,255}));
          connect(Vo.n,Gnd5. p) 
            annotation (points=[70,-40; 70,-50], style(color=3, rgbcolor={0,0,255}));
          connect(Rbias.heatPort,Rin. heatPort) annotation (points=[0,-26; -10,
                -26; -10,-10],
                          style(color=42, rgbcolor={191,0,0}));
          connect(Rload.heatPort,Rin. heatPort) annotation (points=[30,30; -26,
                30; -26,-10; -10,-10],
                                   style(color=42, rgbcolor={191,0,0}));
          connect(Q1.B,Rin. n) 
            annotation (points=[20,0; 0,0], style(color=3, rgbcolor={0,0,255}));
          connect(Q1.E,Gnd3. p) 
            annotation (points=[40,-6; 40,-16], style(color=3, rgbcolor={0,0,255}));
          connect(Q1.C,Rload. n) 
            annotation (points=[40,6; 40,16], style(color=3, rgbcolor={0,0,255}));
          connect(Vo.p,Q1. C) annotation (points=[70,-20; 70,12; 40,12; 40,6], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vin.n,Gnd1. p) annotation (points=[-30,-40; -30,-50], style(color=3,
                rgbcolor={0,0,255}));
          connect(Rin.p,Vin. p) annotation (points=[-20,0; -30,0; -30,-20], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vbias.n,Gnd2. p) 
            annotation (points=[10,-70; 10,-80], style(color=3, rgbcolor={0,0,255}));
          connect(Rbias.n,Vbias. p) 
            annotation (points=[10,-40; 10,-50], style(color=3, rgbcolor={0,0,255}));
          connect(Vload.p,Rload. p) 
            annotation (points=[40,50; 40,44], style(color=3, rgbcolor={0,0,255}));
          connect(Vload.n,Gnd4. p) annotation (points=[40,70; 40,80; 20,80; 20,
                70],
              style(color=3, rgbcolor={0,0,255}));
          connect(FixedTemperature1.port, Qtot.port_b) annotation (points=[-59,
                50; -53,50], style(color=42, rgbcolor={191,0,0}));
          connect(Qtot.port_a, Rload.heatPort) annotation (points=[-31,50; -26,
                50; -26,30; 30,30], style(color=42, rgbcolor={191,0,0}));
          connect(Q1.heatPort, Rload.heatPort) annotation (points=[30,-12; 16,
                -12; 16,30; 30,30], style(color=42, rgbcolor={191,0,0}));
          connect(Q1.S, Vbias.p) annotation (points=[20,-6; 20,-50; 10,-50],
              style(color=2, rgbcolor={0,255,0}));
        end NPNvertical;
      end InverterCircuit;
      
      package HighLevelWrapping 
        "Simple analog inverter circuit in high-level wrapping technology" 
        annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-67.85,54.65; 70,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog inverter circuit"),
            Text(
              extent=[-64,36; 64,4],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built using a single"),
            Text(
              extent=[-68,16; 68,-16],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="NPN bipolar transistor"),
            Text(
              extent=[-80,-4; 80,-38],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using high-level wrapping")),
          Diagram(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-67.85,54.65; 70,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog inverter circuit"),
            Text(
              extent=[-64,36; 64,4],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built using a single"),
            Text(
              extent=[-68,16; 68,-16],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="NPN bipolar transistor"),
            Text(
              extent=[-80,-4; 80,-38],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using high-level wrapping")),
          Documentation(info="<html>
This package contains the same circuit as the <b>InverterCircuit</b> package, this time built in high-level wrapping technology. <p>

The <font color=red><b><i>Modelica-Spice</i></b></font> models were built twice, once using bond graph technology all the way to the full-fledged transistor (high-level wrapping), and once more by wrapping lower-level components inside circuit diagrams, building up the complete transistor from these lower-level circuit diagram components (low-level wrapping). <p>

Bond graph practitioners may prefer to stay within the bond graph methodology throughout, whereas electronic circuit designers may find the circuit diagram framework more familiar and easier to work with. <p>

This package contains a second circuit diagram implementation of the laterally diffused NPN bipolar transistor, called <b>Qnl2</b>, built in high-level wrapping technology.  The top-layer <b>NPNwrapped</b> inverter circuit is identical to the <b>NPNlateral</b> circuit of the <b>InverterCircuit</b> package, except that it calls upon <b>Qnl2</b> instead of <b>Qnl</b>. <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate separately the <b>NPNlateral</b> circuit of the <b>InverterCircuit</b> package and the <b>NPNwrapped</b> circuit of this package.  Verify that the four output variables are identical.
</html>
"));
        model Qnl2 
          "Spice-style laterally diffused NPN bipolar transistor in high-level wrapping technology" 
          constant Real inf=Modelica.Constants.inf;
          
          /* General parameters */
          parameter Integer Level(min=1,max=2) = 2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Real Area=1 "Relative area occupied by device";
          
          /* DC model parameters */
          parameter Real BF=100 
            "|DC model parameters|Maximum forward current gain at reference temperature";
          parameter Real BR=1 
            "|DC model parameters|Maximum reverse current gain at reference temperature";
          parameter Modelica.SIunits.Current IS=1e-16 
            "|DC model parameters|Saturation current at reference temperature";
          parameter Modelica.SIunits.Current ISS=IS 
            "|DC model parameters|Saturation current used for current injection";
          parameter Real NF=1 
            "|DC model parameters|Forward current emission coefficient";
          parameter Real NR=1 
            "|DC model parameters|Reverse current emission coefficient";
          parameter Modelica.SIunits.Voltage VSUB=0 
            "|DC model parameters|Substrate potential (if unconnected)";
          parameter Modelica.SIunits.Time rampTime=0 
            "|DC model parameters|Substrate potential ramping time (if unconnected)";
          
          /* Low current beta degradation effect parameters */
          parameter Real C4(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage current coefficient";
          parameter Modelica.SIunits.Current ISC(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-collector leakage saturation current at reference temperature (ISC = C4*IS)";
          parameter Real C2(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitterr leakage current coefficient";
          parameter Modelica.SIunits.Current ISE(min=0) = 0 
            "|Low current beta degradation effect parameters|Base-emitter leakage saturation current at reference temperature (ISE = C2*IS)";
          parameter Real NC=2 
            "|Low current beta degradation effect parameters|Low-current base-collector leakage emission coefficient";
          parameter Real NE=1.5 
            "|Low current beta degradation effect parameters|Low-current base-emitter leakage emission coefficient";
          
          /* High current beta degradation effect parameters */
          parameter Modelica.SIunits.Current IKF=inf 
            "|High current beta degradation effect|Corner for forward beta high-current roll-off";
          parameter Modelica.SIunits.Current IKR=inf 
            "|High current beta degradation effect|Corner for reverse beta high-current roll-off";
          
          /* Base width modulation parameters */
          parameter Modelica.SIunits.Voltage VAF=inf 
            "|Base width modulation parameters|Forward early voltage";
          parameter Modelica.SIunits.Voltage VAR=inf 
            "|Base width modulation parameters|Reverse early voltage";
          
          /* Parasitic resistor parameters */
          parameter Modelica.SIunits.Current IRB=inf 
            "|Parasitic resistor parameters|Current where base resistance falls halfway to minimum value";
          parameter Modelica.SIunits.Resistance RB(min=0) = 0 
            "|Parasitic resistor parameters|Zero-bias base resistance";
          parameter Modelica.SIunits.Resistance RBM(min=0) = RB 
            "|Parasitic resistor parameters|Minimum base resistance at high currents";
          parameter Modelica.SIunits.Resistance RC(min=0) = 0 
            "|Parasitic resistor parameters|Collector resistance";
          parameter Modelica.SIunits.Resistance RE(min=0) = 0 
            "|Parasitic resistor parameters|Emitter resistance";
          
          /* Junction capacitor parameters */
          parameter Modelica.SIunits.Capacitance CJC(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0)";
          parameter Real MJC(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-collector junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJC=0.75 
            "|Junction capacitor parameters|Base-collector built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJE(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0)";
          parameter Real MJE(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Base-emitter junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJE=0.75 
            "|Junction capacitor parameters|Base-emitter built-in potential at reference temperature";
          parameter Modelica.SIunits.Capacitance CJS(min=Modelica.Constants.small) = 1e-12 
            "|Junction capacitor parameters|Zero-bias substrate depletion capacitance at reference temperature";
          parameter Real MJS(min=0.3,max=0.5) = 0.33 
            "|Junction capacitor parameters|Substrate junction grading coefficient";
          parameter Modelica.SIunits.Voltage VJS=0.75 
            "|Junction capacitor parameters|Substrate built-in potential at reference temperature";
          parameter Real XCJC(min=0,max=1) = 1 
            "|Junction capacitor parameters|Fraction of base-collector depletion capacitance connected to internal base node";
          parameter Real FC(min=0,max=1) = 0.5 
            "|Junction capacitor parameters|Depletion capacitance factor for linearization";
          
          /* Transit time parameters */
          parameter Modelica.SIunits.Time TF(min=0) = 0 
            "|Transit time parameters|Ideal forward transit time";
          parameter Modelica.SIunits.Time TR(min=0) = 0 
            "|Transit time parameters|Ideal reverse transit time";
          
          /* Temperature compensation parameters */
          parameter Real TRB1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of zero-bias base resistance";
          parameter Real TRB2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of zero-bias base resistance";
          parameter Real TRM1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of minimum base resistance";
          parameter Real TRM2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of minimum base resistance";
          parameter Real TRC1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of collector resistance";
          parameter Real TRC2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of collector resistance";
          parameter Real TRE1(unit="Ohm/K") = 0 
            "|Temperature compensation parameters|Linear temperature coefficient of emitter resistance";
          parameter Real TRE2(unit="Ohm/K2") = 0 
            "|Temperature compensation parameters|Quadratic temperature coefficient of emitter resistance";
          parameter Modelica.SIunits.Temperature Tnom=300.15 
            "|Temperature compensation parameters|Reference temperature";
          parameter Real XTI=3 
            "|Temperature compensation parameters|Saturation current temperature exponent";
          parameter Real XTB=0 
            "|Temperature compensation parameters|Forward and reverse beta temperature coefficient";
          parameter Modelica.SIunits.Voltage EG=1.11 
            "|Temperature compensation parameters|Energy gap for temperature effect on saturation current";
          
          /* Numerical parameters */
          parameter Real EMin=-100 
            "|Advanced|Numerical parameters|if x < EMin, the exp(x) function is linearized";
          parameter Real EMax=40 
            "|Advanced|Numerical parameters|if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Conductance GminDC(min=0) = 1e-19 
            "|Advanced|Numerical parameters|Leakage conductance";
          
          /* Compiler parameters */
          parameter Boolean enforceStates = true 
            "|Advanced|Compiler parameters|Use electrical variables as states instead of bond graph variables";
          
        protected 
          parameter Modelica.SIunits.Current ISCeff = if ISC > 0 then ISC else C4*IS;
          parameter Modelica.SIunits.Current ISEeff = if ISE > 0 then ISE else C2*IS;
          annotation (
            Diagram(
        Rectangle(extent=[20,-60; 40,-80],  style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[20,-60; 40,-80],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[14,-70; 20,-70],   style(color=83, rgbcolor={255,85,
                      255})),
        Rectangle(extent=[-70,-70; -50,-90],style(color=83, rgbcolor={255,85,255})),
        Text(
        extent=[-70,-70; -50,-90],
        style(color=83, rgbcolor={255,85,255}),
        string="Connected?"),
              Line(points=[-60,-70; -60,-64], style(color=83, rgbcolor={255,85,
                      255}))),
            Icon(
              Line(points=[-10,40; -10,-40]),
              Line(points=[-10,0; -90,0]),
              Line(points=[91,50; 30,50]),
              Line(points=[30,50; -10,10]),
              Line(points=[-10,-10; 30,-50]),
              Line(points=[30,-50; 91,-50]),
              Polygon(points=[30,-50; 24,-36; 16,-44; 30,-50],     style(fillColor=3,
                      fillPattern=1)),
              Line(points=[0,-40; 0,-91],   style(color=42)),
            Text(
              extent=[12,30; 120,-34],
              string="%Area x",
              style(color=3, rgbcolor={0,0,255})),
              Text(extent=[-100,66; 100,92],    string="%name")),
            Documentation(info="<html>
The <b>Qnl2</b> element implements a full-fledges Spice-stye Gummel-Poon model of the laterally diffused NPN bipolar transistor in high-level wrapping technology [1-3]. <p>

The substrate node is an electrical node, but we decided to paint it green so that the substrate connections wouldn't clutter up the circuit diagram unnecessarily. <p>

The heating port may remain unconnected, in which case the device is being simulated at a constant temperature of <b>Tnom</b>. <p>

The substrate port may remain unconnected, in which case the substrate is assumed to be at a fixed potential of <b>VSUB</b>. <p>

<hr>

<pre>
<b>Parameters:</b>

 <font color=red><b>Level:   Transistor modeling level (default value = 2)
            Level = 1: Ebers-Moll model
            Level = 2: Gummel-Poon model</b></font>

 Area:    Relative area occupied by device (default value = 1) <font color=red>Levels 1,2</font>


<b>DC Model Parameters:</b>

 BF:      Maximum forward current gain at reference temperature (default value = 100) <font color=red>Levels 1,2</font>

 BR:      Maximum reverse current gain at reference temperature (default value = 1) <font color=red>Levels 1,2</font>

 IS:      Saturation current at reference temperature (default value = 1e-16 Amp) <font color=red>Levels 1,2</font>

 ISS:     Saturation current for injection (default value = IS Amp) <font color=red>Levels 2</font>

 NF:      Forward current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 NR:      Reverse current emission coefficient (default value = 1) <font color=red>Levels 1,2</font>

 VSUB:    Substrate potential if unconnected (default value = 0 V) <font color=red>Levels 1,2</font>

 rampTime:Substrate potential ramping time if unconnected (default value = 0 s) <font color=red>Levels 1,2</font>


<b>Low Current Beta Degradation Effect Parameters:</b>

 C4:      Base-collector leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISC:     Base-collector leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISCeff = if ISC &gt; 0 then ISC else C4*IS</b></font>

 C2:      Base-emitter leakage current coefficient (default value = 0) <font color=red>Levels 2</font>

 ISE:     Base-emitter leakage saturation current at reference temperature (default value = 0 Amp) <font color=red>Levels 2</font>
          <font color=blue><b>ISEeff = if ISE &gt; 0 then ISE else C2*IS</b></font>

 NC:      Low-current base-collector leakage emission coefficient (default value = 2) <font color=red>Levels 2</font>

 NE:      Low-current base-emitter leakage emission coefficient (default value = 1.5) <font color=red>Levels 2</font>


<b>High Current Beta Degradation Effect Parameters:</b>

 IKF:     Corner for forward beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>

 IKR:     Corner for reverse beta high-current roll-off (default value = &infin; Amp) <font color=red>Levels 2</font>


<b>Base Width Modulation Parameters:</b>
  
 VAF:     Forward early voltage (default value = &infin; Volt) <font color=red>Levels 1,2</font>

 VAR:     Reverse early voltage (default value = &infin; Volt) <font color=red>Levels 2</font>


<b>Parasitic Resistor Parameters:</b>

 IRB:     Current where base resistance falls halfway to minimum value (default value = &infin; Amp) <font color=red>Levels 2</font>

 RB:      Zero-bias base resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RBM:     Minimum base resistance at high currents (default value = RB Ohm) <font color=red>Levels 2</font>

 RC:      Collector resistance (default value = 0 Ohm) <font color=red>Levels 2</font>

 RE:      Emitter resistance (default value = 0 Ohm) <font color=red>Levels 2</font>


<b>Junction Capacitor Parameters:</b>

 CJC:     Zero-bias base-collector depletion capacitance at reference temperature (CJC > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJC:     Base-collector junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJC:     Base-collector built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJE:     Zero-bias base-emitter depletion capacitance at reference temperature (CJE > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJE:     Base-emitter junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJE:     Base-emitter built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 CJS:     Zero-bias substrate depletion capacitance at reference temperature (CJS > 0) (default value = 1e-12 F) <font color=red>Levels 1,2</font>

 MJS:     Substrate junction grading coefficient (default value = 0.33) <font color=red>Levels 1,2</font>

 VJS:     Substrate built-in potential at reference temperature (default value = 0.75 Volt) <font color=red>Levels 1,2</font>

 XCJC:    Fraction of base-collector depletion capacitance connected to internal base node (default value = 1) <font color=red>Levels 1,2</font>

 FC:      Depletion capacitance factor for linearization (default value = 0.5) <font color=red>Levels 2</font>


<b>Transit Time Parameters:</b>

 TF:      Ideal forward transit time (default value = 0 sec) <font color=red>Levels 1,2</font>

 TR:      Ideal reverse transit time (default value = 0 sec) <font color=red>Levels 1,2</font>


<b>Temperature Compensation and Area Parameters:</b>

 TRB1:    Linear temperature coefficient of zero-bias base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRB2:    Quadratic temperature coefficient of zero-bias base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRM1:    Linear temperature coefficient of mimimum base resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRM2:    Quadratic temperature coefficient of minimum base resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRC1:    Linear temperature coefficient of collector resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRC2:    Quadratic temperature coefficient of collector resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 TRE1:    Linear temperature coefficient of emitter resistance (default value = 0 Ohm/K) <font color=red>Levels 2</font>

 TRE2:    Quadratic temperature coefficient of emitter resistance (default value = 0 Ohm/(K<sup>2</sup>)) <font color=red>Levels 2</font>

 Tnom:    Reference temperature (default value = 300.15 K) <font color=red>Levels 1,2</font>

 XTI:     Saturation current temperature exponent (default value = 3) <font color=red>Levels 1,2</font>

 XTB:     Forward and reverse beta temperature coefficient (default value = 0) <font color=red>Levels 1,2</font>

 EG:      Energy gap for temperature effect on saturation current (default value = 1.11 Volt) <font color=red>Levels 1,2</font>


<b>Numerical Parameters:</b>

 EMin:    Minimum exponent for linearization of junction current (default value = -100) <font color=red>Levels 1,2</font>

 EMax:    Maximum exponent for linearization of junction current (default value = 40) <font color=red>Levels 1,2</font>

 GminDC:  Leakage conductance (default value = 1e-19 Mho) <font color=red>Levels 1,2</font>


<b>Compiler Parameters:</b>

 enforceStates:  State selector (default value = true) <font color=red>Levels 1,2</font>
                   <font color=blue><b>enforceStates = true:   Use (external) capacitive voltages as state variables
                   enforceStates = false:  Use (internal) bond graph efforts as state variables</b></font>
</pre> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li>Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York.
<li>Hild, D.R. and F.E. Cellier (1994), \"Object-oriented electronic circuit modeling using Dymola,\"
    <i>Proc. OOS'94, SCS Object Oriented Simulation Conference</i>, Tempe, AZ, pp.68-75.
<li>Hild, D.R. (1993), <i>Circuit Modeling in Dymola</i>, MS Thesis, Dept. of Electr. &amp; Comp. Engr.,
    University of Arizona, Tucson.
<li> Massobrio, G. and P. Antognetti (1993), <i>Semiconductor Device Modeling with Spice</i>,
     2<sup>nd</sup> edition, McGraw Hill, New York.
<li>Schweisguth, M.C. and F.E. Cellier (1999), \"<a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/BJT.pdf\">A
    bond graph model of the bipolar junction transistor</a>,\" <i>Proc. SCS Intl. Conf. on Bond Graph Modeling</i>,
    San Francisco, CA, pp.344-349.
<li>Schweisguth, M.C. (1997), <a href=\"http://www.inf.ethz.ch/~fcellier/Publikationen/schweisguth_ms.pdf\"><i>Semiconductor
    Modeling with Bondgraphs</i></a>, MS Thesis, Dept. of Electr. &amp; Comp. Engr., University of Arizona, Tucson.
</ol> 
</html>
"));
        public 
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation (extent=[
                -90,-10; -110,10]);
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation (extent=[90,
                40; 110,60]);
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation (extent=[90,
                -40; 110,-60]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "may remain unconnected" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Spice.NPNlat Qnl(
            BF=BF,
            BR=BR,
            IS=IS,
            NF=NF,
            NR=NR,
            GminDC=GminDC,
            NC=NC,
            NE=NE,
            VAF=VAF,
            VAR=VAR,
            IKF=IKF,
            IKR=IKR,
            IRB=IRB,
            RB=RB,
            RBM=RBM,
            RC=RC,
            RE=RE,
            CJC=CJC,
            MJC=MJC,
            VJC=VJC,
            CJE=CJE,
            MJE=MJE,
            VJE=VJE,
            MJS=MJS,
            VJS=VJS,
            XCJC=XCJC,
            TF=TF,
            TR=TR,
            TRB1=TRB1,
            TRB2=TRB2,
            TRM1=TRM1,
            TRM2=TRM2,
            TRC1=TRC1,
            TRC2=TRC2,
            TRE1=TRE1,
            TRE2=TRE2,
            Tnom=Tnom,
            XTI=XTI,
            XTB=XTB,
            EG=EG,
            Area=Area,
            EMin=EMin,
            EMax=EMax,
            FC=FC,
            Level=Level,
            ISC=ISCeff,
            ISE=ISEeff,
            ISS=ISS,
            CJS=CJS)   annotation (extent=[-20,-20; 20,20]);
        protected 
          Bonds.Bond B1         annotation (extent=[-40,-10; -20,10]);
          Bonds.Bond B3         annotation (extent=[-10,20; 10,40], rotation=-90);
          Bonds.Bond B4         annotation (extent=[-10,-40; 10,-20], rotation=-90);
          Electrical.Analog.Interfaces.El2BG El2BG1 
            annotation (extent=[-80,-10; -60,10]);
          Electrical.Analog.Interfaces.El2BG El2BG2 
            annotation (extent=[80,40; 60,60]);
          Electrical.Analog.Interfaces.BG2El BG2El1 
            annotation (extent=[60,-60; 80,-40]);
          Bonds.eBond B2         annotation (extent=[20,-10; 40,10]);
          Bonds.Bond B5         annotation (extent=[-40,-20; -20,0],rotation=-180);
        public 
          Electrical.Analog.Spice.Interfaces.SPin S 
            "Substrate (may remain unconnected)" 
            annotation (extent=[-110,-60; -90,-40]);
        protected 
          Electrical.Analog.Interfaces.BG2El BG2El2 
            annotation (extent=[-30,-40; -50,-20], rotation=270);
          Thermal.HeatTransfer.Interfaces.BG2Heat BG2Heat1 
            annotation (extent=[40,-10; 60,10],   rotation=0);
        public 
          Thermal.HeatTransfer.Switches.HeatBranch Sw1 
            annotation (extent=[-10,-80; 10,-60], rotation=-90);
          Thermal.HeatTransfer.Sources.FixedTemperature T_nom(T=Tnom) 
            annotation (extent=[80,-100; 60,-80]);
          Electrical.Analog.Ideal.IdealCommutingSwitch2 Sw2 
            annotation (extent=[-50,-40; -70,-60], rotation=0);
          Electrical.Analog.Spice.RampingSources.V Vsub(V=VSUB, rampTime=
                rampTime) annotation (extent=[-100,-80; -80,-60], rotation=270);
          Electrical.Analog.Basic.Ground grd 
            annotation (extent=[-100,-102; -80,-84]);
        equation 
          if cardinality(heatPort) <= 1 then
            Sw1.control = false;
          else
            Sw1.control = true;
          end if;
          
          if cardinality(S) <= 1 then
            Sw2.control = false;
          else
            Sw2.control = true;
          end if;
          connect(El2BG1.BondCon1,B1. BondCon1) annotation (points=[-60,0;
                -39.8,0],
              style(color=8, rgbcolor={192,192,192}));
          connect(El2BG1.p,B)  annotation (points=[-80,0; -100,0], style(color=3,
                rgbcolor={0,0,255}));
          connect(El2BG2.p,C)  annotation (points=[80,50; 100,50], style(color=3,
                rgbcolor={0,0,255}));
          connect(B3.BondCon1,El2BG2. BondCon1) annotation (points=[
                -6.00057e-016,39.8; -6.00057e-016,50; 60,50],
                                                style(color=8, rgbcolor={192,192,
                  192}));
          connect(BG2El1.n,E)  annotation (points=[80,-50; 100,-50], style(color=3,
                rgbcolor={0,0,255}));
          connect(B4.BondCon2,BG2El1. BondCon1) annotation (points=[
                6.12303e-016,-40; 40,-40; 40,-50; 60,-50],
                                            style(color=8, rgbcolor={192,192,192}));
          connect(B2.eBondCon1, BG2Heat1.BondCon1) annotation (points=[40,0; 40,
                0], style(color=8, rgbcolor={192,192,192}));
          connect(Sw1.n1, T_nom.port) annotation (points=[5,-81; 12,-81; 12,-90;
                59,-90], style(color=42, rgbcolor={191,0,0}));
          connect(Sw1.n2, heatPort) annotation (points=[6.73533e-016,-81;
                6.73533e-016,-82.5; 0,-82.5; 0,-100], style(color=42, rgbcolor=
                  {191,0,0}));
          connect(Sw1.p, BG2Heat1.port_b) annotation (points=[-6.73533e-016,-59;
                0,-54; 0,-50; 20,-50; 20,-30; 70,-30; 70,0; 60,0], style(color=
                  42, rgbcolor={191,0,0}));
          connect(B5.BondCon2, BG2El2.BondCon1) annotation (points=[-40,-10;
                -40,-20], style(color=8, rgbcolor={192,192,192}));
          connect(Sw2.p, BG2El2.n) annotation (points=[-50,-50; -40,-50; -40,
                -40], style(color=3, rgbcolor={0,0,255}));
          connect(Sw2.n2, S) annotation (points=[-70,-50; -100,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vsub.p, Sw2.n1) annotation (points=[-90,-60; -90,-55; -70,-55],
              style(color=3, rgbcolor={0,0,255}));
          connect(Vsub.n, grd.p) annotation (points=[-90,-80; -90,-84], style(
                color=3, rgbcolor={0,0,255}));
          connect(B1.BondCon2, Qnl.B) annotation (points=[-20,0; -20,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(Qnl.S, B5.BondCon1) annotation (points=[-20,-10; -20.1,-10;
                -20.1,-10; -20.2,-10], style(color=8, rgbcolor={192,192,192}));
          connect(B4.BondCon1, Qnl.E) annotation (points=[-6.00057e-016,-20.2;
                0,-20.2; 0,-20], style(color=8, rgbcolor={192,192,192}));
          connect(B2.fBondCon1, Qnl.H) annotation (points=[20,0; 20,0], style(
                color=8, rgbcolor={192,192,192}));
          connect(B3.BondCon2, Qnl.C) annotation (points=[6.12303e-016,20; 0,20],
              style(color=8, rgbcolor={192,192,192}));
        end Qnl2;
        
        model NPNwrapped 
          "Lateral NPN inverter circuit using high-level wrapping" 
          output Modelica.SIunits.Voltage vin "Input voltage";
          output Modelica.SIunits.Voltage vout "Output voltage";
          output Modelica.SIunits.ActivePower Pel 
            "Electric power dissipated through the two internal current sources";
          output Modelica.SIunits.HeatFlowRate Pth 
            "Total heat dissipated by transistor";
          annotation (
            Icon(
              Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                  color=0,
                  thickness=2,
                  fillColor=9)),
              Text(
                extent=[-67.85,72.65; 70,42],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="Analog inverter circuit"),
              Text(
                extent=[-64,54; 60,22],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="built from a single"),
              Text(
                extent=[-60,34; 58,2],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="laterally diffused"),
              Text(
                extent=[-68,14; 68,-18],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="NPN bipolar transistor"),
              Text(
                extent=[-80,-4; 80,-38],
                style(
                  color=0,
                  thickness=2,
                  fillColor=9,
                  fillPattern=1),
                string="using high-level wrapping")),
            Documentation(info="<html>
Analog electronic inverter circuit built from a single laterally diffused NPN bipolar junction transistor in high-level wrapping technology. <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate separately the <b>NPNlateral</b> circuit of the <b>InverterCircuit</b> package and the <b>NPNwrapped</b> circuit of this package.  Verify that the four output variables are identical.
</html>
"),         experiment(StopTime=4e-007, Interval=5e-009),
            experimentSetupOutput,
            Diagram);
          Electrical.Analog.Spice.R Rin(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=5.6e3) 
                    annotation (extent=[-20,-10; 0,10]);
          Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[-36,-60; -24,-50]);
          Electrical.Analog.Spice.R Rbias(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=10e3) annotation (extent=[0,-40; 20,-12],  rotation=-90);
          Electrical.Analog.Basic.Ground Gnd2 
            annotation (extent=[4,-90; 16,-80]);
          Electrical.Analog.Basic.Ground Gnd3 
            annotation (extent=[34,-26; 46,-16]);
          Electrical.Analog.Spice.R Rload(
            TR1=0,
            TR2=0,
            Area=1,
            Tnom=298.15,
            R=1e3)  annotation (extent=[30,16; 50,44], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd4 
            annotation (extent=[14,60; 26,70]);
          Electrical.Analog.Sensors.VoltageSensor Vo 
            annotation (extent=[80,-40; 60,-20], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd5 
            annotation (extent=[64,-60; 76,-50]);
          Electrical.Analog.Spice.RampingSources.trapV Vin(
            V=6,
            rising=10e-9,
            width=180e-9,
            falling=10e-9,
            period=750e-9,
            nperiod=1,
            offset=0,
            rampTime=60e-9,
            settleTime=40e-9) annotation (extent=[-40,-40; -20,-20], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vbias(        V=-6, rampTime=60e-9) 
            annotation (extent=[0,-70; 20,-50], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vload(        V=6, rampTime=60e-9) 
            annotation (extent=[30,50; 50,70], rotation=90);
          Qnl2 Q1(
            BF=50,
            BR=0.1,
            IS=0.11e-9,
            ISS=0.11e-9,
            NF=1,
            NR=1,
            GminDC=1e-12,
            ISC=1e-16,
            ISE=1e-16,
            NC=2,
            NE=1.5,
            VAF=9e30,
            VAR=9e30,
            IKF=9e30,
            IKR=9e30,
            IRB=0,
            RB=1000,
            RBM=500,
            RC=750,
            RE=13.3,
            CJC=3.6e-12,
            CJE=5.7e-12,
            CJS=11e-12,
            MJC=0.5,
            VJC=0.64,
            MJE=0.5,
            VJE=0.77,
            MJS=0.5,
            VJS=0.75,
            XCJC=1,
            FC=0.5,
            TF=1e-12,
            TR=1e-12,
            TRB1=0.005,
            TRB2=0.0005,
            TRM1=0.005,
            TRM2=0.0005,
            TRC1=0.005,
            TRC2=0.0005,
            TRE1=0.005,
            TRE2=0.0005,
            Tnom=298.15,
            XTI=3,
            XTB=0,
            EG=1.16,
            Area=1,
            Level=2,
            EMin=-100,
            EMax=40,
            VSUB=6,
            rampTime=60e-9) 
                     annotation (extent=[20,-12; 40,12]);
          Thermal.HeatTransfer.Sensors.HeatFlowSensor Qtot 
            annotation (extent=[-32,40; -52,60]);
          Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
                298.15) annotation (extent=[-80,40; -60,60]);
        equation 
          vin = Vin.v;
          vout = Vo.v;
          Pel = -Q1.Qnl.Qni.IB0.e1*Q1.Qnl.Qni.IB0.f1 - Q1.Qnl.Qni.IC0.e1*Q1.Qnl.Qni.IC0.f1;
          Pth = Qtot.Q_flow;
          connect(Rin.n,Rbias. p) annotation (points=[0,0; 10,0; 10,-12],     style(
                color=3, rgbcolor={0,0,255}));
          connect(Vo.n,Gnd5. p) 
            annotation (points=[70,-40; 70,-50], style(color=3, rgbcolor={0,0,255}));
          connect(Rbias.heatPort,Rin. heatPort) annotation (points=[0,-26; -10,
                -26; -10,-10],
                          style(color=42, rgbcolor={191,0,0}));
          connect(Vo.p,Rload. n) annotation (points=[70,-20; 70,12; 40,12; 40,
                16],
              style(color=3, rgbcolor={0,0,255}));
          connect(Rload.heatPort,Rin. heatPort) annotation (points=[30,30; -26,
                30; -26,-10; -10,-10],
                                   style(color=42, rgbcolor={191,0,0}));
          connect(Vin.n,Gnd1. p) annotation (points=[-30,-40; -30,-50], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Rin.p,Vin. p) annotation (points=[-20,0; -30,0; -30,-20], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vbias.n,Gnd2. p) annotation (points=[10,-70; 10,-80], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Rbias.n,Vbias. p) annotation (points=[10,-40; 10,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vload.p,Rload. p) 
            annotation (points=[40,50; 40,44], style(color=3, rgbcolor={0,0,255}));
          connect(Vload.n,Gnd4. p) annotation (points=[40,70; 40,80; 20,80; 20,
                70],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q1.B, Rin.n) annotation (points=[20,0; 0,0], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q1.E, Gnd3.p) annotation (points=[40,-6; 40,-16], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Q1.C, Rload.n) annotation (points=[40,6; 40,16], style(color=
                  3, rgbcolor={0,0,255}));
          connect(FixedTemperature1.port, Qtot.port_b) annotation (points=[-59,
                50; -53,50], style(color=42, rgbcolor={191,0,0}));
          connect(Qtot.port_a, Rload.heatPort) annotation (points=[-31,50; -26,
                50; -26,30; 30,30], style(color=42, rgbcolor={191,0,0}));
          connect(Q1.heatPort, Rload.heatPort) annotation (points=[30,-12; 16,
                -12; 16,30; 30,30], style(color=42, rgbcolor={191,0,0}));
        end NPNwrapped;
      end HighLevelWrapping;
      
      package HifiOperationalAmplifier 
        "Hifi commerecial operational amplifier circuit built from 6 vertical NPNs and 6 lateral PNPs" 
        annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-73.85,54.65; 76,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Hifi operational amplifier"),
            Text(
              extent=[-76,36; 80,4],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built from 6 vertical NPNs"),
            Text(
              extent=[-68,16; 60,-16],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="and 6 lateral PNPs"),
            Text(
              extent=[-46,-12; 76,-46],
              style(
                color=42,
                rgbcolor={172,0,0},
                thickness=2,
                fillColor=9,
                rgbfillColor={160,160,164},
                fillPattern=1),
              string="Transient analysis"),
            Text(
              extent=[-44,-28; 42,-62],
              style(
                color=42,
                rgbcolor={172,0,0},
                thickness=2,
                fillColor=9,
                rgbfillColor={160,160,164},
                fillPattern=1),
              string="DC analysis"),
            Text(
              extent=[-42,-44; 40,-78],
              style(
                color=42,
                rgbcolor={172,0,0},
                thickness=2,
                fillColor=9,
                rgbfillColor={160,160,164},
                fillPattern=1),
              string="AC analysis"),
            Text(
              extent=[-64,-24; -46,-46],
              style(color=42, rgbcolor={172,0,0}),
              string="*"),
            Text(
              extent=[-64,-40; -46,-62],
              style(color=42, rgbcolor={172,0,0}),
              string="*"),
            Text(
              extent=[-64,-56; -46,-78],
              style(color=42, rgbcolor={172,0,0}),
              string="*")),
          Diagram(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-73.85,54.65; 76,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Hifi operational amplifier"),
            Text(
              extent=[-76,36; 80,4],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built from 6 vertical NPNs"),
            Text(
              extent=[-68,16; 60,-16],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="and 6 lateral PNPs")),
          Documentation(info="<html>
This models shows an inverter circuit built from a high-gain operational amplifier.  Such circuits used to be standard modeling elements of analog computers [1]. <p>

Three device parameters were pulled out all the way to the top level: <p>

<ol>
<li><b>Level</b> allows the user to choose between a full-fledged <i>Gummel-Poon</i> transistor model (<b>Level=2</b>) and a simplified <i>Ebers-Moll</i> transistor model (<b>Level=1</b>).
<li><b>rampTime</b> enables the user to change the initial ramping period centrally with a single parameter assignment.
<li><b>settleTime</b> enables the user to change the initial settling period centrally with a single parameter assignment.
</ol> <p>

<hr> <p>

<b>Experimentation:</b> <p>

<b>1. Transient Analysis:</b> <p>

Simulate the model during 0.0001 seconds.  Store only the output variables.  Plot the input voltage, <b>vin</b>, together with the output voltage, <b>vout</b>, on a single plot. <p>

We expect the following results: <p>

<img src=\"../Images/BG_Ex9.png\"> <p>

Here, the ramping trapezoidal input is clearly visible.  In order to reach a true DC OP-point, both the <b>rampTime</b> and the <b>settleTime</b> would have to be chosen considerably longer, but since the circuit converged with the given values, it wasn't necessary to do so. <p>

<hr> <p>

<b>2. DC Analysis:</b> <p>

Before we can attempt an AC analysis, we need to compute a <b>DC OP point</b>.  In <font color=red size=4><b><i>ModelicaSpice</i></b></font>, we would usually enhance the settling time to the full length of the simulation and repeat the transient analysis with the initial ramping and settling periods only. <p>

However in the given case, as clearly visible from the simulation results of the transient analysis, the final value of the transient analysis above represents a perfectly good DC OP point. <p>

The DC OP point, i.e., the final state of the last simulation performed, is automatically saved by Dymola onto the file <b>dsfinal.txt</b>. <p>

<hr> <p>

<b>3. AC Analysis:</b> <p>

In order to perform an AC analysis, we need to return quickly to the <i>Modeling</i> window and select the <b>Inverter</b> model, instead of the previously selected <b>Experiment</b> model.  The <i>Inverter</i> model has been designed as a SISO model.  We then return to the <i>Simulation</i> window. <p>

Now, we need to use the final <b>DC OP point</b> that we saved before as new <b>initial conditions</b>.  To this end, we choose from the <i>Simulation</i> pull-down menu the command <i>Continue/ImportInitial...</i> and point at the previously stored file <i>dsfinal.txt</i>. <p>

We have now set the correct initial conditions and are ready to <b>linearize</b> the model around these initial conditions.  The <i>Linearize</i> command can also be found in the <i>Simulation</i> pull-down menu. <p>

The linearized model is stored by Dymola in the file <b>dslin.mat</b>. <p>

The AC analysis is actually performed in <font color=red><b>Matlab</b></font> rather than in <font color=red><b>Dymola</b></font>.  We enter Matlab, set the default directory to the current working directory (using the <b>cd</b> command), then run the file <b>OpAmp.m</b>: <p>

<img src=\"../Images/BG_Ex11.png\"> <p>

This file loads the linearized model into Matlab, and extracts the four system matrices from the combined system matrix stored by Dymola.  Each BJT represents a 3<sup>rd</sup> order model.  There are 12 BJTs in the <i>Inverter</i> circuit, plus one extra capacitor.  Thus the overall model is of order 37. <p>

The m-file then makes a Matlab system definition out of the four individual system matrices (this requires the <font color=red><b>Controls toolbox</b></font> of Matlab).  The file then draws a Bode diagram: <p>

<img src=\"../Images/BG_Ex12.png\"> <p>

Evidently, the Hifi operational amplifier has a band width of roughly 1 MHz.

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, ISBN: 0-387-97502-0, 755 p.
</ol> <p>
</html>
"));
        model OpAmp "Hifi operational amplifier circuit" 
          parameter Integer Level=2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Modelica.SIunits.Time rampTime=3e-6 "Ramping period";
          parameter Boolean enforceStates = true 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
          annotation (
            Diagram(
              Ellipse(extent=[58,-12; 62,-16], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255})),
              Ellipse(extent=[58,56; 62,52], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Text(
                extent=[-16,94; -2,88],
                string="Vcc",
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1)),
              Text(
                extent=[-34,-76; -20,-82],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="Vee"),
              Text(
                extent=[-78,12; -62,6],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="-VI"),
              Text(
                extent=[8,12; 24,6],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="+VI"),
              Text(
                extent=[98,26; 114,20],
                style(
                  color=10,
                  rgbcolor={95,95,95},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1),
                string="VO"),
              Ellipse(extent=[-2,-80; 2,-84],    style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255}))),
            Coordsys(extent=[-120,-100; 120,100]),
            Icon(
              Ellipse(extent=[-78,64; -10,-56], style(
                  color=3,
                  rgbcolor={0,0,255},
                  thickness=2)),
              Rectangle(extent=[-34,66; 80,-60], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[80,0; -36,62], style(
                  color=3,
                  rgbcolor={0,0,255},
                  thickness=2,
                  fillPattern=1)),
              Line(points=[80,0; -34,-54], style(
                  color=3,
                  rgbcolor={0,0,255},
                  thickness=2,
                  fillPattern=1)),
              Line(points=[-74,30; -110,30], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[-72,-30; -110,-30], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[0,44; 0,90],     style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[0,-38; 0,-90],     style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[80,0; 110,0],style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Text(
                extent=[-76,60; -22,6],
                style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1),
                string="-"),
              Text(
                extent=[-64,-12; -32,-48],
                style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1),
                string="+"),
              Text(
                extent=[22,84; 106,46],
                style(color=3, rgbcolor={0,0,255}),
                string="%name"),
              Line(points=[-60,-48; -60,-100], style(
                  color=42,
                  rgbcolor={203,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1))),
            Documentation(info="<html>
This circuit represents a commercial Hifi operational amplifier built from 12 individual bipolar transistors, six of which are vertically diffused NPN transistors, whereas the other six are laterally diffused PNP transistors. <p>

Each transistor has a (red) heat port and a (green) substrate node.  The heat port uses a different color embracing the philosophy of Modelica libraries, which distinguish different connectors by color to make it visibly unlikely that impossible connections are being made by the user.  The substrate node is an electrical node and should therefore be drawn in blue.  However, we decided to use a different color to prevent the circuit from becoming unreadable due to an abundance of connections. <p>

All internal heat ports need to be tied to the external heat port.  All internal substrate nodes need to be tied to the lowest voltage, i.e., the external <b>Vee</b> port. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, pp. 216 ff.
</ol>
</html>
"));
          Modelica.Electrical.Analog.Interfaces.Pin vi_n 
            "Negative input electrical connector" 
            annotation (extent=[-130,20; -110,40]);
          Modelica.Electrical.Analog.Interfaces.Pin vi_p 
            "Positive input electrical connector" 
            annotation (extent=[-130,-40; -110,-20]);
          Modelica.Electrical.Analog.Interfaces.Pin vo 
            "Output electrical connector" 
            annotation (extent=[110,-10; 130,10]);
          Modelica.Electrical.Analog.Interfaces.Pin vcc 
            "High support electrical connector" 
            annotation (extent=[-10,90; 10,110]);
          Modelica.Electrical.Analog.Interfaces.Pin vee 
            "Low support electrical connector" 
            annotation (extent=[-10,-110; 10,-90]);
          Electrical.Analog.Spice.Qnv Q4(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[80,42; 100,66]);
          Electrical.Analog.Spice.R R1(
            Tnom=298.15,
            Area=1,
            R=1.5e6,
            TR1=0,
            TR2=0)  annotation (extent=[-110,-62; -90,-42], rotation=-90);
          Electrical.Analog.Spice.C C1(
            Tnom=298.15,
            Area=1,
            C=10e-12,
            TC1=0,
            TC2=0,
            enforceStates=enforceStates) 
                    annotation (extent=[34,-24; 54,-4]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "Thermal connector" 
            annotation (extent=[-70,-90; -50,-110],  rotation=-90);
          Electrical.Analog.Spice.Qnv Q5(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[40,28; 60,52]);
          Electrical.Analog.Spice.Qnv Q8(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[40,-2; 60,22]);
          Electrical.Analog.Spice.Qnv Q10(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[40,-48; 60,-24]);
          Electrical.Analog.Spice.Qnv Q11(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[-30,-72; -50,-48]);
          Electrical.Analog.Spice.Qnv Q12(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[-20,-72; 0,-48]);
          Electrical.Analog.Spice.Qpl Q1(
            BF=125,
            BR=0.1,
            IS=1.1e-15,
            NF=1,
            NR=1,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            NC=2,
            NE=1.5,
            VAF=30,
            VAR=9e30,
            IKF=3.6e-3,
            IKR=9e30,
            RB=778,
            RBM=57.6,
            RC=450,
            RE=23.3,
            CJC=0.055e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.652e-12,
            MJE=0.3,
            VJE=0.77,
            CJS=0.01e-12,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            FC=0.5,
            TF=50e-12,
            TR=1e-6,
            TRB1=0.005,
            TRB2=0,
            TRM1=0.005,
            TRM2=0,
            TRC1=0.005,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            Tnom=298.15,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            Area=2,
            EMin=-100,
            EMax=40,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            Level=Level,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[-80,80; -100,56]);
          Electrical.Analog.Spice.Qpl Q2(
            BF=125,
            BR=0.1,
            IS=1.1e-15,
            NF=1,
            NR=1,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            NC=2,
            NE=1.5,
            VAF=30,
            VAR=9e30,
            IKF=3.6e-3,
            IKR=9e30,
            RB=778,
            RBM=57.6,
            RC=450,
            RE=23.3,
            CJC=0.055e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.652e-12,
            MJE=0.3,
            VJE=0.77,
            CJS=0.01e-12,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            FC=0.5,
            TF=50e-12,
            TR=1e-6,
            TRB1=0.005,
            TRB2=0,
            TRM1=0.005,
            TRM2=0,
            TRC1=0.005,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            Tnom=298.15,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            EMin=-100,
            EMax=40,
            Area=1,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            Level=Level,
            VSUB=-5,
            rampTime=rampTime) 
                    annotation (extent=[-46,80; -26,56]);
          Electrical.Analog.Spice.Qpl Q3(
            BF=125,
            BR=0.1,
            IS=1.1e-15,
            NF=1,
            NR=1,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            NC=2,
            NE=1.5,
            VAF=30,
            VAR=9e30,
            IKF=3.6e-3,
            IKR=9e30,
            RB=778,
            RBM=57.6,
            RC=450,
            RE=23.3,
            CJC=0.055e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.652e-12,
            MJE=0.3,
            VJE=0.77,
            CJS=0.01e-12,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            FC=0.5,
            TF=50e-12,
            TR=1e-6,
            TRB1=0.005,
            TRB2=0,
            TRM1=0.005,
            TRM2=0,
            TRC1=0.005,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            Tnom=298.15,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            EMin=-100,
            EMax=40,
            Area=10,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            Level=Level,
            VSUB=-5,
            rampTime=rampTime) 
                     annotation (extent=[40,80; 60,56]);
          Electrical.Analog.Spice.Qpl Q6(
            BF=125,
            BR=0.1,
            IS=1.1e-15,
            NF=1,
            NR=1,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            NC=2,
            NE=1.5,
            VAF=30,
            VAR=9e30,
            IKF=3.6e-3,
            IKR=9e30,
            RB=778,
            RBM=57.6,
            RC=450,
            RE=23.3,
            CJC=0.055e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.652e-12,
            MJE=0.3,
            VJE=0.77,
            CJS=0.01e-12,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            FC=0.5,
            TF=50e-12,
            TR=1e-6,
            TRB1=0.005,
            TRB2=0,
            TRM1=0.005,
            TRM2=0,
            TRC1=0.005,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            Tnom=298.15,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            EMin=-100,
            EMax=40,
            Area=1,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            Level=Level,
            VSUB=-5,
            rampTime=rampTime) 
                    annotation (extent=[-70,28; -50,4]);
          Electrical.Analog.Spice.Qpl Q7(
            BF=125,
            BR=0.1,
            IS=1.1e-15,
            NF=1,
            NR=1,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            NC=2,
            NE=1.5,
            VAF=30,
            VAR=9e30,
            IKF=3.6e-3,
            IKR=9e30,
            RB=778,
            RBM=57.6,
            RC=450,
            RE=23.3,
            CJC=0.055e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.652e-12,
            MJE=0.3,
            VJE=0.77,
            CJS=0.01e-12,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            FC=0.5,
            TF=50e-12,
            TR=1e-6,
            TRB1=0.005,
            TRB2=0,
            TRM1=0.005,
            TRM2=0,
            TRC1=0.005,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            Tnom=298.15,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            EMin=-100,
            EMax=40,
            Area=1,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            Level=Level,
            VSUB=-5,
            rampTime=rampTime) 
                    annotation (extent=[20,28; 0,4]);
          Electrical.Analog.Spice.Qpl Q9(
            BF=125,
            BR=0.1,
            IS=1.1e-15,
            NF=1,
            NR=1,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            NC=2,
            NE=1.5,
            VAF=30,
            VAR=9e30,
            IKF=3.6e-3,
            IKR=9e30,
            RB=778,
            RBM=57.6,
            RC=450,
            RE=23.3,
            CJC=0.055e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.652e-12,
            MJE=0.3,
            VJE=0.77,
            CJS=0.01e-12,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            FC=0.5,
            TF=50e-12,
            TR=1e-6,
            TRB1=0.005,
            TRB2=0,
            TRM1=0.005,
            TRM2=0,
            TRC1=0.005,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            Tnom=298.15,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            EMin=-100,
            EMax=40,
            Area=1,
            ISS=1.1e-15,
            IRB=0,
            enforceStates=enforceStates,
            Level=Level,
            VSUB=-5,
            rampTime=rampTime) 
                    annotation (extent=[80,-2; 100,-26]);
        equation 
          connect(vo,Q4. E) annotation (points=[120,0; 100,0; 100,48], style(color=
                  3, rgbcolor={0,0,255}));
          connect(R1.heatPort,heatPort)  annotation (points=[-110,-52; -110,-72;
                -60,-72; -60,-100], style(color=42, rgbcolor={191,0,0}));
          connect(R1.n,vee)  annotation (points=[-100,-62; -100,-82; 0,-82; 0,
                -100],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q5.E,Q8. C) 
            annotation (points=[60,34; 60,16], style(color=3, rgbcolor={0,0,255}));
          connect(Q8.B,Q5. E) annotation (points=[40,10; 40,24; 60,24; 60,34],style(
                color=3, rgbcolor={0,0,255}));
          connect(C1.n,Q8. E) annotation (points=[54,-14; 60,-14; 60,4], style(
                color=3, rgbcolor={0,0,255}));
          connect(Q8.heatPort,Q5. heatPort) annotation (points=[50,-2; 30,-2;
                30,28; 50,28],
                        style(color=42, rgbcolor={191,0,0}));
          connect(Q10.C,Q8. E) 
            annotation (points=[60,-30; 60,4], style(color=3, rgbcolor={0,0,255}));
          connect(Q10.E,vee)  annotation (points=[60,-42; 60,-82; 0,-82; 0,-100],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q11.C,Q11. B) annotation (points=[-50,-54; -50,-46; -26,-46;
                -26,-60; -30,-60],
                               style(color=3, rgbcolor={0,0,255}));
          connect(Q11.E,vee)  annotation (points=[-50,-66; -50,-82; 0,-82; 0,
                -100],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q11.heatPort,heatPort)  annotation (points=[-40,-72; -60,-72;
                -60,-100],
                       style(color=42, rgbcolor={191,0,0}));
          connect(C1.p,Q10. B) annotation (points=[34,-14; 26,-14; 26,-36; 40,
                -36],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q10.B,Q12. C) annotation (points=[40,-36; 0,-36; 0,-54], style(
                color=3, rgbcolor={0,0,255}));
          connect(Q12.B,Q11. B) annotation (points=[-20,-60; -30,-60], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Q12.E,vee)  annotation (points=[0,-66; 0,-100], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q12.heatPort,Q11. heatPort) annotation (points=[-10,-72; -40,
                -72],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q12.heatPort,Q10. heatPort) annotation (points=[-10,-72; 30,
                -72; 30,-48; 50,-48],
                                 style(color=42, rgbcolor={191,0,0}));
          connect(Q12.S,vee)  annotation (points=[-20,-66; -20,-100; 0,-100], style(
                color=2, rgbcolor={0,255,0}));
          connect(Q11.S,Q12. S) annotation (points=[-30,-66; -20,-66],
                      style(color=2, rgbcolor={0,255,0}));
          connect(Q8.S,Q10. S) annotation (points=[40,4; 24,4; 24,-42; 40,-42],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q8.S,Q12. S) annotation (points=[40,4; 24,4; 24,-86; -20,-86;
                -20,-66],
                      style(color=2, rgbcolor={0,255,0}));
          connect(Q5.S,Q8. S) annotation (points=[40,34; 24,34; 24,4; 40,4], style(
                color=2, rgbcolor={0,255,0}));
          connect(Q4.S,Q5. S) annotation (points=[80,48; 24,48; 24,34; 40,34],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q1.C,R1. p) annotation (points=[-100,62; -100,-42], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q1.E,vcc)  annotation (points=[-100,74; -100,88; 0,88; 0,100],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q1.C,Q1. B) annotation (points=[-100,62; -100,52; -74,52; -74,
                68; -80,68],
                         style(color=3, rgbcolor={0,0,255}));
          connect(Q2.B,Q1. B) annotation (points=[-46,68; -80,68], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q2.S,Q1. S) annotation (points=[-46,74; -80,74], style(color=2,
                rgbcolor={0,255,0}));
          connect(Q2.heatPort,Q1. heatPort) annotation (points=[-36,80; -90,80],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q2.E,vcc)  annotation (points=[-26,74; -26,88; 0,88; 0,100],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q3.B,Q2. B) annotation (points=[40,68; -46,68], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q3.heatPort,Q2. heatPort) annotation (points=[50,80; -36,80],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q3.E,vcc)  annotation (points=[60,74; 60,88; 0,88; 0,100], style(
                color=3, rgbcolor={0,0,255}));
          connect(Q5.C,Q3. C) 
            annotation (points=[60,46; 60,62], style(color=3, rgbcolor={0,0,255}));
          connect(Q3.S,Q2. S) annotation (points=[40,74; 40,84; -46,84; -46,74],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q4.C,vcc)  annotation (points=[100,60; 100,88; 0,88; 0,100],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q3.C,Q4. B) annotation (points=[60,62; 60,54; 80,54], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Q5.B,Q3. C) annotation (points=[40,40; 40,54; 60,54; 60,62],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q6.B,vi_n)  annotation (points=[-70,16; -82,16; -82,30; -120,
                30],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q6.E,Q2. C) annotation (points=[-50,22; -50,30; -26,30; -26,
                62],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q6.C,Q11. C) annotation (points=[-50,10; -50,-54], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q7.C,Q12. C) 
            annotation (points=[0,10; 0,-54], style(color=3, rgbcolor={0,0,255}));
          connect(Q7.E,Q2. C) annotation (points=[0,22; 0,30; -26,30; -26,62],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q7.B,vi_p)  annotation (points=[20,16; 20,-10; -82,-10; -82,
                -30; -120,-30],
                           style(color=3, rgbcolor={0,0,255}));
          connect(Q9.E,vo)  annotation (points=[100,-8; 100,0; 120,0], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Q9.C,vee)  annotation (points=[100,-20; 100,-82; 0,-82; 0,
                -100],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q9.heatPort,Q8. heatPort) annotation (points=[90,-2; 50,-2],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q9.heatPort,Q4. heatPort) annotation (points=[90,-2; 90,42],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q9.B,Q8. E) annotation (points=[80,-14; 60,-14; 60,4], style(
                color=3, rgbcolor={0,0,255}));
          connect(Q9.S,Q4. S) 
            annotation (points=[80,-8; 80,48], style(color=2, rgbcolor={0,255,0}));
          connect(Q7.S,Q5. S) annotation (points=[20,22; 24,22; 24,34; 40,34],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q3.S,Q4. S) annotation (points=[40,74; 40,84; 24,84; 24,48;
                80,48],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q1.S,Q6. S) annotation (points=[-80,74; -70,74; -70,22], style(
                color=2, rgbcolor={0,255,0}));
          connect(Q7.heatPort,Q5. heatPort) annotation (points=[10,28; 50,28],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q7.heatPort,Q6. heatPort) annotation (points=[10,28; -60,28],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q7.heatPort,Q3. heatPort) annotation (points=[10,28; 10,80;
                50,80],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q10.heatPort, Q8.heatPort) annotation (points=[50,-48; 30,-48;
                30,-2; 50,-2], style(color=42, rgbcolor={191,0,0}));
          connect(C1.heatPort, Q8.heatPort) annotation (points=[44,-24; 30,-24;
                30,-2; 50,-2], style(color=42, rgbcolor={191,0,0}));
        end OpAmp;
        
        model Inverter "Inverter circuit built from Hifi operational amplifier" 
          extends Modelica.Blocks.Interfaces.SISO;
          parameter Integer Level=2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Modelica.SIunits.Time rampTime=3e-6 "Ramping period";
          parameter Modelica.SIunits.Time settleTime=2e-6 "Settling period";
          parameter Boolean enforceStates = false 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
          annotation (
            Icon(
              Line(points=[-60,60; -60,-60], style(
                  thickness=2,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[-60,60; 60,0], style(
                  thickness=2,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[-60,-60; 60,0], style(
                  thickness=2,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[-60,0; -100,0], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[60,0; 100,0], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Text(
                extent=[-66,30; -12,-24],
                style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1),
                string="-")),
            Documentation(info="<html>
This model shows an inverter circuit built from a high-gain operational amplifier.  Such circuits used to be standard modeling elements of analog computers [1]. <p>

Three device parameters were pulled out all the way to the top level: <p>

<ol>
<li><b>Level</b> allows the user to choose between a full-fledged <i>Gummel-Poon</i> transistor model (<b>Level=2</b>) and a simplified <i>Ebers-Moll</i> transistor model (<b>Level=1</b>).
<li><b>rampTime</b> enables the user to change the initial ramping period centrally with a single parameter assignment.
<li><b>settleTime</b> enables the user to change the initial settling period centrally with a single parameter assignment.
</ol>
</html>
"),         Diagram);
          Electrical.Analog.Basic.Ground Gnd1 annotation (extent=[-6,-26; 6,-16]);
          Electrical.Analog.Basic.Ground Gnd2 
            annotation (extent=[-66,-66; -54,-56]);
          Electrical.Analog.Basic.Ground Gnd3 annotation (extent=[44,14; 56,24]);
          Electrical.Analog.Basic.Ground Gnd4 annotation (extent=[14,-66; 26,
                -56]);
          Electrical.Analog.Basic.Ground Gnd5 annotation (extent=[54,-66; 66,
                -56]);
          Electrical.Analog.Sensors.VoltageSensor Vo 
            annotation (extent=[70,-46; 50,-26], rotation=-90);
          Electrical.Analog.Spice.R Rf(
            Tnom=298.15,
            TR1=0,
            TR2=0,
            Area=1,
            R=1e6)  annotation (extent=[20,60; 40,80]);
          Electrical.Analog.Spice.R R1(
            Tnom=298.15,
            TR1=0,
            TR2=0,
            Area=1,
            R=1e6)  annotation (extent=[-40,0; -20,20]);
          Electrical.Analog.Spice.RampingSources.V Vee(        V=-5, rampTime=
                rampTime) 
            annotation (extent=[10,-46; 30,-26], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vcc(                       V=5, rampTime=
               rampTime) 
            annotation (extent=[24,34; 44,54], rotation=0);
          OpAmp A1(Level=Level, enforceStates=enforceStates) 
                                annotation (extent=[0,-16; 40,24]);
          Electrical.Analog.Sources.SignalVoltage Vin 
            annotation (extent=[-50,-46; -70,-26], rotation=-90);
          Thermal.HeatTransfer.Sensors.HeatFlowSensor Qtot 
            annotation (extent=[-32,40; -52,60]);
          Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
                298.15) annotation (extent=[-80,40; -60,60]);
        equation 
          connect(Vo.n,Gnd5. p) annotation (points=[60,-46; 60,-56], style(
              color=3,
              rgbcolor={0,0,255},
              pattern=0,
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Vee.n,Gnd4. p) annotation (points=[20,-46; 20,-56], style(color=3,
                rgbcolor={0,0,255}));
          connect(Vcc.n,Gnd3. p) annotation (points=[44,44; 50,44; 50,24], style(
                color=3, rgbcolor={0,0,255}));
          connect(A1.vi_n, R1.n) annotation (points=[0,10; -20,10], style(color=
                 3, rgbcolor={0,0,255}));
          connect(A1.vi_p, Gnd1.p) annotation (points=[0,-2; 0,-16], style(
                color=3, rgbcolor={0,0,255}));
          connect(A1.vee, Vee.p) annotation (points=[20,-16; 20,-26], style(
                color=3, rgbcolor={0,0,255}));
          connect(A1.vo, Vo.p) annotation (points=[40,4; 60,4; 60,-26], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vcc.p, A1.vcc) annotation (points=[24,44; 20,44; 20,24],
              style(color=3, rgbcolor={0,0,255}));
          connect(Rf.p, A1.vi_n) annotation (points=[20,70; 0,70; 0,10], style(
                color=3, rgbcolor={0,0,255}));
          connect(Rf.n, A1.vo) annotation (points=[40,70; 60,70; 60,4; 40,4],
              style(color=3, rgbcolor={0,0,255}));
          connect(R1.heatPort, A1.heatPort) annotation (points=[-30,0; -10,0;
                -10,-28; 10,-28; 10,-16], style(color=42, rgbcolor={191,0,0}));
          connect(Vo.V, y) annotation (points=[70,-36; 80,-36; 80,0; 110,0], style(
                color=74, rgbcolor={0,0,127}));
          connect(Vin.n, Gnd2.p) annotation (points=[-60,-46; -60,-56], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vin.p, R1.p) annotation (points=[-60,-26; -60,10; -40,10],
              style(color=3, rgbcolor={0,0,255}));
          connect(u, Vin.InPort1) annotation (points=[-120,0; -80,0; -80,-36;
                -66,-36], style(color=74, rgbcolor={0,0,127}));
          connect(FixedTemperature1.port,Qtot. port_b) annotation (points=[-59,50;
                -53,50],     style(color=42, rgbcolor={191,0,0}));
          connect(Qtot.port_a, Rf.heatPort) annotation (points=[-31,50; -10,50;
                -10,60; 30,60], style(color=42, rgbcolor={191,0,0}));
          connect(Rf.heatPort, A1.heatPort) annotation (points=[30,60; -10,60;
                -10,-28; 10,-28; 10,-16], style(color=42, rgbcolor={191,0,0}));
        end Inverter;
        
        model Experiment 
          "Experiment description for Hifi operational amplifier circuit" 
          parameter Integer Level=2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Modelica.SIunits.Time rampTime=3e-6 "Ramping period";
          parameter Modelica.SIunits.Time settleTime=2e-6 "Settling period";
          parameter Boolean enforceStates = false 
            "|Advanced||Use electrical variables as states instead of bond graph variables";
          output Modelica.SIunits.Voltage vin "Input voltage";
          output Modelica.SIunits.Voltage vout "Output voltage";
          output Modelica.SIunits.HeatFlowRate Pth 
            "Total heat dissipated by operational amplifier";
          Electrical.Analog.Spice.RampingSources.trapV Vin(
            V=4,
            rising=1e-6,
            width=24e-6,
            falling=1e-6,
            period=50e-6,
            nperiod=2,
            offset=-2,
            rampTime=rampTime,
            settleTime=settleTime) 
                             annotation (extent=[-80,-10; -60,10],  rotation=-90);
          Inverter Inv(
            Level=Level,
            rampTime=rampTime,
            settleTime=settleTime,
            enforceStates=enforceStates) 
                                   annotation (extent=[20,-20; 60,20]);
          Electrical.Analog.Basic.Ground Gnd1 annotation (extent=[-76,-30; -64,
                -20]);
          Electrical.Analog.Basic.Resistor R1(R=1) 
            annotation (extent=[-50,-10; -30,10], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd2 annotation (extent=[-46,-30; -34,
                -20]);
          Electrical.Analog.Sensors.VoltageSensor Vin1 
            annotation (extent=[0,-10; -20,10],  rotation=-90);
          Electrical.Analog.Basic.Ground Gnd3 annotation (extent=[-16,-30; -4,
                -20]);
        equation 
          vin = Inv.u;
          vout = Inv.y;
          Pth = Inv.Qtot.Q_flow;
          annotation (Diagram, Icon(
              Rectangle(extent=[-100,100; 100,-100], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Text(
                extent=[-88,46; 90,4],
                style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1),
                string="Transient and AC analysis"),
              Text(
                extent=[-88,24; 90,-18],
                style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1),
                string="of Hifi operational amplifier")),
            Documentation(info="<html>
This models shows an inverter circuit built from a high-gain operational amplifier.  Such circuits used to be standard modeling elements of analog computers [1]. <p>

Three device parameters were pulled out all the way to the top level: <p>

<ol>
<li><b>Level</b> allows the user to choose between a full-fledged <i>Gummel-Poon</i> transistor model (<b>Level=2</b>) and a simplified <i>Ebers-Moll</i> transistor model (<b>Level=1</b>).
<li><b>rampTime</b> enables the user to change the initial ramping period centrally with a single parameter assignment.
<li><b>settleTime</b> enables the user to change the initial settling period centrally with a single parameter assignment.
</ol> <p>

Usually, we prefer the voltages across capacitors to be used as state variables.  However by default, Dymola will use the effort variables of the underlying bond graph C elements as state variables.  Since these variables are inside the wrapper models and therefore protected, the state derivatives can then not be displayed. <p>

For this reason, we usually make use of Dymola's StateSelect algorithm to force Dymola to use the (external) capacitive voltages as state variables instead of the corresponding (internal) bond graph effort variables. <p>

Unfortunately, Dymola's StateSelect algorithm fails on this example.  For this reason, the Boolean switch <b>enforceStates</b> must be set to false, so that Dymola can use its natural state variables. <p>

<p>

<hr> <p>

<b>Experimentation:</b> <p>

<b>1. Transient Analysis:</b> <p>

Simulate the model during 0.0001 seconds.  Store only the output variables.  Plot the input voltage, <b>vin</b>, together with the output voltage, <b>vout</b>, on a single plot. <p>

We expect the following results: <p>

<img src=\"../Images/BG_Ex9.png\"> <p>

Here, the ramping trapezoidal input is clearly visible.  In order to reach a true DC OP-point, both the <b>rampTime</b> and the <b>settleTime</b> would have to be chosen considerably longer, but since the circuit converged with the given values, it wasn't necessary to do so. <p>

<hr> <p>

<b>2. DC Analysis:</b> <p>

Before we can attempt an AC analysis, we need to compute a <b>DC OP point</b>.  In <font color=red size=4><b><i>ModelicaSpice</i></b></font>, we would usually enhance the settling time to the full length of the simulation and repeat the transient analysis with the initial ramping and settling periods only. <p>

However in the given case, as clearly visible from the simulation results of the transient analysis, the final value of the transient analysis above represents a perfectly good DC OP point. <p>

The DC OP point, i.e., the final state of the last simulation performed, is automatically saved by Dymola onto the file <b>dsfinal.txt</b>. <p>

<hr> <p>

<b>3. AC Analysis:</b> <p>

In order to perform an AC analysis, we need to return quickly to the <i>Modeling</i> window and select the <b>Inverter</b> model, instead of the previously selected <b>Experiment</b> model.  The <i>Inverter</i> model has been designed as a SISO model.  We then return to the <i>Simulation</i> window. <p>

Now, we need to use the final <b>DC OP point</b> that we saved before as new <b>initial conditions</b>.  To this end, we choose from the <i>Simulation</i> pull-down menu the command <i>Continue/ImportInitial...</i> and point at the previously stored file <i>dsfinal.txt</i>. <p>

We have now set the correct initial conditions and are ready to <b>linearize</b> the model around these initial conditions.  The <i>Linearize</i> command can also be found in the <i>Simulation</i> pull-down menu. <p>

The linearized model is stored by Dymola in the file <b>dslin.mat</b>. <p>

The AC analysis is actually performed in <font color=red><b>Matlab</b></font> rather than in <font color=red><b>Dymola</b></font>.  We enter Matlab, set the default directory to the current working directory (using the <b>cd</b> command), then run the file <b>OpAmp.m</b>: <p>

<img src=\"../Images/BG_Ex11.png\"> <p>

This file loads the linearized model into Matlab, and extracts the four system matrices from the combined system matrix stored by Dymola.  Each BJT represents a 3<sup>rd</sup> order model.  There are 12 BJTs in the <i>Inverter</i> circuit, plus one extra capacitor.  Thus the overall model is of order 37. <p>

The m-file then makes a Matlab system definition out of the four individual system matrices (this requires the <font color=red><b>Controls toolbox</b></font> of Matlab).  The file then draws a Bode diagram: <p>

<img src=\"../Images/BG_Ex12.png\"> <p>

Evidently, the Hifi operational amplifier has a band width of roughly 1 MHz.

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, ISBN: 0-387-97502-0, 755 p.
</ol> <p>
</html>
"),         experiment(StopTime=0.0001),
            experimentSetupOutput);
          connect(Vin.n, Gnd1.p) annotation (points=[-70,-10; -70,-20], style(
                color=3, rgbcolor={0,0,255}));
          connect(R1.n, Gnd2.p) annotation (points=[-40,-10; -40,-20], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vin.p, R1.p) annotation (points=[-70,10; -70,20; -40,20; -40,
                10], style(color=3, rgbcolor={0,0,255}));
          connect(Vin1.V, Inv.u) annotation (points=[0,-6.12303e-016; 8,
                -6.12303e-016; 8,0; 16,0], style(color=74, rgbcolor={0,0,127}));
          connect(Vin1.n, Gnd3.p) annotation (points=[-10,-10; -10,-20], style(
                color=3, rgbcolor={0,0,255}));
          connect(R1.p, Vin1.p) annotation (points=[-40,10; -40,20; -10,20; -10,
                10], style(color=3, rgbcolor={0,0,255}));
        end Experiment;
      end HifiOperationalAmplifier;
      
      package OrNorGate "OR/NOR gate built in ECL logic" 
        annotation (preferedView="info",
          Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-67.85,54.65; 70,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog OR/NOR gate"),
            Text(
              extent=[-58,36; 60,4],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built in ECL logic"),
            Text(
              extent=[-72,16; 72,-16],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using 13 vertical NPNs")),
          Diagram(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-67.85,50.65; 70,20],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog OR/NOR gate"),
            Text(
              extent=[-58,32; 60,0],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built in ECL logic"),
            Text(
              extent=[-72,12; 72,-20],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using 13 vertical NPNs")),
          Documentation(info="<html>
This package contains a model of a commercial OR/NOR gate built from thirteen vertically diffused NPN bipolar transistors [1: p.244]. <p>

The circuit uses the device parameters of <b>Process 35</b> proposed in [1: p.208].  Notice the use of the ramping sources both on the two input signals, <b>vA</b> and <b>vB</b>, and on the two supplies of -2 Volts and -5.2 Volts, respectively.  Without ramping sources, this circuit could not be simulated at all using realistic device parameters.  The ramping sources offer a means to slowly approach a <i>DC OP-point</i> before the transient analysis begins. <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate the model during 30 microseconds.  Plot the two input voltages, <b>vA</b> and <b>vB</b>, together with the two output voltages, <b>vOR</b> and <b>vNOR</b>, on a single plot. <p>

<hr> <p>

<b>Simulation Results:</b> <p>

We expect the following results: <p>

<img src=\"../Images/BG_Ex10.png\"> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, ISBN: 0-387-97502-0, 755 p.
</ol> <p>
</html>
"));
        model OrNor "OR/NOR gate built in ECL logic" 
          parameter Integer Level=2 
            "Transistor modeling level (Ebers-Moll = 1; Gummel-Poon = 2)";
          parameter Modelica.SIunits.Time rampTime=3e-6 "Ramping period";
          annotation (
            Coordsys(extent=[-140,-100; 140,100]),
            Diagram(
              Text(
                extent=[-18,-84; 0,-90],
                string="-5.2 V",
                style(color=10, rgbcolor={95,95,95})),
              Text(
                extent=[88,-32; 106,-38],
                style(color=10, rgbcolor={95,95,95}),
                string="-2 V"),
              Text(
                extent=[2,98; 20,92],
                style(color=10, rgbcolor={95,95,95}),
                string="-2 V"),
              Text(
                extent=[4,-94; 22,-100],
                string="-5.2 V",
                style(color=10, rgbcolor={95,95,95})),
              Text(
                extent=[110,46; 128,40],
                style(color=10, rgbcolor={95,95,95}),
                string="A + B"),
              Text(
                extent=[110,26; 128,20],
                style(color=10, rgbcolor={95,95,95}),
                string="A + B"),
              Line(points=[112,26; 124,26], style(color=10, rgbcolor={95,95,95}))),
            Icon(
              Line(points=[78,0; 96,0], style(color=3, rgbcolor={0,0,255})),
              Line(points=[96,-50; 96,50], style(color=3, rgbcolor={0,0,255})),
              Line(points=[96,50; 130,50], style(color=3, rgbcolor={0,0,255})),
              Ellipse(extent=[110,-40; 130,-60], style(color=3, rgbcolor={0,0,
                      255})),
              Line(points=[96,-50; 110,-50], style(color=3, rgbcolor={0,0,255})),
              Text(
                extent=[40,102; 136,66],
                style(color=3, rgbcolor={0,0,255}),
                string="%name"),
              Ellipse(extent=[-94,48; -48,-52], style(color=3, rgbcolor={0,0,
                      255})),
              Rectangle(extent=[-72,-58; -98,54], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[-132,20; -52,20], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-130,-20; -50,-20], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-72,48; -64,48; -38,46; -18,44; -4,42; 6,40; 14,38;
                    22,36; 30,34; 42,30; 50,26; 58,22; 64,18; 72,12; 76,6; 78,0;
                    78,-6; 72,-16; 62,-24; 52,-30; 36,-36; 16,-42; -6,-46; -20,
                    -48], style(color=3, rgbcolor={0,0,255})),
                 Line(points=[-72,-52; -58,-52; -38,-50; -20,-48], style(color=
                      3, rgbcolor={0,0,255}))),
            Documentation(info="<html>
This model describes a commercial OR/NOR gate built in ECL logic.  ECL logic devices are built from bipolar technology. <p>

Each transistor has a (red) heat port and a (green) substrate node.  The heat port uses a different color embracing the philosophy of Modelica libraries, which distinguish different connectors by color to make it visibly unlikely that impossible connections are being made by the user.  The substrate node is an electrical node and should therefore be drawn in blue.  However, we decided to use a different color to prevent the circuit from becoming unreadable due to an abundance of connections.  Presumably, this circuit proves the wisdom of that decision. <p>

All internal heat ports need to be tied to the external heat port.  All internal substrate nodes need to be tied to the lowest voltage, i.e., the bottom port, which is connected externally to a -5.2 Volt supply. <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, p.244.
</ol>
</html>
"));
          
          Electrical.Analog.Spice.Qnv Q4(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[-120,-12; -100,12]);
          Electrical.Analog.Spice.Qnv Q1(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[-80,-12; -60,12]);
          Electrical.Analog.Spice.Qnv Q2(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=5,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[-60,-32; -40,-8]);
          Electrical.Analog.Spice.Qnv Q3(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=5,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[-8,-26; -28,-2]);
          Electrical.Analog.Spice.Qnv Q5(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=5,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[20,-32; 0,-8]);
          Electrical.Analog.Spice.Qnv Q6(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=1,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[-40,-72; -60,-48]);
          Electrical.Analog.Spice.Qnv Q7(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=2,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[0,-72; -20,-48]);
          Electrical.Analog.Spice.Qnv Q8(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=1,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[40,-52; 20,-28]);
          Electrical.Analog.Spice.Qnv Q9(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=1,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[40,-12; 20,12]);
          Electrical.Analog.Spice.Qnv Q10(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=1,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[60,8; 40,32]);
          Electrical.Analog.Spice.Qnv Q11(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            Area=1,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[20,-72; 40,-48]);
          Electrical.Analog.Spice.Qnv Q12(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=25,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[60,28; 80,52]);
          Electrical.Analog.Spice.Qnv Q13(
            BR=0.1,
            RC=750,
            NF=1,
            NR=1,
            NC=2,
            NE=1.5,
            VAR=9e30,
            IKR=9e30,
            IRB=0,
            MJS=0.33,
            VJS=0.75,
            XCJC=1,
            Tnom=298.15,
            EMin=-100,
            EMax=40,
            RE=13.3,
            FC=0.5,
            TRB1=0.005,
            TRM1=0.005,
            TRC1=0.005,
            BF=190,
            IS=1.1e-15,
            GminDC=1e-16,
            ISC=0,
            ISE=0,
            VAF=110,
            IKF=3.6e-3,
            RB=1109.9,
            RBM=368.4,
            CJC=0.36e-12,
            MJC=0.425,
            VJC=0.64,
            CJE=0.597e-12,
            MJE=0.3,
            VJE=0.77,
            TF=50e-12,
            TR=1e-6,
            TRB2=0,
            TRM2=0,
            TRC2=0,
            TRE1=0.0005,
            TRE2=0,
            XTI=2.33,
            XTB=0.006,
            EG=1.205612,
            CJS=0.01e-12,
            Level=Level,
            Area=25,
            ISS=1.1e-15,
            VSUB=-5.2,
            rampTime=rampTime) 
                     annotation (extent=[90,48; 110,72]);
          Electrical.Analog.Spice.R R1(
            Tnom=298.15,
            R=500,
            TR1=0,
            TR2=0,
            Area=1) annotation (extent=[-50,50; -30,70], rotation=-90);
          Electrical.Analog.Spice.R R2(
            Tnom=298.15,
            R=500,
            TR1=0,
            TR2=0,
            Area=1) annotation (extent=[-10,60; 10,80], rotation=-90);
          Electrical.Analog.Spice.R R3(
            Tnom=298.15,
            TR1=0,
            TR2=0,
            Area=1,
            R=540) annotation (extent=[20,22; 40,42], rotation=0);
          Electrical.Analog.Spice.R R4(
            Tnom=298.15,
            TR1=0,
            TR2=0,
            Area=1,
            R=1200) annotation (extent=[10,-90; 30,-70], rotation=-90);
          Electrical.Analog.Spice.R R5(
            Tnom=298.15,
            TR1=0,
            TR2=0,
            Area=1,
            R=2400) annotation (extent=[30,-30; 50,-10], rotation=-90);
          Electrical.Analog.Spice.R R6(
            Tnom=298.15,
            TR1=0,
            TR2=0,
            Area=1,
            R=50) annotation (extent=[70,-20; 90,0], rotation=-90);
          Electrical.Analog.Spice.R R7(
            Tnom=298.15,
            TR1=0,
            TR2=0,
            Area=1,
            R=50) annotation (extent=[100,-20; 120,0], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[-120,60; -100,80]);
          Modelica.Electrical.Analog.Interfaces.Pin Vhigh 
            "High support electrical connector" 
            annotation (extent=[-10,90; 10,110]);
          Modelica.Electrical.Analog.Interfaces.Pin Vlow 
            "Low support electrical connector" 
            annotation (extent=[-10,-110; 10,-90]);
          Modelica.Electrical.Analog.Interfaces.Pin OR 
            "OR output electrical connector" 
            annotation (extent=[130,40; 150,60]);
          Modelica.Electrical.Analog.Interfaces.Pin NOR 
            "NOR output electrical connector" 
            annotation (extent=[130,-60; 150,-40]);
          Modelica.Electrical.Analog.Interfaces.Pin A 
            "First input electrical connector" 
            annotation (extent=[-150,10; -130,30]);
          Modelica.Electrical.Analog.Interfaces.Pin B 
            "Second input electrical connector" 
            annotation (extent=[-150,-30; -130,-10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "Thermal connector" 
            annotation (extent=[60,-90; 80,-110],    rotation=-90);
        equation 
          connect(R2.p, Gnd1.p) annotation (points=[-6.12303e-016,80; -110,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(R1.p, Gnd1.p) annotation (points=[-40,70; -40,80; -110,80],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q4.C, Gnd1.p) annotation (points=[-100,6; -100,20; -80,20;
                -80,80; -110,80], style(color=3, rgbcolor={0,0,255}));
          connect(Q1.C, Gnd1.p) annotation (points=[-60,6; -60,20; -80,20; -80,
                80; -110,80], style(color=3, rgbcolor={0,0,255}));
          connect(Q1.E, Q2.B) annotation (points=[-60,-6; -60,-20], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Q1.B, B) annotation (points=[-80,0; -84,0; -84,-20; -140,-20],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q4.B, A) annotation (points=[-120,0; -128,0; -128,20; -140,20],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q4.E, Q2.B) annotation (points=[-100,-6; -100,-20; -60,-20],
              style(color=3, rgbcolor={0,0,255}));
          connect(R1.n, Q2.C) annotation (points=[-40,50; -40,-14], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Q2.E, Q5.E) annotation (points=[-40,-26; -40,-40; 0,-40; 0,
                -26], style(color=3, rgbcolor={0,0,255}));
          connect(Q7.C, Q2.E) annotation (points=[-20,-54; -20,-40; -40,-40;
                -40,-26], style(color=3, rgbcolor={0,0,255}));
          connect(Q7.B, Q11.B) annotation (points=[0,-60; 20,-60], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Vlow, R4.n) annotation (points=[0,-100; 0,-90; 20,-90], style(
                color=3, rgbcolor={0,0,255}));
          connect(Q7.E, R4.n) annotation (points=[-20,-66; -20,-90; 20,-90],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q6.E, R4.n) annotation (points=[-60,-66; -60,-90; 20,-90],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q8.E, Q11.B) annotation (points=[20,-46; 20,-60], style(color=
                 3, rgbcolor={0,0,255}));
          connect(Q11.E, R4.n) annotation (points=[40,-66; 40,-90; 20,-90],
              style(color=3, rgbcolor={0,0,255}));
          connect(R5.n, Q8.B) annotation (points=[40,-30; 40,-40], style(color=
                  3, rgbcolor={0,0,255}));
          connect(Q9.E, Q5.B) annotation (points=[20,-6; 20,-20], style(color=3,
                rgbcolor={0,0,255}));
          connect(R2.n, Q5.C) annotation (points=[6.12303e-016,60; 0,60; 0,-14],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q3.B, Q3.C) annotation (points=[-8,-14; -8,0; -28,0; -28,-8],
              style(color=3, rgbcolor={0,0,255}));
          connect(R1.n, Q12.B) annotation (points=[-40,50; -40,40; 60,40],
              style(color=3, rgbcolor={0,0,255}));
          connect(R2.n, Q13.B) annotation (points=[6.12303e-016,60; 90,60],
              style(color=3, rgbcolor={0,0,255}));
          connect(R2.p, R3.p) annotation (points=[-6.12303e-016,80; 20,80; 20,
                32], style(color=3, rgbcolor={0,0,255}));
          connect(Q12.C, R2.p) annotation (points=[80,46; 80,80; -6.12303e-016,
                80], style(color=3, rgbcolor={0,0,255}));
          connect(Q13.C, R2.p) annotation (points=[110,66; 110,80;
                -6.12303e-016,80], style(color=3, rgbcolor={0,0,255}));
          connect(Q12.E, R6.p) annotation (points=[80,34; 80,0], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q13.E, R7.p) annotation (points=[110,54; 110,0], style(color=
                  3, rgbcolor={0,0,255}));
          connect(OR, Q13.E) annotation (points=[140,50; 126,50; 126,40; 110,40;
                110,54], style(color=3, rgbcolor={0,0,255}));
          connect(NOR, Q12.E) annotation (points=[140,-50; 126,-50; 126,20; 80,
                20; 80,34], style(color=3, rgbcolor={0,0,255}));
          connect(R6.n, R7.n) annotation (points=[80,-20; 80,-30; 110,-30; 110,
                -20], style(color=3, rgbcolor={0,0,255}));
          connect(R7.n, Vhigh) annotation (points=[110,-20; 110,-30; 148,-30;
                148,100; 0,100], style(color=3, rgbcolor={0,0,255}));
          connect(R3.n, Q10.C) annotation (points=[40,32; 40,26], style(color=3,
                rgbcolor={0,0,255}));
          connect(R3.p, Q9.C) annotation (points=[20,32; 20,6], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q10.E, Q9.B) annotation (points=[40,14; 40,0], style(color=3,
                rgbcolor={0,0,255}));
          connect(Q11.S, Q7.S) annotation (points=[20,-66; 0,-66], style(color=
                  2, rgbcolor={0,255,0}));
          connect(Q7.S, Q6.S) annotation (points=[0,-66; 0,-76; -40,-76; -40,
                -66], style(color=2, rgbcolor={0,255,0}));
          connect(Q6.S, Vlow) annotation (points=[-40,-66; -40,-100; 0,-100],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q2.S, Vlow) annotation (points=[-60,-26; -66,-26; -66,-100; 0,
                -100], style(color=2, rgbcolor={0,255,0}));
          connect(Q1.S, Q2.S) annotation (points=[-80,-6; -80,-26; -60,-26],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q4.S, Q2.S) annotation (points=[-120,-6; -120,-26; -60,-26],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q3.S, Q2.S) annotation (points=[-8,-20; -8,-36; -66,-36; -66,
                -26; -60,-26], style(color=2, rgbcolor={0,255,0}));
          connect(Q5.S, Q3.S) annotation (points=[20,-26; 14,-26; 14,-36; -8,
                -36; -8,-20], style(color=2, rgbcolor={0,255,0}));
          connect(Q8.S, Vlow) annotation (points=[40,-46; 50,-46; 50,-100; 0,
                -100], style(color=2, rgbcolor={0,255,0}));
          connect(Q9.S, Q8.S) annotation (points=[40,-6; 50,-6; 50,-46; 40,-46],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q10.S, Q9.S) annotation (points=[60,14; 60,-6; 40,-6], style(
                color=2, rgbcolor={0,255,0}));
          connect(Q12.S, Q10.S) annotation (points=[60,34; 64,34; 64,14; 60,14],
              style(color=2, rgbcolor={0,255,0}));
          connect(Q13.S, Q10.S) annotation (points=[90,54; 90,14; 60,14], style(
                color=2, rgbcolor={0,255,0}));
          connect(R4.heatPort, Q11.heatPort) annotation (points=[10,-80; 10,-72;
                30,-72], style(color=42, rgbcolor={191,0,0}));
          connect(R4.heatPort, Q7.heatPort) annotation (points=[10,-80; 10,-72;
                -10,-72], style(color=42, rgbcolor={191,0,0}));
          connect(Q7.heatPort, Q6.heatPort) annotation (points=[-10,-72; -50,
                -72], style(color=42, rgbcolor={191,0,0}));
          connect(Q8.heatPort, Q11.heatPort) annotation (points=[30,-52; 26,-52;
                26,-72; 30,-72], style(color=42, rgbcolor={191,0,0}));
          connect(Q11.heatPort, heatPort) annotation (points=[30,-72; 70,-72;
                70,-100], style(color=42, rgbcolor={191,0,0}));
          connect(Q5.heatPort, Q8.heatPort) annotation (points=[10,-32; 12,-32;
                12,-52; 30,-52], style(color=42, rgbcolor={191,0,0}));
          connect(Q3.heatPort, Q5.heatPort) annotation (points=[-18,-26; -18,
                -32; 10,-32], style(color=42, rgbcolor={191,0,0}));
          connect(Q2.heatPort, Q5.heatPort) annotation (points=[-50,-32; 10,-32],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q1.heatPort, Q2.heatPort) annotation (points=[-70,-12; -70,
                -32; -50,-32], style(color=42, rgbcolor={191,0,0}));
          connect(R6.heatPort, heatPort) annotation (points=[70,-10; 70,-100],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q9.heatPort, R5.heatPort) annotation (points=[30,-12; 30,-20],
              style(color=42, rgbcolor={191,0,0}));
          connect(Q10.heatPort, R6.heatPort) annotation (points=[50,8; 56,8; 56,
                -10; 70,-10], style(color=42, rgbcolor={191,0,0}));
          connect(Q12.heatPort, R6.heatPort) annotation (points=[70,28; 70,-10],
              style(color=42, rgbcolor={191,0,0}));
          connect(R7.heatPort, R6.heatPort) annotation (points=[100,-10; 100,
                -26; 70,-26; 70,-10], style(color=42, rgbcolor={191,0,0}));
          connect(Q13.heatPort, Q12.heatPort) annotation (points=[100,48; 100,
                28; 70,28], style(color=42, rgbcolor={191,0,0}));
          connect(Q1.heatPort, Q4.heatPort) annotation (points=[-70,-12; -110,
                -12], style(color=42, rgbcolor={191,0,0}));
          connect(R2.heatPort, R1.heatPort) annotation (points=[-10,70; -10,44;
                -50,44; -50,60], style(color=42, rgbcolor={191,0,0}));
          connect(R2.heatPort, Q13.heatPort) annotation (points=[-10,70; -10,48;
                100,48], style(color=42, rgbcolor={191,0,0}));
          connect(R3.heatPort, R2.heatPort) annotation (points=[30,22; -10,22;
                -10,70], style(color=42, rgbcolor={191,0,0}));
          connect(Q9.heatPort, R3.heatPort) annotation (points=[30,-12; 14,-12;
                14,22; 30,22], style(color=42, rgbcolor={191,0,0}));
          connect(Q6.C, Q2.B) annotation (points=[-60,-54; -60,-40; -56,-40;
                -56,-20; -60,-20], style(color=3, rgbcolor={0,0,255}));
          connect(Q3.B, Q9.E) annotation (points=[-8,-14; -8,-6; 20,-6], style(
                color=3, rgbcolor={0,0,255}));
          connect(Q8.C, Q5.B) annotation (points=[20,-34; 20,-30; 24,-30; 24,
                -20; 20,-20], style(color=3, rgbcolor={0,0,255}));
          connect(Q11.C, Q8.B) annotation (points=[40,-54; 44,-54; 44,-40; 40,
                -40], style(color=3, rgbcolor={0,0,255}));
          connect(R4.p, Q11.B) annotation (points=[20,-70; 24,-70; 24,-60; 20,
                -60], style(color=3, rgbcolor={0,0,255}));
          connect(R5.p, Q9.B) annotation (points=[40,-10; 44,-10; 44,0; 40,0],
              style(color=3, rgbcolor={0,0,255}));
          connect(Q3.E, Q1.E) annotation (points=[-28,-20; -36,-20; -36,-6; -60,
                -6], style(color=3, rgbcolor={0,0,255}));
          connect(Q6.B, Q7.B) annotation (points=[-40,-60; -30,-60; -30,-46; 0,
                -46; 0,-60], style(color=3, rgbcolor={0,0,255}));
          connect(Q10.B, R3.n) annotation (points=[60,20; 60,28; 56,28; 56,32;
                40,32], style(color=3, rgbcolor={0,0,255}));
        end OrNor;
        
        model TestOrNor "Test circuit to evaluate performance of OR/NOR gate" 
          output Modelica.SIunits.Voltage vA "First input voltage";
          output Modelica.SIunits.Voltage vB "Second input voltage";
          output Modelica.SIunits.Voltage vOR "First output voltage";
          output Modelica.SIunits.Voltage vNOR "Second output voltage";
          output Modelica.SIunits.HeatFlowRate Pth 
            "Total heat dissipated by OR/NOR gate";
          Electrical.Analog.Spice.RampingSources.trapV Vb(
            rising=1e-6,
            falling=1e-6,
            period=50e-6,
            V=1,
            width=10e-6,
            nperiod=1,
            offset=-1.8,
            rampTime=3e-6,
            settleTime=7e-6) annotation (extent=[-60,-40; -40,-20], rotation=-90);
          Electrical.Analog.Spice.RampingSources.trapV Va(
            rising=1e-6,
            falling=1e-6,
            period=50e-6,
            V=1,
            width=10e-6,
            nperiod=1,
            offset=-1.8,
            rampTime=3e-6,
            settleTime=2e-6) annotation (extent=[-60,40; -40,20],   rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vee(V=-5.2, rampTime=3e-6) 
            annotation (extent=[-20,-50; 0,-30], rotation=-90);
          Electrical.Analog.Spice.RampingSources.V Vee1(rampTime=3e-6, V=-2) 
            annotation (extent=[-20,50; 0,30],   rotation=-90);
          Electrical.Analog.Basic.Ground Gnd1 annotation (extent=[-76,30; -64,
                40]);
          Electrical.Analog.Basic.Ground Gnd2 annotation (extent=[-56,-60; -44,
                -50]);
          Electrical.Analog.Basic.Ground Gnd3 annotation (extent=[-36,40; -24,
                50]);
          Electrical.Analog.Basic.Ground Gnd4 annotation (extent=[-16,-70; -4,
                -60]);
          Electrical.Analog.Sensors.VoltageSensor Vor 
            annotation (extent=[80,-40; 60,-20], rotation=-90);
          Electrical.Analog.Sensors.VoltageSensor Vnor 
            annotation (extent=[50,-40; 30,-20], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd5 annotation (extent=[34,-60; 46,
                -50]);
          Electrical.Analog.Basic.Ground Gnd6 annotation (extent=[64,-60; 76,
                -50]);
          OrNor OrNor1(Level=2) annotation (extent=[-30,-20; 10,20]);
          Thermal.HeatTransfer.Sensors.HeatFlowSensor Qtot 
            annotation (extent=[30,50; 50,70]);
          Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
                298.15) annotation (extent=[80,50; 60,70]);
        equation 
          vA = Va.v;
          vB = Vb.v;
          vOR = Vor.v;
          vNOR = Vnor.v;
          Pth = Qtot.Q_flow;
          connect(Va.n, Gnd1.p) annotation (points=[-50,40; -50,50; -70,50; -70,
                40], style(color=3, rgbcolor={0,0,255}));
          connect(Vb.n, Gnd2.p) annotation (points=[-50,-40; -50,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vee1.n, Gnd3.p) annotation (points=[-10,50; -10,60; -30,60;
                -30,50], style(color=3, rgbcolor={0,0,255}));
          connect(Vee.n, Gnd4.p) annotation (points=[-10,-50; -10,-60], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vnor.n, Gnd5.p) annotation (points=[40,-40; 40,-50], style(
                color=3, rgbcolor={0,0,255}));
          connect(Vor.n, Gnd6.p) annotation (points=[70,-40; 70,-50], style(
                color=3, rgbcolor={0,0,255}));
          annotation (
            Icon(
              Line(points=[78,10; 96,10],
                                        style(color=3, rgbcolor={0,0,255})),
              Line(points=[96,-40; 96,60], style(color=3, rgbcolor={0,0,255})),
              Line(points=[96,60; 130,60], style(color=3, rgbcolor={0,0,255})),
              Ellipse(extent=[110,-30; 130,-50], style(color=3, rgbcolor={0,0,
                      255})),
              Line(points=[96,-40; 110,-40], style(color=3, rgbcolor={0,0,255})),
              Ellipse(extent=[-94,58; -48,-42], style(color=3, rgbcolor={0,0,255})),
              Rectangle(extent=[-72,-48; -98,64], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[-132,30; -52,30], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-130,-10; -50,-10], style(color=3, rgbcolor={0,0,255})),
              Line(points=[-72,58; -64,58; -38,56; -18,54; -4,52; 6,50; 14,48;
                    22,46; 30,44; 42,40; 50,36; 58,32; 64,28; 72,22; 76,16; 78,
                    10; 78,4; 72,-6; 62,-14; 52,-20; 36,-26; 16,-32; -6,-36;
                    -20,-38], style(color=3, rgbcolor={0,0,255})),
                 Line(points=[-72,-42; -58,-42; -38,-40; -20,-38], style(color=
                      3, rgbcolor={0,0,255}))),
            Coordsys(extent=[-140,-100; 140,100]),
            Documentation(info="<html>
This is a model of a commercial OR/NOR gate built from thirteen vertically diffused NPN bipolar transistors [1: p.244]. <p>

<hr> <p>

<b>Experimentation:</b> <p>

Simulate the model during 30 microseconds.  Plot the two input voltages, <b>vA</b> and <b>vB</b>, together with the two output voltages, <b>vOR</b> and <b>vNOR</b>, on a single plot. <p>

<hr> <p>

<b>Simulation Results:</b> <p>

We expect the following results: <p>

<img src=\"../Images/BG_Ex10.png\"> <p>

<hr> <p>

<b>References:</b> <p>

<ol>
<li> Cellier, F.E. (1991), <i>Continuous System Modeling</i>, Springer-Verlag, New York, ISBN: 0-387-97502-0, 755 p.
</ol> <p>
</html>
"),         Diagram,
            experiment(StopTime=3e-005),
            experimentSetupOutput);
          connect(OrNor1.Vlow, Vee.p) annotation (points=[-10,-20; -10,-30],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(OrNor1.Vhigh, Vee1.p) annotation (points=[-10,20; -10,30],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(OrNor1.A, Va.p) annotation (points=[-30,4; -50,4; -50,20],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(OrNor1.B, Vb.p) annotation (points=[-30,-4; -50,-4; -50,-20],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Vnor.p, OrNor1.NOR) annotation (points=[40,-20; 40,-10; 10,
                -10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Vor.p, OrNor1.OR) annotation (points=[70,-20; 70,10; 10,10],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          
          connect(Qtot.port_b, FixedTemperature1.port) annotation (points=[51,
                60; 59,60], style(color=42, rgbcolor={191,0,0}));
          connect(Qtot.port_a, OrNor1.heatPort) annotation (points=[29,60; 20,
                60; 20,-20; 0,-20], style(color=42, rgbcolor={191,0,0}));
        end TestOrNor;
      end OrNorGate;
      
      model HeatingMOSInverter "Heating MOS Inverter" 
        parameter Integer Level=3 "MOSFET modeling level";
        output Modelica.SIunits.Voltage vin "Sinusoidal input voltage";
        output Modelica.SIunits.Voltage vout "Output voltage";
        output Modelica.SIunits.Temperature TempCap "Circuit temperature";
        output Modelica.SIunits.Temperature TempNMOS 
          "Temperature of NMOS device";
        output Modelica.SIunits.Temperature TempPMOS 
          "Temperature of PMOS device";
        output Modelica.SIunits.HeatFlowRate HeatNMOS 
          "Heat flow of NMOS device";
        output Modelica.SIunits.HeatFlowRate HeatPMOS 
          "Heat flow of PMOS device";
        output Modelica.SIunits.Voltage VfbNMOS 
          "Flat band voltage of NMOS device";
        output Modelica.SIunits.Voltage VfbPMOS 
          "Flat band voltage of PMOS device";
        output Modelica.SIunits.Voltage VthNMOS 
          "Threshold voltage of NMOS device";
        output Modelica.SIunits.Voltage VthPMOS 
          "Threshold voltage of PMOS device";
        output Modelica.SIunits.Voltage vdsatNMOS 
          "Saturation voltage of NMOS device";
        output Modelica.SIunits.Voltage vdsatPMOS 
          "Saturation voltage of PMOS device";
        output Modelica.SIunits.Current idsNMOS 
          "Drain-source current of NMOS device";
        output Modelica.SIunits.Current idsPMOS 
          "Drain-source current of PMOS device";
        Modelica.Electrical.Analog.Basic.Ground G 
        annotation (extent=[-80,-40; -60,-20]);
        Electrical.Analog.Sources.RampVoltage V(
          V=5,
          duration=0.01,
          offset=0,
          startTime=0) annotation (extent=[40,40; 60,60], rotation=-90);
        Electrical.Analog.Sources.SineVoltage Sin(
          V=5,
          phase=0,
          freqHz=1,
          offset=0,
          startTime=0) annotation (extent=[-80,-12; -60,8], rotation=-90);
        Electrical.Analog.Basic.Capacitor Capacitor1(C=1e-5) 
          annotation (extent=[20,0; 40,20], rotation=-90);
        Electrical.Analog.Spice.Mn Mn1(
          Tnom=300.15,
          GAMMA=0.35,
          PHI=0.65,
          LAMBDA=0.02,
          TOX=1e-7,
          NSUB=1e21,
          NSS=1e14,
          NFS=1e14,
          NEFF=5,
          XJ=1e-6,
          TPG=1,
          U0=0.07,
          UCRIT=1e6,
          UEXP=0.1,
          UTRA=0.5,
          DELTA=1,
          ETA=1,
          THETA=0.05,
          ISD=1e-15,
          ISS=1e-15,
          JS=1e-8,
          PB=0.75,
          CJ=2e-4,
          MJ=0.5,
          CJSW=1e-9,
          MJSW=0.33,
          FC=0.5,
          CGB0=2e-10,
          CGD0=4e-11,
          CGS0=4e-11,
          RD=10,
          RS=10,
          TRD1=0.005,
          TRD2=0.0005,
          TRS1=0.005,
          TRS2=0.0005,
          GminDC=1e-16,
          EG=1.16,
          PD=0.8e-6,
          PS=0.8e-6,
          EMin=-100,
          EMax=40,
          L=6e-6,
          W=20e-6,
          Level=Level,
          COX=3.35e-4,
          ECRIT=0,
          XQC=0,
          VMAX=0,
          KAPPA=0.2,
          KP=4.1e-5,
          VT0=0.8,
          LD=1.05e-6,
          WD=1.25e-6,
          AD=1e-7,
          AS=1e-7)     annotation (extent=[-40,0; -20,20]);
        Electrical.Analog.Spice.Mp Mp1(
          Tnom=300.15,
          GAMMA=0.35,
          PHI=0.65,
          LAMBDA=0.02,
          TOX=1e-7,
          NSUB=1e21,
          NSS=1e14,
          NFS=1e14,
          NEFF=5,
          XJ=1e-6,
          TPG=1,
          U0=0.07,
          UCRIT=1e6,
          UEXP=0.1,
          UTRA=0.5,
          DELTA=1,
          ETA=1,
          THETA=0.05,
          ISD=1e-15,
          ISS=1e-15,
          JS=1e-8,
          PB=0.75,
          CJ=2e-4,
          MJ=0.5,
          CJSW=1e-9,
          MJSW=0.33,
          FC=0.5,
          CGB0=2e-10,
          CGD0=4e-11,
          CGS0=4e-11,
          RD=10,
          RS=10,
          TRD1=0.005,
          TRD2=0.0005,
          TRS1=0.005,
          TRS2=0.0005,
          GminDC=1e-16,
          EG=1.16,
          PD=0.8e-6,
          PS=0.8e-6,
          EMin=-100,
          EMax=40,
          L=6e-6,
          W=20e-6,
          Level=Level,
          COX=3.35e-4,
          ECRIT=0,
          XQC=0,
          VMAX=0,
          KAPPA=0.2,
          KP=1.05e-5,
          VT0=-1,
          LD=1.05e-6,
          WD=1.25e-6,
          AD=1e-7,
          AS=1e-7)     annotation (extent=[-40,40; -20,60]);
        annotation (
          Diagram(        Text(extent=[-92,100; 2,72],     string=
                "Heating MOS Inverter")),
          Documentation(info="<html>
The heating MOS inverter shows a heat flow whenever a transistor is leading. <p>

Simulate the circuit during T = 2sec.  Only store the output variables. <p>

Repeat the simulation using all five levels of MOSFET modeling currently available: <p>

<pre>
 <font color=red><b>Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model</b></font>
</pre> <p>

Plot the output variables of the top layer and compare them for the different levels of MOSFET modeling. <p>

<hr> <p>

<b>Release Notes:</b> <p>

<ul>
<li><i>Mai 6, 2004</i> by Christoph Clauss <br>
    realized
</ul>
</html>
"),       Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-53.85,54.65; 58,26],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="MOS Inverter"),
            Text(
              extent=[-72,32; 74,-2],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="with simulation of"),
            Text(
              extent=[-70,4; 74,-28],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="thermic behavior")),
          experiment(StopTime=2),
          experimentSetupOutput(
            states=false,
            derivatives=false,
            inputs=false,
            auxiliaries=false));
        Thermal.HeatTransfer.Passive.ThermalConductor TC1(G=0.01) 
          annotation (extent=[0,-40; 20,-20]);
        Thermal.HeatTransfer.Passive.ThermalConductor TC2(G=0.01) 
          annotation (extent=[0,-70; 20,-50]);
        Thermal.HeatTransfer.Passive.ThermalConductor TC3(G=0.01) 
          annotation (extent=[50,-40; 70,-20]);
        Thermal.HeatTransfer.Passive.HeatCapacitor HeatCapacitor1(C=0.01) 
          annotation (extent=[60,-70; 80,-50], rotation=-90);
        Thermal.HeatTransfer.Sources.FixedTemperature fixedTemperature(T=300) 
          annotation (extent=[100,-40; 80,-20]);
      equation 
        vin = Sin.p.v;
        vout = Capacitor1.p.v;
        TempCap = HeatCapacitor1.port.T;
        TempNMOS = Mn1.heatPort.T;
        TempPMOS = Mp1.heatPort.T;
        HeatNMOS = Mn1.heatPort.Q_flow;
        HeatPMOS = Mp1.heatPort.Q_flow;
        VfbNMOS = Mn1.Mni.IDS0.Vfb;
        VfbPMOS = Mp1.Mpi.IDS0.Vfb;
        VthNMOS = Mn1.Mni.IDS0.Vth;
        VthPMOS = Mp1.Mpi.IDS0.Vth;
        vdsatNMOS = Mn1.Mni.IDS0.vdsat;
        vdsatPMOS = Mp1.Mpi.IDS0.vdsat;
        idsNMOS = Mn1.Mni.IDS0.ids;
        idsPMOS = Mp1.Mpi.IDS0.ids;
        connect(V.n, G.p) annotation (points=[50,40; 50,-20; -70,-20], style(
              color=3, rgbcolor={0,0,255}));
        connect(Sin.n, G.p) annotation (points=[-70,-12; -70,-20], style(color=
                3, rgbcolor={0,0,255}));
        connect(Capacitor1.n, G.p) annotation (points=[30,0; 30,-20; -70,-20],
            style(color=3, rgbcolor={0,0,255}));
        connect(Mn1.S, G.p) annotation (points=[-20,5; -20,-20; -70,-20], style(
              color=3, rgbcolor={0,0,255}));
        connect(Mn1.D, Capacitor1.p) annotation (points=[-20,15; -20,20; 30,20],
            style(color=3, rgbcolor={0,0,255}));
        connect(Sin.p, Mn1.G) annotation (points=[-70,8; -70,14; -40,14; -40,5],
            style(color=3, rgbcolor={0,0,255}));
        connect(Mp1.G, Mn1.G) annotation (points=[-40,45; -40,5], style(color=3,
              rgbcolor={0,0,255}));
        connect(Mp1.S, Mn1.D) annotation (points=[-20,45; -20,15], style(color=
                3, rgbcolor={0,0,255}));
        connect(Mp1.D, V.p) annotation (points=[-20,55; -20,60; 50,60], style(
              color=3, rgbcolor={0,0,255}));
        connect(Mp1.D, Mp1.B) annotation (points=[-20,55; -20,50], style(color=
                3, rgbcolor={0,0,255}));
        connect(Mn1.S, Mn1.B) annotation (points=[-20,5; -20,10], style(color=3,
              rgbcolor={0,0,255}));
        connect(TC3.port_a, TC1.port_b) annotation (points=[49,-30; 21,-30],
            style(color=42, rgbcolor={191,0,0}));
        connect(TC2.port_b, TC3.port_a) annotation (points=[21,-60; 40,-60; 40,
              -30; 49,-30], style(color=42, rgbcolor={191,0,0}));
        connect(HeatCapacitor1.port, TC2.port_b) annotation (points=[60,-60; 21,
              -60], style(color=42, rgbcolor={191,0,0}));
        connect(fixedTemperature.port, TC3.port_b) annotation (points=[79,-30;
              71,-30], style(color=42, rgbcolor={191,0,0}));
        connect(TC1.port_a, Mp1.heatPort) annotation (points=[-1,-30; -10,-30;
              -10,40; -30,40], style(
            color=42,
            rgbcolor={191,0,0},
            fillColor=42,
            rgbfillColor={209,0,0},
            fillPattern=1));
        connect(TC2.port_a, Mn1.heatPort) annotation (points=[-1,-60; -30,-60;
              -30,0], style(
            color=42,
            rgbcolor={191,0,0},
            fillColor=42,
            rgbfillColor={209,0,0},
            fillPattern=1));
      end HeatingMOSInverter;
      
      package NandGate "NAND gate built in CMOS logic" 
        annotation (preferedView="info",Diagram(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-65.85,52.65; 66,22],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog NAND gate"),
            Text(
              extent=[-58,32; 60,0],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built in CMOS logic"),
            Text(
              extent=[-58,-4; 58,-56],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="2 PMOS devices"),
            Text(
              extent=[-64,22; 70,-34],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using 2 NMOS and ")), Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-65.85,52.65; 66,22],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Analog NAND gate"),
            Text(
              extent=[-58,32; 60,0],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="built in CMOS logic"),
            Text(
              extent=[-58,-4; 58,-56],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="2 PMOS devices"),
            Text(
              extent=[-64,22; 70,-34],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using 2 NMOS and ")),
          Documentation(info="<html>
<p>
The nand gate is a basic CMOS building block. It consists of four CMOS transistors.
The output voltage Nand.y.v is low if and only if the two input voltages at Nand.x1.v and Nand.x2.v
are both high. In this way the nand functionality is realized.
<p>
The simulation end time should be set to 1e-7. Please plot the input voltages Nand.x1.v,
d Nand.x2.v, and the output voltage Nand.y.v.
<p>
<dl>
<dt>
<b>Reference:</b>
<dd>
Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 157
<p>
<dt>
<b>Main Authors:</b>
<dd>
<a href=\"http://people.eas.iis.fhg.de/Christoph.Clauss/\">Christoph Clau&szlig;</a>
    &lt;<a href=\"mailto:clauss@eas.iis.fhg.de\">clauss@eas.iis.fhg.de</a>&gt;<br>
    <a href=\"http://people.eas.iis.fhg.de/Andre.Schneider/\">Andr&eacute; Schneider</a>
    &lt;<a href=\"mailto:schneider@eas.iis.fhg.de\">schneider@eas.iis.fhg.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2002, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>
"));
        model Nand "CMOS NAND Gate" 
          parameter Integer Level=3 "MOSFET modeling level";
          Modelica.Electrical.Analog.Interfaces.Pin x1 
            "First input electrical connector" 
                            annotation (extent=[-110,40; -90,60]);
          Modelica.Electrical.Analog.Interfaces.Pin x2 
            "Second input electrical connector" 
                            annotation (extent=[-110,-60; -90,-40]);
          Modelica.Electrical.Analog.Interfaces.Pin Vdd 
            "High support electrical connector" 
                             annotation (extent=[-10,90; 10,110]);
          Modelica.Electrical.Analog.Interfaces.Pin y 
            "NAND output electrical connector" 
                           annotation (extent=[90,-10; 110,10]);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start=
                  300.15)) "Thermal connector" 
            annotation (extent=[-10,-90; 10,-110],   rotation=-90);
          Electrical.Analog.Spice.Mp Mp1(
            Tnom=300.15,
            GAMMA=0.35,
            PHI=0.65,
            LAMBDA=0.02,
            TOX=1e-7,
            NSUB=1e21,
            NSS=1e14,
            NFS=1e14,
            NEFF=5,
            XJ=1e-6,
            TPG=1,
            U0=0.07,
            UCRIT=1e6,
            UEXP=0.1,
            UTRA=0.5,
            DELTA=1,
            ETA=1,
            THETA=0.05,
            ISD=1e-15,
            ISS=1e-15,
            JS=1e-8,
            PB=0.75,
            MJ=0.5,
            MJSW=0.33,
            FC=0.5,
            CGB0=2e-10,
            CGD0=4e-11,
            CGS0=4e-11,
            RD=10,
            RS=10,
            TRD1=0.005,
            TRD2=0.0005,
            TRS1=0.005,
            TRS2=0.0005,
            GminDC=1e-16,
            EG=1.16,
            PD=0.8e-6,
            PS=0.8e-6,
            EMin=-100,
            EMax=40,
            Level=Level,
            COX=3.35e-4,
            ECRIT=0,
            XQC=0,
            VMAX=0,
            KAPPA=0.2,
            VT0=-1,
            KP=1.05e-5,
            W=6.5e-6,
            L=3.1e-6,
            LD=0.25e-6,
            WD=1.25e-6,
            AD=2e-11,
            AS=2e-11,
            CJ=2e-4,
            CJSW=1e-9)   annotation (extent=[10,58; 30,82]);
          Electrical.Analog.Spice.Mp Mp2(
            Tnom=300.15,
            GAMMA=0.35,
            PHI=0.65,
            LAMBDA=0.02,
            TOX=1e-7,
            NSUB=1e21,
            NSS=1e14,
            NFS=1e14,
            NEFF=5,
            XJ=1e-6,
            TPG=1,
            U0=0.07,
            UCRIT=1e6,
            UEXP=0.1,
            UTRA=0.5,
            DELTA=1,
            ETA=1,
            THETA=0.05,
            ISD=1e-15,
            ISS=1e-15,
            JS=1e-8,
            PB=0.75,
            MJ=0.5,
            MJSW=0.33,
            FC=0.5,
            CGB0=2e-10,
            CGD0=4e-11,
            CGS0=4e-11,
            RD=10,
            RS=10,
            TRD1=0.005,
            TRD2=0.0005,
            TRS1=0.005,
            TRS2=0.0005,
            GminDC=1e-16,
            EG=1.16,
            PD=0.8e-6,
            PS=0.8e-6,
            EMin=-100,
            EMax=40,
            Level=Level,
            COX=3.35e-4,
            ECRIT=0,
            XQC=0,
            VMAX=0,
            KAPPA=0.2,
            VT0=-1,
            KP=1.05e-5,
            W=6.5e-6,
            L=3.1e-6,
            LD=0.25e-6,
            WD=1.25e-6,
            AD=2e-11,
            AS=2e-11,
            CJ=2e-4,
            CJSW=1e-9)   annotation (extent=[-50,44; -30,68]);
          Electrical.Analog.Spice.Mn Mn1(
            Tnom=300.15,
            GAMMA=0.35,
            PHI=0.65,
            LAMBDA=0.02,
            TOX=1e-7,
            NSUB=1e21,
            NSS=1e14,
            NFS=1e14,
            NEFF=5,
            XJ=1e-6,
            TPG=1,
            U0=0.07,
            UCRIT=1e6,
            UEXP=0.1,
            UTRA=0.5,
            DELTA=1,
            ETA=1,
            THETA=0.05,
            ISD=1e-15,
            ISS=1e-15,
            JS=1e-8,
            PB=0.75,
            MJ=0.5,
            MJSW=0.33,
            FC=0.5,
            CGB0=2e-10,
            CGD0=4e-11,
            CGS0=4e-11,
            RD=10,
            RS=10,
            TRD1=0.005,
            TRD2=0.0005,
            TRS1=0.005,
            TRS2=0.0005,
            GminDC=1e-16,
            EG=1.16,
            PD=0.8e-6,
            PS=0.8e-6,
            EMin=-100,
            EMax=40,
            Level=Level,
            COX=3.35e-4,
            ECRIT=0,
            XQC=0,
            VMAX=0,
            KAPPA=0.2,
            KP=4.1e-5,
            VT0=0.8,
            W=6.5e-6,
            L=3.1e-6,
            LD=0.25e-6,
            WD=1.25e-6,
            AD=2e-11,
            AS=2e-11,
            CJ=2e-4,
            CJSW=1e-9)   annotation (extent=[10,14; 30,38]);
          Electrical.Analog.Spice.Mn Mn2(
            Tnom=300.15,
            GAMMA=0.35,
            PHI=0.65,
            LAMBDA=0.02,
            TOX=1e-7,
            NSUB=1e21,
            NSS=1e14,
            NFS=1e14,
            NEFF=5,
            XJ=1e-6,
            TPG=1,
            U0=0.07,
            UCRIT=1e6,
            UEXP=0.1,
            UTRA=0.5,
            DELTA=1,
            ETA=1,
            THETA=0.05,
            ISD=1e-15,
            ISS=1e-15,
            JS=1e-8,
            PB=0.75,
            MJ=0.5,
            MJSW=0.33,
            FC=0.5,
            CGB0=2e-10,
            CGD0=4e-11,
            CGS0=4e-11,
            RD=10,
            RS=10,
            TRD1=0.005,
            TRD2=0.0005,
            TRS1=0.005,
            TRS2=0.0005,
            GminDC=1e-16,
            EG=1.16,
            PD=0.8e-6,
            PS=0.8e-6,
            EMin=-100,
            EMax=40,
            Level=Level,
            COX=3.35e-4,
            ECRIT=0,
            XQC=0,
            VMAX=0,
            KAPPA=0.2,
            KP=4.1e-5,
            VT0=0.8,
            W=6.5e-6,
            L=3.1e-6,
            LD=0.25e-6,
            WD=1.25e-6,
            AD=2e-11,
            AS=2e-11,
            CJ=2e-4,
            CJSW=1e-9)   annotation (extent=[10,-28; 30,-4]);
          Electrical.Analog.Spice.C C4(
            Tnom=300.15,
            C=0.4e-13,
            TC1=0,
            TC2=0,
            Area=1) annotation (extent=[50,20; 70,40], rotation=-90);
          annotation (
            Icon(
              Line(points=[-90,-50; -70,-50; -70,-12; -42,-12], style(color=3,
                    rgbcolor={0,0,255})),
              Line(points=[-90,50; -70,50; -70,14; -42,14], style(color=3,
                    rgbcolor={0,0,255})),
              Ellipse(extent=[54,6; 65,-5], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Line(points=[65,0; 91,0], style(color=3, rgbcolor={0,0,255})),
              Ellipse(extent=[-6,32; 54,-32], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Rectangle(extent=[25,33; -30,-37], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Rectangle(extent=[-42,32; 26,-32], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Rectangle(extent=[25,31.5; 27,-31], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Rectangle(extent=[24.5,31; 26.5,-31.5], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[0,90.5; 0,32], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Line(points=[0,-32; 0,-90], style(
                  color=42,
                  rgbcolor={200,0,0},
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1))),
            Coordsys(grid=[2,2], component=[20,20]),
            Diagram(Ellipse(extent=[-2,2; 2,-2], style(
                  color=42,
                  rgbcolor={200,0,0},
                  fillColor=42,
                  rgbfillColor={181,0,0},
                  fillPattern=1)), Ellipse(extent=[28,52; 32,48], style(
                  color=3,
                  rgbcolor={0,0,255},
                  fillColor=3,
                  rgbfillColor={0,0,255},
                  fillPattern=1))),
            Documentation(info="<html>
<p>
The nand gate is a basic CMOS building block. It consists of four CMOS transistors.
<p>
<dl>
<dt>
<b>Reference:</b>
<dd>
Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 157
<p>
<dt>
<b>Main Authors:</b>
<dd>
<a href=\"http://people.eas.iis.fhg.de/Christoph.Clauss/\">Christoph Clau&szlig;</a>
    &lt;<a href=\"mailto:clauss@eas.iis.fhg.de\">clauss@eas.iis.fhg.de</a>&gt;<br>
    <a href=\"http://people.eas.iis.fhg.de/Andre.Schneider/\">Andr&eacute; Schneider</a>
    &lt;<a href=\"mailto:schneider@eas.iis.fhg.de\">schneider@eas.iis.fhg.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2002, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
<font color=red>The circuit was slightly modified by Alfonso Urqua of the Universidad Nacional de Educacin a Distancia in Madrid from the original circuit proposed by Clau&szlig; and Schneider.  Alfonso tied the bulk of the two p-channel MOSFETs to the high voltage (<b>Vdd</b>) instead of the ground, as the original circuit wouldn't simulate correctly for any modeling levels higher than 0.</font>
<p>
</html>
"));
          
          Electrical.Analog.Spice.C C7(
            Tnom=300.15,
            C=0.4e-13,
            TC1=0,
            TC2=0,
            Area=1) annotation (extent=[50,-60; 70,-40], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd1 
            annotation (extent=[44,-28; 56,-16]);
          Electrical.Analog.Basic.Ground Gnd2 annotation (extent=[54,-2; 66,10]);
          Electrical.Analog.Basic.Ground Gnd3 
            annotation (extent=[54,-82; 66,-70]);
          Electrical.Analog.Basic.Ground Gnd4 annotation (extent=[34,14; 46,26]);
        equation 
          connect(Mp1.D, Vdd) annotation (points=[30,76; 30,88; 0,88; 0,100],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mp2.D, Vdd) annotation (points=[-30,62; -30,88; 0,88; 0,100],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mp2.G, x1) annotation (points=[-50,50; -100,50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mp1.G, Mn1.G) annotation (points=[10,64; 10,20], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mn1.S, Mn2.D) annotation (points=[30,20; 30,-10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mn1.G, x2) annotation (points=[10,20; -70,20; -70,-50; -100,
                -50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mp2.G, Mn2.G) annotation (points=[-50,50; -50,-22; 10,-22],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(C4.p, y) annotation (points=[60,40; 60,50; 80,50; 80,0; 100,0],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(C7.p, Mn2.D) annotation (points=[60,-40; 60,-10; 30,-10],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mn2.B, Gnd1.p) annotation (points=[30,-16; 50,-16], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mn2.S, Gnd1.p) annotation (points=[30,-22; 40,-22; 40,-16; 50,
                -16], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(C4.n, Gnd2.p) annotation (points=[60,20; 60,10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(C7.n, Gnd3.p) annotation (points=[60,-60; 60,-70], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Gnd4.p, Mn1.B) annotation (points=[40,26; 30,26], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mp1.B, Mp1.D) 
               annotation (points=[30,70; 30,76], style(color=3, rgbcolor={0,0,255}));
          connect(Mn2.heatPort, heatPort) annotation (points=[20,-28; 0,-28; 0,
                -100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mn1.heatPort, heatPort) annotation (points=[20,14; 0,14; 0,
                -100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mp1.heatPort, heatPort) annotation (points=[20,58; 0,58; 0,
                -100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(Mp2.heatPort, heatPort) annotation (points=[-40,44; -40,0; 0,
                0; 0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(C4.heatPort, heatPort) annotation (points=[50,30; 50,0; 0,0;
                0,-100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=7,
              rgbfillColor={255,255,255},
              fillPattern=1));
          connect(C7.heatPort, heatPort) annotation (points=[50,-50; 0,-50; 0,
                -100], style(
              color=42,
              rgbcolor={191,0,0},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(Mp1.S, C4.p) annotation (points=[30,64; 30,50; 60,50; 60,40],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(Mp1.S, Mn1.D) annotation (points=[30,64; 30,32], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(Mp2.B, Mp2.D) 
               annotation (points=[-30,56; -30,62], style(color=3, rgbcolor={0,0,255}));
          connect(Mp1.S, Mp2.S) annotation (points=[30,64; 30,50; -30,50],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=3,
              rgbfillColor={0,0,255},
              fillPattern=1));
        end Nand;
        
        model TestNand "Test circuit to evaluate performance of NAND gate" 
          parameter Integer Level=3 "MOSFET modeling level";
          output Modelica.SIunits.Voltage vA "First input voltage";
          output Modelica.SIunits.Voltage vB "Second input voltage";
          output Modelica.SIunits.Voltage vNAND "Output voltage";
          output Modelica.SIunits.HeatFlowRate Pth 
            "Total heat dissipated by NAND gate";
          Nand Nand1(Level=Level) annotation (extent=[-20,-20; 20,20]);
          Electrical.Analog.Sources.TrapezoidVoltage VIN1(
            V=3.5,
            rising=1e-9,
            width=19e-9,
            falling=1e-9,
            period=40e-9,
            nperiod=-1,
            offset=0,
            startTime=20e-9) annotation (extent=[-70,-10; -50,10], rotation=-90);
          annotation (
            Diagram,
            Documentation(info="<html>
<p>
The nand gate is a basic CMOS building block. It consists of four CMOS transistors.
The output voltage Nand.y.v is low if and only if the two input voltages at Nand.x1.v and Nand.x2.v
are both high. In this way the nand functionality is realized.
<p>
The simulation end time should be set to 1e-7. Please plot the input voltages Nand.x1.v,
d Nand.x2.v, and the output voltage Nand.y.v.
<p>
Repeat the simulation using all five levels of MOSFET modeling currently available: <p>

<pre>
 <font color=red><b>Level:   MOSFET modeling level (default value = 3)
            Level=0: Static injection model
            Level=1: Shichman-Hodges model
            Level=2: Grove-Frohman model
            Level=3: Empirical model
            Level=4: Simplified Grove-Frohman model</b></font>
</pre>
<p>
<dl>
<dt>
<b>Reference:</b>
<dd>
Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 157
<p>
<dt>
<b>Main Authors:</b>
<dd>
<a href=\"http://people.eas.iis.fhg.de/Christoph.Clauss/\">Christoph Clau&szlig;</a>
    &lt;<a href=\"mailto:clauss@eas.iis.fhg.de\">clauss@eas.iis.fhg.de</a>&gt;<br>
    <a href=\"http://people.eas.iis.fhg.de/Andre.Schneider/\">Andr&eacute; Schneider</a>
    &lt;<a href=\"mailto:schneider@eas.iis.fhg.de\">schneider@eas.iis.fhg.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2002, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>
"),         Icon(
              Line(points=[-90,-50; -70,-50; -70,-12; -42,-12], style(color=3,
                    rgbcolor={0,0,255})),
              Line(points=[-90,50; -70,50; -70,14; -42,14], style(color=3,
                    rgbcolor={0,0,255})),
              Ellipse(extent=[54,6; 65,-5], style(color=3, rgbcolor={0,0,255})),
              Line(points=[65,0; 91,0], style(color=3, rgbcolor={0,0,255})),
              Ellipse(extent=[-6,32; 54,-32], style(color=3, rgbcolor={0,0,255})),
              Rectangle(extent=[25,33; -30,-37], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255})),
              Rectangle(extent=[-42,32; 26,-32], style(color=3, rgbcolor={0,0,
                      255})),
              Rectangle(extent=[25,31.5; 27,-31], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1)),
              Rectangle(extent=[24.5,31; 26.5,-31.5], style(
                  pattern=0,
                  fillColor=7,
                  rgbfillColor={255,255,255},
                  fillPattern=1))),
            experiment(StopTime=1e-007),
            experimentSetupOutput);
          
          Electrical.Analog.Sources.RampVoltage VDD(
            V=5,
            offset=0,
            startTime=0,
            duration=1e-9) 
                         annotation (extent=[40,40; 60,60], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd4 
            annotation (extent=[-66,-32; -54,-20]);
          Electrical.Analog.Sources.TrapezoidVoltage VIN2(
            V=3.5,
            rising=1e-9,
            width=19e-9,
            falling=1e-9,
            period=40e-9,
            nperiod=-1,
            offset=0,
            startTime=10e-9) 
            annotation (extent=[-50,-40; -30,-20], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd5 
            annotation (extent=[-46,-62; -34,-50]);
          Electrical.Analog.Basic.Ground Gnd1 annotation (extent=[44,18; 56,30]);
          Electrical.Analog.Sensors.VoltageSensor Vnand 
            annotation (extent=[90,-30; 70,-10], rotation=-90);
          Electrical.Analog.Basic.Ground Gnd2 
            annotation (extent=[74,-52; 86,-40]);
          Thermal.HeatTransfer.Sensors.HeatFlowSensor Qtot 
            annotation (extent=[10,-80; 30,-60]);
          Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
                300.15) annotation (extent=[60,-80; 40,-60]);
        equation 
          vA = VIN1.v;
          vB = VIN2.v;
          vNAND = Vnand.v;
          Pth = Qtot.Q_flow;
          connect(VIN1.p, Nand1.x1) annotation (points=[-60,10; -20,10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(VIN1.n, Gnd4.p) annotation (points=[-60,-10; -60,-20], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(VIN2.p, Nand1.x2) annotation (points=[-40,-20; -40,-10; -20,
                -10], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(VIN2.n, Gnd5.p) annotation (points=[-40,-40; -40,-50], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(VDD.n, Gnd1.p) annotation (points=[50,40; 50,30], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(VDD.p, Nand1.Vdd) annotation (points=[50,60; 50,70; 0,70; 0,
                20], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(Vnand.p, Nand1.y) annotation (points=[80,-10; 80,0; 20,0],
              style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(Vnand.n, Gnd2.p) annotation (points=[80,-30; 80,-40], style(
              color=3,
              rgbcolor={0,0,255},
              fillColor=42,
              rgbfillColor={181,0,0},
              fillPattern=1));
          connect(Qtot.port_b, FixedTemperature1.port) annotation (points=[31,
                -70; 39,-70], style(color=42, rgbcolor={191,0,0}));
          connect(Qtot.port_a, Nand1.heatPort) annotation (points=[9,-70; 0,-70;
                0,-20], style(color=42, rgbcolor={191,0,0}));
        end TestNand;
      end NandGate;
      
      model OneWayRectifier "One-way rectifier circuit" 
        output Modelica.SIunits.Voltage vout "Output voltage";
        annotation (
          Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-33.85,54.65; 36,26],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="One-way"),
            Text(
              extent=[-58,32; 58,-2],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="rectifier circuit"),
            Text(
              extent=[-96,8; 94,-30],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using Spice-style diode")),
          Diagram,
          experiment(StopTime=0.005),
          experimentSetupOutput,
          Documentation(info="<html>
This circuit demonstrates the use of the Spice-style DIODE model in a simple one-way rectifier circuit. <p>

Simulate the model across 0.005 seconds and plot the output voltage.
</html>
"));
        Electrical.Analog.Sources.SineVoltage V0(
          V=10,
          phase=0,
          freqHz=1000,
          offset=0,
          startTime=0) annotation (extent=[-80,-10; -60,10], rotation=-90);
        Electrical.Analog.Spice.L L1(
          Tnom=300.15,
          L=0.015,
          TL1=0,
          TL2=0) annotation (extent=[-60,20; -40,40]);
        Electrical.Analog.Spice.D D1(
          Tnom=300.15,
          Level=1,
          IS=1e-14,
          RS=5,
          N=1,
          TT=1e-9,
          CJ=0.5e-12,
          PB=0.75,
          MJ=0.33,
          EG=1.16,
          XTI=3,
          FC=0.5,
          BV=0,
          IBV=0,
          TRS=0.005,
          TCV=0,
          TTT1=0,
          TTT2=0,
          ISW=0,
          CJSW=0,
          PHP=0.75,
          MJSW=0.33,
          TOX=1e-7,
          GminDC=1e-16,
          Area=1,
          PJ=1,
          EMin=-100,
          EMax=40) annotation (extent=[-20,20; 0,40]);
        Electrical.Analog.Spice.C C1(
          Tnom=300.15,
          C=50e-6,
          TC1=0,
          TC2=0,
          Area=1) annotation (extent=[10,-10; 30,10], rotation=-90);
        Electrical.Analog.Spice.R R1(
          Tnom=300.15,
          R=125,
          TR1=0.005,
          TR2=0.005,
          Area=1) annotation (extent=[40,-10; 60,10], rotation=-90);
        Electrical.Analog.Basic.Ground Gnd1 
          annotation (extent=[-30,-50; -10,-30]);
        Electrical.Analog.Sensors.VoltageSensor Vout 
          annotation (extent=[90,-10; 70,10], rotation=-90);
        Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
              300.15) annotation (extent=[40,-80; 20,-60]);
      equation 
        vout = Vout.v;
        connect(L1.p, V0.p) annotation (points=[-60,30; -70,30; -70,10], style(
              color=3, rgbcolor={0,0,255}));
        connect(D1.p, L1.n) annotation (points=[-20,30; -40,30], style(color=3,
              rgbcolor={0,0,255}));
        connect(D1.n, C1.p) annotation (points=[0,30; 20,30; 20,10], style(
              color=3, rgbcolor={0,0,255}));
        connect(R1.p, C1.p) annotation (points=[50,10; 50,30; 20,30; 20,10],
            style(color=3, rgbcolor={0,0,255}));
        connect(R1.n, C1.n) annotation (points=[50,-10; 50,-30; 20,-30; 20,-10],
            style(color=3, rgbcolor={0,0,255}));
        connect(C1.n, Gnd1.p) annotation (points=[20,-10; 20,-30; -20,-30],
            style(color=3, rgbcolor={0,0,255}));
        connect(V0.n, Gnd1.p) annotation (points=[-70,-10; -70,-30; -20,-30],
            style(color=3, rgbcolor={0,0,255}));
        connect(R1.heatPort, C1.heatPort) annotation (points=[40,-6.12303e-016;
              40,-20; 0,-20; 0,-6.12303e-016; 10,-6.12303e-016], style(color=42,
              rgbcolor={191,0,0}));
        connect(R1.p, Vout.p) annotation (points=[50,10; 50,30; 80,30; 80,10],
            style(color=3, rgbcolor={0,0,255}));
        connect(Vout.n, R1.n) annotation (points=[80,-10; 80,-30; 50,-30; 50,
              -10], style(color=3, rgbcolor={0,0,255}));
        connect(FixedTemperature1.port, C1.heatPort) annotation (points=[19,-70;
              0,-70; 0,-6.12303e-016; 10,-6.12303e-016], style(color=42,
              rgbcolor={191,0,0}));
        connect(L1.heatPort, C1.heatPort) annotation (points=[-50,20; -50,
              -6.12303e-016; 10,-6.12303e-016], style(color=42, rgbcolor={191,0,
                0}));
        connect(D1.heatPort, C1.heatPort) annotation (points=[-10,20; -10,
              -6.12303e-016; 10,-6.12303e-016], style(color=42, rgbcolor={191,0,
                0}));
      end OneWayRectifier;
      
      model OneWayRectifierBG 
        "One-way rectifier circuit in bond graph interface" 
        output Modelica.SIunits.Voltage vout "Output voltage";
        Sources.sinSe V0(
          e0=10,
          phase=0,
          freqHz=1000,
          offset=0,
          startTime=0) annotation (extent=[-70,-10; -90,10]);
        Bonds.fBond B1 annotation (extent=[-70,-10; -50,10]);
        annotation (
          Diagram,
          Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-33.85,54.65; 36,26],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="One-way"),
            Text(
              extent=[-58,32; 58,-2],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="rectifier circuit"),
            Text(
              extent=[-96,8; 94,-30],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using Spice-style diode"),
            Text(
              extent=[-96,-20; 94,-58],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="in bond graph interface")),
          Documentation(info="<html>
This circuit demonstrates the use of the Spice-style DIODE model in a simple one-way rectifier circuit. This is the same model as the one called <b>OneWayRectifier</b>.  However, this circuit uses the bond graph interface instead of the circuit diagram interface. <p>

Simulate the model across 0.005 seconds and plot the output voltage.
</html>
"),       experiment(StopTime=0.005),
          experimentSetupOutput);
        Junctions.J1p4 J1p4_1 annotation (extent=[-50,-10; -30,10]);
        Bonds.fBond B2 annotation (extent=[-50,10; -30,30], rotation=90);
        Spice.LS L1(
          Tnom=300.15,
          L=0.015,
          TL1=0,
          TL2=0) annotation (extent=[-50,30; -30,70], rotation=90);
        Bonds.eBond B3 annotation (extent=[-50,70; -30,90], rotation=90);
        Bonds.eBond B4 annotation (extent=[-50,-30; -30,-10], rotation=270);
        Spice.DS DS1(
          Level=1,
          Tnom=300.15,
          IS=1e-14,
          RS=5,
          N=1,
          TT=1e-9,
          CJ=0.5e-12,
          PB=0.75,
          MJ=0.33,
          EG=1.16,
          XTI=3,
          FC=0.5,
          BV=0,
          IBV=0,
          TRS=0.005,
          TCV=0,
          TTT1=0,
          TTT2=0,
          ISW=0,
          CJSW=0,
          PHP=0.75,
          MJSW=0.33,
          TOX=1e-7,
          GminDC=1e-16,
          Area=1,
          PJ=1,
          EMin=-100,
          EMax=40) annotation (extent=[-50,-70; -30,-30], rotation=-90);
        Bonds.eBond B5 annotation (extent=[-50,-90; -30,-70], rotation=270);
        Bonds.eBond B6 annotation (extent=[-30,-10; -10,10], rotation=0);
        Junctions.J0p4 J0p4_1 annotation (extent=[-10,-10; 10,10]);
        Bonds.eBond B7 annotation (extent=[-10,10; 10,30], rotation=90);
        Spice.CS C1(
          Tnom=300.15,
          C=50e-6,
          TC1=0,
          TC2=0,
          Area=1) annotation (extent=[-10,30; 10,70], rotation=90);
        Bonds.eBond B8 annotation (extent=[-10,70; 10,90], rotation=90);
        Bonds.fBond B9 annotation (extent=[-10,-30; 10,-10], rotation=-90);
        Spice.RS R1(
          Tnom=300.15,
          R=125,
          TR1=0.005,
          TR2=0.005,
          Area=1) annotation (extent=[-10,-70; 10,-30], rotation=-90);
        Bonds.eBond B10 annotation (extent=[-10,-90; 10,-70], rotation=270);
        Bonds.fBond B11 annotation (extent=[10,-10; 30,10]);
        Sensors.De Vout annotation (extent=[30,-10; 50,10]);
        Junctions.J0p5 J0p5_1 annotation (extent=[20,-60; 40,-40]);
        Bonds.fBond B12 annotation (extent=[40,-60; 60,-40], rotation=180);
        Sources.Se Tdev(e0=300.15) annotation (extent=[60,-62; 90,-38]);
      equation 
        vout = Vout.e;
        connect(B1.eBondCon1, V0.BondCon1) annotation (points=[-70,0; -70,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p4_1.BondCon1, B1.fBondCon1) annotation (points=[-50,0; -50,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B2.eBondCon1, J1p4_1.BondCon4) annotation (points=[-40,10; -40,
              10], style(color=8, rgbcolor={192,192,192}));
        connect(L1.BondCon1, B2.fBondCon1) annotation (points=[-40,30; -40,30],
            style(color=8, rgbcolor={192,192,192}));
        connect(B3.fBondCon1, L1.BondCon2) annotation (points=[-40,70; -40,70],
            style(color=8, rgbcolor={192,192,192}));
        connect(J1p4_1.BondCon3, B4.fBondCon1) annotation (points=[-40,-10; -40,
              -10], style(color=8, rgbcolor={192,192,192}));
        connect(DS1.BondCon1, B4.eBondCon1) annotation (points=[-40,-30; -40,
              -30], style(color=8, rgbcolor={192,192,192}));
        connect(DS1.BondCon2, B5.fBondCon1) annotation (points=[-40,-70; -40,
              -70], style(color=8, rgbcolor={192,192,192}));
        connect(J1p4_1.BondCon2, B6.fBondCon1) annotation (points=[-30,0; -30,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B6.eBondCon1, J0p4_1.BondCon1) annotation (points=[-10,0; -10,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B7.fBondCon1, J0p4_1.BondCon4) annotation (points=[
              -6.12303e-016,10; 0,10], style(color=8, rgbcolor={192,192,192}));
        connect(C1.BondCon1, B7.eBondCon1) annotation (points=[-6.12303e-016,30;
              6.12303e-016,30], style(color=8, rgbcolor={192,192,192}));
        connect(B8.fBondCon1, C1.BondCon2) annotation (points=[-6.12303e-016,70;
              6.12303e-016,70], style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_1.BondCon3, B9.eBondCon1) annotation (points=[0,-10;
              -6.12303e-016,-10], style(color=8, rgbcolor={192,192,192}));
        connect(B9.fBondCon1, R1.BondCon1) annotation (points=[6.12303e-016,-30;
              -6.12303e-016,-30], style(color=8, rgbcolor={192,192,192}));
        connect(R1.BondCon2, B10.fBondCon1) annotation (points=[6.12303e-016,
              -70; 1.83691e-015,-70], style(color=8, rgbcolor={192,192,192}));
        connect(J0p4_1.BondCon2, B11.eBondCon1) annotation (points=[10,0; 10,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B11.fBondCon1, Vout.BondCon1) annotation (points=[30,0; 30,0],
            style(color=8, rgbcolor={192,192,192}));
        connect(B10.eBondCon1, J0p5_1.BondCon3) annotation (points=[
              -1.83691e-015,-90; -1.83691e-015,-94; 26,-94; 26,-60], style(
              color=8, rgbcolor={192,192,192}));
        connect(B5.eBondCon1, J0p5_1.BondCon4) annotation (points=[-40,-90; -40,
              -96; 34,-96; 34,-60], style(color=8, rgbcolor={192,192,192}));
        connect(B8.eBondCon1, J0p5_1.BondCon1) annotation (points=[6.12303e-016,
              90; 6.12303e-016,94; 60,94; 60,-30; 20,-30; 20,-50], style(color=
                8, rgbcolor={192,192,192}));
        connect(B3.eBondCon1, J0p5_1.BondCon5) annotation (points=[-40,90; -40,
              96; 62,96; 62,-32; 30,-32; 30,-40], style(color=8, rgbcolor={192,
                192,192}));
        connect(J0p5_1.BondCon2, B12.fBondCon1) annotation (points=[40,-50; 40,
              -50], style(color=8, rgbcolor={192,192,192}));
        connect(B12.eBondCon1, Tdev.BondCon1) annotation (points=[60,-50; 60,
              -50], style(color=8, rgbcolor={192,192,192}));
      end OneWayRectifierBG;
      
      model DifferentialAmplifier 
        "Differential amplifier circuit made from two p-channel JFETs" 
        output Modelica.SIunits.Voltage vin;
        output Modelica.SIunits.Voltage vout;
        Electrical.Analog.Basic.Ground Gnd1 annotation (extent=[-8,-82; 8,-70]);
        Electrical.Analog.Spice.R Rg(
          Tnom=300.15,
          R=75e3,
          TR1=0,
          TR2=0,
          Area=1) annotation (extent=[-10,-60; 10,-40], rotation=90);
        annotation (
          Diagram,
          experiment(StopTime=0.005),
          experimentSetupOutput,
          Icon(
            Rectangle(extent=[-99.85,100.65; 100,-100],        style(
                color=0,
                thickness=2,
                fillColor=9)),
            Text(
              extent=[-69.85,56.65; 78,24],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="Differential amplifier"),
            Text(
              extent=[-34,26; 28,6],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="circuit"),
            Text(
              extent=[-78,6; 76,-26],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="using two Spice-style"),
            Text(
              extent=[-70,-20; 68,-58],
              style(
                color=0,
                thickness=2,
                fillColor=9,
                fillPattern=1),
              string="p-channel JFETs")),
          Documentation(info="<html>
This circuit describes a differential amplifier built from two p-channel JFETs.  Notice the use of ramping sources.  After ramping, the gate of the right JFET is held constant at 0.5 Volt, whereas the gate of the left JFET varies slightly around that value using a sine-wave.  The output voltage inverts and amplifies that differential voltage by about a factor of 80. <p>

Simulate the circuit across 0.005 seconds and plot the input and output voltages as functions of time.

<font color=red><b><p>
This example does not currently simulate. It simulated correctly under Dymola 6.0b with Modelica 2.2,
but in Dymola 6.1 with Modelica 2.2.1, the simulation dies at time 0, claiming that a Newton iteration
on a non-linear set of equations fails to converge. <p>

We are still investigating what the cause of this anomaly is.
</p></b></font>
</html>
"));
        Electrical.Analog.Spice.Jp Jp1(
          Level=1,
          Tnom=300.15,
          IS=1e-15,
          RS=0.5,
          RD=1.5,
          RG=0,
          BETA=15e-3,
          LAMBDA=0.001,
          TCV=0,
          TRS=0,
          TRD=0,
          TRG=0,
          EG=1.16,
          N=1,
          XTI=3,
          CGS=50e-15,
          CGD=50e-15,
          M=0.5,
          FC=0.5,
          PB=0.2,
          TT=1e-12,
          BEX=0,
          GminDC=1e-16,
          Area=1,
          EMin=-100,
          EMax=40,
          VT0=-1.1) annotation (extent=[-40,-22; -20,2]);
        Electrical.Analog.Spice.Jp Jp2(
          Level=1,
          Tnom=300.15,
          IS=1e-15,
          RS=0.5,
          RD=1.5,
          RG=0,
          BETA=15e-3,
          LAMBDA=0.001,
          TCV=0,
          TRS=0,
          TRD=0,
          TRG=0,
          EG=1.16,
          N=1,
          XTI=3,
          CGS=50e-15,
          CGD=50e-15,
          M=0.5,
          FC=0.5,
          PB=0.2,
          TT=1e-12,
          BEX=0,
          GminDC=1e-16,
          Area=1,
          EMin=-100,
          EMax=40,
          VT0=-1.1) annotation (extent=[40,-22; 20,2]);
        Electrical.Analog.Spice.R Rd1(
          Tnom=300.15,
          TR1=0,
          TR2=0,
          Area=1,
          R=150e3) annotation (extent=[-10,20; -30,40], rotation=90);
        Electrical.Analog.Spice.R Rd2(
          Tnom=300.15,
          TR1=0,
          TR2=0,
          Area=1,
          R=150e3) annotation (extent=[10,20; 30,40], rotation=90);
        Electrical.Analog.Spice.RampingSources.V V1(V=-5, rampTime=1e-3) 
          annotation (extent=[50,20; 70,40], rotation=-90);
        Electrical.Analog.Basic.Ground Gnd2 annotation (extent=[52,-2; 68,10]);
        Electrical.Analog.Spice.RampingSources.V V2(V=0.5, rampTime=1e-3) 
          annotation (extent=[50,-60; 70,-40], rotation=-90);
        Electrical.Analog.Basic.Ground Gnd3 annotation (extent=[52,-82; 68,-70]);
        Electrical.Analog.Spice.RampingSources.sinV Vin(
          offset=0.5,
          freqHz=1e3,
          rampTime=1e-3,
          settleTime=1e-3,
          V=0.02) annotation (extent=[-70,-60; -50,-40], rotation=-90);
        Electrical.Analog.Basic.Ground Gnd4 
          annotation (extent=[-68,-82; -52,-70]);
        Electrical.Analog.Sensors.VoltageSensor Vout 
          annotation (extent=[-10,20; 10,0]);
        Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T=
              300.15) annotation (extent=[-80,60; -60,80]);
      equation 
        vin = Vin.v;
        vout = Vout.v;
        connect(Rg.p, Gnd1.p) annotation (points=[-6.12303e-016,-60; 0,-60; 0,
              -70], style(color=3, rgbcolor={0,0,255}));
        connect(Jp1.S, Rg.n) annotation (points=[-20,-16; -20,-30; 0,-30; 0,-40;
              6.12303e-016,-40], style(color=3, rgbcolor={0,0,255}));
        connect(Jp2.S, Rg.n) annotation (points=[20,-16; 20,-30; 0,-30; 0,-40;
              6.12303e-016,-40], style(color=3, rgbcolor={0,0,255}));
        connect(Rd1.p, Jp1.D) annotation (points=[-20,20; -20,-4], style(color=
                3, rgbcolor={0,0,255}));
        connect(Rd2.p, Jp2.D) annotation (points=[20,20; 20,-4], style(color=3,
              rgbcolor={0,0,255}));
        connect(Rd1.n, Rd2.n) annotation (points=[-20,40; -20,50; 20,50; 20,40],
            style(color=3, rgbcolor={0,0,255}));
        connect(Rd2.n, V1.p) annotation (points=[20,40; 20,50; 60,50; 60,40],
            style(color=3, rgbcolor={0,0,255}));
        connect(V1.n, Gnd2.p) annotation (points=[60,20; 60,10], style(color=3,
              rgbcolor={0,0,255}));
        connect(Jp2.G, V2.p) annotation (points=[40,-16; 60,-16; 60,-40], style(
              color=3, rgbcolor={0,0,255}));
        connect(V2.n, Gnd3.p) annotation (points=[60,-60; 60,-70], style(color=
                3, rgbcolor={0,0,255}));
        connect(Vin.n, Gnd4.p) annotation (points=[-60,-60; -60,-70], style(
              color=3, rgbcolor={0,0,255}));
        connect(Jp1.G, Vin.p) annotation (points=[-40,-16; -60,-16; -60,-40],
            style(color=3, rgbcolor={0,0,255}));
        connect(Vout.p, Rd1.p) annotation (points=[-10,10; -20,10; -20,20],
            style(color=3, rgbcolor={0,0,255}));
        connect(Vout.n, Rd2.p) annotation (points=[10,10; 20,10; 20,20], style(
              color=3, rgbcolor={0,0,255}));
        connect(FixedTemperature1.port, Rd1.heatPort) annotation (points=[-59,
              70; -44,70; -44,30; -30,30], style(color=42, rgbcolor={191,0,0}));
        connect(FixedTemperature1.port, Rd2.heatPort) annotation (points=[-59,
              70; 44,70; 44,30; 30,30], style(color=42, rgbcolor={191,0,0}));
        connect(Jp2.heatPort, Rd2.heatPort) annotation (points=[30,-22; 44,-22; 44,
              30; 30,30], style(
            color=42,
            rgbcolor={191,0,0},
            fillColor=42,
            rgbfillColor={209,0,0},
            fillPattern=1));
        connect(Jp1.heatPort, Rd1.heatPort) annotation (points=[-30,-22; -44,
              -22; -44,30; -30,30], style(
            color=42,
            rgbcolor={191,0,0},
            fillColor=42,
            rgbfillColor={209,0,0},
            fillPattern=1));
        connect(Rg.heatPort, Jp2.heatPort) annotation (points=[10,-50; 30,-50;
              30,-22], style(
            color=42,
            rgbcolor={191,0,0},
            fillColor=42,
            rgbfillColor={209,0,0},
            fillPattern=1));
      end DifferentialAmplifier;
    end ModelicaSpice;
  end Examples;
end BondLib;
